package com.bocloud.cmp.provider.aliyunconvertor;

import com.aliyun.bssopenapi20171214.models.QueryBillResponseBody;
import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: BillingConvertor.java
 * @Description: 阿里云账单转换类
 * @author: ymh
 * @date: 2021年3月15日 上午11:44:34
 */
public class BillingConvertor implements BeanConvertor<PublicCloudBillModel, QueryBillResponseBody.QueryBillResponseBodyDataItemsItem> {

    @Override
    public List<PublicCloudBillModel> convertList(List<? extends QueryBillResponseBody.QueryBillResponseBodyDataItemsItem> objects) {
        List<PublicCloudBillModel> vpcModels = new ArrayList<>();
        for (QueryBillResponseBody.QueryBillResponseBodyDataItemsItem object : objects) {
            vpcModels.add(convertModel(object));
        }
        return vpcModels;
    }

    @Override
    public PublicCloudBillModel convertModel(QueryBillResponseBody.QueryBillResponseBodyDataItemsItem item) {
        PublicCloudBillModel model = null;
        if (null != item) {
            model = new PublicCloudBillModel();
            model.setOrderSn(item.getRecordID());
            model.setStatus(item.getStatus());
            model.setProductCode(item.getProductCode());
            model.setProductName(item.getProductName());
            model.setProductDetail(item.getProductDetail());
            String usageStartTime = item.getUsageStartTime();
            if (!StringUtils.isEmpty(usageStartTime)) {
                model.setFeeBeginTime(DateTools.stringToDateTime(usageStartTime));
            }
            String usageEndTime = item.getUsageEndTime();
            if (!StringUtils.isEmpty(usageEndTime)) {
                model.setFeeEndTime(DateTools.stringToDateTime(usageEndTime));
            }
            // 原价。
            Float pretaxGrossAmount = item.getPretaxGrossAmount() == null ? 0F : item.getPretaxGrossAmount();
            model.setPretaxGrossAmount(new BigDecimal(pretaxGrossAmount).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 优惠金额。
            Float invoiceDiscount = item.getInvoiceDiscount() == null ? 0F : item.getInvoiceDiscount();
            model.setInvoiceDiscount(new BigDecimal(invoiceDiscount).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 抹零金额。
            String roundDownDiscount = item.getRoundDownDiscount() == null ? "0" : item.getRoundDownDiscount();
            model.setRoundDownDiscount(new BigDecimal(roundDownDiscount).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 应付金额。
            Float pretaxAmount = item.getPretaxAmount() == null ? 0F : item.getPretaxAmount();
            model.setPretaxAmount(new BigDecimal(pretaxAmount).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 现金支付。
            Float paymentAmount = item.getPaymentAmount() == null ? 0F : item.getPaymentAmount();
            model.setMoney(new BigDecimal(paymentAmount).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 优惠券抵扣。
            Float deductedByCoupons = item.getDeductedByCoupons() == null ? 0F : item.getDeductedByCoupons();
            model.setDeductedByCoupons(new BigDecimal(deductedByCoupons).setScale(2, BigDecimal.ROUND_HALF_UP));
            // 储值卡支付金额
            Float deductedByPrepaidCard = item.getDeductedByPrepaidCard() == null ? 0F
                    : item.getDeductedByPrepaidCard();
            model.setDeductedByPrepaidCard(new BigDecimal(deductedByPrepaidCard).setScale(2, BigDecimal.ROUND_HALF_UP));
        }
        return model;
    }

}
