package com.bocloud.cmp.provider.aliyunconvertor;

import com.aliyun.ecs20140526.models.DescribeInstanceTypesResponseBody;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 云主机数据转换器
 */
public class FlavorsConvertor implements BeanConvertor<FlavorModel, DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType> {

    @Override
    public List<FlavorModel> convertList(List<? extends DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType> objects) {
        List<FlavorModel> flavorModels = new ArrayList<>();
        for (DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType object : objects) {
            flavorModels.add(convertModel(object));
        }
        return flavorModels;
    }

    @Override
    public FlavorModel convertModel(DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType instanceType) {
        FlavorModel flavorModel = null;
        if (null != instanceType) {
            flavorModel = new FlavorModel();
            flavorModel.setCpu(instanceType.getCpuCoreCount());
            flavorModel.setMemory(new BigDecimal(instanceType.getMemorySize()));
            flavorModel.setFamily(instanceType.getInstanceTypeFamily());
            flavorModel.setPerformanceType(instancesType(instanceType.getInstanceTypeFamily()));
            flavorModel.setName(instanceType.getInstanceTypeId());
            flavorModel.setFlavorUuid(instanceType.getInstanceTypeId());
        }
        return flavorModel;
    }

    // 本文所列实例规格在中国站已全部停售，其中，sn2、sn1、n1、n2和e3在国际站（International）仍然在售。
    private static final String[] SOLD_OUT_TYPE = {"ecs.sn2", "ecs.sn1", "ecs.c4", "ecs.ce4", "ecs.cm4", "ecs.n1",
            "ecs.n2", "ecs.e3"};
    // 通用型
    private static final String[] COMMON_TYPE = {"ecs.g6", "ecs.g5", "ecs.sn2ne", "ecs.sn2"};
    // 计算型
    private static final String[] COMPUTATIONAL_TYPE = {"ecs.ic5", "ecs.c6", "ecs.c5", "ecs.sn1ne", "ecs.sn1"};
    // 内存型
    private static final String[] MEMORY_TYPE = {"ecs.r6", "ecs.r5", "ecs.re4", "ecs.re4e", "ecs.se1ne", "ecs.se1"};
    // 大数据型
    private static final String[] BIG_DATA_TYPE = {"ecs.d1ne", "ecs.d1"};
    // 本地SSD型
    private static final String[] LOCAL_SSD_TYPE = {"ecs.i2", "ecs.i2g", "ecs.i1"};
    // 高主频型
    private static final String[] HIGH_FREQUENCY_TYPE = {"ecs.hfc6", "ecs.hfg6", "ecs.hfr6", "ecs.hfc5", "ecs.hfg5"};
    // 入门级(共享)
    private static final String[] SHARED_TYPE = {"ecs.xn4", "ecs.n4", "ecs.mn4", "ecs.e4", "ecs.n1", "ecs.n2",
            "ecs.e3"};

    private String instancesType(String instanceFamily) {
        if (StringUtils.isEmpty(instanceFamily)) {
            return "其他";
        }
        if (Arrays.asList(COMMON_TYPE).contains(instanceFamily)) {
            return "通用型";
        } else if (Arrays.asList(COMPUTATIONAL_TYPE).contains(instanceFamily)) {
            return "计算型";
        } else if (Arrays.asList(MEMORY_TYPE).contains(instanceFamily)) {
            return "内存型";
        } else if (Arrays.asList(BIG_DATA_TYPE).contains(instanceFamily)) {
            return "大数据型";
        } else if (Arrays.asList(LOCAL_SSD_TYPE).contains(instanceFamily)) {
            return "本地SSD型";
        } else if (Arrays.asList(HIGH_FREQUENCY_TYPE).contains(instanceFamily)) {
            return "高主频型";
        } else if (Arrays.asList(SHARED_TYPE).contains(instanceFamily)) {
            return "入门级(共享)";
        } else {
            return "其他";
        }
    }

}
