package com.bocloud.cmp.provider.aliyunconvertor;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SimplePropertyPreFilter;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

/**
 * 监听器Convertor
 *
 * @author lyy
 * @version 1.0
 * @date 2020/12/22 16:29
 */
public class ListenerConvertor implements BeanConvertor<LbListenerModel, JSONObject> {
    private String loadBalancerId;
    private String protocol;

    public ListenerConvertor(String loadBalancerId, String protocol) {
        this.loadBalancerId = loadBalancerId;
        this.protocol = protocol;
    }

    @Override
    public List<LbListenerModel> convertList(List<? extends JSONObject> list) {
        List<LbListenerModel> lbListenerModels = new ArrayList<>();
        for (JSONObject listener : list) {
            lbListenerModels.add(convertModel(listener));
        }
        return lbListenerModels;
    }

    @Override
    public LbListenerModel convertModel(JSONObject listener) {
        SimplePropertyPreFilter filter = new SimplePropertyPreFilter();
        filter.getExcludes().add("aclStatus");
        filter.getExcludes().add("stickySession");
        filter.getExcludes().add("gzip");
        filter.getExcludes().add("xForwardedFor");
        filter.getExcludes().add("xForwardedFor_SLBID");
        filter.getExcludes().add("xForwardedFor_SLBIP");
        filter.getExcludes().add("xForwardedFor_proto");
        filter.getExcludes().add("healthCheck");
        // TODO 后续优化，这是笨的解决方法
        filter.getExcludes().add("XForwardedFor");
        filter.getExcludes().add("XForwardedFor_SLBID");
        filter.getExcludes().add("XForwardedFor_SLBIP");
        filter.getExcludes().add("XForwardedFor_proto");
        JSONObject object = JSONObject.parseObject(listener.toJSONString());
        String result = JSONObject.toJSONString(object, filter);
        LbListenerModel model = JSONObject.parseObject(result, LbListenerModel.class);
        if (listener != null) {
            if ("sch".equalsIgnoreCase(listener.getString("scheduler"))
                || "tch".equalsIgnoreCase(listener.getString("scheduler"))) {
                model.setHash(listener.getString("scheduler"));
                model.setScheduler("ch");
            }
            model.setAclStatus("on".equalsIgnoreCase(listener.getString("aclStatus")));
            model.setStickySession("on".equalsIgnoreCase(listener.getString("stickySession")));
            model.setGzip("on".equalsIgnoreCase(listener.getString("gzip")));
            model.setXForwardedFor("on".equalsIgnoreCase(listener.getString("xForwardedFor")));
            model.setXForwardedFor_proto("on".equalsIgnoreCase(listener.getString("xForwardedFor_proto")));
            model.setXForwardedFor_SLBID("on".equalsIgnoreCase(listener.getString("xForwardedFor_SLBID")));
            model.setXForwardedFor_SLBIP("on".equalsIgnoreCase(listener.getString("xForwardedFor_SLBIP")));
            model.setHealthCheck("on".equalsIgnoreCase(listener.getString("healthCheck")));
            String description = listener.getString("description");
            model.setName(null == description ? protocol + "_" + listener.getInteger("listenerPort") : description);
            model.setPort(listener.getInteger("listenerPort"));
            model.setConnLimit(listener.getInteger("establishedTimeout"));
            model.setDefaultPoolUuid(listener.getString("vServerGroupId"));
            model.setLbUuid(loadBalancerId);
            model.setProtocol(protocol);
            model.setListenerUuid(loadBalancerId + "_" + protocol + ":" + listener.getInteger("listenerPort"));
        }
        return model;
    }
}
