package com.bocloud.cmp.provider.aliyunconvertor;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.DescribeNetworkInterfacesResponseBody;
import com.aliyun.ecs20140526.models.DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetSecurityGroupIds;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.aliyunconvertor
 * @ClassName: NetworkInterfaceConvertor
 * @Author: 胡文涛
 * @CreateTime: 2020/8/3 14:38
 * @Description: 弹性网卡转换
 */
public class NetworkInterfaceConvertor implements
		BeanConvertor<NetworkInterfaceModel, DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> {
	@Override
	public List<NetworkInterfaceModel> convertList(
			List<? extends DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> list) {
		List<NetworkInterfaceModel> models = new ArrayList<>();
		for (DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterface : list) {
			models.add(convertModel(networkInterface));
		}
		return models;
	}

	@Override
	public NetworkInterfaceModel convertModel(
			DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterface) {
		NetworkInterfaceModel model = new NetworkInterfaceModel();
		if (networkInterface != null) {
			if (StringUtils.isEmpty(networkInterface.getNetworkInterfaceName())) {
				model.setName(networkInterface.getNetworkInterfaceId());
			} else {
				model.setName(networkInterface.getNetworkInterfaceName());
			}
			model.setNetworkInterfaceUuid(networkInterface.getNetworkInterfaceId());
			model.setStatus(networkInterface.getStatus());
			model.setInterfaceType(networkInterface.getType());
			model.setNetworkUuid(networkInterface.getVpcId());
			model.setSubnetUuid(networkInterface.getVSwitchId());
			model.setZone(networkInterface.getZoneId());
			model.setPrimaryPrivateIPAddress(networkInterface.getPrivateIpAddress());// 主私网IP
			model.setMacAddress(networkInterface.getMacAddress());
			model.setInstanceId(networkInterface.getInstanceId());
			List<DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetPrivateIpSetsPrivateIpSet> privateIpSets = new ArrayList<>();
			if (networkInterface.getPrivateIpSets() != null
					&& !ListTool.isEmpty(networkInterface.getPrivateIpSets().getPrivateIpSet())) {
				privateIpSets = networkInterface.getPrivateIpSets().getPrivateIpSet();
			}
			List<String> privateIpAddresssList = new ArrayList<>();
			for (DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetPrivateIpSetsPrivateIpSet privateIpSet : privateIpSets) {
				if (privateIpSet.getPrimary()) {
					continue;
				}
				privateIpAddresssList.add(privateIpSet.getPrivateIpAddress());
			}
			model.setPrivateIpAddress(privateIpAddresssList);// 辅助私网IP
			if (null != networkInterface.getIpv6Sets()
					&& !ListTool.isEmpty(networkInterface.getIpv6Sets().getIpv6Set())) {
				model.setIpv6Address(networkInterface.getIpv6Sets().getIpv6Set().get(0).getIpv6Address());
			}
			DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetSecurityGroupIds securityGroupIds = networkInterface
					.getSecurityGroupIds();
			if (null != securityGroupIds) {
				model.setSecurityGroups(securityGroupIds.getSecurityGroupId());
			}
			model.setRemark(networkInterface.getDescription());
			model.setResourceGroupUuid(networkInterface.getResourceGroupId());
			model.setSecurityGroupUuid(JSONObject.toJSONString(networkInterface.getSecurityGroupIds()));

			List<DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetTagsTag> tags = new ArrayList<>();
			if (networkInterface.getTags() != null) {
				tags = networkInterface.getTags().getTag();
			}
			List<TagModel> tagModels = Lists.newArrayList();
			if (!ListTool.isEmpty(tags)) {
				for (DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSetTagsTag tag : tags) {
					TagModel tagModel = new TagModel();
					tagModel.setKey(tag.getTagKey());
					tagModel.setValue(tag.getTagValue());
					tagModel.setResourceId(networkInterface.getNetworkInterfaceId());
					tagModel.setResourceType(TagType.eni.name());
					tagModels.add(tagModel);
				}
				model.setTagModels(tagModels);
			}
		}
		return model;
	}
}
