package com.bocloud.cmp.provider.aliyunconvertor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.DescribeInstancesResponseBody;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 云主机数据转换器
 */
public class ServerConvertor implements BeanConvertor<ServerModel, DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> {

	@Override
	public List<ServerModel> convertList(List<? extends DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance> objects) {
		List<ServerModel> serverModels = new ArrayList<>();
		for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance object : objects) {
			serverModels.add(convertModel(object));
		}
		return serverModels;
	}

	@Override
	public ServerModel convertModel(DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstance instance) {
		ServerModel model = null;
		if (null != instance) {
			model = new ServerModel();
			model.setInstanceId(instance.getInstanceId());
			model.setResourceGroupUuid(instance.getResourceGroupId());
			model.setRegionId(instance.getRegionId());
			model.setName(instance.getInstanceName());
			model.setFlavorUuid(instance.getInstanceType());
			model.setInternetChargeType(instance.getInternetChargeType());
			model.setInstanceChargeType(instance.getInstanceChargeType());
			model.setInternetMaxBandwidthIn(instance.getInternetMaxBandwidthIn());
			model.setInternetMaxBandwidthOut(instance.getInternetMaxBandwidthOut());
			model.setKeypairName(instance.getKeyPairName());
			model.setCpu(instance.getCpu());
			model.setImageUuid(instance.getImageId());
			model.setMemory(BigDecimal.valueOf(instance.getMemory()).divide(BigDecimal.valueOf(1024), 2,
					BigDecimal.ROUND_HALF_UP));
            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceVpcAttributes vpcAttributes = instance.getVpcAttributes();
            if (null != vpcAttributes) {
            	if (vpcAttributes.getPrivateIpAddress() != null) {
            		model.setPrivateIps(JSONObject.toJSONString(vpcAttributes.getPrivateIpAddress().getIpAddress()));
				}
				model.setVpcUuid(vpcAttributes.getVpcId());
				model.setVSwitchId(vpcAttributes.getVSwitchId());
			}
            if (instance.getPublicIpAddress() != null) {
            	model.setPublicIps(JSONObject.toJSONString(instance.getPublicIpAddress().getIpAddress()));
			}
			model.setGroups(JSONObject.toJSONString(instance.getSecurityGroupIds()));
			model.setSecurityGroups(instance.getSecurityGroupIds().getSecurityGroupId());
			model.setGmtVmCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", instance.getCreationTime(), "Z"));
			model.setRemark(instance.getDescription());
			model.setStatus(instance.getStatus().toUpperCase());
			model.setOsName(instance.getOSName());
			model.setHostName(instance.getHostName());

            DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceEipAddress eipAddress = instance.getEipAddress();
            if (null != eipAddress) {
				String ipAddress = eipAddress.getIpAddress();
				if (!StringUtils.isEmpty(ipAddress)) {
					model.setPublicIps(JSON.toJSONString(new String[] { eipAddress.getIpAddress() }));
					model.setPublicIpAddressUuid(eipAddress.getAllocationId());
				}
			}

            List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceNetworkInterfacesNetworkInterface> networkInterfaces
					= new ArrayList<>();
            if (instance.getNetworkInterfaces() != null) {
            	networkInterfaces = instance.getNetworkInterfaces().getNetworkInterface();
			}
            if (null != networkInterfaces && !networkInterfaces.isEmpty()) {
				List<NetworkCardModel> networkCards = Lists.newArrayList();
				List<NetworkInterfaceModel> nicsModels = new ArrayList<NetworkInterfaceModel>();
				NetworkCardModel networkCard = null;
				NetworkInterfaceModel nicsModel = null;
				for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceNetworkInterfacesNetworkInterface networkInterface : networkInterfaces) {
					networkCard = new NetworkCardModel();
					networkCard.setMac(networkInterface.getMacAddress());
					networkCard.setIpAddress(networkInterface.getPrimaryIpAddress());
					networkCard.setNetworkcardUuid(networkInterface.getNetworkInterfaceId());
					networkCards.add(networkCard);

					nicsModel = new NetworkInterfaceModel();
					nicsModel.setNetworkInterfaceUuid(networkInterface.getNetworkInterfaceId());
					nicsModel.setPrimaryPrivateIPAddress(networkInterface.getPrimaryIpAddress());
					nicsModel.setMacAddress(networkInterface.getMacAddress());
					nicsModels.add(nicsModel);
				}
				model.setNetworkCards(networkCards);
				model.setNetworkInterfaces(nicsModels);
			}

			String osType = instance.getOSType();
			model.setOsCategory(osType);
			if (null != osType && "linux".equalsIgnoreCase(osType)) {
				model.setAccount("root");
			} else if ("windows".equalsIgnoreCase(osType)) {
				model.setAccount("administrator");
			}
			model.setZone(instance.getZoneId());
			model.setTerminationProtected(instance.getDeletionProtection());
            List<DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceTagsTag> tags
					= new ArrayList<>();
            if (instance.getTags() != null) {
            	tags = instance.getTags().getTag();
			}
            List<TagModel> tagModels = Lists.newArrayList();
			if (!ListTool.isEmpty(tags)) {
				for (DescribeInstancesResponseBody.DescribeInstancesResponseBodyInstancesInstanceTagsTag tag : tags) {
					TagModel tagModel = new TagModel();
					tagModel.setKey(tag.getTagKey());
					tagModel.setValue(tag.getTagValue());
					tagModel.setResourceId(instance.getInstanceId());
					tagModel.setResourceType(TagType.instance.name());
					tagModels.add(tagModel);
				}
				model.setTagModels(tagModels);
			}
		}
		return model;
	}

}
