package com.bocloud.cmp.provider.aliyunconvertor;

import com.aliyun.ecs20140526.models.DescribeSnapshotsResponseBody;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 快照数据转换器
 */
public class SnapshotConvertor implements BeanConvertor<SnapshotModel, DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> {

    @Override
    public List<SnapshotModel> convertList(List<? extends DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot> objects) {
        List<SnapshotModel> flavorModels = new ArrayList<>();
        for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot object : objects) {
            flavorModels.add(convertModel(object));
        }
        return flavorModels;
    }

    @Override
    public SnapshotModel convertModel(DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshot snapshot) {
        if (null == snapshot) {
            return null;
        }
        SnapshotModel model = new SnapshotModel();
        if (!StringUtils.isEmpty(snapshot.getSnapshotName())) {
            model.setName(snapshot.getSnapshotName());
        } else {
            model.setName(snapshot.getSnapshotId());
        }
        model.setSnapshotUuid(snapshot.getSnapshotId());
        model.setProgress(snapshot.getProgress());
        model.setVolumeUuid(snapshot.getSourceDiskId());
        model.setSize(new Integer(snapshot.getSourceDiskSize()));
        model.setRemark(snapshot.getDescription());
        String status = snapshot.getStatus();
        if("accomplished".equalsIgnoreCase(status)) {
        	model.setStatus("AVAILABLE");
        } else {
        	model.setStatus(status);
        }
        model.setSnapshotType(snapshot.getSnapshotType());
        model.setResourceGroupUuid(snapshot.getResourceGroupId());
        model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", snapshot.getCreationTime(), "Z"));


        //标签
        List<DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshotTagsTag> tags
                = new ArrayList<>();
        if (snapshot.getTags() != null) {
            tags = snapshot.getTags().getTag();
        }
        List<TagModel> tagModels = Lists.newArrayList();
        if (!ListTool.isEmpty(tags)) {
            for (DescribeSnapshotsResponseBody.DescribeSnapshotsResponseBodySnapshotsSnapshotTagsTag tag : tags) {
                TagModel tagModel = new TagModel();
                tagModel.setKey(tag.getTagKey());
                tagModel.setValue(tag.getTagValue());
                tagModel.setResourceId(snapshot.getSnapshotId());
                tagModel.setResourceType(TagType.snapshot.name());
                tagModels.add(tagModel);
            }
            model.setTagModels(tagModels);
        }

        return model;
    }

}
