package com.bocloud.cmp.provider.aliyunconvertor;

import com.aliyun.ecs20140526.models.DescribeDisksResponseBody;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.entity.enums.VolumeCategory;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 硬盘转换器
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> {

	@Override
	public List<VolumeModel> convertList(List<? extends DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk> objects) {
		List<VolumeModel> volumeModel = new ArrayList<>();
		for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk object : objects) {
			volumeModel.add(convertModel(object));
		}
		return volumeModel;
	}

	@Override
	public VolumeModel convertModel(DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDisk disk) {
		VolumeModel model = new VolumeModel();
		if (null != disk) {
			model.setVolumeUuid(disk.getDiskId());
			model.setZone(disk.getZoneId());
			if (!StringUtils.isEmpty(disk.getDiskName())) {
				model.setName(disk.getDiskName());
			} else {
				model.setName(disk.getDiskId());
			}
			model.setRemark(disk.getDescription());
			String type = disk.getType();
			if (null != type && "system".equalsIgnoreCase(type)) {
				model.setBootVolume(true);
			} else {
				model.setBootVolume(false);
			}
			model.setType(type);
			model.setResourceGroupUuid(disk.getResourceGroupId());
			model.setRegionId(disk.getRegionId());
			model.setCategory(convertDiskCategory(disk.getCategory()));
			model.setSize(disk.getSize());
			model.setImageUuid(disk.getImageId());
			model.setSnapshotUuid(disk.getSourceSnapshotId());
			model.setStatus(convertStatus(disk.getStatus()));
			model.setZone(disk.getZoneId());
			model.setInstanceId(disk.getInstanceId());
			model.setDeleteWithInstance(disk.getDeleteWithInstance());
			model.setDeviceName(disk.getDevice());
			model.setDeleteAutoSnapshot(disk.getDeleteAutoSnapshot());
			model.setEnableAutoSnapshot(disk.getEnableAutoSnapshot());
			model.setCreateTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getCreationTime(), "Z"));
			model.setAttachedTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getAttachedTime(), "Z"));
			model.setDetachedTime(DateTools.transfer(AliyunProvider.DATE_FORMAT, disk.getDetachedTime(), "Z"));
			model.setDiskChargeType(disk.getDiskChargeType());
			model.setEncrypted(disk.getEncrypted());
			model.setPerformanceLevel(disk.getPerformanceLevel());
            List<DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDiskTagsTag> tags
					= new ArrayList<>();
            if (disk.getTags() != null) {
            	tags = disk.getTags().getTag();
			}
            List<TagModel> tagModels = Lists.newArrayList();
			if (!ListTool.isEmpty(tags)) {
				for (DescribeDisksResponseBody.DescribeDisksResponseBodyDisksDiskTagsTag tag : tags) {
					TagModel tagModel = new TagModel();
					tagModel.setKey(tag.getTagKey());
					tagModel.setValue(tag.getTagValue());
					tagModel.setResourceId(disk.getDiskId());
					tagModel.setResourceType(TagType.disk.name());
					tagModels.add(tagModel);
				}
				model.setTagModels(tagModels);
			}
		}
		return model;
	}

	private String convertStatus(String status) {
		if (status.equalsIgnoreCase("In_use")) {
			return "IN_USE";
		}
		if (status.equalsIgnoreCase("Available")) {
			return "AVAILABLE";
		}
		return "EXCEPTION";
	}

	private String convertDiskCategory(String category) {
		if (category.equalsIgnoreCase(VolumeCategory.CLOUD.name())) {
			return VolumeCategory.CLOUD.name();
		}
		if (category.contains(VolumeCategory.EFFICIENCY.name().toLowerCase())) {
			return VolumeCategory.EFFICIENCY.name();
		}
		if (category.contains(VolumeCategory.ESSD.name().toLowerCase())) {
			return VolumeCategory.ESSD.name();
		}
		if (category.contains(VolumeCategory.SSD.name().toLowerCase())) {
			return VolumeCategory.SSD.name();
		}
		return VolumeCategory.CLOUD.name();
	}
}
