package com.bocloud.cmp.provider.aliyunconvertor;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.CidrBlockModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.DescribeVpcsResponseBody.DescribeVpcsResponseBodyVpcsVpc;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;

/**
 * 云主机数据转换器
 */
public class VpcConvertor implements BeanConvertor<VpcModel, DescribeVpcsResponseBodyVpcsVpc> {

	@Override
	public List<VpcModel> convertList(List<? extends DescribeVpcsResponseBodyVpcsVpc> objects) {
		List<VpcModel> vpcModels = new ArrayList<>();
		for (DescribeVpcsResponseBodyVpcsVpc object : objects) {
			vpcModels.add(convertModel(object));
		}
		return vpcModels;
	}

	@Override
	public VpcModel convertModel(DescribeVpcsResponseBodyVpcsVpc vpc) {
		VpcModel model = null;
		if (null != vpc) {
			model = new VpcModel();
			if (!StringUtils.isEmpty(vpc.getVpcName())) {
				model.setName(vpc.getVpcName());
			} else {
				model.setName(vpc.getVpcId());
			}
			String creationTime = vpc.getCreationTime();
			if (null != creationTime && !"".equals(creationTime)) {
				model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", creationTime, "Z"));
			}
			model.setRemark(vpc.getDescription());
			model.setDefaultVpc(vpc.getIsDefault());
			model.setRegionId(vpc.getRegionId());
			String status = vpc.getStatus();
			if (null != status && !"".equals(status)) {
				model.setStatus(status.toUpperCase());
			}
			model.setRouterId(vpc.getVRouterId());
			model.setVswitchIds(JSONObject.toJSONString(vpc.getVSwitchIds()));
			model.setVpcId(vpc.getVpcId());
			model.setUserCidrs(JSONObject.toJSONString(vpc.getUserCidrs()));
			List<CidrBlockModel> cidrBlockModels = Lists.newArrayList();
			List<String> cidrBlocks = Lists.newArrayList();
			CidrBlockModel primaryCidrBlockModel = new CidrBlockModel();
			primaryCidrBlockModel.setType("Primary");
			primaryCidrBlockModel.setTargetId(vpc.getVpcId());
			primaryCidrBlockModel.setCidrBlock(vpc.getCidrBlock());
			cidrBlockModels.add(primaryCidrBlockModel);
			cidrBlocks.add(vpc.getCidrBlock());
			model.setCidrBlockModels(cidrBlockModels);
			model.setCidr(vpc.getCidrBlock());
			model.setCidr(JSON.toJSONString(cidrBlocks));
		}
		return model;
	}

}
