package com.bocloud.cmp.provider.common;

import com.aliyun.bssopenapi20171214.models.QueryInstanceBillRequest;
import com.aliyun.bssopenapi20171214.models.QueryInstanceBillResponse;
import com.aliyun.bssopenapi20171214.models.QueryInstanceBillResponseBody;
import com.bocloud.cmp.model.AliCostModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.CostConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2021/3/5 9:40
 * @Description:阿里云账单
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunCostProvider extends AliyunProvider implements CostProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunCostProvider.class);
    private static final int PAGE_SIZE = 300;
    private BeanConvertor<AliCostModel, QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyDataItemsItem> convertor = new CostConvertor();

    public AliyunCostProvider(Butler butler) {
        super(butler);
    }

    /***
     * 查询指定日期账单
     * @param billingCycle
     * @param granularity
     * @param billingDate
     * @return
     */
    @Override
    public BocloudResult cost(String billingCycle, String granularity, String billingDate) {
        try {
            String periodTime = billingCycle;//默认是MONTHLY时，YYYY-MM
            List<AliCostModel> costModels = new ArrayList<>();
            QueryInstanceBillRequest request = new QueryInstanceBillRequest();
            request.setBillingCycle(billingCycle);
            //是否按照计费项维度拉取数据。
            // false。与费用中心-费用账单-账单明细-实例 账单一致。
            // true。与费用中心-费用账单-账单明细-计费项 账单一致。
            request.setIsBillingItem(false);
            //查询账单的颗粒度。取值：
            //MONTHLY：月。与费用中心-费用账单-账单明细-账期 账单一致。
            //DAILY：日。与费用中心-费用账单-账单明细-按天 账单一致。
            //选择DAILY需指定BillingDate。
            if ("DAILY".equalsIgnoreCase(granularity)) {
                periodTime = billingDate;
                request.setGranularity(granularity);
                request.setBillingDate(billingDate);
            }
            request.setPageNum(1);
            request.setPageSize(PAGE_SIZE);
            QueryInstanceBillResponse response = getBSSClient().queryInstanceBill(request);
            if (null == response || !response.getBody().getSuccess()) {
                return new BocloudResult(false, "查询实例账单失败");
            }
            QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyData data = response.getBody().getData();
            Integer totalCount = data.getTotalCount();
            List<QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyDataItemsItem> items = data.getItems().getItem();
            if (ListTool.isEmpty(items)) {
                return new BocloudResult(true, costModels, "暂无账单信息！");
            }
            for (QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyDataItemsItem item : items) {
                AliCostModel model = convertor.convertModel(item);
                model.setPeriod(granularity);
                model.setPeriodTime(periodTime);
                costModels.add(model);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, costModels, "查询实例账单成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNum(i);
                request.setPageSize(PAGE_SIZE);
                response = getBSSClient().queryInstanceBill(request);
                data = response.getBody().data;
                items = data.getItems().getItem();
                for (QueryInstanceBillResponseBody.QueryInstanceBillResponseBodyDataItemsItem item : items) {
                    AliCostModel model = convertor.convertModel(item);
                    model.setPeriod(granularity);
                    model.setPeriodTime(periodTime);
                    costModels.add(model);
                }
            }
            return new BocloudResult(true, costModels, "查询实例账单成功");
        } catch (Exception e) {
            logger.error("查询实例账单失败:" + e);
            return new BocloudResult(false, "查询实例账单失败", e.getMessage());
        }
    }
}
