package com.bocloud.cmp.provider.common;

import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.FlavorsConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 阿里云云主机实例类型驱动
 *
 * @author : guopeng create at: 2018/11/15 1:52 PM
 * @description:
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunFlavorProvider extends AliyunProvider implements FlavorProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunFlavorProvider.class);

    private BeanConvertor<FlavorModel, DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType> convertor = new FlavorsConvertor();

    public AliyunFlavorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            DescribeInstanceTypesRequest describeInstanceTypesRequest = new DescribeInstanceTypesRequest();
            DescribeInstanceTypesResponse describeInstanceTypesResponse = getECSClient().describeInstanceTypes(describeInstanceTypesRequest);
            List<DescribeInstanceTypesResponseBody.DescribeInstanceTypesResponseBodyInstanceTypesInstanceType> instancesTypes = describeInstanceTypesResponse.getBody().getInstanceTypes().getInstanceType();
            List<FlavorModel> convertList = convertor.convertList(instancesTypes);
            for (FlavorModel flavorModel : convertList) {
                flavorModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "同步成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "同步失败");
        }
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        return null;
    }

    @Override
    public BocloudResult remove(String flavorId) {
        return null;
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        return null;
    }

    @Override
    public BocloudResult listUsedFlavors(String instanceId) {
        try {
            List<String> flavorUuids = new ArrayList<String>();
            DescribeResourcesModificationRequest request = new DescribeResourcesModificationRequest();
            request.setResourceId(instanceId);
            request.setRegionId(this.getRegionId());
            request.setDestinationResource("InstanceType");
            DescribeResourcesModificationResponse acsResponse = getECSClient().describeResourcesModification(request);
            List<DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZone> availableZones = acsResponse.getBody().getAvailableZones().getAvailableZone();
            for (DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZone availableZone : availableZones) {
                if ("Available".equalsIgnoreCase(availableZone.getStatus())) {
                    List<DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResource> availableResources = availableZone.getAvailableResources().getAvailableResource();
                    for (DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResource availableResource : availableResources) {
                        List<DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResourceSupportedResourcesSupportedResource> supportedResources = availableResource.getSupportedResources().getSupportedResource();
                        for (DescribeResourcesModificationResponseBody.DescribeResourcesModificationResponseBodyAvailableZonesAvailableZoneAvailableResourcesAvailableResourceSupportedResourcesSupportedResource supportedResource : supportedResources) {
                            if ("Available".equalsIgnoreCase(supportedResource.getStatus())) {
                                flavorUuids.add(supportedResource.getValue());
                            }
                        }
                    }
                }
            }
            return new BocloudResult(true, flavorUuids, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

}
