package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.slb20140515.models.DescribeZonesRequest;
import com.aliyun.slb20140515.models.DescribeZonesResponse;
import com.aliyun.slb20140515.models.DescribeZonesResponseBody;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ZoneModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 可用区
 *
 * @author lyy
 * @version 1.0
 * @date 2021/1/7 15:38
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunZoneProvider extends AliyunProvider implements ZoneProvider {

    public AliyunZoneProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AliyunZoneProvider.class);

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult query(String type) {
        try {
            switch (type) {
                case "SLB":
                    DescribeZonesRequest request = new DescribeZonesRequest();
                    request.setRegionId(this.getRegionId());
                    DescribeZonesResponse response = getSLBClient().describeZones(request);
                    if (null == response) {
                        return new BocloudResult(false, "获取当前地域下可用域失败");
                    }
                    List<DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone> zones = response.getBody().getZones().getZone();
                    List<ZoneModel> zoneModels = Lists.newArrayList();
                    for (DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZone zone : zones) {
                        ZoneModel zoneModel = new ZoneModel();
                        zoneModel.setName(zone.getLocalName());
                        zoneModel.setZoneId(zone.getZoneId());
                        List<JSONObject> slaves = Lists.newArrayList();
                        List<DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZoneSlaveZonesSlaveZone> slaveZones = zone.getSlaveZones().getSlaveZone();
                        for (DescribeZonesResponseBody.DescribeZonesResponseBodyZonesZoneSlaveZonesSlaveZone slaveZone : slaveZones) {
                            JSONObject slave = new JSONObject();
                            slave.put("zoneId", slaveZone.getZoneId());
                            slave.put("localName", slaveZone.getLocalName());
                            slaves.add(slave);
                        }
                        zoneModel.setSlaveZones(slaves);
                        zoneModels.add(zoneModel);
                    }
                    return new BocloudResult(true, zoneModels, "查询成功");
                default:
                    return new BocloudResult(false, "暂不不支持的类型");
            }
        } catch (Exception e) {
            logger.error("获取当前地域下可用域失败:", e);
            return new BocloudResult(false, "获取当前地域下可用域失败");
        }
    }
}
