package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.aliyun.ecs20140526.models.CancelCopyImageRequest;
import com.aliyun.ecs20140526.models.CancelCopyImageResponse;
import com.aliyun.ecs20140526.models.CopyImageRequest;
import com.aliyun.ecs20140526.models.CopyImageResponse;
import com.aliyun.ecs20140526.models.CreateImageRequest;
import com.aliyun.ecs20140526.models.CreateImageRequest.CreateImageRequestDiskDeviceMapping;
import com.aliyun.ecs20140526.models.CreateImageResponse;
import com.aliyun.ecs20140526.models.DeleteImageRequest;
import com.aliyun.ecs20140526.models.DeleteImageResponse;
import com.aliyun.ecs20140526.models.DescribeImagesRequest;
import com.aliyun.ecs20140526.models.DescribeImagesResponse;
import com.aliyun.ecs20140526.models.DescribeImagesResponseBody;
import com.aliyun.ecs20140526.models.ModifyImageAttributeRequest;
import com.aliyun.ecs20140526.models.ModifyImageAttributeResponse;
import com.bocloud.cmp.entity.enums.TagType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;

/**
 * @author : guopeng create at: 2018/11/15 11:15 AM
 * @description:
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunImageProvider extends AliyunProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunImageProvider.class);
    private static final int PAGE_SIZE = 100;
    private Butler butler;

    public AliyunImageProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        List<ImageModel> models = Lists.newArrayList();
        DescribeImagesRequest request = new DescribeImagesRequest();
        request.setRegionId(this.getRegionId());
        request.setPageNumber(1);
        request.setPageSize(PAGE_SIZE);
        try {
            DescribeImagesResponse response = this.getECSClient().describeImages(request);
            if (null == response) {
                return new BocloudResult(false, "镜像列表查询失败");
            } else if (null == response.getBody().getImages()) {
                return new BocloudResult(false, "镜像列表查询失败");
            }
            Integer totalCount = response.getBody().getTotalCount();
            if (null == totalCount || totalCount == 0) {
                return new BocloudResult(true, "");
            }
            List<DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImage> images =
                response.getBody().getImages().getImage();
            for (DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImage image : images) {
                String imageName = image.getImageName();
                // .raw镜像是定制镜像 对其他资源有严格要求
/*                if (!imageName.endsWith(".raw") && !imageName.contains("amd") && !imageName.contains("_uefi")
                    && !imageName.contains("_tpm")) { */
                if (StringUtils.isNotEmpty(imageName)) {
                    ImageModel convert = this.convert(image);
                    convert.setRegionId(this.getRegionId());
                    models.add(convert);
                }
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "镜像列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getECSClient().describeImages(request);
                images = response.getBody().getImages().getImage();
                for (DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImage image : images) {
                    String imageName = image.getImageName();
//                    if (!imageName.endsWith(".raw")) {
                    if (StringUtils.isNotEmpty(imageName)) {
                        ImageModel convert = this.convert(image);
                        convert.setRegionId(this.getRegionId());
                        models.add(convert);
                    }
                }
            }
            return new BocloudResult(true, models, "镜像列表查询成功");
        } catch (Exception e) {
            logger.error("镜像列表查询失败：", e);
            return new BocloudResult(false, "镜像列表查询失败: " + e);
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
        DescribeImagesRequest request = new DescribeImagesRequest();
        request.setImageId(imageId);
        request.setRegionId(this.getRegionId());
        try {
            DescribeImagesResponse response = this.getECSClient().describeImages(request);
            List<DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImage> images =
                response.getBody().getImages().getImage();
            ImageModel model = convert(images.get(0));
            if (null != model) {
                model.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, model, "镜像详情查询成功");
        } catch (Exception e) {
            logger.error("镜像详情查询失败：", e);
            return new BocloudResult(false, "镜像详情查询失败");
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        DeleteImageRequest request = new DeleteImageRequest();
        request.setImageId(imageId);
        request.setRegionId(this.getRegionId());
        try {
            DeleteImageResponse response = this.getECSClient().deleteImage(request);
            return new BocloudResult(true, response.getBody().getRequestId(), "镜像删除成功");
        } catch (Exception e) {
            logger.error("镜像删除失败：", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        ModifyImageAttributeRequest request = new ModifyImageAttributeRequest();
        request.setImageId(image.getImageUuid());
        request.setRegionId(this.getRegionId());
        if (null != image.getName() && !"".equals(image.getName())) {
            request.setImageName(image.getName());
        }
        if (null != image.getRemark() && !"".equals(image.getRemark())) {
            request.setDescription(image.getRemark());
        }
        try {
            ModifyImageAttributeResponse response = this.getECSClient().modifyImageAttribute(request);
            return new BocloudResult(true, this.detail(image.getImageUuid()).getData(), "镜像修改成功");
        } catch (Exception e) {
            logger.error("镜像修改失败：" + e);
            return new BocloudResult(false, "镜像修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        String snapshotUuid = null;
        AliyunSnapshotProvider snapshotProvider = new AliyunSnapshotProvider(butler);
        try {
            // 根据系统盘快照创建系统盘镜像
            SnapshotModel snapshotModel = new SnapshotModel();
            snapshotModel.setVolumeUuid(imageModel.getInstanceId());
            snapshotModel.setName("snapshot_" + System.nanoTime());
            BocloudResult bocloudResult = snapshotProvider.create(snapshotModel);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            snapshotModel = (SnapshotModel)bocloudResult.getData();
            snapshotUuid = snapshotModel.getSnapshotUuid();
            CreateImageRequest request = new CreateImageRequest();
            List<CreateImageRequestDiskDeviceMapping> diskDeviceMappings = new ArrayList<>();
            CreateImageRequestDiskDeviceMapping diskDeviceMapping = new CreateImageRequestDiskDeviceMapping();
            diskDeviceMapping.setSnapshotId(snapshotUuid);
            diskDeviceMappings.add(diskDeviceMapping);
            request.setDiskDeviceMapping(diskDeviceMappings);

            request.setImageName(imageModel.getName());
            request.setDescription(imageModel.getRemark());
            request.setRegionId(this.getRegionId());
            if (StringUtils.isNoneBlank(imageModel.getResourceGroupUuid())) {
                request.setResourceGroupId(imageModel.getResourceGroupUuid());
            }
            CreateImageResponse response = this.getECSClient().createImage(request);
            String imageId = response.getBody().getImageId();
            logger.info("create task success : " + imageId);
            return new BocloudResult(true, this.progress(imageId).getData(), "镜像创建成功");
        } catch (Exception e) {
            logger.error("镜像创建失败：" + e);
            if (!StringUtils.isEmpty(snapshotUuid)) {
                snapshotProvider.remove(snapshotUuid);
            }
            return new BocloudResult(false, "镜像创建失败：" + e.getMessage());
        }
    }

    private BocloudResult progress(String imageId) {
        BocloudResult result = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                // 通过镜像id查询不出来具体镜像，通过list过滤
                Thread.sleep(5000);
                result = this.list();
                if (result.isFailed()) {
                    return result;
                }
                List<ImageModel> models = (List<ImageModel>)result.getData();
                ImageModel imgModel = null;
                if (!ListTool.isEmpty(models)) {
                    for (ImageModel imageModel : models) {
                        if (imageId.equalsIgnoreCase(imageModel.getImageUuid())) {
                            imgModel = imageModel;
                            break;
                        }
                    }
                    if (null != imgModel && "ACTIVE".equalsIgnoreCase(imgModel.getStatus())) {
                        flag = false;
                        result = new BocloudResult<>(true, imgModel, "创建成功");
                        break;
                    }
                }
                Long endTime = System.currentTimeMillis();
                // 等待600s
                if (endTime - startTime > 20 * 60 * 1000) {
                    flag = false;
                    result = new BocloudResult<>(true, imgModel, "创建成功");
                    break;
                }
            }
            return result;
        } catch (Exception e) {
            logger.error("镜像状态查询失败： " + e);
            return new BocloudResult(false, "镜像状态查询失败：" + e.getMessage());
        }
    }

    public ImageModel convert(DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImage image) {
        if (null == image) {
            return null;
        }
        ImageModel model = new ImageModel();
        if (StringUtils.isEmpty(image.getImageName())) {
            model.setName(image.getImageId());
        } else {
            model.setName(image.getImageName());
        }
        model.setImageUuid(image.getImageId());
        model.setArchitecture(image.getArchitecture());
        model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", image.getCreationTime(), "Z"));
        model.setKernelVersion(image.getImageVersion());
        model.setName(image.getImageName());
        String[] os = image.getOSName().split(" ");
        if (os.length >= 2) {
            String osName = os[0];
            model.setOsName(osName);
            model.setOsType(osName);
            String osVersion = image.getOSName().substring(osName.length()).trim();
            model.setOsVersion(osVersion);
        } else {
            model.setOsName(image.getOSName());
            model.setOsType(image.getOSName());
            model.setOsVersion(image.getOSName());
        }
        String osType = image.getOSType();
        if (null != osType && "Windows".equalsIgnoreCase(osType)) {
            model.setOsCategory("Windows");
        } else {
            model.setOsCategory("Linux");
        }
        model.setRemark(image.getDescription());
        model.setSize(Double.valueOf(image.getSize()));
        model.setMinDisk(Long.parseLong(image.getSize() + ""));
        model.setStatus(convertStatus(image.getStatus()));
        model.setResourceGroupUuid(image.getResourceGroupId());
        String imageOwnerAlias = image.getImageOwnerAlias();
        if (null != imageOwnerAlias) {
            model.setImageType(convertImageType(imageOwnerAlias));
        }

        // 标签
        List<DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImageTagsTag> tags = image.getTags().getTag();
        List<TagModel> tagModels = Lists.newArrayList();
        if (!ListTool.isEmpty(tags)) {
            for (DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImageTagsTag tag : tags) {
                TagModel tagModel = new TagModel();
                tagModel.setKey(tag.getTagKey());
                tagModel.setValue(tag.getTagValue());
                tagModel.setResourceId(image.getImageId());
                tagModel.setResourceType(TagType.image.name());
                tagModels.add(tagModel);
            }
            model.setTagModels(tagModels);
        }

        List<
            DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImageDiskDeviceMappingsDiskDeviceMapping> diskDeviceMappings =
                image.getDiskDeviceMappings().getDiskDeviceMapping();
        if (null != diskDeviceMappings && diskDeviceMappings.size() > 0) {
            List<VolumeModel> volModels = new ArrayList<>();
            VolumeModel volModel = null;
            for (DescribeImagesResponseBody.DescribeImagesResponseBodyImagesImageDiskDeviceMappingsDiskDeviceMapping diskDeviceMapping : diskDeviceMappings) {
                volModel = new VolumeModel();
                volModel.setDeviceName(diskDeviceMapping.getDevice());
                volModel.setSize(Integer.parseInt(diskDeviceMapping.getSize()));
                volModel.setSourceSnapUuid(diskDeviceMapping.getSnapshotId());
                volModel.setType(diskDeviceMapping.getType());
                volModels.add(volModel);
            }
            model.setDiskDeviceMappings(volModels);
        }
        return model;
    }

    private String convertImageType(String imageType) {
        // system：系统公共镜像
        // self：用户的自定义镜像
        // others：其他用户的公开镜像
        // marketplace：镜像市场镜像
        if ("system".equalsIgnoreCase(imageType)) {
            return "PUBLIC";
        }
        if ("self".equalsIgnoreCase(imageType)) {
            return "PRIVATE";
        }
        return imageType == null ? null : imageType.toUpperCase();
    }

    private String convertStatus(String oldStatus) {
        String status = "EXCEPTION";
        if ("Available".equalsIgnoreCase(oldStatus)) {
            status = "ACTIVE";
        } else if ("Creating".equalsIgnoreCase(oldStatus)) {
            status = "SAVING";
        }
        return status;
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult copy(ImageModel model) {
        try {
            CopyImageRequest request = new CopyImageRequest();
            request.setImageId(model.getSourceImageId());
            request.setRegionId(model.getSourceRegion());
            request.setDestinationRegionId(model.getRegionId());
            request.setDestinationImageName(model.getName());
            request.setDestinationDescription(model.getRemark());
            CopyImageResponse response = this.getECSClient().copyImage(request);
            if (null == response) {
                return new BocloudResult(false, "镜像复制失败");
            } else if (null == response.getBody().getImageId()) {
                return new BocloudResult(false, "镜像复制失败");
            }
            return new BocloudResult(true, this.progress(response.getBody().getImageId()).getData(), "镜像复制成功");
        } catch (Exception e) {
            logger.error("镜像复制失败：" + e);
            return new BocloudResult(false, "镜像复制失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult cancelCopy(String imageId) {
        try {
            CancelCopyImageRequest request = new CancelCopyImageRequest();
            request.setImageId(imageId);
            request.setRegionId(this.getRegionId());
            CancelCopyImageResponse response = this.getECSClient().cancelCopyImage(request);
            if (null == response) {
                return new BocloudResult(false, "取消镜像复制失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "取消镜像复制成功");
        } catch (Exception e) {
            logger.error("取消镜像复制失败：" + e);
            return new BocloudResult(false, "取消镜像复制失败：" + e.getMessage());
        }
    }
}
