package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ServerTemplateModel;
import com.bocloud.cmp.model.ServerTemplateVersionModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.ServerTemplateConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.ServerTemplateVersionConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 阿里云 启动模板
 *
 * @author ymh
 * @时间 2019年3月8日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunServerTemplateProvider extends AliyunProvider implements ServerTemplateProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunServerTemplateProvider.class);

    private static final int PAGE_SIZE = 50;

    private BeanConvertor<ServerTemplateModel, DescribeLaunchTemplatesResponseBody.DescribeLaunchTemplatesResponseBodyLaunchTemplateSetsLaunchTemplateSet> templateConvertor = new ServerTemplateConvertor();

    private BeanConvertor<ServerTemplateVersionModel, DescribeLaunchTemplateVersionsResponseBody.DescribeLaunchTemplateVersionsResponseBodyLaunchTemplateVersionSetsLaunchTemplateVersionSet> templateVersionConvertor = new ServerTemplateVersionConvertor();

    public AliyunServerTemplateProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(ServerTemplateModel serverTemplateModel) {
        DescribeLaunchTemplatesRequest describe = new DescribeLaunchTemplatesRequest();
        describe.setPageNumber(1);
        describe.setPageSize(PAGE_SIZE);
        if (null != serverTemplateModel) {
            String templateUuid = serverTemplateModel.getTemplateUuid();
            List<String> launchTemplateIds = new ArrayList<>();
            launchTemplateIds.add(templateUuid);
            describe.setLaunchTemplateId(launchTemplateIds);
        }
        List<ServerTemplateModel> models = Lists.newArrayList();
        try {
            DescribeLaunchTemplatesResponse acsResponse = this.getECSClient().describeLaunchTemplates(describe);
            List<DescribeLaunchTemplatesResponseBody.DescribeLaunchTemplatesResponseBodyLaunchTemplateSetsLaunchTemplateSet> launchTemplateSets = acsResponse.getBody().getLaunchTemplateSets().getLaunchTemplateSet();
            int totalCount = acsResponse.getBody().getTotalCount();
            if (totalCount == 0) {
                return new BocloudResult(true, models, "empty template");
            }
            for (DescribeLaunchTemplatesResponseBody.DescribeLaunchTemplatesResponseBodyLaunchTemplateSetsLaunchTemplateSet launchTemplateSet : launchTemplateSets) {
                ServerTemplateModel convertModel = templateConvertor.convertModel(launchTemplateSet);
                convertModel.setRegion(this.getRegionId());
                models.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                describe.setPageNumber(i);
                describe.setPageSize(PAGE_SIZE);
                acsResponse = this.getECSClient().describeLaunchTemplates(describe);
                launchTemplateSets = acsResponse.getBody().getLaunchTemplateSets().getLaunchTemplateSet();
                for (DescribeLaunchTemplatesResponseBody.DescribeLaunchTemplatesResponseBodyLaunchTemplateSetsLaunchTemplateSet launchTemplateSet : launchTemplateSets) {
                    ServerTemplateModel convertModel = templateConvertor.convertModel(launchTemplateSet);
                    convertModel.setRegion(this.getRegionId());
                    models.add(convertModel);
                }
            }
            return new BocloudResult(true, models, "查询成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询模板失败");
        }

    }

    public BocloudResult listTemplateVersion(ServerTemplateVersionModel serverTemplateVersionModel) {
        DescribeLaunchTemplateVersionsRequest request = new DescribeLaunchTemplateVersionsRequest();
        request.setPageNumber(1);
        request.setPageSize(PAGE_SIZE);
        // 根据ID查询
        if (null != serverTemplateVersionModel && null != serverTemplateVersionModel.getTemplateUuid()) {
            request.setLaunchTemplateId(serverTemplateVersionModel.getTemplateUuid());
        }
        // 根据版本编号查询
        if (null != serverTemplateVersionModel && null != serverTemplateVersionModel.getVersionNumber()) {
            List<Integer> launchTemplateVersions = new ArrayList<>();
            launchTemplateVersions.add(Math.toIntExact(serverTemplateVersionModel.getVersionNumber()));
            request.setLaunchTemplateVersion(launchTemplateVersions);
        }
        List<ServerTemplateVersionModel> models = Lists.newArrayList();
        try {
            DescribeLaunchTemplateVersionsResponse acsResponse = this.getECSClient().describeLaunchTemplateVersions(request);
            List<DescribeLaunchTemplateVersionsResponseBody.DescribeLaunchTemplateVersionsResponseBodyLaunchTemplateVersionSetsLaunchTemplateVersionSet> launchTemplateVersionSets = acsResponse.getBody().getLaunchTemplateVersionSets().getLaunchTemplateVersionSet();
            int totalCount = acsResponse.getBody().getTotalCount();
            if (totalCount == 0) {
                return new BocloudResult(true, models, "empty template");
            }
            for (DescribeLaunchTemplateVersionsResponseBody.DescribeLaunchTemplateVersionsResponseBodyLaunchTemplateVersionSetsLaunchTemplateVersionSet launchTemplateVersionSet : launchTemplateVersionSets) {
                ServerTemplateVersionModel convertModel = templateVersionConvertor
                        .convertModel(launchTemplateVersionSet);
                convertModel.setRegion(this.getRegionId());
                models.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                launchTemplateVersionSets = acsResponse.getBody().getLaunchTemplateVersionSets().getLaunchTemplateVersionSet();
                for (DescribeLaunchTemplateVersionsResponseBody.DescribeLaunchTemplateVersionsResponseBodyLaunchTemplateVersionSetsLaunchTemplateVersionSet launchTemplateVersionSet : launchTemplateVersionSets) {
                    ServerTemplateVersionModel convertModel = templateVersionConvertor
                            .convertModel(launchTemplateVersionSet);
                    convertModel.setRegion(this.getRegionId());
                    models.add(convertModel);
                }
            }
            return new BocloudResult(true, models, "查询模板成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询失败");
        }

    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult create(ServerTemplateModel serverTemplateModel) {
        try {
            CreateLaunchTemplateRequest request = new CreateLaunchTemplateRequest();
            request.setLaunchTemplateName(serverTemplateModel.getName());
            request.setDescription(serverTemplateModel.getRemark());
            request.setRegionId(this.getRegionId());

            List<CreateLaunchTemplateRequest.CreateLaunchTemplateRequestDataDisk> dataDisks = new ArrayList<>();
            List<VolumeModel> volumeModels = serverTemplateModel.getVolumeModels();
            if (null != volumeModels && volumeModels.size() > 0) {
                CreateLaunchTemplateRequest.CreateLaunchTemplateRequestDataDisk dataDisk = null;
                for (VolumeModel volumeModel : volumeModels) {
                    dataDisk = new CreateLaunchTemplateRequest.CreateLaunchTemplateRequestDataDisk();
                    dataDisk.setCategory(volumeModel.getCategory());
                    dataDisk.setSize(volumeModel.getSize());
                    dataDisk.setDeleteWithInstance(volumeModel.getDeleteWithInstance());
                    dataDisk.setDiskName(volumeModel.getName());
                    dataDisk.setDescription(volumeModel.getRemark());
                    dataDisk.setEncrypted("false");
                    dataDisk.setSnapshotId(volumeModel.getSourceSnapshotId());
                    dataDisks.add(dataDisk);
                }
            }
            request.setDataDisk(dataDisks);

            // 默认版本信息
            ServerTemplateVersionModel serverTemplateVersionModel = serverTemplateModel.getServerTemplateVersionModel();
            if (null == serverTemplateVersionModel) {
                return new BocloudResult(false, "参数错误！");
            }
            request.setHostName(serverTemplateVersionModel.getHostName());
            request.setImageId(serverTemplateVersionModel.getImageUuid());
            request.setImageOwnerAlias(serverTemplateVersionModel.getImageOwnerAlias());
            request.setInstanceChargeType(serverTemplateVersionModel.getInstanceChargeType());
            request.setInstanceName(serverTemplateVersionModel.getInstanceName());
            request.setInstanceType(serverTemplateVersionModel.getFlavorUuid());
            request.setInternetChargeType(serverTemplateVersionModel.getInternetChargeType());
            request.setInternetMaxBandwidthOut(serverTemplateVersionModel.getInternetMaxBandwidthOut());
            request.setIoOptimized(serverTemplateVersionModel.getIoOptimized());
            request.setKeyPairName(serverTemplateVersionModel.getKeypairName());
            request.setSecurityGroupId(serverTemplateVersionModel.getSecurityGroupId());

            CreateLaunchTemplateRequest.CreateLaunchTemplateRequestSystemDisk createLaunchTemplateRequestSystemDisk =
                    new CreateLaunchTemplateRequest.CreateLaunchTemplateRequestSystemDisk();
            createLaunchTemplateRequestSystemDisk.setCategory(serverTemplateVersionModel.getSystemDiskCategory());
            createLaunchTemplateRequestSystemDisk.setSize(serverTemplateVersionModel.getSystemDiskSize());
            request.setSystemDisk(createLaunchTemplateRequestSystemDisk);
            request.setVpcId(serverTemplateVersionModel.getVpcUuid());
            request.setVSwitchId(serverTemplateVersionModel.getvSwitchUuid());
            // 实例所在 可以区
            request.setZoneId(serverTemplateVersionModel.getZone());
            request.setVersionDescription(serverTemplateVersionModel.getVersionDescription());
            // 网卡
            List<CreateLaunchTemplateRequest.CreateLaunchTemplateRequestNetworkInterface> networkInterfaces = new ArrayList<>();
            String networkInfo = serverTemplateVersionModel.getNetworkInterfaces();
            if (null != networkInfo && !"".equals(networkInfo)) {
                List<JSONObject> parseArray = JSONObject.parseArray(networkInfo, JSONObject.class);
                CreateLaunchTemplateRequest.CreateLaunchTemplateRequestNetworkInterface netInterface = null;
                for (JSONObject jsonObject : parseArray) {
                    netInterface = new CreateLaunchTemplateRequest.CreateLaunchTemplateRequestNetworkInterface();
                    netInterface.setVSwitchId(jsonObject.getString("vSwitchId"));
                    netInterface.setNetworkInterfaceName(jsonObject.getString("networkInterfaceName"));
                    netInterface.setPrimaryIpAddress(jsonObject.getString("primaryIpAddress"));
                    netInterface.setDescription(jsonObject.getString("description"));
                    netInterface.setSecurityGroupId(jsonObject.getString("securityGroupId"));
                    networkInterfaces.add(netInterface);
                }
            }
            request.setNetworkInterface(networkInterfaces);

            CreateLaunchTemplateResponse acsResponse = this.getECSClient().createLaunchTemplate(request);
            String launchTemplateId = acsResponse.getBody().getLaunchTemplateId();

            ServerTemplateModel templateModel = new ServerTemplateModel(launchTemplateId);
            BocloudResult bocloudResult = list(templateModel);
            List<ServerTemplateModel> models = (List<ServerTemplateModel>) bocloudResult.getData();
            if (null != models && models.size() > 0) {
                templateModel = models.get(0);
            }
            BocloudResult versionResult = listTemplateVersion(new ServerTemplateVersionModel(launchTemplateId));
            List<ServerTemplateVersionModel> versionModels = (List<ServerTemplateVersionModel>) versionResult.getData();
            templateModel.setTemplateVersionModels(versionModels);
            templateModel.setRegion(this.getRegionId());
            return new BocloudResult(true, templateModel, "创建成功");
        } catch (Exception e) {
            logger.error("create server template error:", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult createTemplateVersion(ServerTemplateModel serverTemplateModel) {
        try {
            CreateLaunchTemplateVersionRequest request = new CreateLaunchTemplateVersionRequest();
            request.setLaunchTemplateId(serverTemplateModel.getTemplateUuid());

            List<CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestDataDisk> dataDisks = new ArrayList<>();
            List<VolumeModel> volumeModels = serverTemplateModel.getVolumeModels();
            if (null != volumeModels && volumeModels.size() > 0) {
                CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestDataDisk dataDisk = null;
                for (VolumeModel volumeModel : volumeModels) {
                    dataDisk = new CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestDataDisk();
                    dataDisk.setCategory(volumeModel.getCategory());
                    dataDisk.setSize(volumeModel.getSize());
                    dataDisk.setDeleteWithInstance(volumeModel.getDeleteWithInstance());
                    dataDisk.setDiskName(volumeModel.getName());
                    dataDisk.setDescription(volumeModel.getRemark());
                    dataDisk.setEncrypted("false");
                    dataDisk.setSnapshotId(volumeModel.getSourceSnapshotId());
                    dataDisks.add(dataDisk);
                }
            }
            request.setDataDisk(dataDisks);

            // 默认版本信息
            ServerTemplateVersionModel serverTemplateVersionModel = serverTemplateModel.getServerTemplateVersionModel();
            if (null == serverTemplateVersionModel) {
                return new BocloudResult(false, "参数错误！");
            }
            request.setHostName(serverTemplateVersionModel.getHostName());
            request.setImageId(serverTemplateVersionModel.getImageUuid());
            request.setImageOwnerAlias(serverTemplateVersionModel.getImageOwnerAlias());
            request.setInstanceChargeType(serverTemplateVersionModel.getInstanceChargeType());
            request.setInstanceName(serverTemplateVersionModel.getInstanceName());
            request.setInstanceType(serverTemplateVersionModel.getFlavorUuid());
            request.setInternetChargeType(serverTemplateVersionModel.getInternetChargeType());
            request.setInternetMaxBandwidthOut(serverTemplateVersionModel.getInternetMaxBandwidthOut());
            request.setIoOptimized(serverTemplateVersionModel.getIoOptimized());
            request.setKeyPairName(serverTemplateVersionModel.getKeypairName());
            request.setSecurityGroupId(serverTemplateVersionModel.getSecurityGroupId());
            CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestSystemDisk systemDisk = new CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestSystemDisk();
            systemDisk.setCategory(serverTemplateVersionModel.getSystemDiskCategory());
            systemDisk.setSize(serverTemplateVersionModel.getSystemDiskSize());
            request.setSystemDisk(systemDisk);

            request.setVpcId(serverTemplateVersionModel.getVpcUuid());
            request.setVSwitchId(serverTemplateVersionModel.getvSwitchUuid());
            // 实例所在 可以区
            request.setZoneId(serverTemplateVersionModel.getZone());
            request.setVersionDescription(serverTemplateVersionModel.getVersionDescription());
            // 网卡
            List<CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestNetworkInterface> networkInterfaces = new ArrayList<>();
            String networkInfo = serverTemplateVersionModel.getNetworkInterfaces();
            if (null != networkInfo && !"".equals(networkInfo)) {
                List<JSONObject> parseArray = JSONObject.parseArray(networkInfo, JSONObject.class);
                CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestNetworkInterface netInterface = null;
                for (JSONObject jsonObject : parseArray) {
                    netInterface = new CreateLaunchTemplateVersionRequest.CreateLaunchTemplateVersionRequestNetworkInterface();
                    netInterface.setVSwitchId(jsonObject.getString("vSwitchId"));
                    netInterface.setNetworkInterfaceName(jsonObject.getString("networkInterfaceName"));
                    netInterface.setPrimaryIpAddress(jsonObject.getString("primaryIpAddress"));
                    netInterface.setDescription(jsonObject.getString("description"));
                    netInterface.setSecurityGroupId(jsonObject.getString("securityGroupId"));
                    networkInterfaces.add(netInterface);
                }
            }
            request.setNetworkInterface(networkInterfaces);

            CreateLaunchTemplateVersionResponse acsResponse = this.getECSClient().createLaunchTemplateVersion(request);
            Long launchTemplateVersionNumber = acsResponse.getBody().getLaunchTemplateVersionNumber();
            ServerTemplateVersionModel templateVersionModel = new ServerTemplateVersionModel();
            templateVersionModel.setVersionNumber(launchTemplateVersionNumber);
            templateVersionModel.setTemplateUuid(serverTemplateModel.getTemplateUuid());
            BocloudResult listTemplateVersion = this.listTemplateVersion(templateVersionModel);
            if (listTemplateVersion.isFailed()) {
                return new BocloudResult(false, listTemplateVersion.getMessage());
            }
            List<ServerTemplateVersionModel> versionModels = (List<ServerTemplateVersionModel>) listTemplateVersion
                    .getData();
            return new BocloudResult(true, versionModels.get(0), "创建版本成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(ServerTemplateModel serverTemplateModel) {
        try {
            DeleteLaunchTemplateRequest request = new DeleteLaunchTemplateRequest();
            request.setLaunchTemplateId(serverTemplateModel.getTemplateUuid());
            DeleteLaunchTemplateResponse acsResponse = this.getECSClient().deleteLaunchTemplate(request);
            return new BocloudResult(true, acsResponse.getBody().getRequestId(), "删除成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeTemplateVersion(ServerTemplateModel serverTemplateModel) {
        try {
            DeleteLaunchTemplateVersionRequest request = new DeleteLaunchTemplateVersionRequest();
            request.setLaunchTemplateId(serverTemplateModel.getTemplateUuid());
            List<Long> versions = serverTemplateModel.getVersions();
            List<Integer> integerVersions = new ArrayList<>();
            for (Long version : versions) {
                integerVersions.add(Math.toIntExact(version));
            }
            request.setDeleteVersion(integerVersions);
            DeleteLaunchTemplateVersionResponse acsResponse = this.getECSClient().deleteLaunchTemplateVersion(request);
            return new BocloudResult(true, acsResponse.getBody().getRequestId(), "删除成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ServerTemplateModel serverTemplateModel) {
        try {
            return new BocloudResult(true, null, "查询模板成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "查询模板失败");
        }
    }

    /**
     * 修改默认版本
     */
    @Override
    public BocloudResult modifyDefaultVersion(ServerTemplateModel serverTemplateModel) {
        try {
            ModifyLaunchTemplateDefaultVersionRequest request = new ModifyLaunchTemplateDefaultVersionRequest();
            request.setDefaultVersionNumber(serverTemplateModel.getDefaultVersionNumber());
            request.setLaunchTemplateId(serverTemplateModel.getTemplateUuid());
            ModifyLaunchTemplateDefaultVersionResponse acsResponse = this.getECSClient().modifyLaunchTemplateDefaultVersion(request);
            return new BocloudResult(true, acsResponse.getBody().getRequestId(), "修改成功");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return new BocloudResult(false, "修改失败");
        }
    }

}
