package com.bocloud.cmp.provider.database;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.aliyun.teautil.models.RuntimeOptions;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.rds20140815.models.*;
import com.aliyun.teaopenapi.models.Config;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceRequest;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone.SupportedEngine;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone.SupportedEngine.SupportedEngineVersion;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone.SupportedEngine.SupportedEngineVersion.SupportedCategory;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone.SupportedEngine.SupportedEngineVersion.SupportedCategory.SupportedStorageType;
import com.aliyuncs.rds.model.v20140815.DescribeAvailableResourceResponse.AvailableZone.SupportedEngine.SupportedEngineVersion.SupportedCategory.SupportedStorageType.AvailableResource;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.database.*;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.DBInstanceConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.DBInstanceParameterGroupConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.ResourceGroupsConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import com.google.api.client.util.Maps;

@Provider("bocloud.cmp.aliyun")
public class AliyunDatabaseProvider extends AliyunProvider implements DatabaseProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunDatabaseProvider.class);

	private static final int PAGE_SIZE = 50;
	private static final String DEFAULT_IP = "0.0.0.0/0";

	private ResourceGroupsConvertor convertor = new ResourceGroupsConvertor();
	BeanConvertor<DbInstanceModel, DescribeDBInstanceAttributeResponse> dbInstanceConvertor = new DBInstanceConvertor(
			this.getRegionId());

	BeanConvertor<DbInstanceParameterGroupModel, DescribeParameterGroupResponse> dbInstanceParamConvertor = new DBInstanceParameterGroupConvertor();

	public AliyunDatabaseProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult listInstances() {
		DescribeDBInstancesRequest request = new DescribeDBInstancesRequest();
		request.setPageNumber(1);
		request.setPageSize(PAGE_SIZE);
		request.setRegionId(this.getRegionId());
		try {
			List<DbInstanceModel> models = Lists.newArrayList();
			DescribeDBInstancesResponse response = this.getRDSClient().describeDBInstances(request);
			if (null == response) {
				return new BocloudResult(false, "数据库实例列表查询失败");
			} else if (null == response.getBody().getItems().getDBInstance()) {
				return new BocloudResult(false, "数据库实例列表查询失败");
			}
			List<DescribeDBInstancesResponseBody.DescribeDBInstancesResponseBodyItemsDBInstance> dbInstances = response
					.getBody().getItems().getDBInstance();
			Integer totalCount = response.getBody().totalRecordCount == null ? 0 : response.getBody().totalRecordCount;
//            if (null == totalCount || totalCount == 0) {
//                return new BocloudResult(true, models, "empty DBInstances");
//            }
			logger.info("dbInstances : " + JSON.toJSONString(dbInstances));
			for (DescribeDBInstancesResponseBody.DescribeDBInstancesResponseBodyItemsDBInstance dbInstance : dbInstances) {
				DbInstanceModel attribute = getAttribute(dbInstance.getDBInstanceId());
				if (null != attribute) {
					BocloudResult result = syncUserAndDb(attribute);
					if (result.isFailed()) {
						return new BocloudResult(false, result.getMessage());
					}
					models.add(attribute);
				}
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getRDSClient().describeDBInstances(request);
				dbInstances = response.getBody().items.getDBInstance();
				for (DescribeDBInstancesResponseBody.DescribeDBInstancesResponseBodyItemsDBInstance dbInstance : dbInstances) {
					DbInstanceModel attribute = getAttribute(dbInstance.getDBInstanceId());
					if (null != attribute) {
						BocloudResult result = syncUserAndDb(attribute);
						if (result.isFailed()) {
							return new BocloudResult(false, result.getMessage());
						}
						models.add(attribute);
					}
				}
			}
			return new BocloudResult(true, models, "success");
		} catch (Exception e) {
			logger.error("listInstances error:", e);
			return new BocloudResult(false, "error");
		}
	}

	private BocloudResult syncUserAndDb(DbInstanceModel instanceModel) {
		// 同步当前实例下的用户
		BocloudResult accounts = accounts(instanceModel.getInstanceId(), null);
		if (accounts.isFailed()) {
			return new BocloudResult(false, "获取实例下的用户失败");
		}
		List<DbInstanceUserModel> userModels = (List<DbInstanceUserModel>) accounts.getData();
		instanceModel.setUserModels(userModels);
		// 同步当前实例下的数据库
		BocloudResult dbs = dbs(instanceModel.getInstanceId(), null);
		if (dbs.isFailed()) {
			return new BocloudResult(false, "获取实例下的数据库失败");
		}
		List<DbInstanceDbModel> instanceDbModels = (List<DbInstanceDbModel>) dbs.getData();
		instanceModel.setDbModels(instanceDbModels);
		return new BocloudResult(true, "同步成功");
	}

	@Override
	public BocloudResult createInstance(DbInstanceModel instance) {
		CreateDBInstanceRequest request = new CreateDBInstanceRequest();
		request.setRegionId(this.getRegionId());
		request.setEngine(instance.getEngine());
		request.setEngineVersion(instance.getVersion());
		request.setDBInstanceClass(instance.getInstanceClass());
		request.setDBInstanceStorage(instance.getDbInstanceStorage());
		request.setDBInstanceNetType(instance.getDBInstanceNetType());
		request.setDBInstanceDescription(instance.getName());
		request.setPayType(instance.getPayType());
		request.setZoneId(instance.getZoneId());
		request.setInstanceNetworkType(instance.getInstanceNetworkType());
		request.setVPCId(instance.getVpcId());
		request.setVSwitchId(instance.getVswitchId());
		request.setPeriod(instance.getPeriod());
		request.setUsedTime(instance.getUsedTime());
		if (!StringUtils.isEmpty(instance.getResourceId())) {
			request.setResourceGroupId(instance.getResourceId());
		}
		request.setDBInstanceStorageType(instance.getDbInstanceStorageType());
		request.setAutoRenew(instance.getAutoRenew());
		request.setCategory(instance.getCategory());
		request.setDBParamGroupId(request.getDBParamGroupId());
		request.setDBTimeZone(instance.getDBTimeZone());
		request.setDBIsIgnoreCase(instance.getDBIsIgnoreCase());
		request.setTargetMinorVersion(instance.getTargetMinorVersion());
		request.setSecurityIPList("127.0.0.1");
		try {
//			CreateDBInstanceResponse response = this.getRDSClient().createDBInstance(request);
			Map<String, Integer> map = new HashMap<>();
			map.put("connectTimeout",60000);
			map.put("readTimeout",60000);
			RuntimeOptions build = RuntimeOptions.build(map);
			CreateDBInstanceResponse response = this.getRDSClient().createDBInstanceWithOptions(request,build);
			if (null != response && null != response.getBody().getDBInstanceId()) {
				String instanceId = response.getBody().getDBInstanceId();
				DbInstanceModel instanceModel = getAttribute(instanceId);
				return new BocloudResult(true, instanceModel, "创建成功");
			}
			return new BocloudResult(false, "创建失败");
		} catch (Exception e) {
			logger.error("create instance is Error!", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult modifyInstance(DbInstanceModel instance) {
		return null;
	}

	@Override
	public BocloudResult removeInstance(String instanceId) {
		DeleteDBInstanceRequest request = new DeleteDBInstanceRequest();
		request.setDBInstanceId(instanceId);
		try {
			this.getRDSClient().deleteDBInstance(request);
			return new BocloudResult(true, "释放实例成功");
		} catch (Exception e) {
			logger.error("delete instance is Error!", e);
			return new BocloudResult(true, e.getMessage());
		}
	}

	@Override
	public BocloudResult rebootInstance(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult instanceDetail(String instanceId) {
		DescribeDBInstanceAttributeRequest request = new DescribeDBInstanceAttributeRequest();
		request.setDBInstanceId(instanceId);
		try {
			DescribeDBInstanceAttributeResponse response = this.getRDSClient().describeDBInstanceAttribute(request);
			List<DescribeDBInstanceAttributeResponseBody.DescribeDBInstanceAttributeResponseBodyItemsDBInstanceAttribute> list = response
					.getBody().getItems().getDBInstanceAttribute();
			if (null == list || list.isEmpty()) {
				return new BocloudResult(false, "实例不存在！");
			} else {
				return new BocloudResult(true, dbInstanceConvertor.convertModel(response), "success");
			}
		} catch (Exception e) {
			logger.error("DB Instance Error!", e);
			return new BocloudResult(false, "获取实例异常！");
		}
	}

	@Override
	public BocloudResult getErrLog(DbInstanceModel dbInstanceModel) {
		return null;
	}

	@Override
	public BocloudResult getSlowLog(DbInstanceModel dbInstanceModel) {
		return null;
	}

	@Override
	public BocloudResult getFlavor(String chargeType) {
		try {
			DescribeAvailableResourceRequest request = new DescribeAvailableResourceRequest();
			request.setInstanceChargeType("Postpaid");
			request.setDispenseMode(1);
			DescribeAvailableResourceResponse response = this.getClient().getAcsResponse(request);
			List<AvailableZone> availableZones = response.getAvailableZones();

			List<DbFlavorModel> dbFlavorModels = new ArrayList<>();
			DbFlavorModel dbFlavorModel = new DbFlavorModel();

			Map<String, DbFlavorModel> dbFlavorModelMap = new HashMap<>();
			Set<String> resourceSet = null;
			for (AvailableZone availableZone : availableZones) {
				String status = availableZone.getStatus();
				if (!"Enable".equalsIgnoreCase(status)) {
					continue;
				}
				String regionId = availableZone.getRegionId();
				String zoneId = availableZone.getZoneId();
				List<String> zoneList = new ArrayList<>();
				if (zoneId.contains("MAZ")) {
					String zoneString = zoneId;
					String substring = zoneString.substring(zoneString.indexOf("(") + 1, zoneString.lastIndexOf(")"));
					String zonePrefix = zoneString.substring(0, zoneString.indexOf("MAZ"));
					String[] split = substring.split(",");
					for (String zoneStr : split) {
						zoneList.add(zonePrefix + zoneStr);
					}
				} else {
					zoneList.add(zoneId);
				}
				List<SupportedEngine> supportedEngines = availableZone.getSupportedEngines();
				for (SupportedEngine supportedEngine : supportedEngines) {
					String engine = supportedEngine.getEngine(); // MySQL
					List<SupportedEngineVersion> supportedEngineVersions = supportedEngine.getSupportedEngineVersions();
					for (SupportedEngineVersion supportedEngineVersion : supportedEngineVersions) {
						String version = supportedEngineVersion.getVersion();
						List<SupportedCategory> supportedCategorys = supportedEngineVersion.getSupportedCategorys();
						for (SupportedCategory supportedCategory : supportedCategorys) {
							String category = supportedCategory.getCategory();
							List<SupportedStorageType> supportedStorageTypes = supportedCategory
									.getSupportedStorageTypes();
							for (SupportedStorageType supportedStorageType : supportedStorageTypes) {
								String storageType = supportedStorageType.getStorageType();
								List<AvailableResource> availableResources = supportedStorageType
										.getAvailableResources();
								for (AvailableResource availableResource : availableResources) {
									String dbInstanceClass = availableResource.getDBInstanceClass();
									if (dbFlavorModelMap.containsKey(dbInstanceClass)) {
										dbFlavorModel = dbFlavorModelMap.get(dbInstanceClass);
										resourceSet = new HashSet<>(
												JSON.parseArray(dbFlavorModel.getZoneId(), String.class));
										resourceSet.addAll(zoneList);
										dbFlavorModel.setZoneId(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>(
												JSON.parseArray(dbFlavorModel.getVersion(), String.class));
										resourceSet.add(version);
										dbFlavorModel.setVersion(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>(
												JSON.parseArray(dbFlavorModel.getEngine(), String.class));
										resourceSet.add(engine);
										dbFlavorModel.setEngine(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>(
												JSON.parseArray(dbFlavorModel.getStorageType(), String.class));
										resourceSet.add(storageType);
										dbFlavorModel.setStorageType(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>(
												JSON.parseArray(dbFlavorModel.getCategory(), String.class));
										resourceSet.add(category);
										dbFlavorModel.setCategory(JSON.toJSONString(resourceSet));
									} else {
										dbFlavorModel = new DbFlavorModel();
										dbFlavorModel.setDBInstanceClass(dbInstanceClass);
										dbFlavorModel.setRegionId(regionId);
										dbFlavorModel.setChargeType("Postpaid");
										dbFlavorModel.setStatus(status);
										resourceSet = new HashSet<>();
										resourceSet.addAll(zoneList);
										dbFlavorModel.setZoneId(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>();
										resourceSet.add(version);
										dbFlavorModel.setVersion(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>();
										resourceSet.add(engine);
										dbFlavorModel.setEngine(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>();
										resourceSet.add(storageType);
										dbFlavorModel.setStorageType(JSON.toJSONString(resourceSet));
										resourceSet = new HashSet<>();
										resourceSet.add(category);
										dbFlavorModel.setCategory(JSON.toJSONString(resourceSet));
									}
									dbFlavorModelMap.put(dbInstanceClass, dbFlavorModel);
								}
							}
						}
					}
				}
			}
			List<DbFlavorModel> list = new ArrayList<>(dbFlavorModelMap.values());
			return new BocloudResult<>(true, list, "查询成功");
		} catch (ClientException e) {
			logger.error("DB Flavor Error!", e);
			return new BocloudResult(false, "获取数据库规格异常:" + e);
		}
	}

	public static com.aliyun.rds20140815.Client createClient(String accessKeyId, String accessKeySecret)
			throws Exception {
		Config config = new Config()
				// 您的AccessKey ID
				.setAccessKeyId(accessKeyId)
				// 您的AccessKey Secret
				.setAccessKeySecret(accessKeySecret);
		// 访问的域名
		config.endpoint = "rds.aliyuncs.com";
		return new com.aliyun.rds20140815.Client(config);
	}

	public static void main(String[] args_) throws Exception {
		java.util.List<String> args = java.util.Arrays.asList(args_);
		com.aliyun.rds20140815.Client client = AliyunDatabaseProvider.createClient("LTAI4G4wRz6P26Jg6hM8spJ3",
				"oiEHF3wRFClL7Tijv2cNWWOCshD6tM");
		GrantAccountPrivilegeRequest request = new GrantAccountPrivilegeRequest();
		request.setDBInstanceId("rm-2zej32lg398f7791r");
		request.setAccountName("lmh2");
		request.setDBName("db1");
		request.setAccountPrivilege("ReadOnly");
		// 复制代码运行请自行打印 API 的返回值
		GrantAccountPrivilegeResponse grantAccountPrivilegeResponse = client.grantAccountPrivilege(request);
		System.out.println(grantAccountPrivilegeResponse);
	}

	@Override
	public BocloudResult createParameterGroups(String regionId, DbInstanceParameterGroupModel groupModel) {
		try {
			CreateParameterGroupRequest request = new CreateParameterGroupRequest();
			request.setRegionId(regionId);
			request.setParameterGroupName(groupModel.getName());
			request.setEngine(groupModel.getEngine());
			request.setEngineVersion(groupModel.getEngineVersion());
			request.setParameters(groupModel.getParameters());
			request.setParameterGroupDesc(groupModel.getParameterGroupDesc());
			this.getRDSClient().createParameterGroup(request);
			return new BocloudResult(true, "创建参数模板成功");
		} catch (Exception e) {
			logger.error("create param group is Error!", e);
			return new BocloudResult(true, "创建参数模板失败");
		}
	}

	@Override
	public BocloudResult modifyParameterGroups(String regionId, DbInstanceParameterGroupModel groupModel) {
		try {
			ModifyParameterGroupRequest request = new ModifyParameterGroupRequest();
			request.setRegionId(regionId);
			request.setParameters(groupModel.getParameters());
			request.setParameterGroupDesc(groupModel.getParameterGroupDesc());
			request.setParameterGroupName(groupModel.getName());
			request.setParameterGroupId(groupModel.getParameterGroupId());
			this.getRDSClient().modifyParameterGroup(request);
			return new BocloudResult(true, "修改参数模板成功");
		} catch (Exception e) {
			logger.error("modify param group is Error!", e);
			return new BocloudResult(true, "修改参数模板失败");
		}
	}

	@Override
	public BocloudResult deleteParameterGroups(String regionId, String groupId) {
		try {
			DeleteParameterGroupRequest request = new DeleteParameterGroupRequest();
			request.setRegionId(regionId);
			request.setParameterGroupId(groupId);
			this.getRDSClient().deleteParameterGroup(request);
			return new BocloudResult(true, "删除参数模板成功");
		} catch (Exception e) {
			logger.error("delete param group is Error!", e);
			return new BocloudResult(true, "删除参数模板失败");
		}
	}

	@Override
	public BocloudResult cloneParameterGroups(String regionId, String targetRegionId, String groupId, String GroupName,
			String GroupDesc) {
		try {
			CloneParameterGroupRequest request = new CloneParameterGroupRequest();
			request.setRegionId(regionId);
			request.setTargetRegionId(targetRegionId);
			request.setParameterGroupId(groupId);
			request.setParameterGroupName(GroupName);
			request.setParameterGroupDesc(GroupDesc);
			this.getRDSClient().cloneParameterGroup(request);
			return new BocloudResult(true, "复制参数模板成功");
		} catch (Exception e) {
			logger.error("clone param group is Error!", e);
			return new BocloudResult(true, "复制参数模板失败");
		}
	}

	private static Map<String, String> regionNameMap = new HashMap<>();
	static {
		regionNameMap.put("cn-hangzhou", "华东1(杭州)");
		regionNameMap.put("cn-shanghai", "华东2(上海)");
		regionNameMap.put("cn-qingdao", "华北1(青岛)");
		regionNameMap.put("cn-beijing", "华北2(北京)");

		regionNameMap.put("cn-zhangjiakou", "华北3(张家口)");
		regionNameMap.put("cn-huhehaote", "华北5(呼和浩特)");
		regionNameMap.put("cn-wulanchabu", "华北6(乌兰察布)");
		regionNameMap.put("cn-shenzhen", "华南1(深圳)");

		regionNameMap.put("cn-heyuan", "华南2(河源)");
		regionNameMap.put("cn-guangzhou", "华南3(广州)");
		regionNameMap.put("cn-chengdu", "西南1(成都)");
		regionNameMap.put("cn-hongkong", "中国(香港)");
	}

	@Override
	public BocloudResult getRegions(String type, String volType) {
		DescribeRegionsRequest request = new DescribeRegionsRequest();
		try {
			DescribeRegionsResponse response = this.getRDSClient().describeRegions(request);
			List<DescribeRegionsResponseBody.DescribeRegionsResponseBodyRegionsRDSRegion> rdsRegions = response
					.getBody().getRegions().getRDSRegion();
			Map<String, Object> regionMap = Maps.newHashMap();
			for (DescribeRegionsResponseBody.DescribeRegionsResponseBodyRegionsRDSRegion region : rdsRegions) {
				if (!regionNameMap.keySet().contains(region.getRegionId())) {
					continue;
				}
				if (regionMap.containsKey(region.getRegionId())) {
					Map<String, Object> data = (Map<String, Object>) regionMap.get(region.getRegionId());
					List<Map<String, Object>> zones = (List<Map<String, Object>>) data.get("zones");
					if ("mysql".equalsIgnoreCase(type)) {
						sinicization(region, data, zones, volType);
					}
				} else {
					List<Map<String, Object>> zones = Lists.newArrayList();
					Map<String, Object> data = Maps.newHashMap();
					if ("mysql".equalsIgnoreCase(type)) {
						sinicization(region, data, zones, volType);
						String regionName = region.getRegionId();
						if (regionNameMap.containsKey(region.getRegionId())) {
							regionName = regionNameMap.get(region.getRegionId());
						}
						Map<String, Object> dataMap = Maps.newHashMap();
						dataMap.put("name", regionName);
						dataMap.put("zones", zones);
						regionMap.put(region.getRegionId(), dataMap);
					}
				}
			}
			return new BocloudResult(true, regionMap, "查询用户可选的地域和可用区成功");
		} catch (Exception e) {
			logger.error("query redions and zones is Error!", e);
			return new BocloudResult(false, "查询用户可选的地域和可用区失败！");
		}
	}

	private static void sinicization(DescribeRegionsResponseBody.DescribeRegionsResponseBodyRegionsRDSRegion region,
			Map<String, Object> data, List<Map<String, Object>> zones, String volType) {
		String[] zonesHzName = { "华东 1 可用区I", "华东 1 可用区H", "华东 1 可用区 G", "华东 1 可用区 B", "华东 1 可用区 E", "华东 1 可用区 F",
				"华东1 可用区K" };
		String[] zonesShName = { "华东 2 可用区G", "华东 2 可用区 B", "华东 2 可用区 E", "华东 2 可用区 D", "华东 2 可用区 F", "华东 2 可用区 L" };
		String[] zonesQdName = { "华北 1 可用区 C", "华北 1 可用区 B" };
		String[] zonesBjName = { "华北 2 可用区 H", "华北 2 可用区 F", "华北 2 可用区 A", "华北 2 可用区 G", "华北 2 可用区 C", "华北 2 可用区 D",
				"华北 2 可用区 E", "华北 2 可用区 K" };
		String[] zonesZjkName = { "华北 3 可用区 C", "华北 3 可用区 A", "华北 3 可用区 B" };
		String[] zonesHhhtName = { "华北 5 可用区B", "华北 5 可用区A" };
		String[] zonesWlcbName = { "乌兰察布 可用区A", "乌兰察布 可用区C", "乌兰察布 可用区B" };
		String[] zonesSzName = { "华南 1 可用区 D", "华南 1 可用区 E", "华南 1 可用区 A", "华南 1 可用区 C", "华南 1 可用区 B" };
		String[] zonesHyName = { "河源 可用区B", "河源 可用区A" };
		String[] zonesGzName = { "广州 可用区A" };
		String[] zonesCdName = { "成都 可用区A", "成都 可用区B" };
		String[] zonesXgName = { "香港可用区B", "香港可用区C", "香港可用区D" };
		Map<String, Object> zonesMap = Maps.newHashMap();
		if ("cn-hangzhou".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-hangzhou-i".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesHzName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-hangzhou-h".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesHzName[1]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-hangzhou-g".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesHzName[2]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-hangzhou-k".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHzName[6]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-hangzhou-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHzName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-hangzhou-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHzName[4]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-hangzhou-f".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHzName[5]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-shanghai".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-shanghai-g".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesShName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-shanghai-b".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesShName[1]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-shanghai-f".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesShName[4]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-shanghai-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shanghai-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd".equalsIgnoreCase(volType)) {
				if ("cn-shanghai-l".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[5]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shanghai-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd2".equalsIgnoreCase(volType)) {
				if ("cn-shanghai-l".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[5]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shanghai-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd3".equalsIgnoreCase(volType)) {
				if ("cn-shanghai-l".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[5]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shanghai-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesShName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-qingdao".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-qingdao-c".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesQdName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-qingdao-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesQdName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-qingdao-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesQdName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-beijing".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-beijing-h".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesBjName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-beijing-f".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-k".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[7]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-beijing-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-g".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[4]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[5]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-beijing-e".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[6]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd".equalsIgnoreCase(volType)) {
				if ("cn-beijing-g".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd2".equalsIgnoreCase(volType)) {
				if ("cn-beijing-g".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd3".equalsIgnoreCase(volType)) {
				if ("cn-beijing-g".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesBjName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-zhangjiakou".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-zhangjiakou-c".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesZjkName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-zhangjiakou-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesZjkName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-zhangjiakou-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesZjkName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-huhehaote".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-huhehaote-a".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesHhhtName[1]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-huhehaote-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHhhtName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd".equalsIgnoreCase(volType)) {
				if ("cn-huhehaote-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHhhtName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd2".equalsIgnoreCase(volType)) {
				if ("cn-huhehaote-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHhhtName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd3".equalsIgnoreCase(volType)) {
				if ("cn-huhehaote-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHhhtName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-wulanchabu".equalsIgnoreCase(region.getRegionId())) {
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-wulanchabu-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesWlcbName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-wulanchabu-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesWlcbName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-wulanchabu-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesWlcbName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-shenzhen".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-shenzhen-d".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesSzName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-shenzhen-e".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesSzName[1]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-shenzhen-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesSzName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-shenzhen-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesSzName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shenzhen-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesSzName[3]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-shenzhen-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesSzName[4]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-heyuan".equalsIgnoreCase(region.getRegionId())) {
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-heyuan-b".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHyName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-heyuan-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesHyName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-guangzhou".equalsIgnoreCase(region.getRegionId())) {
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-guangzhou-a".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesGzName[0]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		} else if ("cn-chengdu".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-chengdu-a".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesCdName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			} else if ("cn-chengdu-b".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesCdName[1]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
		} else if ("cn-hongkong".equalsIgnoreCase(region.getRegionId())) {
			if ("cn-hongkong-b".equalsIgnoreCase(region.getZoneId())) {
				zonesMap.put("name", zonesXgName[0]);
				zonesMap.put("zoneId", region.getZoneId());
				zones.add(zonesMap);
			}
			if ("local_ssd".equalsIgnoreCase(volType)) {
				if ("cn-hongkong-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd".equalsIgnoreCase(volType)) {
				if ("cn-hongkong-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-hongkong-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd2".equalsIgnoreCase(volType)) {
				if ("cn-hongkong-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-hongkong-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_essd3".equalsIgnoreCase(volType)) {
				if ("cn-hongkong-d".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[2]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				} else if ("cn-hongkong-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			} else if ("cloud_ssd".equalsIgnoreCase(volType)) {
				if ("cn-hongkong-c".equalsIgnoreCase(region.getZoneId())) {
					zonesMap.put("name", zonesXgName[1]);
					zonesMap.put("zoneId", region.getZoneId());
					zones.add(zonesMap);
				}
			}
		}
	}

	@Override

	public BocloudResult getParameterGroups(String regionId) {
		com.aliyun.rds20140815.models.DescribeParameterGroupsRequest request = new DescribeParameterGroupsRequest();
		request.setRegionId(regionId);
		try {
			DescribeParameterGroupsResponse response = this.getRDSClient().describeParameterGroups(request);
			List<DescribeParameterGroupsResponseBody.DescribeParameterGroupsResponseBodyParameterGroupsParameterGroup> parameterGroup = response
					.getBody().getParameterGroups().getParameterGroup();
			List<DbInstanceParameterGroupModel> groups = Lists.newArrayList();
			for (DescribeParameterGroupsResponseBody.DescribeParameterGroupsResponseBodyParameterGroupsParameterGroup group : parameterGroup) {
				DbInstanceParameterGroupModel groupModel = getParameterGroup(group.getParameterGroupId(), regionId);
				if (null != groupModel) {
					groups.add(groupModel);
				}
			}
			return new BocloudResult(true, groups, "查询参数模板成功");
		} catch (Exception e) {
			logger.error("query param group is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult restartDBInstance(String instanceId) {
		RestartDBInstanceRequest request = new RestartDBInstanceRequest();
		request.setDBInstanceId(instanceId);
		try {
			this.getRDSClient().restartDBInstance(request);
			return new BocloudResult(true, "重启实例成功");
		} catch (Exception e) {
			logger.error("restart is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	private DbInstanceParameterGroupModel getParameterGroup(String groupId, String regionId) {
		DescribeParameterGroupRequest request = new DescribeParameterGroupRequest();
		request.setParameterGroupId(groupId);
		request.setRegionId(regionId);
		try {
			DescribeParameterGroupResponse response = this.getRDSClient().describeParameterGroup(request);
			List<DescribeParameterGroupResponseBody.DescribeParameterGroupResponseBodyParamGroupParameterGroup> group = response
					.getBody().getParamGroup().getParameterGroup();
			if (null == group || group.isEmpty()) {
				return null;
			}
			return dbInstanceParamConvertor.convertModel(response);
		} catch (Exception e) {
			logger.error("get instance param group error:", e);
			return null;
		}
	}

	// public BocloudResult getErrLog(RdsModel rds) {
	// DescribeErrorLogsRequest request = new DescribeErrorLogsRequest();
	// String insId = rds.getInstance_id();
	// request.setDBInstanceId(insId);
	// request.setStartTime(rds.getStart_date());
	// request.setEndTime(rds.getEnd_date());
	// request.setPageNumber(rds.getPage());
	// request.setPageSize(rds.getRows());
	// try {
	// DescribeErrorLogsResponse acsResponse =
	// this.getClient().getAcsResponse(request);
	// Integer totalRecordCount = acsResponse.getTotalRecordCount();
	//// acsResponse.getPageNumber();
	//// List<DescribeSlowLogsResponse.SQLSlowLog> items = acsResponse.getItems();
	// if (null != totalRecordCount) {
	// logger.info("request errorlog rds instance(id:" + insId + ") success");
	// return new BocloudResult(true, acsResponse, "请求rds错误日志成功。");
	// } else {
	// logger.info("request errorlog rds instance(id:" + insId + ") failed");
	// return new BocloudResult(false, "请求rds错误日志失败。");
	// }
	// } catch (ClientException e) {
	// logger.error("request errorlog rds instance(id:" + insId + ") error:", e);
	// return new BocloudResult(false, e.getMessage());
	// }
	// }

	// public BocloudResult getSlowLog(RdsModel rds) {
	// String insId = rds.getInstance_id();
	// DescribeSlowLogsRequest request = new DescribeSlowLogsRequest();
	// request.setDBInstanceId(insId);
	// request.setStartTime(rds.getStart_date());
	// request.setEndTime(rds.getEnd_date());
	// request.setPageNumber(rds.getPage());
	// request.setPageSize(rds.getRows());
	// if (null != rds.getSortKey()) {
	// request.setSortKey(rds.getSortKey());
	// }
	// try {
	// DescribeSlowLogsResponse acsResponse =
	// this.getClient().getAcsResponse(request);
	// if (null != acsResponse.getTotalRecordCount()) {
	// logger.info("request slowLog rds instance(id:" + insId + ") success");
	// return new BocloudResult(true, acsResponse, "请求rds慢日志成功。");
	// } else {
	// logger.info("request slowLog rds instance(id:" + insId + ") failed");
	// return new BocloudResult(false, "请求rds慢日志失败。");
	// }
	// } catch (ClientException e) {
	// logger.error("request errorlog rds instance(id:" + insId + ") error:", e);
	// return new BocloudResult(false, e.getMessage());
	// }
	// }

	/**
	 * 查询数据库实例详情
	 *
	 * @param instanceId
	 * @return
	 */
	private DbInstanceModel getAttribute(String instanceId) {
		DescribeDBInstanceAttributeRequest detailRequest = new DescribeDBInstanceAttributeRequest();
		detailRequest.setDBInstanceId(instanceId);
		DescribeDBInstanceAttributeResponse detailResponse;

		try {
			detailResponse = this.getRDSClient().describeDBInstanceAttribute(detailRequest);
			List<DescribeDBInstanceAttributeResponseBody.DescribeDBInstanceAttributeResponseBodyItemsDBInstanceAttribute> list = detailResponse
					.getBody().getItems().getDBInstanceAttribute();
			if (null == list || list.isEmpty()) {
				return null;
			}
			return dbInstanceConvertor.convertModel(detailResponse);
		} catch (Exception e) {
			logger.error("get instance attribute error:", e);
			return null;
		}
	}

	@Override
	public BocloudResult createAccount(DbInstanceUserModel userModel) {
		try {
			CreateAccountRequest request = new CreateAccountRequest();
			request.setDBInstanceId(userModel.getDBInstanceId());
			request.setAccountName(userModel.getAccountName());
			request.setAccountPassword(userModel.getAccountPassword());
			request.setAccountDescription(userModel.getAccountDescription());
			request.setAccountType(userModel.getAccountType());
			this.getRDSClient().createAccount(request);
			BocloudResult waitAccountDetail = waitAccountDetail(userModel.getDBInstanceId(),
					userModel.getAccountName());
			if (waitAccountDetail.isFailed()) {
				return waitAccountDetail;
			}
			return new BocloudResult(true, waitAccountDetail.getData(), "账号用户成功");
		} catch (Exception e) {
			logger.error("create instance user is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	private BocloudResult waitAccountDetail(String instanceId, String accountName) {
		logger.info("instanceId : " + instanceId + " accountName : " + accountName);
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				Thread.sleep(5000);

				BocloudResult accountResult = this.getAccount(instanceId, accountName);
				if (accountResult.isFailed()) {
					return accountResult;
				}
				DbInstanceUserModel model = (DbInstanceUserModel) accountResult.getData();
				if ("Available".equalsIgnoreCase(model.getAccountStatus())) {
					return new BocloudResult<>(true, model, "查询成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 20 * 60 * 1000) {
					return new BocloudResult<>(false, "操作超时！");
				}
			}
		} catch (Exception e) {
			logger.error("get instances error:", e);
			return new BocloudResult<>(false, "操作失败： " + e);
		}
	}

	@Override
	public BocloudResult resetAccountPassword(String instanceId, String accountName, String accountPassword) {
		try {
			ResetAccountPasswordRequest request = new ResetAccountPasswordRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			request.setAccountPassword(accountPassword);
			this.getRDSClient().resetAccountPassword(request);
			return new BocloudResult(true, "实例用户密码重置成功");
		} catch (Exception e) {
			logger.error("create instance user password reset is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult deleteAccount(String instanceId, String accountName) {
		try {
			DeleteAccountRequest request = new DeleteAccountRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			this.getRDSClient().deleteAccount(request);
			return new BocloudResult(true, "实例用户删除成功");
		} catch (Exception e) {
			logger.error("delete instance user is Error!", e);
			return new BocloudResult(false, "实例用户删除失败！");
		}
	}

	@Override
	public BocloudResult resetAccount(String instanceId, String accountName, String accountPassword) {
		try {
			ResetAccountRequest request = new ResetAccountRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			request.setAccountPassword(accountPassword);
			this.getRDSClient().resetAccount(request);
			return new BocloudResult(true, "重置高权限账号的权限成功");
		} catch (Exception e) {
			logger.error("reset permissions for high-privilege accounts is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult revokeAccountPrivilege(String instanceId, String accountName, String dbName) {
		try {
			RevokeAccountPrivilegeRequest request = new RevokeAccountPrivilegeRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			request.setDBName(dbName);
			this.getRDSClient().revokeAccountPrivilege(request);
			return new BocloudResult(true, "撤销账号对数据库的访问权限成功");
		} catch (Exception e) {
			logger.error("revoke the account's access to the database is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	private BocloudResult getInstanceDetail(String instanceId, String dbName) {
		logger.info("instanceId : " + instanceId + " dbName : " + dbName);
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				Thread.sleep(5000);
				BocloudResult dbDetailResult = this.getDb(instanceId, dbName);
				if (dbDetailResult.isFailed()) {
					return dbDetailResult;
				}
				DbInstanceDbModel model = (DbInstanceDbModel) dbDetailResult.getData();
				if ("Running".equalsIgnoreCase(model.getDBStatus())) {
					return new BocloudResult<>(true, model, "查询成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 20 * 60 * 1000) {
					return new BocloudResult<>(false, "操作超时！");
				}
			}
		} catch (Exception e) {
			logger.error("get instances error:", e);
			return new BocloudResult<>(false, "操作失败： " + e);
		}
	}

	/**
	 * ReadWrite：读写 ReadOnly：只读 DDLOnly：仅执行DDL DMLOnly：只执行DML
	 */
	@Override
	public BocloudResult createDatabase(DbInstanceDbModel dbModel) {
		try {
			CreateDatabaseRequest request = new CreateDatabaseRequest();
			request.setDBInstanceId(dbModel.getDBInstanceId());
			request.setDBName(dbModel.getDBName());
			request.setCharacterSetName(dbModel.getCharacterSetName());
			request.setDBDescription(dbModel.getDBDescription());
			this.getRDSClient().createDatabase(request);
			BocloudResult instanceDetail = getInstanceDetail(dbModel.getDBInstanceId(), dbModel.getDBName());
			if (instanceDetail.isFailed()) {
				return instanceDetail;
			}
			String accountName = dbModel.getAccountName();
			String accountPrivilege = dbModel.getAccountPrivilege();
			DbInstanceUserModel userModel = null;
			if (!StringUtils.isEmpty(accountName) && !StringUtils.isEmpty(accountPrivilege)) {
				BocloudResult grantAccountPrivilege = this.grantAccountPrivilege(dbModel.getDBInstanceId(), accountName,
						dbModel.getDBName(), accountPrivilege);
				if (grantAccountPrivilege.isSuccess()) {
					userModel = (DbInstanceUserModel) grantAccountPrivilege.getData();
				}
			}
			BocloudResult dbDetail = getDb(dbModel.getDBInstanceId(), dbModel.getDBName());
			if (dbDetail.isFailed()) {
				return dbDetail;
			}
			DbInstanceDbModel dbInstanceDbModel = (DbInstanceDbModel) dbDetail.getData();
			if (null != userModel) {
				dbInstanceDbModel.setUserModel(userModel);
			}
			return new BocloudResult(true, dbInstanceDbModel, "创建数据库成功");
		} catch (Exception e) {
			logger.error("create database is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult grantAccountPrivilege(String instanceId, String accountName, String dbName,
			String accountPrivilege) {
		try {
			GrantAccountPrivilegeRequest request = new GrantAccountPrivilegeRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			request.setDBName(dbName);
			request.setAccountPrivilege(accountPrivilege);
			this.getRDSClient().grantAccountPrivilege(request);
			BocloudResult accountDetail = this.getAccount(instanceId, accountName);
			if (accountDetail.isFailed()) {
				return accountDetail;
			}
			return new BocloudResult(true, accountDetail.getData(), "授权账号访问数据库成功");
		} catch (Exception e) {
			logger.error("authorize the account to access the database is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult getAccount(String instanceId, String accountName) {
		try {
			DescribeAccountsRequest request = new DescribeAccountsRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			DescribeAccountsResponse response = this.getRDSClient().describeAccounts(request);
			DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccount account = response
					.getBody().getAccounts().getDBInstanceAccount().get(0);
			DbInstanceUserModel userModel = new DbInstanceUserModel();
			userModel.setAccountStatus(account.getAccountStatus());
			userModel.setAccountDescription(account.getAccountDescription());
			userModel.setDBInstanceId(instanceId);
			userModel.setPrivExceeded(account.getPrivExceeded());
			userModel.setAccountType(account.getAccountType());
			userModel.setAccountName(account.getAccountName());
			JSONArray dbs = new JSONArray();
			List<DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccountDatabasePrivilegesDatabasePrivilege> privilege = account
					.getDatabasePrivileges().getDatabasePrivilege();
			for (DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccountDatabasePrivilegesDatabasePrivilege databasePrivilege : privilege) {
				JSONObject db = new JSONObject();
				db.put("name", databasePrivilege.getDBName());
				db.put("accountPrivilege", databasePrivilege.getAccountPrivilege());
				db.put("accountPrivilegeDetail", databasePrivilege.getAccountPrivilegeDetail());
				dbs.add(db);
			}
			userModel.setDatabasePrivileges(dbs);
			return new BocloudResult(true, userModel, "查询用户信息成功");
		} catch (Exception e) {
			logger.error("get instacne user is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult deleteDatabase(String instanceId, String dBName) {
		try {
			DeleteDatabaseRequest request = new DeleteDatabaseRequest();
			request.setDBInstanceId(instanceId);
			request.setDBName(dBName);
			this.getRDSClient().deleteDatabase(request);
			return new BocloudResult(true, "删除数据库成功");
		} catch (Exception e) {
			logger.error("create database is Error!", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult getDb(String instanceId, String dbName) {
		try {
			DescribeDatabasesRequest request = new DescribeDatabasesRequest();
			request.setDBInstanceId(instanceId);
			request.setDBName(dbName);
			DescribeDatabasesResponse response = this.getRDSClient().describeDatabases(request);
			DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabase db = response.getBody()
					.getDatabases().getDatabase().get(0);
			DbInstanceDbModel dbModel = new DbInstanceDbModel();
			dbModel.setDBDescription(db.getDBDescription());
			dbModel.setDBStatus(db.getDBStatus());
			dbModel.setDBName(db.getDBName());
			dbModel.setDBDescription(db.getDBDescription());
			dbModel.setEngine(db.getEngine());
			dbModel.setCharacterSetName(db.getCharacterSetName());
			JSONArray accounts = new JSONArray();
			List<DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabaseAccountsAccountPrivilegeInfo> accountPrivilegeInfo = db
					.getAccounts().getAccountPrivilegeInfo();
			for (DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabaseAccountsAccountPrivilegeInfo info : accountPrivilegeInfo) {
				JSONObject account = new JSONObject();
				account.put("account", info.getAccount());
				account.put("accountPrivilege", info.getAccountPrivilege());
				account.put("accountPrivilegeDetail", info.getAccountPrivilegeDetail());
				accounts.add(account);
			}
			dbModel.setAccounts(accounts);
			return new BocloudResult(true, dbModel, "查询成功");
		} catch (Exception e) {
			logger.error("get instacne db is Error!", e);
			return new BocloudResult(false, "查询数据库信息失败！");
		}
	}

	@Override
	public BocloudResult accounts(String instanceId, String accountName) {
		try {
			DescribeAccountsRequest request = new DescribeAccountsRequest();
			request.setDBInstanceId(instanceId);
			request.setAccountName(accountName);
			DescribeAccountsResponse response = this.getRDSClient().describeAccounts(request);
			List<DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccount> accounts = response
					.getBody().getAccounts().getDBInstanceAccount();
			List<DbInstanceUserModel> userModels = Lists.newArrayList();
			for (DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccount account : accounts) {
				DbInstanceUserModel userModel = new DbInstanceUserModel();
				userModel.setAccountStatus(account.getAccountStatus());
				userModel.setAccountDescription(account.getAccountDescription());
				userModel.setDBInstanceId(instanceId);
				userModel.setPrivExceeded(account.getPrivExceeded());
				userModel.setAccountType(account.getAccountType());
				userModel.setAccountName(account.getAccountName());
				JSONArray dbs = new JSONArray();
				List<DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccountDatabasePrivilegesDatabasePrivilege> privilege = account
						.getDatabasePrivileges().getDatabasePrivilege();
				for (DescribeAccountsResponseBody.DescribeAccountsResponseBodyAccountsDBInstanceAccountDatabasePrivilegesDatabasePrivilege databasePrivilege : privilege) {
					JSONObject db = new JSONObject();
					db.put("name", databasePrivilege.getDBName());
					db.put("accountPrivilege", databasePrivilege.getAccountPrivilege());
					db.put("accountPrivilegeDetail", databasePrivilege.getAccountPrivilegeDetail());
					dbs.add(db);
				}
				userModel.setDatabasePrivileges(dbs);
				userModels.add(userModel);
			}
			return new BocloudResult(true, userModels, "查询数据库账号列表成功");
		} catch (Exception e) {
			logger.error("get instacne user is Error!", e);
			return new BocloudResult(false, "查询数据库账号列表失败！");
		}
	}

	@Override
	public BocloudResult dbs(String instanceId, String dbName) {
		try {
			DescribeDatabasesRequest request = new DescribeDatabasesRequest();
			request.setDBInstanceId(instanceId);
			request.setDBName(dbName);
			DescribeDatabasesResponse response = this.getRDSClient().describeDatabases(request);
			List<DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabase> database = response
					.getBody().getDatabases().getDatabase();
			List<DbInstanceDbModel> dbs = Lists.newArrayList();
			for (DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabase db : database) {
				DbInstanceDbModel dbModel = new DbInstanceDbModel();
				dbModel.setDBDescription(db.getDBDescription());
				dbModel.setDBStatus(db.getDBStatus());
				dbModel.setDBName(db.getDBName());
				dbModel.setDBDescription(db.getDBDescription());
				dbModel.setEngine(db.getEngine());
				dbModel.setCharacterSetName(db.getCharacterSetName());
				JSONArray accounts = new JSONArray();
				List<DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabaseAccountsAccountPrivilegeInfo> accountPrivilegeInfo = db
						.getAccounts().getAccountPrivilegeInfo();
				for (DescribeDatabasesResponseBody.DescribeDatabasesResponseBodyDatabasesDatabaseAccountsAccountPrivilegeInfo info : accountPrivilegeInfo) {
					JSONObject account = new JSONObject();
					account.put("account", info.getAccount());
					account.put("accountPrivilege", info.getAccountPrivilege());
					account.put("accountPrivilegeDetail", info.getAccountPrivilegeDetail());
					accounts.add(account);
				}
				dbModel.setAccounts(accounts);
				dbs.add(dbModel);
			}
			return new BocloudResult(true, dbs, "查询实例数据库成功");
		} catch (Exception e) {
			logger.error("get instacne db is Error!", e);
			return new BocloudResult(false, "查询实例数据库失败！");
		}
	}
}
