package com.bocloud.cmp.provider.groups;

import com.aliyun.resourcemanager20200331.models.ListResourcesRequest;
import com.aliyun.resourcemanager20200331.models.ListResourcesResponse;
import com.aliyun.resourcemanager20200331.models.ListResourcesResponseBody;
import com.aliyun.vpc20160428.models.MoveResourceGroupRequest;
import com.aliyun.vpc20160428.models.MoveResourceGroupResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ResourceGroupModel;
import com.bocloud.cmp.model.ResourcePoolBean;
import com.bocloud.cmp.model.ResourcePoolModel;
import com.bocloud.cmp.model.agile.AgileResourcePoolModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.compute.ResourcePoolProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.compute
 * @ClassName: AliyunResourceProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/7/29 10:12
 * @Description: 资源
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunResourceProvider extends AliyunProvider implements ResourcePoolProvider {

	private Logger logger = LoggerFactory.getLogger(AliyunResourceProvider.class);
	private static final int PAGE_SIZE = 100;
	public AliyunResourceProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(ResourcePoolModel resourcePoolModel) {
		return null;
	}

	@Override
	public BocloudResult modify(ResourcePoolBean resourcePoolBean) {
		return null;
	}

	@Override
	public BocloudResult remove(String s) {
		return null;
	}

	@Override
	public BocloudResult list(String resourceGroupId) {
		try {
			ListResourcesRequest request = new ListResourcesRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			List<ResourceGroupModel> resourceGroupModels = Lists.newArrayList();
			ListResourcesResponse response = this.getResourceClient().listResources(request);
			if (null == response) {
				return new BocloudResult(false, "查询资源列表失败");
			} else if (null == response.getBody().getResources()) {
				return new BocloudResult(false, "查询资源列表失败");
			}
			List<ListResourcesResponseBody.ListResourcesResponseBodyResourcesResource> resources = response.getBody().getResources().getResource();
			Integer totalCount = response.getBody().totalCount;
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, resourceGroupModels, "empty resource");
			}
			/*for (ListResourcesResponseBody.ListResourcesResponseBodyResourcesResource resource : resources) {
				ResourceGroupModel convertModel = convertor.convertModel(resourceGroup);
				convertModel.setRegionId(this.getRegionId());
				resourceGroupModels.add(convertModel);
			}*/
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, resourceGroupModels, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getResourceClient().listResources(request);
				resources.addAll(response.getBody().getResources().getResource());
			/*	for (ListResourcesResponseBody.ListResourcesResponseBodyResourcesResource resource : resources) {
					ResourceGroupModel convertModel = convertor.convertModel(resourceGroup);
					convertModel.setRegionId(this.getRegionId());
					resourceGroupModels.add(convertModel);
				}*/
			}
			return new BocloudResult(true, resources, "查询资源列表成功");
		} catch (Exception e) {
			logger.error("查询资源列表失败 ： " + e);
			return new BocloudResult(false, "查询资源列表失败", e.getMessage());
		}
	}

	/**
	 * @Author Hu Wentao
	 * @Description 查询相关条件的资源列表
	 * @Date 15:17 2020/7/29
	 * @Param [resourceGroupId, service, region, resourceType, resourceId]
	 * @return com.bocloud.common.model.BocloudResult
	 **/
	public BocloudResult list(String resourceGroupId, String service, String region, String resourceType,
			String resourceId) {
		try {
			ListResourcesRequest request = new ListResourcesRequest();
			request.setResourceGroupId(resourceGroupId);
			request.setService(service);
			request.setResourceType(resourceType);
			request.setResourceId(resourceId);
			ListResourcesResponse response = this.getResourceClient().listResources(request);
			if (null == response || null == response.getBody().getResources()) {
				return new BocloudResult(false, "查询资源列表失败");
			}
			return new BocloudResult(true, response.getBody().getResources(), "查询资源列表成功");
		} catch (Exception e) {
			logger.error("查询资源列表失败 ： " + e);
			return new BocloudResult(false, "查询资源列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(AgileResourcePoolModel model) {
		return null;
	}

	@Override
	public BocloudResult listByResign(AgileResourcePoolModel model) {
		return null;
	}

	@Override
	public BocloudResult removeHuaWeiSdnPool(AgileResourcePoolModel model) {
		return null;
	}

	@Override
	public BocloudResult listResourcePoolLink() {
		return null;
	}

	/**
	 * @Author Hu Wentao
	 * @Description 修改云资源所属的资源组
	 * @Date 15:24 2020/7/29
	 * @Param [regionId, resourceType, resourceId, newResourceGroupId]
	 * @return com.bocloud.common.model.BocloudResult
	 **/
	public BocloudResult moveResourceGroup(String regionId, String resourceType, String resourceId,
			String newResourceGroupId) {
		try {
			MoveResourceGroupRequest request = new MoveResourceGroupRequest();
			request.setRegionId(regionId);
			request.setResourceId(resourceId);
			request.setResourceType(resourceType);
			request.setNewResourceGroupId(newResourceGroupId);
			MoveResourceGroupResponse response = this.getVPCClient().moveResourceGroup(request);
			if (null == response) {
				return new BocloudResult(false, "修改云资源所属的资源组失败");
			}
			return new BocloudResult(true, response.getBody().getRequestId(), "修改云资源所属的资源组成功");
		} catch (Exception e) {
			logger.error("修改云资源所属的资源组失败 ： " + e);
			return new BocloudResult(false, "修改云资源所属的资源组失败", e.getMessage());
		}
	}
}
