package com.bocloud.cmp.provider.loadbalance;

import com.aliyun.slb20140515.models.*;
import com.bocloud.cmp.model.AccessControlModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.AccessControlConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 访问控制
 *
 * @author lyy
 * @version 1.0
 * @date 2021/1/14 9:26
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunAccessControlProvider extends AliyunProvider implements AccessControlProvider {
	private static Logger logger = LoggerFactory.getLogger(AliyunAccessControlProvider.class);

	public AliyunAccessControlProvider(Butler butler) {
		super(butler);
	}

	private static final int PAGE_SIZE = 50;

	private BeanConvertor<AccessControlModel, DescribeAccessControlListAttributeResponse> convertor = new AccessControlConvertor();

	@Override
	public BocloudResult list() {
		try {
			DescribeAccessControlListsRequest request = new DescribeAccessControlListsRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<AccessControlModel> models = Lists.newArrayList();
			DescribeAccessControlListsResponse response = this.getSLBClient().describeAccessControlLists(request);
			if (null == response) {
				return new BocloudResult(false, "访问控制列表查询失败");
			} else if (null == response.getBody().getAcls()) {
				return new BocloudResult(false, "访问控制列表查询失败");
			}
            List<DescribeAccessControlListsResponseBody.DescribeAccessControlListsResponseBodyAclsAcl> acls = response.getBody().getAcls().getAcl();
            Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "empty AccessControls");
			}
			for (DescribeAccessControlListsResponseBody.DescribeAccessControlListsResponseBodyAclsAcl acl : acls) {
				BocloudResult bocloudResult = this.detail(acl.getAclId());
				if (bocloudResult.isFailed()) {
					return new BocloudResult(false, "访问控制列表查询失败");
				}
				AccessControlModel controlModel = (AccessControlModel) bocloudResult.getData();
				models.add(controlModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getSLBClient().describeAccessControlLists(request);
				acls = response.getBody().getAcls().getAcl();
				for (DescribeAccessControlListsResponseBody.DescribeAccessControlListsResponseBodyAclsAcl acl : acls) {
					BocloudResult bocloudResult = this.detail(acl.getAclId());
					if (bocloudResult.isFailed()) {
						return new BocloudResult(false, "访问控制列表查询失败");
					}
					AccessControlModel controlModel = (AccessControlModel) bocloudResult.getData();
					models.add(controlModel);
				}
			}
			return new BocloudResult(true, models, "访问控制列表查询成功");
		} catch (Exception e) {
			logger.error("访问控制列表查询失败：", e);
			return new BocloudResult(false, "访问控制列表查询失败");
		}
	}

	@Override
	public BocloudResult detail(String aclId) {
		try {
			DescribeAccessControlListAttributeRequest request = new DescribeAccessControlListAttributeRequest();
			request.setAclId(aclId);
			request.setRegionId(this.getRegionId());
			DescribeAccessControlListAttributeResponse response = this.getSLBClient().describeAccessControlListAttribute(request);
			if (null == response) {
				return new BocloudResult(false, "访问控制查询失败");
			}
			return new BocloudResult(true, convertor.convertModel(response), "访问控制查询成功");
		} catch (Exception e) {
			logger.error("访问控制查询失败:", e);
			return new BocloudResult(false, "访问控制查询失败：" + e.getMessage());
		}
	}
}
