package com.bocloud.cmp.provider.loadbalance;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.slb20140515.models.*;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.ListenerConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.LoadBalancerConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.MemberConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.PoolConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import com.google.gson.Gson;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/16 13:53
 * @Description: 负载均衡实例
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunLoadBalancerProvider extends AliyunProvider implements LoadBalanceProvider {

    public AliyunLoadBalancerProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AliyunLoadBalancerProvider.class);

    private static final int PAGE_SIZE = 50;

    private BeanConvertor<LbModel, DescribeLoadBalancerAttributeResponse> convertor = new LoadBalancerConvertor();

    @Override
    public BocloudResult list() {
        try {
            DescribeLoadBalancersRequest request = new DescribeLoadBalancersRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            request.setRegionId(this.getRegionId());
            List<LbModel> models = Lists.newArrayList();
            DescribeLoadBalancersResponse response = this.getSLBClient().describeLoadBalancers(request);
            if (null == response) {
                return new BocloudResult(false, "负载均衡实例列表查询失败 : response is null");
            } else if (null == response.getBody().getLoadBalancers()) {
                return new BocloudResult(false, "负载均衡实例列表查询失败 : response body is null");
            }
            List<
                DescribeLoadBalancersResponseBody.DescribeLoadBalancersResponseBodyLoadBalancersLoadBalancer> loadBalancers =
                    response.getBody().getLoadBalancers().getLoadBalancer();
            Integer totalCount = response.getBody().getTotalCount() == null ? 0 : response.getBody().getTotalCount();
            logger.info("totalCount : " + totalCount);
            /*if (null == totalCount || totalCount == 0) {
            	return new BocloudResult(true, models, "empty loadBalancers");
            }*/
            logger.info("loadBalancers : " + JSON.toJSONString(loadBalancers));
            for (DescribeLoadBalancersResponseBody.DescribeLoadBalancersResponseBodyLoadBalancersLoadBalancer loadBalancer : loadBalancers) {
                BocloudResult detail = this.detail(loadBalancer.getLoadBalancerId());
                if(detail.isSuccess()) {
                    LbModel convertModel = JSONObject.parseObject(
                        JSON.toJSONString(detail.getData()), LbModel.class);
                    models.add(convertModel);
                }
                // List<DescribeLoadBalancersResponse.LoadBalancer.Tag> tags = loadBalancer.getTags();
                // List<TagModel> tagModels = Lists.newArrayList();
                // if (!ListTool.isEmpty(tags)) {
                // for (DescribeLoadBalancersResponse.LoadBalancer.Tag tag : tags) {
                // TagModel tagModel = new TagModel();
                // tagModel.setKey(tag.getTagKey());
                // tagModel.setValue(tag.getTagValue());
                // tagModel.setResourceId(loadBalancer.getLoadBalancerId());
                // tagModel.setResourceType(TagType.SLB.name());
                // tagModels.add(tagModel);
                // }
                // convertModel.setTagModels(tagModels);
                // }
            }
            /* int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "success");
            }*/
//            int i = 1;
//            while (loadBalancers.size() > 0) {
//                logger.info("当loadBalancers.size() > 0 查询负载均衡详情 ==== ");
//                request.setPageNumber(++i);
//                request.setPageSize(PAGE_SIZE);
//                response = this.getSLBClient().describeLoadBalancers(request);
//                loadBalancers = response.getBody().getLoadBalancers().getLoadBalancer();
//                logger.info("loadBalancers size " +  loadBalancers.size());
//                logger.info("loadBalancers 第一个元素 : " + JSON.toJSONString(loadBalancers.get(0)));
//                logger.info("loadBalancers : " + JSON.toJSONString(loadBalancers));
//                for (DescribeLoadBalancersResponseBody.DescribeLoadBalancersResponseBodyLoadBalancersLoadBalancer loadBalancer : loadBalancers) {
//                    BocloudResult detail = this.detail(loadBalancer.getLoadBalancerId());
//                    if(detail.isSuccess()) {
//                        LbModel convertModel = JSONObject.parseObject(
//                            JSON.toJSONString(detail.getData()), LbModel.class);
//                        models.add(convertModel);
//                    }
//                    // List<DescribeLoadBalancersResponse.LoadBalancer.Tag> tags = loadBalancer.getTags();
//                    // List<TagModel> tagModels = Lists.newArrayList();
//                    // if (!ListTool.isEmpty(tags)) {
//                    // for (DescribeLoadBalancersResponse.LoadBalancer.Tag tag : tags) {
//                    // TagModel tagModel = new TagModel();
//                    // tagModel.setKey(tag.getTagKey());
//                    // tagModel.setValue(tag.getTagValue());
//                    // tagModel.setResourceId(loadBalancer.getLoadBalancerId());
//                    // tagModel.setResourceType(TagType.SLB.name());
//                    // tagModels.add(tagModel);
//                    // }
//                    // convertModel.setTagModels(tagModels);
//                    // }
//                }
//            }
            /*for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getSLBClient().describeLoadBalancers(request);
                loadBalancers = response.getBody().getLoadBalancers().getLoadBalancer();
                for (DescribeLoadBalancersResponseBody.DescribeLoadBalancersResponseBodyLoadBalancersLoadBalancer loadBalancer : loadBalancers) {
                    LbModel convertModel = JSONObject.parseObject(
                        JSON.toJSONString(this.detail(loadBalancer.getLoadBalancerId()).getData()), LbModel.class);
                    // List<DescribeLoadBalancersResponse.LoadBalancer.Tag> tags = loadBalancer.getTags();
                    // List<TagModel> tagModels = Lists.newArrayList();
                    // if (!ListTool.isEmpty(tags)) {
                    // for (DescribeLoadBalancersResponse.LoadBalancer.Tag tag : tags) {
                    // TagModel tagModel = new TagModel();
                    // tagModel.setKey(tag.getTagKey());
                    // tagModel.setValue(tag.getTagValue());
                    // tagModel.setResourceId(loadBalancer.getLoadBalancerId());
                    // tagModel.setResourceType(TagType.SLB.name());
                    // tagModels.add(tagModel);
                    // }
                    // convertModel.setTagModels(tagModels);
                    // }
                    models.add(convertModel);
                }
            }*/
            return new BocloudResult(true, models, "负载均衡实例列表查询成功");
        } catch (Exception e) {
            logger.error("负载均衡实例列表查询失败：", e);
            return new BocloudResult(false, "负载均衡实例列表查询失败 : " + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(LbModel model) {
        try {
            CreateLoadBalancerRequest request = new CreateLoadBalancerRequest();
            request.setRegionId(this.getRegionId());
            // 付费模式
            request.setPayType(0 == model.getFeeType() ? "PayOnDemand" : "PrePay");
            // 设置主可用域和备可用域
            JSONObject zone = (JSONObject)model.getAvailabilityZones().get(0);
            for (String key : zone.keySet()) {
                if (!StringUtils.isEmpty(zone.getString("zone"))) {
                    if ("master".equalsIgnoreCase(key)) {
                        request.setMasterZoneId(zone.getString("zone"));
                    } else if ("slave".equalsIgnoreCase(key)) {
                        request.setSlaveZoneId(zone.getString("zone"));
                    }
                }
            }
            // 名称，不指定自动生成
            request.setLoadBalancerName(model.getName());
            // 实例规格
            request.setLoadBalancerSpec(model.getSpec());
            /***
             * 负载均衡实例的网络类型。取值： internet：创建公网负载均衡实例后，系统会分配一个公网IP地址，可以转发公网请求。 intranet：创建内网负载均衡实例后，系统会分配一个内网IP地址，仅可转发内网请求。
             */
            request.setAddressType(model.getLoadBalancerType());
            request.setVpcId(model.getVpcUuid());
            request.setVSwitchId(model.getSubnetUuid());
            // 实例的IP版本
            request.setAddressIPVersion(model.getIpAddressType());
            // 公网计费方式
            if ("internet".equalsIgnoreCase(model.getLoadBalancerType())) {
                request.setInternetChargeType(model.getInternetChargeType());
            }
            // 公网宽带值
            if (null != model.getBroadbandValue()) {
                request.setBandwidth(model.getBroadbandValue().intValue());
            }
            if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
                request.setResourceGroupId(model.getResourceGroupUuid());
            }
            CreateLoadBalancerResponse response = this.getSLBClient().createLoadBalancer(request);
            if (null == response) {
                return new BocloudResult(false, "负载均衡实例创建失败");
            } else if (null == response.getBody().getLoadBalancerId()) {
                return new BocloudResult(false, "负载均衡实例创建失败");
            }
            return new BocloudResult(true, this.getAvailable(response.getBody().getLoadBalancerId()), "负载均衡实例创建成功");
        } catch (Exception e) {
            logger.error("负载均衡实例创建失败:", e);
            return new BocloudResult(false, "负载均衡实例创建失败：" + e.getMessage());
        }
    }

    private LbModel getAvailable(String lbId) {
        DescribeLoadBalancerAttributeRequest request = new DescribeLoadBalancerAttributeRequest();
        request.setLoadBalancerId(lbId);
        request.setRegionId(this.getRegionId());
        LbModel model = new LbModel();
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    logger.error("Get error message:", e);
                    Thread.currentThread().interrupt();
                }
                DescribeLoadBalancerAttributeResponse response =
                    this.getSLBClient().describeLoadBalancerAttribute(request);
                if (null == response) {
                    flag = false;
                    break;
                }
                if (("active".equalsIgnoreCase(response.getBody().getLoadBalancerStatus()))) {
                    model = convertor.convertModel(response);
                    flag = false;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    model = convertor.convertModel(response);
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("get loadBalancer error:", e);
        }
        return model;
    }

    @Override
    public BocloudResult update(LbModel model, String action) {
        try {
            switch (action) {
                case "modify":
                    SetLoadBalancerNameRequest request = new SetLoadBalancerNameRequest();
                    request.setLoadBalancerId(model.getLbUuid());
                    request.setRegionId(this.getRegionId());
                    request.setLoadBalancerName(model.getName());
                    SetLoadBalancerNameResponse response = this.getSLBClient().setLoadBalancerName(request);
                    if (null == response) {
                        return new BocloudResult(false, "负载均衡实例修改失败");
                    }
                    return new BocloudResult(true, response.getBody().getRequestId(), "负载均衡实例修改成功");
                case "deleteProtection":
                    SetLoadBalancerDeleteProtectionRequest deleteRequest = new SetLoadBalancerDeleteProtectionRequest();
                    deleteRequest.setRegionId(this.getRegionId());
                    deleteRequest.setDeleteProtection(model.getDeleteProtection());
                    deleteRequest.setLoadBalancerId(model.getLbUuid());
                    SetLoadBalancerDeleteProtectionResponse deleteProtectionResponse =
                        this.getSLBClient().setLoadBalancerDeleteProtection(deleteRequest);
                    if (null == deleteProtectionResponse) {
                        return new BocloudResult(false, "负载均衡实例修改失败");
                    }
                    return new BocloudResult(true, deleteProtectionResponse.getBody().getRequestId(), "负载均衡实例修改成功");
                case "modificationProtection":
                    SetLoadBalancerModificationProtectionRequest modificationProtectionRequest =
                        new SetLoadBalancerModificationProtectionRequest();
                    modificationProtectionRequest.setLoadBalancerId(model.getLbUuid());
                    modificationProtectionRequest.setModificationProtectionStatus(model.getModificationProtection());
                    modificationProtectionRequest
                        .setModificationProtectionReason(model.getModificationProtectionReason());
                    modificationProtectionRequest.setRegionId(this.getRegionId());
                    SetLoadBalancerModificationProtectionResponse modificationProtectionResponse =
                        this.getSLBClient().setLoadBalancerModificationProtection(modificationProtectionRequest);
                    if (null == modificationProtectionResponse) {
                        return new BocloudResult(false, "负载均衡实例修改失败");
                    }
                    return new BocloudResult(true, modificationProtectionResponse.getBody().getRequestId(),
                        "负载均衡实例修改成功");
                default:
                    return new BocloudResult(false, "暂不支持的更新操作");
            }
        } catch (Exception e) {
            logger.error("负载均衡实例修改失败:", e);
            return new BocloudResult(false, "负载均衡实例修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String lbId) {
        try {
            DeleteLoadBalancerRequest request = new DeleteLoadBalancerRequest();
            request.setLoadBalancerId(lbId);
            request.setRegionId(this.getRegionId());
            DeleteLoadBalancerResponse response = this.getSLBClient().deleteLoadBalancer(request);
            if (null == response) {
                return new BocloudResult(false, "负载均衡实例删除失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "负载均衡实例删除成功");
        } catch (Exception e) {
            logger.error("负载均衡实例删除失败:", e);
            return new BocloudResult(false, "负载均衡实例删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String lbId) {
        try {
            DescribeLoadBalancerAttributeRequest request = new DescribeLoadBalancerAttributeRequest();
            request.setLoadBalancerId(lbId);
            request.setRegionId(this.getRegionId());
            DescribeLoadBalancerAttributeResponse response = this.getSLBClient().describeLoadBalancerAttribute(request);
            if (null == response) {
                return new BocloudResult(false, "负载均衡实例详情查询失败");
            }
            return new BocloudResult(true, convertor.convertModel(response), "负载均衡实例详情查询成功");
        } catch (Exception e) {
            logger.error("负载均衡实例详情查询失败:", e);
            return new BocloudResult(false, "负载均衡实例详情查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modifyLbSpec(LbModel lbModel) {
        try {
            ModifyLoadBalancerInstanceSpecRequest request = new ModifyLoadBalancerInstanceSpecRequest();
            request.setLoadBalancerId(lbModel.getLbUuid());
            request.setRegionId(this.getRegionId());
            request.setLoadBalancerSpec(lbModel.getSpec());
            ModifyLoadBalancerInstanceSpecResponse response =
                this.getSLBClient().modifyLoadBalancerInstanceSpec(request);
            if (null == response) {
                return new BocloudResult(false, "负载均衡实例规格修改失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "负载均衡实例规格修改成功");
        } catch (Exception e) {
            logger.error("负载均衡实例规格修改失败:", e);
            return new BocloudResult(false, "负载均衡实例规格修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult setLbDeleteProtection(LbModel lbModel) {
        try {
            SetLoadBalancerDeleteProtectionRequest request = new SetLoadBalancerDeleteProtectionRequest();
            request.setLoadBalancerId(lbModel.getLbUuid());
            request.setRegionId(this.getRegionId());
            /***
             * 负载均衡删除保护状态。 取值：on|off。
             */
            request.setDeleteProtection(lbModel.getDeleteProtection());
            SetLoadBalancerDeleteProtectionResponse response =
                this.getSLBClient().setLoadBalancerDeleteProtection(request);
            if (null == response) {
                return new BocloudResult(false, "实例删除保护状态设置失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "实例删除保护状态设置成功");
        } catch (Exception e) {
            logger.error("实例删除保护状态设置失败:", e);
            return new BocloudResult(false, "实例删除保护状态设置失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult createListener(LbListenerModel lbListenerModel) {
        try {
            String protocol = lbListenerModel.getProtocol();
            Gson gson = new Gson();
            switch (protocol) {
                case "TCP":
                    CreateLoadBalancerTCPListenerRequest tcp = new CreateLoadBalancerTCPListenerRequest();
                    // 监听的带宽峰值
                    tcp.setBandwidth(lbListenerModel.getBandwidth());
                    // 监听器端口
                    tcp.setListenerPort(lbListenerModel.getPort());
                    // 负载均衡器id
                    tcp.setLoadBalancerId(lbListenerModel.getLbUuid());
                    // 调度算法
                    if ("ch".equalsIgnoreCase(lbListenerModel.getScheduler())) {
                        tcp.setScheduler(lbListenerModel.getHash());
                    } else {
                        tcp.setScheduler(lbListenerModel.getScheduler());
                    }
                    // 会话保持的超时时间
                    tcp.setPersistenceTimeout(lbListenerModel.getPersistenceTimeout());
                    // 连接超时时间
                    tcp.setEstablishedTimeout(lbListenerModel.getConnLimit());
                    // 健康检查连续成功多少次后，将后端服务器的健康检查状态由fail判定为success
                    tcp.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    // 健康检查连续失败多少次后，将后端服务器的健康检查状态由success判定为fail
                    tcp.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    // 每次健康检查响应的最大超时时间
                    tcp.setHealthCheckConnectTimeout(lbListenerModel.getHealthCheckConnectTimeout());
                    // 健康检查使用的端口
                    tcp.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    //// 健康检查的时间间隔
                    tcp.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    // 健康检查类型
                    tcp.setHealthCheckType(lbListenerModel.getHealthCheckType());
                    // 服务器组id
                    tcp.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    // 主备服务器组ID
                    tcp.setMasterSlaveServerGroupId(lbListenerModel.getMasterSlaveServerGroupId());
                    // 监听器名称
                    tcp.setDescription(lbListenerModel.getName());
                    //// 健康检查的URI
                    tcp.setHealthCheckURI(lbListenerModel.getHealthCheckURI());
                    // 健康检查的域名
                    tcp.setHealthCheckDomain(lbListenerModel.getHealthCheckDomain());
                    // 健康检查正常的HTTP状态码，多个状态码用逗号（,）分割
                    tcp.setHealthCheckHttpCode(lbListenerModel.getHealthCheckHttpCode());
                    // 是否开启访问控制功能
                    if (null != lbListenerModel.getAclStatus()) {
                        tcp.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                        // 访问控制类型
                        tcp.setAclType(lbListenerModel.getAclType());
                        // 监听绑定的访问策略组ID
                        tcp.setAclId(lbListenerModel.getAclId());
                    }
                    tcp.setRegionId(this.getRegionId());
                    CreateLoadBalancerTCPListenerResponse tcpResponse =
                        this.getSLBClient().createLoadBalancerTCPListener(tcp);
                    if (null == tcpResponse) {
                        return new BocloudResult(false, "负载均衡TCP监听器创建失败");
                    }
                    // 开启监听
                    BocloudResult opTcpResult = this.startLoadBalancerListener(lbListenerModel.getPort(),
                        lbListenerModel.getLbUuid(), protocol);
                    if (opTcpResult.isFailed()) {
                        logger.error("开启监听器失败，请手动重试");
                    }
                    BocloudResult tcpResult =
                        this.detailListener(lbListenerModel.getLbUuid(), lbListenerModel.getPort(), "TCP");
                    LbListenerModel tcpListener = (LbListenerModel)tcpResult.getData();
                    tcpListener.setLbUuid(lbListenerModel.getLbUuid());
                    return new BocloudResult(true, tcpListener, "负载均衡TCP监听器创建成功");
                case "UDP":
                    CreateLoadBalancerUDPListenerRequest udp = new CreateLoadBalancerUDPListenerRequest();
                    udp.setLoadBalancerId(lbListenerModel.getLbUuid());
                    udp.setListenerPort(lbListenerModel.getPort());
                    udp.setDescription(lbListenerModel.getName());
                    udp.setScheduler(lbListenerModel.getScheduler());
                    udp.setRegionId(this.getRegionId());
                    // 是否开启访问控制功能
                    if (null != lbListenerModel.getAclStatus()) {
                        udp.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                        // 访问控制类型
                        udp.setAclType(lbListenerModel.getAclType());
                        // 监听绑定的访问策略组ID
                        udp.setAclId(lbListenerModel.getAclId());
                    }
                    udp.setBandwidth(lbListenerModel.getBandwidth());
                    udp.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    udp.setMasterSlaveServerGroupId(lbListenerModel.getMasterSlaveServerGroupId());
                    udp.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    udp.setHealthCheckConnectTimeout(lbListenerModel.getHealthCheckConnectTimeout());
                    udp.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    udp.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    udp.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    // UDP监听健康检查的请求串
                    udp.setHealthCheckReq(lbListenerModel.getHealthCheckReq());
                    // UDP监听健康检查的响应串
                    udp.setHealthCheckExp(lbListenerModel.getHealthCheckExp());
                    CreateLoadBalancerUDPListenerResponse udpResponse =
                        this.getSLBClient().createLoadBalancerUDPListener(udp);
                    if (null == udpResponse) {
                        return new BocloudResult(false, "负载均衡UDP监听器创建失败");
                    }
                    // 开启监听
                    BocloudResult opUdpResult = this.startLoadBalancerListener(lbListenerModel.getPort(),
                        lbListenerModel.getLbUuid(), protocol);
                    if (opUdpResult.isFailed()) {
                        logger.error("开启监听器失败，请手动重试");
                    }
                    BocloudResult udpResult =
                        this.detailListener(lbListenerModel.getLbUuid(), lbListenerModel.getPort(), "UDP");
                    LbListenerModel udpListener = (LbListenerModel)udpResult.getData();
                    udpListener.setLbUuid(lbListenerModel.getLbUuid());
                    return new BocloudResult(true, udpListener, "负载均衡UDP监听器创建成功");
                case "HTTP":
                    CreateLoadBalancerHTTPListenerRequest http = new CreateLoadBalancerHTTPListenerRequest();
                    http.setLoadBalancerId(lbListenerModel.getLbUuid());
                    http.setListenerPort(lbListenerModel.getPort());
                    http.setDescription(lbListenerModel.getName());
                    http.setScheduler(lbListenerModel.getScheduler());
                    http.setRegionId(this.getRegionId());
                    // 是否开启HTTP至HTTPS的转发
                    http.setListenerForward(lbListenerModel.getListenerForward());
                    // HTTP至HTTPS的监听转发端口
                    http.setForwardPort(lbListenerModel.getForwardPort());
                    // 是否开启会话保持
                    if (null != lbListenerModel.getStickySession()) {
                        http.setStickySession(lbListenerModel.getStickySession() ? "on" : "off");
                    }
                    // cookie的处理方式
                    http.setStickySessionType(lbListenerModel.getStickySessionType());
                    // 服务器上配置的Cookie
                    http.setCookie(lbListenerModel.getCookie());
                    // Cookie超时时间
                    http.setCookieTimeout(lbListenerModel.getCookieTimeout());
                    // 是否开启访问控制功能
                    if (null != lbListenerModel.getAclStatus()) {
                        http.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                        // 访问控制类型
                        http.setAclType(lbListenerModel.getAclType());
                        // 监听绑定的访问策略组ID
                        http.setAclId(lbListenerModel.getAclId());
                    }
                    http.setBandwidth(lbListenerModel.getBandwidth());
                    // 指定连接空闲超时时间
                    http.setIdleTimeout(lbListenerModel.getIdleTimeout());
                    // 指定请求超时时间
                    http.setRequestTimeout(lbListenerModel.getRequestTimeout());
                    // 是否开启Gzip压缩，对特定文件类型进行压缩
                    if (null != lbListenerModel.getGzip()) {
                        http.setGzip(lbListenerModel.getGzip() ? "on" : "off");
                    }
                    // 是否开启通过X-Forwarded-For头字段获取来访者真实 IP
                    if (null != lbListenerModel.getXForwardedFor()) {
                        http.setXForwardedFor(lbListenerModel.getXForwardedFor() ? "on" : "off");
                    }
                    // 是否通过SLB-ID头字段获取负载均衡实例ID
                    if (null != lbListenerModel.getXForwardedFor_SLBID()) {
                    	
                        http.setXForwardedFor_SLBID(lbListenerModel.getXForwardedFor_SLBID() ? "on" : "off");
                    }
                    // 是否通过SLB-IP头字段获取客户端请求的真实IP
                    if (null != lbListenerModel.getXForwardedFor_SLBIP()) {
                        http.setXForwardedFor_SLBIP(lbListenerModel.getXForwardedFor_SLBIP() ? "on" : "off");
                    }
                    // 是否通过X-Forwarded-Proto头字段获取负载均衡实例的监听协议
                    if (null != lbListenerModel.getXForwardedFor_proto()) {
                        http.setXForwardedFor_proto(lbListenerModel.getXForwardedFor_proto() ? "on" : "off");
                    }
                    http.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    // 是否开启健康检查
                    if (null != lbListenerModel.getHealthCheck()) {
                        http.setHealthCheck(lbListenerModel.getHealthCheck() ? "on" : "off");
                    }
                    http.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    http.setHealthCheckURI(lbListenerModel.getHealthCheckURI());
                    http.setHealthCheckDomain(lbListenerModel.getHealthCheckDomain());
                    http.setHealthCheckHttpCode(lbListenerModel.getHealthCheckHttpCode());
                    // 接收来自运行状况检查的响应需要等待的时间
                    http.setHealthCheckTimeout(lbListenerModel.getHealthCheckTimeout());
                    http.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    http.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    http.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    CreateLoadBalancerHTTPListenerResponse httpResponse =
                        this.getSLBClient().createLoadBalancerHTTPListener(http);
                    if (null == httpResponse) {
                        return new BocloudResult(false, "负载均衡HTTP监听器创建失败");
                    }
                    // 开启监听
                    BocloudResult opHttpResult = this.startLoadBalancerListener(lbListenerModel.getPort(),
                        lbListenerModel.getLbUuid(), protocol);
                    if (opHttpResult.isFailed()) {
                        logger.error("开启监听器失败，请手动重试");
                    }
                    BocloudResult httpResult =
                        this.detailListener(lbListenerModel.getLbUuid(), lbListenerModel.getPort(), "HTTP");
                    LbListenerModel httpListener = (LbListenerModel)httpResult.getData();
                    httpListener.setLbUuid(lbListenerModel.getLbUuid());
                    return new BocloudResult(true, httpListener, "负载均衡HTTP监听器创建成功");
                case "HTTPS":
                    CreateLoadBalancerHTTPSListenerRequest https = new CreateLoadBalancerHTTPSListenerRequest();
                    https.setLoadBalancerId(lbListenerModel.getLbUuid());
                    https.setListenerPort(lbListenerModel.getPort());
                    https.setDescription(lbListenerModel.getName());
                    https.setScheduler(lbListenerModel.getScheduler());
                    https.setStickySession(lbListenerModel.getStickySession() ? "on" : "off");
                    https.setStickySessionType(lbListenerModel.getStickySessionType());
                    https.setCookie(lbListenerModel.getCookie());
                    https.setEnableHttp2(lbListenerModel.getEnableHttp2());
                    https.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                    https.setAclType(lbListenerModel.getAclType());
                    https.setAclId(lbListenerModel.getAclId());
                    https.setBandwidth(lbListenerModel.getBandwidth());
                    https.setIdleTimeout(lbListenerModel.getIdleTimeout());
                    https.setRequestTimeout(lbListenerModel.getRequestTimeout());
                    https.setGzip(lbListenerModel.getGzip() ? "on" : "off");
                    https.setXForwardedFor(lbListenerModel.getXForwardedFor() ? "on" : "off");
                    https.setXForwardedFor_SLBID(lbListenerModel.getXForwardedFor_SLBID() ? "on" : "off");
                    https.setXForwardedFor_SLBIP(lbListenerModel.getXForwardedFor_SLBIP() ? "on" : "off");
                    https.setXForwardedFor_proto(lbListenerModel.getXForwardedFor_proto() ? "on" : "off");
                    https.setRegionId(this.getRegionId());
                    // TODO
                    // 因缺少ssl证书，无法继续开发
                    return new BocloudResult(true, "负载均衡HTTPS监听器创建成功");
                default:
                    return new BocloudResult(false, "不支持的负载均衡协议");
            }
        } catch (Exception e) {
            logger.error("负载均衡监听器创建失败:", e);
            return new BocloudResult(false, "负载均衡监听器创建失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult startLoadBalancerListener(Integer listenerPort, String lbId, String listenerProtocol) {
        try {
            StartLoadBalancerListenerRequest request = new StartLoadBalancerListenerRequest();
            request.setListenerPort(listenerPort);
            request.setLoadBalancerId(lbId);
            request.setRegionId(this.getRegionId());
            request.setListenerProtocol(listenerProtocol);
            StartLoadBalancerListenerResponse response = this.getSLBClient().startLoadBalancerListener(request);
            if (null == response) {
                return new BocloudResult(false, "启动监听失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "启动监听成功");
        } catch (Exception e) {
            logger.error("启动监听失败:", e);
            return new BocloudResult(false, "启动监听失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult stopLoadBalancerListener(Integer listenerPort, String lbId, String listenerProtocol) {
        try {
            StopLoadBalancerListenerRequest request = new StopLoadBalancerListenerRequest();
            request.setListenerPort(listenerPort);
            request.setRegionId(this.getRegionId());
            request.setLoadBalancerId(lbId);
            request.setListenerProtocol(listenerProtocol);
            StopLoadBalancerListenerResponse response = this.getSLBClient().stopLoadBalancerListener(request);
            if (null == response) {
                return new BocloudResult(false, "停止监听失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "停止监听成功");
        } catch (Exception e) {
            logger.error("停止监听失败:", e);
            return new BocloudResult(false, "停止监听失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detailListener(String loadBalancerId, Integer listenerPort, String type) {
        try {
            switch (type.toUpperCase()) {
                case "TCP":
                    DescribeLoadBalancerTCPListenerAttributeRequest tcpRequest =
                        new DescribeLoadBalancerTCPListenerAttributeRequest();
                    tcpRequest.setLoadBalancerId(loadBalancerId);
                    tcpRequest.setRegionId(this.getRegionId());
                    tcpRequest.setListenerPort(listenerPort);
                    DescribeLoadBalancerTCPListenerAttributeResponse tcpResponse =
                        this.getSLBClient().describeLoadBalancerTCPListenerAttribute(tcpRequest);
                    if (null == tcpResponse) {
                        return new BocloudResult(false, "查询TCP监听器实例详情失败");
                    }
                    JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(tcpResponse));
                    JSONObject tcpBody = body.getJSONObject("body");
                    BeanConvertor<LbListenerModel, JSONObject> tcpListenerConvertor =
                        new ListenerConvertor(loadBalancerId, "tcp");
                    return new BocloudResult(true, tcpListenerConvertor.convertModel(tcpBody), "查询TCP监听器实例详情成功");
                case "UDP":
                    DescribeLoadBalancerUDPListenerAttributeRequest udpRequest =
                        new DescribeLoadBalancerUDPListenerAttributeRequest();
                    udpRequest.setLoadBalancerId(loadBalancerId);
                    udpRequest.setRegionId(this.getRegionId());
                    udpRequest.setListenerPort(listenerPort);
                    DescribeLoadBalancerUDPListenerAttributeResponse udpResponse =
                        this.getSLBClient().describeLoadBalancerUDPListenerAttribute(udpRequest);
                    if (null == udpResponse) {
                        return new BocloudResult(false, "查询UDP监听器实例详情失败");
                    }
                    JSONObject udpResponseBody = JSONObject.parseObject(JSONObject.toJSONString(udpResponse));
                    JSONObject udpBody = udpResponseBody.getJSONObject("body");
                    BeanConvertor<LbListenerModel, JSONObject> udpListenerConvertor =
                        new ListenerConvertor(loadBalancerId, "udp");
                    return new BocloudResult(true, udpListenerConvertor.convertModel(udpBody), "查询UDP监听器实例详情成功");
                case "HTTP":
                    DescribeLoadBalancerHTTPListenerAttributeRequest httpRequest =
                        new DescribeLoadBalancerHTTPListenerAttributeRequest();
                    httpRequest.setLoadBalancerId(loadBalancerId);
                    httpRequest.setListenerPort(listenerPort);
                    httpRequest.setRegionId(this.getRegionId());
                    DescribeLoadBalancerHTTPListenerAttributeResponse httpResponse =
                        this.getSLBClient().describeLoadBalancerHTTPListenerAttribute(httpRequest);
                    if (null == httpResponse) {
                        return new BocloudResult(false, "查询HTTP监听器实例详情失败");
                    }
                    JSONObject httpResponseBody = JSONObject.parseObject(JSONObject.toJSONString(httpResponse));
                    JSONObject httpBody = httpResponseBody.getJSONObject("body");
                    BeanConvertor<LbListenerModel, JSONObject> httpListenerConvertor =
                        new ListenerConvertor(loadBalancerId, "http");
                    return new BocloudResult(true, httpListenerConvertor.convertModel(httpBody), "查询HTTP监听器实例详情成功");
                default:
                    return new BocloudResult(false, "不支持的监听器协议类型");
            }
        } catch (Exception e) {
            logger.error("查询监听器实例详情失败:", e);
            return new BocloudResult(false, "查询监听器实例详情失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listListener(String lbId) {
        try {
            DescribeLoadBalancerAttributeRequest request = new DescribeLoadBalancerAttributeRequest();
            request.setLoadBalancerId(lbId);
            request.setRegionId(this.getRegionId());
            DescribeLoadBalancerAttributeResponse response = this.getSLBClient().describeLoadBalancerAttribute(request);
            if (null == response) {
                return new BocloudResult(false, "查询负载均衡实例监听器列表失败");
            }
            // 获取所有的监听器端口,协议
            List<LbListenerModel> listeners = Lists.newArrayList();
            List<
                DescribeLoadBalancerAttributeResponseBody.DescribeLoadBalancerAttributeResponseBodyListenerPortsAndProtocalListenerPortAndProtocal> protocal =
                    response.getBody().getListenerPortsAndProtocal().getListenerPortAndProtocal();
            for (DescribeLoadBalancerAttributeResponseBody.DescribeLoadBalancerAttributeResponseBodyListenerPortsAndProtocalListenerPortAndProtocal portAndProtocal : protocal) {
                // 获取监听器实例的数据详情
                BocloudResult result = this.detailListener(response.getBody().getLoadBalancerId(),
                    portAndProtocal.getListenerPort(), portAndProtocal.getListenerProtocal());
                if (result.isFailed()) {
                    return new BocloudResult(false, "查询负载均衡实例监听器失败");
                }
                listeners.add((LbListenerModel)result.getData());
            }
            return new BocloudResult(true, listeners, "查询负载均衡实例监听器列表成功");
        } catch (Exception e) {
            logger.error("查询负载均衡实例监听器列表失败:", e);
            return new BocloudResult(false, "查询负载均衡实例监听器列表失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listBackendServer(String vServerGroupId) {
        try {
            DescribeVServerGroupAttributeRequest request = new DescribeVServerGroupAttributeRequest();
            request.setVServerGroupId(vServerGroupId);
            request.setRegionId(this.getRegionId());
            DescribeVServerGroupAttributeResponse response = this.getSLBClient().describeVServerGroupAttribute(request);
            if (null == response) {
                return new BocloudResult(false, "查询后端服务列表失败");
            }
            BeanConvertor<LbMemberModel,
                DescribeVServerGroupAttributeResponseBody.DescribeVServerGroupAttributeResponseBodyBackendServersBackendServer> poolConvertor =
                    new MemberConvertor(vServerGroupId);
            List<
                DescribeVServerGroupAttributeResponseBody.DescribeVServerGroupAttributeResponseBodyBackendServersBackendServer> backendServer =
                    response.getBody().getBackendServers().getBackendServer();
            return new BocloudResult(true, poolConvertor.convertList(backendServer), "查询后端服务列表成功");
        } catch (Exception e) {
            logger.error("查询后端服务列表失败:", e);
            return new BocloudResult(false, "查询后端服务列表失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listVServerGroup(String loadBalancerId) {
        try {
            DescribeVServerGroupsRequest request = new DescribeVServerGroupsRequest();
            request.setLoadBalancerId(loadBalancerId);
            request.setIncludeListener(true);
            request.setRegionId(this.getRegionId());
            DescribeVServerGroupsResponse response = this.getSLBClient().describeVServerGroups(request);
            if (null == response) {
                return new BocloudResult(false, "查询服务器组列表失败");
            }
            BeanConvertor<LbPoolModel,
                DescribeVServerGroupsResponseBody.DescribeVServerGroupsResponseBodyVServerGroupsVServerGroup> poolConvertor =
                    new PoolConvertor(loadBalancerId);
            List<
                DescribeVServerGroupsResponseBody.DescribeVServerGroupsResponseBodyVServerGroupsVServerGroup> vServerGroup =
                    response.getBody().getVServerGroups().getVServerGroup();
            return new BocloudResult(true, poolConvertor.convertList(vServerGroup), "查询服务器组列表成功");
        } catch (Exception e) {
            logger.error("查询服务器组列表失败:", e);
            return new BocloudResult(false, "查询服务器组列表失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult updateListener(LbListenerModel lbListenerModel) {
        try {
            Gson gson = new Gson();
            String type = lbListenerModel.getProtocol() == null ? "" : lbListenerModel.getProtocol().toUpperCase();
            switch (type) {
                case "TCP":

                    SetLoadBalancerTCPListenerAttributeRequest tcp = new SetLoadBalancerTCPListenerAttributeRequest();
                    tcp.setRegionId(this.getRegionId());
                    // 监听的带宽峰值
                    tcp.setBandwidth(lbListenerModel.getBandwidth());
                    // 监听器端口
                    tcp.setListenerPort(lbListenerModel.getPort());
                    // 负载均衡器id
                    tcp.setLoadBalancerId(lbListenerModel.getLbUuid());
                    // 调度算法
                    tcp.setScheduler(lbListenerModel.getScheduler());
                    // 会话保持的超时时间
                    tcp.setPersistenceTimeout(lbListenerModel.getPersistenceTimeout());
                    // 连接超时时间
                    tcp.setEstablishedTimeout(lbListenerModel.getConnLimit());
                    // 健康检查连续成功多少次后，将后端服务器的健康检查状态由fail判定为success
                    tcp.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    // 健康检查连续失败多少次后，将后端服务器的健康检查状态由success判定为fail
                    tcp.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    // 每次健康检查响应的最大超时时间
                    tcp.setHealthCheckConnectTimeout(lbListenerModel.getHealthCheckConnectTimeout());
                    // 健康检查使用的端口
                    tcp.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    //// 健康检查的时间间隔
                    tcp.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    // 健康检查类型
                    tcp.setHealthCheckType(lbListenerModel.getHealthCheckType());
                    // 服务器组id
                    tcp.setVServerGroup("on");
                    tcp.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    // 主备服务器组ID
                    tcp.setMasterSlaveServerGroupId(lbListenerModel.getMasterSlaveServerGroupId());
                    // 监听器名称
                    tcp.setDescription(lbListenerModel.getName());
                    //// 健康检查的URI
                    tcp.setHealthCheckURI(lbListenerModel.getHealthCheckURI());
                    // 健康检查的域名
                    tcp.setHealthCheckDomain(lbListenerModel.getHealthCheckDomain());
                    // 健康检查正常的HTTP状态码，多个状态码用逗号（,）分割
                    tcp.setHealthCheckHttpCode(lbListenerModel.getHealthCheckHttpCode());
                    // 是否开启访问控制功能
                    tcp.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                    // 访问控制类型
                    tcp.setAclType(lbListenerModel.getAclType());
                    // 监听绑定的访问策略组ID
                    tcp.setAclId(lbListenerModel.getAclId());
                    SetLoadBalancerTCPListenerAttributeResponse tcpResponse =
                        this.getSLBClient().setLoadBalancerTCPListenerAttribute(tcp);
                    if (null == tcpResponse || null == tcpResponse.getBody().getRequestId()) {
                        return new BocloudResult(false, "更新负载均衡TCP监听器失败");
                    }
                    return new BocloudResult(true, lbListenerModel, "更新负载均衡TCP监听器成功");
                case "UDP":
                    SetLoadBalancerUDPListenerAttributeRequest udp = new SetLoadBalancerUDPListenerAttributeRequest();
                    udp.setRegionId(this.getRegionId());
                    udp.setLoadBalancerId(lbListenerModel.getLbUuid());
                    udp.setListenerPort(lbListenerModel.getPort());
                    udp.setDescription(lbListenerModel.getName());
                    udp.setScheduler(lbListenerModel.getScheduler());
                    udp.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                    udp.setAclType(lbListenerModel.getAclType());
                    udp.setAclId(lbListenerModel.getAclId());
                    udp.setVServerGroup("on");
                    udp.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    udp.setMasterSlaveServerGroupId(lbListenerModel.getMasterSlaveServerGroupId());
                    udp.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    udp.setHealthCheckConnectTimeout(lbListenerModel.getHealthCheckConnectTimeout());
                    udp.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    udp.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    udp.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    // UDP监听健康检查的请求串
                    udp.setHealthCheckReq(lbListenerModel.getHealthCheckReq());
                    // UDP监听健康检查的响应串
                    udp.setHealthCheckExp(lbListenerModel.getHealthCheckExp());
                    SetLoadBalancerUDPListenerAttributeResponse udpResponse =
                        this.getSLBClient().setLoadBalancerUDPListenerAttribute(udp);
                    if (null == udpResponse || null == udpResponse.getBody().getRequestId()) {
                        return new BocloudResult(false, "更新负载均衡UDP监听器失败");
                    }
                    return new BocloudResult(true, lbListenerModel, "更新负载均衡UDP监听器成功");
                case "HTTP":
                    SetLoadBalancerHTTPListenerAttributeRequest http =
                        new SetLoadBalancerHTTPListenerAttributeRequest();
                    http.setRegionId(this.getRegionId());
                    http.setLoadBalancerId(lbListenerModel.getLbUuid());
                    http.setListenerPort(lbListenerModel.getPort());
                    http.setDescription(lbListenerModel.getName());
                    http.setScheduler(lbListenerModel.getScheduler());
                    // 是否开启会话保持
                    http.setStickySession(lbListenerModel.getStickySession() ? "on" : "off");
                    // cookie的处理方式
                    http.setStickySessionType(lbListenerModel.getStickySessionType());
                    // 服务器上配置的Cookie
                    http.setCookie(lbListenerModel.getCookie());
                    // Cookie超时时间
                    http.setCookieTimeout(lbListenerModel.getCookieTimeout());
                    http.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                    http.setAclType(lbListenerModel.getAclType());
                    http.setAclId(lbListenerModel.getAclId());
                    http.setBandwidth(lbListenerModel.getBandwidth());
                    // 指定连接空闲超时时间
                    http.setIdleTimeout(lbListenerModel.getIdleTimeout());
                    // 指定请求超时时间
                    http.setRequestTimeout(lbListenerModel.getRequestTimeout());
                    // 是否开启Gzip压缩，对特定文件类型进行压缩
                    http.setGzip(lbListenerModel.getGzip() ? "on" : "off");
                    // 是否开启通过X-Forwarded-For头字段获取来访者真实 IP
                    http.setXForwardedFor(lbListenerModel.getXForwardedFor() ? "on" : "off");
                    // 是否通过SLB-ID头字段获取负载均衡实例ID
                    http.setXForwardedFor_SLBID(lbListenerModel.getXForwardedFor_SLBID() ? "on" : "off");
                    // 是否通过SLB-IP头字段获取客户端请求的真实IP
                    http.setXForwardedFor_SLBIP(lbListenerModel.getXForwardedFor_SLBIP() ? "on" : "off");
                    // 是否通过X-Forwarded-Proto头字段获取负载均衡实例的监听协议
                    http.setXForwardedFor_proto(lbListenerModel.getXForwardedFor_proto() ? "on" : "off");
                    http.setVServerGroupId(lbListenerModel.getDefaultPoolUuid());
                    // 是否开启健康检查
                    http.setHealthCheck(lbListenerModel.getHealthCheck() ? "on" : "off");
                    http.setHealthCheckConnectPort(lbListenerModel.getHealthCheckConnectPort());
                    http.setHealthCheckURI(lbListenerModel.getHealthCheckURI());
                    http.setHealthCheckDomain(lbListenerModel.getHealthCheckDomain());
                    http.setHealthCheckHttpCode(lbListenerModel.getHealthCheckHttpCode());
                    // 接收来自运行状况检查的响应需要等待的时间
                    http.setHealthCheckTimeout(lbListenerModel.getHealthCheckTimeout());
                    http.setHealthCheckInterval(lbListenerModel.getHealthCheckInterval());
                    http.setHealthyThreshold(lbListenerModel.getHealthyThreshold());
                    http.setUnhealthyThreshold(lbListenerModel.getUnhealthyThreshold());
                    SetLoadBalancerHTTPListenerAttributeResponse httpResponse =
                        this.getSLBClient().setLoadBalancerHTTPListenerAttribute(http);
                    if (null == httpResponse || null == httpResponse.getBody().getRequestId()) {
                        return new BocloudResult(false, "更新负载均衡HTTP监听器失败");
                    }
                    return new BocloudResult(true, lbListenerModel, "更新负载均衡HTTP监听器成功");
                case "HTTPS":
                    CreateLoadBalancerHTTPSListenerRequest https = new CreateLoadBalancerHTTPSListenerRequest();
                    https.setRegionId(this.getRegionId());
                    https.setLoadBalancerId(lbListenerModel.getLbUuid());
                    https.setListenerPort(lbListenerModel.getPort());
                    https.setDescription(lbListenerModel.getName());
                    https.setScheduler(lbListenerModel.getScheduler());
                    https.setStickySession(lbListenerModel.getStickySession() ? "on" : "off");
                    https.setStickySessionType(lbListenerModel.getStickySessionType());
                    https.setCookie(lbListenerModel.getCookie());
                    https.setEnableHttp2(lbListenerModel.getEnableHttp2());
                    https.setAclStatus(lbListenerModel.getAclStatus() ? "on" : "off");
                    https.setAclType(lbListenerModel.getAclType());
                    https.setAclId(lbListenerModel.getAclId());
                    https.setBandwidth(lbListenerModel.getBandwidth());
                    https.setIdleTimeout(lbListenerModel.getIdleTimeout());
                    https.setRequestTimeout(lbListenerModel.getRequestTimeout());
                    https.setGzip(lbListenerModel.getGzip() ? "on" : "off");
                    https.setXForwardedFor(lbListenerModel.getXForwardedFor() ? "on" : "off");
                    https.setXForwardedFor_SLBID(lbListenerModel.getXForwardedFor_SLBID() ? "on" : "off");
                    https.setXForwardedFor_SLBIP(lbListenerModel.getXForwardedFor_SLBIP() ? "on" : "off");
                    https.setXForwardedFor_proto(lbListenerModel.getXForwardedFor_proto() ? "on" : "off");
                    // TODO
                    // 因缺少ssl证书，无法继续开发
                    return new BocloudResult(true, lbListenerModel, "负载均衡HTTPS监听器创建成功");
                default:
                    return new BocloudResult(false, "不支持的负载均衡协议");
            }
        } catch (Exception e) {
            logger.error("更新负载均衡监听器失败:", e);
            return new BocloudResult(false, "更新负载均衡监听器失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeListener(Integer port, String loadBalancerId, String protocol) {
        try {
            logger.info("port : " + port);
            DeleteLoadBalancerListenerRequest request = new DeleteLoadBalancerListenerRequest();
            request.setListenerPort(port);
            request.setRegionId(this.getRegionId());
            request.setLoadBalancerId(loadBalancerId);
            request.setListenerProtocol(protocol);
            DeleteLoadBalancerListenerResponse response = this.getSLBClient().deleteLoadBalancerListener(request);
            if (null == response || null == response.getBody().getRequestId()) {
                return new BocloudResult(false, "删除负载均衡监听器失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "删除负载均衡监听器成功");
        } catch (Exception e) {
            logger.error("删除负载均衡监听器失败:", e);
            return new BocloudResult(false, "删除负载均衡监听器失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult createPool(LbPoolModel poolModel, List<LbMemberModel> memberModels) {
        try {
            CreateVServerGroupRequest request = new CreateVServerGroupRequest();
            request.setLoadBalancerId(poolModel.getLbUuid());
            request.setRegionId(this.getRegionId());
            request.setVServerGroupName(poolModel.getName());
            // 拼接members数据
            JSONArray data = new JSONArray();
            for (LbMemberModel memberModel : memberModels) {
                JSONObject member = new JSONObject();
                member.put("ServerId", memberModel.getMemberUuid());
                member.put("Weight", memberModel.getWeight());
                member.put("Type", memberModel.getType());
                member.put("Port", memberModel.getPort());
                data.add(member);
            }
            request.setBackendServers(data.toJSONString());
            CreateVServerGroupResponse response = this.getSLBClient().createVServerGroup(request);
            if (null == response) {
                new BocloudResult(false, "创建虚服务器组失败");
            }
            poolModel.setPoolUuid(response.getBody().getVServerGroupId());
            return new BocloudResult(true, poolModel, "创建虚服务器组成功");
        } catch (Exception e) {
            logger.error("创建虚服务器组失败:", e);
            return new BocloudResult(false, "创建虚服务器组失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult updatePool(LbPoolModel poolModel, List<LbMemberModel> memberModels) {
        try {
            SetVServerGroupAttributeRequest request = new SetVServerGroupAttributeRequest();
            request.setVServerGroupName(poolModel.getName());
            request.setVServerGroupId(poolModel.getPoolUuid());
            request.setRegionId(this.getRegionId());
            // 拼接members数据
            if (!ListTool.isEmpty(memberModels)) {
                JSONArray data = new JSONArray();
                for (LbMemberModel memberModel : memberModels) {
                    JSONObject member = new JSONObject();
                    member.put("ServerId", memberModel.getMemberUuid());
                    member.put("Weight", memberModel.getWeight());
                    member.put("Type", memberModel.getType());
                    member.put("Port", memberModel.getPort());
                    data.add(member);
                }
                request.setBackendServers(data.toJSONString());
            }
            SetVServerGroupAttributeResponse response = this.getSLBClient().setVServerGroupAttribute(request);
            if (null == response) {
                return new BocloudResult(false, "更新虚服务器组失败");
            }
            return new BocloudResult(true, poolModel, "更新虚服务器组成功");
        } catch (Exception e) {
            logger.error("更新虚服务器组失败:", e);
            return new BocloudResult(false, "更新虚服务器组失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removePool(String poolId) {
        try {
            DeleteVServerGroupRequest request = new DeleteVServerGroupRequest();
            request.setVServerGroupId(poolId);
            request.setRegionId(this.getRegionId());
            DeleteVServerGroupResponse response = this.getSLBClient().deleteVServerGroup(request);
            if (null == response) {
                return new BocloudResult(false, "删除虚服务器组失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "删除虚服务器组成功");
        } catch (Exception e) {
            logger.error("删除虚服务器组失败:", e);
            return new BocloudResult(false, "删除虚服务器组失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult addMembers(String vServerGroupId, List<LbMemberModel> memberModels) {
        try {
            AddVServerGroupBackendServersRequest request = new AddVServerGroupBackendServersRequest();
            request.setRegionId(this.getRegionId());
            request.setVServerGroupId(vServerGroupId);
            // 拼接members数据
            JSONArray data = new JSONArray();
            for (LbMemberModel memberModel : memberModels) {
                JSONObject member = new JSONObject();
                member.put("ServerId", memberModel.getMemberUuid());
                member.put("Weight", memberModel.getWeight());
                member.put("Type", memberModel.getType());
                member.put("ServerIp", memberModel.getAddress());
                member.put("Port", memberModel.getPort());
                member.put("Description", memberModel.getName());
                data.add(member);
            }
            request.setBackendServers(data.toJSONString());
            AddVServerGroupBackendServersResponse response = this.getSLBClient().addVServerGroupBackendServers(request);
            if (null == response) {
                return new BocloudResult(false, "新增member失败");
            }
            return new BocloudResult(true, response, "新增member成功");
        } catch (Exception e) {
            logger.error("新增member失败:", e);
            return new BocloudResult(false, "新增member失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeMembers(String vServerGroupId, List<LbMemberModel> memberModels) {
        try {
            RemoveVServerGroupBackendServersRequest request = new RemoveVServerGroupBackendServersRequest();
            request.setRegionId(this.getRegionId());
            request.setVServerGroupId(vServerGroupId);
            // 拼接members数据
            JSONArray data = new JSONArray();
            for (LbMemberModel memberModel : memberModels) {
                JSONObject member = new JSONObject();
                member.put("ServerId", memberModel.getMemberUuid());
                member.put("Weight", memberModel.getWeight());
                member.put("Type", memberModel.getType());
                member.put("ServerIp", memberModel.getAddress());
                member.put("Port", memberModel.getPort());
                member.put("Description", memberModel.getName());
                data.add(member);
            }
            request.setBackendServers(data.toJSONString());
            RemoveVServerGroupBackendServersResponse response =
                this.getSLBClient().removeVServerGroupBackendServers(request);
            if (null == response) {
                return new BocloudResult(false, "删除member失败");
            }
            return new BocloudResult(true, response, "删除member成功");
        } catch (Exception e) {
            logger.error("删除member失败:", e);
            return new BocloudResult(false, "删除member失败:" + e.getMessage());
        }
    }
}
