package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.ecs20140526.models.DescribeNetworkInterfacesRequest;
import com.aliyun.ecs20140526.models.DescribeNetworkInterfacesResponse;
import com.aliyun.ecs20140526.models.DescribeNetworkInterfacesResponseBody;
import com.aliyun.vpc20160428.models.*;
import com.aliyun.vpc20160428.models.DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds;
import com.aliyun.vpc20160428.models.DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewaySnatTableIds;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.NatGatewayConvertor;
import com.bocloud.cmp.provider.aliyunconvertor.NetworkInterfaceConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * NAT 网关
 * 
 * @ClassName: AliyunNatGatewayProvider.java
 * @Description:
 *
 * @author: ymh
 * @date: 2021年5月11日 下午4:54:56
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunNatGatewayProvider extends AliyunProvider implements NatGatewayProvider {

	public AliyunNatGatewayProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(AliyunNatGatewayProvider.class);

	private static final int PAGE_SIZE = 50;

	private BeanConvertor<NatGatewayModel, DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway> convertor = new NatGatewayConvertor();
	private BeanConvertor<NetworkInterfaceModel, DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet> networkConvertor = new NetworkInterfaceConvertor();

	@Override
	public BocloudResult list() {
		try {
			DescribeNatGatewaysRequest request = new DescribeNatGatewaysRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<NatGatewayModel> models = Lists.newArrayList();
			DescribeNatGatewaysResponse response = this.getVPCClient().describeNatGateways(request);
			if (null == response) {
				return new BocloudResult(false, "NAT网关列表查询失败");
			} else if (null == response.getBody().getNatGateways()) {
				return new BocloudResult(false, "NAT网关列表查询失败");
			}
			List<DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway> natGateways = response
					.getBody().getNatGateways().getNatGateway();
			if(!ListTool.isEmpty(natGateways)) {
			    logger.info("nat 网关 : " + JSON.toJSONString(natGateways.get(0)));
			}
			Integer totalCount = response.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "empty natGateways");
			}
			for (DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway natGateway : natGateways) {
				NatGatewayModel convertModel = convertModel(natGateway);
				models.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getVPCClient().describeNatGateways(request);
				natGateways = response.getBody().getNatGateways().getNatGateway();
				for (DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway natGateway : natGateways) {
					NatGatewayModel convertModel = convertModel(natGateway);
					models.add(convertModel);
				}
			}
			return new BocloudResult(true, models, "NAT网关列表查询成功");
		} catch (Exception e) {
			logger.error("NAT网关列表查询失败：" + e);
			return new BocloudResult(false, "查询NAT网关失败");
		}

	}

	@Override
	public BocloudResult create(NatGatewayModel model) {
		try {
			CreateNatGatewayRequest request = new CreateNatGatewayRequest();
			request.setName(model.getName());
			request.setDescription(model.getRemark());
			request.setVpcId(model.getVpcUuid());
			request.setRegionId(this.getRegionId());
			/***
			 * NAT网关的规格，取值： Small（默认值）：小型。 Middle：中型。 Large：大型。 XLarge.1：超大型-1。
			 */
			if (!StringUtils.isEmpty(model.getSpec())) {
				request.setSpec(model.getSpec());
			}
			/***
			 * NAT网关的类型，取值： Normal：普通型NAT网关。 Enhanced：增强型NAT网关
			 */
			request.setNatType(model.getType());
			if ("Enhanced".equalsIgnoreCase(model.getType())) {
				request.setVSwitchId(model.getSubnetUuid());
			}
			CreateNatGatewayResponse response = this.getVPCClient().createNatGateway(request);
			if (null == response) {
				return new BocloudResult(false, "NAT网关创建失败");
			} else if (null == response.getBody().getNatGatewayId()) {
				return new BocloudResult(false, "NAT网关创建失败");
			}
			return new BocloudResult(true, this.getAvailable(response.getBody().getNatGatewayId()), "NAT网关创建成功");
		} catch (Exception e) {
			logger.error("NAT网关创建失败:" + e);
			return new BocloudResult(false, "NAT网关创建失败：" + e.getMessage());
		}
	}

	private NatGatewayModel getAvailable(String gatewayId) {
		DescribeNatGatewaysRequest request = new DescribeNatGatewaysRequest();
		request.setNatGatewayId(gatewayId);
		request.setRegionId(this.getRegionId());
		NatGatewayModel model = new NatGatewayModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeNatGatewaysResponse response = this.getVPCClient().describeNatGateways(request);
				List<DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway> gateways = response
						.getBody().getNatGateways().getNatGateway();
				if (ListTool.isEmpty(gateways)) {
					flag = false;
					break;
				}
				if (("Available".equalsIgnoreCase(gateways.get(0).getStatus()))) {
					model = convertModelForCreate(gateways.get(0));
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertModel(gateways.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get gateway error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult detail(String gatewayId) {
		try {
			DescribeNatGatewaysRequest request = new DescribeNatGatewaysRequest();
			request.setNatGatewayId(gatewayId);
			request.setRegionId(this.getRegionId());
			DescribeNatGatewaysResponse response = this.getVPCClient().describeNatGateways(request);
			if (null == response) {
				return new BocloudResult(false, "NAT网关详情查询失败");
			} else if (null == response.getBody().getNatGateways()) {
				return new BocloudResult(false, "NAT网关详情查询失败");
			}
			return new BocloudResult(true, convertModel(response.getBody().getNatGateways().getNatGateway().get(0)),
					"NAT网关详情查询成功");
		} catch (Exception e) {
			logger.error("NAT网关详情查询失败:" + e);
			return new BocloudResult(false, "NAT网关详情查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(NatGatewayModel model) {
		try {
			logger.info("modify model : " + JSON.toJSONString(model));
			ModifyNatGatewayAttributeRequest request = new ModifyNatGatewayAttributeRequest();
			request.setNatGatewayId(model.getUuid());
			request.setName(model.getName());
			request.setRegionId(this.getRegionId());
			ModifyNatGatewayAttributeResponse response = this.getVPCClient().modifyNatGatewayAttribute(request);
			BocloudResult detailResult = this.detail(model.getUuid());
			logger.info("modify detailResult model : " + JSON.toJSONString(detailResult));
			if (detailResult.isFailed()) {
				return new BocloudResult(false, "NAT网关不存在");
			}
			return new BocloudResult(true, detailResult.getData(), "NAT网关修改成功！");
		} catch (Exception e) {
			logger.error("NAT网关修改失败：" + e);
			return new BocloudResult(false, "NAT网关修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String gatewayId, Boolean force) {
		try {
			DeleteNatGatewayRequest request = new DeleteNatGatewayRequest();
			request.setNatGatewayId(gatewayId);
			request.setForce(force);
			request.setRegionId(this.getRegionId());
			DeleteNatGatewayResponse response = this.getVPCClient().deleteNatGateway(request);
			return new BocloudResult(true, response.getBody().getRequestId(), "NAT网关删除成功！");
		} catch (Exception e) {
			logger.error("NAT网关删除失败：" + e);
			return new BocloudResult(false, "NAT网关删除失败:" + e.getMessage());
		}
	}

	/**
	 * 根据forwardTableId查询DNAT规则
	 *
	 * @param forwardTableId
	 * @return
	 */
	private List<NatGatewayDnatRuleModel> listForward(String forwardTableId) {
		List<NatGatewayDnatRuleModel> models = Lists.newArrayList();
		try {
			DescribeForwardTableEntriesRequest request = new DescribeForwardTableEntriesRequest();
			// 必填
			request.setForwardTableId(forwardTableId);
			request.setRegionId(this.getRegionId());
			// 选填
			// 分页
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			DescribeForwardTableEntriesResponse response = this.getVPCClient().describeForwardTableEntries(request);
			if (null == response) {
				logger.error("DNAT列表查询失败,response为null");
				return models;
			} else if (null == response.getBody().getForwardTableEntries()) {
				logger.error("DNAT列表查询失败,response.getForwardTableEntries为null");
				return models;
			}
			List<DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry> forwardTableEntrys = response
					.getBody().getForwardTableEntries().getForwardTableEntry();
			int totalCount = response.getBody().getTotalCount();
			if (totalCount == 0) {
				return models;
			}
			for (DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry forward : forwardTableEntrys) {
				NatGatewayDnatRuleModel dnatRuleModel = convertForwardModel(forward);
				models.add(dnatRuleModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return models;
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getVPCClient().describeForwardTableEntries(request);
				forwardTableEntrys = response.getBody().getForwardTableEntries().getForwardTableEntry();
				for (DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry forward : forwardTableEntrys) {
					NatGatewayDnatRuleModel dnatRuleModel = convertForwardModel(forward);
					models.add(dnatRuleModel);
				}
			}
			return models;
		} catch (Exception e) {
			logger.error("DNAT列表查询失败：" + e);
			return models;
		}
	}

	/**
	 * 根据snatTableId查询SNAT规则
	 *
	 * @param snatTableId
	 * @return
	 */
	private List<NatGatewaySnatRuleModel> listSnat(String snatTableId) {
		List<NatGatewaySnatRuleModel> models = Lists.newArrayList();
		try {
			DescribeSnatTableEntriesRequest request = new DescribeSnatTableEntriesRequest();
			// 必填
			request.setSnatTableId(snatTableId);
			request.setRegionId(this.getRegionId());
			// 选填
			// 分页
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			DescribeSnatTableEntriesResponse response = this.getVPCClient().describeSnatTableEntries(request);
			if (null == response) {
				logger.error("SNAT列表查询失败,response为null");
				return models;
			} else if (null == response.getBody().getSnatTableEntries()) {
				logger.error("SNAT列表查询失败,response.getForwardTableEntries为null");
				return models;
			}
			List<DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry> snatTableEntrys = response
					.getBody().getSnatTableEntries().getSnatTableEntry();
			int totalCount = response.getBody().getTotalCount();
			if (totalCount == 0) {
				return models;
			}
			for (DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry snat : snatTableEntrys) {
				NatGatewaySnatRuleModel snatRuleModel = convertSnatModel(snat);
				models.add(snatRuleModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return models;
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				response = this.getVPCClient().describeSnatTableEntries(request);
				snatTableEntrys = response.getBody().getSnatTableEntries().getSnatTableEntry();
				for (DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry snat : snatTableEntrys) {
					NatGatewaySnatRuleModel snatRuleModel = convertSnatModel(snat);
					models.add(snatRuleModel);
				}
			}
			return models;
		} catch (Exception e) {
			logger.error("SNAT列表查询失败：" + e);
			return models;
		}
	}

	private NatGatewayModel convertModelForCreate(
			DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway natGateway) {
		NatGatewayModel model = null;
		if (null != natGateway) {
			model = new NatGatewayModel();
			if (StringUtils.isEmpty(natGateway.getName())) {
				model.setName(natGateway.getNatGatewayId());
			} else {
				model.setName(natGateway.getName());
			}
			model.setUuid(natGateway.getNatGatewayId());
			model.setVpcUuid(natGateway.getVpcId());
			model.setSpec(natGateway.getSpec());
			model.setType(natGateway.getNatType());
			model.setResourceGroupUuid(natGateway.getResourceGroupId());
			model.setStatus(natGateway.getStatus());
			model.setRemark(natGateway.getDescription());
			DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayNatGatewayPrivateInfo info = natGateway
					.getNatGatewayPrivateInfo();
			if (null != info) {
				model.setSubnetUuid(info.getVswitchId());
				model.setPrivateIpAddress(info.getPrivateIpAddress());
			}

			if (null != natGateway && null != natGateway.getForwardTableIds()) {
				List<String> forwardTableIds = natGateway.getForwardTableIds().getForwardTableId() == null
						? new ArrayList<>()
						: natGateway.getForwardTableIds().getForwardTableId();
				StringBuffer forwardIdsBuffer = new StringBuffer();
				for (String forwardTableId : forwardTableIds) {
					model.setDnatRules(listForward(forwardTableId));
					forwardIdsBuffer.append(forwardTableId).append(",");
				}
				if (forwardIdsBuffer.length() > 0) {
					model.setForwardTableIds(forwardIdsBuffer.substring(0, forwardIdsBuffer.length() - 1));
				}
			}
			if (null != natGateway && null != natGateway.getSnatTableIds()) {
				List<String> snatTableIds = natGateway.getSnatTableIds().getSnatTableId() == null ? new ArrayList<>()
						: natGateway.getSnatTableIds().getSnatTableId();
				StringBuffer snatIdsBuffer = new StringBuffer();
				for (String snatTableId : snatTableIds) {
					model.setSnatRules(listSnat(snatTableId));
					snatIdsBuffer.append(snatTableId).append(",");
				}
				if (snatIdsBuffer.length() > 0) {
					model.setSnatTableIds(snatIdsBuffer.substring(0, snatIdsBuffer.length() - 1));
				}
			}
			// 网关的弹性网卡信息
			model.setNatType(natGateway.getNatType());
			if (null != natGateway.getNatType()
					&& !NatGatewayModel.NatType.Normal.name().equals(natGateway.getNatType())) {
				DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayNatGatewayPrivateInfo privateInfo = natGateway
						.getNatGatewayPrivateInfo();
				if (null != privateInfo) {
					model.setEniInstanceId(privateInfo.getEniInstanceId());
				}
				BocloudResult result = getNetworkInterface(model.getEniInstanceId());
				model.setNetworkInterfaceModel(result.isSuccess()
						? JSONObject.parseObject(JSONObject.toJSONString(result.getData()), NetworkInterfaceModel.class)
						: null);
			}
		}
		return model;
	}

	private BocloudResult getNetworkInterface(String networkInterfaceId) {
		try {
			DescribeNetworkInterfacesRequest request = new DescribeNetworkInterfacesRequest();
			request.setRegionId(this.getRegionId());
			List<String> networkInterfaceIds = new ArrayList<>();
			networkInterfaceIds.add(networkInterfaceId);
			request.setNetworkInterfaceId(networkInterfaceIds);
			DescribeNetworkInterfacesResponse response = this.getECSClient().describeNetworkInterfaces(request);
			if (null == response) {
				return new BocloudResult(false, "弹性网卡详情查询失败");
			} else if (null == response.getBody().getNetworkInterfaceSets()) {
				return new BocloudResult(false, "弹性网卡详情查询失败");
			}
			DescribeNetworkInterfacesResponseBody.DescribeNetworkInterfacesResponseBodyNetworkInterfaceSetsNetworkInterfaceSet networkInterface = response
					.getBody().getNetworkInterfaceSets().getNetworkInterfaceSet().get(0);
			NetworkInterfaceModel networkInterfaceModel = networkConvertor.convertModel(networkInterface);
			return new BocloudResult(true, networkInterfaceModel, "弹性网卡详情查询成功");
		} catch (Exception e) {
			logger.error("弹性网卡详情查询失败:" + e);
			return new BocloudResult(false, "弹性网卡详情查询失败:" + e.getMessage());
		}
	}

	private NatGatewayModel convertModel(
			DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGateway natGateway) {
		logger.info("natGateway : " + JSON.toJSONString(natGateway));
		NatGatewayModel model = null;
		if (null != natGateway) {
			model = new NatGatewayModel();
			if (StringUtils.isEmpty(natGateway.getName())) {
				model.setName(natGateway.getNatGatewayId());
			} else {
				model.setName(natGateway.getName());
			}
			model.setUuid(natGateway.getNatGatewayId());
			model.setVpcUuid(natGateway.getVpcId());
			model.setSpec(natGateway.getSpec());
			model.setType(natGateway.getNatType());
			model.setResourceGroupUuid(natGateway.getResourceGroupId());
			model.setStatus(natGateway.getStatus());
			model.setRemark(natGateway.getDescription());
			DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayNatGatewayPrivateInfo info = natGateway
					.getNatGatewayPrivateInfo();
			if (null != info) {
				model.setSubnetUuid(info.getVswitchId());
				model.setPrivateIpAddress(info.getPrivateIpAddress());
			}
			DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds describeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds = natGateway
					.getForwardTableIds();
			if (null != describeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds) {
				List<String> forwardTableIds = describeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds
						.getForwardTableId() == null ? new ArrayList<>()
								: describeNatGatewaysResponseBodyNatGatewaysNatGatewayForwardTableIds
										.getForwardTableId();
				if (!ListTool.isEmpty(forwardTableIds)) {
					StringBuffer forwardIdsBuffer = new StringBuffer();
					for (String forwardTableId : forwardTableIds) {
						model.setDnatRules(listForward(forwardTableId));
						forwardIdsBuffer.append(forwardTableId).append(",");
					}
					if (forwardIdsBuffer.length() > 0) {
						model.setForwardTableIds(forwardIdsBuffer.substring(0, forwardIdsBuffer.length() - 1));
					}
				}
			}

			DescribeNatGatewaysResponseBodyNatGatewaysNatGatewaySnatTableIds describeNatGatewaysResponseBodyNatGatewaysNatGatewaySnatTableIds = natGateway
					.getSnatTableIds();
			if (null != describeNatGatewaysResponseBodyNatGatewaysNatGatewaySnatTableIds) {
				List<String> snatTableIds = describeNatGatewaysResponseBodyNatGatewaysNatGatewaySnatTableIds
						.getSnatTableId();
				if (!ListTool.isEmpty(snatTableIds)) {
					StringBuffer snatIdsBuffer = new StringBuffer();
					for (String snatTableId : snatTableIds) {
						model.setSnatRules(listSnat(snatTableId));
						snatIdsBuffer.append(snatTableId).append(",");
					}
					if (snatIdsBuffer.length() > 0) {
						model.setSnatTableIds(snatIdsBuffer.substring(0, snatIdsBuffer.length() - 1));
					}
				}
			}
			// 网关的弹性网卡信息
			model.setNatType(natGateway.getNatType());
			if (null != natGateway.getNatType()
					&& !NatGatewayModel.NatType.Normal.name().equals(natGateway.getNatType())) {
				DescribeNatGatewaysResponseBody.DescribeNatGatewaysResponseBodyNatGatewaysNatGatewayNatGatewayPrivateInfo privateInfo = natGateway
						.getNatGatewayPrivateInfo();
				model.setEniInstanceId(privateInfo.getEniInstanceId());
			}
		}
		return model;
	}

	private NatGatewayDnatRuleModel convertForwardModel(
			DescribeForwardTableEntriesResponseBody.DescribeForwardTableEntriesResponseBodyForwardTableEntriesForwardTableEntry forwardEntry) {
		NatGatewayDnatRuleModel model = null;
		if (null != forwardEntry) {
			model = new NatGatewayDnatRuleModel();
			model.setFloatingIpAddress(forwardEntry.getExternalIp());
			model.setUuid(forwardEntry.getForwardEntryId());
			model.setName(forwardEntry.getForwardEntryName());
			model.setForwardTableId(forwardEntry.getForwardTableId());
			model.setPrivateIp(forwardEntry.getInternalIp());
			model.setStatus(forwardEntry.getStatus());
			if (null != model.getProtocol()
					&& !NatGatewayDnatRuleModel.Protocol.Any.name().equalsIgnoreCase(model.getProtocol())) {
				model.setExternalServicePort(
						null != forwardEntry.getExternalPort() ? Integer.valueOf(forwardEntry.getExternalPort())
								: null);
				model.setInternalServicePort(
						null != forwardEntry.getInternalPort() ? Integer.valueOf(forwardEntry.getInternalPort())
								: null);
			} else {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.Any.name());
			}
			if (null != forwardEntry.getIpProtocol()
					&& NatGatewayDnatRuleModel.Protocol.TCP.name().equalsIgnoreCase(forwardEntry.getIpProtocol())) {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.TCP.name());
			} else if (null != forwardEntry.getIpProtocol()
					&& NatGatewayDnatRuleModel.Protocol.UDP.name().equalsIgnoreCase(forwardEntry.getIpProtocol())) {
				model.setProtocol(NatGatewayDnatRuleModel.Protocol.UDP.name());
			}
		}
		return model;
	}

	private NatGatewaySnatRuleModel convertSnatModel(
			DescribeSnatTableEntriesResponseBody.DescribeSnatTableEntriesResponseBodySnatTableEntriesSnatTableEntry snatTableEntry) {
		NatGatewaySnatRuleModel model = null;
		if (null != snatTableEntry) {
			model = new NatGatewaySnatRuleModel();

			model.setUuid(snatTableEntry.getSnatEntryId());
			model.setName(snatTableEntry.getSnatEntryName());
			model.setFloatingIpAddress(snatTableEntry.getSnatIp());
			model.setSnatTableId(snatTableEntry.getSnatTableId());
			model.setCidr(snatTableEntry.getSourceCIDR());
			model.setSubnetUuid(snatTableEntry.getSourceVSwitchId());
			model.setStatus(snatTableEntry.getStatus());
		}
		return model;
	}
}
