package com.bocloud.cmp.provider.network;


import com.aliyun.vpc20160428.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkAclModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.NetworkAclConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/15 16:21
 * @Description: 网络ACL
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunNetworkAclProvider extends AliyunProvider implements NetworkAclProvider {

    private Logger logger = LoggerFactory.getLogger(AliyunNetworkAclProvider.class);
    private static final int PAGE_SIZE = 50;
    private BeanConvertor<NetworkAclModel, DescribeNetworkAclsResponseBody.DescribeNetworkAclsResponseBodyNetworkAclsNetworkAcl> convertor = new NetworkAclConvertor();

    public AliyunNetworkAclProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeNetworkAclsRequest request = new DescribeNetworkAclsRequest();
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            request.setRegionId(this.getRegionId());
            List<NetworkAclModel> networkAclModels = Lists.newArrayList();
            DescribeNetworkAclsResponse response = this.getVPCClient().describeNetworkAcls(request);
            if (null == response) {
                return new BocloudResult(false, "查看网络ACL列表失败");
            } else if (null == response.getBody().getNetworkAcls()) {
                return new BocloudResult(false, "查看网络ACL列表失败");
            }
            List<DescribeNetworkAclsResponseBody.DescribeNetworkAclsResponseBodyNetworkAclsNetworkAcl> networkAcls = response.getBody().getNetworkAcls().getNetworkAcl();
            int totalCount = Integer.valueOf(response.getBody().getTotalCount());
            if (totalCount == 0) {
                return new BocloudResult(true, networkAclModels, "empty networkAcl");
            }
            for (DescribeNetworkAclsResponseBody.DescribeNetworkAclsResponseBodyNetworkAclsNetworkAcl networkAcl : networkAcls) {
                NetworkAclModel convertModel = convertor.convertModel(networkAcl);
                convertModel.setRegionId(this.getRegionId());
                networkAclModels.add(convertModel);
            }
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, networkAclModels, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getVPCClient().describeNetworkAcls(request);
                networkAcls = response.getBody().getNetworkAcls().getNetworkAcl();
                for (DescribeNetworkAclsResponseBody.DescribeNetworkAclsResponseBodyNetworkAclsNetworkAcl networkAcl : networkAcls) {
                    NetworkAclModel convertModel = convertor.convertModel(networkAcl);
                    convertModel.setRegionId(this.getRegionId());
                    networkAclModels.add(convertModel);
                }
            }
            return new BocloudResult(true, networkAclModels, "查看网络ACL列表成功");
        } catch (Exception e) {
            logger.error("查看网络ACL列表失败:" + e);
            return new BocloudResult(false, "查看网络ACL列表失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String networkAclId) {
        try {
            DescribeNetworkAclsRequest request = new DescribeNetworkAclsRequest();
            request.setNetworkAclId(networkAclId);
            request.setRegionId(this.getRegionId());
            DescribeNetworkAclsResponse response = this.getVPCClient().describeNetworkAcls(request);
            if (null == response) {
                return new BocloudResult(false, "查询网络ACL详情失败");
            } else if (null == response.getBody().getNetworkAcls()) {
                return new BocloudResult(false, "查询网络ACL详情失败");
            }
            DescribeNetworkAclsResponseBody.DescribeNetworkAclsResponseBodyNetworkAclsNetworkAcl networkAcl = response.getBody().getNetworkAcls().getNetworkAcl().get(0);
            NetworkAclModel networkAclModel = convertor.convertModel(networkAcl);
            return new BocloudResult(true, networkAclModel, "查看网络ACL详情成功");
        } catch (Exception e) {
            logger.error("查询网络ACL详情失败:" + e);
            return new BocloudResult(false, "查询网络ACL详情失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String networkAclId) {
        try {
            DeleteNetworkAclRequest request = new DeleteNetworkAclRequest();
            request.setNetworkAclId(networkAclId);
            request.setRegionId(this.getRegionId());
            DeleteNetworkAclResponse response = this.getVPCClient().deleteNetworkAcl(request);
            if (null == response) {
                return new BocloudResult(false, "删除网络ACL失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "删除网络ACL成功");
        } catch (Exception e) {
            logger.error("删除网络ACL失败:" + e);
            return new BocloudResult(false, "删除网络ACL失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(NetworkAclModel model) {
        try {
            ModifyNetworkAclAttributesRequest request = new ModifyNetworkAclAttributesRequest();
            request.setNetworkAclName(model.getName());
            request.setRegionId(this.getRegionId());
            request.setDescription(model.getRemark());
            ModifyNetworkAclAttributesResponse response = this.getVPCClient().modifyNetworkAclAttributes(request);
            if (null == response) {
                return new BocloudResult(false, "修改网络ACL失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "修改网络ACL成功");
        } catch (Exception e) {
            logger.error("修改网络ACL失败:" + e);
            return new BocloudResult(false, "修改网络ACL失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkAclModel model) {
        try {
            CreateNetworkAclRequest request = new CreateNetworkAclRequest();
            request.setNetworkAclName(model.getName());
            request.setDescription(model.getRemark());
            request.setVpcId(model.getVpcUuid());
            request.setRegionId(this.getRegionId());
            CreateNetworkAclResponse response = this.getVPCClient().createNetworkAcl(request);
            if (null == response) {
                return new BocloudResult(false, "创建网络ACL失败");
            }
            return new BocloudResult(true, response.getBody().getNetworkAclId(), "创建网络ACL成功");
        } catch (Exception e) {
            logger.error("创建网络ACL失败:" + e);
            return new BocloudResult(false, "创建网络ACL失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult associate(NetworkAclModel model) {
        try {
            AssociateNetworkAclRequest request = new AssociateNetworkAclRequest();
            request.setRegionId(this.getRegionId());
            request.setNetworkAclId(model.getNetworkACLUuid());
            List<AssociateNetworkAclRequest.AssociateNetworkAclRequestResource> resources = new ArrayList<>();
            for (NetworkAclModel.Resource resource1 : model.getResources()) {
                AssociateNetworkAclRequest.AssociateNetworkAclRequestResource resource = new AssociateNetworkAclRequest.AssociateNetworkAclRequestResource();
                resource.setResourceId(resource1.getResourceId());
                resource.setResourceType(resource1.getResourceType());
                resources.add(resource);
            }
            request.setResource(resources);
            AssociateNetworkAclResponse response = this.getVPCClient().associateNetworkAcl(request);
            if (null == response) {
                return new BocloudResult(false, "绑定网络ACL到交换机失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "绑定网络ACL到交换机成功");
        } catch (Exception e) {
            logger.error("绑定网络ACL到交换机失败:" + e);
            return new BocloudResult(false, "绑定网络ACL到交换机失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult UnAssociate(NetworkAclModel model) {
        try {
            UnassociateNetworkAclRequest request = new UnassociateNetworkAclRequest();
            request.setNetworkAclId(model.getNetworkACLUuid());
            request.setRegionId(this.getRegionId());
            List<UnassociateNetworkAclRequest.UnassociateNetworkAclRequestResource> resources = new ArrayList<>();
            for (NetworkAclModel.Resource resource1 : model.getResources()) {
                UnassociateNetworkAclRequest.UnassociateNetworkAclRequestResource resource = new UnassociateNetworkAclRequest.UnassociateNetworkAclRequestResource();
                resource.setResourceId(resource1.getResourceId());
                resource.setResourceType(resource1.getResourceType());
                resources.add(resource);
            }
            request.setResource(resources);
            UnassociateNetworkAclResponse response = this.getVPCClient().unassociateNetworkAcl(request);
            if (null == response) {
                return new BocloudResult(false, "解除网络ACL与交换机的绑定失败");
            }
            return new BocloudResult(true, response.getBody().getRequestId(), "解除网络ACL与交换机的绑定成功");
        } catch (Exception e) {
            logger.error("解除网络ACL与交换机的绑定失败:" + e);
            return new BocloudResult(false, "解除网络ACL与交换机的绑定失败:" + e.getMessage());
        }
    }
}
