package com.bocloud.cmp.provider.network;

import com.aliyun.vpc20160428.models.CreateRouteEntryRequest;
import com.aliyun.vpc20160428.models.CreateRouteEntryResponse;
import com.aliyun.vpc20160428.models.DeleteRouteEntryRequest;
import com.aliyun.vpc20160428.models.DeleteRouteEntryResponse;
import com.aliyun.vpc20160428.models.DescribeRouteEntryListRequest;
import com.aliyun.vpc20160428.models.DescribeRouteEntryListResponse;
import com.aliyun.vpc20160428.models.DescribeRouteEntryListResponseBody.DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry;
import com.aliyun.vpc20160428.models.ModifyRouteEntryRequest;
import com.aliyun.vpc20160428.models.ModifyRouteEntryResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/11 11:34
 * @Description: 路由条目
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunRouteProvider extends AliyunProvider implements RouteProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunRouteProvider.class);

	public AliyunRouteProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(RouteModel model) {
		try {
			CreateRouteEntryRequest request = new CreateRouteEntryRequest();
			request.setRouteTableId(model.getRouteTableUuid());
			request.setRouteEntryName(model.getName());
			request.setDestinationCidrBlock(model.getDestination());
			request.setRegionId(this.getRegionId());
			request.setNextHopType(model.getTargetType());
			request.setNextHopId(model.getTargetUuid());
			request.setDescription(model.getOrigin());
			CreateRouteEntryResponse createRouteEntry = this.getVPCClient().createRouteEntry(request);
			if (null == createRouteEntry || null == createRouteEntry.getBody()) {
				return new BocloudResult<>(false, "底层创建失败");
			}
			return new BocloudResult(true, detail(model.getRouteTableUuid(), model.getName()).getData(), "创建成功");
		} catch (Exception e) {
			logger.error("创建路由条目异常:" + e);
			return new BocloudResult(false, "创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(RouteModel model) {
		try {
			ModifyRouteEntryRequest request = new ModifyRouteEntryRequest();
			request.setRouteEntryId(model.getUuid());
			request.setRegionId(this.getRegionId());
			request.setRouteEntryName(model.getName());
			ModifyRouteEntryResponse modifyRouteEntry = this.getVPCClient().modifyRouteEntry(request);
			return new BocloudResult(true, "修改成功");
		} catch (Exception e) {
			logger.error("修改路由条目失败:" + e);
			return new BocloudResult(false, "修改路由条目失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult delete(RouteModel model) {
		try {
			DeleteRouteEntryRequest request = new DeleteRouteEntryRequest();
			request.setRouteEntryId(model.getUuid());
			request.setRegionId(this.getRegionId());
			request.setNextHopId(model.getTargetUuid());
			DeleteRouteEntryResponse deleteRouteEntry = this.getVPCClient().deleteRouteEntry(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("刪除路由条目失败:" + e);
			return new BocloudResult(false, "刪除路由条目失败:" + e.getMessage());
		}
	}

	private BocloudResult detail(String routeTableId, String routeEntryName) {
		try {
			DescribeRouteEntryListRequest request = new DescribeRouteEntryListRequest();
			request.setRouteTableId(routeTableId);
			request.setRouteEntryType("Custom");
			request.setRegionId(this.getRegionId());
			request.setRouteEntryName(routeEntryName);
			DescribeRouteEntryListResponse describeRouteEntryList = this.getVPCClient().describeRouteEntryList(request);
			if (null == describeRouteEntryList || null == describeRouteEntryList.getBody()) {
				return new BocloudResult<>(false, "资源不存在");
			}
			List<DescribeRouteEntryListResponseBodyRouteEntrysRouteEntry> routeEntry = describeRouteEntryList.getBody()
					.getRouteEntrys().getRouteEntry();
			if (ListTool.isEmpty(routeEntry)) {
				return new BocloudResult<>(false, "资源不存在");
			}
			return new BocloudResult(true, routeEntry.get(0), "删除成功");
		} catch (Exception e) {
			logger.error("获取路由条目失败:" + e);
			return new BocloudResult(false, "获取路由条目失败:" + e.getMessage());
		}
	}
}
