package com.bocloud.cmp.provider.network;

import com.aliyun.ecs20140526.models.DescribeVRoutersRequest;
import com.aliyun.ecs20140526.models.DescribeVRoutersResponse;
import com.aliyun.ecs20140526.models.DescribeVRoutersResponseBody.DescribeVRoutersResponseBodyVRoutersVRouter;
import com.aliyun.ecs20140526.models.ModifyVRouterAttributeRequest;
import com.aliyun.ecs20140526.models.ModifyVRouterAttributeResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.RouterConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author: yangzhenrong
 * @Date: 2020/12/11 15:01
 * @Description: 路由器
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunRouterProvider extends AliyunProvider implements RouterProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunRouterProvider.class);

	private static final int PAGE_SIZE = 50;

	private BeanConvertor<RouterModel, DescribeVRoutersResponseBodyVRoutersVRouter> convertor = new RouterConvertor();

	public AliyunRouterProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeVRoutersRequest request = new DescribeVRoutersRequest();
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			request.setRegionId(this.getRegionId());
			List<RouterModel> routerModels = Lists.newArrayList();
			DescribeVRoutersResponse describeVRouters = this.getECSClient().describeVRouters(request);
			if (null == describeVRouters) {
				return new BocloudResult(false, "查询路由器列表失败");
			} else if (null == describeVRouters.getBody()) {
				return new BocloudResult(false, "查询路由器列表失败");
			}
			List<DescribeVRoutersResponseBodyVRoutersVRouter> vRouter = describeVRouters.getBody().getVRouters()
					.getVRouter();
			Integer totalCount = describeVRouters.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, routerModels, "empty routers");
			}
			for (DescribeVRoutersResponseBodyVRoutersVRouter router : vRouter) {
				RouterModel convertModel = convertor.convertModel(router);
				convertModel.setRegionId(this.getRegionId());
				routerModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, routerModels, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				describeVRouters = this.getECSClient().describeVRouters(request);
				vRouter = describeVRouters.getBody().getVRouters().getVRouter();
				for (DescribeVRoutersResponseBodyVRoutersVRouter router : vRouter) {
					RouterModel convertModel = convertor.convertModel(router);
					convertModel.setRegionId(this.getRegionId());
					routerModels.add(convertModel);
				}
			}
			return new BocloudResult(true, routerModels, "查询路由器列表成功");
		} catch (Exception e) {
			logger.error("查询路由器列表失败:" + e);
			return new BocloudResult(false, "查询路由器列表失败");
		}
	}

	@Override
	public BocloudResult create(RouterModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(String model) {
		return null;
	}

	@Override
	public BocloudResult detail(String routerId) {
		return null;
	}

	@Override
	public BocloudResult modify(RouterModel model) {
		try {
			ModifyVRouterAttributeRequest request = new ModifyVRouterAttributeRequest();
			request.setVRouterId(model.getRouterUuid());
			request.setVRouterName(model.getName());
			request.setDescription(model.getRemark());
			request.setRegionId(this.getRegionId());
			ModifyVRouterAttributeResponse modifyVRouterAttribute = this.getECSClient().modifyVRouterAttribute(request);
			return new BocloudResult(true, "修改路由器成功");
		} catch (Exception e) {
			logger.error("修改路由器失败：" + e);
			return new BocloudResult(false, "修改路由器失败：" + e.getMessage());
		}

	}

	@Override
	public BocloudResult attach(RouterModel model) {
		return null;
	}

	@Override
	public BocloudResult detach(RouterModel model) {
		return null;
	}

	@Override
	public BocloudResult config(RouterModel model) {
		return null;
	}

	@Override
	public BocloudResult clear(String routerId) {
		return null;
	}

}
