package com.bocloud.cmp.provider.network;

import com.aliyun.ecs20140526.models.*;
import com.aliyun.ecs20140526.models.DescribeVSwitchesResponseBody.DescribeVSwitchesResponseBodyVSwitches;
import com.aliyun.ecs20140526.models.DescribeVSwitchesResponseBody.DescribeVSwitchesResponseBodyVSwitchesVSwitch;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.SubnetConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.network
 * @ClassName: AliyunSubnetProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/7/31 14:40
 * @Description:
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunSubnetProvider extends AliyunProvider implements SubnetProvider {

	public AliyunSubnetProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(AliyunSubnetProvider.class);

	private BeanConvertor<SubnetModel, DescribeVSwitchesResponseBodyVSwitchesVSwitch> convertor = new SubnetConvertor();

	private static final int PAGE_SIZE = 50;

	@Override
	public BocloudResult list() {
		try {
			DescribeVSwitchesRequest request = new DescribeVSwitchesRequest();
			request.setRegionId(this.getRegionId());
			request.setPageNumber(1);
			request.setPageSize(PAGE_SIZE);
			List<SubnetModel> subnetModels = Lists.newArrayList();
			DescribeVSwitchesResponse describeVSwitches = this.getECSClient().describeVSwitches(request);
			if (null == describeVSwitches.getBody()) {
				return new BocloudResult(false, "交换机列表查询失败");
			} else if (null == describeVSwitches.getBody().getVSwitches()) {
				return new BocloudResult(false, "交换机列表查询失败");
			}
			List<DescribeVSwitchesResponseBodyVSwitchesVSwitch> vSwitches = describeVSwitches.getBody().getVSwitches()
					.getVSwitch();
			Integer totalCount = describeVSwitches.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, subnetModels, "empty vSwitches");
			}
			for (DescribeVSwitchesResponseBodyVSwitchesVSwitch vSwitch : vSwitches) {
				SubnetModel convertModel = convertor.convertModel(vSwitch);
				convertModel.setRegionId(this.getRegionId());
				subnetModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, subnetModels, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setPageNumber(i);
				request.setPageSize(PAGE_SIZE);
				describeVSwitches = this.getECSClient().describeVSwitches(request);
				vSwitches = describeVSwitches.getBody().getVSwitches().getVSwitch();
				for (DescribeVSwitchesResponseBodyVSwitchesVSwitch vSwitch : vSwitches) {
					SubnetModel convertModel = convertor.convertModel(vSwitch);
					convertModel.setRegionId(this.getRegionId());
					subnetModels.add(convertModel);
				}
			}
			return new BocloudResult(true, subnetModels, "交换机列表查询成功");
		} catch (Exception e) {
			logger.error("交换机列表查询失败 ： " + e);
			return new BocloudResult(false, "交换机列表查询失败:" + e.getMessage());
		}
	}

	public BocloudResult list(SubnetModel model) {
		try {
			DescribeVSwitchesRequest request = new DescribeVSwitchesRequest();
			request.setRegionId(this.getRegionId());
			request.setVpcId(model.getVpcUuid());
			request.setVSwitchId(model.getSubnetUuid());

			DescribeVSwitchesResponse describeVSwitches = this.getECSClient().describeVSwitches(request);
			if (null == describeVSwitches.getBody()) {
				return new BocloudResult(false, "指定条件下的交换机列表查询失败");
			} else if (null == describeVSwitches.getBody().getVSwitches()) {
				return new BocloudResult(false, "指定条件下的交换机列表查询失败");
			}
			List<DescribeVSwitchesResponseBodyVSwitchesVSwitch> vSwitch = describeVSwitches.getBody().getVSwitches()
					.getVSwitch();
			List<SubnetModel> subnetModels = convertor.convertList(vSwitch);
			return new BocloudResult(true, subnetModels, "指定条件下的交换机列表查询成功");
		} catch (Exception e) {
			logger.error("指定条件下的交换机列表查询失败 ： " + e);
			return new BocloudResult(false, "指定条件下的交换机列表查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SubnetModel model) {
		try {
			CreateVSwitchRequest request = new CreateVSwitchRequest();
			/**
			 * 交换机的网段。交换机网段要求如下： 交换机网段的掩码长度范围为16-29位。 交换机的网段必须从属于所在VPC的网段。
			 * 交换机的网段不能与所在VPC中路由条目的目标网段相同，但可以是目标网段的子集。 如果交换机的网段与所在VPC的网段相同时，VPC只能有一个交换机。
			 */
			request.setCidrBlock(model.getCidr());
			request.setVpcId(model.getVpcUuid());
			request.setZoneId(model.getZone());
			request.setRegionId(model.getRegionId());
			request.setDescription(model.getRemark());
			request.setVSwitchName(model.getName());

			CreateVSwitchResponse createVSwitch = this.getECSClient().createVSwitch(request);

			if (null == createVSwitch || null == createVSwitch.getBody()) {
				return new BocloudResult(false, "子网创建失败");
			}
			return new BocloudResult(true, getAvailable(createVSwitch.getBody().getVSwitchId()), "子网创建成功");
		} catch (Exception e) {
			logger.error("子网创建失败:" + e);
			return new BocloudResult(false, e.getMessage(), "子网创建失败:" + e.getMessage());
		}
	}

	private SubnetModel getAvailable(String vSwitchId) {
		SubnetModel model = new SubnetModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				BocloudResult detail = this.detail(vSwitchId);
				if (detail.isFailed()) {
					return model;
				}
				model = (SubnetModel) detail.getData();
				if ("Available".equalsIgnoreCase(model.getStatus())) {
					model.setRegionId(this.getRegionId());
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					model = (SubnetModel) detail.getData();
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get vSwitch error:", e);
		}
		return model;
	}

	/**
	 * 调用该接口删除交换机时，请注意：
	 * <p>
	 * 删除交换机之前，需要先释放或移走VPC内的所有资源，包括交换机，云产品实例，路由器接口，HAVIP等。 只有处于Available状态的交换机可以被删除。
	 * 交换机所在的VPC正在创建/删除交换机或路由条目时，无法删除交换机。
	 */
	@Override
	public BocloudResult remove(String subnetId) {
		try {
			DeleteVSwitchRequest request = new DeleteVSwitchRequest();
			request.setVSwitchId(subnetId);
			request.setRegionId(this.getRegionId());
			DeleteVSwitchResponse deleteVSwitch = this.getECSClient().deleteVSwitch(request);
			if (null == deleteVSwitch) {
				return new BocloudResult(false, "子网删除失败");
			}
			return new BocloudResult(true, "子网删除成功");
		} catch (Exception e) {
			logger.error("子网删除失败:" + e);
			if(e.getMessage().contains("Specified object has dependent resources") || e.getMessage().contains("VSwitch cannot be deleted when associated with route table") ){
				return new BocloudResult(false, e.getMessage(), "子网具有依赖资源");
			}
			return new BocloudResult(false, e.getMessage(), "子网删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult associateIpv6CidrBlock(String subnetId, String ipv6CidrBlock) {
		try {
			return new BocloudResult(true, "开发中...");
		} catch (Exception e) {
			logger.error("交换机开启IPV6失败:" + e);
			return new BocloudResult(false, e.getMessage(), "交换机开启IPV6失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociateCidrBlock(String subnetId) {
		try {
			return new BocloudResult(true, "开发中...");
		} catch (Exception e) {
			logger.error("交换机关闭IPV6失败:" + e);
			return new BocloudResult(false, e.getMessage(), "交换机关闭IPV6失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SubnetModel model) {
		try {
			ModifyVSwitchAttributeRequest request = new ModifyVSwitchAttributeRequest();
			request.setVSwitchId(model.getSubnetUuid());
			request.setVSwitchName(model.getName());
			request.setRegionId(this.getRegionId());
			request.setDescription(model.getRemark());
			ModifyVSwitchAttributeResponse modifyVSwitchAttribute = this.getECSClient().modifyVSwitchAttribute(request);
			if (null == modifyVSwitchAttribute || null == modifyVSwitchAttribute.getBody()) {
				return new BocloudResult(false, "交换机修改失败");
			}
			return new BocloudResult(true, "交换机修改成功");
		} catch (Exception e) {
			logger.error("交换机修改失败:" + e);
			return new BocloudResult(false, e.getMessage(), "交换机修改失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String subnetId) {
		try {
			DescribeVSwitchesRequest describeVSwitchesRequest = new DescribeVSwitchesRequest();
			describeVSwitchesRequest.setVSwitchId(subnetId);
			describeVSwitchesRequest.setRegionId(this.getRegionId());
			DescribeVSwitchesResponse describeVSwitches = this.getECSClient()
					.describeVSwitches(describeVSwitchesRequest);
			if (null == describeVSwitches || null == describeVSwitches.getBody()) {
				return new BocloudResult(false, "交换机详情查询失败");
			}
			DescribeVSwitchesResponseBodyVSwitches vSwitches = describeVSwitches.getBody().getVSwitches();
			List<DescribeVSwitchesResponseBodyVSwitchesVSwitch> vSwitch = vSwitches.getVSwitch();
			if (ListTool.isEmpty(vSwitch)) {
				return new BocloudResult<>(false, "资源不存在");
			}
			DescribeVSwitchesResponseBodyVSwitchesVSwitch describeVSwitchesResponseBodyVSwitchesVSwitch = vSwitch
					.get(0);
			return new BocloudResult(true, detailResponseToSubnetModel(describeVSwitchesResponseBodyVSwitchesVSwitch),
					"交换机详情查询成功");
		} catch (Exception e) {
			logger.error("交换机详情查询失败：" + e);
			return new BocloudResult(false, "交换机详情查询失败：" + e.getMessage());
		}
	}

	/**
	 * @return com.bocloud.cmp.model.SubnetModel
	 * @Author Hu Wentao
	 * @Description 将查询交换机详情返回的数据封装到subnetmodel中
	 * @Date 15:40 2020/7/31
	 * @Param [response]
	 **/
	public SubnetModel detailResponseToSubnetModel(DescribeVSwitchesResponseBodyVSwitchesVSwitch response) {
		SubnetModel subnetModel = new SubnetModel();
		subnetModel.setRemark(response.getDescription());
		subnetModel.setCidr(response.getCidrBlock());
		String status = response.getStatus();
		if (null != status) {
			subnetModel.setStatus(status);
		}
		subnetModel.setSubnetUuid(response.getVSwitchId());
		if (StringUtils.isEmpty(response.getVSwitchName())) {
			subnetModel.setName(response.getVSwitchId());
		} else {
			subnetModel.setName(response.getVSwitchName());
		}
		subnetModel.setVpcUuid(response.getVpcId());
		subnetModel.setZone(response.getZoneId());
		subnetModel.setIsDefault(response.getIsDefault());
		subnetModel.setIpVersion("V4");
		subnetModel.setResourceGroupUuid(response.getResourceGroupId());
		String creationTime = response.getCreationTime();
		if (null != creationTime && !"".equals(creationTime)) {
			subnetModel.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTime, "Z"));
		}
		return subnetModel;
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
	    try {
            DeleteVSwitchRequest request = new DeleteVSwitchRequest();
            request.setVSwitchId(subnet.getSubnetUuid());
            request.setRegionId(this.getRegionId());
            DeleteVSwitchResponse deleteVSwitch = this.getECSClient().deleteVSwitch(request);
            if (null == deleteVSwitch) {
                return new BocloudResult(false, "交换机删除失败");
            }
            return new BocloudResult(true, "交换机删除成功");
        } catch (Exception e) {
            logger.error("交换机删除失败:" + e);
            return new BocloudResult(false, e.getMessage(), "交换机删除失败:" + e.getMessage());
        }
	}

}
