package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSON;
import com.aliyun.ecs20140526.models.*;
import com.aliyun.ecs20140526.models.DescribeVpcsResponseBody.DescribeVpcsResponseBodyVpcs;
import com.aliyun.ecs20140526.models.DescribeVpcsResponseBody.DescribeVpcsResponseBodyVpcsVpc;
import com.aliyun.vpc20160428.models.AssociateVpcCidrBlockRequest;
import com.aliyun.vpc20160428.models.AssociateVpcCidrBlockResponse;
import com.aliyun.vpc20160428.models.UnassociateVpcCidrBlockRequest;
import com.aliyun.vpc20160428.models.UnassociateVpcCidrBlockResponse;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.VpcConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author ymh
 * @时间 2019年1月24日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunVpcProvider extends AliyunProvider implements VpcProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunVpcProvider.class);

	private static final int PAGE_SIZE = 50;

	private BeanConvertor<VpcModel, DescribeVpcsResponseBodyVpcsVpc> convertor = new VpcConvertor();

	public AliyunVpcProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(VpcModel model) {
		try {
			com.aliyun.ecs20140526.Client ecsClient = this.getECSClient();
			CreateVpcRequest createVpcRequest = new CreateVpcRequest();
			createVpcRequest.setVpcName(model.getName());
			createVpcRequest.setDescription(model.getRemark());
			createVpcRequest.setCidrBlock(model.getCidr());
			createVpcRequest.setRegionId(this.getRegionId());
			// 复制代码运行请自行打印 API 的返回值
			CreateVpcResponse createVpc = ecsClient.createVpc(createVpcRequest);
			if (null == createVpc.getBody()) {
				return new BocloudResult(false, "vpc创建失败");
			}
			String vpcId = createVpc.getBody().getVpcId();
			Thread.sleep(2000);
			return new BocloudResult(true, getAvailable(this.getRegionId(), vpcId), "vpc创建成功");
		} catch (Exception e) {
			logger.error("vpc创建失败:" + e);
			return new BocloudResult(false, "vpc创建失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String vpcId) {
		try {
			DeleteVpcRequest request = new DeleteVpcRequest();
			request.setVpcId(vpcId);
			request.setRegionId(this.getRegionId());
			com.aliyun.ecs20140526.Client ecsClient = this.getECSClient();
			DeleteVpcResponse deleteVpc = ecsClient.deleteVpc(request);
			if (null == deleteVpc) {
				return new BocloudResult(false, "vpc删除失败");
			}
			return new BocloudResult(true, deleteVpc.getBody().getRequestId(), "vpc删除成功！");
		} catch (Exception e) {
			logger.error("vpc删除失败:" + e);
			return new BocloudResult(false, "vpc删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list() {
		DescribeVpcsRequest describe = new DescribeVpcsRequest();
		describe.setPageNumber(1);
		describe.setPageSize(PAGE_SIZE);
		describe.setRegionId(this.getRegionId());
		List<VpcModel> models = Lists.newArrayList();
		try {
			DescribeVpcsResponse describeVpcs = this.getECSClient().describeVpcs(describe);
			DescribeVpcsResponseBodyVpcs vpcsBody = describeVpcs.getBody().getVpcs();
			Integer totalCount = describeVpcs.getBody().getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, models, "empty vpcs");
			}
			List<DescribeVpcsResponseBodyVpcsVpc> vpcs = vpcsBody.getVpc();
			if(!ListTool.isEmpty(vpcs)) {
			    logger.info("网络 VPC : " + JSON.toJSONString(vpcs.get(0)));
			}
			for (DescribeVpcsResponseBodyVpcsVpc vpc : vpcs) {
				VpcModel convertModel = convertor.convertModel(vpc);
				convertModel.setRegionId(this.getRegionId());
				models.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				describe.setPageNumber(i);
				describe.setPageSize(PAGE_SIZE);
				describeVpcs = this.getECSClient().describeVpcs(describe);
				vpcsBody = describeVpcs.getBody().getVpcs();
				for (DescribeVpcsResponseBodyVpcsVpc vpc : vpcs) {
					VpcModel convertModel = convertor.convertModel(vpc);
					convertModel.setRegionId(this.getRegionId());
					models.add(convertModel);
				}
			}
			return new BocloudResult(true, models, "vpc列表查询成功");
		} catch (Exception e) {
			logger.error("vpc列表查询失败:" + e);
			return new BocloudResult(false, "vpc列表查询失败");
		}
	}

	@Override
	public BocloudResult detail(String vpcId, String regionId) {
		try {
			DescribeVpcsRequest request = new DescribeVpcsRequest();
			request.setVpcId(vpcId);
			request.setRegionId(regionId);
			DescribeVpcsResponse response = this.getECSClient().describeVpcs(request);
			DescribeVpcsResponseBodyVpcs vpcs = response.getBody().getVpcs();
			if (null == vpcs) {
				return new BocloudResult(false, "vpc不存在");
			}
			return new BocloudResult(true, convertor.convertModel(vpcs.getVpc().get(0)), "查询成功");
		} catch (Exception e) {
			logger.error("vpc详情查询失败:" + e);
			return new BocloudResult(false, "vpc详情查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociateCidrBlock(String vpcId) {
		try {
			return new BocloudResult(true, "开发中...");
		} catch (Exception e) {
			logger.error("vpc关闭IPV6失败:" + e);
			return new BocloudResult(false, "vpc关闭IPV6失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult associateIpv6CidrBlock(String vpcId) {
		try {
			return new BocloudResult(true, "功能开发中...");
		} catch (Exception e) {
			logger.error("vpc开启IPV6失败:" + e);
			return new BocloudResult(false, "vpc开启IPV6失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult update(VpcModel model) {
		try {
			ModifyVpcAttributeRequest request = new ModifyVpcAttributeRequest();
			request.setVpcId(model.getVpcId());
			request.setRegionId(this.getRegionId());
			request.setVpcName(model.getName());
			request.setDescription(model.getRemark());
			ModifyVpcAttributeResponse modifyVpcAttribute = this.getECSClient().modifyVpcAttribute(request);
			if (null == modifyVpcAttribute.getBody()) {
				return new BocloudResult(false, "vpc修改失败");
			}
			return new BocloudResult(true, this.detail(model.getVpcId(), this.getRegionId()).getData(), "vpc修改成功");
		} catch (Exception e) {
			logger.error("vpc修改失败:" + e);
			return new BocloudResult(false, "vpc修改失败:" + e.getMessage());
		}
	}

	public void delIpv6Gateways(String vpcUuid) {
		try {
		} catch (Exception e) {
			logger.error("查询IPV6网关异常：" + e.getMessage());
		}
	}

	private VpcModel getAvailable(String regionId, String vpcId) {
		DescribeVpcsRequest request = new DescribeVpcsRequest();
		request.setVpcId(vpcId);
		request.setRegionId(this.getRegionId());
		VpcModel model = new VpcModel();
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
				DescribeVpcsResponse describeVpcs = this.getECSClient().describeVpcs(request);
				DescribeVpcsResponseBodyVpcs describeVpcsResponseBodyVpcs = describeVpcs.getBody().getVpcs();
				List<DescribeVpcsResponseBodyVpcsVpc> vpcs = describeVpcsResponseBodyVpcs.getVpc();
				if (ListTool.isEmpty(vpcs)) {
					flag = false;
					break;
				}
				if ("Available".equalsIgnoreCase(vpcs.get(0).getStatus())) {
					model = convertor.convertModel(vpcs.get(0));
					model.setRegionId(this.getRegionId());
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 5 * 60 * 1000) {
					model = convertor.convertModel(vpcs.get(0));
					flag = false;
					break;
				}
			}
		} catch (Exception e) {
			logger.error("get vpc error:", e);
		}
		return model;
	}

	@Override
	public BocloudResult associateCidrBlock(String vpcId, String cidrBlock) {
		try {
			AssociateVpcCidrBlockRequest request = new AssociateVpcCidrBlockRequest();
			request.setVpcId(vpcId);
			request.setSecondaryCidrBlock(cidrBlock);
			AssociateVpcCidrBlockResponse associateVpcCidrBlock = this.getVPCClient().associateVpcCidrBlock(request);
			if (null == associateVpcCidrBlock) {
				return new BocloudResult(false, "添加附加IPv4网段失败");
			}
			return new BocloudResult(true, "添加附加IPv4网段成功");
		} catch (Exception e) {
			logger.error("添加附加IPv4网段失败:" + e);
			return new BocloudResult(false, "添加附加IPv4网段失败" + e.getMessage());
		}
	}

	public BocloudResult disAssociateVpcCidrBlock(String vpcId, String cidrBlock) {
		try {
			UnassociateVpcCidrBlockRequest request = new UnassociateVpcCidrBlockRequest();
			request.setVpcId(vpcId);
			request.setSecondaryCidrBlock(cidrBlock);
			UnassociateVpcCidrBlockResponse unassociateVpcCidrBlock = this.getVPCClient()
					.unassociateVpcCidrBlock(request);
			if (null == unassociateVpcCidrBlock) {
				return new BocloudResult(false, "删除附加IPv4网段失败");
			}
			return new BocloudResult(true, "删除附加IPv4网段成功");
		} catch (Exception e) {
			logger.error("删除附加IPv4网段失败:" + e);
			return new BocloudResult(false, "删除附加IPv4网段失败" + e.getMessage());
		}
	}
}
