package com.bocloud.cmp.provider.security;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.aliyun.ecs20140526.models.*;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.KeypairConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.api.client.util.Lists;

/**
 * 密钥操作类
 *
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunKeypairProvider extends AliyunProvider implements KeypairProvider {

    private static Logger logger = LoggerFactory.getLogger(AliyunKeypairProvider.class);

    private BeanConvertor<KeypairModel,
        DescribeKeyPairsResponseBody.DescribeKeyPairsResponseBodyKeyPairsKeyPair> convertor = new KeypairConvertor();

    private static final int PAGE_SIZE = 100;

    public AliyunKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeKeyPairsRequest request = new DescribeKeyPairsRequest();
            request.setPageNumber(1);
//            request.setPageSize(PAGE_SIZE);
            request.setRegionId(this.getRegionId());
            List<KeypairModel> models = Lists.newArrayList();
            DescribeKeyPairsResponse response = this.getECSClient().describeKeyPairs(request);

            if (null == response.getBody() || null == response.getBody().getKeyPairs()) {
                return new BocloudResult(false, "交换机列表查询失败");
            }

            Integer totalCount = response.getBody().getTotalCount() == null ? 0 : response.getBody().getTotalCount();
            /*if (null == totalCount || totalCount == 0) {
            	return new BocloudResult(true, models, "success");
            }*/

            List<DescribeKeyPairsResponseBody.DescribeKeyPairsResponseBodyKeyPairsKeyPair> keyPairs =
                response.getBody().getKeyPairs().getKeyPair();
            for (DescribeKeyPairsResponseBody.DescribeKeyPairsResponseBodyKeyPairsKeyPair keyPair : keyPairs) {
                KeypairModel convertModel = convertor.convertModel(keyPair);
                convertModel.setRegionId(this.getRegionId());
                models.add(convertModel);
            }
//            int i = 1;
//            while (keyPairs.size() > 0) {
//                logger.info("keyPairs.size() : " + keyPairs.size());
//                request.setPageNumber(++i);
//                request.setPageSize(PAGE_SIZE);
//                response = this.getECSClient().describeKeyPairs(request);
//                keyPairs = response.getBody().getKeyPairs().getKeyPair();
//                for (DescribeKeyPairsResponseBody.DescribeKeyPairsResponseBodyKeyPairsKeyPair keyPair : keyPairs) {
//                    KeypairModel convertModel = convertor.convertModel(keyPair);
//                    convertModel.setRegionId(this.getRegionId());
//                    models.add(convertModel);
//                }
//            }
            /* int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1) {
                return new BocloudResult(true, models, "success");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getECSClient().describeKeyPairs(request);
                keyPairs = response.getBody().getKeyPairs().getKeyPair();
                for (DescribeKeyPairsResponseBody.DescribeKeyPairsResponseBodyKeyPairsKeyPair keyPair : keyPairs) {
                    KeypairModel convertModel = convertor.convertModel(keyPair);
                    convertModel.setRegionId(this.getRegionId());
                    models.add(convertModel);
                }
            }*/
            return new BocloudResult(true, models, "密钥查询成功");
        } catch (Exception e) {
            logger.error("密钥查询失败:" + e.getMessage());
            return new BocloudResult(false, "密钥查询失败" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        try {
            DescribeKeyPairsRequest request = new DescribeKeyPairsRequest();
            request.setKeyPairName(keypairName);
            request.setRegionId(this.getRegionId());
            DescribeKeyPairsResponse response = this.getECSClient().describeKeyPairs(request);
            if (null == response || response.getBody().getKeyPairs().getKeyPair().size() == 0) {
                return new BocloudResult(false, "密钥详情查询失败");
            }
            KeypairModel model = convertor.convertModel(response.getBody().getKeyPairs().getKeyPair().get(0));
            return new BocloudResult(true, model, "密钥详情查询成功");
        } catch (Exception e) {
            logger.error("密钥详情查询失败:" + e);
            return new BocloudResult(false, "密钥详情查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(KeypairModel model) {
        try {
            model.setRegionId(this.getRegionId());
            if (!StringUtils.isEmpty(model.getPublicKey())) {
                ImportKeyPairRequest request = new ImportKeyPairRequest();
                request.setKeyPairName(model.getName());
                request.setRegionId(this.getRegionId());
                request.setPublicKeyBody(model.getPublicKey());
                if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
                    request.setResourceGroupId(model.getResourceGroupUuid());
                }
                ImportKeyPairResponse response = this.getECSClient().importKeyPair(request);
                model.setFingerprint(response.getBody().getKeyPairFingerPrint());
            } else {
                CreateKeyPairRequest request = new CreateKeyPairRequest();
                request.setKeyPairName(model.getName());
                request.setRegionId(this.getRegionId());
                if (!StringUtils.isEmpty(model.getResourceGroupUuid())) {
                    request.setResourceGroupId(model.getResourceGroupUuid());
                }
                CreateKeyPairResponse response = this.getECSClient().createKeyPair(request);
                model.setKeyId(response.getBody().getKeyPairId());
                model.setFingerprint(response.getBody().getKeyPairFingerPrint());
                model.setPrivateKey(response.getBody().getPrivateKeyBody());
            }
            return new BocloudResult(true, model, "密钥创建成功");
        } catch (Exception e) {
            logger.error("密钥创建失败:" + e);
            return new BocloudResult(false, "密钥创建失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            DeleteKeyPairsRequest request = new DeleteKeyPairsRequest();
            request.setRegionId(this.getRegionId());
            List<String> keypairList = new ArrayList<String>();
            keypairList.add(keypairName);
            request.setKeyPairNames(JSON.toJSONString(keypairList));
            DeleteKeyPairsResponse acsResponse = this.getECSClient().deleteKeyPairs(request);
            return new BocloudResult(true, acsResponse.getBody().getRequestId(), "密钥删除成功");
        } catch (Exception e) {
            logger.error("密钥删除失败:" + e);
            return new BocloudResult(false, "密钥删除失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        return null;
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        // TODO Auto-generated method stub
        return null;
    }

}
