package com.bocloud.cmp.provider.storage;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.aliyun.oss.OSS;
import com.aliyun.oss.internal.OSSHeaders;
import com.aliyun.oss.model.CannedAccessControlList;
import com.aliyun.oss.model.GetObjectRequest;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.OSSObjectSummary;
import com.aliyun.oss.model.ObjectListing;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.PutObjectRequest;
import com.aliyun.oss.model.StorageClass;
import com.bocloud.cmp.model.BucketObjectModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.AliyunBucketObjectConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * aliyun oss
 *
 * @author ymh
 * @时间 2019年3月5日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunBucketObjectProvider extends AliyunProvider implements BucketObjectProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunBucketObjectProvider.class);
	
	BeanConvertor<BucketObjectModel, OSSObjectSummary> bucketObjectConvertor = new AliyunBucketObjectConvertor();

	public AliyunBucketObjectProvider(Butler butler) {
		super(butler);
	}
	
	@Override
	public BocloudResult list(String bucketName) {
		try {
			List<BucketObjectModel> syncBucketObjects = syncBucketObjects(bucketName);
			if (!ListTool.isEmpty(syncBucketObjects)) {
				for (BucketObjectModel bucketObjectModel : syncBucketObjects) {
					if ("Folder".equalsIgnoreCase(bucketObjectModel.getType())) {
						String key = bucketObjectModel.getKey();
						bucketObjectModel.setName(key.substring(0, key.length() - 1));
						bucketObjectModel.setType("Folder");
						bucketObjectModel.setIsDirectory(true);
					}
					if ("File".equalsIgnoreCase(bucketObjectModel.getType())) {
						bucketObjectModel.setName(bucketObjectModel.getKey());
						bucketObjectModel.setIsDirectory(false);
						String name = bucketObjectModel.getName();
						if (!StringUtils.isEmpty(name) && name.contains(".")) {
							bucketObjectModel.setType(name.substring(name.lastIndexOf(".") + 1));
						} else {
							bucketObjectModel.setType("未知");
						}
					}
				}
			}
			return new BocloudResult<>(true, syncBucketObjects, "查询成功");
		} catch (Exception e) {
			logger.error("查询文件列表异常：" + e);
			return new BocloudResult<>(false, "查询文件列表异常：" + e);
		}
	}

	static List<BucketObjectModel> models = new ArrayList<BucketObjectModel>();
	
	public List<BucketObjectModel> syncBucketObjects(String bucketName) {
		OSS ossClient = getOSSClient();
		ObjectListing listObjects = ossClient.listObjects(bucketName);
		List<OSSObjectSummary> objectSummaries = listObjects.getObjectSummaries();
		List<BucketObjectModel> syncBuckets = new ArrayList<BucketObjectModel>();
		List<BucketObjectModel> convertList = bucketObjectConvertor.convertList(objectSummaries);
		for (BucketObjectModel model : convertList) {
			String key = model.getKey();
			syncBucketObjects(model, 0, syncBuckets, "", key);
		}
		if (!ListTool.isEmpty(models)) {
			for (BucketObjectModel newModel : models) {
				Boolean flag = true;
				for (BucketObjectModel bucketObjectModel : syncBuckets) {
					if (newModel.getKey().equals(bucketObjectModel.getKey())
							&& newModel.getParentKey().equals(bucketObjectModel.getParentKey())) {
						flag = false;
						break;
					}
				}
				if (flag) {
					syncBuckets.add(newModel);
				}
			}
			models.clear();
		}
		return syncBuckets;
	}

	public void syncBucketObjects(BucketObjectModel bucketObjectModel, Integer index, List<BucketObjectModel> objects,
			String parentKey, String absolutePath) {
		String key = bucketObjectModel.getKey();
		if (!key.contains("/")) {
			bucketObjectModel.setObjectIndex(index);
			bucketObjectModel.setParentKey(parentKey);
			bucketObjectModel.setType("File");
			bucketObjectModel.setAbsolutePath(absolutePath);
			objects.add(bucketObjectModel);
		} else {
			String substring = key.substring(key.indexOf("/") + 1);
			if (StringUtils.isEmpty(substring)) {
				bucketObjectModel.setObjectIndex(index);
				bucketObjectModel.setParentKey(parentKey);
				bucketObjectModel.setAbsolutePath(absolutePath);
				bucketObjectModel.setType("Folder");
				objects.add(bucketObjectModel);
			} else {
				checkExist(key.substring(0, key.indexOf("/") + 1), index, parentKey, objects,
						bucketObjectModel.getBucketName());
				bucketObjectModel.setKey(substring);
				bucketObjectModel.setAbsolutePath(key);
				syncBucketObjects(bucketObjectModel, index + 1, objects, key.substring(0, key.indexOf("/") + 1),
						absolutePath);
			}
		}
	}
	
	private void checkExist(String key, Integer index, String parentKey, List<BucketObjectModel> objects,
			String bucketName) {
		Boolean flag = true;
		for (BucketObjectModel jsonObject : objects) {
			String existKey = jsonObject.getKey();
			if (null != existKey && existKey.equals(key)) {
				flag = false;
				return;
			}
		}
		if (flag) {
			BucketObjectModel bucketObjectModel = new BucketObjectModel();
			for (BucketObjectModel model : models) {
				if (index.equals(model.getObjectIndex()) && parentKey.equals(model.getParentKey())
						&& key.equals(model.getKey())) {
					return;
				}
			}
			bucketObjectModel.setObjectIndex(index);
			bucketObjectModel.setBucketName(bucketName);
			bucketObjectModel.setParentKey(parentKey);
			bucketObjectModel.setKey(key);
			bucketObjectModel.setType("Folder");
			models.add(bucketObjectModel);
		}
	}
	
	@Override
	public BocloudResult remove(BucketObjectModel model) {
		try {
			OSS ossClient = getOSSClient();
			ossClient.deleteObject(model.getBucketName(), model.getKey());
			// 关闭OSSClient。
			ossClient.shutdown();
			return new BocloudResult<>(true, "删除成功");
		} catch (Exception e) {
			return new BocloudResult<>(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(BucketObjectModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public InputStream download(BucketObjectModel model) {
		try {
			GetObjectRequest getObjectRequest = new GetObjectRequest(model.getBucketName(), model.getAbsolutePath());
			OSSObject object = getOSSClient().getObject(getObjectRequest);
			return object.getObjectContent();
		} catch (Exception e) {
			return null;
		}
	}

	@Override
	public BocloudResult upload(BucketObjectModel model) {
		try {
			OSS ossClient = getOSSClient();
			File file = new File(model.getFilepath());
			PutObjectRequest putObjectRequest = new PutObjectRequest(model.getBucketName(),
					model.getAbsolutePath(), file);
			ObjectMetadata metadata = new ObjectMetadata();
			metadata.setHeader(OSSHeaders.OSS_STORAGE_CLASS, StorageClass.Standard.toString());
			metadata.setObjectAcl(CannedAccessControlList.valueOf(model.getObjectAcl()));
			putObjectRequest.setMetadata(metadata);
			// 上传文件。
			ossClient.putObject(putObjectRequest);
			// 关闭OSSClient。
			ossClient.shutdown();
			return new BocloudResult<>(true, "文件夹创建成功");
		} catch (Exception e) {
			return new BocloudResult<>(false, "文件夹创建失败：" + e.getMessage());
		}
	}
	
	@Override
	public BocloudResult acl(BucketObjectModel model) {
		try {
			OSS ossClient = getOSSClient();
			ossClient.setObjectAcl(model.getBucketName(), model.getAbsolutePath(),
					CannedAccessControlList.valueOf(model.getAcl()));
			// 关闭OSSClient。
			ossClient.shutdown();
			return new BocloudResult<>(true, "ACL设置成功");
		} catch (Exception e) {
			return new BocloudResult<>(false, "ACL设置失败：" + e.getMessage());
		}
	}

}
