package com.bocloud.cmp.provider.storage;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.aliyun.oss.OSS;
import com.aliyun.oss.model.AccessControlList;
import com.aliyun.oss.model.Bucket;
import com.aliyun.oss.model.BucketInfo;
import com.aliyun.oss.model.CannedAccessControlList;
import com.aliyun.oss.model.CreateBucketRequest;
import com.aliyun.oss.model.StorageClass;
import com.bocloud.cmp.model.BucketModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AliyunProvider;
import com.bocloud.cmp.provider.aliyunconvertor.AliyunBucketConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * aliyun oss
 *
 * @author ymh
 * @时间 2019年3月5日
 */
@Provider("bocloud.cmp.aliyun")
public class AliyunBucketProvider extends AliyunProvider implements BucketProvider {

	private static Logger logger = LoggerFactory.getLogger(AliyunBucketProvider.class);

	BeanConvertor<BucketModel, BucketInfo> bucketConvertor = new AliyunBucketConvertor();

	public AliyunBucketProvider(Butler butler) {
		super(butler);
	}

	public BocloudResult list() {
		try {
			OSS ossClient = getOSSClient();
			List<Bucket> listBuckets = ossClient.listBuckets();
			if (!ListTool.isEmpty(listBuckets)) {
				listBuckets = listBuckets.stream()
						.filter(bucket -> !StringUtils.isEmpty(bucket.getRegion())
								&& this.getRegionId().equalsIgnoreCase(bucket.getRegion()))
						.collect(Collectors.toList());
			}
			List<BucketModel> convertList = new ArrayList<>();
			listBuckets.forEach((bucket) -> {
				// 获取存储桶的info
				BucketInfo bucketInfo = ossClient.getBucketInfo(bucket.getName());
				convertList.add(bucketConvertor.convertModel(bucketInfo));
			});
			return new BocloudResult<>(true, convertList, "列表查询成功");
		} catch (Exception e) {
			logger.error("查询对象存储异常：" + e);
			return new BocloudResult(false, "查询对象存储异常：" + e);
		}
	}
	
	public BocloudResult create(BucketModel model) {
		try {
			OSS ossClient = getOSSClient();
			CreateBucketRequest createBucketRequest = new CreateBucketRequest(model.getName());
			createBucketRequest.setStorageClass(StorageClass.Standard);
			createBucketRequest.setCannedACL(CannedAccessControlList.valueOf(model.getBucketAcl()));
			Bucket createBucket = ossClient.createBucket(createBucketRequest);
			BucketInfo bucketInfo = ossClient.getBucketInfo(createBucket.getName());
			BucketModel bucketModel = bucketConvertor.convertModel(bucketInfo);
			bucketModel.setRegionId(this.getRegionId());
			return new BocloudResult<>(true, bucketModel, "bucket创建成功");
		} catch (Exception e) {
			logger.error("bucket创建失败：" + e);
			return new BocloudResult(false, e.getMessage());
		}
	}
	
	public BocloudResult remove(BucketModel model) {
		try {
			OSS ossClient = getOSSClient();
			ossClient.deleteBucket(model.getName());
			return new BocloudResult<>(true, "删除bucket成功");
		} catch (Exception e) {
			logger.error("删除bucket异常：" + e);
			return new BocloudResult(false, "删除bucket异常：" + e);
		}
	}

	public BocloudResult remove(String name) {
		return new BocloudResult(false, "暂不支持");
	}

	public BocloudResult exists(String name) {
		try {
			OSS ossClient = getOSSClient();
			boolean exists = ossClient.doesBucketExist(name);
			return new BocloudResult<>(true, exists, exists ? "Bucket已存在" : "不存在");
		} catch (Exception e) {
			logger.error("查询存储空间是否存在异常：" + e);
			return new BocloudResult(false, "查询存储空间是否存在：" + e);
		}
	}
	
	public BocloudResult detail(String bucketName) {
		try {
			OSS ossClient = getOSSClient();
			BucketInfo bucketInfo = ossClient.getBucketInfo(bucketName);
			return new BocloudResult<>(true, bucketConvertor.convertModel(bucketInfo), "查询成功");
		} catch (Exception e) {
			logger.error("查询详情异常：" + e);
			return new BocloudResult(false, "查询详情异常：" + e);
		}
	}

	public BocloudResult detail(Long bucketId) {
		return new BocloudResult(false, "暂不支持");
	}

	public BocloudResult acl(BucketModel model) {
		return new BocloudResult(false, "暂不支持");
	}

	public BocloudResult create(String bucket, boolean share) {
		return new BocloudResult(false, "暂不支持");
	}

	public BocloudResult acl(String bucket, boolean share) {
		return new BocloudResult(false, "暂不支持");
	}

}
