package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.CloudCostModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.costexplorer.model.Group;
import software.amazon.awssdk.services.costexplorer.model.ResultByTime;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

public class CostConvertor implements BeanConvertor<CloudCostModel, ResultByTime> {
    private static final Logger logger = LoggerFactory.getLogger(CostConvertor.class);

    public List<CloudCostModel> convertList(List<? extends ResultByTime> resultByTimes, String granularity, String metrics, String dimension) {
        List<CloudCostModel> cloudCostModels = new ArrayList<>();
        for (ResultByTime resultByTime : resultByTimes) {
            cloudCostModels.addAll(convertModels(resultByTime, granularity, metrics, dimension));
        }
        return cloudCostModels;
    }

    public List<CloudCostModel> convertModels(ResultByTime resultByTime, String granularity, String metrics, String dimension) {
        List<CloudCostModel> cloudCostModels = new ArrayList<>();
        try {
            List<Group> groups = resultByTime.groups();
            if (!ListTool.isEmpty(groups)) {
                for (Group group :
                        groups) {
                    CloudCostModel model = new CloudCostModel();
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss Z");
                    model.setStart(format.parse(resultByTime.timePeriod().start().replace("Z", " UTC")));
                    model.setEnd(format.parse(resultByTime.timePeriod().end().replace("Z", " UTC")));
                    model.setGroupUuid(group.keys().get(0));
                    String key = group.metrics().keySet().stream().findFirst().get();
                    model.setAmount(group.metrics().get(key).amount());
                    model.setUnit(group.metrics().get(key).unit());
                    model.setDimension(dimension);
                    model.setGranularity(granularity);
                    model.setMetrics(metrics);
                    cloudCostModels.add(model);
                }
            } else {
                CloudCostModel model = new CloudCostModel();
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss Z");
                model.setStart(format.parse(resultByTime.timePeriod().start().replace("Z", " UTC")));
                model.setEnd(format.parse(resultByTime.timePeriod().end().replace("Z", " UTC")));
                String key = resultByTime.total().keySet().stream().findFirst().get();
                model.setAmount(resultByTime.total().get(key).amount());
                model.setUnit(resultByTime.total().get(key).unit());
                model.setDimension(dimension);
                model.setGranularity(granularity);
                model.setMetrics(metrics);
                cloudCostModels.add(model);
            }
        } catch (ParseException e) {
            logger.error("Cost Convertor error :", e);
        }

        return cloudCostModels;
    }

    @Override
    public List<CloudCostModel> convertList(List<? extends ResultByTime> models) {
        return null;
    }

    @Override
    public CloudCostModel convertModel(ResultByTime resultByTime) {
        return null;
    }

}
