package com.bocloud.cmp.awsconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import org.springframework.util.CollectionUtils;
import software.amazon.awssdk.services.ec2.model.DeviceType;
import software.amazon.awssdk.services.ec2.model.Image;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ImageConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:11
 */
public class ImageConvertor implements BeanConvertor<ImageModel, Image> {
    @Override
    public List<ImageModel> convertList(List<? extends Image> images) {
        List<ImageModel> imageModel = new ArrayList<>();
        for (Image image : images) {
            //过滤市场镜像
            if(image.hasProductCodes()){
                continue;
            }
            if(CollectionUtils.isEmpty(image.blockDeviceMappings())){
                continue;
            }
            if(image.rootDeviceType()!= DeviceType.EBS){
                continue;
            }
            if(image.platformDetails().equalsIgnoreCase("Linux/UNIX")
                    ||image.platformDetails().equalsIgnoreCase("Windows")){
                imageModel.add(convertModel(image));
            }
        }
        return imageModel;
    }

    @Override
    public ImageModel convertModel(Image image) {
        ImageModel model = new ImageModel();
        model.setImageUuid(image.imageId());
        model.setName(image.name());
        if (!ListTool.isEmpty(image.tags())) {
            for (Tag tag :
                    image.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        model.setOsCategory(image.platformDetails());
        model.setProps(image.name());
        model.setStatus(image.stateAsString());
        model.setOsType(image.imageTypeAsString());
        if ("available".equalsIgnoreCase(image.stateAsString())) {
            model.setStatus("ACTIVE");
        } else {
            model.setStatus(image.stateAsString());
        }
        model.setRemark(image.description());
        model.setArchitecture(image.architectureAsString());
        if (image.publicLaunchPermissions()) {
            model.setImageType("PUBLIC");
        } else {
            model.setImageType("PRIVATE");
        }
        model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", image.creationDate(), "Z"));
        model.setBlockDeviceMappings(JSONObject.parseObject(JSONObject.toJSONString(image.toBuilder())).get("blockDeviceMappings").toString());
        model.setRootDeviceName(image.rootDeviceName());
        model.setRootDeviceType(image.rootDeviceTypeAsString());
        return model;
    }
}
