package com.bocloud.cmp.awsconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.InternetGatewayModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.InternetGateway;
import software.amazon.awssdk.services.ec2.model.InternetGatewayAttachment;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class InternetGatewayConvertor implements BeanConvertor<InternetGatewayModel, InternetGateway> {
    @Override
    public List<InternetGatewayModel> convertList(List<? extends InternetGateway> gateways) {
        List<InternetGatewayModel> gatewayModels = new ArrayList<>();
        for (InternetGateway gateway : gateways) {
            gatewayModels.add(convertModel(gateway));
        }
        return gatewayModels;
    }

    @Override
    public InternetGatewayModel convertModel(InternetGateway gateway) {
        InternetGatewayModel model = JSONObject.parseObject(JSONObject.toJSONString(gateway.toBuilder()), InternetGatewayModel.class);
        model.setInternetGatewayUuid(gateway.internetGatewayId());
        model.setName(gateway.internetGatewayId());
        if (!ListTool.isEmpty(gateway.tags())) {
            for (Tag tag :
                    gateway.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        Optional.ofNullable(gateway.attachments()).ifPresent(attachments -> model.setVpcId(attachments.stream().findFirst().orElse(
                InternetGatewayAttachment.builder().vpcId(null).build()).vpcId()));
        return model;
    }
}
