package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.NetworkInterface;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: NetworkInterfaceConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/11/21 15:07
 */
public class NetworkInterfaceConvertor implements BeanConvertor<NetworkInterfaceModel, NetworkInterface> {
    @Override
    public List<NetworkInterfaceModel> convertList(List<? extends NetworkInterface> models) {
        List<NetworkInterfaceModel> interfaceModels = new ArrayList<>();
        for (NetworkInterface networkInterface : models) {
            interfaceModels.add(convertModel(networkInterface));
        }
        return interfaceModels;
    }

    @Override
    public NetworkInterfaceModel convertModel(NetworkInterface networkInterface) {
        NetworkInterfaceModel model = new NetworkInterfaceModel();
        model.setName(networkInterface.networkInterfaceId());
        if (!ListTool.isEmpty(networkInterface.tagSet())) {
            for (Tag tag :
                    networkInterface.tagSet()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        model.setNetworkInterfaceUuid(networkInterface.networkInterfaceId());
        model.setRemark(networkInterface.description());
        model.setZone(networkInterface.availabilityZone());
        Optional.ofNullable(networkInterface.groups()).ifPresent(groups -> model.setSecurityGroups(groups.stream().map(group -> group.groupId()).collect(Collectors.toList())));
        model.setNetworkUuid(networkInterface.vpcId());
        model.setSubnetUuid(networkInterface.subnetId());
        model.setPublicIPAddress(Optional.ofNullable(networkInterface.association()).map(association -> association.publicIp()).orElse(null));
        model.setPrimaryPrivateIPAddress(networkInterface.privateIpAddress());
        model.setIpv6Address(networkInterface.ipv6Addresses().size() > 0 ? networkInterface.ipv6Addresses().get(0).ipv6Address() : null);
        model.setStatus(networkInterface.statusAsString());
        model.setInterfaceType(networkInterface.interfaceTypeAsString());
        model.setVirtualMachineUuid(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.instanceId()).orElse(null));
        model.setAttachmentUuid(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.attachmentId()).orElse(null));
        model.setDeleteOnTermination(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.deleteOnTermination()).orElse(false));
        model.setDeviceIndex(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.deviceIndex()).orElse(-1));
        model.setAttachedTime(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.attachTime() != null ? DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", attachment.attachTime().toString(), "Z") : null).orElse(null));
        model.setAttchmentStatus(Optional.ofNullable(networkInterface.attachment()).map(attachment -> attachment.statusAsString()).orElse(null));
        return model;
    }
}
