package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.entity.enums.RouteTableType;
import com.bocloud.cmp.model.RouteModel;
import com.bocloud.cmp.model.RouteTableAssociationModel;
import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.ArrayList;
import java.util.List;

public class RouteTableConvertor implements BeanConvertor<RouteTableModel, RouteTable> {
    @Override
    public List<RouteTableModel> convertList(List<? extends RouteTable> list) {
        List<RouteTableModel> models = new ArrayList<>();
        for (RouteTable object : list) {
            models.add(convertModel(object));
        }
        return models;
    }

    @Override
    public RouteTableModel convertModel(RouteTable object) {
        RouteTableModel model = new RouteTableModel();
        model.setName(object.routeTableId());
        if (!ListTool.isEmpty(object.tags())) {
            for (Tag tag :
                    object.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        List<RouteTableAssociation> associations = object.associations();
        List<RouteTableAssociationModel> associationModels = new ArrayList<>();
        if (!ListTool.isEmpty(associations)) {
            for (RouteTableAssociation association :
                    associations) {
                RouteTableAssociationModel associationModel =
                        new RouteTableAssociationModel(
                                association.main(),
                                association.routeTableAssociationId(),
                                association.routeTableId(),
                                association.subnetId(),
                                association.gatewayId(),
                                association.associationState().stateAsString());
                associationModels.add(associationModel);
            }
        }
        model.setAssociations(associationModels);

        List<PropagatingVgw> vgws = object.propagatingVgws();
        List<String> vgwModels = new ArrayList<>();
        if (!ListTool.isEmpty(vgws)) {
            for (PropagatingVgw vgw :
                    vgws) {
                vgwModels.add(vgw.gatewayId());
            }
        }
        model.setPropagatingVgws(vgwModels);

        List<Route> routes = object.routes();
        List<RouteModel> routeModels = new ArrayList<>();
        if (!ListTool.isEmpty(routes)) {
            for (Route route :
                    routes) {
                RouteModel routeModel = new RouteModel();

                if (route.egressOnlyInternetGatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.EGRESS_ONLY_INTERNET_GATEWAY.name());
                    routeModel.setTargetUuid(route.egressOnlyInternetGatewayId());
                } else if (route.instanceId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.INSTANCE.name());
                    routeModel.setTargetUuid(route.instanceId());
                } else if (route.gatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.INTERNET_GATEWAY.name());
                    routeModel.setTargetUuid(route.gatewayId());
                } else if (route.localGatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.LOCAL.name());
                    routeModel.setTargetUuid(route.localGatewayId());
                } else if (route.natGatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.NAT_GATEWAY.name());
                    routeModel.setTargetUuid(route.natGatewayId());
                } else if (route.networkInterfaceId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.NETWORK_INTERFACE.name());
                    routeModel.setTargetUuid(route.networkInterfaceId());
                } else if (route.vpcPeeringConnectionId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.PEERING_CONNECTION.name());
                    routeModel.setTargetUuid(route.vpcPeeringConnectionId());
                } else if (route.transitGatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.TRANSIT_GATEWAY.name());
                    routeModel.setTargetUuid(route.transitGatewayId());
                } else if (route.carrierGatewayId() != null) {
                    routeModel.setTargetType(RouteTableType.RouteTargetType.VIRTUAL_PRIVATE_GATEWAY.name());
                    routeModel.setTargetUuid(route.carrierGatewayId());
                }

                if (route.destinationCidrBlock() != null) {
                    routeModel.setDestinationType(RouteTableType.RouteDestinationType.IPV4.name());
                    routeModel.setDestination(route.destinationCidrBlock());
                } else if (route.destinationIpv6CidrBlock() != null) {
                    routeModel.setDestinationType(RouteTableType.RouteDestinationType.IPV6.name());
                    routeModel.setDestination(route.destinationIpv6CidrBlock());
                } else if (route.destinationPrefixListId() != null) {
                    routeModel.setDestinationType(RouteTableType.RouteDestinationType.PREFIX_LIST.name());
                    routeModel.setDestination(route.destinationPrefixListId());
                }

                routeModel.setOrigin(route.originAsString());
                routeModel.setState(route.stateAsString());
                routeModels.add(routeModel);
            }
        }
        model.setRouteModels(routeModels);

        model.setOwnerUuid(object.ownerId());
        model.setUuid(object.routeTableId());
        model.setVpcUuid(object.vpcId());
        return model;
    }
}
