package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.Snapshot;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SnapshotConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:24
 */
public class SnapshotConvertor implements BeanConvertor<SnapshotModel, Snapshot> {
    @Override
    public List<SnapshotModel> convertList(List<? extends Snapshot> snapshots) {
        List<SnapshotModel> snapshotModels = new ArrayList<>();
        for (Snapshot snapshot : snapshots) {
            snapshotModels.add(convertModel(snapshot));
        }
        return snapshotModels;
    }

    @Override
    public SnapshotModel convertModel(Snapshot snapshot) {
        SnapshotModel model = new SnapshotModel();
        model.setName(snapshot.snapshotId());
        if (!ListTool.isEmpty(snapshot.tags())) {
            for (Tag tag :
                    snapshot.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        model.setRemark(snapshot.description());
        model.setSnapshotUuid(snapshot.snapshotId());
        model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", snapshot.startTime().toString(), "Z"));
        model.setStatus(snapshot.stateAsString());
        if ("completed".equalsIgnoreCase(snapshot.stateAsString())) {
            model.setStatus("AVAILABLE");
        }
        model.setVolumeUuid(snapshot.volumeId());
        model.setSize(snapshot.volumeSize());
        return model;
    }
}
