package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.CidrBlockModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.services.ec2.model.Subnet;
import software.amazon.awssdk.services.ec2.model.SubnetIpv6CidrBlockAssociation;
import software.amazon.awssdk.services.ec2.model.Tag;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SubnetConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:41
 */
public class SubnetConvertor implements BeanConvertor<SubnetModel, Subnet> {
    @Override
    public List<SubnetModel> convertList(List<? extends Subnet> subnets) {
        List<SubnetModel> subnetModels = new ArrayList<>();
        for (Subnet subnet : subnets) {
            subnetModels.add(convertModel(subnet));
        }
        return subnetModels;
    }

    @Override
    public SubnetModel convertModel(Subnet subnet) {
        SubnetModel model = new SubnetModel();
        model.setName(subnet.subnetId());
        if (!ListTool.isEmpty(subnet.tags())) {
            for (Tag tag :
                    subnet.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }
        String cidr = subnet.cidrBlock();
        if (!StringUtils.isEmpty(cidr)) {
            model.setGateway(cidr.substring(0, cidr.lastIndexOf('.')) + ".1");
        }

        List<SubnetIpv6CidrBlockAssociation> ipv6Cidrs = subnet.ipv6CidrBlockAssociationSet();
        if (!ListTool.isEmpty(ipv6Cidrs)) {
            model.setIpv6Cidr(subnet.ipv6CidrBlockAssociationSet().get(0).ipv6CidrBlock());

            List<CidrBlockModel> cidrBlockModels = new ArrayList<>();
            for (SubnetIpv6CidrBlockAssociation ipv6Cidr :
                    ipv6Cidrs) {
                if (!"disassociated".equals(ipv6Cidr.ipv6CidrBlockState().stateAsString())) {
                    CidrBlockModel cidrBlockModel = new CidrBlockModel();
                    cidrBlockModel.setUuid(ipv6Cidr.associationId());
                    cidrBlockModel.setType("IPV6");
                    cidrBlockModel.setTargetId(subnet.subnetId());
                    cidrBlockModel.setCidrBlock(ipv6Cidr.ipv6CidrBlock());
                    cidrBlockModel.setCidrBlockState(ipv6Cidr.ipv6CidrBlockState().stateAsString());
                    cidrBlockModels.add(cidrBlockModel);
                }
            }
            model.setCidrBlockModels(cidrBlockModels);
        }

        model.setZone(subnet.availabilityZone());
        model.setCidr(subnet.cidrBlock());
        model.setStatus(subnet.stateAsString());
        model.setSubnetUuid(subnet.subnetId());
        model.setVpcUuid(subnet.vpcId());
        return model;
    }
}
