package com.bocloud.cmp.awsconvertor;

import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import org.springframework.util.CollectionUtils;
import software.amazon.awssdk.services.ec2.model.Tag;
import software.amazon.awssdk.services.ec2.model.Volume;
import software.amazon.awssdk.services.ec2.model.VolumeAttachment;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: VolumeConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:18
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, Volume> {
    @Override
    public List<VolumeModel> convertList(List<? extends Volume> volumes) {
        List<VolumeModel> volumeModels = new ArrayList<>();
        for (Volume volume : volumes) {
            volumeModels.add(convertModel(volume));
        }
        return volumeModels;
    }

    @Override
    public VolumeModel convertModel(Volume volume) {
        VolumeModel model = new VolumeModel();
        Optional.ofNullable(volume.tags()).ifPresent(tags -> model
                .setName(tags.stream().findFirst().orElse(Tag.builder().value(volume.volumeId()).build()).value()));
        model.setZone(volume.availabilityZone());
        model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss.SSSZ", volume.createTime().toString(), "Z"));
        model.setSize(volume.size());
        model.setIops(volume.iops());
        model.setSourceSnapshotId(volume.snapshotId());
        model.setStatus(volume.state().name());
        model.setVolumeUuid(volume.volumeId());
        model.setCategory(volume.volumeTypeAsString());
        if (!CollectionUtils.isEmpty(volume.attachments())) {
            List<VolumeAttachment> attachments = volume.attachments();
            for (VolumeAttachment volumeAttachment : attachments) {
                model.setInstanceId(volumeAttachment.instanceId());
                model.setDeleteWithInstance(volumeAttachment.deleteOnTermination());
                model.setDevice(volumeAttachment.device());
                /**
                 * 根设备 /dev/sda1   块储存设备 /dev/sda1      2020年1月2日20:22:02  sda1是系统盘
                 */
                model.setBootVolume("/dev/xvda".equalsIgnoreCase(volumeAttachment.device())
                        || "/dev/sda1".equalsIgnoreCase(volumeAttachment.device()));
                model.setAttachedTime(
                        DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", volumeAttachment.attachTime().toString(), "Z"));
            }
        } else {
            model.setDeleteWithInstance(false);
            model.setBootVolume(false);
        }
        return model;
    }
}
