package com.bocloud.cmp.awsconvertor;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.CidrBlockModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import software.amazon.awssdk.services.ec2.model.Tag;
import software.amazon.awssdk.services.ec2.model.Vpc;
import software.amazon.awssdk.services.ec2.model.VpcCidrBlockAssociation;
import software.amazon.awssdk.services.ec2.model.VpcIpv6CidrBlockAssociation;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: VpcConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:37
 */
public class VpcConvertor implements BeanConvertor<VpcModel, Vpc> {
    @Override
    public List<VpcModel> convertList(List<? extends Vpc> vpcs) {
        List<VpcModel> vpcModels = new ArrayList<>();
        for (Vpc vpc : vpcs) {
            vpcModels.add(convertModel(vpc));
        }
        return vpcModels;
    }

    @Override
    public VpcModel convertModel(Vpc vpc) {
        VpcModel model = new VpcModel();
        model.setName(vpc.vpcId());
        if (!ListTool.isEmpty(vpc.tags())) {
            for (Tag tag :
                    vpc.tags()) {
                if ("Name".equals(tag.key())) {
                    model.setName(tag.value());
                    break;
                }
            }
        }

        List<CidrBlockModel> cidrBlockModels = new ArrayList<>();

        List<VpcCidrBlockAssociation> cidrs = vpc.cidrBlockAssociationSet();
        if (!ListTool.isEmpty(cidrs)) {
            List<String> cidrBlocks = new ArrayList<>();
            for (VpcCidrBlockAssociation cidr :
                    cidrs) {
                if (!"disassociated".equals(cidr.cidrBlockState().stateAsString())) {
                    CidrBlockModel cidrBlockModel = new CidrBlockModel();
                    cidrBlockModel.setUuid(cidr.associationId());
                    cidrBlockModel.setType("IPV4");
                    cidrBlockModel.setTargetId(vpc.vpcId());
                    cidrBlockModel.setCidrBlock(cidr.cidrBlock());
                    cidrBlockModel.setCidrBlockState(cidr.cidrBlockState().stateAsString());
                    cidrBlockModels.add(cidrBlockModel);
                    cidrBlocks.add(cidr.cidrBlock());
                }
            }
            model.setCidr(JSON.toJSONString(cidrBlocks));
        }

        List<VpcIpv6CidrBlockAssociation> ipv6Cidrs = vpc.ipv6CidrBlockAssociationSet();
        if (!ListTool.isEmpty(ipv6Cidrs)) {
            for (VpcIpv6CidrBlockAssociation ipv6Cidr :
                    ipv6Cidrs) {
                if (!"disassociated".equals(ipv6Cidr.ipv6CidrBlockState().stateAsString())) {
                    CidrBlockModel cidrBlockModel = new CidrBlockModel();
                    cidrBlockModel.setUuid(ipv6Cidr.associationId());
                    cidrBlockModel.setType("IPV6");
                    cidrBlockModel.setTargetId(vpc.vpcId());
                    cidrBlockModel.setCidrBlock(ipv6Cidr.ipv6CidrBlock());
                    cidrBlockModel.setCidrBlockState(ipv6Cidr.ipv6CidrBlockState().stateAsString());
                    cidrBlockModel.setIpv6Pool(ipv6Cidr.ipv6Pool());
                    cidrBlockModel.setNetworkBorderGroup(ipv6Cidr.networkBorderGroup());
                    cidrBlockModels.add(cidrBlockModel);
                    if (model.getIpv6Cidr() == null) {
                        model.setIpv6Cidr(ipv6Cidr.ipv6CidrBlock());
                    }
                }
            }
        }
        model.setCidrBlockModels(cidrBlockModels);

        if ("available".equalsIgnoreCase(vpc.stateAsString())) {
            model.setStatus("ACTIVE");
        } else {
            model.setStatus(vpc.stateAsString());
        }
        model.setVpcId(vpc.vpcId());
        model.setDefaultVpc(vpc.isDefault());
        return model;
    }
}
