package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.awsconvertor.CostConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.CloudCostModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.costexplorer.CostExplorerClient;
import software.amazon.awssdk.services.costexplorer.model.*;

import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsCostProvider extends AwsProvider implements CostProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsCostProvider.class);
    private final CostConvertor convertor = new CostConvertor();

    public AwsCostProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByResource(String granularity, String metrics, String start, String end) {
        try {
            GroupDefinition groupDefinition = GroupDefinition.builder().type("DIMENSION").key("RESOURCE_ID").build();
            DimensionValues dimensionValues = DimensionValues.builder().key("SERVICE").values("Amazon Elastic Compute Cloud - Compute").build();
            Expression expression = Expression.builder().dimensions(dimensionValues).build();
            GetCostAndUsageWithResourcesRequest request = GetCostAndUsageWithResourcesRequest.builder().filter(expression).granularity(granularity).timePeriod(DateInterval.builder().start(start).end(end).build()).groupBy(groupDefinition).metrics(metrics).build();
            GetCostAndUsageWithResourcesResponse response = ((CostExplorerClient) this.getClient(AwsService.CE)).getCostAndUsageWithResources(request);
            List<CloudCostModel> cloudCostModels = convertor.convertList(response.resultsByTime(), granularity, metrics, "RESOURCE_ID");
            return new BocloudResult(true, cloudCostModels, "list cost success!");
        } catch (Exception e) {
            logger.error("list cost error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
