package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.DescribeInstanceTypesRequest;
import software.amazon.awssdk.services.ec2.model.DescribeInstanceTypesResponse;
import software.amazon.awssdk.services.ec2.model.InstanceTypeInfo;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsFlavorProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:27
 */
@Provider("bocloud.cmp.aws")
public class AwsFlavorProvider extends AwsProvider implements FlavorProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsFlavorProvider.class);

    public AwsFlavorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            List<FlavorModel> flavorModels = new ArrayList<>();
            DescribeInstanceTypesRequest request = DescribeInstanceTypesRequest.builder().build();
            DescribeInstanceTypesResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeInstanceTypes(request);
            List<InstanceTypeInfo> instanceTypeInfos = response.instanceTypes();
            for (InstanceTypeInfo instanceTypeInfo :
                    instanceTypeInfos) {
                FlavorModel model = new FlavorModel();
                model.setName(instanceTypeInfo.instanceTypeAsString());
                model.setFlavorUuid(instanceTypeInfo.instanceTypeAsString());
                model.setCpu(instanceTypeInfo.vCpuInfo().defaultVCpus());
                model.setMemory(new BigDecimal((float) instanceTypeInfo.memoryInfo().sizeInMiB() / 1024));
                flavorModels.add(model);
            }

            if (StringUtils.isNotEmpty(response.nextToken())) {
                request = DescribeInstanceTypesRequest.builder().nextToken(response.nextToken()).build();
                response = ((Ec2Client) this.getClient(AwsService.EC2)).describeInstanceTypes(request);
                instanceTypeInfos = response.instanceTypes();
                for (InstanceTypeInfo instanceTypeInfo :
                        instanceTypeInfos) {
                    FlavorModel model = new FlavorModel();
                    model.setName(instanceTypeInfo.instanceTypeAsString());
                    model.setFlavorUuid(instanceTypeInfo.instanceTypeAsString());
                    model.setCpu(instanceTypeInfo.vCpuInfo().defaultVCpus());
                    model.setMemory(new BigDecimal((float) instanceTypeInfo.memoryInfo().sizeInMiB() / 1024));
                    flavorModels.add(model);
                }
            }

            return new BocloudResult(true, flavorModels, "list flavor success!");
        } catch (Exception e) {
            logger.error("list flavor error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        return null;
    }

    @Override
    public BocloudResult remove(String flavorId) {
        return null;
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        return null;
    }
}
