package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.InternetGatewayConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.InternetGatewayModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsInternetGatewayProvider extends AwsProvider implements InternetGatewayProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsInternetGatewayProvider.class);
    private final BeanConvertor<InternetGatewayModel, InternetGateway> convertor = new InternetGatewayConvertor();

    public AwsInternetGatewayProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult create(String name) {
        try {
            CreateInternetGatewayResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createInternetGateway();
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(name)
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.internetGateway().internetGatewayId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            InternetGatewayModel gatewayModel = convertor.convertModel(response.internetGateway());
            return new BocloudResult(true, gatewayModel, "create InternetGateway success!");
        } catch (Exception e) {
            logger.error("create InternetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String internetGatewayId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            DeleteInternetGatewayRequest request = DeleteInternetGatewayRequest.builder().internetGatewayId(internetGatewayId).build();
            DeleteInternetGatewayResponse response = client.deleteInternetGateway(request);
            return new BocloudResult(true, "remove internetGateway success!");
        } catch (Exception e) {
            logger.error("remove internetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String internetGatewayId, String vpcId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            AttachInternetGatewayRequest request = AttachInternetGatewayRequest.builder().internetGatewayId(internetGatewayId).vpcId(vpcId).build();
            AttachInternetGatewayResponse response = client.attachInternetGateway(request);
            return new BocloudResult(true, "attach InternetGateway success!");
        } catch (Exception e) {
            logger.error("attach InternetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detach(String internetGatewayId, String vpcId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            DetachInternetGatewayRequest request = DetachInternetGatewayRequest.builder().internetGatewayId(internetGatewayId).vpcId(vpcId).build();
            DetachInternetGatewayResponse response = client.detachInternetGateway(request);
            return new BocloudResult(true, "detach InternetGateway success!");
        } catch (Exception e) {
            logger.error("detach InternetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeInternetGatewaysResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeInternetGateways();
            List<InternetGatewayModel> models = convertor.convertList(response.internetGateways());
            return new BocloudResult(true, models, "list addresses success!");
        } catch (Exception e) {
            logger.error("list InternetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult update(InternetGatewayModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getInternetGatewayUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            return new BocloudResult(true, "update InternetGateway success!");
        } catch (Exception e) {
            logger.error("update InternetGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
