package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.ListenerConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.elasticloadbalancingv2.ElasticLoadBalancingV2Client;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.*;

import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsListenerProvider extends AwsProvider implements ListenerProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsListenerProvider.class);
    private final BeanConvertor<LbListenerModel, Listener> convertor = new ListenerConvertor();

    public AwsListenerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String loadBalancerArn) {
        try {
            DescribeListenersRequest request = DescribeListenersRequest.builder().loadBalancerArn(loadBalancerArn).build();
            DescribeListenersResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeListeners(request);
            List<LbListenerModel> models = convertor.convertList(response.listeners());
            return new BocloudResult(true, models, "list listeners success!");
        } catch (Exception e) {
            logger.error("list listeners error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(LbListenerModel model) {
        try {
            CreateListenerRequest.Builder builder = CreateListenerRequest.builder()
                    .loadBalancerArn(model.getLbUuid())
                    .protocol(model.getProtocol())
                    .port(model.getPort());

            List actions = new ArrayList();
            if (model.getDefaultActions() != null && !model.getDefaultActions().isEmpty()) {
                for (Object object :
                        model.getDefaultActions()) {
                    Action action = JSONObject.parseObject(object.toString(), Action.serializableBuilderClass()).build();
                    actions.add(action);
                }
            }
            builder.defaultActions(actions);

            CreateListenerRequest request = builder.build();
            CreateListenerResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).createListener(request);
            return new BocloudResult(true, convertor.convertModel(response.listeners().get(0)), "create listener success!");
        } catch (Exception e) {
            logger.error("create listener error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String listenerArn) {
        try {
            DeleteListenerRequest request = DeleteListenerRequest.builder().listenerArn(listenerArn).build();
            ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).deleteListener(request);
            return new BocloudResult(true, "delete listener success!");
        } catch (Exception e) {
            logger.error("delete listener error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
