package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.NatGatewayConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewayModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsNatGatewayProvider extends AwsProvider implements NatGatewayProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsNatGatewayProvider.class);
    private final BeanConvertor<NatGatewayModel, NatGateway> convertor = new NatGatewayConvertor();

    public AwsNatGatewayProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult create(NatGatewayModel model) {
        try {
            CreateNatGatewayRequest request = CreateNatGatewayRequest.builder().allocationId(model.getAllocationUuid()).subnetId(model.getSubnetUuid()).build();
            CreateNatGatewayResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createNatGateway(request);
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(response.natGateway().natGatewayId())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            NatGatewayModel gatewayModel = (NatGatewayModel) detail(response.natGateway().natGatewayId()).getData();
            return new BocloudResult(true, gatewayModel, "create NatGateway success!");
        } catch (Exception e) {
            logger.error("create NatGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String natGatewayId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            DeleteNatGatewayRequest request = DeleteNatGatewayRequest.builder().natGatewayId(natGatewayId).build();
            DeleteNatGatewayResponse response = client.deleteNatGateway(request);
            return new BocloudResult(true, "remove NatGateway success!");
        } catch (Exception e) {
            logger.error("remove NatGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String natGatewayId) {
        try {
            Ec2Client client = (Ec2Client) this.getClient(AwsService.EC2);
            DescribeNatGatewaysRequest request = DescribeNatGatewaysRequest.builder().natGatewayIds(natGatewayId).build();
            DescribeNatGatewaysResponse response = client.describeNatGateways(request);
            NatGatewayModel gatewayModel = convertor.convertModel(response.natGateways().get(0));
            return new BocloudResult(true, gatewayModel, "detail NatGateway success!");
        } catch (Exception e) {
            logger.error("detail NatGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeNatGatewaysResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeNatGateways();
            List<NatGatewayModel> models = convertor.convertList(response.natGateways());
            return new BocloudResult(true, models, "list addresses success!");
        } catch (Exception e) {
            logger.error("list NatGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(NatGatewayModel model) {
        try {
            Tag tag = Tag.builder()
                    .key("Name")
                    .value(model.getName())
                    .build();
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(model.getUuid())
                    .tags(tag)
                    .build();
            ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            NatGatewayModel gatewayModel = (NatGatewayModel) detail(model.getUuid()).getData();
            return new BocloudResult(true, gatewayModel, "update NatGateway success!");
        } catch (Exception e) {
            logger.error("update NatGateway error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
