package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

@Provider("bocloud.cmp.aws")
public class AwsRouteTableAssociationProvider extends AwsProvider implements RouteTableAssociationProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsRouteTableAssociationProvider.class);

    public AwsRouteTableAssociationProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult replaceAssociation(String routeTableId, String associationId) {
        try {
            ReplaceRouteTableAssociationRequest request = ReplaceRouteTableAssociationRequest.builder().routeTableId(routeTableId).associationId(associationId).build();
            ReplaceRouteTableAssociationResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).replaceRouteTableAssociation(request);
            return new BocloudResult(true, response.newAssociationId(), "set main route table success!");
        } catch (Exception e) {
            logger.error("set main route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult associateRouteTable(String routeTableId, String gatewayId, String subnetId) {
        try {
            AssociateRouteTableRequest.Builder builder = AssociateRouteTableRequest.builder().routeTableId(routeTableId);
            if (StringUtils.isNotEmpty(gatewayId)) {
                builder.gatewayId(gatewayId);
            } else {
                builder.subnetId(subnetId);
            }
            AssociateRouteTableRequest request = builder.build();
            AssociateRouteTableResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).associateRouteTable(request);
            return new BocloudResult(true, response.associationId(), "Associate route table success!");
        } catch (Exception e) {
            logger.error("Associate route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult disassociateRouteTable(String associationId) {
        try {
            DisassociateRouteTableRequest request = DisassociateRouteTableRequest.builder().associationId(associationId).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).disassociateRouteTable(request);
            return new BocloudResult(true, "Associate route table success!");
        } catch (Exception e) {
            logger.error("Associate route table error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
