package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsSecuGroupRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/11/4 10:16
 */
@Provider("bocloud.cmp.aws")
public class AwsSecuGroupRuleProvider extends AwsProvider implements SecurityGroupRuleProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsSecuGroupRuleProvider.class);

    public AwsSecuGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        try {
            IpPermission ipPermission = this.getIpPermission(model);
            if ("ingress".equals(model.getDirection())) {
                AuthorizeSecurityGroupIngressRequest request = AuthorizeSecurityGroupIngressRequest.builder()
                        .groupId(model.getGroupUuid())
                        .ipPermissions(ipPermission)
                        .build();

                ((Ec2Client) this.getClient(AwsService.EC2)).authorizeSecurityGroupIngress(request);
            } else {
                AuthorizeSecurityGroupEgressRequest request = AuthorizeSecurityGroupEgressRequest.builder()
                        .groupId(model.getGroupUuid())
                        .ipPermissions(ipPermission)
                        .build();

                ((Ec2Client) this.getClient(AwsService.EC2)).authorizeSecurityGroupEgress(request);
            }

            return new BocloudResult(true, model, "create securityGroupRule success!");
        } catch (Exception e) {
            logger.error("create securityGroupRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String ruleId) {
        return null;
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel model) {
        try {
            IpPermission ipPermission = this.getIpPermission(model);
            if ("ingress".equals(model.getDirection())) {
                RevokeSecurityGroupIngressRequest request = RevokeSecurityGroupIngressRequest.builder()
                        .groupId(model.getGroupUuid())
                        .ipPermissions(ipPermission)
                        .build();
                ((Ec2Client) this.getClient(AwsService.EC2)).revokeSecurityGroupIngress(request);
            } else {
                RevokeSecurityGroupEgressRequest request = RevokeSecurityGroupEgressRequest.builder()
                        .groupId(model.getGroupUuid())
                        .ipPermissions(ipPermission)
                        .build();
                ((Ec2Client) this.getClient(AwsService.EC2)).revokeSecurityGroupEgress(request);
            }
            return new BocloudResult(true, "remove securityGroupRule success!");
        } catch (Exception e) {
            logger.error("remove securityGroupRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    private IpPermission getIpPermission(SecurityGroupRuleModel model) {
        IpPermission ipPermission = null;
        //protocol为*的不知道如何传入进行删除，暂时没有环境查看 先遗留
//        if(model.getProtocol().equalsIgnoreCase("*")){
//            model.setProtocol("-1");
//        }
        switch (model.getProps()) {
            case "IPV4":
                IpRange ipRange = IpRange.builder()
                        .description(model.getRemark())
                        .cidrIp(model.getSourcePortRange())
                        .build();
                ipPermission = IpPermission.builder()
                        .ipRanges(ipRange)
                        .ipProtocol(model.getProtocol())
                        .fromPort(model.getPortMin())// 起始端口
                        .toPort(model.getPortMax())// 结束端口
                        .build();
                break;
            case "IPV6":
                Ipv6Range ipv6Range = Ipv6Range.builder()
                        .description(model.getRemark())
                        .cidrIpv6(model.getSourcePortRange())
                        .build();
                ipPermission = IpPermission.builder()
                        .ipv6Ranges(ipv6Range)
                        .ipProtocol(model.getProtocol())
                        .fromPort(model.getPortMin())// 起始端口
                        .toPort(model.getPortMax())// 结束端口
                        .build();
                break;
            default:
                break;
        }
        return ipPermission;
    }

}
