package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.awsconvertor.TagConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.ArrayList;
import java.util.List;

@Provider("bocloud.cmp.aws")
public class AwsTagProvider extends AwsProvider implements TagProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsTagProvider.class);
    private final BeanConvertor<TagModel, TagDescription> convertor = new TagConvertor();

    public AwsTagProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult create(List<TagModel> models) {
        try {
            List tags = new ArrayList();
            for (TagModel model :
                    models) {
                tags.add(Tag.builder()
                        .key(model.getKey())
                        .value(model.getValue())
                        .build());
            }
            CreateTagsRequest tagRequest = CreateTagsRequest.builder()
                    .resources(models.get(0).getResourceId())
                    .tags(tags)
                    .build();
            CreateTagsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createTags(tagRequest);
            models = (List<TagModel>) list("resource-id", models.get(0).getResourceId()).getData();
            return new BocloudResult(true, models, "create tag success!");
        } catch (Exception e) {
            logger.error("create tag error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(List<TagModel> models) {
        try {
            List tags = new ArrayList();
            for (TagModel model :
                    models) {
                tags.add(Tag.builder()
                        .key(model.getKey())
                        .build());
            }
            DeleteTagsRequest tagRequest = DeleteTagsRequest.builder()
                    .resources(models.get(0).getResourceId())
                    .tags(tags)
                    .build();
            DeleteTagsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).deleteTags(tagRequest);
            return new BocloudResult(true, "delete tag success!");
        } catch (Exception e) {
            logger.error("delete tag error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeTagsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeTags();
            List<TagModel> models = convertor.convertList(response.tags());
            JSONObject.toJSONString(models);

            return new BocloudResult(true, models, "list tag success!");
        } catch (Exception e) {
            logger.error("list tag error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public BocloudResult list(String key, String value) {
        try {
            Filter filter = Filter.builder().name(key).values(value).build();
            DescribeTagsRequest request = DescribeTagsRequest.builder().filters(filter).build();
            DescribeTagsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeTags(request);
            List<TagModel> models = convertor.convertList(response.tags());
            return new BocloudResult(true, models, "list tag success!");
        } catch (Exception e) {
            logger.error("list tag error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
