package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.awsconvertor.TargetConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.LbTargetGroupModel;
import com.bocloud.cmp.model.LbTargetModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.compute.AwsServerProvider;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.services.elasticloadbalancingv2.ElasticLoadBalancingV2Client;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.DescribeTargetHealthRequest;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.DescribeTargetHealthResponse;
import software.amazon.awssdk.services.elasticloadbalancingv2.model.TargetDescription;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.aws")
public class AwsTargetProvider extends AwsProvider implements TargetProvider {

    private static final Logger logger = LoggerFactory.getLogger(AwsTargetProvider.class);
    private final TargetConvertor convertor = new TargetConvertor();
    private final Butler butler;
    private TargetGroupProvider targetGroupProvider = null;
    private ServerProvider serverProvider = null;


    public AwsTargetProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list(LbTargetGroupModel groupModel, Map<String, String> serverZoneMap) {
        try {
            DescribeTargetHealthRequest request = DescribeTargetHealthRequest.builder().targetGroupArn(groupModel.getUuid()).build();
            DescribeTargetHealthResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeTargetHealth(request);
            List<LbTargetModel> models = convertor.convertList(response.targetHealthDescriptions(), groupModel, serverZoneMap);
            return new BocloudResult(true, models, "list targets success!");
        } catch (Exception e) {
            logger.error("list target error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String targetGroupArn, String id, Integer port) {
        if (targetGroupProvider == null) {
            targetGroupProvider = new AwsTargetGroupProvider(butler);
        }
        if (serverProvider == null) {
            serverProvider = new AwsServerProvider(butler);
        }
        try {
            BocloudResult result = targetGroupProvider.detail(targetGroupArn);
            if (result.isFailed()) {
                return result;
            }
            LbTargetGroupModel targetGroupModel = (LbTargetGroupModel) result.getData();
            Map<String, String> serverZoneMap = new HashMap<>();
            if ("instance".equals(targetGroupModel.getType())) {
                result = serverProvider.detail(id);
                if (result.isFailed()) {
                    return result;
                }
                ServerModel serverModel = (ServerModel) result.getData();
                serverZoneMap.put(serverModel.getInstanceId(), serverModel.getZone());
            }

            DescribeTargetHealthRequest request = DescribeTargetHealthRequest.builder().targetGroupArn(targetGroupArn).targets(TargetDescription.builder().id(id).port(port).build()).build();
            DescribeTargetHealthResponse response = ((ElasticLoadBalancingV2Client) this.getClient(AwsService.ELB)).describeTargetHealth(request);
            LbTargetModel model = convertor.convertModel(response.targetHealthDescriptions().get(0), targetGroupModel, serverZoneMap);
            return new BocloudResult(true, model, "query target success!");
        } catch (Exception e) {
            logger.error("query target error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
