package com.bocloud.cmp.provider.security;

import com.bocloud.cmp.awsconvertor.KeypairConvertor;
import com.bocloud.cmp.common.enums.AwsService;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.AwsProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import software.amazon.awssdk.core.SdkBytes;
import software.amazon.awssdk.services.ec2.Ec2Client;
import software.amazon.awssdk.services.ec2.model.*;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AwsKeypairProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/15 15:53
 */
@Provider("bocloud.cmp.aws")
public class AwsKeypairProvider extends AwsProvider implements KeypairProvider {
    private static final Logger logger = LoggerFactory.getLogger(AwsKeypairProvider.class);

    private final BeanConvertor<KeypairModel, KeyPairInfo> convertor = new KeypairConvertor();

    public AwsKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeKeyPairsResponse describeKeyPairsResponse = ((Ec2Client) this.getClient(AwsService.EC2)).describeKeyPairs();
            List<KeypairModel> keypairModels = convertor.convertList(describeKeyPairsResponse.keyPairs());
            return new BocloudResult(true, keypairModels, "list keypair success!");
        } catch (Exception e) {
            logger.error("list keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        try {
            DescribeKeyPairsRequest request = DescribeKeyPairsRequest.builder()
                    .keyNames(keypairName)
                    .build();
            DescribeKeyPairsResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).describeKeyPairs(request);
            KeypairModel keypairModel = convertor.convertModel(response.keyPairs().get(0));
            keypairModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, keypairModel, "detail keypair success!");
        } catch (Exception e) {
            logger.error("detail keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        try {
            KeypairModel model = new KeypairModel();
            if (!StringUtils.isEmpty(publicKey)) {
                ImportKeyPairRequest pairRequest = ImportKeyPairRequest.builder()
                        .keyName(keypairName)
                        .publicKeyMaterial(SdkBytes.fromUtf8String(publicKey))
                        .build();
                ImportKeyPairResponse importKeyPairResponse = ((Ec2Client) this.getClient(AwsService.EC2)).importKeyPair(pairRequest);
                model.setName(importKeyPairResponse.keyName());
                model.setPublicKey(publicKey);
                model.setFingerprint(importKeyPairResponse.keyFingerprint());
                return new BocloudResult(true, model, "create keypair success!");
            } else {
                CreateKeyPairRequest request = CreateKeyPairRequest.builder()
                        .keyName(keypairName)
                        .build();
                CreateKeyPairResponse response = ((Ec2Client) this.getClient(AwsService.EC2)).createKeyPair(request);
                model.setPrivateKey(response.keyMaterial());
                model.setName(response.keyName());
                model.setFingerprint(response.keyFingerprint());
                return new BocloudResult(true, model, "create keypair success!");
            }
        } catch (Exception e) {
            logger.error("create keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            DeleteKeyPairRequest request = DeleteKeyPairRequest.builder().keyName(keypairName).build();
            ((Ec2Client) this.getClient(AwsService.EC2)).deleteKeyPair(request);
            return new BocloudResult(true, "delete keypair success!");
        } catch (Exception e) {
            logger.error("delete keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        return null;
    }
}
