package com.bocloud.cmp.azureconvertor.account;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.azure.StorageAccountFirewallType;
import com.bocloud.cmp.model.azure.account.StorageAccountFirewallModel;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import com.microsoft.azure.management.storage.StorageAccount;
import com.microsoft.azure.management.storage.StorageAccountKey;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: StorageAccountConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/7/24 13:48
 */
public class StorageAccountConvertor implements BeanConvertor<StorageAccountModel, StorageAccount> {

    @Override
    public StorageAccountModel convertModel(StorageAccount storageAccount) {
        StorageAccountModel model = new StorageAccountModel();
        model.setUuid(storageAccount.id());
        model.setName(storageAccount.name());
        model.setStatus("AVAILABLE");
        model.setRegionId(storageAccount.regionName());
        model.setResourceGroupName(storageAccount.resourceGroupName());
        model.setSku(storageAccount.skuType().name().toString());
        Optional.ofNullable(storageAccount.kind()).map(kind -> kind.toString()).ifPresent(model::setKind);
        Optional.ofNullable(storageAccount.accessTier()).map(accessTier -> accessTier.toString()).ifPresent(model::setAccessTier);
        List<StorageAccountKey> keys = storageAccount.getKeys();
        if (!ListTool.isEmpty(keys)) {
            for (StorageAccountKey key : keys) {
                String keyName = key.keyName();
                if ("key1".equalsIgnoreCase(keyName)) {
                    model.setPrimaryKey(key.value());
                } else {
                    model.setSecondaryKey(key.value());
                }
            }
        }
        Optional.ofNullable(storageAccount.endPoints()).ifPresent(publicEndpoints -> {
            Optional.ofNullable(publicEndpoints.primary()).ifPresent(primary -> {
                model.setPrimaryEndPoint(primary.blob());
                model.setFileEndPointPrimary(primary.file());
                model.setQueueEndPointPrimary(primary.queue());
                model.setTableEndPointPrimary(primary.table());
            });
            Optional.ofNullable(publicEndpoints.secondary()).ifPresent(secondary -> {
                model.setSecondaryEndPoint(secondary.blob());
                model.setFileEndPointSecondary(secondary.file());
                model.setQueueEndPointSecondary(secondary.queue());
                model.setTableEndPointSecondary(secondary.table());
            });
        });

        //自定义域
        Optional.ofNullable(storageAccount.customDomain()).ifPresent(customDomain -> {
            model.setCustomDomain(customDomain.name());
        });

        List<String> ipAddresses = storageAccount.ipAddressesWithAccess();
        if (!CollectionUtils.isEmpty(ipAddresses)) {
            model.setIpAddresses(JSONObject.toJSONString(ipAddresses));
        }
        List<String> ipAddressRanges = storageAccount.ipAddressRangesWithAccess();
        if (!CollectionUtils.isEmpty(ipAddressRanges)) {
            model.setIpAddressRanges(JSONObject.toJSONString(ipAddressRanges));
        }
        model.setIsAccessAllNetworks(storageAccount.isAccessAllowedFromAllNetworks());
        model.setSubnetUuids(storageAccount.networkSubnetsWithAccess());

        List<StorageAccountFirewallModel> firewallModels = new ArrayList<>();
        if (!CollectionUtils.isEmpty(storageAccount.ipAddressesWithAccess())) {
            for (String ip : storageAccount.ipAddressesWithAccess()) {
                firewallModels.add(new StorageAccountFirewallModel(storageAccount.id(), ip, StorageAccountFirewallType.IP.name()));
            }
        }
        if (!CollectionUtils.isEmpty(storageAccount.ipAddressRangesWithAccess())) {
            for (String ip : storageAccount.ipAddressRangesWithAccess()) {
                firewallModels.add(new StorageAccountFirewallModel(storageAccount.id(), ip, StorageAccountFirewallType.IPRANGE.name()));
            }
        }
        model.setFirewallModels(firewallModels);
        model.setTags(null == storageAccount.tags() ? null : JSONObject.toJSONString(storageAccount.tags()));
        Optional.ofNullable(storageAccount.inner()).ifPresent(inner -> {
            Optional.ofNullable(inner.minimumTlsVersion()).map(minimumTlsVersion -> minimumTlsVersion.toString()).ifPresent(model::setMinimumTlsVersion);
        });
        return model;
    }
}