package com.bocloud.cmp.azureconvertor.bill;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.utils.DateTools;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: BillInternationalConvertor
 * @Description:消费转换器
 * @author: zhenpingLi/李振平
 * @date: 2020/12/17 16:56
 */
public class BillConsumptionConvertor implements BeanJsonConvertor<PublicCloudBillModel> {

    @Override
    public PublicCloudBillModel convertModel(JSONObject object) {
        return this.consume(object);
    }

    public PublicCloudBillModel consume(JSONObject object) {
        PublicCloudBillModel model = new PublicCloudBillModel();
        //计费周期UUID+拼接UUID
        model.setOrderSn(object.getString("id").toLowerCase());
        Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
            //订阅ID
            Optional.ofNullable(properties.getString("subscriptionId")).ifPresent(model::setSubscriptionId);
            //订阅名称
            Optional.ofNullable(properties.getString("subscriptionName")).ifPresent(model::setSubscriptionName);
            //资源组
            Optional.ofNullable(properties.getString("resourceGroup")).ifPresent(model::setResourceGroupName);
            //已消费服务或购买的产品名称。 不适用于应用商店。
            Optional.ofNullable(properties.getString("product")).ifPresent(model::setProductName);
            //资源UUID
            Optional.ofNullable(properties.getString("resourceId")).ifPresent(model::setResourceId);
            //资源名称
            Optional.ofNullable(properties.getString("resourceName")).ifPresent(model::setResourceName);
            //资源类型：云主机或者硬盘之类的
            Optional.ofNullable(properties.getString("resourceId")).ifPresent(resourceId -> {
                String productLine;
                String[] split = resourceId.split("/");
                if (split.length > 7) {
                    productLine = split[6] + "/" + split[7];
                } else {
                    productLine = resourceId;
                }
                model.setProductLine(productLine);
            });
            //资源位置。
            Optional.ofNullable(properties.getString("resourceLocation")).ifPresent(model::setRegionId);
            //为使用付费的有效价格。
            Optional.ofNullable(properties.getString("cost")).map(cost -> new BigDecimal(cost)).ifPresent(model::setMoney);
            //单价是适用于您的价格。 （您的 EA 或其他合同价格）。
            Optional.ofNullable(properties.getString("unitPrice")).map(unitPrice -> new BigDecimal(unitPrice)).ifPresent(model::setUnitPrice);
            //使用数量
            Optional.ofNullable(properties.getString("quantity")).map(quantity -> new BigDecimal(quantity)).ifPresent(model::setQuantity);
            //计费币种。
            Optional.ofNullable(properties.getString("billingCurrency")).ifPresent(model::setBillingCurrency);
            //计费期间开始日期。
            Optional.ofNullable(properties.getString("billingPeriodStartDate")).map(billingPeriodStartDate -> DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodStartDate, "Z")).ifPresent(model::setFeeBeginTime);
            //计费期间结束日期。
            Optional.ofNullable(properties.getString("billingPeriodEndDate")).map(billingPeriodEndDate -> DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodEndDate, "Z")).ifPresent(model::setFeeEndTime);
            //使用记录的日期。
            Optional.ofNullable(properties.getString("date")).map(date -> DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", date, "Z")).ifPresent(model::setUseResourceData);
            //资源所属类型
            Optional.ofNullable(properties.getString("consumedService")).ifPresent(model::setConsumedService);
            //资源所属月份
            Optional.ofNullable(properties.getString("billingPeriodStartDate"))
                    .map(billingPeriodStartDate -> DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", billingPeriodStartDate, "Z"))
                    .map(this::getTime)
                    .ifPresent(model::setMonth);
            //获取meterId
            model.setMeterId(properties.getString("meterId"));

            //--废弃字段
            //税前成本金额。
            Optional.ofNullable(properties.getString("cost")).map(cost -> new BigDecimal(cost)).ifPresent(model::setPretaxAmount);
            //消耗的总价
            model.setPretaxGrossAmount(model.getQuantity().multiply(model.getUnitPrice()));

            //meterDetails
            Optional.ofNullable(properties.getJSONObject("meterDetails")).ifPresent(meterDetails -> {
                PublicCloudBillModel.MeterDetailsModel detailsModel = model.new MeterDetailsModel();
                detailsModel.setMeterName(meterDetails.getString("meterName"));
                detailsModel.setMeterCategory(meterDetails.getString("meterCategory"));
                detailsModel.setMeterSubCategory(meterDetails.getString("meterSubCategory"));
                detailsModel.setUnitOfMeasure(meterDetails.getString("unitOfMeasure"));
                model.setMeterDetailsModel(detailsModel);
            });
        });
        return model;
    }

    private String getTime(Date date) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM");
        String time = dateFormat.format(date);
        return time;
    }


}