package com.bocloud.cmp.azureconvertor.blob;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.blob.BlobModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.storage.blob.BlobProperties;
import com.microsoft.azure.storage.blob.CloudBlob;
import com.microsoft.azure.storage.blob.CloudBlockBlob;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.stream.Collectors;

/**
 * blob对象存储数据转换器
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
@Slf4j
public class BlobConvertor implements BeanConvertor<BlobModel, CloudBlob> {

    @Override
    public List<BlobModel> convertList(List<? extends CloudBlob> blockBlobs) {
        return blockBlobs.stream().map(b -> this.convertModel(b)).collect(Collectors.toList());
    }


    @Override
    public BlobModel convertModel(CloudBlob blockBlob) {
        try {
            BlobModel blobModel = new BlobModel();
            blobModel.setName(blockBlob.getName());
            blobModel.setCopyState(JSONObject.toJSONString(blockBlob.getCopyState()));
            blobModel.setMetadata(null == blockBlob.getMetadata() ? null : JSONObject.toJSONString(blockBlob.getMetadata()));
            blobModel.setSnapshotID(blockBlob.getSnapshotID());
            if (null != blockBlob.getSnapshotQualifiedStorageUri()) {
                blobModel.setSnapshotQualifiedStorageUri(JSONObject.toJSONString(blockBlob.getSnapshotQualifiedStorageUri()));
            }
            blobModel.setUrl(null == blockBlob.getUri() ? null : blockBlob.getUri().toString());
            blobModel.setStreamMinimumReadSizeInBytes(blockBlob.getStreamMinimumReadSizeInBytes());
            blobModel.setStreamWriteSizeInBytes(blockBlob.getStreamWriteSizeInBytes());
            blobModel.setSnapshot(blockBlob.isSnapshot());
            if (blockBlob instanceof CloudBlockBlob) {
                CloudBlockBlob cloudBlockBlob = (CloudBlockBlob) blockBlob;
                blobModel.setStreamWriteSizeModified(cloudBlockBlob.isStreamWriteSizeModified());
            }
            BlobProperties properties = blockBlob.getProperties();
            if (null != properties) {
                blobModel.setBlobType(null == properties.getBlobType() ? null : properties.getBlobType().name());
                blobModel.setContentType(properties.getContentType());
                blobModel.setAppendBlobCommittedBlockCount(properties.getAppendBlobCommittedBlockCount());
                blobModel.setCacheControl(properties.getCacheControl());
                blobModel.setContentDisposition(properties.getContentDisposition());
                blobModel.setContentEncoding(properties.getContentEncoding());
                blobModel.setContentLanguage(properties.getContentLanguage());
                blobModel.setContentMD5(properties.getContentMD5());
                blobModel.setCreatedTime(properties.getCreatedTime());
                blobModel.setEtag(properties.getEtag());
                blobModel.setLastModified(properties.getLastModified());
                blobModel.setLeaseStatus(null == properties.getLeaseStatus() ? null : properties.getLeaseStatus().name());
                blobModel.setLeaseState(null == properties.getLeaseState() ? null : properties.getLeaseState().name());
                blobModel.setLeaseDuration(null == properties.getLeaseDuration() ? null : properties.getLeaseDuration().name());
                blobModel.setLength(properties.getLength());
                blobModel.setPageBlobSequenceNumber(properties.getPageBlobSequenceNumber());
                blobModel.setServerEncrypted(properties.isServerEncrypted());
                blobModel.setIncrementalCopy(properties.isIncrementalCopy());
                blobModel.setPremiumPageBlobTier(null == properties.getPremiumPageBlobTier() ? null : properties.getPremiumPageBlobTier().name());
                blobModel.setStandardBlobTier(null == properties.getStandardBlobTier() ? null : properties.getStandardBlobTier().name());
                blobModel.setBlobTierInferredTier(properties.isBlobTierInferred());
                blobModel.setTierChangeTime(properties.getTierChangeTime());
                blobModel.setRehydrationStatus(null == properties.getRehydrationStatus() ? null : properties.getRehydrationStatus().name());
                blobModel.setDeletedTime(properties.getDeletedTime());
                blobModel.setRemainingRetentionDays(properties.getRemainingRetentionDays());
            }
            return blobModel;
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return null;
        }
    }
}
