package com.bocloud.cmp.azureconvertor.blob;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.blob.BlobModel;
import com.bocloud.cmp.model.azure.blob.ContainerModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.storage.blob.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * container数据转换器
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
public class ContainerConvertor implements BeanConvertor<ContainerModel, CloudBlobContainer> {

    private static final BeanConvertor<BlobModel, CloudBlob> CONVERTOR = new BlobConvertor();

    @Override
    public List<ContainerModel> convertList(List<? extends CloudBlobContainer> containers) {
        return containers.stream().map(c -> this.convertModel(c)).collect(Collectors.toList());
    }

    @Override
    public ContainerModel convertModel(CloudBlobContainer container) {
        ContainerModel containerModel = new ContainerModel();
        containerModel.setName(container.getName());
        containerModel.setMetadata(null == container.getMetadata() ? null : JSONObject.toJSONString(container.getMetadata()));
        containerModel.setUrl(null == container.getUri() ? null : container.getUri().toString());
        BlobContainerProperties properties = container.getProperties();
        if (null != properties) {
            containerModel.setLeaseDuration(null == properties.getLeaseDuration() ? null : properties.getLeaseDuration().name());
            containerModel.setLeaseState(null == properties.getLeaseState() ? null : properties.getLeaseState().name());
            containerModel.setLeaseStatus(null == properties.getLeaseStatus() ? null : properties.getLeaseStatus().name());
            containerModel.setHasLegalHold(properties.hasLegalHold());
            containerModel.setHasImmutabilityPolicy(properties.hasImmutabilityPolicy());
            containerModel.setLastModified(properties.getLastModified());
            containerModel.setEtag(properties.getEtag());
            containerModel.setPublicAccess(null == properties.getPublicAccess() ? null : properties.getPublicAccess().name());
        }
        //查询blob存储
        Iterable<ListBlobItem> listBlobItems = container.listBlobs(null, true, EnumSet.of(BlobListingDetails.SNAPSHOTS, BlobListingDetails.METADATA), null, null);
        if (null == listBlobItems) {
            listBlobItems = new ArrayList<>();
        }
        Map<String, CloudBlob> blockBlobMap = new HashMap<>();
        Map<String, CloudBlob> snapshotBlobMap = new HashMap<>();
        try{
            listBlobItems.forEach(b -> {
                if (b instanceof CloudBlockBlob) {
                    CloudBlockBlob blockBlob = (CloudBlockBlob) b;
                    if (blockBlob.isSnapshot()) {
                        snapshotBlobMap.put(blockBlob.getName() + blockBlob.getSnapshotID(), blockBlob);
                    } else {
                        blockBlobMap.put(blockBlob.getName(), blockBlob);
                    }
                } else if (b instanceof CloudAppendBlob) {
                    CloudAppendBlob appendBlob = (CloudAppendBlob) b;
                    if (appendBlob.isSnapshot()) {
                        snapshotBlobMap.put(appendBlob.getName() + appendBlob.getSnapshotID(), appendBlob);
                    } else {
                        blockBlobMap.put(appendBlob.getName(), appendBlob);
                    }
                } else if (b instanceof CloudPageBlob) {
                    CloudPageBlob pageBlob = (CloudPageBlob) b;
                    if (pageBlob.isSnapshot()) {
                        snapshotBlobMap.put(pageBlob.getName() + pageBlob.getSnapshotID(), pageBlob);
                    } else {
                        blockBlobMap.put(pageBlob.getName(), pageBlob);
                    }
                }
            });
        }catch (Exception e){
        }
        List<CloudBlob> blobs = new ArrayList<>(blockBlobMap.values());
        List<CloudBlob> snapshotBlobs = new ArrayList<>(snapshotBlobMap.values());
        containerModel.setBlobs(CONVERTOR.convertList(blobs));
        containerModel.setSnapshots(CONVERTOR.convertList(snapshotBlobs));
        return containerModel;
    }
}
