package com.bocloud.cmp.azureconvertor.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.compute.ComputeSku;
import com.microsoft.azure.management.compute.ResourceSkuCapabilities;
import com.microsoft.azure.management.resources.fluentcore.arm.AvailabilityZoneId;
import com.microsoft.azure.management.resources.fluentcore.arm.Region;
import org.apache.commons.collections.MapUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FlavorRegionConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/20 15:31
 */
public class FlavorConvertor implements BeanConvertor<FlavorModel, ComputeSku> {

    @Override
    public List<FlavorModel> convertList(List<? extends ComputeSku> skus) {
        List<FlavorModel> flavorModels = new ArrayList<>();
        for (ComputeSku computeSku : skus) {
            flavorModels.add(this.convertModel(computeSku));
        }
        return flavorModels;
    }

    @Override
    public FlavorModel convertModel(ComputeSku computeSku) {
        FlavorModel flavorModel = new FlavorModel();
        flavorModel.setName(computeSku.name().toString());
        flavorModel.setFlavorUuid(computeSku.name().toString());
//        flavorModel.setFamily(computeSku.inner().family());

        //sku存在限制
        if (!CollectionUtils.isEmpty(computeSku.restrictions())) {
            flavorModel.setAvailable(false);
        } else {
            flavorModel.setAvailable(true);
        }
        List<ResourceSkuCapabilities> capabilities = computeSku.capabilities();
        if (!CollectionUtils.isEmpty(capabilities)) {
            capabilities.stream().forEach(capabilitie -> {
                switch (capabilitie.name()) {
                    case "vCPUs":
                        flavorModel.setCpu(Integer.parseInt(capabilitie.value()));
                        break;
                    case "MemoryGB":
                        flavorModel.setMemory((BigDecimal.valueOf(Double.parseDouble(capabilitie.value())).setScale(2, BigDecimal.ROUND_HALF_UP)));
                        break;
                    case "MaxResourceVolumeMB":
                        flavorModel.setDisk(Integer.parseInt(capabilitie.value()) / 1024);
                        break;
                    default:
                        break;
                }
            });
        }

        //获取region名称
        if (!CollectionUtils.isEmpty(computeSku.regions())) {
            flavorModel.setRegionId(computeSku.regions().get(0).name());
        }

        //获取规格支持的可用性区域
        Map<Region, Set<AvailabilityZoneId>> zones = computeSku.zones();
        if (!MapUtils.isEmpty(zones)) {
            List<String> zoneIds = new ArrayList<>();
            Collection<Set<AvailabilityZoneId>> sets = computeSku.zones().values();
            if (!CollectionUtils.isEmpty(sets)) {
                for (Set<AvailabilityZoneId> availabilityZoneIds : sets) {
                    for (AvailabilityZoneId availabilityZoneId : availabilityZoneIds) {
                        zoneIds.add(availabilityZoneId.toString());
                    }
                }
            }
            flavorModel.setAvailabilityZoneIds(JSON.toJSONString(zoneIds));
        }

        return flavorModel;
    }

}
