package com.bocloud.cmp.azureconvertor.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.microsoft.azure.management.compute.Snapshot;

import java.util.Optional;

/**
 * 快照数据转换器
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/8/26
 */
public class SnapshotConvertor implements BeanConvertor<SnapshotModel, Snapshot> {

    @Override
    public SnapshotModel convertModel(Snapshot snapshot) {
        SnapshotModel model = new SnapshotModel();
        model.setSnapshotUuid(snapshot.id());
        model.setStatus("AVAILABLE");
        model.setName(snapshot.name());
        //硬盘没有osType，创建出来的快照也没有osType
        model.setProps(Optional.ofNullable(snapshot.osType()).map(operatingSystemTypes -> operatingSystemTypes.toString()).orElse(null));
        model.setSize(snapshot.sizeInGB());
        model.setRegionId(snapshot.regionName());
        model.setResourceGroupName(snapshot.resourceGroupName());
        model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", snapshot.inner().timeCreated().toString(), "Z"));
        model.setVolumeUuid(snapshot.source().sourceId());
        model.setTags(null == snapshot.tags() ? null : JSONObject.toJSONString(snapshot.tags()));
        return model;
    }
}
