package com.bocloud.cmp.azureconvertor.db;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.model.azure.db.ServiceLevelObjectiveModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;

import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDatabaseConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/18 13:58
 */
public class CloudDatabaseServerConvertor implements BeanJsonConvertor<AzureDatabaseServerModel> {

    @Override
    public AzureDatabaseServerModel convertModelParams(JSONObject object, String... params) {
        String Dbtype = params[0];
        AzureDatabaseServerModel model = new AzureDatabaseServerModel();
        model.setDatabaseServerUuid(object.getString("id"));
        model.setName(object.getString("name"));
        model.setRegionId(object.getString("location"));
        model.setResourceGroupName(object.getString("id").split("/")[4]);
        model.setType(object.getString("type"));

        Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
            Optional.ofNullable(properties.getString("fullyQualifiedDomainName")).ifPresent(model::setFullyQualifiedDomain);
            Optional.ofNullable(properties.getString("administratorLogin")).ifPresent(model::setAdministratorName);
            Optional.ofNullable(properties.getString("publicNetworkAccess")).ifPresent(model::setPublicNetworkAccess);
            Optional.ofNullable(properties.getString("sslEnforcement")).ifPresent(model::setSslEnforcement);
            Optional.ofNullable(properties.getString("minimalTlsVersion")).ifPresent(model::setMinimalTlsVersion);
            Optional.ofNullable(properties.getString("version")).ifPresent(model::setVersion);
            Optional.ofNullable(properties.getString("userVisibleState")).ifPresent(model::setStatus);
            Optional.ofNullable(properties.getString("infrastructureEncryption")).ifPresent(model::setInfrastructureEncryption);
            Optional.ofNullable(properties.getJSONObject("storageProfile")).ifPresent(storageProfile -> {
                Optional.ofNullable(storageProfile.getInteger("storageMB")).ifPresent(model::setStorageMb);
                Optional.ofNullable(storageProfile.getInteger("backupRetentionDays")).ifPresent(model::setBackupRetentionDays);
                Optional.ofNullable(storageProfile.getString("geoRedundantBackup")).ifPresent(model::setGeoRedundantBackup);
                Optional.ofNullable(storageProfile.getString("storageAutogrow")).ifPresent(model::setStorageAutogrow);
            });
        });

        ServiceLevelObjectiveModel sloModel = new ServiceLevelObjectiveModel();
        Optional.ofNullable(object.getJSONObject("sku")).ifPresent(sku -> {
            Optional.ofNullable(sku.getString("name")).ifPresent(sloModel::setUuid);
            Optional.ofNullable(sku.getString("tier")).ifPresent(sloModel::setEdition);
            Optional.ofNullable(sku.getString("family")).ifPresent(sloModel::setHardwareGeneration);
            Optional.ofNullable(sku.getInteger("capacity")).ifPresent(sloModel::setvCore);
            sloModel.setRegionId(object.getString("location"));
            sloModel.setDbType(Dbtype);
        });
        model.setSloModel(sloModel);
        return model;
    }
}