package com.bocloud.cmp.azureconvertor.monitor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.monitor.MetricAlertModel;
import com.bocloud.cmp.model.azure.monitor.MetricCriteriaModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: MetricAlertConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/1/11 16:12
 */
public class MetricAlertConvertor implements BeanJsonConvertor<MetricAlertModel> {

    @Override
    public MetricAlertModel convertModel(JSONObject object) {
        MetricAlertModel model = new MetricAlertModel();
        model.setName(object.getString("name"));
        model.setUuid(object.getString("id"));
        model.setResourceGroupName(object.getString("id").split("/")[4]);
        model.setType(object.getString("type"));
        model.setRegionId(object.getString("location"));
        Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
            model.setDescription(properties.getString("description"));
            model.setSeverity(properties.getInteger("severity"));
            model.setEnabled(properties.getBoolean("enabled"));
            model.setEvaluationFrequency(properties.getString("evaluationFrequency"));
            model.setWindowSize(properties.getString("windowSize"));
            model.setAutoMitigate(properties.getBoolean("autoMitigate"));

            String targetResourceType = properties.getString("targetResourceType");
            if (targetResourceType.equalsIgnoreCase("microsoft.cache/redis")) {
                model.setTargetResourceType("Microsoft.Cache/Redis");
            } else {
                model.setTargetResourceType(targetResourceType);
            }
            List<String> scopes = properties.getJSONArray("scopes").toJavaList(String.class);
            model.setResourceUuids(scopes);
            Optional.ofNullable(properties.getJSONObject("criteria")).ifPresent(criteria ->
                    Optional.ofNullable(criteria.getJSONArray("allOf")).map(allof -> allof.stream().map(JSONObject::toJSONString).map(JSONObject::parseObject).map(af -> {
                        MetricCriteriaModel metricCriteriaModel = new MetricCriteriaModel();
                        metricCriteriaModel.setName(af.getString("name"));
                        metricCriteriaModel.setMetricName(af.getString("metricName"));
                        metricCriteriaModel.setMetricNamespace(af.getString("metricNamespace"));
                        metricCriteriaModel.setThreshold(af.getInteger("threshold"));
                        metricCriteriaModel.setOperator(af.getString("operator"));
                        metricCriteriaModel.setTimeAggregation(af.getString("timeAggregation"));
                        metricCriteriaModel.setCriterionType(af.getString("criterionType"));
                        metricCriteriaModel.setMetricAlertUuid(object.getString("id"));
                        return metricCriteriaModel;
                    }).collect(Collectors.toList())).ifPresent(model::setMetricCriteriaModels)
            );
        });
        return model;
    }
}