package com.bocloud.cmp.azureconvertor.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.azure.network.NicIpConfigurationModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.NetworkInterface;
import com.microsoft.azure.management.network.NicIPConfiguration;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: NetworkInterfaceConvertor
 * @Description:网络接口转换器
 * @author: zhenpingLi/李振平
 * @date: 2019/9/27 14:37
 */
public class NetworkInterfaceConvertor implements BeanConvertor<NetworkInterfaceModel, NetworkInterface> {

    @Override
    public NetworkInterfaceModel convertModel(NetworkInterface networkInterface) {
        NetworkInterfaceModel interfaceModel = new NetworkInterfaceModel();
        interfaceModel.setName(networkInterface.name());
        interfaceModel.setResourceGroupName(networkInterface.resourceGroupName());
        interfaceModel.setNetworkInterfaceUuid(networkInterface.id());
        interfaceModel.setRegionId(networkInterface.regionName());
        //获取绑定的云主机
        interfaceModel.setVirtualMachineUuid(networkInterface.virtualMachineId());
        //是否为主网络接口
        if (!ObjectUtils.isEmpty(networkInterface.inner().primary())) {
            interfaceModel.setNicsPrimary(networkInterface.inner().primary());
        } else {
            interfaceModel.setNicsPrimary(false);
        }
        //获取绑定的安全组
        interfaceModel.setSecurityGroupUuid(networkInterface.networkSecurityGroupId());
        //获取vpc uuid
        Optional.ofNullable(networkInterface.primaryIPConfiguration()).map(ipConfiguration -> ipConfiguration.networkId())
                .ifPresent(interfaceModel::setNetworkUuid);
        //获取子网名称
        interfaceModel.setSubnetName(networkInterface.primaryIPConfiguration().subnetName());
        //获取子网uuid
        Optional.ofNullable(networkInterface.primaryIPConfiguration())
                .ifPresent(primaryIPConfiguration ->
                        Optional.ofNullable(primaryIPConfiguration.getNetwork()).ifPresent(network ->
                                Optional.ofNullable(network.subnets()).ifPresent(subnetMap ->
                                        Optional.ofNullable(subnetMap.values()).ifPresent(subnets ->
                                                Optional.ofNullable(subnets.stream().filter(subnet -> subnet.name().equalsIgnoreCase(networkInterface.primaryIPConfiguration().subnetName())).findFirst().get())
                                                        .map(subnet -> subnet.inner().id())
                                                        .ifPresent(interfaceModel::setSubnetUuid)
                                        )
                                )
                        )
                );
        //获取主要私有ip
        Optional.ofNullable(networkInterface.primaryIPConfiguration()).map(ipConfiguration -> ipConfiguration.privateIPAddress())
                .ifPresent(interfaceModel::setPrimaryPrivateIPAddress);
        //获取主要公共ip
        Optional.ofNullable(networkInterface.primaryIPConfiguration()).ifPresent(ipConfiguration -> {
                    Optional.ofNullable(ipConfiguration.getPublicIPAddress()).map(publicIPAddress -> publicIPAddress.ipAddress())
                            .ifPresent(interfaceModel::setPublicIPAddress);
                    Optional.ofNullable(ipConfiguration.getPublicIPAddress()).map(publicIPAddress -> publicIPAddress.id())
                            .ifPresent(interfaceModel::setPublicIpAddressUuid);
                }
        );
        //ip配置  ip配置中包含了上面的主要ip信息
        List<NicIpConfigurationModel> nicIpConfigurationModels = new ArrayList<>();
        for (NicIPConfiguration nicIPConfiguration : networkInterface.ipConfigurations().values()) {
            NicIpConfigurationModel ipConfigurationModel = new NicIpConfigurationModel();
            ipConfigurationModel.setUuid(nicIPConfiguration.inner().id());
            ipConfigurationModel.setName(nicIPConfiguration.name());
            ipConfigurationModel.setIsPrimary(nicIPConfiguration.isPrimary());
            ipConfigurationModel.setIpMethod(nicIPConfiguration.privateIPAllocationMethod().toString());
            ipConfigurationModel.setVpcUuid(nicIPConfiguration.networkId());
            ipConfigurationModel.setPublicIpAddressUuid(nicIPConfiguration.publicIPAddressId());
            ipConfigurationModel.setPrivateIpAddress(nicIPConfiguration.privateIPAddress());
            Optional.ofNullable(nicIPConfiguration.getPublicIPAddress()).map(publicIPAddress -> publicIPAddress.ipAddress()).ifPresent(ipConfigurationModel::setPublicIpAddress);
            nicIpConfigurationModels.add(ipConfigurationModel);
        }
        interfaceModel.setNicIpConfigurationModels(nicIpConfigurationModels);
        interfaceModel.setInterfaceType(networkInterface.type());
        if (!CollectionUtils.isEmpty(networkInterface.dnsServers())) {
            interfaceModel.setDnsServer(JSONObject.toJSONString(networkInterface.dnsServers()));
        }
        interfaceModel.setIpForwarding(networkInterface.isIPForwardingEnabled());
        interfaceModel.setTags(null == networkInterface.tags() ? null : JSONObject.toJSONString(networkInterface.tags()));

        return interfaceModel;
    }
}
