package com.bocloud.cmp.azureconvertor.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.PublicIPAddressModel;
import com.bocloud.cmp.model.azure.network.NicIpConfigurationModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.PublicIPAddress;

import java.util.Optional;

/**
 * @ClassName: PublicIPAddressConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 */
public class PublicIPAddressConvertor implements BeanConvertor<PublicIPAddressModel, PublicIPAddress> {

    @Override
    public PublicIPAddressModel convertModel(PublicIPAddress publicIPAddress) {
        PublicIPAddressModel model = new PublicIPAddressModel();
        model.setUuid(publicIPAddress.id());
        model.setName(publicIPAddress.name());
        model.setResourceGroupName(publicIPAddress.resourceGroupName());
        model.setRegionId(publicIPAddress.regionName());
        model.setIdleTimeoutInMinutes(publicIPAddress.idleTimeoutInMinutes());
        model.setIpAllocationMethod(publicIPAddress.ipAllocationMethod().toString());
        model.setSku(publicIPAddress.sku().sku().name().toString());
        model.setIpAddress(publicIPAddress.ipAddress());
        model.setVersion(publicIPAddress.version().toString());
        model.setFqdn(publicIPAddress.fqdn());
        model.setStatus("AVAILABLE");

        //这段代码用来创建云主机时候，云主机如果创建失败偶尔会自动生成网卡此时还没有入库，有不确定性，所以
        //需求是创建失败的云主机要先解绑ip，然后删除网卡，最后删除ip
        if (!model.getIpAllocationMethod().equals("Dynamic")) {
            Optional.ofNullable(publicIPAddress.getAssignedNetworkInterfaceIPConfiguration()).ifPresent(nicIPConfiguration -> {
                NicIpConfigurationModel ipConfigurationModel = new NicIpConfigurationModel();
                ipConfigurationModel.setUuid(nicIPConfiguration.inner().id());
                ipConfigurationModel.setName(nicIPConfiguration.name());
                ipConfigurationModel.setNetworkInterfaceUuid(nicIPConfiguration.parent().id());
                ipConfigurationModel.setIsPrimary(nicIPConfiguration.isPrimary());
                ipConfigurationModel.setVpcUuid(nicIPConfiguration.networkId());
                ipConfigurationModel.setPublicIpAddressUuid(nicIPConfiguration.publicIPAddressId());
                ipConfigurationModel.setPrivateIpAddress(nicIPConfiguration.privateIPAddress());
                model.setNicIpConfigurationModel(ipConfigurationModel);
            });
        }
        model.setTags(null == publicIPAddress.tags() ? null : JSONObject.toJSONString(publicIPAddress.tags()));
        return model;
    }
}