package com.bocloud.cmp.azureconvertor.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.model.azure.network.RouterTableModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.Route;
import com.microsoft.azure.management.network.RouteTable;
import org.springframework.util.CollectionUtils;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: RouterTableConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/31 14:35
 */
public class RouterTableConvertor implements BeanConvertor<RouterTableModel, RouteTable> {

    @Override
    public RouterTableModel convertModel(RouteTable table) {
        RouterTableModel model = new RouterTableModel();
        model.setName(table.name());
        model.setRegionId(table.regionName());
        model.setResourceGroupName(table.resourceGroupName());
        model.setUuid(table.id());
        //我吐了
        model.setRoutePropagation(!table.isBgpRoutePropagationDisabled());
        Collection<Route> routers = table.routes().values();
        if (!CollectionUtils.isEmpty(routers)) {
            List<RouterModel> routerModels = routers.stream().map(router -> {
                RouterModel routerModel = new RouterModel();
                routerModel.setName(router.name());
                routerModel.setRouterUuid(router.inner().id());
                routerModel.setAddressPrefix(router.destinationAddressPrefix());
                routerModel.setRouterType(router.nextHopType().toString());
                routerModel.setRegionId(router.parent().regionName());
                routerModel.setResourceGroupName(router.parent().resourceGroupName());
                routerModel.setRouterTableUuid(router.parent().id());
                routerModel.setNextHopType(router.nextHopType().toString());
                routerModel.setNextHopIpAddress(router.nextHopIPAddress());
                return routerModel;
            }).collect(Collectors.toList());
            model.setRouterModels(routerModels);
        }
        model.setTags(null == table.tags() ? null : JSONObject.toJSONString(table.tags()));
        return model;
    }
}