package com.bocloud.cmp.azureconvertor.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.NetworkSecurityGroup;
import com.microsoft.azure.management.network.NetworkSecurityRule;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SecurityGroupConvertor
 * @Description:安全组数据转换器
 * @author: zhenpingLi/李振平
 * @date: 2019/9/19 11:07
 */

public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, NetworkSecurityGroup> {

    @Override
    public SecurityGroupModel convertModel(NetworkSecurityGroup group) {
        SecurityGroupModel model = new SecurityGroupModel();
        model.setRegionId(group.regionName());
        model.setGroupUuid(group.id());
        model.setName(group.name());
        model.setResourceGroupName(group.resourceGroupName());
        List<SecurityGroupRuleModel> ruleModels = new ArrayList<>();
        Map<String, NetworkSecurityRule> ruleMap = group.securityRules();
        if (null != ruleMap && !ruleMap.isEmpty()) {
            List<NetworkSecurityRule> securityRules = new ArrayList<>(ruleMap.values());
            for (NetworkSecurityRule securityRule : securityRules) {
                ruleModels.add(this.convertorRule(securityRule, group.id(), false));
            }
        }
        Map<String, NetworkSecurityRule> defaultRuleMap = group.defaultSecurityRules();
        if (null != defaultRuleMap && !defaultRuleMap.isEmpty()) {
            List<NetworkSecurityRule> securityRules = new ArrayList<>(defaultRuleMap.values());
            for (NetworkSecurityRule securityRule : securityRules) {
                ruleModels.add(this.convertorRule(securityRule, group.id(), true));
            }
        }
        model.setModels(ruleModels);
        model.setTags(null == group.tags() ? null : JSONObject.toJSONString(group.tags()));
        return model;
    }

    /**
     * 安全组规则转换
     *
     * @param rule
     * @param groupUuid
     * @return
     */
    private SecurityGroupRuleModel convertorRule(NetworkSecurityRule rule, String groupUuid, Boolean defaultRule) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        model.setRuleUuid(rule.inner().id());
        model.setName(rule.name());
        if (null != rule.sourceAddressPrefix()) {
            model.setSourceAddressPrefix(rule.sourceAddressPrefix());//源地址前缀
        } else {
            model.setSourceAddressPrefix(JSONObject.toJSONString(rule.sourceAddressPrefixes()));//源地址前缀
        }
        if (null != rule.sourcePortRange()) {
            model.setSourcePortRange(rule.sourcePortRange()); //源端口范围
        } else {
            model.setSourcePortRange(JSONObject.toJSONString(rule.sourcePortRanges())); //源端口范围
        }
        if (null != rule.destinationAddressPrefix()) {
            model.setDestinationAddressPrefix(rule.destinationAddressPrefix());//终点地址前缀
        } else {
            model.setDestinationAddressPrefix(JSONObject.toJSONString(rule.destinationAddressPrefixes()));//终点地址前缀
        }
        if (null != rule.destinationPortRange()) {
            model.setDestinationPortRange(rule.destinationPortRange());//终点端口范围
        } else {
            model.setDestinationPortRange(JSONObject.toJSONString(rule.destinationPortRanges()));//终点端口范围
        }
        model.setRemark(rule.description()); //描述
        model.setDirection(rule.direction().toString());//方向
        model.setProtocol(rule.protocol().toString()); //协议
        model.setDefaultRule(defaultRule); //是否默认安全组规则
        model.setGroupUuid(groupUuid); //安全组规格id
        model.setAction(rule.access().toString());//操作 允许/拒绝
        model.setPriority(rule.priority()); //优先级
        return model;
    }
}
