package com.bocloud.cmp.azureconvertor.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.microsoft.azure.management.network.Network;
import com.microsoft.azure.management.network.Subnet;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 虚拟网络数据转换器
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/8/26
 */
public class VpcConvertor implements BeanConvertor<VpcModel, Network> {

    @Override
    public VpcModel convertModel(Network network) {
        VpcModel model = new VpcModel();
        model.setVpcId(network.id());
        model.setName(network.name());
        model.setRegionId(network.regionName());
        model.setResourceGroupName(network.resourceGroupName());
        model.setStatus("ACTIVE");
        List<String> addressSpaces = network.addressSpaces();
        if (!CollectionUtils.isEmpty(addressSpaces)) {
            model.setCidr(JSONArray.toJSONString(addressSpaces));
        }
        List<String> dnsServerIps = network.dnsServerIPs();
        if (!CollectionUtils.isEmpty(dnsServerIps)) {
            model.setDnsServer(JSONArray.toJSONString(dnsServerIps));
        }
        //查询虚拟网络下的子网
        Map<String, Subnet> subnetMap = network.subnets();
        if (null != subnetMap && !subnetMap.isEmpty()) {
            List<Subnet> subnets = new ArrayList<>(subnetMap.values());
            BeanConvertor<SubnetModel, Subnet> subnetConvertor = new SubnetConvertor();
            model.setSubnetModels(subnetConvertor.convertList(subnets));
        }
        model.setDdosPlanUuid(network.ddosProtectionPlanId());
        model.setDdosPlanEnabled(network.isDdosProtectionEnabled());
        model.setTags(null == network.tags() ? null : JSONObject.toJSONString(network.tags()));
        return model;
    }
}
