package com.bocloud.cmp.azureconvertor.redis;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.redis.RedisModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.microsoft.azure.management.resources.fluentcore.arm.Region;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: RedisConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/25 14:17
 */
public class RedisConvertor implements BeanJsonConvertor<RedisModel> {

    @Override
    public RedisModel convertModel(JSONObject object) {
        RedisModel model = new RedisModel();
        model.setRedisUuid(object.getString("id"));
        model.setName(object.getString("name"));
        model.setRegionId(Region.findByLabelOrName(object.getString("location")).name());
        model.setResourceGroupName(object.getString("id").split("/")[4]);
        model.setType(object.getString("type"));
        List<String> zones = JSONArray.parseArray(object.getString("zones"), String.class);
        model.setZones(zones);
        Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
            Optional.ofNullable(properties.getString("provisioningState")).ifPresent(model::setStatus);
            Optional.ofNullable(properties.getString("redisVersion")).ifPresent(model::setRedisVersion);
            Optional.ofNullable(properties.getBooleanValue("enableNonSslPort")).ifPresent(model::setEnableNonSslPort);
            Optional.ofNullable(properties.getString("publicNetworkAccess")).ifPresent(model::setPublicNetworkAccess);
            Optional.ofNullable(properties.getString("hostName")).ifPresent(model::setHostName);
            Optional.ofNullable(properties.getInteger("port")).ifPresent(model::setPort);
            Optional.ofNullable(properties.getInteger("sslPort")).ifPresent(model::setSslPort);
            Optional.ofNullable(properties.getString("subnetId")).ifPresent(model::setSubnetUuid);
            Optional.ofNullable(properties.getString("staticIP")).ifPresent(model::setStaticIp);
            Optional.ofNullable(properties.getInteger("shardCount")).ifPresent(model::setShardCount);
            model.setMinimumTlsVersion(Optional.ofNullable(properties.getString("minimumTlsVersion")).orElse("默认值"));
            Optional.ofNullable(properties.getString("minimumTlsVersion")).ifPresent(model::setMinimumTlsVersion);
            //redis sku
            Optional.ofNullable(properties.getJSONObject("sku")).ifPresent(sku -> {
                model.setSkuName(sku.getString("name"));
                model.setSkuFamily(sku.getString("family"));
                model.setSkuCapacity(sku.getString("capacity"));
            });
            //redis 配置
            Optional.ofNullable(properties.getJSONObject("redisConfiguration")).ifPresent(redisConfiguration -> {
                Optional.ofNullable(redisConfiguration.getInteger("maxclients")).ifPresent(model::setMaxclients);
                Optional.ofNullable(redisConfiguration.getInteger("maxmemory-reserved")).ifPresent(model::setMaxmemoryReserved);
                Optional.ofNullable(redisConfiguration.getInteger("maxfragmentationmemory-reserved")).ifPresent(model::setMaxfragmentationmemoryReserved);
                Optional.ofNullable(redisConfiguration.getString("rdb-backup-enabled")).ifPresent(model::setRdbBackupEnabled);
                Optional.ofNullable(redisConfiguration.getString("rdb-backup-frequency")).ifPresent(model::setRdbBackupFrequency);
                Optional.ofNullable(redisConfiguration.getInteger("rdb-backup-max-snapshot-count")).ifPresent(model::setRdbBackupMaxSnapshotCount);
                Optional.ofNullable(redisConfiguration.getInteger("maxmemory-delta")).ifPresent(model::setMaxmemoryDelta);
                Optional.ofNullable(redisConfiguration.getString("rdb-storage-connection-string")).ifPresent(model::setRdbStorageConnectionString);
            });
            //与缓存关联的 Redis 实例的列表
            List<RedisModel.Instance> instances = Optional.ofNullable(properties.getJSONArray("instances")).map(list -> list.stream().map(JSONObject::toJSONString)
                    .map(JSONObject::parseObject).map(instance -> {
                        RedisModel.Instance redisInstance = model.new Instance();
                        redisInstance.setSslPort(instance.getInteger("sslPort"));
                        redisInstance.setNonSslPort(instance.getInteger("nonSslPort"));
                        redisInstance.setShardId(instance.getInteger("shardId"));
                        redisInstance.setZone(instance.getString("zone"));
                        redisInstance.setIsMaster(instance.getBoolean("isMaster"));
                        return redisInstance;
                    }).collect(Collectors.toList())).orElse(new ArrayList<>());
            model.setInstances(instances);

        });
        return model;
    }

}