package com.bocloud.cmp.azureconvertor.slb;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.azure.slb.OutboundRuleModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: OutboundRuleConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/5/15 15:36
 */
public class OutboundRuleConvertor implements BeanJsonConvertor<OutboundRuleModel> {

    @Override
    public OutboundRuleModel convertModelParams(JSONObject object, String... str) {
        OutboundRuleModel model = new OutboundRuleModel();
        model.setUuid(object.getString("id"));
        model.setName(object.getString("name"));
        model.setEtag(object.getString("etag"));
        model.setType(object.getString("type"));
        model.setSlbUuid(str[0]);
        Optional.ofNullable(object.getJSONObject("properties")).ifPresent(properties -> {
            Optional.ofNullable(properties.getInteger("allocatedOutboundPorts")).ifPresent(model::setAllocatedOutboundPorts);
            Optional.ofNullable(properties.getInteger("idleTimeoutInMinutes")).ifPresent(model::setIdleTimeoutInMinutes);
            Optional.ofNullable(properties.getString("protocol")).ifPresent(model::setProtocol);
            Optional.ofNullable(properties.getBooleanValue("enableTcpReset")).ifPresent(model::setEnableTcpReset);
            List<String> frontendUuid = Optional.ofNullable(properties.getJSONArray("frontendIPConfigurations")).map(list -> list.stream().map(JSONObject::toJSONString)
                    .map(JSONObject::parseObject)
                    .map(frontend -> frontend.getString("id")
                    ).collect(Collectors.toList())).orElse(new ArrayList<>());
            model.setFrontEndUuids(frontendUuid);
            Optional.ofNullable(properties.getJSONObject("backendAddressPool")).ifPresent(backendAddressPool -> {
                model.setBackEndUuid(backendAddressPool.getString("id"));
            });
        });
        return model;
    }


}