package com.bocloud.cmp.azureconvertor.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.microsoft.azure.management.compute.Disk;

import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: VolumeConvertor
 * @Description:硬盘数据转换器
 * @author: zhenpingLi/李振平
 * @date: 2019/9/17 17:55
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, Disk> {

    @Override
    public VolumeModel convertModel(Disk disk) {
        VolumeModel model = new VolumeModel();
        model.setVolumeUuid(disk.id());
        //通过云管创建的硬盘没有osType
        model.setProps(Optional.ofNullable(disk.osType()).map(operatingSystemTypes -> operatingSystemTypes.toString()).orElse(null));
        model.setName(disk.name());
        model.setType(disk.type());
        model.setSize(disk.sizeInGB());
        model.setRegionId(disk.regionName());
        model.setResourceGroupName(disk.resourceGroupName());
        model.setCreateOption(Optional.ofNullable(disk.creationMethod()).map(option -> option.toString()).orElse(null));
        //标准 HDD : Standard_LRS
        //高级 SSD : Premium_LRS
        //标准 SSD : StandardSSD_LRS
        disk.isAttachedToVirtualMachine();
        model.setCategory(disk.sku().toString());
        model.setInstanceId(disk.virtualMachineId());
        if (disk.inner().diskState().toString().toLowerCase().equals("attached")) {
            model.setStatus("IN_USE");
        } else if (disk.inner().diskState().toString().toLowerCase().equals("unattached")) {
            model.setStatus("AVAILABLE");
        } else {
            model.setStatus(disk.inner().diskState().toString().toUpperCase());
        }
        model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", disk.inner().timeCreated().toString(), "Z"));
        model.setSourceSnapshotId(disk.source().sourceId());
        model.setTags(null == disk.tags() ? null : JSONObject.toJSONString(disk.tags()));
        return model;
    }
}
