package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.common.annotations.Provider;
import com.microsoft.azure.AzureEnvironment;
import com.microsoft.azure.credentials.ApplicationTokenCredentials;
import com.microsoft.azure.management.Azure;
import com.microsoft.rest.LogLevel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.TimeUnit;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/27 14:24
 */
@Provider("bocloud.cmp.azure")
public class AzureProvider extends GenericProvider implements SuperProvider {

    private static final Logger LOGGER = LoggerFactory.getLogger(AzureProvider.class);
    private String subscriptionId;//订阅ID
    private String clientId;//应用程序(客户端) ID
    private String secretKey;//密钥
    private String tenant; //租户ID
    private Butler butler;
    private Azure azure;//访问客户端
    private String regionId;//地域
    private String version; //版本

    public AzureProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        this.tenant = butler.getTenant();
        this.subscriptionId = butler.getSubscriptionId();
        this.clientId = butler.getClientId();
        this.secretKey = butler.getSecretKey();
        this.regionId = butler.getRegionId();
        this.version = butler.getVersion();

        AzureEnvironment azureEnvironment;
        switch (version) {
            case "AZURE":
                azureEnvironment = AzureEnvironment.AZURE;
                break;
            case "AZURE_CHINA":
                azureEnvironment = AzureEnvironment.AZURE_CHINA;
                break;
            case "AZURE_US_GOVERNMENT":
                azureEnvironment = AzureEnvironment.AZURE_US_GOVERNMENT;
                break;
            case "AZURE_GERMANY":
                azureEnvironment = AzureEnvironment.AZURE_GERMANY;
                break;
            default:
                azureEnvironment = AzureEnvironment.AZURE_CHINA;
                break;
        }
        ApplicationTokenCredentials credentials = new ApplicationTokenCredentials(this.clientId, this.tenant, this.secretKey, azureEnvironment);
        this.azure = Azure.configure()
                .withLogLevel(LogLevel.NONE)
                .withConnectionTimeout(20, TimeUnit.MINUTES)
                .withReadTimeout(20, TimeUnit.MINUTES)
                .authenticate(credentials)
                .withSubscription(this.subscriptionId);
    }

    public Butler getButler() {
        return butler;
    }

    public Azure getAzure() {
        return azure;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    @Override
    public boolean accessable() {
        boolean flag;
        try {
            this.azure.resourceGroups().list();
            flag = true;
        } catch (Exception e) {
            LOGGER.error("Azure accessable error :", e);
            flag = false;
        }
//        LOGGER.info("AZURE SDK 认证 :" + flag);
        return flag;
    }
}