package com.bocloud.cmp.provider.azure.account;

import com.bocloud.cmp.azureconvertor.account.BatchAccountConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.account.BatchAccountModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.batch.BatchAccount;
import com.microsoft.azure.management.storage.StorageAccount;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureBatchAccountProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/7/24 13:38
 */
@Provider("bocloud.cmp.azure")
public class AzureBatchAccountProvider extends AzureProvider implements BatchAccountProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureBatchAccountProvider.class);

    public AzureBatchAccountProvider(Butler butler) {
        super(butler);
    }

    private BeanConvertor<BatchAccountModel, BatchAccount> convertor = new BatchAccountConvertor();

    @Override
    public BocloudResult list() {
        try {
            List<BatchAccountModel> batchAccountModels = convertor.convertList(this.getAzure().batchAccounts().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, batchAccountModels, "list batchAccount success!");
        } catch (Exception e) {
            logger.error("list batchAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            BatchAccount batchAccount = this.getAzure().batchAccounts().getById(uuid);
            return new BocloudResult(true, convertor.convertModel(batchAccount), "detail batchAccount success!");
        } catch (Exception e) {
            logger.error("detail batchAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().batchAccounts().deleteById(uuid);
            return new BocloudResult(true, "remove batchAccount success!");
        } catch (Exception e) {
            logger.error("remove batchAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(BatchAccountModel model) {
        return new BocloudResult(false, "暂不支持");
    }

    @Override
    public BocloudResult changeStorageAccount(BatchAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getStorageAccountUuid());
            BatchAccount batchAccount = this.getAzure().batchAccounts().getById(model.getUuid()).update()
                    .withExistingStorageAccount(storageAccount)
                    .apply();
            return new BocloudResult(true, convertor.convertModel(batchAccount), "modify batchAccount success!");
        } catch (Exception e) {
            logger.error("batchAccount changeStorageAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(BatchAccountModel model) {
        try {
            StorageAccount storageAccount = this.getAzure().storageAccounts().getById(model.getStorageAccountUuid());
            BatchAccount batchAccount = this.getAzure().batchAccounts().define(model.getName())
                    .withRegion(model.getRegionId())
                    .withExistingResourceGroup(model.getResourceGroupName())
                    .withExistingStorageAccount(storageAccount)
                    .create();
            return new BocloudResult(true, convertor.convertModel(batchAccount), "create batchAccount success!");
        } catch (Exception e) {
            logger.error("remove batchAccount error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}