package com.bocloud.cmp.provider.azure.automation;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.automation.AutomationAccountConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.automation.AutomationAccountModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureAutoAccout
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/8 10:57
 */
@Provider("bocloud.cmp.azure")
public class AzureAutomationAccountProvider extends AzureRestApiProvider implements AutomationAccoutProvider {

    public AzureAutomationAccountProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<AutomationAccountModel> convertor = new AutomationAccountConvertor();

    private static Logger logger = LoggerFactory.getLogger(AzureAutomationAccountProvider.class);

    @Override
    public BocloudResult list() {
        //url -> 订阅id
        String api = "/subscriptions/%s/providers/Microsoft.Automation/automationAccounts?api-version=2015-10-31";
        String url = String.format(api, this.getSubscriptionId());
        String response = this.request(RequestStatus.GET, url, null);
        List<AutomationAccountModel> models = convertor.convertList(JSONObject.parseObject(response).getJSONArray("value"));
        return new BocloudResult(true, models, "成功!");
    }

    @Override
    public BocloudResult detail(AutomationAccountModel model) {
        String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.Automation/automationAccounts/%s?api-version=2015-10-31";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.GET, url, null);
        AutomationAccountModel accountModel = convertor.convertModel(JSONObject.parseObject(response));

        String logAnalyticApi = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.Automation/automationAccounts/%s/linkedWorkspace?api-version=2015-10-31";
        String urlLogAnalytic = String.format(logAnalyticApi, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String responseLogAnalytic = this.request(RequestStatus.GET, urlLogAnalytic, null);
        Optional.ofNullable(responseLogAnalytic).map(logAnalytic -> JSONObject.parseObject(logAnalytic)).ifPresent(logAnalytic -> {
            Optional.ofNullable(logAnalytic.getString("id")).ifPresent(id -> {
                accountModel.setLogAnalyticUuid(id);
                accountModel.setLogAnalyticName(id.split("/")[8]);
            });
        });
        return new BocloudResult(true, accountModel, "成功!");
    }


    @Override
    public BocloudResult create(AutomationAccountModel model) {
        //url -> 订阅id、资源组名称、自动化账户名称
        //params :地域、自动化账户名称
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s?api-version=2015-10-31";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        JSONObject params = new JSONObject();
        params.put("location", model.getRegionId());
        params.put("name", model.getName());
        JSONObject skuParams = new JSONObject();
        skuParams.put("name", "Basic");
        JSONObject skuParam = new JSONObject();
        skuParam.put("sku", skuParams);
        params.put("properties", skuParam);
        String response = this.request(RequestStatus.PUT, url, params.toJSONString());
        return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
    }

    @Override
    public BocloudResult remove(AutomationAccountModel model) {
        String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.Automation/automationAccounts/%s?api-version=2015-10-31";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.DELETE, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "删除成功!");
    }

    @Override
    public BocloudResult getStatusCounts(AutomationAccountModel model, String resourceId) {
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurationMachineRuns/getStatusCounts?api-version=2017-05-15-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        JSONObject params = new JSONObject();
        params.put("softwareUpdateConfigurationRunId", resourceId);
        String response = this.request(RequestStatus.POST, url, params.toJSONString());
        return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
    }

    @Override
    public BocloudResult softwareUpdateConfigurationMachineRuns(AutomationAccountModel model, String historicalRecordName) {
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurationMachineRuns?$filter=properties/correlationId eq 38ca5139-574f-43a5-b21b-e73dee22a8f4&api-version=2017-05-15-preview&_=%s";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.GET, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
    }

    //resourceId = vm计算机名称（不是云主机名称）Name +_+ vmid(不是云主机uuid)
    @Override
    public BocloudResult hybridRunbookWorkerGroups(AutomationAccountModel model, String resourceId) {
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/hybridRunbookWorkerGroups/%s?api-version=2015-10-31";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName(), resourceId);
        String response = this.request(RequestStatus.GET, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
    }


}