package com.bocloud.cmp.provider.azure.automation;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.azure.AutomationTemplates;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.automation.AutomationAccountModel;
import com.bocloud.cmp.model.azure.automation.DeploymentPlanModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.utils.DateTimeTool;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDeploymentProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/8 14:46
 */
@Provider("bocloud.cmp.azure")
public class AzureDeploymentProvider extends AzureRestApiProvider implements DeploymentProvider {

    public AzureDeploymentProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDeploymentProvider.class);

    @Override
    public BocloudResult listDeploymentPlan(DeploymentPlanModel model) {
        //url -> 订阅id、资源组名称、自动化账户名称
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurations?api-version=2017-05-15-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getAutomationAccountName());
        String response = this.request(RequestStatus.GET, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "查询成功!");
    }

    @Override
    public BocloudResult createDeploymentPlan(DeploymentPlanModel model) {
        //url -> 订阅id、资源组名称、自动化账户名称、部署计划名称
        //params 云主机,frequency
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurations/%s?api-version=2017-05-15-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getAutomationAccountName(), model.getName());

        JSONObject params = new JSONObject();
        JSONObject propertiesParam = new JSONObject();
        //1-更新软件更新配置的特定属性
        JSONObject updateConfiguration = new JSONObject();
        //1.1-软件更新配置所针对的Azure云主机的Azure资源ID列表。
        updateConfiguration.put("azureVirtualMachines", model.getVirtualMachines());
        //1.2-软件更新配置运行允许的最大时间。 需要根据 ISO8601 使用 PT_n_H_M_n_n_S 格式指定持续时间
        updateConfiguration.put("duration", "PT2H");
        //1.3-目标机器的操作系统
        updateConfiguration.put("operatingSystem", model.getOperatingSystem());
        //1.4-Windows特定更新配置
        JSONObject osParms = new JSONObject();
        //1.4.1-重新启动软件更新配置设置
        osParms.put("rebootSetting", "IfRequired");
        //1.4.2-软件更新配置中包含的更新分类
        if (model.getOperatingSystem().equalsIgnoreCase("Windows")) {
            osParms.put("includedUpdateClassifications", "Critical,Security,UpdateRollup,FeaturePack,ServicePack,Definition,Tools,Updates");
            updateConfiguration.put("windows", osParms);
        } else if (model.getOperatingSystem().equalsIgnoreCase("Linux")) {
            osParms.put("includedPackageClassifications", "Critical,Other,Security,Unclassified");
            updateConfiguration.put("linux", osParms);
        }
        //1.5-对软件更新配置的目标进行分组
        JSONObject targetsParms = new JSONObject();
        //1.5.1-软件更新配置中的Azure查询列表
        JSONArray azureQuerieArray = new JSONArray();
        JSONObject azureQueries = new JSONObject();
        //1.5.1.1-要将查询范围范围到的位置列表
        azureQueries.put("locations", model.getRegionIds());
        //1.5.1.2-订阅或资源组 ARM ID 的列表
        azureQueries.put("scope", model.getResourceGroupUuids());
        //1.5.1.3-为 VM 标记设置
        JSONObject tagSettingParam = new JSONObject();
        //1.5.1.3.1-按"任意"或"所有指定标记"筛选 VM
        tagSettingParam.put("filterOperator", "All");
        //1.5.1.3.2-带有其值列表的标记字典
        tagSettingParam.put("tags", new JSONObject());
        azureQueries.put("tagSettings", tagSettingParam);
        azureQuerieArray.add(azureQueries);
        targetsParms.put("azureQueries", azureQuerieArray);
        updateConfiguration.put("targets", targetsParms);
        propertiesParam.put("updateConfiguration", updateConfiguration);
        //2-软件更新配置的计划信息
        JSONObject scheduleInfoParms = new JSONObject();
        if (model.getSingle()) {
            //2.1-获取或设置计划的频率
            scheduleInfoParms.put("frequency", "0");
            //2.3-获取或设置计划的开始时间
            scheduleInfoParms.put("startTime", DateTimeTool.getUtcTime(model.getStartTime()));
            //2.5-获取或设置计划的时区
            scheduleInfoParms.put("timeZone", "Asia/Shanghai");
        } else {
            //2.1-获取或设置计划的频率
            scheduleInfoParms.put("frequency", model.getFrequency());
            //2.2-获取或设置计划的时间间隔
            scheduleInfoParms.put("interval", model.getInterval());
            //2.3-获取或设置计划的开始时间
            scheduleInfoParms.put("startTime", DateTimeTool.getUtcTime(model.getStartTime()));
            //2.4-获取或设置计划的结束时间
            Optional.ofNullable(model.getExpiryTime()).ifPresent(expiryTime -> scheduleInfoParms.put("expiryTime", DateTimeTool.getUtcTime(model.getExpiryTime())));
//            scheduleInfoParms.put("nextRun", "2020-08-10T18:43:00");
            //2.5-获取或设置计划的时区
            scheduleInfoParms.put("timeZone", "Asia/Shanghai");
        }
//            //2.6-获取或设置高级计划
//            JSONObject advancedScheduleParam = new JSONObject();
//            //2.6.1-作业应执行的一周中的天数
//            String[] weekDays = new String[]{"Monday", "Thursday"};
//            advancedScheduleParam.put("weekDays", weekDays);
//            scheduleInfoParms.put("advancedSchedule", advancedScheduleParam);
        propertiesParam.put("scheduleInfo", scheduleInfoParms);
        params.put("properties", propertiesParam);
        String response = this.request(RequestStatus.PUT, url, params.toJSONString());
        return new BocloudResult(true, JSONObject.parseObject(response), "创建成功!");
    }

    @Override
    public BocloudResult removeDeploymentPlan(DeploymentPlanModel model) {
        //url -> 订阅id、资源组名称、自动化账户名称
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurations/%s?api-version=2017-05-15-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getAutomationAccountName(), model.getName());
        String response = this.request(RequestStatus.DELETE, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "删除成功!");
    }

    @Override
    public BocloudResult deploymentAutomationAccount(AutomationAccountModel model, String deploymentName) {
        try {
            //url -> 订阅id、资源组名称、部署名称
            //params :地域、自动化账户名称，LogAnalytic名称
            String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Resources/deployments/%s?api-version=2019-10-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), deploymentName);

//            String deploymentPath = this.getClass().getClassLoader().getResource("deploymentTemplate/automationAccount.json").getPath();
//            File file = new File(deploymentPath);
//            String content = FileUtils.readFileToString(file, "UTF-8");
            String content = AutomationTemplates.AUTOMATIONACCOUNT.getValue();
            JSONObject object = JSONObject.parseObject(content);
            object.getJSONObject("properties").getJSONObject("template").getJSONObject("parameters")
                    .getJSONObject("workspaceName").put("defaultValue", model.getLogAnalyticName());
            object.getJSONObject("properties").getJSONObject("template").getJSONObject("parameters")
                    .getJSONObject("automationAccountName")
                    .put("defaultValue", model.getName());
            object.getJSONObject("properties").getJSONObject("parameters").getJSONObject("location")
                    .put("value", model.getRegionId());
            String response = this.request(RequestStatus.PUT, url, object.toJSONString());
            return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
        } catch (Exception e) {
            logger.error("Deployment AutomationAccount Error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult softwareUpdateConfigurationRuns(AutomationAccountModel model) {
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Automation/automationAccounts/%s/softwareUpdateConfigurationRuns?api-version=2017-05-15-preview&_=%s";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName(), System.currentTimeMillis());
        String response = this.request(RequestStatus.GET, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "查询成功!");
    }

}