package com.bocloud.cmp.provider.azure.automation;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.automation.LogAnalyticConvertor;
import com.bocloud.cmp.entity.enums.azure.AutomationTemplates;
import com.bocloud.cmp.entity.enums.azure.LogAnalyticSku;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.automation.LogAnalyticModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.DateTimeTool;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureLogAnalytic
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/8/8 10:57
 */
@Provider("bocloud.cmp.azure")
public class AzureLogAnalyticProvider extends AzureRestApiProvider implements LogAnalyticProvider {

    public AzureLogAnalyticProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<LogAnalyticModel> convertor = new LogAnalyticConvertor();

    private static Logger logger = LoggerFactory.getLogger(AzureLogAnalyticProvider.class);

    @Override
    public BocloudResult list() {
        //url -> 订阅id
        String api = "/subscriptions/%s/providers/Microsoft.OperationalInsights/workspaces?api-version=2020-03-01-preview";
        String url = String.format(api, this.getSubscriptionId());
        String response = this.request(RequestStatus.GET, url, null);
        List<LogAnalyticModel> models = convertor.convertList(JSONObject.parseObject(response).getJSONArray("value"));
        List<LogAnalyticModel> modelList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(models)) {
            models.stream().forEach(accountModel -> {
                modelList.add(JSONObject.parseObject(JSONObject.toJSONString(this.ListsharedKeys(accountModel).getData()), LogAnalyticModel.class));
            });
        }
        return new BocloudResult(true, modelList, "查询成功!");
    }

    public BocloudResult ListsharedKeys(LogAnalyticModel model) {
        String apiKey = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s/sharedKeys?api-version=2020-03-01-preview";
        String urlKey = String.format(apiKey, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String responseKey = this.request(RequestStatus.POST, urlKey, null);
        Optional.ofNullable(responseKey).map(key -> JSONObject.parseObject(key)).ifPresent(jsonKey -> {
            model.setPrimarySharedKey(jsonKey.getString("primarySharedKey"));
            model.setSecondarySharedKey(jsonKey.getString("secondarySharedKey"));
        });
        return new BocloudResult(true, model, "成功!");
    }

    @Override
    public BocloudResult remove(LogAnalyticModel model) {
        //url -> 订阅id
        String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s?api-version=2020-03-01-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.DELETE, url, null);
        return new BocloudResult(true, JSONObject.parseObject(response), "删除成功!");
    }

    @Override
    public BocloudResult detail(LogAnalyticModel model) {
        String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s?api-version=2020-03-01-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.GET, url, null);
        LogAnalyticModel analyticModel = convertor.convertModel(JSONObject.parseObject(response));

        String apiKey = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s/sharedKeys?api-version=2020-03-01-preview";
        String urlKey = String.format(apiKey, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String responseKey = this.request(RequestStatus.POST, urlKey, null);
        Optional.ofNullable(responseKey).map(key -> JSONObject.parseObject(key)).ifPresent(jsonKey -> {
            analyticModel.setPrimarySharedKey(jsonKey.getString("primarySharedKey"));
            analyticModel.setSecondarySharedKey(jsonKey.getString("secondarySharedKey"));
        });
        return new BocloudResult(true, analyticModel, "查询成功!");
    }

    @Override
    public BocloudResult sharedKeys(LogAnalyticModel model) {
        try {
            //url -> 订阅id、资源组名称、工作区名称
            String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s/sharedKeys?api-version=2020-03-01-preview";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.POST, url, null);
            return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
        } catch (Exception e) {
            logger.error("logAnalytic sharedKeys error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(LogAnalyticModel model) {
        //url -> 订阅id、资源组名称、工作区名称
        //params : 地域、工作区名称
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s?api-version=2020-03-01-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        JSONObject params = new JSONObject();
        params.put("location", model.getRegionId());
        params.put("name", model.getName());
        JSONObject propertiesParam = new JSONObject();
        propertiesParam.put("publicNetworkAccessForIngestion", "Enabled");//用于访问 Log Analytics 引入的网络访问类型。
        propertiesParam.put("publicNetworkAccessForQuery", "Enabled");//用于访问 Log Analytics 查询的网络访问类型。
        propertiesParam.put("retentionInDays", 30);//工作区数据保持期（天）。 -1 表示无限制 Sku 的无限制保留。 所有其他 Sku 允许的最大值为730天。
        JSONObject skuParam = new JSONObject();
        skuParam.put("name", LogAnalyticSku.PERGB2018.getValue());
//            skuParam.put("maxCapacityReservationLevel", 3000);//选择 CapacityReservation sku 时，可用于此工作区的最大容量预留级别。
        propertiesParam.put("sku", skuParam);
        JSONObject workspaceCappingParam = new JSONObject();
        workspaceCappingParam.put("dailyQuotaGb", -1);//引入的每日工作区配额。 -1 表示无限制。
        propertiesParam.put("workspaceCapping", workspaceCappingParam);
        params.put("properties", propertiesParam);
        String response = this.request(RequestStatus.PUT, url, params.toJSONString());
        return new BocloudResult(true, JSONObject.parseObject(response), "创建成功!");
    }

    @Override
    public BocloudResult linkedServices(LogAnalyticModel model) {
        //url -> 订阅id、资源组名称、工作区名称
        //params : 自动化账户uuid
        String api = "/subscriptions/%s/resourcegroups/%s/providers/Microsoft.OperationalInsights/workspaces/%s/linkedServices/Automation?api-version=2020-03-01-preview";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        JSONObject params = new JSONObject();
        JSONObject propertiesParam = new JSONObject();
        propertiesParam.put("writeAccessResourceId", model.getAutomationAccountUuid());
        params.put("properties", propertiesParam);
        String response = this.request(RequestStatus.PUT, url, params.toJSONString());
        return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
    }

    @Override
    public BocloudResult queryLogAnalytic(LogAnalyticModel model, String operation, String resourceId) {
        try {
            String api = "/subscriptions/%s/resourcegroups/%s/providers/microsoft.operationalinsights/workspaces/%s/query?api-version=2017-10-01&%s";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName(), operation);
//            String fileResource = null;
//            switch (operation) {
//                case "q_OrchestratorExtension.DataModels.Computer":
//                    fileResource = "queryLogAnalytic/computer.json";
//                    break;
//                case "q_OrchestratorExtension.DataModels.MissingUpdate2":
//                    fileResource = "queryLogAnalytic/missingUpdate2.json";
//                    break;
//                default:
//                    break;
//            }
//
//            String deploymentPath = this.getClass().getClassLoader().getResource(fileResource).getPath();
//            File file = new File(deploymentPath);
//            String content = FileUtils.readFileToString(file, "UTF-8");
            JSONObject replaceObject;
            String queryStr;
            String content = null;
            String startTime = DateTimeTool.getUTCtime(0, new Date());
            String endTime = DateTimeTool.getUTCtime(24, new Date());
            String timespanStr = startTime + "/" + endTime;
            switch (operation) {
                case "q_OrchestratorExtension.DataModels.Computer":
                    content = AutomationTemplates.COMPUTER.getValue();
                    break;
                case "q_OrchestratorExtension.DataModels.MissingUpdate2":
                    content = AutomationTemplates.MISSINGUPDATE2.getValue();
                    break;
                case "q_OrchestratorExtension.DataModels.DeployedUpdatesSummary2":
                    content = AutomationTemplates.DEPLOYEDUPDATESSUMMARY2.getValue();
                    replaceObject = JSONObject.parseObject(content);
                    queryStr = replaceObject.getString("query");
                    queryStr = queryStr.replace("38ca5139-574f-43a5-b21b-e73dee22a8f4", resourceId);
                    replaceObject.put("query", queryStr);
                    replaceObject.put("timespan", timespanStr);
                    content = replaceObject.toJSONString();
                    break;
                case "q_OrchestratorExtension.DataModels.DeployedUpdateInMultiMachineView":
                    content = AutomationTemplates.DEPLOYEDUPDATEINMULTIMACHINEVIEW.getValue();
                    replaceObject = JSONObject.parseObject(content);
                    queryStr = replaceObject.getString("query");
                    queryStr = queryStr.replace("38ca5139-574f-43a5-b21b-e73dee22a8f4", resourceId);
                    replaceObject.put("query", queryStr);
                    replaceObject.put("timespan", timespanStr);
                    content = replaceObject.toJSONString();
                    break;
                default:
                    break;
            }
            JSONObject object = JSONObject.parseObject(content);
            String response = this.request(RequestStatus.POST, url, object.toJSONString());
            return new BocloudResult(true, JSONObject.parseObject(response), "成功!");
        } catch (Exception e) {
            logger.error("query LogAnalytic Error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }
}