package com.bocloud.cmp.provider.azure.checkname;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureResourceCheckName
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/3/10 10:50
 */
@Provider("bocloud.cmp.azure")
public class AzureResourceCheckNameProvider extends AzureRestApiProvider implements ResourceCheckNameProvider {

    public AzureResourceCheckNameProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureResourceCheckNameProvider.class);

    public Map<String, String> map = new HashMap<String, String>() {{
        //mysql
        put("Microsoft.DBforMySQL", "subscriptions/%s/providers/Microsoft.DBForMySQL/checkNameAvailability?api-version=2017-12-01");
        //redis
        put("Microsoft.Cache/Redis", "subscriptions/%s/providers/Microsoft.Cache/CheckNameAvailability?api-version=2020-06-01");
    }};

    @Override
    public BocloudResult checkName(String name, String type) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = map.get(type);
            String url = String.format(api, this.getSubscriptionId());
            JSONObject params = new JSONObject();
            params.put("name", name);
            params.put("type", type);
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            if (null == jsonObject) {
                jsonObject = new JSONObject();
            }
            switch (type) {
                case "Microsoft.DBforMySQL":
                    if (jsonObject.getBoolean("nameAvailable")) {
                        jsonObject.put("result", true);
                        jsonObject.put("message", "名称可用");
                    } else {
                        jsonObject.put("result", false);
                    }
                    break;
                case "Microsoft.Cache/Redis":
                    jsonObject.put("result", true);
                    jsonObject.put("message", "名称可用");
                    break;
                default:
                    jsonObject.put("result", false);
                    jsonObject.put("message", "名称校验失败！");
                    break;
            }
            return new BocloudResult(true, jsonObject.toJSONString(), LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            JSONObject dataObject = new JSONObject();
            dataObject.put("result", false);
            dataObject.put("message", e.getMessage());
            return new BocloudResult(true, dataObject.toJSONString(), e.getMessage());
        }
    }
}