package com.bocloud.cmp.provider.azure.db;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.PerformanceTierConvertor;
import com.bocloud.cmp.entity.enums.azure.PerformanceTierType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.PerformanceTierModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzurePerformanceTierProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/22 14:03
 */
@Provider("bocloud.cmp.azure")
public class AzurePerformanceTierProvider extends AzureRestApiProvider implements PerformanceTierProvider {

    private static Logger logger = LoggerFactory.getLogger(AzurePerformanceTierProvider.class);

    public AzurePerformanceTierProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<PerformanceTierModel> convertor = new PerformanceTierConvertor();

    @Override
    public BocloudResult listByMySql() {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "/subscriptions/%s/providers/Microsoft.DBForMySQL/locations/%s/performanceTiers?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), this.getRegionId());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PerformanceTierModel> models = convertor.convertListParams(jsonObject.getJSONArray("value"), this.getRegionId(), PerformanceTierType.MYSQL.getValue());
            return new BocloudResult(true, models, "查询成功!");
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}