package com.bocloud.cmp.provider.azure.db.administrator;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.DbAdministratorConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.model.azure.db.DbAdministratorModel;
import com.bocloud.cmp.model.azure.db.DbFirewallRuleModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDbAdministratorProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/2 10:07
 */
@Provider("bocloud.cmp.azure")
public class AzureDbAdministratorProvider extends AzureRestApiProvider implements DbAdministratorProvider {

    public AzureDbAdministratorProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDbAdministratorProvider.class);

    private BeanJsonConvertor<DbAdministratorModel> convertor = new DbAdministratorConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/administrators?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<DbAdministratorModel> administratorModels = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, administratorModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(DbAdministratorModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/administrators/activeDirectory?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(DbAdministratorModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/administrators/activeDirectory?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbAdministratorModel administratorModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            return new BocloudResult(true, administratorModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult create(DbAdministratorModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/administrators/activeDirectory?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("administratorType", model.getAdministratorType());
            properties.put("login", model.getLogin());
            properties.put("sid", model.getSid());
            properties.put("tenantId", model.getTenantId());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DbFirewallRuleModel ruleModel;
            return null;
//            if (null != jsonObject.getString("operation")) {
//                //数据库处于创建状态，需要根据名称查询
//                ruleModel = this.getDbFwRule(model);
//            } else {
//                ruleModel = convertor.convertModelParams(jsonObject, model.getDatabaseUuid());
//            }
//            if (ObjectUtils.isEmpty(ruleModel)) {
//                return new BocloudResult(false, "创建失败");
//            } else {
//                return new BocloudResult(true, ruleModel, "success!");
//            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}