package com.bocloud.cmp.provider.azure.db.database;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.CloudDatabaseConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseModel;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.ObjectUtils;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureCloudDatabaseProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/2 15:07
 */
@Provider("bocloud.cmp.azure")
public class AzureCloudDatabaseProvider extends AzureRestApiProvider implements CloudDatabaseProvider {

    public AzureCloudDatabaseProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureCloudDatabaseProvider.class);

    private BeanJsonConvertor<AzureDatabaseModel> convertor = new CloudDatabaseConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/databases?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<AzureDatabaseModel> databaseModels = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, databaseModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(AzureDatabaseModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/databases/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(AzureDatabaseModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/databases/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("charset", model.getDbCharset());
            properties.put("collation", model.getDbCollation());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseModel databaseModel;
            if (null != jsonObject.getString("operation")) {
                //数据库处于创建状态，需要根据名称查询
                databaseModel = this.getResource(model);
            } else {
                databaseModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            }
            if (ObjectUtils.isEmpty(databaseModel)) {
                return new BocloudResult(false, "创建失败");
            } else {
                return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public AzureDatabaseModel getResource(AzureDatabaseModel databaseModel) {
        AzureDatabaseModel model = null;
        try {
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 30 * 60 * 1000) {
                logger.info("执行轮询数据库：{}", databaseModel.getName());
                Thread.sleep(60 * 1000);
                BocloudResult detail = this.detail(databaseModel);
                if (detail.isSuccess()) {
                    model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), AzureDatabaseModel.class);
                    break;
                }
                endTime = System.currentTimeMillis();
            }
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
        }
        return model;
    }

    @Override
    public BocloudResult detail(AzureDatabaseModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/databases/%s?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getDatabaseServerName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            AzureDatabaseModel databaseModel = convertor.convertModelParams(jsonObject, model.getDatabaseServerUuid());
            return new BocloudResult(true, databaseModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}