package com.bocloud.cmp.provider.azure.db.log;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.db.CloudDatabaseServerLogConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerLogModel;
import com.bocloud.cmp.model.azure.db.AzureDatabaseServerModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureCloudDatabaseServerLogProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/2/4 14:44
 */
@Provider("bocloud.cmp.azure")
public class AzureCloudDatabaseServerLogProvider extends AzureRestApiProvider implements CloudDatabaseServerLogProvider {

    public AzureCloudDatabaseServerLogProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureCloudDatabaseServerLogProvider.class);

    private BeanJsonConvertor<AzureDatabaseServerLogModel> convertor = new CloudDatabaseServerLogConvertor();

    @Override
    public BocloudResult list(AzureDatabaseServerModel databaseModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s/logFiles?api-version=2017-12-01";
            String url = String.format(api, this.getSubscriptionId(), databaseModel.getResourceGroupName(), databaseModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<AzureDatabaseServerLogModel> ruleModels = convertor.convertListParams(jsonObject.getJSONArray("value"), databaseModel.getDatabaseServerUuid());
            return new BocloudResult(true, ruleModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}