package com.bocloud.cmp.provider.azure.ddos;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.ddos.DdosCustomPolicieConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.ddos.DdosCustomPolicieModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureDdosCustomPolicieProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/3/4 11:10
 */
@Provider("bocloud.cmp.azure")
public class AzureDdosCustomPolicieProvider extends AzureRestApiProvider implements DdosCustomPolicieProvider {

    public AzureDdosCustomPolicieProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureDdosCustomPolicieProvider.class);

    private BeanJsonConvertor<DdosCustomPolicieModel> convertor = new DdosCustomPolicieConvertor();


    @Override
    public BocloudResult create(DdosCustomPolicieModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/ddosCustomPolicies/%s?api-version=2020-07-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            JSONArray protocolCustomSettings = new JSONArray();
            properties.put("protocolCustomSettings", protocolCustomSettings);
            params.put("location", model.getRegionId());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DdosCustomPolicieModel customPolicieModel = convertor.convertModel(jsonObject);
            return new BocloudResult(true, customPolicieModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(DdosCustomPolicieModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/ddosCustomPolicies/%s?api-version=2020-07-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject tags = new JSONObject();
            if (StringUtils.hasText(model.getTags())) {
                tags = JSONObject.parseObject(model.getTags());
            }
            params.put("tags", tags);
            String response = this.request(RequestStatus.PATCH, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            DdosCustomPolicieModel customPolicieModel = convertor.convertModel(jsonObject);
            return new BocloudResult(true, customPolicieModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(DdosCustomPolicieModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/ddosCustomPolicies/%s?api-version=2020-07-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DdosCustomPolicieModel customPolicieModel = convertor.convertModel(jsonObject);
            return new BocloudResult(true, customPolicieModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(DdosCustomPolicieModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/ddosCustomPolicies/%s?api-version=2020-07-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}