package com.bocloud.cmp.provider.azure.ddos;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.ddos.DdosProtectionPlanModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.PagedList;
import com.microsoft.azure.management.network.DdosProtectionPlan;
import com.microsoft.azure.management.network.implementation.DdosProtectionPlanInner;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/3/1
 */
@Slf4j
@Provider("bocloud.cmp.azure")
public class AzureDdosProtectionPlanProvider extends AzureProvider implements DdosProtectionPlanProvider {

    public AzureDdosProtectionPlanProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String resourceGroup) {
        PagedList<DdosProtectionPlan> plans = StringUtils.hasText(resourceGroup) ? this.getAzure().ddosProtectionPlans().listByResourceGroup(resourceGroup) : this.getAzure().ddosProtectionPlans().list();
        List<DdosProtectionPlanModel> items = plans.stream().map(d -> convertor(d)).collect(Collectors.toList());
        return new BocloudResult(true, items, "查询成功");
    }

    @Override
    public BocloudResult create(DdosProtectionPlanModel model) {
        try {
            DdosProtectionPlan ddosProtectionPlan = this.getAzure().ddosProtectionPlans()
                    .define(model.getName())
                    .withRegion(model.getRegionId())
                    .withExistingResourceGroup(model.getResourceGroupName())
                    .create();
            Assert.notNull(ddosProtectionPlan, "创建失败");
            return new BocloudResult(true, this.getDdos(ddosProtectionPlan.id()), "创建成功");
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(DdosProtectionPlanModel model) {
        try {
            DdosProtectionPlan ddosProtectionPlan = this.getAzure().ddosProtectionPlans().getById(model.getDdosUuid());
            Assert.notNull(ddosProtectionPlan, "数据不存在");
            String tags = model.getTags();
            if (StringUtils.isEmpty(tags)) {
                ddosProtectionPlan = ddosProtectionPlan.update().withTags(new HashMap<>()).apply();
            } else {
                ddosProtectionPlan = ddosProtectionPlan.update().withTags(JSONObject.parseObject(tags, Map.class)).apply();
            }
            return new BocloudResult(true, this.getDdos(ddosProtectionPlan.id()), "修改成功");
        } catch (Exception e) {
            log.error("Get error message:{}", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String planId) {
        try {
            log.info("删除ddos：{}", planId);
            this.getAzure().ddosProtectionPlans().deleteById(planId);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            log.error("删除ddos失败:{}", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public DdosProtectionPlanModel getDdos(String planId) {
        DdosProtectionPlanModel model = null;
        Long startTime = System.currentTimeMillis();
        while (true) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            DdosProtectionPlan plan = this.getAzure().ddosProtectionPlans().getById(planId);
            if (null == plan) {
                break;
            }
            if (null != plan.provisioningState() && "Succeeded".equalsIgnoreCase(plan.provisioningState().toString())) {
                model = convertor(plan);
                break;
            }
            Long endTime = System.currentTimeMillis();
            if (endTime - startTime > 60 * 1000) {
                break;
            }
        }
        return model;
    }

    private DdosProtectionPlanModel convertor(DdosProtectionPlan plan) {
        DdosProtectionPlanModel model = new DdosProtectionPlanModel();
        model.setDdosUuid(plan.id());
        model.setName(plan.name());
        model.setResourceGroupName(plan.resourceGroupName());
        model.setRegionId(plan.regionName());
        model.setStatus(plan.provisioningState().toString());
        model.setResourceGuid(plan.resourceGuid());
        model.setType(plan.type());
        model.setPlanKey(plan.key());
        model.setTags(null == plan.tags() ? null : JSONObject.toJSONString(plan.tags()));
        DdosProtectionPlanInner inner = plan.inner();
        if (null != inner) {
            model.setEtag(inner.etag());
        }
        return model;
    }
}
