package com.bocloud.cmp.provider.azure.redis.firewall;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.redis.RedisFirewallRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.redis.RedisFirewallRuleModel;
import com.bocloud.cmp.model.azure.redis.RedisModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureRedisFirewallRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/3/1 13:40
 */
@Provider("bocloud.cmp.azure")
public class AzureRedisFirewallRuleProvider extends AzureRestApiProvider implements RedisFirewallRuleProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureRedisFirewallRuleProvider.class);

    private BeanJsonConvertor<RedisFirewallRuleModel> convertor = new RedisFirewallRuleConvertor();

    public AzureRedisFirewallRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(RedisModel redisModel) {
        try {
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/firewallRules?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), redisModel.getResourceGroupName(), redisModel.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<RedisFirewallRuleModel> ruleModels = convertor.convertListParams(jsonObject.getJSONArray("value"), redisModel.getRedisUuid());
            return new BocloudResult(true, ruleModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(RedisFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/firewallRules/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getRedisName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(RedisFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/firewallRules/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getRedisName(), model.getName());
            JSONObject params = new JSONObject();
            JSONObject properties = new JSONObject();
            properties.put("startIP", model.getStartIp());
            properties.put("endIP", model.getEndIp());
            params.put("properties", properties);
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisFirewallRuleModel ruleModel = convertor.convertModelParams(jsonObject, model.getRedisUuid());
            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(RedisFirewallRuleModel model) {
        try {
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s/firewallRules/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getRedisName(), model.getName());
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            RedisFirewallRuleModel ruleModel = convertor.convertModelParams(jsonObject, model.getRedisUuid());
            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}