package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.BackEndConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.BackEndModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancerBackend;
import com.microsoft.azure.management.network.model.HasNetworkInterfaces;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: AzureBackEndProvider
 * @Description:后端池
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureBackEndProvider extends AzureProvider implements BackEndProvider {

    public AzureBackEndProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureBackEndProvider.class);

    private BeanConvertor<BackEndModel, LoadBalancerBackend> convertor = new BackEndConvertor();

    @Override
    public BocloudResult create(BackEndModel model) {
        try {
            List<HasNetworkInterfaces> vms = new ArrayList();
            model.getVirtualMachineUuids().forEach(vmId -> {
                vms.add(this.getAzure().virtualMachines().getById(vmId));
            });
            LoadBalancer loadBalancer;
            if (!CollectionUtils.isEmpty(vms)) {
                loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                        .update()
                        .defineBackend(model.getName())
                        .withExistingVirtualMachines(vms)
                        .attach()
                        .apply();
            } else {
                loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                        .update()
                        .defineBackend(model.getName())
                        .attach()
                        .apply();
            }
            return new BocloudResult(true, convertor.convertModel(loadBalancer.backends().get(model.getName())), "create backEnd success!");
        } catch (Exception e) {
            logger.error("create backEnd error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(BackEndModel model) {
        try {
            BocloudResult removeResult = this.remove(model);
            if (removeResult.isFailed()) {
                return removeResult;
            }
            BocloudResult modifyResult = this.create(model);
            if (modifyResult.isFailed()) {
                List<String> oldVmUuids = model.getOldVmUuids();
                model.setVirtualMachineUuids(oldVmUuids);
                this.create(model);
            }
            return modifyResult;
        } catch (Exception e) {
            logger.error("modify backEnd error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(BackEndModel model) {
        try {
            this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .withoutBackend(model.getName())
                    .apply();
            return new BocloudResult(true, "remove backEnd success!");
        } catch (Exception e) {
            logger.error("remove backEnd error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}