package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.PrivateFrontEndConvertor;
import com.bocloud.cmp.azureconvertor.slb.PublicFrontEndConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.FrontEndModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancerPrivateFrontend;
import com.microsoft.azure.management.network.LoadBalancerPublicFrontend;
import com.microsoft.azure.management.network.Network;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @ClassName: AzureFrontEndProvider
 * @Description:前端ip配置
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureFrontEndProvider extends AzureProvider implements FrontEndProvider {

    public AzureFrontEndProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureFrontEndProvider.class);

    private BeanConvertor<FrontEndModel, LoadBalancerPublicFrontend> publicConvertor = new PublicFrontEndConvertor();

    private BeanConvertor<FrontEndModel, LoadBalancerPrivateFrontend> privateConvertor = new PrivateFrontEndConvertor();

    @Override
    public BocloudResult create(FrontEndModel model) {
        try {
            LoadBalancer loadBalancer = null;
            switch (model.getFrontEndType()) {
                case "INTERNAL":
                    Network network = this.getAzure().networks().getById(model.getVpcUuid());
                    loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                            .update()
                            .definePrivateFrontend(model.getName())
                            .withExistingSubnet(network, model.getSubnetName())
                            .attach()
                            .apply();
                    return new BocloudResult(true, privateConvertor.convertModel(loadBalancer.privateFrontends().get(model.getName())), "create frontEnd success!");
                case "EXTERNAL":
                    loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                            .update()
                            .definePublicFrontend(model.getName())
                            .withExistingPublicIPAddress(model.getPublicIpUuid())
                            .attach()
                            .apply();
                    return new BocloudResult(true, publicConvertor.convertModel(loadBalancer.publicFrontends().get(model.getName())), "create frontEnd success!");
                default:
                    break;
            }
            return new BocloudResult(false, "create frontEnd error!");
        } catch (Exception e) {
            logger.error("create frontEnd error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult remove(FrontEndModel model) {
        try {
            this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .withoutFrontend(model.getName())
                    .apply();
            return new BocloudResult(true, "remove frontEnd success!");
        } catch (Exception e) {
            logger.error("remove frontEnd error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}