package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.InboundNatRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.InboundNatRuleModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancerInboundNatRule;
import com.microsoft.azure.management.network.TransportProtocol;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @ClassName: AzureInboundNatRuleProvider
 * @Description:入站NAT规则
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureInboundNatRuleProvider extends AzureProvider implements InboundNatRuleProvider {

    public AzureInboundNatRuleProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureInboundNatRuleProvider.class);

    private BeanConvertor<InboundNatRuleModel, LoadBalancerInboundNatRule> convertor = new InboundNatRuleConvertor();

    @Override
    public BocloudResult create(InboundNatRuleModel model) {
        try {
            LoadBalancer loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .defineInboundNatRule(model.getName())
                    .withProtocol(TransportProtocol.fromString(model.getProtocol()))
                    .fromFrontend(model.getFrontEndName())
                    .fromFrontendPort(model.getFrontEndPort())
                    .toBackendPort(model.getBackEndPort())
                    .withFloatingIP(model.getFloatingIPEnabled())
                    .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes())
                    .attach()
                    .apply();
            return new BocloudResult(true, convertor.convertModel(loadBalancer.inboundNatRules().get(model.getName())), "create inboundNatRule success!");
//            //20210421采用api方式创建-因为sdk不支持选择nic的ip配置
//            LoadBalancer loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid());
//            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
//            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/loadBalancers/%s/inboundNatRules/%s?api-version=2020-11-01";
//            String url = String.format(api, provider.getSubscriptionId(), loadBalancer.resourceGroupName(), loadBalancer.name(), model.getName());
//            JSONObject params = new JSONObject();
//            params.put("name", model.getName());
//            JSONObject properties = new JSONObject();
//            properties.put("backendPort", model.getBackEndPort());
//            properties.put("enableFloatingIP", model.getFloatingIPEnabled());
//            properties.put("enableTcpReset", false);
//            JSONObject frontendIPConfiguration = new JSONObject();
//            frontendIPConfiguration.put("id", model.getFrontEndUuid());
//            properties.put("frontendIPConfiguration", frontendIPConfiguration);
//            properties.put("frontendPort", model.getFrontEndPort());
//            properties.put("idleTimeoutInMinutes", model.getIdleTimeoutInMinutes());
//            properties.put("protocol", model.getProtocol());
//            JSONObject backendIPConfiguration = new JSONObject();
//            backendIPConfiguration.put("id", model.getNicIpConfigurationUuid());
//            properties.put("backendIPConfiguration", backendIPConfiguration);
//            params.put("properties", properties);
//            String response = provider.request(AzureRestApiProvider.RequestStatus.PUT, url, params.toJSONString());
//            JSONObject jsonObject = JSONObject.parseObject(response);
//            RedisFirewallRuleModel ruleModel = convertor.convertModelParams(jsonObject, model.getRedisUuid());
//            return new BocloudResult(true, ruleModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
//            return null;
        } catch (Exception e) {
            logger.error("create inboundNatRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(InboundNatRuleModel model) {
        try {
            this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .withoutInboundNatRule(model.getName())
                    .apply();
            return new BocloudResult(true, "remove inboundNatRule success!");
        } catch (Exception e) {
            logger.error("remove inboundNatRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}