package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.LoadBalanceRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.LoadBalanceRuleModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancingRule;
import com.microsoft.azure.management.network.LoadDistribution;
import com.microsoft.azure.management.network.TransportProtocol;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @ClassName: AzureLoadBalanceRuleProvider
 * @Description:负载均衡规则
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureLoadBalanceRuleProvider extends AzureProvider implements LoadBalanceRuleProvider {

    public AzureLoadBalanceRuleProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureLoadBalanceRuleProvider.class);

    private BeanConvertor<LoadBalanceRuleModel, LoadBalancingRule> convertor = new LoadBalanceRuleConvertor();

    @Override
    public BocloudResult create(LoadBalanceRuleModel model) {
        try {
            LoadBalancer loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .defineLoadBalancingRule(model.getName())
                    .withProtocol(TransportProtocol.fromString(model.getProtocol()))
                    .fromFrontend(model.getFrontEndName())
                    .fromFrontendPort(model.getFrontEndPort())
                    .toBackend(model.getBackEndName())
                    .toBackendPort(model.getBackEndPort())
                    .withProbe(model.getProbeName())
                    .withLoadDistribution(LoadDistribution.fromString(model.getLoadDistribution()))//会话持续行
                    .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes())//会话超时（分钟）
                    .withFloatingIP(model.getFloatingIpEnabled())
                    .attach()
                    .apply();
            return new BocloudResult(true, convertor.convertModel(loadBalancer.loadBalancingRules().get(model.getName())), "create loadBalanceRule success!");
        } catch (Exception e) {
            logger.error("create loadBalanceRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(LoadBalanceRuleModel model) {
        try {
            LoadBalancer loadBalancer = this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .updateLoadBalancingRule(model.getName())
                    .withProtocol(TransportProtocol.fromString(model.getProtocol()))
                    .fromFrontend(model.getFrontEndName())
                    .fromFrontendPort(model.getFrontEndPort())
                    .toBackendPort(model.getBackEndPort())
                    .withProbe(model.getProbeName())
                    .withLoadDistribution(LoadDistribution.fromString(model.getLoadDistribution()))//会话持续行
                    .withIdleTimeoutInMinutes(model.getIdleTimeoutInMinutes())//会话超时（分钟）
                    .withFloatingIP(model.getFloatingIpEnabled())
                    .parent()
                    .apply();
            return new BocloudResult(true, convertor.convertModel(loadBalancer.loadBalancingRules().get(model.getName())), "create loadBalanceRule success!");
        } catch (Exception e) {
            logger.error("modify loadBalanceRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult remove(LoadBalanceRuleModel model) {
        try {
            this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .withoutLoadBalancingRule(model.getName())
                    .apply();
            return new BocloudResult(true, "remove loadBalanceRule success!");
        } catch (Exception e) {
            logger.error("remove loadBalanceRule error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}