package com.bocloud.cmp.provider.azure.slb;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.slb.OutboundRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.OutboundRuleModel;
import com.bocloud.cmp.model.azure.slb.SlbModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureOutboundRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/5/15 15:25
 */
@Provider("bocloud.cmp.azure")
public class AzureOutboundRuleProvider extends AzureProvider implements OutboundRuleProvider {

    public AzureOutboundRuleProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<OutboundRuleModel> convertor = new OutboundRuleConvertor();

    private static Logger logger = LoggerFactory.getLogger(AzureOutboundRuleProvider.class);

    @Override
    public BocloudResult list(SlbModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            //url -> 订阅id、资源组名称、自动化账户名称
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/loadBalancers/%s/outboundRules?api-version=2020-11-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<OutboundRuleModel> models = convertor.convertListParams(jsonObject.getJSONArray("value"), model.getUuid());
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    public BocloudResult create(OutboundRuleModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/loadBalancers/%s?api-version=2020-11-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getSlbName());
            String oldSlbString = provider.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject oldSlb = JSONObject.parseObject(oldSlbString);
            JSONObject oldProperties = oldSlb.getJSONObject("properties");
            JSONArray oldOutboundRules = oldProperties.getJSONArray("outboundRules");

            JSONObject outboundRule = new JSONObject();
            outboundRule.put("protocol", model.getProtocol());
            outboundRule.put("allocatedOutboundPorts", model.getAllocatedOutboundPorts());
            outboundRule.put("enableTcpReset", model.getEnableTcpReset());
            outboundRule.put("idleTimeoutInMinutes", model.getIdleTimeoutInMinutes());
            JSONObject backendAddressPool = new JSONObject();
            backendAddressPool.put("id", model.getBackEndUuid());
            outboundRule.put("backendAddressPool", backendAddressPool);
            JSONArray frontendIPConfigurations = new JSONArray();
            for (String frontEndUuid : model.getFrontEndUuids()) {
                JSONObject frontendIPConfiguration = new JSONObject();
                frontendIPConfiguration.put("id", frontEndUuid);
                frontendIPConfigurations.add(frontendIPConfiguration);
            }
            outboundRule.put("frontendIPConfigurations", frontendIPConfigurations);
            oldOutboundRules.add(outboundRule);
            oldProperties.put("outboundRules", oldOutboundRules);
            oldSlb.put("properties", oldProperties);
            String s = oldSlb.toJSONString();
            String response = provider.request(AzureRestApiProvider.RequestStatus.PUT, url, oldSlb.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            return null;
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public BocloudResult remove(OutboundRuleModel model) {
        try {
            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Network/loadBalancers/%s/outboundRules/%s?api-version=2020-11-01";
            String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getSlbName(), model.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.DELETE, url, null);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }


}