package com.bocloud.cmp.provider.azure.slb;

import com.bocloud.cmp.azureconvertor.slb.ProbeHttpConvertor;
import com.bocloud.cmp.azureconvertor.slb.ProbeTcpConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.slb.ProbeModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.LoadBalancer;
import com.microsoft.azure.management.network.LoadBalancerHttpProbe;
import com.microsoft.azure.management.network.LoadBalancerTcpProbe;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @ClassName: AzureProbeProvider
 * @Description:运行状况探测
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureProbeProvider extends AzureProvider implements ProbeProvider {

    public AzureProbeProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureFrontEndProvider.class);

    private BeanConvertor<ProbeModel, LoadBalancerTcpProbe> tcpConvertor = new ProbeTcpConvertor();
    private BeanConvertor<ProbeModel, LoadBalancerHttpProbe> httpConvertor = new ProbeHttpConvertor();

    @Override
    public BocloudResult create(ProbeModel model) {
        try {
            LoadBalancer.Update update = this.getAzure().loadBalancers().getById(model.getSlbUuid()).update();
            LoadBalancer loadBalancer;
            ProbeModel probeModel = null;
            if (model.getProtocol().equalsIgnoreCase("TCP")) {
                loadBalancer = update.defineTcpProbe(model.getName())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .attach()
                        .apply();
                probeModel = tcpConvertor.convertModel(loadBalancer.tcpProbes().get(model.getName()));
            } else if (model.getProtocol().equalsIgnoreCase("HTTP")) {
                loadBalancer = update.defineHttpProbe(model.getName())
                        .withRequestPath(model.getRequestPath())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .attach()
                        .apply();
                probeModel = httpConvertor.convertModel(loadBalancer.httpProbes().get(model.getName()));
            } else if (model.getProtocol().equalsIgnoreCase("HTTPS")) {
                loadBalancer = update.defineHttpsProbe(model.getName())
                        .withRequestPath(model.getRequestPath())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .attach()
                        .apply();
                probeModel = httpConvertor.convertModel(loadBalancer.httpsProbes().get(model.getName()));
            }
            return new BocloudResult(true, probeModel, "create probe success!");
        } catch (Exception e) {
            logger.error("create probe error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ProbeModel model) {
        try {
            LoadBalancer.Update update = this.getAzure().loadBalancers().getById(model.getSlbUuid()).update();
            LoadBalancer loadBalancer;
            ProbeModel probeModel = null;
            if (model.getProtocol().equalsIgnoreCase("TCP")) {
                loadBalancer = update
                        .updateTcpProbe(model.getName())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .parent()
                        .apply();
                probeModel = tcpConvertor.convertModel(loadBalancer.tcpProbes().get(model.getName()));
            } else if (model.getProtocol().equalsIgnoreCase("HTTP")) {
                loadBalancer = update.updateHttpProbe(model.getName())
                        .withRequestPath(model.getRequestPath())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .parent()
                        .apply();
                probeModel = httpConvertor.convertModel(loadBalancer.httpProbes().get(model.getName()));
            } else if (model.getProtocol().equalsIgnoreCase("HTTPS")) {
                loadBalancer = update.updateHttpsProbe(model.getName())
                        .withRequestPath(model.getRequestPath())
                        .withPort(model.getPort())
                        .withIntervalInSeconds(model.getIntervalInSeconds())
                        .withNumberOfProbes(model.getNumberOfProbes())
                        .parent()
                        .apply();
                probeModel = httpConvertor.convertModel(loadBalancer.httpsProbes().get(model.getName()));
            }
            return new BocloudResult(true, probeModel, "modify probe success!");
        } catch (Exception e) {
            logger.error("modify probe error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(ProbeModel model) {
        try {
            this.getAzure().loadBalancers().getById(model.getSlbUuid())
                    .update()
                    .withoutProbe(model.getName())
                    .apply();
            return new BocloudResult(true, "remove probe success!");
        } catch (Exception e) {
            logger.error("remove probe error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}