package com.bocloud.cmp.provider.azure.tags;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.azure.AzureConsumedServiceEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.tags.TagModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureTagProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/6/8 9:37
 */
@Provider("bocloud.cmp.azure")
public class AzureTagProvider extends AzureProvider implements TagProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureTagProvider.class);

    public AzureTagProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult tags(TagModel model) {
        try {
            AzureConsumedServiceEnum serviceEnum = AzureConsumedServiceEnum.valueOf(model.getAction());
            switch (serviceEnum) {
                case DDOS:
                    this.getAzure()
                            .ddosProtectionPlans()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case STORAGEACCOUNT:
                    this.getAzure()
                            .storageAccounts()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case VM:
                    this.getAzure()
                            .virtualMachines()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case EIP:
                    this.getAzure()
                            .publicIPAddresses()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case ROUTERTABLE:
                    this.getAzure()
                            .routeTables()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case VPC:
                    this.getAzure()
                            .networks()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case ASET:
                    this.getAzure()
                            .availabilitySets()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case SNAPSHOT:
                    this.getAzure()
                            .snapshots()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case VOLUME:
                    this.getAzure()
                            .disks()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case SLB:
                    this.getAzure()
                            .loadBalancers()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case RESOURCEGROUP:
                    this.getAzure()
                            .resourceGroups()
                            .getByName(model.getResourceName())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case NICS:
                    this.getAzure()
                            .networkInterfaces()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case SECURITYGROUP:
                    this.getAzure()
                            .networkSecurityGroups()
                            .getById(model.getResourceUuid())
                            .update()
                            .withTags(model.getTagMap())
                            .apply();
                    break;
                case MYSQL:
                    String mysqlResponse = this.putMysql(model);
                    break;
                case REDIS:
                    String redisResponse = this.putRedis(model);
                    break;
                case KEYPAIR:
                    String keysResponse = this.putKeys(model);
                    break;
                default:
                    break;
            }
            return new BocloudResult(true, "修改成功");
        } catch (Exception e) {
            logger.error("tags error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    private String putMysql(TagModel model) throws Exception {
        AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
        String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.DBForMySQL/servers/%s?api-version=2017-12-01";
        JSONObject params = new JSONObject();
        params.put("tags", model.getTagMap());
        String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getResourceName());
        return provider.request(AzureRestApiProvider.RequestStatus.PATCH, url, params.toJSONString());

    }

    private String putRedis(TagModel model) throws Exception {
        AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
        String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Cache/redis/%s?api-version=2020-06-01";
        JSONObject params = new JSONObject();
        params.put("tags", model.getTagMap());
        String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getResourceName());
        return provider.request(AzureRestApiProvider.RequestStatus.PATCH, url, params.toJSONString());
    }

    private String putKeys(TagModel model) throws Exception {
        AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
        String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/sshPublicKeys/%s?api-version=2021-03-01";
        JSONObject params = new JSONObject();
        params.put("tags", model.getTagMap());
        String url = String.format(api, provider.getSubscriptionId(), model.getResourceGroupName(), model.getResourceName());
        return provider.request(AzureRestApiProvider.RequestStatus.PATCH, url, params.toJSONString());
    }

}