package com.bocloud.cmp.provider.blob;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.blob.BlobConvertor;
import com.bocloud.cmp.azureconvertor.blob.ContainerApiConvertor;
import com.bocloud.cmp.azureconvertor.blob.ContainerConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.model.azure.blob.BlobModel;
import com.bocloud.cmp.model.azure.blob.ContainerModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.storage.OperationContext;
import com.microsoft.azure.storage.StorageCredentials;
import com.microsoft.azure.storage.StorageCredentialsAccountAndKey;
import com.microsoft.azure.storage.StorageUri;
import com.microsoft.azure.storage.blob.*;
import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;

import java.net.URI;
import java.util.*;

/**
 * 存储容器接口
 *
 * @author wangyu
 * @version 5.0.0-SNAPSHOT
 * @since 2021/2/1
 */
@Slf4j
@Provider("bocloud.cmp.azure")
public class AzureContainerProvider extends AzureProvider implements ContainerProvider {

    private static BeanConvertor<ContainerModel, CloudBlobContainer> containerConvertor = new ContainerConvertor();

    private BeanJsonConvertor<ContainerModel> apiConvertor = new ContainerApiConvertor();

    private static BeanConvertor<BlobModel, CloudBlob> blobConvertor = new BlobConvertor();

    public AzureContainerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(StorageAccountModel account) {
        try {
//            Assert.notNull(account.getName(), "存储账号为空");
//            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
//            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
//            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
//            URI primaryUri = new URI(account.getPrimaryEndPoint());
//            URI secondaryUri = null;
//            if (StringUtils.hasText(account.getSecondaryEndPoint())) {
//                secondaryUri = new URI(account.getSecondaryEndPoint());
//            }
//            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
//            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
//
//            Iterable<CloudBlobContainer> containers = client.listContainers(null, ContainerListingDetails.ALL, null, null);
//            if (null == containers) {
//                containers = new ArrayList<>();
//            }
//            List<CloudBlobContainer> blobContainers = new ArrayList<>();
//           for(CloudBlobContainer cloudBlobContainer:containers){
//               blobContainers.add(cloudBlobContainer);
//           }
//            List<ContainerModel> containerModels = containerConvertor.convertList(blobContainers);
//            return new BocloudResult(true, containerModels, "查询成功");

            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Storage/storageAccounts/%s/blobServices/default/containers?api-version=2021-01-01";
            String url = String.format(api, provider.getSubscriptionId(), account.getResourceGroupName(), account.getName());
            String response = provider.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<ContainerModel> models = apiConvertor.convertList(jsonObject.getJSONArray("value"));
            for (ContainerModel containerModel : models) {
                URI uri = new URI(account.getPrimaryEndPoint() + containerModel.getName());
                StorageUri storageUri = new StorageUri(uri);
                StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
                CloudBlobContainer container = new CloudBlobContainer(storageUri, credentials);
                this.apiBlobConvertor(container, containerModel);
            }
            return new BocloudResult(true, models, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "获取容器列表失败：" + e.getMessage());
        }
    }

    private void apiBlobConvertor(CloudBlobContainer container, ContainerModel containerModel) {
        containerModel.setMetadata(null == container.getMetadata() ? null : JSONObject.toJSONString(container.getMetadata()));
        containerModel.setUrl(null == container.getUri() ? null : container.getUri().toString());
        //查询blob存储
        Iterable<ListBlobItem> listBlobItems = container.listBlobs(null, true, EnumSet.of(BlobListingDetails.SNAPSHOTS, BlobListingDetails.METADATA), null, null);
        if (null == listBlobItems) {
            listBlobItems = new ArrayList<>();
        }
        Map<String, CloudBlob> blockBlobMap = new HashMap<>();
        Map<String, CloudBlob> snapshotBlobMap = new HashMap<>();
        try {
            listBlobItems.forEach(b -> {
                if (b instanceof CloudBlockBlob) {
                    CloudBlockBlob blockBlob = (CloudBlockBlob) b;
                    if (blockBlob.isSnapshot()) {
                        snapshotBlobMap.put(blockBlob.getName() + blockBlob.getSnapshotID(), blockBlob);
                    } else {
                        blockBlobMap.put(blockBlob.getName(), blockBlob);
                    }
                } else if (b instanceof CloudAppendBlob) {
                    CloudAppendBlob appendBlob = (CloudAppendBlob) b;
                    if (appendBlob.isSnapshot()) {
                        snapshotBlobMap.put(appendBlob.getName() + appendBlob.getSnapshotID(), appendBlob);
                    } else {
                        blockBlobMap.put(appendBlob.getName(), appendBlob);
                    }
                } else if (b instanceof CloudPageBlob) {
                    CloudPageBlob pageBlob = (CloudPageBlob) b;
                    if (pageBlob.isSnapshot()) {
                        snapshotBlobMap.put(pageBlob.getName() + pageBlob.getSnapshotID(), pageBlob);
                    } else {
                        blockBlobMap.put(pageBlob.getName(), pageBlob);
                    }
                }
            });
        } catch (Exception e) {
            log.error("获取容器文件失败,容器名称:" + containerModel.getName(), e);
        }
        List<CloudBlob> blobs = new ArrayList<>(blockBlobMap.values());
        List<CloudBlob> snapshotBlobs = new ArrayList<>(snapshotBlobMap.values());
        containerModel.setBlobs(blobConvertor.convertList(blobs));
        containerModel.setSnapshots(blobConvertor.convertList(snapshotBlobs));
    }


    /**
     * 创建容器
     *
     * @param model
     * @param account
     * @return
     */
    @Override
    public BocloudResult create(ContainerModel model, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(model.getName());
            BlobRequestOptions options = new BlobRequestOptions();
            OperationContext context = new OperationContext();
            boolean flag = container.createIfNotExists(BlobContainerPublicAccessType.valueOf(model.getPublicAccess()), options, context);
            Assert.isTrue(flag, "容器创建失败");
            container = client.getContainerReference(model.getName());
            Assert.notNull(container, "未找到创建成功的容器数据");
            return new BocloudResult(true, getContainer(client, container.getName()), "容器创建成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "容器创建失败：" + e.getMessage());
        }
    }

    public ContainerModel getContainer(CloudBlobClient client, String name) {
        ContainerModel model = null;
        Long startTime = System.currentTimeMillis();
        while (true) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                log.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            Iterable<CloudBlobContainer> cloudBlobContainers = client.listContainers(name, ContainerListingDetails.ALL, null, null);
            if (null == cloudBlobContainers) {
                break;
            }
            for (CloudBlobContainer container : cloudBlobContainers) {
                if (null != container.getProperties() && null != container.getProperties().getLeaseState()) {
                    model = containerConvertor.convertModel(container);
                    break;
                }
            }
            Long endTime = System.currentTimeMillis();
            if (endTime - startTime > 60 * 1000) {
                break;
            }
        }
        return model;
    }


    /**
     * 删除容器
     *
     * @param name    容器数据
     * @param account 存储账号
     * @return
     */
    @Override
    public BocloudResult delete(String name, StorageAccountModel account) {
        try {
            log.info("删除容器：{}", name);
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(name);
            boolean flag = container.deleteIfExists();
            Assert.isTrue(flag, "容器删除失败");
            return new BocloudResult(true, "容器删除成功");
        } catch (Exception e) {
            log.error("删除容器失败：{}", e);
            return new BocloudResult(false, "容器删除失败：" + e.getMessage());
        }
    }

    /**
     * 更新容器元数据
     *
     * @param name     容器名称
     * @param metadata 元数据
     * @param account  存储账号
     * @return
     */
    @Override
    public BocloudResult updateMetadata(String name, Map<String, String> metadata, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(name);
            container.setMetadata((HashMap<String, String>) metadata);
            container.uploadMetadata();
            return new BocloudResult(true, getContainer(client, name), "元数据更新成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "元数据更新失败：" + e.getMessage());
        }
    }

    /**
     * 更新公共访问级别
     *
     * @param name             容器名称
     * @param publicAccessType 公共访问级别
     * @param account          存储账号
     * @return
     */
    @Override
    public BocloudResult updatePublicAccess(String name, String publicAccessType, StorageAccountModel account) {
        try {
            Assert.notNull(account.getName(), "存储账号为空");
            Assert.notNull(account.getPrimaryKey(), "存储账号primary key");
            Assert.notNull(account.getPrimaryEndPoint(), "存储账号primary endpoint");
            StorageCredentials credentials = new StorageCredentialsAccountAndKey(account.getName(), account.getPrimaryKey());
            URI primaryUri = new URI(account.getPrimaryEndPoint());
            URI secondaryUri = null;
            if (StringUtils.hasText(account.getSecondaryEndPoint())) {
                secondaryUri = new URI(account.getSecondaryEndPoint());
            }
            StorageUri storageUri = new StorageUri(primaryUri, secondaryUri);
            CloudBlobClient client = new CloudBlobClient(storageUri, credentials);
            CloudBlobContainer container = client.getContainerReference(name);
            BlobContainerPermissions blobContainerPermissions = new BlobContainerPermissions();
            blobContainerPermissions.setPublicAccess(BlobContainerPublicAccessType.valueOf(publicAccessType));
            container.uploadPermissions(blobContainerPermissions);
            return new BocloudResult(true, "更新成功");
        } catch (Exception e) {
            log.error("Get error message：{}", e);
            return new BocloudResult(false, "更新失败：" + e.getMessage());
        }
    }
}
