package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.bill.BillConsumptionConvertor;
import com.bocloud.cmp.azureconvertor.bill.BillPeriodConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.model.azure.bill.AzureCloudBillModel;
import com.bocloud.cmp.model.azure.bill.BillPeriodModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureBillProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/17 16:50
 */
@Provider("bocloud.cmp.azure")
public class AzureBillProvider extends AzureRestApiProvider implements BillingProvider {

    public AzureBillProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureBillProvider.class);

    private BeanJsonConvertor<PublicCloudBillModel> billConsumptionConvertor = new BillConsumptionConvertor();

    private BeanJsonConvertor<BillPeriodModel> billPeriodConvertor = new BillPeriodConvertor();

    //获取计费周期
    @Override
    public BocloudResult billPeriods() {
        try {
            String api = "/subscriptions/%s/providers/Microsoft.Billing/billingPeriods?api-version=2017-04-24-preview";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<BillPeriodModel> billPeriodModels = billPeriodConvertor.convertList(jsonObject.getJSONArray("value"));
            logger.info("Azure 底层获取计费周期API成功！");
            return new BocloudResult(true, billPeriodModels, "成功！");
        } catch (Exception e) {
            logger.error("billPeriods error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    //根据计费周期获取账单
    @Override
    public BocloudResult periodPrices(String periodUuid) {
        try {
            AzureCloudBillModel model = new AzureCloudBillModel();
            String api = "%s/providers/Microsoft.Consumption/usageDetails?api-version=2019-10-01";
            String url = String.format(api, periodUuid);
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PublicCloudBillModel> billModels = billConsumptionConvertor.convertList(jsonObject.getJSONArray("value"));
            String nextLink = jsonObject.getString("nextLink");
            model.setModels(billModels);
            model.setNextLink(nextLink);
            logger.info("Azure 底层获取账单API成功！");
            return new BocloudResult(true, model, "成功!");
        } catch (Exception e) {
            logger.error("periodPrices error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    //根据分页获取账单
    @Override
    public BocloudResult nextLinkPrices(String nextLink) {
        try {
            AzureCloudBillModel model = new AzureCloudBillModel();
            String response = this.requestNoEndpoint(AzureRestApiProvider.RequestStatus.GET, nextLink, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PublicCloudBillModel> billModels = billConsumptionConvertor.convertList(jsonObject.getJSONArray("value"));
            String newNextLink = jsonObject.getString("nextLink");
            model.setModels(billModels);
            model.setNextLink(newNextLink);
            logger.info("Azure 底层分页账单API成功！");
            return new BocloudResult(true, model, "成功!");
        } catch (Exception e) {
            logger.error("nextLinkPrices error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    //根据计费周期按照时间段获取账单
    @Override
    public BocloudResult periodByTimePrices(String periodUuid, String startTime, String endTime) {
        try {
            logger.info("Azure 底层接受到请求账单操作......");
            AzureCloudBillModel model = new AzureCloudBillModel();
            String api = "%s/providers/Microsoft.Consumption/usageDetails?$filter=properties/usageStart+ge+'%s'+and+properties/usageEnd+lt+'%s'+&$expand=+properties/meterDetails+&api-version=2019-10-01";
            String url = String.format(api, periodUuid, startTime, endTime);
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<PublicCloudBillModel> billModels = billConsumptionConvertor.convertList(jsonObject.getJSONArray("value"));
            String nextLink = jsonObject.getString("nextLink");
            model.setModels(billModels);
            model.setNextLink(nextLink);
            logger.info("Azure 底层获取账单API成功！");
            return new BocloudResult(true, model, "成功!");
        } catch (Exception e) {
            logger.error("periodPrices error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}