package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.azure.AvailabilitySetModel;
import com.bocloud.cmp.model.azure.PublicIPAddressModel;
import com.bocloud.cmp.model.azure.account.BatchAccountModel;
import com.bocloud.cmp.model.azure.account.StorageAccountModel;
import com.bocloud.cmp.model.azure.blob.ContainerModel;
import com.bocloud.cmp.model.azure.db.*;
import com.bocloud.cmp.model.azure.ddos.DdosProtectionPlanModel;
import com.bocloud.cmp.model.azure.monitor.MetricAlertModel;
import com.bocloud.cmp.model.azure.network.RouterTableModel;
import com.bocloud.cmp.model.azure.redis.RedisFirewallRuleModel;
import com.bocloud.cmp.model.azure.redis.RedisModel;
import com.bocloud.cmp.model.azure.slb.SlbModel;
import com.bocloud.cmp.model.azure.sync.AzureSyncModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.azure.account.AzureBatchAccountProvider;
import com.bocloud.cmp.provider.azure.account.AzureStorageAccountProvider;
import com.bocloud.cmp.provider.azure.account.BatchAccountProvider;
import com.bocloud.cmp.provider.azure.account.StorageAccountProvider;
import com.bocloud.cmp.provider.azure.db.AzureCloudDatabaseServerProvider;
import com.bocloud.cmp.provider.azure.db.AzurePerformanceTierProvider;
import com.bocloud.cmp.provider.azure.db.CloudDatabaseServerProvider;
import com.bocloud.cmp.provider.azure.db.PerformanceTierProvider;
import com.bocloud.cmp.provider.azure.db.configuration.AzureDbConfigurationProvider;
import com.bocloud.cmp.provider.azure.db.configuration.DbConfigurationProvider;
import com.bocloud.cmp.provider.azure.db.database.AzureCloudDatabaseProvider;
import com.bocloud.cmp.provider.azure.db.database.CloudDatabaseProvider;
import com.bocloud.cmp.provider.azure.db.firewall.AzureDbFirewallRuleProvider;
import com.bocloud.cmp.provider.azure.db.firewall.DbFirewallRuleProvider;
import com.bocloud.cmp.provider.azure.db.log.AzureCloudDatabaseServerLogProvider;
import com.bocloud.cmp.provider.azure.db.log.CloudDatabaseServerLogProvider;
import com.bocloud.cmp.provider.azure.db.virtualnetwork.AzureDbVnetRuleProvider;
import com.bocloud.cmp.provider.azure.db.virtualnetwork.DbVnetRuleProvider;
import com.bocloud.cmp.provider.azure.ddos.AzureDdosCustomPolicieProvider;
import com.bocloud.cmp.provider.azure.ddos.AzureDdosProtectionPlanProvider;
import com.bocloud.cmp.provider.azure.ddos.DdosCustomPolicieProvider;
import com.bocloud.cmp.provider.azure.ddos.DdosProtectionPlanProvider;
import com.bocloud.cmp.provider.azure.network.AzureRouterTableProvider;
import com.bocloud.cmp.provider.azure.network.RouterTableProvider;
import com.bocloud.cmp.provider.azure.redis.AzureRedisProvider;
import com.bocloud.cmp.provider.azure.redis.RedisProvider;
import com.bocloud.cmp.provider.azure.redis.firewall.AzureRedisFirewallRuleProvider;
import com.bocloud.cmp.provider.azure.redis.firewall.RedisFirewallRuleProvider;
import com.bocloud.cmp.provider.azure.slb.AzureLoadBalanceProvider;
import com.bocloud.cmp.provider.azure.slb.LoadBalanceProvider;
import com.bocloud.cmp.provider.blob.AzureContainerProvider;
import com.bocloud.cmp.provider.blob.ContainerProvider;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.groups.AzureResourceGroupProvider;
import com.bocloud.cmp.provider.groups.ResourceGroupsProvider;
import com.bocloud.cmp.provider.monitor.AzureMetricAlertProvider;
import com.bocloud.cmp.provider.monitor.MetricAlertProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.AzureKeypairProvider;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.storage.AzureVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureLocationProvider
 * @Description:
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureLocationProvider extends AzureProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureLocationProvider.class);

    private Butler butler;
    private ServerProvider serverProvider;
    private ImageProvider imageProvider;
    private SecurityGroupProvider securityGroupProvider;
    private SnapshotProvider snapshotProvider;
    private VolumeProvider volumeProvider;
    private VpcProvider vpcProvider;
    private ResourceGroupsProvider resourceGroupsProvider;
    private FlavorProvider flavorProvider;
    private NetworkInterfaceProvider networkInterfaceProvider;
    private BatchAccountProvider batchAccountProvider;
    private StorageAccountProvider storageAccountProvider;
    private LoadBalanceProvider loadBalanceProvider;
    private PublicIPAddressProvider publicIPAddressProvider;
    private AvailabilitySetProvider availabilitySetProvider;
    private CloudDatabaseServerProvider cloudDatabaseServerProvider;
    private PerformanceTierProvider performanceTierProvider;
    private AzureRegionProvider azureRegionProvider;
    private KeypairProvider keypairProvider;
    private RouterTableProvider routerTableProvider;
    private MetricAlertProvider metricAlertProvider;
    private DbFirewallRuleProvider dbFirewallRuleProvider;
    private ContainerProvider containerProvider;
    private CloudDatabaseProvider cloudDatabaseProvider;
    private CloudDatabaseServerLogProvider cloudDatabaseServerLogProvider;
    private DbConfigurationProvider cloudDatabaseServerConfigurationProvider;
    private RedisProvider redisProvider;
    private RedisFirewallRuleProvider redisFirewallRuleProvider;
    private DdosProtectionPlanProvider planProvider;
    private DdosCustomPolicieProvider policieProvider;
    private DbVnetRuleProvider dbVnetRuleProvider;

    public AzureLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        resourceGroupsProvider = new AzureResourceGroupProvider(butler);
        serverProvider = new AzureServerProvider(butler);
        imageProvider = new AzureImageProvider(butler);
        volumeProvider = new AzureVolumeProvider(butler);
        snapshotProvider = new AzureSnapshotsProvider(butler);
        vpcProvider = new AzureVpcProvider(butler);
        securityGroupProvider = new AzureSecurityGroupProvider(butler);
        flavorProvider = new AzureFlavorProvider(butler);
        networkInterfaceProvider = new AzureNetworkInterfaceProvider(butler);
        batchAccountProvider = new AzureBatchAccountProvider(butler);
        storageAccountProvider = new AzureStorageAccountProvider(butler);
        loadBalanceProvider = new AzureLoadBalanceProvider(butler);
        publicIPAddressProvider = new AzurePublicIPAddressProvider(butler);
        availabilitySetProvider = new AzureAvailabilitySetProvider(butler);
        cloudDatabaseServerProvider = new AzureCloudDatabaseServerProvider(butler);
        performanceTierProvider = new AzurePerformanceTierProvider(butler);
        azureRegionProvider = new AzureRegionProvider(butler);
        keypairProvider = new AzureKeypairProvider(butler);
        routerTableProvider = new AzureRouterTableProvider(butler);
        metricAlertProvider = new AzureMetricAlertProvider(butler);
        dbFirewallRuleProvider = new AzureDbFirewallRuleProvider(butler);
        containerProvider = new AzureContainerProvider(butler);
        cloudDatabaseProvider = new AzureCloudDatabaseProvider(butler);
        cloudDatabaseServerLogProvider = new AzureCloudDatabaseServerLogProvider(butler);
        cloudDatabaseServerConfigurationProvider = new AzureDbConfigurationProvider(butler);
        redisProvider = new AzureRedisProvider(butler);
        redisFirewallRuleProvider = new AzureRedisFirewallRuleProvider(butler);
        planProvider = new AzureDdosProtectionPlanProvider(butler);
        policieProvider = new AzureDdosCustomPolicieProvider(butler);
        dbVnetRuleProvider = new AzureDbVnetRuleProvider(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            BocloudResult bocloudResult;
            AzureSyncModel syncModel = new AzureSyncModel();

            logger.info("{} - {} - resourceGroup -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = resourceGroupsProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setResourceGroupModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), ResourceGroupModel.class));
            logger.info("{} - {} - resourceGroup -->End", this.butler.getRegionId(), this.butler.getName());

            List<ResourceGroupModel> resourceGroupModels = syncModel.getResourceGroupModels();
            List<DdosProtectionPlanModel> ddosProtectionPlanModels = new ArrayList<>();
            if (!ListTool.isEmpty(resourceGroupModels)) {
                logger.info("{} - {} - ddos plan and policy-->Start", this.butler.getRegionId(), this.butler.getName());
                for (ResourceGroupModel groupModel : resourceGroupModels) {
                    bocloudResult = planProvider.list(groupModel.getName());
                    if (bocloudResult.isSuccess()) {
                        List<DdosProtectionPlanModel> planModels = JSONArray.parseArray(JSONObject.toJSONString(bocloudResult.getData()), DdosProtectionPlanModel.class);
                        if (!ListTool.isEmpty(planModels)) {
                            ddosProtectionPlanModels.addAll(planModels);
                        }
                    }
//                   bocloudResult=policieProvider.list(groupModel.getName());
//                   if(bocloudResult.isSuccess()){
//                       List<DdosProtectionPlanModel> planModels= JSONArray.parseArray(JSONObject.toJSONString(bocloudResult.getData()),DdosProtectionPlanModel.class);
//                       syncModel.setPlanModels(planModels);
//                   }
                }
                logger.info("{} - {} - ddos plan and policy-->Start", this.butler.getRegionId(), this.butler.getName());
            }
            syncModel.setPlanModels(ddosProtectionPlanModels);
            logger.info("{} - {} - networkInterface -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = networkInterfaceProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setNetworkInterfaceModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), NetworkInterfaceModel.class));
            logger.info("{} - {} - networkInterface -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - flavor -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = flavorProvider.sync();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setFlavorModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), FlavorModel.class));
            logger.info("{} - {} - flavor -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - volume -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = volumeProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setVolumeModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), VolumeModel.class));
            logger.info("{} - {} - volume -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - snapshot -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = snapshotProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setSnapshotModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), SnapshotModel.class));
            logger.info("{} - {} - snapshot -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - network -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = vpcProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setVpcModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), VpcModel.class));
            logger.info("{} - {} - network -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - cloudServer -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = serverProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setServerModels(JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), ServerModel.class));
            logger.info("{} - {} - cloudServer -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - securityGroup -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = securityGroupProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setSecurityGroupModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), SecurityGroupModel.class));
            logger.info("{} - {} - securityGroup -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - batchAccount -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = batchAccountProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setBatchAccountModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), BatchAccountModel.class));
            logger.info("{} - {} - batchAccount -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - storageAccount -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = storageAccountProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            List<StorageAccountModel> storageAccountModels = JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), StorageAccountModel.class);
            for (StorageAccountModel accountModel : storageAccountModels) {
                logger.info("start sync container of storage account:{}", accountModel.getName());
                bocloudResult = containerProvider.list(accountModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                accountModel.setContainers(JSONArray.parseArray(JSONObject.toJSONString(bocloudResult.getData()), ContainerModel.class));
                logger.info("end sync container of storage account:{}", accountModel.getName());
            }
            syncModel.setStorageAccountModels(storageAccountModels);
            logger.info("{} - {} - storageAccount -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - loadBalance -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = loadBalanceProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setSlbModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), SlbModel.class));
            logger.info("{} - {} - loadBalance -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - publicIPAddress -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = publicIPAddressProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setPublicIPAddressModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), PublicIPAddressModel.class));
            logger.info("{} - {} - publicIPAddress -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - availabilitySet -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = availabilitySetProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setAvailabilitySetModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), AvailabilitySetModel.class));
            logger.info("{} - {} - availabilitySet -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - keypair -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = keypairProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setKeypairModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), KeypairModel.class));
            logger.info("{} - {} - keypair -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - routerTable -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = routerTableProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setRouterTableModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), RouterTableModel.class));
            logger.info("{} - {} - routerTable -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - metricAlert -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = metricAlertProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setMetricAlertModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), MetricAlertModel.class));
            logger.info("{} - {} - metricAlert -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - performanceTier -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = performanceTierProvider.listByMySql();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setPerformanceTierModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), PerformanceTierModel.class));
            logger.info("{} - {} - performanceTier -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - databaseServer -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = cloudDatabaseServerProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            List<AzureDatabaseServerModel> azureDatabaseServerModels = JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), AzureDatabaseServerModel.class);
            for (AzureDatabaseServerModel azureDatabaseServerModel : azureDatabaseServerModels) {
                logger.info("{} - {} - databaseServer firewall rule -->Start", this.butler.getRegionId(), this.butler.getName());
                bocloudResult = dbFirewallRuleProvider.list(azureDatabaseServerModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                azureDatabaseServerModel.setDbFirewallRuleModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), DbFirewallRuleModel.class));
                logger.info("{} - {} - databaseServer firewall rule -->End", this.butler.getRegionId(), this.butler.getName());

//                logger.info("{} - {} - databaseServer database -->Start", this.butler.getRegionId(), this.butler.getName());
//                bocloudResult = cloudDatabaseProvider.list(azureDatabaseServerModel);
//                if (bocloudResult.isFailed()) {
//                    return bocloudResult;
//                }
//                azureDatabaseServerModel.setAzureDatabaseModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), AzureDatabaseModel.class));
//                logger.info("{} - {} - databaseServer database -->End", this.butler.getRegionId(), this.butler.getName());

                logger.info("{} - {} - databaseServer log -->Start", this.butler.getRegionId(), this.butler.getName());
                bocloudResult = cloudDatabaseServerLogProvider.list(azureDatabaseServerModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                azureDatabaseServerModel.setAzureDatabaseServerLogModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), AzureDatabaseServerLogModel.class));
                logger.info("{} - {} - databaseServer log -->End", this.butler.getRegionId(), this.butler.getName());

                logger.info("{} - {} - databaseServer configuration -->Start", this.butler.getRegionId(), this.butler.getName());
                bocloudResult = cloudDatabaseServerConfigurationProvider.list(azureDatabaseServerModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                azureDatabaseServerModel.setDbConfigurationModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), DbConfigurationModel.class));
                logger.info("{} - {} - databaseServer configuration -->End", this.butler.getRegionId(), this.butler.getName());

                logger.info("{} - {} - databaseServer vnet rule -->Start", this.butler.getRegionId(), this.butler.getName());
                bocloudResult = dbVnetRuleProvider.list(azureDatabaseServerModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                azureDatabaseServerModel.setDbVnetRuleModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), DbVnetRuleModel.class));
                logger.info("{} - {} - databaseServer vnet rule -->End", this.butler.getRegionId(), this.butler.getName());

            }
            syncModel.setAzureDatabaseServerModels(azureDatabaseServerModels);
            logger.info("{} - {} - databaseServer -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - redis -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = redisProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            List<RedisModel> redisModels = JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), RedisModel.class);
            for (RedisModel redisModel : redisModels) {
                logger.info("{} - {} - redis firewall rule -->Start", this.butler.getRegionId(), this.butler.getName());
                bocloudResult = redisFirewallRuleProvider.list(redisModel);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                redisModel.setRedisFirewallRuleModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), RedisFirewallRuleModel.class));
                logger.info("{} - {} - redis firewall rule -->End", this.butler.getRegionId(), this.butler.getName());
            }
            syncModel.setRedisModels(redisModels);
            logger.info("{} - {} - redis -->End", this.butler.getRegionId(), this.butler.getName());

            logger.info("{} - {} - image -->Start", this.butler.getRegionId(), this.butler.getName());
            bocloudResult = imageProvider.list();
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            syncModel.setImageModels(JSONArray.parseArray(JSONArray.toJSONString(bocloudResult.getData()), ImageModel.class));
            logger.info("{} - {} - image -->End", this.butler.getRegionId(), this.butler.getName());

            return new BocloudResult(true, syncModel, "success");
        } catch (Exception e) {
            logger.error("Azure sync failure :", e);
            return new BocloudResult(false, "Azure sync failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listRegions() {
        try {
            List<RegionModel> regionModels = Lists.newArrayList();
            BocloudResult bocloudResult = azureRegionProvider.list();
            if (bocloudResult.isSuccess()) {
                regionModels = JSONObject.parseArray(JSONObject.toJSONString(bocloudResult.getData()), RegionModel.class);
            }
            return new BocloudResult(true, regionModels, "list region success!");
        } catch (Exception e) {
            logger.error("list region failure :", e);
            return new BocloudResult(false, "list region failure :" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

    @Override
    public BocloudResult syncImages() {
        try {
            AzureSyncModel syncModel = new AzureSyncModel();
            List<ImageModel> imageModels = new ArrayList<>();
            BocloudResult imageResult = imageProvider.list();
            if (imageResult.isSuccess()) {
                imageModels = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()),
                        ImageModel.class);
            }
            syncModel.setImageModels(imageModels);
            return new BocloudResult(true, syncModel, "success!");
        } catch (Exception e) {
            logger.error("list image failure :", e);
            return new BocloudResult(false, "list image failure :" + e.getMessage());
        }
    }
}
