package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.azureconvertor.compute.AvailabilitySetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.azure.AvailabilitySetModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.compute.AvailabilitySet;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: AzureAvailabilitySetProvider
 * @Description:
 * @author: zhenpingLi/李振平
 */
@Provider("bocloud.cmp.azure")
public class AzureAvailabilitySetProvider extends AzureProvider implements AvailabilitySetProvider {

    public AzureAvailabilitySetProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureAvailabilitySetProvider.class);

    private BeanConvertor<AvailabilitySetModel, AvailabilitySet> convertor = new AvailabilitySetConvertor();

    @Override
    public BocloudResult list() {
        try {
            List<AvailabilitySetModel> regionModels = convertor.convertList(this.getAzure().availabilitySets().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list availabilitySet success!");
        } catch (Exception e) {
            logger.error("list availabilitySet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(AvailabilitySetModel model) {
        try {
            AvailabilitySet availabilitySet = this.getAzure().availabilitySets()
                    .define(model.getName())
                    .withRegion(model.getRegionId())
                    .withExistingResourceGroup(model.getResourceGroupName())
                    .withFaultDomainCount(model.getFaultDomainCount())//容错域
                    .withUpdateDomainCount(model.getUpdateDomainCount())//更新域
                    .create();
            return new BocloudResult(true, convertor.convertModel(availabilitySet), "create availabilitySet success!");
        } catch (Exception e) {
            logger.error("create availabilitySet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.getAzure().availabilitySets().deleteById(uuid);
            return new BocloudResult(true, "remove availabilitySet success!");
        } catch (Exception e) {
            logger.error("remove availabilitySet error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


//    public BocloudResult tag(String uuid) {
//        try {
//            this.getAzure().availabilitySets().getById(uuid)
//                    .update().withTags()
//            return new BocloudResult(true, "remove availabilitySet success!");
//        } catch (Exception e) {
//            logger.error("remove availabilitySet error :", e);
//            return new BocloudResult(false, e.getMessage());
//        }
//    }
}