package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.azureconvertor.compute.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.compute.Snapshot;
import com.microsoft.azure.management.compute.SnapshotUpdate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureSnapshotsProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/16 9:00
 */
@Provider("bocloud.cmp.azure")
public class AzureSnapshotsProvider extends AzureProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureSnapshotsProvider.class);

    private BeanConvertor<SnapshotModel, Snapshot> convertor = new SnapshotConvertor();


    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public AzureSnapshotsProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<SnapshotModel> regionModels = convertor.convertList(this.getAzure().snapshots().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list snapshot success!");
        } catch (Exception e) {
            logger.error("list snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            this.getAzure().snapshots().deleteById(model.getSnapshotUuid());
            return new BocloudResult(true, "remove snapshot success!");
        } catch (Exception e) {
            logger.error("remove snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            Snapshot snapshot = this.getAzure().snapshots().getById(model.getSnapshotUuid());
            return new BocloudResult(true, convertor.convertModel(snapshot), "detail snapshot success!");
        } catch (Exception e) {
            logger.error("detail snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            SnapshotUpdate snapshotUpdate = new SnapshotUpdate();
            snapshotUpdate.withDiskSizeGB(model.getSize());
            this.getAzure().snapshots().inner().update(model.getResourceGroupName(), model.getName(), snapshotUpdate);
            return new BocloudResult(true, "update snapshot success!");
        } catch (Exception e) {
            logger.error("update snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        try {
            Snapshot snapshot = this.getAzure().snapshots().getById(snapshotId);
            return new BocloudResult(true, convertor.convertModel(snapshot), "detail snapshot success!");
        } catch (Exception e) {
            logger.error("detail snapshot error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        return null;
    }

}
