package com.bocloud.cmp.provider.groups;

import com.bocloud.cmp.azureconvertor.ResourceGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ResourceGroupModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.resources.ResourceGroup;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureResourceGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/8/6 14:53
 */
@Provider("bocloud.cmp.azure")
public class AzureResourceGroupProvider extends AzureProvider implements ResourceGroupsProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureResourceGroupProvider.class);

    private BeanConvertor<ResourceGroupModel, ResourceGroup> convertor = new ResourceGroupConvertor();

    public AzureResourceGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<ResourceGroupModel> regionModels = convertor.convertList(this.getAzure().resourceGroups().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list resourceGroups success!");
        } catch (Exception e) {
            logger.error("list resourceGroups error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ResourceGroupModel model) {
        try {
            ResourceGroup resourceGroup = this.getAzure().resourceGroups()
                    .define(model.getName())
                    .withRegion(model.getRegionId())
                    .create();
            return new BocloudResult(true, convertor.convertModel(resourceGroup), "create resourceGroups success!");
        } catch (Exception e) {
            logger.error("create resourceGroups error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String resourceGroupName) {
        try {
            this.getAzure().resourceGroups().deleteByName(resourceGroupName);
            return new BocloudResult(true, "remove resourceGroups success!");
        } catch (Exception e) {
            logger.error("remove resourceGroups error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String resourceGroupName) {
        try {
            ResourceGroup resourceGroup = this.getAzure().resourceGroups().getByName(resourceGroupName);
            return new BocloudResult(true, convertor.convertModel(resourceGroup), "detail resourceGroups success!");
        } catch (Exception e) {
            logger.error("detail resourceGroups error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}
