package com.bocloud.cmp.provider.monitor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.monitor.AlarmConvertor;
import com.bocloud.cmp.model.AlarmModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureAlarmProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/29 14:34
 */
@Provider("bocloud.cmp.azure")
public class AzureAlarmProvider extends AzureRestApiProvider implements AlarmProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureAlarmProvider.class);

    private BeanJsonConvertor<AlarmModel> convertor = new AlarmConvertor();

    public AzureAlarmProvider(Butler butler) {
        super(butler);
    }

    //获取订阅下的告警信息
    @Override
    public BocloudResult sync() {
        try {
            String api = "%s/providers/Microsoft.AlertsManagement/alerts?api-version=2019-03-01";
            String url = String.format(api, "subscriptions/" + this.getSubscriptionId());
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            List<AlarmModel> models = convertor.convertList(JSONObject.parseObject(response).getJSONArray("value"));
            return new BocloudResult(true, models, "查询成功!");
        } catch (Exception e) {
            logger.error("list Alarm error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(AlarmModel alarmModel) {
        return null;
    }

    @Override
    public BocloudResult update(AlarmModel alarmModel) {
        try {
            String alarmUuid = alarmModel.getAlarmId();
            String[] alarmSplit = alarmUuid.split("/");
            String alarmId = alarmSplit[alarmSplit.length - 1];
            String api = "%s/providers/Microsoft.AlertsManagement/alerts/%s/changestate?api-version=2019-03-01&newState=%s";
            String url = String.format(api, alarmModel.getTarget(), alarmId, alarmModel.getStatus());
            String response = this.request(AzureRestApiProvider.RequestStatus.POST, url, null);
            AlarmModel model = convertor.convertModel(JSONObject.parseObject(response));
            return new BocloudResult(true, model, "修改成功!");
        } catch (Exception e) {
            logger.error("update Alarm error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }

    @Override
    public BocloudResult confirm(AlarmModel alarmModel) {
        return null;
    }
}