package com.bocloud.cmp.provider.monitor;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureMonitorProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/28 19:48
 */
@Provider("bocloud.cmp.azure")
public class AzureMonitorProvider extends AzureRestApiProvider implements MonitorProvider {

    public AzureMonitorProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureMonitorProvider.class);

    @Override
    public BocloudResult query(String time, String name, String category, List<String> types) {
        return null;
    }

    @Override
    public BocloudResult metricNamespaces(String resourceUri) {
        try {
            String api = "%s/providers/microsoft.insights/metricNamespaces?api-version=2017-12-01-preview";
            String url = String.format(api, resourceUri);
            String response = this.request(RequestStatus.GET, url, null);
            return new BocloudResult(true, response, "查询成功!");
        } catch (Exception e) {
            logger.error("list metricNamespaces error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult metricDefinitions(String resourceUri, String metricNamespace) {
        try {
            String api = "%s/providers/microsoft.insights/metricDefinitions?api-version=2018-01-01&metricnamespace=%s";
            String url = String.format(api, resourceUri, metricNamespace);
            String response = this.request(RequestStatus.GET, url, null);
            return new BocloudResult(true, response, "查询成功!");
        } catch (Exception e) {
            logger.error("list metricDefinitions error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult metrics(String resourceUri, String timespan, String interval, String metricnames, String aggregation, String metricNamespace) {
        try {
            String api = "%s/providers/microsoft.insights/metrics?timespan=%s&interval=%s&metricnames=%s&aggregation=%s&api-version=2018-01-01&metricnamespace=%s";
            String url = String.format(api, resourceUri, timespan, interval, metricnames, aggregation, metricNamespace);
            logger.info("监控指标API查询->url:{}", url);
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            return new BocloudResult(true, response, "查询成功!");
        } catch (Exception e) {
            logger.error("list metrics error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}