package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.azureconvertor.network.RouterTableConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RouterModel;
import com.bocloud.cmp.model.azure.network.RouterTableModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.RouteNextHopType;
import com.microsoft.azure.management.network.RouteTable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureRouterProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/31 13:57
 */
@Provider("bocloud.cmp.azure")
public class AzureRouterProvider extends AzureProvider implements RouterProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureRouterProvider.class);

    private BeanConvertor<RouterTableModel, RouteTable> convertor = new RouterTableConvertor();

    public AzureRouterProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(RouterModel model) {
        try {
            RouteTable routeTable;
            if (model.getNextHopType().equalsIgnoreCase("VirtualAppliance")) {
                routeTable = this.getAzure()
                        .routeTables()
                        .getById(model.getRouterTableUuid())
                        .update()
                        .defineRoute(model.getName())
                        .withDestinationAddressPrefix(model.getAddressPrefix())
                        .withNextHopToVirtualAppliance(model.getNextHopIpAddress())
                        .attach()
                        .apply();
            } else {
                routeTable = this.getAzure()
                        .routeTables()
                        .getById(model.getRouterTableUuid())
                        .update()
                        .defineRoute(model.getName())
                        .withDestinationAddressPrefix(model.getAddressPrefix())
                        .withNextHop(RouteNextHopType.fromString(model.getNextHopType()))
                        .attach()
                        .apply();
            }
            RouterModel routerModel = convertor.convertModel(routeTable).getRouterModels().stream().filter(router -> router.getName().equalsIgnoreCase(model.getName())).findFirst().get();
            return new BocloudResult(true, routerModel, "create router success!");
        } catch (Exception e) {
            logger.error("create router error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(RouterModel model) {
        try {
            this.getAzure()
                    .routeTables()
                    .getById(model.getRouterTableUuid())
                    .update()
                    .withoutRoute(model.getName())
                    .apply();
            return new BocloudResult(true, "remove router success!");
        } catch (Exception e) {
            logger.error("remove router error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult remove(String routerId) {
        return null;
    }

    @Override
    public BocloudResult detail(String routerId) {
        return null;
    }

    @Override
    public BocloudResult modify(RouterModel model) {
        return null;
    }

    @Override
    public BocloudResult attach(RouterModel model) {
        return null;
    }

    @Override
    public BocloudResult detach(RouterModel model) {
        return null;
    }

    @Override
    public BocloudResult config(RouterModel model) {
        return null;
    }

    @Override
    public BocloudResult clear(String routerId) {
        return null;
    }

}