package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.azureconvertor.network.SecurityGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.microsoft.azure.management.network.NetworkSecurityGroup;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureSecurityGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/16 13:37
 */
@Provider("bocloud.cmp.azure")
public class AzureSecurityGroupProvider extends AzureProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureSecurityGroupProvider.class);

    private BeanConvertor<SecurityGroupModel, NetworkSecurityGroup> convertor = new SecurityGroupConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public AzureSecurityGroupProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<SecurityGroupModel> regionModels = convertor.convertList(this.getAzure().networkSecurityGroups().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list networkSecurityGroup success!");
        } catch (Exception e) {
            logger.error("list networkSecurityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult detail(String sgId) {
        try {
            NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups().getById(sgId);
            return new BocloudResult(true, convertor.convertModel(networkSecurityGroup), "detail networkSecurityGroup success!");
        } catch (Exception e) {
            logger.error("detail networkSecurityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            //创建安全组同时创建自定义规则，后续优化
            NetworkSecurityGroup networkSecurityGroup = this.getAzure().networkSecurityGroups()
                    .define(group.getName())
                    .withRegion(group.getRegionId())
                    .withExistingResourceGroup(group.getResourceGroupName())
                    .create();
            return new BocloudResult(true, convertor.convertModel(networkSecurityGroup), "create networkSecurityGroup success!");
        } catch (Exception e) {
            logger.error("create networkSecurityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            this.getAzure().networkSecurityGroups().deleteById(sgId);
            return new BocloudResult(true, "remove networkSecurityGroup success!");
        } catch (Exception e) {
            logger.error("remove networkSecurityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            return new BocloudResult(false, "nonsupport");
        } catch (Exception e) {
            logger.error("update networkSecurityGroup error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
