package com.bocloud.cmp.provider.security;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.security.KeypairConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureKeypairProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2020/12/30 13:53
 */
@Provider("bocloud.cmp.azure")
public class AzureKeypairProvider extends AzureRestApiProvider implements KeypairProvider {

    private BeanJsonConvertor<KeypairModel> convertor = new KeypairConvertor();

    public AzureKeypairProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(AzureKeypairProvider.class);

    @Override
    public BocloudResult list() {
        try {
            String api = "/subscriptions/%s/providers/Microsoft.Compute/sshPublicKeys?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId());
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            List<KeypairModel> models = convertor.convertList(JSONObject.parseObject(response).getJSONArray("value")).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, models, "查询成功!");
        } catch (Exception e) {
            logger.error("list keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(KeypairModel model) {
        try {
            String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/sshPublicKeys/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            String response = this.request(AzureRestApiProvider.RequestStatus.GET, url, null);
            KeypairModel keypairModel = convertor.convertModel(JSONObject.parseObject(response));
            return new BocloudResult(true, keypairModel, "查询成功!");
        } catch (Exception e) {
            logger.error("detail keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(KeypairModel model) {
        try {
            String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/sshPublicKeys/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            this.request(RequestStatus.DELETE, url, null);
            return new BocloudResult(true, "删除成功!");
        } catch (Exception e) {
            logger.error("delete keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(KeypairModel model) {
        try {
            String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/sshPublicKeys/%s?api-version=2020-06-01";
            String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
            JSONObject params = new JSONObject();
            params.put("location", model.getRegionId());
            if (!StringUtils.isEmpty(model.getPublicKey())) {
                JSONObject properties = new JSONObject();
                properties.put("publicKey", model.getPublicKey());
                params.put("properties", properties);
            }
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject object = JSONObject.parseObject(response);
            model.setFingerprint(object.getString("id"));
            JSONObject properties = object.getJSONObject("properties");
            if (null != properties) {
                model.setPublicKey(properties.getString("publicKey"));
            }
            KeypairModel keypairModel = this.generateKeyPair(model);
            return new BocloudResult(true, keypairModel, "创建成功!");
        } catch (Exception e) {
            logger.error("create keypair error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public KeypairModel generateKeyPair(KeypairModel model) {
        String api = "/subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/sshPublicKeys/%s/generateKeyPair?api-version=2020-06-01";
        String url = String.format(api, this.getSubscriptionId(), model.getResourceGroupName(), model.getName());
        String response = this.request(RequestStatus.POST, url, null);
        JSONObject object = JSONObject.parseObject(response);
        model.setFingerprint(object.getString("id"));
        model.setPrivateKey(object.getString("privateKey"));
        model.setPublicKey(object.getString("publicKey"));
        return model;
    }


    @Override
    public BocloudResult detail(String keypairName) {
        return null;
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        return null;
    }

    @Override
    public BocloudResult delete(String keypairName) {
        return null;
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        return null;
    }
}