package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.azureconvertor.compute.SnapshotConvertor;
import com.bocloud.cmp.azureconvertor.storage.VolumeConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.AzureProvider;
import com.bocloud.cmp.provider.AzureRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.DateTools;
import com.microsoft.azure.management.compute.*;
import com.microsoft.azure.management.compute.implementation.VirtualMachineInner;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: AzureVolumeProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/9/17 17:33
 */
@Provider("bocloud.cmp.azure")
public class AzureVolumeProvider extends AzureProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(AzureVolumeProvider.class);

    private BeanConvertor<VolumeModel, Disk> convertor = new VolumeConvertor();

    private BeanConvertor<SnapshotModel, Snapshot> convertorSnap = new SnapshotConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public AzureVolumeProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            List<VolumeModel> regionModels = convertor.convertList(this.getAzure().disks().list()).stream().filter(model -> model.getRegionId().equalsIgnoreCase(this.getRegionId())).collect(Collectors.toList());
            return new BocloudResult(true, regionModels, "list disk success!");
        } catch (Exception e) {
            logger.error("list disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
//            Disk disk = null;
//            if (null != volume.getSourceSnapshotId() && !"".equals(volume.getSourceSnapshotId())) {
//                if (volume.getProps().equalsIgnoreCase("Linux")) {
//                    disk = this.getAzure().disks()
//                            .define(volume.getName())
//                            .withRegion(volume.getRegionId())
//                            .withExistingResourceGroup(volume.getAzureResourceGroupName())
//                            .withLinuxFromSnapshot(volume.getSourceSnapshotId())
//                            .create();
//                } else if (volume.getProps().equalsIgnoreCase("Windows")) {
//                    disk = this.getAzure().disks()
//                            .define(volume.getName())
//                            .withRegion(volume.getRegionId())
//                            .withExistingResourceGroup(volume.getAzureResourceGroupName())
//                            .withWindowsFromSnapshot(volume.getSourceSnapshotId())
//                            .create();
//                }
//            } else {
//                disk = this.getAzure().disks()
//                        .define(volume.getName())
//                        .withRegion(volume.getRegionId())
//                        .withExistingResourceGroup(volume.getAzureResourceGroupName())
//                        .withData()
//                        .withSizeInGB(volume.getSize())
//                        .withSku(DiskSkuTypes.fromStorageAccountType(DiskStorageAccountTypes.fromString(volume.getCategory())))
//                        .create();
//            }
//            return new BocloudResult(true, convertor.convertModel(disk), "create disk success!");

            AzureRestApiProvider provider = new AzureRestApiProvider(this.getButler());
            String api = "subscriptions/%s/resourceGroups/%s/providers/Microsoft.Compute/disks/%s?api-version=2020-12-01";
            String url = String.format(api, provider.getSubscriptionId(), volume.getResourceGroupName(), volume.getName());
            JSONObject params = new JSONObject();
            if (null != volume.getSourceSnapshotId() && !"".equals(volume.getSourceSnapshotId())) {
                params.put("location", volume.getRegionId());
                JSONObject properties = new JSONObject();
                JSONObject creationData = new JSONObject();
                creationData.put("createOption", "Copy");
                creationData.put("sourceResourceId", volume.getSourceSnapshotId());
                properties.put("creationData", creationData);
                params.put("properties", properties);
            } else {
                params.put("location", volume.getRegionId());
                JSONObject sku = new JSONObject();
                sku.put("name", volume.getCategory());
                params.put("sku", sku);
                JSONObject properties = new JSONObject();
                JSONObject creationData = new JSONObject();
                creationData.put("createOption", "Empty");
                properties.put("creationData", creationData);
                properties.put("diskSizeGB", volume.getSize());
                //磁盘是否加密
                if (!StringUtils.isEmpty(volume.getDiskEncryptionUuid())) {
                    JSONObject encryption = new JSONObject();
                    encryption.put("diskEncryptionSetId", volume.getDiskEncryptionUuid());
                    properties.put("encryption", encryption);
                }
                params.put("properties", properties);
            }
            provider.request(AzureRestApiProvider.RequestStatus.PUT, url, params.toJSONString());
            Thread.sleep(2000);
            String response = provider.request(AzureRestApiProvider.RequestStatus.GET, url, params.toJSONString());
            JSONObject object = JSONObject.parseObject(response);
            return new BocloudResult(true, apiConvertor(object), "create disk success!");
        } catch (Exception e) {
            logger.error("create disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    private VolumeModel apiConvertor(JSONObject jsonObject) {
        VolumeModel model = new VolumeModel();
        model.setVolumeUuid(jsonObject.getString("id"));
        model.setResourceGroupName(jsonObject.getString("id").split("/")[4]);
        model.setName(jsonObject.getString("name"));
        model.setType(jsonObject.getString("type"));
        model.setRegionId(jsonObject.getString("location"));
        model.setStatus("AVAILABLE");
        Optional.ofNullable(jsonObject.getJSONObject("sku")).ifPresent(sku -> model.setCategory(sku.getString("name")));
        Optional.ofNullable(jsonObject.getJSONObject("properties")).ifPresent(properties -> {
            Optional.ofNullable(properties.getJSONObject("creationData")).ifPresent(creationData -> {
                model.setCreateOption(creationData.getString("createOption"));
                model.setSourceSnapshotId(creationData.getString("sourceResourceId"));
            });
            model.setSize(properties.getInteger("diskSizeGB"));
            model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ss", properties.getString("timeCreated"), "Z"));
            Optional.ofNullable(properties.getJSONObject("encryption")).ifPresent(encryption -> {
                model.setDiskEncryptionUuid(encryption.getString("diskEncryptionSetId"));
            });
        });
        return model;
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        try {
            this.getAzure().disks().getById(volume.getVolumeUuid()).update().withSizeInGB(volume.getSize()).apply();
            return new BocloudResult(true, "update disk success!");
        } catch (Exception e) {
            logger.error("update disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            this.getAzure().disks().deleteById(volumeId);
            return new BocloudResult(true, "remove disk success!");
        } catch (Exception e) {
            logger.error("remove disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }


    @Override
    public BocloudResult detail(String volumeId) {
        try {
            Disk disk = this.getAzure().disks().getById(volumeId);
            return new BocloudResult(true, convertor.convertModel(disk), "detail disk success!");
        } catch (Exception e) {
            logger.error("detail disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        try {
            Snapshot snapshot = this.getAzure().snapshots().define(snapshotModel.getName())
                    .withRegion(snapshotModel.getRegionId())
                    .withExistingResourceGroup(snapshotModel.getResourceGroupName())
                    .withDataFromDisk(snapshotModel.getVolumeUuid())
                    .create();
            SnapshotModel newSnapModel = convertorSnap.convertModel(snapshot);
            newSnapModel.setStatus("CREATING");  //为service业务逻辑 设置状态
            return new BocloudResult(true, newSnapModel, "create snapshot success!");
        } catch (Exception e) {
            logger.error("create snapshot error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        try {
            Disk disk = this.getAzure().disks().getById(volumeId);
            this.getAzure().virtualMachines().getById(serverId).update().withExistingDataDisk(disk).apply();
            return new BocloudResult(true, "attach disk success!");
        } catch (Exception e) {
            logger.error("attach disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        return null;
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            //获取硬盘详情
            Disk disk = this.getAzure().disks().getById(volumeId);
            //获取主机对象
            VirtualMachine vm = this.getAzure().virtualMachines().getById(serverId);
            //获取主机内部信息
            VirtualMachineInner vmInner = vm.inner();
            //获取存储配置
            StorageProfile storageProfile = vmInner.storageProfile();
            //获取挂载盘
            List<DataDisk> dataDisks = storageProfile.dataDisks();
            for (DataDisk dataDisk : dataDisks) {
                if (dataDisk.name().equals(disk.name())) {
                    dataDisks.remove(dataDisk);
                    break;
                }
            }
            //更新挂载盘
            storageProfile.withDataDisks(dataDisks);
            //更新存储配置
            vmInner.withStorageProfile(storageProfile);
            //更新主机
            this.getAzure().virtualMachines().inner().beginCreateOrUpdate(vm.resourceGroupName(), vm.name(), vmInner);
            return new BocloudResult(true, "detach disk success!");
        } catch (Exception e) {
            logger.error("detach disk error :", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        return null;
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel volume) {
        return null;
    }
}
