package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.*;
import com.bocloud.cmp.provider.network.EasyStackGwQosProvider;
import com.bocloud.cmp.provider.network.EasyStackVpnConnProvider;
import com.bocloud.cmp.provider.network.EasyStackVpnProvider;
import com.bocloud.cmp.provider.utils.ResourcePercolator;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.api.OSClient;
import org.openstack4j.core.transport.Config;
import org.openstack4j.model.common.Identifier;
import org.openstack4j.model.compute.*;
import org.openstack4j.model.compute.ext.AvailabilityZone;
import org.openstack4j.model.compute.ext.Hypervisor;
import org.openstack4j.model.identity.v2.Tenant;
import org.openstack4j.model.identity.v3.Domain;
import org.openstack4j.model.identity.v3.Project;
import org.openstack4j.model.image.v2.Image;
import org.openstack4j.model.network.*;
import org.openstack4j.model.network.SecurityGroup;
import org.openstack4j.model.network.ext.*;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeSnapshot;
import org.openstack4j.model.storage.block.VolumeType;
import org.openstack4j.openstack.OSFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * easystack数据同步操作类
 */
@Slf4j
@Provider("bocloud.cmp.easystack")
public class EasyStackLocationProvider extends EasyStackProvider implements LocationProvider {

    private Butler butler;

    public EasyStackLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult listRegions() {
        return null;
    }

    @Override
    public BocloudResult syncHostAggregate() {
        return null;
    }


    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult sync() {
        OSSyncModel model = new OSSyncModel();
        try {

            //todo easy纳管时去掉 -- 目前测试使用openstack环境，创建资源需要选择项目，所以暂先不要注释
//            log.info("starting sync project......");
//            List<OSTenantModel> osTenants = new ArrayList<>();
//            try {
//                if (Version.V2 == this.getVersion()) {
//                    List<? extends Tenant> tenants = this.getClient2().identity().tenants().list();
//                    for (Tenant tenant : tenants) {
//                        OSTenantModel osTenant = new OSTenantModel(tenant.getName(), tenant.getDescription(), tenant.getId(),
//                                tenant.isEnabled(), null, null);
//                        osTenants.add(osTenant);
//                    }
//                } else {
//                    List<? extends Project> projects = this.getClient3().identity().projects().list();
//                    for (Project project : projects) {
//                        Domain domain = project.getDomain();
//                        String domainName;
//                        if (null != domain) {
//                            domainName = domain.getName();
//                        } else {
//                            domainName = project.getDomainId();
//                        }
//                        OSTenantModel tenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
//                                project.isEnabled(), project.getDomainId(), domainName);
//                        osTenants.add(tenant);
//                    }
//                }
//            } catch (Exception e) {
//                log.error("EasyStack项目数据异常：{}", e);
//            }
//            model.setTenants(osTenants);
//            log.info("end sync project !");
            //easy纳管时去掉

            log.info("starting sync AvailabilityZone......");

            List<? extends AvailabilityZone> zones = this.getClient().compute().zones().list(true);
            if (!ListTool.isEmpty(zones)) {
                List<RegionModel> regions = new ArrayList<>();
                for (AvailabilityZone aZones : zones) {
                    if(aZones.getZoneName().equalsIgnoreCase("internal")){
                        continue;
                    }
                    RegionModel region = new RegionModel();
                    region.setCode(aZones.getZoneName());
                    region.setName(aZones.getZoneName());
                    regions.add(region);
                }
                model.setRegions(regions);
            }
        } catch (Exception e) {
            log.error("EasyStack可用域异常：{}", e);
        }
        log.info("end sync AvailabilityZone !");

        log.info("starting sync HostAggregate......");
        try {
            List<? extends HostAggregate> aggregates = this.getClient().compute().hostAggregates().list();
            if (!ListTool.isEmpty(aggregates)) {
                List<ClusterModel> clusterModels = new ArrayList<>();
                for (HostAggregate aggregate : aggregates) {
                    clusterModels.add(convertAggregate(aggregate));
                }
                model.setClusterModels(clusterModels);
            }
        } catch (Exception e) {
            log.error("EasyStack主机组数据异常：{}", e);
        }
        log.info("end sync HostAggregate !");

        log.info("starting sync Hypervisor......");
        try {
            List<? extends Hypervisor> hypervisors = this.getClient().compute().hypervisors().list();
            if (!ListTool.isEmpty(hypervisors)) {
                BeanConvertor<HostModel, Hypervisor> convertHyperV = new HypervisorConvertor();
                model.setHostModels(convertHyperV.convertList(hypervisors));
            }
        } catch (Exception e) {
            log.error("EasyStack宿主机数据异常：{}", e);
        }
        log.info("end sync Hypervisor !");

        log.info("starting sync Server......");
        try {
            List<? extends Server> servers = this.getClient().compute().servers().listAll(true);
            List<Server> filterServers = new ArrayList<>();
            if (!ListTool.isEmpty(servers)) {
                for (Server server : servers) {
                    if (ResourcePercolator.serverFilter(server.getStatus())) {
                        filterServers.add(server);
                    }
                }
            }
            if (!ListTool.isEmpty(filterServers)) {
                BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
                model.setServerModels(convertServer.convertList(filterServers));
            }
        } catch (Exception e) {
            log.error("EasyStack云主机数据异常：{}", e);
        }
        log.info("end sync Server !");

        log.info("starting sync qos policy rules......");
        try {
            // 同步qos policy rules
            EasyStackGwQosProvider gwQosProvider = new EasyStackGwQosProvider(butler);
            BocloudResult qosPoliciesList = gwQosProvider.qosPoliciesList(null);
            if (qosPoliciesList.isSuccess()) {
                List<GwQosModel> gwQosModels = (List<GwQosModel>) qosPoliciesList.getData();
                if (null != gwQosModels && gwQosModels.size() > 0) {
                    for (GwQosModel gwQosModel : gwQosModels) {
                        BocloudResult qosRulesList = gwQosProvider.qosRulesList(gwQosModel);
                        if (qosRulesList.isSuccess()) {
                            gwQosModel.setQosRules((List<GwQosModel>) qosRulesList.getData());
                        }
                    }
                }
                model.setGwQosModel(gwQosModels);
            }
        } catch (Exception e) {
            log.error("EasyStack带宽数据异常：{}", e);
        }
        log.info("end sync qos policy rules !");

        log.info("starting sync Volume......");
        Map<String, String> filterParams = new HashMap<>();
        filterParams.put("all_tenants", "true");
        try {
            List<? extends Volume> volumes = this.getClient().blockStorage().volumes().list(filterParams);
            if (!ListTool.isEmpty(volumes)) {
                BeanConvertor<VolumeModel, Volume> convertVolumes = new VolumeConvertor();
                List<VolumeModel> volumeModels = new ArrayList<>();
                for (Volume volume : volumes) {
                    if (ResourcePercolator.volumeFilter(volume.getStatus())) {
                        volumeModels.add(convertVolumes.convertModel(volume));
                    }
                }
                model.setVolumeModels(volumeModels);
            }
        } catch (Exception e) {
            log.error("EasyStack硬盘数据异常：{}", e);
        }
        log.info("end sync Volume !");

        log.info("starting sync VolumeType......");
        try {
            List<? extends VolumeType> volumeTypes = this.getClient().blockStorage().volumes().listVolumeTypes();
            List<VolumeTypeModel> typeModels = new ArrayList<>();
            for (VolumeType type : volumeTypes) {
                VolumeTypeModel typeModel = new VolumeTypeModel();
                typeModel.setVolumeTypeUuid(type.getId());
                typeModel.setName(type.getName());
                if (null != type.getExtraSpecs() && !type.getExtraSpecs().isEmpty()) {
                    typeModel.setExtraSpecs(JSONObject.toJSONString(type.getExtraSpecs()));
                }
                typeModels.add(typeModel);
            }
            model.setTypeModels(typeModels);
        } catch (Exception e) {
            log.error("EasyStack硬盘类型数据异常：{}", e);
        }
        log.info("end sync VolumeType !");

        log.info("starting sync VolumeSnapshot......");
        try {
            List<? extends VolumeSnapshot> snapshots = this.getClient().blockStorage().snapshots().list(filterParams);
            if (!ListTool.isEmpty(snapshots)) {
                BeanConvertor<SnapshotModel, VolumeSnapshot> convertSnapshot = new SnapshotConvertor();
                List<SnapshotModel> snapshotModels = new ArrayList<>();
                for (VolumeSnapshot snapshot : snapshots) {
                    if (ResourcePercolator.snapshotFilter(snapshot.getStatus())) {
                        snapshotModels.add(convertSnapshot.convertModel(snapshot));
                    }
                }
                model.setSnapshotModels(snapshotModels);
            }
        } catch (Exception e) {
            log.error("EasyStack快照数据异常：{}", e);
        }
        log.info("end sync VolumeSnapshot !");

        log.info("starting sync Network......");
        try {
            List<? extends Network> networks = this.getClient().networking().network().list();
            // 过滤当前租户下的网络
            List<Network> filterNetworks = new ArrayList<>();
            if (!ListTool.isEmpty(networks)) {
                for (Network network : networks) {
                    // 过滤租户的或者共享的网络
                    if (ResourcePercolator.networkFilter(network.getStatus())) {
                        filterNetworks.add(network);
                    }
                }
            }
            if (!ListTool.isEmpty(filterNetworks)) {
                BeanConvertor<NetworkModel, Network> convertNetworks = new NetworkConvertor();
                model.setNetworkModels(convertNetworks.convertList(filterNetworks));
            }
        } catch (Exception e) {
            log.error("EasyStack网络数据异常：{}", e);
        }
        log.info("end sync Network !");

        log.info("starting sync image......");
        try {
            Map<String, String> imageParams = new HashMap<>();
            imageParams.put("limit", Integer.MAX_VALUE + "");
            List<? extends Image> images = this.getClient().imagesV2().list(imageParams);
            if (!ListTool.isEmpty(images)) {
                BeanConvertor<ImageModel, Image> imageConvertor = new V2ImageConvertor();
                List<ImageModel> imageModels = new ArrayList<>();
                for (Image image : images) {
                    if (ResourcePercolator.imageFilter(image.getStatus())) {
                        imageModels.add(imageConvertor.convertModel(image));
                    }
                }
                model.setImageModels(imageModels);
            }
        } catch (Exception e) {
            log.error("EasyStack快照数据异常：{}", e);
        }
        log.info("end sync image !");

        log.info("starting sync Flavor......");
        try {
            List<? extends Flavor> flavors = this.getClient().compute().flavors().list(true);
            if (!ListTool.isEmpty(flavors)) {
                BeanConvertor<FlavorModel, Flavor> convertFlavor = new FlavorConvertor();
                model.setFlavorModels(convertFlavor.convertList(flavors));
            }
        } catch (Exception e) {
            log.error("EasyStack规格数据异常：{}", e);
        }
        log.info("end sync Flavor !");

        log.info("starting sync Port......");
        try {
            List<? extends Port> ports = this.getClient().networking().port().list();
            if (!ListTool.isEmpty(ports)) {
                BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                model.setPortModels(convertPort.convertList(ports));
            }
        } catch (Exception e) {
            log.error("EasyStack端口数据异常：{}", e);
        }
        log.info("end sync Port !");

        log.info("starting sync Router......");
        try {
            List<? extends Router> routers = this.getClient().networking().router().list();
            if (!ListTool.isEmpty(routers)) {
                BeanConvertor<RouterModel, Router> convertRouter = new RouterConvertor();
                model.setRouterModels(convertRouter.convertList(routers));
            }
        } catch (Exception e) {
            log.error("EasyStack路由数据异常：{}", e);
        }
        log.info("end sync Router !");

        log.info("starting sync SecurityGroup......");
        try {
            List<? extends SecurityGroup> sGroups = this.getClient().networking().securitygroup().list();
            if (!ListTool.isEmpty(sGroups)) {
                BeanConvertor<SecurityGroupModel, SecurityGroup> convertGroup = new SecurityGroupConvertor();
                model.setGroupModels(convertGroup.convertList(sGroups));
            }
        } catch (Exception e) {
            log.error("EasyStack安全组数据异常：{}", e);
        }
        log.info("end sync SecurityGroup !");

        log.info("starting sync NetFloatingIP......");
        try {
            List<? extends NetFloatingIP> floatingIPs = this.getClient().networking().floatingip().list();
            if (!ListTool.isEmpty(floatingIPs)) {
                BeanConvertor<FloatingIpModel, NetFloatingIP> convertFip = new FloatingIpConvertor();
                List<FloatingIpModel> fips = new ArrayList<>();
                BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                for (NetFloatingIP filterFip : floatingIPs) {
                    if (!StringUtils.isEmpty(filterFip.getPortId())) {
                        Port port = this.getClient().networking().port().get(filterFip.getPortId());
                        List<PortModel> portModels = new ArrayList<>();
                        if (null != port) {
                            portModels.add(convertPort.convertModel(port));
                        }
                        FloatingIpModel fIpModel = convertFip.convertModel(filterFip);
                        fIpModel.setPorts(portModels);
                        fips.add(fIpModel);
                    } else {
                        fips.add(convertFip.convertModel(filterFip));
                    }
                }
                model.setfIpModels(fips);
            }
        } catch (Exception e) {
            log.error("EasyStack浮动IP数据异常：{}", e);
        }
        log.info("end sync NetFloatingIP !");

        log.info("starting sync Firewall......");
        try {
            List<? extends Firewall> firewalls = this.getClient().networking().firewalls().firewall().list();
            if (!ListTool.isEmpty(firewalls)) {
                BeanConvertor<FirewallModel, Firewall> convertFirewall = new FirewallConvertor();
                model.setFirewallModels(convertFirewall.convertList(firewalls));
            }
            List<? extends FirewallPolicy> policys = this.getClient().networking().firewalls().firewallpolicy().list();
            if (!ListTool.isEmpty(policys)) {
                BeanConvertor<FirewallPolicyModel, FirewallPolicy> convertPolicy = new FirewallPolicyConvertor();
                model.setFirewallPolicyModels(convertPolicy.convertList(policys));
            }
            List<? extends FirewallRule> rules = this.getClient().networking().firewalls().firewallrule().list();
            if (!ListTool.isEmpty(rules)) {
                BeanConvertor<FirewallRuleModel, FirewallRule> convertRule = new FirewallRuleConvertor();
                model.setFirewallRuleModels(convertRule.convertList(rules));
            }
        } catch (Exception e) {
            log.error("EasyStack防火墙数据异常：{}", e);
        }
        log.info("end sync Firewall !");

        log.info("starting sync LbListenerModel......");
        try {
            Map<String, LbListenerModel> listenerMap = new HashMap<>();
            List<? extends ListenerV2> listeners = this.getClient().networking().lbaasV2().listener().list();
            if (!ListTool.isEmpty(listeners)) {
                BeanConvertor<LbListenerModel, ListenerV2> convertListener = new LbListenerConvertor();
                List<LbListenerModel> listenerModels = convertListener.convertList(listeners);
                BeanConvertor<LbPoolModel, LbPoolV2> convertLbPool = new LbPoolConvertor();
                BeanConvertor<HealthMonitorModel, HealthMonitorV2> convertHm = new HealthMonitorConvertor();
                for (LbListenerModel listenerModel : listenerModels) {
                    String poolUuid = listenerModel.getDefaultPoolUuid();
                    if (StringUtils.isEmpty(poolUuid)) {
                        listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
                        continue;
                    }
                    LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().get(poolUuid);
                    LbPoolModel poolModel = convertLbPool.convertModel(pool);
                    if (!StringUtils.isEmpty(poolModel.getHmUuid())) {
                        HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor()
                                .get(poolModel.getHmUuid());
                        HealthMonitorModel monitorModel = convertHm.convertModel(monitor);
                        poolModel.setModel(monitorModel);
                    }
                    listenerModel.setPoolModel(poolModel);
                    List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool()
                            .listMembers(poolUuid);
                    if (!ListTool.isEmpty(members)) {
                        BeanConvertor<LbMemberModel, MemberV2> convertMember = new LbMemberConvertor();
                        List<LbMemberModel> memberModels = convertMember.convertList(members);
                        for (LbMemberModel memberModel : memberModels) {
                            memberModel.setPoolUuid(poolUuid);
                        }
                        listenerModel.setModels(memberModels);
                    }
                    listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
                }
            }
            // 获取负责均衡相关对象
            List<? extends LoadBalancerV2> lbs = this.getClient().networking().lbaasV2().loadbalancer().list();
            if (!ListTool.isEmpty(lbs)) {
                BeanConvertor<LbModel, LoadBalancerV2> convertLb = new LbConvertor();
                List<LbModel> lbModels = convertLb.convertList(lbs);
                for (LbModel lbModel : lbModels) {
                    List<LbListenerModel> listenerModels = new ArrayList<>();
                    List<String> list = lbModel.getListeners();
                    for (String uuid : list) {
                        listenerModels.add(listenerMap.get(uuid));
                    }
                    lbModel.setModels(listenerModels);
                }
                model.setLbModels(lbModels);
            }
        } catch (Exception e) {
            log.error("EasyStack负载均衡器数据异常：{}", e);
        }
        log.info("end sync LbListenerModel !");

        log.info("starting sync HostResource......");
        try {
            // hostResource
            List<? extends HostResource> hostResources = this.getClient().compute().host().list();
            if (!ListTool.isEmpty(hostResources)) {
                List<ClusterModel> hostResourcesModels = new ArrayList<>();
                for (HostResource hostResource : hostResources) {
                    if ("compute".equalsIgnoreCase(hostResource.getService())) {
                        hostResourcesModels.add(hostResoucesConvertor(hostResource));
                    }
                }
                model.setHostResourcesModels(hostResourcesModels);
            }
        } catch (Exception e) {
            log.error("EasyStack主机资源数据异常：{}", e);
        }
        log.info("end sync HostResource !");


        log.info("starting sync Keypair......");
        try {
            List<? extends Keypair> keypairs = this.getClient().compute().keypairs().list();
            if (!ListTool.isEmpty(keypairs)) {
                BeanConvertor<KeypairModel, Keypair> convertKeypair = new KeypairConvertor();
                model.setKeypairModels(convertKeypair.convertList(keypairs));
            }
        } catch (Exception e) {
            log.error("EasyStack密钥数据异常：{}", e);
        }
        log.info("end sync Keypair !");

        log.info("starting sync Vpn......");
        try {
            // 同步vpn service
            EasyStackVpnProvider openStackVpnProvider = new EasyStackVpnProvider(butler);
            BocloudResult bocloudResult = openStackVpnProvider.list();
            if (bocloudResult.isSuccess()) {
                List<VpnModel> vpnModels = (List<VpnModel>) bocloudResult.getData();
                model.setVpnModels(vpnModels);
            }
            // 同步 vpn policy
            List<VpnPolicyModel> vpnPolicyModels = new ArrayList<>();
            BocloudResult vpnIkePolicyResult = openStackVpnProvider.vpnIkePolicylist();
            if (vpnIkePolicyResult.isSuccess()) {
                List<VpnPolicyModel> vpnIkePolicylist = (List<VpnPolicyModel>) vpnIkePolicyResult.getData();
                if (!ListTool.isEmpty(vpnIkePolicylist)) {
                    vpnPolicyModels.addAll(vpnIkePolicylist);
                }
            }
            BocloudResult vpnIPsecPolicyResult = openStackVpnProvider.vpnIPsecPolicylist();
            if (vpnIPsecPolicyResult.isSuccess()) {
                List<VpnPolicyModel> vpnIPsecPolicylist = (List<VpnPolicyModel>) vpnIPsecPolicyResult.getData();
                if (!ListTool.isEmpty(vpnIPsecPolicylist)) {
                    vpnPolicyModels.addAll(vpnIPsecPolicylist);
                }
            }
            model.setVpnPolicyModels(vpnPolicyModels);
            // 同步vpn connection
            EasyStackVpnConnProvider openStackVpnConnProvider = new EasyStackVpnConnProvider(butler);
            BocloudResult vpnConnResult = openStackVpnConnProvider.list();
            if (vpnConnResult.isSuccess()) {
                List<VpnConnModel> vpnConnModes = (List<VpnConnModel>) vpnConnResult.getData();
                model.setVpnConnModes(vpnConnModes);
            }
        } catch (Exception e) {
            log.error("EasyStack vpn数据异常：{}", e);
        }
        log.info("end sync Vpn !");

//todo  easystack纳管放开
        log.info("starting sync project......");
        try {
            List<OSTenantModel> osTenants = new ArrayList<>();
            if (Version.V2 == this.getVersion()) {
                List<? extends Tenant> tenants = this.getClient2().identity().tenants().list();
                for (Tenant tenant : tenants) {
                    OSTenantModel osTenant = new OSTenantModel(tenant.getName(), tenant.getDescription(), tenant.getId(),
                            tenant.isEnabled(), null, null);
                    osTenants.add(osTenant);
                }
            } else {
                Identifier domainIdentifier = Identifier.byName(butler.getDomainName());
                Config config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(new SSLContextBuilder().build())
                        .withReadTimeout(50 * 1000 * 1000).withConnectionTimeout(50 * 1000 * 1000);
                OSClient.OSClientV3 projectsClient = OSFactory.builderV3().endpoint(butler.getUrl() + "/v3")
                        .credentials(butler.getUsername(), butler.getPassword(), domainIdentifier)
                        .scopeToDomain(Identifier.byName(butler.getDomainName())).withConfig(config)
                        .authenticate();
                List<? extends Project> projects = projectsClient.identity().projects().list();
                log.info("OS项目数量：{}", projects.size());
                for (Project project : projects) {
                    Domain domain = project.getDomain();
                    String domainName;
                    if (null != domain) {
                        domainName = domain.getName();
                    } else {
                        domainName = project.getDomainId();
                    }
                    OSTenantModel tenant = new OSTenantModel(project.getName(), project.getDescription(), project.getId(),
                            project.isEnabled(), project.getDomainId(), domainName);
                    osTenants.add(tenant);
                }
            }
            List<OSTenantModel> newOsTenants = new ArrayList<>();
            for (OSTenantModel osTenantModel : osTenants) {
                if (!osTenantModel.getName().equalsIgnoreCase("service")) {
                    newOsTenants.add(osTenantModel);
                }
            }
            model.setTenants(newOsTenants);
        } catch (Exception e) {
            log.error("获取EasyStack租户数据异常：{}", e);
        }
        log.info("end sync project !");
        return new BocloudResult(true, model, "success");
    }

    public ClusterModel hostResoucesConvertor(HostResource hostResource) {
        ClusterModel model = new ClusterModel();
        if (null != hostResource) {
            model.setName(hostResource.getHostName());
            model.setCpu(hostResource.getCpu());
            model.setDisk(hostResource.getDiskInGb());
            model.setMemory(hostResource.getMemoryInMb());
            model.setService(hostResource.getService());
            model.setZone(hostResource.getZone());
        }
        return model;
    }

    @Override
    public BocloudResult syncSecurityGroups() {
        List<? extends SecurityGroup> sGroups = this.getClient().networking().securitygroup().list();
        List<SecurityGroupModel> groupModels = new ArrayList<>();
        if (!ListTool.isEmpty(sGroups)) {
            BeanConvertor<SecurityGroupModel, SecurityGroup> convertGroup = new SecurityGroupConvertor();
            groupModels = convertGroup.convertList(sGroups);
        }
        return new BocloudResult(true, groupModels, "查询成功");
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        OSSyncModel model = new OSSyncModel();
        List<? extends Port> ports = this.getClient().networking().port().list();
        if (null != ports && !ports.isEmpty()) {
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            List<PortModel> portModels = convertPort.convertList(ports);
            List<PortModel> result = new ArrayList<>();
            if (portModels != null && portModels.size() > 0) {
                for (PortModel portModel : portModels) {
                    if (portModel.getNetworkUuid().contains(networkUuid)) {
                        result.add(portModel);
                    }
                }
            }
            model.setPortModels(result);
        }

        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult getHostModel(HostModel hostModel) {
        return null;
    }

    private ClusterModel convertAggregate(HostAggregate aggregate) {
        ClusterModel clusterModel = new ClusterModel();
        clusterModel.setName(aggregate.getName());
        clusterModel.setRegion(aggregate.getAvailabilityZone());
        clusterModel.setUuid(aggregate.getId());
        clusterModel.setHostNames(aggregate.getHosts());
        Map<String, String> meteDataMap = aggregate.getMetadata();
        if (null != meteDataMap) {
            clusterModel.setRemark(JSONObject.toJSONString(aggregate.getMetadata()));
        }
        return clusterModel;
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncStack(LbListenerModel lbListenerModel) {
        try {
            OSSyncModel model = new OSSyncModel();
            Map<String, LbListenerModel> listenerMap = new HashMap<>();
            String poolUuid = lbListenerModel.getDefaultPoolUuid();
            String lbUuid = lbListenerModel.getLbUuid();
            String listenerUuid = lbListenerModel.getListenerUuid();
            BeanConvertor<LbPoolModel, LbPoolV2> convertLbPool = new LbPoolConvertor();
            BeanConvertor<HealthMonitorModel, HealthMonitorV2> convertHm = new HealthMonitorConvertor();
            ListenerV2 listenerV2 = this.getClient().networking().lbaasV2().listener().get(listenerUuid);
            BeanConvertor<LbListenerModel, ListenerV2> convertListener = new LbListenerConvertor();
            LbListenerModel listenerModel = convertListener.convertModel(listenerV2);
            LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().get(poolUuid);
            LbPoolModel poolModel = convertLbPool.convertModel(pool);
            if (!StringUtils.isEmpty(poolModel.getHmUuid())) {
                HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor()
                        .get(poolModel.getHmUuid());
                HealthMonitorModel monitorModel = convertHm.convertModel(monitor);
                poolModel.setModel(monitorModel);
            }
            listenerModel.setPoolModel(poolModel);
            List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool().listMembers(poolUuid); // poolUuid
            if (!ListTool.isEmpty(members)) {
                BeanConvertor<LbMemberModel, MemberV2> convertMember = new LbMemberConvertor();
                List<LbMemberModel> memberModels = convertMember.convertList(members);
                for (LbMemberModel memberModel : memberModels) {
                    memberModel.setPoolUuid(poolUuid);
                }
                listenerModel.setModels(memberModels);
            }
            listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
            LoadBalancerV2 loadBalancerV2 = this.getClient().networking().lbaasV2().loadbalancer().get(lbUuid);
            BeanConvertor<LbModel, LoadBalancerV2> convertLb = new LbConvertor();
            LbModel lbModels = convertLb.convertModel(loadBalancerV2);
            List<LbListenerModel> listenerModels = new ArrayList<>();
            List<String> list = lbModels.getListeners();
            for (String uuid : list) {
                listenerModels.add(listenerMap.get(uuid));
            }
            lbModels.setModels(listenerModels);
            List<LbModel> lbList = new ArrayList<>();
            lbList.add(lbModels);
            model.setLbModels(lbList);

            String vipPortUuid = lbModels.getVipPortUuid();
            if (!StringUtils.isEmpty(vipPortUuid)) {
                Port port = this.getClient().networking().port().get(vipPortUuid);
                BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                List<PortModel> portModels = new ArrayList<>();
                portModels.add(convertPort.convertModel(port));
                model.setPortModels(portModels);
            }
            return new BocloudResult(true, model, "success");
        } catch (Exception e) {
            return new BocloudResult(false, "伸缩组同步负载均衡异常：" + e.getMessage());
        }
    }

}
