package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.monitor.AlarmProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.openstack4j.model.compute.QuotaSet;
import org.openstack4j.model.compute.QuotaSetUpdate;
import org.openstack4j.model.network.NetQuota;
import org.openstack4j.model.storage.block.BlockQuotaSet;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: EasyStackQuotaProvider
 * @Description: 配额
 * @Author lxy
 * @Date 2022/1/4 16:09
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackQuotaProvider extends EasyStackProvider {

    public EasyStackQuotaProvider(Butler butler) {
        super(butler);
    }

    public BocloudResult quota(String id) {
        try {
            QuotaSet quotaSet = this.getClient().compute().quotaSets().get(id);
            BlockQuotaSet blockQuotaSet = this.getClient().blockStorage().quotaSets().get(id);
            NetQuota netQuota = this.getClient().networking().quotas().get(id);
            List<Object> list = new ArrayList<>();
            list.add(quotaSet);
            list.add(blockQuotaSet);
            list.add(netQuota);
            return new BocloudResult(true, list, "1");
        } catch (Exception e) {
            e.printStackTrace();
            return new BocloudResult(false, e.getMessage(), "1");
        }
    }

    public BocloudResult quotaupdate(String tenantId, QuotaSetUpdate qs, BlockQuotaSet bq, NetQuota nq) {
        try {
            QuotaSet quotaSet = this.getClient().compute().quotaSets().updateForTenant(tenantId,qs);
            BlockQuotaSet blockQuotaSet = this.getClient().blockStorage().quotaSets().updateForTenant(tenantId, bq);
            NetQuota netQuota = this.getClient().networking().quotas().updateForTenant(tenantId, nq);
            List<Object> list = new ArrayList<>();
            list.add(quotaSet);
            list.add(blockQuotaSet);
            list.add(netQuota);
            return new BocloudResult(true, list, "1");
        } catch (Exception e) {
            e.printStackTrace();
            return new BocloudResult(false, e.getMessage(), "1");
        }
    }
}
