package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.HypervisorConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.openstack4j.model.compute.HostResource;
import org.openstack4j.model.compute.ext.Hypervisor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * easystack宿主机操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackHyperVProvider extends EasyStackProvider implements HostProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackHyperVProvider.class);

    public EasyStackHyperVProvider(Butler butler) {
        super(butler);
    }

    /**
     * 计算主机删除
     */
    @Override
    public BocloudResult remove(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult enterMode(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult exitMode(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult disconn(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult reconn(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult moveTo(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult shutdown(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult reboot(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult detail(HostOperateModel hostOperateModel) {
        return null;
    }

    @Override
    public BocloudResult listHosts() {
        try {
            List<ClusterModel> hostResourcesModels = new ArrayList<>();
            List<? extends HostResource> hostResources = this.getClient().compute().host().list();
            if (!ListTool.isEmpty(hostResources)) {
                for (HostResource hostResource : hostResources) {
                    if ("compute".equalsIgnoreCase(hostResource.getService())) {
                        hostResourcesModels.add(hostResoucesConvertor(hostResource));
                    }
                }
            }
            return new BocloudResult(true, hostResourcesModels, "同步成功！");
        } catch (Exception e) {
            logger.error("同步host异常：" + e.getMessage());
            return new BocloudResult(false, "同步host异常：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        List<? extends Hypervisor> hyperVs = this.getClient().compute().hypervisors().list();
        if (null == hyperVs || hyperVs.isEmpty()) {
            hyperVs = new ArrayList<>();
        }
        BeanConvertor<HostModel, Hypervisor> convertHyperV = new HypervisorConvertor();
        return new BocloudResult(true, convertHyperV.convertList(hyperVs), "success");
    }

    @Override
    public BocloudResult monitor() {
        return null;
    }

    @Override
    public BocloudResult listPerf() {
        List<HostPerfModel> hostPerfModel = new ArrayList<>();
        List<? extends Hypervisor> hypervs = this.getClient().compute().hypervisors().list();
        for (Hypervisor hyper : hypervs) {
            List<? extends HostResource> hostResources = this.getClient().compute().host()
                    .hostDescribe(hyper.getHypervisorHostname());
            hostPerfModel.add(convert(hostResources));
        }
        return new BocloudResult(true, hostPerfModel, "success");
    }

    @Override
    public BocloudResult summary(Long aLong) {
        return null;
    }

    @Override
    public BocloudResult detail(Long aLong) {
        return null;
    }

    @Override
    public BocloudResult list(String s) {
        return null;
    }

    /**
     * 转化参数
     *
     * @param hostResources
     * @return
     */
    private HostPerfModel convert(List<? extends HostResource> hostResources) {
        HostPerfModel hostPerfModel = new HostPerfModel();
        if (null == hostResources) {
            return null;
        }
        for (HostResource hostResource : hostResources) {
            if ("(total)".equalsIgnoreCase(hostResource.getProject())) {
                hostPerfModel.setCpuTotal(hostResource.getCpu());
                hostPerfModel.setDiskTotal(hostResource.getDiskInGb());
                hostPerfModel.setMemTotal(hostResource.getMemoryInMb());
                hostPerfModel.setHostName(hostResource.getHost());
            } else if ("(used_now)".equalsIgnoreCase(hostResource.getProject())) {
                hostPerfModel.setCpuUsed(hostResource.getCpu());
                hostPerfModel.setDiskUsed(hostResource.getDiskInGb());
                hostPerfModel.setMemUsed(hostResource.getMemoryInMb());
            }
            hostPerfModel.setGmtCreate(new Date());
        }
        return hostPerfModel;
    }

    public ClusterModel hostResoucesConvertor(HostResource hostResource) {
        ClusterModel model = new ClusterModel();
        if (null != hostResource) {
            model.setName(hostResource.getHostName());
            model.setCpu(hostResource.getCpu());
            model.setDisk(hostResource.getDiskInGb());
            model.setMemory(hostResource.getMemoryInMb());
            model.setService(hostResource.getService());
            model.setZone(hostResource.getZone());
        }
        return model;
    }
}
