package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotBackupModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.storage.block.VolumeBackup;
import org.openstack4j.model.storage.block.VolumeBackupCreate;
import org.openstack4j.model.storage.block.VolumeBackupRestore;
import org.openstack4j.openstack.storage.block.domain.CinderVolumeBackupCreate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack快照备份操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackSnapBackupProvider extends EasyStackProvider implements SnapshotBackupProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackSnapBackupProvider.class);

    public EasyStackSnapBackupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<SnapshotBackupModel> models = new ArrayList<>();
        List<? extends VolumeBackup> backups = new ArrayList<>();
        backups = this.getClient().blockStorage().backups().list();
        if (null != backups) {
            for (VolumeBackup snapshot : backups) {
                models.add(convert(snapshot));
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(SnapshotBackupModel snapshotBackup) {
        VolumeBackupCreate vbc = CinderVolumeBackupCreate.builder().snapshotId(snapshotBackup.getSnapshotUuid())
                .name(snapshotBackup.getName()).volumeId(snapshotBackup.getVolumeUuid()).force(true).build();
        VolumeBackup volumeBackup = this.getClient().blockStorage().backups().create(vbc);
        return new BocloudResult(true, convert(volumeBackup), "success");
    }

    @Override
    public BocloudResult remove(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult remove(SnapshotBackupModel model) {
        ActionResponse response = null;
        response = this.getClient().blockStorage().backups().delete(model.getBackupUuid());
        if (response.isSuccess()) {
            return new BocloudResult(true, "删除成功!");
        } else {
            logger.error("remove snapshot [{}] error: {}", model.getSnapshotUuid(), response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult restore(SnapshotBackupModel snapshotBackupModel) {
        try {
            VolumeBackupRestore restore = this.getClient().blockStorage().backups().restore(snapshotBackupModel.getBackupUuid(), null,
                    snapshotBackupModel.getVolumeUuid());
            return new BocloudResult(true, restore, "success");
        } catch (Exception e) {
            return new BocloudResult(false, "restore failed");
        }
    }

    private SnapshotBackupModel convert(VolumeBackup backup) {
        if (null == backup) {
            return null;
        }
        SnapshotBackupModel model = new SnapshotBackupModel();
        model.setCreateTime(backup.getCreated());
        model.setName(backup.getName());
        model.setSize(backup.getSize());
        model.setSnapshotUuid(backup.getSnapshotId());
        model.setBackupUuid(backup.getId());
        model.setVolumeUuid(backup.getVolumeId());
        model.setStatus(backup.getStatus() != null ? backup.getStatus().name() : null);
        return model;
    }

    @Override
    public BocloudResult detail(SnapshotBackupModel model) {
        VolumeBackup snapshot = this.getClient().blockStorage().backups().get(model.getBackupUuid());
        if (null == snapshot) {
            logger.error("snapshot [{}] doesnot exsit!", model.getSnapshotUuid());
            return new BocloudResult(false, "快照不存在！");
        }
        return new BocloudResult(true, convert(snapshot), "success");
    }

}
