package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ImageModel;
import org.openstack4j.model.compute.Image;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * easystack镜像(compute)数据转换器
 */
public class ComputerImageConvertor implements BeanConvertor<ImageModel, Image> {

    @Override
    public List<ImageModel> convertList(List<? extends Image> images) {
        List<ImageModel> imageModels = new ArrayList<>();
        for (Image image : images) {
            imageModels.add(convertModel(image));
        }
        return imageModels;
    }

    @Override
    public ImageModel convertModel(Image image) {
        BigDecimal size = new BigDecimal(0);
        ImageModel imageModel = new ImageModel();
        imageModel.setGmtCreate(image.getCreated());
        imageModel.setImageUuid(image.getId());
        imageModel.setMinDisk(Long.valueOf(image.getMinDisk()));
        imageModel.setName(image.getName());
        if (0 != image.getSize()) {
            size = new BigDecimal(image.getSize()).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP);
        }
        imageModel.setSize(size.doubleValue());
        imageModel.setStatus(image.getStatus().name());
        Map<String, Object> propMap = image.getMetaData();
        if (null != propMap) {
            imageModel.setQgaInstalled(false);
            if (propMap.containsKey("hw_qemu_guest_agent") && "yes".equals(propMap.get("hw_qemu_guest_agent"))) {
                imageModel.setQgaInstalled(true);
            }
            String key = "oscategory";
            if (propMap.containsKey(key)) {
                imageModel.setOsCategory(null == propMap.get(key) ? null : propMap.get(key).toString());
            }
            key = "osname";
            if (propMap.containsKey(key)) {
                imageModel.setOsName(null == propMap.get(key) ? null : propMap.get(key).toString());
            }
            key = "osversion";
            if (propMap.containsKey(key)) {
                imageModel.setOsVersion(null == propMap.get(key) ? null : propMap.get(key).toString());
            }
            key = "default_user";
            if (propMap.containsKey(key)) {
                imageModel.setDefaultUser(null == propMap.get(key) ? null : propMap.get(key).toString());
            }
        }
        return imageModel;
    }
}
