package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ImageModel;
import org.openstack4j.model.image.Image;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * easystack镜像数据转换器
 */
public class ImageConvertor implements BeanConvertor<ImageModel, Image> {

    @Override
    public List<ImageModel> convertList(List<? extends Image> images) {
        List<ImageModel> imageModels = new ArrayList<>();
        for (Image image : images) {
            imageModels.add(convertModel(image));
        }
        return imageModels;
    }

    @Override
    public ImageModel convertModel(Image image) {
        if (null == image) {
            return null;
        }
        BigDecimal size = new BigDecimal(0);// edit by ouyangwulin
        ImageModel model = new ImageModel();
        model.setGmtCreate(image.getCreatedAt());
        model.setImageUuid(image.getId());
        model.setMinDisk(image.getMinDisk());
        model.setName(image.getName());
        if (null != image.getSize()) {
            size = new BigDecimal(image.getSize()).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP);
        }
        model.setSize(size.doubleValue());
        if (null != image.getStatus()) {
            model.setStatus(image.getStatus().name());
        }
        Map<String, String> propMap = image.getProperties();
        if (null != propMap) {
            if (propMap.containsKey("hw_qemu_guest_agent")) {
                if ("yes".equals(propMap.get("hw_qemu_guest_agent"))) {
                    model.setQgaInstalled(true);
                } else {
                    model.setQgaInstalled(false);
                }
            } else {
                model.setQgaInstalled(false);
            }
            if (propMap.containsKey("oscategory")) {
                model.setOsCategory(propMap.get("oscategory"));
            }
            if (propMap.containsKey("osname")) {
                model.setOsName(propMap.get("osname"));
            }
            if (propMap.containsKey("osversion")) {
                model.setOsVersion(propMap.get("osversion"));
            }
            if (propMap.containsKey("default_user")) {
                model.setDefaultUser(propMap.get("default_user"));
            }
            model.setOsType(image.getDiskFormat().name());
        }
        return model;
    }

}
