package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.utils.StatusConverter;
import com.bocloud.common.utils.ListTool;
import org.openstack4j.model.network.ext.LoadBalancerV2;
import org.openstack4j.openstack.networking.domain.ext.ListItem;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack负载均衡数据转换器
 */
public class LbConvertor implements BeanConvertor<LbModel, LoadBalancerV2> {

    @Override
    public List<LbModel> convertList(List<? extends LoadBalancerV2> loadBalancerV2s) {
        List<LbModel> models = new ArrayList<>();
        for (LoadBalancerV2 balancer : loadBalancerV2s) {
            models.add(convertModel(balancer));
        }
        return models;
    }

    @Override
    public LbModel convertModel(LoadBalancerV2 balancer) {
        LbModel model = new LbModel();
        model.setLbUuid(balancer.getId());
        model.setName(balancer.getName());
        model.setRemark(balancer.getDescription());
        model.setTenantUuid(balancer.getTenantId());
        model.setSubnetUuid(balancer.getVipSubnetId());
        model.setVipAddress(balancer.getVipAddress());
        model.setVipPortUuid(balancer.getVipPortId());
        model.setAdminStateUp(balancer.isAdminStateUp());
        List<ListItem> items = balancer.getListeners();
        List<String> listeners = new ArrayList<>();
        if (!ListTool.isEmpty(items)) {
            for (ListItem item : items) {
                listeners.add(item.getId());
            }
        }
        model.setListeners(listeners);
        model.setProvisioningStatus(StatusConverter.Network.converter(balancer.getProvisioningStatus().name()));
        model.setOperatingStatus(balancer.getOperatingStatus().name());
        model.setProvider(balancer.getProvider());
        return model;
    }
}
