package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.utils.StatusConverter;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.network.IP;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.State;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * easystack端口数据转换器
 */
public class PortConvertor implements BeanConvertor<PortModel, Port> {

    @Override
    public List<PortModel> convertList(List<? extends Port> models) {
        List<PortModel> portModels = new ArrayList<>();
        for (Port port : models) {
            State state = port.getState();
            if (State.ACTIVE == state || State.DOWN == state || State.BUILD == state || State.ERROR == state || State.UNRECOGNIZED == state) {
                portModels.add(convertModel(port));
            }
        }
        return portModels;
    }

    @Override
    public PortModel convertModel(Port port) {
        PortModel portModel = new PortModel();
        if (null != port) {
            String name = port.getName();
            String portId = port.getId();
            // 如果name，取ID的前12位
            if (StringUtils.isEmpty(name)) {
                name = portId.substring(0, 13);
            }
            Set<? extends IP> fixedIps = port.getFixedIps();
            List<PortFixedIpModel> portFixedIpModels = new ArrayList<PortFixedIpModel>();
            // 固定ip的处理
            for (IP fixedIp : fixedIps) {
                portFixedIpModels.add(new PortFixedIpModel(fixedIp.getIpAddress(), fixedIp.getSubnetId()));
            }
            portModel = new PortModel(name, StatusConverter.Network.converter(port.getState().name()),
                    port.getDeviceId(), port.getDeviceOwner(), port.getNetworkId(), port.getMacAddress(), portId,
                    portFixedIpModels);
            portModel.setTenantUuid(port.getTenantId());
        }
        return portModel;
    }

}
