package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ImageModel;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.image.v2.Image;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

/**
 * easystack镜像(v2)数据转换器
 */
public class V2ImageConvertor implements BeanConvertor<ImageModel, Image> {

    @Override
    public List<ImageModel> convertList(List<? extends Image> images) {
        List<ImageModel> imageModels = new ArrayList<>();
        for (Image image : images) {
            imageModels.add(convertModel(image));
        }
        return imageModels;
    }

    @Override
    public ImageModel convertModel(Image image) {
        if (null == image) {
            return null;
        }
        BigDecimal size = new BigDecimal(0);
        ImageModel model = new ImageModel();
        model.setTenantUuid(image.getOwner());
        model.setImageType(image.getVisibility().name());
        model.setGmtCreate(image.getCreatedAt());
        model.setImageUuid(image.getId());
        model.setMinDisk(image.getMinDisk());
        model.setName(image.getName());
        if (null != image.getSize()) {
            size = new BigDecimal(image.getSize()).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP);
        }
        model.setSize(size.doubleValue());
        if (null != image.getStatus()) {
            model.setStatus(image.getStatus().name());
        }
        if (null != image.getDiskFormat()) {
            model.setOsType(image.getDiskFormat().name());
        }
        model.setOsCategory(image.getAdditionalPropertyValue("oscategory"));
        model.setOsName(image.getAdditionalPropertyValue("osname"));
        model.setOsVersion(image.getAdditionalPropertyValue("osversion"));
        model.setDefaultUser(image.getAdditionalPropertyValue("default_user"));
        String agent = image.getAdditionalPropertyValue("hw_qemu_guest_agent");
        if (StringUtils.isEmpty(agent)) {
            model.setQgaInstalled(false);
        } else if ("yes".equals(agent)) {
            model.setQgaInstalled(true);
        } else if ("no".equals(agent)) {
            model.setQgaInstalled(false);
        }
        return model;
    }

}
