package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeAttachment;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack存储卷数据转换器
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, Volume> {

    @Override
    public List<VolumeModel> convertList(List<? extends Volume> volumes) {
        List<VolumeModel> models = new ArrayList<>();
        for (Volume volume : volumes) {
            models.add(convertModel(volume));
        }
        return models;
    }

    @Override
    public VolumeModel convertModel(Volume volume) {
        VolumeModel model = new VolumeModel();
        model.setCategory(volume.getVolumeType());
        model.setCreateTime(volume.getCreated());
        model.setImageUuid(volume.getImageRef());
        String name = volume.getName();
        if (StringUtils.isEmpty(name)) {
            name = volume.getDisplayName();
        }
        if (StringUtils.isEmpty(name)) {
            name = volume.getId();
        }
        String remark = volume.getDescription();
        if (StringUtils.isEmpty(remark)) {
            remark = volume.getDisplayDescription();
        }
        model.setRemark(remark);
        model.setName(name);
        model.setSize(volume.getSize());
        model.setStatus(volume.getStatus().name());
        model.setType(volume.getVolumeType());
        model.setVolumeUuid(volume.getId());
        model.setVolumeType(volume.getVolumeType());
        model.setZone(volume.getZone());
        model.setBootVolume(volume.bootable());
        model.setTenantUuid(volume.getTenantId());
        List<ServerVolumeModel> models = new ArrayList<>();
        List<? extends VolumeAttachment> attachments = volume.getAttachments();
        if (!ListTool.isEmpty(attachments)) {
            for (VolumeAttachment attachment : attachments) {
                models.add(new ServerVolumeModel(attachment.getServerId(), attachment.getVolumeId(),
                        attachment.getDevice()));
            }
        }
        model.setModels(models);
        return model;
    }
}
