package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PortConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.network.Port;
import org.openstack4j.model.network.builder.PortBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack端口操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackPortProvider extends EasyStackProvider implements PortProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackPortProvider.class);

    public EasyStackPortProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        List<PortModel> models = new ArrayList<>();
        List<? extends Port> ports = this.getClient().networking().port().list();
        if (null != ports) {
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            models = convertPort.convertList(ports);
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(PortModel model) {
        PortBuilder portbuilder = Builders.port().name(model.getName()).networkId(model.getNetworkUuid());
        List<PortFixedIpModel> fixedIpModels = model.getPortFixedIps();
        if (!ListTool.isEmpty(fixedIpModels)) {
            for (PortFixedIpModel portFixedIpModel : fixedIpModels) {
                String ipAddress = portFixedIpModel.getIpAddress();
                String subnetUuid = portFixedIpModel.getSubnetUuid();
                if (StringUtils.isEmpty(ipAddress)) {
                    ipAddress = null;
                }
                portbuilder.fixedIp(ipAddress, subnetUuid);
            }
        }
        Port port = portbuilder.build();
        try {
            port = this.getClient().networking().port().create(port);
            if (null != port) {
                BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
                model = convertPort.convertModel(port);
                return new BocloudResult(true, model, "创建成功!");
            }
            return new BocloudResult(false, null, "创建失败!");
        } catch (Exception e) {
            return new BocloudResult(false, null, "创建失败:" + e.getMessage());
        }

    }

    @Override
    public BocloudResult modify(PortModel model) {
        // 构建port
        PortBuilder portbuilder = Builders.port().name(model.getName()).networkId(model.getNetworkUuid());
        if (model.getPortFixedIps() != null && model.getPortFixedIps().size() > 0) {
            for (PortFixedIpModel portFixedIpModel : model.getPortFixedIps()) {
                portbuilder.fixedIp(portFixedIpModel.getIpAddress(), portFixedIpModel.getSubnetUuid());
            }
        }
        Port port = portbuilder.build();
        port.setId(model.getPortUuid());
        port = this.getClient().networking().port().update(port);
        if (null != port) {
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            model = convertPort.convertModel(port);
            return new BocloudResult(true, model, "修改成功!");
        }
        return new BocloudResult(false, null, "修改失败!");
    }

    @Override
    public BocloudResult remove(String portId) {
        Port port = this.getClient().networking().port().get(portId);
        if (null == port) {
            return new BocloudResult(true, "port not exist");
        }
        ActionResponse response = this.getClient().networking().port().delete(portId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "端口删除成功！");
        } else {
            logger.error("remove port [{}] failed :{}", portId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult detail(String portId) {
        Port port = this.getClient().networking().port().get(portId);
        if (null != port) {
            BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
            PortModel portModel = convertPort.convertModel(port);
            return new BocloudResult(true, portModel, "查询成功!");
        } else {
            logger.error("get port error");
            return new BocloudResult(false, "查询失败!");
        }
    }

}
