package com.bocloud.cmp.provider.network;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpnModel;
import com.bocloud.cmp.model.VpnPolicyModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VpnConvertor;
import com.bocloud.cmp.provider.convertor.VpnPolicyConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * easystack vpn
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackVpnProvider extends EasyStackProvider implements VpnProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackVpnProvider.class);

    private BeanConvertor<VpnModel, JSONObject> vpnConvertor = new VpnConvertor();

    private BeanConvertor<VpnPolicyModel, JSONObject> policyConvertor = new VpnPolicyConvertor();

    public EasyStackVpnProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/v2.0/vpn/vpnservices", null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("vpnservices"), JSONObject.class);
            List<VpnModel> convertList = vpnConvertor.convertList(list);
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult vpnIkePolicylist() {
        try {
            Result result = this.doGet("/v2.0/vpn/ikepolicies", null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("ikepolicies"), JSONObject.class);
            List<VpnPolicyModel> convertList = policyConvertor.convertList(list);
            if (null != convertList && convertList.size() > 0) {
                for (VpnPolicyModel vpnPolicyModel : convertList) {
                    vpnPolicyModel.setCategory("IKE");
                }
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult vpnIPsecPolicylist() {
        try {
            Result result = this.doGet("/v2.0/vpn/ipsecpolicies", null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            List<JSONObject> list = JSONArray.parseArray(object.getString("ipsecpolicies"), JSONObject.class);
            List<VpnPolicyModel> convertList = policyConvertor.convertList(list);
            if (null != convertList && convertList.size() > 0) {
                for (VpnPolicyModel vpnPolicyModel : convertList) {
                    vpnPolicyModel.setCategory("IPSEC");
                }
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String vpnId) {
        try {
            Result result = this.doGet("/v2.0/vpn/vpnservices/" + vpnId, null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "数据不存在");
            }
            VpnModel convertModel = vpnConvertor.convertModel(JSONObject.parseObject(object.getString("vpnservice")));
            return new BocloudResult(true, convertModel, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(VpnModel model) {
        try {
            JSONObject param = new JSONObject();
            param.put("subnet_id", model.getSubnetUuid());
            param.put("router_id", model.getRouterUuid());
            param.put("description", model.getRemark());
            param.put("name", model.getName());
            param.put("admin_state_up", model.getAdminStateUp());
            param.put("flavor_id", null);
            if (!StringUtils.isEmpty(model.getVpnType())) {
                param.put("vpn_type", model.getVpnType());
            }
            JSONObject vpnservice = new JSONObject();
            vpnservice.put("vpnservice", param);
            Result result = this.doPost("/v2.0/vpn/vpnservices", vpnservice, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            VpnModel convertModel = vpnConvertor.convertModel(JSONObject.parseObject(object.getString("vpnservice")));
            return new BocloudResult(true, getActive(convertModel.getVpnUuid()).getData(), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    private BocloudResult getActive(String vpnUuid) {
        BocloudResult detail = new BocloudResult(false, "查询失败！");
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            detail = this.detail(vpnUuid);
            if (detail.isFailed()) {
                flag = false;
                break;
            }
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            VpnModel convertModel = (VpnModel) detail.getData();
            if (null != convertModel && ("ACTIVE".equalsIgnoreCase(convertModel.getStatus()) || "PENDING_CREATE".equalsIgnoreCase(convertModel.getStatus()))) {
                flag = false;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待60s
            if (endTime - startTime > 60 * 1000 * 50) {
                flag = false;
                break;
            }
        }
        return detail;
    }

    @Override
    public BocloudResult delete(String vpnUuid) {
        try {
            BocloudResult bocloudResult = this.detail(vpnUuid);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            Result result = this.doDelete("/v2.0/vpn/vpnservices/" + vpnUuid, null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult modify(VpnModel model) {
        return new BocloudResult(true, "修改成功！");
    }

    @Override
    public BocloudResult createIkePolicy(VpnPolicyModel vpnPolicyModel) {
        try {
            JSONObject param = new JSONObject();
            param.put("auth_algorithm", vpnPolicyModel.getAuthorizationAlgorithm());
            param.put("phase1_negotiation_mode", vpnPolicyModel.getMode());
            param.put("encryption_algorithm", vpnPolicyModel.getEncryptionAlgorithm());
            param.put("pfs", vpnPolicyModel.getGroupSecret());
            param.put("ike_version",
                    vpnPolicyModel.getVersion() == null ? null : vpnPolicyModel.getVersion().toLowerCase());
            param.put("name", vpnPolicyModel.getName());
            param.put("description", vpnPolicyModel.getRemark());
            JSONObject lifetime = new JSONObject();
            lifetime.put("units", vpnPolicyModel.getKeyUnit().toLowerCase());
            lifetime.put("value", vpnPolicyModel.getKeyValue());
            param.put("lifetime", lifetime);
            JSONObject ikepolicy = new JSONObject();
            ikepolicy.put("ikepolicy", param);
            Result result = this.doPost("/v2.0/vpn/ikepolicies", ikepolicy, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            VpnPolicyModel ikePolicyModel = policyConvertor
                    .convertModel(JSONObject.parseObject(object.getString("ikepolicy")));
            ikePolicyModel.setCategory("IKE");
            return new BocloudResult(true, ikePolicyModel, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modfiyIkePolicy(VpnPolicyModel vpnPolicyModel) {
        try {
            JSONObject param = new JSONObject();
            param.put("auth_algorithm", vpnPolicyModel.getAuthorizationAlgorithm());
            param.put("phase1_negotiation_mode", vpnPolicyModel.getMode());
            param.put("encryption_algorithm", vpnPolicyModel.getEncryptionAlgorithm());
            param.put("pfs", vpnPolicyModel.getGroupSecret());
            param.put("ike_version", vpnPolicyModel.getVersion());
            param.put("name", vpnPolicyModel.getName());
            param.put("description", vpnPolicyModel.getRemark());
            JSONObject lifetime = new JSONObject();
            lifetime.put("units", vpnPolicyModel.getKeyUnit());
            lifetime.put("value", vpnPolicyModel.getKeyValue());
            param.put("lifetime", lifetime);
            JSONObject ikepolicy = new JSONObject();
            ikepolicy.put("ikepolicy", param);

            Result result = this.doPut("/v2.0/vpn/ikepolicies/" + vpnPolicyModel.getPolicyUuid(), ikepolicy, "neutron",
                    "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "编辑失败");
            }
            VpnPolicyModel ikePolicyModel = policyConvertor
                    .convertModel(JSONObject.parseObject(object.getString("ikepolicy")));
            return new BocloudResult(true, ikePolicyModel, "编辑成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "编辑失败");
        }
    }

    @Override
    public BocloudResult removeIkePolicy(String policyUuid) {
        try {
            Result result = this.doDelete("/v2.0/vpn/ikepolicies/" + policyUuid, null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult createIPsecPolicy(VpnPolicyModel vpnPolicyModel) {
        try {
            JSONObject param = new JSONObject();
            param.put("name", vpnPolicyModel.getName());
            param.put("description", vpnPolicyModel.getRemark());
            param.put("transform_protocol", vpnPolicyModel.getProtocol());
            param.put("auth_algorithm", vpnPolicyModel.getAuthorizationAlgorithm());
            param.put("encapsulation_mode", vpnPolicyModel.getMode());
            param.put("encryption_algorithm", vpnPolicyModel.getEncryptionAlgorithm());
            param.put("pfs", vpnPolicyModel.getGroupSecret());

            JSONObject lifetime = new JSONObject();
            lifetime.put("units", vpnPolicyModel.getKeyUnit().toLowerCase());
            lifetime.put("value", vpnPolicyModel.getKeyValue());
            param.put("lifetime", lifetime);

            JSONObject ipsecpolicy = new JSONObject();
            ipsecpolicy.put("ipsecpolicy", param);
            Result result = this.doPost("/v2.0/vpn/ipsecpolicies", ipsecpolicy, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            VpnPolicyModel ipsecPolicyModel = policyConvertor
                    .convertModel(JSONObject.parseObject(object.getString("ipsecpolicy")));
            ipsecPolicyModel.setCategory("IPSEC");
            return new BocloudResult(true, ipsecPolicyModel, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modfiyIPsecPolicy(VpnPolicyModel vpnPolicyModel) {
        try {
            JSONObject param = new JSONObject();
            param.put("name", vpnPolicyModel.getName());
            param.put("description", vpnPolicyModel.getRemark());
            param.put("transform_protocol", vpnPolicyModel.getProtocol());
            param.put("auth_algorithm", vpnPolicyModel.getAuthorizationAlgorithm());
            param.put("encapsulation_mode", vpnPolicyModel.getMode());
            param.put("encryption_algorithm", vpnPolicyModel.getEncryptionAlgorithm());
            param.put("pfs", vpnPolicyModel.getGroupSecret());

            JSONObject lifetime = new JSONObject();
            lifetime.put("units", vpnPolicyModel.getKeyUnit().toLowerCase());
            lifetime.put("value", vpnPolicyModel.getKeyValue());
            param.put("lifetime", lifetime);

            JSONObject ipsecpolicy = new JSONObject();
            ipsecpolicy.put("ipsecpolicy", param);
            Result result = this.doPut("/v2.0/vpn/ipsecpolicies/" + vpnPolicyModel.getPolicyUuid(), ipsecpolicy,
                    "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "修改失败");
            }
            VpnPolicyModel ikePolicyModel = policyConvertor
                    .convertModel(JSONObject.parseObject(object.getString("ipsecpolicy")));
            return new BocloudResult(true, ikePolicyModel, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult removeIPsecPolicy(String policyUuid) {
        try {
            Result result = this.doDelete("/v2.0/vpn/ipsecpolicies/" + policyUuid, null, "neutron", "network");
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

}
