package com.bocloud.cmp.provider.security;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.EasyStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.KeypairConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.Keypair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * easystack密钥操作类
 */
@Provider("bocloud.cmp.easystack")
public class EasyStackKeypairProvider extends EasyStackProvider implements KeypairProvider {

    private static Logger logger = LoggerFactory.getLogger(EasyStackKeypairProvider.class);

    public EasyStackKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        List<KeypairModel> models = new ArrayList<>();
        List<? extends Keypair> keypairs = this.getClient().compute().keypairs().list();
        if (!ListTool.isEmpty(keypairs)) {
            BeanConvertor<KeypairModel, Keypair> convertKeypair = new KeypairConvertor();
            models = convertKeypair.convertList(keypairs);
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult detail(String keypairName) {
        BeanConvertor<KeypairModel, Keypair> convertKeypair = new KeypairConvertor();
        Keypair keypair = this.getClient().compute().keypairs().get(keypairName);
        if (null == keypair) {
            return new BocloudResult(false, "keypair is not exist");
        }
        return new BocloudResult(true, convertKeypair.convertModel(keypair), "success");
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        BocloudResult BocloudResult = null;
        try {
            Keypair exist = this.getClient().compute().keypairs().get(keypairName);
            if (null != exist) {
                return new BocloudResult(false, "密钥已存在");
            }
            // publicKey为null时将自动生成公钥
            Keypair result = this.getClient().compute().keypairs().create(keypairName, publicKey);
            if (null == result) {
                BocloudResult = new BocloudResult(false, "create keypair fail");
            } else {
                BeanConvertor<KeypairModel, Keypair> convertKeypair = new KeypairConvertor();
                KeypairModel model = convertKeypair.convertModel(result);
                BocloudResult = new BocloudResult(true, model, "create keypair success");
            }
            return BocloudResult;
        } catch (ClientResponseException e) {
            logger.error("创建密钥失败:{}", e);
            return new BocloudResult(false, "创建密钥失败！【" + e.getMessage() + "】");
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        Keypair keypair = this.getClient().compute().keypairs().get(keypairName);
        if (null == keypair) {
            return new BocloudResult(true, "keypair is not exist");
        }
        ActionResponse response = this.getClient().compute().keypairs().delete(keypairName);
        if (response.isSuccess()) {
            return new BocloudResult(true, "success");
        } else {
            logger.error("delete keypair fail:", response.getFault());
            return new BocloudResult(false, "delete fail");
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        // TODO Auto-generated method stub
        return null;
    }
}
