package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

/**
 * f5连接验证通用类
 *
 * @author liuyuanyuan
 */
public class F5Provider extends GenericProvider implements SuperProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5Provider.class);
    private String endpoint;// 平台认证路径
    private String username;
    private String password;
    private CloseableHttpClient client;
    private boolean accessable;
    private String error; // 错误信息

    @Override
    public boolean accessable() {
        return this.accessable;
    }

    public F5Provider(Butler butler) {
        super(butler);
        this.endpoint = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        init();
    }

    /**
     * 初始化连接
     */
    private void init() {
        if (paramCheck().isFailed()) {// 基本的参数校验不通过，即用户名，密码或者认证路径为空
            this.accessable = false;
            logger.error("Driver init error : {}", paramCheck().getMessage());
            this.error = paramCheck().getMessage();
            return;
        }
        try {
            CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
            HttpClientBuilder httpClientBuilder = HttpClientBuilder.create()
                    .setDefaultCredentialsProvider(credentialsProvider);
            // 忽略掉ssl认证
            TrustManager[] trustAllCerts = new TrustManager[1];
            TrustManager tm = new MiTM();
            trustAllCerts[0] = tm;
            SSLContext sc = SSLContext.getInstance("TLSv1.2");
            sc.init(null, trustAllCerts, null);
            HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
            httpClientBuilder.setSSLContext(sc);
            httpClientBuilder.setSSLHostnameVerifier(new NoopHostnameVerifier());
            client = httpClientBuilder.build();
            //认证
            String url = endpoint + "/mgmt/tm/";
            HttpClient httpClient = new HttpClient(30000);
            Map<String, Object> header = buildHeader();
            Result result = httpClient.get(header, null, url);
            if (result.isFailed()) {
                logger.info("f5 authentication failure : ", result.getMessage());
                accessable = false;
            } else {
                accessable = true;
            }
        } catch (Exception e) {
            logger.error("Get error message：", e);
            accessable = false;
        }
    }

    static class MiTM implements TrustManager, X509TrustManager {
        @Override
        public java.security.cert.X509Certificate[] getAcceptedIssuers() {
            return null;
        }

        public boolean isServerTrusted(X509Certificate[] certs) {
            return true;
        }

        public boolean isClientTrusted(X509Certificate[] certs) {
            return true;
        }

        @Override
        public void checkServerTrusted(X509Certificate[] certs, String authType) throws CertificateException {
            return;
        }

        @Override
        public void checkClientTrusted(X509Certificate[] certs, String authType) throws CertificateException {
            return;
        }
    }

    /**
     * 构建header
     *
     * @return
     */
    protected Map<String, Object> buildHeader() {
        Map<String, Object> header = new HashMap<>();
        header.put("accept", "application/json");
        header.put("content-type", "application/json");
        header.put("Authorization",
                "Basic " + Base64.getUrlEncoder().encodeToString((getUsername() + ":" + getPassword()).getBytes()));
        return header;
    }

    private Result paramCheck() {
        if (StringUtils.isEmpty(this.endpoint)) {
            return new Result(false, "Endpoint is Empty!");
        }
        if (StringUtils.isEmpty(this.username)) {
            return new Result(false, "Username is Empty!");
        }
        if (StringUtils.isEmpty(this.password)) {
            return new Result(false, "Password is Empty!");
        }
        return new Result(true, "success");
    }

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public boolean isAccessable() {
        return accessable;
    }

    public void setAccessable(boolean accessable) {
        this.accessable = accessable;
    }

    public CloseableHttpClient getClient() {
        return client;
    }

    public void setClient(CloseableHttpClient client) {
        this.client = client;
    }

    public String getError() {
        return error;
    }

    public void setError(String error) {
        this.error = error;
    }

}
