package com.bocloud.cmp.provider.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * F5认证操作类
 *
 * @author weiwei
 * @since 2018/12/03
 */
@Provider("bocloud.cmp.f5")
public class F5AuthProvider extends F5Provider implements AuthProvider {
	private static Logger logger = LoggerFactory.getLogger(F5AuthProvider.class);
	private Butler butler;

	public F5AuthProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult auth() {
		if (this.isAccessable()) {
			// 认证f5信息
			String url = "/mgmt/tm/";
			F5LocationProvider f5LocationProvider = new F5LocationProvider(this.butler);
			BocloudResult result = f5LocationProvider.authLogin(url);
			if (result.isFailed()) {
				logger.info("info : " + result.getMessage());
			}
			result = new BocloudResult(result.isSuccess(), result.getMessage());
			return result;
		}
		return new BocloudResult(this.isAccessable(), "认证信息失败");
	}
}
