package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.F5BasicObject;
import com.bocloud.cmp.entity.enums.F5MonitorType;
import com.bocloud.cmp.entity.enums.F5ProfileCategory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.f5.*;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.cmp.provider.f5.*;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.UnsupportedEncodingException;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.*;

/**
 * F5同步数据组装类
 *
 * @author weiwei
 * @since 2018/12/03
 */
@Provider("bocloud.cmp.f5")
public class F5LocationProvider extends F5Provider implements LocationProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5LocationProvider.class);
    private static final String basicUrl = "/mgmt/tm/ltm/";
    private F5NodeProvider f5NodeProvider = null;
    private F5ProfileProvider f5ProfileProvider = null;
    private F5PoolProvider f5PoolProvider = null;
    private F5MonitorProvider f5MonitorProvider = null;
    private F5VirtualServerProvider f5VirtualServerProvider = null;
    private F5IRulesProvider f5iRulesProvider = null;
    private F5DnsResolversProvider f5DnsResolversProvider = null;
    private F5VlansProvider f5VlansProvider = null;
    private F5AddressTranslationProvider f5AddressTranslationProvider = null;
    private F5TunnelProvider f5TunnelProvider = null;
    private F5PartitionProvider f5PartitionProvider = null;
    private F5DataGroupProvider f5DataGroupProvider = null;

    private Butler butler;

    public F5LocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        f5NodeProvider = new F5NodeProvider(butler);
        f5ProfileProvider = new F5ProfileProvider(butler);
        f5PoolProvider = new F5PoolProvider(butler);
        f5MonitorProvider = new F5MonitorProvider(butler);
        f5VirtualServerProvider = new F5VirtualServerProvider(butler);
        f5iRulesProvider = new F5IRulesProvider(butler);
        f5DnsResolversProvider = new F5DnsResolversProvider(butler);
        f5VlansProvider = new F5VlansProvider(butler);
        f5AddressTranslationProvider = new F5AddressTranslationProvider(butler);
        f5TunnelProvider = new F5TunnelProvider(butler);
        f5PartitionProvider = new F5PartitionProvider(butler);
        f5DataGroupProvider = new F5DataGroupProvider(butler);
    }

    @Override
    public BocloudResult listRegions() {
        return null;
    }

    @Override
    public BocloudResult sync() {
        F5SyncModel model = new F5SyncModel();
        //获取数据组
        BocloudResult dataGroupResult = f5DataGroupProvider.list("internal");
        logger.info("Start assembling data group data...");
        if (dataGroupResult.isFailed()) {
            logger.info("Failed to get data group data ...");
            return new BocloudResult(false, dataGroupResult.getMessage());
        }
        JSONObject dataGroupData = JSONObject.parseObject(dataGroupResult.getMessage());
        if (null != dataGroupData.getJSONArray("items")) {
            List<DataGroupModel> dataGroups = dataGroupData.getJSONArray("items")
                    .toJavaList(DataGroupModel.class);
            model.setDataGroupModels(dataGroups);
        }
        logger.info("End assembling data group data...");
        // 获取客户端分区数据
        BocloudResult partitionResult = f5PartitionProvider.list();
        logger.info("Start assembling partition data...");
        if (partitionResult.isFailed()) {
            logger.info("Failed to get partition data ...");
            return new BocloudResult(false, partitionResult.getMessage());
        }
        JSONObject partitionResultData = JSONObject.parseObject(partitionResult.getMessage());
        if (null != partitionResultData.getJSONArray("items")) {
            List<PartitionModel> partitionModels = partitionResultData.getJSONArray("items")
                    .toJavaList(PartitionModel.class);
            model.setPartitionModels(partitionModels);
        }
        logger.info("End assembling partition data...");

        // 获取客户端tunnel数据
        BocloudResult tunnelResult = f5TunnelProvider.list("tunnels/tunnel");
        logger.info("Start assembling tunnel data...");
        if (tunnelResult.isFailed()) {
            logger.info("Failed to get tunnel data ...");
            return new BocloudResult(false, tunnelResult.getMessage());
        }
        JSONObject tunnelResultData = JSONObject.parseObject(tunnelResult.getMessage());
        if (null != tunnelResultData.getJSONArray("items")) {
            List<TunnelModel> tunnelModels = tunnelResultData.getJSONArray("items").toJavaList(TunnelModel.class);
            model.setTunnelModels(tunnelModels);
        }
        logger.info("End assembling tunnel data...");

        // 获取客户端vlan数据
        BocloudResult vlanResult = f5VlansProvider.list();
        logger.info("Start assembling vlan data...");
        if (vlanResult.isFailed()) {
            logger.info("Failed to get vlan data ...");
            return new BocloudResult(false, vlanResult.getMessage());
        }
        JSONObject vlanResultData = JSONObject.parseObject(vlanResult.getMessage());
        if (null != vlanResultData.getJSONArray("items")) {
            List<VlanModel> vlanModels = vlanResultData.getJSONArray("items").toJavaList(VlanModel.class);
            model.setVlanModels(vlanModels);
        }
        logger.info("End assembling vlan data...");

        // 获取客户端地址转化数据
        // 目前先写死，时间紧，后期所有类型做成枚举类,目前只同步snatpool的数据
        BocloudResult snatPoolResult = f5AddressTranslationProvider.list("snatpool");
        logger.info("Start assembling snatPool data...");
        if (snatPoolResult.isFailed()) {
            logger.info("Failed to get snatPool data ...");
            return new BocloudResult(false, snatPoolResult.getMessage());
        }
        JSONObject snatPoolResultData = JSONObject.parseObject(snatPoolResult.getMessage());
        if (null != snatPoolResultData.getJSONArray("items")) {
            List<SnantPoolModel> snantPoolModels = snatPoolResultData.getJSONArray("items")
                    .toJavaList(SnantPoolModel.class);
            model.setSnantPoolModels(snantPoolModels);
        }
        logger.info("End assembling snatPool data...");

        // 获取客户端dns解析器数据
        BocloudResult dnsResolversResult = f5DnsResolversProvider.list();
        logger.info("Start assembling dnsResolvers data...");
        if (dnsResolversResult.isFailed()) {
            logger.info("Failed to get dnsResolvers data ...");
            return new BocloudResult(false, dnsResolversResult.getMessage());
        }
        JSONObject dnsResolversResultData = JSONObject.parseObject(dnsResolversResult.getMessage());
        if (null != dnsResolversResultData.getJSONArray("items")) {
            List<DnsResolversModel> resolversModels = dnsResolversResultData.getJSONArray("items")
                    .toJavaList(DnsResolversModel.class);
            model.setDnsResolversModels(resolversModels);
        }
        logger.info("End assembling dnsResolvers data...");

        // 获取客户端rule的数据
        BocloudResult ruleResult = f5iRulesProvider.list();
        logger.info("Start assembling rule data...");
        if (ruleResult.isFailed()) {
            logger.info("Failed to get rule data ...");
            return new BocloudResult(false, ruleResult.getMessage());
        }
        JSONObject ruleResultData = JSONObject.parseObject(ruleResult.getMessage());
        if (null != ruleResultData.getJSONArray("items")) {
            List<RuleModel> rules = ruleResultData.getJSONArray("items").toJavaList(RuleModel.class);
            model.setRuleModels(rules);
        }
        logger.info("End assembling rule data...");

        // 获取客户端node的数据
        BocloudResult nodeResult = f5NodeProvider.list();
        logger.info("Start assembling node data...");
        if (nodeResult.isFailed()) {
            logger.info("Failed to get node data ...");
            return new BocloudResult(false, nodeResult.getMessage());
        }
        JSONObject nodeResultData = JSONObject.parseObject(nodeResult.getMessage());
        if (null != nodeResultData.getJSONArray("items")) {
            List<NodeModel> nodes = nodeResultData.getJSONArray("items").toJavaList(NodeModel.class);
            // 获取node的状态
            for (NodeModel nodeModel : nodes) {
                String name = new String(nodeModel.getName());
                try {
                    name = URLEncoder.encode(name, "UTF-8");
                } catch (UnsupportedEncodingException e) {
                    logger.error("url编码转换异常", e);
                }
                String fullPath = "~" + nodeModel.getPartition() + "~";
                if (null != nodeModel.getSubPath()) {
                    fullPath += nodeModel.getSubPath() + "~";
                }
                String status = getStatus(f5NodeProvider.stats(fullPath + name), null,
                        F5BasicObject.NODE, name, null, nodeModel.getPartition(), null, nodeModel.getSubPath(), null);
                nodeModel.setStatus(status);
            }
            model.setNodeModels(nodes);
        }
        logger.info("End assembling node data...");

        // 获取客户端profile的数据
        logger.info("Start assembling profile data...");
        F5ProfileCategory[] profileCategorys = F5ProfileCategory.values();
        List<ProfileModel> profileModelLists = new ArrayList<>();
        for (F5ProfileCategory f5ProfileCategory : profileCategorys) {
            String afterCategroy = StringUtils.substringAfter(String.valueOf(f5ProfileCategory), "_").replaceAll("_",
                    "-");
            String category = StringUtils.substringBefore(String.valueOf(f5ProfileCategory), "_") + "/" + afterCategroy;
            BocloudResult profileResult = f5ProfileProvider.list(category);
            if (profileResult.isFailed()) {
                logger.info("Failed to get profile data ...");
                return new BocloudResult(false, profileResult.getMessage());
            }
            JSONObject profileResultData = JSONObject.parseObject(profileResult.getMessage());
            if (null != profileResultData.getJSONArray("items")) {
                List<ProfileModel> profiles = profileResultData.getJSONArray("items").toJavaList(ProfileModel.class);
                profileModelLists.addAll(profiles);
            }
        }
        model.setProfileModels(profileModelLists);
        logger.info("End assembling profile data...");

        // 获取客户端monitor的数据
        logger.info("Start assembling monitor data...");
        List<String> types = buildMonitorType();
        List<MonitorModel> monitorData = new ArrayList<>();
        for (String type : types) {
            BocloudResult monitorResult = f5MonitorProvider.list(type);
            if (monitorResult.isFailed()) {
                logger.info("Failed to get profile data ...");
                return new BocloudResult(false, monitorResult.getMessage());
            }
            JSONObject monitorResultData = JSONObject.parseObject(monitorResult.getMessage());
            if (null != monitorResultData.getJSONArray("items")) {
                List<MonitorModel> monitors = monitorResultData.getJSONArray("items").toJavaList(MonitorModel.class);
                switch (type) {
                    case F5MonitorType.HTTP:
                    case F5MonitorType.GATEWAY_ICMP:
                    case F5MonitorType.HTTPS:
                    case F5MonitorType.ICMP:
                    case F5MonitorType.INBAND:
                    case F5MonitorType.REAL_SERVER:
                    case F5MonitorType.SNMP_DCA:
                    case F5MonitorType.TCP:
                    case F5MonitorType.TCP_ECHO:
                    case F5MonitorType.TCP_HALF_OPEN:
                    case F5MonitorType.UDP:
                        monitorData.addAll(monitors);
                        break;
                    default:
                        Iterator<MonitorModel> iterator = monitors.iterator();
                        while (iterator.hasNext()) {
                            if (iterator.next().getName().equals(type.replace("-", "_"))) {
                                iterator.remove();
                            }
                        }
                        monitorData.addAll(monitors);
                        break;
                }
            }
        }
        monitorData.removeAll(Collections.singleton(null));
        model.setMonitorModels(monitorData);
        logger.info("End assembling monitor data...");

        // 获取客户端pool以及poolMember的数据
        BocloudResult poolResult = f5PoolProvider.list();
        logger.info("Start assembling pool data...");
        if (poolResult.isFailed()) {
            logger.info("Failed to get pool data ...");
            return new BocloudResult(false, poolResult.getMessage());
        }
        JSONObject poolResultData = JSONObject.parseObject(poolResult.getMessage());
        if (null != poolResultData.getJSONArray("items")) {
            List<PoolModel> pools = poolResultData.getJSONArray("items").toJavaList(PoolModel.class);
            // 获取客户端pool member的数据
            for (PoolModel poolModel : pools) {
                String name = new String(poolModel.getName());
                try {
                    name = URLEncoder.encode(name, "UTF-8");
                } catch (UnsupportedEncodingException e) {
                    logger.error("url编码转换异常", e);
                }
                // 获取pool的状态
                String fullPath = "~" + poolModel.getPartition() + "~";
                if (null != poolModel.getSubPath()) {
                    fullPath += poolModel.getSubPath() + "~";
                }
                BocloudResult result = f5PoolProvider.stats(basicUrl + F5BasicObject.POOL + "/" + fullPath + poolModel.getName() + "/stats");
                if (result.isFailed()) {
                    return new BocloudResult(false, poolResult.getMessage());
                }
                String poolStatus = getStatus(result, null, F5BasicObject.POOL, name, null, poolModel.getPartition(), null, poolModel.getSubPath(), null);
                poolModel.setStatus(poolStatus);
                String full = "~" + poolModel.getPartition() + "~";
                if (null != poolModel.getSubPath()) {
                    full += poolModel.getSubPath() + "~";
                }
                String url = basicUrl + F5BasicObject.POOL + "/" + full + poolModel.getName() + "/members";
                BocloudResult poolMemberResult = f5PoolProvider.detail(url);
                if (poolMemberResult.isFailed()) {
                    logger.info("Failed to get pool member data ...");
                    return new BocloudResult(false, poolMemberResult.getMessage());
                }
                JSONObject poolMemberResultData = JSONObject.parseObject(poolMemberResult.getMessage());
                if (null != poolMemberResultData.getJSONArray("items")) {
                    List<PoolMemberModel> poolMembers = poolMemberResultData.getJSONArray("items")
                            .toJavaList(PoolMemberModel.class);
                    for (PoolMemberModel poolMemberModel : poolMembers) {
                        // 获取poolmember的状态
                        String poolFullPatch = "~" + poolMemberModel.getPartition() + "~";
                        if (null != poolMemberModel.getSubPath()) {
                            poolFullPatch += poolMemberModel.getSubPath() + "~";
                        }
                        String poolMenmberStatus = getStatus(
                                f5PoolProvider.stats(url + "/" + poolFullPatch + poolMemberModel.getName() + "/stats"),
                                F5BasicObject.POOL, F5BasicObject.POOL_MEMBER, name, poolMemberModel.getName(),
                                poolMemberModel.getPartition(), poolModel.getPartition(), poolMemberModel.getSubPath(), poolModel.getSubPath());
                        poolMemberModel.setStatus(poolMenmberStatus);
                    }
                    poolModel.setPoolMemberModels(poolMembers);
                }
            }
            model.setPoolModels(pools);
        }
        logger.info("End assembling pool data...");

        // 获取客户端virtual server的数据
        BocloudResult virtualServerResult = f5VirtualServerProvider.list();
        logger.info("Start assembling virtual server data...");
        if (virtualServerResult.isFailed()) {
            logger.info("Failed to get virtual server data ...");
            return new BocloudResult(false, virtualServerResult.getMessage());
        }
        JSONObject virtualServerResultData = JSONObject.parseObject(virtualServerResult.getMessage());
        if (null != virtualServerResultData.getJSONArray("items")) {
            List<VirtualServerModel> virtualServers = virtualServerResultData.getJSONArray("items")
                    .toJavaList(VirtualServerModel.class);
            for (VirtualServerModel virtualServerModel : virtualServers) {
                String name = new String(virtualServerModel.getName());
                // 获取vs的状态
                String vsFullPatch = "~" + virtualServerModel.getPartition() + "~";
                if (null != virtualServerModel.getSubPath()) {
                    vsFullPatch += virtualServerModel.getSubPath() + "~";
                }
                BocloudResult stats = f5VirtualServerProvider.stats(vsFullPatch + virtualServerModel.getName());
                if (stats.isFailed()) {
                    return new BocloudResult(false, stats.getMessage());
                }
                String status = getStatus(stats, null,
                        F5BasicObject.VIRTUAL_SERVER, name, null, virtualServerModel.getPartition(), null, virtualServerModel.getSubPath(), null);
                virtualServerModel.setStatus(status);
                String pFullPatch = "~" + virtualServerModel.getPartition() + "~";
                if (null != virtualServerModel.getSubPath()) {
                    pFullPatch += virtualServerModel.getSubPath() + "~";
                }
                BocloudResult vsProfilesResult = f5VirtualServerProvider.getProfiles(pFullPatch + virtualServerModel.getName());
                if (vsProfilesResult.isFailed()) {
                    logger.info("Failed to get virtual server of profiles data ...");
                    return new BocloudResult(false, vsProfilesResult.getMessage());
                }
                JSONObject profilesResultData = JSONObject.parseObject(vsProfilesResult.getMessage());
                List<ProfileModel> profileModels = profilesResultData.getJSONArray("items")
                        .toJavaList(ProfileModel.class);
                virtualServerModel.setProfileModels(profileModels);
            }
            model.setVirtualServerModels(virtualServers);
        }
        logger.info("End assembling virtual server data...");
        return new BocloudResult(true, model, "同步成功");
    }

    // 获取对象的状态
    private String getStatus(BocloudResult result, String parentType, String type, String parentName, String name,
                             String partition, String parentPartition, String partitionSub, String parentPartitionSub) {
        JSONObject resultData = JSONObject.parseObject(result.getMessage());
        JSONObject entries = resultData.getJSONObject("entries");
        String description = null;
        String version = butler.getVersion();
        if (null != partitionSub) {
            partition = partition + "~" + partitionSub;
        }
        if (null != parentPartitionSub) {
            parentPartition = parentPartition + "~" + parentPartitionSub;
        }
        switch (version) {
            case "V11":
                // 获取V11版本的节点状态
                description = entries.getJSONObject("status.availabilityState").getString("description");
                break;
            case "V13":
                if (F5BasicObject.POOL_MEMBER.equals(type)) {
                    String key = "https://localhost/mgmt/tm/ltm/" + parentType + "/~" + parentPartition + "~" + parentName
                            + "/" + type + "/~" + partition + "~" + name + "/~" + partition + "~" + name;
                    description = entries.getJSONObject(key + "/stats").getJSONObject("nestedStats")
                            .getJSONObject("entries").getJSONObject("status.availabilityState").getString("description");
                } else {
                    String key = "https://localhost/mgmt/tm/ltm/" + type + "/~" + partition + "~" + parentName + "/~"
                            + partition + "~" + parentName + "/stats";
                    description = entries.getJSONObject(key).getJSONObject("nestedStats").getJSONObject("entries")
                            .getJSONObject("status.availabilityState").getString("description");
                }
                break;
            default:
                break;
        }
        return description;
    }

    /**
     * 拼接monitor的类型
     *
     * @return
     */
    public List<String> buildMonitorType() {
        F5MonitorType f5MonitorType = new F5MonitorType();
        List<String> typeList = new ArrayList<>();
        Class<? extends F5MonitorType> studentClass = (Class<? extends F5MonitorType>) f5MonitorType.getClass();
        Field[] fields = studentClass.getDeclaredFields();
        for (int i = 0; i < fields.length; i++) {
            Field field = fields[i];
            field.setAccessible(true);
            try {
                typeList.add(String.valueOf(field.get(f5MonitorType)));
            } catch (Exception e) {
                logger.error("拼接monitor类型失败{}", e);
            }
        }
        return typeList;
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

    public BocloudResult authLogin(String url) {
        // 处理请求路径
        url = butler.getUrl() + url;
        HttpGet httpGet = new HttpGet(url);
        httpGet.addHeader("Authorization", "Basic " + Base64.getUrlEncoder()
                .encodeToString(((butler.getUsername() + ":" + butler.getPassword())).getBytes()));
        httpGet.addHeader("accept", "application/json");
        try {
            F5Provider f5Provider = new F5Provider(butler);
            CloseableHttpResponse httpResponse = f5Provider.getClient().execute(httpGet);
            String resultData = null;
            StatusLine statusLine = httpResponse.getStatusLine();
            int statusCode = statusLine.getStatusCode();
            HttpEntity entity = httpResponse.getEntity();
            switch (statusCode) {
                case 200:
                    return new BocloudResult(true, resultData);
                case 404:
                    return new BocloudResult(false, "Url Not Found");
                case 401:
                    resultData = EntityUtils.toString(entity);
                    return new BocloudResult(false, parseResultData(resultData));
                default:
                    return new BocloudResult(false, "认证失败，请检查！");
            }
        } catch (Exception e) {
            logger.error("CloseableHttpResponse execute ", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    private String parseResultData(String resultData) {
        return StringUtils.substringBefore(StringUtils.substringAfter(resultData, "<p>"), "</p>");
    }

}
