package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * AddressTranslation 地址转化操作接口实现
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5AddressTranslationProvider extends F5Provider implements AddressTranslationProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5AddressTranslationProvider.class);
    private Butler butler;

    public F5AddressTranslationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list(String category) {
        // 获取Address Translation列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/" + category;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Address Translation数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

}
