package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.DataGroupModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * @author lyy
 * @version 1.0
 * @date 2020/11/10 17:56
 */
@Provider("bocloud.cmp.f5")
public class F5DataGroupProvider extends F5Provider implements DataGroupProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5NodeProvider.class);
    private Butler butler;

    public F5DataGroupProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list(String category) {
        String url = "/mgmt/tm/ltm/data-group/" + category;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Data Group数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult create(String category, DataGroupModel dataGroupModel) {
        String url = "/mgmt/tm/ltm/data-group/" + category;
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(dataGroupModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Data Group失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult update(String category, DataGroupModel dataGroupModel) {
        String url = "/mgmt/tm/ltm/data-group/" + category + "/" + dataGroupModel.getName();
        url = butler.getUrl() + url;
        buildDataGroupModel(dataGroupModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(dataGroupModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Data Group失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    private void buildDataGroupModel(DataGroupModel dataGroupModel) {
        dataGroupModel.setName(null);
        dataGroupModel.setType(null);
    }

    @Override
    public BocloudResult remove(String category, String name) {
        String url = "/mgmt/tm/ltm/data-group/" + category + "/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Data Group失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }
}
