package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * Created by liuyuanyuan on 2019/5/7 14:14
 */
@Provider("bocloud.cmp.f5")
public class F5DeviceManagementProvider extends F5Provider implements DeviceManagementProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5DeviceManagementProvider.class);
    private Butler butler;

    public F5DeviceManagementProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult deviceGroup(String url) {
        // 处理请求路径
        url = butler.getUrl() + "/mgmt/tm/cm/device-group/" + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Device Group数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult device(String url) {
        // 处理请求路径
        url = butler.getUrl() + "/mgmt/tm/cm/device/" + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Device数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult syncStats(String url) {
        // 处理请求路径
        url = butler.getUrl() + "/mgmt/tm/cm/sync-status/" + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult sync(JSONObject params) {
        // 处理请求路径
        String url = "/mgmt/tm/cm";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, params, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "同步成功");
    }
}
