package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.MonitorModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * Monitor操作接口实现类
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5MonitorProvider extends F5Provider implements MonitorProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5MonitorProvider.class);
    private Butler butler;

    public F5MonitorProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(MonitorModel monitorModel, String type) {
        // 开始创建monitor
        // 处理请求路径
        String url = "/mgmt/tm/ltm/monitor/" + type;
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(monitorModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Monitor失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name, String type) {
        // 执行删除monitor
        // 处理请求路径
        String url = "/mgmt/tm/ltm/monitor/" + type + "/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Monitor失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

    @Override
    public BocloudResult update(MonitorModel monitorModel, String type) {
        // 开始更新monitor
        // 处理请求路径
        String url = "/mgmt/tm/ltm/monitor/" + type + "/" + monitorModel.getName();
        url = butler.getUrl() + url;
        buildMonitorModel(monitorModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(monitorModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Monitor失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    @Override
    public BocloudResult list(String type) {
        // 获取monitor
        // 处理请求路径
        String url = "/mgmt/tm/ltm/monitor/" + type;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Monitor数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildMonitorModel(MonitorModel monitorModel) {
        monitorModel.setName(null);
        monitorModel.setDefaultsFrom(null);
    }

}
