package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.NodeModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * node操作接口实现类
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5NodeProvider extends F5Provider implements NodeProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5NodeProvider.class);
    private Butler butler;

    public F5NodeProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(NodeModel nodeModel) {
        // 开始创建node
        // 处理请求路径
        String url = "/mgmt/tm/ltm/node/";
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(nodeModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Node失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name) {
        // 执行删除node
        // 处理请求路径
        String url = "/mgmt/tm/ltm/node/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Node失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

    @Override
    public BocloudResult update(NodeModel nodeModel) {
        // 开始更新node
        // 处理请求路径
        String url = "/mgmt/tm/ltm/node/" + nodeModel.getName();
        url = butler.getUrl() + url;
        buildNodeModel(nodeModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(nodeModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Node失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    @Override
    public BocloudResult list() {
        // 获取node列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/node/";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Node数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildNodeModel(NodeModel nodeModel) {
        nodeModel.setName(null);
        nodeModel.setAddress(null);
    }

    @Override
    public BocloudResult stats(String nodeName) {
        // 获取node状态。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/node/" + nodeName + "/stats";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Node状态失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

}
