package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.PartitionModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

@Provider("bocloud.cmp.f5")
public class F5PartitionProvider extends F5Provider implements PartitionProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5PartitionProvider.class);
    private Butler butler;

    public F5PartitionProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        // 获取partition列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/auth/partition/";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取partition数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult create(PartitionModel partitionModel) {
        // 开始创建partition
        // 处理请求路径
        String url = "/mgmt/tm/auth/partition";
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(partitionModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建partition失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name) {
        // 执行删除partition
        // 处理请求路径
        String url = "/mgmt/tm/auth/partition/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除partition失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

}
