package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.PoolMemberModel;
import com.bocloud.cmp.model.f5.PoolModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * pool操作接口实现类
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5PoolProvider extends F5Provider implements PoolProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5PoolProvider.class);
    private Butler butler;

    public F5PoolProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(PoolModel poolModel) {
        // 开始创建pool
        // 处理请求路径
        String url = "/mgmt/tm/ltm/pool/";
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(poolModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Pool失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name) {
        // 执行删除pool
        // 处理请求路径
        String url = "/mgmt/tm/ltm/pool/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Pool失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

    @Override
    public BocloudResult update(PoolModel poolModel) {
        // 开始更新pool
        // 处理请求路径
        String url = "/mgmt/tm/ltm/pool/" + poolModel.getName();
        url = butler.getUrl() + url;
        buildPoolModel(poolModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(poolModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Pool失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    @Override
    public BocloudResult list() {
        // 获取pool列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/pool/";
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Pool数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult removePoolMember(String url) {
        // 删除poolMember。。。
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除PoolMember失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult createPoolMember(PoolMemberModel poolMemberModel, String url) {
        // 新增poolMember。。。
        // 处理请求路径
        url = butler.getUrl() + url;
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(poolMemberModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        return new BocloudResult(true, result, "");
    }

    @Override
    public BocloudResult detail(String url) {
        // 获取poolMember信息。。。
        // 处理请求路径
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取PoolMember详情失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    @Override
    public BocloudResult updatePoolMember(String url, PoolMemberModel poolMemberModel) {
        // 更新poolMember信息。。。
        // 处理请求路径
        url = butler.getUrl() + url;
        buildPoolMemberModel(poolMemberModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(poolMemberModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新PoolMember失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildPoolModel(PoolModel poolModel) {
        poolModel.setName(null);
        poolModel.setMembers(null);
    }

    @Override
    public BocloudResult stats(String url) {
        // 获取poolMember状态。。。
        // 处理请求路径
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);

        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取PoolMember状态失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildPoolMemberModel(PoolMemberModel poolMemberModel) {
        poolMemberModel.setName(null);
        poolMemberModel.setAddress(null);
    }

}
