package com.bocloud.cmp.provider.f5;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.f5.ProfileModel;
import com.bocloud.cmp.provider.F5Provider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * profile操作接口实现类
 *
 * @author liuyuanyuan
 */
@Provider("bocloud.cmp.f5")
public class F5ProfileProvider extends F5Provider implements ProfileProvider {
    private static final Logger logger = LoggerFactory.getLogger(F5ProfileProvider.class);
    private Butler butler;

    public F5ProfileProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(ProfileModel profileModel, String category) {
        // 开始创建profile
        // 处理请求路径
        String url = "/mgmt/tm/ltm/" + category;
        url = butler.getUrl() + url;
        profileModel.setDnsResolverId(null);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(profileModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.post(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "创建Profile失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "创建成功");
    }

    @Override
    public BocloudResult remove(String name, String categroy) {
        // 执行删除profile
        // 处理请求路径
        String url = "/mgmt/tm/ltm/" + categroy + "/" + name;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.delete(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "删除Profile失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "删除成功");
    }

    @Override
    public BocloudResult update(ProfileModel profileModel, String categroy) {
        // 开始更新profile
        // 处理请求路径
        String url = "/mgmt/tm/ltm/" + categroy + "/" + profileModel.getName();
        url = butler.getUrl() + url;
        buildProfileModel(profileModel);
        JSONObject body = JSONObject.parseObject(JSONObject.toJSONString(profileModel));
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        httpClient.setDataFormat(PostDataFormat.RAW);
        Result result = httpClient.patch(header, body, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "更新Profile失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage(), "更新成功");
    }

    @Override
    public BocloudResult list(String category) {
        // 获取profile列表。。。
        // 处理请求路径
        String url = "/mgmt/tm/ltm/" + category;
        url = butler.getUrl() + url;
        HttpClient httpClient = new HttpClient(30000);
        Map<String, Object> header = buildHeader();
        Result result = httpClient.get(header, null, url);
        if (result.isFailed()) {
            logger.error(result.getMessage());
            return new BocloudResult(false, "获取Profile数据失败:" + result.getMessage());
        }
        return new BocloudResult(true, result.getMessage());
    }

    private void buildProfileModel(ProfileModel profileModel) {
        profileModel.setName(null);
        profileModel.setProxyType(null);
        profileModel.setDnsResolverId(null);
    }

}
