package com.bocloud.cmp.fusioncloudconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 华为私有云镜像转换器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2018年9月18日
 *
 */
public class ImageConvertor implements BeanConvertor<ImageModel, JSONObject> {

	private static final Logger logger = LoggerFactory.getLogger(ImageConvertor.class);

	@Override
	public List<ImageModel> convertList(List<? extends JSONObject> objects) {
		List<ImageModel> imageModels = new ArrayList<ImageModel>();
		for (JSONObject object : objects) {
			imageModels.add(convertModel(object));
		}
		return imageModels;
	}

	@Override
	public ImageModel convertModel(JSONObject object) {
		ImageModel model = new ImageModel();
		model.setName(object.getString("name"));
		model.setStatus(null == object.getString("status") ? "ERROR" : object.getString("status").toUpperCase());
		// 不确定有没有描述和创建时间
		model.setRemark(object.getString("__description"));
		try {
			// 根据具体的时间数值来看，FusionCloud返回给我的时间应该是UTC时间，比东8区早了8小时。
			// 但是从具体的时间格式来看它却没有注明时区，所以这里if判断下如果确定没有时区就给加上时区
			String createAt = object.getString("created_at");
			if (!createAt.contains("+")) {
				createAt += "+0000";
			}
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'Z");
			model.setGmtCreate(format.parse(createAt));
		} catch (ParseException e) {
			logger.error("Get error message：", e);
			model.setGmtCreate(new Date());
		}
		model.setOsCategory(object.getString("__platform"));
		model.setOsVersion(object.getString("__os_version"));
		model.setOsType(object.getString("__os_type"));
		model.setImageUuid(object.getString("id"));
		model.setArchitecture(object.getString("__os_bit")); // 系统位数
		model.setImageSourceType(object.getString("_image_source_type"));
		model.setImageType(object.getString("__imagetype")); // 镜像类型，支持公共镜像：gold 私有镜像：private 共享镜像：shared
		model.setMinRam(object.getLong("min_ram"));
		model.setIsRegistered(object.getBoolean("_isregistered")); // 是否注册过的镜像
		//model.setOwnerId(object.getLong("owner"));
		model.setFilePath(object.getString("file"));
		Double size = object.getDouble("_image_size");
		if (size == null) {
			size = object.getDouble("size");
		}
		if (null != size) {
			size = new BigDecimal(size).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
					.doubleValue();
		}
		model.setSize(size);
		Double virtualSize = object.getDouble("virtual_size");
		if (null != virtualSize) {
			virtualSize = new BigDecimal(virtualSize).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
					.doubleValue();
		}
		model.setVirtualSize(virtualSize);
		model.setMinDisk(object.getLong("min_disk"));
		return model;
	}
}
