package com.bocloud.cmp.fusioncloudconvertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 华为私有云云主机数据转换器
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/9/19
 */
public class ServerConvertor implements BeanConvertor<ServerModel, JSONObject> {

	private static final Logger logger = LoggerFactory.getLogger(ServerConvertor.class);

	@Override
	public List<ServerModel> convertList(List<? extends JSONObject> objects) {
		List<ServerModel> serverModels = new ArrayList<ServerModel>();
		for (JSONObject object : objects) {
			serverModels.add(convertModel(object));
		}
		return serverModels;
	}

	@Override
	public ServerModel convertModel(JSONObject object) {
		ServerModel model = new ServerModel();
		model.setTenantUuid(object.getString("tenant_id"));
		model.setName(object.getString("name"));
		String privatesIps = "";
		String managerIp = "";
		List<JSONObject> newIps = new ArrayList<JSONObject>();
		JSONObject addresses = JSONObject.parseObject(object.getString("addresses"), JSONObject.class);
		if (null != addresses) {
			for (String networkId : addresses.keySet()) {
				List<JSONObject> ips = JSONArray.parseArray(addresses.getString(networkId), JSONObject.class);
				for (JSONObject ip : ips) {
					JSONObject newIp = new JSONObject();
					newIp.put("networkId", networkId);
					newIp.put("address", ip.getString("addr"));
					if (StringUtils.isEmpty(managerIp)) {
						managerIp = ip.getString("addr");
					}
					newIps.add(newIp);
				}
			}
		}
		privatesIps = JSONArray.toJSONString(newIps);
		model.setPrivateIps(privatesIps);
		model.setManagerIp(managerIp);
		model.setZone(object.getString("OS-EXT-AZ:availability_zone"));
		model.setInstanceId(object.getString("id"));
		List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
		String volsJson = object.getString("os-extended-volumes:volumes_attached");
		if (!StringUtils.isEmpty(volsJson)) {
			List<JSONObject> vols = JSONArray.parseArray(volsJson, JSONObject.class);
			for (JSONObject vol : vols) {
				VolumeModel volumeModel = new VolumeModel();
				volumeModel.setVolumeUuid(vol.getString("id"));
				volumeModels.add(volumeModel);
			}
		}
		model.setDiskDevices(volumeModels);
		model.setHostName(object.getString("OS-EXT-SRV-ATTR:host"));
		String imageJson = object.getString("image");
		if (!StringUtils.isEmpty(imageJson)) {
			JSONObject imageObj = JSONObject.parseObject(imageJson);
			model.setImageUuid(imageObj.getString("id"));
		}
		try {
			// 根据具体的时间数值来看，FusionCloud返回给我的时间应该是UTC时间，比东8区早了8小时。
			// 但是从具体的时间格式来看它却没有注明时区，所以这里if判断下如果确定没有时区就给加上时区
			String createAt = object.getString("created");
			if (!createAt.contains("+")) {
				createAt += "+0000";
			}
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'Z");
			model.setGmtVmCreate(format.parse(createAt));
		} catch (ParseException e) {
			logger.error("Get error message：", e);
			model.setGmtVmCreate(new Date());
		}
		model.setKeypairName(object.getString("key_name"));
		String flavorJson = object.getString("flavor");
		if (!StringUtils.isEmpty(flavorJson)) {
			JSONObject flavorObj = JSONObject.parseObject(flavorJson, JSONObject.class);
			model.setFlavorUuid(flavorObj.getString("id"));
		}
		String secgJson = object.getString("security_groups");
		if (!StringUtils.isEmpty(secgJson)) {
			List<JSONObject> secgObjs = JSONArray.parseArray(secgJson, JSONObject.class);
			List<String> secgList = new ArrayList<String>();
			for (JSONObject secgObj : secgObjs) {
				secgList.add(secgObj.getString("name"));
			}
			model.setSecurityGroups(secgList);
			model.setGroups(JSONArray.toJSONString(secgList));
		}
		String status = convert(object.getString("status"));
		model.setStatus(status);
		return model;
	}

	private String convert(String status) {
		if (StringUtils.isEmpty(status)) {
			return BocloudStatus.Server.EXCEPTION.name();
		} else if (status.equalsIgnoreCase("active")) {
			return BocloudStatus.Server.RUNNING.name();
		} else if (status.equalsIgnoreCase(BocloudStatus.Server.PAUSED.name())) {
			return BocloudStatus.Server.PAUSED.name();
		} else if (status.equalsIgnoreCase("stopped") || status.equalsIgnoreCase("shutoff")) {
			return BocloudStatus.Server.STOPPED.name();
		} else if (status.equalsIgnoreCase("BUILD")) {
			return BocloudStatus.Server.BUILDING.name();
		} else if (status.equalsIgnoreCase(BocloudStatus.Server.SUSPENDED.name())) {
			return BocloudStatus.Server.SUSPENDED.name();
		} else if (status.equalsIgnoreCase("VERIFY_RESIZE")) {
			return BocloudStatus.Server.RESIZING.name();
		} else if (status.equalsIgnoreCase("SHELVED") || status.equalsIgnoreCase("SHELVED_OFFLOADED")) {
			return BocloudStatus.Server.SHELVED.name();
		} else if (status.equalsIgnoreCase("error")) {
			return BocloudStatus.Server.ERROR.name();
		} else {
			return BocloudStatus.Server.EXCEPTION.name();
		}
	}
}
