package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.common.model.FusionCloudAuth;
import com.bocloud.cmp.common.model.FusionCloudCatalog;
import com.bocloud.cmp.common.model.FusionCloudDomain;
import com.bocloud.cmp.common.model.FusionCloudEndpoint;
import com.bocloud.cmp.common.model.FusionCloudIdentity;
import com.bocloud.cmp.common.model.FusionCloudPassword;
import com.bocloud.cmp.common.model.FusionCloudProject;
import com.bocloud.cmp.common.model.FusionCloudScope;
import com.bocloud.cmp.common.model.FusionCloudToken;
import com.bocloud.cmp.common.model.FusionCloudUser;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.utils.TokenCache;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.http.ResultBuilder;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPatch;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:07 2018/11/14 @ Modified
 * By：
 */
public class FusionCloudProvider extends GenericProvider implements SuperProvider {

	private static final Logger logger = LoggerFactory.getLogger(FusionCloudProvider.class);
	private static final String CODE = "bocloud.cmp.fusioncloud";
	private String endpoint;// 平台认证路径
	private String username;// 管理员账号
	private String password;// 管理员密码
	private String domain;// 所属域
	private String project;// 项目ID
	private boolean accessable;
	private FusionCloudToken token;// token信息
	private String error; // 错误信息
	private HttpClient client = new HttpClient(3 * 60 * 1000, PostDataFormat.RAW);

	private String monitorUrl;
	private String monitorUserName;
	private String monitorPwd;
	private String monitorToken;
	private String monitorRoaRand;
	private Butler butler;

	/**
	 * @param butler
	 */
	public FusionCloudProvider(Butler butler) {
		super(butler);
		this.butler = butler;
		this.endpoint = butler.getUrl();
		this.username = butler.getUsername();
		this.password = butler.getPassword();
		this.domain = butler.getDomainName();
		this.project = butler.getProjectId();
		initToken();
		/*token = FusionCloudClientFactory.getInstance().getToken(butler.getUuid());
		if (null != token && null != token.getToken()) {
			this.accessable = true;
		} else {
			init();
			if (accessable) {
				ButlerRepository repository = ButlerRepository.getInstance();
				if (!repository.contains(CODE)) {
					repository.addListener(new FusionCloudButlerListener(CODE));
				}
				FusionCloudClientFactory.getInstance().refreshToken(butler.getUuid(), this);
			}
		}*/
	}

	private void initToken() {
//		String monitorTokenCache = TokenCache.getKey("monitorToken." + butler.getUuid());
//		String monitorRoaRandCache = TokenCache.getKey("monitorRoaRand." + butler.getUuid());
//		if (null == monitorTokenCache || null == monitorRoaRandCache) {
//			initMonitorToken();
//		} else {
//			this.monitorToken = monitorTokenCache;
//			this.monitorRoaRand = monitorRoaRandCache;
//		}
		String fusionCloudToken = TokenCache.getKey("token." + butler.getUuid());
		if (null == fusionCloudToken) {
			init();
		} else {
			this.token = JSON.parseObject(fusionCloudToken, FusionCloudToken.class);
			this.accessable = true;
		}
	}

	@Override
	public boolean accessable() {
		return this.accessable;
	}

	protected Map<String, Object> buildMonitorHeader() {
		Map<String, Object> header = new HashMap<String, Object>();
		header.put("accept", "application/json");
		header.put("content-type", "application/json");
		header.put("X-Auth-Token", this.getMonitorToken());
		header.put("roaRand", this.getMonitorRoaRand());
		return header;
	}

	/**
	 * 初始化连接
	 */
	private void init() {
		if (paramCheck().isFailed()) {// 基本的参数校验不通过，即用户名，密码或者认证路径为空
			this.accessable = false;
			logger.error("Driver init error : {}", paramCheck().getMessage());
			this.error = paramCheck().getMessage();
			return;
		}
		String url = this.endpoint + "/v3/auth/tokens";
		FusionCloudDomain domain = new FusionCloudDomain(this.domain);
		FusionCloudUser user = new FusionCloudUser(this.username, this.password, domain);
		FusionCloudPassword password = new FusionCloudPassword(user);
		List<String> methods = new ArrayList<String>();
		methods.add("password");
		FusionCloudIdentity identity = new FusionCloudIdentity(password, methods);
		FusionCloudProject project = new FusionCloudProject(this.project);
		FusionCloudScope scope = new FusionCloudScope(project);
		//FusionCloudScope scope = new FusionCloudScope(domain);
		FusionCloudAuth auth = new FusionCloudAuth(identity, scope);
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("auth", auth);
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("accept", "application/json");
		httpPost.addHeader("content-type", "application/json");
		StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
		httpPost.setEntity(httpEntity);

		logger.debug("==== Get fusion-cloud token start ====>>");
		logger.debug("== url: " + url);
		logger.debug("== param: " + JSON.toJSONString(params));
		try {
			CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPost);
			Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
			logger.debug("== result: " + JSON.toJSONString(result));
			logger.debug("==== Get fusion-cloud token end ====<<");
			if (result.isFailed()) {
				this.accessable = false;
				logger.error("Driver init error : {}", result.getMessage());
				this.error = result.getMessage();
				return;
			}
			JSONObject content = JSONObject
					.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
			this.token = JSONObject.parseObject(content.getString("token"), FusionCloudToken.class);
			if (null == this.token) {
				this.accessable = false;
				logger.error("Driver init error : {}", "Token is empty !");
				this.error = "Token is empty !";
				return;
			}
			String authToken = findToken(httpResponse);
			if (StringUtils.isEmpty(authToken)) {
				this.accessable = false;
				logger.error("Driver init error : {}", "Token is empty !");
				this.error = "Token is empty !";
				return;
			}
			token.setToken(authToken);
			TokenCache.setKey("token." + butler.getUuid(), JSON.toJSONString(token));
			this.accessable = true;
		} catch (Exception e) {
			logger.error("Get error message：", e);
			this.accessable = false;
		}
	}

	/**
	 * 从响应头部获取会话X-Subject-Token
	 *
	 * @param httpResponse
	 * @return
	 */
	private String findToken(CloseableHttpResponse httpResponse) {
		String token = "";
		Header[] headers = httpResponse.getHeaders("X-Subject-Token");
		if (null == headers || headers.length == 0) {
			return token;
		}
		for (Header header : headers) {
			if ("X-Subject-Token".equalsIgnoreCase(header.getName())) {
				token = header.getValue();
				break;
			}
		}
		return token;
	}

	public String getUrl() { // https://ip:26335
		return this.monitorUrl;
	}

	public void initMonitorToken() {
		try {
			String url = getUrl() + "/rest/plat/smapp/v1/oauth/token";
			JSONObject params = new JSONObject();
			params.put("grantType", "password");
			params.put("userName", this.getMonitorUserName());
			params.put("value", this.getMonitorPwd());
			HttpPut httpPost = new HttpPut(url);
			httpPost.addHeader("accept", "application/json");
			httpPost.addHeader("content-type", "application/json");
			StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
			httpPost.setEntity(httpEntity);
			CloseableHttpResponse httpResponse = this.getClient().getHttpClient().execute(httpPost);
			Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
			JSONObject content = JSONObject
					.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
			this.monitorToken = content.getString("accessSession");
			this.monitorRoaRand = content.getString("roaRand");
			TokenCache.setKey("monitorToken." + butler.getUuid(), monitorToken);
			TokenCache.setKey("monitorRoaRand." + butler.getUuid(), monitorRoaRand);
		} catch (Exception e) {
			logger.info("初始化监控token Error : " + e);
		}
	}

	/**
	 * 基本参数信息认证，即确保认证路径，认证账号和认证密码为非空
	 *
	 * @return
	 */
	private Result paramCheck() {
		if (StringUtils.isEmpty(this.endpoint)) {
			return new Result(false, "Endpoint is Empty!");
		}
		if (StringUtils.isEmpty(this.username)) {
			return new Result(false, "Username is Empty!");
		}
		if (StringUtils.isEmpty(this.password)) {
			return new Result(false, "Password is Empty!");
		}
		if (StringUtils.isEmpty(this.domain)) {
			return new Result(false, "Domain is Empty!");
		}
		if (StringUtils.isEmpty(this.project)) {
			return new Result(false, "Project is Empty!");
		}
		return new Result(true, "success");
	}

	public Result doGet(String url, Map<String, Object> params) {
		try {
			client.setDataFormat(null);
			logger.debug("url: {}", url);
			logger.debug("params: {}", JSON.toJSONString(params));
			Result result = this.client.get(buildHeader(), params, url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}	
	
	public Result doGet(String url, Map<String, Object> params, FusionCloudApiName apiName,
			FusionCloudApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", JSON.toJSONString(params));
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.get(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, Map<String, Object> params) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			logger.debug("url: {}", url);
			logger.debug("params: {}", JSON.toJSONString(params));
			Result result = this.client.post(buildHeader(), params, url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, Map<String, Object> params, FusionCloudApiName apiName,
			FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", JSON.toJSONString(params));
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.post(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, String params) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			logger.debug("url: {}", url);
			logger.debug("params: {}", JSON.toJSONString(params));
			Result result = this.client.post(buildHeader(), params, url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, String params, FusionCloudApiName apiName, FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", params);
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.post(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPut(String url, Map<String, Object> params, FusionCloudApiName apiName,
			FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", JSON.toJSONString(params));
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.put(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPutFile(String url, String key, File file, FusionCloudApiName apiName, FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.FORM_DATA);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("file: {} = {}", key, file.getName());
			FileBody fileBody = new FileBody(file, ContentType.MULTIPART_FORM_DATA);
			HttpEntity httpEntity = MultipartEntityBuilder.create().addPart(key, fileBody).build();
			HttpPut httpPut = new HttpPut(apiUrl + url);
			httpPut.setEntity(httpEntity);
			httpPut.setHeader("X-Auth-Token", this.token.getToken());
			httpPut.setHeader("X-Project-id", this.token.getProject().getId());
			//httpPut.setHeader(HttpHeaders.CONTENT_TYPE, ContentType.MULTIPART_FORM_DATA.getMimeType());
			httpPut.setHeader(HttpHeaders.ACCEPT, ContentType.APPLICATION_JSON.getMimeType());
			CloseableHttpResponse response = this.client.getHttpClient().execute(httpPut);
			ResultBuilder resultBuilder = new CommonResultBuilder();
			Result result = resultBuilder.build(response.getStatusLine().getStatusCode());
			if (response.getEntity() != null) {
				result.setMessage(IOUtils.toString(response.getEntity().getContent(), StandardCharsets.UTF_8));
			}
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} catch (ClientProtocolException e) {
			logger.error("HttpClient ClientProtocolException:", e);
			return new Result(false, e.getMessage());
		} catch (IOException e) {
			logger.error("HttpClient IOException:", e);
			return new Result(false, e.getMessage());
		} finally {
			this.client.close();
		}
	}

	public Result doPatch(String url, Map<String, Object> params, FusionCloudApiName apiName,
			FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", JSON.toJSONString(params));
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.patch(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doPatch(String url, String params, FusionCloudApiName apiName, FusionCloudApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", params);
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			HttpEntity entity = new StringEntity(params, StandardCharsets.UTF_8);

			HttpPatch httpPatch = new HttpPatch(apiUrl + url);
			httpPatch.setEntity(entity);

			httpPatch.setHeader("X-Auth-Token", this.token.getToken());
			httpPatch.setHeader("X-Project-id", this.token.getProject().getId());
			httpPatch.setHeader("accept", "application/json");
			httpPatch.setHeader("content-type", "application/json");
			CloseableHttpResponse response = this.client.getHttpClient().execute(httpPatch);
			ResultBuilder resultBuilder = new CommonResultBuilder();
			Result result = resultBuilder.build(response.getStatusLine().getStatusCode());
			if (response.getEntity() != null) {
				result.setMessage(IOUtils.toString(response.getEntity().getContent(), StandardCharsets.UTF_8));
			}
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} catch (ClientProtocolException e) {
			logger.error("HttpClient ClientProtocolException:", e);
			return new Result(false, e.getMessage());
		} catch (IOException e) {
			logger.error("HttpClient IOException:", e);
			return new Result(false, e.getMessage());
		} finally {
			this.client.close();
		}
	}

	public Result doDelete(String url, Map<String, Object> params) {
		try {
			client.setDataFormat(null);
			logger.debug("url: {}", url);
			logger.debug("params: {}", JSON.toJSONString(params));
			Result result = this.client.delete(buildHeader(), params, url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result doDelete(String url, Map<String, Object> params, FusionCloudApiName apiName,
			FusionCloudApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = readApiUrl(apiName, apiType);
			logger.debug("url: {}", apiUrl + url);
			logger.debug("params: {}", JSON.toJSONString(params));
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
			}
			Result result = this.client.delete(buildHeader(), params, apiUrl + url);
			logger.debug("result: {}", JSON.toJSONString(result));
			return result;
		} finally {
			this.client.close();
		}
	}

	public Result waitImageTask(String jobId) {
		try {
			client.setDataFormat(null);
			String url = readApiUrl(FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (!StringUtils.isEmpty(url) && url.contains("v2")) {
				url = url.substring(0, url.indexOf("v2") - 1);
			}
			return this.client.get(buildHeader(), null,
					url + "/v1/" + this.token.getProject().getId() + "/jobs/" + jobId);
		} finally {
			this.client.close();
		}
	}

	/**
	 * 获取api的网关地址
	 *
	 * @param apiName
	 * @param apiType
	 * @return
	 */
	private String readApiUrl(FusionCloudApiName apiName, FusionCloudApiType apiType) {
		String url = null;
		Map<String, String> apiMap = new HashMap<String, String>();
		List<FusionCloudCatalog> catalogs = this.token.getCatalog();
		if (ListTool.isEmpty(catalogs)) {
			return url;
		}
		for (FusionCloudCatalog catalog : catalogs) {
			List<FusionCloudEndpoint> endpoints = catalog.getEndpoints();
			if (ListTool.isEmpty(endpoints)) {
				continue;
			}
			apiMap.put(catalog.getName() + ":" + catalog.getType(), endpoints.get(0).getUrl());
		}
		String name = "";
		if (null != apiName) {
			name = apiName.name();
		}
		String type = "";
		if (null != type) {
			type = apiType.name();
		}
		url = apiMap.get(name + ":" + type);
		return url;
	}

	private Map<String, Object> buildHeader() {
		Map<String, Object> header = new HashMap<String, Object>();
		header.put("accept", "application/json");
		header.put("content-type", "application/json");
		header.put("X-Auth-Token", this.token.getToken());
		header.put("X-Project-id", this.token.getProject().getId());
		return header;
	}

	/**
	 * @return the endpoint
	 */
	public String getEndpoint() {
		return endpoint;
	}

	/**
	 * @param endpoint
	 *            the endpoint to set
	 */
	public void setEndpoint(String endpoint) {
		this.endpoint = endpoint;
	}

	/**
	 * @return the username
	 */
	public String getUsername() {
		return username;
	}

	/**
	 * @param username
	 *            the username to set
	 */
	public void setUsername(String username) {
		this.username = username;
	}

	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @param password
	 *            the password to set
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @return the domain
	 */
	public String getDomain() {
		return domain;
	}

	/**
	 * @param domain
	 *            the domain to set
	 */
	public void setDomain(String domain) {
		this.domain = domain;
	}

	/**
	 * @return the project
	 */
	public String getProject() {
		return project;
	}

	/**
	 * @param project
	 *            the project to set
	 */
	public void setProject(String project) {
		this.project = project;
	}

	/**
	 * @return the accessable
	 */
	public boolean isAccessable() {
		return accessable;
	}

	/**
	 * @param accessable
	 *            the accessable to set
	 */
	public void setAccessable(boolean accessable) {
		this.accessable = accessable;
	}

	/**
	 * @return the token
	 */
	public FusionCloudToken getToken() {
		return token;
	}

	/**
	 * @param token
	 *            the token to set
	 */
	public void setToken(FusionCloudToken token) {
		this.token = token;
	}

	/**
	 * @return the error
	 */
	public String getError() {
		return error;
	}

	/**
	 * @param error
	 *            the error to set
	 */
	public void setError(String error) {
		this.error = error;
	}

	public HttpClient getClient() {
		return client;
	}

	public void setClient(HttpClient client) {
		this.client = client;
	}

	public String getMonitorUrl() {
		return monitorUrl;
	}

	public void setMonitorUrl(String monitorUrl) {
		this.monitorUrl = monitorUrl;
	}

	public String getMonitorUserName() {
		return monitorUserName;
	}

	public void setMonitorUserName(String monitorUserName) {
		this.monitorUserName = monitorUserName;
	}

	public String getMonitorPwd() {
		return monitorPwd;
	}

	public void setMonitorPwd(String monitorPwd) {
		this.monitorPwd = monitorPwd;
	}

	public String getMonitorToken() {
		return monitorToken;
	}

	public void setMonitorToken(String monitorToken) {
		this.monitorToken = monitorToken;
	}

	public String getMonitorRoaRand() {
		return monitorRoaRand;
	}

	public void setMonitorRoaRand(String monitorRoaRand) {
		this.monitorRoaRand = monitorRoaRand;
	}

}