package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.FlavorConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云云主机规格操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudFlavorProvider extends FusionCloudProvider implements FlavorProvider {

	private static final Logger logger = LoggerFactory.getLogger(FusionCloudFlavorProvider.class);
	private BeanConvertor<FlavorModel, JSONObject> convertor = new FlavorConvertor();

	/**
	 * @param butler
	 */
	public FusionCloudFlavorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		try {
			Result result = doGet("/flavors/detail", null, FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("flavors"), JSONObject.class);
			if (null == objects) {
				return new BocloudResult(false, "查询失败");
			}
			List<FlavorModel> convertList = this.convertor.convertList(objects);
			List<FlavorModel> resultFavor = new ArrayList<FlavorModel>();
			for (FlavorModel flavorModel : convertList) {
				Result extraSpecs = doGet("/flavors/" + flavorModel.getFlavorUuid() + "/os-extra_specs", null,
						FusionCloudApiName.ecs, FusionCloudApiType.compute);

				JSONObject extraSpecsObj = JSONObject.parseObject(extraSpecs.getMessage());
				JSONObject jsonObject = extraSpecsObj.getJSONObject("extra_specs");
				String performancetype = null;
				String extBootType = null;
				if (null != jsonObject) {
					performancetype = jsonObject.getString("ecs:performancetype");
					extBootType = jsonObject.getString("huawei:extBootType");
				}
				if ((null != performancetype) && (null != extBootType) && ("normal".equalsIgnoreCase(performancetype))
						&& ("volume".equalsIgnoreCase(extBootType))) {
					resultFavor.add(flavorModel);
				}
			}
			return new BocloudResult(true, resultFavor, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "查询失败");
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		try {
			JSONObject object = new JSONObject();
			object.put("name", flavor.getName());
			object.put("vcpus", flavor.getCpu());
			object.put("ram", flavor.getMemory());
			object.put("disk", flavor.getDisk());
			Result result = this.doPost("/flavors", object, FusionCloudApiName.ecs, FusionCloudApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject flavorObj = JSONObject.parseObject(result.getMessage());
			return new BocloudResult(true, convertor.convertModel(flavorObj.getJSONObject("flavor")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult remove(String flavorId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		// TODO Auto-generated method stub
		return null;
	}

}
