package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.RegionConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RegionModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云可用域操作driver
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/9/21
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudRegionProvider extends FusionCloudProvider implements ZoneProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudRegionProvider.class);
    private BeanConvertor<RegionModel, JSONObject> convertor = new RegionConvertor();

    /**
     * @param butler
     */
    public FusionCloudRegionProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/os-availability-zone", null, FusionCloudApiName.ecs,
                    FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "可用分区列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("availabilityZoneInfo"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "可用分区列表为空");
            }
            List<RegionModel> models = new ArrayList<RegionModel>();
            for (JSONObject obj : objects) {
                models.add(convertor.convertModel(obj));
            }
            return new BocloudResult(true, models, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
