package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.HostAggregateConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 华为私有云服务器组操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月17日
 */
public class FusionCloudHostAggregateProvider extends FusionCloudProvider implements HostAggregateProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudHostAggregateProvider.class);
    private BeanConvertor<ClusterModel, JSONObject> convertor = new HostAggregateConvertor();

    /**
     * @param butler
     */
    public FusionCloudHostAggregateProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(ClusterModel clusterModel) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", clusterModel.getName());
            params.put("policies", clusterModel.getPolicies());
            JSONObject clusterParam = new JSONObject();
            clusterParam.put("server_group", params);
            Result result = this.doPost("/os-server-groups", clusterParam, FusionCloudApiName.ecs, FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            JSONObject jsonObject = object.getJSONObject("server_group");
            if (null == jsonObject) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, convertor.convertModel(jsonObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(ClusterModel clusterModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(String aggregateId) {
        try {
            Result result = this.doDelete("/os-server-groups/" + aggregateId, null, FusionCloudApiName.ecs, FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult addHost(String aggregateId, String hostName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removeHost(String aggregateId, String hostName) {
        // TODO Auto-generated method stub
        return null;
    }

    public BocloudResult detail(String aggregateId) {
        try {
            Result result = this.doGet("/os-server-groups/" + aggregateId, null, FusionCloudApiName.ecs, FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "主机组不存在");
            }
            JSONObject hostAggrObj = JSONObject.parseObject(object.getString("server_group"));
            if (null == hostAggrObj) {
                return new BocloudResult(false, "主机组不存在");
            }
            return new BocloudResult(true, convertor.convertModel(hostAggrObj), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

}
