package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudSnapshotProvider extends FusionCloudProvider implements SnapshotProvider {
    private static final Logger logger = LoggerFactory.getLogger(FusionCloudSnapshotProvider.class);
    private BeanConvertor<SnapshotModel, JSONObject> snapshotConvertor = new SnapshotConvertor();

    public FusionCloudSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = doGet("/snapshots", null, FusionCloudApiName.evsv2, FusionCloudApiType.volumev2);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("snapshots"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, this.snapshotConvertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
        }
        return new BocloudResult(false, "查询失败");
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            Result result = doDelete("/snapshots/" + model.getSnapshotUuid(), null, FusionCloudApiName.evsv2,
                    FusionCloudApiType.volumev2);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "删除成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
        }
        return new BocloudResult(false, "删除失败");
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            Result result = doGet("/snapshots/" + model.getSnapshotUuid(), null, FusionCloudApiName.evsv2,
                    FusionCloudApiType.volumev2);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
            return new BocloudResult(true, this.snapshotConvertor.convertModel(jsonObject), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
        }
        return new BocloudResult(false, "查询失败");
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("description", model.getRemark());
            params.put("name", model.getName());
            JSONObject volumeParam = new JSONObject();
            volumeParam.put("snapshot", params);

            Result result = doPut("/snapshots/" + model.getSnapshotUuid(), volumeParam, FusionCloudApiName.evsv2,
                    FusionCloudApiType.volumev2);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
            return new BocloudResult(true, this.snapshotConvertor.convertModel(jsonObject), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
        }
        return new BocloudResult(false, "修改失败");
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("volume_id", model.getVolumeUuid());
            jsonObject.put("name", model.getVolumeName());
            JSONObject params = new JSONObject();
            params.put("rollback", jsonObject);
            Result result = doPost("/os-vendor-snapshots/" + model.getSnapshotUuid() + "/rollback", params,
                    FusionCloudApiName.evsv2, FusionCloudApiType.volumev2);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "success");
        } catch (Exception e) {
            logger.error("Get error message:", e);
        }
        return new BocloudResult(false, "fail");
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        SnapshotModel model = new SnapshotModel();
        model.setSnapshotUuid(snapshotId);
        BocloudResult result = new BocloudResult(false, "操作失败");
        Boolean flag = Boolean.valueOf(true);
        Long startTime = Long.valueOf(System.currentTimeMillis());
        while (flag.booleanValue()) {
            try {
                Thread.sleep(5000L);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            result = detail(model);
            if (result.isFailed()) {
                flag = Boolean.valueOf(false);
                break;
            }
            SnapshotModel snapshotModel = (SnapshotModel) JSONObject
                    .parseObject(JSONObject.toJSONString(result.getData()), SnapshotModel.class);
            if (null == snapshotModel) {
                flag = Boolean.valueOf(false);
                result = new BocloudResult(false, "fail");
                break;
            }
            if (!"creating".equalsIgnoreCase(snapshotModel.getStatus())) {
                flag = Boolean.valueOf(false);
                break;
            }
            Long endTime = Long.valueOf(System.currentTimeMillis());
            if (endTime.longValue() - startTime.longValue() > 900000L) {
                flag = Boolean.valueOf(false);
                model.setStatus("EXCEPTION");
                break;
            }
        }
        return result;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        return null;
    }
}
