package com.bocloud.cmp.provider.loadbalance;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.LbConvertor;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionCloudLbProvider
 * @Description:负载均衡
 * @author: zhenpingLi/李振平
 * @date: 2020/2/27 11:06
 */
public class FusionCloudLbProvider extends FusionCloudProvider implements LoadBalanceProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudLbProvider.class);
    private BeanConvertor<LbModel, JSONObject> convertor = new LbConvertor();

    /**
     * @param butler
     */
    public FusionCloudLbProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            //todo FusionCloudApiName FusionCloudApiType
            Result result = this.doGet("/v2.0/lbaas/loadbalancers", null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "Please check the network connection!");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("loadbalancers"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "负载均衡列表为空");
            }
            return new BocloudResult(true, convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(LbModel lb) {
        return null;
    }

    @Override
    public BocloudResult update(LbModel lb) {
        return null;
    }

    @Override
    public BocloudResult remove(String lbId) {
        return null;
    }

    @Override
    public BocloudResult detail(String lbId) {
        return null;
    }

    @Override
    public BocloudResult listListener() {
        return null;
    }

    @Override
    public BocloudResult listListener(String lbId) {
        return null;
    }

    @Override
    public BocloudResult createListener(LbListenerModel lbListenerModel) {
        return null;
    }

    @Override
    public BocloudResult updateListener(LbListenerModel lbListenerModel) {
        return null;
    }

    @Override
    public BocloudResult removeListener(String listenerId) {
        return null;
    }

    @Override
    public BocloudResult detailListener(String listenerId) {
        return null;
    }

    @Override
    public BocloudResult listHealMonitor() {
        return null;
    }

    @Override
    public BocloudResult createHealMonitor(HealthMonitorModel hmModel) {
        return null;
    }

    @Override
    public BocloudResult updateHealMonitor(HealthMonitorModel hmModel) {
        return null;
    }

    @Override
    public BocloudResult removeHealMonitor(String hmId) {
        return null;
    }

    @Override
    public BocloudResult detailHealMonitor(String hmId) {
        return null;
    }

    @Override
    public BocloudResult listMember(String poolId) {
        return null;
    }

    @Override
    public BocloudResult createMember(LbMemberModel mbModel) {
        return null;
    }

    @Override
    public BocloudResult updateMember(LbMemberModel mbModel) {
        return null;
    }

    @Override
    public BocloudResult removeMember(String poolId, String mbId) {
        return null;
    }

    @Override
    public BocloudResult detailMember(String poolId, String mbId) {
        return null;
    }

    @Override
    public BocloudResult listPool() {
        return null;
    }

    @Override
    public BocloudResult createPool(LbPoolModel poolModel) {
        return null;
    }

    @Override
    public BocloudResult removePool(String poolId) {
        return null;
    }

    @Override
    public BocloudResult updatePool(LbPoolModel poolModel) {
        return null;
    }

    @Override
    public BocloudResult detailPool(String poolId) {
        return null;
    }
}
