package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.FloatingIpConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 华为私有云浮动IP操作driver
 *
 * @Date: Created in 2018/9/20
 * @Author: guopeng
 * @Description:
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudFloatingIPProvider extends FusionCloudProvider implements FloatingIPProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudFloatingIPProvider.class);
    private BeanConvertor<FloatingIpModel, JSONObject> convertor = new FloatingIpConvertor();

    /**
     * @param butler
     */
    public FusionCloudFloatingIPProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/v2.0/floatingips", null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "浮动IP列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("floatingips"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "浮动IP列表为空");
            }
            return new BocloudResult(true, convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(FloatingIpModel floatingIpModel) {
        try {
            JSONObject params = new JSONObject();
            params.put("floating_network_id", floatingIpModel.getFloatingNetworkUuid());
            params.put("port_id", floatingIpModel.getPortUuid());
            JSONObject floatingIpParams = new JSONObject();
            floatingIpParams.put("floatingip", params);
            Result result = this.doPost("/v2.0/floatingips", floatingIpParams, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "Failed to create floatingIp ,Please check the network connection!");
            }
            JSONObject jsonObject = object.getJSONObject("floatingip");
            if (null == jsonObject) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, convertor.convertModel(jsonObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult modify(FloatingIpModel floatingIpModel) {
        try {
            JSONObject params = new JSONObject();
            params.put("port_id", floatingIpModel.getPortUuid());
            JSONObject floatingIpParams = new JSONObject();
            floatingIpParams.put("floatingip", params);
            Result result = this.doPut("/v2.0/floatingips/" + floatingIpModel.getFloatingIpUuid(), floatingIpParams,
                    FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "修改失败");
            }
            JSONObject jsonObject = object.getJSONObject("floatingip");
            if (null == jsonObject) {
                return new BocloudResult(false, "修改失败");
            }
            return new BocloudResult(true, convertor.convertModel(jsonObject), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult remove(String fipId) {
        try {
            Result result = this.doDelete("/v2.0/floatingips/" + fipId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String fipId) {
        return null;
    }

    @Override
    public BocloudResult associate(String fipId, String portId) {
        try {
            JSONObject params = new JSONObject();
            params.put("port_id", portId);
            JSONObject floatingIpParams = new JSONObject();
            floatingIpParams.put("floatingip", params);
            Result result = this.doPut("/v2.0/floatingips/" + fipId, floatingIpParams, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "Failed to associate floatingIp ,Please check the network connection!");
            }
            JSONObject jsonObject = object.getJSONObject("floatingip");
            if (null == jsonObject) {
                return new BocloudResult(false, "分配失败");
            }
            return new BocloudResult(true, this.convertor.convertModel(jsonObject), "floatingIp associate success!");
        } catch (Exception e) {
            logger.error("floatingIp associate error:", e.getMessage());
            return new BocloudResult(false, "floatingIp associate error!");
        }
    }

    @Override
    public BocloudResult disassociate(String fipId) {
        try {
            JSONObject params = new JSONObject();
            // port_id 为空为解绑
            params.put("port_id", "");
            JSONObject floatingIpParams = new JSONObject();
            floatingIpParams.put("floatingip", params);
            Result result = this.doPut("/v2.0/floatingips/" + fipId, floatingIpParams,
                    FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "Failed to disassociate floatingIp ,Please check the network connection!");
            }
            JSONObject jsonObject = object.getJSONObject("floatingip");
            if (null == jsonObject) {
                return new BocloudResult(false, "解绑失败失败");
            }
            return new BocloudResult(true, this.convertor.convertModel(jsonObject), "floatingIp disassociate success!");
        } catch (Exception e) {
            logger.error("floatingIp disassociate error:", e.getMessage());
            return new BocloudResult(false, "floatingIp disassociate error!");
        }
    }

    @Override
    public BocloudResult associate(FloatingIpModel modeld) {
        // TODO Auto-generated method stub
        return null;
    }
}
