package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.NetworkConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云网络操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudNetworkProvider extends FusionCloudProvider implements NetworkProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudNetworkProvider.class);
    private BeanConvertor<NetworkModel, JSONObject> convertor = new NetworkConvertor();

    private Butler butler;

    /**
     * @param butler
     */
    public FusionCloudNetworkProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = doGet("/v2.0/networks", null, FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询失败");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("networks"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "查询失败");
            }
            List<JSONObject> currentNets = new ArrayList<JSONObject>();
            for (JSONObject jsonObject : objects) {
                String tenantId = jsonObject.getString("tenant_id");
                Boolean shared = jsonObject.getBoolean("shared");
                if (Boolean.TRUE.equals(shared) || this.butler.getProjectId().equals(tenantId)) {
                    currentNets.add(jsonObject);
                }
            }
            return new BocloudResult(true, this.convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
        return new BocloudResult(false, "查询失败");
    }

    @Override
    public BocloudResult create(NetworkModel model) {
        try {
            JSONObject network = new JSONObject();
            JSONObject param = new JSONObject();
            param.put("tenant_id", this.getProject());
            param.put("shared", model.getShared());
            param.put("name", model.getName());
            param.put("admin_state_up", model.getAdminStateup());
            network.put("network", param);
            Result result = this.doPost("/v2.0/networks", network, FusionCloudApiName.vpc, FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, convertor.convertModel(object.getJSONObject("network")), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");

        }
    }

    @Override
    public BocloudResult remove(String networkId) {
        try {
            BocloudResult bocloudResult = this.detail(networkId);
            if (null == bocloudResult.getData()) {
                return new BocloudResult(true, "删除成功");
            }
            Result result = this.doDelete("/v2.0/networks/" + networkId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(String networkId) {
        try {
            Result result = this.doGet("/v2.0/networks/" + networkId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "路由不存在");
            }
            return new BocloudResult(true, convertor.convertModel(object.getJSONObject("network")), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult modify(NetworkModel model) {
        try {
            JSONObject params = new JSONObject();
            JSONObject network = new JSONObject();
            network.put("name", model.getName());
            network.put("shared", model.getShared());
            network.put("admin_state_up", model.getAdminStateup());
            params.put("network", network);
            Result result = this.doPut("/v2.0/networks/" + model.getNetworkUuid(), params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject routerObject = JSONObject.parseObject(result.getMessage(), JSONObject.class);
            return new BocloudResult(true, convertor.convertModel(routerObject.getJSONObject("network")), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "修改失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult createDSwitch(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createVSwitch(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removeDSwitch(String name) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult renameDSwitch(String oldName, String newName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updateDVSwitch(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult removePortGroup(String hostName, String pgName, String type) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult addHost(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult listVSwitch(SwitchModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult listProfile() {
        // TODO Auto-generated method stub
        return null;
    }

}
