package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 华为私有云安全组操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudSecurityGroupProvider extends FusionCloudProvider implements SecurityGroupProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudSecurityGroupProvider.class);

    private BeanConvertor<SecurityGroupModel, JSONObject> convertor = new SecurityGroupConvertor();

    /**
     * @param butler
     */
    public FusionCloudSecurityGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        Result result = this.doGet("/v2.0/security-groups", null, FusionCloudApiName.vpc, FusionCloudApiType.network);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        JSONObject object = JSONObject.parseObject(result.getMessage());
        if (null == object) {
            return new BocloudResult(false, "安全组列表为空");
        }
        List<JSONObject> objects = JSONArray.parseArray(object.getString("security_groups"), JSONObject.class);
        if (null == objects) {
            return new BocloudResult(false, "硬盘列表为空");
        }
        return new BocloudResult(true, convertor.convertList(objects), "查询成功");
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            JSONObject groupParam = new JSONObject();
            groupParam.put("name", group.getName());
            groupParam.put("description", group.getRemark());
            JSONObject params = new JSONObject();
            params.put("security_group", groupParam);
            Result result = this.doPost("/v2.0/security-groups", params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("security_group");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            Result result = this.doDelete("/v2.0/security-groups/" + sgId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败！");
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        try {
            Result result = this.doGet("/v2.0/security-groups/" + sgId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("security_group");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败！");
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            JSONObject groupParam = new JSONObject();
            groupParam.put("name", model.getName());
            groupParam.put("description", model.getRemark());
            JSONObject params = new JSONObject();
            params.put("security_group", groupParam);
            Result result = this.doPut("/v2.0/security-groups/" + model.getGroupUuid(), params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("security_group");
            return new BocloudResult(true, convertor.convertModel(jsonObject), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败！");
        }
    }

}
