package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 华为私有云安全组规则操作driver
 *
 * @author ymh
 * @date 2018年9月21日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudSecurityGroupRuleProvider extends FusionCloudProvider implements SecurityGroupRuleProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudSecurityGroupRuleProvider.class);

    /**
     * @param butler
     */
    public FusionCloudSecurityGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        try {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("security_group_id", model.getGroupUuid());
            jsonObject.put("protocol", model.getProtocol());
            jsonObject.put("direction", model.getDirection());
            jsonObject.put("remote_ip_prefix", model.getRemoteIpPrefix());
            jsonObject.put("remote_group_id", model.getRemoteGroupUuid());
            jsonObject.put("port_range_max", model.getPortMax());
            jsonObject.put("port_range_min", model.getPortMin());
            JSONObject params = new JSONObject();
            params.put("security_group_rule", jsonObject);
            Result result = this.doPost("/v2.0/security-group-rules", params, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject groupRule = JSONObject.parseObject(result.getMessage()).getJSONObject("security_group_rule");
            SecurityGroupRuleModel rule = new SecurityGroupRuleModel();
            rule.setName(groupRule.getString("tenant_id"));
            rule.setPortMax(groupRule.getInteger("port_range_max"));
            rule.setPortMin(groupRule.getInteger("port_range_min"));
            if (null != groupRule.getString("protocol")) {
                rule.setProtocol(groupRule.getString("protocol").toUpperCase());
            }
            rule.setRuleUuid(groupRule.getString("id"));
            rule.setGroupUuid(groupRule.getString("security_group_id"));
            rule.setDirection(groupRule.getString("direction"));
            rule.setRemoteIpPrefix(groupRule.getString("remote_ip_prefix"));
            rule.setEtherType(groupRule.getString("ethertype"));
            rule.setRemoteGroupUuid(groupRule.getString("remote_group_id"));
            return new BocloudResult(true, rule, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult remove(String ruleId) {
        try {
            Result result = this.doDelete("/v2.0/security-group-rules/" + ruleId, null, FusionCloudApiName.vpc,
                    FusionCloudApiType.network);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, result.getMessage(), "删除成功！");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败！");
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
        return null;
    }

}
