package com.bocloud.cmp.provider.security;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.FusionCloudApiName;
import com.bocloud.cmp.common.enums.FusionCloudApiType;
import com.bocloud.cmp.fusioncloudconvertor.KeypairConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.FusionCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为私有云密钥对操作driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.fusioncloud")
public class FusionCloudKeypairProvider extends FusionCloudProvider implements KeypairProvider {

    private static final Logger logger = LoggerFactory.getLogger(FusionCloudKeypairProvider.class);
    private BeanConvertor<KeypairModel, JSONObject> convertor = new KeypairConvertor();

    /**
     * @param butler
     */
    public FusionCloudKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/os-keypairs", null, FusionCloudApiName.ecs, FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "密钥列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("keypairs"), JSONObject.class);
            if (null == objects) {
                return new BocloudResult(false, "密钥列表为空");
            }
            List<KeypairModel> models = new ArrayList<KeypairModel>();
            for (JSONObject obj : objects) {
                models.add(convertor.convertModel(obj.getJSONObject("keypair")));
            }
            return new BocloudResult(true, models, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        try {
            Result result = this.doGet("/os-keypairs/" + keypairName, null, FusionCloudApiName.ecs,
                    FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            JSONObject keypairObj = object.getJSONObject("keypair");
            return new BocloudResult(true, convertor.convertModel(keypairObj), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        try {
            JSONObject params = new JSONObject();
            JSONObject keypair = new JSONObject();
            keypair.put("name", keypairName);
            keypair.put("public_key", publicKey);
            params.put("keypair", keypair);
            Result result = this.doPost("/os-keypairs", params, FusionCloudApiName.ecs, FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            JSONObject keypairObj = object.getJSONObject("keypair");
            return new BocloudResult(true, convertor.convertModel(keypairObj), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            Result result = this.doDelete("/os-keypairs/" + keypairName, null, FusionCloudApiName.ecs,
                    FusionCloudApiType.compute);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "删除失败");
        }
    }

	@Override
	public BocloudResult modify(KeypairModel model) {
		// TODO Auto-generated method stub
		return null;
	}

}
