package com.bocloud.cmp.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServerConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 17:56
 */
public class ServerConvertor implements BeanJsonConvertor<ServerModel> {

    @Override
    public ServerModel convertModel(JSONObject object) {
        ServerModel model = new ServerModel();
        model.setInstanceId(object.getString("urn"));
        model.setUri(object.getString("uri"));
        model.setName(object.getString("name"));
        model.setArch(object.getString("arch"));
        model.setRemark(object.getString("description"));
        model.setHostUuid(object.getString("hostUrn"));
        model.setClusterUuid(object.getString("clusterUrn"));
        model.setDatastores(object.getString("dataStoreUrns"));
        model.setIsBindingHost(object.getBoolean("isBindingHost"));

        //光驱状态
        model.setCdRomStatus(object.getString("cdRomStatus"));
        model.setStatus(status(object.getString("status")));
        model.setIsTemplate(object.getBoolean("isTemplate"));
        model.setToolsInstalled(!object.getString("toolInstallStatus").equalsIgnoreCase("empty"));
        model.setToolsRunning(object.getString("toolInstallStatus").equalsIgnoreCase("fill"));
        //OsName  OsVersion 系统盘大小 系统盘类 操作系统类型
        Optional.ofNullable(object.getJSONObject("osOptions")).ifPresent(osOptions -> {
            model.setOsCategory(osOptions.getString("osType"));
            model.setOsVersion(osOptions.getString("osVersion"));
            model.setOsName(osOptions.getString("guestOSName"));
            model.setPassword(osOptions.getString("password"));
        });

        Optional.ofNullable(object.getJSONObject("vmConfig")).ifPresent(vmConfig -> {
            Optional.ofNullable(vmConfig.getJSONObject("cpu")).ifPresent(cpu -> {
                model.setCpu(cpu.getInteger("quantity"));
            });
            Optional.ofNullable(vmConfig.getJSONObject("memory")).ifPresent(memory -> {
                model.setMemory(memory.getBigDecimal("quantityMB").divide(new BigDecimal("1024"), 0, BigDecimal.ROUND_HALF_UP));
            });
            Optional.ofNullable(vmConfig.getJSONArray("disks"))
                    .map(disks -> disks
                            .stream()
                            .map(JSONObject::toJSONString)
                            .map(JSONObject::parseObject)
                            .map(disk -> {
                                VolumeModel volumeModel = new VolumeModel();
                                volumeModel.setName(disk.getString("diskName"));
                                volumeModel.setVolumeUuid(disk.getString("volumeUrn"));
                                //拼接uri
                                String volumeUrn = disk.getString("volumeUrn");
                                String volumeUri = volumeUrn.replaceAll(":", "/");
                                String uri = volumeUri.replaceFirst("urn", "/service");
                                volumeModel.setUri(uri);
                                //拼接uri
                                volumeModel.setSize(disk.getInteger("quantityGB"));
                                volumeModel.setPciType(disk.getString("pciType"));
                                volumeModel.setSequenceNum(disk.getInteger("sequenceNum"));
                                volumeModel.setIoMode(disk.getString("ioMode"));
                                volumeModel.setAccessMode(disk.getInteger("accessMode"));
                                volumeModel.setDataStoreUuid(disk.getString("datastoreUrn"));
                                volumeModel.setBootVolume(disk.getBoolean("systemVolume"));
                                volumeModel.setVolumeType(disk.getString("volType"));//磁盘配置模式 0：普通卷 1：延迟置零卷
                                volumeModel.setType(disk.getString("type"));//卷类型，取值为normal（普通卷）、share（共享卷）；
                                volumeModel.setThinlyProvisioned(disk.getBoolean("isThin"));
                                return volumeModel;
                            }).collect(Collectors.toList())
                    ).ifPresent(model::setDiskDevices);
            Optional.ofNullable(vmConfig.getJSONArray("nics"))
                    .map(nics -> nics
                            .stream()
                            .map(JSONObject::toJSONString)
                            .map(JSONObject::parseObject)
                            .map(nic -> {
                                NetworkCardModel networkCardModel = new NetworkCardModel();
                                networkCardModel.setName(nic.getString("name"));
                                networkCardModel.setNetworkcardUuid(nic.getString("urn"));
                                networkCardModel.setUri(nic.getString("uri"));
                                networkCardModel.setPortGroupUuid(nic.getString("portGroupUrn"));
                                networkCardModel.setMac(nic.getString("mac"));
                                networkCardModel.setIpAddress(nic.getString("ip"));
                                networkCardModel.setType(nic.getString("virtIo"));
                                networkCardModel.setSequenceNum(nic.getInteger("sequenceNum"));
                                networkCardModel.setEnableSecurityGroup(nic.getBoolean("enableSecurityGroup"));
                                networkCardModel.setSecurityGroupUuid(nic.getString("securityGroupId"));
                                Optional.ofNullable(nic.getJSONObject("nicConfig")).ifPresent(nicConfig -> {
                                    networkCardModel.setVringbuf(nicConfig.getInteger("vringbuf"));
                                    networkCardModel.setQueues(nicConfig.getInteger("queues"));
                                });
                                return networkCardModel;
                            }).collect(Collectors.toList())
                    ).ifPresent(model::setNetworkCards);
        });

        //二次回填设置系统盘和数据盘,系统盘类型
        int diskSize = 0;
        int systemDiskSize = 0;
        String systemDiskCategory = "normal";
        for (VolumeModel volumeModel : model.getDiskDevices()) {
            if (volumeModel.getBootVolume()) {
                systemDiskSize += volumeModel.getSize();
                systemDiskCategory = volumeModel.getType();
            } else {
                diskSize += volumeModel.getSize();
            }
        }
        model.setSystemDiskCategory(systemDiskCategory);//系统盘类型
        model.setSystemDiskSize(systemDiskSize);//系统盘大小
        model.setDataDisk(diskSize);//数据盘大小
        model.setDisk(diskSize + systemDiskSize);//总共磁盘大小

        //二次回填ip信息
        if (!model.getIsTemplate()) {
            List<String> privateIps = model.getNetworkCards().stream().map(networkCardModel -> networkCardModel.getIpAddress()).collect(Collectors.toList());
            model.setPrivateIps(JSONObject.toJSONString(privateIps));
        } else {
            List<String> privateIps = new ArrayList<>();
            for (NetworkCardModel networkCardModel : model.getNetworkCards()) {
                privateIps.add("0.0.0.0");
            }
            model.setPrivateIps(JSONObject.toJSONString(privateIps));
        }
        //二次回填安全组信息
        List<String> securityGroups = new ArrayList<>();
        model.getNetworkCards().stream().forEach(cardModel -> {
            if (null != cardModel.getSecurityGroupUuid()) {
                securityGroups.add(cardModel.getSecurityGroupUuid());
            }
        });
        model.setSecurityGroups(securityGroups);

        object.getString("createTime");
        object.getJSONObject("vncAcessInfo");
        object.getJSONObject("vmRebootConfig");
        object.getString("group");
        object.getString("location");
        object.getJSONArray("dataStoreUrns");
        return model;
    }

    @Override
    public ServerModel convertModelParams(JSONObject object, String... params) {
        ServerModel model = new ServerModel();
        model.setInstanceId(object.getString("urn"));
        model.setUri(object.getString("uri"));
        model.setName(object.getString("name"));
        object.getString("arch");
        model.setRemark(object.getString("description"));
        object.getString("group");
        object.getString("location");
        model.setHostUuid(object.getString("hostUrn"));
        model.setClusterUuid(object.getString("clusterUrn"));
        object.getJSONArray("dataStoreUrns");
        model.setStatus(status(object.getString("status")));
        model.setIsTemplate(object.getBoolean("isTemplate"));
        model.setToolsInstalled(!object.getString("toolInstallStatus").equalsIgnoreCase("empty"));
        model.setToolsRunning(object.getString("toolInstallStatus").equalsIgnoreCase("fill"));
//        //OsName  OsVersion 系统盘大小 系统盘类 操作系统类型
//        Optional.ofNullable(object.getJSONObject("osOptions")).ifPresent(osOptions -> {
//            model.setOsCategory(osOptions.getString("osType"));
//            model.setOsVersion(osOptions.getString("osVersion"));
//            model.setOsName(osOptions.getString("guestOSName"));
//            model.setPassword(osOptions.getString("password"));
//        });
//
//        Optional.ofNullable(object.getJSONObject("vmConfig")).ifPresent(vmConfig -> {
//            Optional.ofNullable(vmConfig.getJSONObject("cpu")).ifPresent(cpu -> {
//                model.setCpu(cpu.getInteger("quantity"));
//            });
//            Optional.ofNullable(vmConfig.getJSONObject("Memory")).ifPresent(memory -> {
//                model.setMemory(memory.getBigDecimal("quantityMB"));
//            });
//            Optional.ofNullable(vmConfig.getJSONArray("disks"))
//                    .map(disks -> disks
//                            .stream()
//                            .map(JSONObject::toJSONString)
//                            .map(JSONObject::parseObject)
//                            .map(disk -> {
//                                VolumeModel volumeModel = new VolumeModel();
//                                volumeModel.setName(disk.getString("diskName"));
//                                volumeModel.setUri(disk.getString("volumeUrl"));
//                                volumeModel.setVolumeUuid(disk.getString("volumeUrn"));
//                                volumeModel.setSize(disk.getInteger("quantityGB"));
//                                volumeModel.setPciType(disk.getString("pciType"));
//                                volumeModel.setSequenceNum(disk.getInteger("sequenceNum"));
//                                volumeModel.setIoMode(disk.getString("ioMode"));
//                                volumeModel.setAccessMode(disk.getInteger("accessMode"));
//                                volumeModel.setDataStoreUuid(disk.getString("datastoreUrn"));
//                                volumeModel.setBootVolume(disk.getBoolean("systemVolume"));
//                                volumeModel.setVolumeType(disk.getString("volType"));//磁盘配置模式 0：普通卷 1：延迟置零卷
//                                volumeModel.setType(disk.getString("type"));//卷类型，取值为normal（普通卷）、share（共享卷）；
//                                return volumeModel;
//                            }).collect(Collectors.toList())
//                    ).ifPresent(model::setDiskDevices);
//            Optional.ofNullable(vmConfig.getJSONArray("nics"))
//                    .map(nics -> nics
//                            .stream()
//                            .map(JSONObject::toJSONString)
//                            .map(JSONObject::parseObject)
//                            .map(nic -> {
//                                NetworkCardModel networkCardModel = new NetworkCardModel();
//                                networkCardModel.setName(nic.getString("name"));
//                                networkCardModel.setNetworkcardUuid(nic.getString("urn"));
//                                networkCardModel.setUri(nic.getString("uri"));
//                                networkCardModel.setPortGroupUuid(nic.getString("portGroupUrn"));
//                                networkCardModel.setMac(nic.getString("mac"));
//                                networkCardModel.setIpAddress(nic.getString("ip"));
//                                networkCardModel.setType(nic.getString("virtIo"));
//                                networkCardModel.setSequenceNum(nic.getInteger("sequenceNum"));
//                                networkCardModel.setEnableSecurityGroup(nic.getBoolean("enableSecurityGroup"));
//                                networkCardModel.setSecurityGroupUuid(nic.getString("securityGroupId"));
//                                networkCardModel.setVringburf(nic.getInteger("vringburf"));
//                                networkCardModel.setQueues(nic.getInteger("queues"));
//                                return networkCardModel;
//                            }).collect(Collectors.toList())
//                    ).ifPresent(model::setNetworkCards);
//        });
//
//        //二次回填设置系统盘和数据盘,系统盘类型
//        int diskSize = 0;
//        int systemDiskSize = 0;
//        String systemDiskCategory = "normal";
//        for (VolumeModel volumeModel : model.getDiskDevices()) {
//            if (volumeModel.getBootVolume()) {
//                systemDiskSize += volumeModel.getSize();
//                systemDiskCategory = volumeModel.getType();
//            } else {
//                diskSize += volumeModel.getSize();
//            }
//        }
//        model.setSystemDiskCategory(systemDiskCategory);//系统盘类型
//        model.setSystemDiskSize(systemDiskSize);//系统盘大小
//        model.setDataDisk(diskSize);//数据盘大小
//        model.setDisk(diskSize + systemDiskSize);//总共磁盘大小
//
//        //二次回填ip信息
//        List<String> privateIps = model.getNetworkCards().stream().map(networkCardModel -> networkCardModel.getIpAddress()).collect(Collectors.toList());
//        model.setPrivateIps(JSONObject.toJSONString(privateIps));
//
//        object.getString("createTime");
//        object.getJSONObject("vncAcessInfo");
//        object.getJSONObject("vmRebootConfig");
        if (null != params && 0 != params.length) {
            String vdcUri = params[0];
            model.setVdcUri(vdcUri);
        }
        return model;
    }

    private String status(String status) {
        String newStatus;
        switch (status) {
            case "running":
                newStatus = "RUNNING";
                break;
            case "stopped":
                newStatus = "STOPPED";
                break;
            case "unknown":
                newStatus = "UNKNOWN";
                break;
            case "hibernated":
                newStatus = "HIBERNATED";
                break;
            case "creating":
                newStatus = "BUILDING";
                break;
            case "shutting-down":
                newStatus = "DELETING";
                break;
            case "migrating":
                newStatus = status;
                break;
            case "fault-resuming":
                newStatus = status;
                break;
            case "starting":
                newStatus = "STARTING";
                break;
            case "stopping":
                newStatus = "STOPPING";
                break;
            case "hibernating":
                newStatus = "HIBERNATING";
                break;
            case "pause":
                newStatus = "PAUSED";
                break;
            case "recycling":
                newStatus = status;
                break;
            default:
                newStatus = "ERROR";
                break;
        }
        return newStatus;
    }

}