package com.bocloud.cmp.convertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;

import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @ClassName: SnapshotConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/10/28 15:19
 */
public class SnapshotConvertor implements BeanJsonConvertor<SnapshotModel> {
    @Override
    public SnapshotModel convertModelParams(JSONObject object, String... params) {
        SnapshotModel model = new SnapshotModel();
        Optional.ofNullable(object.getString("urn")).ifPresent(model::setSnapshotUuid);
        Optional.ofNullable(object.getString("uri")).ifPresent(model::setUri);
        Optional.ofNullable(object.getString("name")).ifPresent(model::setName);
        Optional.ofNullable(object.getString("description")).ifPresent(model::setRemark);
        Optional.ofNullable(object.getString("type").toUpperCase()).ifPresent(model::setSnapshotType);
        Optional.ofNullable(object.getString("status")).ifPresent(model::setStatus);
        Long snapProvisionSize = object.getLong("snapProvisionSize");
        if (null != snapProvisionSize && snapProvisionSize != 0) {
            Long l = snapProvisionSize / 1024L;
            model.setSize(l.intValue());
        }else {
            model.setSize(0);
        }
        if (null != params && params.length != 0) {
            String parentSnapshoutUuid = params[0];
            model.setParentSnapshotUuid(parentSnapshoutUuid);
        }
        JSONArray childSnapshots = object.getJSONArray("childSnapshots");
        Optional.ofNullable(childSnapshots).map(
                child -> convertListParams(child, object.getString("urn"))
        ).ifPresent(model::setChildSnapshots);

        return model;
    }
}
