package com.bocloud.cmp.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereUpLinkModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereVlanPoolModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;

import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SwitchConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/25 11:53
 */
public class SwitchConvertor implements BeanJsonConvertor<SwitchModel> {

    @Override
    public SwitchModel convertModelParams(JSONObject object, String... params) {
        SwitchModel model = new SwitchModel();
        Optional.ofNullable(object.getString("urn")).ifPresent(model::setSwitcherVal);
        Optional.ofNullable(object.getString("uri")).ifPresent(model::setUri);
        Optional.ofNullable(object.getString("name")).ifPresent(model::setName);
        Optional.ofNullable(object.getString("description")).ifPresent(model::setRemark);
        Optional.ofNullable(object.getString("type")).ifPresent(model::setType);
        Optional.ofNullable(object.getBoolean("isIgmpSnooping")).ifPresent(model::setIsIgmpSnooping);
        Optional.ofNullable(object.getInteger("mtu")).ifPresent(model::setMtu);
        Optional.ofNullable(object.getInteger("portGroupNum")).ifPresent(model::setPortGroupNum);
        if (null!=params && 0 != params.length) {
            String vdcUri = params[0];
            model.setVdcUri(vdcUri);
        }
        return model;
    }

    @Override
    public SwitchModel convertModel(JSONObject object) {
        SwitchModel model = new SwitchModel();
        Optional.ofNullable(object.getString("urn")).ifPresent(model::setSwitcherVal);
        Optional.ofNullable(object.getString("uri")).ifPresent(model::setUri);
        Optional.ofNullable(object.getString("name")).ifPresent(model::setName);
        Optional.ofNullable(object.getString("description")).ifPresent(model::setRemark);
        Optional.ofNullable(object.getString("type")).ifPresent(model::setType);
        Optional.ofNullable(object.getBoolean("isIgmpSnooping")).ifPresent(model::setIsIgmpSnooping);
        Optional.ofNullable(object.getInteger("mtu")).ifPresent(model::setMtu);

        Optional.ofNullable(object.getJSONArray("hostPortSet"))
                .map(resources -> resources
                        .stream()
                        .map(JSONObject::toJSONString)
                        .map(JSONObject::parseObject)
                        .map(resource -> {
                            FusionSphereUpLinkModel sonModel = new FusionSphereUpLinkModel();
                            sonModel.setHostUuid(resource.getString("hostUrn"));
                            sonModel.setHostName(resource.getString("hostName"));
                            sonModel.setPortUuid(resource.getString("portUrn"));
                            sonModel.setPortName(resource.getString("portName"));
                            sonModel.setStatus(resource.getString("status"));
                            return sonModel;
                        }).collect(Collectors.toList())
                ).ifPresent(model::setUpLinkModels);

        Optional.ofNullable(object.getJSONArray("vlanPoolSet"))
                .map(resources -> resources
                        .stream()
                        .map(JSONObject::toJSONString)
                        .map(JSONObject::parseObject)
                        .map(resource -> {
                            FusionSphereVlanPoolModel sonModel = new FusionSphereVlanPoolModel();
                            sonModel.setStartVlanId(resource.getInteger("startVlanId"));
                            sonModel.setEndVlanId(resource.getInteger("endVlanId"));
                            return sonModel;
                        }).collect(Collectors.toList())
                ).ifPresent(model::setVlanPoolModels);
        return model;
    }
}