package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.ClusterConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereClusterProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 13:52
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereClusterProvider extends FusionSphereRestApiProvider implements ClusterProvider {

    public FusionSphereClusterProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<ClusterModel> convertor = new ClusterConvertor();

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            String api = "%s/clusters";
            String url = String.format(api, siteUri);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<ClusterModel> models = convertor.convertListParams(jsonObject.getJSONArray("clusters"), siteUri).stream().collect(Collectors.toList());
            List<ClusterModel> newModels = new ArrayList<>();
            for (ClusterModel clusterModel : models) {
                String detailResponse = this.request(RequestStatus.GET, clusterModel.getUri(), null);
                JSONObject detailObject = JSONObject.parseObject(detailResponse);
                ClusterModel newModel = convertor.convertModel(detailObject);
                newModel.setVdcUri(siteUri);
                newModels.add(newModel);
            }
            return new BocloudResult(true, newModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ClusterModel model) {
        try {
            String api = "%s/clusters";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("arch", model.getArch());//必选
            params.put("parentObjUrn", model.getParentObjUrn());//必选
            params.put("description", model.getRemark());
            params.put("tag", model.getTags());
            params.put("isMemOvercommit", model.getIsMemOvercommit());
            params.put("isEnableHa", model.getIsEnableHa());
            if (null != model.getIsEnableHa() && model.getIsEnableHa()) {
                // params.put("haSetting",model.getArch());
            }
            params.put("isEnableDrs", model.getIsEnableDrs());
            if (null != model.getIsEnableDrs() && model.getIsEnableDrs()) {
                // params.put("drsSetting",model.getArch());
            }
            params.put("isEnableImc", model.getIsEnableImc());
            if (null != model.getIsEnableImc() && model.getIsEnableImc()) {
                // params.put("imcSetting",model.getArch());
            }
            params.put("resStrategy", model.getResStrategy());
            params.put("enableGuestNuma", model.getEnableGuestNuma());
            params.put("enableVmDrs", model.getEnableVmDrs());
            params.put("enableHostNumaDRS", model.getEnableHostNumaDRS());
            params.put("enableIOTailor", model.getEnableIoTailor());
            params.put("isAutoAdjustNuma", model.getIsAutoAdjustNuma());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);

            model.setClusterVal(jsonObject.getString("urn"));
            model.setUri(jsonObject.getString("uri"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ClusterModel model) {
        try {
            String url = model.getUri();
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("arch", model.getArch());
            params.put("parentObjUrn", model.getParentObjUrn());
            params.put("description", model.getRemark());
            params.put("tag", model.getTags());
            params.put("isMemOvercommit", model.getIsMemOvercommit());
            params.put("isEnableHa", model.getIsEnableHa());
            if (null != model.getIsEnableHa() && model.getIsEnableHa()) {
                // params.put("haSetting",model.getArch());
            }
            params.put("isEnableDrs", model.getIsEnableDrs());
            if (null != model.getIsEnableDrs() && model.getIsEnableDrs()) {
                // params.put("drsSetting",model.getArch());
            }
            params.put("isEnableImc", model.getIsEnableImc());
            if (null != model.getIsEnableImc() && model.getIsEnableImc()) {
                // params.put("imcSetting",model.getArch());
            }
            params.put("resStrategy", model.getResStrategy());
            params.put("enableGuestNuma", model.getEnableGuestNuma());
            params.put("enableVmDrs", model.getEnableVmDrs());
            params.put("enableHostNumaDRS", model.getEnableHostNumaDRS());
            params.put("enableIOTailor", model.getEnableIoTailor());
            params.put("isAutoAdjustNuma", model.getIsAutoAdjustNuma());
            this.request(RequestStatus.PUT, url, params.toJSONString());
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String clusterUri) {
        try {
            this.request(RequestStatus.DELETE, clusterUri, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult resourceStatistics(String uri) {
        try {
            String response = this.request(RequestStatus.GET, uri + "/computeresource", null);
            return new BocloudResult(true, response, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}