package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.HostConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.fusionsphere.TaskModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereHostProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 17:21
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereHostProvider extends FusionSphereRestApiProvider implements HostProvider {

    public FusionSphereHostProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<HostModel> convertor = new HostConvertor();

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            List<HostModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/hosts";
            String url = String.format(api, siteUri);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("hosts"), siteUri).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("hosts"), siteUri).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            //列表没有arch信息，单独查询一次
            for (HostModel model : models) {
                String detailResponse = this.request(RequestStatus.GET, model.getUri(), null);
                JSONObject detailObject = JSONObject.parseObject(detailResponse);
                model.setArch(detailObject.getString("arch"));
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(HostModel model) {
        try {
            String api = "%s/hosts";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("description", model.getRemark());
            params.put("ip", model.getHostIp());//必选
            params.put("clusterUrn", model.getClusterUuid());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            jsonObject.getString("urn");
            jsonObject.getString("taskUrn");

            TaskModel taskModel = TaskProvider.get2Task(jsonObject.getString("taskUri"), this.getButler());
            if (taskModel.getBusinessStatus()) {
                return this.detail(jsonObject.getString("uri"));
            } else {
                return new BocloudResult(false, taskModel.getReasonDes());
            }
//            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
//            if (taskStatus) {
//                return this.detail(jsonObject.getString("uri"));
//            } else {
//                return new BocloudResult(false, "任务失败");
//            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uri) {
        try {
            String response = this.request(RequestStatus.GET, uri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            HostModel hostModel = convertor.convertModelParams(jsonObject, null);
            return new BocloudResult(true, hostModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(HostModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            String response = this.request(RequestStatus.PUT, model.getUri(), params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(HostModel model) {
        try {
            String response = this.request(RequestStatus.DELETE, model.getUri(), null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult maintain(HostModel model) {
        try {
            String api;
            if (model.getMaintained()) {
                api = "%s/action/enterMaintenanceMode";
            } else {
                api = "%s/action/exitMaintenanceMode";
            }
            String url = String.format(api, model.getUri());
            this.request(RequestStatus.POST, url, null);
            return new BocloudResult(true, "任务执行成功");
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult moveTo(HostModel model, String clusterUrn) {
        try {
            String api = "%s/action/move";
            String url = String.format(api, model.getUri());
            JSONObject params = new JSONObject();
            params.put("clusterUrn", clusterUrn);
            this.request(RequestStatus.POST, url, params.toJSONString());
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}