package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.ServerConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.fusionsphere.CdromModel;
import com.bocloud.cmp.model.fusionsphere.NicSpecificationModel;
import com.bocloud.cmp.model.fusionsphere.TaskModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereServerProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 17:35
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereServerProvider extends FusionSphereRestApiProvider implements ServerProvider {

    public FusionSphereServerProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<ServerModel> convertor = new ServerConvertor();

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            String api = "%s/vms";
            String url = String.format(api, siteUri);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<ServerModel> models = convertor.convertListParams(jsonObject.getJSONArray("vms"), siteUri).stream().collect(Collectors.toList());
            List<ServerModel> newModels = new ArrayList<>();
            for (ServerModel serverModel : models) {
                String detailResponse = this.request(RequestStatus.GET, serverModel.getUri(), null);
                JSONObject detailJsonObject = JSONObject.parseObject(detailResponse);
                ServerModel newServerModel = convertor.convertModel(detailJsonObject);
                newServerModel.setVdcUri(siteUri);
                newModels.add(newServerModel);
            }
            return new BocloudResult(true, newModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ServerModel model) {
        try {
            JSONObject jsonObject;
            switch (model.getCreateMode()) {
                case "Default":
                    log.info("底层执行默认创建中......");
                    jsonObject = createDefault(model);
                    break;
                case "Template":
                    log.info("底层执行模板克隆中......");
                    jsonObject = createTemplate(model);
                    break;
                case "FromServer":
                    log.info("底层执行云主机克隆中......");
                    jsonObject = createFromServer(model);
                    break;
                default:
                    jsonObject = null;
                    break;
            }
            if (jsonObject == null) {
                return new BocloudResult(false, "非法创建");
            }

            TaskModel taskModel = TaskProvider.get2Task(jsonObject.getString("taskUri"), this.getButler());
            if (taskModel.getBusinessStatus()) {
                return this.detail(jsonObject.getString("uri"));
            } else {
                return new BocloudResult(false, taskModel.getReasonDes());
            }
//            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
//            if (taskStatus) {
//                return this.detail(jsonObject.getString("uri"));
//            } else {
//                return new BocloudResult(false, "任务失败");
//            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    public JSONObject createDefault(ServerModel model) {
        String api = "%s/vms";
        String url = String.format(api, model.getVdcUri());
        JSONObject params = new JSONObject();
        params.put("name", model.getName());//必选
        params.put("arch", model.getArch());
        params.put("description", model.getRemark());
        params.put("location", model.getVmlocation());
        params.put("parentObjUrn", model.getVdcUuid());
        params.put("isBindingHost", model.getIsBindingHost());
        params.put("autoBoot", true);
        params.put("enableImc", false);
        params.put("isMultiDiskSpeedup", false);
        params.put("hasSetStoreResAssignState", false);
        JSONObject osOptions = new JSONObject();
        osOptions.put("osType", model.getOsCategory());
        osOptions.put("osVersion", Integer.parseInt(model.getOsVersion()));
        params.put("osOptions", osOptions);
        params.put("vmConfig", this.vmConfig(model));
        String response = this.request(RequestStatus.POST, url, params.toJSONString());
        JSONObject jsonObject = JSONObject.parseObject(response);
        return jsonObject;
    }

    public JSONObject createTemplate(ServerModel model) {
        String api = "%s/action/clone";
        String url = String.format(api, model.getUri());
        JSONObject params = new JSONObject();
        params.put("name", model.getName());//必选
        params.put("description", model.getRemark());
        params.put("isBindingHost", model.getIsBindingHost());
        params.put("parentObjUrn", model.getVdcUuid());
        params.put("location", model.getVmlocation());
        params.put("hasSetStoreResAssignState", false);
        params.put("isTemplate", false);
        params.put("isMultiDiskSpeedup", false);
        params.put("autoBoot", true);
        params.put("enableImc", false);

        JSONObject osOptions = new JSONObject();
        osOptions.put("osType", model.getOsCategory());
        osOptions.put("osVersion", Integer.parseInt(model.getOsVersion()));
        params.put("osOptions", osOptions);
        params.put("vmConfig", this.vmConfig(model));
        params.put("vmCustomization", this.customizationConfig(model));
        String response = this.request(RequestStatus.POST, url, params.toJSONString());
        JSONObject jsonObject = JSONObject.parseObject(response);
        return jsonObject;
    }

    public JSONObject createFromServer(ServerModel model) {
        String api = "%s/action/clone";
        String url = String.format(api, model.getUri());
        JSONObject params = new JSONObject();
        params.put("name", model.getName());//必选
        params.put("description", model.getRemark());
        params.put("isBindingHost", model.getIsBindingHost());
        params.put("parentObjUrn", model.getVdcUuid());
        params.put("location", model.getVmlocation());
        params.put("hasSetStoreResAssignState", false);
        params.put("isLinkClone", false);
        params.put("isTemplate", false);
        params.put("isMultiDiskSpeedup", false);
        params.put("autoBoot", true);
        params.put("enableImc", false);

        JSONObject osOptions = new JSONObject();
        osOptions.put("osType", model.getOsCategory());
        osOptions.put("osVersion", Integer.parseInt(model.getOsVersion()));
        params.put("osOptions", osOptions);
        params.put("vmConfig", this.vmConfig(model));
        String response = this.request(RequestStatus.POST, url, params.toJSONString());
        JSONObject jsonObject = JSONObject.parseObject(response);
        return jsonObject;
    }

    private JSONObject vmConfig(ServerModel model) {
        JSONObject params = new JSONObject();
        params.put("numaNodes", 0);//云主机NUMA节点数，取值范围[0-8], 默认值0。

 /*       //云主机显卡类型
        JSONObject graphicsCard = new JSONObject();
        graphicsCard.put("type", "virtio");
        if(model.getArch().equalsIgnoreCase("arm")){
            graphicsCard.put("type","virtio");
        }else{
            graphicsCard.put("type","vga");
            graphicsCard.put("type","cirrus");
        }
        params.put("graphicsCard", graphicsCard);*/

        //cpu
        JSONObject cpu = new JSONObject();
        cpu.put("cpuPolicy", "shared");
        cpu.put("cpuThreadPolicy", "prefer");
        cpu.put("weight", model.getCpu() * 500);
        cpu.put("reservation", 0);
        cpu.put("quantity", model.getCpu());
        cpu.put("slotNum", model.getCpu());
        cpu.put("coresPerSocket", 1);
        cpu.put("limit", 0);
        params.put("cpu", cpu);
        //内存
        JSONObject memory = new JSONObject();
        memory.put("quantityMB", model.getMemory().intValue() * 1024);
        memory.put("reservation", 0);
        memory.put("weight", model.getMemory().intValue() * 1024 * 10);
        memory.put("hugePage", "4K");
        memory.put("limit", 0);
        memory.put("unit", "GB");
        params.put("memory", memory);
        //磁盘
        JSONArray disks = new JSONArray();
        for (VolumeModel volumeModel : model.getDiskDevices()) {
            JSONObject disk = new JSONObject();
            disk.put("datastoreUrn", volumeModel.getDataStoreUuid());
            disk.put("name", volumeModel.getName());
            disk.put("quantityGB", volumeModel.getSize());
            disk.put("sequenceNum", volumeModel.getSequenceNum());
            disk.put("indepDisk", volumeModel.getIndepDisk());
            disk.put("persistentDisk", volumeModel.getPersistentDisk());
            disk.put("isThin", volumeModel.getThinlyProvisioned());
            disk.put("pciType", volumeModel.getPciType());
            disk.put("volType", volumeModel.getVolumeType());
            disk.put("bootOrder", -1);
            disks.add(disk);
        }
        params.put("disks", disks);
        //网卡
        JSONArray nics = new JSONArray();
        for (NetworkCardModel cardModel : model.getNetworkCards()) {
            JSONObject nic = new JSONObject();
            nic.put("sequenceNum", cardModel.getSequenceNum());
            nic.put("portGroupUrn", cardModel.getPortGroupUuid());
            nic.put("virtIo", cardModel.getType());
            JSONObject nicConfig = new JSONObject();
            nicConfig.put("vringbuf", cardModel.getVringbuf());
            nicConfig.put("queues", cardModel.getQueues());
            nic.put("nicConfig", nicConfig);
            nic.put("enableSecurityGroup", cardModel.getEnableSecurityGroup());
            if (cardModel.getEnableSecurityGroup()) {
                nic.put("securityGroupId", cardModel.getSecurityGroupUuid());
            }
            nic.put("bootOrder", -1);
            nics.add(nic);
        }
        params.put("nics", nics);

        //properties
        JSONObject properties = new JSONObject();
        properties.put("clockMode", "freeClock");//云主机的时钟模式，包括自由时(freeClock)钟和同步时钟(synchClock)。可选，默认自由时钟。
//        启动引导固件: UEFI , BIOS 。 可选，默认BIOS
//        if(model.getArch().equalsIgnoreCase("arm")){
//            properties.put("bootFirmware", "UEFI");
//        }else{
//            properties.put("bootFirmware", "BIOS");
//        }
//        properties.put("bootFirmwareTime", 0);//启动引导延迟时间，取值范围：[0，10000]注：UEFI不支持设置
        properties.put("bootOption", "disk");//pxe,cdrom,disk,exact
        properties.put("evsAffinity", false);//EVS亲和云主机开关，可选，默认值false。true:开启EVS亲和。false：关闭EVS亲和。
        properties.put("isEnableHa", false);
        properties.put("vmVncKeymapSetting", 7);//vnc登陆VM键盘类型，可选，默认使用站点高级配置中的键盘类型。可通过接口3.4.50获取系统支持的”vnc登陆VM键盘类型” id列表。
        properties.put("isAutoUpgrade", true);//pv driver是否自动化升级，true：自动升级，false:手动升级。可选，默认为true。
        properties.put("attachType", false);//块设备的挂卷方式，是否支持基本共享存储向存储虚拟化热迁移，false：不支持，true：支持。可选，默认为false。
        properties.put("isEnableMemVol", false);//是否有内存卷。可选，默认为true
        properties.put("isEnableFt", false);//云主机是否支持FT(预留)。
        properties.put("cdRomBootOrder", -1);//可选。光驱启动顺序，默认为-1，表示不设置启动顺序。
        properties.put("consoleLogTextState", false);
        properties.put("isAutoAdjustNuma", false);// 可选，是否开启NUMA结构自动调整。
        properties.put("isHpet", false);//是否开启高精度时钟，仅查询时有效。
        properties.put("realtime", false);//高精度实时性云主机标识，默认未非高精度实时性云主机。True：高精度实时性云主机False：非高精度实时性云主机。
        properties.put("consolelog", 1);
        params.put("properties", properties);
        return params;
    }

    private JSONObject customizationConfig(ServerModel model) {
        JSONObject params = new JSONObject();
        if (model.getCustomization()) {
            params.put("hostname", model.getVmHostName());
            params.put("osType", model.getOsCategory());
            if (null != model.getPassword() && !"".equals(model.getPassword())) {
                params.put("password", model.getPassword());
            }
            params.put("updatePassword", false);
            JSONArray nicSpecification = new JSONArray();
            for (NicSpecificationModel specificationModel : model.getNicSpecificationModels()) {
                JSONObject ip = new JSONObject();
                ip.put("ipVersion", specificationModel.getIpVersion());
                ip.put("sequenceNum", specificationModel.getSequenceNum());
                ip.put("ip", specificationModel.getIp());
                ip.put("netmask", specificationModel.getNetmask());
                ip.put("gateway", specificationModel.getGateway());
                ip.put("setdns", specificationModel.getSetdns());
                nicSpecification.add(ip);
            }
            params.put("nicSpecification", nicSpecification);
        } else {
            params.put("isUpdateVmPassword", false);
            params.put("osType", model.getOsCategory());
        }
        return params;
    }

    @Override
    public BocloudResult modify(ServerModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//可选
            params.put("description", model.getRemark());//可选
            params.put("location", model.getVmlocation());//可选

            //cpu规格
            if (null != model.getCpu() && model.getCpu() != 0) {
                JSONObject cpu = new JSONObject();
                cpu.put("cpuPolicy", "shared");
                cpu.put("cpuThreadPolicy", "prefer");
                cpu.put("weight", model.getCpu() * 500);
                cpu.put("reservation", 0);
                cpu.put("quantity", model.getCpu());
                cpu.put("slotNum", model.getCpu());
                cpu.put("coresPerSocket", 1);
                cpu.put("limit", 0);
                params.put("cpu", cpu);
            }
            if (null != model.getMemory() && !model.getMemory().equals(BigDecimal.ZERO)) {
                //内存
                JSONObject memory = new JSONObject();
                memory.put("quantityMB", model.getMemory().intValue() * 1024);
                memory.put("reservation", 0);
                memory.put("weight", model.getMemory().intValue() * 1024 * 10);
                memory.put("hugePage", "4K");
                memory.put("limit", 0);
                params.put("memory", memory);
            }

            if (null != model.getIsTemplate() && model.getIsTemplate()) {
                params.put("isTemplate", true);
            }
            String response = this.request(RequestStatus.PUT, model.getUri(), params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            if (taskStatus) {
                return this.detail(model.getUri());
            } else {
                return new BocloudResult(false, "任务失败");
            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String serverUri) {
        try {
            String response = this.request(RequestStatus.GET, serverUri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            ServerModel serverModel = convertor.convertModel(jsonObject);
            return new BocloudResult(true, serverModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String serverUri, String name) {
        try {
            String response = this.request(RequestStatus.DELETE, serverUri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult start(String serverUri) {
        try {
            String api = "%s/action/start";
            String url = String.format(api, serverUri);
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult stop(String serverUri) {
        try {
            String api = "%s/action/stop";
            String url = String.format(api, serverUri);
            JSONObject params = new JSONObject();
            params.put("mode", "safe");
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult reboot(String serverUri) {
        try {
            String api = "%s/action/reboot";
            String url = String.format(api, serverUri);
            JSONObject params = new JSONObject();
            params.put("mode", "safe");
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult hibernated(String serverUri) {
        try {
            String api = "%s/action/hibernate";
            String url = String.format(api, serverUri);
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        try {
            String api = "%s/action/migrate";
            String url = String.format(api, server.getUri());
            JSONObject params = new JSONObject();
            params.put("location", server.getHostUuid());//必选
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult mountToolsInstaller(String serverUri) {
        try {
            String api = "%s/action/installtools";
            String url = String.format(api, serverUri);
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, "tools" + LogMapUtils.MAP.get("attach"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult unmountToolsInstaller(String serverUri) {
        try {
            String api = "%s/action/uninstalltools";
            String url = String.format(api, serverUri);
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, "tools" + LogMapUtils.MAP.get("detach"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult joinSGroup(String networkCardUri, String group) {
        try {
            String api = "%s/securitygroup";
            String url = String.format(api, networkCardUri);
            JSONObject params = new JSONObject();
            params.put("enableSecurityGroup", true);//可选，云主机网卡是否开启安全组：true表示开启，false表示未开启，默认为false。
            params.put("securityGroupId", Long.valueOf(group));//安全组Id，当enableSecurityGroup为true时有效。
            this.request(RequestStatus.PUT, url, params.toJSONString());
            return new BocloudResult(true, Thread.currentThread().getStackTrace()[1].getMethodName() + LogMapUtils.MAP.get("operation"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult leaveSGroup(String networkCardUri, String group) {
        try {
            String api = "%s/securitygroup";
            String url = String.format(api, networkCardUri);
            JSONObject params = new JSONObject();
            params.put("enableSecurityGroup", false);//可选，云主机网卡是否开启安全组：true表示开启，false表示未开启，默认为false。
            this.request(RequestStatus.PUT, url, params.toJSONString());
            return new BocloudResult(true, Thread.currentThread().getStackTrace()[1].getMethodName() + LogMapUtils.MAP.get("operation"));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attachCdrom(CdromModel cdromModel) {
        try {
            String api = "%s/action/attachCdrom";
            String url = String.format(api, cdromModel.getVmUuid());
            JSONObject params = new JSONObject();
            params.put("devicePath", cdromModel.getDevicePath());
            params.put("username", cdromModel.getUsername());
            params.put("password", cdromModel.getPassword());
            params.put("protocol", cdromModel.getProtocol());

            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detachCdrom(CdromModel cdromModel) {
        try {
            String api = "%s/action/detachCdrom";
            String url = String.format(api, cdromModel.getVmUuid());
            String response = this.request(RequestStatus.POST, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult getVmIps(String serverUri, String address) {
        try {
            boolean taskStatus = false;
            ServerModel serverModel = null;
            Long startTime = System.currentTimeMillis();
            Long endTime = System.currentTimeMillis();
            while (endTime - startTime < 10 * 60 * 1000) {
                log.info("执行IP获取任务查询：要匹配的IP为：{}", address);
                Thread.sleep(10 * 1000);
                String response = this.request(RequestStatus.GET, serverUri, null);
                JSONObject jsonObject = JSONObject.parseObject(response);
                serverModel = convertor.convertModel(jsonObject);
                log.info("执行IP获取任务查询：获取目标机器的IP为：{}", serverModel.getPrivateIps());
                List<String> ipList = JSONArray.parseArray(serverModel.getPrivateIps(), String.class);
                for (String ip : ipList) {
                    if (ip.equals(address)) {
                        log.info("执行IP获取任务查询：匹配成功退出循环！");
                        taskStatus = true;
                        break;
                    }
                }
                if (taskStatus) {
                    break;
                }
                endTime = System.currentTimeMillis();
            }
            if (taskStatus) {
                return new BocloudResult(true, serverModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            } else {
                return new BocloudResult(false, "匹配ip地址失败");
            }
        } catch (Exception e) {
            log.error("getVmIps error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}