package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.VdcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VDCModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.common.VdcProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereSiteProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/14 14:21
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereVdcProvider extends FusionSphereRestApiProvider implements VdcProvider {

    private BeanJsonConvertor<VDCModel> convertor = new VdcConvertor();

    public FusionSphereVdcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            String api = "/service/sites";
            String url = String.format(api);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<VDCModel> models = convertor.convertList(jsonObject.getJSONArray("sites")).stream().collect(Collectors.toList());
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }

    }


}