package com.bocloud.cmp.provider.fusionsphere;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.OsVersionConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.fusionsphere.FusionSphereOsVersionModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereOsVersionProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/11/3 11:25
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereOsVersionProvider extends FusionSphereRestApiProvider implements OsVersionProvider {

    private BeanJsonConvertor<FusionSphereOsVersionModel> convertor = new OsVersionConvertor();

    public FusionSphereOsVersionProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String vdc) {
        try {
            List<FusionSphereOsVersionModel> models = new ArrayList<>();
            String api = "%s/vms/osversions";
            String url = String.format(api, vdc);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("linux"), vdc).stream().collect(Collectors.toList()));
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("windows"), vdc).stream().collect(Collectors.toList()));
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("other"), vdc).stream().collect(Collectors.toList()));
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}