package com.bocloud.cmp.provider.fusionsphere;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.StorageDeviceConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.fusionsphere.StorageDeviceModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereStorageDeviceProvider
 * @Description:存储设备
 * @author: zhenpingLi/李振平
 * @date: 2021/10/20 15:42
 */

@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereStorageDeviceProvider extends FusionSphereRestApiProvider implements StorageDeviceProvider {

    private BeanJsonConvertor<StorageDeviceModel> convertor = new StorageDeviceConvertor();

    public FusionSphereStorageDeviceProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String vdc) {
        try {
            List<StorageDeviceModel> models = new ArrayList<>();
            Integer offset = 0;
            String api = "%s/storageunits/querybypage";
            String url = String.format(api, vdc);
            JSONObject params = new JSONObject();
            params.put("deviceType", 0);
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("storageUnits"), vdc).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("storageUnits"), vdc).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}