package com.bocloud.cmp.provider.fusionsphere;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.UpLinkPortConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.fusionsphere.FusionSphereUpLinkPortModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.provider.fusionsphere.UpLinkPortProvider;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSpherePortProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/25 15:29
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereUpLinkPortProvider extends FusionSphereRestApiProvider implements UpLinkPortProvider {

    private BeanJsonConvertor<FusionSphereUpLinkPortModel> convertor = new UpLinkPortConvertor();

    public FusionSphereUpLinkPortProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listBySwitcher(String siteUri, String dvsUrn) {
        try {
            List<FusionSphereUpLinkPortModel> models = new ArrayList<>();
            Integer offset = 0;

            String api = "%s/uplinkports";
            String url = String.format(api, siteUri);
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            JSONArray array = new JSONArray();
            array.add(dvsUrn);
            params.put("dvsurns", array);
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("uplinkPorts"), siteUri).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.POST, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("uplinkPorts"), siteUri).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}