package com.bocloud.cmp.provider.monitor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.fusionsphere.FusionSphereMonitorModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereMonitorRealTimeModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereMonitorProvider extends FusionSphereRestApiProvider implements MonitorProvider {

    public FusionSphereMonitorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult metrics(FusionSphereMonitorModel model, String siteUri) {
        try {
            String api = "%s/monitors/objectmetric-curvedata";
            String url = String.format(api, siteUri);
            JSONArray array = new JSONArray();
            JSONObject params = new JSONObject();
            params.put("urn", model.getUrn());
            params.put("metricId", model.getMetricId());
            params.put("startTime", model.getStartTime());
            params.put("endTime", model.getEndTime());
            params.put("interval", model.getInterval());
            array.add(params);
            String response = this.requestMonitor(RequestStatus.POST, url, array.toJSONString());
            return new BocloudResult(true, response, "查询成功");
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult realMetrics(FusionSphereMonitorRealTimeModel model) {
        try {
            String api = "%s/monitors/objectmetric-realtimedata";
            String url = String.format(api, model.getSiteUri());
            JSONArray array = new JSONArray();
            JSONObject params = new JSONObject();
            params.put("urn", model.getUrn());
            params.put("metricId", model.getMetricId());
            array.add(params);
            String response = this.requestMonitor(RequestStatus.POST, url, array.toJSONString());
            return new BocloudResult(true, response, "查询成功");
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}
