package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.Random;

/**
 * @ClassName: FusionSphereNetworkCardProvider
 * @Description:
 * @Author lxy
 * @Date 2021/10/27 17:07
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereNetworkCardProvider extends FusionSphereRestApiProvider implements NetworkCardProvider{
    public FusionSphereNetworkCardProvider(Butler butler) {
        super(butler);
    }

    //暂未提供查询全部网卡接口，网卡直接在云主机数据中进行同步
    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult remove(String networkCardUri) {
        try {

            String uri = networkCardUri.replaceFirst("nics", "virtualNics");
            String response = this.request(RequestStatus.DELETE, uri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(NetworkCardModel model) {
        try {
            String api = "%s/virtualNics";
            String url = String.format(api, model.getServerUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//网卡别名,可选，长度[0,256]
            params.put("portGroupUrn", model.getPortGroupUuid());//portGroup 标识，必选
            params.put("mac", model.getMac());//mac,可选。系统支持自动分配能力
            params.put("sequenceNum", model.getSequenceNum());//网卡对应的总线编号，0-15。可选。不可重复
            /**
             * 网卡类型, 可选。1：virtio（默认值）
             * 2：e1000 (KVM未支持)
             * 3：rtl8139 (KVM未支持)
             * 4：vhost-user
             */
            params.put("virtIo", model.getType());
//            params.put("portId", ); //暂未存储  云主机交换机端口 ID，可选。
            if ("virtio".equalsIgnoreCase(model.getType())) {
                JSONObject nicConfig = new JSONObject();
                nicConfig.put("vringbuf", model.getVringbuf());//IO环大小，默认为256，取值范围为256、512、1024、2048、4096
                nicConfig.put("queues", model.getQueues());//队列数，默认为1，取值范围为[1,8]
                params.put("nicConfig", nicConfig);//虚拟网卡配置参数,仅对virtio类型网卡有意义
            }

            params.put("enableSecurityGroup", model.getEnableSecurityGroup());
            if (model.getEnableSecurityGroup()) {
                params.put("securityGroupId", model.getSecurityGroupUuid());
            }

            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            model.setNetworkcardUuid(jsonObject.getString("urn"));
            model.setUri(jsonObject.getString("uri"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String networkCardId) {
        return null;
    }

    @Override
    public BocloudResult attachNetworkCard(NetworkCardModel networkCardModel) {
        return null;
    }

    @Override
    public BocloudResult detachNetworkCard(NetworkCardModel networkCardModel) {
        return null;
    }
}
