package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.PortGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSpherePortGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/26 14:09
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSpherePortGroupProvider extends FusionSphereRestApiProvider implements PortGroupProvider {

    private BeanJsonConvertor<PortGroupModel> convertor = new PortGroupConvertor();

    public FusionSpherePortGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listBySwitcher(String uuid) {
        try {
            List<PortGroupModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/portgroups";
            String url = String.format(api, uuid);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertList(jsonObject.getJSONArray("portGroups")).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertList(jsonObject.getJSONArray("portGroups")).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(PortGroupModel model) {
        try {
            String api = "%s/portgroups";
            String url = String.format(api, model.getSwitcherUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("vlanId", model.getVlanId());//必选
            params.put("description", model.getRemark());
            params.put("portType", Integer.parseInt(model.getType()));
            params.put("vlanRange", model.getVlanRange());
            params.put("txLimit", model.getOutAverageBandWidth());
            params.put("txPeakLimit", model.getOutPeakBandWidth());
            params.put("txBurstSize", model.getOutBurstSize());
            params.put("rxLimit", model.getInAverageBandWidth());
            params.put("rxPeakLimit", model.getInPeakBandWidth());
            params.put("rxBurstSize", model.getInBurstSize());
            params.put("isIpMacBind", model.getIsIpMacBind());
            params.put("isDhcpIsolation", model.getIsDhcpIsolation());
            params.put("isCalcTCPCheckSum", model.getIsCalcTcpCheckSum());
            params.put("ipBcstSuppress", model.getIpBcstSuppress());

            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            model.setGroupVal(jsonObject.getString("urn"));
            model.setUri(jsonObject.getString("uri"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(PortGroupModel model) {
        try {
            String url = model.getUri();
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("vlanId", model.getVlanId());//必选
            params.put("description", model.getRemark());
            if (StringUtils.isNotBlank(model.getType())) {
                params.put("portType", Integer.parseInt(model.getType()));
            }
            params.put("vlanRange", model.getVlanRange());
            params.put("txLimit", model.getOutAverageBandWidth());
            params.put("txPeakLimit", model.getOutPeakBandWidth());
            params.put("txBurstSize", model.getOutBurstSize());
            params.put("rxLimit", model.getInAverageBandWidth());
            params.put("rxPeakLimit", model.getInPeakBandWidth());
            params.put("rxBurstSize", model.getInBurstSize());
            params.put("isIpMacBind", model.getIsIpMacBind());
            params.put("isDhcpIsolation", model.getIsDhcpIsolation());
            params.put("isCalcTCPCheckSum", model.getIsCalcTcpCheckSum());
            params.put("ipBcstSuppress", model.getIpBcstSuppress());
            String response = this.request(RequestStatus.PUT, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            this.request(RequestStatus.DELETE, uuid, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }


}