package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.SecurityGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereSecurityGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/22 10:08
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereSecurityGroupProvider extends FusionSphereRestApiProvider implements SecurityGroupProvider {

    private BeanJsonConvertor<SecurityGroupModel> convertor = new SecurityGroupConvertor();

    public FusionSphereSecurityGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            List<SecurityGroupModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/securitygroups";
            String url = String.format(api, siteUri);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("securityGroups"),siteUri).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("securityGroups"),siteUri).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel model) {
        try {
            String api = "%s/securitygroups/addsecuritygroup";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("sgName", model.getName());//必选
            params.put("sgDescription", model.getRemark());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            model.setGroupUuid(jsonObject.getString("sgId"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            String api = "%s/securitygroups/%s";
            String url = String.format(api, model.getVdcUri(), model.getGroupUuid());
            JSONObject params = new JSONObject();
            params.put("sgName", model.getName());//必选
            params.put("sgDescription", model.getRemark());
            this.request(RequestStatus.PUT, url, params.toJSONString());
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupModel model) {
        try {
            String api = "%s/securitygroups/deletesecuritygroup";
            String url = String.format(api, model.getVdcUri());
            JSONArray array = new JSONArray();
            array.add(Integer.valueOf(model.getGroupUuid()));
            JSONObject params = new JSONObject();
            params.put("sgIds", array);//必选
            this.request(RequestStatus.POST, url, params.toJSONString());
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}