package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.SecurityGroupRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereSecurityGroupRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/22 10:09
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereSecurityGroupRuleProvider extends FusionSphereRestApiProvider implements SecurityGroupRuleProvider {

    private BeanJsonConvertor<SecurityGroupRuleModel> convertor = new SecurityGroupRuleConvertor();

    public FusionSphereSecurityGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String sgId, String vdcId) {
        try {
            List<SecurityGroupRuleModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/securitygroups/%s/action/rules";
            String url = String.format(api, vdcId, sgId);
            String response = this.request(RequestStatus.GET, url, params.toJSONString()    );
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertList(jsonObject.getJSONArray("securityGroupRules")).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertList(jsonObject.getJSONArray("securityGroupRules")).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        try {
            String api = "%s/securitygroups/%s/action/addrules";
            String url = String.format(api, model.getVdcUri(), model.getGroupUuid());
            JSONObject ruleParam = new JSONObject();
            ruleParam.put("ipProtocol", model.getProtocol());//必选
            ruleParam.put("ipRanges", model.getIpRanges());
            ruleParam.put("fromPort", model.getPortMin());//必选
            ruleParam.put("toPort", model.getPortMax());
            ruleParam.put("direction", Integer.parseInt(model.getDirection()));//必选
            JSONArray rulesParam = new JSONArray();
            rulesParam.add(ruleParam);
            JSONObject params = new JSONObject();
            params.put("rules", rulesParam);
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel model) {
        try {
            String api = "%s/securitygroups/%s/action/delrules";
            String url = String.format(api, model.getVdcUri(), model.getGroupUuid());
            JSONArray array = new JSONArray();
            array.add(model.getRuleUuid());
            JSONObject params = new JSONObject();
            params.put("ruleIds", array);//必选
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}