package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.SwitchConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.model.fusionsphere.FusionSphereUpLinkModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereSwitchProvider
 * @Description:交换机
 * @author: zhenpingLi/李振平
 * @date: 2021/10/25 11:48
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereSwitchProvider extends FusionSphereRestApiProvider implements SwitchProvider {

    private BeanJsonConvertor<SwitchModel> convertor = new SwitchConvertor();


    public FusionSphereSwitchProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            String api = "%s/dvswitchs";
            String url = String.format(api, siteUri);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<SwitchModel> models = convertor.convertListParams(jsonObject.getJSONArray("dvSwitchs"), siteUri).stream().collect(Collectors.toList());
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SwitchModel model) {
        try {
            String api = "%s/dvswitchs";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("description", model.getRemark());
            params.put("type", model.getType());
            params.put("isIgmpSnooping", model.getIsIgmpSnooping());
            params.put("mtu", model.getMtu());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            model.setSwitcherVal(jsonObject.getString("urn"));
            model.setUri(jsonObject.getString("uri"));
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SwitchModel model) {
        try {
            String url = model.getUri();
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("description", model.getRemark());
            params.put("isIgmpSnooping", model.getIsIgmpSnooping());
            params.put("mtu", model.getMtu());
            this.request(RequestStatus.PUT, url, params.toJSONString());
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String switchUri) {
        try {
            this.request(RequestStatus.DELETE, switchUri, null);
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult addUpLink(SwitchModel model) {
        try {
            String api = "%s/action/addhost";
            String url = String.format(api, model.getUri());
            JSONObject params = new JSONObject();
            JSONArray array = new JSONArray();
            model.getUpLinkModels().stream().forEach(upLink -> {
                JSONObject upLinkObj = new JSONObject();
                upLinkObj.put("hostUrn", upLink.getHostUuid());
                upLinkObj.put("portUrn", upLink.getPortUuid());
                array.add(upLinkObj);
            });
            params.put("hostPortSet", array);//必选
            this.request(RequestStatus.POST, url, params.toJSONString());
            return this.detail(model.getUri());
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeUpLink(SwitchModel model) {
        try {
            String api = "%s/action/delhost";
            String url = String.format(api, model.getUri());
            JSONObject params = new JSONObject();
            JSONArray array = new JSONArray();
            model.getUpLinkModels().stream().forEach(upLink -> {
                JSONObject upLinkObj = new JSONObject();
                upLinkObj.put("hostUrn", upLink.getHostUuid());
                upLinkObj.put("portUrn", upLink.getPortUuid());
                array.add(upLinkObj);
            });
            params.put("hostPortSet", array);//必选
            this.request(RequestStatus.POST, url, params.toJSONString());
            return this.detail(model.getUri());
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult addVlanPool(SwitchModel model) {
        try {
            String api = "%s/action/addvlanpool";
            String url = String.format(api, model.getUri());
            JSONObject params = new JSONObject();
            JSONArray array = new JSONArray();
            model.getVlanPoolModels().stream().forEach(vlan -> {
                JSONObject obj = new JSONObject();
                obj.put("startVlanId", vlan.getStartVlanId());
                obj.put("endVlanId", vlan.getEndVlanId());
                array.add(obj);
            });
            params.put("vlanPoolSet", array);//必选
            this.request(RequestStatus.POST, url, params.toJSONString());
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult removeVlanPool(SwitchModel model) {
        try {
            String api = "%s/action/delvlanpool";
            String url = String.format(api, model.getUri());
            JSONObject params = new JSONObject();
            JSONArray array = new JSONArray();
            model.getVlanPoolModels().stream().forEach(vlan -> {
                JSONObject obj = new JSONObject();
                obj.put("startVlanId", vlan.getStartVlanId());
                obj.put("endVlanId", vlan.getEndVlanId());
                array.add(obj);
            });
            params.put("vlanPoolSet", array);//必选
            this.request(RequestStatus.POST, url, params.toJSONString());
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String switchUri) {
        try {
            String response = this.request(RequestStatus.GET, switchUri, null);
            JSONObject object = JSONObject.parseObject(response);
            return new BocloudResult(true, convertor.convertModel(object), LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}