package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.CdromFileConvertor;
import com.bocloud.cmp.convertor.DataStoreConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.DataStoreModel;
import com.bocloud.cmp.model.fusionsphere.CdromFileModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereDataStoreProvider
 * @Description:数据存储
 * @author: zhenpingLi/李振平
 * @date: 2021/10/20 16:21
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereDataStoreProvider extends FusionSphereRestApiProvider implements DataStoreProvider {

    private BeanJsonConvertor<DataStoreModel> convertor = new DataStoreConvertor();

    public FusionSphereDataStoreProvider(Butler butler) {
        super(butler);
    }

    private BeanJsonConvertor<CdromFileModel> fileConvertor = new CdromFileConvertor();

    @Override
    public BocloudResult listByVdc(String vdc) {
        try {
            List<DataStoreModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/datastores";
            String url = String.format(api, vdc);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("datastores"), vdc).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("datastores"), vdc).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(DataStoreModel model) {
        try {
            String api = "%s/datastores";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("storageUnitUrn", model.getStoreDeviceUuid());//必选
            params.put("useType", model.getUseType());//必选
            params.put("hostUrn", model.getHostUuid());
            params.put("description", model.getRemark());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            if (taskStatus) {
                return this.detail(jsonObject.getString("uri"));
            } else {
                return new BocloudResult(false, "任务失败");
            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(DataStoreModel model) {
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            this.request(RequestStatus.PUT, model.getUri(), params.toJSONString());
            return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uri) {
        try {
            String response = this.request(RequestStatus.DELETE, uri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uri) {
        try {
            String response = this.request(RequestStatus.GET, uri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            DataStoreModel hostModel = convertor.convertModelParams(jsonObject);
            return new BocloudResult(true, hostModel, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult listFile(String uri) {
        try {
            List<CdromFileModel> models = new ArrayList<>();
            Integer offset = 0;
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String api = "%s/file";
            String url = String.format(api, uri);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(fileConvertor.convertList(jsonObject.getJSONArray("fileUnits")).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");
            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(fileConvertor.convertList(jsonObject.getJSONArray("fileUnits")).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}