package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.HostPortConvertor;
import com.bocloud.cmp.convertor.UplinkPortAggrConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.fusionsphere.HostPortModel;
import com.bocloud.cmp.model.fusionsphere.UplinkPortAggrModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: FusionSphereHostPortProvider
 * @Description:查询服务器网口信息 hostport =全部口= UplinkPortAggr+普通口
 * @author: zhenpingLi/李振平
 * @date: 2021/11/2 11:56
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereHostPortProvider extends FusionSphereRestApiProvider implements HostPortProvider {

    private BeanJsonConvertor<HostPortModel> convertor = new HostPortConvertor();
    private BeanJsonConvertor<UplinkPortAggrModel> aggrConvertor = new UplinkPortAggrConvertor();


    public FusionSphereHostPortProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String hostUri) {
        try {
            String api = "%s/ports";
            String url = String.format(api, hostUri);
            String response = this.request(RequestStatus.GET, url, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            List<HostPortModel> models = convertor.convertList(jsonObject.getJSONArray("ports")).stream().collect(Collectors.toList());
            //--------------------关系处理--------------
            Map<String, HostPortModel> portModelMap = models.stream().collect(Collectors.toMap(portModel -> portModel.getPortUuid(), portModel -> portModel));

            List<HostPortModel> aggrPortModels = models.stream().filter(portModel -> portModel.getFlag() == 1).collect(Collectors.toList());
            for (HostPortModel aggrPortModel : aggrPortModels) {
                String aggrResponse = this.request(RequestStatus.GET, aggrPortModel.getUri(), null);
                JSONObject aggrjsonObject = JSONObject.parseObject(aggrResponse);
                UplinkPortAggrModel aggrModel = aggrConvertor.convertModel(aggrjsonObject);
                List<HostPortModel> aggrModelHostPortModels = aggrModel.getHostPortModels();
                for (HostPortModel aggrModelHostPortModel : aggrModelHostPortModels) {
                    HostPortModel portModel = portModelMap.get(aggrModelHostPortModel.getPortUuid());
                    if (null != portModel) {
                        portModel.setParentUuid(aggrPortModel.getPortUuid());
                    }
                }
            }
            List<HostPortModel> hostPortModels = new ArrayList<>(portModelMap.values());
            return new BocloudResult(true, hostPortModels, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}