package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.convertor.VolumeConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.fusionsphere.TaskModel;
import com.bocloud.cmp.provider.FusionSphereRestApiProvider;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.utils.LogMapUtils;
import com.bocloud.cmp.utils.TaskProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: FusionSphereVolumeProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2021/10/21 13:58
 */
@Slf4j
@Provider("bocloud.cmp.fusionsphere")
public class FusionSphereVolumeProvider extends FusionSphereRestApiProvider implements VolumeProvider {

    private BeanJsonConvertor<VolumeModel> convertor = new VolumeConvertor();

    public FusionSphereVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String siteUri) {
        try {
            List<VolumeModel> models = new ArrayList<>();
            Integer offset = 0;

            String api = "%s/volumes";
            String url = String.format(api, siteUri);
            JSONObject params = new JSONObject();
            params.put("offset", offset);
            params.put("limit", LIMIT);
            String response = this.request(RequestStatus.GET, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            models.addAll(convertor.convertListParams(jsonObject.getJSONArray("volumes"), siteUri).stream().collect(Collectors.toList()));
            Integer total = jsonObject.getInteger("total");

            while (total >= LIMIT + offset) {
                params.put("offset", offset += LIMIT);
                response = this.request(RequestStatus.GET, url, params.toJSONString());
                jsonObject = JSONObject.parseObject(response);
                models.addAll(convertor.convertListParams(jsonObject.getJSONArray("volumes"), siteUri).stream().collect(Collectors.toList()));
                total = jsonObject.getInteger("total");
            }
            return new BocloudResult(true, models, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VolumeModel model) {
        try {
            String api = "%s/volumes";
            String url = String.format(api, model.getVdcUri());
            JSONObject params = new JSONObject();
            params.put("name", model.getName());//必选
            params.put("quantityGB", model.getSize());//必选
            params.put("datastoreUrn", model.getDataStoreUuid());//必选
            params.put("type", model.getType());//必选
            params.put("isThin", model.getThinlyProvisioned());
            params.put("persistentDisk", model.getPersistentDisk());
            params.put("indepDisk", model.getIndepDisk());
            if (StringUtils.isNotBlank(model.getVolumeType())) {
                params.put("volType", Integer.parseInt(model.getVolumeType()));
            }
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);

            TaskModel taskModel = TaskProvider.get2Task(jsonObject.getString("taskUri"), this.getButler());
            if (taskModel.getBusinessStatus()) {
                return this.detail(jsonObject.getString("uri"));
            } else {
                return new BocloudResult(false, taskModel.getReasonDes());
            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uri) {
        try {
            String response = this.request(RequestStatus.GET, uri, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            VolumeModel model = convertor.convertModelParams(jsonObject);
            return new BocloudResult(true, model, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeModel model) {
        try {
            JSONObject params = new JSONObject();
            //编辑磁盘名称
            if (null != model.getName() && "" != model.getName()) {
                params.put("name", model.getName());
                this.request(RequestStatus.PUT, model.getUri(), params.toJSONString());
                return new BocloudResult(true, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
            } else {
                //云主机磁盘扩容
                if (null != model.getInstanceId() && "" != model.getInstanceId()) {
                    params.put("size", model.getSize() * 1024);
                    params.put("volUrn", model.getVolumeUuid());
                    String api = "%s/action/expandvol";
                    String url = String.format(api, model.getInstanceId());
                    String response = this.request(RequestStatus.POST, url, params.toJSONString());
                    JSONObject jsonObject = JSONObject.parseObject(response);
                    boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
                    return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
                } else {//普通磁盘扩容
                    params.put("size", model.getSize() * 1024);
                    String api = "%s/action/expandVol";
                    String url = String.format(api, model.getUri());
                    String response = this.request(RequestStatus.POST, url, params.toJSONString());
                    JSONObject jsonObject = JSONObject.parseObject(response);
                    boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
                    return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
                }
            }
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String volumeId) {
        try {
            String response = this.request(RequestStatus.DELETE, volumeId, null);
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult attach(String serverUri, VolumeModel model) {
        try {
            String api = "%s/action/attachvol";
            String url = String.format(api, serverUri);
            JSONObject params = new JSONObject();
            params.put("volUrn", model.getVolumeUuid());//必选
            params.put("pciType", model.getPciType());
            params.put("sequenceNum", model.getSequenceNum());//必选
            params.put("ioMode", model.getIoMode());
            params.put("accessMode", model.getAccessMode());
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, "硬盘" + LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detach(String serverUri, String volumeId) {
        try {
            String api = "%s/action/detachvol";
            String url = String.format(api, serverUri);
            JSONObject params = new JSONObject();
            params.put("volUrn", volumeId);//必选
            String response = this.request(RequestStatus.POST, url, params.toJSONString());
            JSONObject jsonObject = JSONObject.parseObject(response);
            boolean taskStatus = TaskProvider.getTask(jsonObject.getString("taskUri"), this.getButler());
            return new BocloudResult(taskStatus, "硬盘" + LogMapUtils.MAP.get(Thread.currentThread().getStackTrace()[1].getMethodName()));
        } catch (Exception e) {
            log.error(this.getClass().getName() + " " + Thread.currentThread().getStackTrace()[1].getMethodName() + " error :", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

}