package com.bocloud.cmp.provider;

import java.io.IOException;
import java.math.BigDecimal;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.apache.http.HttpHost;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.h3c.entity.RsTaskMsg;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.common.H3cThreadPool;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.model.Result;

/**
 * 华三认证provider
 * 
 * @author ymh
 *
 */
public abstract class H3cProvider extends GenericProvider implements SuperProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cProvider.class);
	/**
	 * endpoint 没有 http:// 这样的前缀
	 */
	private String endpoint;
	private Integer port;
	private String username;
	private String password;
	private CloseableHttpClient client;
	private boolean accessable;
	private String protocol = "http";

	@SuppressWarnings("deprecation")
	public DefaultHttpClient defaultHttpClient = null;

	public H3cProvider(Butler butler) {
		super(butler);
		String newUrl = butler.getUrl().toLowerCase();
		String url = "";
		if (newUrl.indexOf("http://") == 0) {
			url = newUrl.substring(7);
		} else if (newUrl.indexOf("https://") == 0) {
			url = newUrl.substring(8);
			protocol = "https";
		}
		String[] endPointAndPort = url.split(":");
		if (endPointAndPort.length == 2) {
			this.endpoint = endPointAndPort[0];
			this.port = Integer.parseInt(endPointAndPort[1]);
			this.username = butler.getUsername();
			this.password = butler.getPassword();
			this.login(endpoint, port, username, password, protocol);
			// this.init();
		} else {
			this.accessable = false;
			throw new RuntimeException("错误的参数");
		}
	}

	public void init() {
		try {
			CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
			credentialsProvider.setCredentials(new AuthScope(this.endpoint, this.port, "VMC RESTful Web Services"),
					new UsernamePasswordCredentials(this.username, this.password));
			HttpClientBuilder httpClientBuilder = HttpClientBuilder.create().setDefaultCredentialsProvider(
					credentialsProvider);
			client = httpClientBuilder.build();
			accessable = true;
		} catch (Exception e) {
			logger.error("", e);
			accessable = false;
		}
	}

	/**
	 * @return the accessable
	 */
	public boolean isAccessable() {
		return accessable;
	}

	public String getEndpoint() {
		return endpoint;
	}

	public void setEndpoint(String endpoint) {
		this.endpoint = endpoint;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public Integer getPort() {
		return port;
	}

	public void setPort(Integer port) {
		this.port = port;
	}

	public void setClient(CloseableHttpClient client) {
		this.client = client;
	}

	public CloseableHttpClient getClient() {
		return client;
	}

	public Result doGet(String url) throws Exception {
		HttpGet httpGet = new HttpGet("http://" + this.getEndpoint() + ":" + this.getPort() + url);
		httpGet.addHeader("accept", "application/xml");
		CloseableHttpResponse httpResponse = client.execute(httpGet);
		Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
		try {
			result.setMessage(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
		} catch (Exception e) {
			logger.error("请求io异常{}", ExceptionUtils.getStackTrace(e));
		} finally {
			closeClient();
		}
		return result;
	}

	public Result doPost(String url, String body) throws Exception {
		HttpPost httpPost = new HttpPost("http://" + this.getEndpoint() + ":" + this.getPort() + url);
		httpPost.addHeader("accept", "application/xml");
		httpPost.addHeader("content-type", "application/xml");
		if (body != null) {
			StringEntity httpEntity = new StringEntity(body, "gbk");
			httpPost.setEntity(httpEntity);
		}
		CloseableHttpResponse httpResponse = client.execute(httpPost);
		Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
		try {
			result.setMessage(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
		} catch (Exception e) {
			logger.error("请求io异常{}", ExceptionUtils.getStackTrace(e));
		} finally {
			closeClient();
		}
		return result;
	}

	public Result doPut(String url, String body) throws Exception {
		HttpPut httpPut = new HttpPut("http://" + this.getEndpoint() + ":" + this.getPort() + url);
		httpPut.addHeader("accept", "application/xml");
		httpPut.addHeader("content-type", "application/xml");
		if (body != null) {
			StringEntity httpEntity = new StringEntity(body, "utf-8");
			httpPut.setEntity(httpEntity);
		}
		CloseableHttpResponse httpResponse = client.execute(httpPut);
		Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
		try {
			result.setMessage(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
		} catch (Exception e) {
			logger.error("请求io异常{}", ExceptionUtils.getStackTrace(e));
		} finally {
			closeClient();
		}
		return result;
	}

	public Result doDelete(String url) throws Exception {
		HttpDelete httpDelete = new HttpDelete("http://" + this.getEndpoint() + ":" + this.getPort() + url);
		httpDelete.addHeader("accept", "application/xml");
		CloseableHttpResponse httpResponse = client.execute(httpDelete);
		Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
		try {
			if (httpResponse.getEntity() != null) {
				result.setMessage(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
			}
		} catch (Exception e) {
			logger.error("请求io异常{}", ExceptionUtils.getStackTrace(e));
		} finally {
			closeClient();
		}
		return result;
	}

	/**
	 * 注册事件监听
	 *
	 * @return
	 * @throws Exception
	 */
	public void doEventRegister(String eventType) throws Exception {
		H3cThreadPool.createThread(this.endpoint, this.port, this.username, this.password, eventType);
	}

	public void closeClient() {
		try {
			this.client.close();
		} catch (IOException e) {
			logger.error("关闭client失败", e);
		}
	}

	public Double kbMbGbToGb(String str) {
		try {
			if (str.contains("KB")) {
				str = str.replace("KB", "");
				return new BigDecimal(str).divide(new BigDecimal(1024 * 1024), 2, BigDecimal.ROUND_HALF_UP)
						.doubleValue();
			} else if (str.contains("MB")) {
				str = str.replace("MB", "");
				return new BigDecimal(str).divide(new BigDecimal(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
			} else if (str.contains("GB")) {
				str = str.replace("GB", "");
				return new BigDecimal(str).doubleValue();
			} else if (str.contains("TB")) {
				str = str.replace("TB", "");
				return new BigDecimal(str).multiply(new BigDecimal(1024)).setScale(2, BigDecimal.ROUND_HALF_UP)
						.doubleValue();
			}
		} catch (Exception e) {
			logger.error("error", e);
			return 0d;
		}
		return 0d;
	}

	@Override
	public boolean accessable() {
		return this.accessable;
	}

	@SuppressWarnings("deprecation")
	public void login(String ip, Integer port, String loginName, String password, String protocol) {
		// Http 连接客户端。
		defaultHttpClient = new DefaultHttpClient();
		HttpHost defaultHost = new HttpHost(ip, port, protocol);
		HttpParams params = defaultHttpClient.getParams();
		HttpConnectionParams.setConnectionTimeout(params, 5 * 1000);
		HttpConnectionParams.setSocketBufferSize(params, 8192);
		params.setParameter(ClientPNames.DEFAULT_HOST, defaultHost);
		// 认证信息
		defaultHttpClient.getCredentialsProvider().setCredentials(new AuthScope(ip, port, "VMC RESTful Web Services"),
				new UsernamePasswordCredentials(loginName, password)); // 用户名，密码
		this.accessable = true;
	}

	@SuppressWarnings("deprecation")
	public void logout() {
		if (defaultHttpClient == null) {
			logger.error("您还没有登录。");
		}
		defaultHttpClient.getConnectionManager().shutdown();
	}

	public void login(String ip, Integer port, String loginName, String password) {
		login(ip, port, loginName, password, "http");
	}

	public RsTaskMsg waitForComplete(Long msgId) {
		String uri = "/cas/casrs/message/" + msgId;
		RsTaskMsg task = RestUtil.httpGet(defaultHttpClient, uri, RsTaskMsg.class);
		while (!task.isCompleted()) {
			try {
				Thread.sleep(2000);
			} catch (InterruptedException e) {
				logger.error("查询任务信息异常：" + e.getMessage());
			}
			task = RestUtil.httpGet(defaultHttpClient, uri, RsTaskMsg.class);
		}
		return task;
	}

}