package com.bocloud.cmp.provider.builder;

import java.lang.reflect.Field;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

import com.bocloud.cmp.model.h3c.utils.ElementUtils;
import com.bocloud.cmp.provider.annotation.H3cXmlAnotation;

public abstract class H3cBasicBuilder {

    protected static Logger logger = LoggerFactory.getLogger(H3cBasicBuilder.class);

    public abstract H3cBasicBuilder initByModel(Object model);

    protected abstract Object getModel();


    protected String toXml() {
        Object model = getModel();
        if (model == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        Field[] fields = model.getClass().getFields();
        for (Field field : fields) {
            H3cXmlAnotation annotation = field.getAnnotation(H3cXmlAnotation.class);
            if (annotation != null) {
                String name = annotation.name();
                sb = sb.append("<").append(name).append(">");
                field.setAccessible(true);
                try {
                    sb = sb.append(field.get(this));
                } catch (IllegalAccessException e) {
                    logger.error("访问字段出错");
                }
                sb.append("</").append(name).append(">");
            }
        }
        return sb.toString();
    }

    protected Element addElement(Element parent, String fieldName) {
        Object model = this.getModel();
        if (model == null) {
            return null;
        }
        try {
            Field field = model.getClass().getDeclaredField(fieldName);
            field.setAccessible(true);
            Object value = field.get(model);
            return ElementUtils.addChild(parent, fieldName, value);
        } catch (Exception e) {
            logger.error("反射报错", e);
        }
        return null;
    }


    protected String toElementXml(String name, Object value) {
        StringBuilder sb = new StringBuilder();
        sb = sb.append("<").append(name).append(">");
        sb = sb.append(value);
        sb.append("</").append(name).append(">");
        return sb.toString();
    }
}
