package com.bocloud.cmp.provider.builder;

import java.util.List;

import org.w3c.dom.Element;

import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.h3c.H3cNetWorkModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.model.h3c.H3cStorageModel;
import com.bocloud.cmp.model.h3c.utils.ElementUtils;
import com.bocloud.common.utils.ListTool;

public class H3cServerModelBuilder extends H3cBasicBuilder {

    private H3cServerModel h3cServerModel = null;

    @Override
    public H3cBasicBuilder initByModel(Object model) {
        if (model == null) {
            return null;
        } else if (model instanceof ServerModel) {
            // 将公共模型转h3c模型 ，此部分代码已经删除不用了
            return this;
        } else if (model instanceof H3cServerModel) {
            this.h3cServerModel = (H3cServerModel) model;
            return this;
        }
        return null;
    }

    @Override
    protected Object getModel() {
        return this.h3cServerModel;
    }


    public String buildCreateXml() {
        if (this.h3cServerModel == null) {
            return "";
        }
        try {
            Element domain = ElementUtils.creatRoot("domain");
            // 云主机名称
            addElement(domain, "name");
            // 云主机显示名称
            addElement(domain, "title");
            // 描述
            addElement(domain, "description");
            // 内存
            addElement(domain, "memory");
            // 内存大小，单位为由memoryUnit决定,换算成MB后，必须与memory指定的值相同。
            addElement(domain, "memoryInit");
            // 内存显示单位，字符串。枚举类型：GB,MB
            addElement(domain, "memoryUnit");
            // 内存资源优先级。枚举值：0=低，50=中，100=高
            addElement(domain, "memoryPriority");
            // 内存预留。内存预留百分比
            addElement(domain, "memoryBacking");
            // CPU个数，最小值为1，最大值由主机实际情况决定。
            addElement(domain, "cpuSockets");
            // cpu核数 CPU核数，最小值为1，最大值由主机实际情况决定
            addElement(domain, "cpuCores");
            // CPU调度优先级，枚举值：1024：高，512：中，256：低。
            addElement(domain, "cpuShares");
            // 主机允许的最大CPU核数。
            addElement(domain, "maxCpuSocket");
            // I/O优先级，枚举值：500：高，300：中，200：低。
            addElement(domain, "blkiotune");
            // 操作系统类型，枚举值：0：Windows,1：Linux。
            addElement(domain, "system");
            // 操作系统版本
            addElement(domain, "osVersion");
            // 是否允许云主机自动迁移，实现资源动态调整，枚举值：1=允许，0=不允许。
            addElement(domain, "autoMigrate");
            // 启动优先级，当集群启用HA后有效，枚举值：0：低级，1：中级，2：高级。
            addElement(domain, "priority");
            // 主机id
            addElement(domain, "hostId");
            addElement(domain, "hostPoolId");
            // 是否可用。 1
            addElement(domain, "formatEnable");
//            // 绑定物理CPU配置。 自定义类型，可选项
//            addElement(domain, "bindcpu");  // 加了这个之后会报错
            // 云主机网络配置。自定义类型，必须
            List<H3cNetWorkModel> h3cNetworks = this.h3cServerModel.getNetworks();
            /**
             * 初始化网卡
             */
            if (!ListTool.isEmpty(h3cNetworks)) {
                for (H3cNetWorkModel networkModel : h3cNetworks) {
                    H3cNetworkModelBuilder h3cNetworkModelBuilder = new H3cNetworkModelBuilder();
                    h3cNetworkModelBuilder.initByModel(networkModel);
                    h3cNetworkModelBuilder.addElementToCreateVmXml(domain);
                }
            }
            /**
             * 初始化存储
             */
            List<H3cStorageModel> h3CStorageModels = this.h3cServerModel.getStorages();
            if (!ListTool.isEmpty(h3CStorageModels)) {
                for (H3cStorageModel h3CStorageModel : h3CStorageModels) {
                    H3cStorageModelBuilder h3cStorageModelBuilder = new H3cStorageModelBuilder();
                    h3cStorageModelBuilder.initByModel(h3CStorageModel);
                    h3cStorageModelBuilder.addElementToCreateVmXml(domain);
                }
            }

            return ElementUtils.toFormatedXML(domain);
        } catch (Exception e) {
            logger.error("", e);
        }
        return null;
    }


}
