package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.DataStoreFileModel;
import com.bocloud.cmp.model.DataStoreModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.IpModel;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.model.VDCModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cHostModel;
import com.bocloud.cmp.model.h3c.H3cNetWorkModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.model.h3c.H3cStorageModel;
import com.bocloud.cmp.model.h3c.H3cSyncModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsDomain;
import com.bocloud.cmp.model.h3c.entity.RsSnapshot;
import com.bocloud.cmp.model.h3c.entity.RsStoragePool;
import com.bocloud.cmp.model.h3c.entity.RsStorageVolume;
import com.bocloud.cmp.model.h3c.entity.RsVSwitch;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.compute.H3cClusterProvider;
import com.bocloud.cmp.provider.compute.H3cHostPoolProvider;
import com.bocloud.cmp.provider.compute.H3cHostProvider;
import com.bocloud.cmp.provider.compute.H3cServerProvider;
import com.bocloud.cmp.provider.compute.H3cSnapshotProvider;
import com.bocloud.cmp.provider.network.H3cNetworkProvider;
import com.bocloud.cmp.provider.storage.H3cStoragePoolProvider;
import com.bocloud.cmp.provider.storage.H3cVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云同步操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cLocationProvider extends H3cProvider implements LocationProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cLocationProvider.class);
	private Butler butler;

	public H3cLocationProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult listRegions() {
		return null;
	}

	@Override
	public BocloudResult<H3cSyncModel> sync() {
		H3cSyncModel h3cModel = new H3cSyncModel();
		logger.info("start sync vdc  data...... ");
		H3cHostPoolProvider hostPoolProvider = new H3cHostPoolProvider(butler);
		BocloudResult<List<VDCModel>> vdcModelResult = hostPoolProvider.list();
		if (vdcModelResult.isFailed()) {
			return new BocloudResult<H3cSyncModel>(false, "查询VDC数据异常：" + vdcModelResult.getMessage());
		}
		h3cModel.setVdcModels(vdcModelResult.getData());
		logger.info("end sync vdc  data...... ");

		logger.info("start sync cluster  data...... ");
		H3cClusterProvider clusterProvider = new H3cClusterProvider(butler);
		BocloudResult<List<ClusterModel>> clusterResult = clusterProvider.list();
		if (clusterResult.isFailed()) {
			return new BocloudResult<H3cSyncModel>(false, "查询集群数据异常：" + clusterResult.getMessage());
		}
		List<ClusterModel> clusterModels = clusterResult.getData();
		logger.info("end sync cluster  data....... ");

		H3cHostProvider hostProvider = new H3cHostProvider(butler);
		BocloudResult<List<HostModel>> hostResult = hostProvider.list();
		if (hostResult.isFailed()) {
			return new BocloudResult<H3cSyncModel>(false, "查询主机数据异常：" + hostResult.getMessage());
		}
		List<HostModel> hostModels = hostResult.getData();

		List<RsDomain> allVms = new ArrayList<RsDomain>();
		H3cServerProvider serverProvider = new H3cServerProvider(butler);
		for (HostModel hostModel : hostModels) {
			BocloudResult<List<RsDomain>> vmListResult = serverProvider.list(hostModel.getUuid());
			if (vmListResult.isSuccess()) {
				hostModel.setVmNum(vmListResult.getData() == null ? 0 : vmListResult.getData().size());
				allVms.addAll(vmListResult.getData());
			}
			logger.info("start sync switch ... hostId : [" + hostModel.getUuid() + "]");
			initHostSwitch(hostModel);
			logger.info("end sync switch ... hostId : [" + hostModel.getUuid() + "]");
			logger.info("start sync storagePool ... hostId : [" + hostModel.getUuid() + "]");
			initStoragePool(hostModel);
			logger.info("end sync storagePool ... hostId : [" + hostModel.getUuid() + "]");
		}
		Map<Long, Integer> clusterVmNums = new HashMap<Long, Integer>();
		H3cSnapshotProvider snapshotProvider = new H3cSnapshotProvider(butler);
		for (RsDomain rsDomain : allVms) {
			// 统计集群下的云主机数量
			if (clusterVmNums.containsKey(rsDomain.getClusterId())) {
				Integer clusterVms = clusterVmNums.get(rsDomain.getClusterId());
				clusterVmNums.put(rsDomain.getClusterId(), clusterVms + 1);
			} else {
				clusterVmNums.put(rsDomain.getClusterId(), 1);
			}
			// 获取云主机下的快照
			initSnapshot(rsDomain, snapshotProvider);
		}
		for (ClusterModel clusterModel : clusterModels) {
			Long clusterId = Long.parseLong(clusterModel.getUuid());
			clusterModel.setVmNum(clusterVmNums.get(clusterId) == null ? 0 : clusterVmNums.get(clusterId));
		}
		h3cModel.setServerModels(allVms);
		h3cModel.setHostModels(hostModels);
		h3cModel.setClusterModels(clusterModels);
		logger.info("start sync vmTemplate  data....... ");
		BocloudResult<List<RsDomain>> listVmTemplates = listVmTemplates();
		if (listVmTemplates.isSuccess()) {
			h3cModel.setVmTemplateModels(listVmTemplates.getData());
		}
		logger.info("end sync vmTemplate  data....... ");
		logger.info("h3cModel : " + JSON.toJSONString(h3cModel));
		return new BocloudResult<H3cSyncModel>(true, h3cModel, "查询成功！");
	}

	public BocloudResult<List<RsDomain>> listVmTemplates() {
		String uri = "/cas/casrs/vmTemplate/vmTemplateList?offset=0&limit=" + Integer.MAX_VALUE;
		try {
			List<RsDomain> tempRsDomain = new ArrayList<RsDomain>();
			@SuppressWarnings("unchecked")
			RsCommList<RsDomain> templates = RestUtil.httpGet(defaultHttpClient, uri, RsCommList.class);
			if (null == templates || null == templates.getData()) {
				return new BocloudResult<List<RsDomain>>(true, "暂无云主机模板信息");
			}
			List<RsDomain> data = templates.getData();
			for (RsDomain rsDomain : data) {
				String casuri = "/cas/casrs/vm/" + rsDomain.getId();
				RsDomain result = RestUtil.httpGet(defaultHttpClient, casuri, RsDomain.class);
				if (null != result) {
					result.setStorageCapacity(rsDomain.getStorageCapacity());
					tempRsDomain.add(result);
				}
			}
			return new BocloudResult<List<RsDomain>>(true, tempRsDomain, "模板查询成功！");
		} catch (Exception e) {
			logger.error("查询详情错误：", e);
			return new BocloudResult<List<RsDomain>>(false, e.getMessage());
		}
	}

	private void initSnapshot(RsDomain rsDomain, H3cSnapshotProvider snapshotProvider) {
		BocloudResult<List<RsSnapshot>> snapshotResult = snapshotProvider.list(rsDomain.getId());
		if (snapshotResult.isSuccess()) {
			List<RsSnapshot> data = snapshotResult.getData();
			rsDomain.setRsSnapshots(data);
		}
	}

	/**
	 * 生成主机模型
	 *
	 * @param hostsInHostPool
	 * @return
	 */
	private List<HostModel> newHostModels(List<HostModel> hostsInHostPool) {
		List<HostModel> hostModels = new ArrayList<>();
		if (!ListTool.isEmpty(hostsInHostPool)) {
			for (HostModel host : hostsInHostPool) {
				HostModel newHost = newHostModel(host);
				hostModels.add(newHost);
			}
		}
		return hostModels;
	}

	@SuppressWarnings("unchecked")
	private void initStoragePool(HostModel host) {
		H3cStoragePoolProvider h3cStoragePoolProvider = new H3cStoragePoolProvider(butler);
		BocloudResult bocloudResult = h3cStoragePoolProvider.list(host.getUuid());
		if (bocloudResult.isSuccess()) {
			List<RsStoragePool> rsStoragePools = (List<RsStoragePool>) bocloudResult.getData();
			H3cVolumeProvider h3cVolumeProvider = new H3cVolumeProvider(butler);
			for (RsStoragePool rsStoragePool : rsStoragePools) {
				BocloudResult volumeListResult = h3cVolumeProvider.list(host.getUuid(), rsStoragePool.getName());
				if (volumeListResult.isSuccess()) {
					rsStoragePool.setRsStorageVolumes((List<RsStorageVolume>) volumeListResult.getData());
				}
			}
			host.setRsStoragePools(rsStoragePools);
		}
	}

	private HostModel newHostModel(HostModel host) {
		HostModel newHost = getMoreHostModel(host);
		initSwitch(newHost);
		initServers(newHost);
		initDataStores(newHost);
		return newHost;
	}

	@Override
	public BocloudResult getHostModel(HostModel host) {
		HostModel newHost = getMoreHostModel(host);
		initSwitch(newHost);
		initServers(newHost);
		initDataStores(newHost);
		return new BocloudResult(true, newHost, "success");
	}

	/**
	 * 初始化 集群里的主机
	 *
	 * @param hostPoolId
	 * @param clusters
	 */
	private void initHostsInClusters(Long hostPoolId, List<ClusterModel> clusters) {
		for (ClusterModel cluster : clusters) {
			BocloudResult listResult = new H3cHostProvider(butler).list(cluster.getUuid());
			if (listResult.isSuccess()) {
				@SuppressWarnings("unchecked")
				List<HostModel> hosts = (List<HostModel>) listResult.getData();
				cluster.setHosts(newHostModels(hosts));
			}
		}
	}

	/**
	 * 将h3c的主机模型转通用模型
	 *
	 * @param h3cHosts
	 * @return
	 */
	private List<HostModel> convertHostModel(List<H3cHostModel> h3cHosts) {
		List<HostModel> hosts = new ArrayList<>();
		if (!ListTool.isEmpty(h3cHosts)) {
			for (H3cHostModel h3cHost : h3cHosts) {
				HostModel hostModel = new HostModel();
				hostModel.setName(h3cHost.getName());
				hostModel.setHostIp(h3cHost.getIp());
				hostModel.setUuid(String.valueOf(h3cHost.getId()));
				hosts.add(hostModel);
			}
		}
		return hosts;
	}

	/**
	 * 获取主机的详细信息
	 *
	 * @param host
	 * @return
	 */
	private HostModel getMoreHostModel(HostModel host) {
		H3cHostProvider h3cHostProvider = new H3cHostProvider(butler);
		BocloudResult bocloudResult = h3cHostProvider.detail(Long.valueOf(host.getUuid()));
		if (bocloudResult.isSuccess()) {
			HostModel retModel = (HostModel) bocloudResult.getData();
			h3cHostProvider = new H3cHostProvider(butler);
			BocloudResult summaryResult = h3cHostProvider.summary(Long.valueOf(host.getUuid()));
			if (summaryResult.isSuccess()) {
				if (retModel.getStatus() == null) {
					retModel.setStatus(((HostModel) summaryResult.getData()).getStatus());
					retModel.setPowerState(retModel.getStatus());
				}
			}
			return retModel;
		}
		return host;
	}

	@SuppressWarnings("unchecked")
	private void initDataStores(HostModel host) {
		H3cStoragePoolProvider h3cStoragePoolProvider = new H3cStoragePoolProvider(butler);
		BocloudResult bocloudResult = h3cStoragePoolProvider.list(host.getUuid());
		if (bocloudResult.isSuccess()) {
			List<DataStoreModel> dataStores = (List<DataStoreModel>) bocloudResult.getData();
			for (DataStoreModel dataStore : dataStores) {
				String storeVal = dataStore.getStoreVal();
				if (storeVal.contains("###")) {
					storeVal = storeVal.split("###")[1];
				}
				H3cVolumeProvider h3cVolumeProvider = new H3cVolumeProvider(butler);
				BocloudResult volumeListResult = h3cVolumeProvider.list(host.getUuid(), storeVal);
				if (volumeListResult.isSuccess()) {
					List<H3cVolumeModel> h3cVolumeList = (List<H3cVolumeModel>) volumeListResult.getData();
					List<DataStoreFileModel> files = convertStoreFiles(h3cVolumeList);
					dataStore.setFiles(files);
				}
			}
			host.setDataStores(dataStores);
		}
	}

	private List<DataStoreFileModel> convertStoreFiles(List<H3cVolumeModel> h3cVolumeList) {
		if (ListTool.isEmpty(h3cVolumeList)) {
			return null;
		}
		List<DataStoreFileModel> models = new ArrayList<>();
		for (H3cVolumeModel h3cVolumeModel : h3cVolumeList) {
			DataStoreFileModel dataStoreFile = new DataStoreFileModel();
			dataStoreFile.setName(h3cVolumeModel.getVolName());
			dataStoreFile.setSize(h3cVolumeModel.getCapacity());
			models.add(dataStoreFile);
		}
		return models;
	}

	@SuppressWarnings("unchecked")
	private void initServers(HostModel hostModel) {
		H3cServerProvider h3cServerProvider = new H3cServerProvider(butler);
		BocloudResult bocloudResult = h3cServerProvider.list(hostModel.getUuid());
		if (bocloudResult.isSuccess()) {
			hostModel.setServers((List<ServerModel>) bocloudResult.getData());
			getMoreServerInfo(hostModel);
		}
	}

	private void getMoreServerInfo(HostModel hostModel) {
		List<ServerModel> servers = hostModel.getServers();
		for (ServerModel server : servers) {
			String serverId = server.getInstanceId();
			H3cServerProvider h3cServerProvider = new H3cServerProvider(butler);
			BocloudResult detail = h3cServerProvider.detail(serverId);
			List<IpModel> ips = new ArrayList<>();
			if (detail.isSuccess()) {
				H3cServerModel h3cServer = (H3cServerModel) detail.getData();
				server.setCpu(h3cServer.getCpu());
				server.setMemory(BigDecimal.valueOf(h3cServer.getMemory()).divide(BigDecimal.valueOf(1024), 2,
						BigDecimal.ROUND_HALF_UP));
				server.setDiskDevices(getServerDiskDevice(h3cServer.getStorages()));
				Integer size = 0;
				for (VolumeModel volumeModel : server.getDiskDevices()) {
					size += volumeModel.getSize();
				}
				// mb转gb
				server.setDisk(size / 1024);
				server.setIsoPath(getServerIsoPath(h3cServer.getStorages()));
				switch (h3cServer.getStatus()) {
				case 2:
					server.setStatus(BocloudStatus.Server.RUNNING.name());
					break;
				case 3:
					server.setStatus(BocloudStatus.Server.STOPPED.name());
					break;
				case 4:
					server.setStatus(BocloudStatus.Server.PAUSED.name());
					break;
				}
			}
			// 通过这个获取ip信息
			h3cServerProvider = new H3cServerProvider(butler);
			BocloudResult netCardResult = h3cServerProvider.getNetCard(serverId);
			if (netCardResult.isSuccess()) {
				@SuppressWarnings("unchecked")
				List<H3cNetWorkModel> h3cNetWorkModels = (List<H3cNetWorkModel>) netCardResult.getData();
				List<NetworkCardModel> networkcardModels = new ArrayList<>();
				for (H3cNetWorkModel h3cNetWorkModel : h3cNetWorkModels) {
					NetworkCardModel networkCardModel = new NetworkCardModel();
					networkCardModel.setMac(h3cNetWorkModel.getMac());

					networkCardModel.setSwitcher(h3cNetWorkModel.getVsName());
					if (!StringUtils.isEmpty(h3cNetWorkModel.getIpAddr()) && server.getManagerIp() == null) {
						server.setManagerIp(h3cNetWorkModel.getIpAddr());
					}
					IpModel ip = new IpModel();
					ip.setIp(h3cNetWorkModel.getIpAddr() + "###" + networkCardModel.getSwitcher());
					ips.add(ip);
					networkcardModels.add(networkCardModel);
				}
				server.setNetcards(networkcardModels);
				server.setPrivateIps(JSONObject.toJSONString(ips));
			}
			initSnapShot(server);
		}
	}

	private String getServerIsoPath(List<H3cStorageModel> storages) {
		/**
		 * <storage> <storeFile>/vms/isos/castools.iso</storeFile>
		 * <capacity>70</capacity> <targetBus>ide</targetBus> <type>file</type>
		 * <device>hda</device> <diskDevice>cdrom</diskDevice> </storage>
		 */
		for (H3cStorageModel storage : storages) {
			if ("cdrom".equalsIgnoreCase(storage.getDiskDevice())) {
				return storage.getStoreFile();
			}
		}
		return null;
	}

	private List<VolumeModel> getServerDiskDevice(List<H3cStorageModel> storages) {
		List<VolumeModel> volumeModels = new ArrayList<>();
		for (H3cStorageModel storage : storages) {
			if ("disk".equalsIgnoreCase(storage.getDiskDevice()) || storage.getDevice().startsWith("vd")) {
				VolumeModel volumeModel = new VolumeModel();
				volumeModel.setName(storage.getStoreFile());
				volumeModel.setSize(storage.getCapacity());
				volumeModels.add(volumeModel);
			}
		}
		return volumeModels;
	}

	/**
	 * 查询交换机
	 *
	 * @param host
	 */
	private void initHostSwitch(HostModel host) {
		H3cNetworkProvider h3cNetworkProvider = new H3cNetworkProvider(butler);
		SwitchModel switchModel = new SwitchModel();
		switchModel.setHostId(Long.valueOf(host.getUuid()));
		BocloudResult<List<RsVSwitch>> listVSwitch = h3cNetworkProvider.listVSwitch(switchModel);
		if (listVSwitch.isSuccess()) {
			host.setH3cSwitchers(listVSwitch.getData());
		}
	}

	/**
	 * 查询交换机
	 *
	 * @param host
	 */
	@SuppressWarnings("unchecked")
	private void initSwitch(HostModel host) {
		H3cNetworkProvider h3cNetworkProvider = new H3cNetworkProvider(butler);
		SwitchModel query = new SwitchModel();
		query.setHostId(Long.valueOf(host.getUuid()));
		BocloudResult bocloudResult = h3cNetworkProvider.listVSwitch(query);
		if (bocloudResult.isSuccess()) {
			host.setSwitchers((List<SwitchModel>) bocloudResult.getData());
		}
	}

	/**
	 * 查询快照
	 *
	 * @param serverModel
	 */
	@SuppressWarnings("unchecked")
	private void initSnapShot(ServerModel serverModel) {
		H3cSnapshotProvider h3cSnapshotProvider = new H3cSnapshotProvider(butler);
		BocloudResult bocloudResult = h3cSnapshotProvider.list(Long.valueOf(serverModel.getInstanceId()));
		if (bocloudResult.isSuccess()) {
			serverModel.setSnapshots((List<SnapshotModel>) bocloudResult.getData());
		}
	}

	@Override
	public BocloudResult listZones(String region) {
		return null;
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		// TODO Auto-generated method stub
		return null;
	}

}
