package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.h3c.H3cTaskMsgModel;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 华三云任务消息操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/16
 */
@Provider("bocloud.cmp.h3c")
public class H3cTaskMsgProvider extends H3cProvider implements TaskMsgProvider {

    private static Logger logger = LoggerFactory.getLogger(H3cTaskMsgProvider.class);

    public H3cTaskMsgProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult get(Long msgId) {
        try {
            Result result = this.doGet("/cas/casrs/message/" + msgId);
            if (result.isSuccess()) {
                String message = result.getMessage();
                H3cTaskMsgModel h3cTaskMsgModel = null;
                Element taskMsg = XmlUtils.getSingleNode(message, "taskMsg");
                if (taskMsg != null) {
                    h3cTaskMsgModel = new H3cTaskMsgModel();
                    h3cTaskMsgModel.setMsgId(XmlUtils.readLong(taskMsg, "msgId"));
                    h3cTaskMsgModel.setName(XmlUtils.readString(taskMsg, "name"));
                    h3cTaskMsgModel.setTargetId(XmlUtils.readLong(taskMsg, "targetId"));
                    h3cTaskMsgModel.setTargetName(XmlUtils.readString(taskMsg, "targetName"));
                    h3cTaskMsgModel.setDetail(XmlUtils.readString(taskMsg, "detail"));
                    h3cTaskMsgModel.setCompleted(XmlUtils.readBoolean(taskMsg, "completed"));
                    h3cTaskMsgModel.setResult(XmlUtils.readInteger(taskMsg, "result"));
                    h3cTaskMsgModel.setFailMsg(XmlUtils.readString(taskMsg, "failMsg"));
                    h3cTaskMsgModel.setEventType(XmlUtils.readInteger(taskMsg, "eventType"));
                }
                return new BocloudResult(true, h3cTaskMsgModel, "获取result成功");
            } else {
                return new BocloudResult(false, result.getMessage());
            }
        } catch (Exception e) {
            logger.error("错误", e);
            return new BocloudResult(false, e.getMessage());
        } finally {
            closeClient();
        }
    }

}
