package com.bocloud.cmp.provider.common;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.Authenticator;
import java.net.HttpURLConnection;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.provider.factory.H3cEventHandlerFactory;

/**
 * 华三云线程池操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/16
 */
public class H3cThreadPool {

    public static String getRegisterKey(String endpoint, int port, String username, String password, String eventType) {
        return endpoint + "," + port + "," + username + "," + password + "," + eventType;
    }

    private static ConcurrentHashMap<String, Thread> threads = new ConcurrentHashMap<>();

    public static void createThread(String endpoint, int port, String username, String password, String eventType) {
        String registerKey = getRegisterKey(endpoint, port, username, password, eventType);
        if (!threads.contains(registerKey)) {
            Thread thread = new HandleResultThread(endpoint, port, username, password, eventType);
            thread.start();
            threads.put(registerKey, thread);
        } else {
            Thread thread = threads.get(registerKey);
            if (!thread.isAlive()) {
                thread.start();
            }
        }

    }

    /**
     * 处理回调的线程
     */
    private static class HandleResultThread extends Thread {

        private static Logger logger = LoggerFactory.getLogger(HandleResultThread.class);

        private String endpoint;

        private int port;

        private String username;

        private String password;

        private String eventType;

        public HandleResultThread(String endpoint, int port, String username, String password, String eventType) {
            this.endpoint = endpoint;
            this.port = port;
            this.username = username;
            this.password = password;
            this.eventType = eventType;
        }

        @Override
        public void run() {
            try {
                // 处理回调
                Authenticator.setDefault(new Authenticator() {
                    @Override
                    protected PasswordAuthentication getPasswordAuthentication() {
                        return new PasswordAuthentication(username, password.toCharArray());
                    }
                });
                URL url = new URL("http://" + endpoint + ":" + port + "/cas/casrs/events?type=" + eventType);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("GET");
                conn.setRequestProperty("accept", "application/xml");
                if (conn.getResponseCode() == 202) {
                    BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                    String line = null;
                    StringBuilder sb = new StringBuilder();
                    while ((line = reader.readLine()) != null) {
                        if (line.length() != 0) {
                            // 事件信息未结束，继续读取
                            sb.append(line);
                        } else {
                            // 读取到一个完整的事件，进行处理
                            String str = sb.toString();
                            sb.delete(0, sb.length());
                            if (!"#HEART_BEAT".equals(str)) {
                                // 判断事件为非心跳事件，对事件信息进行处理
                                H3cEventHandlerFactory.getHandler(str).handle();
                            }
                        }
                    }
                }

            } catch (Exception e) {
                logger.error(ExceptionUtils.getStackTrace(e));
            }
        }
    }

}
