package com.bocloud.cmp.provider.compute;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.ClusterOperateModel;
import com.bocloud.cmp.model.h3c.entity.RsCluster;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsTaskMsg;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云集群操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cClusterProvider extends H3cProvider implements ClusterProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cClusterProvider.class);
	private Butler butler;

	public H3cClusterProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult<List<ClusterModel>> list() {
		logger.info("start sync cluster .... ");
		List<RsCluster> result = null;
		String casuri = "/cas/casrs/cluster/clusters";
		@SuppressWarnings("unchecked")
		RsCommList<RsCluster> comlist = RestUtil.httpGet(defaultHttpClient, casuri, RsCommList.class);
		if (comlist != null) {
			result = comlist.getData();
		}
		logger.info("end sync cluster .... ");
		List<ClusterModel> convertCluster = convertClusters(result);
		return new BocloudResult<List<ClusterModel>>(true, convertCluster, "查询成功！");
	}

	private List<ClusterModel> convertClusters(List<RsCluster> h3cClusters) {
		List<ClusterModel> clusterModels = new ArrayList<>();
		if (!ListTool.isEmpty(h3cClusters)) {
			for (RsCluster h3cCluster : h3cClusters) {
				clusterModels.add(convertCluster(h3cCluster));
			}
		}
		return clusterModels;
	}

	private ClusterModel convertCluster(RsCluster h3cCluster) {
		ClusterModel clusterModel = new ClusterModel();
		if (null != h3cCluster) {
			clusterModel.setName(h3cCluster.getName());
			clusterModel.setUuid(String.valueOf(h3cCluster.getId()));
			clusterModel.setClusterVal(String.valueOf(h3cCluster.getId()));
			clusterModel.setVdcUuid(String.valueOf(h3cCluster.getHostPoolId()));
			clusterModel.setHostNum(h3cCluster.getChildNum());
			clusterModel.setRemark(h3cCluster.getDescription());
			getMoreClusterInfo(clusterModel);
		}
		return clusterModel;
	}

	private void getMoreClusterInfo(ClusterModel clusterModel) {
		logger.info("start sync cluster summary ...");
		BocloudResult summary = this.summary(clusterModel.getClusterVal());
		if (summary.isSuccess()) {
			ClusterModel summaryData = (ClusterModel) summary.getData();
			clusterModel.setMemTotal(summaryData.getMemTotal());
			clusterModel.setCpuTotal(summaryData.getCpuTotal());
			clusterModel.setDiskTotal(summaryData.getDiskTotal());
			clusterModel.setDiskUsed(summaryData.getDiskUsed());
		}
		logger.info("end sync cluster summary ...");
	}

	@Override
	public BocloudResult<ClusterModel> create(ClusterOperateModel model) {
		try {
			String casuri = "/cas/casrs/cluster/add";
			RsCluster rsCluster = new RsCluster();
			rsCluster.setHostPoolId(model.getVdcId());
			rsCluster.setName(model.getClusterName());
			RestUtil.httpPost(defaultHttpClient, casuri, rsCluster, String.class);
			return new BocloudResult<ClusterModel>(true, this.queryByName(model.getClusterName()).getData(), "集群创建成功");
		} catch (Exception e) {
			logger.error("集群创建失败：" + e.getMessage());
			return new BocloudResult<ClusterModel>(false, "集群创建失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsCluster> remove(ClusterOperateModel model) {
		try {
			String uri = "/cas/casrs/cluster/delete/" + model.getClusterId();
			RsTaskMsg rsTaskMsg = RestUtil.httpDelete(defaultHttpClient, uri, RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(rsTaskMsg.getMsgId());
			return new BocloudResult<RsCluster>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "删除成功。" : "删除失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("集群删除失败：" + e.getMessage());
			return new BocloudResult<RsCluster>(false, "集群删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String clusterVal) {
		return null;
	}

	@Override
	public BocloudResult create(ClusterModel clusterModel) {
		return null;
	}

	@Override
	public BocloudResult modify(ClusterModel clusterModel) {
		return null;
	}

	@Override
	public BocloudResult remove(String s) {
		return null;
	}

	@Override
	public BocloudResult addHost(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult removeHost(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult<ClusterModel> queryByName(String name) {
		String url = "/cas/casrs/cluster/name/" + name;
		try {
			RsCluster rsCluster = RestUtil.httpGet(defaultHttpClient, url, RsCluster.class);
			ClusterModel convertCluster = convertCluster(rsCluster);
			return new BocloudResult<ClusterModel>(true, convertCluster, "查询成功！");
		} catch (Exception e) {
			logger.error("根据名称查询集群异常：" + e.getMessage());
			return new BocloudResult<ClusterModel>(false, "根据名称查询集群异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult summary(String clusterVal) {
		String casuri = "/cas/casrs/cluster/summary/" + clusterVal;
		String summary = null;
		try {
			summary = RestUtil.httpGet(defaultHttpClient, casuri, String.class);
		} catch (Exception e) {
			logger.error("查询集群summary信息异常：", e);
			return new BocloudResult(false, e.getMessage());
		}
		if (!StringUtils.isEmpty(summary)) {
			String xmlStr = summary;
			List<org.dom4j.Element> keyvalues = XmlUtils.getNodesByXpath(xmlStr, "//list/keyValue");
			ClusterModel clusterModel = new ClusterModel();
			Long cpuTotal = 0L;
			Double memTotal = 0d;
			Double diskAvail = 0d;
			Double diskTotal = 0d;
			if (!ListTool.isEmpty(keyvalues)) {
				for (org.dom4j.Element keyvalue : keyvalues) {
					if ("CPU总核数".equals(keyvalue.element("key").getTextTrim())) {
						cpuTotal = Long.valueOf(keyvalue.element("value").getTextTrim());
					} else if ("总内存".equals(keyvalue.element("key").getTextTrim())) {
						memTotal = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					} else if ("本地总存储".equals(keyvalue.element("key").getTextTrim())) {
						diskTotal = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					} else if ("本地总可用存储".equals(keyvalue.element("key").getTextTrim())) {
						diskAvail = kbMbGbToGb(keyvalue.element("value").getTextTrim());
					}
				}
			}
			clusterModel.setDiskUsed(new BigDecimal(diskTotal).subtract(new BigDecimal(diskAvail))
					.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
			clusterModel.setDiskTotal(diskTotal);
			clusterModel.setCpuTotal(cpuTotal);
			clusterModel.setMemTotal(memTotal);
			return new BocloudResult(true, clusterModel, "查询成功");
		} else {
			return new BocloudResult(false, "查询失败");
		}
	}

}
