package com.bocloud.cmp.provider.compute;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.HostOperateModel;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.h3c.H3cTaskMsgModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsHost;
import com.bocloud.cmp.model.h3c.entity.RsPhysicalNIC;
import com.bocloud.cmp.model.h3c.entity.RsTaskMsg;
import com.bocloud.cmp.model.h3c.utils.ElementUtils;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云宿主机操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cHostProvider extends H3cProvider implements HostProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cHostProvider.class);

	public H3cHostProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult<RsTaskMsg> create(HostOperateModel model) {
		String casuri = "/cas/casrs/host/add";
		try {
			RsHost rsHost = new RsHost();
			rsHost.setUser(model.getUsername());
			rsHost.setPwd(model.getPassword());
			rsHost.setHostPoolId(model.getVdcId());
			rsHost.setClusterId(model.getClusterId());
			rsHost.setName(model.getHostIp());
			// FIXME 是否启用HA，1：启用，0：未启用，结合clusterID使用。
			rsHost.setEnableHA(0);
			// FIXME 是否忽略重复管理。 布尔 （Boolean）类型
			rsHost.setIgnore(false);
			RsTaskMsg rsTaskMsg = RestUtil.httpPost(defaultHttpClient, casuri, rsHost, RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(rsTaskMsg.getMsgId());
			logger.info("主机创建结果：" + JSON.toJSONString(JSON.toJSONString(waitForComplete)));
			return new BocloudResult<RsTaskMsg>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "主机创建成功。" : "主机创建失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("主机创建失败,失败原因：" + e.getMessage());
			return new BocloudResult<RsTaskMsg>(false, e.getMessage());
		}
	}

	public List<HostModel> convertList(List<? extends RsHost> objects) {
		List<HostModel> hostModels = new ArrayList<HostModel>();
		if (ListTool.isEmpty(objects)) {
			return hostModels;
		}
		HostModel hostModel = null;
		for (RsHost object : objects) {
			hostModel = new HostModel();
			hostModel.setHostIp(object.getIp());
			hostModel.setName(object.getName());
			hostModel.setStatus(object.getStatus() + "");
			hostModel.setClusterUuid(String.valueOf(object.getClusterId()));
			hostModels.add(hostModel);
		}
		return hostModels;
	}

	@Override
	public BocloudResult<List<HostModel>> list() {
		List<HostModel> hostModels = new ArrayList<HostModel>();
		String casuri = "/cas/casrs/host";
		@SuppressWarnings("unchecked")
		RsCommList<RsHost> comlist = RestUtil.httpGet(defaultHttpClient, casuri, RsCommList.class);
		if (comlist == null || null == comlist.getData()) {
			return new BocloudResult<List<HostModel>>(false, hostModels, "查询成功！");
		}
		List<RsHost> data = comlist.getData();
		for (RsHost rsHost : data) {
			BocloudResult<HostModel> detail = this.detail(rsHost.getId());
			if (detail.isSuccess()) {
				HostModel hostDetail = detail.getData();
				hostDetail.setClusterUuid(String.valueOf(rsHost.getClusterId()));
				hostModels.add(hostDetail);
			}
		}
		return new BocloudResult<List<HostModel>>(true, hostModels, "查询成功！");
	}

	@Override
	public BocloudResult monitor() {
		return null;
	}

	@Override
	public BocloudResult<List<HostModel>> list(String clusterId) {
		List<HostModel> hostModels = new ArrayList<HostModel>();
		String url = "/cas/casrs/cluster/hosts?offset=0&limit=" + Integer.MAX_VALUE + "&clusterId=" + clusterId;
		try {
			@SuppressWarnings("unchecked")
			RsCommList<RsHost> comlist = RestUtil.httpGet(defaultHttpClient, url, RsCommList.class);
			if (comlist == null || null == comlist.getData()) {
				return new BocloudResult<List<HostModel>>(false, hostModels, "查询成功！");
			}
			List<RsHost> data = comlist.getData();
			for (RsHost rsHost : data) {
				BocloudResult<HostModel> detail = this.detail(rsHost.getId());
				if (detail.isSuccess()) {
					HostModel hostDetail = detail.getData();
					hostDetail.setClusterUuid(String.valueOf(rsHost.getClusterId()));
					hostModels.add(hostDetail);
				}
			}
			return new BocloudResult<List<HostModel>>(true, hostModels, "查询成功！");
		} catch (Exception e) {
			logger.error("根据集群ID查询宿主机异常：" + e.getMessage());
			return new BocloudResult<List<HostModel>>(false, "根据集群ID查询宿主机异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(HostOperateModel model) {
		String deleteUrl = "/cas/casrs/host/delete?id=" + model.getHostId();
		Result result = null;
		try {
			result = this.doDelete(deleteUrl);
			if (result.isSuccess()) {
				String message = result.getMessage();
				JSONObject object = XmlUtils.xml2FastJSONObject(message);
				H3cTaskMsgModel msgModel = JSONObject.toJavaObject(object.getJSONObject("taskMsg"),
						H3cTaskMsgModel.class);
				return new BocloudResult(true, msgModel, "删除主机成功");
			}
			return new BocloudResult(false, result.getMessage());
		} catch (Exception e) {
			logger.error(ExceptionUtils.getStackTrace(e));
			return new BocloudResult(false, "删除主机失败");
		} finally {
			closeClient();
		}
	}

	/**
	 * 进入维护模式
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult enterMode(HostOperateModel model) {
		String enterUrl = "/cas/casrs/host/into";
		try {
			Element hostEle = ElementUtils.creatRoot("host");
			ElementUtils.addChild(hostEle, "id", model.getHostIp());
			ElementUtils.addChild(hostEle, "name", model.getHostName());
			ElementUtils.addChild(hostEle, "ip", model.getHostIp());
			Result result = this.doPut(enterUrl, ElementUtils.toFormatedXML(hostEle));
			if (result.isSuccess()) {
				String message = result.getMessage();
				JSONObject object = XmlUtils.xml2FastJSONObject(message);
				H3cTaskMsgModel msgModel = JSONObject.toJavaObject(object.getJSONObject("taskMsg"),
						H3cTaskMsgModel.class);
				return new BocloudResult(true, msgModel, "发出进入维护模式消息成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}

	}

	@Override
	public BocloudResult exitMode(HostOperateModel model) {
		String enterUrl = "/cas/casrs/host/exit";
		try {
			Element hostEle = ElementUtils.creatRoot("host");
			ElementUtils.addChild(hostEle, "id", model.getHostIp());
			ElementUtils.addChild(hostEle, "name", model.getHostName());
			ElementUtils.addChild(hostEle, "ip", model.getHostIp());
			Result result = this.doPut(enterUrl, ElementUtils.toFormatedXML(hostEle));
			if (result.isSuccess()) {
				String message = result.getMessage();
				JSONObject object = XmlUtils.xml2FastJSONObject(message);
				H3cTaskMsgModel msgModel = JSONObject.toJavaObject(object.getJSONObject("taskMsg"),
						H3cTaskMsgModel.class);
				return new BocloudResult(true, msgModel, "发出退出维护模式消息成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}
	}

	/**
	 * not implement FIXME
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult disconn(HostOperateModel model) {
		return new BocloudResult(false, "没有实现这个接口");
	}

	/**
	 * not implement FIXME
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult reconn(HostOperateModel model) {
		return new BocloudResult(false, "没有实现这个接口");
	}

	/**
	 * not implement FIXME
	 *
	 * @param model
	 * @return
	 */
	@Override
	public BocloudResult moveTo(HostOperateModel model) {
		return new BocloudResult(false, "没有实现这个接口");
	}

	@Override
	public BocloudResult shutdown(HostOperateModel model) {
		try {
			Element hostEle = ElementUtils.creatRoot("host");
			ElementUtils.addChild(hostEle, "id", model.getHostId());
			ElementUtils.addChild(hostEle, "name", model.getHostName());
			ElementUtils.addChild(hostEle, "ip", model.getHostIp());
			Result result = this.doPut("/cas/casrs/host/shutoff", ElementUtils.toFormatedXML(hostEle));
			if (result.isSuccess()) {
				String message = result.getMessage();
				JSONObject object = XmlUtils.xml2FastJSONObject(message);
				H3cTaskMsgModel msgModel = JSONObject.toJavaObject(object.getJSONObject("taskMsg"),
						H3cTaskMsgModel.class);
				return new BocloudResult(true, msgModel, "发送关闭主机请求成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}
	}

	@Override
	public BocloudResult reboot(HostOperateModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult detail(HostOperateModel model) {
		return detail(model.getHostId());
	}

	@Override
	public BocloudResult<HostModel> detail(Long hostId) {
		HostModel hostModel = new HostModel();
		String uri = "/cas/casrs/host/id/" + hostId;
		try {
			RsHost host = RestUtil.httpGet(defaultHttpClient, uri, RsHost.class);
			if (null != host) {
				// 处理网卡
				hostModel.setCpuCore(host.getCpuCount());
				hostModel.setUuid(String.valueOf(host.getId()));
				hostModel.setHostIp(host.getIp());
				hostModel.setName(host.getName() == null ? host.getIp() : host.getName());
				hostModel.setCpuModel(host.getCpuModel());
				hostModel.setDiskTotal(host.getDiskSize().doubleValue());
				if (null != host.getMemorySize()) {
					double memory = new BigDecimal(host.getMemorySize()).divide(new BigDecimal(1024), 2,
							RoundingMode.HALF_UP).doubleValue();
					hostModel.setMemTotal(memory);
				}
				hostModel.setVendor(host.getVendor());
				hostModel.setOsCategory(host.getModel());
				hostModel.setCpuTotal(host.getCpuFrequence().longValue() * host.getCpuCount());
				hostModel.setCpuNum(host.getCpuSockets());
				hostModel.setStatus(host.getStatus() == 1 ? BocloudStatus.Server.RUNNING.name()
						: BocloudStatus.Server.EXCEPTION.name());
				hostModel.setPowerState(host.getStatus() == 1 ? BocloudStatus.Server.RUNNING.name()
						: BocloudStatus.Server.EXCEPTION.name());

				List<RsPhysicalNIC> pnics = host.getPnicList();
				List<NetworkCardModel> networkcards = new ArrayList<>();
				for (RsPhysicalNIC pnic : pnics) {
					NetworkCardModel cardModel = new NetworkCardModel();
					cardModel.setName(pnic.getName());
					cardModel.setUsed("1".equals(pnic.getStatus() + ""));
					cardModel.setMac(pnic.getMacAddr());
					cardModel.setRemark(pnic.getDescription());
					networkcards.add(cardModel);
				}
				hostModel.setNetcards(networkcards);
				return new BocloudResult<HostModel>(true, hostModel, "查询成功");
			} else {
				return new BocloudResult<HostModel>(false, hostModel, "查询主机数据异常！");
			}
		} catch (Exception e) {
			logger.error("请求错误");
		}
		return new BocloudResult<HostModel>(false, "调用接口失败");
	}

	@Override
	public BocloudResult summary(Long id) {
		String url = "/cas/casrs/host/summary/" + id;
		try {
			Result result = doGet(url);
			if (result.isSuccess()) {
				String xml = result.getMessage();
				JSONArray jsonArray = JSONObject.parseObject(XmlUtils.xml2JSON(xml)).getJSONObject("list")
						.getJSONArray("keyValue");
				for (Object o : jsonArray) {
					JSONObject jsonObject = (JSONObject) o;
					String key = jsonObject.getString("key");
					if ("run".equals(key)) {
						Integer run = jsonObject.getInteger("value");
						HostModel hostModel = new HostModel();
						// 只能通过是否有正在运行的云主机判断主机是否正在运行
						if (run >= 1) {
							hostModel.setStatus(BocloudStatus.Server.RUNNING.name());
						}
						return new BocloudResult(true, hostModel, "查询成功");
					}
				}
			} else {
				return new BocloudResult(false, result.getMessage());
			}

		} catch (Exception e) {
			logger.error("请求错误", e);
		}
		return new BocloudResult(false, "调用接口失败");
	}

	@Override
	public BocloudResult listPerf() {
		// TODO Auto-generated method stub
		return null;
	}
}
