package com.bocloud.cmp.provider.compute;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.IpModel;
import com.bocloud.cmp.model.OSMigrateModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cNetWorkModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.model.h3c.H3cStorageModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.model.h3c.entity.PortProfile;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsDeployVmOsInfo;
import com.bocloud.cmp.model.h3c.entity.RsDomain;
import com.bocloud.cmp.model.h3c.entity.RsDomainDeploy;
import com.bocloud.cmp.model.h3c.entity.RsDomainDetail;
import com.bocloud.cmp.model.h3c.entity.RsDomainForModify;
import com.bocloud.cmp.model.h3c.entity.RsDomainForModify.BasicConfig;
import com.bocloud.cmp.model.h3c.entity.RsDomainForModify.CpuConfig;
import com.bocloud.cmp.model.h3c.entity.RsDomainForModify.MemoryConfig;
import com.bocloud.cmp.model.h3c.entity.RsDomainNetwork;
import com.bocloud.cmp.model.h3c.entity.RsDomainStorage;
import com.bocloud.cmp.model.h3c.entity.RsSnapshot;
import com.bocloud.cmp.model.h3c.entity.RsStoragePool;
import com.bocloud.cmp.model.h3c.entity.RsStore;
import com.bocloud.cmp.model.h3c.entity.RsTaskMsg;
import com.bocloud.cmp.model.h3c.entity.RsVolume;
import com.bocloud.cmp.model.h3c.entity.VmParams;
import com.bocloud.cmp.model.h3c.utils.ElementUtils;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.model.h3c.utils.VmEnum;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.builder.H3cServerModelBuilder;
import com.bocloud.cmp.provider.storage.H3cStoragePoolProvider;
import com.bocloud.cmp.provider.storage.H3cVolumeProvider;
import com.bocloud.cmp.provider.utils.H3cToolsUtils;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云主机操作
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cServerProvider extends H3cProvider implements ServerProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cServerProvider.class);
	private Butler butler;

	public H3cServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult<List<RsDomain>> list() {
		return null;
	}

	// 云主机状态。 2：运行 3：关闭 4：暂停。
	@Override
	public BocloudResult<List<RsDomain>> list(String hostId) {
		List<RsDomain> servers = new ArrayList<>();
		try {
			String casuri = "/cas/casrs/host/id/" + hostId + "/vm";
			@SuppressWarnings("unchecked")
			RsCommList<RsDomain> comlist = RestUtil.httpGet(defaultHttpClient, casuri, RsCommList.class);
			if (comlist == null || null == comlist.getData()) {
				return new BocloudResult<List<RsDomain>>(true, servers, "暂无云主机信息");
			}
			List<RsDomain> data = comlist.getData();
			if (!ListTool.isEmpty(data)) {
				for (RsDomain rsDomain : data) {
					BocloudResult<RsDomain> detail = this.detail(String.valueOf(rsDomain.getId()));
					if (detail.isSuccess()) {
						RsDomain rsDomainDetail = detail.getData();
						rsDomainDetail.setClusterId(rsDomain.getClusterId());
						rsDomainDetail.setOsDesc(rsDomain.getOsDesc());
						rsDomainDetail.setStorageCapacity(rsDomain.getStorageCapacity());
						rsDomainDetail.setCreateDate(rsDomain.getCreateDate());
						servers.add(rsDomainDetail);
					}
				}
			}
			return new BocloudResult<List<RsDomain>>(true, servers, "查询成功");
		} catch (Exception e) {
			logger.error("查询失败{}", ExceptionUtils.getStackTrace(e));
			return new BocloudResult<List<RsDomain>>(false, "查询失败");
		}
	}

	/**
	 * @param server
	 * @return
	 */
	@Override
	public BocloudResult<RsDomain> create(ServerModel server) {
		try {
			logger.info("创建云主机参数：" + JSON.toJSONString(server));
			String tempInstId = server.getTempInstId();
			if (!StringUtils.isEmpty(tempInstId)) {
				return deployTemplate(server);
			}
			String uri = "/cas/casrs/vm/add";
			BocloudResult<RsDomain> convertData = convertData(server);
			logger.info("convertData : " + JSON.toJSONString(convertData));
			if (convertData.isFailed()) {
				return convertData;
			}
			RsTaskMsg rsTaskMsg = RestUtil.httpPost(defaultHttpClient, uri, convertData.getData(), RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(rsTaskMsg.getMsgId());
			if (waitForComplete.getResult() == 0) {
				Long targetId = waitForComplete.getTargetId();
				BocloudResult<RsDomain> detail = this.detail(String.valueOf(targetId));
				RsDomain data = detail.getData();
				data.setClusterId(Long.parseLong(server.getClusterUuid()));
				logger.info("create result : " + JSON.toJSONString(data));
				return new BocloudResult<RsDomain>(true, data, "创建成功！");
			} else {
				logger.info("create result : 创建失败 " + waitForComplete.getFailMsg());
				return new BocloudResult<RsDomain>(false, "创建失败: " + waitForComplete.getFailMsg());
			}
		} catch (Exception e) {
			logger.error("云主机创建失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "云主机创建失败：" + e.getMessage());
		}
	}

	public BocloudResult<RsDomain> deployTemplate(ServerModel server) {
		logger.info("***********模板部署云主机begin************");
		RsDomainDeploy rs = new RsDomainDeploy();
		rs.setId(Long.parseLong(server.getTempInstId()));
		rs.setDomainName(server.getName());
		rs.setTargetHostId(server.getHostId());
		rs.setDeployMode(1);
		rs.setProtectMode(0);
		rs.setVmNum(0);
		rs.setDeployType(0);
		rs.setTitle(server.getName());
		List<RsStore> storages = new ArrayList<RsStore>();
		List<JSONObject> dataDiskList = server.getDataDiskList();
		for (JSONObject jsonObject : dataDiskList) {
			String storeFile = jsonObject.getString("storeFile");
			RsStore store = new RsStore();
			store.setSrc(storeFile.substring(storeFile.lastIndexOf("/") + 1));
			store.setDest(server.getName() + "_" + System.nanoTime());
			store.setPool(server.getStoreName());
			storages.add(store);
		}
		rs.setStorages(storages);

		List<IpModel> ips = server.getIps();
		if (!ListTool.isEmpty(ips)) {
			IpModel ipModel = ips.get(0);
			RsDeployVmOsInfo osInfo = new RsDeployVmOsInfo();
			osInfo.setSysIp(ipModel.getIp());
			osInfo.setSysMask(ipModel.getMask());
			osInfo.setIsBindIp(true);
			osInfo.setSysName("localhost");
			rs.setOsInfo(osInfo);
		}
		String uri = "/cas/casrs/vm/deploy";
		RsTaskMsg rsTaskMsg = RestUtil.httpPost(defaultHttpClient, uri, rs, RsTaskMsg.class);
		RsTaskMsg waitForComplete = waitForComplete(rsTaskMsg.getMsgId());
		if (waitForComplete.getResult() == 0) {
			BocloudResult<RsDomain> findVm = this.findVmByName(server);
			if (findVm.isFailed()) {
				return findVm;
			}
			RsDomain currentVm = findVm.getData();
			BocloudResult<RsDomain> detail = this.detail(String.valueOf(currentVm.getId()));
			RsDomain data = detail.getData();
			data.setClusterId(Long.parseLong(server.getClusterUuid()));
			logger.info("create result : " + JSON.toJSONString(data));
			return new BocloudResult<RsDomain>(true, data, "创建成功！");
		} else {
			logger.info("create result : 创建失败 " + waitForComplete.getFailMsg());
			return new BocloudResult<RsDomain>(false, "创建失败: " + waitForComplete.getFailMsg());
		}
	}

	private BocloudResult findVmByName(ServerModel server) {
		try {
			String ipAddress = null;
			List<IpModel> ips = server.getIps();
			if (!ListTool.isEmpty(ips)) {
				IpModel ipModel = ips.get(0);
				ipAddress = ipModel.getIp();
			}
			BocloudResult<List<RsDomain>> vmListResult = this.list(String.valueOf(server.getHostId()));
			if (vmListResult.isFailed()) {
				return vmListResult;
			}
			List<RsDomain> data = vmListResult.getData();
			if (ListTool.isEmpty(data)) {
				return new BocloudResult(false, "云主机查询失败！");
			}
			for (RsDomain rsDomain2 : data) {
				String vmIp = "";
				List<RsDomainNetwork> networks = rsDomain2.getNetworks();
				if (!ListTool.isEmpty(networks)) {
					RsDomainNetwork rsDomainNetwork = networks.get(0);
					vmIp = rsDomainNetwork.getIpAddr();
				}
				if (!StringUtils.isEmpty(ipAddress) && ipAddress.equalsIgnoreCase(vmIp)
						&& server.getName().equalsIgnoreCase(rsDomain2.getName())) {
					return new BocloudResult(true, rsDomain2, "云主机查询成功！");
				}
				continue;
			}
			return new BocloudResult(false, "暂未查询到云主机信息");
		} catch (Exception e) {
			logger.error("查询云主机异常：" + e.getMessage());
			return new BocloudResult(false, "查询云主机异常：" + e.getMessage());
		}
	}

	/**
	 * @param server
	 * @return
	 */
	@Override
	public BocloudResult create(H3cServerModel server) {
		String url = "/cas/casrs/vm/add";
		try {
			H3cServerModelBuilder h3cBasicBuilder = new H3cServerModelBuilder();
			h3cBasicBuilder.initByModel(server);
			String xml = h3cBasicBuilder.buildCreateXml();

			Result result = doPost(url, xml);
			if (result.isSuccess()) {
				String resultXml = result.getMessage();
				String msgId = XmlUtils.getXmlText(resultXml, "msgId");
				return new BocloudResult(true, msgId, resultXml);
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	private BocloudResult<RsDomain> convertData(ServerModel model) {
		RsDomain rsDomain = new RsDomain();
		rsDomain.setName(model.getName());
		rsDomain.setTitle(model.getName());
		if (!StringUtils.isEmpty(model.getRemark())) {
			rsDomain.setDescription(model.getRemark());
		}
		rsDomain.setHostId(model.getHostId());
		rsDomain.setClusterId(Long.parseLong(model.getClusterUuid()));
		rsDomain.setHostPoolId(model.getVdcId());

		rsDomain.setEnableHA(0);
		// 启动优先级，当集群启用HA后有效，枚举值：0：低级，1：中级，2：高级。整型（Integer）类型，必须。
		rsDomain.setPriority(1);
		// 是否允许云主机自动迁移，实现资源动态调整，枚举值：1=允许，0=不允许。整型（Integer）类型，必须。
		rsDomain.setAutoMigrate(0);

		rsDomain.setMemory(model.getMemory().longValue() * 1024L);
		// 内存预留。内存预留百分比。 整型（Integer）类型，取值范围:0-100。必填。
		rsDomain.setMemoryBacking(10);
		// 内存资源优先级。枚举值：0=低，50=中，100=高 整型（Integer）类型，取值:0,50,100。必填。
		rsDomain.setMemoryPriority(0);
		// 内存大小，单位为由memoryUnit决定,换算成MB后，必须与memory指定的值相同。整型（Integer）类型，必须。
		rsDomain.setMemoryInit(model.getMemory().doubleValue());
		// 内存显示单位，字符串。枚举类型：GB,MB。
		rsDomain.setMemoryUnit("GB");

		// CPU个数，最小值为1，最大值由主机实际情况决定。整型（Integer）类型，必须。
		rsDomain.setCpuSockets(model.getCpu());
		// CPU核数，最小值为1，最大值由主机实际情况决定。整型（Integer）类型，必须。
		rsDomain.setCpuCores(1);
		// CPU调度优先级，枚举值：1024：高，512：中，256：低。整型（Integer）类型，必须。
		rsDomain.setCpuShares(512);

		// 是否可用。整型（Integer）类型，设置为1，必须。
		rsDomain.setFormatEnable(1);
		rsDomain.setMaxCpuSocket(24);
		// I/O优先级，枚举值：500：高，300：中，200：低。整型（Integer）类型，必须。
		rsDomain.setBlkiotune(300);
		rsDomain.setOsBit("x86_64");
		// rsDomain.setImgFileName((String)md.get("imgFileName"));
		rsDomain.setImgFileType("raw");
		// 操作系统类型，枚举值：0：Windows,1：Linux。整型（Integer）类型，必须。
		rsDomain.setSystem("Windows".equalsIgnoreCase(model.getOsCategory()) ? 0 : 1);
		// 显示驱动
		rsDomain.setDrive("cirrus");
		// rsDomain.setOsVersion("Red Hat Enterprise Linux7(64-bit)");
		rsDomain.setOsVersion(model.getOsName());

		// 显示类型
		rsDomain.setViewType("vnc");
		// 自动配置
		rsDomain.setAuto(0);
		// 引导设备
		rsDomain.setBootingDevice(1);
		// 是否自动启动
		rsDomain.setAutoBooting(1);
		rsDomain.setAutoLoadVirtio(false);

		BocloudResult<List<RsDomainStorage>> domainStorages = domainStorages(model);
		if (domainStorages.isFailed()) {
			return new BocloudResult<RsDomain>(false, domainStorages.getMessage());
		}
		if (!ListTool.isEmpty(domainStorages.getData())) {
			rsDomain.setStorages(domainStorages.getData());
		}
		BocloudResult<List<RsDomainNetwork>> domainNetworks = domainNetworks(model);
		if (domainNetworks.isFailed()) {
			return new BocloudResult<RsDomain>(false, domainNetworks.getMessage());
		}
		if (!ListTool.isEmpty(domainNetworks.getData())) {
			rsDomain.setNetworks(domainNetworks.getData());
		}
		return new BocloudResult<RsDomain>(true, rsDomain, "success");
	}

	public BocloudResult<List<RsDomainNetwork>> domainNetworks(ServerModel model) {
		List<RsDomainNetwork> rsListNetWork = new ArrayList<RsDomainNetwork>();
		try {
			List<JSONObject> h3cNetworkCards = model.getH3cNetworkCards();
			if (!ListTool.isEmpty(h3cNetworkCards)) {
				RsDomainNetwork rsDomainNetwork = null;
				Long profileId = 1L;
				PortProfile networkProfile = getNetworkProfile();
				if (null != networkProfile) {
					profileId = networkProfile.getId();
				}
				for (JSONObject jsonObject : h3cNetworkCards) {
					rsDomainNetwork = new RsDomainNetwork();
					// 虚拟交换机名称。字符串（String）类型，必须。
					// rsDomainNetwork.setVsName("vswitch0");
					rsDomainNetwork.setVsName(jsonObject.getString("vsName"));
					// 虚拟交换机id。长整型（Long）类型，必须。
					// rsDomainNetwork.setVsId(1L);
					rsDomainNetwork.setVsId(jsonObject.getLong("vsId"));
					// 网络策略模板id。长整型（Long）类型，必须。
					rsDomainNetwork.setProfileId(profileId);
					// 网卡设备型号，枚举值：rtl8139：普通网卡;virtio：Virtio网卡;e1000：Intel
					// e1000网卡。
					rsDomainNetwork.setDeviceModel("virtio");
					// 是否设置内核加速，枚举值：1：代表设置内核加速，0：代表不设置。整型（Integer）类型，默认为0。
					rsDomainNetwork.setKernelAccelerated(0);
					// 网卡绑定IP地址。字符串（String）类型，符合IPv4格式，可选项。
					String ipAddr = jsonObject.getString("ipAddress");
					if (!StringUtils.isEmpty(ipAddr)) {
						rsDomainNetwork.setIpAddr(ipAddr);
					}
					rsListNetWork.add(rsDomainNetwork);
				}
			}
			logger.info("rsListNetWork : " + JSON.toJSONString(rsListNetWork));
			return new BocloudResult<List<RsDomainNetwork>>(true, rsListNetWork, "success");
		} catch (Exception e) {
			logger.error("获取网卡数据异常：" + e.getMessage());
			return new BocloudResult<List<RsDomainNetwork>>(false, "获取网卡数据异常：" + e.getMessage());
		}
	}

	public PortProfile getNetworkProfile() {
		String casuri = "/cas/casrs/profile";
		@SuppressWarnings("unchecked")
		RsCommList<PortProfile> httpGet = RestUtil.httpGet(defaultHttpClient, casuri, RsCommList.class);
		List<PortProfile> data = httpGet.getData();
		if (ListTool.isEmpty(data)) {
			return null;
		}
		return data.get(0);
	}

	public BocloudResult<List<RsDomainStorage>> domainStorages(ServerModel model) {
		List<RsDomainStorage> rsListStorage = new ArrayList<RsDomainStorage>();
		List<JSONObject> dataDiskList = model.getDataDiskList();
		logger.info("dataDiskList : " + JSON.toJSONString(dataDiskList));
		if (!ListTool.isEmpty(dataDiskList)) {
			RsDomainStorage rsDomainStorage = null;
			for (JSONObject jsonObject : dataDiskList) {
				rsDomainStorage = new RsDomainStorage();
				String device = jsonObject.getString("diskDevice");
				rsDomainStorage.setDevice(device);
				if ("disk".equalsIgnoreCase(device)) {
					BocloudResult<RsDomainStorage> buildDisk = buildDisk(jsonObject, String.valueOf(model.getHostId()),
							model);
					if (buildDisk.isFailed()) {
						return new BocloudResult<List<RsDomainStorage>>(false, buildDisk.getMessage());
					}
					rsListStorage.add(buildDisk.getData());
				}
				if ("cdrom".equalsIgnoreCase(device)) {
					rsListStorage.add(buildCdromDisk(rsDomainStorage, jsonObject));
				}
				if ("floppy".equalsIgnoreCase(device)) {
					rsListStorage.add(buildFloppyDisk(rsDomainStorage, jsonObject));
				}
			}
		}
		logger.info("rsListStorage : " + JSON.toJSONString(rsListStorage));
		return new BocloudResult<List<RsDomainStorage>>(true, rsListStorage, "success");
	}

	private RsDomainStorage buildFloppyDisk(RsDomainStorage rsDomainStorage, JSONObject jsonObject) {
		rsDomainStorage.setDevice("floppy");
		rsDomainStorage.setStoreFile(jsonObject.getString("storeFile") == null ? "autoVirtio" : jsonObject
				.getString("storeFile"));
		rsDomainStorage.setType(jsonObject.getString("type") == null ? "file" : jsonObject.getString("type"));
		rsDomainStorage.setDriveType(jsonObject.getString("driveType") == null ? "raw" : jsonObject
				.getString("driveType"));
		return rsDomainStorage;
	}

	private RsDomainStorage buildCdromDisk(RsDomainStorage rsDomainStorage, JSONObject jsonObject) {
		rsDomainStorage.setDevice("cdrom");
		rsDomainStorage.setStoreFile(jsonObject.getString("storeFile"));
		rsDomainStorage.setType(jsonObject.getString("type") == null ? "file" : jsonObject.getString("type"));
		rsDomainStorage.setDriveType(jsonObject.getString("driveType") == null ? "raw" : jsonObject
				.getString("driveType"));
		return rsDomainStorage;
	}

	private BocloudResult<RsDomainStorage> buildDisk(JSONObject jsonObject, String hostId, ServerModel model) {
		RsDomainStorage rsDomainStorage = new RsDomainStorage();
		try {
			Long size = jsonObject.getLong("capacity");
			String rsStoragePoolName = jsonObject.getString("storeName");
			String volumeName = jsonObject.getString("volumeName");

			RsStoragePool storagePool = getRandomStoragePool(hostId, rsStoragePoolName);
			if (null == storagePool) {
				return new BocloudResult<RsDomainStorage>(false, "存储卷创建失败!");
			}
			logger.info("storagePool : " + JSON.toJSONString(storagePool));
			if (StringUtils.isEmpty(rsStoragePoolName)) {
				rsStoragePoolName = storagePool.getName();
			}
			if (StringUtils.isEmpty(volumeName)) {
				volumeName = "vm_" + System.nanoTime();
			}
			RsVolume rsVolume = new RsVolume();
			rsVolume.setHostId(Long.parseLong(hostId));
			rsVolume.setSpName(rsStoragePoolName);
			rsVolume.setVolName(volumeName);
			rsVolume.setCapacity(1024L * size);
			rsVolume.setFormat("qcow2");
			rsVolume.setBaseFile(null);
			String uri = "/cas/casrs/storage/volume/add";
			RestUtil.httpPost(defaultHttpClient, uri, rsVolume, null);
			// 磁盘缓存方，枚举值：directsync，writethrough，writeback，none。字符串（String）类型，当存储卷为文件类型时不能为空。
			String cache = jsonObject.getString("cache");
			if (StringUtils.isEmpty(cache)) {
				cache = "directsync";
			}
			rsDomainStorage.setCacheType(cache);
			// 存储卷类型，必须和使用的存储卷的类型保持一致，枚举值：qcow2，raw 字符串（String）类型，必须。
			String driveType = jsonObject.getString("driveType");
			if (StringUtils.isEmpty(driveType)) {
				driveType = "qcow2";
			}
			rsDomainStorage.setDriveType(driveType);

			// 存储类型，枚举值：disk：硬盘， cdrom：光驱， floppy：软驱。字符串（String）类型，必须。
			rsDomainStorage.setDevice("disk");
			/**
			 * 存储设备类型，枚举值：ide：IDE硬盘， scsi：SCSI硬盘， usb：USB硬盘;，virtio：Virtio 硬盘。
			 * 字符串（String）类型，必须。
			 */
			rsDomainStorage.setTargetBus("virtio");
			// 存储文件路径,当增加软驱时，若值为autoVirtio，则根据操作系统类型自动加载一个软驱设备。字符串（String）类型，必须。
			rsDomainStorage.setStoreFile(storagePool.getPath() + "/" + volumeName);
			/** 分配类型:0指定 1动态分配。 **/
			rsDomainStorage.setAssignType(0);
			// 类型。枚举值：file 文件， block 块设备。字符串（String）类型，必须。
			String file = jsonObject.getString("file");
			if (StringUtils.isEmpty(file)) {
				file = "file";
			}
			rsDomainStorage.setType(file);
			return new BocloudResult<RsDomainStorage>(true, rsDomainStorage, "success");
		} catch (Exception e) {
			logger.error("创建磁盘失败：" + e.getMessage());
			return new BocloudResult<RsDomainStorage>(false, e.getMessage());
		}
	}

	public RsStoragePool getRandomStoragePool(String hostId, String storagePoolName) {
		RsStoragePool rsStoragePool = null;
		H3cStoragePoolProvider storagePoolProvider = new H3cStoragePoolProvider(butler);
		if (!StringUtils.isEmpty(storagePoolName)) {
			BocloudResult<RsStoragePool> detail = storagePoolProvider.detail(hostId, storagePoolName);
			if (detail.isSuccess()) {
				rsStoragePool = detail.getData();
			}
		} else {
			BocloudResult<List<RsStoragePool>> bocloudResult = storagePoolProvider.list(hostId);
			if (bocloudResult.isSuccess()) {
				List<RsStoragePool> rsStoragePools = (List<RsStoragePool>) bocloudResult.getData();
				if (!ListTool.isEmpty(rsStoragePools)) {
					rsStoragePool = rsStoragePools.get(0);
					for (RsStoragePool storagePool : rsStoragePools) {
						if (storagePool.getFreeSize() > rsStoragePool.getFreeSize()) {
							rsStoragePool = storagePool;
						}
					}
				}
			}
		}
		return rsStoragePool;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel model) {
		String url = "/cas/casrs/vm/migrate";
		try {
			Element vmParamter = ElementUtils.creatRoot("vmParamter");
			ElementUtils.addChild(vmParamter, "id", model.getId());
			ElementUtils.addChild(vmParamter, "domainName", model.getDomainName());
			ElementUtils.addChild(vmParamter, "title", model.getTitle());
			ElementUtils.addChild(vmParamter, "targetHostId", model.getTargetHostId());
			ElementUtils.addChild(vmParamter, "onlineMigrate", model.getOnlineMigrate());
			ElementUtils.addChild(vmParamter, "migrateType", model.getMigrateType());
			ElementUtils.addChild(vmParamter, "targetPath", model.getTargetPath());
			ElementUtils.addChild(vmParamter, "targetPool", model.getTargetPool());
			ElementUtils.addChild(vmParamter, "dstPersist", model.getDstPersist());
			ElementUtils.addChild(vmParamter, "dstSuspend", model.getDstSuspend());
			Result result = doPost(url, ElementUtils.toFormatedXML(vmParamter));
			if (result.isSuccess()) {
				return new BocloudResult(true, "任务下发成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel model) {
		return null;
	}

	@Override
	public BocloudResult<RsTaskMsg> snapshot(SnapshotModel snapshotModel) {
		String uri = "/cas/casrs/vm/snapshot";
		try {
			RsSnapshot rsSnapshot = new RsSnapshot();
			rsSnapshot.setVmId(Long.parseLong(snapshotModel.getInstanceId()));
			rsSnapshot.setName(snapshotModel.getName());
			rsSnapshot.setDesc(snapshotModel.getRemark());
			RsTaskMsg task = RestUtil.httpPost(defaultHttpClient, uri, rsSnapshot, RsTaskMsg.class);
			if (null == task) {
				new BocloudResult<RsTaskMsg>(false, "快照创建失败,失败原因：接口异常！");
			}
			RsTaskMsg waitForComplete = waitForComplete(task.getMsgId());
			return new BocloudResult<RsTaskMsg>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "快照创建成功!" : "快照创建失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("快照创建失败：" + e.getMessage());
			return new BocloudResult<RsTaskMsg>(false, "快照创建失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(ServerModel server) {
		try {
			String uri = "/cas/casrs/vm/detail/" + server.getInstanceId();
			RsDomainDetail rsDomainDetail = RestUtil.httpGet(defaultHttpClient, uri, RsDomainDetail.class);
			String casuri = "/cas/casrs/vm/modify";
			Integer cpu = server.getCpu();
			if (null != cpu && 0 != cpu) {
				// 修改CPU
				RsDomainForModify rsDomain = new RsDomainForModify();
				rsDomain.setId(Long.parseLong(server.getInstanceId()));
				CpuConfig cpuConfig = rsDomainDetail.getCpuConfig();
				rsDomain.setCpuConfig(cpuConfig);
				cpuConfig.setCpuCores(cpu / cpuConfig.getCpuSockets());
				RestUtil.httpPut(defaultHttpClient, casuri, rsDomain, RsTaskMsg.class);
			}
			BigDecimal memoryDecimal = server.getMemory();
			if (null != memoryDecimal && memoryDecimal.longValue() != 0) {
				// 修改内存
				RsDomainForModify rsDomain = new RsDomainForModify();
				rsDomain.setId(Long.parseLong(server.getInstanceId()));
				MemoryConfig memory = rsDomainDetail.getMemory();
				rsDomain.setMemory(memory);
				memory.setMemoryInit(memoryDecimal.doubleValue());
				String memoryUnit = memory.getMemoryUnit();
				Long memorySize = 0L;
				if ("MB".equals(memoryUnit)) {
					memorySize = memoryDecimal.longValue();
				} else {
					memorySize = memoryDecimal.longValue() * 1024;
				}
				memory.setSize(memorySize);
				RestUtil.httpPut(defaultHttpClient, casuri, rsDomain, RsTaskMsg.class);
			}
			String remark = server.getRemark();
			if (!StringUtils.isEmpty(remark)) {
				// 编辑描述
				RsDomainForModify rsDomain = new RsDomainForModify();
				rsDomain.setId(Long.parseLong(server.getInstanceId()));
				BasicConfig basicConfig = rsDomainDetail.getBasicConfig();
				basicConfig.setDesc(remark);
				rsDomain.setBasicConfig(basicConfig);
				RestUtil.httpPut(defaultHttpClient, casuri, rsDomain, RsTaskMsg.class);
			}
			if (!StringUtils.isEmpty(server.getName())) {
				String renameUri = "/cas/casrs/vm/rename/" + server.getInstanceId() + "/" + server.getName();
				RestUtil.httpPut(defaultHttpClient, renameUri, null, RsTaskMsg.class);
			}
			return new BocloudResult<RsDomain>(true, "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "编辑失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsDomain> start(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_START);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "启动成功。" : "启动失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("云主机启动失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "云主机启动失败：" + e.getMessage());
		}
	}

	public RsTaskMsg operateVm(VmParams params) {
		String uri = null;
		String operateType = params.getOperateType();
		if (operateType.equals("close")) {
			uri = "/cas/casrs/vm/stop/" + params.getVmId();
		} else {
			uri = "/cas/casrs/vm/" + operateType + "/" + params.getVmId();
		}
		RsTaskMsg rsTaskMsg = RestUtil.httpPut(defaultHttpClient, uri, null, RsTaskMsg.class);
		return waitForComplete(rsTaskMsg.getMsgId());
	}

	@Override
	public BocloudResult<RsDomain> stop(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_POWEROFF);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "关机成功。" : "关机失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("关机失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "关机失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsDomain> reboot(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_RESTART);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "重启成功。" : "重启失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("重启失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "重启失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsDomain> pause(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_PAUSE);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "停止成功。" : "停止失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("停止失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "停止失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsDomain> unPause(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_RESTORE);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "恢复成功。" : "恢复失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("恢复失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "恢复失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsDomain> suspend(String instanceId) {
		try {
			VmParams params = new VmParams();
			params.setVmId(Long.parseLong(instanceId));
			params.setOperateType(VmEnum.CAS_VM_OPERRATE_SLEEP);
			RsTaskMsg operateVm = operateVm(params);
			return new BocloudResult<RsDomain>(operateVm.getResult() == 0, this.detail(instanceId).getData(),
					operateVm.getResult() == 0 ? "休眠成功。" : "休眠失败,失败原因：" + operateVm.getFailMsg());
		} catch (Exception e) {
			logger.error("休眠失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "休眠失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult resume(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult<RsDomain> remove(String instanceId, String name) {
		try {
			String uri = "/cas/casrs/vm/delete/" + instanceId;
			RsTaskMsg rsTaskMsg = RestUtil.httpDelete(defaultHttpClient, uri, RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(rsTaskMsg.getMsgId());
			return new BocloudResult<RsDomain>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "删除成功。" : "删除失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("云主机删除失败：" + e.getMessage());
			return new BocloudResult<RsDomain>(false, "云主机删除失败：" + e.getMessage());
		}
	}

	private void removeSystemDisk(RsDomain rsDomain) {
		try {
			List<RsDomainStorage> storages = rsDomain.getStorages();
			if (ListTool.isEmpty(storages)) {
				return;
			}
			H3cVolumeProvider volumeProvider = new H3cVolumeProvider(butler);
			for (RsDomainStorage rsDomainStorage : storages) {
				logger.info("storages : " + JSON.toJSONString(storages));
				if ("disk".equalsIgnoreCase(rsDomainStorage.getDiskDevice())
						&& "vda".equalsIgnoreCase(rsDomainStorage.getDevice())) {
					String storeFile = rsDomainStorage.getStoreFile();
					String volumeName = storeFile.substring(storeFile.lastIndexOf("/") + 1);
					String path = storeFile.substring(0, storeFile.lastIndexOf("/"));
					RsStoragePool storagePool = getStoragePoolByPath(path, String.valueOf(rsDomain.getHostId()));
					logger.info("storagePool : " + JSON.toJSONString(storagePool));
					if (null != storagePool) {
						H3cVolumeModel volumeModel = new H3cVolumeModel();
						volumeModel.setHostId(rsDomain.getHostId());
						volumeModel.setSpName(storagePool.getName());
						volumeModel.setVolName(volumeName);
						volumeProvider.remove(volumeModel);
						logger.info("系统盘删除成功！");
					}
				}
			}
		} catch (Exception e) {
			logger.error("删除系统盘异常：" + e.getMessage());
		}
	}

	public RsStoragePool getStoragePoolByPath(String path, String hostId) {
		RsStoragePool rsStoragePool = null;
		H3cStoragePoolProvider poolProvider = new H3cStoragePoolProvider(butler);
		BocloudResult<List<RsStoragePool>> list = poolProvider.list(hostId);
		if (list.isFailed()) {
			return rsStoragePool;
		}
		List<RsStoragePool> data = list.getData();
		if (!ListTool.isEmpty(data)) {
			for (RsStoragePool storagePool : data) {
				if (path.equalsIgnoreCase(storagePool.getPath()) && 1 == storagePool.getStatus()) {
					rsStoragePool = storagePool;
					break;
				}
			}
		}
		return rsStoragePool;
	}

	@Override
	public BocloudResult<RsDomain> detail(String instanceId) {
		String casuri = "/cas/casrs/vm/" + instanceId;
		try {
			RsDomain rsDomainDetail = RestUtil.httpGet(defaultHttpClient, casuri, RsDomain.class);
			if (null != rsDomainDetail) {
				return new BocloudResult<RsDomain>(true, rsDomainDetail, "查询成功");
			} else {
				return new BocloudResult<RsDomain>(false, "云主机不存在！");
			}
		} catch (Exception e) {
			logger.error("查询详情错误：", e);
			return new BocloudResult<RsDomain>(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult getJnlpXml(Long vmId, String address) {
		String jnlp = "<?xml version=\"1.0\" encoding=\"utf-8\"?>" + "<jnlp spec=\"1.5+\" codebase=\""
				+ address
				+ "/cas\">"
				+ "<information>"
				+ "<title>VirtDesktop</title>"
				+ "<vendor>VirtCompany</vendor>"
				+ "<homepage href=\"http://www.virtDesktop.com\"/>"
				+ "<description>Virt Desktop</description>"
				+ "<icon kind=\"splash\" href=\"gxt/images/default/s.gif\"/>"
				+ "</information>"
				+ "<security><all-permissions/></security>"
				+ "<update check=\"timeout\" policy=\"always\"/>"
				+ "<resources>"
				+ "<jar  href=\"jar/vnc.jar\"/><java version=\"1.6+\" java-vm-args=\"-Xnosplash\" href=\"http://java.sun.com/products/autodl/j2se\"/>"
				+ "</resources>" + "<application-desc main-class=\"com.virtual.vnc.VncViewer\">"
				+ "<argument>DOMAINID</argument>" + "<argument>"
				+ H3cToolsUtils.encryptStringToText(String.valueOf(vmId)) + "</argument>"
				+ "<argument>USERNAME</argument>" + "<argument>" + this.getUsername() + "</argument>"
				+ "<argument>PASSWORD</argument>" + "<argument>"
				+ H3cToolsUtils.encryptStringToText(this.getPassword()) + "</argument>"
				+ "<argument>PROTOCOL</argument>" + "<argument>http</argument>" + "<argument>HOSTIP</argument>"
				+ "<argument>" + this.getEndpoint() + "</argument>" + "<argument>HOSTPORT</argument>" + "<argument>"
				+ this.getPort() + "</argument>" + "</application-desc>" + "</jnlp>";
		logger.info("jnlp : " + jnlp);
		return new BocloudResult(true, jnlp, "查询成功");
	}

	private List<H3cNetWorkModel> getNetworks(List<org.dom4j.Element> networks) {
		List<H3cNetWorkModel> h3cStorages = new ArrayList<>();
		for (org.dom4j.Element storage : networks) {
			JSONObject jsonObject = XmlUtils.xml2FastJSONObject(storage.asXML());
			H3cNetWorkModel networkModel = JSONObject.toJavaObject(jsonObject.getJSONObject("network"),
					H3cNetWorkModel.class);
			if (networkModel != null) {
				h3cStorages.add(networkModel);
			}
		}
		return h3cStorages;
	}

	private List<H3cStorageModel> getStorages(List<org.dom4j.Element> storages) {
		List<H3cStorageModel> h3cStorages = new ArrayList<>();
		for (org.dom4j.Element storage : storages) {
			JSONObject jsonObject = XmlUtils.xml2FastJSONObject(storage.asXML());
			H3cStorageModel storageModel = JSONObject.toJavaObject(jsonObject.getJSONObject("storage"),
					H3cStorageModel.class);
			if (storageModel != null) {
				h3cStorages.add(storageModel);
			}
		}
		return h3cStorages;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		return null;
	}

	@Override
	public BocloudResult joinSGroup(String instanceId, String groupId) {
		return null;
	}

	@Override
	public BocloudResult leaveSGroup(String instanceId, String groupId) {
		return null;
	}

	@Override
	public BocloudResult changePassword(String instanceId, String password) {
		return null;
	}

	@Override
	public BocloudResult attachNetwork(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult detachNetwork(String instanceId, String portUuid) {
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId) {
		return null;
	}

	@Override
	public BocloudResult confirmResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean memHotEnable) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel server) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String instanceId) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String instanceId, String options) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String instanceId) {
		String url = "/cas/casrs/vm/network/" + instanceId;
		List<H3cNetWorkModel> networks = new ArrayList<>();
		try {
			Result result = doGet(url);
			if (result.isSuccess()) {
				String xml = result.getMessage();
				List<org.dom4j.Element> networkEles = XmlUtils.getNodesByXpath(xml, "list/network");
				if (!ListTool.isEmpty(networkEles)) {
					for (org.dom4j.Element element : networkEles) {
						String hostXml = element.asXML();
						String jsonStr = XmlUtils.xml2JSON(hostXml);
						JSONObject jsonObject = JSONObject.parseObject(jsonStr);
						H3cNetWorkModel h3cNetworkModel = JSONObject.toJavaObject(jsonObject.getJSONObject("network"),
								H3cNetWorkModel.class);
						networks.add(h3cNetworkModel);
					}

				}
				return new BocloudResult(true, networks, "查询成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("查询失败{}", ExceptionUtils.getStackTrace(e));
		}

		return new BocloudResult(false, "查询失败");
	}

	@Override
	public BocloudResult getVmIps(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult softReboot(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
		return null;
	}

	@Override
	public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getMetric(String type) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String instanceId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
		return null;
	}

	@Override
	public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
		// TODO Auto-generated method stub
		return null;
	}

}
