package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsSnapshot;
import com.bocloud.cmp.model.h3c.entity.RsTaskMsg;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

/**
 * 华三云快照操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/16
 */
@Provider("bocloud.cmp.h3c")
public class H3cSnapshotProvider extends H3cProvider implements SnapshotProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cSnapshotProvider.class);

	public H3cSnapshotProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult<List<RsSnapshot>> list(Long cloudServerId) {
		List<RsSnapshot> snapshots = new ArrayList<RsSnapshot>();
		String uri = "/cas/casrs/vm/snapshot/" + cloudServerId;
		@SuppressWarnings("unchecked")
		RsCommList<RsSnapshot> comList = RestUtil.httpGet(defaultHttpClient, uri, RsCommList.class);
		if (comList != null) {
			List<RsSnapshot> allSnapshots = comList.getData();
			addSnapshot(snapshots, allSnapshots);
		}
		return new BocloudResult<List<RsSnapshot>>(true, snapshots, "查询成功！");
	}

	private void addSnapshot(List<RsSnapshot> snapshots, List<RsSnapshot> allSnapshots) {
		if (!ListTool.isEmpty(allSnapshots)) {
			for (RsSnapshot rsSnapshot : allSnapshots) {
				snapshots.add(rsSnapshot);
				RsCommList<RsSnapshot> subSnapshot = rsSnapshot.getSnapshots();
				if (null != subSnapshot && !ListTool.isEmpty(subSnapshot.getData())) {
					addSnapshot(snapshots, subSnapshot.getData());
				}
			}
		}
	}

	@Override
	public BocloudResult list() {
		return null;
	}

	@Override
	public BocloudResult<RsTaskMsg> remove(SnapshotModel model) {
		try {
			String url = "/cas/casrs/vm/snapshot/" + model.getInstanceId() + "/" + model.getName();
			RsTaskMsg task = RestUtil.httpDelete(defaultHttpClient, url, RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(task.getMsgId());
			return new BocloudResult<RsTaskMsg>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "快照删除成功!" : "快照删除失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("云主机快照创建失败:" + e.getMessage());
			return new BocloudResult<RsTaskMsg>(false, "云主机快照创建失败！");
		}
	}

	@Override
	public BocloudResult detail(SnapshotModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult modify(SnapshotModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult<RsTaskMsg> revert(SnapshotModel model) {
		String url = "/cas/casrs/vm/snapshot/resume";
		try {
			RsSnapshot rsSnapshot = new RsSnapshot();
			rsSnapshot.setVmId(Long.parseLong(model.getInstanceId()));
			rsSnapshot.setName(model.getName());
			RsTaskMsg task = RestUtil.httpPut(defaultHttpClient, url, rsSnapshot, RsTaskMsg.class);
			RsTaskMsg waitForComplete = waitForComplete(task.getMsgId());
			return new BocloudResult<RsTaskMsg>(waitForComplete.getResult() == 0,
					waitForComplete.getResult() == 0 ? "快照恢复成功。" : "快照恢复失败,失败原因：" + waitForComplete.getFailMsg());
		} catch (Exception e) {
			logger.error("revert snapshot error:" + e.getMessage());
			return new BocloudResult<RsTaskMsg>(false, "恢复失败 ： " + e.getMessage());
		}
	}

	@Override
	public BocloudResult removeByVolumeId(String volumeId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getSnapshot(String snapshotId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getAvaliableSnap(String snapshotId) {
		return null;
	}

}
