package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortGroupModel;
import com.bocloud.cmp.model.SwitchModel;
import com.bocloud.cmp.model.h3c.H3cPortProfileModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsVSwitch;
import com.bocloud.cmp.model.h3c.utils.ElementUtils;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.utils.XmlUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华三云网络操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cNetworkProvider extends H3cProvider implements NetworkProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cNetworkProvider.class);

	public H3cNetworkProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		return null;
	}

	@Override
	public BocloudResult createDSwitch(SwitchModel switchModel) {
		return new BocloudResult(false, "创建分布式交换机失败");
	}

	@Override
	public BocloudResult createVSwitch(SwitchModel switchModel) {
		String enterUrl = "/cas/casrs/vswitch";
		try {
			Element hostEle = ElementUtils.creatRoot("vSwitch");
			ElementUtils.addChild(hostEle, "name", switchModel.getName());
			ElementUtils.addChild(hostEle, "hostId", switchModel.getHostId());
			ElementUtils.addChild(hostEle, "portNum", switchModel.getPortNum());
			ElementUtils.addChild(hostEle, "mode", 0);
			ElementUtils.addChild(hostEle, "status", switchModel.getStatus());
			ElementUtils.addChild(hostEle, "address", "");
			ElementUtils.addChild(hostEle, "enableLacp", false);
			ElementUtils.addChild(hostEle, "bondMode", "balance-tcp");
			ElementUtils.addChild(hostEle, "description", switchModel.getRemark());
			ElementUtils.addChild(hostEle, "vlanId", switchModel.getVlanId());
			Result result = this.doPost(enterUrl, ElementUtils.toFormatedXML(hostEle));
			if (result.isSuccess()) {
				return new BocloudResult(true, "创建成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}
	}

	@Override
	public BocloudResult removeDSwitch(String name) {
		return new BocloudResult(false, "删除分布式交换机失败");
	}

	@Override
	public BocloudResult renameDSwitch(String oldName, String newName) {
		return new BocloudResult(false, "修改分布式交换机失败");
	}

	@Override
	public BocloudResult updateDVSwitch(SwitchModel model) {
		return new BocloudResult(false, "分布式交换机更新失败：");

	}

	@Override
	public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
		// FIXME 这个方法调用不同
		String enterUrl = "/cas/casrs/vswitch/" + vswitchName;
		try {

			Result result = this.doDelete(enterUrl);
			if (result.isSuccess()) {
				return new BocloudResult(true, "删除成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}

	}

	@Override
	public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel switchModel) {
		String enterUrl = "/cas/casrs/vswitch";
		try {
			Element hostEle = ElementUtils.creatRoot("vSwitch");
			ElementUtils.addChild(hostEle, "id", vswitchName);
			ElementUtils.addChild(hostEle, "name", switchModel.getName());
			ElementUtils.addChild(hostEle, "hostId", switchModel.getHostId());
			ElementUtils.addChild(hostEle, "portNum", switchModel.getPortNum());
			ElementUtils.addChild(hostEle, "mode", 0);
			ElementUtils.addChild(hostEle, "status", switchModel.getStatus());
			ElementUtils.addChild(hostEle, "address", "");
			ElementUtils.addChild(hostEle, "enableLacp", false);
			ElementUtils.addChild(hostEle, "bondMode", "balance-tcp");
			ElementUtils.addChild(hostEle, "description", switchModel.getRemark());
			ElementUtils.addChild(hostEle, "vlanId", switchModel.getVlanId());
			Result result = this.doPut(enterUrl, ElementUtils.toFormatedXML(hostEle));
			if (result.isSuccess()) {
				return new BocloudResult(true, "更新成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}
	}

	@Override
	public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
		return new BocloudResult(false, "创建端口组失败");
	}

	@Override
	public BocloudResult removePortGroup(String hostName, String pgName, String type) {
		return new BocloudResult(false, "删除端口组失败");
	}

	@Override
	public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
		return new BocloudResult(false, "端口组更新失败");
	}

	@Override
	public BocloudResult addHost(SwitchModel model) {
		return new BocloudResult(false, "添加主机失败：");
	}

	@Override
	@SuppressWarnings("unchecked")
	public BocloudResult<List<RsVSwitch>> listVSwitch(SwitchModel model) {
		String uri = "/cas/casrs/host/id/" + model.getHostId() + "/vswitch";
		List<RsVSwitch> vSwitchs = new ArrayList<RsVSwitch>();
		try {
			RsCommList<RsVSwitch> result = RestUtil.httpGet(defaultHttpClient, uri, RsCommList.class);
			if (null != result.getData()) {
				List<RsVSwitch> data = result.getData();
				return new BocloudResult<List<RsVSwitch>>(true, data, "查询成功");
			} else {
				return new BocloudResult<List<RsVSwitch>>(false, vSwitchs, "暂无交换机信息");
			}
		} catch (Exception e) {
			logger.error("查询交换机错误", e);
			return new BocloudResult<List<RsVSwitch>>(false, vSwitchs, e.getMessage());
		} finally {
			logout();
		}
	}

	// @Override
	// public BocloudResult listVSwitch(SwitchModel model) {
	// String enterUrl = "/cas/casrs/host/id/" + model.getHostId() + "/vswitch";
	// try {
	// Result result = this.doGet(enterUrl);
	// if (result.isSuccess()) {
	// List<SwitchModel> switchModels = new ArrayList<>();
	// String xml = result.getMessage();
	// List<org.dom4j.Element> elements = XmlUtils.getNodesByXpath(xml,
	// "list/vSwitch");
	// if (elements != null && elements.size() > 0) {
	//
	// for (org.dom4j.Element element : elements) {
	// SwitchModel switchModel = new SwitchModel();
	// switchModel.setHostId(XmlUtils.readLong(element, "hostId"));
	// Integer vlanId = XmlUtils.readInteger(element, "vlanId");
	// if (vlanId != null) {
	// switchModel.setVlanId(vlanId);
	// }
	// switchModel.setRemark(XmlUtils.readString(element, "description"));
	// switchModel.setPortNum(XmlUtils.readLong(element, "portNum"));
	// switchModel.setSwitcherVal(XmlUtils.readString(element, "id"));
	// switchModel.setName(XmlUtils.readString(element, "name"));
	// NetworkModel network = new NetworkModel();
	// List<SubnetModel> subnets = new ArrayList<>();
	// SubnetModel subnet = new SubnetModel();
	// subnet.setAddress(XmlUtils.readString(element, "address"));
	// subnet.setGateway(XmlUtils.readString(element, "gateway"));
	// int netmask = H3cToolsUtils.getCidr(XmlUtils.readString(element,
	// "netmask"));
	// String gateway = subnet.getGateway();
	// if (StringUtils.isEmpty(gateway)) {
	// gateway = subnet.getAddress();
	// }
	// subnet.setCidr(gateway + "/" + netmask);
	// subnet.setIpPools(IPAddress.getIPAddress(gateway).getPools(gateway,
	// subnet.getCidr()));
	// subnets.add(subnet);
	// network.setSubnets(subnets);
	// switchModel.setNetwork(network);
	// switchModels.add(switchModel);
	// }
	// }
	//
	// return new BocloudResult(true, switchModels, "查询成功");
	// } else {
	// return new BocloudResult(false, result.getMessage());
	// }
	// } catch (Exception e) {
	// logger.error("错误", e);
	// return new BocloudResult(false, e.getMessage());
	// } finally {
	// closeClient();
	// }
	// }

	@Override
	public BocloudResult create(NetworkModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(String networkId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult detail(String networkId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult modify(NetworkModel model) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult listProfile() {
		String enterUrl = "/cas/casrs/profile/";
		try {
			Result result = this.doGet(enterUrl);
			if (result.isSuccess()) {
				List<H3cPortProfileModel> portProfileModels = new ArrayList<>();
				String xml = result.getMessage();
				List<org.dom4j.Element> elements = XmlUtils.getNodesByXpath(xml, "list/portProfile");
				if (elements != null && elements.size() > 0) {
					for (org.dom4j.Element element : elements) {
						element.asXML();
						JSONObject jsonObject = XmlUtils.xml2FastJSONObject(element.asXML());
						H3cPortProfileModel portProfile = JSONObject.parseObject(jsonObject.getString("portProfile"),
								H3cPortProfileModel.class);
						portProfileModels.add(portProfile);
					}
				}
				return new BocloudResult(true, portProfileModels, "查询成功");
			} else {
				return new BocloudResult(false, result.getMessage());
			}
		} catch (Exception e) {
			logger.error("错误", e);
			return new BocloudResult(false, e.getMessage());
		} finally {
			closeClient();
		}
	}

}
