package com.bocloud.cmp.provider.storage;

import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.h3c.H3cStoragePoolModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsStoragePool;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华三云存储池操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cStoragePoolProvider extends H3cProvider implements StoragePoolProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cStoragePoolProvider.class);

	public H3cStoragePoolProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult create(H3cStoragePoolModel storagePoolModel) {
		String url = "/cas/casrs/storage/add";
		String xml = null;
		try {
			xml = storagePoolModel.toCreateString();
		} catch (Exception e) {
			logger.error("{}", e);
			return new BocloudResult(false, "生成xml出错");
		}

		try {
			Result result = doPost(url, xml);
			return new BocloudResult(result.isSuccess(), result.getMessage());
		} catch (Exception e) {
			logger.error("{}", e);
			return new BocloudResult(false, "远程调用失败");
		}
	}

	@Override
	public BocloudResult start(H3cStoragePoolModel storagePoolModel) {
		String url = "/cas/casrs/storage/start?id=" + storagePoolModel.getHostId() + "&poolName="
				+ storagePoolModel.getName() + "&hostName=" + storagePoolModel.getHostName();
		try {
			Result result = doGet(url);
			return new BocloudResult(result.isSuccess(), result.getMessage());
		} catch (Exception e) {
			logger.error("{}", e);
			return new BocloudResult(false, "远程调用失败");
		}
	}

	@Override
	public BocloudResult remove(H3cStoragePoolModel storagePoolModel) {
		String url = "/cas/casrs/storage/delete?id=" + storagePoolModel.getHostId() + "&poolName="
				+ storagePoolModel.getName() + "&hostName=" + storagePoolModel.getHostName();
		try {
			Result result = doDelete(url);
			return new BocloudResult(result.isSuccess(), result.getMessage());
		} catch (Exception e) {
			logger.error("{}", e);
			return new BocloudResult(false, "远程调用失败");
		}
	}

	@Override
	public BocloudResult<List<RsStoragePool>> list(String hostId) {
		String url = "/cas/casrs/storage/pool?hostId=" + hostId;
		try {
			@SuppressWarnings("unchecked")
			RsCommList<RsStoragePool> rsStoragePoolList = RestUtil.httpGet(defaultHttpClient, url, RsCommList.class);
			if (rsStoragePoolList != null) {
				return new BocloudResult<List<RsStoragePool>>(true, rsStoragePoolList.getData(), "查询成功！");
			} else {
				return new BocloudResult<List<RsStoragePool>>(false, "查询存储池失败！");
			}
		} catch (Exception e) {
			logger.error("查询存储池异常：" + e.getMessage());
			return new BocloudResult<List<RsStoragePool>>(false, "查询存储池异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult<RsStoragePool> detail(String hostId, String poolName) {
		String uri = "/cas/casrs/storage/info?hostId=" + hostId + "&poolName=" + poolName;
		try {
			RsStoragePool rsStoragePool = RestUtil.httpGet(defaultHttpClient, uri, RsStoragePool.class);
			if (null != rsStoragePool) {
				return new BocloudResult<RsStoragePool>(true, rsStoragePool, "查询成功！");
			} else {
				return new BocloudResult<RsStoragePool>(false, "当前存储池不存在！");
			}
		} catch (Exception e) {
			logger.error("查询存储池异常:" + e.getMessage());
			return new BocloudResult<RsStoragePool>(false, "查询存储池异常:" + e.getMessage());
		}
	}

}
