package com.bocloud.cmp.provider.storage;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.model.h3c.entity.RsCommList;
import com.bocloud.cmp.model.h3c.entity.RsStorageVolume;
import com.bocloud.cmp.model.h3c.utils.RestUtil;
import com.bocloud.cmp.provider.H3cProvider;
import com.bocloud.cmp.provider.builder.H3cVolumeModelBuilder;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华三云卷操作类
 * 
 * @author ymh
 *
 */
@Provider("bocloud.cmp.h3c")
public class H3cVolumeProvider extends H3cProvider implements VolumeProvider {

	private static Logger logger = LoggerFactory.getLogger(H3cVolumeProvider.class);

	public H3cVolumeProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult<List<RsStorageVolume>> list(String hostId, String storagePoolName) {
		String uri = "/cas/casrs/storage/volume?hostId=" + hostId + "&poolName=" + storagePoolName;
		@SuppressWarnings("unchecked")
		RsCommList<RsStorageVolume> comList = RestUtil.httpGet(defaultHttpClient, uri, RsCommList.class);
		if (comList != null) {
			return new BocloudResult<List<RsStorageVolume>>(true, comList.getData(), "查询成功！");
		} else {
			return new BocloudResult<List<RsStorageVolume>>(false, "暂无数据！");
		}
	}

	@Override
	public BocloudResult list() {
		return null;
	}

	@Override
	public BocloudResult create(VolumeModel volume) {
		String url = "/cas/casrs/storage/volume/add";
		try {
			H3cVolumeModelBuilder h3cBasicBuilder = new H3cVolumeModelBuilder();
			h3cBasicBuilder.initByModel(volume);
			String xml = h3cBasicBuilder.buildCreateXml();
			Result result = doPost(url, xml);
			if (result.isSuccess()) {
				String resultXml = result.getMessage();
				return new BocloudResult(true, resultXml, "创建成功");
			}
			return new BocloudResult(false, result.getMessage());
		} catch (Exception e) {
			logger.error("{}", e);
			return new BocloudResult(false, "生成xml出错");
		}
	}

	@Override
	public BocloudResult modify(VolumeModel volumeModel) {
		return null;
	}

	@Override
	public BocloudResult remove(String s) {
		return null;
	}

	@Override
	public BocloudResult detail(String s) {
		return null;
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		return null;
	}

	@Override
	public BocloudResult attach(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult attach(String s, String s1, String s2) {
		return null;
	}

	@Override
	public BocloudResult detach(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult resetStatus(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult uploadToImage(String s, ImageModel imageModel) {
		return null;
	}

	@Override
	public BocloudResult extend(String s, Integer integer) {
		return null;
	}

	@Override
	public BocloudResult<RsStorageVolume> remove(H3cVolumeModel volume) {
		try {
			String uri = "/cas/casrs/storage/volume/delete/" + volume.getHostId() + "/" + volume.getSpName() + "/"
					+ volume.getVolName();
			RestUtil.httpDelete(defaultHttpClient, uri);
			return new BocloudResult<RsStorageVolume>(true, "存储卷删除成功！");
		} catch (Exception e) {
			logger.error("存储卷删除失败:" + e.getMessage());
			return new BocloudResult<RsStorageVolume>(true, "存储卷删除失败:" + e.getMessage());
		}
	}

}
