package com.bocloud.cmp.provider.utils;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.json.JSONObject;
import org.json.XML;

import java.util.ArrayList;
import java.util.List;

public class XmlUtils {
    /**
     * 得到要结点下对应nodeName的所有结点信息
     *
     * @param xml
     * @param nodeName
     * @return
     */
    @SuppressWarnings("unchecked")
    public static List<Element> getNodes(String xml, String nodeName) {
        Document document;
        List<Element> list = new ArrayList<>();
        try {
            document = DocumentHelper.parseText(xml);
            Element root = document.getRootElement();
            list = root.elements(nodeName);
        } catch (DocumentException e) {
            return null;
        }
        return list;
    }

    /**
     * 获取根结点的属性
     *
     * @param xml
     * @param attrName
     * @return
     */
    public static String getRootAttribute(String xml, String attrName) {
        Document document;
        try {
            document = DocumentHelper.parseText(xml);
            Element root = document.getRootElement();
            return root.attribute(attrName).getText();
        } catch (DocumentException e) {
            return null;
        }
    }

    /**
     * 查找节点 例如：xpath= "//Server/Service/Connector[@protocol='HTTP/1.1']"
     *
     * @param xml
     * @param xpath
     * @return
     */
    public static Element getSingleNode(String xml, String xpath) {
        Document document;
        Element e = null;
        try {
            document = DocumentHelper.parseText(xml);
            e = (Element) document.selectSingleNode(xpath);
        } catch (DocumentException e1) {
            return null;
        }
        return e;
    }

    @SuppressWarnings("unchecked")
    public static List<Element> getNodesByXpath(String xml, String xpath) {
        Document document;
        List<Element> e = null;
        try {
            document = DocumentHelper.parseText(xml);
            e = document.selectNodes(xpath);
        } catch (DocumentException e1) {
            return null;
        }
        return e;
    }

    /**
     * 获取某节点的内容
     *
     * @param xml
     * @param nodeName
     * @return
     */
    public static String getXmlText(String xml, String nodeName) {
        String returnValue = "";
        try {
            Document doc = DocumentHelper.parseText(xml);
            Element root = doc.getRootElement();
            @SuppressWarnings("unchecked")
            List<Element> elements = root.elements();
            for (Element element : elements) {
                if (nodeName.equals(element.getName())) {
                    returnValue = element.getText();
                }
            }
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        return returnValue;
    }

    public static Long readLong(Element element, String nodeName) {
        try {
            return Long.valueOf(readString(element, nodeName));
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public static String readString(Element element, String nodeName) {
        Element child = element.element(nodeName);
        if (child == null) {
            return null;
        }
        return child.getStringValue();
    }

    public static Integer readInteger(Element element, String nodeName) {
        try {
            return Integer.valueOf(readString(element, nodeName));
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public static String xml2JSON(String xml) {
        JSONObject json = XML.toJSONObject(xml);
        return json.toString();
    }

    public static JSONObject xml2JSONObject(String xml) {
        JSONObject json = XML.toJSONObject(xml);
        return json;
    }

    public static String json2XML(String json) {
        return XML.toString(json);
    }

    public static Boolean readBoolean(Element element, String nodeName) {
        try {
            return Boolean.valueOf(readString(element, nodeName));
        } catch (NumberFormatException e) {
            return null;
        }
    }

    public static com.alibaba.fastjson.JSONObject xml2FastJSONObject(String objXml) {
        JSONObject jsonObject = xml2JSONObject(objXml);
        return com.alibaba.fastjson.JSONObject.parseObject(jsonObject.toString());
    }
}

