package com.bocloud.cmp.provider.common.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.hmc.SyncModel;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedIPLConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemCapabilities;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemIOConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemMemoryConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemProcessorConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.HostEthernetAdapterResponse;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileEntry;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemEntry;
import com.bocloud.cmp.model.hmc.sdk.module.SharedProcessorPoolResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualNetworkBridgeResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSwitchResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7HostEthernetAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionProfileService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7ManagedSystemService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7SharedProcessorPoolService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualNetworkBridgeService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualSwitchService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VolumeGroupService;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedIPLConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedSystemCapabilities;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedSystemIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedSystemMemoryConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedSystemProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartition;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystem;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystemContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystemEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystemResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7SharedProcessorPoolResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualNetworkBridgeResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSwitchResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupResponse;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * HMC同步数据组装类
 *
 * @author lkx007
 */
public class HmcLocationProvider extends HmcAccessableProvider implements LocationProvider {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    public HmcLocationProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        if (!accessable()) {
            return new BocloudResult(false, "HMC无法获取有效的sessionId");
        }

        HmcAuthInfo auth = this.getAuth();

        // 服务器列表
        logger.info("正在获取服务器数据...");
        List<V7ManagedSystemEntry> systemEntries = new ArrayList<>();
        BocloudResult result = V7ManagedSystemService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，服务器列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V7ManagedSystemResponse) data)
                .map(V7ManagedSystemResponse::getEntries)
                .ifPresent(systemEntries::addAll);

        // 获取vios逻辑分区列表
        logger.info("正在获取VIOS数据...");
        List<V7VirtualIOServerEntry> viosEntries = new ArrayList<>();
        result = V7VirtualIOServerService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，vios列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V7VirtualIOServerResponse) data)
                .map(V7VirtualIOServerResponse::getEntries)
                .ifPresent(viosEntries::addAll);

        // 获取(AIX/Linux)逻辑分区列表
        logger.info("正在获取逻辑分区数据...");
        List<V7LogicalPartitionEntry> lparEntries = new ArrayList<>();
        result = V7LogicalPartitionService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，lpar列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V7LogicalPartitionResponse) data)
                .map(V7LogicalPartitionResponse::getEntries)
                .ifPresent(lparEntries::addAll);

        List<SyncModel> syncModels = systemEntries.stream()
                .map(systemEntry -> this.getManagedSystemSyncInfo(systemEntry, viosEntries, lparEntries))
                .collect(Collectors.toList());
        return new BocloudResult(true, syncModels, "同步成功");

    }

    @Override
    public BocloudResult getManagedSystemSyncInfo(String uuid) {
       HmcAuthInfo auth = this.getAuth();

        // 获取服务器信息
        BocloudResult result = V7ManagedSystemService.getByUuid(auth, uuid);
        if (result.isFailed()) {
            String message = result.getMessage();
            return new BocloudResult(false, "获取服务器信息失败(" + message + ")");
        }
        if (null == result.getData()) {
            return new BocloudResult(false, "服务器信息为空");
        }

        // 获取vios逻辑分区列表
        List<V7VirtualIOServerEntry> viosEntries = new ArrayList<>();
        result = V7VirtualIOServerService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，vios列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V7VirtualIOServerResponse) data)
                .map(V7VirtualIOServerResponse::getEntries)
                .ifPresent(viosEntries::addAll);

        // 获取(AIX/Linux)逻辑分区列表
        List<V7LogicalPartitionEntry> lparEntries = new ArrayList<>();
        result = V7LogicalPartitionService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，lpar列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V7LogicalPartitionResponse) data)
                .map(V7LogicalPartitionResponse::getEntries)
                .ifPresent(lparEntries::addAll);

        V7ManagedSystemEntry entry = (V7ManagedSystemEntry) result.getData();
        SyncModel syncModel = getManagedSystemSyncInfo(entry, viosEntries, lparEntries);

        return new BocloudResult(true, syncModel, "服务器数据获取成功");
    }

    private SyncModel getManagedSystemSyncInfo(
            V7ManagedSystemEntry entry, List<V7VirtualIOServerEntry> viosEntries, List<V7LogicalPartitionEntry> lparEntries
    ) {
        String systemName = Optional.ofNullable(entry)
                .map(V7ManagedSystemEntry::getContent)
                .map(V7ManagedSystemContent::getManagedSystem)
                .map(V7ManagedSystem::getSystemName)
                .orElse("未知服务器名称");

        V7ManagedSystemContent content;
        V7ManagedSystem system;

        HmcAuthInfo auth = this.getAuth();
        SyncModel syncModel = new SyncModel();
        syncModel.setUuid(entry.getId());
        syncModel.setSystemEntry(JSON.parseObject(JSON.toJSONString(entry), ManagedSystemEntry.class));

        // 获取虚拟交换机信息
        logger.info("正在获取服务器[{}]的虚拟交换机数据...", systemName);
        BocloudResult vswitchResult = V7VirtualSwitchService.get(auth, entry.getId());
        if (vswitchResult.isSuccess() && null != vswitchResult.getData()) {
            V7VirtualSwitchResponse responseV7 = (V7VirtualSwitchResponse) vswitchResult.getData();
            VirtualSwitchResponse response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualSwitchResponse.class);
            syncModel.setVswitchEntries(response.getEntries());
        }

        // 获取虚拟网桥信息
        logger.info("正在获取服务器[{}]的虚拟网桥数据...", systemName);
        BocloudResult vbridgeResult = V7VirtualNetworkBridgeService.get(auth, entry.getId());
        if (vbridgeResult.isSuccess() && null != vbridgeResult.getData()) {
            V7VirtualNetworkBridgeResponse responseV7 = (V7VirtualNetworkBridgeResponse) vbridgeResult.getData();
            VirtualNetworkBridgeResponse response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualNetworkBridgeResponse.class);
            syncModel.setNetworkBridgeEntries(response.getEntries());
        }

        // 获取主机以太网物理信息
        logger.info("正在获取服务器[{}]的HEA数据...", systemName);
        BocloudResult heaResult = V7HostEthernetAdapterService.get(auth, entry.getId());
        if (heaResult.isSuccess() && null != heaResult.getData()) {
            V7HostEthernetAdapterResponse responseV7 = (V7HostEthernetAdapterResponse) heaResult.getData();
            HostEthernetAdapterResponse response = JSON.parseObject(JSON.toJSONString(responseV7), HostEthernetAdapterResponse.class);
            syncModel.setHostEthernetAdapterEntries(response.getEntries());
        }

        //获取主机共享存储池的信息
        logger.info("正在获取服务器[{}]的共享存储池数据...", systemName);
        BocloudResult poolResult = V7SharedProcessorPoolService.get(auth, entry.getId());
        if (poolResult.isSuccess() && null != poolResult.getData()) {
            V7SharedProcessorPoolResponse responseV7 = (V7SharedProcessorPoolResponse) poolResult.getData();
            SharedProcessorPoolResponse response = JSON.parseObject(JSON.toJSONString(responseV7), SharedProcessorPoolResponse.class);
            syncModel.setSharedProcessorPoolEntries(response.getEntries());
        }

        content = entry.getContent();
        if (content != null) {
            system = content.getManagedSystem();
            if (system != null) {
                V7AssociatedIPLConfiguration iplConfiguration = system.getAssociatedIPLConfiguration();
                V7AssociatedSystemCapabilities systemCapabilities = system.getAssociatedSystemCapabilities();
                V7AssociatedSystemMemoryConfiguration systemMemoryConfiguration = system.getAssociatedSystemMemoryConfiguration();
                V7AssociatedSystemProcessorConfiguration systemProcessorConfiguration = system.getAssociatedSystemProcessorConfiguration();
                V7AssociatedSystemIOConfiguration systemIOConfiguration = system.getAssociatedSystemIOConfiguration();

                syncModel.setIplConfiguration(JSON.parseObject(JSON.toJSONString(iplConfiguration), AssociatedIPLConfiguration.class));
                syncModel.setSystemCapabilities(JSON.parseObject(JSON.toJSONString(systemCapabilities), AssociatedSystemCapabilities.class));
                syncModel.setSystemMemoryConfiguration(JSON.parseObject(JSON.toJSONString(systemMemoryConfiguration), AssociatedSystemMemoryConfiguration.class));
                syncModel.setSystemProcessorConfiguration(JSON.parseObject(JSON.toJSONString(systemProcessorConfiguration), AssociatedSystemProcessorConfiguration.class));
                syncModel.setSystemIOConfiguration(JSON.parseObject(JSON.toJSONString(systemIOConfiguration), AssociatedSystemIOConfiguration.class));
            }
        }

        // 添加vios分区
        Map<String, List<V7LogicalPartitionProfileEntry>> profileEntrys = new HashMap<>();
        Map<String, List<V7VolumeGroupEntry>> volumeGroupEntries = new HashMap<>();
        if (null != viosEntries) {
            List<V7VirtualIOServerEntry> serverViosEntrys = new ArrayList<>();
            for (V7VirtualIOServerEntry viosEntry : viosEntries) {
                String viosName = Optional.ofNullable(viosEntry)
                        .map(V7VirtualIOServerEntry::getContent)
                        .map(V7VirtualIOServerContent::getVirtualIOServer)
                        .map(V7VirtualIOServer::getPartitionName)
                        .orElse("未知的VIOS名称");
                String[] values = viosEntry.getContent().getVirtualIOServer().getAssociatedManagedSystem().getHref().split("/");
                if (entry.getId().equals(values[values.length - 1])) {
                    serverViosEntrys.add(viosEntry);
                    // get the profile of the vios
                    String uuid = viosEntry.getContent().getVirtualIOServer().getPartitionUUID();
                    logger.info("正在获取服务器[{}]的VIOS[{}]的概要文件数据", systemName, viosName);
                    BocloudResult result = V7LogicalPartitionProfileService.getByVirtualIOServerUuid(auth, uuid);
                    if (result.isSuccess()) {
                        V7LogicalPartitionProfileResponse response = (V7LogicalPartitionProfileResponse) result.getData();
                        profileEntrys.put(uuid, response.getEntries());
                    }
                    logger.info("正在获取服务器[{}]的VolumeGroup[{}]的概要文件数据", systemName, viosName);
                    result = V7VolumeGroupService.get(auth, uuid);
                    if (result.isSuccess()) {
                        if (null != result.getData()) {
                            V7VolumeGroupResponse response = (V7VolumeGroupResponse) result.getData();
                            volumeGroupEntries.put(uuid, response.getEntries());
                        }
                    }
                }
            }
            if (!serverViosEntrys.isEmpty()) {
                syncModel.setViosEntries(JSON.parseArray(JSON.toJSONString(serverViosEntrys), VirtualIOServerEntry.class));
            }
        }

        // 添加lpar分区
        if (null != lparEntries) {
            List<V7LogicalPartitionEntry> serveLparEntrys = new ArrayList<>();
            for (V7LogicalPartitionEntry lparEntry : lparEntries) {
                String lparName = Optional.ofNullable(lparEntry)
                        .map(V7LogicalPartitionEntry::getContent)
                        .map(V7LogicalPartitionContent::getLogicalPartition)
                        .map(V7LogicalPartition::getPartitionName)
                        .orElse("未知的逻辑分区名称");
                String[] values = lparEntry.getContent().getLogicalPartition().getAssociatedManagedSystem().getHref().split("/");
                if (entry.getId().equals(values[values.length - 1])) {
                    serveLparEntrys.add(lparEntry);
                    // get the profile of the lpar
                    String uuid = lparEntry.getContent().getLogicalPartition().getPartitionUUID();
                    logger.info("正在获取服务器[{}]的逻辑分区[{}]的概要文件数据", systemName, lparName);
                    BocloudResult result = V7LogicalPartitionProfileService.getByLogicalPartitionUuid(auth, uuid);
                    if (result.isSuccess()) {
                        V7LogicalPartitionProfileResponse response = (V7LogicalPartitionProfileResponse) result.getData();
                        profileEntrys.put(uuid, response.getEntries());
                    }
                }
            }
            if (!serveLparEntrys.isEmpty()) {
                syncModel.setLparEntries(JSON.parseArray(JSON.toJSONString(serveLparEntrys), LogicalPartitionEntry.class));
            }
        }
        if (!profileEntrys.isEmpty()) {
            Map<String, List<LogicalPartitionProfileEntry>> profileEntryResponses = new HashMap<>();
            for (Map.Entry<String, List<V7LogicalPartitionProfileEntry>> profileEntry : profileEntrys.entrySet()) {
                List<LogicalPartitionProfileEntry> profileEntryResponseValue = JSON.parseArray(JSON.toJSONString(profileEntry.getValue()), LogicalPartitionProfileEntry.class);
                profileEntryResponses.put(profileEntry.getKey(), profileEntryResponseValue);
            }
            syncModel.setProfileEntries(profileEntryResponses);
        }
        if (!volumeGroupEntries.isEmpty()) {
            Map<String, List<VolumeGroupEntry>> groupEntryResponses = new HashMap<>();
            for (Map.Entry<String, List<V7VolumeGroupEntry>> groupEntry : volumeGroupEntries.entrySet()) {
                List<VolumeGroupEntry> groupEntryResponseValue = JSON.parseArray(JSON.toJSONString(groupEntry.getValue()), VolumeGroupEntry.class);
                groupEntryResponses.put(groupEntry.getKey(), groupEntryResponseValue);
            }
            syncModel.setVolumeGroupEntries(groupEntryResponses);
        }

        // 2019-08-05 : 获取逻辑分区和vios的RMC链接状态
        Map<String, String> rmcMap = new HashMap<>();
        if (syncModel.getViosEntries() != null && syncModel.getViosEntries().size() > 0) {
            for (VirtualIOServerEntry viosEntry : syncModel.getViosEntries()) {
                String uuid = viosEntry.getContent().getVirtualIOServer().getPartitionUUID();
                BocloudResult bocloudResult = V7VirtualIOServerService.getQuickProperty(auth, uuid, VirtualIOServerQuickProperty.RMCState);
                if (bocloudResult.isSuccess()) {
                    String rmcStat = (String) bocloudResult.getData();
                    rmcStat = Optional.ofNullable(rmcStat).map(String::toLowerCase).orElse(null);
                    rmcMap.put(uuid, rmcStat);
                }
            }
        }
        if (syncModel.getLparEntries() != null && syncModel.getLparEntries().size() > 0) {
            for (LogicalPartitionEntry lparEntry : syncModel.getLparEntries()) {
                String uuid = lparEntry.getContent().getLogicalPartition().getPartitionUUID();
                BocloudResult bocloudResult = V7LogicalPartitionService.getQuickProperty(auth, uuid, LogicalPartitionQuickProperty.RMCState);
                if (bocloudResult.isSuccess()) {
                    String rmcStat = (String) bocloudResult.getData();
                    rmcStat = Optional.ofNullable(rmcStat).map(String::toLowerCase).orElse(null);
                    rmcMap.put(uuid, rmcStat);
                }
            }
        }
        syncModel.setRmcStatus(rmcMap);
        return syncModel;
    }

    @Override
    public BocloudResult listRegions() {
        return null;
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

}
