package com.bocloud.cmp.provider.common.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.hmc.SyncModel;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedIPLConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemCapabilities;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemIOConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemMemoryConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.AssociatedSystemProcessorConfiguration;
import com.bocloud.cmp.model.hmc.sdk.module.HostEthernetAdapterResponse;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileEntry;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemEntry;
import com.bocloud.cmp.model.hmc.sdk.module.SharedProcessorPoolResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualNetworkBridgeResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSwitchResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8HostEthernetAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionProfileService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8ManagedSystemService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8SharedProcessorPoolService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualNetworkBridgeService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualSwitchService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VolumeGroupService;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedIPLConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedSystemCapabilities;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedSystemIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedSystemMemoryConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedSystemProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartition;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ManagedSystem;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ManagedSystemContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ManagedSystemEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ManagedSystemResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8SharedProcessorPoolResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualNetworkBridgeResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSwitchResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupResponse;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * HMC同步数据组装类
 *
 * @author lkx007
 */
public class HmcLocationProvider extends HmcAccessableProvider implements LocationProvider {

    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    public HmcLocationProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        if (!accessable()) {
            return new BocloudResult(false, "HMC无法获取有效的sessionId");
        }

        HmcAuthInfo auth = this.getAuth();

        // 服务器列表
        logger.info("正在获取服务器数据...");
        List<V8ManagedSystemEntry> systemEntries = new ArrayList<>();
        BocloudResult result = V8ManagedSystemService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，服务器列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V8ManagedSystemResponse) data)
                .map(V8ManagedSystemResponse::getEntries)
                .ifPresent(systemEntries::addAll);

        // 获取vios逻辑分区列表
        logger.info("正在获取VIOS数据...");
        List<V8VirtualIOServerEntry> viosEntries = new ArrayList<>();
        result = V8VirtualIOServerService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，vios列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V8VirtualIOServerResponse) data)
                .map(V8VirtualIOServerResponse::getEntries)
                .ifPresent(viosEntries::addAll);

        // 获取(AIX/Linux)逻辑分区列表
        logger.info("正在获取逻辑分区数据...");
        List<V8LogicalPartitionEntry> lparEntries = new ArrayList<>();
        result = V8LogicalPartitionService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，lpar列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V8LogicalPartitionResponse) data)
                .map(V8LogicalPartitionResponse::getEntries)
                .ifPresent(lparEntries::addAll);

        List<SyncModel> syncModels = systemEntries.stream()
                .map(systemEntry -> this.getManagedSystemSyncInfo(systemEntry, viosEntries, lparEntries))
                .collect(Collectors.toList());
        return new BocloudResult(true, syncModels, "同步成功");

    }

    @Override
    public BocloudResult getManagedSystemSyncInfo(String uuid) {
       HmcAuthInfo auth = this.getAuth();

        // 获取服务器信息
        BocloudResult result = V8ManagedSystemService.getByUuid(auth, uuid);
        if (result.isFailed()) {
            String message = result.getMessage();
            return new BocloudResult(false, "获取服务器信息失败(" + message + ")");
        }
        if (null == result.getData()) {
            return new BocloudResult(false, "服务器信息为空");
        }

        // 获取vios逻辑分区列表
        List<V8VirtualIOServerEntry> viosEntries = new ArrayList<>();
        result = V8VirtualIOServerService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，vios列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V8VirtualIOServerResponse) data)
                .map(V8VirtualIOServerResponse::getEntries)
                .ifPresent(viosEntries::addAll);

        // 获取(AIX/Linux)逻辑分区列表
        List<V8LogicalPartitionEntry> lparEntries = new ArrayList<>();
        result = V8LogicalPartitionService.get(auth);
        if (result.isFailed()) {
            return new BocloudResult(false, "同步HMC时，lpar列表同步失败，终止同步操作(" + result.getMessage() + ")");
        }
        Optional.ofNullable(result.getData())
                .map(data -> (V8LogicalPartitionResponse) data)
                .map(V8LogicalPartitionResponse::getEntries)
                .ifPresent(lparEntries::addAll);

        V8ManagedSystemEntry entry = (V8ManagedSystemEntry) result.getData();
        SyncModel syncModel = getManagedSystemSyncInfo(entry, viosEntries, lparEntries);

        return new BocloudResult(true, syncModel, "服务器数据获取成功");
    }

    private SyncModel getManagedSystemSyncInfo(
            V8ManagedSystemEntry entry, List<V8VirtualIOServerEntry> viosEntries, List<V8LogicalPartitionEntry> lparEntries
    ) {
        String systemName = Optional.ofNullable(entry)
                .map(V8ManagedSystemEntry::getContent)
                .map(V8ManagedSystemContent::getManagedSystem)
                .map(V8ManagedSystem::getSystemName)
                .orElse("未知服务器名称");
        
        V8ManagedSystemContent content;
        V8ManagedSystem system;

        HmcAuthInfo auth = this.getAuth();
        SyncModel syncModel = new SyncModel();
        syncModel.setUuid(entry.getId());
        syncModel.setSystemEntry(JSON.parseObject(JSON.toJSONString(entry), ManagedSystemEntry.class));

        // 获取虚拟交换机信息
        logger.info("正在获取服务器[{}]的虚拟交换机数据...", systemName);
        BocloudResult vswitchResult = V8VirtualSwitchService.get(auth, entry.getId());
        if (vswitchResult.isSuccess() && null != vswitchResult.getData()) {
            V8VirtualSwitchResponse responseV8 = (V8VirtualSwitchResponse) vswitchResult.getData();
            VirtualSwitchResponse response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualSwitchResponse.class);
            syncModel.setVswitchEntries(response.getEntries());
        }

        // 获取虚拟网桥信息
        logger.info("正在获取服务器[{}]的虚拟网桥数据...", systemName);
        BocloudResult vbridgeResult = V8VirtualNetworkBridgeService.get(auth, entry.getId());
        if (vbridgeResult.isSuccess() && null != vbridgeResult.getData()) {
            V8VirtualNetworkBridgeResponse responseV8 = (V8VirtualNetworkBridgeResponse) vbridgeResult.getData();
            VirtualNetworkBridgeResponse response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualNetworkBridgeResponse.class);
            syncModel.setNetworkBridgeEntries(response.getEntries());
        }

        // 获取主机以太网物理信息
        logger.info("正在获取服务器[{}]的HEA数据...", systemName);
        BocloudResult heaResult = V8HostEthernetAdapterService.get(auth, entry.getId());
        if (heaResult.isSuccess() && null != heaResult.getData()) {
            V8HostEthernetAdapterResponse responseV8 = (V8HostEthernetAdapterResponse) heaResult.getData();
            HostEthernetAdapterResponse response = JSON.parseObject(JSON.toJSONString(responseV8), HostEthernetAdapterResponse.class);
            syncModel.setHostEthernetAdapterEntries(response.getEntries());
        }

        //获取主机共享存储池的信息
        logger.info("正在获取服务器[{}]的共享存储池数据...", systemName);
        BocloudResult poolResult = V8SharedProcessorPoolService.get(auth, entry.getId());
        if (poolResult.isSuccess() && null != poolResult.getData()) {
            V8SharedProcessorPoolResponse responseV8 = (V8SharedProcessorPoolResponse) poolResult.getData();
            SharedProcessorPoolResponse response = JSON.parseObject(JSON.toJSONString(responseV8), SharedProcessorPoolResponse.class);
            syncModel.setSharedProcessorPoolEntries(response.getEntries());
        }

        content = entry.getContent();
        if (content != null) {
            system = content.getManagedSystem();
            if (system != null) {
                V8AssociatedIPLConfiguration iplConfiguration = system.getAssociatedIPLConfiguration();
                V8AssociatedSystemCapabilities systemCapabilities = system.getAssociatedSystemCapabilities();
                V8AssociatedSystemMemoryConfiguration systemMemoryConfiguration = system.getAssociatedSystemMemoryConfiguration();
                V8AssociatedSystemProcessorConfiguration systemProcessorConfiguration = system.getAssociatedSystemProcessorConfiguration();
                V8AssociatedSystemIOConfiguration systemIOConfiguration = system.getAssociatedSystemIOConfiguration();

                syncModel.setIplConfiguration(JSON.parseObject(JSON.toJSONString(iplConfiguration), AssociatedIPLConfiguration.class));
                syncModel.setSystemCapabilities(JSON.parseObject(JSON.toJSONString(systemCapabilities), AssociatedSystemCapabilities.class));
                syncModel.setSystemMemoryConfiguration(JSON.parseObject(JSON.toJSONString(systemMemoryConfiguration), AssociatedSystemMemoryConfiguration.class));
                syncModel.setSystemProcessorConfiguration(JSON.parseObject(JSON.toJSONString(systemProcessorConfiguration), AssociatedSystemProcessorConfiguration.class));
                syncModel.setSystemIOConfiguration(JSON.parseObject(JSON.toJSONString(systemIOConfiguration), AssociatedSystemIOConfiguration.class));
            }
        }

        // 添加vios分区
        Map<String, List<V8LogicalPartitionProfileEntry>> profileEntrys = new HashMap<>();
        Map<String, List<V8VolumeGroupEntry>> volumeGroupEntries = new HashMap<>();
        if (null != viosEntries) {
            List<V8VirtualIOServerEntry> serverViosEntrys = new ArrayList<>();
            for (V8VirtualIOServerEntry viosEntry : viosEntries) {
                String viosName = Optional.ofNullable(viosEntry)
                        .map(V8VirtualIOServerEntry::getContent)
                        .map(V8VirtualIOServerContent::getVirtualIOServer)
                        .map(V8VirtualIOServer::getPartitionName)
                        .orElse("未知的VIOS名称");
                String[] values = viosEntry.getContent().getVirtualIOServer().getAssociatedManagedSystem().getHref().split("/");
                if (entry.getId().equals(values[values.length - 1])) {
                    serverViosEntrys.add(viosEntry);
                    // get the profile of the vios
                    String uuid = viosEntry.getContent().getVirtualIOServer().getPartitionUUID();
                    logger.info("正在获取服务器[{}]的VIOS[{}]的概要文件数据", systemName, viosName);
                    BocloudResult result = V8LogicalPartitionProfileService.getByVirtualIOServerUuid(auth, uuid);
                    if (result.isSuccess()) {
                        V8LogicalPartitionProfileResponse response = (V8LogicalPartitionProfileResponse) result.getData();
                        profileEntrys.put(uuid, response.getEntries());
                    }
                    logger.info("正在获取服务器[{}]的VolumeGroup[{}]的概要文件数据", systemName, viosName);
                    result = V8VolumeGroupService.get(auth, uuid);
                    if (result.isSuccess()) {
                        if (null != result.getData()) {
                            V8VolumeGroupResponse response = (V8VolumeGroupResponse) result.getData();
                            volumeGroupEntries.put(uuid, response.getEntries());
                        }
                    }
                }
            }
            if (!serverViosEntrys.isEmpty()) {
                syncModel.setViosEntries(JSON.parseArray(JSON.toJSONString(serverViosEntrys), VirtualIOServerEntry.class));
            }
        }

        // 添加lpar分区
        if (null != lparEntries) {
            List<V8LogicalPartitionEntry> serveLparEntrys = new ArrayList<>();
            for (V8LogicalPartitionEntry lparEntry : lparEntries) {
                String lparName = Optional.ofNullable(lparEntry)
                        .map(V8LogicalPartitionEntry::getContent)
                        .map(V8LogicalPartitionContent::getLogicalPartition)
                        .map(V8LogicalPartition::getPartitionName)
                        .orElse("未知的逻辑分区名称");
                String[] values = lparEntry.getContent().getLogicalPartition().getAssociatedManagedSystem().getHref().split("/");
                if (entry.getId().equals(values[values.length - 1])) {
                    serveLparEntrys.add(lparEntry);
                    // get the profile of the lpar
                    String uuid = lparEntry.getContent().getLogicalPartition().getPartitionUUID();
                    logger.info("正在获取服务器[{}]的逻辑分区[{}]的概要文件数据", systemName, lparName);
                    BocloudResult result = V8LogicalPartitionProfileService.getByLogicalPartitionUuid(auth, uuid);
                    if (result.isSuccess()) {
                        V8LogicalPartitionProfileResponse response = (V8LogicalPartitionProfileResponse) result.getData();
                        profileEntrys.put(uuid, response.getEntries());
                    }
                }
            }
            if (!serveLparEntrys.isEmpty()) {
                syncModel.setLparEntries(JSON.parseArray(JSON.toJSONString(serveLparEntrys), LogicalPartitionEntry.class));
            }
        }
        if (!profileEntrys.isEmpty()) {
            Map<String, List<LogicalPartitionProfileEntry>> profileEntryResponses = new HashMap<>();
            for (Map.Entry<String, List<V8LogicalPartitionProfileEntry>> profileEntry : profileEntrys.entrySet()) {
                List<LogicalPartitionProfileEntry> profileEntryResponseValue = JSON.parseArray(JSON.toJSONString(profileEntry.getValue()), LogicalPartitionProfileEntry.class);
                profileEntryResponses.put(profileEntry.getKey(), profileEntryResponseValue);
            }
            syncModel.setProfileEntries(profileEntryResponses);
        }
        if (!volumeGroupEntries.isEmpty()) {
            Map<String, List<VolumeGroupEntry>> groupEntryResponses = new HashMap<>();
            for (Map.Entry<String, List<V8VolumeGroupEntry>> groupEntry : volumeGroupEntries.entrySet()) {
                List<VolumeGroupEntry> groupEntryResponseValue = JSON.parseArray(JSON.toJSONString(groupEntry.getValue()), VolumeGroupEntry.class);
                groupEntryResponses.put(groupEntry.getKey(), groupEntryResponseValue);
            }
            syncModel.setVolumeGroupEntries(groupEntryResponses);
        }

        // 2019-08-05 : 获取逻辑分区和vios的RMC链接状态
        Map<String, String> rmcMap = new HashMap<>();
        if (syncModel.getViosEntries() != null && syncModel.getViosEntries().size() > 0) {
            for (VirtualIOServerEntry viosEntry : syncModel.getViosEntries()) {
                String uuid = viosEntry.getContent().getVirtualIOServer().getPartitionUUID();
                BocloudResult bocloudResult = V8VirtualIOServerService.getQuickProperty(auth, uuid, VirtualIOServerQuickProperty.RMCState);
                if (bocloudResult.isSuccess()) {
                    rmcMap.put(uuid, (String) bocloudResult.getData());
                }
            }
        }
        if (syncModel.getLparEntries() != null && syncModel.getLparEntries().size() > 0) {
            for (LogicalPartitionEntry lparEntry : syncModel.getLparEntries()) {
                String uuid = lparEntry.getContent().getLogicalPartition().getPartitionUUID();
                BocloudResult bocloudResult = V8LogicalPartitionService.getQuickProperty(auth, uuid, LogicalPartitionQuickProperty.RMCState);
                if (bocloudResult.isSuccess()) {
                    rmcMap.put(uuid, (String) bocloudResult.getData());
                }
            }
        }
        syncModel.setRmcStatus(rmcMap);
        return syncModel;
    }

    @Override
    public BocloudResult listRegions() {
        return null;
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

}
