package com.bocloud.cmp.provider.hmc.sdk.api;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.hmc.sdk.enums.RequestMethod;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.model.Result;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.entity.StringEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * HTTP调用的基础服务类
 *
 * @author sxp
 * @since May 2, 2018
 */
public class BaseService {
    private static final int timeout = 1000 * 60 * 60;
    private static Logger logger = LoggerFactory.getLogger(BaseService.class);

    public static Result invokeWithLog(RequestMethod method, String uri, Map<String, Object> header, String body) {
        logger.debug("========调用HMC Rest API========");
        logger.debug("==>> uri: " + uri);
        logger.debug("==>> method: " + method.name());
        logger.debug("==>> header: " + JSON.toJSONString(header));
        logger.debug("==>> body: " + Optional.ofNullable(body)
                .map(s -> s.replaceAll("\r", "").replaceAll("\n", ""))
                .orElse("")
        );
        Result result = new HttpClient(timeout).request(uri, method.name(), header, new HashMap<>(), body);
        logger.debug("==<< code: " + result.getErrorMsg());
        logger.debug("==<< response: " + Optional.ofNullable(result.getMessage())
                .map(s -> s.replaceAll("\r", "").replaceAll("\n", ""))
                .orElse("")
        );
        logger.debug("===============================");
        return result;
    }

    private static class HttpClient extends com.bocloud.common.http.HttpClient {

        @Override
        protected void handleResponse() throws UnsupportedOperationException, IOException {
            super.handleResponse();
            Optional.ofNullable(this.result.getMessage()).ifPresent(msg -> this.result.setMessage(new String(msg.getBytes(StandardCharsets.ISO_8859_1))));
        }

        public HttpClient(int timeout) {
            super(timeout);
            this.resultBuilder = new CommonResultBuilder() {
                @Override
                public Result build(int status) {
                    Result result = super.build(status);
                    result.setErrorMsg(String.valueOf(status));
                    return result;
                }
            };
        }

        public Result request(String uri, String method, Map<String, Object> header, Map<String, Object> params, String body) {
            try {
                uri = this.buildURL(uri, params);
                HttpEntityEnclosingRequestBase httpGet = new HttpEntityEnclosingRequestBase() {
                    @Override
                    public String getMethod() {
                        return method;
                    }
                };
                httpGet.setURI(new URI(uri));
                this.buildHeader(httpGet, header);
                httpGet.setEntity(new StringEntity(Optional.ofNullable(body).orElse("")));
                httpGet.setConfig(this.config);
                this.response = this.httpClient.execute(httpGet);
                this.handleResponse();
            } catch (IOException | URISyntaxException e) {
                logger.error("HttpClient IOException:", e);
                this.result = new Result(false, e.getMessage());
            } finally {
                this.close();
            }
            return this.result;
        }
    }
}
