package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ClientNetworkAdapterEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ClientNetworkAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import java.util.Optional;

/**
 * 客户端网络适配器服务
 * <p>
 * 属于虚拟适配器
 * </p>
 *
 * @author sxp
 * @since Jun 27, 2018
 */
public class V7ClientNetworkAdapterService extends V7BaseService {
    private static final String TYPE = "ClientNetworkAdapter";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; type=" + TYPE;
    private static final String API = "/rest/api/uom/LogicalPartition/%s/" + TYPE;
    private static final String SPECIFIED_API = API + "/%s";

    /**
     * 查询所有的客户端网络适配器
     *
     * @author sxp
     * @since Jun 27, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String lparUuid) {
        String api = String.format(API, lparUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ClientNetworkAdapterResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ClientNetworkAdapterResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询指定唯一标识的客户端网络适配器
     *
     * @author sxp
     * @since Jun 27, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String lparUuid, String clientNetworkAdapterUuid) {
        String api = String.format(SPECIFIED_API, lparUuid, clientNetworkAdapterUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7ClientNetworkAdapterEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7ClientNetworkAdapterEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

}
