package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionFileQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;

import java.util.*;

/**
 * 逻辑分区概要文件服务
 * 
 * @author sxp
 * @since May 11, 2018
 */
public class V7LogicalPartitionProfileService extends V7BaseService {
    private static final String TYPE = "LogicalPartitionProfile";
    private static final String API_1 = "/rest/api/uom/LogicalPartition/%s/LogicalPartitionProfile";
    private static final String API_2 = "/rest/api/uom/LogicalPartition/%s/LogicalPartitionProfile/%s";
    private static final String LPAR_CREATE_API = API_1;
    private static final String LPAR_DELETE_API = API_2;
    private static final String LPAR_UPDATE_API = API_2;
    private static final String API_3 = "/rest/api/uom/VirtualIOServer/%s/LogicalPartitionProfile";
    private static final String API_4 = "/rest/api/uom/VirtualIOServer/%s/LogicalPartitionProfile/%s";
    private static final String VIOS_CREATE_API = API_3;
    private static final String VIOS_DELETE_API = API_4;
    private static final String VIOS_UPDATE_API = API_4;
    private static final String QUICK_PROPERTY_API_2 = API_2 + "/quick/%s";
    private static final String QUICK_PROPERTY_API_4 = API_4 + "/quick/%s";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=LogicalPartitionProfile";

    /**
     * 查询指定逻辑分区的概要文件信息
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getByLogicalPartitionUuid(HmcAuthInfo auth, String lparUuid) {
        String api = String.format(API_1, lparUuid);
        return getResponse(auth, api);
    }

    /**
     * 查询指定逻辑分区的指定uuid的概要文件信息
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getByLogicalPartitionUuid(HmcAuthInfo auth, String logicalPartitionUuid, String profileUuid) {
        String api = String.format(API_2, logicalPartitionUuid, profileUuid);
        return getEntry(auth, api);
    }

    /**
     * 查询指定虚拟IO服务器的概要文件信息
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getByVirtualIOServerUuid(HmcAuthInfo auth, String viosUuid) {
        String api = String.format(API_3, viosUuid);
        return getResponse(auth, api);
    }

    /**
     * 查询指定虚拟IO服务器的指定uuid的概要文件信息
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getByVirtualIOServerUuid(HmcAuthInfo auth, String virtualIOServerUuid, String profileUuid) {
        String api = String.format(API_4, virtualIOServerUuid, profileUuid);
        return getEntry(auth, api);
    }

    private static BocloudResult getResponse(HmcAuthInfo auth, String api) {
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7LogicalPartitionProfileResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7LogicalPartitionProfileResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    private static BocloudResult getEntry(HmcAuthInfo auth, String api) {
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7LogicalPartitionProfileEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7LogicalPartitionProfileEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定逻辑分区的指定uuid概要文件的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getQuickPropertyByLogicalPartitionUuid(HmcAuthInfo auth, String logicalPartitionUuid, String profileUuid, LogicalPartitionFileQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API_2, logicalPartitionUuid, profileUuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 快速查询指定虚拟IO服务器的指定uuid概要文件的某个属性信息
     * <p>
     * 只获取指定属性的值
     * </p>
     * 
     * @author sxp
     * @since May 11, 2018
     */
    public static BocloudResult getQuickPropertyByVirtualIOServerUuid(HmcAuthInfo auth, String virtualIOServerUuid, String profileUuid, LogicalPartitionFileQuickProperty property) {
        String api = String.format(QUICK_PROPERTY_API_4, virtualIOServerUuid, profileUuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }

    /**
     * 创建概要文件
     * 
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult create(HmcAuthInfo auth, boolean isVIOS, String partitionUuid, V7LogicalPartitionProfile profile) {
        String api;
        if (isVIOS) {
            api = String.format(VIOS_CREATE_API, partitionUuid);
        } else {
            api = String.format(LPAR_CREATE_API, partitionUuid);
        }
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(profile);
        xmlText = XmlUtils.appendHmcNamespace(profile.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        V7LogicalPartitionProfileEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7LogicalPartitionProfileEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 删除概要文件
     * 
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult delete(HmcAuthInfo auth, boolean isVIOS, String partitionUuid, String profileUuid) {
        String api;
        if (isVIOS) {
            api = String.format(VIOS_DELETE_API, partitionUuid, profileUuid);
        } else {
            api = String.format(LPAR_DELETE_API, partitionUuid, profileUuid);
        }
        String url = auth.getEndpoint() + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpDelete(auth, url, header, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    /**
     * 修改概要文件
     * 
     * @author sxp
     * @since Jun 15, 2018
     */
    public static BocloudResult update(HmcAuthInfo auth, boolean isVIOS, String partitionUuid, String profileUuid, V7LogicalPartitionProfileForUpdate updateProfile) {
        String api;
        BocloudResult profileResult;
        if (isVIOS) {
            api = String.format(VIOS_UPDATE_API, partitionUuid, profileUuid);
            profileResult = getByVirtualIOServerUuid(auth, partitionUuid, profileUuid);
        } else {
            api = String.format(LPAR_UPDATE_API, partitionUuid, profileUuid);
            profileResult = getByLogicalPartitionUuid(auth, partitionUuid, profileUuid);
        }
        if (profileResult.isFailed()) {
            return new BocloudResult(false, profileResult.getMessage());
        }
        V7LogicalPartitionProfileEntry profileEntry = (V7LogicalPartitionProfileEntry) profileResult.getData();
        Optional<V7LogicalPartitionProfile> profileOpt = Optional.ofNullable(profileEntry)
                .map(V7LogicalPartitionProfileEntry::getContent)
                .map(V7LogicalPartitionProfileContent::getLogicalPartitionProfile);
        if (!profileOpt.isPresent()) {
            return new BocloudResult(false, "根据唯一标识获取概要文件信息为空");
        }

        V7LogicalPartitionProfile profile = profileOpt.get();

        copyProperties(updateProfile, profile);

        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(profile);
        xmlText = XmlUtils.appendHmcNamespace(profile.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPost(auth, url, header, xmlText);
        // 这里要返回Result.errorMsg到BocloudResult.code，这个是错误码：401 404...
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage(), result.getErrorMsg(), null);
        }

        V7LogicalPartitionProfileEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7LogicalPartitionProfileEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }


    private static void copyProperties(V7LogicalPartitionProfileForUpdate source, V7LogicalPartitionProfile target) {
        V7HostEthernetAdapterLogicalPorts hostEthernetAdapterLogicalPorts = source.getHostEthernetAdapterLogicalPorts();
        V7IOConfigurationInstance iOConfigurationInstance = source.getIoConfigurationInstance();
        V7ProcessorAttributes processorAttributes = source.getProcessorAttributes();
        V7ProfileMemory profileMemory = source.getProfileMemory();

        Set<String> ignoreProperties = new HashSet<>();
        ignoreProperties.add("hostEthernetAdapterLogicalPorts");
        ignoreProperties.add("iOConfigurationInstance");
        ignoreProperties.add("processorAttributes");
        ignoreProperties.add("profileMemory");
        EnhancedBeanUtils.copyNotEmptyProperties(target, source, ignoreProperties);

        Boolean assignAllResources = Optional.ofNullable(source.getAssignAllResources()).map(Boolean::parseBoolean).orElse(false);
        if (assignAllResources) {
            // 使用系统所有资源时，要强制将下面的属性值为空
            target.setHostEthernetAdapterLogicalPorts(null);
            target.setIoConfigurationInstance(null);
            target.setProcessorAttributes(null);
            target.setProfileMemory(null);
        } else {
            if (hostEthernetAdapterLogicalPorts != null) {
                if (target.getHostEthernetAdapterLogicalPorts() == null) {
                    target.setHostEthernetAdapterLogicalPorts(hostEthernetAdapterLogicalPorts);
                } else {
                    V7HostEthernetAdapterLogicalPorts dest = target.getHostEthernetAdapterLogicalPorts();
                    EnhancedBeanUtils.copyNotEmptyProperties(dest, hostEthernetAdapterLogicalPorts);
                }
            }

            if (iOConfigurationInstance != null) {
                if (target.getIoConfigurationInstance() == null) {
                    target.setIoConfigurationInstance(iOConfigurationInstance);
                } else {
                    V7IOConfigurationInstance dest = target.getIoConfigurationInstance();
                    EnhancedBeanUtils.copyNotEmptyProperties(dest, iOConfigurationInstance);
                }
            }

            if (processorAttributes != null) {
                if (target.getProcessorAttributes() == null) {
                    target.setProcessorAttributes(processorAttributes);
                } else {
                    V7ProcessorAttributes dest = target.getProcessorAttributes();
                    EnhancedBeanUtils.copyNotEmptyProperties(dest, processorAttributes);
                }
            }

            if (profileMemory != null) {
                if (target.getProfileMemory() == null) {
                    target.setProfileMemory(profileMemory);
                } else {
                    V7ProfileMemory dest = target.getProfileMemory();
                    EnhancedBeanUtils.copyNotEmptyProperties(dest, profileMemory);
                }
            }
        }
    }

}
