package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.enums.RequestMethod;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7Atom;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7LogonRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7LogonResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.common.V7Metadata;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7Password;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7UserID;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * HMC登陆
 * <p>
 * 获取X-API-Session
 * </p>
 * 
 * @author sxp
 * @since May 2, 2018
 */
public class V7LogonService extends V7BaseService {
    private static final Logger logger = LoggerFactory.getLogger(V7LogonService.class);

    public static final Map<HmcAuthInfo, String> sessionHolder = new HashMap<>();

    public static void clearSession(HmcAuthInfo auth) {
        sessionHolder.remove(auth);
    }

    public synchronized static String getSession(HmcAuthInfo auth) {
        if (sessionHolder.containsKey(auth)) {
            return sessionHolder.get(auth);
        }
        String session = getSessionFromHmc(auth);
        sessionHolder.put(auth, session);
        return session;
    }

    /**
     * 获取X-API-Session
     * 
     * @author sxp
     * @since May 2, 2018
     */
    private static String getSessionFromHmc(HmcAuthInfo auth) {
        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, "application/vnd.ibm.powervm.web+xml");
        String url = auth.getEndpoint() + "/rest/api/web/Logon";
        Result result = invokeWithErrorParse(RequestMethod.PUT, url, header, genLogonRequestXml(auth));
        if (result.isFailed()) {
            String message = String.format("从HMC获取Session失败[%s]: %s", result.getErrorMsg(), result.getMessage());
            throw new RuntimeException(message);
        }
        V7LogonResponse response = XmlUtils.parseObject(V7LogonResponse.class, result.getMessage());
        String session = response.getSession().getValue();
        logger.info("Get HMC[{}]‘s session from HMC rest api: {}", auth.getEndpoint(), session);
        return session;
    }

    /**
     * 生成登陆请求的XML
     * 
     * @author sxp
     * @since May 3, 2018
     */
    private static String genLogonRequestXml(HmcAuthInfo auth) {
        V7LogonRequest request = new V7LogonRequest();
        V7Metadata metadata = new V7Metadata(new V7Atom());

        V7UserID userId = new V7UserID(auth.getUsername());
        userId.setKb("CUR");
        userId.setKxe("false");

        V7Password password = new V7Password(auth.getPassword());
        password.setKb("CUR");
        password.setKxe("false");

        request.setMetadata(metadata);
        request.setUserId(userId);
        request.setPassword(password);

        return XmlUtils.toXmlString(request);
    }

}
