package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSCSIClientAdapterEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSCSIClientAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import java.util.Optional;

/**
 * 虚拟SCSI客户端适配器服务
 * <p>
 * 属于虚拟适配器
 * </p>
 * 
 * @author sxp
 * @since Jun 27, 2018
 */
public class V7VirtualSCSIClientAdapterService extends V7BaseService {
    private static final String TYPE = "VirtualSCSIClientAdapter";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; type=" + TYPE;
    private static final String API = "/rest/api/uom/LogicalPartition/%s/" + TYPE;
    private static final String SPECIFIED_API = API + "/%s";

    public static BocloudResult get(HmcAuthInfo auth, String lparUuid) {
        String api = String.format(API, lparUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSCSIClientAdapterResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSCSIClientAdapterResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    public static BocloudResult getByHref(HmcAuthInfo auth, String href) {
        Result result = httpGet(auth, href, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSCSIClientAdapterEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSCSIClientAdapterEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    public static BocloudResult get(HmcAuthInfo auth, String lparUuid, String VirtualSCSIClientAdapterUuid) {
        String api = String.format(SPECIFIED_API, lparUuid, VirtualSCSIClientAdapterUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSCSIClientAdapterEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSCSIClientAdapterEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

}
