package com.bocloud.cmp.provider.hmc.sdk.api.v7;

import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.EnhancedBeanUtils;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.http.HttpHeaders;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * PowerVM Virtual Switch APIs
 * 
 * @author lkx
 *
 */
public class V7VirtualSwitchService extends V7BaseService {
    private static final String TYPE = "VirtualSwitch";
    private static final String API = "/rest/api/uom/ManagedSystem/%s/VirtualSwitch";
    private static final String UUID_API = "/rest/api/uom/ManagedSystem/%s/VirtualSwitch/%s";
    private static final String SPECIFIED_API = API + "/%s";

    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml;" + " Type=VirtualSwitch";

    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(API, managedSystemUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSwitchResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSwitchResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid, String VirtualSwitchUuid) {
        String api = String.format(UUID_API, managedSystemUuid, VirtualSwitchUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSwitchEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSwitchEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 创建虚拟网络交换机
     */
    public static BocloudResult create(HmcAuthInfo auth, String serverUuid, V7VirtualSwitch virtualSwitch) {
        String api = String.format(API, serverUuid);
        String url = auth.getEndpoint() + api;

        String xmlText = XmlUtils.toXmlString(virtualSwitch);
        xmlText = XmlUtils.appendHmcNamespace(virtualSwitch.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpPut(auth, url, header, xmlText);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V7VirtualSwitchEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V7VirtualSwitchEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    public static BocloudResult delete(HmcAuthInfo auth, String serverUuid, String virtualSwitchUuid) {
        String api = String.format(SPECIFIED_API, serverUuid, virtualSwitchUuid);
        String url = auth.getEndpoint() + api;

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        Result result = httpDelete(auth, url, header, null);
        return new BocloudResult(result.isSuccess(), result.getMessage(), result.getMessage());
    }

    /**
     * 修改虚拟网络交换机
     */
    public static BocloudResult update(HmcAuthInfo auth, String serverUuid, String virtualSwitchUuid, V7VirtualSwitchForUpdate virtualSwitchForUpdate) {
        BocloudResult getResult = get(auth, serverUuid, virtualSwitchUuid);
        if (getResult.isFailed()) {
            return getResult;
        }
        V7VirtualSwitchEntry entry = (V7VirtualSwitchEntry) getResult.getData();

        Optional<V7VirtualSwitch> virtualSwitchOpt = Optional.ofNullable(entry)
                .map(V7VirtualSwitchEntry::getContent)
                .map(V7VirtualSwitchContent::getVirtualSwitch);
        if (!virtualSwitchOpt.isPresent()) {
            return new BocloudResult(false, "系统中不存在唯一标识为[" + virtualSwitchUuid + "]的虚拟网络交换机");
        }

        V7VirtualSwitch virtualSwitch = virtualSwitchOpt.get();

        EnhancedBeanUtils.copyNotEmptyProperties(virtualSwitch, virtualSwitchForUpdate);

        String endpoint = auth.getEndpoint();
        String api = String.format(SPECIFIED_API, serverUuid, virtualSwitchUuid);
        String url = endpoint + api;

        String xmlText = XmlUtils.toXmlString(virtualSwitch);
        xmlText = XmlUtils.appendHmcNamespace(virtualSwitch.getClass(), xmlText);

        Map<String, Object> header = new HashMap<>();
        header.put(HttpHeaders.CONTENT_TYPE, CONTENT_TYPE);

        // 这里不能直接拿postResult的message转换成V7VirtualSwitchEntry再返回，因为postResult的返回结果是修改之前的
        // 也许是HMC的bug，暂不确定870是否有此问题
        Result postResult = httpPost(auth, url, header, xmlText);
        if (postResult.isFailed()) {
            return new BocloudResult(false, postResult.getMessage());
        }
        return get(auth, serverUuid, virtualSwitchUuid);
    }
}
