package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleEntry;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleResponse;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.common.V8ManagementConsoleEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.common.V8ManagementConsoleResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import java.util.Optional;

/**
 * 管理控制台
 * 
 * @author sxp
 * @since May 3, 2018
 */
public class V8ManagementConsoleService extends V8BaseService {
    private static final String TYPE = "ManagementConsole";
    private static final String API = "/rest/api/uom/ManagementConsole";
    private static final String SPECIFIED_API = API + "/%s";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.web+xml";

    /**
     * 获取控制台信息
     * 
     * <pre>
     * Management Console lists the following information:
     * 1. Links to the Systems Managed by this Management Console.
     * 2. Network Information regarding the Management Console.
     * 3. Version of the Management Console
     * </pre>
     * 
     * @author sxp
     * @since May 4, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth) {
        Result result = httpGet(auth, auth.getEndpoint() + API, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8ManagementConsoleResponse responseV8 = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8ManagementConsoleResponse.class, msg))
                .orElse(null);
        ManagementConsoleResponse response = JSON.parseObject(JSON.toJSONString(responseV8), ManagementConsoleResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获取指定唯一标识的控制台信息
     * 
     * @author sxp
     * @since May 7, 2018
     */
    public static BocloudResult getByUuid(HmcAuthInfo auth, String uuid) {
        String api = String.format(SPECIFIED_API, uuid);

        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        V8ManagementConsoleEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8ManagementConsoleEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 增加受管系统(ManagedSystem)到HMC平台
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult addManagedSystem(HmcAuthInfo auth, String consoleUuid, String hostIp, String password) {
        return V8JobService.addManagedSystem(auth, consoleUuid, hostIp, password);
    }

    /**
     * 从HMC平台删除受管系统(ManagedSystem)的连接
     * 
     * @author sxp
     * @since Jun 7, 2018
     */
    public static BocloudResult removeManagedSystemConnection(HmcAuthInfo auth, String consoleUuid, String managedSystemUuid) {
        return V8JobService.removeManagedSystemConnection(auth, consoleUuid, managedSystemUuid);
    }

}
