package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.bocloud.cmp.provider.hmc.sdk.enums.SharedProcessorPoolQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8SharedProcessorPoolEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8SharedProcessorPoolResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import java.util.Optional;

/**
 * CPU共享池
 * 
 * @author sxp
 * @since May 15, 2018
 */
public class V8SharedProcessorPoolService extends V8BaseService {
    private static final String TYPE = "SharedProcessorPool";
    private static final String API_1 = "/rest/api/uom/ManagedSystem/%s/SharedProcessorPool";
    private static final String API_2 = "/rest/api/uom/ManagedSystem/%s/SharedProcessorPool/%s";
    private static final String QUICK_API = "/rest/api/uom/ManagedSystem/%s/SharedProcessorPool/%s/quick/%s";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; Type=SharedProcessorPool";

    /**
     * 查询CPU共享池
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(API_1, managedSystemUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8SharedProcessorPoolResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8SharedProcessorPoolResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询指定CPU共享池
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid, String poolUuid) {
        String api = String.format(API_2, managedSystemUuid, poolUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8SharedProcessorPoolEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8SharedProcessorPoolEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 快速查询指定CPU共享池的属性值
     * 
     * @author sxp
     * @since May 15, 2018
     */
    public static BocloudResult getQuickProperty(HmcAuthInfo auth, String managedSystemUuid, String poolUuid, SharedProcessorPoolQuickProperty property) {
        String api = String.format(QUICK_API, managedSystemUuid, poolUuid, property.name());
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        String value = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace("\"", ""))
                .orElse(null);
        return new BocloudResult(true, value, "执行成功");
    }
}
