package com.bocloud.cmp.provider.hmc.sdk.api.v8;

import com.bocloud.cmp.model.hmc.sdk.module.VirtualNetworkEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcAuthInfo;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualNetworkResponse;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import java.util.Optional;

/**
 * 虚拟网络服务
 * 
 * @author sxp
 * @since Jun 25, 2018
 */
public class V8VirtualNetworkService extends V8BaseService {
    private static final String TYPE = "VirtualNetwork";
    private static final String CONTENT_TYPE = "application/vnd.ibm.powervm.uom+xml; type=" + TYPE;
    private static final String API = "/rest/api/uom/ManagedSystem/%s/" + TYPE;
    private static final String SPECIFIED_API = API + "/%s";

    /**
     * 查询虚拟网络列表
     * 
     * @author sxp
     * @since Jun 25, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid) {
        String api = String.format(API, managedSystemUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        V8VirtualNetworkResponse response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(V8VirtualNetworkResponse.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 查询指定虚拟网络
     * 
     * @author sxp
     * @since Jun 25, 2018
     */
    public static BocloudResult get(HmcAuthInfo auth, String managedSystemUuid, String virtualNetworkUuid) {
        String api = String.format(SPECIFIED_API, managedSystemUuid, virtualNetworkUuid);
        Result result = httpGet(auth, auth.getEndpoint() + api, CONTENT_TYPE);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        VirtualNetworkEntry response = Optional.ofNullable(result.getMessage())
                .map(msg -> msg.replace(TYPE + ":" + TYPE, TYPE))
                .map(msg -> XmlUtils.parseObject(VirtualNetworkEntry.class, msg))
                .orElse(null);
        return new BocloudResult(true, response, "执行成功");
    }

}
