package com.bocloud.cmp.provider.hmc.sdk.helper.v7;

import com.bocloud.cmp.model.hmc.HmcProfileFibreAdapterModel;
import com.bocloud.cmp.model.hmc.HmcProfileIoslotModel;
import com.bocloud.cmp.model.hmc.HmcProfileLheaModel;
import com.bocloud.cmp.model.hmc.HmcProfileNetworkAdapterModel;
import com.bocloud.cmp.model.hmc.HmcProfileScsiAdapterModel;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterLogicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileClientNetworkAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileTrunkAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileVirtualFibreChannelClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileVirtualIOAdapterSubclass;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileVirtualIOAdapters;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileVirtualSCSIClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileVirtualSCSIServerAdapter;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class V7HmcDriverHelper {

    static void initProfileAdapterForUpdate(V7IOConfigurationInstance ioConfig) {
        if (null == ioConfig) {
            ioConfig = new V7IOConfigurationInstance();
        }
        V7ProfileVirtualIOAdapters vAdapters = ioConfig.getProfileVirtualIoAdapters();
        if (null == vAdapters) {
            vAdapters = new V7ProfileVirtualIOAdapters();
            ioConfig.setProfileVirtualIoAdapters(vAdapters);
        }
        List<V7ProfileVirtualIOAdapterSubclass> subclasses = vAdapters.getSubclasses();
        if (null == subclasses) {
            subclasses = new ArrayList<>();
            vAdapters.setSubclasses(subclasses);
        }
    }

    static void initProfileIOSlots(V7IOConfigurationInstance ioConfig) {
        if (null == ioConfig) {
            ioConfig = new V7IOConfigurationInstance();
        }
        V7ProfileIOSlots profileIoSlots = ioConfig.getProfileIoSlots();
        if (null == profileIoSlots) {
            profileIoSlots = new V7ProfileIOSlots();
            ioConfig.setProfileIoSlots(profileIoSlots);
        }
        List<V7ProfileIOSlot> listProfileIoSlots = profileIoSlots.getProfileIOSlot();
        if (null == listProfileIoSlots) {
            listProfileIoSlots = new ArrayList<>();
            profileIoSlots.setProfileIOSlot(listProfileIoSlots);
        }
    }

    static Boolean updateProfileNetworkAdapter(V7IOConfigurationInstance ioConfig, List<HmcProfileNetworkAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }
        initProfileAdapterForUpdate(ioConfig);
        if (null != ioConfig) {
            V7ProfileVirtualIOAdapters ioAdapters = ioConfig.getProfileVirtualIoAdapters();
            if (null != ioAdapters) {
                List<V7ProfileVirtualIOAdapterSubclass> subclasses = ioAdapters.getSubclasses();
                for (HmcProfileNetworkAdapterModel network : adapters) {
                    V7ProfileVirtualIOAdapterSubclass adapterSubclass = new V7ProfileVirtualIOAdapterSubclass();
                    if (null != network.getPriority()) {
                        V7ProfileTrunkAdapter trunckAdapter = new V7ProfileTrunkAdapter();
                        trunckAdapter.setIsRequired(network.getIsRequired().toString());
                        trunckAdapter.setPortVlanID(network.getVirtualVlanId().toString());
                        trunckAdapter.setTaggedVlanIds(network.getTaggedVlanIds());
                        trunckAdapter.setVirtualSlotNumber(network.getAdapterId().toString());
                        trunckAdapter.setIsTaggedVlanSupported(network.getIsTaggedVlanSupported().toString());
                        trunckAdapter.setTrunkPriority(network.getPriority().toString());
                        trunckAdapter.setVirtualSwitchID(network.getVirtualSwitchId().toString());
                        adapterSubclass.setProfileTrunkAdapter(trunckAdapter);
                    } else {
                        V7ProfileClientNetworkAdapter networkAdapter = new V7ProfileClientNetworkAdapter();
                        networkAdapter.setIsRequired(network.getIsRequired().toString());
                        networkAdapter.setPortVlanID(network.getVirtualVlanId().toString());
                        networkAdapter.setTaggedVlanIds(network.getTaggedVlanIds());
                        networkAdapter.setVirtualSlotNumber(network.getAdapterId().toString());
                        networkAdapter.setIsTaggedVlanSupported(network.getIsTaggedVlanSupported().toString());
                        networkAdapter.setVirtualSwitchID(network.getVirtualSwitchId().toString());
                        adapterSubclass.setProfileClientNetworkAdapter(networkAdapter);
                    }
                    subclasses.add(adapterSubclass);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileFibreAdapter(V7IOConfigurationInstance ioConfig, List<HmcProfileFibreAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }

        initProfileAdapterForUpdate(ioConfig);
        if (null != ioConfig) {
            V7ProfileVirtualIOAdapters ioAdapters = ioConfig.getProfileVirtualIoAdapters();
            if (null != ioAdapters) {
                List<V7ProfileVirtualIOAdapterSubclass> subclasses = ioAdapters.getSubclasses();
                for (HmcProfileFibreAdapterModel fibre : adapters) {
                    V7ProfileVirtualIOAdapterSubclass adapterSubclass = new V7ProfileVirtualIOAdapterSubclass();
                    V7ProfileVirtualFibreChannelClientAdapter fibreAdapter = new V7ProfileVirtualFibreChannelClientAdapter();
                    fibreAdapter.setIsRequired(fibre.getIsRequired().toString());
                    fibreAdapter.setConnectingPartitionID(fibre.getConnectingPartitionId().toString());
                    fibreAdapter.setConnectingVirtualSlotNumber(fibre.getConnectingVirtualSlotNumber().toString());
                    fibreAdapter.setVirtualSlotNumber(fibre.getAdapterId().toString());
                    fibreAdapter.setAdapterType(fibre.getType());
                    adapterSubclass.setProfileVirtualFibreChannelClientAdapter(fibreAdapter);
                    subclasses.add(adapterSubclass);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileScsiAdapter(V7IOConfigurationInstance ioConfig, List<HmcProfileScsiAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }

        initProfileAdapterForUpdate(ioConfig);
        List<V7ProfileVirtualIOAdapterSubclass> subclasses = ioConfig.getProfileVirtualIoAdapters().getSubclasses();
        for (HmcProfileScsiAdapterModel scsi : adapters) {
            V7ProfileVirtualIOAdapterSubclass adapterSubclass = new V7ProfileVirtualIOAdapterSubclass();
            if ("Client".equalsIgnoreCase(scsi.getType())) {
                V7ProfileVirtualSCSIClientAdapter scsiAdapter = new V7ProfileVirtualSCSIClientAdapter();
                scsiAdapter.setIsRequired(scsi.getIsRequired().toString());
                scsiAdapter.setRemoteSlotNumber(scsi.getRemoteSlotNumber().toString());
                scsiAdapter.setRemotePartitionID(scsi.getRemotePartitionId().toString());
                scsiAdapter.setVirtualSlotNumber(scsi.getAdapterId().toString());
                scsiAdapter.setAdapterType(scsi.getType().toString());
                adapterSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);
            }
            if ("Server".equalsIgnoreCase(scsi.getType())) {
                V7ProfileVirtualSCSIServerAdapter scsiAdapter = new V7ProfileVirtualSCSIServerAdapter();
                scsiAdapter.setIsRequired(scsi.getIsRequired().toString());
                if (null != scsi.getRemoteSlotNumber()) {
                    scsiAdapter.setRemoteSlotNumber(scsi.getRemoteSlotNumber().toString());
                }
                if (null != scsi.getRemotePartitionId()) {
                    scsiAdapter.setRemotePartitionID(scsi.getRemotePartitionId().toString());
                }
                scsiAdapter.setVirtualSlotNumber(scsi.getAdapterId().toString());
                scsiAdapter.setAdapterType(scsi.getType().toString());
                adapterSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);
            }
            subclasses.add(adapterSubclass);
        }

        return true;
    }

    static Boolean updateProfileIoslots(V7IOConfigurationInstance ioConfig, List<HmcProfileIoslotModel> ioslots) {
        if (null == ioslots || ioslots.isEmpty()) {
            return false;
        }

        initProfileIOSlots(ioConfig);
        if (null != ioConfig) {
            V7ProfileIOSlots slots = ioConfig.getProfileIoSlots();
            if (null != slots) {
                List<V7ProfileIOSlot> listProfileIOSlots = slots.getProfileIOSlot();
                for (HmcProfileIoslotModel ioslot : ioslots) {
                    V7ProfileIOSlot profileIOSlot = new V7ProfileIOSlot();
                    profileIOSlot.setIsRequired(ioslot.getIsRequired().toString());
                    V7AssociatedIOSlot associatedIOSlot = new V7AssociatedIOSlot();
                    associatedIOSlot.setSlotDynamicReconfigurationConnectorIndex(ioslot.getAdapterId().toString());
                    profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
                    listProfileIOSlots.add(profileIOSlot);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileAdapter(
            V7IOConfigurationInstance ioConfig,
            List<HmcProfileNetworkAdapterModel> networkerAdapters,
            List<HmcProfileFibreAdapterModel> fibreAdapters,
            List<HmcProfileScsiAdapterModel> scsiAdapters
    ) {
        Boolean result = false;
        if (updateProfileNetworkAdapter(ioConfig, networkerAdapters)) {
            result = true;
        }
        if (updateProfileFibreAdapter(ioConfig, fibreAdapters)) {
            result = true;
        }
        if (updateProfileScsiAdapter(ioConfig, scsiAdapters)) {
            result = true;
        }

        return result;
    }

    // 更新概要文件中的IOConfigurationInstance
    public static Boolean updateProfileIOConfiguration(
            V7IOConfigurationInstance ioConfig, Integer maxVirtualIoslots,
            List<HmcProfileNetworkAdapterModel> networkerAdapters,
            List<HmcProfileFibreAdapterModel> fibreAdapters,
            List<HmcProfileScsiAdapterModel> scsiAdapters,
            List<HmcProfileIoslotModel> ioslots
    ) {
        Boolean result = false;
        // 更机虚拟适配器设置
        if (updateProfileAdapter(ioConfig, networkerAdapters, fibreAdapters, scsiAdapters)) {
            result = true;
        }
        // 更机虚拟适配器设置
        if (updateProfileIoslots(ioConfig, ioslots)) {
            result = true;
        }
        // 更机最大虚拟适配器数设置
        if (null != maxVirtualIoslots) {
            if (null == ioConfig) {
                return false;
            }
            ioConfig.setMaximumVirtualIoSlots(maxVirtualIoslots.toString());
            result = true;
        }

        return result;
    }

    public static Boolean updateProfileHEALogicalPort(V7HostEthernetAdapterLogicalPorts logicalPorts, List<HmcProfileLheaModel> lheas) {
        if (null == lheas) {
            return false;
        }

        // 初始HostEthernetAdapterLogicalPorts化结构体
        if (null == logicalPorts) {
            logicalPorts = new V7HostEthernetAdapterLogicalPorts();
        }
        List<V7HostEthernetAdapterLogicalPort> listLogicalPort = logicalPorts.getHostEthernetAdapterLogicalPortList();
        if (null == listLogicalPort) {
            listLogicalPort = new ArrayList<>();
            logicalPorts.setHostEthernetAdapterLogicalPortList(listLogicalPort);
        }

        // 添加主机以太网逻辑端口设置
        for (HmcProfileLheaModel lhea : lheas) {
            V7HostEthernetAdapterLogicalPort logicalPort = new V7HostEthernetAdapterLogicalPort();
            logicalPort.setAdapterID(lhea.getAdapterId().toString());
            logicalPort.setLogicalPortID(lhea.getLogicalPortId().toString());
            logicalPort.setPhysicalPortID(lhea.getPhysicalPortId().toString());
            logicalPort.setAllowedVLANIDs(StringUtils.isEmpty(lhea.getAllowVlanIds()) ? null : lhea.getAllowVlanIds());
            logicalPort.setAllowedOSMACAddresses(StringUtils.isEmpty(lhea.getAllowMacAddresses()) ? null : lhea.getAllowMacAddresses());
            logicalPort.setPortGroupID(lhea.getPortGroupId().toString());
            listLogicalPort.add(logicalPort);
        }
        return true;
    }
}
