package com.bocloud.cmp.provider.hmc.sdk.helper.v8;

import com.bocloud.cmp.model.hmc.HmcProfileFibreAdapterModel;
import com.bocloud.cmp.model.hmc.HmcProfileIoslotModel;
import com.bocloud.cmp.model.hmc.HmcProfileLheaModel;
import com.bocloud.cmp.model.hmc.HmcProfileNetworkAdapterModel;
import com.bocloud.cmp.model.hmc.HmcProfileScsiAdapterModel;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterLogicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileClientNetworkAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileTrunkAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileVirtualFibreChannelClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileVirtualIOAdapterSubclass;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileVirtualIOAdapters;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileVirtualSCSIClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileVirtualSCSIServerAdapter;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

public class V8HmcDriverHelper {

    static void initProfileAdapterForUpdate(V8IOConfigurationInstance ioConfig) {
        if (null == ioConfig) {
            ioConfig = new V8IOConfigurationInstance();
        }
        V8ProfileVirtualIOAdapters vAdapters = ioConfig.getProfileVirtualIoAdapters();
        if (null == vAdapters) {
            vAdapters = new V8ProfileVirtualIOAdapters();
            ioConfig.setProfileVirtualIoAdapters(vAdapters);
        }
        List<V8ProfileVirtualIOAdapterSubclass> subclasses = vAdapters.getSubclasses();
        if (null == subclasses) {
            subclasses = new ArrayList<>();
            vAdapters.setSubclasses(subclasses);
        }
    }

    static void initProfileIOSlots(V8IOConfigurationInstance ioConfig) {
        if (null == ioConfig) {
            ioConfig = new V8IOConfigurationInstance();
        }
        V8ProfileIOSlots profileIoSlots = ioConfig.getProfileIoSlots();
        if (null == profileIoSlots) {
            profileIoSlots = new V8ProfileIOSlots();
            ioConfig.setProfileIoSlots(profileIoSlots);
        }
        List<V8ProfileIOSlot> listProfileIoSlots = profileIoSlots.getProfileIOSlot();
        if (null == listProfileIoSlots) {
            listProfileIoSlots = new ArrayList<>();
            profileIoSlots.setProfileIOSlot(listProfileIoSlots);
        }
    }

    static Boolean updateProfileNetworkAdapter(V8IOConfigurationInstance ioConfig, List<HmcProfileNetworkAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }
        initProfileAdapterForUpdate(ioConfig);
        if (null != ioConfig) {
            V8ProfileVirtualIOAdapters ioAdapters = ioConfig.getProfileVirtualIoAdapters();
            if (null != ioAdapters) {
                List<V8ProfileVirtualIOAdapterSubclass> subclasses = ioAdapters.getSubclasses();
                for (HmcProfileNetworkAdapterModel network : adapters) {
                    V8ProfileVirtualIOAdapterSubclass adapterSubclass = new V8ProfileVirtualIOAdapterSubclass();
                    if (null != network.getPriority()) {
                        V8ProfileTrunkAdapter trunckAdapter = new V8ProfileTrunkAdapter();
                        trunckAdapter.setIsRequired(network.getIsRequired().toString());
                        trunckAdapter.setPortVlanID(network.getVirtualVlanId().toString());
                        trunckAdapter.setTaggedVlanIds(network.getTaggedVlanIds());
                        trunckAdapter.setVirtualSlotNumber(network.getAdapterId().toString());
                        trunckAdapter.setIsTaggedVlanSupported(network.getIsTaggedVlanSupported().toString());
                        trunckAdapter.setTrunkPriority(network.getPriority().toString());
                        trunckAdapter.setVirtualSwitchID(network.getVirtualSwitchId().toString());
                        String switchName = network.getVirtualSwitchName();
                        // 需要把交换机名称中的(Default)移除，用真正的名称来代替
                        switchName = switchName.replace("(Default)", "");
                        trunckAdapter.setVirtualSwitchName(switchName);
                        adapterSubclass.setProfileTrunkAdapter(trunckAdapter);
                    } else {
                        V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
                        networkAdapter.setIsRequired(network.getIsRequired().toString());
                        networkAdapter.setPortVlanID(network.getVirtualVlanId().toString());
                        networkAdapter.setTaggedVlanIds(network.getTaggedVlanIds());
                        networkAdapter.setVirtualSlotNumber(network.getAdapterId().toString());
                        networkAdapter.setIsTaggedVlanSupported(network.getIsTaggedVlanSupported().toString());
                        networkAdapter.setVirtualSwitchID(network.getVirtualSwitchId().toString());
                        String switchName = network.getVirtualSwitchName();
                        // 需要把交换机名称中的(Default)移除，用真正的名称来代替
                        switchName = switchName.replace("(Default)", "");
                        networkAdapter.setVirtualSwitchName(switchName);
                        adapterSubclass.setProfileClientNetworkAdapter(networkAdapter);
                    }
                    subclasses.add(adapterSubclass);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileFibreAdapter(V8IOConfigurationInstance ioConfig, List<HmcProfileFibreAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }

        initProfileAdapterForUpdate(ioConfig);
        if (null != ioConfig) {
            V8ProfileVirtualIOAdapters ioAdapters = ioConfig.getProfileVirtualIoAdapters();
            if (null != ioAdapters) {
                List<V8ProfileVirtualIOAdapterSubclass> subclasses = ioAdapters.getSubclasses();
                for (HmcProfileFibreAdapterModel fibre : adapters) {
                    V8ProfileVirtualIOAdapterSubclass adapterSubclass = new V8ProfileVirtualIOAdapterSubclass();
                    V8ProfileVirtualFibreChannelClientAdapter fibreAdapter = new V8ProfileVirtualFibreChannelClientAdapter();
                    fibreAdapter.setIsRequired(fibre.getIsRequired().toString());
                    fibreAdapter.setConnectingPartitionID(fibre.getConnectingPartitionId().toString());
                    fibreAdapter.setConnectingVirtualSlotNumber(fibre.getConnectingVirtualSlotNumber().toString());
                    fibreAdapter.setVirtualSlotNumber(fibre.getAdapterId().toString());
                    fibreAdapter.setAdapterType(fibre.getType());
                    adapterSubclass.setProfileVirtualFibreChannelClientAdapter(fibreAdapter);
                    subclasses.add(adapterSubclass);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileScsiAdapter(V8IOConfigurationInstance ioConfig, List<HmcProfileScsiAdapterModel> adapters) {
        if (null == adapters || adapters.isEmpty()) {
            return false;
        }

        initProfileAdapterForUpdate(ioConfig);
        List<V8ProfileVirtualIOAdapterSubclass> subclasses = ioConfig.getProfileVirtualIoAdapters().getSubclasses();
        for (HmcProfileScsiAdapterModel scsi : adapters) {
            V8ProfileVirtualIOAdapterSubclass adapterSubclass = new V8ProfileVirtualIOAdapterSubclass();
            if ("Client".equalsIgnoreCase(scsi.getType())) {
                V8ProfileVirtualSCSIClientAdapter scsiAdapter = new V8ProfileVirtualSCSIClientAdapter();
                scsiAdapter.setIsRequired(scsi.getIsRequired().toString());
                scsiAdapter.setRemoteSlotNumber(scsi.getRemoteSlotNumber().toString());
                scsiAdapter.setRemotePartitionID(scsi.getRemotePartitionId().toString());
                scsiAdapter.setVirtualSlotNumber(scsi.getAdapterId().toString());
                scsiAdapter.setAdapterType(scsi.getType().toString());
                adapterSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);
            }
            if ("Server".equalsIgnoreCase(scsi.getType())) {
                V8ProfileVirtualSCSIServerAdapter scsiAdapter = new V8ProfileVirtualSCSIServerAdapter();
                scsiAdapter.setIsRequired(scsi.getIsRequired().toString());
                if (null != scsi.getRemoteSlotNumber()) {
                    scsiAdapter.setRemoteSlotNumber(scsi.getRemoteSlotNumber().toString());
                }
                if (null != scsi.getRemotePartitionId()) {
                    scsiAdapter.setRemotePartitionID(scsi.getRemotePartitionId().toString());
                }
                scsiAdapter.setVirtualSlotNumber(scsi.getAdapterId().toString());
                scsiAdapter.setAdapterType(scsi.getType().toString());
                adapterSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);
            }
            subclasses.add(adapterSubclass);
        }

        return true;
    }

    static Boolean updateProfileIoslots(V8IOConfigurationInstance ioConfig, List<HmcProfileIoslotModel> ioslots) {
        if (null == ioslots || ioslots.isEmpty()) {
            return false;
        }

        initProfileIOSlots(ioConfig);
        if (null != ioConfig) {
            V8ProfileIOSlots slots = ioConfig.getProfileIoSlots();
            if (null != slots) {
                List<V8ProfileIOSlot> listProfileIOSlots = slots.getProfileIOSlot();
                for (HmcProfileIoslotModel ioslot : ioslots) {
                    V8ProfileIOSlot profileIOSlot = new V8ProfileIOSlot();
                    profileIOSlot.setIsRequired(ioslot.getIsRequired().toString());
                    V8AssociatedIOSlot associatedIOSlot = new V8AssociatedIOSlot();
                    associatedIOSlot.setSlotDynamicReconfigurationConnectorIndex(ioslot.getAdapterId().toString());
                    profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
                    listProfileIOSlots.add(profileIOSlot);
                }
            }
        }
        return true;
    }

    static Boolean updateProfileAdapter(
            V8IOConfigurationInstance ioConfig,
            List<HmcProfileNetworkAdapterModel> networkerAdapters,
            List<HmcProfileFibreAdapterModel> fibreAdapters,
            List<HmcProfileScsiAdapterModel> scsiAdapters
    ) {
        Boolean result = false;
        if (updateProfileNetworkAdapter(ioConfig, networkerAdapters)) {
            result = true;
        }
        if (updateProfileFibreAdapter(ioConfig, fibreAdapters)) {
            result = true;
        }
        if (updateProfileScsiAdapter(ioConfig, scsiAdapters)) {
            result = true;
        }

        return result;
    }

    // 更新概要文件中的IOConfigurationInstance
    public static Boolean updateProfileIOConfiguration(
            V8IOConfigurationInstance ioConfig, Integer maxVirtualIoslots,
            List<HmcProfileNetworkAdapterModel> networkerAdapters,
            List<HmcProfileFibreAdapterModel> fibreAdapters,
            List<HmcProfileScsiAdapterModel> scsiAdapters,
            List<HmcProfileIoslotModel> ioslots
    ) {
        Boolean result = false;
        // 更机虚拟适配器设置
        if (updateProfileAdapter(ioConfig, networkerAdapters, fibreAdapters, scsiAdapters)) {
            result = true;
        }
        // 更机虚拟适配器设置
        if (updateProfileIoslots(ioConfig, ioslots)) {
            result = true;
        }
        // 更机最大虚拟适配器数设置
        if (null != maxVirtualIoslots) {
            if (null == ioConfig) {
                return false;
            }
            ioConfig.setMaximumVirtualIoSlots(maxVirtualIoslots.toString());
            result = true;
        }

        return result;
    }

    public static Boolean updateProfileHEALogicalPort(V8HostEthernetAdapterLogicalPorts logicalPorts, List<HmcProfileLheaModel> lheas) {
        if (null == lheas) {
            return false;
        }

        // 初始HostEthernetAdapterLogicalPorts化结构体
        if (null == logicalPorts) {
            logicalPorts = new V8HostEthernetAdapterLogicalPorts();
        }
        List<V8HostEthernetAdapterLogicalPort> listLogicalPort = logicalPorts.getHostEthernetAdapterLogicalPortList();
        if (null == listLogicalPort) {
            listLogicalPort = new ArrayList<>();
            logicalPorts.setHostEthernetAdapterLogicalPortList(listLogicalPort);
        }

        // 添加主机以太网逻辑端口设置
        for (HmcProfileLheaModel lhea : lheas) {
            V8HostEthernetAdapterLogicalPort logicalPort = new V8HostEthernetAdapterLogicalPort();
            logicalPort.setAdapterID(lhea.getAdapterId().toString());
            logicalPort.setLogicalPortID(lhea.getLogicalPortId().toString());
            logicalPort.setPhysicalPortID(lhea.getPhysicalPortId().toString());
            logicalPort.setAllowedVLANIDs(StringUtils.isEmpty(lhea.getAllowVlanIds()) ? null : lhea.getAllowVlanIds());
            logicalPort.setAllowedOSMACAddresses(StringUtils.isEmpty(lhea.getAllowMacAddresses()) ? null : lhea.getAllowMacAddresses());
            logicalPort.setPortGroupID(lhea.getPortGroupId().toString());
            listLogicalPort.add(logicalPort);
        }
        return true;
    }
}
