package com.bocloud.cmp.provider.hmc.sdk.model;

import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;

import java.util.HashMap;
import java.util.Map;

/**
 * HMC的JOB相关常量
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class HmcJobConstant {

    public static final String JOB_COMPLETE = "COMPLETED_OK";
    public static final String JOB_COMPLETED_WITH_WARNINGS = "COMPLETED_WITH_WARNINGS";
    public static final String JOB_COMPLETED_WITH_ERROR = "COMPLETED_WITH_ERROR";
    public static final String JOB_FAILED_BEFORE_COMPLETION = "FAILED_BEFORE_COMPLETION";

    // 去指明这个job请求的xml文件是否包含xmlns:JobRequest命令空间
    private static final Map<String, Boolean> HAS_JOB_NAMESPACE_MAP = new HashMap<>();
    static {
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.EventLogger_ManagementConsole.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.CLIRunner_ManagementConsole.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.AddManagedSystem_ManagementConsole.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOn_ManagedSystem.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOff_ManagedSystem.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ResetConnection_ManagedSystem.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.RemoveConnection_ManagedSystem.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.QueryReservedMemoryRequiredForPartition_ManagedSystem.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ModifySRIOVAdapterMode_ManagedSystem.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ClearSRIOVPhysicalPortStatistics_ManagedSystem.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOff_LogicalPartition.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.Migrate_LogicalPartition.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOn_LogicalPartition.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.MigrateValidate_LogicalPartition.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.MigrateRecover_LogicalPartition.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.MigrateAbort_LogicalPartition.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOff_VirtualIOServer.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.PowerOn_VirtualIOServer.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ConfigDevice_VirtualIOServer.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.Create_Cluster.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.Delete_Cluster.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.CreateLogicalUnit_Cluster.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.LULinkedClone_Cluster.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_LogicalPartition.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_VirtualIOServer.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_LogicalPartition.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_VirtualIOServer.name(), true);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.CreatePowerEnterprisePool_PowerEnterprisePool.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.SetMasterConsole_PowerEnterprisePool.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.SyncSystemPool_PowerEnterprisePool.name(), false);
        HAS_JOB_NAMESPACE_MAP.put(HmcJobType.UpdatePowerEnterprisePool_PowerEnterprisePool.name(), false);
    }

    // 去指明这个job请求的xml文件是否包含xmlns:ns2命令空间
    private static final Map<String, Boolean> HAS_JOB_NS2_MAP = new HashMap<>();
    static {
        HAS_JOB_NS2_MAP.put(HmcJobType.EventLogger_ManagementConsole.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.CLIRunner_ManagementConsole.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.AddManagedSystem_ManagementConsole.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOn_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOff_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ResetConnection_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.RemoveConnection_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.QueryReservedMemoryRequiredForPartition_ManagedSystem.name(), false);
        HAS_JOB_NS2_MAP.put(HmcJobType.ModifySRIOVAdapterMode_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ClearSRIOVPhysicalPortStatistics_ManagedSystem.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOff_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.Migrate_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOn_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.MigrateValidate_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.MigrateRecover_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.MigrateAbort_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOff_VirtualIOServer.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.PowerOn_VirtualIOServer.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ConfigDevice_VirtualIOServer.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.Create_Cluster.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.Delete_Cluster.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.CreateLogicalUnit_Cluster.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.LULinkedClone_Cluster.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_VirtualIOServer.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_LogicalPartition.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_VirtualIOServer.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.CreatePowerEnterprisePool_PowerEnterprisePool.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.SetMasterConsole_PowerEnterprisePool.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.SyncSystemPool_PowerEnterprisePool.name(), true);
        HAS_JOB_NS2_MAP.put(HmcJobType.UpdatePowerEnterprisePool_PowerEnterprisePool.name(), true);
    }

    // 去指明这个job请求的URL地址
    private static final Map<String, String> JOB_REQUEST_RESOURCE = new HashMap<>();
    static {
        JOB_REQUEST_RESOURCE.put(HmcJobType.EventLogger_ManagementConsole.name(), "/rest/api/uom/ManagementConsole/%s/do/EventLogger");
        JOB_REQUEST_RESOURCE.put(HmcJobType.CLIRunner_ManagementConsole.name(), "/rest/api/uom/ManagementConsole/%s/do/CLIRunner");
        JOB_REQUEST_RESOURCE.put(HmcJobType.AddManagedSystem_ManagementConsole.name(), "/rest/api/uom/ManagementConsole/%s/do/AddManagedSystem");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOn_ManagedSystem.name(), "/rest/api/uom/ManagedSystem/%s/do/PowerOn");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOff_ManagedSystem.name(), "/rest/api/uom/ManagedSystem/%s/do/PowerOff");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ResetConnection_ManagedSystem.name(), "/rest/api/uom/ManagementConsole/%s/ManagedSystem/%s/do/ResetConnection");
        JOB_REQUEST_RESOURCE.put(HmcJobType.RemoveConnection_ManagedSystem.name(), "/rest/api/uom/ManagementConsole/%s/ManagedSystem/%s/do/RemoveConnection");
        JOB_REQUEST_RESOURCE.put(HmcJobType.QueryReservedMemoryRequiredForPartition_ManagedSystem.name(), "/rest/api/uom/ManagedSystem/%s/do/QueryReservedMemoryRequiredForPartition");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ModifySRIOVAdapterMode_ManagedSystem.name(), "/rest/api/uom/ManagedSystem/%s/do/ModifySRIOVAdapterMode");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ClearSRIOVPhysicalPortStatistics_ManagedSystem.name(), "/rest/api/uom/ManagedSystem/%s/do/ClearSRIOVPhysicalPortStatistics");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOff_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/PowerOff");
        JOB_REQUEST_RESOURCE.put(HmcJobType.Migrate_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/Migrate");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOn_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/PowerOn");
        JOB_REQUEST_RESOURCE.put(HmcJobType.MigrateValidate_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/MigrateValidate");
        JOB_REQUEST_RESOURCE.put(HmcJobType.MigrateRecover_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/MigrateRecover");
        JOB_REQUEST_RESOURCE.put(HmcJobType.MigrateAbort_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/do/MigrateAbort");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOff_VirtualIOServer.name(), "/rest/api/uom/VirtualIOServer/%s/do/PowerOff");
        JOB_REQUEST_RESOURCE.put(HmcJobType.PowerOn_VirtualIOServer.name(), "/rest/api/uom/VirtualIOServer/%s/do/PowerOn");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ConfigDevice_VirtualIOServer.name(), "/rest/api/uom/VirtualIOServer/%s/do/ConfigDevice");
        JOB_REQUEST_RESOURCE.put(HmcJobType.Create_Cluster.name(), "/rest/api/uom/Cluster/do/Create");
        JOB_REQUEST_RESOURCE.put(HmcJobType.Delete_Cluster.name(), "/rest/api/uom/Cluster/%s/do/Delete");
        JOB_REQUEST_RESOURCE.put(HmcJobType.CreateLogicalUnit_Cluster.name(), "/rest/api/uom/Cluster/%s/do/CreateLogicalUnit");
        JOB_REQUEST_RESOURCE.put(HmcJobType.LULinkedClone_Cluster.name(), "/rest/api/uom/Cluster/%s/do/LULinkedClone");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/SRIOVEthernetLogicalPort/%s/do/ClearStatistics");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ClearStatistics_SRIOVEthernetLogicalPort_VirtualIOServer.name(), "/rest/api/uom/VirtualIOServer/%s/SRIOVEthernetLogicalPort/%s/do/ClearStatistics");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_LogicalPartition.name(), "/rest/api/uom/LogicalPartition/%s/SRIOVFibreChannelOverEthernetLogicalPort/%s/do/ClearStatistics");
        JOB_REQUEST_RESOURCE.put(HmcJobType.ClearStatistics_SRIOVFibreChannelOverEthernetLogicalPort_VirtualIOServer.name(), "/rest/api/uom/VirtualIOServer/%s/SRIOVFibreChannelOverEthernetLogicalPort/%s/do/ClearStatistics");
        JOB_REQUEST_RESOURCE.put(HmcJobType.CreatePowerEnterprisePool_PowerEnterprisePool.name(), "/rest/api/uom/PowerEnterprisePool/do/CreatePowerEnterprisePool");
        JOB_REQUEST_RESOURCE.put(HmcJobType.SetMasterConsole_PowerEnterprisePool.name(), "/rest/api/uom/PowerEnterprisePool/do/SetMasterConsole");
        JOB_REQUEST_RESOURCE.put(HmcJobType.SyncSystemPool_PowerEnterprisePool.name(), "/rest/api/uom/PowerEnterprisePool/do/SyncSystemPool");
        JOB_REQUEST_RESOURCE.put(HmcJobType.UpdatePowerEnterprisePool_PowerEnterprisePool.name(), "/rest/api/uom/PowerEnterprisePool/do/UpdatePowerEnterprisePool");
    }

    public static Map<String, Boolean> getHasJobNamespaceMap() {
        return HAS_JOB_NAMESPACE_MAP;
    }

    public static Map<String, Boolean> getHasJobNs2Map() {
        return HAS_JOB_NS2_MAP;
    }

    public static Map<String, String> getJobRequestResource() {
        return JOB_REQUEST_RESOURCE;
    }
}
