package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcPartitionModel;
import com.bocloud.cmp.model.hmc.HmcProfileModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSCSIClientAdapterEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.PartitionProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualSCSIClientAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcOperationName;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.helper.v7.V7HmcDriverHelper;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobParameter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobParameters;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7RequestedOperation;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7DedicatedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartition;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionProfileForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PartitionMemoryConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PartitionProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7SharedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSCSIClientAdapterEntry;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class HmcPartitionProvider extends HmcAccessableProvider implements PartitionProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcPartitionProvider.class);

    public HmcPartitionProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult createLogicalPartition(String managedSystemUuid, HmcPartitionModel par) {
        V7LogicalPartition logicalPartition = new V7LogicalPartition();
        HmcProfileModel profileSetting = par.getHmcLparProfile();
        logicalPartition.setPartitionName(par.getName());
        logicalPartition.setPartitionType(par.getPartitionType());

        // CurrentProfileSync有三种情况，On 开启, Disabled 关闭, Suspended 挂起
        // 无论如何，先设定为关闭同步，避免后面修改概要文件的时候报错
        logicalPartition.setCurrentProfileSync("Disabled");

        // 概要文件是否要使用系统所有资源，需要分开处理
        if (profileSetting.getAssignAllResources() == null || !profileSetting.getAssignAllResources()) {
            // 处理器配置
            V7PartitionProcessorConfiguration processorConfiguration = new V7PartitionProcessorConfiguration();
            setPartitionProcessor(processorConfiguration, profileSetting);
            logicalPartition.setPartitionProcessorConfiguration(processorConfiguration);

            // 内存设置
            V7PartitionMemoryConfiguration memoryConfiguration = new V7PartitionMemoryConfiguration();
            memoryConfiguration.setMinimumMemory(profileSetting.getMinMemory().toString());
            memoryConfiguration.setDesiredMemory(profileSetting.getDesMemory().toString());
            memoryConfiguration.setMaximumMemory(profileSetting.getMaxMemory().toString());
            logicalPartition.setPartitionMemoryConfiguration(memoryConfiguration);

            // 更新IO设置
            V7PartitionIOConfiguration ioConfiguration = new V7PartitionIOConfiguration();
            ioConfiguration.setMaximumVirtualIOSlots(profileSetting.getMaxVirtualIoslots().toString());
            logicalPartition.setPartitionIOConfiguration(ioConfiguration);
        } else {
            return new BocloudResult(false, "当前版本HMC暂不支持创建分区时使用系统中所有资源");
        }

        // 概要文件设置
        V7LogicalPartitionProfileForUpdate updateProfile = new V7LogicalPartitionProfileForUpdate();
        setProfileForUpdate(updateProfile, profileSetting);

        BocloudResult result = V7LogicalPartitionService.create(getAuth(), managedSystemUuid, logicalPartition, updateProfile);
        if (result.isFailed()) {
            return result;
        }
        V7LogicalPartitionEntry responseV7 = (V7LogicalPartitionEntry) result.getData();

        if (result.isSuccess() && !"Disabled".equals(par.getCurrentProfileSync())) {
            // 如果需要开启同步或挂起，由于上面暂时处理成了关闭，所以这里要改回来
            logger.info("重新将逻辑分区[{}]设定为开启同步或挂起状态...", par.getName());
            V7LogicalPartitionForUpdate updateLpar = new V7LogicalPartitionForUpdate();
            updateLpar.setCurrentProfileSync(par.getCurrentProfileSync());
            result = V7LogicalPartitionService.update(getAuth(), managedSystemUuid, responseV7.getId(), updateLpar);
            if (result.isFailed()) {
                logger.error("重新将逻辑分区[{}]设定为开启同步或挂起状态失败：{}", par.getName(), result.getMessage());
                // 删除已经创建的分区
                V7LogicalPartitionService.delete(getAuth(), managedSystemUuid, responseV7.getId());
                return new BocloudResult(false, "创建逻辑分区出错：" + result.getMessage());
            }
        }

        responseV7 = (V7LogicalPartitionEntry) result.getData();
        LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult createVirtualIOServer(String managedSystemUuid, HmcPartitionModel par) {
        V7VirtualIOServer virtualIOServer = new V7VirtualIOServer();
        HmcProfileModel profileSetting = par.getHmcLparProfile();
        virtualIOServer.setPartitionName(par.getName());
        virtualIOServer.setPartitionType(par.getPartitionType());

        // CurrentProfileSync有三种情况，On 开启, Disabled 关闭, Suspended 挂起
        // 无论如何，先设定为关闭同步，避免后面修改概要文件的时候报错
        virtualIOServer.setCurrentProfileSync("Disabled");

        // 概要文件是否要使用系统所有资源，需要分开处理
        if (profileSetting.getAssignAllResources() == null || !profileSetting.getAssignAllResources()) {
            // 处理器配置
            V7PartitionProcessorConfiguration processorConfiguration = new V7PartitionProcessorConfiguration();
            setPartitionProcessor(processorConfiguration, profileSetting);
            virtualIOServer.setPartitionProcessorConfiguration(processorConfiguration);

            // 内存设置
            V7PartitionMemoryConfiguration memoryConfiguration = new V7PartitionMemoryConfiguration();
            memoryConfiguration.setMinimumMemory(profileSetting.getMinMemory().toString());
            memoryConfiguration.setDesiredMemory(profileSetting.getDesMemory().toString());
            memoryConfiguration.setMaximumMemory(profileSetting.getMaxMemory().toString());
            virtualIOServer.setPartitionMemoryConfiguration(memoryConfiguration);

            // 更新IO设置
            V7PartitionIOConfiguration ioConfiguration = new V7PartitionIOConfiguration();
            ioConfiguration.setMaximumVirtualIOSlots(profileSetting.getMaxVirtualIoslots().toString());
            virtualIOServer.setPartitionIOConfiguration(ioConfiguration);
        } else {
            return new BocloudResult(false, "当前版本HMC暂不支持创建分区时使用系统中所有资源");
        }

        // 概要文件设置
        V7LogicalPartitionProfileForUpdate updateProfile = new V7LogicalPartitionProfileForUpdate();
        setProfileForUpdate(updateProfile, profileSetting);

        BocloudResult result = V7VirtualIOServerService.create(getAuth(), managedSystemUuid, virtualIOServer, updateProfile);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create VIOS {}", par.getName());
        V7VirtualIOServerEntry responseV7 = (V7VirtualIOServerEntry) result.getData();

        if (result.isSuccess() && !"Disabled".equals(par.getCurrentProfileSync())) {
            // 如果需要开启同步或挂起，由于上面暂时处理成了关闭，所以这里要改回来
            logger.info("重新将VIOS[{}]设定为开启同步或挂起状态...", par.getName());
            V7VirtualIOServerForUpdate updateVios = new V7VirtualIOServerForUpdate();
            updateVios.setCurrentProfileSync(par.getCurrentProfileSync());
            result = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, responseV7.getId(), updateVios);
            if (result.isFailed()) {
                logger.error("重新将VIOS[{}]设定为开启同步或挂起状态失败：{}", par.getName(), result.getMessage());
                // 删除已经创建的VIOS
                V7LogicalPartitionService.delete(getAuth(), managedSystemUuid, responseV7.getId());
                return new BocloudResult(false, "创建VIOS出错：" + result.getMessage());
            }
        }

        responseV7 = (V7VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    private void setPartitionProcessor(V7PartitionProcessorConfiguration processorConfiguration, HmcProfileModel profileSetting) {
        if (profileSetting.getSharedProcessor() != null && profileSetting.getSharedProcessor()) {
            V7SharedProcessorConfiguration sharedConfiguration = new V7SharedProcessorConfiguration();
            processorConfiguration.setHasDedicatedProcessors("false");
            sharedConfiguration.setMinimumProcessingUnits(profileSetting.getMinProUnits().toString());
            sharedConfiguration.setDesiredProcessingUnits(profileSetting.getDesProUnits().toString());
            sharedConfiguration.setMaximumProcessingUnits(profileSetting.getMaxProUnits().toString());
            sharedConfiguration.setSharedProcessorPoolID(profileSetting.getSharedProPoolId().toString());
            sharedConfiguration.setMinimumVirtualProcessors(profileSetting.getMinProVirt().toString());
            sharedConfiguration.setDesiredVirtualProcessors(profileSetting.getDesProVirt().toString());
            sharedConfiguration.setMaximumVirtualProcessors(profileSetting.getMaxProVirt().toString());
            sharedConfiguration.setSharedProcessorPoolID(profileSetting.getSharedProPoolId().toString());
            if ("uncapped".equals(profileSetting.getProSharingMode()) && null != profileSetting.getUncappedWeight()) {
                sharedConfiguration.setUncappedWeight(profileSetting.getUncappedWeight().toString());
            }
            processorConfiguration.setSharedProcessorConfiguration(sharedConfiguration);
            processorConfiguration.setSharingMode(profileSetting.getProSharingMode());
        } else {
            V7DedicatedProcessorConfiguration dedicatedConfiguration = new V7DedicatedProcessorConfiguration();
            processorConfiguration.setHasDedicatedProcessors("true");
            dedicatedConfiguration.setMinimumProcessors(profileSetting.getMinPro().toString());
            dedicatedConfiguration.setDesiredProcessors(profileSetting.getDesPro().toString());
            dedicatedConfiguration.setMaximumProcessors(profileSetting.getMaxPro().toString());
            processorConfiguration.setDedicatedProcessorConfiguration(dedicatedConfiguration);
            processorConfiguration.setSharingMode("sre idle proces");
        }
    }

    private void setProfileForUpdate(V7LogicalPartitionProfileForUpdate updateProfile, HmcProfileModel profileSetting) {
        updateProfile.setProfileName(profileSetting.getName());
        updateProfile.setAssignAllResources(profileSetting.getAssignAllResources() == null ? null : profileSetting.getAssignAllResources().toString());
        updateProfile.setConnectionMonitoringEnabled(profileSetting.getConnMonitor() == null ? null : profileSetting.getConnMonitor().toString());
        updateProfile.setAutoStart(profileSetting.getAutoStart() == null ? null : profileSetting.getAutoStart().toString());
        updateProfile.setRedundantErrorPathReportingEnabled(profileSetting.getRedundantErrorPathReport() == null ? null : profileSetting.getRedundantErrorPathReport().toString());
        updateProfile.setBootMode(profileSetting.getBootMode());

        // 概要文件虚拟适配器设置
        V7IOConfigurationInstance ioConfig = new V7IOConfigurationInstance();
        if (V7HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profileSetting.getMaxVirtualIoslots(),
                profileSetting.getHmcProfileNetworkAdapters(), profileSetting.getHmcProfileFibreAdapters(),
                profileSetting.getHmcProfileScsiAdapters(), profileSetting.getHmcProfileIoslot())) {
            updateProfile.setIoConfigurationInstance(ioConfig);
        }

        // 主机以太网逻辑端口设置
        V7HostEthernetAdapterLogicalPorts logicalPorts = new V7HostEthernetAdapterLogicalPorts();
        if (null != profileSetting.getHmcProfileLheas()) {
            V7HmcDriverHelper.updateProfileHEALogicalPort(logicalPorts, profileSetting.getHmcProfileLheas());
            updateProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
        }
    }

    /**
     * Hmc分区删除
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult delete(String managedSystemUuid, String partitionUuid, Boolean isVios) {
        BocloudResult result;
        if (isVios) {
            result = V7VirtualIOServerService.delete(getAuth(), managedSystemUuid, partitionUuid);
            logger.info("Hmc driver completely delete VIOS partition");
        } else {
            result = V7LogicalPartitionService.delete(getAuth(), managedSystemUuid, partitionUuid);
            logger.info("Hmc driver completely delete logical partition");
        }
        return result;
    }

    /**
     * Hmc逻辑分区编辑
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult updateLogicalPartition(HmcPartitionModel partition, String managedSystemUuid, String partitionUuid) {
        V7LogicalPartitionForUpdate updatePar = new V7LogicalPartitionForUpdate();
        if (null != partition.getCurrentProfileSync()) {
            updatePar.setCurrentProfileSync(partition.getCurrentProfileSync());
        }
        if (null != partition.getName()) {
            updatePar.setPartitionName(partition.getName());
        }
        if (null != partition.getKeyLockPosition()) {
            updatePar.setKeylockPosition(partition.getKeyLockPosition());
        }
        // 更新允许收集性能信息设置
        if (null != partition.getAllowPerformanceCollection()) {
            updatePar.setAllowPerformanceDataCollection(partition.getAllowPerformanceCollection().toString());
        }

        BocloudResult result = V7LogicalPartitionService.update(getAuth(), managedSystemUuid, partitionUuid, updatePar);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update partition");

        V7LogicalPartitionEntry responseV7 = (V7LogicalPartitionEntry) result.getData();
        LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * Hmc VIOS分区编辑
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult updateVirtualIOServer(HmcPartitionModel partition, String managedSystemUuid, String partitionUuid) {
        V7VirtualIOServerForUpdate updatePar = new V7VirtualIOServerForUpdate();
        if (null != partition.getCurrentProfileSync()) {
            updatePar.setCurrentProfileSync(partition.getCurrentProfileSync());
        }
        if (null != partition.getName()) {
            updatePar.setPartitionName(partition.getName());
        }
        if (null != partition.getKeyLockPosition()) {
            updatePar.setKeylockPosition(partition.getKeyLockPosition());
        }
        // 更新允许收集性能信息设置
        if (null != partition.getAllowPerformanceCollection()) {
            updatePar.setAllowPerformanceDataCollection(partition.getAllowPerformanceCollection().toString());
        }

        BocloudResult result = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, partitionUuid, updatePar);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update VIOS partition");
        V7VirtualIOServerEntry responseV7 = (V7VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult getPartitionAttribute(String partitionUuid, Boolean isVios) {
        BocloudResult result;
        if (isVios) {
            result = V7VirtualIOServerService.get(getAuth(), partitionUuid);
            if (result.isFailed()) {
                return result;
            }
            V7VirtualIOServerEntry responseV7 = (V7VirtualIOServerEntry) result.getData();
            VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualIOServerEntry.class);
            return new BocloudResult(true, response, "执行成功");
        } else {
            result = V7LogicalPartitionService.get(getAuth(), partitionUuid);
            if (result.isFailed()) {
                return result;
            }
            V7LogicalPartitionEntry responseV7 = (V7LogicalPartitionEntry) result.getData();
            LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionEntry.class);
            return new BocloudResult(true, response, "执行成功");
        }
    }

    @Override
    public BocloudResult powerOffPartition(String uuid, Boolean isVios) {
        V7JobRequest jobRequest = generatePowerOffJobRequest(false);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V7VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V7LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_LogicalPartition);
        }
    }

    @Override
    public BocloudResult restartPartition(String uuid, Boolean isVios) {
        V7JobRequest jobRequest = generatePowerOffJobRequest(true);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V7VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V7LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_LogicalPartition);
        }
    }

    private V7JobRequest generatePowerOffJobRequest(Boolean restart) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOff.name());
        requestedOperation.setProgressType("DISCRETE");
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameter jobParameter;

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("immediate");
        jobParameter.setParameterValue("true");
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("restart");
        jobParameter.setParameterValue(restart.toString());
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("operation");
        jobParameter.setParameterValue("shutdown");
        listJobParameters.add(jobParameter);

        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return jobRequest;
    }

    @Override
    public BocloudResult powerOnPartition(String uuid, String profileUuid, Boolean isVios) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOn.name());
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameter jobParameter;

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("force");
        jobParameter.setParameterValue("false");
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("LogicalPartitionProfile");
        jobParameter.setParameterValue(profileUuid);
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("novsi");
        jobParameter.setParameterValue("true");
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("bootmode");
        jobParameter.setParameterValue("norm");
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("keylock");
        jobParameter.setParameterValue("norm");
        listJobParameters.add(jobParameter);

        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V7VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOn_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V7LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOn_LogicalPartition);
        }
    }

    @Override
    public BocloudResult queryPartitionState(String uuid, Boolean isVios) {
        if (isVios) {
            return V7VirtualIOServerService.getQuickProperty(getAuth(), uuid, VirtualIOServerQuickProperty.PartitionState);
        } else {
            return V7LogicalPartitionService.getQuickProperty(getAuth(), uuid, LogicalPartitionQuickProperty.PartitionState);
        }
    }

    @Override
    public BocloudResult queryPartitionRMCState(String uuid, Boolean isVios) {
        BocloudResult bocloudResult;
        if (isVios) {
            bocloudResult = V7VirtualIOServerService.getQuickProperty(getAuth(), uuid, VirtualIOServerQuickProperty.RMCState);
        } else {
            bocloudResult = V7LogicalPartitionService.getQuickProperty(getAuth(), uuid, LogicalPartitionQuickProperty.RMCState);
        }
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        String rmcStat = (String) bocloudResult.getData();
        rmcStat = Optional.ofNullable(rmcStat).map(String::toLowerCase).orElse(null);
        return new BocloudResult(true, rmcStat, "执行成功");
    }

    @Override
    public BocloudResult getVirtualScsiAdapters(String href) {
        BocloudResult result = V7VirtualSCSIClientAdapterService.getByHref(getAuth(), href);
        if (result.isFailed()) {
            return result;
        }
        V7VirtualSCSIClientAdapterEntry responseV7 = (V7VirtualSCSIClientAdapterEntry) result.getData();
        VirtualSCSIClientAdapterEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualSCSIClientAdapterEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
