package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleEntry;
import com.bocloud.cmp.model.hmc.sdk.common.ManagementConsoleResponse;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystem;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemEntry;
import com.bocloud.cmp.model.hmc.sdk.module.ManagedSystemResponse;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.ServerProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7ManagedSystemService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7ManagementConsoleService;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcOperationName;
import com.bocloud.cmp.provider.hmc.sdk.enums.ManagedSystemQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobParameter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobParameters;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.job.V7RequestedOperation;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystem;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ManagedSystemResponse;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class HmcServerProvider extends HmcAccessableProvider implements ServerProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcServerProvider.class);

    private static final String PCM_ERROR_ID_4002 = "4002";
    private static final String HMC_PCM_TIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ssZ";
    private static final String UI_TIME_FORMATE = "yyyy-MM-dd HH:mm:ss";

    public HmcServerProvider(Butler butler) {
        super(butler);
    }
    
    /**
     * 获取所有物理服务器的信息
     * 
     * @author sxp
     * @since Nov 20, 2018
     */
    @Override
    public BocloudResult get() {
        BocloudResult result = V7ManagedSystemService.get(this.getAuth());
        ManagedSystemResponse response = JSON.parseObject(JSON.toJSONString(result.getData()), ManagedSystemResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult update(String managedSystemUuid, ManagedSystem managedSystem) {
        V7ManagedSystem v7ManagedSystem = JSON.parseObject(JSON.toJSONString(managedSystem), V7ManagedSystem.class);
        BocloudResult result = V7ManagedSystemService.update(this.getAuth(), managedSystemUuid, v7ManagedSystem);
        ManagedSystemEntry response = JSON.parseObject(JSON.toJSONString(result.getData()), ManagedSystemEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * HMC服务器开机
     * 
     * @author sxp
     * @since Jun 12, 2018
     */
    @Override
    public BocloudResult powerOnManagedSystem(String managedSystemUuid) {
        return V7ManagedSystemService.powerOnManagedSystem(this.getAuth(), managedSystemUuid);
    }

    /**
     * HMC服务器关机
     * 
     * @author sxp
     * @since Jun 12, 2018
     */
    @Override
    public BocloudResult powerOffManagedSystem(String managedSystemUuid) {
        return V7ManagedSystemService.powerOffManagedSystem(this.getAuth(), managedSystemUuid);
    }

    /**
     * HMC服务器重置连接
     */
    @Override
    public BocloudResult resetConnection(String managedSystemUuid) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.ResetConnection.name());
        requestedOperation.setGroupName(HmcGroupName.ManagedSystem.name());
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return V7ManagedSystemService.doJobRequest(getAuth(), managedSystemUuid, jobRequest, HmcJobType.ResetConnection_ManagedSystem);
    }

    /**
     * HMC服务器删除连接
     */
    @Override
    public BocloudResult removeConnection(String managedSystemUuid) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.RemoveConnection.name());
        requestedOperation.setGroupName(HmcGroupName.ManagedSystem.name());
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return V7ManagedSystemService.doJobRequest(getAuth(), managedSystemUuid, jobRequest, HmcJobType.RemoveConnection_ManagedSystem);
    }

    /**
     * HMC服务器删除连接
     */
    @Override
    public BocloudResult powerOn(String managedSystemUuid) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOn.name());
        requestedOperation.setGroupName(HmcGroupName.ManagedSystem.name());
        requestedOperation.setProgressType("LINEAR");
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameter jobParameter;

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("operation");
        jobParameter.setParameterValue("on");
        listJobParameters.add(jobParameter);

        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return V7ManagedSystemService.doJobRequest(getAuth(), managedSystemUuid, jobRequest, HmcJobType.PowerOn_ManagedSystem);
    }

    /**
     * HMC服务器删除连接
     */
    @Override
    public BocloudResult powerOff(String managedSystemUuid) {
        V7JobRequest jobRequest = new V7JobRequest();
        V7RequestedOperation requestedOperation = new V7RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOff.name());
        requestedOperation.setGroupName(HmcGroupName.ManagedSystem.name());
        requestedOperation.setProgressType("LINEAR");
        jobRequest.setRequestedOperation(requestedOperation);

        List<V7JobParameter> listJobParameters = new ArrayList<>();
        V7JobParameter jobParameter;
        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("immediate");
        jobParameter.setParameterValue("true");
        listJobParameters.add(jobParameter);

        jobParameter = new V7JobParameter();
        jobParameter.setParameterName("restart");
        jobParameter.setParameterValue("false");
        listJobParameters.add(jobParameter);

        V7JobParameters jobParmeters = new V7JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return V7ManagedSystemService.doJobRequest(getAuth(), managedSystemUuid, jobRequest, HmcJobType.PowerOff_ManagedSystem);
    }
    
    /**
     * 查询HMC服务器的状态
     * 
     * @param managedSystemUuid
     *            HMC服务器的uuid
     * @author sxp
     * @since Jun 14, 2018
     */
    @Override
    public BocloudResult queryManagedSystemState(String managedSystemUuid) {
        return V7ManagedSystemService.getStateByUuid(this.getAuth(), managedSystemUuid);
    }

    /**
     * 查询HMC服务器的属性
     * 
     * @param managedSystemUuid
     *            HMC服务器的uuid
     */
    @Override
    public BocloudResult queryManagedSystemAttribute(String managedSystemUuid) {
        BocloudResult result = V7ManagedSystemService.getByUuid(this.getAuth(), managedSystemUuid);
        ManagedSystemEntry response = JSON.parseObject(JSON.toJSONString(result.getData()), ManagedSystemEntry.class);
        return new BocloudResult(true, response, "执行成功");

    }

    /**
     * 获取当前HMC平台的控制台唯一标识
     * 
     * @author sxp
     * @since Jun 19, 2018
     */
    @Override
    public BocloudResult queryManagementConsoleUuid() {
        BocloudResult result = V7ManagementConsoleService.get(this.getAuth());
        if (result.isFailed()) {
            return result;
        }

        ManagementConsoleResponse response = (ManagementConsoleResponse) result.getData();
        List<ManagementConsoleEntry> entries = response.getEntries();

        ManagementConsoleEntry entry;
        if (entries == null || entries.isEmpty() || (entry = entries.get(0)) == null) {
            return new BocloudResult(false, "系统中未获取到HMC控制台信息");
        }

        String uuid = entry.getId();
        if (StringUtils.isEmpty(uuid)) {
            return new BocloudResult(false, "系统中未获取到HMC控制台唯一标识");
        }

        return new BocloudResult(true, uuid, "success");
    }

    /**
     * 增加受管系统(ManagedSystem)到HMC平台
     * 
     * @author sxp
     * @since Jun 19, 2018
     */
    @Override
    public BocloudResult addManagedSystem(String consoleUuid, String hostIp, String password) {
        return V7ManagementConsoleService.addManagedSystem(this.getAuth(), consoleUuid, hostIp, password);
    }

    /**
     * 从HMC平台删除受管系统(ManagedSystem)的连接
     * 
     * @author sxp
     * @since Jun 19, 2018
     */
    @Override
    public BocloudResult removeManagedSystemConnection(String consoleUuid, String managedSystemUuid) {
        return V7ManagementConsoleService.removeManagedSystemConnection(this.getAuth(), consoleUuid, managedSystemUuid);
    }
}
