package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcVirtualFibreChannelMappingModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionVfcResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VirtualFibreChannelMappingProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7LogicalPartitionVfcResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7Port;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ServerAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualFibreChannelMapping;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualFibreChannelMappings;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerForUpdate;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.BooleanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author luyanchao
 * @since 2018/12/18 16:53
 */
public class HmcVirtualFibreChannelMappingProvider extends HmcAccessableProvider implements VirtualFibreChannelMappingProvider {

    private static Logger logger = LoggerFactory.getLogger(HmcVirtualFibreChannelMappingProvider.class);

    public HmcVirtualFibreChannelMappingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult addVfcToVios(String managedSystemUuid, String viosUuid, HmcVirtualFibreChannelMappingModel mapping) {
        // 获取现有的vios数据
        BocloudResult getOldResult = V7VirtualIOServerService.get(getAuth(), viosUuid);
        if (getOldResult.isFailed()) {
            return getOldResult;
        }
        V7VirtualIOServer oldVios = ((V7VirtualIOServerEntry) getOldResult.getData()).getContent().getVirtualIOServer();

        // 初始化用来更新的vios数据
        V7VirtualIOServerForUpdate updateVios = new V7VirtualIOServerForUpdate();
        updateVios.setVirtualFibreChannelMappings(new V7VirtualFibreChannelMappings());
        updateVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());

        // 将现有的vfcMapping添加进来
        if (oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList() != null) {
            for (V7VirtualFibreChannelMapping oldMapping : oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList()) {
                oldMapping.getServerAdapter().setPhysicalPort(null);
                updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(oldMapping);
            }
        } else {
            oldVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());
        }

        // 添加新的vfcMapping
        V7VirtualFibreChannelMapping newMapping = new V7VirtualFibreChannelMapping();

        V7Port port = new V7Port();
        port.setUniqueDeviceID(mapping.getPhysicalUniqueDeviceId());
        port.setPortName(mapping.getFcName());
        newMapping.setPort(port);

        V7ClientAdapter clientAdapter = new V7ClientAdapter();
        clientAdapter.setAdapterType("Client");
        clientAdapter.setLocalPartitionId(mapping.getClientPartitionId().toString());
        clientAdapter.setVirtualSlotNumber(mapping.getClientPartitionSlot() == null ? null : mapping.getClientPartitionSlot().toString());
        clientAdapter.setConnectingPartitionID(mapping.getServerViosId().toString());
        clientAdapter.setConnectingVirtualSlotNumber(mapping.getServerViosSlot() == null ? null : mapping.getServerViosSlot().toString());
        clientAdapter.setVariedOn(BooleanUtils.toString(mapping.getClientViredOn(), "true", "false", "false"));
        clientAdapter.setRequiredAdapter(BooleanUtils.toString(mapping.getClientRequired(), "true", "false", "false"));
        newMapping.setClientAdapter(clientAdapter);

        V7ServerAdapter serverAdapter = new V7ServerAdapter();
        serverAdapter.setAdapterType("Server");
        serverAdapter.setLocalPartitionId(mapping.getServerViosId().toString());
        serverAdapter.setVirtualSlotNumber(mapping.getServerViosSlot() == null ? null : mapping.getServerViosSlot().toString());
        serverAdapter.setConnectingPartitionID(mapping.getClientPartitionId().toString());
        serverAdapter.setConnectingVirtualSlotNumber(mapping.getClientPartitionSlot() == null ? null : mapping.getClientPartitionSlot().toString());
        serverAdapter.setVariedOn(BooleanUtils.toString(mapping.getServerViredOn(), "true", "false", "false"));
        serverAdapter.setRequiredAdapter(BooleanUtils.toString(mapping.getServerRequired(), "true", "false", "false"));
        serverAdapter.setMapPort(mapping.getFcName());
        newMapping.setServerAdapter(serverAdapter);

        updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(newMapping);

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V7PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V7ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V7ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V7ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V7ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V7AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 保存vios
        BocloudResult bocloudResult = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V7VirtualIOServerEntry serverEntryV7 = (V7VirtualIOServerEntry) bocloudResult.getData();
        VirtualIOServerEntry serverEntry = JSON.parseObject(JSON.toJSONString(serverEntryV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, serverEntry, "执行成功");
    }

    @Override
    public BocloudResult removeVfcFromVios(String managedSystemUuid, String viosUuid, HmcVirtualFibreChannelMappingModel mapping) {
        // 获取现有的vios数据
        BocloudResult getOldResult = V7VirtualIOServerService.get(getAuth(), viosUuid);
        if (getOldResult.isFailed()) {
            return getOldResult;
        }
        V7VirtualIOServer oldVios = ((V7VirtualIOServerEntry) getOldResult.getData()).getContent().getVirtualIOServer();

        // 初始化用来更新的vios数据
        V7VirtualIOServerForUpdate updateVios = new V7VirtualIOServerForUpdate();
        updateVios.setVirtualFibreChannelMappings(new V7VirtualFibreChannelMappings());
        updateVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());

        // 将现有的vfcMapping添加进来，除了需要删除的那个
        if (oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList() != null) {
            for (V7VirtualFibreChannelMapping oldMapping : oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList()) {
                if (oldMapping.getServerAdapter().getLocalPartitionId().equals(mapping.getServerViosId().toString()) &&
                        oldMapping.getServerAdapter().getVirtualSlotNumber().equals(mapping.getServerViosSlot().toString())) {
                    continue;
                }
                oldMapping.getServerAdapter().setPhysicalPort(null);
                updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(oldMapping);
            }
        }

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V7PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V7ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V7ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V7ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V7ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V7AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 保存vios
        BocloudResult bocloudResult = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V7VirtualIOServerEntry serverEntryV7 = (V7VirtualIOServerEntry) bocloudResult.getData();
        VirtualIOServerEntry serverEntry = JSON.parseObject(JSON.toJSONString(serverEntryV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, serverEntry, "执行成功");
    }

    @Override
    public BocloudResult getVfcFromPartition(String partitionUuid) {
        BocloudResult bocloudResult = V7LogicalPartitionService.getVfc(getAuth(), partitionUuid);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V7LogicalPartitionVfcResponse responseV7 = (V7LogicalPartitionVfcResponse) bocloudResult.getData();
        LogicalPartitionVfcResponse response = JSON.parseObject(JSON.toJSONString(responseV7), LogicalPartitionVfcResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }

}
