package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcVirtualSwitchModel;
import com.bocloud.cmp.model.hmc.sdk.module.ClientNetworkAdapterResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSwitchEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VirtualNetworkProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7ClientNetworkAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualSwitchService;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ClientNetworkAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSwitch;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSwitchEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSwitchForUpdate;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HmcVirtualNetworkProvider extends HmcAccessableProvider implements VirtualNetworkProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcVirtualNetworkProvider.class);

    public HmcVirtualNetworkProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult createVirtualSwitch(HmcVirtualSwitchModel virtualSwitch, String serverUuid) {
        V7VirtualSwitch vswitch = new V7VirtualSwitch();
        vswitch.setSwitchName(virtualSwitch.getName());
        BocloudResult result = V7VirtualSwitchService.create(getAuth(), serverUuid, vswitch);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create virtual switch");
        V7VirtualSwitchEntry responseV7 = (V7VirtualSwitchEntry) result.getData();
        VirtualSwitchEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualSwitchEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult deleteVirtualSwitch(String serverUuid, String vswitchUuid) {
        BocloudResult result = V7VirtualSwitchService.delete(getAuth(), serverUuid, vswitchUuid);
        logger.info("Hmc driver completely delete virtual switch");
        return result;
    }

    @Override
    public BocloudResult updateVirtualSwitch(HmcVirtualSwitchModel virtualSwitch, String serverUuid, String vswitchUuid) {
        V7VirtualSwitchForUpdate virtualSwitchForUpdate = new V7VirtualSwitchForUpdate();
        if (null != virtualSwitch) {
            virtualSwitchForUpdate.setSwitchName(virtualSwitch.getName());
        }
        BocloudResult result = V7VirtualSwitchService.update(getAuth(), serverUuid, vswitchUuid, virtualSwitchForUpdate);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update virtual switch");
        V7VirtualSwitchEntry responseV7 = (V7VirtualSwitchEntry) result.getData();
        VirtualSwitchEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualSwitchEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public  BocloudResult getClientNetworkAdapter(String laprUuid) {
        BocloudResult result = V7ClientNetworkAdapterService.get(getAuth(), laprUuid);
        if (result.isFailed()) {
            return result;
        }
        V7ClientNetworkAdapterResponse responseV7 = (V7ClientNetworkAdapterResponse) result.getData();
        ClientNetworkAdapterResponse response = JSON.parseObject(JSON.toJSONString(responseV7), ClientNetworkAdapterResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
