package com.bocloud.cmp.provider.hmc.v7;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroup;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupResponse;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VolumeGroupProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v7.V7VolumeGroupService;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7PhysicalVolume;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7ServerAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7Storage;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualDisk;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualDisks;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualIOServerForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSCSIMapping;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VirtualSCSIMappings;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroup;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v7.sdk.module.V7VolumeGroupResponse;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * hmc volume group driver
 *
 * @author lkx007
 */
public class HmcVolumeGroupProvider extends HmcAccessableProvider implements VolumeGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(HmcVolumeGroupProvider.class);

    public HmcVolumeGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult getVolumeGroup(String viosUuid) {
        if (viosUuid == null) {
            return new BocloudResult(false, "参数异常");
        }
        BocloudResult getResult = V7VolumeGroupService.get(getAuth(), viosUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }
        V7VolumeGroupResponse responseV7 = (V7VolumeGroupResponse) getResult.getData();
        VolumeGroupResponse response = JSON.parseObject(JSON.toJSONString(responseV7), VolumeGroupResponse.class);
        return new BocloudResult(true, response, "查询虚拟磁盘成功");

    }

    @Override
    public BocloudResult getVolumeGroup(String viosUuid, String volumeGroupUuid) {
        if (viosUuid == null || volumeGroupUuid == null) {
            return new BocloudResult(false, "参数异常");
        }
        BocloudResult getResult = V7VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get VolumeGroupEntry failed:{}", getResult);
            return getResult;
        }
        V7VolumeGroupEntry responseV7 = (V7VolumeGroupEntry) getResult.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "查询虚拟磁盘成功");
    }

    @Override
    public BocloudResult createVirtualDisk(String viosUuid, String volumeGroupUuid, Double diskCapacity, String diskName) {
        BocloudResult getResult = V7VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V7VolumeGroupEntry entry = (V7VolumeGroupEntry) getResult.getData();
        Optional<V7VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V7VolumeGroupEntry::getContent)
                .map(V7VolumeGroupContent::getVolumeGroup)
                .map(V7VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V7VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V7VirtualDisk> disks = virtualDisksOpt.map(V7VirtualDisks::getDisks).orElse(new ArrayList<>());
        if (disks.stream().anyMatch(disk -> Objects.equals(disk.getDiskName(), diskName))) {
            logger.error("virtual disk {} exists", diskName);
            return new BocloudResult(false, "虚拟磁盘[" + diskName + "]已经存在");
        }
        V7VirtualDisk disk = new V7VirtualDisk();
        disk.setDiskCapacity(diskCapacity.toString());
        disk.setDiskName(diskName);
        disks.add(disk);
        virtualDisks.setDisks(disks);

        V7VolumeGroupForUpdate updateBean = new V7VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);
        BocloudResult result = V7VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create virtual disk {}, result: {}", diskName, result.isSuccess());
        V7VolumeGroupEntry responseV7 = (V7VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult updateVirtualDisk(String viosUuid, String volumeGroupUuid, Double diskCapacity, String diskName) {
        BocloudResult getResult = V7VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V7VolumeGroupEntry entry = (V7VolumeGroupEntry) getResult.getData();
        Optional<V7VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V7VolumeGroupEntry::getContent)
                .map(V7VolumeGroupContent::getVolumeGroup)
                .map(V7VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V7VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V7VirtualDisk> disks = virtualDisksOpt.map(V7VirtualDisks::getDisks).orElse(new ArrayList<>());
        Optional<V7VirtualDisk> diskOpt = disks.stream().filter(disk -> Objects.equals(disk.getDiskName(), diskName)).findFirst();
        if (!diskOpt.isPresent()) {
            logger.error("virtual disk {} not exists", diskName);
            return new BocloudResult(false, "虚拟磁盘[" + diskName + "]不存在");
        }
        diskOpt.get().setDiskCapacity(diskCapacity.toString());
        virtualDisks.setDisks(disks);

        V7VolumeGroupForUpdate updateBean = new V7VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);
        BocloudResult result = V7VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update virtual disk {}, result: {}", diskName, result.isSuccess());
        V7VolumeGroupEntry responseV7 = (V7VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult deleteVirtualDisk(String viosUuid, String volumeGroupUuid, String diskName) {
        BocloudResult getResult = V7VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V7VolumeGroupEntry entry = (V7VolumeGroupEntry) getResult.getData();
        Optional<V7VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V7VolumeGroupEntry::getContent)
                .map(V7VolumeGroupContent::getVolumeGroup)
                .map(V7VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V7VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V7VirtualDisk> disks = virtualDisksOpt.map(V7VirtualDisks::getDisks).orElse(new ArrayList<>());
        disks.removeIf(oldDisk -> diskName.equals(oldDisk.getDiskName()));
        virtualDisks.setDisks(disks);

        V7VolumeGroupForUpdate updateBean = new V7VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);

        BocloudResult result = V7VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely delete virtual disk {}, result: {}", diskName, result.isSuccess());
        V7VolumeGroupEntry responseV7 = (V7VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult assign(String managedSystemUuid, String viosUuid, String lparUuid, String diskName, String diskCapacity, Integer viosId, Integer lparId, Integer lparSlot, boolean isPhysical) {
        if (diskName == null || diskCapacity == null) {
            logger.error("disk info {} , {} error.", diskName, diskCapacity);
            return new BocloudResult(false, "分配失败，磁盘信息不正确");
        }

        BocloudResult result = V7VirtualIOServerService.get(getAuth(), viosUuid);
        if (result.isFailed()) {
            logger.error("get vios info error:{}", result);
            return result;
        }
        V7VirtualIOServerEntry virtualIOServerEntry = (V7VirtualIOServerEntry) result.getData();
        V7VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V7VirtualIOServerForUpdate updateVios = new V7VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V7PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V7ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V7ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V7ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V7ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V7AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V7VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V7VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V7VirtualSCSIMapping> newMappingList = new ArrayList<>();
        if (virtualScsiMappingList == null) {
            virtualScsiMappingList = new ArrayList<>();
        }

        V7VirtualSCSIMapping virtualScsiMapping = null;
        for (V7VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if (null != oldMap.getStorage() && null != oldMap.getStorage().getVirtualDisk() && diskName.equals(oldMap.getStorage().getVirtualDisk().getDiskName())) {
                // for virtual disk
                virtualScsiMapping = oldMap;
            } else if (null != oldMap.getStorage() && null != oldMap.getStorage().getPhysicalVolume() && diskName.equals(oldMap.getStorage().getPhysicalVolume().getVolumeName())) {
                // for physical disk
                virtualScsiMapping = oldMap;
            } else {
                newMappingList.add(oldMap);
            }
        }

        if (virtualScsiMapping == null) {
            // 不存在则新增
            V7Storage storage = new V7Storage();
            storage.setSchemaVersion(null);
            if (isPhysical) {
                storage.setPhysicalVolume(new V7PhysicalVolume(diskCapacity, diskName));
            } else {
                storage.setVirtualDisk(new V7VirtualDisk(diskCapacity, diskName));
            }
            V7ServerAdapter serverAdapter = new V7ServerAdapter(viosId.toString(), lparId == null ? null : lparId.toString(), "true");
            if (lparId != null) {
                V7ClientAdapter clientAdapter = new V7ClientAdapter(lparId.toString(), viosId.toString(), "true");
                clientAdapter.setVariedOn("true");
                if (lparSlot != null) {
                    clientAdapter.setVirtualSlotNumber(lparSlot.toString());
                    serverAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                virtualScsiMapping = new V7VirtualSCSIMapping(clientAdapter, serverAdapter, storage);
            } else {
                virtualScsiMapping = new V7VirtualSCSIMapping(serverAdapter, storage);
            }
        } else {
            // 已存在则更新分配信息
            if (null != virtualScsiMapping.getServerAdapter()) {
                V7ServerAdapter oldServerAdapter = virtualScsiMapping.getServerAdapter();
                oldServerAdapter.setRemoteLogicalPartitionId(lparId.toString());
                if (lparSlot != null) {
                    oldServerAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                oldServerAdapter.setVirtualSlotNumber(null);// 这句话必须的，不能删除
                virtualScsiMapping.setServerAdapter(oldServerAdapter);
            }
            V7ClientAdapter clientAdapter = new V7ClientAdapter(lparId.toString(), viosId.toString(), "true");
            clientAdapter.setVariedOn("true");
            if (lparSlot != null) {
                clientAdapter.setVirtualSlotNumber(lparSlot.toString());
            }
            virtualScsiMapping.setClientAdapter(clientAdapter);
            virtualScsiMapping.setAssociatedLogicalPartition(null);
        }
        newMappingList.add(virtualScsiMapping);
        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely assign disk {} to {}-{}, result: {}", diskName, lparId, lparSlot, result.isSuccess());
        V7VirtualIOServerEntry responseV7 = (V7VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult unAssign(String managedSystemUuid, String viosUuid, String diskName, boolean isPhysical) {
        BocloudResult result = V7VirtualIOServerService.get(getAuth(), viosUuid);
        if (result.isFailed()) {
            logger.error("get vios info error:{}", result);
            return result;
        }
        V7VirtualIOServerEntry virtualIOServerEntry = (V7VirtualIOServerEntry) result.getData();
        V7VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V7VirtualIOServerForUpdate updateVios = new V7VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V7PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V7ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V7ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V7ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V7ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V7AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V7VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V7VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V7VirtualSCSIMapping> newMappingList = new ArrayList<>();
        if (virtualScsiMappingList == null) {
            logger.warn("VirtualSCSIMapping信息为空，取消分配成功");
            return new BocloudResult(true, "取消分配成功");
        }

        for (V7VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if (isPhysical) {
                if (null != diskName && null != oldMap.getStorage() && null != oldMap.getStorage().getPhysicalVolume() && diskName.equals(oldMap.getStorage().getPhysicalVolume().getVolumeName())) {
                    // 这里是该物理卷原来的分配信息,不做任何处理
                } else {
                    newMappingList.add(oldMap);
                }
            } else {
                if (null != diskName && null != oldMap.getStorage() && null != oldMap.getStorage().getVirtualDisk() && diskName.equals(oldMap.getStorage().getVirtualDisk().getDiskName())) {
                    // 这里是该虚拟磁盘原来的分配信息,不做任何处理
                } else {
                    newMappingList.add(oldMap);
                }
            }
        }

        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V7VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely unAssign disk {}, result: {}", diskName, result.isSuccess());
        V7VirtualIOServerEntry responseV7 = (V7VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV7), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult createVolumeGroup(String viosUuid, VolumeGroup volumeGroup) {
        V7VolumeGroup v7VolumeGroup = JSON.parseObject(JSON.toJSONString(volumeGroup), V7VolumeGroup.class);
        BocloudResult bocloudResult = V7VolumeGroupService.create(getAuth(), viosUuid, v7VolumeGroup);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V7VolumeGroupEntry entryV7 = (V7VolumeGroupEntry) bocloudResult.getData();
        VolumeGroupEntry entry = JSON.parseObject(JSON.toJSONString(entryV7), VolumeGroupEntry.class);
        return new BocloudResult(true, entry, "执行成功");
    }

    @Override
    public BocloudResult deleteVolumeGroup(String viosUuid, String volumeGroupUuid) {
        return V7VolumeGroupService.delete(getAuth(), viosUuid, volumeGroupUuid);
    }
}
