package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcHeaPhysicalPortModel;
import com.bocloud.cmp.model.hmc.HmcHostEthernetAdapterModel;
import com.bocloud.cmp.model.hmc.sdk.module.HostEthernetAdapterEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.HEAProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8HostEthernetAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterPhysicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterPortGroup;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class HmcHEAProvider extends HmcAccessableProvider implements HEAProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcHEAProvider.class);

    public HmcHEAProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult updatePhysicalPorts(HmcHostEthernetAdapterModel hmcHostEthernetAdapter, String serverUuid, String heaUuid) {
        V8HostEthernetAdapterForUpdate heaForUpdate = new V8HostEthernetAdapterForUpdate();

        // 从HMC获取主机以太网信息
        BocloudResult result = V8HostEthernetAdapterService.get(getAuth(), serverUuid, heaUuid);
        if (result.isFailed()) {
            return result;
        }

        V8HostEthernetAdapterEntry entry = (V8HostEthernetAdapterEntry) result.getData();
        heaForUpdate.setPortGroups(entry.getContent().getHostEthernetAdapter().getPortGroups());

        // 设置新的主机以太网配置
        List<V8HostEthernetAdapterPortGroup> portGroups = heaForUpdate.getPortGroups().getHostEthernetAdapterPortGroup();
        List<HmcHeaPhysicalPortModel> physicalPorts = hmcHostEthernetAdapter.getHmcHeaPhysicalPorts();
        for (V8HostEthernetAdapterPortGroup portGroup : portGroups) {
            Integer groupId = Integer.parseInt(portGroup.getPortGroupID());
            for (HmcHeaPhysicalPortModel physicalPort : physicalPorts) {
                // 不是同一个组内的物理端口不更新
                if (!physicalPort.getPortGroupId().equals(groupId)) {
                    continue;
                }
                updatePhysicalPort(portGroup.getPhysicalPorts().getHostEthernetAdapterPhysicalPort(), physicalPort);

                if (null != physicalPort.getMultiCoreScaling()) {
                    portGroup.setPendingMultiCoreScaling(physicalPort.getMultiCoreScaling().toString());
                }
            }
        }

        result = V8HostEthernetAdapterService.update(getAuth(), serverUuid, heaUuid, heaForUpdate);
        if (result.isFailed()) {
            return result;
        }
        V8HostEthernetAdapterEntry responseV8 = (V8HostEthernetAdapterEntry) result.getData();
        HostEthernetAdapterEntry responseCommon = JSON.parseObject(JSON.toJSONString(responseV8), HostEthernetAdapterEntry.class);
        return new BocloudResult(true, responseCommon, "执行完成");
    }

    private void updatePhysicalPort(List<V8HostEthernetAdapterPhysicalPort> physicalPorts, HmcHeaPhysicalPortModel hmcHeaPhysicalPort) {
        if (null == physicalPorts || physicalPorts.isEmpty()) {
            return;
        }

        for (V8HostEthernetAdapterPhysicalPort physicalPort : physicalPorts) {
            Integer portId = Integer.parseInt(physicalPort.getPhysicalPortID());
            if (!portId.equals(hmcHeaPhysicalPort.getPortId())) {
                continue;
            }

            if (null != hmcHeaPhysicalPort.getConnectionSpeed()) {
                physicalPort.setPendingConnectionSpeed(hmcHeaPhysicalPort.getConnectionSpeed());
            }
            if (null != hmcHeaPhysicalPort.getDuplexMode()) {
                physicalPort.setPendingDuplexMode(hmcHeaPhysicalPort.getDuplexMode());
            }
            if (null != hmcHeaPhysicalPort.getMaxReceivePacket()) {
                physicalPort.setMaximumReceivePacketSize(hmcHeaPhysicalPort.getMaxReceivePacket());
            }
            if (null != hmcHeaPhysicalPort.getFlowControl()) {
                physicalPort.setPendingFlowControlEnabled(hmcHeaPhysicalPort.getFlowControl().toString());
            }
            if (null != hmcHeaPhysicalPort.getOwner()) {
                physicalPort.setOwner(hmcHeaPhysicalPort.getOwner());
            }
        }
    }
}
