package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcPartitionModel;
import com.bocloud.cmp.model.hmc.HmcProfileModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSCSIClientAdapterEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.PartitionProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualSCSIClientAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcGroupName;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcJobType;
import com.bocloud.cmp.provider.hmc.sdk.enums.HmcOperationName;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.helper.v8.V8HmcDriverHelper;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8JobParameter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8JobParameters;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8JobRequest;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.job.V8RequestedOperation;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssignAllResources;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8DedicatedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartition;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PartitionMemoryConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PartitionProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8SharedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSCSIClientAdapterEntry;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class HmcPartitionProvider extends HmcAccessableProvider implements PartitionProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcPartitionProvider.class);

    public HmcPartitionProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult createLogicalPartition(String managedSystemUuid, HmcPartitionModel par) {
        V8LogicalPartition logicalPartition = new V8LogicalPartition();
        HmcProfileModel profileSetting = par.getHmcLparProfile();
        logicalPartition.setPartitionName(par.getName());
        logicalPartition.setPartitionType(par.getPartitionType());

        // CurrentProfileSync有三种情况，On 开启, Disabled 关闭, Suspended 挂起
        // 无论如何，先设定为关闭同步，避免后面修改概要文件的时候报错
        logicalPartition.setCurrentProfileSync("Disabled");

        // 概要文件是否要使用系统所有资源，需要分开处理
        if (profileSetting.getAssignAllResources() == null || !profileSetting.getAssignAllResources()) {
            // 设定为不使用系统所有资源
            V8AssignAllResources assignAllResources = new V8AssignAllResources();
            assignAllResources.setKsv("V1_6_0");
            assignAllResources.setValue("false");
            logicalPartition.setAssignAllResources(assignAllResources);

            // 处理器配置
            V8PartitionProcessorConfiguration processorConfiguration = new V8PartitionProcessorConfiguration();
            setPartitionProcessor(processorConfiguration, profileSetting);
            logicalPartition.setPartitionProcessorConfiguration(processorConfiguration);

            // 内存设置
            V8PartitionMemoryConfiguration memoryConfiguration = new V8PartitionMemoryConfiguration();
            memoryConfiguration.setMinimumMemory(profileSetting.getMinMemory().toString());
            memoryConfiguration.setDesiredMemory(profileSetting.getDesMemory().toString());
            memoryConfiguration.setMaximumMemory(profileSetting.getMaxMemory().toString());
            logicalPartition.setPartitionMemoryConfiguration(memoryConfiguration);

            // 更新IO设置
            V8PartitionIOConfiguration ioConfiguration = new V8PartitionIOConfiguration();
            ioConfiguration.setMaximumVirtualIOSlots(profileSetting.getMaxVirtualIoslots().toString());
            logicalPartition.setPartitionIOConfiguration(ioConfiguration);
        } else {
            // 设定为rangwo使用系统所有资源
            V8AssignAllResources assignAllResources = new V8AssignAllResources();
            assignAllResources.setKsv("V1_6_0");
            assignAllResources.setValue("true");
            logicalPartition.setAssignAllResources(assignAllResources);
        }

        // 概要文件设置
        V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();
        setProfileForUpdate(updateProfile, profileSetting);

        BocloudResult result = V8LogicalPartitionService.create(getAuth(), managedSystemUuid, logicalPartition, updateProfile);
        if (result.isFailed()) {
            return result;
        }
        V8LogicalPartitionEntry responseV8 = (V8LogicalPartitionEntry) result.getData();

        if (result.isSuccess() && !"Disabled".equals(par.getCurrentProfileSync())) {
            // 如果需要开启同步或挂起，由于上面暂时处理成了关闭，所以这里要改回来
            logger.info("重新将逻辑分区[{}]设定为开启同步或挂起状态...", par.getName());
            V8LogicalPartitionForUpdate updateLpar = new V8LogicalPartitionForUpdate();
            updateLpar.setCurrentProfileSync(par.getCurrentProfileSync());
            result = V8LogicalPartitionService.update(getAuth(), managedSystemUuid, responseV8.getId(), updateLpar);
            if (result.isFailed()) {
                logger.error("重新将逻辑分区[{}]设定为开启同步或挂起状态失败：{}", par.getName(), result.getMessage());
                // 删除已经创建的分区
                V8LogicalPartitionService.delete(getAuth(), managedSystemUuid, responseV8.getId());
                return new BocloudResult(false, "创建逻辑分区出错：" + result.getMessage());
            }
        }

        responseV8 = (V8LogicalPartitionEntry) result.getData();
        LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult createVirtualIOServer(String managedSystemUuid, HmcPartitionModel par) {
        V8VirtualIOServer virtualIOServer = new V8VirtualIOServer();
        HmcProfileModel profileSetting = par.getHmcLparProfile();
        virtualIOServer.setPartitionName(par.getName());
        virtualIOServer.setPartitionType(par.getPartitionType());

        // CurrentProfileSync有三种情况，On 开启, Disabled 关闭, Suspended 挂起
        // 无论如何，先设定为关闭同步，避免后面修改概要文件的时候报错
        virtualIOServer.setCurrentProfileSync("Disabled");

        // 概要文件是否要使用系统所有资源，需要分开处理
        if (profileSetting.getAssignAllResources() == null || !profileSetting.getAssignAllResources()) {
            // 设定为不使用系统所有资源
            V8AssignAllResources assignAllResources = new V8AssignAllResources();
            assignAllResources.setKsv("V1_6_0");
            assignAllResources.setValue("false");
            virtualIOServer.setAssignAllResources(assignAllResources);

            // 处理器配置
            V8PartitionProcessorConfiguration processorConfiguration = new V8PartitionProcessorConfiguration();
            setPartitionProcessor(processorConfiguration, profileSetting);
            virtualIOServer.setPartitionProcessorConfiguration(processorConfiguration);

            // 内存设置
            V8PartitionMemoryConfiguration memoryConfiguration = new V8PartitionMemoryConfiguration();
            memoryConfiguration.setMinimumMemory(profileSetting.getMinMemory().toString());
            memoryConfiguration.setDesiredMemory(profileSetting.getDesMemory().toString());
            memoryConfiguration.setMaximumMemory(profileSetting.getMaxMemory().toString());
            virtualIOServer.setPartitionMemoryConfiguration(memoryConfiguration);

            // 更新IO设置
            V8PartitionIOConfiguration ioConfiguration = new V8PartitionIOConfiguration();
            ioConfiguration.setMaximumVirtualIOSlots(profileSetting.getMaxVirtualIoslots().toString());
            virtualIOServer.setPartitionIOConfiguration(ioConfiguration);
        } else {
            // 设定为rangwo使用系统所有资源
            V8AssignAllResources assignAllResources = new V8AssignAllResources();
            assignAllResources.setKsv("V1_6_0");
            assignAllResources.setValue("true");
            virtualIOServer.setAssignAllResources(assignAllResources);
        }

        // 概要文件设置
        V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();
        setProfileForUpdate(updateProfile, profileSetting);

        BocloudResult result = V8VirtualIOServerService.create(getAuth(), managedSystemUuid, virtualIOServer, updateProfile);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create VIOS {}", par.getName());
        V8VirtualIOServerEntry responseV8 = (V8VirtualIOServerEntry) result.getData();

        if (result.isSuccess() && !"Disabled".equals(par.getCurrentProfileSync())) {
            // 如果需要开启同步或挂起，由于上面暂时处理成了关闭，所以这里要改回来
            logger.info("重新将VIOS[{}]设定为开启同步或挂起状态...", par.getName());
            V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();
            updateVios.setCurrentProfileSync(par.getCurrentProfileSync());
            result = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, responseV8.getId(), updateVios);
            if (result.isFailed()) {
                logger.error("重新将VIOS[{}]设定为开启同步或挂起状态失败：{}", par.getName(), result.getMessage());
                // 删除已经创建的VIOS
                V8LogicalPartitionService.delete(getAuth(), managedSystemUuid, responseV8.getId());
                return new BocloudResult(false, "创建VIOS出错：" + result.getMessage());
            }
        }

        responseV8 = (V8VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    private void setPartitionProcessor(V8PartitionProcessorConfiguration processorConfiguration, HmcProfileModel profileSetting) {
        if (profileSetting.getSharedProcessor() != null && profileSetting.getSharedProcessor()) {
            V8SharedProcessorConfiguration sharedConfiguration = new V8SharedProcessorConfiguration();
            processorConfiguration.setHasDedicatedProcessors("false");
            sharedConfiguration.setMinimumProcessingUnits(profileSetting.getMinProUnits().toString());
            sharedConfiguration.setDesiredProcessingUnits(profileSetting.getDesProUnits().toString());
            sharedConfiguration.setMaximumProcessingUnits(profileSetting.getMaxProUnits().toString());
            sharedConfiguration.setSharedProcessorPoolID(profileSetting.getSharedProPoolId().toString());
            sharedConfiguration.setMinimumVirtualProcessors(profileSetting.getMinProVirt().toString());
            sharedConfiguration.setDesiredVirtualProcessors(profileSetting.getDesProVirt().toString());
            sharedConfiguration.setMaximumVirtualProcessors(profileSetting.getMaxProVirt().toString());
            sharedConfiguration.setSharedProcessorPoolID(profileSetting.getSharedProPoolId().toString());
            if ("uncapped".equals(profileSetting.getProSharingMode()) && null != profileSetting.getUncappedWeight()) {
                sharedConfiguration.setUncappedWeight(profileSetting.getUncappedWeight().toString());
            }
            processorConfiguration.setSharedProcessorConfiguration(sharedConfiguration);
            processorConfiguration.setSharingMode(profileSetting.getProSharingMode());
        } else {
            V8DedicatedProcessorConfiguration dedicatedConfiguration = new V8DedicatedProcessorConfiguration();
            processorConfiguration.setHasDedicatedProcessors("true");
            dedicatedConfiguration.setMinimumProcessors(profileSetting.getMinPro().toString());
            dedicatedConfiguration.setDesiredProcessors(profileSetting.getDesPro().toString());
            dedicatedConfiguration.setMaximumProcessors(profileSetting.getMaxPro().toString());
            processorConfiguration.setDedicatedProcessorConfiguration(dedicatedConfiguration);
            processorConfiguration.setSharingMode("sre idle proces");
        }
    }

    private void setProfileForUpdate(V8LogicalPartitionProfileForUpdate updateProfile, HmcProfileModel profileSetting) {
        updateProfile.setProfileName(profileSetting.getName());
        updateProfile.setAssignAllResources(profileSetting.getAssignAllResources() == null ? null : profileSetting.getAssignAllResources().toString());
        updateProfile.setConnectionMonitoringEnabled(profileSetting.getConnMonitor() == null ? null : profileSetting.getConnMonitor().toString());
        updateProfile.setAutoStart(profileSetting.getAutoStart() == null ? null : profileSetting.getAutoStart().toString());
        updateProfile.setRedundantErrorPathReportingEnabled(profileSetting.getRedundantErrorPathReport() == null ? null : profileSetting.getRedundantErrorPathReport().toString());
        updateProfile.setBootMode(profileSetting.getBootMode());

        // 概要文件虚拟适配器设置
        V8IOConfigurationInstance ioConfig = new V8IOConfigurationInstance();
        if (V8HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profileSetting.getMaxVirtualIoslots(),
                profileSetting.getHmcProfileNetworkAdapters(), profileSetting.getHmcProfileFibreAdapters(),
                profileSetting.getHmcProfileScsiAdapters(), profileSetting.getHmcProfileIoslot())) {
            updateProfile.setIoConfigurationInstance(ioConfig);
        }

        // 主机以太网逻辑端口设置
        V8HostEthernetAdapterLogicalPorts logicalPorts = new V8HostEthernetAdapterLogicalPorts();
        if (null != profileSetting.getHmcProfileLheas()) {
            V8HmcDriverHelper.updateProfileHEALogicalPort(logicalPorts, profileSetting.getHmcProfileLheas());
            updateProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
        }
    }

    /**
     * Hmc分区删除
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult delete(String managedSystemUuid, String partitionUuid, Boolean isVios) {
        BocloudResult result;
        if (isVios) {
            result = V8VirtualIOServerService.delete(getAuth(), managedSystemUuid, partitionUuid);
            logger.info("Hmc driver completely delete VIOS partition");
        } else {
            result = V8LogicalPartitionService.delete(getAuth(), managedSystemUuid, partitionUuid);
            logger.info("Hmc driver completely delete logical partition");
        }
        return result;
    }

    /**
     * Hmc逻辑分区编辑
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult updateLogicalPartition(HmcPartitionModel partition, String managedSystemUuid, String partitionUuid) {
        V8LogicalPartitionForUpdate updatePar = new V8LogicalPartitionForUpdate();
        if (null != partition.getCurrentProfileSync()) {
            updatePar.setCurrentProfileSync(partition.getCurrentProfileSync());
        }
        if (null != partition.getName()) {
            updatePar.setPartitionName(partition.getName());
        }
        if (null != partition.getKeyLockPosition()) {
            updatePar.setKeylockPosition(partition.getKeyLockPosition());
        }
        // 更新允许收集性能信息设置
        if (null != partition.getAllowPerformanceCollection()) {
            updatePar.setAllowPerformanceDataCollection(partition.getAllowPerformanceCollection().toString());
        }

        BocloudResult result = V8LogicalPartitionService.update(getAuth(), managedSystemUuid, partitionUuid, updatePar);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update partition");

        V8LogicalPartitionEntry responseV8 = (V8LogicalPartitionEntry) result.getData();
        LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * Hmc VIOS分区编辑
     *
     * @param managedSystemUuid
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult updateVirtualIOServer(HmcPartitionModel partition, String managedSystemUuid, String partitionUuid) {
        V8VirtualIOServerForUpdate updatePar = new V8VirtualIOServerForUpdate();
        if (null != partition.getCurrentProfileSync()) {
            updatePar.setCurrentProfileSync(partition.getCurrentProfileSync());
        }
        if (null != partition.getName()) {
            updatePar.setPartitionName(partition.getName());
        }
        if (null != partition.getKeyLockPosition()) {
            updatePar.setKeylockPosition(partition.getKeyLockPosition());
        }
        // 更新允许收集性能信息设置
        if (null != partition.getAllowPerformanceCollection()) {
            updatePar.setAllowPerformanceDataCollection(partition.getAllowPerformanceCollection().toString());
        }

        BocloudResult result = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, partitionUuid, updatePar);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update VIOS partition");
        V8VirtualIOServerEntry responseV8 = (V8VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualIOServerEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult getPartitionAttribute(String partitionUuid, Boolean isVios) {
        BocloudResult result;
        if (isVios) {
            result = V8VirtualIOServerService.get(getAuth(), partitionUuid);
            if (result.isFailed()) {
                return result;
            }
            V8VirtualIOServerEntry responseV8 = (V8VirtualIOServerEntry) result.getData();
            VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualIOServerEntry.class);
            return new BocloudResult(true, response, "执行成功");
        } else {
            result = V8LogicalPartitionService.get(getAuth(), partitionUuid);
            if (result.isFailed()) {
                return result;
            }
            V8LogicalPartitionEntry responseV8 = (V8LogicalPartitionEntry) result.getData();
            LogicalPartitionEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionEntry.class);
            return new BocloudResult(true, response, "执行成功");
        }
    }

    @Override
    public BocloudResult powerOffPartition(String uuid, Boolean isVios) {
        V8JobRequest jobRequest = generatePowerOffJobRequest(false);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V8VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V8LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_LogicalPartition);
        }
    }

    @Override
    public BocloudResult restartPartition(String uuid, Boolean isVios) {
        V8JobRequest jobRequest = generatePowerOffJobRequest(true);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V8VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V8LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOff_LogicalPartition);
        }
    }

    private V8JobRequest generatePowerOffJobRequest(Boolean restart) {
        V8JobRequest jobRequest = new V8JobRequest();
        V8RequestedOperation requestedOperation = new V8RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOff.name());
        requestedOperation.setProgressType("DISCRETE");
        jobRequest.setRequestedOperation(requestedOperation);

        List<V8JobParameter> listJobParameters = new ArrayList<>();
        V8JobParameter jobParameter;

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("immediate");
        jobParameter.setParameterValue("true");
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("restart");
        jobParameter.setParameterValue(restart.toString());
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("operation");
        jobParameter.setParameterValue("shutdown");
        listJobParameters.add(jobParameter);

        V8JobParameters jobParmeters = new V8JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);

        return jobRequest;
    }

    @Override
    public BocloudResult powerOnPartition(String uuid, String profileUuid, Boolean isVios) {
        V8JobRequest jobRequest = new V8JobRequest();
        V8RequestedOperation requestedOperation = new V8RequestedOperation();
        requestedOperation.setOperationName(HmcOperationName.PowerOn.name());
        jobRequest.setRequestedOperation(requestedOperation);

        List<V8JobParameter> listJobParameters = new ArrayList<>();
        V8JobParameter jobParameter;

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("force");
        jobParameter.setParameterValue("false");
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("LogicalPartitionProfile");
        jobParameter.setParameterValue(profileUuid);
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("novsi");
        jobParameter.setParameterValue("true");
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("bootmode");
        jobParameter.setParameterValue("norm");
        listJobParameters.add(jobParameter);

        jobParameter = new V8JobParameter();
        jobParameter.setParameterName("keylock");
        jobParameter.setParameterValue("norm");
        listJobParameters.add(jobParameter);

        V8JobParameters jobParmeters = new V8JobParameters();
        jobParmeters.setJobParameters(listJobParameters);
        jobRequest.setJobParameters(jobParmeters);
        if (isVios) {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.VirtualIOServer.name());
            return V8VirtualIOServerService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOn_VirtualIOServer);
        } else {
            jobRequest.getRequestedOperation().setGroupName(HmcGroupName.LogicalPartition.name());
            return V8LogicalPartitionService.doJobRequest(getAuth(), uuid, jobRequest, HmcJobType.PowerOn_LogicalPartition);
        }
    }

    @Override
    public BocloudResult queryPartitionState(String uuid, Boolean isVios) {
        if (isVios) {
            return V8VirtualIOServerService.getQuickProperty(getAuth(), uuid, VirtualIOServerQuickProperty.PartitionState);
        } else {
            return V8LogicalPartitionService.getQuickProperty(getAuth(), uuid, LogicalPartitionQuickProperty.PartitionState);
        }
    }

    @Override
    public BocloudResult queryPartitionRMCState(String uuid, Boolean isVios) {
        if (isVios) {
            return V8VirtualIOServerService.getQuickProperty(getAuth(), uuid, VirtualIOServerQuickProperty.RMCState);
        } else {
            return V8LogicalPartitionService.getQuickProperty(getAuth(), uuid, LogicalPartitionQuickProperty.RMCState);
        }
    }

    @Override
    public BocloudResult getVirtualScsiAdapters(String href) {
        BocloudResult result = V8VirtualSCSIClientAdapterService.getByHref(getAuth(), href);
        if (result.isFailed()) {
            return result;
        }
        V8VirtualSCSIClientAdapterEntry responseV8 = (V8VirtualSCSIClientAdapterEntry) result.getData();
        VirtualSCSIClientAdapterEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualSCSIClientAdapterEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
