package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcPartitionModel;
import com.bocloud.cmp.model.hmc.HmcProfileLheaModel;
import com.bocloud.cmp.model.hmc.HmcProfileModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.ProfileProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionProfileService;
import com.bocloud.cmp.provider.hmc.sdk.helper.v8.V8HmcDriverHelper;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8DedicatedProcessorConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterLogicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterLogicalPorts;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8IOConfigurationInstance;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfile;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProcessorAttributes;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileMemory;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8SharedProcessorConfiguration;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;

public class HmcProfileProvider extends HmcAccessableProvider implements ProfileProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcProfileProvider.class);

    public HmcProfileProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(HmcPartitionModel partition, HmcProfileModel profile) {
        V8LogicalPartitionProfile hmcProfile = new V8LogicalPartitionProfile();
        hmcProfile.setProfileName(profile.getName());

        Boolean isAssignAllResources = profile.getAssignAllResources();
        if (null == isAssignAllResources) {
            isAssignAllResources = false;
        }
        hmcProfile.setAssignAllResources(isAssignAllResources.toString());
        if (false == isAssignAllResources) {
            // 设置处理器
            V8ProcessorAttributes processorAttributes = new V8ProcessorAttributes();
            setProcessorAttributes(processorAttributes, profile);
            hmcProfile.setProcessorAttributes(processorAttributes);

            // 设置内存
            V8ProfileMemory memory = new V8ProfileMemory();
            memory.setMaximumMemory(profile.getMaxMemory().toString());
            memory.setMinimumMemory(profile.getMinMemory().toString());
            memory.setDesiredMemory(profile.getDesMemory().toString());
            hmcProfile.setProfileMemory(memory);

            // 概要文件虚拟适配器设置
            V8IOConfigurationInstance ioConfig = new V8IOConfigurationInstance();
            if (V8HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profile.getMaxVirtualIoslots(),
                    profile.getHmcProfileNetworkAdapters(), profile.getHmcProfileFibreAdapters(),
                    profile.getHmcProfileScsiAdapters(), profile.getHmcProfileIoslot())) {
                hmcProfile.setIoConfigurationInstance(ioConfig);
            }

            // 设置hea
            if (profile.getHmcProfileLheas() != null) {
                V8HostEthernetAdapterLogicalPorts logicalPorts = new V8HostEthernetAdapterLogicalPorts();
                logicalPorts.setHostEthernetAdapterLogicalPortList(new ArrayList<>());
                for (HmcProfileLheaModel hmcProfileLhea : profile.getHmcProfileLheas()) {
                    V8HostEthernetAdapterLogicalPort logicalPort = new V8HostEthernetAdapterLogicalPort();
                    logicalPort.setAdapterID(hmcProfileLhea.getAdapterId() == null ? null :hmcProfileLhea.getAdapterId().toString());
                    logicalPort.setAllowedVLANIDs(StringUtils.isEmpty(hmcProfileLhea.getAllowVlanIds()) ? null : hmcProfileLhea.getAllowVlanIds());
                    logicalPort.setAllowedOSMACAddresses(StringUtils.isEmpty(hmcProfileLhea.getAllowMacAddresses()) ? null : hmcProfileLhea.getAllowMacAddresses());
                    logicalPort.setLogicalPortID(hmcProfileLhea.getLogicalPortId() == null ? null : hmcProfileLhea.getLogicalPortId().toString());
                    logicalPort.setPhysicalPortID(hmcProfileLhea.getPhysicalPortId() == null ? null : hmcProfileLhea.getPhysicalPortId().toString());
                    logicalPort.setPortGroupID(hmcProfileLhea.getPortGroupId() == null ? null : hmcProfileLhea.getPortGroupId().toString());
                    logicalPorts.getHostEthernetAdapterLogicalPortList().add(logicalPort);
                }
                hmcProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
            }

            // 设置其它
            hmcProfile.setBootMode(profile.getBootMode().toString());
            hmcProfile.setAutoStart(profile.getAutoStart().toString());
            hmcProfile.setConnectionMonitoringEnabled(profile.getConnMonitor().toString());
            hmcProfile.setRedundantErrorPathReportingEnabled(profile.getRedundantErrorPathReport().toString());
            if (null != profile.getProcessorCompatibilityMode()) {
                hmcProfile.setDesiredProcessorCompatibilityMode(profile.getProcessorCompatibilityMode());
            }
        }

        boolean isVios = false;
        if ("Virtual IO Server".equals(partition.getPartitionType())) {
            isVios = true;
        }
        BocloudResult result = V8LogicalPartitionProfileService.create(getAuth(), isVios, partition.getUuid(), hmcProfile);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create partition");

        V8LogicalPartitionProfileEntry responseV8 = (V8LogicalPartitionProfileEntry) result.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    private void setProcessorAttributes(V8ProcessorAttributes processorAttributes, HmcProfileModel profile) {
        processorAttributes.setSharingMode(profile.getProSharingMode());
        if (profile.getSharedProcessor()) {
            processorAttributes.setHasDedicatedProcessors("false");
            V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
            sharedConfig.setDesiredProcessingUnits(profile.getDesProUnits().toString());
            sharedConfig.setDesiredVirtualProcessors(profile.getDesProVirt().toString());
            sharedConfig.setMaximumProcessingUnits(profile.getMaxProUnits().toString());
            sharedConfig.setMaximumVirtualProcessors(profile.getMaxProVirt().toString());
            sharedConfig.setMinimumProcessingUnits(profile.getMinProUnits().toString());
            sharedConfig.setMinimumVirtualProcessors(profile.getMinProVirt().toString());
            sharedConfig.setSharedProcessorPoolID(profile.getSharedProPoolId().toString());
            if (profile.getUncappedWeight() != null) {
                sharedConfig.setUncappedWeight(profile.getUncappedWeight().toString());
            }

            processorAttributes.setSharedProcessorConfiguration(sharedConfig);
        } else {
            processorAttributes.setHasDedicatedProcessors("true");
            V8DedicatedProcessorConfiguration dedicatedConfig = new V8DedicatedProcessorConfiguration();
            dedicatedConfig.setDesiredProcessors(profile.getDesPro().toString());
            dedicatedConfig.setMaximumProcessors(profile.getMaxPro().toString());
            dedicatedConfig.setMinimumProcessors(profile.getMinPro().toString());

            processorAttributes.setDedicatedProcessorConfiguration(dedicatedConfig);
        }
    }

    /**
     * Hmc概要文件编辑
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult update(HmcProfileModel profile, boolean isVIOS, String partitionUuid, String profileUuid) {
        V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();
        if (null != profile.getName()) {
            updateProfile.setProfileName(profile.getName());
        }
        if (null != profile.getConnMonitor()) {
            updateProfile.setConnectionMonitoringEnabled(profile.getConnMonitor().toString());
        }
        if (null != profile.getBootMode()) {
            updateProfile.setBootMode(profile.getBootMode());
        }
        if (null != profile.getAutoStart()) {
            updateProfile.setAutoStart(profile.getAutoStart().toString());
        }
        if (null != profile.getRedundantErrorPathReport()) {
            updateProfile.setRedundantErrorPathReportingEnabled(profile.getRedundantErrorPathReport().toString());
        }
        if (null != profile.getProcessorCompatibilityMode()) {
            updateProfile.setDesiredProcessorCompatibilityMode(profile.getProcessorCompatibilityMode());
        }

        // 设置处理器
        if (null != profile.getSharedProcessor()) {
            V8ProcessorAttributes processorAttributes = new V8ProcessorAttributes();
            setProcessorAttributes(processorAttributes, profile);
            updateProfile.setProcessorAttributes(processorAttributes);
        }

        // 设置内存
        // 内存如果要修改的话，所有的值都要传过来
        if (null != profile.getMaxMemory()) {
            V8ProfileMemory memory = new V8ProfileMemory();
            memory.setMinimumMemory(profile.getMinMemory().toString());
            memory.setMaximumMemory(profile.getMaxMemory().toString());
            memory.setDesiredMemory(profile.getDesMemory().toString());
            memory.setBarrierSynchronizationRegisterArrayCount(profile.getMemoryBsrCount().toString());
            memory.setMinimumHugePageCount(profile.getMemoryMinPageCount().toString());
            memory.setDesiredHugePageCount(profile.getMemoryDesPageCount().toString());
            memory.setMaximumHugePageCount(profile.getMemoryMaxPageCount().toString());
            updateProfile.setProfileMemory(memory);
        }

        V8IOConfigurationInstance ioConfig = new V8IOConfigurationInstance();
        if (V8HmcDriverHelper.updateProfileIOConfiguration(ioConfig, profile.getMaxVirtualIoslots(),
                profile.getHmcProfileNetworkAdapters(), profile.getHmcProfileFibreAdapters(),
                profile.getHmcProfileScsiAdapters(), profile.getHmcProfileIoslot())) {
            updateProfile.setIoConfigurationInstance(ioConfig);
        }

        // 主机以太网逻辑端口设置
        V8HostEthernetAdapterLogicalPorts logicalPorts = new V8HostEthernetAdapterLogicalPorts();
        if (null != profile.getHmcProfileLheas()) {
            V8HmcDriverHelper.updateProfileHEALogicalPort(logicalPorts, profile.getHmcProfileLheas());
            updateProfile.setHostEthernetAdapterLogicalPorts(logicalPorts);
        }

        BocloudResult result = V8LogicalPartitionProfileService.update(getAuth(), isVIOS, partitionUuid, profileUuid, updateProfile);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update partition profile");
        V8LogicalPartitionProfileEntry responseV8 = (V8LogicalPartitionProfileEntry) result.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * Hmc分区概要文件删除
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult delete(boolean isVios, String partitionUuid, String profileUuid) {
        BocloudResult result = V8LogicalPartitionProfileService.delete(getAuth(), isVios, partitionUuid, profileUuid);
        logger.info("Hmc driver completely delete partition profile");
        return result;
    }

    /**
     * 获得Hmc分区概要文件
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult getProfile(boolean isVios, String partitionUuid, String profileUuid) {
        BocloudResult result;
        if (isVios) {
            result = V8LogicalPartitionProfileService.getByVirtualIOServerUuid(getAuth(), partitionUuid, profileUuid);
        } else {
            result = V8LogicalPartitionProfileService.getByLogicalPartitionUuid(getAuth(), partitionUuid, profileUuid);
        }
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely get partition profile");
        V8LogicalPartitionProfileEntry responseV8 = (V8LogicalPartitionProfileEntry) result.getData();
        LogicalPartitionProfileEntry response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionProfileEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    /**
     * 获得Hmc分区概要文件
     * 
     * @param partitionUuid
     * @return
     */
    @Override
    public BocloudResult getProfile(boolean isVios, String partitionUuid) {
        BocloudResult result;
        if (isVios) {
            result = V8LogicalPartitionProfileService.getByVirtualIOServerUuid(getAuth(), partitionUuid);
        } else {
            result = V8LogicalPartitionProfileService.getByLogicalPartitionUuid(getAuth(), partitionUuid);
        }
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely get partition profile");
        V8LogicalPartitionProfileResponse responseV8 = (V8LogicalPartitionProfileResponse) result.getData();
        LogicalPartitionProfileResponse response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionProfileResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
