package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcVirtualFibreChannelMappingModel;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionVfcResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VirtualFibreChannelMappingProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8LogicalPartitionVfcResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8Port;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ServerAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualFibreChannelMapping;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualFibreChannelMappings;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerForUpdate;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.BooleanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author luyanchao
 * @since 2018/12/18 16:53
 */
public class HmcVirtualFibreChannelMappingProvider extends HmcAccessableProvider implements VirtualFibreChannelMappingProvider {

    private static Logger logger = LoggerFactory.getLogger(HmcVirtualFibreChannelMappingProvider.class);

    public HmcVirtualFibreChannelMappingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult addVfcToVios(String managedSystemUuid, String viosUuid, HmcVirtualFibreChannelMappingModel mapping) {
        // 获取现有的vios数据
        BocloudResult getOldResult = V8VirtualIOServerService.get(getAuth(), viosUuid);
        if (getOldResult.isFailed()) {
            return getOldResult;
        }
        V8VirtualIOServer oldVios = ((V8VirtualIOServerEntry) getOldResult.getData()).getContent().getVirtualIOServer();

        // 初始化用来更新的vios数据
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();
        updateVios.setVirtualFibreChannelMappings(new V8VirtualFibreChannelMappings());
        updateVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());

        // 将现有的vfcMapping添加进来
        if (oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList() != null) {
            for (V8VirtualFibreChannelMapping oldMapping : oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList()) {
                oldMapping.getServerAdapter().setPhysicalPort(null);
                updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(oldMapping);
            }
        } else {
            oldVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());
        }

        // 添加新的vfcMapping
        V8VirtualFibreChannelMapping newMapping = new V8VirtualFibreChannelMapping();

        V8Port port = new V8Port();
        port.setUniqueDeviceID(mapping.getPhysicalUniqueDeviceId());
        port.setPortName(mapping.getFcName());
        newMapping.setPort(port);

        V8ClientAdapter clientAdapter = new V8ClientAdapter();
        clientAdapter.setAdapterType("Client");
        clientAdapter.setLocalPartitionId(mapping.getClientPartitionId().toString());
        clientAdapter.setVirtualSlotNumber(mapping.getClientPartitionSlot() == null ? null : mapping.getClientPartitionSlot().toString());
        clientAdapter.setConnectingPartitionID(mapping.getServerViosId().toString());
        clientAdapter.setConnectingVirtualSlotNumber(mapping.getServerViosSlot() == null ? null : mapping.getServerViosSlot().toString());
        clientAdapter.setVariedOn(BooleanUtils.toString(mapping.getClientViredOn(), "true", "false", "false"));
        clientAdapter.setRequiredAdapter(BooleanUtils.toString(mapping.getClientRequired(), "true", "false", "false"));
        newMapping.setClientAdapter(clientAdapter);

        V8ServerAdapter serverAdapter = new V8ServerAdapter();
        serverAdapter.setAdapterType("Server");
        serverAdapter.setLocalPartitionId(mapping.getServerViosId().toString());
        serverAdapter.setVirtualSlotNumber(mapping.getServerViosSlot() == null ? null : mapping.getServerViosSlot().toString());
        serverAdapter.setConnectingPartitionID(mapping.getClientPartitionId().toString());
        serverAdapter.setConnectingVirtualSlotNumber(mapping.getClientPartitionSlot() == null ? null : mapping.getClientPartitionSlot().toString());
        serverAdapter.setVariedOn(BooleanUtils.toString(mapping.getServerViredOn(), "true", "false", "false"));
        serverAdapter.setRequiredAdapter(BooleanUtils.toString(mapping.getServerRequired(), "true", "false", "false"));
        serverAdapter.setMapPort(mapping.getFcName());
        newMapping.setServerAdapter(serverAdapter);

        updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(newMapping);

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 保存vios
        BocloudResult bocloudResult = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V8VirtualIOServerEntry serverEntryV8 = (V8VirtualIOServerEntry) bocloudResult.getData();
        VirtualIOServerEntry serverEntry = JSON.parseObject(JSON.toJSONString(serverEntryV8), VirtualIOServerEntry.class);
        return new BocloudResult(true, serverEntry, "执行成功");
    }

    @Override
    public BocloudResult removeVfcFromVios(String managedSystemUuid, String viosUuid, HmcVirtualFibreChannelMappingModel mapping) {
        // 获取现有的vios数据
        BocloudResult getOldResult = V8VirtualIOServerService.get(getAuth(), viosUuid);
        if (getOldResult.isFailed()) {
            return getOldResult;
        }
        V8VirtualIOServer oldVios = ((V8VirtualIOServerEntry) getOldResult.getData()).getContent().getVirtualIOServer();

        // 初始化用来更新的vios数据
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();
        updateVios.setVirtualFibreChannelMappings(new V8VirtualFibreChannelMappings());
        updateVios.getVirtualFibreChannelMappings().setVirtualFibreChannelMappingList(new ArrayList<>());

        // 将现有的vfcMapping添加进来，除了需要删除的那个
        if (oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList() != null) {
            for (V8VirtualFibreChannelMapping oldMapping : oldVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList()) {
                if (oldMapping.getServerAdapter().getLocalPartitionId().equals(mapping.getServerViosId().toString()) &&
                        oldMapping.getServerAdapter().getVirtualSlotNumber().equals(mapping.getServerViosSlot().toString())) {
                    continue;
                }
                oldMapping.getServerAdapter().setPhysicalPort(null);
                updateVios.getVirtualFibreChannelMappings().getVirtualFibreChannelMappingList().add(oldMapping);
            }
        }

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 保存vios
        BocloudResult bocloudResult = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V8VirtualIOServerEntry serverEntryV8 = (V8VirtualIOServerEntry) bocloudResult.getData();
        VirtualIOServerEntry serverEntry = JSON.parseObject(JSON.toJSONString(serverEntryV8), VirtualIOServerEntry.class);
        return new BocloudResult(true, serverEntry, "执行成功");
    }

    @Override
    public BocloudResult getVfcFromPartition(String partitionUuid) {
        BocloudResult bocloudResult = V8LogicalPartitionService.getVfc(getAuth(), partitionUuid);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V8LogicalPartitionVfcResponse responseV8 = (V8LogicalPartitionVfcResponse) bocloudResult.getData();
        LogicalPartitionVfcResponse response = JSON.parseObject(JSON.toJSONString(responseV8), LogicalPartitionVfcResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }

}
