package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.HmcVirtualSwitchModel;
import com.bocloud.cmp.model.hmc.sdk.module.ClientNetworkAdapterResponse;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualSwitchEntry;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VirtualNetworkProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8ClientNetworkAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualSwitchService;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ClientNetworkAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSwitch;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSwitchEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSwitchForUpdate;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HmcVirtualNetworkProvider extends HmcAccessableProvider implements VirtualNetworkProvider {
    private static Logger logger = LoggerFactory.getLogger(HmcVirtualNetworkProvider.class);

    public HmcVirtualNetworkProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult createVirtualSwitch(HmcVirtualSwitchModel virtualSwitch, String serverUuid) {
        V8VirtualSwitch vswitch = new V8VirtualSwitch();
        vswitch.setSwitchName(virtualSwitch.getName());
        BocloudResult result = V8VirtualSwitchService.create(getAuth(), serverUuid, vswitch);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create virtual switch");
        V8VirtualSwitchEntry responseV8 = (V8VirtualSwitchEntry) result.getData();
        VirtualSwitchEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualSwitchEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult deleteVirtualSwitch(String serverUuid, String vswitchUuid) {
        BocloudResult result = V8VirtualSwitchService.delete(getAuth(), serverUuid, vswitchUuid);
        logger.info("Hmc driver completely delete virtual switch");
        return result;
    }

    @Override
    public BocloudResult updateVirtualSwitch(HmcVirtualSwitchModel virtualSwitch, String serverUuid, String vswitchUuid) {
        V8VirtualSwitchForUpdate virtualSwitchForUpdate = new V8VirtualSwitchForUpdate();
        if (null != virtualSwitch) {
            virtualSwitchForUpdate.setSwitchName(virtualSwitch.getName());
        }
        BocloudResult result = V8VirtualSwitchService.update(getAuth(), serverUuid, vswitchUuid, virtualSwitchForUpdate);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create virtual switch");
        V8VirtualSwitchEntry responseV8 = (V8VirtualSwitchEntry) result.getData();
        VirtualSwitchEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VirtualSwitchEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public  BocloudResult getClientNetworkAdapter(String laprUuid) {
        BocloudResult result = V8ClientNetworkAdapterService.get(getAuth(), laprUuid);
        if (result.isFailed()) {
            return result;
        }
        V8ClientNetworkAdapterResponse responseV8 = (V8ClientNetworkAdapterResponse) result.getData();
        ClientNetworkAdapterResponse response = JSON.parseObject(JSON.toJSONString(responseV8), ClientNetworkAdapterResponse.class);
        return new BocloudResult(true, response, "执行成功");
    }
}
