package com.bocloud.cmp.provider.hmc.v8;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroup;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupResponse;
import com.bocloud.cmp.provider.HmcAccessableProvider;
import com.bocloud.cmp.provider.hmc.VolumeGroupProvider;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VolumeGroupService;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8AssociatedIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ClientAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PartitionIOConfiguration;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8PhysicalVolume;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlot;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ProfileIOSlots;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8ServerAdapter;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8Storage;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualDisk;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualDisks;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServer;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualIOServerForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSCSIMapping;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VirtualSCSIMappings;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroup;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupContent;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8VolumeGroupResponse;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * hmc volume group driver
 *
 * @author lkx007
 */
public class HmcVolumeGroupProvider extends HmcAccessableProvider implements VolumeGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(HmcVolumeGroupProvider.class);

    public HmcVolumeGroupProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult getVolumeGroup(String viosUuid) {
        if (viosUuid == null) {
            return new BocloudResult(false, "参数异常");
        }
        BocloudResult getResult = V8VolumeGroupService.get(getAuth(), viosUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }
        V8VolumeGroupResponse responseV8 = (V8VolumeGroupResponse) getResult.getData();
        VolumeGroupResponse response = JSON.parseObject(JSON.toJSONString(responseV8), VolumeGroupResponse.class);
        return new BocloudResult(true, response, "查询虚拟磁盘成功");

    }

    @Override
    public BocloudResult getVolumeGroup(String viosUuid, String volumeGroupUuid) {
        if (viosUuid == null || volumeGroupUuid == null) {
            return new BocloudResult(false, "参数异常");
        }
        BocloudResult getResult = V8VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get VolumeGroupEntry failed:{}", getResult);
            return getResult;
        }
        V8VolumeGroupEntry responseV8 = (V8VolumeGroupEntry) getResult.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "查询虚拟磁盘成功");
    }

    @Override
    public BocloudResult createVirtualDisk(String viosUuid, String volumeGroupUuid, Double diskCapacity, String diskName) {
        BocloudResult getResult = V8VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V8VolumeGroupEntry entry = (V8VolumeGroupEntry) getResult.getData();
        Optional<V8VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V8VolumeGroupEntry::getContent)
                .map(V8VolumeGroupContent::getVolumeGroup)
                .map(V8VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V8VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V8VirtualDisk> disks = virtualDisksOpt.map(V8VirtualDisks::getDisks).orElse(new ArrayList<>());
        if (disks.stream().anyMatch(disk -> Objects.equals(disk.getDiskName(), diskName))) {
            logger.error("virtual disk {} exists", diskName);
            return new BocloudResult(false, "虚拟磁盘[" + diskName + "]已经存在");
        }
        V8VirtualDisk disk = new V8VirtualDisk();
        disk.setDiskCapacity(diskCapacity.toString());
        disk.setDiskName(diskName);
        disks.add(disk);
        virtualDisks.setDisks(disks);

        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);
        BocloudResult result = V8VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely create virtual disk {}, result: {}", diskName, result.isSuccess());
        V8VolumeGroupEntry responseV8 = (V8VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult updateVirtualDisk(String viosUuid, String volumeGroupUuid, Double diskCapacity, String diskName) {
        BocloudResult getResult = V8VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V8VolumeGroupEntry entry = (V8VolumeGroupEntry) getResult.getData();
        Optional<V8VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V8VolumeGroupEntry::getContent)
                .map(V8VolumeGroupContent::getVolumeGroup)
                .map(V8VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V8VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V8VirtualDisk> disks = virtualDisksOpt.map(V8VirtualDisks::getDisks).orElse(new ArrayList<>());
        Optional<V8VirtualDisk> diskOpt = disks.stream().filter(disk -> Objects.equals(disk.getDiskName(), diskName)).findFirst();
        if (!diskOpt.isPresent()) {
            logger.error("virtual disk {} not exists", diskName);
            return new BocloudResult(false, "虚拟磁盘[" + diskName + "]不存在");
        }
        diskOpt.get().setDiskCapacity(diskCapacity.toString());
        virtualDisks.setDisks(disks);

        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);
        BocloudResult result = V8VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely update virtual disk {}, result: {}", diskName, result.isSuccess());
        V8VolumeGroupEntry responseV8 = (V8VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult deleteVirtualDisk(String viosUuid, String volumeGroupUuid, String diskName) {
        BocloudResult getResult = V8VolumeGroupService.get(getAuth(), viosUuid, volumeGroupUuid);
        if (getResult.isFailed()) {
            logger.info("Hmc driver get volume group failed:{}", getResult);
            return getResult;
        }

        V8VolumeGroupEntry entry = (V8VolumeGroupEntry) getResult.getData();
        Optional<V8VirtualDisks> virtualDisksOpt = Optional.ofNullable(entry)
                .map(V8VolumeGroupEntry::getContent)
                .map(V8VolumeGroupContent::getVolumeGroup)
                .map(V8VolumeGroup::getVirtualDisks);
        if (!virtualDisksOpt.isPresent()) {
            logger.error("can't virtual disks of group with uuid {} ", volumeGroupUuid);
            return new BocloudResult(false, "存储池[" + volumeGroupUuid + "]的虚拟盘信息不存在");
        }
        V8VirtualDisks virtualDisks = virtualDisksOpt.get();
        List<V8VirtualDisk> disks = virtualDisksOpt.map(V8VirtualDisks::getDisks).orElse(new ArrayList<>());
        disks.removeIf(oldDisk -> diskName.equals(oldDisk.getDiskName()));
        virtualDisks.setDisks(disks);

        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        updateBean.setVirtualDisks(virtualDisks);

        BocloudResult result = V8VolumeGroupService.update(getAuth(), viosUuid, volumeGroupUuid, updateBean);
        if (result.isFailed()) {
            return result;
        }
        logger.info("Hmc driver completely delete virtual disk {}, result: {}", diskName, result.isSuccess());
        V8VolumeGroupEntry responseV8 = (V8VolumeGroupEntry) result.getData();
        VolumeGroupEntry response = JSON.parseObject(JSON.toJSONString(responseV8), VolumeGroupEntry.class);
        return new BocloudResult(true, response, "执行成功");
    }

    @Override
    public BocloudResult assign(String managedSystemUuid, String viosUuid, String lparUuid, String diskName, String diskCapacity, Integer viosId, Integer lparId, Integer lparSlot, boolean isPhysical) {
        if (diskName == null || diskCapacity == null) {
            logger.error("disk info {} , {} error.", diskName, diskCapacity);
            return new BocloudResult(false, "分配失败，磁盘信息不正确");
        }

        BocloudResult result = V8VirtualIOServerService.get(getAuth(), viosUuid);
        if (result.isFailed()) {
            logger.error("get vios info error:{}", result);
            return result;
        }
        V8VirtualIOServerEntry virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        V8VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V8VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V8VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V8VirtualSCSIMapping> newMappingList = new ArrayList<>();
        if (virtualScsiMappingList == null) {
            virtualScsiMappingList = new ArrayList<>();
        }

        V8VirtualSCSIMapping virtualScsiMapping = null;
        for (V8VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if (null != oldMap.getStorage() && null != oldMap.getStorage().getVirtualDisk() && diskName.equals(oldMap.getStorage().getVirtualDisk().getDiskName())) {
                // for virtual disk
                virtualScsiMapping = oldMap;
            } else if (null != oldMap.getStorage() && null != oldMap.getStorage().getPhysicalVolume() && diskName.equals(oldMap.getStorage().getPhysicalVolume().getVolumeName())) {
                // for physical disk
                virtualScsiMapping = oldMap;
            } else {
                newMappingList.add(oldMap);
            }
        }

        if (virtualScsiMapping == null) {
            // 不存在则新增
            V8Storage storage = new V8Storage();
            storage.setSchemaVersion(null);
            if (isPhysical) {
                storage.setPhysicalVolume(new V8PhysicalVolume(diskCapacity, diskName));
            } else {
                storage.setVirtualDisk(new V8VirtualDisk(diskCapacity, diskName));
            }
            V8ServerAdapter serverAdapter = new V8ServerAdapter(viosId.toString(), lparId == null ? null : lparId.toString(), "true");
            if (lparId != null) {
                V8ClientAdapter clientAdapter = new V8ClientAdapter(lparId.toString(), viosId.toString(), "true");
                clientAdapter.setVariedOn("true");
                if (lparSlot != null) {
                    clientAdapter.setVirtualSlotNumber(lparSlot.toString());
                    serverAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                virtualScsiMapping = new V8VirtualSCSIMapping(clientAdapter, serverAdapter, storage);
            } else {
                virtualScsiMapping = new V8VirtualSCSIMapping(serverAdapter, storage);
            }
        } else {
            // 已存在则更新分配信息
            if (null != virtualScsiMapping.getServerAdapter()) {
                V8ServerAdapter oldServerAdapter = virtualScsiMapping.getServerAdapter();
                oldServerAdapter.setRemoteLogicalPartitionId(lparId.toString());
                if (lparSlot != null) {
                    oldServerAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                oldServerAdapter.setVirtualSlotNumber(null);// 这句话必须的，不能删除
                virtualScsiMapping.setServerAdapter(oldServerAdapter);
            }
            V8ClientAdapter clientAdapter = new V8ClientAdapter(lparId.toString(), viosId.toString(), "true");
            clientAdapter.setVariedOn("true");
            if (lparSlot != null) {
                clientAdapter.setVirtualSlotNumber(lparSlot.toString());
            }
            virtualScsiMapping.setClientAdapter(clientAdapter);
            virtualScsiMapping.setAssociatedLogicalPartition(null);
        }
        newMappingList.add(virtualScsiMapping);
        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(virtualIOServerEntry), VirtualIOServerEntry.class);
        logger.info("Hmc driver completely assign disk {} to {}-{}, result: {}", diskName, lparId, lparSlot, result.isSuccess());
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult unAssign(String managedSystemUuid, String viosUuid, String diskName, boolean isPhysical) {
        BocloudResult result = V8VirtualIOServerService.get(getAuth(), viosUuid);
        if (result.isFailed()) {
            logger.error("get vios info error:{}", result);
            return result;
        }
        V8VirtualIOServerEntry virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        V8VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V8VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V8VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V8VirtualSCSIMapping> newMappingList = new ArrayList<>();
        if (virtualScsiMappingList == null) {
            logger.warn("VirtualSCSIMapping信息为空，取消分配成功");
            return new BocloudResult(true, "取消分配成功");
        }

        for (V8VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if (isPhysical) {
                if (null != diskName && null != oldMap.getStorage() && null != oldMap.getStorage().getPhysicalVolume() && diskName.equals(oldMap.getStorage().getPhysicalVolume().getVolumeName())) {
                    // 这里是该物理卷原来的分配信息,不做任何处理
                } else {
                    newMappingList.add(oldMap);
                }
            } else {
                if (null != diskName && null != oldMap.getStorage() && null != oldMap.getStorage().getVirtualDisk() && diskName.equals(oldMap.getStorage().getVirtualDisk().getDiskName())) {
                    // 这里是该虚拟磁盘原来的分配信息,不做任何处理
                } else {
                    newMappingList.add(oldMap);
                }
            }
        }

        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V8VirtualIOServerService.update(getAuth(), managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        VirtualIOServerEntry response = JSON.parseObject(JSON.toJSONString(virtualIOServerEntry), VirtualIOServerEntry.class);
        logger.info("Hmc driver completely unAssign disk {}, result: {}", diskName, result.isSuccess());
        return new BocloudResult(true, response, "执行成功");

    }

    @Override
    public BocloudResult createVolumeGroup(String viosUuid, VolumeGroup volumeGroup) {
        V8VolumeGroup V8VolumeGroup = JSON.parseObject(JSON.toJSONString(volumeGroup), V8VolumeGroup.class);
        BocloudResult bocloudResult = V8VolumeGroupService.create(getAuth(), viosUuid, V8VolumeGroup);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        V8VolumeGroupEntry entryV8 = (V8VolumeGroupEntry) bocloudResult.getData();
        VolumeGroupEntry entry = JSON.parseObject(JSON.toJSONString(entryV8), VolumeGroupEntry.class);
        return new BocloudResult(true, entry, "执行成功");
    }

    @Override
    public BocloudResult deleteVolumeGroup(String viosUuid, String volumeGroupUuid) {
        return V8VolumeGroupService.delete(getAuth(), viosUuid, volumeGroupUuid);
    }
}
