package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.model.hmc.sdk.module.HostEthernetAdapterEntry;
import com.bocloud.cmp.model.hmc.sdk.module.HostEthernetAdapterResponse;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8HostEthernetAdapterService;
import com.bocloud.cmp.provider.hmc.sdk.enums.HostEthernetAdapterQuickproperty;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterEntry;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterForUpdate;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterPhysicalPort;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.V8HostEthernetAdapterPortGroup;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.junit.Assert;
import org.junit.Test;

import java.util.List;

public class V8HostEthernetAdapterServiceTest extends BaseTest {

	@Test
	public void testGet1() {
		BocloudResult result = V8HostEthernetAdapterService.get(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
		HostEthernetAdapterResponse response = (HostEthernetAdapterResponse) result.getData();
		System.out.println(response.getEntries().get(0).getContent().getHostEthernetAdapter().getAdapterID());
	}

	@Test
	public void testGet2() {
		BocloudResult result = V8HostEthernetAdapterService.get(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a",
				"21360b8f-a0fe-3ecf-8a56-2c62020b61ec");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
		HostEthernetAdapterEntry response = (HostEthernetAdapterEntry) result.getData();
		System.out.println(response.getContent().getHostEthernetAdapter().getAdapterID());
	}

	@Test
	public void testGetQuickProperty() {
		BocloudResult result;
		HostEthernetAdapterQuickproperty[] values = HostEthernetAdapterQuickproperty.values();

		for (HostEthernetAdapterQuickproperty property : values) {
			result = V8HostEthernetAdapterService.getQuickProperty(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a",
					"21360b8f-a0fe-3ecf-8a56-2c62020b61ec", property);
			// Assert.assertTrue(result.getMessage(), result.isSuccess());
			System.out.println(property.name() + "=" + result.getData());
		}
	}

	@Test
	public void testUpdate() {
		V8HostEthernetAdapterForUpdate heaUpdate = new V8HostEthernetAdapterForUpdate();

		BocloudResult getResult = V8HostEthernetAdapterService.get(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a",
				"21360b8f-a0fe-3ecf-8a56-2c62020b61ec");
		Assert.assertTrue(getResult.getMessage(), getResult.isSuccess());
		V8HostEthernetAdapterEntry response = (V8HostEthernetAdapterEntry) getResult.getData();

		heaUpdate.setPortGroups(response.getContent().getHostEthernetAdapter().getPortGroups());

		List<V8HostEthernetAdapterPortGroup> portGroups = heaUpdate.getPortGroups().getHostEthernetAdapterPortGroup();
		for (V8HostEthernetAdapterPortGroup portGroup : portGroups) {
			portGroup.setLogicalPorts(null);
			V8HostEthernetAdapterPhysicalPort physicalPort = portGroup.getPhysicalPorts()
					.getHostEthernetAdapterPhysicalPort().get(0);
			physicalPort.setMaximumReceivePacketSize("Jumbo 9000 Byte");
		}

		BocloudResult result = V8HostEthernetAdapterService.update(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a",
				"21360b8f-a0fe-3ecf-8a56-2c62020b61ec", heaUpdate);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}
}
