package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionProfileResponse;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionProfileService;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionFileQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.DateUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.apache.commons.lang3.StringUtils;
import org.junit.Assert;
import org.junit.Test;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

public class V8LogicalPartitionProfileServiceTest extends BaseTest {
	private static String lparProfileUuid = null;
	private static String viosProfileUuid = null;
	private static String lparProfileName;
	private static String viosProfileName;

	@Test
	public void testGetByLogicalPartitionUuid() {
		BocloudResult result = V8LogicalPartitionProfileService.getByLogicalPartitionUuid(auth,
				"18EFEB4F-820B-4AB6-A638-263873EF9F90");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileResponse response = (LogicalPartitionProfileResponse) result.getData();
		System.out.println(response);
	}

	@Test
	public void testGetByLogicalPartitionUuid2() {
		BocloudResult result = V8LogicalPartitionProfileService.getByLogicalPartitionUuid(auth,
//				"0DB94AE2-E59D-44AA-88A5-A1D29D383A9E", "678f32c0-b2b0-3896-b5ec-2009fb584cd6");
				
				"1E35D624-31D1-4D5F-A67F-4CE875CF016C","ad9ffadf-998a-36d5-a734-88cadeb62166");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetByVirtualIOServerUuid() {
//		5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3  sxp-vios
//		6D18F91B-6BCB-4631-B9B4-BFBCF0111B55  vios
		BocloudResult result = V8LogicalPartitionProfileService.getByVirtualIOServerUuid(auth,
				"5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetByVirtualIOServerUuid2() {
		BocloudResult result = V8LogicalPartitionProfileService.getByVirtualIOServerUuid(auth,
				"5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3", "2f1e6ecd-e4cb-3785-a5ab-8d3ecbeec7e9");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetQuickPropertyByLogicalPartitionUuid2() {
		BocloudResult result;
		LogicalPartitionFileQuickProperty[] values = LogicalPartitionFileQuickProperty.values();

		for (LogicalPartitionFileQuickProperty property : values) {
			result = V8LogicalPartitionProfileService.getQuickPropertyByLogicalPartitionUuid(auth,
					"74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E", "c2223fcf-b529-3dd7-a9ec-fa75ba16a75a", property);
			Assert.assertTrue(result.getMessage(), result.isSuccess());
			System.out.println(property.name() + "=" + result.getData());
		}
	}

	@Test
	public void testGetQuickPropertyByVirtualIOServerUuid2() {
		BocloudResult result;
		LogicalPartitionFileQuickProperty[] values = LogicalPartitionFileQuickProperty.values();

		for (LogicalPartitionFileQuickProperty property : values) {
			result = V8LogicalPartitionProfileService.getQuickPropertyByVirtualIOServerUuid(auth,
					"00646977-6ADC-48AC-ACBF-FE6A11A893B0", "bfc04c7e-a3a0-3872-8943-6755395f5c0d", property);
			Assert.assertTrue(result.getMessage(), result.isSuccess());
			System.out.println(property.name() + "=" + result.getData());
		}
	}

	@Test
	public void testCreateLpar() {
		if (StringUtils.isNotEmpty(lparProfileUuid)) {
			return;
		}

		boolean isVIOS = false;
		String partitionUuid = "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E";
		V8LogicalPartitionProfile profile = new V8LogicalPartitionProfile();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.3");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.3");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		Date today = Calendar.getInstance().getTime();
		String suffix = DateUtils.dateToString(today, "HHmmss");
		lparProfileName = "sdk-profile-" + suffix;
		profile.setProfileName(lparProfileName);

		BocloudResult result = V8LogicalPartitionProfileService.create(auth, isVIOS, partitionUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileEntry entry = (LogicalPartitionProfileEntry) result.getData();
		lparProfileUuid = entry.getId();

		System.out.println("lparProfileUuid=" + lparProfileUuid);
	}

	@Test
	public void testUpdateLpar() {
		// lparProfileUuid = "7ae3e558-3679-38b1-b518-e49e0c14aab4";
		if (StringUtils.isEmpty(lparProfileUuid)) {
			testCreateLpar();
		}

		boolean isVIOS = false;
		String partitionUuid = "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E";
		V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();
		updateProfile.setProfileName(lparProfileName + "-modify");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, lparProfileUuid,
				updateProfile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	@Test
	public void testDeleteLpar() {
		// lparProfileUuid = "7ae3e558-3679-38b1-b518-e49e0c14aab4";
		if (StringUtils.isEmpty(lparProfileUuid)) {
			testCreateLpar();
		}

		boolean isVIOS = false;
		String partitionUuid = "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E";
		BocloudResult result = V8LogicalPartitionProfileService.delete(auth, isVIOS, partitionUuid, lparProfileUuid);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	@Test
	public void testCreateVios() {
		if (StringUtils.isNotEmpty(viosProfileUuid)) {
			return;
		}

		boolean isVIOS = true;
		String partitionUuid = "00646977-6ADC-48AC-ACBF-FE6A11A893B0";
		V8LogicalPartitionProfile profile = new V8LogicalPartitionProfile();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.3");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.3");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		Date today = Calendar.getInstance().getTime();
		String suffix = DateUtils.dateToString(today, "HHmmss");
		viosProfileName = "sdk-profile-" + suffix;
		profile.setProfileName(viosProfileName);

		BocloudResult result = V8LogicalPartitionProfileService.create(auth, isVIOS, partitionUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileEntry entry = (LogicalPartitionProfileEntry) result.getData();
		viosProfileUuid = entry.getId();

		System.out.println("viosProfileUuid=" + viosProfileUuid);
	}

	@Test
	public void testUpdateVios() {
		// viosProfileUuid = "7ae3e558-3679-38b1-b518-e49e0c14aab4";
		if (StringUtils.isEmpty(viosProfileUuid)) {
			testCreateVios();
		}

		boolean isVIOS = true;
		String partitionUuid = "00646977-6ADC-48AC-ACBF-FE6A11A893B0";
		V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();
		updateProfile.setProfileName(viosProfileName + "-modify");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, viosProfileUuid,
				updateProfile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	@Test
	public void testDeleteVios() {
		// viosProfileUuid = "555f990b-1c5b-3284-9eb7-6d70d88168f5";
		if (StringUtils.isEmpty(viosProfileUuid)) {
			testCreateVios();
		}

		boolean isVIOS = true;
		String partitionUuid = "00646977-6ADC-48AC-ACBF-FE6A11A893B0";
		BocloudResult result = V8LogicalPartitionProfileService.delete(auth, isVIOS, partitionUuid, viosProfileUuid);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	/**
	 * 创建逻辑分区虚拟以太网适配器和虚拟SCSI适配器
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testCreateLparVirtualIOAdapters() {
		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";
		V8LogicalPartitionProfile profile = new V8LogicalPartitionProfile();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.1");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.1");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		profile.setProfileName("sxp-test");

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("10");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(4);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("2");// 必选项
		networkAdapter.setPortVlanID("6");// 必选项

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIClientAdapter scsiAdapter = new V8ProfileVirtualSCSIClientAdapter();
		scsiSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("3");// 必选项
		scsiAdapter.setRemotePartitionID("1");// 必选项
		scsiAdapter.setRemoteSlotNumber("15");// 必选项

		// 虚拟光纤通道适配器
		V8ProfileVirtualIOAdapterSubclass fclSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(fclSubclass);

		V8ProfileVirtualFibreChannelClientAdapter fclAdapter = new V8ProfileVirtualFibreChannelClientAdapter();
		fclSubclass.setProfileVirtualFibreChannelClientAdapter(fclAdapter);

		fclAdapter.setVirtualSlotNumber("5");// 必选项
		fclAdapter.setConnectingPartitionID("1");// 必选项
		fclAdapter.setConnectingVirtualSlotNumber("11");// 必选项

		BocloudResult result = V8LogicalPartitionProfileService.create(auth, isVIOS, partitionUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileEntry entry = (LogicalPartitionProfileEntry) result.getData();
		String profileId = entry.getId();

		System.out.println("profileId=" + profileId);
	}

	/**
	 * 更新逻辑分区虚拟以太网适配器和虚拟SCSI适配器
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testUpdateLparVirtualIOAdapters() {
		String profileId = "5f5094c5-515c-3d60-b6a4-d368d99e75be";

		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";

		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("10");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(3);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("3");// 必选项
		networkAdapter.setPortVlanID("7");// 必选项
		networkAdapter.setIsRequired("true");
		networkAdapter.setIsTaggedVlanSupported("true");
		networkAdapter.setTaggedVlanIds("3 4");

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIClientAdapter scsiAdapter = new V8ProfileVirtualSCSIClientAdapter();
		scsiSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("4");// 必选项
		scsiAdapter.setRemotePartitionID("1");// 必选项
		scsiAdapter.setRemoteSlotNumber("16");// 必选项
		scsiAdapter.setIsRequired("true");

		// 虚拟光纤通道适配器
		V8ProfileVirtualIOAdapterSubclass fclSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(fclSubclass);

		V8ProfileVirtualFibreChannelClientAdapter fclAdapter = new V8ProfileVirtualFibreChannelClientAdapter();
		fclSubclass.setProfileVirtualFibreChannelClientAdapter(fclAdapter);

		fclAdapter.setVirtualSlotNumber("5");// 必选项
		fclAdapter.setConnectingPartitionID("1");// 必选项
		fclAdapter.setConnectingVirtualSlotNumber("11");// 必选项
		fclAdapter.setIsRequired("true");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileId, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	/**
	 * 删除逻辑分区虚拟以太网适配器
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testDeleteLparClientNetworkAdapter() {
		String profileId = "5f5094c5-515c-3d60-b6a4-d368d99e75be";

		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";

		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("10");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(2);
		adapters.setSubclasses(subclasses);

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIClientAdapter scsiAdapter = new V8ProfileVirtualSCSIClientAdapter();
		scsiSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("4");// 必选项
		scsiAdapter.setRemotePartitionID("1");// 必选项
		scsiAdapter.setRemoteSlotNumber("16");// 必选项
		scsiAdapter.setIsRequired("true");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileId, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	/**
	 * 删除逻辑分区虚拟SCSI适配器
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testDeleteLparAndVirtualSCSIClientAdapter() {
		String profileId = "5f5094c5-515c-3d60-b6a4-d368d99e75be";

		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";

		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("10");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(2);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("3");// 必选项
		networkAdapter.setPortVlanID("7");// 必选项
		networkAdapter.setIsRequired("true");
		networkAdapter.setIsTaggedVlanSupported("true");
		networkAdapter.setTaggedVlanIds("3 4");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileId, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

	/**
	 * 创建逻辑分区虚拟以太网适配器(isQualityOfServicePriorityEnabled=true)
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testCreateLparTrunkAdapter() {
		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";
		V8LogicalPartitionProfile profile = new V8LogicalPartitionProfile();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.1");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.1");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("9");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(2);
		adapters.setSubclasses(subclasses);

		V8ProfileVirtualIOAdapterSubclass subclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(subclass);

		V8ProfileTrunkAdapter adapter = new V8ProfileTrunkAdapter();
		subclass.setProfileTrunkAdapter(adapter);

		// adapter.setIsQualityOfServicePriorityEnabled("true");
		adapter.setTrunkPriority("2");
		adapter.setVirtualSlotNumber("3");// 必选项
		adapter.setPortVlanID("6");// 必选项

		profile.setProfileName("sxp-TrunkAdapter");// uuid=e214fd79-ee57-3f22-97b9-1d3ae22fc44d

		BocloudResult result = V8LogicalPartitionProfileService.create(auth, isVIOS, partitionUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileEntry entry = (LogicalPartitionProfileEntry) result.getData();
		String profileId = entry.getId();

		System.out.println("profileId=" + profileId);
	}

	/**
	 * 更新逻辑分区虚拟以太网适配器(isQualityOfServicePriorityEnabled=true)
	 * 
	 * @author sxp
	 * @since Jun 29, 2018
	 */
	@Test
	public void testUpdateLparTrunkAdapter() {
		String profileId = "e214fd79-ee57-3f22-97b9-1d3ae22fc44d";

		boolean isVIOS = false;
		String partitionUuid = "553947E3-E6FC-42C4-8FF0-ACC319AD3D8B";

		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("8");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(2);
		adapters.setSubclasses(subclasses);

		V8ProfileVirtualIOAdapterSubclass subclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(subclass);

		V8ProfileTrunkAdapter adapter = new V8ProfileTrunkAdapter();
		subclass.setProfileTrunkAdapter(adapter);

		// adapter.setIsQualityOfServicePriorityEnabled("true");
		adapter.setVirtualSlotNumber("4");// 必选项
		adapter.setPortVlanID("7");// 必选项
		adapter.setIsRequired("true");

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileId, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}
	
	/**
	 * 创建VIOS虚拟适配器
	 * 
	 * @author sxp
	 * @since Jul 4, 2018
	 */
	@Test
	public void testCreateViosVirtualIOAdapters() {
		boolean isVIOS = true;
		String partitionUuid = "5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3";
		V8LogicalPartitionProfile profile = new V8LogicalPartitionProfile();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.2");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.3");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		String viosProfileName = "sdk-sxp-vios-profile";
		profile.setProfileName(viosProfileName);
		
		// 虚拟适配器
		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);
		instance.setMaximumVirtualIoSlots("20");

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(4);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("12");// 必选项
		networkAdapter.setPortVlanID("1");// 必选项

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIServerAdapter scsiAdapter = new V8ProfileVirtualSCSIServerAdapter();
		scsiSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("14");// 必选项
		scsiAdapter.setLocalPartitionID("11");// 必选项
		

		// 虚拟光纤通道适配器
		V8ProfileVirtualIOAdapterSubclass fcSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(fcSubclass);

		V8ProfileVirtualFibreChannelServerAdapter fcAdapter = new V8ProfileVirtualFibreChannelServerAdapter();
		fcSubclass.setProfileVirtualFibreChannelServerAdapter(fcAdapter);

		fcAdapter.setVirtualSlotNumber("13");// 必选项
		fcAdapter.setConnectingPartitionID("1");// 必选项
		fcAdapter.setConnectingVirtualSlotNumber("11");// 必选项
		

		BocloudResult result = V8LogicalPartitionProfileService.create(auth, isVIOS, partitionUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionProfileEntry entry = (LogicalPartitionProfileEntry) result.getData();
		String viosProfileUuid = entry.getId();

		System.out.println("viosProfileUuid=" + viosProfileUuid);
	}
	
	/**
	 * 修改VIOS虚拟适配器
	 * 
	 * @author sxp
	 * @since Jul 4, 2018
	 */
	@Test
	public void testUpdateViosVirtualIOAdapters() {
		boolean isVIOS = true;		
		String partitionUuid = "5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3";
		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.2");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.3");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		String viosProfileName = "sdk-sxp-vios-profile";
		profile.setProfileName(viosProfileName);
		
		// 虚拟适配器
		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);
		instance.setMaximumVirtualIoSlots("20");

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(4);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("11");// 必选项
		networkAdapter.setPortVlanID("2");// 必选项

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIServerAdapter scsiAdapter = new V8ProfileVirtualSCSIServerAdapter();
		scsiSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("12");// 必选项
		scsiAdapter.setLocalPartitionID("11");// 必选项
		

		// 虚拟光纤通道适配器
		V8ProfileVirtualIOAdapterSubclass fcSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(fcSubclass);

		V8ProfileVirtualFibreChannelServerAdapter fcAdapter = new V8ProfileVirtualFibreChannelServerAdapter();
		fcSubclass.setProfileVirtualFibreChannelServerAdapter(fcAdapter);

		fcAdapter.setVirtualSlotNumber("13");// 必选项
		fcAdapter.setConnectingPartitionID("1");// 必选项
		fcAdapter.setConnectingVirtualSlotNumber("11");// 必选项
		
		String profileUuid = "41aacb9a-23e9-3b4b-b3cb-cce99e0f80e4";

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}
	
	/**
	 * 删除VIOS虚拟适配器
	 * 
	 * @author sxp
	 * @since Jul 4, 2018
	 */
	@Test
	public void testDeleteViosVirtualIOAdapters() {
		boolean isVIOS = true;		
		String partitionUuid = "5FFC8CE9-E0B5-4E49-9C37-85149AE2AFC3";
		V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

		V8ProcessorAttributes cpuConfig = new V8ProcessorAttributes();
		cpuConfig.setHasDedicatedProcessors("false");
		V8SharedProcessorConfiguration sharedConfig = new V8SharedProcessorConfiguration();
		cpuConfig.setSharedProcessorConfiguration(sharedConfig);
		sharedConfig.setDesiredProcessingUnits("0.2");
		sharedConfig.setDesiredVirtualProcessors("1");
		sharedConfig.setMaximumProcessingUnits("0.3");
		sharedConfig.setMaximumVirtualProcessors("1");
		sharedConfig.setMinimumProcessingUnits("0.1");
		sharedConfig.setMinimumVirtualProcessors("1");
		cpuConfig.setSharingMode("capped");
		profile.setProcessorAttributes(cpuConfig);

		V8ProfileMemory profileMemory = new V8ProfileMemory();
		profileMemory.setDesiredMemory("128");
		profileMemory.setMaximumMemory("128");
		profileMemory.setMinimumMemory("128");
		profile.setProfileMemory(profileMemory);

		String viosProfileName = "sdk-sxp-vios-profile";
		profile.setProfileName(viosProfileName);
		
		// 虚拟适配器
		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		profile.setIoConfigurationInstance(instance);
		instance.setMaximumVirtualIoSlots("20");

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(4);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("11");// 必选项
		networkAdapter.setPortVlanID("2");// 必选项

//		// SCSI适配器
//		ProfileVirtualIOAdapterSubclass scsiSubclass = new ProfileVirtualIOAdapterSubclass();
//		subclasses.add(scsiSubclass);
//
//		ProfileVirtualSCSIServerAdapter scsiAdapter = new ProfileVirtualSCSIServerAdapter();
//		scsiSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);
//
//		scsiAdapter.setVirtualSlotNumber("12");// 必选项
//		scsiAdapter.setLocalPartitionID("11");// 必选项
//		
//
//		// 虚拟光纤通道适配器
//		ProfileVirtualIOAdapterSubclass fcSubclass = new ProfileVirtualIOAdapterSubclass();
//		subclasses.add(fcSubclass);
//
//		ProfileVirtualFibreChannelServerAdapter fcAdapter = new ProfileVirtualFibreChannelServerAdapter();
//		fcSubclass.setProfileVirtualFibreChannelServerAdapter(fcAdapter);
//
//		fcAdapter.setVirtualSlotNumber("13");// 必选项
//		fcAdapter.setConnectingPartitionID("1");// 必选项
//		fcAdapter.setConnectingVirtualSlotNumber("11");// 必选项
		
		String profileUuid = "41aacb9a-23e9-3b4b-b3cb-cce99e0f80e4";

		BocloudResult result = V8LogicalPartitionProfileService.update(auth, isVIOS, partitionUuid, profileUuid, profile);

		Assert.assertTrue(result.getMessage(), result.isSuccess());
	}

}
