package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionEntry;
import com.bocloud.cmp.model.hmc.sdk.module.LogicalPartitionResponse;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.enums.LogicalPartitionQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.junit.Assert;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class V8LogicalPartitionServiceTest extends BaseTest {

	@Test
	public void testGet1() {
		BocloudResult result = V8LogicalPartitionService.get(auth);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGet2() {
		BocloudResult result = V8LogicalPartitionService.get(auth, "0DB94AE2-E59D-44AA-88A5-A1D29D383A9E");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetByManagedSystemUuid1() {
		BocloudResult result = V8LogicalPartitionService.getByManagedSystemUuid(auth,
				"25cc2a4e-b946-3b98-acaa-cdf90628809a");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetByManagedSystemUuid2() {
		BocloudResult result = V8LogicalPartitionService.getByManagedSystemUuid(auth,
				"25cc2a4e-b946-3b98-acaa-cdf90628809a", "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetQuickProperty() {
		BocloudResult result;
		LogicalPartitionQuickProperty[] values = LogicalPartitionQuickProperty.values();

		for (LogicalPartitionQuickProperty property : values) {
			result = V8LogicalPartitionService.getQuickProperty(auth, "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E", property);
			// Assert.assertTrue(result.getMessage(), result.isSuccess());
			System.out.println(property.name() + "=" + result.getData());
		}
	}

	// @Test
	public void testCUD() {
		testCreate();
		testUpdate();
		testDelete();
	}

	public void testCreate() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";

		V8LogicalPartition lpar = new V8LogicalPartition();
		lpar.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		lpar.setPartitionType("AIX/Linux");
		lpar.setPartitionName("lpar-bocloud.sdk");

		V8PartitionMemoryConfiguration memConf = new V8PartitionMemoryConfiguration();
		memConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		memConf.setDesiredMemory("256");
		memConf.setMinimumMemory("128");
		memConf.setMaximumMemory("256");
		lpar.setPartitionMemoryConfiguration(memConf);

		V8PartitionProcessorConfiguration cpuConf = new V8PartitionProcessorConfiguration();
		cpuConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		cpuConf.setHasDedicatedProcessors("false");
		cpuConf.setSharingMode("capped");

		V8SharedProcessorConfiguration shareConf = new V8SharedProcessorConfiguration();
		shareConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		shareConf.setDesiredProcessingUnits("0.2");
		shareConf.setMinimumProcessingUnits("0.1");
		shareConf.setMaximumProcessingUnits("0.2");
		shareConf.setDesiredVirtualProcessors("1");
		shareConf.setMinimumVirtualProcessors("1");
		shareConf.setMaximumVirtualProcessors("1");
		

		cpuConf.setSharedProcessorConfiguration(shareConf);
		lpar.setPartitionProcessorConfiguration(cpuConf);

		BocloudResult result = V8LogicalPartitionService.create(auth, managedSystemUuid, lpar);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	 @Test
	public void testCreate2() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";

		V8LogicalPartition lpar = new V8LogicalPartition();
		lpar.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		lpar.setPartitionType("AIX/Linux");
		lpar.setPartitionName("lpar-bocloud.sdk.create2");

		V8PartitionMemoryConfiguration memConf = new V8PartitionMemoryConfiguration();
		memConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		memConf.setDesiredMemory("256");
		memConf.setMinimumMemory("128");
		memConf.setMaximumMemory("256");
		lpar.setPartitionMemoryConfiguration(memConf);

		V8PartitionProcessorConfiguration cpuConf = new V8PartitionProcessorConfiguration();
		cpuConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		cpuConf.setHasDedicatedProcessors("false");
		cpuConf.setSharingMode("capped");

		V8SharedProcessorConfiguration shareConf = new V8SharedProcessorConfiguration();
		shareConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
		shareConf.setDesiredProcessingUnits("0.2");
		shareConf.setMinimumProcessingUnits("0.1");
		shareConf.setMaximumProcessingUnits("0.2");
		shareConf.setDesiredVirtualProcessors("1");
		shareConf.setMinimumVirtualProcessors("1");
		shareConf.setMaximumVirtualProcessors("1");

		cpuConf.setSharedProcessorConfiguration(shareConf);
		lpar.setPartitionProcessorConfiguration(cpuConf);

		// 概要文件
		V8LogicalPartitionProfileForUpdate updateProfile = getProfile();

		BocloudResult result = V8LogicalPartitionService.create(auth, managedSystemUuid, lpar, updateProfile);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	private V8LogicalPartitionProfileForUpdate getProfile() {
		V8LogicalPartitionProfileForUpdate updateProfile = new V8LogicalPartitionProfileForUpdate();

		V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
		updateProfile.setIoConfigurationInstance(instance);

		instance.setMaximumVirtualIoSlots("30");// 必选项

		V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
		instance.setProfileVirtualIoAdapters(adapters);

		List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(3);
		adapters.setSubclasses(subclasses);

		// 以太网适配器
		V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(networkSubclass);

		V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
		networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

		networkAdapter.setVirtualSlotNumber("3");// 必选项
		networkAdapter.setPortVlanID("7");// 必选项
		networkAdapter.setIsRequired("true");
		networkAdapter.setIsTaggedVlanSupported("true");
		networkAdapter.setTaggedVlanIds("3 4");

		// SCSI适配器
		V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(scsiSubclass);

		V8ProfileVirtualSCSIClientAdapter scsiAdapter = new V8ProfileVirtualSCSIClientAdapter();
		scsiSubclass.setProfileVirtualSCSIClientAdapter(scsiAdapter);

		scsiAdapter.setVirtualSlotNumber("4");// 必选项
		scsiAdapter.setRemotePartitionID("1");// 必选项
		scsiAdapter.setRemoteSlotNumber("16");// 必选项
		scsiAdapter.setIsRequired("true");

		// 虚拟光纤通道适配器
		V8ProfileVirtualIOAdapterSubclass fclSubclass = new V8ProfileVirtualIOAdapterSubclass();
		subclasses.add(fclSubclass);

		V8ProfileVirtualFibreChannelClientAdapter fclAdapter = new V8ProfileVirtualFibreChannelClientAdapter();
		fclSubclass.setProfileVirtualFibreChannelClientAdapter(fclAdapter);

		fclAdapter.setVirtualSlotNumber("5");// 必选项
		fclAdapter.setConnectingPartitionID("1");// 必选项
		fclAdapter.setConnectingVirtualSlotNumber("11");// 必选项
		fclAdapter.setIsRequired("true");
		return updateProfile;
	}

	public void testUpdate() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";
		String lparUuid = null;

		BocloudResult result = V8LogicalPartitionService.get(auth);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionResponse response = (LogicalPartitionResponse) result.getData();
		List<LogicalPartitionEntry> entries = response.getEntries();

		if (entries != null) {
			String PARTITIONNAME = "lpar-bocloud.sdk";
			String name;
			for (LogicalPartitionEntry entry : entries) {
				try {
					name = entry.getContent().getLogicalPartition().getPartitionName();
				} catch (NullPointerException e) {
					continue;
				}
				if (PARTITIONNAME.equals(name)) {
					lparUuid = entry.getId();
				}
			}
			Assert.assertTrue("未找到名称为[" + PARTITIONNAME + "]的逻辑分区", lparUuid != null);
		}

		V8LogicalPartitionForUpdate updateLpar = new V8LogicalPartitionForUpdate();
		updateLpar.setPartitionName("lpar-bocloud.sdk-update");
		updateLpar.setKeylockPosition("manual");

		result = V8LogicalPartitionService.update(auth, managedSystemUuid, lparUuid, updateLpar);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());

	}

	public void testDelete() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";
		String lparUuid = null;

		BocloudResult result = V8LogicalPartitionService.get(auth);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		LogicalPartitionResponse response = (LogicalPartitionResponse) result.getData();
		List<LogicalPartitionEntry> entries = response.getEntries();

		if (entries != null) {
			String PARTITIONNAME = "lpar-bocloud.sdk-update";
			String name;
			for (LogicalPartitionEntry entry : entries) {
				try {
					name = entry.getContent().getLogicalPartition().getPartitionName();
				} catch (NullPointerException e) {
					continue;
				}
				if (PARTITIONNAME.equals(name)) {
					lparUuid = entry.getId();
				}
			}
			Assert.assertTrue("未找到名称为[" + PARTITIONNAME + "]的逻辑分区", lparUuid != null);
		}

		result = V8LogicalPartitionService.delete(auth, managedSystemUuid, lparUuid);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	// @Test
	public void powerOnPartition() {
		BocloudResult result = V8LogicalPartitionService.powerOnPartition(auth, "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	// @Test
	public void powerOffPartition() {
		BocloudResult result = V8LogicalPartitionService.powerOffPartition(auth, "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	 @Test
	public void getVfc() {
		BocloudResult result = V8LogicalPartitionService.getVfc(auth, "74A2E73F-9AD4-47A5-8CA2-8D3CDA37C49E");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

}
