package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.entity.enums.HmcPartitionType;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8LogicalPartitionService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8ManagedSystemService;
import com.bocloud.cmp.provider.hmc.sdk.enums.ManagedSystemQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.junit.Assert;
import org.junit.Test;

public class V8ManagedSystemServiceTest extends BaseTest {
	@Test
	public void testGet() {
		BocloudResult result = V8ManagedSystemService.get(auth);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetByUuid() {
		BocloudResult result = V8ManagedSystemService.getByUuid(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a");
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void testGetQuickProperty() {
		BocloudResult result;
		ManagedSystemQuickProperty[] values = ManagedSystemQuickProperty.values();

		for (ManagedSystemQuickProperty property : values) {
			result = V8ManagedSystemService.getQuickProperty(auth, "25cc2a4e-b946-3b98-acaa-cdf90628809a", property);
			// Assert.assertTrue(result.getMessage(), result.isSuccess());
			System.out.println(property.name() + "=" + result.getData());
		}
	}

	//@Test
	public void powerOffManagedSystem() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";
		BocloudResult result = V8ManagedSystemService.powerOffManagedSystem(auth, managedSystemUuid);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	//@Test
	public void powerOnManagedSystem() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";
		BocloudResult result = V8ManagedSystemService.powerOnManagedSystem(auth, managedSystemUuid);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

	@Test
	public void update() {
		String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";

		// 寻找一个IBMI的分区
		String ibmiHref = null;
		BocloudResult result = V8LogicalPartitionService.getByManagedSystemUuid(auth, managedSystemUuid);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		V8LogicalPartitionResponse lpars = (V8LogicalPartitionResponse) result.getData();
		for (V8LogicalPartitionEntry entry : lpars.getEntries()) {
			if (HmcPartitionType.IBMI.getType().equals(entry.getContent().getLogicalPartition().getPartitionType())) {
				for (V8Link link : entry.getLink()) {
					if ("SELF".equals(link.getRel())) {
						ibmiHref = link.getHref();
					}
				}
			}
		}
		Assert.assertNotNull("未找到iBMI分区", ibmiHref);

		V8ManagedSystem system = new V8ManagedSystem();
		system.setSystemName("Server-8203");
		system.setSystemLocation(new V8SystemLocation());
		system.getSystemLocation().setValue("this is system location test by lyc");
		system.setDescription(new V8Description());
		system.getDescription().setValue("this is description test by lyc");
		system.setServicePartition(new V8ServicePartition());
		system.getServicePartition().setHref(ibmiHref);
		system.setAssociatedIPLConfiguration(new V8AssociatedIPLConfiguration());
		system.getAssociatedIPLConfiguration().setPowerOffWhenLastLogicalPartitionIsShutdown("false");
		// 供电端: Temporary 临时  Permanent 永久
		system.getAssociatedIPLConfiguration().setPendingPowerOnSide("Temporary");
		// 键锁位置: normal 正常  manual 手动
		system.getAssociatedIPLConfiguration().setPendingSystemKeylock("normal");
		// 分区启动策略: autostart 始终自动启动  userinit 由用户启动   autorecovery 自动启动以执行自动回复
		system.getAssociatedIPLConfiguration().setPendingPowerOnLogicalPartitionStartPolicy(new V8PendingPowerOnLogicalPartitionStartPolicy());
		system.getAssociatedIPLConfiguration().getPendingPowerOnLogicalPartitionStartPolicy().setValue("autorecovery");

		result = V8ManagedSystemService.update(auth, managedSystemUuid, system);
		Assert.assertTrue(result.getMessage(), result.isSuccess());
		System.out.println(result.getData());
	}

}
