package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VirtualIOServerResponse;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.enums.VirtualIOServerQuickProperty;
import com.bocloud.cmp.provider.hmc.sdk.model.HmcSdkConstant;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.junit.Assert;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

/**
 * Test for VirtualIOServerService
 *
 * @author sxp
 * @since May 11, 2018
 */
public class V8VirtualIOServerServiceTest extends BaseTest {

    private static final String viosUuid = "1C29BA46-9158-4397-ACA5-B0A067BF1E49";
    private static final String managedSystemUuid = "25cc2a4e-b946-3b98-acaa-cdf90628809a";

    @Test
    public void testGet() {
        BocloudResult result = V8VirtualIOServerService.get(auth);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void testGet2() {
        BocloudResult result = V8VirtualIOServerService.get(auth, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void testGetQuickProperty() {
        BocloudResult result;
        VirtualIOServerQuickProperty[] values = VirtualIOServerQuickProperty.values();

        for (VirtualIOServerQuickProperty property : values) {
            result = V8VirtualIOServerService.getQuickProperty(auth, viosUuid, property);
            // Assert.assertTrue(result.getMessage(), result.isSuccess());
            System.out.println(property.name() + "=" + result.getData());
        }
    }

    // @Test
    public void testCUD() {
        testCreate();
        testUpdate();
        testDelete();
    }

    public void testCreate() {
        V8VirtualIOServer vios = new V8VirtualIOServer();
        vios.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        vios.setPartitionType("Virtual IO Server");
        vios.setPartitionName("vios-bocloud.sdk");

        V8PartitionMemoryConfiguration memConf = new V8PartitionMemoryConfiguration();
        memConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        memConf.setDesiredMemory("256");
        memConf.setMinimumMemory("256");
        memConf.setMaximumMemory("256");
        vios.setPartitionMemoryConfiguration(memConf);

        V8PartitionProcessorConfiguration cpuConf = new V8PartitionProcessorConfiguration();
        cpuConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        cpuConf.setHasDedicatedProcessors("false");
        cpuConf.setSharingMode("capped");

        V8SharedProcessorConfiguration shareConf = new V8SharedProcessorConfiguration();
        shareConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        shareConf.setDesiredProcessingUnits("0.5");
        shareConf.setDesiredVirtualProcessors("1");
        shareConf.setMaximumProcessingUnits("0.5");
        shareConf.setMaximumVirtualProcessors("1");
        shareConf.setMinimumProcessingUnits("0.5");
        shareConf.setMinimumVirtualProcessors("1");

        cpuConf.setSharedProcessorConfiguration(shareConf);
        vios.setPartitionProcessorConfiguration(cpuConf);

        BocloudResult result = V8VirtualIOServerService.create(auth, managedSystemUuid, vios);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void testCreate2() {
        V8VirtualIOServer vios = new V8VirtualIOServer();
        vios.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        vios.setPartitionType("Virtual IO Server");
        vios.setPartitionName("vios-bocloud.sdk.create2");

        V8PartitionMemoryConfiguration memConf = new V8PartitionMemoryConfiguration();
        memConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        memConf.setDesiredMemory("256");
        memConf.setMinimumMemory("256");
        memConf.setMaximumMemory("256");
        vios.setPartitionMemoryConfiguration(memConf);

        V8PartitionProcessorConfiguration cpuConf = new V8PartitionProcessorConfiguration();
        cpuConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        cpuConf.setHasDedicatedProcessors("false");
        cpuConf.setSharingMode("capped");

        V8SharedProcessorConfiguration shareConf = new V8SharedProcessorConfiguration();
        shareConf.setSchemaVersion(HmcSdkConstant.SCHEMA_VERSION_V120);
        shareConf.setDesiredProcessingUnits("0.5");
        shareConf.setDesiredVirtualProcessors("1");
        shareConf.setMaximumProcessingUnits("0.5");
        shareConf.setMaximumVirtualProcessors("1");
        shareConf.setMinimumProcessingUnits("0.5");
        shareConf.setMinimumVirtualProcessors("1");

        cpuConf.setSharedProcessorConfiguration(shareConf);
        vios.setPartitionProcessorConfiguration(cpuConf);

        // 概要文件
        V8LogicalPartitionProfileForUpdate updateProfile = getProfile();

        BocloudResult result = V8VirtualIOServerService.create(auth, managedSystemUuid, vios, updateProfile);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    private V8LogicalPartitionProfileForUpdate getProfile() {
        V8LogicalPartitionProfileForUpdate profile = new V8LogicalPartitionProfileForUpdate();

        // 虚拟适配器
        V8IOConfigurationInstance instance = new V8IOConfigurationInstance();
        profile.setIoConfigurationInstance(instance);
        instance.setMaximumVirtualIoSlots("20");

        V8ProfileVirtualIOAdapters adapters = new V8ProfileVirtualIOAdapters();
        instance.setProfileVirtualIoAdapters(adapters);

        List<V8ProfileVirtualIOAdapterSubclass> subclasses = new ArrayList<>(4);
        adapters.setSubclasses(subclasses);

        // 以太网适配器
        V8ProfileVirtualIOAdapterSubclass networkSubclass = new V8ProfileVirtualIOAdapterSubclass();
        subclasses.add(networkSubclass);

        V8ProfileClientNetworkAdapter networkAdapter = new V8ProfileClientNetworkAdapter();
        networkSubclass.setProfileClientNetworkAdapter(networkAdapter);

        networkAdapter.setVirtualSlotNumber("12");// 必选项
        networkAdapter.setPortVlanID("1");// 必选项

        // SCSI适配器
        V8ProfileVirtualIOAdapterSubclass scsiSubclass = new V8ProfileVirtualIOAdapterSubclass();
        subclasses.add(scsiSubclass);

        V8ProfileVirtualSCSIServerAdapter scsiAdapter = new V8ProfileVirtualSCSIServerAdapter();
        scsiSubclass.setProfileVirtualSCSIServerAdapter(scsiAdapter);

        scsiAdapter.setVirtualSlotNumber("14");// 必选项
        scsiAdapter.setLocalPartitionID("11");// 必选项

        // 虚拟光纤通道适配器
        V8ProfileVirtualIOAdapterSubclass fcSubclass = new V8ProfileVirtualIOAdapterSubclass();
        subclasses.add(fcSubclass);

        V8ProfileVirtualFibreChannelServerAdapter fcAdapter = new V8ProfileVirtualFibreChannelServerAdapter();
        fcSubclass.setProfileVirtualFibreChannelServerAdapter(fcAdapter);

        fcAdapter.setVirtualSlotNumber("13");// 必选项
        fcAdapter.setConnectingPartitionID("1");// 必选项
        fcAdapter.setConnectingVirtualSlotNumber("11");// 必选项

        return profile;
    }

    public void testUpdate() {
        String viosUuid = null;

        BocloudResult result = V8VirtualIOServerService.get(auth);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        VirtualIOServerResponse response = (VirtualIOServerResponse) result.getData();
        List<VirtualIOServerEntry> entries = response.getEntries();

        if (entries != null) {
            String PARTITIONNAME = "vios-bocloud.sdk";
            String name;
            for (VirtualIOServerEntry entry : entries) {
                try {
                    name = entry.getContent().getVirtualIOServer().getPartitionName();
                } catch (NullPointerException e) {
                    continue;
                }
                if (PARTITIONNAME.equals(name)) {
                    viosUuid = entry.getId();
                }
            }
            Assert.assertTrue("未找到名称为[" + PARTITIONNAME + "]的VIOS", viosUuid != null);
        }

        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();
        updateVios.setPartitionName("vios-bocloud.sdk-update");
        updateVios.setKeylockPosition("manual");

        result = V8VirtualIOServerService.update(auth, managedSystemUuid, viosUuid, updateVios);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());

    }

    @Test
    public void testCreateVirtualScsiMappings() throws Exception {
        V8VirtualDisk vd = new V8VirtualDisk("300", "server");
        Integer viosId = 1;
        Integer lparId = 2;
        Integer lparSlot = 5;


        BocloudResult result = V8VirtualIOServerService.get(auth, viosUuid);

        Assert.assertTrue(result.isSuccess());
        V8VirtualIOServerEntry virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        V8VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V8VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V8VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V8VirtualSCSIMapping> newMappingList = new ArrayList<>();
        if (virtualScsiMappingList == null) {
            virtualScsiMappingList = new ArrayList<>();
        }

        V8VirtualSCSIMapping virtualScsiMapping = null;
        for (V8VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if (null != oldMap.getStorage() && null != oldMap.getStorage().getVirtualDisk()
                    && vd.getDiskName().equals(oldMap.getStorage().getVirtualDisk().getDiskName())) {
                virtualScsiMapping = oldMap;
            } else if (null != oldMap.getServerAdapter()
                    && vd.getDiskName().equals(oldMap.getServerAdapter().getBackingDeviceName())) {
                // for physical disk
                virtualScsiMapping = oldMap;
            } else {
                newMappingList.add(oldMap);
            }
        }
        System.out.println(virtualScsiMapping);

        if (virtualScsiMapping == null) {
            // 不存在则新增
            V8Storage storage = null;
            if (vd.getDiskCapacity() != null) {
                storage = new V8Storage();
                storage.setSchemaVersion(null);
                storage.setVirtualDisk(vd);
            }
            V8ServerAdapter serverAdapter = new V8ServerAdapter(viosId.toString(),
                    lparId == null ? null : lparId.toString(), "true");
            if (lparId != null) {
                V8ClientAdapter clientAdapter = new V8ClientAdapter(lparId.toString(), viosId.toString(), "true");
                clientAdapter.setVariedOn("true");
                if (lparSlot != null) {
                    clientAdapter.setVirtualSlotNumber(lparSlot.toString());
                    serverAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                virtualScsiMapping = new V8VirtualSCSIMapping(clientAdapter, serverAdapter, storage);
            }
        } else {
            // 已存在则更新分配信息
            if (null != virtualScsiMapping.getServerAdapter()) {
                V8ServerAdapter oldServerAdapter = virtualScsiMapping.getServerAdapter();
                oldServerAdapter.setRemoteLogicalPartitionId(lparId.toString());
                if (lparSlot != null) {
                    oldServerAdapter.setRemoteSlotNumber(lparSlot.toString());
                }
                oldServerAdapter.setVirtualSlotNumber(null);// 这句话必须的，不能删除
                virtualScsiMapping.setServerAdapter(oldServerAdapter);
            }
            V8ClientAdapter clientAdapter = new V8ClientAdapter(lparId.toString(), viosId.toString(), "true");
            clientAdapter.setVariedOn("true");
            if (lparSlot != null) {
                clientAdapter.setVirtualSlotNumber(lparSlot.toString());
            }
            virtualScsiMapping.setClientAdapter(clientAdapter);
            virtualScsiMapping.setAssociatedLogicalPartition(null);
        }
        System.out.println(virtualScsiMapping);
        newMappingList.add(virtualScsiMapping);
        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V8VirtualIOServerService.update(auth, managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        System.out.println(result.getData());

    }

    @Test
    public void testUnassign() {
        BocloudResult result = V8VirtualIOServerService.get(auth, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        V8VirtualIOServerEntry virtualIOServerEntry = (V8VirtualIOServerEntry) result.getData();
        V8VirtualIOServer oldVios = virtualIOServerEntry.getContent().getVirtualIOServer();
        V8VirtualIOServerForUpdate updateVios = new V8VirtualIOServerForUpdate();

        // 这些设置只是为了调用分配的时候不报错，没有实际作用
        V8PartitionIOConfiguration partitionIOConfiguration = oldVios.getPartitionIOConfiguration();
        V8ProfileIOSlots profileIOSlots = partitionIOConfiguration.getProfileIOSlots();
        List<V8ProfileIOSlot> profileIOSlotList = profileIOSlots.getProfileIOSlot();
        List<V8ProfileIOSlot> newProfileIOSlotList = new ArrayList<>();
        for (V8ProfileIOSlot profileIOSlot : profileIOSlotList) {
            V8AssociatedIOSlot associatedIOSlot = profileIOSlot.getAssociatedIOSlot();
            associatedIOSlot.setRelatedIBMiIOSlot(null);
            associatedIOSlot.setRelatedIOAdapter(null);
            profileIOSlot.setAssociatedIOSlot(associatedIOSlot);
            newProfileIOSlotList.add(profileIOSlot);
        }
        profileIOSlots.setProfileIOSlot(newProfileIOSlotList);
        partitionIOConfiguration.setProfileIOSlots(profileIOSlots);
        updateVios.setPartitionIOConfiguration(partitionIOConfiguration);

        // 获取virtualSCSIMappings信息
        V8VirtualSCSIMappings virtualSCSIMappings = oldVios.getVirtualSCSIMappings();
        List<V8VirtualSCSIMapping> virtualScsiMappingList = virtualSCSIMappings.getVirtualScsiMapping();
        List<V8VirtualSCSIMapping> newMappingList = new ArrayList<>();

        for (V8VirtualSCSIMapping oldMap : virtualScsiMappingList) {
            if ("14".equals(oldMap.getServerAdapter().getVirtualSlotNumber())
                    || "15".equals(oldMap.getServerAdapter().getVirtualSlotNumber())
                    || "19".equals(oldMap.getServerAdapter().getVirtualSlotNumber())) {
                // 这里是该磁盘原来的分配信息,不做任何处理
            } else {
                newMappingList.add(oldMap);
            }
        }

        virtualSCSIMappings.setVirtualScsiMapping(newMappingList);
        updateVios.setVirtualSCSIMappings(virtualSCSIMappings);
        result = V8VirtualIOServerService.update(auth, managedSystemUuid, viosUuid, updateVios, 1000 * 60 * 10);
        System.out.println(result);

    }

    public void testDelete() {
        String viosUuid = null;

        BocloudResult result = V8VirtualIOServerService.get(auth);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        VirtualIOServerResponse response = (VirtualIOServerResponse) result.getData();
        List<VirtualIOServerEntry> entries = response.getEntries();

        if (entries != null) {
            String PARTITIONNAME = "vios-bocloud.sdk-update";
            String name;
            for (VirtualIOServerEntry entry : entries) {
                try {
                    name = entry.getContent().getVirtualIOServer().getPartitionName();
                } catch (NullPointerException e) {
                    continue;
                }
                if (PARTITIONNAME.equals(name)) {
                    viosUuid = entry.getId();
                }
            }
            Assert.assertTrue("未找到名称为[" + PARTITIONNAME + "]的VIOS", viosUuid != null);
        }

        result = V8VirtualIOServerService.delete(auth, managedSystemUuid, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    // @Test
    public void powerOnPartition() {
        BocloudResult result = V8VirtualIOServerService.powerOnPartition(auth, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    // @Test
    public void powerOffPartition() {
        BocloudResult result = V8VirtualIOServerService.powerOffPartition(auth, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

}
