package com.bocloud.sdk.hmc.api;

import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupEntry;
import com.bocloud.cmp.model.hmc.sdk.module.VolumeGroupResponse;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VirtualIOServerService;
import com.bocloud.cmp.provider.hmc.sdk.api.v8.V8VolumeGroupService;
import com.bocloud.cmp.provider.hmc.sdk.model.v8.sdk.module.*;
import com.bocloud.cmp.provider.hmc.sdk.util.XmlUtils;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.sdk.hmc.api.base.BaseTest;
import org.junit.Assert;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

public class V8VolumeGroupServiceTest extends BaseTest {
    private static final String viosUuid = "53A5ACB5-C1A8-4378-822E-0DB5F5211571";
    /**
     * 使用devvg
     */
    private static final String volumeGroupUuid = "2ee8352a-fdb5-30b2-80f8-ed44957d4758";

    @Test
    public void get1() {
        BocloudResult result = V8VolumeGroupService.get(auth, viosUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result);
    }

    @Test
    public void get2() {
        BocloudResult result = V8VolumeGroupService.get(auth, viosUuid, volumeGroupUuid);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void create() {
        BocloudResult groupResult = V8VolumeGroupService.get(auth, viosUuid, volumeGroupUuid);
        Assert.assertTrue(groupResult.isSuccess());

        V8VolumeGroup target;
        try {
            V8VolumeGroupEntry entry = (V8VolumeGroupEntry) groupResult.getData();
            target = entry.getContent().getVolumeGroup();
        } catch (Exception e) {
            target = null;
        }
        assert target != null;

        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        V8VirtualDisks virtualDisks = target.getVirtualDisks();
        List<V8VirtualDisk> disks = target.getVirtualDisks().getDisks();
        V8VirtualDisk disk = new V8VirtualDisk();

        disk.setDiskLabel("MB");
        disk.setDiskCapacity("1.000976");
        disk.setDiskName("api_vdiks2");
        disks.add(disk);
        virtualDisks.setDisks(disks);
        updateBean.setVirtualDisks(virtualDisks);

        BocloudResult result = V8VolumeGroupService.update(auth, viosUuid, volumeGroupUuid, updateBean);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void createVG() {
        // 查询所有pv
        BocloudResult viosResult = V8VirtualIOServerService.get(auth, viosUuid);
        assert viosResult.isSuccess();
        V8VirtualIOServerEntry vios = (V8VirtualIOServerEntry) viosResult.getData();
        List<V8PhysicalVolume> pvs = vios.getContent().getVirtualIOServer().getPhysicalVolumes().getPhysicalVolumeList();
        // 查询所有vg
        BocloudResult vgResult = V8VolumeGroupService.get(auth, viosUuid);
        assert vgResult.isSuccess();
        V8VolumeGroupResponse vgResponse = (V8VolumeGroupResponse) vgResult.getData();
        List<V8VolumeGroupEntry> vgs = vgResponse.getEntries();
        // 获取尚未使用的pv
        pvs.removeIf(pv -> {
            for (V8VolumeGroupEntry vg : vgs) {
                for (V8PhysicalVolume pvUsed : vg.getContent().getVolumeGroup().getPhysicalVolumes().getPhysicalVolumeList()) {
                    if (pvUsed.getVolumeUniqueId().equals(pv.getVolumeUniqueId())) {
                        return true;
                    }
                }
            }
            return false;
        });

        V8PhysicalVolumes pvss = new V8PhysicalVolumes();
        pvss.setPhysicalVolumeList(pvs);

        V8VolumeGroup vg = new V8VolumeGroup();
        vg.setGroupName("lycvg");
        vg.setPhysicalVolumes(pvss);

        String xmlText = XmlUtils.toXmlString(vg);
        xmlText = XmlUtils.appendHmcNamespace(vg.getClass(), xmlText);

        System.out.println(xmlText);

        BocloudResult createResult = V8VolumeGroupService.create(auth, viosUuid, vg);
        System.out.println(createResult.getMessage());
        System.out.println(createResult.getData());
        assert createResult.isSuccess();
    }

    @Test
    public void delete() {
        BocloudResult groupResult = V8VolumeGroupService.get(auth, viosUuid, volumeGroupUuid);
        Assert.assertTrue(groupResult.isSuccess());

        V8VolumeGroup target;
        try {
            V8VolumeGroupEntry entry = (V8VolumeGroupEntry) groupResult.getData();
            target = entry.getContent().getVolumeGroup();
        } catch (Exception e) {
            target = null;
        }

        assert target != null;

        List<V8VirtualDisk> oldDisks = target.getVirtualDisks().getDisks();
        List<V8VirtualDisk> disks = new ArrayList<>(2);
        for (V8VirtualDisk vd : oldDisks) {
            if ("test".equals(vd.getDiskName())) {
                continue;
            }
            disks.add(vd);
        }

        target.getVirtualDisks().setDisks(disks);
        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        updateBean.setVirtualDisks(target.getVirtualDisks());

        BocloudResult result = V8VolumeGroupService.update(auth, viosUuid, volumeGroupUuid, updateBean);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

    @Test
    public void deleteVG() {
        final String vgName = "lycvg";

        BocloudResult vgsResult = V8VolumeGroupService.get(auth, viosUuid);
        assert vgsResult.isSuccess();
        VolumeGroupResponse vgs = (VolumeGroupResponse) vgsResult.getData();
        String vgUuid = null;
        for (VolumeGroupEntry vg : vgs.getEntries()) {
            if (vgName.equals(vg.getContent().getVolumeGroup().getGroupName())) {
                vgUuid = vg.getContent().getVolumeGroup().getMetadata().getAtom().getAtomId();
            }
        }
        assert vgUuid != null;

        BocloudResult opResult = V8VolumeGroupService.delete(auth, viosUuid, vgUuid);
        assert opResult.isSuccess();
        System.out.println(opResult.getData());
    }

    @Test
    public void update() {
        BocloudResult groupResult = V8VolumeGroupService.get(auth, viosUuid, volumeGroupUuid);
        Assert.assertTrue(groupResult.isSuccess());

        V8VolumeGroup target;
        try {
            V8VolumeGroupEntry entry = (V8VolumeGroupEntry) groupResult.getData();
            target = entry.getContent().getVolumeGroup();
        } catch (Exception e) {
            target = null;
        }

        assert target != null;

        List<V8VirtualDisk> oldDisks = target.getVirtualDisks().getDisks();
        List<V8VirtualDisk> disks = new ArrayList<>(2);
        for (V8VirtualDisk vd : oldDisks) {
            if ("api_vdiks1".equals(vd.getDiskName())) {
                vd.setDiskCapacity("3");
            }
            disks.add(vd);
        }

        target.getVirtualDisks().setDisks(disks);
        V8VolumeGroupForUpdate updateBean = new V8VolumeGroupForUpdate();
        updateBean.setVirtualDisks(target.getVirtualDisks());

        BocloudResult result = V8VolumeGroupService.update(auth, viosUuid, volumeGroupUuid, updateBean);
        Assert.assertTrue(result.getMessage(), result.isSuccess());
        System.out.println(result.getData());
    }

}
