package com.bocloud.cmp.provider.common;

import java.util.List;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonDesktopPoolModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonFarmModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonSessionModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonSyncModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonUserOrGroupModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonVcenterModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.horizon.VmwareHorizonDesktopPoolProvider;
import com.bocloud.cmp.provider.horizon.VmwareHorizonFarmProvider;
import com.bocloud.cmp.provider.horizon.VmwareHorizonSessionProvider;
import com.bocloud.cmp.provider.horizon.VmwareHorizonUserOrGroupProvider;
import com.bocloud.cmp.provider.horizon.VmwareHorizonVcenterProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * vmware horizon同步类
 * 
 * @author HP
 *
 */
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonLocationProvider extends VmwareHorizonProvider implements LocationProvider {

	private VmwareHorizonDesktopPoolProvider poolProvider;
	private VmwareHorizonSessionProvider sessionProvider;
	private VmwareHorizonVcenterProvider vcenterProvider;
	private VmwareHorizonUserOrGroupProvider userOrGroupProvider;
	private VmwareHorizonFarmProvider farmProvider;

	public VmwareHorizonLocationProvider(Butler butler) {
		super(butler);
		poolProvider = new VmwareHorizonDesktopPoolProvider(butler);
		sessionProvider = new VmwareHorizonSessionProvider(butler);
		vcenterProvider = new VmwareHorizonVcenterProvider(butler);
		userOrGroupProvider = new VmwareHorizonUserOrGroupProvider(butler);
		farmProvider = new VmwareHorizonFarmProvider(butler);
	}

	@SuppressWarnings("unchecked")
	@Override
	public BocloudResult sync() {
		VmwareHorizonSyncModel model = new VmwareHorizonSyncModel();
		// 查询桌面池
		BocloudResult result = poolProvider.list();
		if (result.isSuccess()) {
			model.setPoolModels((List<VmwareHorizonDesktopPoolModel>) result.getData());
		}
		// 查询场
		result = farmProvider.list();
		if (result.isSuccess()) {
			model.setFarmModels((List<VmwareHorizonFarmModel>) result.getData());
		}
		// 查询会话
		result = sessionProvider.list();
		if (result.isSuccess()) {
			model.setSessionModels((List<VmwareHorizonSessionModel>) result.getData());
		}
		// 查询vcenter
		result = vcenterProvider.list();
		if (result.isSuccess()) {
			model.setVcenterModels((List<VmwareHorizonVcenterModel>) result.getData());
		}
		// 查询用户，组
		result = userOrGroupProvider.list(false);
		if (result.isSuccess()) {
			model.setUserOrGroupModels((List<VmwareHorizonUserOrGroupModel>) result.getData());
		}
		return new BocloudResult(true, model, "success");
	}

}
