package com.bocloud.cmp.provider.horizon;

import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonDesktopPoolModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.utils.JsonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

/**
 * horizon桌面池
 * 
 * @author HP
 *
 */
@Slf4j
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonDesktopPoolProvider extends VmwareHorizonProvider implements HorizonDesktopPoolProvider {

	public VmwareHorizonDesktopPoolProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			String url = "/inventory/v1/desktop-pools";
			if ("8.0".equalsIgnoreCase(this.getVersion())) {
				url = "/inventory/v2/desktop-pools";
			}
			Result result = this.doGet(url, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "结果为空");
			List<JSONObject> resultList = JSONArray.parseArray(message, JSONObject.class);
			List<VmwareHorizonDesktopPoolModel> models = resultList.stream().map(obj -> convertor(obj))
					.collect(Collectors.toList());
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VmwareHorizonDesktopPoolModel model) {
		try {
			String url = "/inventory/v1/desktop-pools";
			if ("8.0".equalsIgnoreCase(this.getVersion())) {
				url = "/inventory/v2/desktop-pools";
			}
			Result result = this.doPost(url, convertorMap(model));
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "创建结果为空");
			JSONObject object = JSONObject.parseObject(message);
			return new BocloudResult(true, convertor(object), "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	/**
	 * 数据转换
	 * 
	 * @param model
	 * @return
	 */
	private JSONObject convertorMap(VmwareHorizonDesktopPoolModel model) {
		String humpToLine = JsonUtils.humpToLine(JSONObject.toJSONString(model));
		JSONObject object = JSONObject.parseObject(humpToLine);
		String uuid = object.getString("uuid");
		object.remove("uuid");
		object.put("id", uuid);
		return object;
	}

	/**
	 * 桌面池数据转换
	 * 
	 * @param object
	 * @return
	 */
	private VmwareHorizonDesktopPoolModel convertor(JSONObject object) {
		VmwareHorizonDesktopPoolModel model = new VmwareHorizonDesktopPoolModel();
		String json = JsonUtils.lineToHump(object.toJSONString());
		model = JSONObject.parseObject(json, VmwareHorizonDesktopPoolModel.class);
		model.setUuid(object.getString("id"));
		// v1 api下的参数都封装在settings下
		String settings = object.getString("settings");
		if (!StringUtils.isEmpty(settings)) {
			String settingsJson = JsonUtils.lineToHump(settings);
			JSONObject settingsObj = JSONObject.parseObject(settingsJson);
			model.setDeleteInProgress(settingsObj.getBoolean("deleteInProgress"));
			model.setEnableClientRestrictions(settingsObj.getBoolean("enableClientRestrictions"));
			model.setAllowRdsPoolMultiSessionPerUser(settingsObj.getBoolean("allowRdsPoolMultiSessionPerUser"));
			model.setSessionType(settingsObj.getString("sessionType"));
			model.setCloudManaged(settingsObj.getBoolean("cloudManaged"));
			model.setCloudAssigned(settingsObj.getBoolean("cloudAssigned"));
			model.setSessionSettings(settingsObj.getString("sessionSettings"));
			model.setDisplayProtocolSettings(settingsObj.getString("displayProtocolSettings"));
		}
		return model;
	}

}
