package com.bocloud.cmp.provider.horizon;

import java.util.List;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonFarmModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.utils.JsonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

/**
 * horizon场管理
 * 
 * @author HP
 *
 */
@Slf4j
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonFarmProvider extends VmwareHorizonProvider implements HorizonFarmProvider {

	public VmwareHorizonFarmProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/inventory/v1/farms", null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "结果为空");
			List<JSONObject> resultList = JSONArray.parseArray(message, JSONObject.class);
			List<VmwareHorizonFarmModel> models = resultList.stream().map(obj -> convertor(obj))
					.collect(Collectors.toList());
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VmwareHorizonFarmModel model) {
		try {
			Result result = this.doPost("/inventory/v1/farms", convertorMap(model));
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "创建结果为空");
			JSONObject object = JSONObject.parseObject(message);
			return new BocloudResult(true, convertor(object), "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	public JSONObject convertorMap(VmwareHorizonFarmModel model) {
		String humpToLine = JsonUtils.humpToLine(JSONObject.toJSONString(model));
		JSONObject object = JSONObject.parseObject(humpToLine);
		String uuid = object.getString("uuid");
		object.remove("uuid");
		object.put("id", uuid);
		return object;
	}

	/**
	 * horizon场数据转换
	 * 
	 * @param object
	 * @return
	 */
	private VmwareHorizonFarmModel convertor(JSONObject object) {
		VmwareHorizonFarmModel model = new VmwareHorizonFarmModel();
		String json = JsonUtils.lineToHump(object.toJSONString());
		model = JSONObject.parseObject(json, VmwareHorizonFarmModel.class);
		model.setUuid(object.getString("id"));
		model.setRemark(object.getString("description"));
		return model;
	}

}
