package com.bocloud.cmp.provider.horizon;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonSessionModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.utils.JsonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

/**
 * horizon会话
 * 
 * @author HP
 *
 */
@Slf4j
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonSessionProvider extends VmwareHorizonProvider implements HorizonSessionProvider {

	public VmwareHorizonSessionProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/inventory/v1/sessions", null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "结果为空");
			List<JSONObject> resultList = JSONArray.parseArray(message, JSONObject.class);
			List<VmwareHorizonSessionModel> models = resultList.stream().map(obj -> convertor(obj))
					.collect(Collectors.toList());
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	/**
	 * 会话数据转换
	 * 
	 * @param object
	 * @return
	 */
	private VmwareHorizonSessionModel convertor(JSONObject object) {
		Long disconnectedTime = object.getLong("disconnected_time");
		Long startTime = object.getLong("start_time");
		object.remove("disconnected_time");
		object.remove("start_time");
		VmwareHorizonSessionModel model = new VmwareHorizonSessionModel();
		String json = JsonUtils.lineToHump(object.toJSONString());
		model = JSONObject.parseObject(json, VmwareHorizonSessionModel.class);
		model.setUuid(object.getString("id"));
		if (null != disconnectedTime) {
			model.setDisconnectedTime(new Date(disconnectedTime));
		}
		if (null != startTime) {
			model.setStartTime(new Date(startTime));
		}
		return model;
	}

}
