package com.bocloud.cmp.provider.horizon;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonUserOrGroupModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.utils.JsonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.MapTools;

import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

/**
 * horizon 用户，组管理
 * 
 * @author HP
 *
 */
@Slf4j
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonUserOrGroupProvider extends VmwareHorizonProvider implements HorizonUserOrGroupProvider {

	public VmwareHorizonUserOrGroupProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list(Boolean groupOnly) {
		try {
			Map<String, Object> paramMap = MapTools.simpleMap("group_only", groupOnly);
			Result result = this.doGet("/external/v1/ad-users-or-groups", paramMap);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "结果为空");
			List<JSONObject> resultList = JSONArray.parseArray(message, JSONObject.class);
			List<VmwareHorizonUserOrGroupModel> models = resultList.stream().map(obj -> convertor(obj))
					.collect(Collectors.toList());
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	/**
	 * 数据转换
	 * 
	 * @param model
	 * @return
	 */
	@SuppressWarnings("unused")
	private JSONObject convertorMap(VmwareHorizonUserOrGroupModel model) {
		String humpToLine = JsonUtils.humpToLine(JSONObject.toJSONString(model));
		JSONObject object = JSONObject.parseObject(humpToLine);
		String uuid = object.getString("uuid");
		object.remove("uuid");
		object.put("id", uuid);
		String remark = object.getString("remark");
		object.remove("remark");
		object.put("description", remark);
		return object;
	}

	/**
	 * horizon 用户，组数据转换
	 * 
	 * @param object
	 * @return
	 */
	private VmwareHorizonUserOrGroupModel convertor(JSONObject object) {
		VmwareHorizonUserOrGroupModel model = new VmwareHorizonUserOrGroupModel();
		String json = JsonUtils.lineToHump(object.toJSONString());
		model = JSONObject.parseObject(json, VmwareHorizonUserOrGroupModel.class);
		model.setUuid(object.getString("id"));
		model.setRemark(object.getString("description"));
		return model;
	}

}
