package com.bocloud.cmp.provider.horizon;

import java.util.List;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.horizon.VmwareHorizonDesktopPoolModel;
import com.bocloud.cmp.model.horizon.VmwareHorizonVcenterModel;
import com.bocloud.cmp.provider.VmwareHorizonProvider;
import com.bocloud.cmp.provider.utils.JsonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

import io.jsonwebtoken.lang.Assert;
import lombok.extern.slf4j.Slf4j;

/**
 * horizon vcenter管理
 * 
 * @author HP
 *
 */
@Slf4j
@Provider("bocloud.cmp.horizon")
public class VmwareHorizonVcenterProvider extends VmwareHorizonProvider implements HorizonVcenterProvider {

	public VmwareHorizonVcenterProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/config/v1/virtual-centers", null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			String message = result.getMessage();
			Assert.notNull(message, "结果为空");
			List<JSONObject> resultList = JSONArray.parseArray(message, JSONObject.class);
			List<VmwareHorizonVcenterModel> models = resultList.stream().map(obj -> convertor(obj))
					.collect(Collectors.toList());
			return new BocloudResult(true, models, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	public JSONObject convertorMap(VmwareHorizonDesktopPoolModel model) {
		String humpToLine = JsonUtils.humpToLine(JSONObject.toJSONString(model));
		JSONObject object = JSONObject.parseObject(humpToLine);
		String uuid = object.getString("uuid");
		object.remove("uuid");
		object.put("id", uuid);
		return object;
	}

	/**
	 * horizon vcenter数据转换
	 * 
	 * @param object
	 * @return
	 */
	private VmwareHorizonVcenterModel convertor(JSONObject object) {
		VmwareHorizonVcenterModel model = new VmwareHorizonVcenterModel();
		String json = JsonUtils.lineToHump(object.toJSONString());
		model = JSONObject.parseObject(json, VmwareHorizonVcenterModel.class);
		model.setUuid(object.getString("id"));
		return model;
	}

}
