package com.bocloud.cmp.huaweiconvertor;

import java.security.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.BackupModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.cbr.v1.model.BackupExtendInfo;
import com.huaweicloud.sdk.cbr.v1.model.BackupResp;

/**
 * @ClassName: BackupConvertor.java
 * @Description: 备份转换类
 *
 * @author: ymh
 * @date: 2021年3月19日 上午10:03:02
 */
public class BackupConvertor implements BeanConvertor<BackupModel, BackupResp> {

	static DateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.getDefault());

	@Override
	public List<BackupModel> convertList(List<? extends BackupResp> objects) {
		List<BackupModel> backupPolicyModels = new ArrayList<>();
		for (BackupResp object : objects) {
			if ("deleting".equalsIgnoreCase(object.getStatus().toString())) {
				continue;
			}
			backupPolicyModels.add(convertModel(object));
		}
		return backupPolicyModels;
	}

	@Override
	public BackupModel convertModel(BackupResp info) {
		BackupModel backupModel = new BackupModel();
		if (null != info) {
			backupModel.setBackupUuid(info.getId());
			backupModel.setName(info.getName());
			backupModel.setCheckpointUuid(info.getCheckpointId());
			backupModel.setResourceType(info.getResourceType().toString());
			backupModel.setRemark(info.getDescription());
			backupModel.setCreateTime(
					stringToDateTime(info.getCreatedAt().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss"))));
			backupModel.setUpdateTime(
					stringToDateTime(info.getUpdatedAt().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss"))));
			BackupExtendInfo extendInfo = info.getExtendInfo();
			backupModel.setExtendInfo(JSON.toJSONString(extendInfo));
			if (null != extendInfo) {
				backupModel.setSnapshotUuid(extendInfo.getSnapshotId());
				backupModel.setBootable(extendInfo.getBootable());
			}
			backupModel.setResourceSize(info.getResourceSize());
			backupModel.setResourceUuid(info.getResourceId());
			backupModel.setResourceName(info.getResourceName());
			backupModel.setResourceAz(info.getResourceAz());
			backupModel.setProjectUuid(info.getProjectId());
			backupModel.setVaultsUuid(info.getVaultId());
			backupModel.setImageType(info.getImageType().getValue());
			backupModel.setStatus(info.getStatus() == null ? null : info.getStatus().toString().toUpperCase());
		}
		return backupModel;
	}

	public static Date stringToDateTime(String dateString) {
		Date date = null;
		try {
			if ((dateString != null) && (dateString.length() != 0)) {
				format.setTimeZone(TimeZone.getTimeZone("GMT+0"));
				date = format.parse(dateString);
			}
		} catch (ParseException e) {
		}
		return date;
	}

}
