package com.bocloud.cmp.huaweiconvertor;

import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.bss.v2.model.MonthlyBillRecord;
import com.huaweicloud.sdk.core.utils.StringUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @title: BillingConvertor
 * @Author Hu Wentao
 * @Date: 2021/4/20 14:54
 * @Describle:
 */
public class BillingConvertor implements BeanConvertor<PublicCloudBillModel, MonthlyBillRecord> {

	private static Logger logger = LoggerFactory.getLogger(BillingConvertor.class);
	
	@Override
	public List<PublicCloudBillModel> convertList(List<? extends MonthlyBillRecord> models) {
		List<PublicCloudBillModel> list = new ArrayList<>();
		if (ListTool.isEmpty(models)) {
			return list;
		}
		for (MonthlyBillRecord model : models) {
			list.add(convertModel(model));
		}
		return list;
	}

    @Override
    public PublicCloudBillModel convertModel(MonthlyBillRecord model) {
        //计费模式。 1:包年/包月3:按需10:预留实例
        Map<Integer, String> chargeTypeMap = new HashMap<>(8);
        chargeTypeMap.put(1, "包年/包月");
        chargeTypeMap.put(3, "按需");
        chargeTypeMap.put(10, "预留实例");

        //账单类型。 1:消费-新购2:消费-续订3:消费-变更4:退款-退订5:消费-使用8:消费-自动续订9:调账-补偿14:消费-服务支持计划月末扣费16:调账-扣费
        Map<Integer, String> billTypeMap = new HashMap<>(8);
        billTypeMap.put(1, "消费-新购");
        billTypeMap.put(2, "消费-续订");
        billTypeMap.put(3, "消费-变更");
        billTypeMap.put(4, "退款-退订");
        billTypeMap.put(5, "消费-使用");
        billTypeMap.put(8, "消费-自动续订");
        billTypeMap.put(9, "调账-补偿");
        billTypeMap.put(14, "消费-服务支持计划月末扣费");
        billTypeMap.put(16, "调账-扣费");

        //支付状态。 1:已支付2:未结清3:未结算
        Map<Integer, String> statusMap = new HashMap<>(8);
        statusMap.put(1, "已支付");
        statusMap.put(2, "未清算");
        statusMap.put(3, "未结算");

        PublicCloudBillModel publicCloudBillModel = new PublicCloudBillModel();
        if (model != null) {
            publicCloudBillModel.setMonth(model.getBillCycle());
            publicCloudBillModel.setProductCode(model.getServiceTypeCode());//云服务类型编码

            publicCloudBillModel.setResourceId(model.getResourceTypeCode());//资源类型编码

            publicCloudBillModel.setRegionId(model.getRegionCode());
            if(null != model.getChargingMode()) {
            	publicCloudBillModel.setSubscriptionId(String.valueOf(model.getChargingMode()));
            }
            if(null != model.getChargingMode() && chargeTypeMap.containsKey(model.getChargingMode())) {
            	publicCloudBillModel.setSubscriptionType(chargeTypeMap.get(model.getChargingMode()));
            }
            String consumeTime = model.getConsumeTime();
			if (!StringUtils.isEmpty(consumeTime)) {
				if (consumeTime.contains("Z-")) {
					String[] split = consumeTime.split("Z-");
					if (null != split && split.length > 0 && !StringUtils.isEmpty(split[0])) {
						publicCloudBillModel
								.setFeeBeginTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", split[0] + "Z", "Z"));
					}
					if (null != split && split.length > 1 && !StringUtils.isEmpty(split[1])) {
						publicCloudBillModel.setFeeEndTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", split[1], "Z"));
					}
				} else {
					publicCloudBillModel
							.setFeeBeginTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", consumeTime + "Z", "Z"));
					publicCloudBillModel.setFeeEndTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", consumeTime, "Z"));
				}

			}
            if(!StringUtils.isEmpty(model.getTradeTime())) {
            	Date transfer = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", model.getTradeTime(), "Z");
            	if(null != transfer) {
            		publicCloudBillModel.setPayTime(String.valueOf(transfer));
            	}
            }
//            model.getProviderType(); //服务商。 1:华为云2:云市场
            publicCloudBillModel.setOrderSn(model.getTradeId());
            if(null != model.getBillType() && billTypeMap.containsKey(model.getBillType())) {
            	publicCloudBillModel.setReduceType(billTypeMap.get(model.getBillType()));
            }
            if(null != model.getStatus() && statusMap.containsKey(model.getStatus())) {
            	publicCloudBillModel.setStatus(statusMap.get(model.getStatus()));
            }
            if(null != model.getOfficialAmount()) {
            	publicCloudBillModel.setPretaxGrossAmount(BigDecimal.valueOf(model.getOfficialAmount())); //官网价
            }
            if(null != model.getEraseAmount()) {
            	publicCloudBillModel.setRoundDownDiscount(BigDecimal.valueOf(model.getEraseAmount())); //抹零金额
            }
            if(null != model.getConsumeAmount()) {
            	publicCloudBillModel.setMoney(BigDecimal.valueOf(model.getConsumeAmount())); //应付金额
            }
            if(null != model.getCouponAmount()) {
            	publicCloudBillModel.setDeductedByCashCoupons(BigDecimal.valueOf(model.getCouponAmount())); //代金券支付金额
            }
            if(null != model.getFlexipurchaseCouponAmount()) {
            	publicCloudBillModel.setDeductedByCoupons(BigDecimal.valueOf(model.getFlexipurchaseCouponAmount()));// 现金卷
            }
            if(null != model.getStoredValueCardAmount()) {
            	publicCloudBillModel.setDeductedByPrepaidCard(BigDecimal.valueOf(model.getStoredValueCardAmount())); //储值卡支付金额
            }
            if(null != model.getDebtAmount()) {
            	publicCloudBillModel.setOutstandingAmount(BigDecimal.valueOf(model.getDebtAmount())); // 欠费金额
            }
            publicCloudBillModel.setRegionName(model.getRegionName());
            publicCloudBillModel.setBillingCurrency("RMB");
        }
        return publicCloudBillModel;
    }

}
