package com.bocloud.cmp.huaweiconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.eip.v2.model.PublicipShowResp;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

public class ElasticIpConvertor implements BeanConvertor<ElasticIpModel, PublicipShowResp> {
	@Override
	public List<ElasticIpModel> convertList(List<? extends PublicipShowResp> publicips) {
		List<ElasticIpModel> elasticIpModels = new ArrayList<>();
		for (PublicipShowResp publicip : publicips) {
			elasticIpModels.add(convertModel(publicip));
		}
		return elasticIpModels;
	}

	@Override
	public ElasticIpModel convertModel(PublicipShowResp publicip) {
		ElasticIpModel model = JSONObject.parseObject(JSONObject.toJSONString(publicip), ElasticIpModel.class);
		model.setElasticIpUuid(publicip.getId());
		model.setStatus(publicip.getStatus().getValue());
		model.setBandwidthShareType(publicip.getBandwidthShareType().getValue());
		model.setIpVersion(publicip.getIpVersion().getValue().toString());
		model.setPublicIp(publicip.getPublicIpAddress());
		model.setBandwidth(publicip.getBandwidthSize().toString());
		model.setBandwidthUuid(publicip.getBandwidthId());
		model.setPrivateIpAddress(publicip.getPrivateIpAddress());
		model.setPortId(publicip.getPortId());
		String enterpriseProjectId = publicip.getEnterpriseProjectId();
		if (!StringUtils.isEmpty(enterpriseProjectId)) {
			model.setTenantUuid(enterpriseProjectId);
		}
		return model;
	}
}
