package com.bocloud.cmp.huaweiconvertor;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.OffsetDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.EnterpriseProjectModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.eps.v1.model.EpDetail;

/**
 * @ClassName: BackupConvertor.java
 * @Description: 企业项目
 *
 * @author: ymh
 * @date: 2021年3月19日 上午10:03:02
 */
public class EnterpriseConvertor implements BeanConvertor<EnterpriseProjectModel, EpDetail> {

	public static DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static Logger logger = LoggerFactory.getLogger(EnterpriseConvertor.class);

	@Override
	public List<EnterpriseProjectModel> convertList(List<? extends EpDetail> objects) {
		List<EnterpriseProjectModel> enterpriseProjectModels = new ArrayList<>();
		for (EpDetail object : objects) {
			enterpriseProjectModels.add(convertModel(object));
		}
		return enterpriseProjectModels;
	}

	@Override
	public EnterpriseProjectModel convertModel(EpDetail info) {
		EnterpriseProjectModel enterpriseProjectModel = new EnterpriseProjectModel();
		if (null != info) {
			enterpriseProjectModel.setName(info.getName());
			enterpriseProjectModel.setRemark(info.getDescription());
			enterpriseProjectModel.setEnterpriseProjectUuid(info.getId());
			Integer status = info.getStatus();
			if (null != status && 1 == status) {
				enterpriseProjectModel.setStatus("NORMAL");
				enterpriseProjectModel.setEnabled(true);
			} else {
				enterpriseProjectModel.setStatus("ABNORMAL");
				enterpriseProjectModel.setEnabled(false);
			}
			try {
				OffsetDateTime createdAt = info.getCreatedAt();
				if (null != createdAt) {
					String createTime = createdAt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
					enterpriseProjectModel.setCreateTime(format.parse(createTime));
				}
				OffsetDateTime updatedAt = info.getUpdatedAt();
				if (null != updatedAt) {
					String updateTime = updatedAt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
					enterpriseProjectModel.setUpdateTime(format.parse(updateTime));
				}
			} catch (ParseException e) {
				logger.error("时间转换异常： " + e);
			}
		}
		return enterpriseProjectModel;
	}

}
