package com.bocloud.cmp.huaweiconvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.ecs.v2.model.Flavor;
import com.huaweicloud.sdk.ecs.v2.model.FlavorExtraSpec;

/**
 * 地域 转换
 *
 * @author ymh
 * @时间 2018年11月22日
 */
public class FlavorConvertor implements BeanConvertor<FlavorModel, Flavor> {

    @Override
    public List<FlavorModel> convertList(List<? extends Flavor> objects) {
        List<FlavorModel> regionModels = new ArrayList<FlavorModel>();
        for (Flavor object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public FlavorModel convertModel(Flavor info) {
        FlavorModel model = new FlavorModel();
        if (null != info) {
            FlavorModel flavorModel = new FlavorModel();
            flavorModel.setCpuArchitecture(info.getId().startsWith("k") ? "kunpeng" : "x86");
            flavorModel.setName(info.getName());
            flavorModel.setCpu(Integer.valueOf(info.getVcpus()));
            flavorModel.setDisk(Integer.valueOf(info.getDisk()));
            flavorModel.setFlavorUuid(info.getId());
            flavorModel.setMemory(
                    BigDecimal.valueOf(info.getRam()).divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP));
            flavorModel.setUuid(info.getId());
            FlavorExtraSpec fosExtraSpecs = info.getOsExtraSpecs();
            if ("-1".equals(fosExtraSpecs.getQuotaMaxRate())) {
                fosExtraSpecs.setQuotaMaxRate(fosExtraSpecs.getQuotaMinRate());
            }
            flavorModel.setOsExtraSpecs(info.getOsExtraSpecs());
            String osExtraSpecs = JSONObject.toJSONString(info.getOsExtraSpecs());
            if (osExtraSpecs.contains("\"ecsPerformancetype\":\"computingv3\"")) {
                flavorModel.setPerformanceType("通用计算增强型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"normal\"")) {
                flavorModel.setPerformanceType("通用计算型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"highmem\"")) {
                flavorModel.setPerformanceType("内存优化型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"saphana\"")) {
                flavorModel.setPerformanceType("超大内存型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"highcpu\"")) {
                flavorModel.setPerformanceType("高性能计算型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"diskintensive\"")) {
                flavorModel.setPerformanceType("磁盘增强型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"highio\"")) {
                flavorModel.setPerformanceType("超高I/O型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"gpu\"")) {
                flavorModel.setPerformanceType("GPU加速型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"fpga\"")) {
                flavorModel.setPerformanceType("FPGA加速型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"ascend\"")) {
                flavorModel.setPerformanceType("AI加速型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"entry\"")) {
                flavorModel.setPerformanceType("通用入门型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"kunpeng_computing\"")) {
                flavorModel.setPerformanceType("鲲鹏通用计算增强型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"kunpeng_highmem\"")) {
                flavorModel.setPerformanceType("鲲鹏内存优化型");
            } else if (osExtraSpecs.contains("\"ecsPerformancetype\":\"kunpeng_highio\"")) {
                flavorModel.setPerformanceType("鲲鹏超高IO型");
            }
            
            flavorModel.setAttachableQuantity(info.getAttachableQuantity());
            List<String> finalZone = new ArrayList<>();
            Optional.ofNullable(info.getOsExtraSpecs()).flatMap(flavorExtraSpec -> Optional.ofNullable(flavorExtraSpec.getCondOperationAz())).ifPresent(az -> {
                List<String> zones = Arrays.stream(az.split(",")).
                        filter(zone -> zone.contains("normal")).collect(Collectors.toList());
                if (zones.size() != 0) {
                    zones.forEach(zone -> {
                        finalZone.add(zone.substring(0, zone.indexOf("(")));
                    });
                }
            });
            flavorModel.setZone(JSONObject.toJSONString(finalZone));

            return flavorModel;
        }
        return model;
    }
}
