package com.bocloud.cmp.huaweiconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.ims.v2.model.ImageInfo;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class ImageConvertor implements BeanConvertor<ImageModel, ImageInfo> {

	public static final String IMAGE_TYPE = "__imagetype"; // 公共镜像：gold 私有镜像：private 共享镜像：shared
	public static final String OS_VERSION = "__os_version";
	public static final String VIRTUAL_ENV_TYPE = "virtual_env_type";

	@Override
	public List<ImageModel> convertList(List<? extends ImageInfo> objects) {
		List<ImageModel> imageModels = new ArrayList<ImageModel>();
		for (ImageInfo object : objects) {
			String imageType = object.getImagetype() == null ? "" : object.getImagetype().getValue();
			if ("gold".equals(imageType)) {
				publicImage(object, imageModels);
			} else if ("private".equals(imageType)) { // 私有镜像
				imageModels.add(convertModel(object));
			}
		}
		return imageModels;
	}

	private void publicImage(ImageInfo object, List<ImageModel> imageModels) {
		if ("ACTIVE".equalsIgnoreCase(object.getStatus().getValue())) {
			imageModels.add(convertModel(object));
		}
	}

	@Override
	public ImageModel convertModel(ImageInfo image) {
		ImageModel model = new ImageModel();
		model.setName(image.getName());
		model.setImageUuid(image.getId());
		model.setStatus(null == image.getStatus() ? "ERROR" : image.getStatus().getValue().toUpperCase());
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try {
			model.setGmtCreate(formatter.parse(image.getCreatedAt()));
		} catch (ParseException e) {
			model.setGmtCreate(new Date());
		}
		model.setOsType(image.getPlatform().getValue());
		model.setOsCategory(image.getOsType().getValue());
		model.setOsName(image.getOsType().getValue());
		model.setOsVersion(image.getOsVersion());
		if ("gold".equalsIgnoreCase(image.getImagetype().getValue())) {
			model.setImageType("PUBLIC");
		} else {
			model.setImageType(image.getImagetype().getValue().toUpperCase());
		}
		String architecture = image.getArchitecture() != null ? image.getArchitecture().getValue() : "";
		model.setArchitecture(architecture + "_" + image.getOsBit().getValue());
		model.setVirtualEnvType(image.getVirtualEnvType().getValue());

		List<String> supportString = new ArrayList<>();
		if (image.getSupportArm() != null) {
			if (Boolean.valueOf(image.getSupportArm().getValue())) {
				supportString.add("Arm");
			}
		}
		if (image.getSupportDiskintensive() != null) {
			if (Boolean.valueOf(image.getSupportDiskintensive())) {
				supportString.add("Diskintensive");
			}
		}
		if (image.getSupportHighperformance() != null) {
			if (Boolean.valueOf(image.getSupportHighperformance())) {
				supportString.add("Highperformance");
			}
		}
		if (image.getSupportKvm() != null) {
			if (Boolean.valueOf(image.getSupportKvm())) {
				supportString.add("Kvm");
			}
		}
		if (image.getSupportKvmGpuType() != null) {
			supportString.add("KvmGpuType:" + image.getSupportKvmGpuType());
		}
		if (image.getSupportKvmInfiniband() != null) {
			if (Boolean.valueOf(image.getSupportKvmInfiniband())) {
				supportString.add("KvmInfiniband");
			}
		}
		if (image.getSupportFcInject() != null) {
			if (Boolean.valueOf(image.getSupportFcInject().getValue())) {
				supportString.add("FcInject");
			}
		}
		if (image.getSupportLargememory() != null) {
			if (Boolean.valueOf(image.getSupportLargememory())) {
				supportString.add("Largememory");
			}
		}
		if (image.getSupportXen() != null) {
			if (Boolean.valueOf(image.getSupportXen())) {
				supportString.add("Xen");
			}
		}
		if (image.getSupportXenGpuType() != null) {
			supportString.add("XenGpuType:" + image.getSupportXenGpuType());
		}
		if (image.getSupportXenHana() != null) {
			if (Boolean.valueOf(image.getSupportXenHana())) {
				supportString.add("SupportXenHana");
			}
		}
		model.setSupport(JSONObject.toJSONString(supportString));

		Double size = Double.parseDouble(image.getMinDisk() + "");
		model.setSize(size);
		model.setMinDisk(Long.parseLong(image.getMinDisk() + ""));
		return model;
	}
}
