package com.bocloud.cmp.huaweiconvertor;

import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.elb.v2.model.CreateListenerReq;
import com.huaweicloud.sdk.elb.v2.model.ListenerResp;
import com.huaweicloud.sdk.elb.v2.model.LoadbalancerResp;
import com.huaweicloud.sdk.elb.v2.model.ResourceList;
import org.apache.commons.collections.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class LbListenerConvertor implements BeanConvertor<LbListenerModel, ListenerResp> {

	@Override
	public List<LbListenerModel> convertList(List<? extends ListenerResp> objects) {
		List<LbListenerModel> volumeModels = new ArrayList<>();
		for (ListenerResp object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public LbListenerModel convertModel(ListenerResp info) {
		LbListenerModel model = new LbListenerModel();
		if (null != info) {
			model.setListenerUuid(info.getId());
			model.setName(info.getName());
			model.setRemark(info.getDescription());
			
			model.setAdminStateUp(info.getAdminStateUp());
			model.setProtocol(info.getProtocol().getValue());
			model.setPort(info.getProtocolPort());
			
			model.setDefaultPoolUuid(info.getDefaultPoolId());
			model.setConnLimit(info.getConnectionLimit());
			model.setDefTlsContainerRef(info.getDefaultTlsContainerRef());
			
			List<ResourceList> loadbalancers = info.getLoadbalancers();
			if (CollectionUtils.isNotEmpty(loadbalancers)) {
				model.setLbUuid(loadbalancers.get(0).getId());
				List<String> lbUuids = new ArrayList<>();
				for (ResourceList resource : loadbalancers) {
					lbUuids.add(resource.getId());
				}
				model.setLbUuids(lbUuids);
			}
		}
		return model;
	}

	private Date dateConver(String dateStr) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS");
			Date date = format.parse(dateStr);

			return date;

		} catch (Exception e) {
			return null;
		}
	}

}
