package com.bocloud.cmp.huaweiconvertor;

import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.elb.v2.model.PoolResp;
import com.huaweicloud.sdk.elb.v2.model.PoolResp;
import com.huaweicloud.sdk.elb.v2.model.ResourceList;
import com.huaweicloud.sdk.elb.v2.model.SessionPersistence;
import org.apache.commons.collections.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


public class LbPoolConvertor implements BeanConvertor<LbPoolModel, PoolResp> {

    @Override
    public List<LbPoolModel> convertList(List<? extends PoolResp> objects) {
        List<LbPoolModel> volumeModels = new ArrayList<>();
        for (PoolResp object : objects) {
            volumeModels.add(convertModel(object));
        }
        return volumeModels;
    }

    @Override
    public LbPoolModel convertModel(PoolResp info) {
        LbPoolModel model = new LbPoolModel();
        if (null != info) {

            model.setName(info.getName());
            model.setPoolUuid(info.getId());
            model.setRemark(info.getDescription());
            model.setProtocol(info.getProtocol().getValue());
            //private String lbMethod;// 负载均衡方法(ROUND_ROBIN/LEAST_CONNECTIONS/SOURCE_IP)
            model.setLbMethod(info.getLbAlgorithm().getValue());
            //private String type;// 会话持久类型(APP_COOKIE/HTTP_COOKIE/SOURCE_IP)
            SessionPersistence sessionPersistence = info.getSessionPersistence();
            if(null != sessionPersistence){
                model.setType(sessionPersistence.getType().getValue());
                model.setCookieName(sessionPersistence.getCookieName());
                model.setPersistenceTimeout( sessionPersistence.getPersistenceTimeout());
            }
            model.setAdminStateUp(info.getAdminStateUp());
            //监听
            List<ResourceList> listeners = info.getListeners();
            if(CollectionUtils.isNotEmpty(listeners)){
                List<String> listenerList = new ArrayList<>();
                model.setListenerUuid(listeners.get(0).getId());
                for(ResourceList resource:listeners){
                    listenerList.add(resource.getId());
                }
                model.setListenerUuids(listenerList);
            }
            //后端服务器
            List<ResourceList> members = info.getMembers();
            if(CollectionUtils.isNotEmpty(members)){
                List<String> memberList = new ArrayList<>();
                for(ResourceList resource:members){
                    memberList.add(resource.getId());
                }
                model.setMemberUuids(memberList);
            }

            //负载均衡
            List<ResourceList> loadbalancers = info.getLoadbalancers();
            if(CollectionUtils.isNotEmpty(loadbalancers)){
                model.setLbUuid(loadbalancers.get(0).getId());
            }
            model.setHmUuid(info.getHealthmonitorId());
        }
        return model;
    }

}
