package com.bocloud.cmp.huaweiconvertor;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.elb.v2.model.LoadbalancerResp;
import com.huaweicloud.sdk.elb.v2.model.ResourceList;
import org.apache.commons.collections.CollectionUtils;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class LoadBalanceConvertor implements BeanConvertor<LbModel, LoadbalancerResp> {

	@Override
	public List<LbModel> convertList(List<? extends LoadbalancerResp> objects) {
		List<LbModel> volumeModels = new ArrayList<>();
		for (LoadbalancerResp object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public LbModel convertModel(LoadbalancerResp info) {
		LbModel model = new LbModel();
		System.out.println(JSON.toJSONString(info));
		if (null != info) {

			model.setLbUuid(info.getId());
			model.setName(info.getName());
			model.setCreateTime(dateConver(info.getCreatedAt()));
			model.setRemark(info.getDescription());

			model.setVipSubnetUuid(info.getVipSubnetId());

			model.setAdminStateUp(info.getAdminStateUp());

			model.setVipAddress(info.getVipAddress());
			model.setVipPortUuid(info.getVipPortId());

			model.setStatus(info.getProvisioningStatus().getValue());
			// 监听
			List<ResourceList> listeners = info.getListeners();
			if (CollectionUtils.isNotEmpty(listeners)) {
				List<String> listenerList = new ArrayList<>();
				for (ResourceList resource : listeners) {
					listenerList.add(resource.getId());
				}
				model.setListeners(listenerList);
			}
			// 服务器组 resourceGroupUuid
			List<ResourceList> pools = info.getPools();
			if (CollectionUtils.isNotEmpty(pools)) {
				List<String> lbPoolUuids = new ArrayList<>();
				for (ResourceList resource : pools) {
					lbPoolUuids.add(resource.getId());
				}
				model.setLbPoolUuids(lbPoolUuids);
			}
			// model.setLoadBalancerType();
			// model.setVpcUuid(info.getVp);
			// model.setAvailabilityZones(info.get);
		}
		return model;
	}

	private Date dateConver(String dateStr) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
			Date date = format.parse(dateStr.replace("Z", " UTC"));

			return date;

		} catch (Exception e) {
			return null;
		}
	}

}
