package com.bocloud.cmp.huaweiconvertor;

import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.vpc.v2.model.FixedIp;
import com.huaweicloud.sdk.vpc.v2.model.Port;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

public class PortConvertor implements BeanConvertor<PortModel, Port> {
	@Override
	public List<PortModel> convertList(List<? extends Port> objects) {
		List<PortModel> portModels = new ArrayList<PortModel>();
		for (Port object : objects) {
			portModels.add(convertModel(object));
		}
		return portModels;
	}

	@Override
	public PortModel convertModel(Port port) {
		PortModel model = new PortModel();
		String name = port.getName();
		if (StringUtils.isEmpty(name)) {
			name = port.getId();
		}
		model.setName(name);
		model.setPortUuid(port.getId());
		model.setStatus(port.getStatus().getValue());
		model.setDeviceOwner(port.getDeviceOwner().getValue());
		model.setDeviceUuid(port.getDeviceId());
		model.setNetworkUuid(port.getNetworkId());
		model.setMacAddress(port.getMacAddress());
		List<FixedIp> fixedIps = port.getFixedIps();
		List<PortFixedIpModel> fixedIpModels = new ArrayList<PortFixedIpModel>();
		if (null != fixedIps && fixedIps.size() > 0) {
			for (FixedIp fixedIp : fixedIps) {
				PortFixedIpModel fixedIpModel = new PortFixedIpModel();
				fixedIpModel.setIpAddress(fixedIp.getIpAddress());
				fixedIpModel.setSubnetUuid(fixedIp.getSubnetId());
				fixedIpModels.add(fixedIpModel);
			}
		}
		model.setPortFixedIps(fixedIpModels);
		return model;
	}
}
