package com.bocloud.cmp.huaweiconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.vpc.v2.model.SecurityGroup;
import com.huaweicloud.sdk.vpc.v2.model.SecurityGroupRule;

public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, SecurityGroup> {

	private BeanConvertor<SecurityGroupRuleModel, SecurityGroupRule> ruleConvertor = new SecurityGroupRuleConvertor();

	@Override
	public List<SecurityGroupModel> convertList(List<? extends SecurityGroup> objects) {
		List<SecurityGroupModel> volumeModels = new ArrayList<SecurityGroupModel>();
		for (SecurityGroup object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public SecurityGroupModel convertModel(SecurityGroup info) {
		SecurityGroupModel model = new SecurityGroupModel();
		if (null != info) {
			model.setRemark(info.getDescription());
			String name = info.getName();
			if ("default".equals(name)) {
				name = "Sys-" + name;
			}
			String enterpriseProjectId = info.getEnterpriseProjectId();
			if (!StringUtils.isEmpty(enterpriseProjectId)) {
				model.setTenantUuid(enterpriseProjectId);
			}
			model.setName(name);
			model.setGroupUuid(info.getId());
			// model.setTenantUuid(info.getTenantId());
			List<? extends SecurityGroupRule> rules = info.getSecurityGroupRules();
			List<SecurityGroupRuleModel> convertList = ruleConvertor.convertList(rules);
			for (SecurityGroupRuleModel securityGroupRuleModel : convertList) {
				securityGroupRuleModel.setGroupUuid(info.getId());
			}
			model.setModels(convertList);
			model.setVpcUuid(info.getVpcId());
		}
		return model;
	}

}
