package com.bocloud.cmp.huaweiconvertor;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.ecs.v2.model.ServerAddress;
import com.huaweicloud.sdk.ecs.v2.model.ServerAddress.OsEXTIPSTypeEnum;
import com.huaweicloud.sdk.ecs.v2.model.ServerDetail;
import com.huaweicloud.sdk.ecs.v2.model.ServerExtendVolumeAttachment;
import com.huaweicloud.sdk.ecs.v2.model.ServerFlavor;
import com.huaweicloud.sdk.ecs.v2.model.ServerSecurityGroup;

/**
 * 华为私有云云主机数据转换器
 *
 * @author ymh
 * @时间 2018年12月11日
 */

public class ServerConvertor implements BeanConvertor<ServerModel, ServerDetail> {

	@Override
	public List<ServerModel> convertList(List<? extends ServerDetail> objects) {
		List<ServerModel> serverModels = new ArrayList<>();
		for (ServerDetail object : objects) {
			serverModels.add(convertModel(object));
		}
		return serverModels;
	}

	@Override
	public ServerModel convertModel(ServerDetail server) {
		ServerModel model = new ServerModel();
		ServerFlavor flavor = server.getFlavor();
		if (null != flavor) {
			model.setFlavorUuid(flavor.getId());
		}
		String enterpriseProjectId = server.getEnterpriseProjectId();
		if (!StringUtils.isEmpty(enterpriseProjectId)) {
			model.setTenantUuid(enterpriseProjectId);
		}
		model.setName(server.getName());
		this.vpcConvertor(server, model);
		model.setZone(server.getOsEXTAZAvailabilityZone());
		model.setInstanceId(server.getId());
		this.volumeConvertor(server, model);
		this.imageConvertor(server, model);
		model.setGmtVmCreate(dateConver(server.getCreated()));
		model.setKeypairName(server.getKeyName());

		this.securityConvertor(server, model);
		String status = convert(server.getStatus());
		if (null != status && !"".equals(status)) {
			model.setStatus(status.toUpperCase());
		} else {
			model.setStatus("Exception");
		}

		List<TagModel> tagModels = new ArrayList<>();
		for (String tag : server.getTags()) {
			tagModels.add(new TagModel(tag.split("=")[0], tag.split("=")[1], model.getInstanceId(), "instance"));
		}

		model.setTagModels(JSONObject.parseArray(JSONObject.toJSONString(tagModels), TagModel.class));

		return model;
	}

	private void vpcConvertor(ServerDetail server, ServerModel model) {
		String privatesIps = "";
		String managerIp = "";
		List<JSONObject> newIps = new ArrayList<JSONObject>();
		Map<String, List<ServerAddress>> addresses = server.getAddresses();
		String vpcId = "";
		if (null != addresses) {
			for (String networkId : addresses.keySet()) {
				vpcId = networkId;
				List<ServerAddress> ips = addresses.get(vpcId);
				for (ServerAddress ip : ips) {
					JSONObject newIp = new JSONObject();
					OsEXTIPSTypeEnum osEXTIPSType = ip.getOsEXTIPSType();
					if (null != osEXTIPSType) {
						newIp.put("type", osEXTIPSType.getValue());
					}
					newIp.put("networkId", networkId);
					newIp.put("address", ip.getAddr());
					if (StringUtils.isEmpty(managerIp)) {
						managerIp = ip.getAddr();
					}
					newIp.put("version", ip.getVersion());
					newIp.put("portId", ip.getOsEXTIPSPortId());
					newIp.put("macAddr", ip.getOsEXTIPSMACMacAddr());
					newIps.add(newIp);
				}
			}
		}
		model.setNetworkUuid(vpcId);
		privatesIps = JSONArray.toJSONString(newIps);
		model.setPrivateIps(privatesIps);
		model.setManagerIp(managerIp);
	}

	private void securityConvertor(ServerDetail server, ServerModel model) {

		List<ServerSecurityGroup> securityGroups = server.getSecurityGroups();
		if (null != securityGroups && securityGroups.size() > 0) {
			List<String> secgList = new ArrayList<String>();
			for (ServerSecurityGroup group : securityGroups) {
				String name = group.getName();
				if ("default".equals(name)) {
					name = "Sys-" + name;
				}
				secgList.add(name);
			}
			model.setSecurityGroups(secgList);
			model.setGroups(JSONArray.toJSONString(secgList));
		}
	}

	private void imageConvertor(ServerDetail server, ServerModel model) {
		// Image image = server.getImage();
		Map<String, String> metadata = server.getMetadata();

		if (null != metadata) {
			model.setImageUuid(metadata.get("metering.image_id"));
			model.setOsCategory(metadata.get("os_type"));
			model.setOsName(metadata.get("image_name"));
		}
	}

	private void volumeConvertor(ServerDetail server, ServerModel model) {
		List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();

		List<ServerExtendVolumeAttachment> volumes = server.getOsExtendedVolumesVolumesAttached();

		if (null != volumes && volumes.size() > 0) {
			VolumeModel volumeModel = null;
			for (ServerExtendVolumeAttachment vol : volumes) {
				volumeModel = new VolumeModel();
				volumeModel.setVolumeUuid(vol.getId());
				volumeModel.setDevice(vol.getDevice());
				volumeModels.add(volumeModel);
			}
		}
		model.setDiskDevices(volumeModels);
	}

	private String convert(String status) {
		if (StringUtils.isEmpty(status)) {
			return BocloudStatus.Server.EXCEPTION.name();
		} else if (status.equalsIgnoreCase("active")) {
			return BocloudStatus.Server.RUNNING.name();
		} else if (status.equalsIgnoreCase(BocloudStatus.Server.PAUSED.name())) {
			return BocloudStatus.Server.PAUSED.name();
		} else if (status.equalsIgnoreCase("stopped") || status.equalsIgnoreCase("shutoff")) {
			return BocloudStatus.Server.STOPPED.name();
		} else if (status.equalsIgnoreCase("BUILD")) {
			return BocloudStatus.Server.BUILDING.name();
		} else if (status.equalsIgnoreCase(BocloudStatus.Server.SUSPENDED.name())) {
			return BocloudStatus.Server.SUSPENDED.name();
		} else if (status.equalsIgnoreCase("VERIFY_RESIZE")) {
			return BocloudStatus.Server.RESIZING.name();
		} else if (status.equalsIgnoreCase("RESIZE")) {
			return BocloudStatus.Server.RESIZING.name();
		} else if (status.equalsIgnoreCase("SHELVED") || status.equalsIgnoreCase("SHELVED_OFFLOADED")) {
			return BocloudStatus.Server.SHELVED.name();
		} else if (status.equalsIgnoreCase("error")) {
			return BocloudStatus.Server.ERROR.name();
		} // 新增
		else if (status.equalsIgnoreCase("REBOOT") || status.equalsIgnoreCase("HARD_REBOOT")) {
			return BocloudStatus.Server.RESTARTING.name();
		} else {
			return BocloudStatus.Server.EXCEPTION.name();
		}

	}

	private Date dateConver(String dateStr) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss Z");
			Date date = format.parse(dateStr.replace("Z", " UTC"));

			return date;

		} catch (Exception e) {
			return null;
		}
	}

}
