package com.bocloud.cmp.huaweiconvertor;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.beanutils.converters.DateConverter;
import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.compute.HuaweiImageProvider;
import com.bocloud.cmp.provider.compute.HuaweiSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.evs.v2.model.SnapshotList;

public class SnapshotConvertor implements BeanConvertor<SnapshotModel, SnapshotList> {

	@Override
	public List<SnapshotModel> convertList(List<? extends SnapshotList> objects) {
		List<SnapshotModel> snapshotModels = new ArrayList<SnapshotModel>();
		for (SnapshotList object : objects) {
			snapshotModels.add(convertModel(object));
		}
		return snapshotModels;
	}

	@Override
	public SnapshotModel convertModel(SnapshotList snapshot) {
		SnapshotModel model = new SnapshotModel();
		if (null != snapshot) {
			model.setSize(snapshot.getSize());
			model.setSnapshotUuid(snapshot.getId());
			model.setName(snapshot.getName());
			String status = snapshot.getStatus().toString();
			if (!StringUtils.isEmpty(status) && "error".equalsIgnoreCase(status)) {
				status = "EXCEPTION";
			}
			model.setStatus(status.toUpperCase());
			model.setRemark(snapshot.getDescription());
			model.setVolumeUuid(snapshot.getVolumeId());
			model.setCreateTime(dateConver(snapshot.getCreatedAt()));
		}
		return model;
	}
	
	private static Date dateConver(String dateStr) {
		try {
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
			dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
			Date date = dateFormat.parse(dateStr);
			return date;
		} catch (Exception e) {
			return null;
		}
	}

}
