package com.bocloud.cmp.huaweiconvertor;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;
import org.apache.commons.lang3.StringUtils;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VaultsModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.cbr.v1.model.Billing;
import com.huaweicloud.sdk.cbr.v1.model.ResourceResp;
import com.huaweicloud.sdk.cbr.v1.model.Vault;

/**
 * 存储库转换
 *
 * @author ymh
 * @时间 2018年11月22日
 */
public class VaultsConvertor implements BeanConvertor<VaultsModel, Vault> {

	static DateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.getDefault());

	@Override
	public List<VaultsModel> convertList(List<? extends Vault> objects) {
		List<VaultsModel> vaultsModels = new ArrayList<>();
		for (Vault object : objects) {
			vaultsModels.add(convertModel(object));
		}
		return vaultsModels;
	}

	@Override
	public VaultsModel convertModel(Vault info) {
		VaultsModel vaultsModel = new VaultsModel();
		if (null != info) {
			vaultsModel.setVaultsUuid(info.getId());
			vaultsModel.setName(info.getName());
			vaultsModel.setAutoBind(info.getAutoBind());
			vaultsModel.setAutoExpand(info.getAutoExpand());
			String createdAt = info.getCreatedAt();
			if (!StringUtils.isEmpty(createdAt)) {
				vaultsModel.setCreateTime(stringToDateTime(createdAt));
			}

			Billing billing = info.getBilling();
			if (null != billing) {
				vaultsModel.setObjectType(billing.getObjectType().getValue());
				vaultsModel.setVaultSize(billing.getSize());
				vaultsModel.setStatus(billing.getStatus() == null ? "" : billing.getStatus().getValue().toUpperCase());
				vaultsModel.setProtectType(billing.getProtectType().getValue());
				Integer used = billing.getUsed();
				if (null != used) {
					vaultsModel.setUsed(used.longValue() / 1024);
				}
				vaultsModel.setAllocated(billing.getAllocated().longValue());
			}

			List<ResourceResp> resources = info.getResources();
			// 获取存储库下的资源
			this.resources(resources, vaultsModel);
		}
		return vaultsModel;
	}

	private void resources(List<ResourceResp> resources, VaultsModel vaultsModel) {
		if (ListTool.isEmpty(resources)) {
			return;
		}
		List<String> resourceUuids = new ArrayList<>();
		List<VolumeModel> volumeModels = new ArrayList<>();
		List<ServerModel> serverModels = new ArrayList<>();

		for (ResourceResp object : resources) {
			String type = object.getType();
			if ("OS::Nova::Server".equalsIgnoreCase(type)) {
				ServerModel serverModel = new ServerModel();
				serverModel.setInstanceId(object.getId());
				resourceUuids.add(object.getId());
				serverModel.setName(object.getName());
				serverModel.setSystemDiskSize(object.getSize());
				Integer backupSize = object.getBackupSize() == null ? 0 : object.getBackupSize();
				serverModel.setBackupSize(String.valueOf(backupSize / 1024));
				serverModel.setBackupCount(String.valueOf(object.getBackupCount()));
				serverModels.add(serverModel);
			}
			if ("OS::Cinder::Volume".equalsIgnoreCase(type)) {
				VolumeModel volumeModel = new VolumeModel();
				volumeModel.setVolumeUuid(object.getId());
				resourceUuids.add(object.getId());
				volumeModel.setName(object.getName());
				volumeModel.setSize(object.getSize());
				Integer backupSize = object.getBackupSize() == null ? 0 : object.getBackupSize();
				volumeModel.setBackupSize(String.valueOf(backupSize / 1024));
				volumeModel.setBackupCount(String.valueOf(object.getBackupCount()));
				volumeModels.add(volumeModel);
			}
		}
		vaultsModel.setResources(resourceUuids);
		vaultsModel.setVolumeModels(volumeModels);
		vaultsModel.setServerModels(serverModels);
	}

	public static Date stringToDateTime(String dateString) {
		Date date = null;
		try {
			if ((dateString != null) && (dateString.length() != 0)) {
				format.setTimeZone(TimeZone.getTimeZone("GMT+0"));
				date = format.parse(dateString);
			}
		} catch (ParseException e) {
		}
		return date;
	}

}
