package com.bocloud.cmp.huaweiconvertor;

import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.TagModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.evs.v2.model.Attachment;
import com.huaweicloud.sdk.evs.v2.model.VolumeDetail;

public class VolumeConvertor implements BeanConvertor<VolumeModel, VolumeDetail> {

	@Override
	public List<VolumeModel> convertList(List<? extends VolumeDetail> objects) {
		List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
		for (VolumeDetail object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public VolumeModel convertModel(VolumeDetail volume) {
		VolumeModel model = new VolumeModel();
		if (volume != null) {
			List<ServerVolumeModel> models = new ArrayList<>();
			ServerVolumeModel volumeModel = null;
			List<? extends Attachment> attachments = volume.getAttachments();
			for (Attachment volumeAttachment : attachments) {
				volumeModel = new ServerVolumeModel();
				volumeModel.setDevice(volumeAttachment.getDevice()); //
				volumeModel.setInstanceId(volumeAttachment.getServerId());
				volumeModel.setVolumeUuid(volumeAttachment.getVolumeId());
				models.add(volumeModel);
			}
			String enterpriseProjectId = volume.getEnterpriseProjectId();
			if (!StringUtils.isEmpty(enterpriseProjectId)) {
				model.setTenantUuid(enterpriseProjectId);
			}
			model.setBootVolume("true".equals(volume.getBootable()));
			model.setCreateTime(dateConver(volume.getCreatedAt()));
			model.setVolumeUuid(volume.getId());
			model.setRemark(volume.getDescription());
			model.setImageUuid(String.valueOf(volume.getVolumeImageMetadata().get("image_id")));
			model.setName(volume.getName());
			model.setType(volume.getVolumeType());
			model.setSize(volume.getSize());
			model.setSourceSnapshotId(volume.getSnapshotId());
			model.setSourceVolidUuid(volume.getSourceVolid());
			model.setZone(volume.getAvailabilityZone());
			model.setModels(models);
			model.setRemark(volume.getDescription());
			String status = volume.getStatus().toString();
			if (!StringUtils.isEmpty(status) && status.equalsIgnoreCase("in-use")) {
				status = "IN_USE";
			} else if (!StringUtils.isEmpty(status) && status.equalsIgnoreCase("error")) {
				status = "EXCEPTION";
			}
			model.setStatus(status.toUpperCase());

			List<TagModel> tagModels = new ArrayList<>();
			for (String key : volume.getTags().keySet()) {
				tagModels.add(new TagModel(key, volume.getTags().get(key), model.getVolumeUuid(), "volume"));
			}

			model.setTagModels(JSONObject.parseArray(JSONObject.toJSONString(tagModels), TagModel.class));
		}
		return model;
	}

	private Date dateConver(String dateStr) {
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss",  Locale.getDefault());
			format.setTimeZone(TimeZone.getTimeZone("GMT+0"));
			Date date = format.parse(dateStr);

			return date;

		} catch (Exception e) {
			return null;
		}
	}

}
