package com.bocloud.cmp.huaweiconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumeTypeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.evs.v2.model.VolumeType;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;


public class VolumeTypeConvertor implements BeanConvertor<VolumeTypeModel, VolumeType> {

    @Override
    public List<VolumeTypeModel> convertList(List<? extends VolumeType> objects) {
        List<VolumeTypeModel> volumeModels = new ArrayList<VolumeTypeModel>();
        for (VolumeType object : objects) {
            volumeModels.add(convertModel(object));
        }
        return volumeModels;
    }

    @Override
    public VolumeTypeModel convertModel(VolumeType volumeType) {
        VolumeTypeModel model = new VolumeTypeModel();
        if (volumeType != null) {
            model.setName(volumeType.getName());
            model.setVolumeTypeUuid(volumeType.getId());
            Optional.ofNullable(volumeType.getExtraSpecs()).ifPresent(volumeTypeExtraSpecs -> {
                model.setExtraSpecs(JSONObject.toJSONString(volumeType.getExtraSpecs()));
                Optional.ofNullable(volumeTypeExtraSpecs.getReSKEYAvailabilityZones()).ifPresent(zones -> {
                    model.setZones(Arrays.asList(zones
                            .split(",")).stream().map(s -> (s.trim())).collect(Collectors.toList()));
                });
                Optional.ofNullable(volumeTypeExtraSpecs.getOsVendorExtendedSoldOutAvailabilityZones()).ifPresent(soldZones -> {
                    model.setSoldOutZones(Arrays.asList(soldZones
                            .split(",")).stream().map(s -> (s.trim())).collect(Collectors.toList()));
                });

            });


        }
        return model;
    }


}
