package com.bocloud.cmp.provider.common;

import java.text.SimpleDateFormat;
import java.util.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.BillingConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PublicCloudBillModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.bss.v2.BssClient;
import com.huaweicloud.sdk.bss.v2.model.*;

/**
 * @title: HuaweiBillingProvider
 * @Author Hu Wentao
 * @Date: 2021/4/20 11:48
 * @Describle:
 */
@Provider("bocloud.cmp.huawei")
public class HuaweiBillingProvider extends HuaweiProvider implements BillingProvider {

    private static Logger logger = LoggerFactory.getLogger(HuaweiBillingProvider.class);

    private BeanConvertor<PublicCloudBillModel, MonthlyBillRecord> convertor = new BillingConvertor();

    public HuaweiBillingProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult<List<PublicCloudBillModel>> billDetailList(String month) {
        try {
            List<PublicCloudBillModel> bill = new ArrayList<>();
            // int allMonth = new Date().getMonth() + 12;
            int allMonth = 1;
            for (int i = 0; i <= allMonth; i++) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM");
                Calendar c = Calendar.getInstance();
                c.setTime(new Date());
                c.add(Calendar.MONTH, -i);
                Date time = c.getTime();
                bill.addAll(bill(format.format(time)));
            }

            return new BocloudResult<>(true, bill, "查询账单成功");
        } catch (Exception e) {
        	e.printStackTrace();
            logger.error("查询账单失败：" + e.getMessage());
            return new BocloudResult<>(false, "查询账单失败：" + e.getMessage());
        }
    }

    // @Override
    // public BocloudResult<List<PublicCloudBillModel>> billDetailList(String month) {
    // try {
    // List<PublicCloudBillModel> bill = new ArrayList<>();
    // SimpleDateFormat format = new SimpleDateFormat("yyyy");
    // Calendar c = Calendar.getInstance();
    // c.setTime(new Date());
    // c.add(Calendar.YEAR, -1);
    // Date time = c.getTime();
    // String year = format.format(time);
    // System.out.println(format.format(time));
    // for (int i = 1; i<= 12; i++) {
    // String s = format.format(time) + "-" + i;
    // System.out.println(s);
    // bill.addAll(bill(s));
    // }
    //
    // return new BocloudResult<>(true, bill, "查询账单成功");
    // } catch (Exception e) {
    // logger.error("查询账单失败：" + e.getMessage());
    // return new BocloudResult<>(false,"查询账单失败：" + e.getMessage());
    // }
    // }

    public List<PublicCloudBillModel> bill(String month) {
    	logger.info("month : " + month);
        Map<String, String> serviceTypeMap = serviceTypeMap();
        Map<String, String> resourceTypeMap = resourceTypeMap();

        BssClient client = (BssClient)getClient(HuaweiIndex.service.BSS);

        ListCustomerBillsFeeRecordsRequest request = new ListCustomerBillsFeeRecordsRequest();
        request.withBillCycle(month);

        ListCustomerBillsFeeRecordsResponse response = client.listCustomerBillsFeeRecords(request);
        Integer totalCount = response.getTotalCount();
        // Integer totalPage = 1;
        Integer limit = 1000;
        Integer totalPage = totalCount % limit == 0 ? totalCount / limit : totalCount / limit + 1;
        List<MonthlyBillRecord> billRecordList = new ArrayList<>(8);
        for (int i = 0; i <= totalPage; i++) {
            request.withBillCycle(month).withOffset(i * limit).withLimit(limit);
            response = client.listCustomerBillsFeeRecords(request);

            billRecordList.addAll(response.getRecords());
        }
        logger.info("start sync : " + month);
		if (!ListTool.isEmpty(billRecordList)) {
			logger.info("record : " + JSON.toJSONString(billRecordList.get(0)));
		}
        List<PublicCloudBillModel> publicCloudBillModels = convertor.convertList(billRecordList);
        logger.info("start convertor : " + month);
        for (PublicCloudBillModel model : publicCloudBillModels) {
        	if(null != model.getProductCode()) {
        		model.setProductCode(serviceTypeMap.get(model.getProductCode()));
        	}
            model.setProductName(model.getProductCode());
            if(null != model.getResourceId()) {
            	model.setProductDetail(resourceTypeMap.get(model.getResourceId()));
            }
        }
        if (!ListTool.isEmpty(publicCloudBillModels)) {
            System.out.println(JSON.toJSONString(publicCloudBillModels.get(0)));
        }
        return publicCloudBillModels;
    }

    @Override
    public BocloudResult billResourceSummaryList() {
        return null;
    }

    /**
     * 通过云服务类型code查询云服务类型
     * 
     * @return
     */
    public Map<String, String> serviceTypeMap() {
        try {
            Map<String, String> serviceTypeMap = new HashMap<>(8);
            BssClient client = (BssClient)getClient(HuaweiIndex.service.BSS);
            ListServiceTypesRequest request = new ListServiceTypesRequest();

            ListServiceTypesResponse response = client.listServiceTypes(request);
            List<ServiceType> serviceTypes = response.getServiceTypes();
            if (!ListTool.isEmpty(serviceTypes)) {
                for (ServiceType serviceType : serviceTypes) {
                    serviceTypeMap.put(serviceType.getServiceTypeCode(), serviceType.getServiceTypeName());
                }
            }
            return serviceTypeMap;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 通过资源类型code查询资源类型
     * 
     * @return
     */
    public Map<String, String> resourceTypeMap() {
        try {
            Map<String, String> resourceTypeMap = new HashMap<>(8);
            BssClient client = (BssClient)getClient(HuaweiIndex.service.BSS);
            ListResourceTypesRequest request = new ListResourceTypesRequest();
            ListResourceTypesResponse response = client.listResourceTypes(request);
            List<ResourceType> resourceTypes = response.getResourceTypes();
            if (!ListTool.isEmpty(resourceTypes)) {
                for (ResourceType resourceType : resourceTypes) {
                    resourceTypeMap.put(resourceType.getResourceTypeCode(), resourceType.getResourceTypeName());
                }
            }
            return resourceTypeMap;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
