package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.EnterpriseConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.EnterpriseProjectModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.eps.v1.EpsClient;
import com.huaweicloud.sdk.eps.v1.model.CreateEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.CreateEnterpriseProjectResponse;
import com.huaweicloud.sdk.eps.v1.model.DisableAction;
import com.huaweicloud.sdk.eps.v1.model.DisableEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.EnableAction;
import com.huaweicloud.sdk.eps.v1.model.EnableEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.EnterpriseProject;
import com.huaweicloud.sdk.eps.v1.model.EpDetail;
import com.huaweicloud.sdk.eps.v1.model.ListEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.ListEnterpriseProjectResponse;
import com.huaweicloud.sdk.eps.v1.model.ShowEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.ShowEnterpriseProjectResponse;
import com.huaweicloud.sdk.eps.v1.model.UpdateEnterpriseProjectRequest;
import com.huaweicloud.sdk.eps.v1.model.UpdateEnterpriseProjectResponse;
import com.huaweicloud.sdk.eps.v1.model.EnableAction.ActionEnum;

/**
 * HCSO企业项目
 *
 */
@Provider("bocloud.cmp.huawei")
public class HuaweiEnterpriseProvider extends HuaweiProvider implements EnterpriseProvider {

	private static Logger logger = LoggerFactory.getLogger(HuaweiEnterpriseProvider.class);

	private BeanConvertor<EnterpriseProjectModel, EpDetail> enterpriseConvertor = new EnterpriseConvertor();

	private static final int PAGE_SIZE = 1000;

	private static final DisableEnterpriseProjectRequest disableEnterpriseProjectRequest = null;

	public HuaweiEnterpriseProvider(Butler butler) {
		super(butler);
	}

	public BocloudResult listEnterpriseProject() {
		try {
			List<EnterpriseProjectModel> enterpriseProjectModels = new ArrayList<>();
			EpsClient epsClient = (EpsClient) getClient(HuaweiIndex.service.EPS);
			ListEnterpriseProjectRequest enterpriseProjectRequest = new ListEnterpriseProjectRequest();
			enterpriseProjectRequest.setOffset(0);
			enterpriseProjectRequest.setLimit(PAGE_SIZE);
			ListEnterpriseProjectResponse enterpriseProjectResponse = epsClient
					.listEnterpriseProject(enterpriseProjectRequest);
			Integer totalCount = enterpriseProjectResponse.getTotalCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, enterpriseProjectModels, "企业列表为空");
			}
			List<EpDetail> enterpriseProjects = enterpriseProjectResponse.getEnterpriseProjects();
			enterpriseProjectModels = enterpriseConvertor.convertList(enterpriseProjects);
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				logger.info("企业项目数量 ： " + enterpriseProjectModels.size());
				return new BocloudResult(true, enterpriseProjectModels, "快照列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				enterpriseProjectRequest.setOffset(PAGE_SIZE * (i - 1));
				enterpriseProjectRequest.withLimit(PAGE_SIZE);
				enterpriseProjectResponse = epsClient.listEnterpriseProject(enterpriseProjectRequest);
				enterpriseProjects = enterpriseProjectResponse.getEnterpriseProjects();
				enterpriseProjectModels.addAll(enterpriseConvertor.convertList(enterpriseProjects));
			}
			logger.info("企业项目数量 ： " + enterpriseProjectModels.size());
			return new BocloudResult<>(true, enterpriseProjectModels, "查询成功！");
		} catch (Exception e) {
			logger.error("查询企业项目异常：" + e);
			return new BocloudResult(false, "查询企业项目异常：" + e);
		}
	}

	public BocloudResult enterpriseProjectEnable(EnterpriseProjectModel model) {
		try {
			Boolean enabled = model.getEnabled();
			if (null == enabled) {
				return new BocloudResult<>(false, "操作成功");
			}
			EpsClient epsClient = (EpsClient) getClient(HuaweiIndex.service.EPS);
			if (enabled) {
				EnableEnterpriseProjectRequest enableEnterpriseProjectRequest = new EnableEnterpriseProjectRequest();
				enableEnterpriseProjectRequest.setEnterpriseProjectId(model.getEnterpriseProjectUuid());
				EnableAction enableBody = new EnableAction();
				enableBody.withAction(ActionEnum.ENABLE);
				enableEnterpriseProjectRequest.setBody(enableBody);
				epsClient.enableEnterpriseProject(enableEnterpriseProjectRequest);

			} else {
				DisableEnterpriseProjectRequest disableRequest = new DisableEnterpriseProjectRequest();
				DisableAction disableBody = new DisableAction();
				disableBody.withAction(com.huaweicloud.sdk.eps.v1.model.DisableAction.ActionEnum.DISABLE);
				disableRequest.setBody(disableBody);
				disableRequest.setEnterpriseProjectId(model.getEnterpriseProjectUuid());
				epsClient.disableEnterpriseProject(disableRequest);
			}
			ShowEnterpriseProjectRequest showRequest = new ShowEnterpriseProjectRequest();
			showRequest.setEnterpriseProjectId(model.getEnterpriseProjectUuid());
			ShowEnterpriseProjectResponse showEnterpriseProject = epsClient.showEnterpriseProject(showRequest);
			return new BocloudResult<>(true,
					enterpriseConvertor.convertModel(showEnterpriseProject.getEnterpriseProject()), "项目创建成功");
		} catch (Exception e) {
			logger.error("企业项目创建失败：" + e);
			return new BocloudResult<>(false, "企业项目创建失败：" + e);
		}
	}

	public BocloudResult modifyEnterpriseProject(EnterpriseProjectModel model) {
		try {
			EpsClient epsClient = (EpsClient) getClient(HuaweiIndex.service.EPS);
			UpdateEnterpriseProjectRequest updateProjectRequest = new UpdateEnterpriseProjectRequest();
			updateProjectRequest.setEnterpriseProjectId(model.getEnterpriseProjectUuid());
			EnterpriseProject enterpriseProjectBody = new EnterpriseProject();
			if (!StringUtils.isEmpty(model.getRemark())) {
				enterpriseProjectBody.setDescription(model.getRemark());
			}
			if (!StringUtils.isEmpty(model.getName())) {
				enterpriseProjectBody.setName(model.getName());
			}
			updateProjectRequest.setBody(enterpriseProjectBody);
			UpdateEnterpriseProjectResponse updateEnterpriseProject = epsClient
					.updateEnterpriseProject(updateProjectRequest);
			return new BocloudResult<>(true,
					enterpriseConvertor.convertModel(updateEnterpriseProject.getEnterpriseProject()), "项目创建成功");
		} catch (Exception e) {
			logger.error("企业项目创建失败：" + e);
			return new BocloudResult<>(false, "企业项目创建失败：" + e);
		}
	}

	public BocloudResult createEnterpriseProject(EnterpriseProjectModel model) {
		try {
			EpsClient epsClient = (EpsClient) getClient(HuaweiIndex.service.EPS);
			CreateEnterpriseProjectRequest createEnterpriseProjectRequest = new CreateEnterpriseProjectRequest();
			EnterpriseProject enterpriseProjectBody = new EnterpriseProject();
			enterpriseProjectBody.setName(model.getName());
			enterpriseProjectBody.setDescription(model.getRemark());
			createEnterpriseProjectRequest.setBody(enterpriseProjectBody);
			CreateEnterpriseProjectResponse createEnterpriseProject = epsClient
					.createEnterpriseProject(createEnterpriseProjectRequest);
			EpDetail enterpriseProject = createEnterpriseProject.getEnterpriseProject();
			return new BocloudResult<>(true, enterpriseConvertor.convertModel(enterpriseProject), "项目创建成功");
		} catch (Exception e) {
			logger.error("企业项目创建失败：" + e);
			return new BocloudResult<>(false, "企业项目创建失败：" + e);
		}
	}
}
