package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.FlavorConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.Flavor;
import com.huaweicloud.sdk.ecs.v2.model.ListFlavorsRequest;
import com.huaweicloud.sdk.ecs.v2.model.ListFlavorsResponse;
import com.huaweicloud.sdk.ecs.v2.model.ListResizeFlavorsRequest;
import com.huaweicloud.sdk.ecs.v2.model.ListResizeFlavorsResponse;
import com.huaweicloud.sdk.ecs.v2.model.ListResizeFlavorsResult;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 华为云规格
 *
 * @author ymh
 * @时间 2018年11月29日
 */
@Provider("bocloud.cmp.huawei")
public class HuaweiFlavorProvider extends HuaweiProvider implements FlavorProvider {

	private static Logger logger = LoggerFactory.getLogger(HuaweiFlavorProvider.class);

	private BeanConvertor<FlavorModel, Flavor> convertor = new FlavorConvertor();

	public HuaweiFlavorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ListFlavorsRequest request = new ListFlavorsRequest();
			ListFlavorsResponse response = client.listFlavors(request);
			List<FlavorModel> flavorModels = convertor.convertList(response.getFlavors());
			flavorModels.stream().forEach(flavorModel -> flavorModel.setRegionId(this.getRegionId()));
			return new BocloudResult(true, flavorModels, "同步成功");
		} catch (Exception e) {
			logger.error("失败 ： " + e);
			return new BocloudResult(false, "查询失败:" + e.getMessage());
		}
	}

	/**
	 * 查询云服务器规格变更支持列表
	 */
	@Override
	public BocloudResult listUsedFlavors(String instanceId) {
		try {
			List<String> flavorUuids = new ArrayList<>();
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			ListResizeFlavorsRequest request = new ListResizeFlavorsRequest();
			request.withInstanceUuid(instanceId);
			ListResizeFlavorsResponse response = client.listResizeFlavors(request);
			List<ListResizeFlavorsResult> flavors = response.getFlavors();
			if (!ListTool.isEmpty(flavors)) {
				for (ListResizeFlavorsResult listResizeFlavorsResult : flavors) {
					flavorUuids.add(listResizeFlavorsResult.getId());
				}
			}
			return new BocloudResult(true, flavorUuids, "成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		// Flavor builtF =
		// Builders.flavor().name("safe_to_delete_flavor").vcpus(1).disk(2)
		// .isPublic(true).rxtxFactor(2).ephemeral(1).ram(128).id("delete_1").swap(1).build();
		// Flavor f = osv3().compute().flavors().create(builtF);
		return null;
	}

	@Override
	public BocloudResult remove(String flavorId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		// TODO Auto-generated method stub
		return null;
	}

}
