package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.ZoneConvertor;
import com.bocloud.cmp.model.BackupModel;
import com.bocloud.cmp.model.BackupPolicyModel;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.model.EnterpriseProjectModel;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.HealthMonitorModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.HuaweiSyncModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.LbListenerPolicyModel;
import com.bocloud.cmp.model.LbListenerRuleModel;
import com.bocloud.cmp.model.LbMemberModel;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.model.NatGatewayDnatRuleModel;
import com.bocloud.cmp.model.NatGatewayModel;
import com.bocloud.cmp.model.NatGatewaySnatRuleModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.RegionModel;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VaultsModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumeTypeModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.model.ZoneModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.backup.HuaweiBackupProvider;
import com.bocloud.cmp.provider.backup.HuaweiVaultsProvider;
import com.bocloud.cmp.provider.compute.HuaweiImageProvider;
import com.bocloud.cmp.provider.compute.HuaweiServerProvider;
import com.bocloud.cmp.provider.compute.HuaweiSnapshotProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.loadbalance.HuaweiLoadBalanceProvider;
import com.bocloud.cmp.provider.network.HuaweiElasticIpProvider;
import com.bocloud.cmp.provider.network.HuaweiNatGatewayDnatRuleProvider;
import com.bocloud.cmp.provider.network.HuaweiNatGatewayProvider;
import com.bocloud.cmp.provider.network.HuaweiNatGatewaySnatRuleProvider;
import com.bocloud.cmp.provider.network.HuaweiPortProvider;
import com.bocloud.cmp.provider.network.HuaweiSecurityGroupProvider;
import com.bocloud.cmp.provider.network.HuaweiSubnetProvider;
import com.bocloud.cmp.provider.network.HuaweiVpcProvider;
import com.bocloud.cmp.provider.security.HuaweiKeypairProvider;
import com.bocloud.cmp.provider.storage.HuaweiVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.NovaAvailabilityZone;
import com.huaweicloud.sdk.ecs.v2.model.NovaListAvailabilityZonesRequest;
import com.huaweicloud.sdk.ecs.v2.model.NovaListAvailabilityZonesResponse;
import com.huaweicloud.sdk.iam.v3.region.IamRegion;
import com.huaweicloud.sdk.ims.v2.ImsClient;

@Provider("bocloud.cmp.huawei")
public class HuaweiLocationProvider extends HuaweiProvider implements LocationProvider {

	private static final Logger logger = LoggerFactory.getLogger(HuaweiLocationProvider.class);
	private Butler butler;

	public HuaweiLocationProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	private BeanConvertor<ZoneModel, NovaAvailabilityZone> zoneConvertor = new ZoneConvertor();

	@Override
	public BocloudResult sync() {
		try {
			logger.info("开始同步 " + butler.getRegionId() + " 数据... ");
			HuaweiSyncModel model = new HuaweiSyncModel();
			logger.info("地域 同步 ...");
			BocloudResult result = syncRegions(model);
			if (result.isFailed()) {
				logger.warn("地域{}无法同步数据", this.getRegionId());
				return result;
			}

			logger.info("同步企业项目...");
			syncEnterpriseProject(model);

			logger.info("地域 规格 ...");
			syncFlavor(model);
			logger.info("端口 数据 ...");
			syncPorts(model);
			logger.info("弹性IP 数据 ...");
			syncPublicIps(model);
			logger.info("镜像 数据...");
			syncImages(model);
			logger.info("秘钥 数据...");
			syncKeypairs(model);
			logger.info("同步VPC...");
			syncVpcs(model);
			logger.info("安全组 数据...");
			syncSecurityGroups(model);
			logger.info("云硬盘类型 数据...");
			syncVolumeTypes(model);
			logger.info("云硬盘 数据...");
			syncVolumes(model);
			logger.info("云主机 数据...");
			syncServers(model);
			logger.info("快照 数据...");
			syncSnapshots(model);
			logger.info("nat网关 数据...");
			syncNatGateways(model);
			logger.info("nat网关dnat规则 数据...");
			syncNatGatewayDnatRules(model);
			logger.info("nat网关snat规则 数据...");
			syncNatGatewaySnatRules(model);

			logger.info("负载均衡 数据...");
			syncLoadBalances(model);

			logger.info("监听器 数据...");
			syncLbListeners(model);

			logger.info("服务器组 数据...");
			syncLbPools(model);

			logger.info("健康检查 数据...");
			syncHealMonitors(model);

			logger.info("转发策略及规则 数据...");
			syncL7policyAndRule(model);

			/*
			 * logger.info("同步 备份策略 数据 ..."); syncBackupPolicy(model);
			 * logger.info("同步 存储库 数据 ..."); syncVaults(model); logger.info("同步 备份资源...");
			 * syncBackupResources(model);
			 */
			return new BocloudResult(true, model, "同步成功");
		} catch (Exception e) {
			logger.error("同步失败 ： " + e);
			return new BocloudResult(false, "同步失败 ");
		}
	}

	private void syncEnterpriseProject(HuaweiSyncModel model) {
		try {
			List<EnterpriseProjectModel> enterpriseProjectModels = new ArrayList<>();
			if (!StringUtils.isEmpty(this.getEndpoint())) {
				HuaweiEnterpriseProvider enterpriseProvider = new HuaweiEnterpriseProvider(butler);
				BocloudResult listEnterpriseProject = enterpriseProvider.listEnterpriseProject();
				if (listEnterpriseProject.isSuccess()) {
					enterpriseProjectModels = (List<EnterpriseProjectModel>) listEnterpriseProject.getData();
				}
			}
			model.setEnterpriseProjectModels(enterpriseProjectModels);
		} catch (Exception e) {
			logger.error("同步企业项目异常：" + e);
		}
	}

	private void syncBackupPolicy(HuaweiSyncModel model) {
		HuaweiBackupProvider backupProvider = new HuaweiBackupProvider(butler);
		BocloudResult listBackupPolicy = backupProvider.listBackupPolicy(null);
		List<BackupPolicyModel> backupPolicyModels = new ArrayList<>();
		if (listBackupPolicy.isSuccess() && null != listBackupPolicy.getData()) {
			backupPolicyModels = JSONArray.parseArray(JSONArray.toJSONString(listBackupPolicy.getData()),
					BackupPolicyModel.class);
			for (BackupPolicyModel backupPolicyModel : backupPolicyModels) {
				backupPolicyModel.setRegionId(this.getRegionId());
			}
		}
		model.setBackupPolicyModels(backupPolicyModels);
	}

	private void syncBackupResources(HuaweiSyncModel model) {
		HuaweiBackupProvider backupProvider = new HuaweiBackupProvider(butler);
		BocloudResult backupResult = backupProvider.list(null);
		List<BackupModel> backupModels = new ArrayList<>();
		if (backupResult.isSuccess()) {
			backupModels = JSONArray.parseArray(JSONArray.toJSONString(backupResult.getData()), BackupModel.class);
		}
		model.setBackupModels(backupModels);
	}

	private void syncVaults(HuaweiSyncModel model) {
		HuaweiVaultsProvider vaultsProvider = new HuaweiVaultsProvider(butler);
		BocloudResult vaultsResult = vaultsProvider.list(null);
		List<VaultsModel> vaultsModels = new ArrayList<>();
		if (vaultsResult.isSuccess()) {
			vaultsModels = JSONArray.parseArray(JSONArray.toJSONString(vaultsResult.getData()), VaultsModel.class);
		}
		model.setVaultsModels(vaultsModels);
	}

	private void syncSnapshots(HuaweiSyncModel model) {
		HuaweiSnapshotProvider snapshotProvider = new HuaweiSnapshotProvider(butler);
		BocloudResult snapResult = snapshotProvider.list();
		List<SnapshotModel> snapshotModels = new ArrayList<>();
		if (snapResult.isSuccess()) {
			snapshotModels = JSONArray.parseArray(JSONArray.toJSONString(snapResult.getData()), SnapshotModel.class);
			for (SnapshotModel snapshotModel : snapshotModels) {
				snapshotModel.setRegionId(this.getRegionId());
			}
			model.setSnapshotModels(snapshotModels);
		} else {
			model.setSnapshotModels(null);
		}
	}

	private void syncServers(HuaweiSyncModel model) {
		HuaweiServerProvider serverProvider = new HuaweiServerProvider(butler);
		BocloudResult serverResult = serverProvider.list();
		List<ServerModel> serverModels = new ArrayList<>();
		if (serverResult.isSuccess()) {
			serverModels = JSONArray.parseArray(JSONArray.toJSONString(serverResult.getData()), ServerModel.class);
			for (ServerModel serverModel : serverModels) {
				serverModel.setRegionId(this.getRegionId());
			}
			model.setServerModels(serverModels);
		} else {
			model.setServerModels(null);
		}
	}

	private void syncVolumes(HuaweiSyncModel model) {
		HuaweiVolumeProvider volumeProvider = new HuaweiVolumeProvider(butler);
		Map<String, List<VolumeModel>> volumeMap = new HashMap<>();
		List<VolumeModel> attachVols = new ArrayList<>();
		BocloudResult volumeResult = volumeProvider.list();
		List<VolumeModel> volumeModels = new ArrayList<>();
		if (volumeResult.isSuccess()) {
			volumeModels = JSONArray.parseArray(JSONArray.toJSONString(volumeResult.getData()), VolumeModel.class);
			for (VolumeModel volumeModel : volumeModels) {
				List<ServerVolumeModel> models = volumeModel.getModels();
				if (null != models && models.size() > 0) {
					ServerVolumeModel serverVolumeModel = models.get(0);
					// 获取云主机挂载的所有盘
					if (volumeMap.containsKey(serverVolumeModel.getInstanceId())) {
						List<VolumeModel> list = volumeMap.get(serverVolumeModel.getInstanceId());
						list.add(volumeModel);
						volumeMap.put(serverVolumeModel.getInstanceId(), list);
					} else {
						attachVols.add(volumeModel);
						volumeMap.put(serverVolumeModel.getInstanceId(), attachVols);
					}
				}
				volumeModel.setRequestId(this.getRegionId());
			}
			model.setVolumeModels(volumeModels);
		} else {
			model.setVolumeModels(null);
		}
	}

	private void syncVolumeTypes(HuaweiSyncModel model) {
		HuaweiVolumeProvider volumeProvider = new HuaweiVolumeProvider(butler);
		BocloudResult result = volumeProvider.listVolumeTypes();
		List<VolumeTypeModel> typeModels = new ArrayList<>();
		if (result.isSuccess()) {
			typeModels = JSONArray.parseArray(JSONArray.toJSONString(result.getData()), VolumeTypeModel.class);
			model.setVolumeTypeModels(typeModels);
		} else {
			model.setVolumeTypeModels(null);
		}
	}

	private void syncSecurityGroups(HuaweiSyncModel model) {
		HuaweiSecurityGroupProvider secgProvider = new HuaweiSecurityGroupProvider(butler);
		BocloudResult secgResult = secgProvider.list();
		List<SecurityGroupModel> secgModels = new ArrayList<>();
		if (secgResult.isSuccess()) {
			secgModels = JSONArray.parseArray(JSONArray.toJSONString(secgResult.getData()), SecurityGroupModel.class);
			for (SecurityGroupModel securityGroupModel : secgModels) {
				securityGroupModel.setRegionId(this.getRegionId());
			}
			model.setGroupModels(secgModels);
		} else {
			model.setGroupModels(null);
		}
	}

	private void syncVpcs(HuaweiSyncModel model) {
		HuaweiSubnetProvider subnetProvider = new HuaweiSubnetProvider(butler);
		HuaweiVpcProvider vpcProvider = new HuaweiVpcProvider(butler);
		BocloudResult vpcList = vpcProvider.list();
		List<VpcModel> vpcModels = new ArrayList<>();
		if (vpcList.isSuccess()) {
			vpcModels = JSONArray.parseArray(JSONArray.toJSONString(vpcList.getData()), VpcModel.class);
			for (VpcModel vpcModel : vpcModels) {
				vpcModel.setRegionId(this.getRegionId());
				// 查询VPC下的子网数据
				String vpcId = vpcModel.getVpcId();
				BocloudResult subnetResult = subnetProvider.list(vpcId);
				List<SubnetModel> subnetModels = new ArrayList<SubnetModel>();
				if (subnetResult.isSuccess()) {
					subnetModels = JSONArray.parseArray(JSONArray.toJSONString(subnetResult.getData()),
							SubnetModel.class);
				}
				vpcModel.setSubnetModels(subnetModels);
			}
			model.setVpcModels(vpcModels);
		} else {
			model.setVpcModels(null);
		}
	}

	private void syncKeypairs(HuaweiSyncModel model) {
		HuaweiKeypairProvider keypairProvider = new HuaweiKeypairProvider(butler);
		BocloudResult keypairResult = keypairProvider.list();
		List<KeypairModel> keypairs = new ArrayList<>();
		if (keypairResult.isSuccess()) {
			keypairs = JSONArray.parseArray(JSONArray.toJSONString(keypairResult.getData()), KeypairModel.class);
			model.setKeypairModels(keypairs);
		} else {
			model.setKeypairModels(null);
		}
	}

	private void syncImages(HuaweiSyncModel model) {
		HuaweiImageProvider imageProvider = new HuaweiImageProvider(butler);
		List<ImageModel> imageModels = new ArrayList<>();
		// 查询私有镜像需要指定企业项目ID
		List<EnterpriseProjectModel> enterpriseProjectModels = model.getEnterpriseProjectModels();
		if (!ListTool.isEmpty(enterpriseProjectModels)) {
			ImsClient client = (ImsClient) getClient(HuaweiIndex.service.IMS);
			for (EnterpriseProjectModel enterpriseProjectModel : enterpriseProjectModels) {
				// 公共镜像只查询默认项目下的
				if ("0".equalsIgnoreCase(enterpriseProjectModel.getEnterpriseProjectUuid())) {
					BocloudResult listPublicImages = imageProvider.listImagesByEnterpriseProject(client,
							enterpriseProjectModel.getEnterpriseProjectUuid(), "gold");
					if (listPublicImages.isSuccess()) {
						List<ImageModel> parseArray = JSONArray
								.parseArray(JSONArray.toJSONString(listPublicImages.getData()), ImageModel.class);
						if (!ListTool.isEmpty(parseArray)) {
							imageModels.addAll(parseArray);
						}
					}
				}
				BocloudResult listPrivateImages = imageProvider.listImagesByEnterpriseProject(client,
						enterpriseProjectModel.getEnterpriseProjectUuid(), "private");
				if (listPrivateImages.isSuccess()) {
					List<ImageModel> parseArray = JSONArray
							.parseArray(JSONArray.toJSONString(listPrivateImages.getData()), ImageModel.class);
					if (!ListTool.isEmpty(parseArray)) {
						imageModels.addAll(parseArray);
					}
				}
			}
		} else {
			BocloudResult imageResult = imageProvider.list();
			if (imageResult.isSuccess()) {
				imageModels = JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()), ImageModel.class);
			}
		}
		model.setImageModels(imageModels);
		logger.info("镜像数量 ： " + imageModels.size());
	}

	private void syncFlavor(HuaweiSyncModel model) {
		HuaweiFlavorProvider flavorProvider = new HuaweiFlavorProvider(butler);
		BocloudResult result = flavorProvider.sync();
		List<FlavorModel> flavorModels = new ArrayList<>();
		if (result.isSuccess()) {
			flavorModels = JSONArray.parseArray(JSONArray.toJSONString(result.getData()), FlavorModel.class);
			for (FlavorModel imageModel : flavorModels) {
				imageModel.setRegionId(this.getRegionId());
			}
			model.setFlavorModels(flavorModels);
		} else {
			model.setFlavorModels(null);
		}
	}

	private BocloudResult syncRegions(HuaweiSyncModel model) {
		BocloudResult result = this.listZones(this.getRegionId());
		if (result.isFailed()) {
			return result;
		}
		List<RegionModel> regions = new ArrayList<>();
		RegionModel regionModel = new RegionModel();
		regionModel.setCode(this.getRegionId());
		List<ZoneModel> zones = JSON.parseArray(JSON.toJSONString(result.getData()), ZoneModel.class);
		regionModel.setZones(zones);
		regions.add(regionModel);
		model.setRegions(regions);
		return new BocloudResult(true, "查询成功！");
	}

	private void syncPublicIps(HuaweiSyncModel model) {
		HuaweiElasticIpProvider elasticIpProvider = new HuaweiElasticIpProvider(butler);
		BocloudResult result = elasticIpProvider.list();
		List<ElasticIpModel> elasticIpModels = new ArrayList<>();
		if (result.isSuccess()) {
			elasticIpModels = JSONArray.parseArray(JSONArray.toJSONString(result.getData()), ElasticIpModel.class);
			for (ElasticIpModel elasticIpModel : elasticIpModels) {
				elasticIpModel.setRegionId(this.getRegionId());
			}
			model.setElasticIpModels(elasticIpModels);
		} else {
			model.setElasticIpModels(null);
		}
	}

	private void syncPorts(HuaweiSyncModel model) {
		HuaweiPortProvider huaweiPortProvider = new HuaweiPortProvider(butler);
		BocloudResult result = huaweiPortProvider.list();
		List<PortModel> portModels = new ArrayList<>();
		if (result.isSuccess()) {
			portModels = JSONArray.parseArray(JSONArray.toJSONString(result.getData()), PortModel.class);
			for (PortModel portModel : portModels) {
				portModel.setRegionId(this.getRegionId());
			}
			model.setPortModels(portModels);
		} else {
			model.setPortModels(null);
		}
	}

	private void syncNatGateways(HuaweiSyncModel model) {
		HuaweiNatGatewayProvider natGatewayProvider = new HuaweiNatGatewayProvider(butler);
		BocloudResult natGatewayResult = natGatewayProvider.list();
		List<NatGatewayModel> natGatewayModels = new ArrayList<>();
		if (natGatewayResult.isSuccess()) {
			natGatewayModels = JSONArray.parseArray(JSONArray.toJSONString(natGatewayResult.getData()),
					NatGatewayModel.class);
			for (NatGatewayModel natGatewayModel : natGatewayModels) {
				natGatewayModel.setRegionId(this.getRegionId());
			}
			model.setNatGatewayModels(natGatewayModels);
		} else {
			model.setNatGatewayModels(null);
		}
	}

	private void syncNatGatewayDnatRules(HuaweiSyncModel model) {
		HuaweiNatGatewayDnatRuleProvider natGatewayProvider = new HuaweiNatGatewayDnatRuleProvider(butler);
		BocloudResult natGatewayResult = natGatewayProvider.list();
		List<NatGatewayDnatRuleModel> natGatewayModels = new ArrayList<>();
		if (natGatewayResult.isSuccess()) {
			natGatewayModels = JSONArray.parseArray(JSONArray.toJSONString(natGatewayResult.getData()),
					NatGatewayDnatRuleModel.class);
			for (NatGatewayDnatRuleModel natGatewayModel : natGatewayModels) {
				natGatewayModel.setRegionId(this.getRegionId());
			}
			model.setDnatRuleModels(natGatewayModels);
		} else {
			model.setDnatRuleModels(null);
		}
	}

	private void syncNatGatewaySnatRules(HuaweiSyncModel model) {
		HuaweiNatGatewaySnatRuleProvider natGatewayProvider = new HuaweiNatGatewaySnatRuleProvider(butler);
		BocloudResult natGatewayResult = natGatewayProvider.list();
		List<NatGatewaySnatRuleModel> natGatewayModels = new ArrayList<>();
		if (natGatewayResult.isSuccess()) {
			natGatewayModels = JSONArray.parseArray(JSONArray.toJSONString(natGatewayResult.getData()),
					NatGatewaySnatRuleModel.class);
			for (NatGatewaySnatRuleModel natGatewayModel : natGatewayModels) {
				natGatewayModel.setRegionId(this.getRegionId());
			}
			model.setSnatRuleModels(natGatewayModels);
		} else {
			model.setSnatRuleModels(null);
		}
	}

	private void syncLoadBalances(HuaweiSyncModel model) {
		HuaweiLoadBalanceProvider loadBalanceProvider = new HuaweiLoadBalanceProvider(butler);
		BocloudResult lbResult = loadBalanceProvider.list();
		List<LbModel> lbModels = new ArrayList<>();
		if (lbResult.isSuccess()) {
			lbModels = JSONArray.parseArray(JSONArray.toJSONString(lbResult.getData()), LbModel.class);
			for (LbModel lbModel : lbModels) {
				lbModel.setRegionId(this.getRegionId());
			}
			model.setLbModels(lbModels);
		} else {
			model.setLbModels(null);
		}
	}

	private void syncLbListeners(HuaweiSyncModel model) {
		HuaweiLoadBalanceProvider loadBalanceProvider = new HuaweiLoadBalanceProvider(butler);
		BocloudResult lbListenerResult = loadBalanceProvider.listListener();
		List<LbListenerModel> lbListeners = new ArrayList<>();
		if (lbListenerResult.isSuccess()) {
			lbListeners = JSONArray.parseArray(JSONArray.toJSONString(lbListenerResult.getData()),
					LbListenerModel.class);
			model.setLbListenerModels(lbListeners);
		} else {
			model.setLbListenerModels(null);
		}
	}

	private void syncLbPools(HuaweiSyncModel model) {
		HuaweiLoadBalanceProvider loadBalanceProvider = new HuaweiLoadBalanceProvider(butler);
		BocloudResult lbPoolsResult = loadBalanceProvider.listPool();
		List<LbPoolModel> lbPools = new ArrayList<>();
		if (lbPoolsResult.isSuccess()) {
			lbPools = JSONArray.parseArray(JSONArray.toJSONString(lbPoolsResult.getData()), LbPoolModel.class);
			if (CollectionUtils.isNotEmpty(lbPools)) {
				List<LbMemberModel> lbMembers = new ArrayList<>();
				for (LbPoolModel pool : lbPools) {
					List<LbMemberModel> temp = new ArrayList();
					BocloudResult lbMembersResult = loadBalanceProvider.listMember(pool.getPoolUuid());
					if (lbMembersResult.isSuccess()) {
						temp = JSONArray.parseArray(JSONArray.toJSONString(lbMembersResult.getData()),
								LbMemberModel.class);
					}
					if (CollectionUtils.isNotEmpty(temp)) {
						lbMembers.addAll(temp);
					}
				}
				model.setLbMemberModels(lbMembers);
			}
			model.setLbPoolModels(lbPools);
		} else {
			model.setLbPoolModels(null);
		}
	}

	private void syncHealMonitors(HuaweiSyncModel model) {
		HuaweiLoadBalanceProvider loadBalanceProvider = new HuaweiLoadBalanceProvider(butler);
		BocloudResult healMonitorsResult = loadBalanceProvider.listHealMonitor();
		List<HealthMonitorModel> healthMonitors = new ArrayList<>();
		if (healMonitorsResult.isSuccess()) {
			healthMonitors = JSONArray.parseArray(JSONArray.toJSONString(healMonitorsResult.getData()),
					HealthMonitorModel.class);

			model.setHealthMonitorModels(healthMonitors);

		} else {
			model.setHealthMonitorModels(null);
		}
	}

	private void syncL7policyAndRule(HuaweiSyncModel model) {
		HuaweiLoadBalanceProvider loadBalanceProvider = new HuaweiLoadBalanceProvider(butler);
		BocloudResult l7policyResult = loadBalanceProvider.listListenerPolicy();
		List<LbListenerPolicyModel> lbL7policyModels = new ArrayList<>();
		if (l7policyResult.isSuccess()) {
			lbL7policyModels = JSONArray.parseArray(JSONArray.toJSONString(l7policyResult.getData()),
					LbListenerPolicyModel.class);

			if (CollectionUtils.isNotEmpty(lbL7policyModels)) {

				List<LbListenerRuleModel> rules = new ArrayList<>();
				for (LbListenerPolicyModel policy : lbL7policyModels) {

					List<LbListenerRuleModel> temp = new ArrayList<>();

					BocloudResult lbMembersResult = loadBalanceProvider.listListenerRule(policy.getPolicyUuid());
					if (lbMembersResult.isSuccess()) {
						temp = JSONArray.parseArray(JSONArray.toJSONString(lbMembersResult.getData()),
								LbListenerRuleModel.class);
					}

					if (CollectionUtils.isNotEmpty(temp)) {
						rules.addAll(temp);
					}
				}

				model.setLbL7RuleModels(rules);
			}

			model.setLbL7PolicyModels(lbL7policyModels);
		} else {
			model.setLbL7PolicyModels(null);
		}
	}

	@Override
	public BocloudResult listZones(String region) {
		try {
			EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
			NovaListAvailabilityZonesRequest request = new NovaListAvailabilityZonesRequest();
			NovaListAvailabilityZonesResponse response = client.novaListAvailabilityZones(request);
			return new BocloudResult(true, zoneConvertor.convertList(response.getAvailabilityZoneInfo()), "查询成功！");
		} catch (Exception e) {
			logger.error("查询地域可用区失败 ： ", e);
			return new BocloudResult(false, "查询地域可用区失败");
		}
	}

	@Override
	public BocloudResult getHostModel(HostModel host) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult listRegions() {
		return new HuaweiRegionProvider(butler).list();
	}
}
