package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.RegionConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.RegionModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.iam.v3.IamClient;
import com.huaweicloud.sdk.iam.v3.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 华为私有云可用域操作driver
 *
 * @author ymh
 * @时间 2018年12月11日
 */

@Provider("bocloud.cmp.huawei")
public class HuaweiRegionProvider extends HuaweiProvider implements ZoneProvider {

	public HuaweiRegionProvider(Butler butler) {
		super(butler);
	}

	private static final Logger logger = LoggerFactory.getLogger(HuaweiRegionProvider.class);

	private BeanConvertor<RegionModel, ProjectResult> regionConvertor = new RegionConvertor();

	private static Map<String, String> regionMap = new HashMap<>();

	@Override
	public BocloudResult list() {
		try {
			IamClient client = (IamClient) getClient(HuaweiIndex.service.IAM);
			KeystoneListRegionsRequest requestRegion = new KeystoneListRegionsRequest();
			KeystoneListRegionsResponse regionsResponse = client.keystoneListRegions(requestRegion);
			regionMap = regionsResponse.getRegions().stream()
					.collect(Collectors.toMap(Region::getId, region -> region.getLocales().getZhCn()));
			KeystoneListProjectsRequest request = new KeystoneListProjectsRequest();
			KeystoneListProjectsResponse response = client.keystoneListProjects(request);
			List<RegionModel> regionModels = regionConvertor.convertList(response.getProjects());
			regionModels.stream().forEach(regionModel -> regionModel.setName(regionMap.get(regionModel.getCode())));
			return new BocloudResult(true, regionModels, "success");
		} catch (Exception e) {
			logger.error("list regions error:", e);
			return new BocloudResult(false, e.getMessage());
		}
	}

}
