package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.evs.v2.EvsClient;
import com.huaweicloud.sdk.evs.v2.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/*
*
 * 快照
 * 
 * @author ymh
 * @时间 2018年11月29日

*/

@Provider("bocloud.cmp.huawei")
public class HuaweiSnapshotProvider extends HuaweiProvider implements SnapshotProvider {

	private static Logger logger = LoggerFactory.getLogger(HuaweiSnapshotProvider.class);

	private BeanConvertor<SnapshotModel, SnapshotList> convertor = new SnapshotConvertor();

	private static final String SERVICE_NAME = "evs";

	public HuaweiSnapshotProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			List<SnapshotModel> snapshotModels = new ArrayList<>();
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			ListSnapshotsRequest request = new ListSnapshotsRequest();
			request.setOffset(0);
			request.withLimit(PAGE_SIZE);
			ListSnapshotsResponse response = client.listSnapshots(request);
			Integer totalCount = response.getCount();
			if (null == totalCount || totalCount == 0) {
				return new BocloudResult(true, snapshotModels, "快照列表为空");
			}
			List<SnapshotList> snapshots = response.getSnapshots();
			for (SnapshotList snapshotList : snapshots) {
				SnapshotModel convertModel = convertor.convertModel(snapshotList);
				convertModel.setRegionId(this.getRegionId());
				snapshotModels.add(convertModel);
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, snapshotModels, "快照列表查询成功");
			}
			for (int i = 2; i <= totalPage; i++) {
				request.setOffset(PAGE_SIZE * (i - 1));
				request.withLimit(PAGE_SIZE);
				response = client.listSnapshots(request);
				snapshots = response.getSnapshots();
				for (SnapshotList snapshotList : snapshots) {
					SnapshotModel convertModel = convertor.convertModel(snapshotList);
					convertModel.setRegionId(this.getRegionId());
					snapshotModels.add(convertModel);
				}
			}
			logger.info("快照count : " + snapshotModels.size());
			return new BocloudResult(true, snapshotModels, "查询成功");
		} catch (Exception e) {
			logger.error("查询失败 ： " + e);
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SnapshotModel model) {
		try {
			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			DeleteSnapshotRequest request = new DeleteSnapshotRequest();
			request.withSnapshotId(model.getSnapshotUuid());
			DeleteSnapshotResponse response = client.deleteSnapshot(request);

			return new BocloudResult(true, response.getHttpStatusCode(), "删除成功");

		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除快照" + model.getName() + "失败: " + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(SnapshotModel model) {
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			ListSnapshotsRequest request = new ListSnapshotsRequest();
			request.withId(model.getSnapshotUuid());
			ListSnapshotsResponse response = client.listSnapshots(request);

//            if(response.getCount()<1){
//                return new BocloudResult(false, "查询详情失败");
//            }

			/*
			 * ShowSnapshotRequest request = new ShowSnapshotRequest();
			 * request.withSnapshotId(model.getSnapshotUuid()); ShowSnapshotResponse
			 * response = client.showSnapshot(request);
			 */

			SnapshotModel convertModel = convertor.convertModel(response.getSnapshots().get(0));
			convertModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, convertModel, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败 ： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SnapshotModel model) {
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);
			UpdateSnapshotRequest request = new UpdateSnapshotRequest();

			UpdateSnapshotRequestBody body = new UpdateSnapshotRequestBody();

			UpdateSnapshotOption snapshot = new UpdateSnapshotOption();
			snapshot.withName(model.getName()).withDescription(model.getRemark());
			body.withSnapshot(snapshot);
			request.withSnapshotId(model.getSnapshotUuid()).withBody(body);
			UpdateSnapshotResponse response = client.updateSnapshot(request);

			// 查询
			ListSnapshotsRequest requestQuery = new ListSnapshotsRequest();
			requestQuery.withId(model.getSnapshotUuid());
			ListSnapshotsResponse responseQuery = client.listSnapshots(requestQuery);

			if (responseQuery.getCount() < 1) {
				return new BocloudResult(false, "修改失败");
			}

			SnapshotModel convertModel = convertor.convertModel(responseQuery.getSnapshots().get(0));
			convertModel.setRegionId(this.getRegionId());

			return new BocloudResult(true, convertModel, "修改成功");

		} catch (Exception e) {
			logger.error("修改失败： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult revert(SnapshotModel model) {
		try {

			EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

			RollbackSnapshotRequest request = new RollbackSnapshotRequest();

			RollbackSnapshotRequestBody body = new RollbackSnapshotRequestBody();
			RollbackSnapshotOption rollback = new RollbackSnapshotOption();
			rollback.withVolumeId(model.getVolumeUuid());
			body.withRollback(rollback);

			request.withSnapshotId(model.getSnapshotUuid()).withBody(body);
			RollbackSnapshotResponse response = client.rollbackSnapshot(request);

			return new BocloudResult(true, "恢复成功", "恢复成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "恢复失败！");
		}
	}

	@Override
	public BocloudResult removeByVolumeId(String volumeId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult getSnapshot(String snapshotId) {
		BocloudResult result = new BocloudResult(false, "查询超时或快照不存在");
		if (null == snapshotId) {
			return result;
		}
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}

				EvsClient client = (EvsClient) getClient(HuaweiIndex.service.EVS);

				ListSnapshotsRequest request = new ListSnapshotsRequest();
				request.withId(snapshotId);
				ListSnapshotsResponse response = client.listSnapshots(request);

//                if(response.getCount()<1){
//                    return new BocloudResult(false, "查询详情失败");
//                }

				SnapshotList snapshotList = response.getSnapshots().get(0);

				if (null != snapshotList && "AVAILABLE".equalsIgnoreCase(snapshotList.getStatus())) {
					SnapshotModel convertModel = convertor.convertModel(response.getSnapshots().get(0));
					convertModel.setRegionId(this.getRegionId());
					result = new BocloudResult(true, convertModel, "查询成功");
					flag = false;
					break;
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 10 * 60 * 1000) {
					flag = false;
					break;
				}
			}
			return result;
		} catch (Exception e) {
			logger.error("查询失败 ： " + e);
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult getAvaliableSnap(String snapshotId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult list(Long cloudServerId) {
		// TODO Auto-generated method stub
		return null;
	}

}
