package com.bocloud.cmp.provider.loadbalance;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.LbHealthMonitorConvertor;
import com.bocloud.cmp.huaweiconvertor.LbL7policyConvertor;
import com.bocloud.cmp.huaweiconvertor.LbL7ruleConvertor;
import com.bocloud.cmp.huaweiconvertor.LbListenerConvertor;
import com.bocloud.cmp.huaweiconvertor.LbMemberConvertor;
import com.bocloud.cmp.huaweiconvertor.LbPoolConvertor;
import com.bocloud.cmp.huaweiconvertor.LoadBalanceConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HealthMonitorModel;
import com.bocloud.cmp.model.LbListenerModel;
import com.bocloud.cmp.model.LbListenerPolicyModel;
import com.bocloud.cmp.model.LbListenerRuleModel;
import com.bocloud.cmp.model.LbMemberModel;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.elb.v2.ElbClient;
import com.huaweicloud.sdk.elb.v2.model.*;

@Provider("bocloud.cmp.huawei")
public class HuaweiLoadBalanceProvider extends HuaweiProvider implements LoadBalanceProvider {
	public HuaweiLoadBalanceProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiLoadBalanceProvider.class);

	private BeanConvertor<LbModel, LoadbalancerResp> convetor = new LoadBalanceConvertor();

	private BeanConvertor<LbListenerModel, ListenerResp> convetorListener = new LbListenerConvertor();

	private BeanConvertor<HealthMonitorModel, HealthmonitorResp> convetorHealth = new LbHealthMonitorConvertor();

	private BeanConvertor<LbMemberModel, MemberResp> convetorMember = new LbMemberConvertor();

	private BeanConvertor<LbPoolModel, PoolResp> convetorPool = new LbPoolConvertor();

	private BeanConvertor<LbListenerPolicyModel, L7policyResp> convetorL7policy = new LbL7policyConvertor();

	private BeanConvertor<LbListenerRuleModel, L7ruleResp> convetorL7rule = new LbL7ruleConvertor();

	@Override
	public BocloudResult list() {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListLoadbalancersRequest request = new ListLoadbalancersRequest();
			ListLoadbalancersResponse response = client.listLoadbalancers(request);

			List<LoadbalancerResp> loadbalancers = response.getLoadbalancers();
			List<LbModel> convertList = convetor.convertList(loadbalancers);
			for (LbModel lbModel : convertList) {
				lbModel.setRegionId(this.getRegionId());
			}

			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败 : " + e);
		}
	}

	@Override
	public BocloudResult create(LbModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreateLoadbalancerRequest request = new CreateLoadbalancerRequest();
			CreateLoadbalancerRequestBody body = new CreateLoadbalancerRequestBody();
			CreateLoadbalancerReq loadbalancer = new CreateLoadbalancerReq();
			loadbalancer.withName(lb.getName()).withDescription(lb.getRemark()).withVipSubnetId(lb.getVipSubnetUuid());
			loadbalancer.withAdminStateUp(true).withVipAddress(lb.getVipAddress());
			loadbalancer.withProvider(CreateLoadbalancerReq.ProviderEnum.VLB);
			if (StringUtils.isNotEmpty(lb.getVipAddress())) {
				loadbalancer.withVipAddress(lb.getVipAddress());
			}
			body.withLoadbalancer(loadbalancer);
			request.withBody(body);

			CreateLoadbalancerResponse response = client.createLoadbalancer(request);

			LbModel lbModel = convetor.convertModel(response.getLoadbalancer());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult update(LbModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);
			UpdateLoadbalancerRequest request = new UpdateLoadbalancerRequest();
			UpdateLoadbalancerRequestBody body = new UpdateLoadbalancerRequestBody();
			UpdateLoadbalancerReq loadbalancer = new UpdateLoadbalancerReq();
			loadbalancer.withName(lb.getName()).withDescription(lb.getRemark()).withAdminStateUp(true);
			body.withLoadbalancer(loadbalancer);
			request.withLoadbalancerId(lb.getLbUuid()).withBody(body);
			UpdateLoadbalancerResponse response = client.updateLoadbalancer(request);
			LbModel lbModel = convetor.convertModel(response.getLoadbalancer());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String lbId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteLoadbalancerRequest request = new DeleteLoadbalancerRequest();
			request.withLoadbalancerId(lbId);
			DeleteLoadbalancerResponse response = client.deleteLoadbalancer(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String lbId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ShowLoadbalancerRequest request = new ShowLoadbalancerRequest();
			request.setLoadbalancerId(lbId);

			ShowLoadbalancerResponse response = client.showLoadbalancer(request);

			LbModel lbModel = convetor.convertModel(response.getLoadbalancer());
			lbModel.setRegionId(this.getRegionId());

			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listListener() {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListListenersRequest request = new ListListenersRequest();
			ListListenersResponse response = client.listListeners(request);

			List<ListenerResp> listeners = response.getListeners();
			List<LbListenerModel> convertList = convetorListener.convertList(listeners);

			for (LbListenerModel lbModel : convertList) {
				lbModel.setRegionId(this.getRegionId());
			}

			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败 : " + e);
		}
	}

	@Override
	public BocloudResult createListener(LbListenerModel lbListenerModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreateListenerRequest request = new CreateListenerRequest();
			CreateListenerRequestBody body = new CreateListenerRequestBody();
			CreateListenerReq listener = new CreateListenerReq();
			listener.withName(lbListenerModel.getName()).withDescription(lbListenerModel.getRemark())
					.withAdminStateUp(true);
			CreateListenerReq.ProtocolEnum protocolEnum = CreateListenerReq.ProtocolEnum
					.valueOf(lbListenerModel.getProtocol());
			listener.withLoadbalancerId(lbListenerModel.getLbUuid()).withProtocol(protocolEnum)
					.withProtocolPort(lbListenerModel.getPort());

			listener.withDefaultPoolId(lbListenerModel.getDefaultPoolUuid());

			body.withListener(listener);
			request.withBody(body);
			CreateListenerResponse response = client.createListener(request);

			LbListenerModel lbModel = convetorListener.convertModel(response.getListener());
			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updateListener(LbListenerModel lbListenerModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);
			UpdateListenerRequest request = new UpdateListenerRequest();
			UpdateListenerRequestBody body = new UpdateListenerRequestBody();
			UpdateListenerReq listener = new UpdateListenerReq();
			if (!StringUtils.isEmpty(lbListenerModel.getName())) {
				listener.withName(lbListenerModel.getName());
			}
			if (!StringUtils.isEmpty(lbListenerModel.getRemark())) {
				listener.withDescription(lbListenerModel.getRemark());
			}
			listener.withAdminStateUp(true);
			if (StringUtils.isNotBlank(lbListenerModel.getDefaultPoolUuid())) {
				listener.setDefaultPoolId(lbListenerModel.getDefaultPoolUuid());
			}
			body.withListener(listener);
			request.withListenerId(lbListenerModel.getListenerUuid()).withBody(body);
			UpdateListenerResponse response = client.updateListener(request);
			LbListenerModel lbModel = convetorListener.convertModel(response.getListener());
			return new BocloudResult(true, lbModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removeListener(String listenerId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteListenerRequest request = new DeleteListenerRequest();
			request.withListenerId(listenerId);
			DeleteListenerResponse response = client.deleteListener(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailListener(String listenerId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ShowListenerRequest request = new ShowListenerRequest();
			request.withListenerId(listenerId);

			ShowListenerResponse response = client.showListener(request);

			LbListenerModel lbModel = convetorListener.convertModel(response.getListener());

			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listHealMonitor() {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListHealthmonitorsRequest request = new ListHealthmonitorsRequest();

			ListHealthmonitorsResponse response = client.listHealthmonitors(request);

			List<HealthmonitorResp> healthmonitors = response.getHealthmonitors();
			List<HealthMonitorModel> convertList = convetorHealth.convertList(healthmonitors);

			return new BocloudResult(true, convertList, "查询列表成功");

			// return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败 : " + e);
		}
	}

	@Override
	public BocloudResult createHealMonitor(HealthMonitorModel hmModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreateHealthmonitorRequest request = new CreateHealthmonitorRequest();

			CreateHealthmonitorRequestBody body = new CreateHealthmonitorRequestBody();

			CreateHealthmonitorReq healthmonitor = new CreateHealthmonitorReq();
			// 必填项
			CreateHealthmonitorReq.TypeEnum typeEnum = CreateHealthmonitorReq.TypeEnum.fromValue(hmModel.getType());
			healthmonitor.withTimeout(hmModel.getTimeout()).withType(typeEnum).withDelay(hmModel.getDelay());
			healthmonitor.withMaxRetries(hmModel.getMaxRetries()).withPoolId(hmModel.getPoolUuid());
			// 非必填项
			healthmonitor.withName(hmModel.getName()).withAdminStateUp(hmModel.getAdminStateUp());
			healthmonitor.withExpectedCodes(hmModel.getExpectedCodes());
			healthmonitor.withUrlPath(hmModel.getUrlPath()).withHttpMethod(hmModel.getHttpMethod());

			body.withHealthmonitor(healthmonitor);
			request.withBody(body);
			CreateHealthmonitorResponse response = client.createHealthmonitor(request);

			return new BocloudResult(true, convetorHealth.convertModel(response.getHealthmonitor()), "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updateHealMonitor(HealthMonitorModel hmModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			UpdateHealthmonitorRequest request = new UpdateHealthmonitorRequest();
			UpdateHealthmonitorRequestBody body = new UpdateHealthmonitorRequestBody();
			UpdateHealthmonitorReq healthmonitor = new UpdateHealthmonitorReq();
			healthmonitor.withName(hmModel.getName()).withAdminStateUp(hmModel.getAdminStateUp());
			healthmonitor.withTimeout(hmModel.getTimeout()).withExpectedCodes(hmModel.getExpectedCodes());
			healthmonitor.withUrlPath(hmModel.getUrlPath()).withHttpMethod(hmModel.getHttpMethod());

			healthmonitor.withDelay(hmModel.getDelay()).withMaxRetries(hmModel.getMaxRetries());
			CreateHealthmonitorReq.TypeEnum typeEnum = CreateHealthmonitorReq.TypeEnum.fromValue(hmModel.getType());
			healthmonitor.withType(hmModel.getType());

			body.withHealthmonitor(healthmonitor);
			request.withHealthmonitorId(hmModel.getHealthUuid()).withBody(body);
			UpdateHealthmonitorResponse response = client.updateHealthmonitor(request);

			return new BocloudResult(true, convetorHealth.convertModel(response.getHealthmonitor()), "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removeHealMonitor(String hmId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteHealthmonitorRequest request = new DeleteHealthmonitorRequest();
			request.withHealthmonitorId(hmId);
			DeleteHealthmonitorResponse response = client.deleteHealthmonitor(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailHealMonitor(String hmId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ShowHealthmonitorsRequest request = new ShowHealthmonitorsRequest();
			request.withHealthmonitorId(hmId);
			ShowHealthmonitorsResponse response = client.showHealthmonitors(request);
			return new BocloudResult(true, convetorHealth.convertModel(response.getHealthmonitor()), "查询成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listMember(String poolId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListMembersRequest request = new ListMembersRequest();
			request.withPoolId(poolId);
			ListMembersResponse response = client.listMembers(request);

			List<MemberResp> members = response.getMembers();
			List<LbMemberModel> convertList = convetorMember.convertList(members);

			for (LbMemberModel lbMemberModel : convertList) {
				lbMemberModel.setPoolUuid(poolId);
			}

			return new BocloudResult(true, convertList, "查询列表成功");

			// return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult createMember(LbMemberModel mbModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);
			CreateMemberRequest request = new CreateMemberRequest();
			CreateMemberRequestBody body = new CreateMemberRequestBody();
			CreateMemberReq member = new CreateMemberReq();
			// 必填项
			member.withProtocolPort(mbModel.getPort()).withSubnetId(mbModel.getSubnetUuid())
					.withAddress(mbModel.getAddress());
			Integer weight = mbModel.getWeight();
			if (null != weight) {
				member.withWeight(mbModel.getWeight());
			}
			// 非必填项
			if (!StringUtils.isEmpty(mbModel.getName())) {
				member.withName(mbModel.getName());
			}
			Boolean adminStateUp = mbModel.getAdminStateUp();
			if (null != adminStateUp && adminStateUp) {
				member.withAdminStateUp(adminStateUp);
			}
			body.withMember(member);
			request.withPoolId(mbModel.getPoolUuid()).withBody(body);
			CreateMemberResponse response = client.createMember(request);

			LbMemberModel lbMemberModel = convetorMember.convertModel(response.getMember());
			lbMemberModel.setPoolUuid(mbModel.getPoolUuid());
			return new BocloudResult(true, lbMemberModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult updateMember(LbMemberModel mbModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			UpdateMemberRequest request = new UpdateMemberRequest();
			UpdateMemberRequestBody body = new UpdateMemberRequestBody();
			UpdateMemberReq member = new UpdateMemberReq();
			member.withName(mbModel.getName()).withAdminStateUp(mbModel.getAdminStateUp())
					.withWeight(mbModel.getWeight());
			body.withMember(member);
			request.withPoolId(mbModel.getPoolUuid()).withMemberId(mbModel.getMemberUuid()).withBody(body);

			UpdateMemberResponse response = client.updateMember(request);

			LbMemberModel lbMemberModel = convetorMember.convertModel(response.getMember());
			lbMemberModel.setPoolUuid(mbModel.getPoolUuid());

			return new BocloudResult(true, lbMemberModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removeMember(String poolId, String mbId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteMemberRequest request = new DeleteMemberRequest();
			request.withPoolId(poolId).withMemberId(mbId);
			DeleteMemberResponse response = client.deleteMember(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailMember(String poolId, String mbId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ShowMemberRequest request = new ShowMemberRequest();
			request.withPoolId(poolId).withMemberId(mbId);
			ShowMemberResponse response = client.showMember(request);

			LbMemberModel lbMemberModel = convetorMember.convertModel(response.getMember());
			lbMemberModel.setPoolUuid(poolId);
			return new BocloudResult(true, lbMemberModel, "查询成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listPool() {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListPoolsRequest request = new ListPoolsRequest();

			ListPoolsResponse response = client.listPools(request);

			List<PoolResp> pools = response.getPools();
			List<LbPoolModel> convertList = convetorPool.convertList(pools);

			return new BocloudResult(true, convertList, "查询列表成功");

		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败 : " + e);
		}
	}

	@Override
	public BocloudResult createPool(LbPoolModel poolModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreatePoolRequest request = new CreatePoolRequest();

			CreatePoolRequestBody body = new CreatePoolRequestBody();

			CreatePoolReq pool = new CreatePoolReq();
			CreatePoolReq.ProtocolEnum protocolEnum = CreatePoolReq.ProtocolEnum.fromValue(poolModel.getProtocol());
			// 必填项
			pool.withProtocol(protocolEnum).withLbAlgorithm(poolModel.getLbMethod());
			// 非必填项

			pool.withLoadbalancerId(poolModel.getLbUuid());
			pool.withListenerId(poolModel.getListenerUuid()).withName(poolModel.getName())
					.withDescription(poolModel.getRemark());
			pool.withAdminStateUp(poolModel.getAdminStateUp());

			String type = poolModel.getType();
			if (StringUtils.isNotBlank(type)) {
				SessionPersistence sessionPersistence = new SessionPersistence();
				sessionPersistence.setType(SessionPersistence.TypeEnum.fromValue(type));
				sessionPersistence.setCookieName(poolModel.getCookieName());
				sessionPersistence.setPersistenceTimeout(poolModel.getPersistenceTimeout());
				pool.withSessionPersistence(sessionPersistence);
			}

			body.withPool(pool);
			request.withBody(body);
			CreatePoolResponse response = client.createPool(request);

			LbPoolModel lbPoolModel = convetorPool.convertModel(response.getPool());

			return new BocloudResult(true, lbPoolModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removePool(String poolId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeletePoolRequest request = new DeletePoolRequest();
			request.withPoolId(poolId);
			DeletePoolResponse response = client.deletePool(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updatePool(LbPoolModel poolModel) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			UpdatePoolRequest request = new UpdatePoolRequest();

			UpdatePoolRequestBody body = new UpdatePoolRequestBody();

			UpdatePoolReq pool = new UpdatePoolReq();

			if(!StringUtils.isEmpty(poolModel.getLbMethod())) {
				pool.withLbAlgorithm(poolModel.getLbMethod());
			}
			if(!StringUtils.isEmpty(poolModel.getName())) {
				pool.withName(poolModel.getName());
			}
			if(!StringUtils.isEmpty(poolModel.getRemark())) {
				pool.withDescription(poolModel.getRemark());
			}
			String type = poolModel.getType();
			if (StringUtils.isNotBlank(type)) {
				SessionPersistence sessionPersistence = new SessionPersistence();
				sessionPersistence.setType(SessionPersistence.TypeEnum.fromValue(type));
				sessionPersistence.setCookieName(poolModel.getCookieName());
				// sessionPersistence.setPersistenceTimeout(poolModel.getT);
				pool.withSessionPersistence(sessionPersistence);
			}

			body.withPool(pool);

			request.withPoolId(poolModel.getPoolUuid()).withBody(body);

			UpdatePoolResponse response = client.updatePool(request);

			LbPoolModel lbPoolModel = convetorPool.convertModel(response.getPool());

			return new BocloudResult(true, lbPoolModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailPool(String poolId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ShowPoolRequest request = new ShowPoolRequest();
			request.withPoolId(poolId);
			ShowPoolResponse response = client.showPool(request);

			LbPoolModel lbPoolModel = convetorPool.convertModel(response.getPool());
			return new BocloudResult(true, lbPoolModel, "查询成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listListenerPolicy() {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListL7policiesRequest request = new ListL7policiesRequest();
			ListL7policiesResponse response = client.listL7policies(request);
			List<L7policyResp> l7policies = response.getL7policies();
			List<LbListenerPolicyModel> convertList = convetorL7policy.convertList(l7policies);
			for (LbListenerPolicyModel lbModel : convertList) {
				lbModel.setRegionId(this.getRegionId());
			}

			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败 : " + e);
		}
	}

	@Override
	public BocloudResult createListenerPolicy(LbListenerPolicyModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreateL7policyRequest request = new CreateL7policyRequest();
			CreateL7policyRequestBody body = new CreateL7policyRequestBody();
			CreateL7policyReq l7policy = new CreateL7policyReq();

			CreateL7policyReq.ActionEnum action = CreateL7policyReq.ActionEnum.fromValue(lb.getAction());
			l7policy.withName(lb.getName()).withAction(action).withDescription(lb.getRemark())
					.withListenerId(lb.getListenerUuid());
			l7policy.withPosition(lb.getPosition()).withRedirectListenerId(lb.getRedirectListenerUuid())
					.withRedirectPoolId(lb.getRedirectPoolUuid()).withRedirectUrl(lb.getRedirectUrl());
			List<LbListenerRuleModel> rules = lb.getRules();
			if (!CollectionUtils.isEmpty(rules)) {
				List<CreateL7ruleReqInPolicy> l7rules = new ArrayList<CreateL7ruleReqInPolicy>();

				for (LbListenerRuleModel model : rules) {
					CreateL7ruleReqInPolicy l7rule = new CreateL7ruleReqInPolicy();
					CreateL7ruleReqInPolicy.TypeEnum type = CreateL7ruleReqInPolicy.TypeEnum.fromValue(model.getType());
					l7rule.withType(type).withCompareType(model.getCompareType()).withKey(model.getRuleKey())
							.withValue(model.getRuleValue()).withInvert(model.getInvert());
					l7rules.add(l7rule);
				}
				l7policy.withRules(l7rules);
			}

			body.withL7policy(l7policy);
			request.withBody(body);
			CreateL7policyResponse response = client.createL7policy(request);

			LbListenerPolicyModel lbModel = convetorL7policy.convertModel(response.getL7policy());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updateListenerPolicy(LbListenerPolicyModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);
			UpdateL7policiesRequest request = new UpdateL7policiesRequest();
			UpdateL7policiesRequestBody body = new UpdateL7policiesRequestBody();
			UpdateL7policyReq l7policy = new UpdateL7policyReq();
			l7policy.withName(lb.getName()).withDescription(lb.getRemark())
					.withRedirectListenerId(lb.getRedirectListenerUuid()).withRedirectPoolId(lb.getRedirectPoolUuid());
			body.withL7policy(l7policy);
			request.withL7policyId(lb.getPolicyUuid()).withBody(body);
			UpdateL7policiesResponse response = client.updateL7policies(request);
			LbListenerPolicyModel lbModel = convetorL7policy.convertModel(response.getL7policy());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removeListenerPolicy(String policyId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteL7policyRequest request = new DeleteL7policyRequest();
			request.withL7policyId(policyId);
			DeleteL7policyResponse response = client.deleteL7policy(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailListenerPolicy(String policyId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListL7policiesRequest request = new ListL7policiesRequest();
			request.withId(policyId);
			ListL7policiesResponse response = client.listL7policies(request);
			List<L7policyResp> l7policies = response.getL7policies();
			if (CollectionUtils.isEmpty(l7policies)) {
				return new BocloudResult(false, "查询详情失败");
			}

			LbListenerPolicyModel lbModel = convetorL7policy.convertModel(l7policies.get(0));
			lbModel.setRegionId(this.getRegionId());

			return new BocloudResult(true, lbModel, "查询成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult listListenerRule(String policyId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListL7rulesRequest request = new ListL7rulesRequest();
			request.withL7policyId(policyId);
			ListL7rulesResponse response = client.listL7rules(request);
			List<L7ruleResp> l7rule = response.getRules();
			List<LbListenerRuleModel> convertList = convetorL7rule.convertList(l7rule);
			for (LbListenerRuleModel lbModel : convertList) {
				lbModel.setRegionId(this.getRegionId());
				lbModel.setPolicyUuid(policyId);
			}

			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult createListenerRule(LbListenerRuleModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			CreateL7ruleRequest request = new CreateL7ruleRequest();
			CreateL7ruleRequestBody body = new CreateL7ruleRequestBody();
			CreateL7ruleReq rule = new CreateL7ruleReq();

			CreateL7ruleReq.TypeEnum type = CreateL7ruleReq.TypeEnum.fromValue(lb.getType());
			rule.withType(type).withCompareType(lb.getCompareType()).withKey(lb.getRuleKey())
					.withValue(lb.getRuleValue()).withInvert(lb.getInvert());
			request.withL7policyId(lb.getPolicyUuid()).withBody(body.withRule(rule));
			CreateL7ruleResponse response = client.createL7rule(request);

			LbListenerRuleModel lbModel = convetorL7rule.convertModel(response.getRule());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "查询列表成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updateListenerRule(LbListenerRuleModel lb) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);
			UpdateL7ruleRequest request = new UpdateL7ruleRequest();
			UpdateL7ruleRequestBody body = new UpdateL7ruleRequestBody();
			UpdateL7ruleReq rule = new UpdateL7ruleReq();
			rule.withCompareType(lb.getCompareType()).withKey(lb.getRuleKey()).withValue(lb.getRuleValue())
					.withInvert(lb.getInvert());
			body.withRule(rule);
			request.withL7policyId(lb.getPolicyUuid()).withL7ruleId(lb.getRuleUuid()).withBody(body);
			UpdateL7ruleResponse response = client.updateL7rule(request);
			LbListenerRuleModel lbModel = convetorL7rule.convertModel(response.getRule());
			lbModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, lbModel, "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult removeListenerRule(String policyId, String ruleId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			DeleteL7ruleRequest request = new DeleteL7ruleRequest();
			request.withL7policyId(policyId).withL7ruleId(ruleId);
			DeleteL7ruleResponse response = client.deleteL7rule(request);

			return new BocloudResult(true, "移除成功");
		} catch (Exception e) {
			logger.error("移除失败： " + e);
			return new BocloudResult(false, "移除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detailListenerRule(String policyId, String ruleId) {
		try {
			ElbClient client = (ElbClient) getClient(HuaweiIndex.service.ELB);

			ListL7rulesRequest request = new ListL7rulesRequest();
			request.withL7policyId(policyId);
			ListL7rulesResponse response = client.listL7rules(request);
			List<L7ruleResp> l7rule = response.getRules();
			if (CollectionUtils.isEmpty(l7rule)) {
				return new BocloudResult(false, "查询详情失败");
			}

			LbListenerRuleModel lbModel = convetorL7rule.convertModel(l7rule.get(0));
			lbModel.setRegionId(this.getRegionId());
			lbModel.setPolicyUuid(policyId);
			return new BocloudResult(true, lbModel, "查询成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败", e.getMessage());
		}
	}

}
