package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.ElasticIpConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.eip.v2.EipClient;
import com.huaweicloud.sdk.eip.v2.model.*;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.huawei")
public class HuaweiElasticIpProvider extends HuaweiProvider implements ElasticIpProvider {

	public HuaweiElasticIpProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiElasticIpProvider.class);

	private BeanConvertor<ElasticIpModel, PublicipShowResp> convetor = new ElasticIpConvertor();

	@Override
	public BocloudResult list() {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			// 获取带宽
			Map<String, BandwidthResp> bandwidthRespMap = buildBandwidthMap(client);
			List<PublicipShowResp> allPublicips = new ArrayList<>();
			ListPublicipsRequest request = new ListPublicipsRequest();
			request.withLimit(PAGE_SIZE);
			ListPublicipsResponse response = client.listPublicips(request);
			List<PublicipShowResp> publicips = response.getPublicips();
			allPublicips.addAll(publicips);
			while (null != publicips && publicips.size() == PAGE_SIZE) {
				PublicipShowResp publicipShowResp = publicips.get(publicips.size() - 1);
				request.withMarker(publicipShowResp.getId());
				response = client.listPublicips(request);
				publicips = response.getPublicips();
				allPublicips.addAll(publicips);
			}
			List<ElasticIpModel> elasticIpModels = convetor.convertList(allPublicips);
			for (ElasticIpModel elasticIpModel : elasticIpModels) {
				elasticIpModel.setBandwidthChargeMode(
						bandwidthRespMap.get(elasticIpModel.getBandwidthUuid()).getChargeMode().getValue());
			}
			return new BocloudResult(true, elasticIpModels, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询浮动IP列表失败： " + e);
			return new BocloudResult(false, "查询列表失败", e.getMessage());
		}
	}

	public Map<String, BandwidthResp> buildBandwidthMap(EipClient client) {
		Map<String, BandwidthResp> bandwidthRespMap = new HashMap<>();
		try {
			List<BandwidthResp> allBandwidthResps = new ArrayList<>();
			ListBandwidthsRequest bandwidthsRequest = new ListBandwidthsRequest();
			bandwidthsRequest.withLimit(PAGE_SIZE);
			ListBandwidthsResponse bandwidthsResponse = client.listBandwidths(bandwidthsRequest);
			List<BandwidthResp> bandwidthResps = bandwidthsResponse.getBandwidths();
			allBandwidthResps.addAll(bandwidthResps);
			while (null != bandwidthResps && bandwidthResps.size() == PAGE_SIZE) {
				BandwidthResp bandwidthResp = bandwidthResps.get(bandwidthResps.size() - 1);
				bandwidthsRequest.withMarker(bandwidthResp.getId());
				bandwidthsResponse = client.listBandwidths(bandwidthsRequest);
				bandwidthResps = bandwidthsResponse.getBandwidths();
				allBandwidthResps.addAll(bandwidthResps);
			}
			if (!ListTool.isEmpty(allBandwidthResps)) {
				for (BandwidthResp bandwidthResp : allBandwidthResps) {
					bandwidthRespMap.put(bandwidthResp.getId(), bandwidthResp);
				}
			}
		} catch (Exception e) {
			logger.error("获取 BandwidthResp 异常 ： " + e);
		}
		return bandwidthRespMap;
	}

	public static void main(String[] args) {
		Butler newButler = new Butler();
		newButler.setUrl("hhcloud.cn");
		newButler.setAccessKey("OYLRP3ZPQHIPVCWAEDXT");
		newButler.setSecretKey("66jz27svEcJHUaSFeVN89bjBJbW5oU04HbT578rx");
		newButler.setRegionId("cn-central-220");
		HuaweiVpcProvider imageProvider = new HuaweiVpcProvider(newButler);
		imageProvider.list();
	}

	@Override
	public BocloudResult allocate(ElasticIpModel model) {
		try {
//        	BGP("5_bgp"),			// 全动态BGP
//        	SBGP("5_sbgp"),			// 静态BGP
//        	Telcom("5_telcom"),		// 中国电信
//        	Union("5_union"),		// 中国联通
//        	Mobile("5_chinamobile");// 中国移动
			String type = StringUtils.isEmpty(model.getType()) ? "5_bgp" : model.getType();
			// 大连特殊处理
			if ("cn-northeast-1".equalsIgnoreCase(this.getRegionId())) {
				type = "5_telcom";
			}
			// 西南-贵阳一
			if ("cn-southwest-2".equalsIgnoreCase(this.getRegionId())) {
				type = "5_sbgp";
			}
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			CreatePublicipRequest request = new CreatePublicipRequest();
			CreatePublicipRequestBody body = new CreatePublicipRequestBody();
			CreatePublicipBandwidthOption bandwidthOption = new CreatePublicipBandwidthOption();
			String bandwidthShareType = model.getBandwidthShareType();
			bandwidthOption.withShareType(CreatePublicipBandwidthOption.ShareTypeEnum.valueOf(bandwidthShareType));
			switch (bandwidthShareType) {
			case "WHOLE":
				bandwidthOption.withId(model.getBandwidthUuid());
				break;
			case "PER":
				bandwidthOption
						.withChargeMode(
								CreatePublicipBandwidthOption.ChargeModeEnum.valueOf(model.getBandwidthChargeMode()))
						.withName(model.getBandwidthName()).withSize(Integer.valueOf(model.getBandwidth()));
				break;
			default:
				break;
			}
			CreatePublicipOption publicipOption = new CreatePublicipOption();
			publicipOption.withType(type)
					.withIpVersion(CreatePublicipOption.IpVersionEnum.valueOf(Integer.valueOf(model.getIpVersion())));
			if (StringUtils.isNoneEmpty(model.getPublicIp())) {
				publicipOption.withIpAddress(model.getPublicIp());
			}
			body.withBandwidth(bandwidthOption).withPublicip(publicipOption);

			String enterpriseProjectUuid = model.getEnterpriseProjectUuid();
			if (!StringUtils.isEmpty(enterpriseProjectUuid)) {
				body.withEnterpriseProjectId(enterpriseProjectUuid);
			}

			request.withBody(body);
			CreatePublicipResponse response = client.createPublicip(request);

			Long startTime = System.currentTimeMillis();
			Long endTime = System.currentTimeMillis();
			while (endTime - startTime < 5 * 60 * 1000) {
				// 创建时间不确定，增加休眠减少轮询次数
				Thread.sleep(5000);
				if (response.getPublicip().getBandwidthSize() != null) {
					break;
				}
				endTime = System.currentTimeMillis();
			}

			model = (ElasticIpModel) detail(response.getPublicip().getId()).getData();
			model.setRegionId(this.getRegionId());
			return new BocloudResult(true, model, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败：" + this.getChineseMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult allocatePrePaid(ElasticIpModel model) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			CreatePrePaidPublicipRequest request = new CreatePrePaidPublicipRequest();
			CreatePrePaidPublicipRequestBody body = new CreatePrePaidPublicipRequestBody();
			CreatePublicipBandwidthOption bandwidthOption = new CreatePublicipBandwidthOption();
			bandwidthOption
					.withShareType(CreatePublicipBandwidthOption.ShareTypeEnum.valueOf(model.getBandwidthShareType()))
					.withName(model.getBandwidthName())
					.withChargeMode(
							CreatePublicipBandwidthOption.ChargeModeEnum.valueOf(model.getBandwidthChargeMode()))
					.withId(model.getBandwidthUuid()).withSize(Integer.valueOf(model.getBandwidth()));
			CreatePrePaidPublicipOption publicipOption = new CreatePrePaidPublicipOption();
			publicipOption.withType(model.getType())
					.withIpVersion(
							CreatePrePaidPublicipOption.IpVersionEnum.valueOf(Integer.valueOf(model.getIpVersion())))
					.withType(model.getType());
			CreatePrePaidPublicipExtendParamOption extendParamOption = new CreatePrePaidPublicipExtendParamOption();
			extendParamOption
					.withChargeMode(
							CreatePrePaidPublicipExtendParamOption.ChargeModeEnum.valueOf(model.getChargeMode()))
					.withIsAutoPay(model.getIsAutoPay()).withIsAutoRenew(model.getIsAutoRenew())
					.withPeriodNum(model.getPeriodNum()).withPeriodType(
							CreatePrePaidPublicipExtendParamOption.PeriodTypeEnum.valueOf(model.getPeriodType()));

			body.withBandwidth(bandwidthOption).withPublicip(publicipOption).withExtendParam(extendParamOption);
			request.withBody(body);
			CreatePrePaidPublicipResponse response = client.createPrePaidPublicip(request);
			model = (ElasticIpModel) detail(response.getPublicip().getId()).getData();
			return new BocloudResult(true, model, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult release(String publicipId) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			DeletePublicipRequest request = new DeletePublicipRequest();
			request.withPublicipId(publicipId);
			DeletePublicipResponse response = client.deletePublicip(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("删除失败： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String publicipId) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			ShowPublicipRequest request = new ShowPublicipRequest();
			request.withPublicipId(publicipId);
			ShowPublicipResponse response = client.showPublicip(request);
			ElasticIpModel elasticIpModel = convetor.convertModel(response.getPublicip());

			ShowBandwidthRequest bandwidthRequest = new ShowBandwidthRequest();
			bandwidthRequest.withBandwidthId(elasticIpModel.getBandwidthUuid());
			ShowBandwidthResponse bandwidthResponse = client.showBandwidth(bandwidthRequest);
			elasticIpModel.setBandwidthChargeMode(bandwidthResponse.getBandwidth().getChargeMode().getValue());
			return new BocloudResult(true, elasticIpModel, "查询成功");
		} catch (Exception e) {
			logger.error("查询失败： " + e);
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult update(ElasticIpModel model) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			UpdatePublicipRequest request = new UpdatePublicipRequest();
			UpdatePublicipsRequestBody body = new UpdatePublicipsRequestBody();
			UpdatePublicipOption option = new UpdatePublicipOption();
			if (StringUtils.isNotEmpty(model.getIpVersion())) {
				option.withIpVersion(UpdatePublicipOption.IpVersionEnum.valueOf(Integer.valueOf(model.getIpVersion())));
			} else {
				option.withPortId(model.getPortId());
			}
			body.withPublicip(option);
			request.withPublicipId(model.getElasticIpUuid()).withBody(body);
			UpdatePublicipResponse response = client.updatePublicip(request);
			return new BocloudResult(true, convetor.convertModel(response.getPublicip()), "更新成功");
		} catch (Exception e) {
			logger.error("更新失败： " + e);
			return new BocloudResult(false, "更新失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult associate(ElasticIpModel model) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			NeutronUpdateFloatingIpRequest request = new NeutronUpdateFloatingIpRequest();
			NeutronUpdateFloatingIpRequestBody body = new NeutronUpdateFloatingIpRequestBody();
			UpdateFloatingIpOption floatingip = new UpdateFloatingIpOption();
			floatingip.setPortId(model.getPortId());
			body.setFloatingip(floatingip);
			request.setBody(body);
			request.setFloatingipId(model.getElasticIpUuid());
			NeutronUpdateFloatingIpResponse neutronUpdateFloatingIp = client.neutronUpdateFloatingIp(request);
			String portId = neutronUpdateFloatingIp.getFloatingip() == null ? ""
					: neutronUpdateFloatingIp.getFloatingip().getPortId();
			return new BocloudResult(true, portId, "关联成功");
		} catch (Exception e) {
			logger.error("关联失败： " + e);
			return new BocloudResult(false, "关联失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult disassociate(ElasticIpModel model) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			NeutronUpdateFloatingIpRequest request = new NeutronUpdateFloatingIpRequest();
			NeutronUpdateFloatingIpRequestBody body = new NeutronUpdateFloatingIpRequestBody();
			UpdateFloatingIpOption floatingip = new UpdateFloatingIpOption();
			floatingip.setPortId(null);
			body.setFloatingip(floatingip);
			request.setBody(body);
			request.setFloatingipId(model.getElasticIpUuid());
			NeutronUpdateFloatingIpResponse neutronUpdateFloatingIp = client.neutronUpdateFloatingIp(request);
			return new BocloudResult(true, neutronUpdateFloatingIp.getFloatingip(), "解除成功");
		} catch (Exception e) {
			logger.error("解除失败： " + e);
			return new BocloudResult(false, "解除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult updateBandwidth(ElasticIpModel model) {
		try {
			EipClient client = (EipClient) getClient(HuaweiIndex.service.EIP_VPC);
			UpdateBandwidthRequest request = new UpdateBandwidthRequest();
			UpdateBandwidthRequestBody body = new UpdateBandwidthRequestBody();
			UpdateBandwidthOption bandwidth = new UpdateBandwidthOption();

			UpdateBandwidthOption.ChargeModeEnum chargeMode = UpdateBandwidthOption.ChargeModeEnum
					.fromValue(model.getBandwidthChargeMode());
			bandwidth.withName(model.getBandwidthName()).withChargeMode(chargeMode);

			bandwidth.withSize(Integer.parseInt(model.getBandwidth()));
			body.withBandwidth(bandwidth);
			request.withBandwidthId(model.getBandwidthUuid()).withBody(body);
			UpdateBandwidthResponse response = client.updateBandwidth(request);

			BandwidthResp responseBandwidth = response.getBandwidth();
			model.setBandwidthName(responseBandwidth.getName());
			model.setBandwidthChargeMode(responseBandwidth.getChargeMode().getValue());
			model.setBandwidth(responseBandwidth.getSize().toString());

			return new BocloudResult(true, model, "修改成功");
		} catch (Exception e) {
			logger.error("修改失败： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}
}
