package com.bocloud.cmp.provider.network;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.NatGatewayDnatRuleConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.NatGatewayDnatRuleModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.huaweicloud.sdk.nat.v2.NatClient;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayDnatOption;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayDnatRuleOption;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayDnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.CreateNatGatewayDnatRuleResponse;
import com.huaweicloud.sdk.nat.v2.model.DeleteNatGatewayDnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewayDnatRulesRequest;
import com.huaweicloud.sdk.nat.v2.model.ListNatGatewayDnatRulesResponse;
import com.huaweicloud.sdk.nat.v2.model.NatGatewayDnatRuleResponseBody;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewayDnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.ShowNatGatewayDnatRuleResponse;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayDnatRuleOption;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayDnatRuleRequest;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayDnatRuleRequestBody;
import com.huaweicloud.sdk.nat.v2.model.UpdateNatGatewayDnatRuleResponse;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.FixedIp;
import com.huaweicloud.sdk.vpc.v2.model.Port;
import com.huaweicloud.sdk.vpc.v2.model.ShowPortRequest;

import io.jsonwebtoken.lang.Assert;

@Provider("bocloud.cmp.huawei")
public class HuaweiNatGatewayDnatRuleProvider extends HuaweiProvider implements NatGatewayDnatRuleProvider {

	public HuaweiNatGatewayDnatRuleProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiNatGatewayDnatRuleProvider.class);

	private BeanConvertor<NatGatewayDnatRuleModel, NatGatewayDnatRuleResponseBody> convetor = new NatGatewayDnatRuleConvertor();

	@Override
	public BocloudResult list() {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ListNatGatewayDnatRulesRequest request = new ListNatGatewayDnatRulesRequest();
			request.withLimit(2000);
			ListNatGatewayDnatRulesResponse response = client.listNatGatewayDnatRules(request);
			List<NatGatewayDnatRuleModel> convertList = convetor.convertList(response.getDnatRules());
			convertList = this.getAddress(convertList);
			return new BocloudResult(true, convertList, "查询列表成功");
		} catch (Exception e) {
			logger.error("查询列表失败： " + e);
			return new BocloudResult(false, "查询列表失败:" + e);
		}
	}

	private List<NatGatewayDnatRuleModel> getAddress(List<NatGatewayDnatRuleModel> convertList) {
		try {
			if (ListTool.isEmpty(convertList)) {
				return convertList;
			}
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			for (NatGatewayDnatRuleModel natGatewayDnatRuleModel : convertList) {
				String privateIp = natGatewayDnatRuleModel.getPrivateIp();
				if (!StringUtils.isEmpty(privateIp)) {
					continue;
				}
				String portUuid = natGatewayDnatRuleModel.getPortUuid();
				ShowPortRequest request = new ShowPortRequest();
				request.setPortId(portUuid);
				Port resultPort = client.showPort(request).getPort();
				List<FixedIp> fixedIps = resultPort.getFixedIps();
				if (ListTool.isEmpty(fixedIps)) {
					continue;
				}
				FixedIp fixedIp = fixedIps.get(0);
				if (null != fixedIp) {
					natGatewayDnatRuleModel.setPrivateIp(fixedIp.getIpAddress());
				}
			}
		} catch (Exception e) {
			logger.error("NatGatewayDnatRule 获取IP地址异常：" + e.getMessage());
		}
		return convertList;
	}

	/*
	 * 创建规则时,要求网关状态status = ACTIVE,要求网关管理员状态admin_state_up = True。
	 * port_id和private_ip不能同时生效。 对于all port类型的规则,要求internal_service_port =
	 * 0,external_service_port = 0, protocol = ANY.
	 */
	@Override
	public BocloudResult create(NatGatewayDnatRuleModel model) {
		try {
			Assert.isTrue(StringUtils.isNotEmpty(model.getNatGatewayId()), "参数natGatewayId不能为空");
			Assert.isTrue(model.getInternalServicePort() != null, "参数internalServicePort不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getFloatingIpId()), "参数floatingIpId不能为空");
			Assert.isTrue(model.getExternalServicePort() != null, "参数enternalServicePort不能为空");
			Assert.isTrue(StringUtils.isNotEmpty(model.getProtocol()), "参数protocol不能为空");
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			CreateNatGatewayDnatRuleRequest request = new CreateNatGatewayDnatRuleRequest();
			CreateNatGatewayDnatRuleOption body = new CreateNatGatewayDnatRuleOption();
			CreateNatGatewayDnatOption dnatRulebody = new CreateNatGatewayDnatOption();
			// protocol选择any全部端口时，internalServicePort和externalServicePort默认为0
			// 选择tcp和upd时，InternalServicePortRange和ExternalServicePortRange优先生效
			dnatRulebody.withNatGatewayId(model.getNatGatewayId())
					.withInternalServicePort(model.getInternalServicePort()).withFloatingIpId(model.getFloatingIpId())
					.withExternalServicePort(model.getExternalServicePort()).withProtocol(model.getProtocol())
					.withInternalServicePortRange(model.getInternalServicePortRange())
					.withExternalServicePortRange(model.getExternalServicePortRange())
					.withDescription(model.getRemark());
			// 用户私有IP地址,对应专线、云连接场景,与port_id参数二选一。
			if (model.getPortUuid() != null && !"".equalsIgnoreCase(model.getPortUuid())) {
				dnatRulebody.withPortId(model.getPortUuid());
			} else if (model.getPrivateIp() != null && !"".equalsIgnoreCase(model.getPrivateIp())) {
				dnatRulebody.withPrivateIp(model.getPrivateIp());
			}

			body.withDnatRule(dnatRulebody);
			request.withBody(body);
			CreateNatGatewayDnatRuleResponse response = client.createNatGatewayDnatRule(request);
			return new BocloudResult(true, waitForServerStatus(response.getDnatRule().getId()).getData(), "创建dnat规则成功");
		} catch (Exception e) {
			logger.info("创建dnat规则失败" + e.getMessage());
			return new BocloudResult(false, "创建dnat规则失败:" + e.getMessage());
		}
	}

	private BocloudResult waitForServerStatus(String dnatRuleId) {
		BocloudResult result = new BocloudResult(false, "创建失败");
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			result = this.detail(dnatRuleId);
			if (result.isFailed()) {
				return result;
			}
			NatGatewayDnatRuleModel gateway = JSON.parseObject(JSON.toJSONString(result.getData()),
					NatGatewayDnatRuleModel.class);
			if (null == gateway) {
				return result;
			}
			if ("ACTIVE".equalsIgnoreCase(gateway.getStatus())) {
				flag = false;
				break;
			} else {
				try {
					Thread.sleep(5000);
				} catch (InterruptedException e) {
					logger.error("Get error message:", e);
					Thread.currentThread().interrupt();
				}
			}
			Long endTime = System.currentTimeMillis();
			// 等待三分钟
			if (endTime - startTime > 15 * 60 * 1000) {
				flag = false;
				break;
			}
		}
		return result;
	}

	/*
	 * 更新操作时,要求DNAT规则状态status = ACTIVE,要求网关管理员状态admin_state_up =
	 * True。port_id和private_ip不能同时生效。 对于all port类型的规则,要求internal_service_port =
	 * 0,external_service_port = 0,protocol = ANY。
	 * 更新操作涉及以下字段更新时,要求这些字段必须一起更新。包括:port_id、private_ip、internal_service_port、
	 * external_service_port、floating_ip_id、protocol、internal_service_port_range、
	 * external_service_port_range。
	 */
	@Override
	public BocloudResult modify(NatGatewayDnatRuleModel model) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			UpdateNatGatewayDnatRuleRequest request = new UpdateNatGatewayDnatRuleRequest();
			request.withDnatRuleId(model.getUuid());
			UpdateNatGatewayDnatRuleRequestBody body = new UpdateNatGatewayDnatRuleRequestBody();
			UpdateNatGatewayDnatRuleOption dnatRulebody = new UpdateNatGatewayDnatRuleOption();
			dnatRulebody.withNatGatewayId(model.getNatGatewayId())
					.withInternalServicePort(model.getInternalServicePort()).withFloatingIpId(model.getFloatingIpId())
					.withExternalServicePort(model.getExternalServicePort())
					.withProtocol(UpdateNatGatewayDnatRuleOption.ProtocolEnum.fromValue(model.getProtocol()))
					.withPortId(model.getPortUuid()).withDescription(model.getRemark());
			if (!StringUtils.isEmpty(model.getInternalServicePortRange())) {
				dnatRulebody.withInternalServicePortRange(model.getInternalServicePortRange());
			}
			if (!StringUtils.isEmpty(model.getExternalServicePortRange())) {
				dnatRulebody.withExternalServicePortRange(model.getExternalServicePortRange());
			}
			body.withDnatRule(dnatRulebody);
			request.withBody(body);
			UpdateNatGatewayDnatRuleResponse response = client.updateNatGatewayDnatRule(request);
			NatGatewayDnatRuleModel data = (NatGatewayDnatRuleModel) waitForServerStatus(response.getDnatRule().getId())
					.getData();
			data.setPrivateIp(model.getPrivateIp());
			return new BocloudResult(true, data, "修改dnat规则成功");
		} catch (Exception e) {
			logger.info("修改dnat规则失败" + e.getMessage());
			return new BocloudResult(false, "修改dnat规则失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String gatewayId, String dnanRuleId) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			DeleteNatGatewayDnatRuleRequest request = new DeleteNatGatewayDnatRuleRequest();
			request.withNatGatewayId(gatewayId);
			request.withDnatRuleId(dnanRuleId);
			client.deleteNatGatewayDnatRule(request);
			return new BocloudResult(true, "删除dnat规则成功");
		} catch (Exception e) {
			logger.info("删除dnat规则失败" + e.getMessage());
			return new BocloudResult(false, "删除dnat规则失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String ruleId) {
		try {
			NatClient client = (NatClient) getClient(HuaweiIndex.service.NAT);
			ShowNatGatewayDnatRuleRequest request = new ShowNatGatewayDnatRuleRequest();
			request.withDnatRuleId(ruleId);
			ShowNatGatewayDnatRuleResponse response = client.showNatGatewayDnatRule(request);
			NatGatewayDnatRuleModel dnatRuleModel = convetor.convertModel(response.getDnatRule());
			dnatRuleModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, dnatRuleModel, "查询详情成功");
		} catch (Exception e) {
			logger.error("查询详情失败： " + e);
			return new BocloudResult(false, "查询详情失败:" + e.getMessage());
		}
	}

}
