package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.SecurityGroupRuleConvertor;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.huaweicloud.sdk.vpc.v2.VpcClient;
import com.huaweicloud.sdk.vpc.v2.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

@Provider("bocloud.cmp.huawei")
public class HuaweiSecuGroupRuleProvider extends HuaweiProvider implements SecurityGroupRuleProvider {

	public HuaweiSecuGroupRuleProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(HuaweiSecuGroupRuleProvider.class);

	private BeanConvertor<SecurityGroupRuleModel, SecurityGroupRule> ruleConvertor = new SecurityGroupRuleConvertor();

	@Override
	public BocloudResult create(SecurityGroupRuleModel model) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			CreateSecurityGroupRuleRequest request = new CreateSecurityGroupRuleRequest();
			CreateSecurityGroupRuleRequestBody body = new CreateSecurityGroupRuleRequestBody();
			CreateSecurityGroupRuleOption securityGroupRulebody = new CreateSecurityGroupRuleOption();
			securityGroupRulebody.withSecurityGroupId(model.getGroupUuid()).withDirection(model.getDirection())
					.withEthertype(model.getEtherType()).withProtocol(model.getProtocol())
					.withPortRangeMin(model.getPortMin()).withPortRangeMax(model.getPortMax())
					.withRemoteIpPrefix(model.getRemoteIpPrefix());
			body.withSecurityGroupRule(securityGroupRulebody);
			request.withBody(body);
			CreateSecurityGroupRuleResponse response = client.createSecurityGroupRule(request);
			return new BocloudResult(true, ruleConvertor.convertModel(response.getSecurityGroupRule()), "创建成功");
		} catch (Exception e) {
			logger.error("创建失败： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel model) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			DeleteSecurityGroupRuleRequest request = new DeleteSecurityGroupRuleRequest();
			request.withSecurityGroupRuleId(model.getRuleUuid());
			client.deleteSecurityGroupRule(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("删除失败： " + e.getMessage());
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String ruleId) {
		try {
			VpcClient client = (VpcClient) getClient(HuaweiIndex.service.VPC);
			DeleteSecurityGroupRuleRequest request = new DeleteSecurityGroupRuleRequest();
			request.withSecurityGroupRuleId(ruleId);
			client.deleteSecurityGroupRule(request);
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("删除失败： " + e.getMessage());
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

}
