package com.bocloud.cmp.provider.security;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.api.model.HuaweiIndex;
import com.bocloud.cmp.huaweiconvertor.KeypairConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.HuaweiProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.huaweicloud.sdk.ecs.v2.EcsClient;
import com.huaweicloud.sdk.ecs.v2.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 秘钥
 *
 * @author ymh
 * @时间 2018年11月29日
 */

@Provider("bocloud.cmp.huawei")
public class HuaweiKeypairProvider extends HuaweiProvider implements KeypairProvider {

    private static Logger logger = LoggerFactory.getLogger(HuaweiKeypairProvider.class);

    private KeypairConvertor convertor = new KeypairConvertor();

    public HuaweiKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
            NovaListKeypairsRequest request = new NovaListKeypairsRequest();
            NovaListKeypairsResponse response = client.novaListKeypairs(request);

            List<KeypairModel> convertList = convertor.convertList(response.getKeypairs());
            for (KeypairModel keypairModel : convertList) {
                keypairModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询列表成功");
        } catch (Exception e) {
            logger.error("查询列表失败 ： " + e);
            return new BocloudResult(false, e.getMessage(), "查询列表失败");
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        try {
            EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
            NovaShowKeypairRequest request = new NovaShowKeypairRequest();
            request.setKeypairName(keypairName);
            NovaShowKeypairResponse response = client.novaShowKeypair(request);
            NovaSimpleKeypair keypair = JSONObject.parseObject(JSONObject.toJSONString(response.getKeypair()), NovaSimpleKeypair.class);
            NovaListKeypairsResult keypairsResult = new NovaListKeypairsResult();
            keypairsResult.setKeypair(keypair);
            KeypairModel convertModel = convertor.convertModelDetail(response.getKeypair());
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "查询详情成功");
        } catch (Exception e) {
            logger.error("查询详情失败 ： " + e);
            return new BocloudResult(false, e.getMessage(), "查询详情失败");
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        try {
            if (publicKey == null || "".equals(publicKey)) {
                publicKey = null;
            }
            EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
            NovaCreateKeypairRequest request = new NovaCreateKeypairRequest();
            NovaCreateKeypairRequestBody body = new NovaCreateKeypairRequestBody();
            NovaCreateKeypairOption option = new NovaCreateKeypairOption();
            option.setName(keypairName);
            option.setPublicKey(publicKey);
            body.setKeypair(option);
            request.setBody(body);
            NovaCreateKeypairResponse response = client.novaCreateKeypair(request);
            KeypairModel convertModel = convertor.convertModelPair(response.getKeypair());
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "创建成功");
        } catch (Exception e) {
            logger.error("创建失败 ： " + e);
            return new BocloudResult(false, "创建失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(String keypairName) {
        try {
            EcsClient client = (EcsClient) getClient(HuaweiIndex.service.ECS);
            NovaDeleteKeypairRequest request = new NovaDeleteKeypairRequest();
            request.setKeypairName(keypairName);
            NovaDeleteKeypairResponse response = client.novaDeleteKeypair(request);
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("删除失败 ： " + e);
            return new BocloudResult(false, "删除失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        // TODO Auto-generated method stub
        return null;
    }

}
