package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.common.JdCloudLocationProvider;
import com.jdcloud.sdk.auth.CredentialsProvider;
import com.jdcloud.sdk.auth.StaticCredentialsProvider;
import com.jdcloud.sdk.service.cps.client.CpsClient;
import com.jdcloud.sdk.service.disk.client.DiskClient;
import com.jdcloud.sdk.service.mongodb.client.MongodbClient;
import com.jdcloud.sdk.service.mongodb.model.AvailableZones;
import com.jdcloud.sdk.service.mongodb.model.DescribeAvailableZonesRequest;
import com.jdcloud.sdk.service.mongodb.model.DescribeAvailableZonesResponse;
import com.jdcloud.sdk.service.mongodb.model.DescribeAvailableZonesResult;
import com.jdcloud.sdk.service.vm.client.VmClient;
import com.jdcloud.sdk.service.vpc.client.VpcClient;

import java.util.List;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:23 2018/11/14 @ Modified
 */
public class JdCloudProvider extends GenericProvider implements SuperProvider {

    private String accessKey;// 访问key
    private String secretKey;// 安全key
    private String region;// 地域
    /*
     * "cn-south-1" 华南-广州 "cn-east-1" 华东-宿迁 "cn-east-2" 华东-上海 "cn-north-1" 华北-北京
     */
    private String regionId = "cn-north-1";
    private CredentialsProvider credentialsProvider;// 证书提供者
    private Butler butler;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public JdCloudProvider(Butler butler) {
        super(butler);
        this.accessKey = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.region = butler.getRegionId() == null ? regionId : butler.getRegionId();
        this.regionId = butler.getRegionId() == null ? regionId : butler.getRegionId();
        this.credentialsProvider = new StaticCredentialsProvider(accessKey, secretKey);
        this.butler = butler;
    }

    /**
     * @return the accessKey
     */
    public String getAccessKey() {
        return accessKey;
    }

    /**
     * @param accessKey the accessKey to set
     */
    public void setAccessKey(String accessKey) {
        this.accessKey = accessKey;
    }

    /**
     * @return the secretKey
     */
    public String getSecretKey() {
        return secretKey;
    }

    /**
     * @param secretKey the secretKey to set
     */
    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    /**
     * @return the region
     */
    public String getRegion() {
        return region;
    }

    /**
     * @param region the region to set
     */
    public void setRegion(String region) {
        this.region = region;
    }

    /**
     * @return the vmClient
     */
    public VmClient getVmClient() {
        return VmClient.builder().credentialsProvider(credentialsProvider).build();
    }

    public MongodbClient getMongodbClient() {
        return MongodbClient.builder().credentialsProvider(credentialsProvider).build();
    }

    public VpcClient getVpcClient() {
        return VpcClient.builder().credentialsProvider(credentialsProvider).build();
    }

    public DiskClient getDiskClient() {
        return DiskClient.builder().credentialsProvider(credentialsProvider).build();
    }

    public CpsClient getCpsClient() {
        return CpsClient.builder().credentialsProvider(credentialsProvider).build();
    }

    /**
     * 查询当前region下的az 默认当前域下的第一个az
     *
     * @return
     */
    public List<AvailableZones> zoneList() {
        DescribeAvailableZonesRequest request = new DescribeAvailableZonesRequest();
        request.setRegionId(this.getRegionId());
        DescribeAvailableZonesResponse availableZones = this.getMongodbClient().describeAvailableZones(request);
        DescribeAvailableZonesResult result = availableZones.getResult();
        List<AvailableZones> zones = result.getAvailableZones();
        return zones;
    }

    @Override
    public boolean accessable() {
        JdCloudLocationProvider location = new JdCloudLocationProvider(butler);
        return location.listRegions().isSuccess();
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

}