package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.InstanceTypeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vm.model.DescribeInstanceTypesRequest;
import com.jdcloud.sdk.service.vm.model.DescribeInstanceTypesResponse;
import com.jdcloud.sdk.service.vm.model.DescribeInstanceTypesResult;
import com.jdcloud.sdk.service.vm.model.InstanceType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.List;

/**
 * @author : guopeng create at: 2018/11/15 3:26 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudFlavorProvider extends JdCloudProvider implements FlavorProvider {

	private static Logger logger = LoggerFactory.getLogger(JdCloudFlavorProvider.class);
	private BeanConvertor<FlavorModel, InstanceType> instTypeConvertor = new InstanceTypeConvertor();

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler
	 *            云平台认证信息
	 */
	public JdCloudFlavorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		try {
			// 云主机实例类型 默认同步第一个az下的 因为创建云主机的时候指定为第一个az 后续加上可用区 可用域
			DescribeInstanceTypesRequest request = new DescribeInstanceTypesRequest();
			request.setRegionId(this.getRegionId());
			DescribeInstanceTypesResponse response = this.getVmClient().describeInstanceTypes(request);
			DescribeInstanceTypesResult result = response.getResult();
			List<InstanceType> instanceTypes = result.getInstanceTypes();
			List<FlavorModel> convertList = instTypeConvertor.convertList(instanceTypes);
			for (FlavorModel flavorModel : convertList) {
				flavorModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, convertList, "同步成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "同步失败");
		}
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		return null;
	}

	@Override
	public BocloudResult remove(String flavorId) {
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		return null;
	}

}
