package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.compute.JdCloudImageProvider;
import com.bocloud.cmp.provider.compute.JdCloudServerProvider;
import com.bocloud.cmp.provider.compute.JdCloudSnapshotProvider;
import com.bocloud.cmp.provider.network.JdCloudElasticIpProvider;
import com.bocloud.cmp.provider.network.JdCloudRouterTableProvider;
import com.bocloud.cmp.provider.network.JdCloudSecurityProvider;
import com.bocloud.cmp.provider.network.JdCloudVpcProvider;
import com.bocloud.cmp.provider.security.JdCloudKeypairProvider;
import com.bocloud.cmp.provider.storage.JdCloudVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.JdcloudSdkException;

/**
 * @author ymh
 * @时间 2019年1月17日
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudLocationProvider extends JdCloudProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudLocationProvider.class);

    private Butler butler;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    /**
     * 通过京东工单反应 京东云目前没有云主机的可用区接口 目前根据区域代码写死
     */
    @Override
    public BocloudResult listRegions() {
        try {
            // 北京区域 编码
            RegionModel bj = new RegionModel();
            String bjrid = "cn-north-1";
            String bjName = "华北-北京";
            bj.setRegionId(bjrid);
            bj.setCode(bjrid);
            bj.setName(bjName);
            List<ZoneModel> zones = new ArrayList<>();
            ZoneModel zoneModel = null;
            for (int i = 1; i <= 3; i++) {
                zoneModel = new ZoneModel();
                zoneModel.setName(bjName + " " + "可用区" + (char)(64 + i));
                zoneModel.setRegionId(bjrid);
                zoneModel.setZoneId(bjrid + (char)(96 + i));
                zones.add(zoneModel);
            }
            bj.setZones(zones);
            // 宿迁区域编码
            RegionModel sq = new RegionModel();
            String sqrid = "cn-east-1";
            String sqName = "华东-宿迁";
            sq.setRegionId(sqrid);
            sq.setCode(sqrid);
            sq.setName(sqName);
            List<ZoneModel> zqZones = new ArrayList<>();
            zoneModel = new ZoneModel();
            zoneModel.setName(sqName + " " + "可用区" + (char)65);
            zoneModel.setRegionId(sqrid);
            zoneModel.setZoneId(sqrid + (char)97);
            zqZones.add(zoneModel);
            sq.setZones(zqZones);

            // 上海区域 编码
            RegionModel sh = new RegionModel();
            String shrid = "cn-east-2";
            String shName = "华东-上海";
            sh.setRegionId(shrid);
            sh.setCode(shrid);
            sh.setName(shName);
            List<ZoneModel> shZones = new ArrayList<>();
            for (int i = 1; i <= 3; i++) {
                zoneModel = new ZoneModel();
                zoneModel.setName(shName + " " + "可用区" + (char)(64 + i));
                zoneModel.setRegionId(shrid);
                zoneModel.setZoneId(shrid + (char)(96 + i));
                shZones.add(zoneModel);
            }
            sh.setZones(shZones);

            // 宿迁区域编码
            RegionModel gz = new RegionModel();
            String gzrid = "cn-south-1";
            String gzName = "华南-广州";
            gz.setRegionId(gzrid);
            gz.setCode(gzrid);
            gz.setName(gzName);
            List<ZoneModel> gzZones = new ArrayList<>();
            zoneModel = new ZoneModel();
            zoneModel.setName(gzName + " " + "可用区" + (char)65);
            zoneModel.setRegionId(gzrid);
            zoneModel.setZoneId(gzrid + (char)97);
            gzZones.add(zoneModel);
            gz.setZones(gzZones);

            List<RegionModel> models = new ArrayList<>();
            models.add(bj);
            models.add(sq);
            models.add(gz);
            models.add(sh);
            return new BocloudResult(true, models, "查询成功！");
        } catch (JdcloudSdkException e) {
            logger.error("查询区域失败 ： " + e.getMessage());
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    public BocloudResult sync() {
        try {
            logger.info("sync data : " + this.getRegionId());
            JdSyncModel syncModel = new JdSyncModel();
            // 同步区域
            BocloudResult listRegions = this.listRegions();
            if (listRegions.isFailed()) {
                return new BocloudResult(false, "同步region失败 : " + listRegions.getMessage());
            }
            List<RegionModel> models = (List<RegionModel>)listRegions.getData();
            for (RegionModel regionModel : models) {
                if (this.getRegionId().equals(regionModel.getRegionId())) {
                    syncModel.setZoneModels(regionModel.getZones());
                }
            }

            // 云主机
            JdCloudServerProvider serverProvider = new JdCloudServerProvider(butler);
            BocloudResult serverResult = serverProvider.list();
            if (serverResult.isFailed()) {
                return new BocloudResult(false, "同步云主机失败 : " + serverResult.getMessage());
            }
            List<ServerModel> serverModels = (List<ServerModel>)serverResult.getData();
            syncModel.setServerModels(serverModels);

            // 镜像
            JdCloudImageProvider imageProvider = new JdCloudImageProvider(butler);
            BocloudResult imageResult = imageProvider.list();
            if (imageResult.isFailed()) {
                return new BocloudResult(false, "同步镜像失败 : " + imageResult.getMessage());
            }
            List<ImageModel> imageModels = (List<ImageModel>)imageResult.getData();
            syncModel.setImageModels(imageModels);

            // 规格
            JdCloudFlavorProvider flavorProvider = new JdCloudFlavorProvider(butler);
            BocloudResult flavorResult = flavorProvider.sync();
            if (flavorResult.isFailed()) {
                return new BocloudResult(false, "同步规格失败 ： " + flavorResult.getMessage());
            }
            List<FlavorModel> flavorModels = (List<FlavorModel>)flavorResult.getData();
            syncModel.setFlavorModels(flavorModels);

            // 硬盘
            JdCloudVolumeProvider volumePivider = new JdCloudVolumeProvider(butler);
            BocloudResult volumeResult = volumePivider.list();
            if (volumeResult.isFailed()) {
                return new BocloudResult(false, "同步硬盘失败 ： " + volumeResult.getMessage());
            }
            List<VolumeModel> volumeModels = (List<VolumeModel>)volumeResult.getData();
            syncModel.setVolumeModels(volumeModels);

            // 快照
            JdCloudSnapshotProvider JdCloudSnapshotProvider = new JdCloudSnapshotProvider(butler);
            BocloudResult snapshotResult = JdCloudSnapshotProvider.list();
            if (volumeResult.isFailed()) {
                return new BocloudResult(false, "同步快照失败 ： " + snapshotResult.getMessage());
            }
            List<SnapshotModel> snapshotModels = (List<SnapshotModel>)snapshotResult.getData();
            syncModel.setSnapshotModels(snapshotModels);

            // VPC
            JdCloudVpcProvider vpcPrivider = new JdCloudVpcProvider(butler);
            BocloudResult vpcResult = vpcPrivider.list();
            if (vpcResult.isFailed()) {
                return new BocloudResult(false, "同步vpc失败 ： " + vpcResult.getMessage());
            }
            List<VpcModel> vpcModels = (List<VpcModel>)vpcResult.getData();
            vpcModels.forEach(vpcModel -> vpcModel.setRegionId(this.getRegionId()));
            syncModel.setVpcModels(vpcModels);

            // 路由表
            JdCloudRouterTableProvider routerTableProvider = new JdCloudRouterTableProvider(butler);
            BocloudResult routeTableResult = routerTableProvider.list();
            if (routeTableResult.isFailed()) {
                return new BocloudResult(false, "同步路由表失败 ： " + routeTableResult.getMessage());
            }
            List<RouteTableModel> routeTableModels = (List<RouteTableModel>)routeTableResult.getData();
            routeTableModels.forEach(subnetModel -> subnetModel.setRegionId(this.getRegionId()));
            syncModel.setRouteTableModels(routeTableModels);

            // 安全组
            JdCloudSecurityProvider securityProvider = new JdCloudSecurityProvider(butler);
            BocloudResult securityResult = securityProvider.list();
            if (securityResult.isFailed()) {
                return new BocloudResult(false, "同步安全组失败 ： " + securityResult.getMessage());
            }
            List<SecurityGroupModel> securityGroupModels = (List<SecurityGroupModel>)securityResult.getData();
            syncModel.setSecurityGroupModels(securityGroupModels);
            // 秘钥
            JdCloudKeypairProvider keypairProvider = new JdCloudKeypairProvider(butler);
            BocloudResult keypairResult = keypairProvider.list();
            if (securityResult.isFailed()) {
                return new BocloudResult(false, "同步安全组失败 ： " + keypairResult.getMessage());
            }
            List<KeypairModel> keypairModels = (List<KeypairModel>)keypairResult.getData();
            syncModel.setKeypairModels(keypairModels);

            // 弹性公网IP
            JdCloudElasticIpProvider jdCloudElasticIpProvider = new JdCloudElasticIpProvider(butler);
            BocloudResult elasticIp = jdCloudElasticIpProvider.list();
            if (elasticIp.isFailed()) {
                return new BocloudResult(false, "同步弹性公网IP失败 ： " + elasticIp.getMessage());
            }
            List<ElasticIpModel> elasticIpModels = (List<ElasticIpModel>)elasticIp.getData();
            elasticIpModels.forEach(subnetModel -> subnetModel.setRegionId(this.getRegionId()));
            syncModel.setElasticIpModels(elasticIpModels);

            return new BocloudResult(true, syncModel, "获取同步数据成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "获取同步数据失败");
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

}
