package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.JdImageConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vm.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author : guopeng create at: 2018/11/15 2:48 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudImageProvider extends JdCloudProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudImageProvider.class);
    private BeanConvertor<ImageModel, Image> imageConvertor = new JdImageConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public JdCloudImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<Image> images = new ArrayList<>();

            List<Image> publicImages = selectImage("public");
            System.out.println("publicImages:" + publicImages.size());
            List<Image> privateImages = selectImage("private");
            System.out.println("privateImages:" + privateImages.size());
//			List<Image> sharedImages = selectImage("shared");
//			List<Image> thirdpartyImages = selectImage("thirdparty");

            images.addAll(publicImages);
            images.addAll(privateImages);
//			images.addAll(sharedImages);
//			images.addAll(thirdpartyImages);
            List<ImageModel> convertList = imageConvertor.convertList(images);
            for (ImageModel imageModel : convertList) {
                imageModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list image error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    private List<Image> selectImage(String aPublic) {
        int pageNumber = 1;
        DescribeImagesRequest describeImagesRequest = new DescribeImagesRequest();
        describeImagesRequest.setPageSize(PAGE_SIZE);
        describeImagesRequest.setPageNumber(pageNumber);
        // 所属区域
        describeImagesRequest.setRegionId(this.getRegion());
        // 操作系统发行版(suse, debian, ubuntu, centos, windows-server)
//			describeImagesRequest.setPlatform("CentOS");
        // 获取公共镜像
        describeImagesRequest.setImageSource(aPublic);
        // status ready
        describeImagesRequest.setStatus("ready");
        DescribeImagesResponse describeImagesResponse = this.getVmClient().describeImages(describeImagesRequest);
        DescribeImagesResult result = describeImagesResponse.getResult();
        if (null == result) {
            logger.error("查询镜像列表:totalCount为0");
            return new ArrayList<Image>();
        }
        Integer totalCount = result.getTotalCount().intValue();
        if (totalCount == 0) {
            logger.info("查询" + aPublic + "镜像列表:totalCount为0");
            return new ArrayList<Image>();
        }
        List<Image> images = result.getImages();
        int totalPage = (result.getTotalCount().intValue() % PAGE_SIZE) == 0
                ? result.getTotalCount().intValue() / PAGE_SIZE : (result.getTotalCount().intValue() / PAGE_SIZE + 1);
        if (totalPage > pageNumber) {
            for (int i = pageNumber + 1; i <= totalPage; i++) {
                describeImagesRequest.setPageNumber(i);
                describeImagesRequest.setPageSize(PAGE_SIZE);
                describeImagesResponse = this.getVmClient().describeImages(describeImagesRequest);
                images.addAll(describeImagesResponse.getResult().getImages());
            }
        }
        return images;
    }

    @Override
    public BocloudResult detail(String imageId) {
        try {
            DescribeImageRequest request = new DescribeImageRequest();
            request.setRegionId(this.getRegion());
            request.setImageId(imageId);
            DescribeImageResponse response = this.getVmClient().describeImage(request);
            if (null == response) {
                return new BocloudResult(false, "查询失败");
            }
            DescribeImageResult result = response.getResult();
            if (null == result) {
                return new BocloudResult(false, "查询失败");
            }
            Image image = result.getImage();
            if (null == image) {
                return new BocloudResult(false, "镜像不存在");
            }
            return new BocloudResult(true, imageConvertor.convertModel(image), "查询成功");
        } catch (Exception e) {
            logger.error("query image error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        return null;
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        return null;
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
            CreateImageRequest request = new CreateImageRequest();
            request.setRegionId(this.getRegionId());
            request.setInstanceId(imageModel.getInstanceId());
            request.setName(imageModel.getName());
            request.setDescription(imageModel.getRemark());
            CreateImageResponse createImage = this.getVmClient().createImage(request);
            CreateImageResult result = createImage.getResult();
            String imageId = result.getImageId();
            return new BocloudResult(true, imageId, "查询成功");
        } catch (Exception e) {
            logger.error("query image error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

}
