package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.JdSnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.disk.model.*;

/**
 * @author : guopeng create at: 2018/11/15 4:30 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudSnapshotProvider extends JdCloudProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(JdCloudSnapshotProvider.class);

    private BeanConvertor<SnapshotModel, Snapshot> convertor = new JdSnapshotConvertor();

    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler
     *            云平台认证信息
     */
    public JdCloudSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            DescribeSnapshotsRequest request = new DescribeSnapshotsRequest();
            request.setRegionId(this.getRegionId());
            request.setPageNumber(1);
            request.setPageSize(PAGE_SIZE);
            DescribeSnapshotsResponse response = this.getDiskClient().describeSnapshots(request);
            if (response == null || response.getResult() == null) {
                return new BocloudResult(false, "查询快照列表失败, 接口返回response或response.getResult()为空");
            }
            int totalCount = response.getResult().getTotalCount() == null ? 0 : response.getResult().getTotalCount();
            if (totalCount == 0) {
                logger.info("查询快照列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<SnapshotModel>(), "empty");
            }
            DescribeSnapshotsResult result = response.getResult();
            List<Snapshot> totalVpcs = new ArrayList<>(result.getSnapshots());
            int totalPage = totalCount / PAGE_SIZE + 1;
            if (totalPage == 1 || totalCount == PAGE_SIZE) {
                return new BocloudResult(true, convertor.convertList(totalVpcs), "快照列表查询成功");
            }
            for (int i = 2; i <= totalPage; i++) {
                request.setPageNumber(i);
                request.setPageSize(PAGE_SIZE);
                response = this.getDiskClient().describeSnapshots(request);
                totalVpcs.addAll(response.getResult().getSnapshots());
            }
            return new BocloudResult(true, convertor.convertList(totalVpcs), "快照列表查询成功");
        } catch (Exception e) {
            logger.error("查询快照列表失败 ： " + e);
            return new BocloudResult(false, "快照列表查询失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            DeleteSnapshotRequest request = new DeleteSnapshotRequest();
            request.setRegionId(this.getRegionId());
            request.setSnapshotId(model.getSnapshotUuid());
            DeleteSnapshotResponse deleteSnapshotResponse = this.getDiskClient().deleteSnapshot(request);
            return new BocloudResult(true, deleteSnapshotResponse.getResult(), "删除成功");
        } catch (Exception e) {
            logger.error("delete volume error :", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            DescribeSnapshotRequest request = new DescribeSnapshotRequest();
            request.setSnapshotId(model.getSnapshotUuid());
            request.setRegionId(this.getRegionId());
            DescribeSnapshotResponse response = this.getDiskClient().describeSnapshot(request);
            if (response == null || response.getResult() == null || response.getResult().getSnapshot() == null) {
                return new BocloudResult(false, "查询快照详情失败, 接口返回response或response.getResult()为空");
            }
            DescribeSnapshotResult result = response.getResult();
            SnapshotModel snapshotModel = convertor.convertModel(result.getSnapshot());
            snapshotModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, snapshotModel, "查询快照详情成功");
        } catch (Exception e) {
            logger.error("qeury snapshot error :", e);
            return new BocloudResult(false, "查询快照详情失败");
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            ModifySnapshotAttributeRequest request = new ModifySnapshotAttributeRequest();
            request.setRegionId(this.getRegionId());
            request.setSnapshotId(model.getSnapshotUuid());
            request.setDescription(model.getRemark());
            request.setName(model.getName());
            this.getDiskClient().modifySnapshotAttribute(request);
            BocloudResult detail = detail(model);
            if (detail.isFailed()) {
                return new BocloudResult(false, detail.getMessage());
            }
            return new BocloudResult(true, detail.getData(), "修改成功");
        } catch (Exception e) {
            logger.error("modify snapshot error :", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        return null;
    }

}
