package com.bocloud.cmp.provider.jdconvertor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import com.jdcloud.sdk.service.charge.model.Charge;
import com.jdcloud.sdk.service.disk.model.Disk;
import com.jdcloud.sdk.service.vm.model.*;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author : guopeng create at: 2018/11/15 3:40 PM
 * @description:
 */
public class InstanceConvertor implements BeanConvertor<ServerModel, Instance> {
	@Override
	public List<ServerModel> convertList(List<? extends Instance> instances) {
		List<ServerModel> models = Lists.newArrayList();
		if (null != instances && instances.size() > 0) {
			for (Instance instance : instances) {
				models.add(convertModel(instance));
			}
		}
		return models;
	}

	@Override
	public ServerModel convertModel(Instance instance) {
		ServerModel serverModel = new ServerModel();
		serverModel.setName(instance.getInstanceName());
		serverModel.setStatus(convert(instance.getStatus()));
		serverModel.setRemark(instance.getDescription());
		if (!ListTool.isEmpty(instance.getKeyNames())) {
			serverModel.setKeypairName(JSONObject.toJSONString(instance.getKeyNames()));
		}

		List<VolumeModel> volumeList = new ArrayList<>();
		if (null != instance.getSystemDisk()) {
			InstanceDiskAttachment systemDisk = instance.getSystemDisk();
			Disk disk = systemDisk.getCloudDisk();
			if (null != disk) {
				serverModel.setSystemDiskSize(disk.getDiskSizeGB());
				serverModel.setDisk(disk.getDiskSizeGB());
			} else {
			    LocalDisk localDisk = systemDisk.getLocalDisk();
				serverModel.setSystemDiskSize(localDisk.getDiskSizeGB());
				serverModel.setDisk(localDisk.getDiskSizeGB());
			}
			serverModel.setSystemDiskCategory(systemDisk.getDiskCategory());
		}
		if (!ListTool.isEmpty(instance.getDataDisks())) {
			List<InstanceDiskAttachment> dataDisks = instance.getDataDisks();
			VolumeModel attachmentModel = null;
			for (InstanceDiskAttachment instanceDiskAttachment : dataDisks) {
				attachmentModel = new VolumeModel();
				attachmentModel.setBootVolume(false);
				attachmentModel.setDeleteWithInstance(instanceDiskAttachment.getAutoDelete());
				attachmentModel.setDeviceName(instanceDiskAttachment.getDeviceName());
				attachmentModel.setCategory(instanceDiskAttachment.getDiskCategory());
				Disk cloudDisk = instanceDiskAttachment.getCloudDisk();
				attachmentModel.setName(cloudDisk.getName());
				attachmentModel.setZone(cloudDisk.getAz());
				attachmentModel.setRemark(cloudDisk.getDescription());
				attachmentModel.setVolumeUuid(cloudDisk.getDiskId());
				attachmentModel.setSize(cloudDisk.getDiskSizeGB());
				String status = cloudDisk.getStatus();
				if (null != status) {
					attachmentModel.setStatus(status.toUpperCase());
				}
				Date createTime = attachmentModel.getCreateTime();
				attachmentModel.setCreateTime(createTime);//
				volumeList.add(attachmentModel);
			}
			serverModel.setDataDisks(JSONObject.toJSONString(instance.getDataDisks()));
		}
		serverModel.setDiskDevices(volumeList);
		String privateIpAddress = instance.getPrivateIpAddress();
		ArrayList<String> arrayList = new ArrayList<>();
		arrayList.add(privateIpAddress);
		serverModel.setPrivateIps(JSON.toJSONString(arrayList));
		serverModel.setPublicIps(instance.getElasticIpAddress());
		serverModel.setZone(instance.getAz());
		serverModel.setImageUuid(instance.getImageId());
		serverModel.setInstanceId(instance.getInstanceId());
		serverModel.setInstanceType(instance.getInstanceType());
		serverModel.setFlavorUuid(instance.getInstanceType());
		serverModel.setManagerIp(instance.getPrivateIpAddress());
		serverModel.setVpcId(instance.getVpcId());
		serverModel.setSubnetUuid(instance.getSubnetId());
		InstanceNetworkInterfaceAttachment networkInterfaceAttachment = instance.getPrimaryNetworkInterface();
		if (null != networkInterfaceAttachment) {
			List<SecurityGroupSimple> securityGroups = networkInterfaceAttachment.getNetworkInterface()
					.getSecurityGroups();
			if (!ListTool.isEmpty(securityGroups)) {
				List<String> sGroups = Lists.newArrayList();
				for (SecurityGroupSimple securityGroup : securityGroups) {
					sGroups.add(securityGroup.getGroupId());
				}
				serverModel.setSecurityGroups(sGroups);
			}
		}
		Charge charge = instance.getCharge();
		if (null != charge) {
			serverModel.setInstanceChargeType(charge.getChargeMode());
			serverModel.setInstanceType(instance.getInstanceType());
		}
		return serverModel;
	}

	private String convert(String status) {
		if (status.equalsIgnoreCase("running")) {
			return BocloudStatus.Server.RUNNING.name();
		} else if (status.equalsIgnoreCase("stopped")) {
			return BocloudStatus.Server.STOPPED.name();
		} else {
			return BocloudStatus.Server.EXCEPTION.name();
		}
	}
}
