package com.bocloud.cmp.provider.jdconvertor;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.google.api.client.util.Lists;
import com.jdcloud.sdk.service.charge.model.Charge;
import com.jdcloud.sdk.service.disk.model.Disk;
import com.jdcloud.sdk.service.disk.model.DiskAttachment;

/**
 * @author 作者
 * @version 创建时间：2019年1月17日 上午11:15:47 类说明
 */
public class JdDiskConvertor implements BeanConvertor<VolumeModel, Disk> {

  private static Logger logger = LoggerFactory.getLogger(JdDiskConvertor.class);

  @Override
  public List<VolumeModel> convertList(List<? extends Disk> disks) {
    List<VolumeModel> models = Lists.newArrayList();
    if (null == disks || disks.size() <= 0) {
      return models;
    }
    for (Disk disk : disks) {
      String status = disk.getStatus();
      if (null != status && !"DELETED".equalsIgnoreCase(status)) {
        models.add(convertModel(disk));
      }
    }
    return models;
  }

  @Override
  public VolumeModel convertModel(Disk disk) {
    VolumeModel model = new VolumeModel();
    if (null != disk) {
      model.setZone(disk.getAz());
      model.setRemark(disk.getDescription());
      model.setVolumeUuid(disk.getDiskId());
      model.setSize(disk.getDiskSizeGB());
      model.setType(disk.getDiskType());
      model.setName(disk.getName());
      model.setSourceSnapUuid(disk.getSnapshotId());
      String status = disk.getStatus();
      if (null != status) {
        model.setStatus(disk.getStatus().toUpperCase());
      }
      List<DiskAttachment> attachments = disk.getAttachments();
      if (null != attachments && attachments.size() > 0) {
        DiskAttachment diskAttachment = attachments.get(0);
        model.setInstanceId(diskAttachment.getInstanceId());
        try {
          String attachTime = diskAttachment.getAttachTime();
          if (null != attachTime && !"".equals(attachTime)) {
            model.setAttachedTime(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss").parse(attachTime));
          }
        } catch (ParseException e) {
          logger.error("时间转换ERROR : " + e);
        }
      }
      try {
        String createTime = disk.getCreateTime();
        if (null != createTime && !"".equals(createTime)) {
          model.setCreateTime(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss").parse(createTime));
        }
      } catch (ParseException e) {
        logger.error("时间转换ERROR : " + e);
      }
      Charge charge = disk.getCharge();
      if (null != charge) {
        model.setDiskChargeType(charge.getChargeMode());
        try {
          String chargeExpiredTime = charge.getChargeExpiredTime();
          if (null != chargeExpiredTime && !"".equals(chargeExpiredTime)) {
            model.setDiskExpireTime(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss").parse(chargeExpiredTime));
          }
        } catch (ParseException e) {
          logger.error("时间转换ERROR : " + e);
        }
      }
    }
    return model;
  }

}
