package com.bocloud.cmp.provider.jdconvertor;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.RouteTableModel;
import com.bocloud.cmp.model.RouteTableRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.jdcloud.sdk.service.vpc.model.RouteTable;
import com.jdcloud.sdk.service.vpc.model.RouteTableRule;

/**
 * 路由表转换器
 */
public class RouteTableConvertor implements BeanConvertor<RouteTableModel, RouteTable> {

    private static Logger logger = LoggerFactory.getLogger(RouteTableConvertor.class);

    @Override
    public List<RouteTableModel> convertList(List<? extends RouteTable> objects) {
        List<RouteTableModel> imageModels = new ArrayList<RouteTableModel>();
        for (RouteTable object : objects) {
            imageModels.add(convertModel(object));
        }
        return imageModels;
    }

    @Override
    public RouteTableModel convertModel(RouteTable subnet) {
        RouteTableModel model = null;
        if (null != subnet) {
            model = new RouteTableModel();
            model.setUuid(subnet.getRouteTableId());
            model.setName(subnet.getRouteTableName());
            // 路由表类型，default：默认路由表，custom：自定义路由表
            model.setType(subnet.getRouteTableType());
            model.setRemark(subnet.getDescription());
            model.setVpcUuid(subnet.getVpcId());
            model.setSubnetUuids(subnet.getSubnetIds());
            model.setRuleModels(convertRuleList(subnet.getRouteTableRules()));
            String creationTime = subnet.getCreatedTime();
            if (null != creationTime && !"".equals(creationTime)) {
                model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", creationTime, "Z"));
            }
        }
        return model;
    }

    private List<RouteTableRuleModel> convertRuleList(List<RouteTableRule> objects) {
        List<RouteTableRuleModel> subnetModels = new ArrayList<RouteTableRuleModel>();
        for (RouteTableRule object : objects) {
            subnetModels.add(convertModel(object));
        }
        return subnetModels;
    }

    private RouteTableRuleModel convertModel(RouteTableRule subnet) {
        RouteTableRuleModel model = null;
        if (null != subnet) {
            model = new RouteTableRuleModel();
            model.setRuleUuid(subnet.getRuleId());
            model.setPriority(subnet.getPriority().intValue());
            model.setNextHopId(subnet.getNextHopId());
            model.setNextHopType(subnet.getNextHopType());
            model.setAddressPrefix(subnet.getAddressPrefix());
        }
        return model;
    }

}
