package com.bocloud.cmp.provider.jdconvertor;

import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import com.google.api.client.util.Lists;
import com.jdcloud.sdk.service.vpc.model.NetworkSecurityGroup;
import com.jdcloud.sdk.service.vpc.model.SecurityGroupRule;
import com.jdcloud.sdk.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * @ClassName: SecurityGroupConvertor
 * @Description: 安全组
 * @Author lxy
 * @Date 2021/6/16 18:02
 */
public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, NetworkSecurityGroup> {
    private static Logger logger = LoggerFactory.getLogger(SecurityGroupConvertor.class);
    private BeanConvertor<SecurityGroupRuleModel, SecurityGroupRule> ruleConvertor = new SecurityGroupRuleConvertor();

    @Override
    public List<SecurityGroupModel> convertList(List<? extends NetworkSecurityGroup> networkSecurityGroups) {
        List<SecurityGroupModel> models = Lists.newArrayList();
        for (NetworkSecurityGroup networkSecurityGroup : networkSecurityGroups) {
            models.add(convertModel(networkSecurityGroup));
        }
        return models;
    }

    @Override
    public SecurityGroupModel convertModel(NetworkSecurityGroup networkSecurityGroup) {
        SecurityGroupModel secGroupModel = new SecurityGroupModel();
        secGroupModel.setName(networkSecurityGroup.getNetworkSecurityGroupName());
        secGroupModel.setGroupUuid(networkSecurityGroup.getNetworkSecurityGroupId());
        secGroupModel.setRemark(StringUtils.isEmpty(networkSecurityGroup.getDescription()) ? null : networkSecurityGroup.getDescription());
        try {
            String createTime = networkSecurityGroup.getCreatedTime();
            if (null != createTime && !"".equals(createTime)) {
                secGroupModel.setCreateTime(new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss").parse(createTime));
            }
        } catch (ParseException e) {
            logger.error("时间转换ERROR : " + e);
        }
        secGroupModel.setVpcUuid(networkSecurityGroup.getVpcId());
        List<SecurityGroupRule> securityGroupRules = networkSecurityGroup.getSecurityGroupRules();
        if (!ListTool.isEmpty(securityGroupRules)) {
            List<SecurityGroupRuleModel> securityGroupRuleModels = ruleConvertor.convertList(securityGroupRules);
            for (SecurityGroupRuleModel securityGroupRuleModel : securityGroupRuleModels) {
                securityGroupRuleModel.setGroupUuid(networkSecurityGroup.getNetworkSecurityGroupId());
            }
            secGroupModel.setModels(securityGroupRuleModels);
        }

        return secGroupModel;
    }


}
