package com.bocloud.cmp.provider.jdconvertor;

import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.jdconvertor.enums.DirectionEnum;
import com.bocloud.cmp.provider.jdconvertor.enums.ProtocolEnum;
import com.google.api.client.util.Lists;
import com.jdcloud.sdk.service.vpc.model.SecurityGroupRule;
import com.jdcloud.sdk.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @ClassName: SecurityGroupRuleConvertor
 * @Description: 安全组规则
 * @Author lxy
 * @Date 2021/6/17 10:45
 */
public class SecurityGroupRuleConvertor implements BeanConvertor<SecurityGroupRuleModel, SecurityGroupRule> {
    private static Logger logger = LoggerFactory.getLogger(SecurityGroupRuleConvertor.class);

    @Override
    public List<SecurityGroupRuleModel> convertList(List<? extends SecurityGroupRule> securityGroupRules) {
        List<SecurityGroupRuleModel> models = Lists.newArrayList();
        for (SecurityGroupRule securityGroupRule : securityGroupRules) {
            models.add(convertModel(securityGroupRule));
        }
        return models;
    }

    @Override
    public SecurityGroupRuleModel convertModel(SecurityGroupRule securityGroupRule) {
        SecurityGroupRuleModel securityGroupRuleModel = new SecurityGroupRuleModel();
        securityGroupRuleModel.setRuleUuid(securityGroupRule.getRuleId());
        String direction = "";
        if (StringUtils.isNotBlank(securityGroupRule.getDirection().toString())) {
            switch (DirectionEnum.getByNum(securityGroupRule.getDirection().intValue())) {
                case EGRESS:
                    direction = DirectionEnum.EGRESS.getName();
                    break;
                case INGRESS:
                    direction = DirectionEnum.INGRESS.getName();
                    break;
                default:
                    break;
            }
        }
        securityGroupRuleModel.setDirection(direction);
        String protocol = "";
        if (StringUtils.isNotBlank(securityGroupRule.getProtocol().toString())) {
            switch (ProtocolEnum.getByNum(securityGroupRule.getProtocol().intValue())) {
                case ALL:
                    protocol = ProtocolEnum.ALL.getName();
                    break;
                case TCP:
                    protocol = ProtocolEnum.TCP.getName();
                    break;
                case UDP:
                    protocol = ProtocolEnum.UDP.getName();
                    break;
                case ICMP:
                    protocol = ProtocolEnum.ICMP.getName();
                    break;
                default:
                    break;
            }
        }
        securityGroupRuleModel.setProtocol(protocol);
        securityGroupRuleModel.setRemoteIpPrefix(securityGroupRule.getAddressPrefix());
        securityGroupRuleModel.setEtherType(securityGroupRule.getIpVersion() == null ? null : "IPv" + securityGroupRule.getIpVersion());
        securityGroupRuleModel.setPortMin(securityGroupRule.getFromPort().intValue());
        securityGroupRuleModel.setPortMax(securityGroupRule.getToPort().intValue());
        securityGroupRuleModel.setRemark(securityGroupRule.getDescription());
        //创建时间没做存储
        return securityGroupRuleModel;
    }
}
