package com.bocloud.cmp.provider.jdconvertor;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.jdcloud.sdk.service.vpc.model.Subnet;

/**
 * 子网转换器
 */
public class SubnetConvertor implements BeanConvertor<SubnetModel, Subnet> {

    private static Logger logger = LoggerFactory.getLogger(SubnetConvertor.class);

    @Override
    public List<SubnetModel> convertList(List<? extends Subnet> objects) {
        List<SubnetModel> imageModels = new ArrayList<SubnetModel>();
        for (Subnet object : objects) {
            imageModels.add(convertModel(object));
        }
        return imageModels;
    }

    @Override
    public SubnetModel convertModel(Subnet subnet) {
        SubnetModel model = null;
        if (null != subnet) {
            model = new SubnetModel();
            model.setCidr(subnet.getAddressPrefix());
            model.setSubnetUuid(subnet.getSubnetId());
            model.setName(subnet.getSubnetName());
            model.setVpcUuid(subnet.getVpcId());
            model.setRemark(subnet.getDescription());
            model.setRouterTableUuid(subnet.getRouteTableId());
            String startIp = subnet.getStartIp();
            String endIp = subnet.getEndIp();
            model.setIpPools(JSON.toJSONString(new String[] {startIp + "-" + endIp}));
            String creationTime = subnet.getCreatedTime();
            if (null != creationTime && !"".equals(creationTime)) {
                model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", creationTime, "Z"));
            }
        }
        return model;
    }

}
