package com.bocloud.cmp.provider.jdconvertor;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.jdcloud.sdk.service.vpc.model.Subnet;
import com.jdcloud.sdk.service.vpc.model.Vpc;

/**
 * VPC转换器
 * 
 * @author ymh
 * @时间 2018年11月16日
 */
public class VpcConvertor implements BeanConvertor<VpcModel, Vpc> {

    private static Logger logger = LoggerFactory.getLogger(VpcConvertor.class);

    @Override
    public List<VpcModel> convertList(List<? extends Vpc> objects) {
        List<VpcModel> imageModels = new ArrayList<VpcModel>();
        for (Vpc object : objects) {
            imageModels.add(convertModel(object));
        }
        return imageModels;
    }

    @Override
    public VpcModel convertModel(Vpc vpc) {
        VpcModel model = null;
        if (null != vpc) {
            model = new VpcModel();
            model.setCidr(vpc.getAddressPrefix());
            model.setRemark(vpc.getDescription());
            model.setVpcId(vpc.getVpcId());
            model.setName(vpc.getVpcName());
            String creationTime = vpc.getCreatedTime();
            if (null != creationTime && !"".equals(creationTime)) {
                model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mmZ", creationTime, "Z"));
            }
            List<Subnet> subnets = vpc.getSubnets();
            if (null != subnets && subnets.size() > 0) {
                model.setSubnetModels(new SubnetConvertor().convertList(subnets));
            }
        }
        return model;
    }

}
