package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.JdCloudProvider;
import com.bocloud.cmp.provider.jdconvertor.enums.DirectionEnum;
import com.bocloud.cmp.provider.jdconvertor.enums.ProtocolEnum;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.jdcloud.sdk.service.vpc.model.*;
import com.jdcloud.sdk.utils.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author : guopeng create at: 2018/11/15 4:23 PM
 * @description:
 */
@Provider("bocloud.cmp.jdcloud")
public class JdCloudSecurityRuleProvider extends JdCloudProvider implements SecurityGroupRuleProvider {

    private static Logger logger =
            LoggerFactory.getLogger(JdCloudSecurityRuleProvider.class);

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public JdCloudSecurityRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel group) {
        try {
            AddNetworkSecurityGroupRulesRequest request = new AddNetworkSecurityGroupRulesRequest();
            request.setRegionId(this.getRegion());
            request.setNetworkSecurityGroupId(group.getGroupUuid());
            List<AddSecurityGroupRules> addSecurityGroupRules = new ArrayList<>();
            addSecurityGroupRules.add(createRules(group));
            request.setNetworkSecurityGroupRuleSpecs(addSecurityGroupRules);
            AddNetworkSecurityGroupRulesResponse response = this.getVpcClient().addNetworkSecurityGroupRules(request);
            AddNetworkSecurityGroupRulesResult result = response.getResult();
            if (null == response.getRequestId()) {
                return new BocloudResult(false, "创建失败");
            }
            //可能会存在起始和结束端口不统一，需要同步之后才会统一，因为无法查询定位到最新创建的rule是哪一条
            return new BocloudResult(true, group, "创建成功");
        } catch (Exception e) {
            logger.error("创建失败： " + e);
            return new BocloudResult(false, "创建失败", e.getMessage());
        }
    }

    private AddSecurityGroupRules createRules(SecurityGroupRuleModel group) {
        AddSecurityGroupRules addSecurityGroupRules = new AddSecurityGroupRules();
        addSecurityGroupRules.setDescription(StringUtils.isNotBlank(group.getRemark()) ? group.getRemark() : null);
        addSecurityGroupRules.setAddressPrefix(group.getRemoteIpPrefix());
        Number direction = null;
        if (StringUtils.isNotBlank(group.getDirection().toString())) {
            switch (DirectionEnum.getByName(group.getDirection())) {
                case EGRESS:
                    direction = DirectionEnum.EGRESS.getNum();
                    break;
                case INGRESS:
                    direction = DirectionEnum.INGRESS.getNum();
                    break;
                default:
                    break;
            }
        }
        addSecurityGroupRules.setDirection(direction);
        Number protocol = null;
        if (StringUtils.isNotBlank(group.getProtocol().toString())) {
            switch (ProtocolEnum.getByName(group.getProtocol())) {
                case ALL:
                    protocol = ProtocolEnum.ALL.getNum();
                    break;
                case TCP:
                    protocol = ProtocolEnum.TCP.getNum();
                    break;
                case UDP:
                    protocol = ProtocolEnum.UDP.getNum();
                    break;
                case ICMP:
                    protocol = ProtocolEnum.ICMP.getNum();
                    break;
                default:
                    break;
            }
        }
        addSecurityGroupRules.setProtocol(protocol);
        addSecurityGroupRules.setFromPort(StringUtils.isNotBlank(String.valueOf(group.getPortMin())) ? group.getPortMin() : null);
        addSecurityGroupRules.setToPort(StringUtils.isNotBlank(String.valueOf(group.getPortMax())) ? group.getPortMax() : null);
        return addSecurityGroupRules;
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel model) {
        try {
            RemoveNetworkSecurityGroupRulesRequest request = new RemoveNetworkSecurityGroupRulesRequest();
            request.setRegionId(this.getRegion());
            request.setNetworkSecurityGroupId(model.getGroupUuid());
            List<String> ruleIds = new ArrayList<>();
            ruleIds.add(model.getRuleUuid());
            request.setRuleIds(ruleIds);
            RemoveNetworkSecurityGroupRulesResponse response = this.getVpcClient().removeNetworkSecurityGroupRules(request);
            if (null == response.getRequestId()) {
                return new BocloudResult(false, "安全组规则删除失败");
            }
            return new BocloudResult(true, "安全组规则删除成功");
        } catch (Exception e) {
            logger.error("删除失败： " + e.getMessage());
            return new BocloudResult(false, "安全组规则删除失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        return null;
    }

}
