package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.kubernetes.DaemonSetModel;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import io.fabric8.kubernetes.api.model.Container;
import io.fabric8.kubernetes.api.model.LabelSelector;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PodSpec;
import io.fabric8.kubernetes.api.model.PodTemplateSpec;
import io.fabric8.kubernetes.api.model.Quantity;
import io.fabric8.kubernetes.api.model.ResourceRequirements;
import io.fabric8.kubernetes.api.model.apps.DaemonSet;
import io.fabric8.kubernetes.api.model.apps.DaemonSetSpec;
import io.fabric8.kubernetes.api.model.apps.DaemonSetUpdateStrategy;
import io.fabric8.kubernetes.api.model.apps.RollingUpdateDaemonSet;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class DaemonSetConvertor implements BeanConvertor<DaemonSetModel, DaemonSet> {

	@Override
	public List<DaemonSetModel> convertList(List<? extends DaemonSet> objects) {
		List<DaemonSetModel> nodeModels = new ArrayList<>();
		for (DaemonSet object : objects) {
			DaemonSetModel model = convertModel(object);
			if (null != model) {
				nodeModels.add(model);
			}
		}
		return nodeModels;
	}

	@Override
	public DaemonSetModel convertModel(DaemonSet info) {
		try {
			DaemonSetModel model = null;
			if (null != info) {
				model = new DaemonSetModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				model.setKind(info.getKind());
				model.setApiVersion(info.getApiVersion());
				ObjectMeta metadata = info.getMetadata();
				model.setUid(metadata.getUid());
				model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
				String creationTimestamp = metadata.getCreationTimestamp();
				if (!StringUtils.isEmpty(creationTimestamp)) {
					model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
				}
				if (null != metadata.getLabels()) {
					model.setLabels(JSON.toJSONString(metadata.getLabels()));
					model.setLabelsMap(metadata.getLabels());
				}
				model.setName(metadata.getName());
				model.setNamespace(metadata.getNamespace());
				model.setInstanceId(metadata.getUid());

				DaemonSetSpec spec = info.getSpec();
				LabelSelector selector = spec.getSelector();
				if (null != selector) {
					model.setSelectorMap(selector.getMatchLabels());
					model.setSelector(JSON.toJSONString(selector.getMatchLabels()));
				}

				DaemonSetUpdateStrategy updateStrategy = spec.getUpdateStrategy();
				if (null != updateStrategy) {
					model.setStrategy(updateStrategy.getType());
					RollingUpdateDaemonSet rollingUpdate = updateStrategy.getRollingUpdate();
					if (null != rollingUpdate) {
						model.setRollingUpdateMaxUnavailable(rollingUpdate.getMaxUnavailable().getStrVal());
					}
				}
				PodTemplateSpec template = spec.getTemplate();
				if (null != template) {
					ObjectMeta tempMeta = template.getMetadata();
					model.setTemplateLabels(JSON.toJSONString(tempMeta.getLabels()));
					PodSpec podSpec = template.getSpec();
					if (null != podSpec) {
						List<Container> containers = podSpec.getContainers();
						if (!ListTool.isEmpty(containers)) {
							model.setContainers(JSON.toJSONString(containers));
							Container container = containers.get(0);
							ResourceRequirements resources = container.getResources();
							Map<String, Quantity> requests = resources.getRequests();
							if (null != requests) {
								Quantity cpuQuantity = requests.get("cpu");
								if (null != cpuQuantity) {
									model.setRequestsCpu(cpuQuantity.getAmount());
								}
								Quantity memoryQuantity = requests.get("memory");
								if (null != memoryQuantity) {
									model.setRequestsMemory(memoryQuantity.getAmount());
								}
							}
							Map<String, Quantity> limits = resources.getLimits();
							if (null != limits) {
								Quantity cpuQuantity = limits.get("cpu");
								if (null != cpuQuantity) {
									model.setLimitsCpu(cpuQuantity.getAmount());
								}
								Quantity memoryQuantity = limits.get("memory");
								if (null != memoryQuantity) {
									model.setLimitsMemory(memoryQuantity.getAmount());
								}
							}
						}
						model.setDnsPolicy(podSpec.getDnsPolicy());
						model.setServiceAccount(podSpec.getServiceAccount());
						model.setServiceAccountName(podSpec.getServiceAccountName());
						model.setRestartPolicy(podSpec.getRestartPolicy());
						model.setTerminationGracePeriodSeconds(podSpec.getTerminationGracePeriodSeconds());
					}
				}
			}
			return model;
		} catch (JsonProcessingException e) {
			return null;
		}
	}
}
