package com.bocloud.cmp.provider.convertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.kubernetes.NodeModel;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;

import io.fabric8.kubernetes.api.model.Node;
import io.fabric8.kubernetes.api.model.NodeAddress;
import io.fabric8.kubernetes.api.model.NodeCondition;
import io.fabric8.kubernetes.api.model.NodeSpec;
import io.fabric8.kubernetes.api.model.NodeStatus;
import io.fabric8.kubernetes.api.model.NodeSystemInfo;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Quantity;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class NodeConvertor implements BeanConvertor<NodeModel, Node> {

	@Override
	public List<NodeModel> convertList(List<? extends Node> objects) {
		List<NodeModel> nodeModels = new ArrayList<>();
		for (Node object : objects) {
			nodeModels.add(convertModel(object));
		}
		return nodeModels;
	}

	@Override
	public NodeModel convertModel(Node info) {
		NodeModel model = new NodeModel();
		if (null != info) {
			model.setApiVersion(info.getApiVersion());
			model.setKind(info.getKind());
			ObjectMeta metadata = info.getMetadata();
			model.setUid(metadata.getUid());
			model.setName(metadata.getName());
			model.setInstanceId(metadata.getUid());
			Map<String, String> labels = metadata.getLabels();
			model.setLabels(JSON.toJSONString(labels));
			String roleKey = "kubernetes.io/role";
			if (null != labels && labels.containsKey(roleKey)) {
				model.setRole(labels.get(roleKey));
			}
			String creationTimestamp = metadata.getCreationTimestamp();
			if (null != creationTimestamp && !"".equals(creationTimestamp)) {
				model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
			}
			model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
			NodeSpec spec = info.getSpec();
			model.setPodCidr(spec.getPodCIDR());
			model.setSchedulable(true);
			if (null != spec.getUnschedulable() && spec.getUnschedulable()) {
				model.setSchedulable(false);
			}
			model.setSchedulable(spec.getUnschedulable() == null);
			String nodeIp = "";
			NodeStatus status = info.getStatus();
			List<NodeAddress> addresses = status.getAddresses();
			List<String> adds = new ArrayList<>();
			for (NodeAddress nodeAddress : addresses) {
				nodeIp = nodeAddress.getAddress();
				String type = nodeAddress.getType();
				adds.add(type + ":" + nodeIp);
			}
			model.setNodeIp(nodeIp);
			model.setAddress(JSON.toJSONString(adds));
			NodeSystemInfo nodeInfo = status.getNodeInfo();
			model.setArchitecture(nodeInfo.getArchitecture());
			model.setBootId(nodeInfo.getBootID());
			model.setContainerRuntimeVersion(nodeInfo.getContainerRuntimeVersion());
			model.setKernelVersion(nodeInfo.getKernelVersion());
			model.setKubeProxyVersion(nodeInfo.getKubeProxyVersion());
			model.setKubeletVersion(nodeInfo.getKubeletVersion());
			model.setMachineId(nodeInfo.getMachineID());
			model.setOperatingSystem(nodeInfo.getOperatingSystem());
			model.setOsImage(nodeInfo.getOsImage());
			model.setSystemUuid(nodeInfo.getSystemUUID());
			// 总量
			Map<String, Quantity> capacity = status.getCapacity();
			Quantity cpuQuantity = capacity.get("cpu");
			String cpuAmount = cpuQuantity.getAmount();
			if (cpuAmount.contains("m")) {
				cpuAmount = cpuAmount.substring(0, cpuAmount.indexOf("m"));
				model.setCpu(Integer.valueOf(cpuAmount));
			} else {
				model.setCpu(Integer.valueOf(cpuAmount) * 1000);
			}

			Quantity memoryQuantity = capacity.get("memory");
			String memoryAmount = memoryQuantity.getAmount();
			if (memoryAmount.contains("Ki")) {
				memoryAmount = memoryAmount.substring(0, memoryAmount.indexOf("Ki"));
			}
			if (memoryAmount.contains("Mi")) {
				memoryAmount = memoryAmount.substring(0, memoryAmount.indexOf("Mi"));
				Integer limitsMi = Integer.valueOf(memoryAmount) * 1024;
				memoryAmount = String.valueOf(limitsMi);
			}
			if (memoryAmount.contains("M")) {
				memoryAmount = memoryAmount.substring(0, memoryAmount.indexOf("M"));
				Integer limitsM = Integer.valueOf(memoryAmount) * 1024;
				memoryAmount = String.valueOf(limitsM);
			}
			if (memoryAmount.contains("G")) {
				memoryAmount = memoryAmount.substring(0, memoryAmount.indexOf("G"));
				Integer limitsM = Integer.valueOf(memoryAmount) * 1024 * 1024;
				memoryAmount = String.valueOf(limitsM);
			}
			model.setMemory(BigDecimal.valueOf(Long.valueOf(memoryAmount)));
			List<NodeCondition> conditions = status.getConditions();
			if (!ListTool.isEmpty(conditions)) {
				for (NodeCondition nodeCondition : conditions) {
					String type = nodeCondition.getType();
					String conditionStatus = nodeCondition.getStatus();
					if ("Ready".equalsIgnoreCase(type) && "True".equalsIgnoreCase(conditionStatus)) {
						model.setStatus("Ready");
						break;
					}
				}
			}
			if (StringUtils.isEmpty(model.getStatus())) {
				model.setStatus("NotReady");
			}
		}
		return model;
	}
}
