package com.bocloud.cmp.provider.convertor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeClaimModel;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.fabric8.kubernetes.api.model.KubernetesResource;
import io.fabric8.kubernetes.api.model.LabelSelector;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaim;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaimSpec;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaimStatus;
import io.fabric8.kubernetes.api.model.PersistentVolumeSpec;
import io.fabric8.kubernetes.api.model.Quantity;
import io.fabric8.kubernetes.api.model.ResourceRequirements;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class PersistentVolumeClaimConvertor
		implements BeanConvertor<PersistentVolumeClaimModel, PersistentVolumeClaim> {

	@Override
	public List<PersistentVolumeClaimModel> convertList(List<? extends PersistentVolumeClaim> objects) {
		List<PersistentVolumeClaimModel> nodeModels = new ArrayList<>();
		for (PersistentVolumeClaim object : objects) {
			PersistentVolumeClaimModel convertModel = convertModel(object);
			if (null != convertModel) {
				nodeModels.add(convertModel);
			}
		}
		return nodeModels;
	}

	@Override
	public PersistentVolumeClaimModel convertModel(PersistentVolumeClaim info) {
		PersistentVolumeClaimModel model = null;
		try {
			if (null != info) {
				model = new PersistentVolumeClaimModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				model.setKind(info.getKind());
				model.setApiVersion(info.getApiVersion());

				ObjectMeta metadata = info.getMetadata();
				model.setNamespace(metadata.getNamespace());
				model.setUid(metadata.getUid());
				model.setName(metadata.getName());
				model.setInstanceId(metadata.getUid());
				model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
				Map<String, String> labels = metadata.getLabels();
				if (null != labels) {
					model.setLabels(JSON.toJSONString(labels));
				}
				String creationTimestamp = metadata.getCreationTimestamp();
				if (null != creationTimestamp && !"".equals(creationTimestamp)) {
					model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
				}
				model.setFinalizers(JSON.toJSONString(metadata.getFinalizers()));

				PersistentVolumeClaimSpec spec = info.getSpec();
				List<String> accessModes = spec.getAccessModes();
				if (!ListTool.isEmpty(accessModes)) {
					model.setAccessModes(accessModes.get(0));
				}
				ResourceRequirements resources = spec.getResources();
				Map<String, Quantity> requests = resources.getRequests();
				if (null != requests) {
					Quantity quantity = requests.get("storage");
					if (null != quantity) {
						model.setStorageAmount(quantity.getAmount());
					}
				}
				model.setVolumeMode(spec.getVolumeMode());
				model.setVolumeName(spec.getVolumeName());
				model.setStorageClass(spec.getStorageClassName());
				model.setStorageClassName(spec.getStorageClassName());
				LabelSelector selector = spec.getSelector();
				if (null != selector) {
					model.setSelector(JSON.toJSONString(selector));
					model.setMatchLabels(selector.getMatchLabels());
				}
				PersistentVolumeClaimStatus status = info.getStatus();
				model.setStatus(status.getPhase());
			}
			return model;
		} catch (Exception e) {
			return null;
		}
	}

	public String source(PersistentVolumeSpec spec) {
		Class<?> clazz = spec.getClass();
		Annotation[] annotations = clazz.getAnnotations();
		List<String> values = null;
		for (Annotation annotation : annotations) {
			if (annotation instanceof JsonPropertyOrder) {
				values = Arrays.asList(((JsonPropertyOrder) annotation).value());
				break;
			}
		}
		JSONObject source = new JSONObject();
		for (String value : values) {
			try {
				Method method = clazz.getMethod("get" + value.substring(0, 1).toUpperCase() + value.substring(1));
				if (null == method) {
					continue;
				}
				if (KubernetesResource.class.isAssignableFrom(method.getReturnType())) {
					Object object = method.invoke(spec);
					if (null != object) {
						source.put(value, object);
					}
				}
			} catch (Exception e) {
			}
		}
		return source.toJSONString();
	}
}
