package com.bocloud.cmp.provider.convertor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeModel;
import com.bocloud.common.utils.DateTools;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.ObjectMapper;

import io.fabric8.kubernetes.api.model.KubernetesResource;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PersistentVolume;
import io.fabric8.kubernetes.api.model.PersistentVolumeSpec;
import io.fabric8.kubernetes.api.model.PersistentVolumeStatus;
import io.fabric8.kubernetes.api.model.Quantity;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class PersistentVolumeConvertor implements BeanConvertor<PersistentVolumeModel, PersistentVolume> {

    @Override
    public List<PersistentVolumeModel> convertList(List<? extends PersistentVolume> objects) {
        List<PersistentVolumeModel> nodeModels = new ArrayList<>();
        for (PersistentVolume object : objects) {
            PersistentVolumeModel convertModel = convertModel(object);
            if (null != convertModel) {
                nodeModels.add(convertModel);
            }
        }
        return nodeModels;
    }

    @Override
    public PersistentVolumeModel convertModel(PersistentVolume info) {
        PersistentVolumeModel model = null;
        try {
            if (null != info) {
                model = new PersistentVolumeModel();
                ObjectMapper mapper = new ObjectMapper();
                model.setYamlText(mapper.writeValueAsString(info));
                model.setKind(info.getKind());
                model.setApiVersion(info.getApiVersion());
                ObjectMeta metadata = info.getMetadata();
                model.setUid(metadata.getUid());
                model.setName(metadata.getName());
                model.setInstanceId(metadata.getUid());
                model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
                model.setLabels(JSON.toJSONString(metadata.getLabels()));

                String creationTimestamp = metadata.getCreationTimestamp();
    			if (null != creationTimestamp && !"".equals(creationTimestamp)) {
    				model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
    			}
                model.setFinalizers(JSON.toJSONString(metadata.getFinalizers()));

                PersistentVolumeSpec spec = info.getSpec();

                Map<String, Quantity> capacity = spec.getCapacity();
                model.setCapacity(JSON.toJSONString(capacity));
                Quantity quantity = capacity.get("storage");
                if (null != quantity) {
                    model.setStorageAmount(quantity.getAmount());
                }
                model.setClaimRef(JSON.toJSONString(spec.getClaimRef()));

                model.setVolumeMode(spec.getVolumeMode());
                model.setAccessModes(JSON.toJSONString(spec.getAccessModes()));
                model.setStorageClass(spec.getStorageClassName());
                model.setReclaimPolicy(spec.getPersistentVolumeReclaimPolicy());
                model.setSource(source(spec));
                PersistentVolumeStatus status = info.getStatus();
                model.setStatus(status.getPhase());
                model.setMessage(status.getMessage());
                model.setReason(status.getReason());
            }
            return model;
        } catch (Exception e) {
            return null;
        }
    }

    public String source(PersistentVolumeSpec spec) {
        Class<?> clazz = spec.getClass();
        Annotation[] annotations = clazz.getAnnotations();
        List<String> values = null;
        for (Annotation annotation : annotations) {
            if (annotation instanceof JsonPropertyOrder) {
                values = Arrays.asList(((JsonPropertyOrder) annotation).value());
                break;
            }
        }
        JSONObject source = new JSONObject();
        for (String value : values) {
            try {
                Method method = clazz.getMethod("get" + value.substring(0, 1).toUpperCase() + value.substring(1));
                if (null == method) {
                    continue;
                }
                if (KubernetesResource.class.isAssignableFrom(method.getReturnType())) {
                    Object object = method.invoke(spec);
                    if (null != object) {
                        source.put(value, object);
                    }
                }
            } catch (Exception e) {
            }
        }
        return source.toJSONString();
    }
}
