package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;
import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.kubernetes.ContainerModel;
import com.bocloud.cmp.model.kubernetes.PodModel;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.fabric8.kubernetes.api.model.ContainerState;
import io.fabric8.kubernetes.api.model.ContainerStateRunning;
import io.fabric8.kubernetes.api.model.ContainerStateTerminated;
import io.fabric8.kubernetes.api.model.ContainerStateWaiting;
import io.fabric8.kubernetes.api.model.ContainerStatus;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.OwnerReference;
import io.fabric8.kubernetes.api.model.Pod;
import io.fabric8.kubernetes.api.model.PodSpec;
import io.fabric8.kubernetes.api.model.PodStatus;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class PodConvertor implements BeanConvertor<PodModel, Pod> {

	@Override
	public List<PodModel> convertList(List<? extends Pod> objects) {
		List<PodModel> nodeModels = new ArrayList<>();
		for (Pod object : objects) {
			PodModel convertModel = convertModel(object);
			if (null != convertModel) {
				nodeModels.add(convertModel);
			}
		}
		return nodeModels;
	}

	@Override
	public PodModel convertModel(Pod info) {
		PodModel model = null;
		try {
			if (null != info) {
				model = new PodModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				ObjectMeta metadata = info.getMetadata();
				model.setUid(metadata.getUid());
				List<OwnerReference> ownerReferences = metadata.getOwnerReferences();
				if (!ListTool.isEmpty(ownerReferences)) {
					OwnerReference ownerReference = ownerReferences.get(0);
					model.setParentResourceUid(ownerReference.getUid());
					model.setParentResourceType(ownerReference.getKind());
					model.setParentResourceName(ownerReference.getName());
					model.setParentResourceVersion(ownerReference.getApiVersion());
				}
				String creationTimestamp = metadata.getCreationTimestamp();
				if (null != creationTimestamp && !"".equals(creationTimestamp)) {
					model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
				}
				model.setName(metadata.getName());
				model.setInstanceId(metadata.getUid());
				model.setKind(info.getKind());
				model.setApiVersion(info.getApiVersion());
				model.setLabelsMap(metadata.getLabels());
				model.setLabels(JSON.toJSONString(metadata.getLabels()));
				model.setGenerateName(metadata.getGenerateName());
				model.setNamespace(metadata.getNamespace());
				PodSpec spec = info.getSpec();
				model.setDnsPolicy(spec.getDnsPolicy());
				model.setHostNetwork(spec.getHostNetwork());
				model.setNodeSelector(JSON.toJSONString(spec.getNodeSelector()));
				model.setNodeName(spec.getNodeName());
				model.setRestartPolicy(spec.getRestartPolicy());
				model.setSecurityContext(JSON.toJSONString(spec.getSecurityContext()));
				model.setServiceAccount(spec.getServiceAccount());
				model.setServiceAccountName(spec.getServiceAccountName());
				model.setTerminationGracePeriodSeconds(spec.getTerminationGracePeriodSeconds());
				model.setVolumes(JSON.toJSONString(spec.getVolumes()));
				PodStatus status = info.getStatus();
				model.setContainerStatuses(JSON.toJSONString(status.getContainerStatuses()));
				model.setStatus(status.getPhase() == null ? null : status.getPhase().toUpperCase());
				model.setPhase(status.getPhase() == null ? null : status.getPhase().toUpperCase());
				model.setHostIp(status.getHostIP());
				model.setPodIp(status.getPodIP());
				String imageName = null;
				List<ContainerStatus> containerStatuses = status.getContainerStatuses();
				if (!ListTool.isEmpty(containerStatuses)) {
					List<ContainerModel> containerModels = new ArrayList<>();
					ContainerModel containerModel = null;
					for (ContainerStatus containerStatus : containerStatuses) {
						containerModel = new ContainerModel();
						containerModel.setImageName(containerStatus.getImage());
						imageName = containerStatus.getImage();
						containerModel.setImageUuid(containerStatus.getImageID());
						containerModel.setContainerId(containerStatus.getContainerID());
						containerModel.setName(containerStatus.getName());
						containerModel.setUid(containerStatus.getContainerID());
						containerModel.setReady(containerStatus.getReady());
						containerModel.setRestartCount(containerStatus.getRestartCount());
						ContainerState state = containerStatus.getState();
						ContainerStateRunning running = state.getRunning();
						if (null != running) {
							containerModel.setStatus("RUNNING");
						}
						ContainerStateTerminated terminated = state.getTerminated();
						if (null != terminated) {
							containerModel.setStatus(terminated.getReason());
						}
						ContainerStateWaiting waiting = state.getWaiting();
						if (null != waiting) {
							containerModel.setStatus(waiting.getReason());
						}
						containerModels.add(containerModel);
					}
					model.setKubeContainers(containerModels);
				}
				model.setImageName(imageName);
			}
			return model;
		} catch (JsonProcessingException e) {
			return null;
		}
	}
}
