package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import com.bocloud.cmp.model.kubernetes.ReplicaModel;
import com.bocloud.common.utils.ListTool;
import com.fasterxml.jackson.databind.ObjectMapper;

import io.fabric8.kubernetes.api.model.Container;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.OwnerReference;
import io.fabric8.kubernetes.api.model.PodSpec;
import io.fabric8.kubernetes.api.model.PodTemplateSpec;
import io.fabric8.kubernetes.api.model.Quantity;
import io.fabric8.kubernetes.api.model.ResourceRequirements;
import io.fabric8.kubernetes.api.model.apps.ReplicaSet;
import io.fabric8.kubernetes.api.model.apps.ReplicaSetSpec;

/**
 * 副本集转换类
 * 
 * @时间 2019年8月21日
 */
public class ReplicaConvertor implements BeanConvertor<ReplicaModel, ReplicaSet> {

	@Override
	public List<ReplicaModel> convertList(List<? extends ReplicaSet> objects) {
		List<ReplicaModel> nodeModels = new ArrayList<>();
		for (ReplicaSet object : objects) {
			ReplicaModel model = convertModel(object);
			if (null != model) {
				nodeModels.add(model);
			}
		}
		return nodeModels;
	}

	@Override
	public ReplicaModel convertModel(ReplicaSet info) {
		try {
			ReplicaModel model = null;
			if (null != info) {
				model = new ReplicaModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				model.setKind(info.getKind());
				model.setApiVersion(info.getApiVersion());
				ObjectMeta metadata = info.getMetadata();
				model.setName(metadata.getName());
				model.setNamespace(metadata.getNamespace());
				model.setLabelsMap(metadata.getLabels());
				model.setUid(metadata.getUid());
				List<OwnerReference> ownerReferences = metadata.getOwnerReferences();
				if (!ListTool.isEmpty(ownerReferences)) {
					OwnerReference ownerReference = ownerReferences.get(0);
					model.setParentResourceUid(ownerReference.getUid());
					model.setParentResourceName(ownerReference.getName());
					model.setParentResourceType(ownerReference.getKind());
					model.setParentResourceVersion(ownerReference.getApiVersion());
				}
				ReplicaSetSpec spec = info.getSpec();
				if (null == spec) {
					return model;
				}
				PodTemplateSpec template = spec.getTemplate();
				if (null == template) {
					return model;
				}
				PodSpec podSpec = template.getSpec();
				if (null == podSpec) {
					return model;
				}
				List<Container> containers = podSpec.getContainers();
				if (!ListTool.isEmpty(containers)) {
					Container container = containers.get(0);
					ResourceRequirements resources = container.getResources();
					Map<String, Quantity> requests = resources.getRequests();
					if (null != requests) {
						Quantity cpuQuantity = requests.get("cpu");
						if (null != cpuQuantity) {
							model.setRequestsCpu(cpuQuantity.getAmount());
						}
						Quantity memoryQuantity = requests.get("memory");
						if (null != memoryQuantity) {
							model.setRequestsMemory(memoryQuantity.getAmount());
						}
					}
					Map<String, Quantity> limits = resources.getLimits();
					if (null != limits) {
						Quantity cpuQuantity = limits.get("cpu");
						if (null != cpuQuantity) {
							model.setLimitsCpu(cpuQuantity.getAmount());
						}
						Quantity memoryQuantity = limits.get("memory");
						if (null != memoryQuantity) {
							model.setLimitsMemory(memoryQuantity.getAmount());
						}
					}
				}
			}
			return model;
		} catch (Exception e) {
			return null;
		}
	}
}
