package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.kubernetes.SecretModel;
import com.bocloud.common.utils.DateTools;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Secret;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class SecretConvertor implements BeanConvertor<SecretModel, Secret> {

	@Override
	public List<SecretModel> convertList(List<? extends Secret> objects) {
		List<SecretModel> nodeModels = new ArrayList<>();
		for (Secret object : objects) {
			SecretModel convertModel = convertModel(object);
			if (null != convertModel) {
				nodeModels.add(convertModel);
			}
		}
		return nodeModels;
	}

	@Override
	public SecretModel convertModel(Secret info) {
		SecretModel model = null;
		try {
			if (null != info) {
				model = new SecretModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				model.setApiVersion(info.getApiVersion());
				model.setKind(info.getKind());
				ObjectMeta metadata = info.getMetadata();
				model.setUid(metadata.getUid());
				model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
				String creationTimestamp = metadata.getCreationTimestamp();
				if (null != creationTimestamp && !"".equals(creationTimestamp)) {
					model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
				}
				if (null != metadata.getLabels()) {
					model.setLabels(JSON.toJSONString(metadata.getLabels()));
					model.setLablesMap(metadata.getLabels());
				}
				model.setName(metadata.getName());
				model.setNamespace(metadata.getNamespace());
				model.setInstanceId(metadata.getUid());
				String type = info.getType();
				model.setType(type);
				Map<String, String> data = info.getData();
				if (null != data) {
					if ("kubernetes.io/dockerconfigjson".equalsIgnoreCase(type)) {
						model.setData(data.get(".dockerconfigjson").toString());
					}
				}
			}
			return model;
		} catch (JsonProcessingException e) {
			return null;
		}
	}
}
