package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.kubernetes.ServiceModel;
import com.bocloud.common.utils.DateTools;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.Service;
import io.fabric8.kubernetes.api.model.ServicePort;
import io.fabric8.kubernetes.api.model.ServiceSpec;

/**
 * 转换类
 * 
 * @时间 2019年8月21日
 */
public class ServiceConvertor implements BeanConvertor<ServiceModel, Service> {

	@Override
	public List<ServiceModel> convertList(List<? extends Service> objects) {
		List<ServiceModel> nodeModels = new ArrayList<>();
		for (Service object : objects) {
			ServiceModel convertModel = convertModel(object);
			if (null != convertModel) {
				nodeModels.add(convertModel);
			}
		}
		return nodeModels;
	}

	@Override
	public ServiceModel convertModel(Service info) {
		ServiceModel model = null;
		try {
			if (null != info) {
				model = new ServiceModel();
				ObjectMapper mapper = new ObjectMapper();
				model.setYamlText(mapper.writeValueAsString(info));
				model.setApiVersion(info.getApiVersion());
				model.setKind(info.getKind());
				ObjectMeta metadata = info.getMetadata();
				model.setUid(metadata.getUid());
				model.setAnnotations(JSON.toJSONString(metadata.getAnnotations()));
				String creationTimestamp = metadata.getCreationTimestamp();
				if (null != creationTimestamp && !"".equals(creationTimestamp)) {
					model.setCreationTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", creationTimestamp, "Z"));
				}
				if (null != metadata.getLabels()) {
					model.setLabels(JSON.toJSONString(metadata.getLabels()));
				}
				model.setName(metadata.getName());
				model.setNamespace(metadata.getNamespace());
				model.setInstanceId(metadata.getUid());
				ServiceSpec spec = info.getSpec();
				model.setClusterIp(spec.getClusterIP());
				List<String> internalEndpoints = new ArrayList<>();
				List<ServicePort> ports = spec.getPorts();
				for (ServicePort servicePort : ports) {
					Integer port = servicePort.getPort();
					String protocol = servicePort.getProtocol();
					internalEndpoints.add(model.getName() + "." + model.getNamespace() + ":" + port + " " + protocol);
					if (null != servicePort.getNodePort()) {
						internalEndpoints.add(model.getName() + "." + model.getNamespace() + ":"
								+ servicePort.getNodePort() + " " + protocol);
					}
				}
				model.setPorts(JSON.toJSONString(ports));
				model.setInternalEndpoint(JSON.toJSONString(internalEndpoints));
				if (null != spec.getSelector()) {
					model.setSelectorMap(spec.getSelector());
					model.setSelector(JSON.toJSONString(spec.getSelector()));
				}
				model.setSessionAffinity(spec.getSessionAffinity());
				model.setType(spec.getType());
			}
			return model;
		} catch (JsonProcessingException e) {
			return null;
		}
	}
}
