package com.bocloud.cmp.provider.kubernetes;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.DeploymentModel;
import com.bocloud.cmp.model.kubernetes.PodModel;
import com.bocloud.cmp.model.kubernetes.ReplicaModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.DeploymentConvertor;
import com.bocloud.cmp.provider.convertor.PodConvertor;
import com.bocloud.cmp.provider.convertor.ReplicaConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import io.fabric8.kubernetes.api.model.Affinity;
import io.fabric8.kubernetes.api.model.Container;
import io.fabric8.kubernetes.api.model.ContainerPort;
import io.fabric8.kubernetes.api.model.EmptyDirVolumeSource;
import io.fabric8.kubernetes.api.model.IntOrString;
import io.fabric8.kubernetes.api.model.LabelSelector;
import io.fabric8.kubernetes.api.model.LabelSelectorRequirement;
import io.fabric8.kubernetes.api.model.LocalObjectReference;
import io.fabric8.kubernetes.api.model.NodeAffinity;
import io.fabric8.kubernetes.api.model.NodeSelector;
import io.fabric8.kubernetes.api.model.NodeSelectorRequirement;
import io.fabric8.kubernetes.api.model.NodeSelectorTerm;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaimVolumeSource;
import io.fabric8.kubernetes.api.model.Pod;
import io.fabric8.kubernetes.api.model.PodAffinity;
import io.fabric8.kubernetes.api.model.PodAffinityTerm;
import io.fabric8.kubernetes.api.model.PodAntiAffinity;
import io.fabric8.kubernetes.api.model.PodList;
import io.fabric8.kubernetes.api.model.PodSpec;
import io.fabric8.kubernetes.api.model.PodTemplateSpec;
import io.fabric8.kubernetes.api.model.PreferredSchedulingTerm;
import io.fabric8.kubernetes.api.model.Quantity;
import io.fabric8.kubernetes.api.model.ResourceRequirements;
import io.fabric8.kubernetes.api.model.Volume;
import io.fabric8.kubernetes.api.model.VolumeMount;
import io.fabric8.kubernetes.api.model.apps.Deployment;
import io.fabric8.kubernetes.api.model.apps.DeploymentList;
import io.fabric8.kubernetes.api.model.apps.DeploymentSpec;
import io.fabric8.kubernetes.api.model.apps.DeploymentStrategy;
import io.fabric8.kubernetes.api.model.apps.ReplicaSet;
import io.fabric8.kubernetes.api.model.apps.ReplicaSetList;
import io.fabric8.kubernetes.api.model.apps.RollingUpdateDeployment;

/**
 * k8s deployment
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeDepolymentProvider extends KubernetesProvider implements DeploymentProvider {

	private BeanConvertor<DeploymentModel, Deployment> deploymentConvertor = new DeploymentConvertor();
	private BeanConvertor<ReplicaModel, ReplicaSet> replicaConvertor = new ReplicaConvertor(); // 副本集
	private BeanConvertor<PodModel, Pod> podConvertor = new PodConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubeDepolymentProvider.class);

	public KubeDepolymentProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			DeploymentList deploymentList = this.getClient().apps().deployments().list();
			List<DeploymentModel> deploymentModels = deploymentConvertor.convertList(deploymentList.getItems());
			return new BocloudResult(true, deploymentModels, "查询成功！");
		} catch (Exception e) {
			logger.error("查询部署失败：" + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(DeploymentModel model) {
		try {
			String yamlText = CommonUtils.yamlAnalysis(model.getYamlText());
			if (StringUtils.isEmpty(yamlText)) {
				return createDeployment(model);
			}
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Deployment deployment = this.getClient().apps().deployments().load(bais).get();
			ObjectMeta metadata = deployment.getMetadata();
			if (null != metadata.getResourceVersion()) {
				metadata.setResourceVersion(null);
			}
			if (null != metadata.getUid()) {
				metadata.setUid(null);
			}
			if (null != metadata.getSelfLink()) {
				metadata.setSelfLink(null);
			}
			deployment.setMetadata(metadata);
			Deployment create = this.getClient().apps().deployments().create(deployment);
			Thread.sleep(5000);
			DeploymentModel deploymentModel = deploymentConvertor.convertModel(create);
			List<PodModel> listPods = listPods(deploymentModel);
			if (!ListTool.isEmpty(listPods)) {
				for (PodModel podModel : listPods) {
					podModel.setDeploymentApiVersion(deploymentModel.getApiVersion());
					podModel.setDeploymentName(deploymentModel.getName());
					podModel.setDeploymentUid(deploymentModel.getUid());
				}
				deploymentModel.setPodModels(listPods);
				deploymentModel.setPodCount(listPods.size());
			}
			return new BocloudResult(true, deploymentModel, "Deployment创建成功！");
		} catch (Exception e) {
			logger.error("新增失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	private BocloudResult createDeployment(DeploymentModel model) {
		try {
			String namespace = model.getNamespace();
			if (StringUtils.isEmpty(namespace)) {
				namespace = "default";
			}
			String name = model.getName();
			Map<String, String> labelsMap = model.getLabelsMap();
			Map<String, String> selectorMap = model.getSelectorMap();
			Deployment deployment = new Deployment();
			ObjectMeta metadata = new ObjectMeta();
			metadata.setNamespace(namespace);
			metadata.setName(name);
			metadata.setLabels(labelsMap);
			deployment.setMetadata(metadata);
			DeploymentSpec spec = new DeploymentSpec();
			/**
			 * 副本集
			 */
			spec.setReplicas(model.getReplica() == null ? 1 : model.getReplica());
			LabelSelector selector = new LabelSelector();
			selector.setMatchLabels(selectorMap);
			spec.setSelector(selector);
			/**
			 * 策略
			 */
			String strategyType = model.getStrategyType();
			if (StringUtils.isEmpty(strategyType)) {
				strategyType = "RollingUpdate";
			}
			DeploymentStrategy strategy = new DeploymentStrategy();
			if ("RollingUpdate".equalsIgnoreCase(strategyType)) {
				RollingUpdateDeployment rollingUpdate = new RollingUpdateDeployment();
				rollingUpdate.setMaxSurge(new IntOrString(25));
				rollingUpdate.setMaxUnavailable(new IntOrString(25));
				strategy.setRollingUpdate(rollingUpdate);
			}
			strategy.setType(strategyType);
			spec.setStrategy(strategy);
			deployment.setSpec(spec);

			/**
			 * SPEC
			 */
			PodSpec podTemplateSpecSpec = new PodSpec();
			/**
			 * 设置存储卷
			 */
			Map<String, String> specAnnotationsMap = new HashMap<>();
			String storageVolumes = model.getVolumes();
			// 设置volume
			List<VolumeMount> volumeMounts = null;
			if (!StringUtils.isEmpty(storageVolumes)) {
				List<JSONObject> stroageVolumes = JSONObject.parseArray(storageVolumes, JSONObject.class);
				if (!ListTool.isEmpty(stroageVolumes)) {
					List<Volume> volumes = new ArrayList<>();
					volumeMounts = new ArrayList<>();
					Volume volume = null;
					VolumeMount volumeMount = null;
					for (JSONObject jsonObject : stroageVolumes) {
						String volumeType = jsonObject.getString("volumeType");
						if ("log".equalsIgnoreCase(volumeType)) {
							specAnnotationsMap.put("kubernetes.customized/logging_emptydir_name",
									jsonObject.getString("name"));
						}
						volume = new Volume();
						volume.setName(jsonObject.getString("name"));
						if ("pvc".equalsIgnoreCase(volumeType)) {
							PersistentVolumeClaimVolumeSource persistentVolumeClaim = new PersistentVolumeClaimVolumeSource();
							persistentVolumeClaim.setClaimName(jsonObject.getString("claimName"));
							volume.setPersistentVolumeClaim(persistentVolumeClaim);
						} else {
							volume.setEmptyDir(new EmptyDirVolumeSource());
						}
						volumes.add(volume);
						volumeMount = new VolumeMount();
						volumeMount.setName(jsonObject.getString("name"));
						volumeMount.setMountPath(jsonObject.getString("mountPath"));
						volumeMounts.add(volumeMount);
					}
					podTemplateSpecSpec.setVolumes(volumes);
				}
			}
			/**
			 * template
			 */
			PodTemplateSpec template = new PodTemplateSpec();
			ObjectMeta podTemplateSpecMetadata = new ObjectMeta();
			podTemplateSpecMetadata.setLabels(labelsMap);

			if (null != specAnnotationsMap && specAnnotationsMap.keySet().size() > 0) {
				podTemplateSpecMetadata.setAnnotations(specAnnotationsMap);
			}
			template.setMetadata(podTemplateSpecMetadata);

			List<Container> containers = new ArrayList<>();
			Container container = new Container();
			if (!ListTool.isEmpty(volumeMounts)) {
				container.setVolumeMounts(volumeMounts);
			}
			String containerName = "k8s-" + model.getName() + "-" + model.getNamespace() + "-";
			if (containerName.length() < 63) {
				containerName = containerName + strRandom(63 - containerName.length()).toLowerCase();
			}

			container.setName(containerName);
			container.setImage(model.getImage());
			/**
			 * 端口号
			 */
			String bocoPlatformPorts = model.getBocoPlatformPorts();
			if (!StringUtils.isEmpty(bocoPlatformPorts)) {
				List<ContainerPort> ports = new ArrayList<>();
				ContainerPort containerPort = null;
				List<JSONObject> portObjects = JSONObject.parseArray(bocoPlatformPorts, JSONObject.class);
				for (JSONObject jsonObject : portObjects) {
					containerPort = new ContainerPort();
					containerPort
							.setName(jsonObject.getString("protocol").toLowerCase() + jsonObject.getString("port"));
					containerPort.setProtocol(jsonObject.getString("protocol").toUpperCase());
					containerPort.setContainerPort(Integer.valueOf(jsonObject.getString("port")));
					ports.add(containerPort);
				}
				container.setPorts(ports);
			}
			/**
			 * resources: {minCpu: "200m", maxCpu: "200m", minMem: "256M", maxMem: "256M"}
			 * 
			 */
			String resources = model.getResources();
			if (!StringUtils.isEmpty(resources)) {
				JSONObject resourceObject = JSONObject.parseObject(resources);
				ResourceRequirements resourceRequirements = new ResourceRequirements();
				Map<String, Quantity> limits = new HashMap<>();
				limits.put("cpu", new Quantity(resourceObject.getString("maxCpu")));
				limits.put("memory", new Quantity(resourceObject.getString("maxMem")));
				String maxGpu = resourceObject.getString("maxGpu");
				if (!StringUtils.isEmpty(maxGpu)) {
					limits.put("nvidia.com/gpu", new Quantity(maxGpu));
				}
				resourceRequirements.setLimits(limits);
				Map<String, Quantity> requests = new HashMap<>();
				requests.put("cpu", new Quantity(resourceObject.getString("minCpu")));
				requests.put("memory", new Quantity(resourceObject.getString("minMem")));
				resourceRequirements.setRequests(requests);
				container.setResources(resourceRequirements);
			}

			String imagePullPolicy = model.getImagePullPolicy();
			if (StringUtils.isEmpty(imagePullPolicy)) {
				imagePullPolicy = "Always";
			}
			container.setImagePullPolicy(imagePullPolicy);
			containers.add(container);
			podTemplateSpecSpec.setContainers(containers);
			// secret
			String secretName = model.getSecretName();
			if (!StringUtils.isEmpty(secretName)) {
				List<LocalObjectReference> imagePullSecrets = new ArrayList<>();
				LocalObjectReference imagePullSecret = new LocalObjectReference(secretName);
				imagePullSecrets.add(imagePullSecret);
				podTemplateSpecSpec.setImagePullSecrets(imagePullSecrets);
			}

			// 设置亲和性
			String affinitys = model.getAffinitys();
			if (!StringUtils.isEmpty(affinitys)) {
				podTemplateSpecSpec = this.createAffinitys(namespace, name, affinitys, podTemplateSpecSpec);
				if (null == podTemplateSpecSpec) {
					return new BocloudResult(false, "调度配置异常！");
				}
			}
			template.setSpec(podTemplateSpecSpec);
			spec.setTemplate(template);

			Deployment create = this.getClient().apps().deployments().create(deployment);
			Thread.sleep(1000);
			DeploymentModel deploymentModel = deploymentConvertor.convertModel(create);
			List<PodModel> pods = listPods(deploymentModel);
			List<PodModel> listPods = list(pods);
			if (!ListTool.isEmpty(listPods)) {
				for (PodModel podModel : listPods) {
					podModel.setDeploymentApiVersion(deploymentModel.getApiVersion());
					podModel.setDeploymentName(deploymentModel.getName());
					podModel.setDeploymentUid(deploymentModel.getUid());
				}
				deploymentModel.setPodModels(listPods);
				deploymentModel.setPodCount(listPods.size());
			}
			return new BocloudResult(true, deploymentModel, "服务创建成功！");
		} catch (Exception e) {
			logger.error("新增失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	public List<ReplicaModel> listReplica(String depUuid, String namespace) {
		List<ReplicaModel> replicaModelList = new ArrayList<>();
		try {
			ReplicaSetList replicaSetList = this.getClient().apps().replicaSets().inNamespace(namespace).list();
			List<ReplicaModel> convertList = replicaConvertor.convertList(replicaSetList.getItems());
			if (!ListTool.isEmpty(convertList)) {
				for (ReplicaModel replicaModel : convertList) {
					if (depUuid.equalsIgnoreCase(replicaModel.getParentResourceUid())) {
						replicaModelList.add(replicaModel);
					}
				}
			}
			return replicaModelList;
		} catch (Exception e) {
			logger.error("查询副本集ERROR:" + e.getMessage());
			return replicaModelList;
		}
	}

	private List<PodModel> listPods(DeploymentModel deploymentModel) {
		List<PodModel> arrayList = new ArrayList<>();
		try {
			List<ReplicaModel> replicaModels = listReplica(deploymentModel.getUid(), deploymentModel.getNamespace());
			if (ListTool.isEmpty(replicaModels)) {
				return arrayList;
			}
			deploymentModel.setReplicaModels(replicaModels);
			for (ReplicaModel replicaModel : replicaModels) {
				PodList podList = this.getClient().pods().inNamespace(replicaModel.getNamespace()).list();
				List<PodModel> podConvertors = podConvertor.convertList(podList.getItems());
				for (PodModel podModel : podConvertors) {
					if (replicaModel.getKind().equalsIgnoreCase(podModel.getParentResourceType())
							&& replicaModel.getUid().equalsIgnoreCase(podModel.getParentResourceUid())) {
						podModel.setDeploymentName(deploymentModel.getName());
						podModel.setDeploymentUid(deploymentModel.getUid());
						podModel.setDeploymentApiVersion(deploymentModel.getApiVersion());
						podModel.setReplicaName(replicaModel.getName());
						Integer podCount = deploymentModel.getPodCount();
						if (null == podCount || 0 == podCount) {
							deploymentModel.setPodCount(1);
						} else {
							deploymentModel.setPodCount(deploymentModel.getPodCount() + 1);
						}
						arrayList.add(podModel);
					}
				}
			}
			return arrayList;
		} catch (Exception e) {
			logger.error("查询POD列表失败：" + e.getMessage());
			return arrayList;
		}
	}

	public List<PodModel> list(List<PodModel> listPods) {
		List<PodModel> convertList = null;
		try {
			Long startTime = System.currentTimeMillis();
			boolean flag = false;
			List<Pod> items = null;
			while (true) {
				Thread.sleep(2000);
				items = new ArrayList<>();
				for (PodModel podModel : listPods) {
					items.add(this.getClient().pods().inNamespace(podModel.getNamespace()).withName(podModel.getName())
							.get());
				}
				if (!ListTool.isEmpty(items)) {
					boolean podStatus = true;
					for (Pod pod : items) {
						if (!"running".equalsIgnoreCase(pod.getStatus().getPhase())) {
							podStatus = false;
						}
					}
					flag = podStatus;
				}
				// 等待三分钟
				if (flag || System.currentTimeMillis() - startTime > 5 * 60 * 1000) {
					break;
				}
			}
			items = new ArrayList<>();
			for (PodModel podModel : listPods) {
				items.add(this.getClient().pods().inNamespace(podModel.getNamespace()).withName(podModel.getName())
						.get());
			}
			convertList = podConvertor.convertList(items);
		} catch (Exception e) {
			logger.error("query pod list fail : " + e.getMessage());
		}
		return convertList;
	}

	/**
	 * 设置亲和性
	 * 
	 * @param affinitys
	 * @param podTemplateSpecSpec
	 */
	private PodSpec createAffinitys(String namespace, String applicationName, String affinitys,
			PodSpec podTemplateSpecSpec) {
		try {
			// affinitys:[ {affinityType:'nodeAffinity' , affinityRequired:
			// true, operator:'In'
			// ,matchExpressions:{node1:'node1',nodetype:'loadbalance' } } ]
			List<JSONObject> affinitysObjects = JSONObject.parseArray(affinitys, JSONObject.class);
			// 亲和性
			Affinity affinity = new Affinity();
			for (JSONObject affinitysObject : affinitysObjects) {
				String affinityType = affinitysObject.getString("affinityType");
				String operator = affinitysObject.getString("operator");
				Boolean affinityRequired = affinitysObject.getBoolean("affinityRequired");
				// 节点亲和性
				if ("nodeAffinity".equalsIgnoreCase(affinityType)) {
					JSONObject matchExpressionsMaps = affinitysObject.getJSONObject("matchExpressions");
					NodeAffinity nodeAffinity = new NodeAffinity();
					// matchExpressions
					List<NodeSelectorRequirement> matchExpressions = new ArrayList<>();
					NodeSelectorRequirement matchExpression = null;
					Set<Entry<String, Object>> entrySet = matchExpressionsMaps.entrySet();
					for (Entry<String, Object> entry : entrySet) {
						matchExpression = new NodeSelectorRequirement();
						matchExpression.setKey(entry.getKey());
						matchExpression.setOperator(operator);
						if ("In".equalsIgnoreCase(operator) || "NotIn".equalsIgnoreCase(operator)) {
							matchExpression.setValues(Arrays.asList(new String[] { entry.getValue().toString() }));
						}
						if ("Lt".equalsIgnoreCase(operator) || "Gt".equalsIgnoreCase(operator)) {
							String labelValue = affinitysObject.getString("labelValue");
							matchExpression.setValues(Arrays.asList(new String[] { labelValue }));
						}
						matchExpressions.add(matchExpression);
					}
					if (affinityRequired) {
						// 调度强制约束 硬亲和性；节点必须满足设定条件，Pod才能被调度到这个节点。
						NodeSelector requiredDuringSchedulingIgnoredDuringExecution = new NodeSelector();
						List<NodeSelectorTerm> nodeSelectorTerms = new ArrayList<>();
						nodeSelectorTerms.add(new NodeSelectorTerm(matchExpressions, new ArrayList<>()));
						requiredDuringSchedulingIgnoredDuringExecution.setNodeSelectorTerms(nodeSelectorTerms);
						nodeAffinity.setRequiredDuringSchedulingIgnoredDuringExecution(
								requiredDuringSchedulingIgnoredDuringExecution);
					} else {
						// 调度非强制约束
						// 软亲和性；不管节点上能否满足设定条件，Pod都可以被调度，只是Pod优先被调度到符合条件多的节点上。
						Integer weight = affinitysObject.getInteger("weight");
						List<PreferredSchedulingTerm> preferredDuringSchedulingIgnoredDuringExecutions = new ArrayList<>();
						PreferredSchedulingTerm rreferredSchedulingTerm = new PreferredSchedulingTerm();
						rreferredSchedulingTerm.setWeight(weight);
						rreferredSchedulingTerm.setPreference(new NodeSelectorTerm(matchExpressions, new ArrayList<>()));
						preferredDuringSchedulingIgnoredDuringExecutions.add(rreferredSchedulingTerm);
						nodeAffinity.setPreferredDuringSchedulingIgnoredDuringExecution(
								preferredDuringSchedulingIgnoredDuringExecutions);
					}
					affinity.setNodeAffinity(nodeAffinity);
				}
				if ("podAffinity".equalsIgnoreCase(affinityType) || "podAntiAffinity".equalsIgnoreCase(affinityType)) {
					// 服务内亲和设置
					List<PodAffinityTerm> requiredDuringSchedulingIgnoredDuringExecution = new ArrayList<>();
					PodAffinityTerm podAffinityTerm = new PodAffinityTerm();
					LabelSelector labelSelector = new LabelSelector();
					List<LabelSelectorRequirement> matchExpressions = new ArrayList<>();
					LabelSelectorRequirement matchExpression = new LabelSelectorRequirement();
					if (affinityRequired) {
						matchExpression.setKey("applicationName");
						matchExpression.setValues(Arrays.asList(new String[] { applicationName }));
					} else {
						List<String> podMatchExpressions = JSON
								.parseArray(affinitysObject.getString("podMatchExpressions"), String.class);
						matchExpression.setKey("kubernetes.customized/pod-affinity");
						matchExpression.setValues(podMatchExpressions);
						podAffinityTerm.setNamespaces(Arrays.asList(new String[] { namespace }));
					}
					matchExpression.setOperator("In");
					matchExpressions.add(matchExpression);
					labelSelector.setMatchExpressions(matchExpressions);
					podAffinityTerm.setLabelSelector(labelSelector);
					podAffinityTerm.setTopologyKey("kubernetes.io/hostname");
					requiredDuringSchedulingIgnoredDuringExecution.add(podAffinityTerm);
					if ("podAffinity".equalsIgnoreCase(affinityType)) {
						// 服务实例亲和调度
						PodAffinity podAffinity = new PodAffinity();
						podAffinity.setRequiredDuringSchedulingIgnoredDuringExecution(
								requiredDuringSchedulingIgnoredDuringExecution);
						affinity.setPodAffinity(podAffinity);
					}
					if ("podAntiAffinity".equalsIgnoreCase(affinityType)) {
						// 服务实例反亲和调度
						PodAntiAffinity podAntiAffinity = new PodAntiAffinity();
						podAntiAffinity.setRequiredDuringSchedulingIgnoredDuringExecution(
								requiredDuringSchedulingIgnoredDuringExecution);
						affinity.setPodAntiAffinity(podAntiAffinity);
					}
				}
			}
			podTemplateSpecSpec.setAffinity(affinity);
			return podTemplateSpecSpec;
		} catch (Exception e) {
			logger.error("设置亲和性异常：" + e.getMessage());
			return null;
		}
	}

	@Override
	public BocloudResult remove(DeploymentModel model) {
		try {
			Deployment deployment = this.getClient().apps().deployments().inNamespace(model.getNamespace())
					.withName(model.getName()).get();
			if (null == deployment) {
				return new BocloudResult(true, "资源已被删除！");
			}
			List<Deployment> deployments = new ArrayList<>();
			deployments.add(deployment);
			Boolean delete = this.getClient().apps().deployments().delete(deployments);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("查询部署失败：" + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(DeploymentModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(DeploymentModel model) {
		try {
			String yamlText = model.getYamlText();
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Deployment deployment = this.getClient().apps().deployments().load(bais).get();
			ObjectMeta metadata = deployment.getMetadata();
			Deployment replace = this.getClient().apps().deployments().inNamespace(metadata.getNamespace())
					.withName(metadata.getName()).replace(deployment);
			return new BocloudResult(true, deploymentConvertor.convertModel(replace), "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult extendReplicas(DeploymentModel model) {
		try {
			Deployment deployment = this.getClient().apps().deployments().inNamespace(model.getNamespace())
					.withName(model.getName()).get();
			if (null == deployment) {
				return new BocloudResult(false, "资源不存在！");
			}
			DeploymentSpec spec = deployment.getSpec();
			spec.setReplicas(model.getReplica());
			deployment.setSpec(spec);
			Deployment replace = this.getClient().apps().deployments().inNamespace(model.getNamespace())
					.withName(model.getName()).replace(deployment);
			DeploymentModel deploymentModel = deploymentConvertor.convertModel(replace);
			boolean check = list(deploymentModel, model.getReplica());
			if (!check) {
				return new BocloudResult(false, "扩容缩容失败！");
			}
			List<PodModel> pods = listPods(deploymentModel);
			List<PodModel> listPods = list(pods);
			if (!ListTool.isEmpty(listPods)) {
				for (PodModel podModel : listPods) {
					podModel.setDeploymentApiVersion(deploymentModel.getApiVersion());
					podModel.setDeploymentName(deploymentModel.getName());
					podModel.setDeploymentUid(deploymentModel.getUid());
				}
				deploymentModel.setPodModels(listPods);
				deploymentModel.setPodCount(listPods.size());
			}
			return new BocloudResult(true, deploymentModel, "扩容缩容成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	public boolean list(DeploymentModel deploymentModel, Integer replica) {
		List<PodModel> pods = listPods(deploymentModel);
		boolean flag = false;
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				if (replica == pods.size()) {
					flag = true;
					break;
				}
				// 等待三分钟
				if (System.currentTimeMillis() - startTime > 5 * 60 * 1000) {
					break;
				}
				pods = listPods(deploymentModel);
			}
		} catch (Exception e) {
			logger.error("query pod list fail : " + e.getMessage());
		}
		return flag;
	}

}
