package com.bocloud.cmp.provider.kubernetes;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.NamespaceModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.NamespaceConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import io.fabric8.kubernetes.api.model.Namespace;
import io.fabric8.kubernetes.api.model.NamespaceList;
import io.fabric8.kubernetes.api.model.NamespaceSpec;
import io.fabric8.kubernetes.api.model.ObjectMeta;

/**
 * @ namespace
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeNamespaceProvider extends KubernetesProvider implements NamespaceProvider {

	private static Logger logger = LoggerFactory.getLogger(KubeNamespaceProvider.class);

	private BeanConvertor<NamespaceModel, Namespace> convertor = new NamespaceConvertor();

	public KubeNamespaceProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			NamespaceList list = this.getClient().namespaces().list();
			List<Namespace> items = list.getItems();
			List<NamespaceModel> convertList = convertor.convertList(items);
			return new BocloudResult(true, convertList, "查询Namespace成功！");
		} catch (Exception e) {
			logger.error("query namespace list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(NamespaceModel model) {
		try {
			Namespace item = new Namespace();
			ObjectMeta metadata = new ObjectMeta();
			metadata.setName(model.getName());
			item.setMetadata(metadata);
			Namespace create = this.getClient().namespaces().create(item);
			return new BocloudResult(true, convertor.convertModel(create), "创建Namespace成功！");
		} catch (Exception e) {
			logger.error("create namespace fail : " + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult remove(NamespaceModel model) {
		try {
			Namespace namespace = this.getClient().namespaces().withName(model.getName()).get();
			NamespaceSpec spec = namespace.getSpec();
			if (null != spec) {
				namespace.setSpec(null);
			}
			Boolean delete = this.getClient().namespaces().delete(namespace);
			return new BocloudResult(delete, "", delete ? "删除成功！" : "删除失败！");
		} catch (Exception e) {
			logger.error("delete namespace fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(NamespaceModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(NamespaceModel model) {
		return null;
	}

}
