package com.bocloud.cmp.provider.kubernetes;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.NodeModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.NodeConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;

import io.fabric8.kubernetes.api.model.DoneableNode;
import io.fabric8.kubernetes.api.model.Node;
import io.fabric8.kubernetes.api.model.NodeList;
import io.fabric8.kubernetes.api.model.NodeSpec;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.client.dsl.Resource;

/**
 * k8s node
 *
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeNodeProvider extends KubernetesProvider implements NodeProvider {

	private BeanConvertor<NodeModel, Node> convertor = new NodeConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubeNodeProvider.class);

	public KubeNodeProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			NodeList list = this.getClient().nodes().list();
			List<Node> items = list.getItems();
			List<NodeModel> convertList = convertor.convertList(items);
			return new BocloudResult(true, convertList, "查询Node成功！");
		} catch (Exception e) {
			logger.error("query node list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(NodeModel model) {
		return null;
	}

	@Override
	public BocloudResult remove(NodeModel model) {
		return null;
	}

	@Override
	public BocloudResult detail(NodeModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(NodeModel model) {
		try {
			String labels = model.getLabels();
			List<JSONObject> parseArray = JSON.parseArray(labels, JSONObject.class);
			Node node = this.getClient().nodes().withName(model.getName()).get();
			if (null == node) {
				return new BocloudResult(false, "节点数据异常！");
			}
			ObjectMeta metadata = node.getMetadata();
			Map<String, String> labelsMap = metadata.getLabels();
			if (!ListTool.isEmpty(parseArray)) {
				for (JSONObject object : parseArray) {
					labelsMap.put(object.getString("key"), object.getString("value"));
				}
			}
			Node createOrReplace = this.getClient().nodes().createOrReplace(node);
			return new BocloudResult(true, convertor.convertModel(createOrReplace), "策略调整成功！");
		} catch (Exception e) {
			logger.error("modify node schedule fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult modifySchedule(NodeModel model) {
		try {
			Resource<Node> currentNode = this.getClient().nodes().withName(model.getName());
			Node node = currentNode.get();
			if (null == node) {
				return new BocloudResult(false, "节点信息不存在！");
			}
			NodeSpec spec = node.getSpec();
			spec.setUnschedulable(!model.getSchedulable());
			node.setSpec(spec);
			Node newNode = currentNode.createOrReplace(node);
			return new BocloudResult(true, convertor.convertModel(newNode), "策略调整成功！");
		} catch (Exception e) {
			logger.error("modify node schedule fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult removeNodeLabels(NodeModel model) {
		try {
			String labelKey = model.getLabelKey();
			Node node = this.getClient().nodes().withName(model.getName()).get();
			if (null == node) {
				return new BocloudResult(false, "节点数据异常！");
			}
			ObjectMeta metadata = node.getMetadata();
			Map<String, String> labelsMap = metadata.getLabels();
			if (!StringUtils.isEmpty(labelKey)) {
				if (labelsMap.containsKey(labelKey)) {
					labelsMap.remove(labelKey);
				}
			}
			Node createOrReplace = this.getClient().nodes().createOrReplace(node);
			return new BocloudResult(true, convertor.convertModel(createOrReplace), "删除标签成功！");
		} catch (Exception e) {
			logger.error("删除标签 fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult addNodeLabels(NodeModel model) {
		try {
			String labels = model.getLabels();
			JSONObject object = JSON.parseObject(labels, JSONObject.class);
			Node node = this.getClient().nodes().withName(model.getName()).get();
			if (null == node) {
				return new BocloudResult(false, "节点数据异常！");
			}
			ObjectMeta metadata = node.getMetadata();
			Map<String, String> labelsMap = metadata.getLabels();
			if (!StringUtils.isEmpty(labels)) {
				labelsMap.put(object.getString("key"), object.getString("value"));
			}
			Node createOrReplace = this.getClient().nodes().createOrReplace(node);
			return new BocloudResult(true, convertor.convertModel(createOrReplace), "添加标签成功！");
		} catch (Exception e) {
			logger.error("添加标签  fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

}
