package com.bocloud.cmp.provider.kubernetes;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeClaimModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PersistentVolumeClaimConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaim;
import io.fabric8.kubernetes.api.model.PersistentVolumeClaimSpec;
import io.fabric8.kubernetes.api.model.Quantity;
import io.fabric8.kubernetes.api.model.ResourceRequirements;

/**
 * k8s PersistentVolume
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubePersistentVolumeClaimProvider extends KubernetesProvider implements PersistentVolumeClaimProvider {

	public KubePersistentVolumeClaimProvider(Butler butler) {
		super(butler);
	}

	private BeanConvertor<PersistentVolumeClaimModel, PersistentVolumeClaim> persistentVolumeConvertor = new PersistentVolumeClaimConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubePersistentVolumeClaimProvider.class);

	@Override
	public BocloudResult list() {
		return null;
	}

	@Override
	public BocloudResult create(PersistentVolumeClaimModel model) {
		try {
			String accessModes = model.getAccessModes();
			if (null == accessModes) {
				return new BocloudResult(false, "缺少访问模式！");
			}
			PersistentVolumeClaim item = new PersistentVolumeClaim();
			ObjectMeta metadata = new ObjectMeta();
			metadata.setName(model.getName());
			metadata.setNamespace(model.getNamespace());
			String labels = model.getLabels();
			if (!StringUtils.isEmpty(labels)) {
				Map<String, String> label = new HashMap<>();
				List<JSONObject> array = JSONObject.parseArray(labels, JSONObject.class);
				for (JSONObject jsonObject : array) {
					label.put(jsonObject.getString("key"), jsonObject.getString("value"));
				}
				metadata.setLabels(label);
			}
			item.setMetadata(metadata);
			String storageAmount = model.getStorageAmount();
			String unit = model.getUnit();
			if (unit.contains("B")) {
				unit = unit.substring(0, unit.indexOf("B"));
			}
			PersistentVolumeClaimSpec spec = new PersistentVolumeClaimSpec();
			ResourceRequirements resources = new ResourceRequirements();
			Map<String, Quantity> requests = new HashMap<String, Quantity>();
			requests.put("storage", new Quantity(storageAmount + unit));
			resources.setRequests(requests);
			spec.setResources(resources);
			spec.setAccessModes(Arrays.asList(new String[] { accessModes }));
			String storageClassType = model.getStorageClassType();
			String storageClassName = model.getStorageClassName();
			if ("storageClassName".equalsIgnoreCase(storageClassType) && !StringUtils.isEmpty(storageClassName)) {
				spec.setStorageClassName(model.getStorageClassName());
			}
			item.setSpec(spec);
			PersistentVolumeClaim persistentVolumeClaim = this.getClient().persistentVolumeClaims().create(item);
			return new BocloudResult(true, waitStatus(persistentVolumeConvertor.convertModel(persistentVolumeClaim)),
					"新增成功！");
		} catch (Exception e) {
			logger.error("新增存储卷失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	public PersistentVolumeClaimModel waitStatus(PersistentVolumeClaimModel model) {
		PersistentVolumeClaimModel persistentVolumeClaimModel = null;
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				Thread.sleep(200);
				PersistentVolumeClaim persistentVolumeClaim = this.getClient().persistentVolumeClaims()
						.inNamespace(model.getNamespace()).withName(model.getName()).get();
				persistentVolumeClaimModel = persistentVolumeConvertor.convertModel(persistentVolumeClaim);
				if (null == persistentVolumeClaimModel
						|| "Bound".equalsIgnoreCase(persistentVolumeClaimModel.getStatus())) {
					break;
				}
				// 等待2分钟
				if (System.currentTimeMillis() - startTime > 1 * 60 * 1000) {
					break;
				}
			}
		} catch (Exception e) {
			logger.error("query pod list fail : " + e.getMessage());
		}
		return persistentVolumeClaimModel;
	}

	@Override
	public BocloudResult remove(PersistentVolumeClaimModel model) {
		try {
			PersistentVolumeClaim persistentVolumeClaim = this.getClient().persistentVolumeClaims()
					.inNamespace(model.getNamespace()).withName(model.getName()).get();
			if (null == persistentVolumeClaim) {
				return new BocloudResult(true, "删除成功！");
			}
			Boolean delete = this.getClient().persistentVolumeClaims().delete(persistentVolumeClaim);
			return new BocloudResult(delete, "", delete ? "删除成功！" : "删除失败！");
		} catch (Exception e) {
			logger.error("删除PersistentVolume失败：" + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(PersistentVolumeClaimModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(PersistentVolumeClaimModel model) {
		try {
			return new BocloudResult(true, "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑存储卷失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

}
