package com.bocloud.cmp.provider.kubernetes;

import java.io.ByteArrayInputStream;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.PersistentVolumeModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.PersistentVolumeConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import io.fabric8.kubernetes.api.model.DoneablePersistentVolume;
import io.fabric8.kubernetes.api.model.ObjectMeta;
import io.fabric8.kubernetes.api.model.PersistentVolume;
import io.fabric8.kubernetes.client.dsl.Resource;

/**
 * k8s PersistentVolume
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubePersistentVolumeProvider extends KubernetesProvider implements PersistentVolumeProvider {

	public KubePersistentVolumeProvider(Butler butler) {
		super(butler);
	}

	private BeanConvertor<PersistentVolumeModel, PersistentVolume> persistentVolumeConvertor = new PersistentVolumeConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubePersistentVolumeProvider.class);

	@Override
	public BocloudResult list() {
		return null;
	}

	@Override
	public BocloudResult create(PersistentVolumeModel model) {
		try {
			String yt = model.getYamlText();
			if (StringUtils.isEmpty(yt)) {
				return createPersistentVolume(model);
			}
			String yamlText = CommonUtils.yamlAnalysis(yt);
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			PersistentVolume persistentVolume = this.getClient().persistentVolumes().load(bais).get();
			ObjectMeta metadata = persistentVolume.getMetadata();
			if (null != metadata.getResourceVersion()) {
				metadata.setResourceVersion(null);
			}
			if (null != metadata.getUid()) {
				metadata.setUid(null);
			}
			if (null != metadata.getSelfLink()) {
				metadata.setSelfLink(null);
			}
			persistentVolume.setMetadata(metadata);
			PersistentVolume create = this.getClient().persistentVolumes().create(persistentVolume);
			return new BocloudResult(true, persistentVolumeConvertor.convertModel(create), "新增成功！");
		} catch (Exception e) {
			logger.error("新增存储卷失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	private BocloudResult createPersistentVolume(PersistentVolumeModel model) {
		try {
			return new BocloudResult(true, "存储卷创建成功！");
		} catch (Exception e) {
			logger.error("创建存储卷异常：" + e.getMessage());
			return new BocloudResult(false, "创建存储卷异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(PersistentVolumeModel model) {
		try {
			PersistentVolume persistentVolume = this.getClient().persistentVolumes().withName(model.getName()).get();
			if (null == persistentVolume) {
				return new BocloudResult(true, "删除成功！");
			}
			Boolean delete = this.getClient().persistentVolumes().delete(persistentVolume);
			return new BocloudResult(delete, "", delete ? "删除成功 ！" : "删除失败！");
		} catch (Exception e) {
			logger.error("删除PersistentVolume失败：" + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(PersistentVolumeModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(PersistentVolumeModel model) {
		try {
			String yamlText = model.getYamlText();
			ByteArrayInputStream bais = new ByteArrayInputStream(yamlText.getBytes());
			Resource<PersistentVolume> load = this.getClient().persistentVolumes().load(bais);
			PersistentVolume persistentVolume = load.get();
			ObjectMeta metadata = persistentVolume.getMetadata();
			PersistentVolume replace = this.getClient().persistentVolumes().withName(metadata.getName())
					.replace(persistentVolume);
			return new BocloudResult(true, persistentVolumeConvertor.convertModel(replace), "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑存储卷失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

}
