package com.bocloud.cmp.provider.kubernetes;

import java.util.HashMap;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.kubernetes.SecretModel;
import com.bocloud.cmp.provider.KubernetesProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.SecretConvertor;
import com.bocloud.cmp.provider.utils.CommonUtils;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.api.model.SecretBuilder;

/**
 * k8s service
 * 
 * @时间 2019年8月21日
 */
@Provider("bocloud.cmp.kubernetes")
public class KubeSecretProvider extends KubernetesProvider implements SecretProvider {

	public KubeSecretProvider(Butler butler) {
		super(butler);
	}

	BeanConvertor<SecretModel, Secret> convertor = new SecretConvertor();

	private static Logger logger = LoggerFactory.getLogger(KubeSecretProvider.class);

	@Override
	public BocloudResult list() {
		try {
			return new BocloudResult(true, null, "查询Node成功！");
		} catch (Exception e) {
			logger.error("query node list fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SecretModel model) {
		try {
			String nameSpace = model.getNamespace();
			String secretName = model.getName();
			Map<String, String> secretData = new HashMap<String, String>(1);
			secretData.put(".dockerconfigjson", model.getDockerconfigjson());
			SecretBuilder secretBuilder = new SecretBuilder();
			secretBuilder.withNewMetadata().withName(secretName).withNamespace(nameSpace).endMetadata()
					.withData(secretData).withType(model.getType());
			Secret create = this.getClient().secrets().inNamespace(nameSpace).create(secretBuilder.build());
			return new BocloudResult(true, convertor.convertModel(create), "新增成功！");
		} catch (Exception e) {
			logger.error("新增Services失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

	@Override
	public BocloudResult remove(SecretModel model) {
		try {
			Secret secret = this.getClient().secrets().inNamespace(model.getNamespace()).withName(model.getName())
					.get();
			if (null == secret) {
				return new BocloudResult(true, "删除成功！");
			}
			Boolean delete = this.getClient().secrets().delete(secret);
			return new BocloudResult(delete, "", delete ? "删除成功！" : "删除失败！");
		} catch (Exception e) {
			logger.error("delete service fail : " + e.getMessage());
			return new BocloudResult(false, e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(SecretModel model) {
		return null;
	}

	@Override
	public BocloudResult modify(SecretModel model) {
		try {
			return new BocloudResult(true, null, "编辑成功！");
		} catch (Exception e) {
			logger.error("编辑失败：" + e.getMessage());
			return new BocloudResult(false, CommonUtils.getMessage(e.getMessage()));
		}
	}

}
