package com.bocloud.cmp.kingconvertor;

import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.ksc.internal.SdkInternalList;
import com.ksc.network.vpc.model.SecurityGroups.SecurityGroup;
import com.ksc.network.vpc.model.SecurityGroups.SecurityGroupRule;

import java.util.ArrayList;
import java.util.List;

/**
 * 区域转换
 * 
 * @author ymh
 * @时间 2019年9月12日
 */
public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, SecurityGroup> {

	@Override
	public List<SecurityGroupModel> convertList(List<? extends SecurityGroup> objects) {
		List<SecurityGroupModel> regionModels = new ArrayList<>();
		for (SecurityGroup object : objects) {
			regionModels.add(convertModel(object));
		}
		return regionModels;
	}

	@Override
	public SecurityGroupModel convertModel(SecurityGroup info) {
		SecurityGroupModel model = new SecurityGroupModel();
		if (null != info) {
			model.setGroupUuid(info.getSecurityGroupId());
			model.setName(info.getSecurityGroupName());
			model.setVpcUuid(info.getVpcId());
			String creationTime = info.getCreateTime();
			if (null != creationTime && !"".equals(creationTime)) {
				model.setCreateTime(DateTools.stringToDateTime(creationTime));
			}
			SdkInternalList<SecurityGroupRule> securityGroupEntrySet = info.getSecurityGroupEntrySet();
			if (null != securityGroupEntrySet && securityGroupEntrySet.size() > 0) {
				model.setModels(convertRuleModel(securityGroupEntrySet));
			}
		}
		return model;
	}

	private List<SecurityGroupRuleModel> convertRuleModel(SdkInternalList<SecurityGroupRule> securityGroupEntrySet) {
		List<SecurityGroupRuleModel> groupRuleModelList = new ArrayList<>();
		SecurityGroupRuleModel groupRuleModel = null;
		for (SecurityGroupRule securityGroupRule : securityGroupEntrySet) {
			groupRuleModel = new SecurityGroupRuleModel();
			groupRuleModel.setAction("ACCEPT");
			groupRuleModel.setDirection(convertDirection(securityGroupRule.getDirection()));
			groupRuleModel.setPortMin(securityGroupRule.getPortRangeFrom());
			groupRuleModel.setPortMax(securityGroupRule.getPortRangeTo());
			groupRuleModel.setProtocol(
					securityGroupRule.getProtocol() == null ? "" : securityGroupRule.getProtocol().toUpperCase());
			groupRuleModel.setRemoteIpPrefix(securityGroupRule.getCidrBlock());
			groupRuleModel.setGroupUuid(securityGroupRule.getSecurityGroupId());
			groupRuleModel.setRuleUuid(securityGroupRule.getSecurityGroupEntryId());
			groupRuleModel.setRemark(securityGroupRule.getDescription());
			groupRuleModelList.add(groupRuleModel);
		}
		return groupRuleModelList;
	}

	private String convertDirection(String direction) {
		if ("in".equalsIgnoreCase(direction)) {
			return "ingress";
		}
		if ("out".equalsIgnoreCase(direction)) {
			return "egress";
		}
		return "ingress";
	}
}
