package com.bocloud.cmp.kingconvertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 云主机转换
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
public class ServerConvertor implements BeanConvertor<ServerModel, JSONObject> {

    @Override
    public List<ServerModel> convertList(List<? extends JSONObject> objects) {
        List<ServerModel> regionModels = new ArrayList<>();
        for (JSONObject object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public ServerModel convertModel(JSONObject info) {
        ServerModel model = new ServerModel();
        if (null != info) {
            model.setInstanceId(info.getString("InstanceId"));
            model.setName(info.getString("InstanceName"));
            model.setFlavorUuid(info.getString("InstanceType"));
            this.convertConfigures(info, model);
            model.setImageUuid(info.getString("ImageId"));
            model.setSubnetUuid(info.getString("SubnetId"));
            model.setPrivateIps(info.getString("PrivateIpAddress"));
            JSONObject instanceState = info.getJSONObject("InstanceState");
            model.setStatus(
                    instanceState.getString("Name") == null ? null : instanceState.getString("Name").toUpperCase());
            String createTime = info.getString("CreationDate");
            if (null != createTime && !"".equals(createTime)) {
                model.setGmtVmCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createTime, "Z"));
            }
            model.setZone(info.getString("AvailabilityZone"));
            model.setInstanceChargeType(info.getString("ChargeType"));
            this.convertKeypairs(info, model);
            this.convertSystemDisks(info, model);
            this.convertNetworks(info, model);
        }
        return model;
    }

    private void convertConfigures(JSONObject info, ServerModel model) {
        try {
            JSONObject instanceConfigure = info.getObject(info.getString("InstanceConfigure"), JSONObject.class);
            if (null != instanceConfigure) {
                model.setCpu(instanceConfigure.getInteger("VCPU"));
                model.setMemory(new BigDecimal(instanceConfigure.getInteger("MemoryGb")));
            }
        } catch (Exception e) {
        }
    }

    private void convertSystemDisks(JSONObject info, ServerModel model) {
        try {
            JSONObject systemDisk = info.getJSONObject("SystemDisk");
            if (null != systemDisk) {
                model.setSystemDiskSize(systemDisk.getInteger("DiskSize"));
                model.setDisk(systemDisk.getInteger("DiskSize"));
                model.setSystemDiskCategory(systemDisk.getString("DiskType"));
            }
        } catch (Exception e) {
        }
    }

    private void convertKeypairs(JSONObject info, ServerModel model) {
        try {
            JSONArray keySets = info.getJSONArray("KeySet");
            if (null != keySets && keySets.size() > 0) {
                String[] keyArrays = new String[keySets.size()];
                for (int i = 0; i < keyArrays.length; i++) {
                    keyArrays[i] = keySets.getString(i);
                }
                model.setKeypairIds(keyArrays);
            }
        } catch (Exception e) {
        }
    }

    private void convertNetworks(JSONObject info, ServerModel model) {
        try {
            String networkInterfaceSetInfo = info.getString("NetworkInterfaceSet");
            if (null != networkInterfaceSetInfo) {
                List<NetworkModel> networks = new ArrayList<>();
                List<String> securityGroups = new ArrayList<>();
                List<JSONObject> networkInterfaceSets = JSONObject.parseArray(networkInterfaceSetInfo,
                        JSONObject.class);
                for (JSONObject jsonObject : networkInterfaceSets) {
                    NetworkModel networkModel = new NetworkModel();
                    networkModel.setNetworkUuid(jsonObject.getString("VpcId"));
                    networkModel.setSubnetIds(Arrays.asList(new String[] { jsonObject.getString("SubnetId") }));
                    networkModel.setFixedIp(jsonObject.getString("PrivateIpAddress"));
                    networkModel.setType(jsonObject.getString("NetworkInterfaceType"));
                    networks.add(networkModel);

                    String securityGroupSet = jsonObject.getString("SecurityGroupSet");
                    List<JSONObject> securityGroupSetArray = JSONObject.parseArray(securityGroupSet, JSONObject.class);
                    for (JSONObject jsonObject2 : securityGroupSetArray) {
                        securityGroups.add(jsonObject2.getString("SecurityGroupId"));
                    }
                }
                model.setNetworks(networks);
            }
        } catch (Exception e) {
        }
    }
}
