package com.bocloud.cmp.provider;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.util.HttpClientHelper;
import com.bocloud.cmp.provider.util.KingCloudSignerV4Util;
import com.ksc.HttpMethod;
import com.ksc.auth.AWSCredentials;
import com.ksc.auth.BasicAWSCredentials;
import com.ksc.kec.KSCKECClient;
import com.ksc.kec.model.DescribeRegionsRequest;
import com.ksc.ket.util.Header;
import com.ksc.network.vpc.KSCVPCClient;
import com.ksc.regions.InMemoryRegionImpl;
import com.ksc.regions.Region;

public class KingCloudProvider extends GenericProvider implements SuperProvider {

    private String secretId;
    private String secretKey;
    private String regionId = "cn-shanghai-2"; // 选择大区

    private AWSCredentials credentials = null;
    private Region region = null;

    public KingCloudProvider(Butler butler) {
        super(butler);
        if (StringUtils.isEmpty(butler.getAccessKey())) {
            throw new IllegalArgumentException("Access key cannot be null.");
        }
        if (StringUtils.isEmpty(butler.getSecretKey())) {
            throw new IllegalArgumentException("Secret key cannot be null.");
        }
        this.secretId = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        if (StringUtils.isNotEmpty(butler.getRegionId())) {
            this.regionId = butler.getRegionId();
        }
        init();
    }

    private void init() {
        credentials = new BasicAWSCredentials(secretId, secretKey);
        region = new Region(new InMemoryRegionImpl(this.regionId, null));
    }

    public KSCKECClient getKecClient() {
        KSCKECClient kecClient = new KSCKECClient(credentials);
        kecClient.setRegion(region);
        return kecClient;
    }

    public KSCVPCClient getVpcClient() {
        KSCVPCClient vpcClient = new KSCVPCClient(credentials);
        vpcClient.setRegion(region);
        return vpcClient;
    }

    @Override
    public boolean accessable() {
        try {
            this.getKecClient().describeRegions(new DescribeRegionsRequest());
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public String getSecretId() {
        return secretId;
    }

    public void setSecretId(String secretId) {
        this.secretId = secretId;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public String call(String endpoint, Map<String, String> basicParams, String action, String version,
            String serviceName, HttpMethod httpMethoid) throws Exception {
        String result = "";
        Map<String, String> parameters = new HashMap<String, String>();
        parameters.put("Action", action);
        parameters.put("Version", version);
        if (null != basicParams) {
            parameters.putAll(basicParams);
        }
        switch (httpMethoid) {
        case GET:
            result = apiGet(endpoint, serviceName, parameters);
            break;
        case POST:
            result = apiPost(endpoint, serviceName, parameters, JSON.toJSONString(basicParams));
            break;
        default:
            break;
        }
        return result;
    }

    private String apiPost(String endpoint, String serviceName, Map<String, String> parameters, String params)
            throws Exception {
        URI uri = URI.create(endpoint);
        Map<String, String> headers = new HashMap<String, String>();
        InputStream body = new ByteArrayInputStream(params.getBytes("ISO-8859-1"));
        List<Header> header = KingCloudSignerV4Util.getAuthHeaderForPost(uri, parameters, body, headers, serviceName,
                regionId, secretId, secretKey);
        return HttpClientHelper.sendPost(endpoint + "/?" + MapToString(parameters), params, header);
    }

    private String apiGet(String endpoint, String serviceName, Map<String, String> parameters) throws Exception {
        URI uri = URI.create(endpoint);
        Map<String, String> authHeaderForGet = KingCloudSignerV4Util.getAuthHeaderForGet(uri, parameters, null,
                serviceName, regionId, secretId, secretKey);
        return HttpClientHelper.sendGet(endpoint + "/", MapToString(parameters), authHeaderForGet);
    }

    public String MapToString(Map<String, String> parameters) {
        String param = "";
        for (String key : parameters.keySet()) {
            param = param + key + "=" + parameters.get(key) + "&";
        }
        return param;
    }

}