package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.kec.model.DescribeRegionsRequest;
import com.ksc.kec.model.DescribeRegionsResult;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 金山云连接验证
 * 
 * @author ymh
 * @时间 2019年10月8日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudAuthProvider extends KingCloudProvider implements AuthProvider {

    public KingCloudAuthProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudAuthProvider.class);

    @Override
    public boolean accessable() {
        try {
            DescribeRegionsRequest request = new DescribeRegionsRequest();
            this.getKecClient().describeRegions(request);
            return true;
        } catch (Exception e) {
            logger.error("连接失败：" + e.getMessage());
            return false;
        }
    }

    @Override
    public BocloudResult auth() {
        try {
            DescribeRegionsRequest request = new DescribeRegionsRequest();
            DescribeRegionsResult describeRegions = this.getKecClient().describeRegions(request);
            return new BocloudResult(true, describeRegions, "金山云连接成功！");
        } catch (Exception e) {
            logger.error("获取连接失败：" + e.getMessage());
            return new BocloudResult(false, "金山云连接失败：" + e.getMessage());
        }
    }

}
