package com.bocloud.cmp.provider.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.kingconvertor.FlavorConvertor;
import com.bocloud.cmp.kingconvertor.ImageConvertor;
import com.bocloud.cmp.kingconvertor.LocalVolumeConvertor;
import com.bocloud.cmp.kingconvertor.RegionConvertor;
import com.bocloud.cmp.kingconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.kingconvertor.SubnetConvertor;
import com.bocloud.cmp.kingconvertor.VpcConvertor;
import com.bocloud.cmp.kingconvertor.ZoneConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.KingCloudSyncModel;
import com.bocloud.cmp.model.RegionModel;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.model.ZoneModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.compute.KingCloudServerProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.security.KingCloudKeypairProvider;
import com.bocloud.cmp.provider.storage.KingCloudVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.kec.model.AvailabilityZone;
import com.ksc.kec.model.DescribeAvailabilityZonesRequest;
import com.ksc.kec.model.DescribeAvailabilityZonesResult;
import com.ksc.kec.model.DescribeImagesRequest;
import com.ksc.kec.model.DescribeImagesResult;
import com.ksc.kec.model.DescribeInstanceTypeConfigsRequest;
import com.ksc.kec.model.DescribeInstanceTypeConfigsResult;
import com.ksc.kec.model.DescribeLocalVolumesRequest;
import com.ksc.kec.model.DescribeLocalVolumesResult;
import com.ksc.kec.model.DescribeRegionsRequest;
import com.ksc.kec.model.DescribeRegionsResult;
import com.ksc.kec.model.Image;
import com.ksc.kec.model.InstanceTypeConfig;
import com.ksc.kec.model.LocalVolume;
import com.ksc.kec.model.Region;
import com.ksc.model.Filter;
import com.ksc.network.vpc.model.SecurityGroups.DescribeSecurityGroupsRequest;
import com.ksc.network.vpc.model.SecurityGroups.DescribeSecurityGroupsResult;
import com.ksc.network.vpc.model.SecurityGroups.SecurityGroup;
import com.ksc.network.vpc.model.subnet.DescribeSubnetsRequest;
import com.ksc.network.vpc.model.subnet.DescribeSubnetsResult;
import com.ksc.network.vpc.model.subnet.Subnet;
import com.ksc.network.vpc.model.vpc.DescribeVpcsRequest;
import com.ksc.network.vpc.model.vpc.DescribeVpcsResult;
import com.ksc.network.vpc.model.vpc.Vpc;

/**
 * 金山云同步
 * 
 * @author ymh
 * @时间 2019年9月12日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudLocationProvider extends KingCloudProvider implements LocationProvider {

    private Butler butler;

    public KingCloudLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudLocationProvider.class);
    private BeanConvertor<RegionModel, Region> regionConvertor = new RegionConvertor();
    private BeanConvertor<ZoneModel, AvailabilityZone> zoneConvertor = new ZoneConvertor();
    private BeanConvertor<ImageModel, Image> imageConvertor = new ImageConvertor();
    private BeanConvertor<FlavorModel, InstanceTypeConfig> flavorConvertor = new FlavorConvertor();
    private BeanConvertor<VpcModel, Vpc> vpcConvertor = new VpcConvertor();
    private BeanConvertor<SubnetModel, Subnet> subnetConvertor = new SubnetConvertor();
    private BeanConvertor<SecurityGroupModel, SecurityGroup> securityGroupConvertor = new SecurityGroupConvertor();
    private BeanConvertor<VolumeModel, LocalVolume> localVolumeConvertor = new LocalVolumeConvertor();

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult sync() {
        try {
            KingCloudSyncModel syncModel = new KingCloudSyncModel();
            RegionModel regionModel = new RegionModel();
            regionModel.setCode(this.getRegionId());
            regionModel.setRegionId(this.getRegionId());
            // 查询zone
            DescribeAvailabilityZonesRequest describeAvailabilityZonesRequest = new DescribeAvailabilityZonesRequest();
            DescribeAvailabilityZonesResult availabilityZones = this.getKecClient()
                    .describeAvailabilityZones(describeAvailabilityZonesRequest);
            List<AvailabilityZone> availabilityZoneSet = availabilityZones.getAvailabilityZoneSet();
            List<ZoneModel> zoneModels = zoneConvertor.convertList(availabilityZoneSet);
            regionModel.setZones(zoneModels);
            syncModel.setRegionModels(Arrays.asList(new RegionModel[] { regionModel }));
            // 同步镜像
            DescribeImagesRequest imageRequest = new DescribeImagesRequest();
            DescribeImagesResult describeImages = getKecClient().describeImages(imageRequest);
            List<ImageModel> imageModels = imageConvertor.convertList(describeImages.getImagesSet());
            syncModel.setImageModels(imageModels);
            // 同步规格
            DescribeInstanceTypeConfigsRequest InstanceTypeConfigRequest = new DescribeInstanceTypeConfigsRequest();
            DescribeInstanceTypeConfigsResult describeInstanceTypeConfigs = getKecClient()
                    .describeInstanceTypeConfigs(InstanceTypeConfigRequest);
            List<InstanceTypeConfig> instanceTypeConfigSet = describeInstanceTypeConfigs.getInstanceTypeConfigSet();
            syncModel.setFlavorModels(flavorConvertor.convertList(instanceTypeConfigSet));
            // vpc同步
            DescribeVpcsRequest request = new DescribeVpcsRequest();
            DescribeVpcsResult describeVpcs = getVpcClient().describeVpcs(request);
            List<VpcModel> vpcModels = vpcConvertor.convertList(describeVpcs.getVpcSet());
            // 获取vpc下的子网信息
            if (null != vpcModels && vpcModels.size() > 0) {
                DescribeSubnetsRequest subnetRequest = new DescribeSubnetsRequest();
                Collection<Filter> filters = new ArrayList<>();
                Filter filter = null;
                for (VpcModel vpcModel : vpcModels) {
                    filter = new Filter();
                    filter.setName("vpc-id");
                    filter.setValues(Arrays.asList(new String[] { vpcModel.getVpcId() }));
                    filters.add(filter);
                    subnetRequest.setFilters(filters);
                    DescribeSubnetsResult describeSubnets = getVpcClient().describeSubnets(subnetRequest);
                    List<Subnet> subnetSet = describeSubnets.getSubnetSet();
                    List<SubnetModel> subnetModels = subnetConvertor.convertList(subnetSet);
                    vpcModel.setSubnetModels(subnetModels);
                    filters.clear();
                }
            }
            syncModel.setVpcModels(vpcModels);
            // 同步云硬盘
            List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
            KingCloudVolumeProvider volumeProvider = new KingCloudVolumeProvider(butler);
            BocloudResult volumeList = volumeProvider.list();
            if (volumeList.isFailed()) {
                return new BocloudResult(false, volumeList.getMessage());
            }
            List<VolumeModel> cloudVolumeModels = (List<VolumeModel>) volumeList.getData();
            if (null != cloudVolumeModels && cloudVolumeModels.size() > 0) {
                volumeModels.addAll(cloudVolumeModels);
            }
            // 同步本地硬盘
            DescribeLocalVolumesRequest describeLocalVolumesRequest = new DescribeLocalVolumesRequest();
            DescribeLocalVolumesResult describeLocalVolumes = getKecClient()
                    .describeLocalVolumes(describeLocalVolumesRequest);
            List<LocalVolume> localVolumeSet = describeLocalVolumes.getLocalVolumeSet();
            List<VolumeModel> localVolumeModels = localVolumeConvertor.convertList(localVolumeSet);
            for (VolumeModel volumeModel : localVolumeModels) {
                volumeModel.setRegionId(this.getRegionId());
            }
            if (null != localVolumeModels) {
                volumeModels.addAll(localVolumeModels);
            }
            syncModel.setVolumeModels(volumeModels);
            // 同步安全组
            DescribeSecurityGroupsRequest describeSecurityGroupsRequest = new DescribeSecurityGroupsRequest();
            DescribeSecurityGroupsResult describeSecurityGroups = getVpcClient()
                    .describeSecurityGroups(describeSecurityGroupsRequest);
            List<SecurityGroup> securityGroupSet = describeSecurityGroups.getSecurityGroupSet();
            syncModel.setSecurityGroupModels(securityGroupConvertor.convertList(securityGroupSet));
            // 同步云主机
            KingCloudServerProvider serverProvider = new KingCloudServerProvider(butler);
            BocloudResult serverList = serverProvider.list();
            if (serverList.isFailed()) {
                return new BocloudResult(false, serverList.getMessage());
            }
            List<ServerModel> serverModels = (List<ServerModel>) serverList.getData();
            syncModel.setServerModels(serverModels);
            // 同步秘钥
            KingCloudKeypairProvider keypairProvider = new KingCloudKeypairProvider(butler);
            BocloudResult keypairList = keypairProvider.list();
            if (keypairList.isFailed()) {
                return new BocloudResult(false, keypairList.getMessage());
            }
            List<KeypairModel> keypairModels = (List<KeypairModel>) keypairList.getData();
            syncModel.setKeypairModels(keypairModels);
            return new BocloudResult(true, syncModel, "同步成功！");
        } catch (Exception e) {
            logger.error("金山云同步失败：" + e.getMessage());
            return new BocloudResult(false, "金山云同步失败：" + e.getMessage());
        }
    }

    public static void main(String[] args) {
        String ak = "AKLTN3Hbk45USVqVriho6ZpgqQ";
        String sk = "OLyFJlGQQ6DHQ2gFavA4qilsStg1YaESE6GW6Wmep620/4VQ1z/FyzuE4kY+NIYH4A==";
        Butler butler = new Butler();
        butler.setAccessKey(ak);
        butler.setSecretKey(sk);
        butler.setRegionId("cn-shanghai-2");
        KingCloudLocationProvider location = new KingCloudLocationProvider(butler);
        System.out.println(location.sync());
    }


    @Override
    public BocloudResult listRegions() {
        try {
            DescribeRegionsRequest request = new DescribeRegionsRequest();
            DescribeRegionsResult describeRegions = getKecClient().describeRegions(request);
            List<Region> regionSet = describeRegions.getRegionSet();
            return new BocloudResult(true, regionConvertor.convertList(regionSet), "查询成功");
        } catch (Exception e) {
            logger.error("查询region失败：" + e.getMessage());
            return new BocloudResult(false, "查询region失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        try {
            DescribeAvailabilityZonesRequest describeAvailabilityZonesRequest = new DescribeAvailabilityZonesRequest();
            DescribeAvailabilityZonesResult availabilityZones = getKecClient()
                    .describeAvailabilityZones(describeAvailabilityZonesRequest);
            List<AvailabilityZone> availabilityZoneSet = availabilityZones.getAvailabilityZoneSet();
            return new BocloudResult(true, zoneConvertor.convertList(availabilityZoneSet), "查询成功");
        } catch (Exception e) {
            logger.error("查询zone失败：" + e.getMessage());
            return new BocloudResult(false, "查询zone失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        // TODO Auto-generated method stub
        return null;
    }

}
