package com.bocloud.cmp.provider.compute;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.kingconvertor.ImageConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.kec.model.CreateImageRequest;
import com.ksc.kec.model.CreateImageResult;
import com.ksc.kec.model.DescribeImagesRequest;
import com.ksc.kec.model.DescribeImagesResult;
import com.ksc.kec.model.Image;
import com.ksc.kec.model.ModifyImageAttributeRequest;
import com.ksc.kec.model.ModifyImageAttributeResult;
import com.ksc.kec.model.RemoveImagesRequest;
import com.ksc.kec.model.RemoveImagesResult;

/**
 * 镜像
 * 
 * @author ymh
 * @时间 2019年9月25日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudImageProvider extends KingCloudProvider implements ImageProvider {

    public KingCloudImageProvider(Butler butler) {
        super(butler);
    }

    private BeanConvertor<ImageModel, Image> imageConvertor = new ImageConvertor();
    private static Logger logger = LoggerFactory.getLogger(KingCloudImageProvider.class);

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult detail(String imageId) {
        return null;
    }

    @Override
    public BocloudResult remove(String imageId) {
        try {
            RemoveImagesRequest removeImagesRequest = new RemoveImagesRequest();
            removeImagesRequest.setImageId(imageId);
            RemoveImagesResult removeImages = getKecClient().removeImages(removeImagesRequest);
            return new BocloudResult(true, removeImages.getReturnSet(), "删除成功！");
        } catch (Exception e) {
            logger.error("镜像删除失败：" + e.getMessage());
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        try {
            ModifyImageAttributeRequest modifyImageAttributeRequest = new ModifyImageAttributeRequest();
            modifyImageAttributeRequest.setImageId(image.getImageUuid());
            modifyImageAttributeRequest.setName(image.getName());
            ModifyImageAttributeResult modifyImageAttribute = getKecClient()
                    .modifyImageAttribute(modifyImageAttributeRequest);
            return new BocloudResult(modifyImageAttribute.getReturn(),
                    modifyImageAttribute.getReturn() ? "编辑成功！" : "编辑失败！");
        } catch (Exception e) {
            logger.error("镜像编辑失败：" + e.getMessage());
            return new BocloudResult(false, "镜像编辑失败：" + e.getMessage());
        }
    }

    private BocloudResult imageProcess(String imageId) {
        Image image = null;
        try {
            DescribeImagesRequest describeImagesRequest = new DescribeImagesRequest();
            describeImagesRequest.setImageId(imageId);
            Boolean createResult = false;
            String message = "查询失败或超时！";
            Integer flag = 10 * 10; // 5 min
            while (flag > 0) {
                DescribeImagesResult describeImages = getKecClient().describeImages(describeImagesRequest);
                List<Image> imagesSet = describeImages.getImagesSet();
                if (imagesSet.size() > 0) {
                    image = imagesSet.get(0);
                }
                if (null != image && "active".equalsIgnoreCase(image.getImageState())) {
                    flag = 0;
                    createResult = true;
                    message = "查询镜像成功！";
                    break;
                } else {
                    try {
                        Thread.sleep(6000);
                    } catch (Exception e) {
                        logger.error("线程睡觉异常 : " + e);
                    }
                    flag--;
                }
            }
            return new BocloudResult(createResult, image, message);
        } catch (Exception e) {
            logger.error("查询镜像失败：" + e.getMessage());
            return new BocloudResult(false, "查询镜像失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        try {
            CreateImageRequest createImageRequest = new CreateImageRequest();
            createImageRequest.setInstanceId(imageModel.getInstanceId());
            createImageRequest.setName(imageModel.getName());
            CreateImageResult createImage = getKecClient().createImage(createImageRequest);
            String imageId = createImage.getImageId();
            BocloudResult imageProcess = imageProcess(imageId);
            Image image = (Image) imageProcess.getData();
            ImageModel convertModel = imageConvertor.convertModel(image);
            convertModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, convertModel, "镜像创建成功！");
        } catch (Exception e) {
            logger.error("镜像创建失败：" + e.getMessage());
            return new BocloudResult(false, "镜像创建失败：" + e.getMessage());
        }
    }

}
