package com.bocloud.cmp.provider.compute;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.kingconvertor.SnapshotConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.kec.model.DeleteLocalVolumeSnapshotRequest;
import com.ksc.kec.model.DeleteLocalVolumeSnapshotResult;
import com.ksc.kec.model.DescribeLocalVolumeSnapshotsRequest;
import com.ksc.kec.model.DescribeLocalVolumeSnapshotsResult;
import com.ksc.kec.model.LocalVolumeSnapshot;
import com.ksc.kec.model.RollbackLocalVolumeRequest;
import com.ksc.kec.model.RollbackLocalVolumeResult;

/**
 * 快照操作
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudSnapshotProvider extends KingCloudProvider implements SnapshotProvider {

    public KingCloudSnapshotProvider(Butler butler) {
        super(butler);
        // TODO Auto-generated constructor stub
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudSnapshotProvider.class);
    private BeanConvertor<SnapshotModel, LocalVolumeSnapshot> snapshotConvertor = new SnapshotConvertor();

    @Override
    public BocloudResult list() {
        try {
            DescribeLocalVolumeSnapshotsRequest describeLocalVolumeSnapshotsRequest = new DescribeLocalVolumeSnapshotsRequest();
            DescribeLocalVolumeSnapshotsResult describeLocalVolumeSnapshots = getKecClient()
                    .describeLocalVolumeSnapshots(describeLocalVolumeSnapshotsRequest);
            List<LocalVolumeSnapshot> localVolumeSnapshotSet = describeLocalVolumeSnapshots.getLocalVolumeSnapshotSet();
            return new BocloudResult(true, snapshotConvertor.convertList(localVolumeSnapshotSet), "查询快照列表成功！");
        } catch (Exception e) {
            logger.error("查询快照列表失败：" + e.getMessage());
            return new BocloudResult(false, "查询快照列表失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            DeleteLocalVolumeSnapshotRequest deleteLocalVolumeSnapshotRequest = new DeleteLocalVolumeSnapshotRequest();
            deleteLocalVolumeSnapshotRequest.withLocalVolumeSnapshotIds(new String[] { model.getSnapshotUuid() });
            DeleteLocalVolumeSnapshotResult deleteLocalVolumeSnapshot = getKecClient()
                    .deleteLocalVolumeSnapshot(deleteLocalVolumeSnapshotRequest);
            return new BocloudResult(true, deleteLocalVolumeSnapshot.getReturnSet(), "快照删除成功！");
        } catch (Exception e) {
            logger.error("快照删除失败：" + e.getMessage());
            return new BocloudResult(false, "快照删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            RollbackLocalVolumeRequest rollbackLocalVolumeRequest = new RollbackLocalVolumeRequest();
            rollbackLocalVolumeRequest.setLocalVolumeId(model.getVolumeUuid());
            rollbackLocalVolumeRequest.setLocalVolumeSnapshotId(model.getSnapshotUuid());
            RollbackLocalVolumeResult rollbackLocalVolume = getKecClient()
                    .rollbackLocalVolume(rollbackLocalVolumeRequest);
            return new BocloudResult(true, rollbackLocalVolume.getReturn(), "快照回滚成功！");
        } catch (Exception e) {
            logger.error("快照回滚失败：" + e.getMessage());
            return new BocloudResult(false, "快照回滚失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        // TODO Auto-generated method stub
        return null;
    }

}
