package com.bocloud.cmp.provider.network;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.kingconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.network.vpc.model.SecurityGroups.CreateSecurityGroupRequest;
import com.ksc.network.vpc.model.SecurityGroups.CreateSecurityGroupResult;
import com.ksc.network.vpc.model.SecurityGroups.DeleteSecurityGroupRequest;
import com.ksc.network.vpc.model.SecurityGroups.DeleteSecurityGroupResult;
import com.ksc.network.vpc.model.SecurityGroups.ModifySecurityGroupRequest;
import com.ksc.network.vpc.model.SecurityGroups.ModifySecurityGroupResult;
import com.ksc.network.vpc.model.SecurityGroups.SecurityGroup;

/**
 * 安全组
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudSecuGroupProvider extends KingCloudProvider implements SecurityGroupProvider {

    public KingCloudSecuGroupProvider(Butler butler) {
        super(butler);
        // TODO Auto-generated constructor stub
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudSecuGroupProvider.class);
    private BeanConvertor<SecurityGroupModel, SecurityGroup> securityGroupConvertor = new SecurityGroupConvertor();

    @Override
    public BocloudResult list() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(SecurityGroupModel group) {
        try {
            CreateSecurityGroupRequest createSecurityGroupRequest = new CreateSecurityGroupRequest();
            createSecurityGroupRequest.setDescription(group.getRemark());
            createSecurityGroupRequest.setSecurityGroupName(group.getName());
            createSecurityGroupRequest.setVpcId(group.getVpcUuid());
            CreateSecurityGroupResult createSecurityGroup = getVpcClient()
                    .createSecurityGroup(createSecurityGroupRequest);
            return new BocloudResult(true, securityGroupConvertor.convertModel(createSecurityGroup.getSecurityGroup()),
                    "新建安全组成功！");
        } catch (Exception e) {
            logger.error("新建安全组失败：" + e.getMessage());
            return new BocloudResult(false, "新建安全组失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        try {
            DeleteSecurityGroupRequest deleteSecurityGroupRequest = new DeleteSecurityGroupRequest();
            deleteSecurityGroupRequest.setSecurityGroupId(sgId);
            DeleteSecurityGroupResult deleteSecurityGroup = getVpcClient()
                    .deleteSecurityGroup(deleteSecurityGroupRequest);
            return new BocloudResult(true, deleteSecurityGroup.getReturn(), "删除安全组成功！");
        } catch (Exception e) {
            logger.error("删除安全组失败：" + e.getMessage());
            return new BocloudResult(false, "删除安全组失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        try {
            ModifySecurityGroupRequest modifySecurityGroupRequest = new ModifySecurityGroupRequest();
            modifySecurityGroupRequest.setSecurityGroupId(model.getGroupUuid());
            modifySecurityGroupRequest.setSecurityGroupName(model.getName());
            ModifySecurityGroupResult modifySecurityGroup = getVpcClient()
                    .modifySecurityGroup(modifySecurityGroupRequest);
            return new BocloudResult(true, modifySecurityGroup.getSecurityGroupName(), "编辑安全组成功！");
        } catch (Exception e) {
            logger.error("编辑安全组失败：" + e.getMessage());
            return new BocloudResult(false, "编辑安全组失败：" + e.getMessage());
        }
    }

}
