package com.bocloud.cmp.provider.network;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.kingconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.network.vpc.model.SecurityGroups.AuthorizeSecurityGroupEntryRequest;
import com.ksc.network.vpc.model.SecurityGroups.AuthorizeSecurityGroupEntryResult;
import com.ksc.network.vpc.model.SecurityGroups.DescribeSecurityGroupsRequest;
import com.ksc.network.vpc.model.SecurityGroups.DescribeSecurityGroupsResult;
import com.ksc.network.vpc.model.SecurityGroups.SecurityGroup;

/**
 * 安全组规则
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudSecuGroupRuleProvider extends KingCloudProvider implements SecurityGroupRuleProvider {

	public KingCloudSecuGroupRuleProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(KingCloudSecuGroupRuleProvider.class);
	private BeanConvertor<SecurityGroupModel, SecurityGroup> securityGroupConvertor = new SecurityGroupConvertor();

	@Override
	public BocloudResult create(SecurityGroupRuleModel model) {
		try {

			AuthorizeSecurityGroupEntryRequest authorizeSecurityGroupEntryRequest = new AuthorizeSecurityGroupEntryRequest();
			authorizeSecurityGroupEntryRequest.setProtocol(model.getProtocol());
			authorizeSecurityGroupEntryRequest.setCidrBlock(model.getRemoteIpPrefix());
			authorizeSecurityGroupEntryRequest.setDescription(model.getRemark());
			/**
			 * 安全组规则方向，in为入站规则，out为出站规则
			 */
			authorizeSecurityGroupEntryRequest.setDirection(convertDirection(model.getDirection()));
			if ("icmp".equalsIgnoreCase(model.getProtocol())) {
				/**
				 * ICMP协议，ICMP类型，只有协议为ICMP类型，才必填 -1全部协议
				 */
				authorizeSecurityGroupEntryRequest.setIcmpCode(-1);
				authorizeSecurityGroupEntryRequest.setIcmpType(-1);
			} else {
				authorizeSecurityGroupEntryRequest.setPortRangeFrom(model.getPortMin());
				authorizeSecurityGroupEntryRequest.setPortRangeTo(model.getPortMax());
			}
			authorizeSecurityGroupEntryRequest.setSecurityGroupId(model.getGroupUuid());
			AuthorizeSecurityGroupEntryResult authorizeSecurityGroupEntry = getVpcClient()
					.authorizeSecurityGroupEntry(authorizeSecurityGroupEntryRequest);
			if (null != authorizeSecurityGroupEntry && authorizeSecurityGroupEntry.getReturn()) {
				DescribeSecurityGroupsRequest describeSecurityGroupsRequest = new DescribeSecurityGroupsRequest();
				Collection<String> securityGroupIds = Arrays.asList(new String[] { model.getGroupUuid() });
				describeSecurityGroupsRequest.setSecurityGroupIds(securityGroupIds);
				DescribeSecurityGroupsResult describeSecurityGroups = getVpcClient()
						.describeSecurityGroups(describeSecurityGroupsRequest);
				List<SecurityGroup> securityGroupSet = describeSecurityGroups.getSecurityGroupSet();
				return new BocloudResult(true, securityGroupConvertor.convertList(securityGroupSet), "新建安全组规则成功！");
			}
			return new BocloudResult(false, "规则添加失败！");
		} catch (Exception e) {
			logger.error("新建安全组规则失败：" + e.getMessage());
			return new BocloudResult(false, "新建安全组规则失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String ruleId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
		// TODO Auto-generated method stub
		return null;
	}

	private String convertDirection(String direction) {
		if ("ingress".equalsIgnoreCase(direction)) {
			return "in";
		}
		if ("egress".equalsIgnoreCase(direction)) {
			return "out";
		}
		return "ingress";
	}

}
