package com.bocloud.cmp.provider.network;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.kingconvertor.SubnetConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.network.vpc.model.subnet.CreateSubnetRequest;
import com.ksc.network.vpc.model.subnet.CreateSubnetResult;
import com.ksc.network.vpc.model.subnet.DeleteSubnetRequest;
import com.ksc.network.vpc.model.subnet.DeleteSubnetResult;
import com.ksc.network.vpc.model.subnet.ModifySubnetRequest;
import com.ksc.network.vpc.model.subnet.ModifySubnetResult;
import com.ksc.network.vpc.model.subnet.Subnet;

/**
 * 子网操作
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudSubnetProvider extends KingCloudProvider implements SubnetProvider {

    public KingCloudSubnetProvider(Butler butler) {
        super(butler);
    }

    private BeanConvertor<SubnetModel, Subnet> subnetConvertor = new SubnetConvertor();
    private static Logger logger = LoggerFactory.getLogger(KingCloudSubnetProvider.class);

    @Override
    public BocloudResult list() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        try {
            CreateSubnetRequest createSubnetRequest = new CreateSubnetRequest();
            createSubnetRequest.setAvailabilityZone(subnet.getZone());
            createSubnetRequest.setCidrBlock(subnet.getCidr());
            createSubnetRequest.setVpcId(subnet.getVpcUuid());
            if (null != subnet.getDnsNames()) {
                List<String> dnsList = JSONObject.parseArray(subnet.getDnsNames(), String.class);
                if (dnsList.size() > 0) {
                    createSubnetRequest.setDns1(dnsList.get(0));
                }
                if (dnsList.size() > 1) {
                    createSubnetRequest.setDns2(dnsList.get(1));
                }
            }
            if (null != subnet.getGateway()) {
                createSubnetRequest.setGatewayIp(subnet.getGateway());
            }
            if (null != subnet.getIpPools()) {
                List<String> ipPoolsList = JSONObject.parseArray(subnet.getIpPools(), String.class);
                if (null != ipPoolsList && ipPoolsList.size() > 0) {
                    String[] split = ipPoolsList.get(0).split("-");
                    createSubnetRequest.setDhcpIpFrom(split[0]);
                    createSubnetRequest.setDhcpIpTo(split[1]);
                }
            }
            createSubnetRequest.setSubnetName(subnet.getName());
            /**
             * 子网的类型，终端子网（Reserve）、普通子网（Normal）
             */
            createSubnetRequest.setSubnetType("Reserve");
            CreateSubnetResult createSubnet = getVpcClient().createSubnet(createSubnetRequest);
            return new BocloudResult(true, subnetConvertor.convertModel(createSubnet.getSubnet()), "新建子网成功！");
        } catch (Exception e) {
            logger.error("新建子网失败：" + e.getMessage());
            return new BocloudResult(false, "新建子网失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SubnetModel subnet) {
        try {
            ModifySubnetRequest modifySubnetRequest = new ModifySubnetRequest();
            if (null != subnet.getDnsNames()) {
                List<String> dnsList = JSONObject.parseArray(subnet.getDnsNames(), String.class);
                if (dnsList.size() > 0) {
                    modifySubnetRequest.setDns1(dnsList.get(0));
                }
                if (dnsList.size() > 1) {
                    modifySubnetRequest.setDns2(dnsList.get(1));
                }
            }
            modifySubnetRequest.setSubnetId(subnet.getSubnetUuid());
            modifySubnetRequest.setSubnetName(subnet.getName());
            ModifySubnetResult modifySubnet = getVpcClient().modifySubnet(modifySubnetRequest);
            return new BocloudResult(true, subnetConvertor.convertModel(modifySubnet.getSubnet()), "编辑子网成功！");
        } catch (Exception e) {
            logger.error("编辑子网失败：" + e.getMessage());
            return new BocloudResult(false, "编辑子网失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            DeleteSubnetRequest deleteSubnetRequest = new DeleteSubnetRequest();
            deleteSubnetRequest.setSubnetId(subnetId);
            DeleteSubnetResult deleteSubnet = getVpcClient().deleteSubnet(deleteSubnetRequest);
            return new BocloudResult(true, deleteSubnet.getReturn(), "删除子网成功！");
        } catch (Exception e) {
            logger.error("删除子网失败：" + e.getMessage());
            return new BocloudResult(false, "删除子网失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        // TODO Auto-generated method stub
        return null;
    }

}
