package com.bocloud.cmp.provider.network;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.kingconvertor.VpcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.network.vpc.model.vpc.CreateVpcRequest;
import com.ksc.network.vpc.model.vpc.CreateVpcResult;
import com.ksc.network.vpc.model.vpc.DeleteVpcRequest;
import com.ksc.network.vpc.model.vpc.DeleteVpcResult;
import com.ksc.network.vpc.model.vpc.Vpc;

/**
 * king cloud vpc
 * 
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudVpcProvider extends KingCloudProvider implements VpcProvider {

    public KingCloudVpcProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(KingCloudVpcProvider.class);
    private BeanConvertor<VpcModel, Vpc> vpcConvertor = new VpcConvertor();

    @Override
    public BocloudResult create(VpcModel model) {
        try {
            CreateVpcRequest createVpcRequest = new CreateVpcRequest();
            createVpcRequest.setCidrBlock(model.getCidr());
            if (null != model.getDefaultVpc()) {
                createVpcRequest.setDefault(model.getDefaultVpc());
            }
            createVpcRequest.setVpcName(model.getName());
            CreateVpcResult createVpc = getVpcClient().createVpc(createVpcRequest);
            return new BocloudResult(true, vpcConvertor.convertModel(createVpc.getVpc()), "新建VPC成功！");
        } catch (Exception e) {
            logger.error("新建VPC失败：" + e.getMessage());
            return new BocloudResult(false, "新建VPC失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String vpcId) {
        try {
            DeleteVpcRequest deleteVpcRequest = new DeleteVpcRequest();
            deleteVpcRequest.setVpcId(vpcId);
            DeleteVpcResult deleteVpc = getVpcClient().deleteVpc(deleteVpcRequest);
            return new BocloudResult(true, deleteVpc.getReturn(), "删除VPC成功！");
        } catch (Exception e) {
            logger.error("删除VPC失败：" + e.getMessage());
            return new BocloudResult(false, "删除VPC失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult update(VpcModel model) {
        // TODO Auto-generated method stub
        return null;
    }

}
