package com.bocloud.cmp.provider.security;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.kingconvertor.KeypairConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.KingCloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.util.KingCloudConstants;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.ksc.HttpMethod;

/**
 * 秘钥
 * 
 * @author ymh
 * @时间 2019年9月25日
 */
@Provider("bocloud.cmp.kingcloud")
public class KingCloudKeypairProvider extends KingCloudProvider implements KeypairProvider {

    public KingCloudKeypairProvider(Butler butler) {
        super(butler);
        // TODO Auto-generated constructor stub
    }

    private static final String ENDPOIN = KingCloudConstants.SKS_ENDPOINT;
    private static final String SERVICE_NAME = KingCloudConstants.SKS;
    private static final String VERSION = KingCloudConstants.VERSION_20151101;
    private BeanConvertor<KeypairModel, JSONObject> keypairConvertor = new KeypairConvertor();
    private static Logger logger = LoggerFactory.getLogger(KingCloudKeypairProvider.class);

    @Override
    public BocloudResult list() {
        try {
            String action = "DescribeKeys";
            Map<String, String> parameters = new HashMap<String, String>();
            String call = call(ENDPOIN, parameters, action, VERSION, SERVICE_NAME, HttpMethod.GET);
            if (null == call) {
                return new BocloudResult(false, "查询秘钥失败！");
            }
            JSONObject keypariResult = JSONObject.parseObject(call);
            String keySets = keypariResult.getString("KeySet");
            List<JSONObject> keys = JSONObject.parseArray(keySets, JSONObject.class);
            List<KeypairModel> convertList = keypairConvertor.convertList(keys);
            return new BocloudResult(true, convertList, "查询秘钥成功！");
        } catch (Exception e) {
            logger.error("查询秘钥列表失败：" + e.getMessage());
            return new BocloudResult(false, "查询秘钥列表失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult delete(String keypairName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        // TODO Auto-generated method stub
        return null;
    }

}
