package com.bocloud.cmp.manageoneconvertor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FirewallRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

/**
 * 防火墙转换类
 * 
 * @author admin
 *
 */
public class FirewallRuleConvertor implements BeanConvertor<FirewallRuleModel, JSONObject> {

	@Override
	public List<FirewallRuleModel> convertList(List<? extends JSONObject> models) {
		List<FirewallRuleModel> flavorModels = new ArrayList<FirewallRuleModel>();
		for (JSONObject model : models) {
			flavorModels.add(convertModel(model));
		}
		return flavorModels;
	}

	@Override
	public FirewallRuleModel convertModel(JSONObject model) {
		FirewallRuleModel firewallRuleModel = new FirewallRuleModel();
		if (null != model) {
			firewallRuleModel.setRuleUuid(model.getString("id"));
			firewallRuleModel.setPosition(model.getInteger("position"));
			String action = model.getString("action");
			if (!StringUtils.isEmpty(action)) {
				action = action.toUpperCase();
			}
			firewallRuleModel.setAction(action);
			firewallRuleModel.setName(model.getString("name"));
			firewallRuleModel.setIpVersion(model.getString("ip_version"));
			firewallRuleModel.setEnabled(model.getBoolean("enabled"));
			firewallRuleModel.setTenantUuid(model.getString("tenant_id"));
			firewallRuleModel.setShared(model.getBoolean("shared"));
			firewallRuleModel.setPolicyUuid(model.getString("firewall_policy_id"));
			firewallRuleModel.setRemark(model.getString("description"));
			String protocol = model.getString("protocol");
			if (StringUtils.isEmpty(protocol)) {
				protocol = "ANY";
			}
			firewallRuleModel.setProtocol(protocol.toUpperCase());
			List<String> sourceIpAddresses = JSON.parseArray(model.getString("source_ip_addresses"), String.class);
			if (!ListTool.isEmpty(sourceIpAddresses)) {
				firewallRuleModel.setSourceIp(JSON.toJSONString(sourceIpAddresses));
			}
			List<String> sourcePorts = JSON.parseArray(model.getString("source_ports"), String.class);
			if (!ListTool.isEmpty(sourcePorts)) {
				String sourcePort = sourcePorts.get(0);
				if (null != sourcePort && sourcePort.contains(":")) {
					sourcePort = sourcePort.replaceAll(":", "-");
				}
				firewallRuleModel.setSourcePort(sourcePort);
			}
			List<String> destinationIpAddresses = JSON.parseArray(model.getString("destination_ip_addresses"),
					String.class);
			if (!ListTool.isEmpty(destinationIpAddresses)) {
				firewallRuleModel.setDestIp(JSON.toJSONString(destinationIpAddresses));
			}
			List<String> destinationPorts = JSON.parseArray(model.getString("destination_ports"), String.class);
			if (!ListTool.isEmpty(destinationPorts)) {
				String destinationPort = destinationPorts.get(0);
				if (null != destinationPort && destinationPort.contains(":")) {
					destinationPort = destinationPort.replaceAll(":", "-");
				}
				firewallRuleModel.setDestPort(destinationPort);
			}
		}
		return firewallRuleModel;
	}
}
