package com.bocloud.cmp.manageoneconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @Date: Created in 2018/9/19
 * @Author: guopeng
 * @Description:
 */
public class FlavorConvertor implements BeanConvertor<FlavorModel, JSONObject> {

	@Override
	public List<FlavorModel> convertList(List<? extends JSONObject> models) {
		List<FlavorModel> flavorModels = new ArrayList<FlavorModel>();
		for (JSONObject model : models) {
			Integer disk = model.getInteger("disk");
			if (null != disk && disk > 0) {
				continue;
			}
			JSONObject osExtraSpecs = model.getJSONObject("os_extra_specs");
			if (null == osExtraSpecs) {
				continue;
			}
			String cpuInfo = osExtraSpecs.getString("capabilities:cpu_info:arch");
			if (!"x86_64".equalsIgnoreCase(cpuInfo)) {
				continue;
			}
			String cpuInfoVendor = osExtraSpecs.getString("capabilities:cpu_info:vendor");
			if (!"Intel".equalsIgnoreCase(cpuInfoVendor)) {
				continue;
			}
			String extBootType = osExtraSpecs.getString("huawei:extBootType");
			if (!"Volume".equalsIgnoreCase(extBootType)) {
				continue;
			}
			flavorModels.add(convertModel(model));
		}
		return flavorModels;
	}

	@Override
	public FlavorModel convertModel(JSONObject model) {
		FlavorModel flavorModel = new FlavorModel();
		flavorModel.setName(model.getString("name"));
		flavorModel.setCpu(model.getInteger("vcpus"));
		flavorModel.setDisk(model.getInteger("disk"));
		flavorModel.setFlavorUuid(model.getString("id"));
		flavorModel.setMemory(BigDecimal.valueOf(model.getInteger("ram")).divide(BigDecimal.valueOf(1024), 2,
				BigDecimal.ROUND_HALF_UP));
		flavorModel.setUuid(model.getString("id"));
		JSONObject osExtraSpecs = model.getJSONObject("os_extra_specs");
		if (null != osExtraSpecs) {
			flavorModel.setOsExtraSpecs(osExtraSpecs.toJSONString());
		}

		return flavorModel;
	}
}
