package com.bocloud.cmp.manageoneconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 宿主机
 * 
 * @author Administrator
 */
public class HostConvertor implements BeanConvertor<HostModel, JSONObject> {

	@Override
	public List<HostModel> convertList(List<? extends JSONObject> models) {
		List<HostModel> hostModels = new ArrayList<HostModel>();
		for (JSONObject model : models) {
			hostModels.add(convertModel(model));
		}
		return hostModels;
	}

	@Override
	public HostModel convertModel(JSONObject model) {
		HostModel hostModel = new HostModel();
		if (null != model) {
			hostModel.setHostIp(model.getString("ipAddress"));
			hostModel.setName(model.getString("name"));
			String status = model.getString("status");
			if(null != status && "normal".equalsIgnoreCase(status)) {
				status = "RUNNING";
			} else {
				status = "STOPPED";
			}
			hostModel.setStatus(status);
			hostModel.setPowerState(status);
			hostModel.setCategory(model.getString("category"));
			hostModel.setUuid(model.getString("id"));

			hostModel.setCpuTotal(model.getLong("totalVcpuCores"));
			hostModel.setCpuUsed(model.getLong("allocatedVcpuCores"));

			double diskTotal = new BigDecimal(model.getLong("totalDiskSizeMB"))
					.divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setDiskTotal(diskTotal);

			double diskUsed = new BigDecimal(model.getLong("allocatedDiskSizeMB"))
					.divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setDiskUsed(diskUsed);

			double memory = new BigDecimal(model.getLong("totalVmemoryMB"))
					.divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setMemTotal(memory);

			double memUsed = new BigDecimal(model.getLong("allocatedVmemoryMB"))
					.divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setMemUsed(memUsed);
		}
		return hostModel;
	}
}
