package com.bocloud.cmp.manageoneconvertor;

import java.math.BigDecimal;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.google.common.collect.Lists;

/**
 * 宿主机转换
 * 
 * @author admin
 *
 */
public class HypervisorConvertor implements BeanConvertor<HostModel, JSONObject> {

	@Override
	public List<HostModel> convertList(List<? extends JSONObject> hyperVs) {
		List<HostModel> hostModels = Lists.newArrayList();
		for (JSONObject hyperv : hyperVs) {
			hostModels.add(convertModel(hyperv));
		}
		return hostModels;
	}

	@Override
	public HostModel convertModel(JSONObject object) {
		HostModel hostModel = new HostModel();
		if (null != object) {
			hostModel.setCatalog("ManageOne");
			hostModel.setUuid(object.getString("id"));
			hostModel.setName(object.getString("hypervisor_hostname"));
			hostModel.setHostName(object.getString("hypervisor_hostname"));
			hostModel.setHostIp(object.getString("host_ip"));
			hostModel.setPowerState(object.getString("state"));
			hostModel.setStatus(object.getString("status"));
			hostModel.setVmNum(object.getInteger("running_vms"));

			hostModel.setCpuUsed(object.getLong("vcpus_used"));
			hostModel.setCpuTotal(object.getLong("vcpus"));

			double memory = new BigDecimal(object.getLongValue("memory_mb")).divide(BigDecimal.valueOf(1024), 2,
					BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setMemTotal(memory);
			double memUsed = new BigDecimal(object.getLongValue("memory_mb_used")).divide(BigDecimal.valueOf(1024), 2,
					BigDecimal.ROUND_HALF_UP).doubleValue();
			hostModel.setMemUsed(memUsed);

			hostModel.setDiskTotal(object.getDouble("local_gb"));
			hostModel.setDiskUsed(object.getDouble("local_gb_used"));

			String cpuInfo = object.getString("cpu_info");
			if (!StringUtils.isEmpty(cpuInfo)) {
				JSONObject parseObject = JSONObject.parseObject(cpuInfo, JSONObject.class);
				hostModel.setCpuModel(parseObject.getString("model"));
				hostModel.setModelName(parseObject.getString("model_name"));
			}
		}
		return hostModel;
	}

}