package com.bocloud.cmp.manageoneconvertor;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

/**
 * 华为manage one镜像转换器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2018年9月18日
 *
 */
public class ImageConvertor implements BeanConvertor<ImageModel, JSONObject> {

	private static final Logger logger = LoggerFactory.getLogger(ImageConvertor.class);

	@Override
	public List<ImageModel> convertList(List<? extends JSONObject> objects) {
		List<ImageModel> imageModels = new ArrayList<ImageModel>();
		for (JSONObject object : objects) {
			String architecture = object.getString("architecture");
			Integer minRam = object.getInteger("min_ram") == null ? 0 : object.getInteger("min_ram");
			// 目前只获取x86镜像
			if ("x86_64".equalsIgnoreCase(architecture) && minRam <= 0) {
				imageModels.add(convertModel(object));
			}
		}
		return imageModels;
	}

	@Override
	public ImageModel convertModel(JSONObject object) {
		ImageModel model = new ImageModel();
		model.setName(object.getString("name"));
		model.setStatus(null == object.getString("status") ? "ERROR" : object.getString("status").toUpperCase());
		model.setRemark(object.getString("__description"));
		SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
		try {
			model.setGmtCreate(format.parse(object.getString("created_at")));
		} catch (ParseException e) {
			logger.error("Get error message：", e);
			model.setGmtCreate(new Date());
		}
		model.setOsCategory(object.getString("__platform"));
		model.setOsName(object.getString("__os_version"));
		model.setOsType(object.getString("__os_type"));
		model.setImageUuid(object.getString("id"));
		model.setArchitecture(object.getString("architecture"));
		model.setImageType(object.getString("visibility"));
		Double size = object.getDouble("size");
		if (null != size) {
			size = new BigDecimal(size).divide(new BigDecimal(1024 * 1024 * 1024L), 2, RoundingMode.HALF_UP)
					.doubleValue();
		}
		model.setSize(size);
		model.setMinDisk(object.getLong("min_disk"));
		model.setMinRam(BigDecimal.valueOf(object.getInteger("min_ram")).divide(BigDecimal.valueOf(1024), 2,
				BigDecimal.ROUND_HALF_UP).longValue());
		return model;
	}
}
