package com.bocloud.cmp.manageoneconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.PortFixedIpModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.utils.StatusConverter;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one端口转换器
 *
 * @author wangyu
 * @version 3.0.0-SNAPSHOT
 * @since 2018/9/18
 */
public class PortConvertor implements BeanConvertor<PortModel, JSONObject> {
	@Override
	public List<PortModel> convertList(List<? extends JSONObject> objects) {
		List<PortModel> portModels = new ArrayList<PortModel>();
		for (JSONObject object : objects) {
			portModels.add(convertModel(object));
		}
		return portModels;
	}

	@Override
	public PortModel convertModel(JSONObject object) {
		PortModel model = new PortModel();
		String name = object.getString("name");
		if (StringUtils.isEmpty(name)) {
			name = object.getString("id");
		}
		model.setTenantUuid(object.getString("tenant_id"));
		model.setName(name);
		model.setPortUuid(object.getString("id"));
		model.setStatus(StatusConverter.Network.converter(object.getString("status")));
		model.setDeviceOwner(object.getString("device_owner"));
		model.setDeviceUuid(object.getString("device_id"));
		model.setNetworkUuid(object.getString("network_id"));
		model.setMacAddress(object.getString("mac_address"));
		List<JSONObject> fixedIps = JSONArray.parseArray(object.getString("fixed_ips"), JSONObject.class);
		List<PortFixedIpModel> fixedIpModels = new ArrayList<PortFixedIpModel>();
		if (!ListTool.isEmpty(fixedIps)) {
			for (JSONObject fixedIp : fixedIps) {
				PortFixedIpModel fixedIpModel = new PortFixedIpModel();
				fixedIpModel.setIpAddress(fixedIp.getString("ip_address"));
				fixedIpModel.setSubnetUuid(fixedIp.getString("subnet_id"));
				fixedIpModels.add(fixedIpModel);
			}
		}
		model.setPortFixedIps(fixedIpModels);
		return model;
	}
}
