package com.bocloud.cmp.manageoneconvertor;

import java.util.ArrayList;
import java.util.List;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, JSONObject> {

	@Override
	public List<SecurityGroupModel> convertList(List<? extends JSONObject> objects) {
		List<SecurityGroupModel> volumeModels = new ArrayList<SecurityGroupModel>();
		for (JSONObject object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public SecurityGroupModel convertModel(JSONObject json) {
		SecurityGroupModel model = new SecurityGroupModel();
		model.setGroupUuid(json.getString("id"));
		model.setName(json.getString("name"));
		model.setRemark(json.getString("description"));
		model.setTenantUuid(json.getString("tenant_id"));
		JSONArray jsonArray = json.getJSONArray("security_group_rules");
		List<SecurityGroupRuleModel> models = new ArrayList<SecurityGroupRuleModel>();
		for (Object object : jsonArray) {
			JSONObject jsonObject = JSONObject.parseObject(object.toString(), JSONObject.class);
			SecurityGroupRuleModel ruleModel = this.convertSecurityGroupRuleModel(jsonObject);
			models.add(ruleModel);
		}
		model.setModels(models);
		return model;
	}

	private SecurityGroupRuleModel convertSecurityGroupRuleModel(JSONObject groupRule) {
		SecurityGroupRuleModel rule = new SecurityGroupRuleModel();
		rule.setName(groupRule.getString("tenant_id"));
		rule.setPortMax(groupRule.getInteger("port_range_max"));
		rule.setPortMin(groupRule.getInteger("port_range_min"));
		if (null != groupRule.getString("protocol")) {
			rule.setProtocol(groupRule.getString("protocol").toUpperCase());
		}
		rule.setRuleUuid(groupRule.getString("id"));
		rule.setGroupUuid(groupRule.getString("security_group_id"));
		rule.setDirection(groupRule.getString("direction"));
		rule.setRemoteIpPrefix(groupRule.getString("remote_ip_prefix"));
		rule.setEtherType(groupRule.getString("ethertype"));
		rule.setRemoteGroupUuid(groupRule.getString("remote_group_id"));
		return rule;
	}

}
