package com.bocloud.cmp.manageoneconvertor;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

public class SnapshotConvertor implements BeanConvertor<SnapshotModel, JSONObject> {

	private static final Logger logger = LoggerFactory.getLogger(SnapshotConvertor.class);

	@Override
	public List<SnapshotModel> convertList(List<? extends JSONObject> objects) {
		List<SnapshotModel> snapshotModels = new ArrayList<SnapshotModel>();
		for (JSONObject object : objects) {
			snapshotModels.add(convertModel(object));
		}
		return snapshotModels;
	}

	@Override
	public SnapshotModel convertModel(JSONObject jsonObject) {
		SnapshotModel snapshot = new SnapshotModel();
		snapshot.setSize(jsonObject.getInteger("size"));
		snapshot.setSnapshotUuid(jsonObject.getString("id"));
		snapshot.setName(jsonObject.getString("name"));
		String status = jsonObject.getString("status");
		if (!StringUtils.isEmpty(status) && "error".equalsIgnoreCase(status)) {
			status = "EXCEPTION";
		}
		snapshot.setStatus(status.toUpperCase());
		snapshot.setRemark(jsonObject.getString("description"));
		snapshot.setVolumeUuid(jsonObject.getString("volume_id"));
		try {
			// 根据具体的时间数值来看，FusionCloud返回给我的时间应该是UTC时间，比东8区早了8小时。
			// 但是从具体的时间格式来看它却没有注明时区，所以这里if判断下如果确定没有时区就给加上时区
			String createAt = jsonObject.getString("created_at");
			if (!createAt.contains("+")) {
				createAt += "+0000";
			}
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSSSSZ");
			snapshot.setCreateTime(format.parse(createAt));
		} catch (ParseException e) {
			logger.error("Get error message：", e);
			snapshot.setCreateTime(new Date());
		}

		return snapshot;
	}

}
