package com.bocloud.cmp.manageoneconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

public class SubnetConvertor implements BeanConvertor<SubnetModel, JSONObject> {

	@Override
	public List<SubnetModel> convertList(List<? extends JSONObject> objects) {
		List<SubnetModel> subnetModels = new ArrayList<SubnetModel>();
		for (JSONObject object : objects) {
			subnetModels.add(convertModel(object));
		}
		return subnetModels;
	}

	@Override
	public SubnetModel convertModel(JSONObject model) {
		SubnetModel subnet = new SubnetModel();
		String networkId = model.getString("network_id");
		if (StringUtils.isEmpty(model.getString("name"))) {
			subnet.setName(networkId);
		} else {
			subnet.setName(model.getString("name"));
		}
		subnet.setNetworkUuid(networkId);
		subnet.setCidr(model.getString("cidr"));
		subnet.setDhcpEnabled(model.getBoolean("enable_dhcp"));
		subnet.setSubnetUuid(model.getString("id"));
		subnet.setIpVersion(model.getString("ip_version"));
		subnet.setGateway(model.getString("gateway_ip"));
		subnet.setHostRoutes(model.getString("host_routes")); // 类型为数组类型
		List<String> ipPool = new ArrayList<String>();
		String ipPools = model.getString("allocation_pools");
		if (!StringUtils.isEmpty(ipPools)) {
			List<JSONObject> pools = JSONArray.parseArray(ipPools, JSONObject.class);
			for (JSONObject pool : pools) {
				ipPool.add(pool.getString("start") + "-" + pool.getString("end"));
			}
		}
		subnet.setIpPools(JSONArray.toJSONString(ipPool));
		subnet.setDnsNames(model.getString("dns_nameservers"));
		return subnet;
	}

}
