package com.bocloud.cmp.manageoneconvertor;

import io.jsonwebtoken.lang.Assert;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.manageone.VdcProjectModel;
import com.bocloud.cmp.model.manageone.VdcRegionModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

/**
 * vdc项目数据转换器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月17日
 *
 */
public class VdcProjectConvertor implements BeanConvertor<VdcProjectModel, JSONObject> {

	@Override
	public List<VdcProjectModel> convertList(List<? extends JSONObject> models) {
		List<VdcProjectModel> projectModels = new ArrayList<>();
		for (JSONObject model : models) {
			projectModels.add(convertModel(model));
		}
		return projectModels;
	}

	@Override
	public VdcProjectModel convertModel(JSONObject model) {
		VdcProjectModel projectModel = new VdcProjectModel();
		Assert.notNull(model, "项目数据为空");
		projectModel.setProjectId(model.getString("id"));
		projectModel.setName(model.getString("name"));
		projectModel.setDisplayName(model.getString("display_name"));
		projectModel.setRemark(model.getString("description"));
		projectModel.setDomainId(model.getString("domain_id"));
		projectModel.setEnabled(model.getBoolean("enabled"));
		projectModel.setVdcTenantId(model.getString("tenant_id"));
		projectModel.setShared(model.getBoolean("is_shared"));
		projectModel.setVdcTenantName(model.getString("tenant_name"));
		projectModel.setCreateUserId(model.getString("create_user_id"));
		projectModel.setCreateUserName(model.getString("create_user_name"));
		String regions = model.getString("regions");
		if (!StringUtils.isEmpty(regions)) {
			List<JSONObject> regionList = JSONArray.parseArray(regions, JSONObject.class);
			List<VdcRegionModel> regionModels = new ArrayList<>();
			for (JSONObject region : regionList) {
				VdcRegionModel regionModel = new VdcRegionModel();
				regionModel.setRegionId(region.getString("region_id"));
				regionModel.setRegionType(region.getString("region_type"));
				regionModel.setRegionStatus(region.getString("status"));
				regionModel.setRegionNames(region.getString("region_name"));
				regionModels.add(regionModel);
			}
			projectModel.setRegions(regionModels);
		}
		return projectModel;
	}

}
