package com.bocloud.cmp.manageoneconvertor;

import io.jsonwebtoken.lang.Assert;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.manageone.VdcUserModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;

/**
 * vdc用户数据转换器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月17日
 *
 */
public class VdcUserConvertor implements BeanConvertor<VdcUserModel, JSONObject> {

	private static Logger logger = LoggerFactory.getLogger(VdcUserConvertor.class);

	@Override
	public List<VdcUserModel> convertList(List<? extends JSONObject> models) {
		List<VdcUserModel> userModels = new ArrayList<>();
		for (JSONObject model : models) {
			userModels.add(convertModel(model));
		}
		return userModels;
	}

	@Override
	public VdcUserModel convertModel(JSONObject model) {
		VdcUserModel userModel = new VdcUserModel();
		Assert.notNull(model, "用户数据为空");
		userModel.setEnabled(model.getBoolean("enabled"));
		userModel.setRemark(model.getString("description"));
		userModel.setEmail(model.getString("email"));
		userModel.setAreacode(model.getString("areacode"));
		userModel.setPhone(model.getString("phone"));
		userModel.setRoles(model.getString("roles"));
		userModel.setResourceTenantId(model.getString("resource_tenant_id"));
		userModel.setDisplayName(model.getString("display_name"));
		userModel.setVdcId(model.getString("vdc_id"));
		userModel.setDomainId(model.getString("domain_id"));
		userModel.setDomainName(model.getString("domain_name"));
		userModel.setUserType(model.getString("user_type"));
		userModel.setName(model.getString("name"));
		userModel.setUserId(model.getString("id"));
		String createAt = model.getString("create_at");
		if (!StringUtils.isEmpty(createAt)) {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.z");
			try {
				userModel.setCreateAt(format.parse(createAt));
			} catch (ParseException e) {
				logger.error("Get error message：{}", e.getMessage());
			}
		}

		return userModel;
	}

}
