package com.bocloud.cmp.manageoneconvertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class VolumeConvertor implements BeanConvertor<VolumeModel, JSONObject> {

	private static final Logger logger = LoggerFactory.getLogger(VolumeConvertor.class);

	@Override
	public List<VolumeModel> convertList(List<? extends JSONObject> objects) {
		List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
		for (JSONObject object : objects) {
			volumeModels.add(convertModel(object));
		}
		return volumeModels;
	}

	@Override
	public VolumeModel convertModel(JSONObject jsonObject) {
		VolumeModel model = new VolumeModel();
		String name = jsonObject.getString("name");
		if (StringUtils.isEmpty(name)) {
			name = "--";
		}
		model.setName(name);
		model.setVolumeUuid(jsonObject.getString("id"));

		String status = jsonObject.getString("status");
		if (!StringUtils.isEmpty(status) && status.equalsIgnoreCase("in-use")) {
			status = "IN_USE";
		} else if (!StringUtils.isEmpty(status) && status.equalsIgnoreCase("error")) {
			status = "EXCEPTION";
		}
		model.setStatus(status.toUpperCase());
		model.setSize(jsonObject.getInteger("size"));
		model.setZone(jsonObject.getString("availability_zone"));
		model.setType(jsonObject.getString("volume_type"));
		model.setVolumeType(jsonObject.getString("volume_type"));
		model.setRemark(jsonObject.getString("description"));
		model.setBootVolume(jsonObject.getBoolean("bootable"));
		model.setMultiattach(jsonObject.getBoolean("multiattach"));
		List<JSONObject> attachments = JSONArray.parseArray(jsonObject.getString("attachments"), JSONObject.class);
		List<ServerVolumeModel> models = new ArrayList<ServerVolumeModel>();
		if (!ListTool.isEmpty(attachments)) {
			for (JSONObject attachment : attachments) {
				models.add(new ServerVolumeModel(attachment.getString("server_id"), attachment.getString("volume_id"),
						attachment.getString("device")));
			}
		}
		model.setModels(models);
		JSONObject imageJson = jsonObject.getJSONObject("volume_image_metadata");
		if (null != imageJson) {
			model.setImageUuid(imageJson.getString("image_id"));
		}
		try {
			// 根据具体的时间数值来看，FusionCloud返回给我的时间应该是UTC时间，比东8区早了8小时。
			// 但是从具体的时间格式来看它却没有注明时区，所以这里if判断下如果确定没有时区就给加上时区
			String createAt = jsonObject.getString("created_at");
			if (!createAt.contains("+")) {
				createAt += "+0000";
			}
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSSSSZ");
			model.setCreateTime(format.parse(createAt));
		} catch (ParseException e) {
			logger.error("Get error message：", e);
			model.setCreateTime(new Date());
		}
		return model;
	}

}
