package com.bocloud.cmp.provider;

import io.jsonwebtoken.lang.Assert;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.model.ManageOneToken;
import com.bocloud.cmp.model.Butler;

/**
 * 华为manage one client工厂类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月16日
 */
public class ManageOneClientFactory {

	private static Logger logger = LoggerFactory.getLogger(ManageOneClientFactory.class);
	private static Map<String, ManageOneToken> tokens = new ConcurrentHashMap<>(8, 0.75f);

	/**
	 * 私有构造方法
	 */
	private ManageOneClientFactory() {
		super();
		if (null != FactoryContainer.instance) {// 防止利用反射机制实现多个对象的漏洞
			throw new RuntimeException("ClientFactory遭受反射攻击！");
		}
		tokens = new ConcurrentHashMap<>(10, 0.75f);
	}

	/**
	 * 获取配置实例标准方法
	 *
	 * @return
	 */
	public static ManageOneClientFactory getInstance() {
		return FactoryContainer.instance;
	}

	/**
	 * 获取fusionCloud可用token
	 *
	 * @param key
	 * @return
	 */
	public ManageOneToken getToken(String key) {
		Assert.hasLength(key, "");
		Assert.notNull(ButlerRepository.getInstance().get(key), "");
		// 判断当前UUID的FusionCloud认证信息是否存在于缓存中
		if (tokens.keySet().contains(key)) {// 认证信息存在于缓存
			ManageOneToken token = tokens.get(key);
			Date current = new Date();
			if (null != token && null != token.getToken()) {
				// 判断当前时间是否超过了会话超时时间，如果超时之后进行重新认证
				if (current.after(token.getExpiresAt())) {
					logger.info("Cloud Provider [{}] token is Expired! rebuild one!",
							ButlerRepository.getInstance().get(key).getName());
					return null;
				}
				return token;
			}
		}
		return null;
	}

	/**
	 * 刷新认证
	 *
	 * @param butler
	 */
	public void updateToken(Butler butler) {
		Assert.notNull(butler, "");
		if (tokens.containsKey(butler.getUuid())) {
			tokens.remove(butler.getUuid());
		}
		new ManageOneProvider(butler);
	}

	/**
	 * 刷新认证
	 *
	 * @param key
	 * @param provider
	 */
	public void refreshToken(String key, ManageOneProvider provider) {
		if (provider.accessable()) {
			tokens.put(key, provider.getToken());
		}
	}

	public void removeToken(String key) {
		tokens.remove(key);
	}

	private static class FactoryContainer {
		private static ManageOneClientFactory instance = new ManageOneClientFactory();
	}

}
