package com.bocloud.cmp.provider;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.common.model.ManageOneAuth;
import com.bocloud.cmp.common.model.ManageOneCatalog;
import com.bocloud.cmp.common.model.ManageOneDomain;
import com.bocloud.cmp.common.model.ManageOneEndpoint;
import com.bocloud.cmp.common.model.ManageOneIdentity;
import com.bocloud.cmp.common.model.ManageOnePassword;
import com.bocloud.cmp.common.model.ManageOneProject;
import com.bocloud.cmp.common.model.ManageOneScope;
import com.bocloud.cmp.common.model.ManageOneToken;
import com.bocloud.cmp.common.model.ManageOneUser;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.utils.TokenCache;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.CommonResultBuilder;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.http.ResultBuilder;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

import lombok.Data;

/**
 * @ Description： @ Author ：丁明威 @ Date ：Created in 17:07 2018/11/14 @ Modified
 * By：
 */
@Data
public class ManageOneProvider extends GenericProvider implements SuperProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneProvider.class);
	private String endpoint;// 平台认证路径
	private String username;// 管理员账号
	private String password;// 管理员密码
	private String domain;// 所属域
	private String project;// 项目ID
	private boolean accessable;
	private ManageOneToken token;// token信息
	private String error; // 错误信息
	private HttpClient client = new HttpClient(60 * 1000, PostDataFormat.RAW);
	private String monitorUrl;
	private String monitorUserName;
	private String monitorPwd;
	private String monitorToken;
	private String monitorRoaRand;
	private Butler butler;

	/**
	 * @param butler
	 */
	public ManageOneProvider(Butler butler) {
		super(butler);
		this.butler = butler;
		this.endpoint = butler.getUrl();
		this.username = butler.getUsername();
		this.password = butler.getPassword();
		this.domain = butler.getDomainName();
		this.project = butler.getProjectId();
		initToken();
	}

	private void initToken() {
		String manageOneToken = TokenCache.getKey("token." + butler.getProjectId() + "." + butler.getUuid());
		if (null == manageOneToken) {
			init();
		} else {
			this.token = JSON.parseObject(manageOneToken, ManageOneToken.class);
			this.accessable = true;
		}
	}

	@Override
	public boolean accessable() {
		return this.accessable;
	}

	protected Map<String, Object> buildMonitorHeader() {
		Map<String, Object> header = new HashMap<String, Object>();
		header.put("accept", "application/json");
		header.put("content-type", "application/json");
		header.put("X-Auth-Token", this.getMonitorToken());
		header.put("roaRand", this.getMonitorRoaRand());
		return header;
	}

	/**
	 * 初始化连接
	 */
	private void init() {
		if (paramCheck().isFailed()) {// 基本的参数校验不通过，即用户名，密码或者认证路径为空
			this.accessable = false;
			logger.error("Driver init error : {}", paramCheck().getMessage());
			this.error = paramCheck().getMessage();
			return;
		}
		String url = this.endpoint + "/v3/auth/tokens";
		ManageOneDomain domain = new ManageOneDomain(this.domain);
		ManageOneUser user = new ManageOneUser(this.username, this.password, domain);
		ManageOnePassword password = new ManageOnePassword(user);
		List<String> methods = new ArrayList<String>();
		methods.add("password");
		ManageOneIdentity identity = new ManageOneIdentity(password, methods);

		ManageOneProject project = new ManageOneProject(this.project);
		ManageOneScope scope = new ManageOneScope(project);
		ManageOneAuth auth = new ManageOneAuth(identity, scope);
		Map<String, Object> params = new HashMap<String, Object>();
		params.put("auth", auth);
		HttpPost httpPost = new HttpPost(url);
		httpPost.addHeader("accept", "application/json");
		httpPost.addHeader("content-type", "application/json");
		StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
		httpPost.setEntity(httpEntity);
		try {
			CloseableHttpResponse httpResponse = client.getHttpClient().execute(httpPost);
			Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
			if (result.isFailed()) {
				this.accessable = false;
				logger.error("Driver init error : {}", result.getMessage());
				this.error = result.getMessage();
				return;
			}
			JSONObject content = JSONObject
					.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
			logger.info("认证token: " + content.toJSONString());
			this.token = JSONObject.parseObject(content.getString("token"), ManageOneToken.class);
			if (null == this.token) {
				this.accessable = false;
				logger.error("Driver init error : {}", "Token is empty !");
				this.error = "Token is empty !";
				return;
			}
			String authToken = findToken(httpResponse);
			if (StringUtils.isEmpty(authToken)) {
				this.accessable = false;
				logger.error("Driver init error : {}", "Token is empty !");
				this.error = "Token is empty !";
				return;
			}
			token.setToken(authToken);
			TokenCache.setKey("token." + butler.getProjectId() + "." + butler.getUuid(), JSON.toJSONString(token));
			this.accessable = true;
		} catch (Exception e) {
			logger.error("Get error message：", e);
			this.accessable = false;
		}
	}

	/**
	 * 从响应头部获取会话X-Subject-Token
	 *
	 * @param httpResponse
	 * @return
	 */
	private String findToken(CloseableHttpResponse httpResponse) {
		String token = "";
		Header[] headers = httpResponse.getHeaders("X-Subject-Token");
		if (null == headers || headers.length == 0) {
			return token;
		}
		for (Header header : headers) {
			if ("X-Subject-Token".equalsIgnoreCase(header.getName())) {
				token = header.getValue();
				break;
			}
		}
		return token;
	}

	public String getUrl() { // https://ip:26335
		return this.monitorUrl;
	}

	public void initMonitorToken() {
		try {
			String url = getUrl() + "/rest/plat/smapp/v1/oauth/token";
			JSONObject params = new JSONObject();
			params.put("grantType", "password");
			params.put("userName", this.getMonitorUserName());
			params.put("value", this.getMonitorPwd());
			HttpPut httpPost = new HttpPut(url);
			httpPost.addHeader("accept", "application/json");
			httpPost.addHeader("content-type", "application/json");
			StringEntity httpEntity = new StringEntity(JSONObject.toJSON(params).toString(), "UTF-8");
			httpPost.setEntity(httpEntity);
			CloseableHttpResponse httpResponse = this.getClient().getHttpClient().execute(httpPost);
			Result result = new CommonResultBuilder().build(httpResponse.getStatusLine().getStatusCode());
			logger.info("result:{}", JSONObject.toJSON(result));
			JSONObject content = JSONObject
					.parseObject(IOUtils.toString(httpResponse.getEntity().getContent(), "utf-8"));
			this.monitorToken = content.getString("accessSession");
			this.monitorRoaRand = content.getString("roaRand");
			TokenCache.setKey("monitorToken." + butler.getProjectId() + "." + butler.getUuid(), monitorToken);
			TokenCache.setKey("monitorRoaRand." + butler.getProjectId() + "." + butler.getUuid(), monitorRoaRand);
		} catch (Exception e) {
			logger.info("初始化监控token Error : " + e);
		}
	}

	/**
	 * 基本参数信息认证，即确保认证路径，认证账号和认证密码为非空
	 *
	 * @return
	 */
	private Result paramCheck() {
		if (StringUtils.isEmpty(this.endpoint)) {
			return new Result(false, "Endpoint is Empty!");
		}
		if (StringUtils.isEmpty(this.username)) {
			return new Result(false, "Username is Empty!");
		}
		if (StringUtils.isEmpty(this.password)) {
			return new Result(false, "Password is Empty!");
		}
		if (StringUtils.isEmpty(this.domain)) {
			return new Result(false, "Domain is Empty!");
		}
		if (StringUtils.isEmpty(this.project)) {
			return new Result(false, "Project is Empty!");
		}
		return new Result(true, "success");
	}

	public Result doGet(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.get(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result listFlavors(String url, Map<String, Object> params, ManageOneApiName apiName,
			ManageOneApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				apiUrl = apiUrl.replace("v2", "v1");
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.get(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.post(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result getVncConsole(String serverId) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = readApiUrl(ManageOneApiName.ecs, ManageOneApiType.compute);
			if (StringUtils.isEmpty(apiUrl)) {
				return Result.FAILED("ecs service is unavailable");
			}
			apiUrl = apiUrl.replaceFirst("v2", "v2.1");
			Map<String, Object> header = buildHeader();
			header.put("X-Openstack-Nova-Api-Version", "2.6");
			Map<String, Object> params = new HashMap<>();
			JSONObject vnc = new JSONObject();
			vnc.put("type", "novnc");
			vnc.put("protocol", "vnc");
			params.put("remote_console", vnc);
			// /v2.1/{tenant_id}/servers/{server_id}/remote-consoles
			return this.client.post(header, params, apiUrl + "/servers/" + serverId + "/remote-consoles");
		} finally {
			this.client.close();
		}
	}

	public Result doPost(String url, String params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.post(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doPostForV1(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				apiUrl = apiUrl.replace("v2", "v1");
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.post(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doPut(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.put(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doPatch(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.patch(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doPatch(String url, String params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(PostDataFormat.RAW);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.patch(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	public Result doDelete(String url, Map<String, Object> params, ManageOneApiName apiName, ManageOneApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			return this.client.delete(buildHeader(), params, apiUrl + url);
		} finally {
			this.client.close();
		}
	}

	protected ResultBuilder resultBuilder = new CommonResultBuilder();

	/**
	 * 哎 无奈之举
	 * 
	 * @param url
	 * @param params
	 * @param apiName
	 * @param apiType
	 * @return
	 */
	public Result doDeleteWithParams(String url, Map<String, Object> params, ManageOneApiName apiName,
			ManageOneApiType apiType) {
		try {
			client.setDataFormat(null);
			String apiUrl = this.endpoint;
			if (null != apiName || null != apiType) {
				apiUrl = readApiUrl(apiName, apiType);
				if (StringUtils.isEmpty(apiUrl)) {
					return Result.FAILED(apiName.name() + ":" + apiType.name() + " service is unavailable");
				}
			}
			HttpDeleteWithBody deleteWithBody = new HttpDeleteWithBody(apiUrl + url);
			StringEntity entity = new StringEntity(JSON.toJSONString(params), "utf-8");
			deleteWithBody.setEntity(entity);
			deleteWithBody.setHeader("accept", "application/json");
			deleteWithBody.setHeader("content-type", "application/json");
			deleteWithBody.setHeader("X-Auth-Token", this.token.getToken());
			deleteWithBody.setHeader("X-Project-id", this.token.getProject().getId());
			CloseableHttpClient httpClient = client.getHttpClient();
			CloseableHttpResponse response = httpClient.execute(deleteWithBody);
			int status = response.getStatusLine().getStatusCode();
			String content = null;
			Result result = resultBuilder.build(status);
			HttpEntity httpEntity = response.getEntity();
			if (null != httpEntity) {
				content = IOUtils.toString(httpEntity.getContent(), "utf-8");
			}
			result.setMessage(content);
			return result;
		} catch (Exception e) {
			logger.error("远程接口异常：" + e.getMessage());
			return Result.FAILED("远程接口异常：" + e.getMessage());
		} finally {
			this.client.close();
		}
	}

	class HttpDeleteWithBody extends HttpEntityEnclosingRequestBase {

		public static final String METHOD_NAME = "DELETE";

		/**
		 * 获取方法（必须重载）
		 *
		 * @return
		 */
		@Override
		public String getMethod() {
			return METHOD_NAME;
		}

		public HttpDeleteWithBody(final String uri) {
			super();
			setURI(URI.create(uri));
		}

		public HttpDeleteWithBody(final URI uri) {
			super();
			setURI(uri);
		}

		public HttpDeleteWithBody() {
			super();
		}
	}

	public Result waitImageTask(String jobId) {
		try {
			client.setDataFormat(null);
			String url = readApiUrl(ManageOneApiName.ecs, ManageOneApiType.compute);
			if (!StringUtils.isEmpty(url) && url.contains("v2")) {
				url = url.substring(0, url.indexOf("v2") - 1);
			}
			return this.client.get(buildHeader(), null,
					url + "/v1/" + this.token.getProject().getId() + "/jobs/" + jobId);
		} finally {
			this.client.close();
		}
	}

	/**
	 * 获取api的网关地址
	 *
	 * @param apiName
	 * @param apiType
	 * @return
	 */
	private String readApiUrl(ManageOneApiName apiName, ManageOneApiType apiType) {
		String url = null;
		Map<String, String> apiMap = new HashMap<String, String>();
		List<ManageOneCatalog> catalogs = this.token.getCatalog();
		if (ListTool.isEmpty(catalogs)) {
			return url;
		}
		for (ManageOneCatalog catalog : catalogs) {
			List<ManageOneEndpoint> endpoints = catalog.getEndpoints();
			if (ListTool.isEmpty(endpoints)) {
				continue;
			}
			apiMap.put(catalog.getName() + ":" + catalog.getType(), endpoints.get(0).getUrl());
		}
		String name = "";
		if (null != apiName) {
			name = apiName.name();
		}
		String type = "";
		if (null != type) {
			type = apiType.name();
		}
		url = apiMap.get(name + ":" + type);
		return url;
	}

	private Map<String, Object> buildHeader() {
		Map<String, Object> header = new HashMap<String, Object>();
		header.put("accept", "application/json");
		header.put("content-type", "application/json");
		header.put("X-Auth-Token", this.token.getToken());
		header.put("X-Project-id", this.token.getProject().getId());
		return header;
	}

}