package com.bocloud.cmp.provider.common;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.manageoneconvertor.ProjectConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华为manage one认证driver
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneAuthProvider extends ManageOneProvider implements AuthProvider {

    private static final Logger logger = LoggerFactory.getLogger(ManageOneAuthProvider.class);

    /**
     * @param butler
     */
    public ManageOneAuthProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult auth() {
        if (this.isAccessable()) {
            return new BocloudResult(true, "" + this.isAccessable());
        }
        return new BocloudResult(this.isAccessable(), "认证信息失败");
    }

    @Override
    public BocloudResult listProjects() {
        try {
            JSONObject params = new JSONObject();
            Result result = this.doGet("/v3/auth/projects", params, null, null);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "云主机列表为空");
            }
            List<JSONObject> objects = JSONArray.parseArray(object.getString("projects"), JSONObject.class);
            ProjectConvertor convertor = new ProjectConvertor();
            return new BocloudResult(true, convertor.convertList(objects), "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult(false, "查询失败");
        }

    }
}
