package com.bocloud.cmp.provider.common;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.FlavorConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * 华为manage one云主机规格操作driver
 *
 * @author wangyu
 * @version 1.0
 * @complete gp
 * @since 2018年9月17日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneFlavorProvider extends ManageOneProvider implements FlavorProvider {

	private static final Logger logger = LoggerFactory.getLogger(ManageOneFlavorProvider.class);
	private BeanConvertor<FlavorModel, JSONObject> convertor = new FlavorConvertor();

	/**
	 * @param butler
	 */
	public ManageOneFlavorProvider(Butler butler) {
		super(butler);
	}
	
	public static void main(String[] args) {
    	Butler testButler = new Butler();
    	testButler.setUrl("https://iam-apigateway-proxy.hcslab.com.cn");
    	testButler.setUsername("byyg_admin");
    	testButler.setPassword("User@123");
    	testButler.setDomainName("博云");
    	testButler.setProjectId("c0c568d034c1477fad8e35cd4c6098db");
    	ManageOneFlavorProvider provider = new ManageOneFlavorProvider(testButler );
		System.out.println(provider.sync());
    }
	

	@Override
	public BocloudResult sync() {
		try {
//			Result result = doGet("/flavors/detail", null, ManageOneApiName.ecs, ManageOneApiType.compute);
			Result result = listFlavors("/cloudservers/flavors", null, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("flavors"), JSONObject.class);
			List<FlavorModel> convertList = this.convertor.convertList(objects);
			return new BocloudResult(true, convertList, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "查询失败");
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		try {
			JSONObject object = new JSONObject();
			object.put("name", flavor.getName());
			object.put("vcpus", flavor.getCpu());
			object.put("ram", flavor.getMemory());
			object.put("disk", flavor.getDisk());
			Result result = this.doPost("/flavors", object, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject flavorObj = JSONObject.parseObject(result.getMessage());
			return new BocloudResult(true, convertor.convertModel(flavorObj.getJSONObject("flavor")), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "创建失败");
		}
	}

	@Override
	public BocloudResult remove(String flavorId) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		// TODO Auto-generated method stub
		return null;
	}

}
