package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.manageone.ManageOneVdcProjectProvider;
import com.bocloud.cmp.provider.manageone.ManageOneVdcProvider;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.security.ManageOneFirewallProvider;
import com.bocloud.cmp.provider.security.ManageOneKeypairProvider;
import com.bocloud.cmp.provider.storage.ManageOneVolumeProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.ResultTools;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.List;

/**
 * 华为manage one数据组装类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年9月25日
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneLocationProvider extends ManageOneProvider implements LocationProvider {

    private static final Logger logger = LoggerFactory.getLogger(ManageOneLocationProvider.class);
    private ZoneProvider regionProvider = null;
    private FlavorProvider flavorProvider = null;
    private ImageProvider imageProvider = null;
    private KeypairProvider keypairProvider = null;
    private NetworkProvider networkProvider = null;
    private PortProvider portProvider = null;
    private RouterProvider routerProvider = null;
    private SecurityGroupProvider secgProvider = null;
    private ManageOneServerProvider serverProvider = null;
    private SnapshotProvider snapshotProvider = null;
    private SubnetProvider subnetProvider = null;
    private VolumeProvider volumeProvider = null;
    private ManageOneHyperVProvider hyperVProvider = null;
    private ManageOneFirewallProvider firewallProvider = null;
    private ManageOneFloatingIPProvider floatingIpProvider = null;
    private ManageOneVdcProvider vdcProvider = null;
    private ManageOneVdcProjectProvider projectProvider = null;
    private ManageOneVpcProvider vpcProvider = null;
    private Butler butler;
    private ManageOneAuthProvider authProvider = null;

    /**
     * @param butler
     */
    public ManageOneLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult listRegions() {
        return ResultTools.noImplementedResult();
    }

    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult sync() {
        try {
            authProvider = new ManageOneAuthProvider(butler);
            BocloudResult listProjects = authProvider.listProjects();
            if (listProjects.isFailed()) {
                return listProjects;
            }
            List<ManageOneProjectModel> manageOneProjectModels = (List<ManageOneProjectModel>)listProjects.getData();

            ManageOneSyncModel model = new ManageOneSyncModel();
            List<RegionModel> regions = new ArrayList<>();
            List<FlavorModel> flavors = new ArrayList<>();
            List<ImageModel> images = new ArrayList<>();
            List<KeypairModel> keypairs = new ArrayList<>();
            List<PortModel> portModels = new ArrayList<>();
            List<NetworkModel> networkModels = new ArrayList<>();
            List<RouterModel> routerModels = new ArrayList<>();
            List<SecurityGroupModel> secgModels = new ArrayList<>();
            List<ServerModel> serverModels = new ArrayList<>();
            List<VolumeTypeModel> volumeTypeModels = new ArrayList<>();
            List<VolumeModel> volumeModels = new ArrayList<>();
            List<SnapshotModel> snapshotModels = new ArrayList<>();
            List<FirewallModel> firewallModels = new ArrayList<>();
            List<FirewallPolicyModel> firewallPolicyModels = new ArrayList<>();
            List<FirewallRuleModel> firewallRuleModels = new ArrayList<>();
            List<FloatingIpModel> floatingIpModels = new ArrayList<>();
            List<HostModel> hostModels = new ArrayList<>();

//            for (ManageOneProjectModel manageOneProjectModel : manageOneProjectModels) {
//                butler.setProjectId(manageOneProjectModel.getProjectUuid());
                regionProvider = new ManageOneRegionProvider(butler);
                flavorProvider = new ManageOneFlavorProvider(butler);
                imageProvider = new ManageOneImageProvider(butler);
                keypairProvider = new ManageOneKeypairProvider(butler);
                networkProvider = new ManageOneNetworkProvider(butler);
                portProvider = new ManageOnePortProvider(butler);
                routerProvider = new ManageOneRouterProvider(butler);
                secgProvider = new ManageOneSecurityGroupProvider(butler);
                serverProvider = new ManageOneServerProvider(butler);
                snapshotProvider = new ManageOneSnapshotProvider(butler);
                subnetProvider = new ManageOneSubnetProvider(butler);
                volumeProvider = new ManageOneVolumeProvider(butler);
                hyperVProvider = new ManageOneHyperVProvider(butler);
                firewallProvider = new ManageOneFirewallProvider(butler);
                floatingIpProvider = new ManageOneFloatingIPProvider(butler);
                vdcProvider = new ManageOneVdcProvider(butler);
                projectProvider = new ManageOneVdcProjectProvider(butler);
                vpcProvider = new ManageOneVpcProvider(butler);

                logger.info("Start assembling region data...");
                BocloudResult regionResult = regionProvider.list();
                if (regionResult.isSuccess()) {
                    regions.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(regionResult.getData()), RegionModel.class));
                }
                logger.info("End assembling region data...");

                logger.info("Start assembling flavor data...");
                BocloudResult flavorResult = flavorProvider.sync();
                if (flavorResult.isSuccess()) {
                    flavors.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(flavorResult.getData()), FlavorModel.class));
                }
                logger.info("End assembling flavor data...");
                logger.info("Start assembling image data...");
                BocloudResult imageResult = imageProvider.list();
                if (imageResult.isSuccess()) {
                    images
                        .addAll(JSONArray.parseArray(JSONArray.toJSONString(imageResult.getData()), ImageModel.class));
                }
                logger.info("End assembling image data...");
                
                logger.info("Start assembling keypair data...");
                BocloudResult keypairResult = keypairProvider.list();
                if (keypairResult.isSuccess()) {
                    keypairs.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(keypairResult.getData()), KeypairModel.class));
                }
                logger.info("End assembling keypair data...");

                logger.info("Start assembling port data...");
                BocloudResult portResult = portProvider.sync();
                if (portResult.isSuccess()) {
                    portModels
                        .addAll(JSONArray.parseArray(JSONArray.toJSONString(portResult.getData()), PortModel.class));
                }
                logger.info("End assembling port data...");

                BocloudResult hyperVProviderResult = hyperVProvider.list();
                if (hyperVProviderResult.isSuccess()) {
                    hostModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(hyperVProviderResult.getData()), HostModel.class));
                }

                BocloudResult networkResult = networkProvider.list();
                logger.info("Start assembling Network data...");
                if (networkResult.isSuccess()) {
                    List<NetworkModel> networkModelsByProject =
                        JSONArray.parseArray(JSONArray.toJSONString(networkResult.getData()), NetworkModel.class);
                    for (NetworkModel networkModel : networkModelsByProject) {
                        List<SubnetModel> subnetModels = networkModel.getSubnets();
                        List<SubnetModel> newSubModels = new ArrayList<>();
                        for (SubnetModel subnetModel : subnetModels) {
                            BocloudResult subnetResult = subnetProvider.detail(subnetModel.getSubnetUuid());
                            Assert.isTrue(subnetResult.isSuccess(), subnetResult.getMessage());
                            newSubModels.add((SubnetModel)subnetResult.getData());
                        }
                        networkModel.setSubnets(newSubModels);
                    }
                    networkModels.addAll(networkModelsByProject);
                }

                logger.info("End assembling Network data...");

                logger.info("Start assembling router data...");
                BocloudResult routerResult = routerProvider.list();
                if (routerResult.isSuccess()) {
                    routerModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(routerResult.getData()), RouterModel.class));
                }
                logger.info("End assembling router data...");

                logger.info("Start assembling security group data...");
                BocloudResult secgResult = secgProvider.list();
                if (secgResult.isSuccess()) {
                    secgModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(secgResult.getData()), SecurityGroupModel.class));
                }
                logger.info("End assembling security group data...");

                logger.info("Start assembling server data...");
                BocloudResult serverResult = serverProvider.list();
                if (serverResult.isSuccess()) {
                    serverModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(serverResult.getData()), ServerModel.class));
                }
                logger.info("End assembling server data...");

                logger.info("Start assembling volume type data...");
                BocloudResult volumeTypeResult = volumeProvider.listVolumeTypes();
                if (volumeTypeResult.isSuccess()) {
                    volumeTypeModels.addAll(JSONArray.parseArray(JSONArray.toJSONString(volumeTypeResult.getData()),
                        VolumeTypeModel.class));
                }
                logger.info("End assembling volume type data...");

                logger.info("Start assembling volume data...");
                BocloudResult volumeResult = volumeProvider.list();
                if (volumeResult.isSuccess()) {
                    volumeModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(volumeResult.getData()), VolumeModel.class));
                }
                logger.info("End assembling volume data...");

                logger.info("Start assembling snapshot data...");
                BocloudResult snapResult = snapshotProvider.list();
                if (snapResult.isSuccess()) {
                    snapshotModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(snapResult.getData()), SnapshotModel.class));
                }
                logger.info("End assembling snapshot data...");

                // 同步防火墙数据
                logger.info("Start assembling firewall data...");
                BocloudResult firewallResult = firewallProvider.list();
                if (firewallResult.isSuccess()) {
                    firewallModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(firewallResult.getData()), FirewallModel.class));
                }
                logger.info("End assembling firewall data...");

                logger.info("Start assembling firewall policy data...");
                BocloudResult listPolicy = firewallProvider.listPolicy();
                if (listPolicy.isSuccess()) {
                    firewallPolicyModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(listPolicy.getData()), FirewallPolicyModel.class));
                }
                logger.info("End assembling firewall policy data...");
                logger.info("Start assembling firewall rule data...");

                BocloudResult listRule = firewallProvider.listRule();
                if (listRule.isSuccess()) {
                    firewallRuleModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(listRule.getData()), FirewallRuleModel.class));
                }
                logger.info("End assembling firewall rule data...");

                logger.info("Start assembling floatingIp data...");
                BocloudResult floatIpResult = floatingIpProvider.list();
                if (floatIpResult.isSuccess()) {
                    floatingIpModels.addAll(
                        JSONArray.parseArray(JSONArray.toJSONString(floatIpResult.getData()), FloatingIpModel.class));
                }
                logger.info("End assembling floatingIp data...");

//            }

            model.setFloatingIpModels(floatingIpModels == null ? new ArrayList<>() : floatingIpModels);
            model.setFirewallRuleModels(firewallRuleModels == null ? new ArrayList<>() : firewallRuleModels);
            model.setFirewallPolicyModels(firewallPolicyModels == null ? new ArrayList<>() : firewallPolicyModels);
            model.setFirewallModels(firewallModels == null ? new ArrayList<>() : firewallModels);
            model.setSnapshotModels(snapshotModels);
            model.setVolumeModels(volumeModels);
            model.setVolumeTypeModels(volumeTypeModels);
            model.setServerModels(serverModels);
            model.setGroupModels(secgModels);
            model.setRouterModels(routerModels);
            model.setRegions(regions);
            model.setFlavorModels(flavors);
            model.setImageModels(images);
            model.setKeypairModels(keypairs);
            model.setPortModels(portModels);
            model.setHostModels(new ArrayList<>());
            model.setNetworkModels(networkModels);
            model.setVdcModels(new ArrayList<>());
            model.setStorageModels(new ArrayList<>());
            model.setHostModels(hostModels);
            if(!ListTool.isEmpty(model.getHostModels())) {
                logger.info("host : " + JSON.toJSONString(model.getHostModels().get(0)));
            }
            logger.info("Start assembling Vdc data...");
            // BocloudResult vdcResult = vdcProvider.list();
            // Assert.isTrue(vdcResult.isSuccess(), vdcResult.getMessage());
            // List<VdcModel> vdcModels =
            // JSONArray.parseArray(JSONArray.toJSONString(vdcResult.getData()), VdcModel.class);
            // if (!ListTool.isEmpty(vdcModels)) {
            // for (VdcModel vdcModel : vdcModels) {
            // BocloudResult projectResult = projectProvider.list(vdcModel.getVdcId());
            // Assert.isTrue(projectResult.isSuccess(), projectResult.getMessage());
            // List<VdcProjectModel> projectModels =
            // JSONArray.parseArray(JSONArray.toJSONString(projectResult.getData()), VdcProjectModel.class);
            // vdcModel.setProjectModels(projectModels);
            // }
            // }
            model.setVdcModels(new ArrayList<>());
            logger.info("Start assembling Vdc data...");
            logger.info("Start sync storage pool data...");
            BocloudResult storageCapacity = serverProvider.storageCapacity();
            if (storageCapacity.isSuccess()) {
                List<StorageModel> storageModels = (List<StorageModel>)storageCapacity.getData();
                model.setStorageModels(storageModels == null ? new ArrayList<>() : storageModels);
            }
            logger.info("End sync storage pool data...");
            return new BocloudResult(true, model, "同步成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        return ResultTools.noImplementedResult();
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        FusionCloudSyncModel model = new FusionCloudSyncModel();
        BocloudResult portResult = portProvider.sync();
        List<PortModel> portModels = new ArrayList<PortModel>();
        if (portResult.isSuccess()) {
            portModels = JSONArray.parseArray(JSONArray.toJSONString(portResult.getData()), PortModel.class);
        }
        List<PortModel> result = new ArrayList<PortModel>();
        for (PortModel portModel : portModels) {
            if (portModel.getNetworkUuid().contains(networkUuid)) {
                result.add(portModel);
            }
        }
        model.setPortModels(result);
        return new BocloudResult(true, model, "success");
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

}
