package com.bocloud.cmp.provider.compute;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.HypervisorConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HostOperateModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * hypervisors
 * 
 * @author admin
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneHyperVProvider extends ManageOneProvider implements HostProvider {

	private HypervisorConvertor convertor = new HypervisorConvertor();

	public ManageOneHyperVProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(ManageOneHyperVProvider.class);

	@Override
	public BocloudResult list() {
		try {
			Result result = doGet("/os-hypervisors/detail", null, ManageOneApiName.ecs, ManageOneApiType.compute);
			if (result.isFailed()) {
				logger.error("同步hypervisors异常：" + result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				logger.error("同步hypervisors异常：" + JSON.toJSONString(result));
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("hypervisors"), JSONObject.class);
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message:", e);
			return new BocloudResult(false, "查询失败");
		}

	}

	/**
	 * 计算主机删除
	 */
	@Override
	public BocloudResult remove(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult enterMode(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult exitMode(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult disconn(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult reconn(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult moveTo(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult shutdown(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult reboot(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult detail(HostOperateModel hostOperateModel) {
		return null;
	}

	@Override
	public BocloudResult listHosts() {
		try {
			return new BocloudResult(true, "暂未开发！");
		} catch (Exception e) {
			logger.error("同步host异常：" + e.getMessage());
			return new BocloudResult(false, "同步host异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult monitor() {
		return null;
	}

	@Override
	public BocloudResult listPerf() {
		return new BocloudResult(true, "暂未开发！");
	}

	@Override
	public BocloudResult summary(Long aLong) {
		return null;
	}

	@Override
	public BocloudResult detail(Long aLong) {
		return null;
	}

	@Override
	public BocloudResult list(String s) {
		return null;
	}

}
