package com.bocloud.cmp.provider.compute;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.ImageConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

@Provider("bocloud.cmp.manageone")
public class ManageOneImageProvider extends ManageOneProvider implements ImageProvider {
	private static final Logger logger = LoggerFactory.getLogger(ManageOneImageProvider.class);
	private BeanConvertor<ImageModel, JSONObject> convertor = new ImageConvertor();

	public ManageOneImageProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = doGet("/v2/cloudimages", null, ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("images"), JSONObject.class);
			return new BocloudResult(true, this.convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "查询失败");
	}
	
	@Override
	public BocloudResult detail(String imageId) {
		try {
			Result result = doGet("/v2/images/" + imageId, null, ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "查询失败");
			}
			return new BocloudResult(true, this.convertor.convertModel(object), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
			return new BocloudResult(false, "查询失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String imageId) {
		try {
			BocloudResult bocloudResult = detail(imageId);
			if (null == bocloudResult.getData()) {
				return new BocloudResult(true, "删除失败");
			}
			Result result = doDelete("/v2/images/" + imageId, null, ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "删除失败");
	}

	@Override
	public BocloudResult modify(ImageModel image) {
		try {
			List<JSONObject> params = new ArrayList<JSONObject>();
			JSONObject nameObj = new JSONObject();
			nameObj.put("op", "replace");
			nameObj.put("path", "/name");
			nameObj.put("value", image.getName());
			params.add(nameObj);
			if (null != image.getRemark()) {
				JSONObject descObj = new JSONObject();
				descObj.put("op", "replace");
				descObj.put("path", "/__description");
				descObj.put("value", image.getRemark());
				params.add(descObj);
			}
			Result result = doPatch("/v2/cloudimages/" + image.getImageUuid(), JSONArray.toJSONString(params),
					ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject imageObj = (JSONObject) JSONObject.parseObject(result.getMessage(), JSONObject.class);
			return new BocloudResult(true, this.convertor.convertModel(imageObj), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "修改失败");
	}

	@Override
	public BocloudResult create(ImageModel imageModel) {
		try {
			JSONObject params = new JSONObject();
			params.put("name", imageModel.getName());
			params.put("description", imageModel.getRemark());
			params.put("instance_id", imageModel.getInstanceId());
			Result result = doPost("/v2/cloudimages/action", params, ManageOneApiName.ims, ManageOneApiType.image);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			String jobId = object.getString("job_id");
			BocloudResult waitTaskResult = waitTaskResult(jobId);
			if (waitTaskResult.isFailed()) {
				return new BocloudResult(false, "创建失败:" + waitTaskResult.getMessage());
			}
			return new BocloudResult(true, waitTaskResult.getData(), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message��", e);
		}
		return new BocloudResult(false, "创建失败");
	}

	private BocloudResult waitTaskResult(String jobId) {
		Boolean flag = Boolean.valueOf(true);
		Long startTime = Long.valueOf(System.currentTimeMillis());
		Result result = null;
		JSONObject jobResult = null;
		BocloudResult bocloudResult = new BocloudResult(false, "操作失败！");
		while (flag.booleanValue()) {
			try {
				Thread.sleep(8000L);
			} catch (InterruptedException e) {
				logger.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			result = waitImageTask(jobId);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			jobResult = JSONObject.parseObject(result.getMessage());
			logger.info("jobResult : " + JSON.toJSONString(jobResult));
			if ("FAIL".equalsIgnoreCase(jobResult.getString("status"))) {
				return new BocloudResult(false, jobResult.getString("entities"));
			}
			Long endTime = Long.valueOf(System.currentTimeMillis());
			if ("SUCCESS".equalsIgnoreCase(jobResult.getString("status"))
					|| endTime.longValue() - startTime.longValue() > 40 * 60 * 1000L) {
				JSONObject imageObj = JSONObject.parseObject(jobResult.getString("entities"));
				String imageId = imageObj.getString("image_id");
				BocloudResult detailResult = detail(imageId);
				if (detailResult.isSuccess()) {
					return detailResult;
				} else {
					return new BocloudResult(false, "镜像创建失败！");
				}
			}
		}
		return bocloudResult;
	}

	@Override
	public BocloudResult shareImage(ImageModel imageModel) {
		return null;
	}

	@Override
	public BocloudResult syncImages(ImageModel imageModel) {
		return null;
	}
}
