package com.bocloud.cmp.provider.manageone;

import io.jsonwebtoken.lang.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.manageoneconvertor.VdcUserConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.manageone.VdcUserModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one 用户管理
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月17日
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneUserProvider extends ManageOneProvider implements VdcUserProvider {

	private static Logger logger = LoggerFactory.getLogger(ManageOneUserProvider.class);
	private BeanConvertor<VdcUserModel, JSONObject> convertor = new VdcUserConvertor();
	/**
	 * 华为manage one limit取值范围：0-1000，默认20
	 */
	private static final Integer LIMIT = 100;

	public ManageOneUserProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list(String tenantId) {
		try {
			Map<String, Object> paramMap = new HashMap<>();
			paramMap.put("start", 0);
			paramMap.put("limit", LIMIT);
			paramMap.put("tenant_id", tenantId);
			Result result = this.doGet("/rest/vdc/v3.0/users", paramMap, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject object = JSONObject.parseObject(result.getMessage());
			// 返回总条数
			Long total = object.getLong("total");
			if (null == total || total == 0L) {
				return new BocloudResult(true, "查询成功");
			}
			List<VdcUserModel> VdcUserModels = new ArrayList<>();
			// 第一次查询的结果集
			List<JSONObject> objects = JSONArray.parseArray(object.getString("users"), JSONObject.class);
			VdcUserModels.addAll(convertor.convertList(objects));
			Long totalPage = total / LIMIT + 1;
			if (totalPage.equals(1L)) {
				return new BocloudResult(true, VdcUserModels, "查询成功");
			}
			for (int i = 1; i <= totalPage; i++) {
				paramMap.clear();
				paramMap.put("start", i);
				paramMap.put("limit", LIMIT);
				paramMap.put("tenant_id", tenantId);
				result = this.doGet("/rest/vdc/v3.0/users", paramMap, null, null);
				Assert.isTrue(result.isSuccess(), result.getMessage());
				object = JSONObject.parseObject(result.getMessage());
				objects = JSONArray.parseArray(object.getString("users"), JSONObject.class);
				if (!ListTool.isEmpty(objects)) {
					VdcUserModels.addAll(convertor.convertList(objects));
				}
			}
			return new BocloudResult(true, VdcUserModels, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VdcUserModel model) {
		try {
			JSONObject params = new JSONObject();
			params.put("tenant_id", model.getVdcId());
			params.put("limit_access", false);
			JSONObject userParam = new JSONObject();
			userParam.put("name", model.getName());
			userParam.put("display_name", model.getDisplayName());
			userParam.put("password", model.getPassword());
			userParam.put("enabled", true);
			userParam.put("description", model.getRemark());
			userParam.put("roles", model.getRoleList());
			userParam.put("projects", model.getProjects());
			params.put("user", userParam);
			Result result = this.doPost("/rest/vdc/v3.0/users", params, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("user");
			return new BocloudResult(true, this.detail(jsonObject.getString("id")).getData(), "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "创建失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String userId) {
		try {
			Result result = this.doDelete("/rest/vdc/v3.0/users/" + userId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String userId) {
		try {
			Result result = this.doGet("/rest/vdc/v3.0/users/" + userId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("user");
			return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}
}
