package com.bocloud.cmp.provider.manageone;

import io.jsonwebtoken.lang.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.manageoneconvertor.VdcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.manageone.VdcModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

/**
 * 华为manage one vdc管理
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月17日
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneVdcProvider extends ManageOneProvider implements VdcProvider {

	private static Logger logger = LoggerFactory.getLogger(ManageOneVdcProvider.class);
	private BeanConvertor<VdcModel, JSONObject> convertor = new VdcConvertor();
	/**
	 * 华为manage one limit取值范围：0-1000，默认20
	 */
	private static final Integer LIMIT = 1000;

	public ManageOneVdcProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			Map<String, Object> paramMap = new HashMap<>();
			paramMap.put("start", 0);
			paramMap.put("limit", LIMIT);
			Result result = this.doGet("/rest/vdc/v3.0/vdcs", paramMap, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject object = JSONObject.parseObject(result.getMessage());
			// 返回总条数
			Long total = object.getLong("total");
			if (null == total || total == 0L) {
				return new BocloudResult(true, "查询成功");
			}
			List<VdcModel> vdcModels = new ArrayList<>();
			// 第一次查询的结果集
			List<JSONObject> objects = JSONArray.parseArray(object.getString("vdcs"), JSONObject.class);
			vdcModels.addAll(convertor.convertList(objects));
			Long totalPage = total / LIMIT + 1;
			if (totalPage.equals(1L)) {
				return new BocloudResult(true, vdcModels, "查询成功");
			}
			for (int i = 1; i <= totalPage; i++) {
				paramMap.clear();
				paramMap.put("start", i);
				paramMap.put("limit", LIMIT);
				result = this.doGet("/rest/vdc/v3.0/vdcs", paramMap, null, null);
				Assert.isTrue(result.isSuccess(), result.getMessage());
				object = JSONObject.parseObject(result.getMessage());
				objects = JSONArray.parseArray(object.getString("vdcs"), JSONObject.class);
				if (!ListTool.isEmpty(objects)) {
					vdcModels.addAll(convertor.convertList(objects));
				}
			}
			return new BocloudResult(true, vdcModels, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(VdcModel model) {
		try {
			JSONObject vdcParam = new JSONObject();
			vdcParam.put("upper_vdc_id", model.getUpperVdcId());
			vdcParam.put("name", model.getName());
			vdcParam.put("description", model.getRemark());
			vdcParam.put("mfa_status", model.getMfaStatus());
			vdcParam.put("ldap_id", model.getLdapId());
			vdcParam.put("processes", model.getProcesses());
			vdcParam.put("third_type", model.getThirdType());
			vdcParam.put("cloud_federation_rate", model.getCloudFederationRate());
			vdcParam.put("other_cloud_federation_rate", model.getOtherCloudFederationRate());
			vdcParam.put("enterprise_id", model.getEnterpriseId());
			vdcParam.put("status", model.getStatus());
			vdcParam.put("tag", model.getTag());
			vdcParam.put("enterprise_detail", model.getEnterpriseDetail());
			JSONObject params = new JSONObject();
			params.put("vdc", vdcParam);
			Result result = this.doPost("/rest/vdc/v3.1/vdcs", params, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("vdc");
			String vdcId = jsonObject.getString("id");
			VdcModel vdcModel = (VdcModel) this.detail(vdcId).getData();
			// 授权区域
			this.assignRegion(vdcModel);
			return new BocloudResult(true, vdcModel, "创建成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "创建失败：" + e.getMessage());
		}
	}

	private void assignRegion(VdcModel model) {
		String vdcId = model.getVdcId();
		// 获取地域列表
		Result regionResult = this.doGet("/rest/serviceaccess/v3.0/regions", null, null, null);
		if (regionResult.isFailed()) {
			return;
		}
		List<JSONObject> regions = JSONArray
				.parseArray(JSONObject.parseObject(regionResult.getMessage()).getString("records"), JSONObject.class);
		List<JSONObject> assignRegions = new ArrayList<>();
		List<JSONObject> externalNetworks = new ArrayList<>();
		for (JSONObject region : regions) {
			JSONObject assignRegion = new JSONObject();
			assignRegion.put("region_id", region.getString("id"));
			assignRegion.put("action", "bind");
			List<JSONObject> assignCloudInfras = new ArrayList<>();
			String cloudInfras = region.getString("cloud_infras");
			if (!StringUtils.isEmpty(cloudInfras)) {
				List<JSONObject> cloudInfraList = JSONArray.parseArray(cloudInfras, JSONObject.class);
				for (JSONObject cloudInfra : cloudInfraList) {
					Result networkResult = doGet(
							"/rest/serviceaccess/v3.0/external-networks?cloud_infra_id=" + cloudInfra.getString("id"),
							null, null, null);
					if (networkResult.isSuccess()) {
						List<JSONObject> networks = JSONArray.parseArray(
								JSONObject.parseObject(networkResult.getMessage()).getString("external_networks"),
								JSONObject.class);
						for (JSONObject network : networks) {
							JSONObject externalNetwork = new JSONObject();
							externalNetwork.put("network_id", network.get("id"));
							externalNetwork.put("cloudInfraId", cloudInfra.getString("id"));
							externalNetwork.put("networkName", network.getString("name"));
							externalNetworks.add(externalNetwork);
						}
					}
					JSONObject assignCloudInfra = new JSONObject();
					assignCloudInfra.put("cloud_infra_id", cloudInfra.getString("id"));
					assignCloudInfra.put("action", "bind");
					// 查询可用域
					Result azResult = this.doGet(
							"/rest/serviceaccess/v3.0/available-zones?cloud_infra_id=" + cloudInfra.getString("id"),
							null, null, null);
					if (azResult.isFailed()) {
						return;
					}
					List<JSONObject> azs = JSONArray.parseArray(
							JSONObject.parseObject(azResult.getMessage()).getString("records"), JSONObject.class);
					List<JSONObject> assignAzs = new ArrayList<>();
					for (JSONObject az : azs) {
						JSONObject assignAz = new JSONObject();
						assignAz.put("az_id", az.get("az_id"));
						assignAz.put("action", "bind");
						assignAzs.add(assignAz);
					}
					if (!ListTool.isEmpty(assignAzs)) {
						assignCloudInfra.put("available_zones", assignAzs);
					}
					assignCloudInfras.add(assignCloudInfra);
				}
			}
			if (!ListTool.isEmpty(assignCloudInfras)) {
				assignRegion.put("cloud_infras", assignCloudInfras);
			}
			assignRegions.add(assignRegion);
		}
		if (!ListTool.isEmpty(assignRegions)) {
			JSONObject params = new JSONObject();
			params.put("regions", assignRegions);
			Result result = this.doPut("/rest/vdc/v3.1/vdcs/" + vdcId + "/regions", params, null, null);
			logger.info("VDC授权区域结果：{}", JSONObject.toJSON(result));
		}
		// 授权外部网络
		if ("0".equals(model.getUpperVdcId()) && !ListTool.isEmpty(externalNetworks)) {
			for (JSONObject externalNetwork : externalNetworks) {
				String networkId = externalNetwork.getString("network_id");
				externalNetwork.put("resource_id", vdcId);
				externalNetwork.put("associateAction", true);
				externalNetwork.put("inherit", true);
				externalNetwork.remove("network_id");
				Result result = this.doPut("/rest/vdc/v3.0/vdcs/external-networks/" + networkId, externalNetwork, null,
						null);
				logger.info("VDC授权网络结果：{}", JSONObject.toJSON(result));
			}
		}
	}

	@Override
	public BocloudResult remove(String vdcId) {
		try {
			Result result = this.doDelete("/rest/vdc/v3.0/vdcs/" + vdcId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "删除失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String vdcId) {
		try {
			Result result = this.doGet("/rest/vdc/v3.0/vdcs/" + vdcId, null, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("vdc");
			return new BocloudResult(true, convertor.convertModel(jsonObject), "查询详情成功！");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult update(VdcModel model) {
		try {
			JSONObject vdcParam = new JSONObject();
			vdcParam.put("upper_vdc_id", model.getUpperVdcId());
			vdcParam.put("name", model.getName());
			vdcParam.put("description", model.getRemark());
			vdcParam.put("mfa_status", model.getMfaStatus());
			vdcParam.put("ldap_id", model.getLdapId());
			vdcParam.put("processes", model.getProcesses());
			vdcParam.put("third_type", model.getThirdType());
			vdcParam.put("cloud_federation_rate", model.getCloudFederationRate());
			vdcParam.put("other_cloud_federation_rate", model.getOtherCloudFederationRate());
			vdcParam.put("enterprise_id", model.getEnterpriseId());
			vdcParam.put("status", model.getStatus());
			vdcParam.put("tag", model.getTag());
			vdcParam.put("enterprise_detail", model.getEnterpriseDetail());
			JSONObject params = new JSONObject();
			params.put("vdc", vdcParam);
			Result result = this.doPut("/rest/vdc/v3.1/vdcs/" + model.getVdcId(), params, null, null);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("vdc");
			return new BocloudResult(true, this.detail(jsonObject.getString("id")).getData(), "修改成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "修改失败：" + e.getMessage());
		}
	}
}
