package com.bocloud.cmp.provider.network;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.enums.ManageOneApiName;
import com.bocloud.cmp.common.enums.ManageOneApiType;
import com.bocloud.cmp.manageoneconvertor.VpcConvertor;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.ManageOneProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;

/**
 * manageone vpc
 * 
 * @author admin
 *
 */
@Provider("bocloud.cmp.manageone")
public class ManageOneVpcProvider extends ManageOneProvider implements VpcProvider {

	private static Logger logger = LoggerFactory.getLogger(ManageOneVpcProvider.class);
	private Butler butler;

	public ManageOneVpcProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	private BeanConvertor<VpcModel, JSONObject> convertor = new VpcConvertor();

	@Override
	public BocloudResult create(VpcModel model) {
		try {
			return new BocloudResult(true, "开发中...");
		} catch (Exception e) {
			return new BocloudResult(false, e.getMessage());
		}
	}

	public BocloudResult getVpcDetail(String vpcId, String regionId) {
		BocloudResult result = null;
		try {
			Boolean flag = true;
			Long startTime = System.currentTimeMillis();
			while (flag) {
				Thread.sleep(100);
				Long endTime = System.currentTimeMillis();
				// 等待60s
				if (endTime - startTime > 3 * 60 * 1000) {
					flag = false;
					result = this.detail(vpcId, regionId);
					break;
				}
			}
			return result;
		} catch (Exception e) {
			logger.error("查询状态失败： " + e);
			return new BocloudResult(false, "查询状态失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String vpcId) {
		try {
			return new BocloudResult(true, "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult list() {
		try {
			Result result = this.doGet("/v1/" + butler.getProjectId() + "/vpcs", null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(true, "列表为空");
			}
			List<JSONObject> objects = JSONArray.parseArray(object.getString("vpcs"), JSONObject.class);
			return new BocloudResult(true, convertor.convertList(objects), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult detail(String vpcId, String regionId) {
		try {
			Result result = this.doGet("/v1/" + butler.getProjectId() + "/vpcs/" + vpcId, null, ManageOneApiName.vpc,
					ManageOneApiType.network);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(true, "资源不存在！");
			}
			return new BocloudResult(true, convertor.convertModel(object.getJSONObject("vpc")), "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败");
		}
	}

	@Override
	public BocloudResult update(VpcModel model) {
		try {
			return new BocloudResult(true, "修改成功");
		} catch (Exception e) {
			logger.error("修改失败 ： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

	@Override
	public boolean accessable() {
		return super.accessable();
	}

}
