package com.bocloud.cmp.utils;

import java.util.Date;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.common.model.ManageOneToken;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;

public class TokenCache {

	private static Logger logger = LoggerFactory.getLogger(TokenCache.class);

	private static LoadingCache<String, String> localcache =
			// ,1000是设置缓存的初始化容量，maximumSize是设置缓存最大容量
			CacheBuilder.newBuilder().initialCapacity(1000).maximumSize(10000).expireAfterWrite(1700, TimeUnit.SECONDS)
					// build里面要实现一个匿名抽象类
					.build(new CacheLoader<String, String>() {
						@Override
						public String load(String s) throws Exception {
							return "null";
						}
					});

	/*
	 * 添加本地缓存
	 */
	public static void setKey(String key, String value) {
		localcache.put(key, value);
	}

	/*
	 * 得到本地缓存
	 */
	public static String getKey(String key) {
		String value = null;
		try {
			value = localcache.get(key);
			if ("null".equals(value)) {
				return null;
			}
			if (key.startsWith("token.")) {
				ManageOneToken token = JSON.parseObject(value, ManageOneToken.class);
				if (null != token && null != token.getToken()) {
					Date current = new Date();
					// 判断当前时间是否超过了会话超时时间，如果超时之后进行重新认证
					if (current.after(token.getExpiresAt())) {
						localcache.invalidate(key);
						return null;
					}
				}
			}
			return value;
		} catch (ExecutionException e) {
			logger.error("getKey()方法错误", e);
		}
		return null;
	}

}