package com.bocloud.cmp.provider.common;

import java.math.BigDecimal;
import java.util.List;
import java.util.UUID;

import org.openstack4j.api.Builders;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.Flavor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.FlavorConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;

/**
 * openstack云主机规格操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackFlavorProvider extends OpenStackProvider implements FlavorProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackFlavorProvider.class);

    public OpenStackFlavorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        List<FlavorModel> models = Lists.newArrayList();
        List<? extends Flavor> flavors = this.getClient().compute().flavors().list();
        if (null != flavors) {
            for (Flavor flavor : flavors) {
                FlavorModel model = new FlavorModel();
                model.setCpu(flavor.getVcpus());
                model.setMemory(BigDecimal.valueOf(flavor.getRam()).divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP));
                model.setName(flavor.getName());
                model.setFamily(null);
                model.setDisk(flavor.getDisk());
                model.setFlavorUuid(flavor.getId());
                models.add(model);
            }
        }
        return new BocloudResult(true, models, "success");
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        String id = UUID.randomUUID().toString();
        Flavor flavorResult = null;
        try {
            flavorResult = this.getClient().compute().flavors().create(Builders.flavor().name(flavor.getName())
                    .ram(flavor.getMemory().multiply(BigDecimal.valueOf(1024)).intValue()).vcpus(flavor.getCpu()).disk(flavor.getDisk()).id(id).build());
            flavor.setFlavorUuid(flavorResult.getId());
            return new BocloudResult(true, flavor, "创建成功!");
        } catch (Exception e) {
            logger.error("Flavor Exception {}", e);
            return new BocloudResult(false, "创建失败!");
        }
    }

    @Override
    public BocloudResult remove(String flavorId) {
        ActionResponse response = this.getClient().compute().flavors().delete(flavorId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "配置删除成功！");
        } else {
            logger.error("remove flavor [{}] failed :{}", flavorId, response.getFault());
            return new BocloudResult(false, "配置删除失败！");
        }
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        try {
            Flavor update = Builders.flavor().id(flavorModel.getFlavorUuid()).name(flavorModel.getName())
                    .disk(flavorModel.getDisk()).vcpus(flavorModel.getCpu()).ram(flavorModel.getMemory().multiply(BigDecimal.valueOf(1024)).intValue()).build();
            this.getClient().compute().flavors().delete(flavorModel.getFlavorUuid());
            Flavor result = this.getClient().compute().flavors().create(update);
            if (null == result) {
                return new BocloudResult(false, "配置修改失败");
            }
            BeanConvertor<FlavorModel, Flavor> convertorFlavor = new FlavorConvertor();
            return new BocloudResult(true, convertorFlavor.convertModel(result), "配置修改成功");
        } catch (Exception e) {
            logger.error("update flavor failed :", e);
            return new BocloudResult(false, "配置修改失败");
        }
    }

}
