package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.OpenStackClientFactory;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.compute.OpenStackSnapshotProvider;
import com.bocloud.cmp.provider.convertor.*;
import com.bocloud.cmp.provider.network.OpenStackGwQosProvider;
import com.bocloud.cmp.provider.network.OpenStackVpnConnProvider;
import com.bocloud.cmp.provider.network.OpenStackVpnProvider;
import com.bocloud.cmp.provider.storage.OpenStackVolumeProvider;
import com.bocloud.cmp.provider.utils.ResourcePercolator;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.compute.*;
import org.openstack4j.model.compute.ext.AvailabilityZone;
import org.openstack4j.model.compute.ext.Hypervisor;
import org.openstack4j.model.identity.v2.Tenant;
import org.openstack4j.model.identity.v3.Domain;
import org.openstack4j.model.identity.v3.Project;
import org.openstack4j.model.image.v2.Image;
import org.openstack4j.model.network.*;
import org.openstack4j.model.network.SecurityGroup;
import org.openstack4j.model.network.ext.*;
import org.openstack4j.model.storage.object.SwiftContainer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * openstack数据同步操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackLocationProvider extends OpenStackProvider implements LocationProvider {

	private Butler butler;
	
	private static Logger logger = LoggerFactory.getLogger(OpenStackLocationProvider.class);

	public OpenStackLocationProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	@Override
	public BocloudResult listRegions() {
		return null;
	}

	@Override
	public BocloudResult syncHostAggregate() {
		return null;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public BocloudResult sync() {
		try {
			OSSyncModel model = new OSSyncModel();
			logger.info("start sync project data ......");
			List<OSTenantModel> osTenants = new ArrayList<>();
			if (Version.V2 == this.getVersion()) {
				List<? extends Tenant> tenants = this.getClient2().identity().tenants().list();
				for (Tenant tenant : tenants) {
					OSTenantModel osTenant = new OSTenantModel(tenant.getName(), tenant.getDescription(),
							tenant.getId(), tenant.isEnabled(), null, null);
					osTenants.add(osTenant);
				}
			} else {
				List<? extends Project> projects = this.getClient3().identity().projects().list();
				for (Project project : projects) {
					Domain domain = project.getDomain();
					String domainName = null;
					if (null != domain) {
						domainName = domain.getName();
					} else {
						domainName = project.getDomainId();
					}
					OSTenantModel tenant = new OSTenantModel(project.getName(), project.getDescription(),
							project.getId(), project.isEnabled(), project.getDomainId(), domainName);
					osTenants.add(tenant);
				}
			}
			model.setTenants(osTenants);
			logger.info("start sync availabilityzone data ......");
			List<? extends AvailabilityZone> zones = this.getClient().compute().zones().list(true);
			logger.info("start sync hostaggregate data ......");
			List<? extends HostAggregate> aggregates = this.getClient().compute().hostAggregates().list();
			logger.info("start sync hypervisor data ......");
			List<? extends Hypervisor> hypervisors = this.getClient().compute().hypervisors().list();
			logger.info("start sync server data ......");
			List<? extends Server> servers = this.getClient().compute().servers().listAll(true);
			List<Server> filterServers = Lists.newArrayList();
			if (!ListTool.isEmpty(servers)) {
				for (Server server : servers) {
					if (ResourcePercolator.serverFilter(server.getStatus())) {
						filterServers.add(server);
					}
				}
			}
			// 同步qos policy rules
			logger.info("start sync gwqos data ......");
			OpenStackGwQosProvider gwQosProvider = new OpenStackGwQosProvider(butler);
			BocloudResult qosPoliciesList = gwQosProvider.qosPoliciesList(null);
			if (qosPoliciesList.isSuccess()) {
				List<GwQosModel> gwQosModels = (List<GwQosModel>) qosPoliciesList.getData();
				if (null != gwQosModels && gwQosModels.size() > 0) {
					for (GwQosModel gwQosModel : gwQosModels) {
						BocloudResult qosRulesList = gwQosProvider.qosRulesList(gwQosModel);
						if (qosRulesList.isSuccess()) {
							gwQosModel.setQosRules((List<GwQosModel>) qosRulesList.getData());
						}
					}
				}
				model.setGwQosModel(gwQosModels);
			}
			Map<String, String> filterParams = new HashMap<>();
			filterParams.put("all_tenants", "true");
			// 同步云硬盘
			logger.info("start sync volume data ......");
			OpenStackVolumeProvider volumeProvider = new OpenStackVolumeProvider(butler);
			BocloudResult volumeResult = volumeProvider.list();
			if (volumeResult.isFailed()) {
				return volumeResult;
			}
			List<VolumeModel> volumeModels = (List<VolumeModel>) volumeResult.getData();
			model.setVolumeModels(volumeModels);
			// 同步云硬盘类型
			try {
				logger.info("start sync volume type data ......");
				JSONObject params = new JSONObject();
				Result result = blockStorageFromGet("/types", params);
				if (result.isSuccess()) {
					JSONObject object = JSONObject.parseObject(result.getMessage());
					if (null != object) {
						List<JSONObject> list = JSONArray.parseArray(object.getString("volume_types"),
								JSONObject.class);
						if (!ListTool.isEmpty(list)) {
							List<VolumeTypeModel> typeModels = new ArrayList<>();
							for (JSONObject typeObject : list) {
								VolumeTypeModel typeModel = new VolumeTypeModel();
								typeModel.setVolumeTypeUuid(typeObject.getString("id"));
								typeModel.setName(typeObject.getString("name"));
								if (!StringUtils.isEmpty(typeObject.getString("extra_specs"))) {
									typeModel.setExtraSpecs(typeObject.getString("extra_specs"));
								}
								typeModels.add(typeModel);
							}
							model.setTypeModels(typeModels);
						}
					}
				}
			} catch (Exception e) {
				logger.error("query volume type error：", e);
				return new BocloudResult<>(false, "查询硬盘类型失败：" + e);
			}
			// 同步快照列表
			logger.info("start sync snapshot data ......");
			OpenStackSnapshotProvider snapshotProvider = new OpenStackSnapshotProvider(butler);
			BocloudResult snapshotResult = snapshotProvider.list();
			if (snapshotResult.isFailed()) {
				return snapshotResult;
			}
			List<SnapshotModel> snapshotModels = (List<SnapshotModel>) snapshotResult.getData();
			model.setSnapshotModels(snapshotModels);
			// 同步网络
			logger.info("start sync network data ......");
			List<? extends Network> networks = this.getClient().networking().network().list();
			// 过滤当前租户下的网络
			List<Network> filterNetworks = Lists.newArrayList();
			if (!ListTool.isEmpty(networks)) {
				for (Network network : networks) {
					// 过滤租户的或者共享的网络
					if (ResourcePercolator.networkFilter(network.getStatus())) {
						filterNetworks.add(network);
					}
				}
			}
			Map<String, String> imageParams = new HashMap<>();
			imageParams.put("limit", Integer.MAX_VALUE + "");
			logger.info("start sync image data ......");
			List<? extends Image> images = this.getClient().imagesV2().list(imageParams);
			logger.info("start sync flavors data ......");
			List<? extends Flavor> flavors = this.getClient().compute().flavors().list(true);
			logger.info("start sync ports data ......");
			List<? extends Port> ports = this.getClient().networking().port().list();
			logger.info("start sync router data ......");
			List<? extends Router> routers = this.getClient().networking().router().list();
			logger.info("start sync security data ......");
			List<? extends SecurityGroup> sGroups = this.getClient().networking().securitygroup().list();
			logger.info("start sync floatingIP data ......");
			List<? extends NetFloatingIP> floatingIPs = this.getClient().networking().floatingip().list();
			logger.info("start sync firewall data ......");
			List<? extends Firewall> firewalls = this.getClient().networking().firewalls().firewall().list();
			logger.info("start sync firewallPolicy data ......");
			List<? extends FirewallPolicy> policys = this.getClient().networking().firewalls().firewallpolicy().list();
			logger.info("start sync firewallRule data ......");
			List<? extends FirewallRule> rules = this.getClient().networking().firewalls().firewallrule().list();
			// 获取负责均衡相关对象
			logger.info("start sync loadbalance data ......");
			List<? extends LoadBalancerV2> lbs = this.getClient().networking().lbaasV2().loadbalancer().list();
			List<? extends ListenerV2> listeners = this.getClient().networking().lbaasV2().listener().list();
			// hostResource
			logger.info("start sync hostResources data ......");
			List<? extends HostResource> hostResources = this.getClient().compute().host().list();
			if (!ListTool.isEmpty(hostResources)) {
				List<ClusterModel> hostResourcesModels = new ArrayList<>();
				for (HostResource hostResource : hostResources) {
					if ("compute".equalsIgnoreCase(hostResource.getService()))
						hostResourcesModels.add(hostResoucesConvertor(hostResource));
				}
				model.setHostResourcesModels(hostResourcesModels);
			}
			if (this.getClient().supportsHeat()) {
				// 云编排 同步 栈 模板版本
				logger.info("start sync heat data ......");
				OpenStackHeatProvider provider = new OpenStackHeatProvider(butler);
				BocloudResult listStack = provider.listStack(null);
				if (listStack.isSuccess()) {
					model.setStackModels((List<HeatStackModel>) listStack.getData());
				}
				BocloudResult templateVersions = provider.templateVersions(null);
				if (templateVersions.isSuccess()) {
					model.setStackVersionModels((List<HeatStackModel>) templateVersions.getData());
				}
			}
			// 对象存储
			List<? extends SwiftContainer> containers = null;
			if (this.getClient().supportsObjectStorage()) {
				logger.info("start sync swiftcontainer data ......");
				containers = this.getClient().objectStorage().containers().list();
			}
			logger.info("start sync keypairs data ......");
			List<? extends Keypair> keypairs = this.getClient().compute().keypairs().list();
			if (!ListTool.isEmpty(zones)) {
				List<RegionModel> regions = Lists.newArrayList();
				for (AvailabilityZone aZones : zones) {
					RegionModel region = new RegionModel();
					region.setCode(aZones.getZoneName());
					region.setName(aZones.getZoneName());
					regions.add(region);
				}
				model.setRegions(regions);
			}
			if (!ListTool.isEmpty(hypervisors)) {
				BeanConvertor<HostModel, Hypervisor> convertHyperV = new HypervisorConvertor();
				model.setHostModels(convertHyperV.convertList(hypervisors));
			}
			if (!ListTool.isEmpty(filterServers)) {
				BeanConvertor<ServerModel, Server> convertServer = new ServerConvertor();
				model.setServerModels(convertServer.convertList(filterServers));
			}
			if (!ListTool.isEmpty(filterNetworks)) {
				BeanConvertor<NetworkModel, Network> convertNetworks = new NetworkConvertor();
				model.setNetworkModels(convertNetworks.convertList(filterNetworks));
			}
			if (!ListTool.isEmpty(images)) {
				BeanConvertor<ImageModel, Image> imageConvertor = new V2ImageConvertor();
				List<ImageModel> imageModels = Lists.newArrayList();
				for (Image image : images) {
					if (ResourcePercolator.imageFilter(image.getStatus())) {
						imageModels.add(imageConvertor.convertModel(image));
					}
				}
				model.setImageModels(imageModels);
			}
			if (!ListTool.isEmpty(flavors)) {
				BeanConvertor<FlavorModel, Flavor> convertFlavor = new FlavorConvertor();
				model.setFlavorModels(convertFlavor.convertList(flavors));
			}
			if (!ListTool.isEmpty(ports)) {
				BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
				model.setPortModels(convertPort.convertList(ports));
			}
			if (!ListTool.isEmpty(routers)) {
				BeanConvertor<RouterModel, Router> convertRouter = new RouterConvertor();
				model.setRouterModels(convertRouter.convertList(routers));
			}
			if (!ListTool.isEmpty(sGroups)) {
				BeanConvertor<SecurityGroupModel, SecurityGroup> convertGroup = new SecurityGroupConvertor();
				model.setGroupModels(convertGroup.convertList(sGroups));
			}
			if (!ListTool.isEmpty(floatingIPs)) {
				BeanConvertor<FloatingIpModel, NetFloatingIP> convertFip = new FloatingIpConvertor();
				List<FloatingIpModel> fips = Lists.newArrayList();
				BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
				for (NetFloatingIP filterFip : floatingIPs) {
					if (!StringUtils.isEmpty(filterFip.getPortId())) {
						Port port = this.getClient().networking().port().get(filterFip.getPortId());
						List<PortModel> portModels = Lists.newArrayList();
						if (null != port) {
							portModels.add(convertPort.convertModel(port));
						}
						FloatingIpModel fIpModel = convertFip.convertModel(filterFip);
						fIpModel.setPorts(portModels);
						fips.add(fIpModel);
					} else {
						fips.add(convertFip.convertModel(filterFip));
					}
				}
				model.setfIpModels(fips);
			}
			if (!ListTool.isEmpty(firewalls)) {
				BeanConvertor<FirewallModel, Firewall> convertFirewall = new FirewallConvertor();
				model.setFirewallModels(convertFirewall.convertList(firewalls));
			}
			if (!ListTool.isEmpty(policys)) {
				BeanConvertor<FirewallPolicyModel, FirewallPolicy> convertPolicy = new FirewallPolicyConvertor();
				model.setFirewallPolicyModels(convertPolicy.convertList(policys));
			}
			if (!ListTool.isEmpty(rules)) {
				BeanConvertor<FirewallRuleModel, FirewallRule> convertRule = new FirewallRuleConvertor();
				model.setFirewallRuleModels(convertRule.convertList(rules));
			}
			if (!ListTool.isEmpty(keypairs)) {
				BeanConvertor<KeypairModel, Keypair> convertKeypair = new KeypairConvertor();
				model.setKeypairModels(convertKeypair.convertList(keypairs));
			}
			if (!ListTool.isEmpty(aggregates)) {
				List<ClusterModel> clusterModels = Lists.newArrayList();
				for (HostAggregate aggregate : aggregates) {
					clusterModels.add(convertAggregate(aggregate));
				}
				model.setClusterModels(clusterModels);
			}
			Map<String, LbListenerModel> listenerMap = Maps.newHashMap();
			if (!ListTool.isEmpty(listeners)) {
				BeanConvertor<LbListenerModel, ListenerV2> convertListener = new LbListenerConvertor();
				List<LbListenerModel> listenerModels = convertListener.convertList(listeners);
				BeanConvertor<LbPoolModel, LbPoolV2> convertLbPool = new LbPoolConvertor();
				BeanConvertor<HealthMonitorModel, HealthMonitorV2> convertHm = new HealthMonitorConvertor();
				for (LbListenerModel listenerModel : listenerModels) {
					String poolUuid = listenerModel.getDefaultPoolUuid();
					if (StringUtils.isEmpty(poolUuid)) {
						listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
						continue;
					}
					LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().get(poolUuid);
					LbPoolModel poolModel = convertLbPool.convertModel(pool);
					if (!StringUtils.isEmpty(poolModel.getHmUuid())) {
						HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor()
								.get(poolModel.getHmUuid());
						HealthMonitorModel monitorModel = convertHm.convertModel(monitor);
						poolModel.setModel(monitorModel);
					}
					listenerModel.setPoolModel(poolModel);
					List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool()
							.listMembers(poolUuid);
					if (!ListTool.isEmpty(members)) {
						BeanConvertor<LbMemberModel, MemberV2> convertMember = new LbMemberConvertor();
						List<LbMemberModel> memberModels = convertMember.convertList(members);
						for (LbMemberModel memberModel : memberModels) {
							memberModel.setPoolUuid(poolUuid);
						}
						listenerModel.setModels(memberModels);
					}
					listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
				}
			}
			if (!ListTool.isEmpty(lbs)) {
				BeanConvertor<LbModel, LoadBalancerV2> convertLb = new LbConvertor();
				List<LbModel> lbModels = convertLb.convertList(lbs);
				for (LbModel lbModel : lbModels) {
					List<LbListenerModel> listenerModels = Lists.newArrayList();
					List<String> list = lbModel.getListeners();
					for (String uuid : list) {
						listenerModels.add(listenerMap.get(uuid));
					}
					lbModel.setModels(listenerModels);
				}
				model.setLbModels(lbModels);
			}
			if (!ListTool.isEmpty(containers)) {
				BeanConvertor<ObjectStorageContainerModel, SwiftContainer> covertOsc = new ObjectStorageContainerConvertor(
						this);
				model.setObjectStorageContainerModels(covertOsc.convertList(containers));
			}
			// 同步vpn service
			OpenStackVpnProvider openStackVpnProvider = new OpenStackVpnProvider(butler);
			logger.info("start sync vpn data ......");
			BocloudResult bocloudResult = openStackVpnProvider.list();
			if (bocloudResult.isSuccess()) {
				List<VpnModel> vpnModels = (List<VpnModel>) bocloudResult.getData();
				model.setVpnModels(vpnModels);
			}
			// 同步 vpn policy
			List<VpnPolicyModel> vpnPolicyModels = new ArrayList<>();
			BocloudResult vpnIkePolicyResult = openStackVpnProvider.vpnIkePolicylist();
			if (vpnIkePolicyResult.isSuccess()) {
				List<VpnPolicyModel> vpnIkePolicylist = (List<VpnPolicyModel>) vpnIkePolicyResult.getData();
				if (!ListTool.isEmpty(vpnIkePolicylist)) {
					vpnPolicyModels.addAll(vpnIkePolicylist);
				}
			}
			BocloudResult vpnIPsecPolicyResult = openStackVpnProvider.vpnIPsecPolicylist();
			if (vpnIPsecPolicyResult.isSuccess()) {
				List<VpnPolicyModel> vpnIPsecPolicylist = (List<VpnPolicyModel>) vpnIPsecPolicyResult.getData();
				if (!ListTool.isEmpty(vpnIPsecPolicylist)) {
					vpnPolicyModels.addAll(vpnIPsecPolicylist);
				}
			}
			model.setVpnPolicyModels(vpnPolicyModels);
			// 同步vpn connection
			OpenStackVpnConnProvider openStackVpnConnProvider = new OpenStackVpnConnProvider(butler);
			BocloudResult vpnConnResult = openStackVpnConnProvider.list();
			if (vpnConnResult.isSuccess()) {
				List<VpnConnModel> vpnConnModes = (List<VpnConnModel>) vpnConnResult.getData();
				model.setVpnConnModes(vpnConnModes);
			}
			return new BocloudResult(true, model, "success");
		} catch (Exception e) {
			logger.info("OpenStack资源同步失败：" + e);
			return new BocloudResult<>(false, "OpenStack资源同步失败：" + e);
		}
	}

	public ClusterModel hostResoucesConvertor(HostResource hostResource) {
		ClusterModel model = new ClusterModel();
		if (null != hostResource) {
			model.setName(hostResource.getHostName());
			model.setCpu(hostResource.getCpu());
			model.setDisk(hostResource.getDiskInGb());
			model.setMemory(hostResource.getMemoryInMb());
			model.setService(hostResource.getService());
			model.setZone(hostResource.getZone());
		}
		return model;
	}

	@Override
	public BocloudResult syncSecurityGroups() {
		List<? extends SecurityGroup> sGroups = this.getClient().networking().securitygroup().list();
		List<SecurityGroupModel> groupModels = new ArrayList<>();
		if (!ListTool.isEmpty(sGroups)) {
			BeanConvertor<SecurityGroupModel, SecurityGroup> convertGroup = new SecurityGroupConvertor();
			groupModels = convertGroup.convertList(sGroups);
		}
		return new BocloudResult(true, groupModels, "查询成功");
	}

	@Override
	public BocloudResult syncPortByNetworkUuid(String networkUuid) {
		OSSyncModel model = new OSSyncModel();
		List<? extends Port> ports = this.getClient().networking().port().list();
		if (null != ports && !ports.isEmpty()) {
			BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
			List<PortModel> portModels = convertPort.convertList(ports);
			List<PortModel> result = new ArrayList<>();
			if (portModels != null && portModels.size() > 0) {
				for (PortModel portModel : portModels) {
					if (portModel.getNetworkUuid().contains(networkUuid)) {
						result.add(portModel);
					}
				}
			}
			model.setPortModels(result);
		}

		return new BocloudResult(true, model, "success");
	}

	@Override
	public BocloudResult getHostModel(HostModel hostModel) {
		return null;
	}

	private ClusterModel convertAggregate(HostAggregate aggregate) {
		ClusterModel clusterModel = new ClusterModel();
		clusterModel.setName(aggregate.getName());
		clusterModel.setRegion(aggregate.getAvailabilityZone());
		clusterModel.setUuid(aggregate.getId());
		clusterModel.setHostNames(aggregate.getHosts());
		Map<String, String> meteDataMap = aggregate.getMetadata();
		if (null != meteDataMap) {
			clusterModel.setRemark(JSONObject.toJSONString(aggregate.getMetadata()));
		}
		return clusterModel;
	}

	@Override
	public BocloudResult listZones(String region) {
		return null;
	}

	@Override
	public BocloudResult syncStack(LbListenerModel lbListenerModel) {
		try {
			OSSyncModel model = new OSSyncModel();
			Map<String, LbListenerModel> listenerMap = Maps.newHashMap();
			String poolUuid = lbListenerModel.getDefaultPoolUuid();
			String lbUuid = lbListenerModel.getLbUuid();
			String listenerUuid = lbListenerModel.getListenerUuid();
			BeanConvertor<LbPoolModel, LbPoolV2> convertLbPool = new LbPoolConvertor();
			BeanConvertor<HealthMonitorModel, HealthMonitorV2> convertHm = new HealthMonitorConvertor();
			ListenerV2 listenerV2 = this.getClient().networking().lbaasV2().listener().get(listenerUuid);
			BeanConvertor<LbListenerModel, ListenerV2> convertListener = new LbListenerConvertor();
			LbListenerModel listenerModel = convertListener.convertModel(listenerV2);
			LbPoolV2 pool = this.getClient().networking().lbaasV2().lbPool().get(poolUuid);
			LbPoolModel poolModel = convertLbPool.convertModel(pool);
			if (!StringUtils.isEmpty(poolModel.getHmUuid())) {
				HealthMonitorV2 monitor = this.getClient().networking().lbaasV2().healthMonitor()
						.get(poolModel.getHmUuid());
				HealthMonitorModel monitorModel = convertHm.convertModel(monitor);
				poolModel.setModel(monitorModel);
			}
			listenerModel.setPoolModel(poolModel);
			List<? extends MemberV2> members = this.getClient().networking().lbaasV2().lbPool().listMembers(poolUuid); // poolUuid
			if (!ListTool.isEmpty(members)) {
				BeanConvertor<LbMemberModel, MemberV2> convertMember = new LbMemberConvertor();
				List<LbMemberModel> memberModels = convertMember.convertList(members);
				for (LbMemberModel memberModel : memberModels) {
					memberModel.setPoolUuid(poolUuid);
				}
				listenerModel.setModels(memberModels);
			}
			listenerMap.put(listenerModel.getListenerUuid(), listenerModel);
			LoadBalancerV2 loadBalancerV2 = this.getClient().networking().lbaasV2().loadbalancer().get(lbUuid);
			BeanConvertor<LbModel, LoadBalancerV2> convertLb = new LbConvertor();
			LbModel lbModels = convertLb.convertModel(loadBalancerV2);
			List<LbListenerModel> listenerModels = Lists.newArrayList();
			List<String> list = lbModels.getListeners();
			for (String uuid : list) {
				listenerModels.add(listenerMap.get(uuid));
			}
			lbModels.setModels(listenerModels);
			List<LbModel> lbList = new ArrayList<>();
			lbList.add(lbModels);
			model.setLbModels(lbList);

			String vipPortUuid = lbModels.getVipPortUuid();
			if (!StringUtils.isEmpty(vipPortUuid)) {
				Port port = this.getClient().networking().port().get(vipPortUuid);
				BeanConvertor<PortModel, Port> convertPort = new PortConvertor();
				List<PortModel> portModels = new ArrayList<>();
				portModels.add(convertPort.convertModel(port));
				model.setPortModels(portModels);
			}
			return new BocloudResult(true, model, "success");
		} catch (Exception e) {
			return new BocloudResult(false, "伸缩组同步负载均衡异常：" + e.getMessage());
		}
	}

}
