package com.bocloud.cmp.provider.compute;

import java.util.Map;

import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.HostAggregate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.ClusterOperateModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * openstack主机组操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackHostAggregateProvider extends OpenStackProvider implements ClusterProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackHostAggregateProvider.class);

    public OpenStackHostAggregateProvider(Butler butler) {
        super(butler);
    }

    /**
     * 创建集群
     *
     * @param model
     * @return
     */
    @Override
    public BocloudResult create(ClusterOperateModel model) {
        return null;
    }

    /**
     * 删除集群
     *
     * @param model
     * @return
     */
    @Override
    public BocloudResult remove(ClusterOperateModel model) {
        return null;
    }

    /**
     * 获取集群信息
     * <p>
     * 用于vCenter云平台，只查询cpu、内存、磁盘的总量和使用信息
     * </p>
     *
     * @param clusterVal 集群的标识
     * @return 集群信息
     * @author sxp
     * @since Apr 26, 2018
     */
    @Override
    public BocloudResult detail(String clusterVal) {
        return null;
    }

    @Override
    public BocloudResult create(ClusterModel clusterModel) {
        try {
            HostAggregate aggregate = this.getClient().compute().hostAggregates().create(clusterModel.getName(),
                    clusterModel.getRegion());
            if (null == aggregate) {
                return new BocloudResult(false, "创建失败");
            }
            aggregate = this.getClient().compute().hostAggregates().get(aggregate.getId());
            if (null == aggregate) {
                return new BocloudResult(false, "创建失败");
            }
            return new BocloudResult(true, convertAggregate(aggregate), "创建成功");
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ClusterModel clusterModel) {
        try {
            HostAggregate aggregate = this.getClient().compute().hostAggregates().update(clusterModel.getUuid(),
                    clusterModel.getName(), clusterModel.getRegion());
            if (null == aggregate) {
                return new BocloudResult(false, "修改失败");
            }
            return new BocloudResult(true, convertAggregate(aggregate), "修改成功");
        } catch (Exception e) {
            logger.error("Get error message", e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String aggregateId) {
        HostAggregate aggregate = this.getClient().compute().hostAggregates().get(aggregateId);
        if (null == aggregate) {
            logger.info("host aggregate not exist!");
            return new BocloudResult(true, "主机聚合不存在");
        }
        ActionResponse response = this.getClient().compute().hostAggregates().delete(aggregateId);
        if (response.isSuccess()) {
            return new BocloudResult(true, "删除成功！");
        } else {
            logger.error("remove host aggregate [{}] failed :{}", aggregateId, response.getFault());
            return new BocloudResult(false, response.getFault());
        }
    }

    @Override
    public BocloudResult addHost(String aggregateId, String hostName) {
        HostAggregate aggregate = this.getClient().compute().hostAggregates().addHost(aggregateId, hostName);
        if (null == aggregate) {
            return new BocloudResult(false, "主机添加失败");
        }
        return new BocloudResult(true, "主机添加成功");
    }

    @Override
    public BocloudResult removeHost(String aggregateId, String hostName) {
        HostAggregate aggregate = this.getClient().compute().hostAggregates().removeHost(aggregateId, hostName);
        if (null == aggregate) {
            return new BocloudResult(false, "主机移除失败");
        }
        return new BocloudResult(true, "主机移除成功");
    }

    @Override
    public BocloudResult summary(String clusterVal) {
        return null;
    }

    @Override
    public BocloudResult queryByName(String name) {
        return null;
    }

    private ClusterModel convertAggregate(HostAggregate aggregate) {
        ClusterModel clusterModel = new ClusterModel();
        clusterModel.setName(aggregate.getName());
        clusterModel.setRegion(aggregate.getAvailabilityZone());
        clusterModel.setUuid(aggregate.getId());
        clusterModel.setHostNames(aggregate.getHosts());
        Map<String, String> meteDataMap = aggregate.getMetadata();
        if (null != meteDataMap) {
            clusterModel.setRemark(JSONObject.toJSONString(aggregate.getMetadata()));
        }
        return clusterModel;
    }
}

