package com.bocloud.cmp.provider.convertor;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.telemetry.Alarm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.AlarmModel;
import com.google.common.collect.Lists;

/**
 * Openstack告警数据转换器
 * 
 * @author weiwei
 *
 */
public class AlarmConvertor implements BeanConvertor<AlarmModel, Alarm> {
	private static Logger logger = LoggerFactory.getLogger(AlarmConvertor.class);

	@Override
	public List<AlarmModel> convertList(List<? extends Alarm> models) {
		List<AlarmModel> alarmModels = Lists.newArrayList();
		for (Alarm alarm : models) {
			alarmModels.add(convertModel(alarm));
		}
		return alarmModels;
	}

	@Override
	public AlarmModel convertModel(Alarm alarm) {
		if (null == alarm) {
			return null;
		}
		AlarmModel model = new AlarmModel();
		model.setName(alarm.getName());
		model.setType(null != alarm.getType() ? alarm.getType().toString() : "");
		model.setRemark(alarm.getDescription());
		String tigger = alarm.getTimestamp();
		if (!StringUtils.isEmpty(tigger)) {
			model.setGmtTrigger(stringToDate(tigger));
		}
		String confirm = alarm.getStateTimestamp();
		if (!StringUtils.isEmpty(tigger)) {
			model.setGmtConfirm(stringToDate(confirm));
		}
		model.setLevel(alarm.getState());
		model.setAlarmVal(alarm.getAlarmId());
		model.setConfirmer(alarm.getUserId());
		String gmtConfirm = alarm.getTimestamp();
		if (!StringUtils.isEmpty(gmtConfirm)) {
			model.setGmtConfirm(stringToDate(gmtConfirm));
		}
		model.setTarget(alarm.getDescription());

		return model;
	}

	private Date stringToDate(String timetamp) {
		DateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		try {
			return fmt.parse(timetamp.replace("T", " "));
		} catch (ParseException e) {
			logger.error("String to Date Error {}", e);
		}
		return null;
	}
}
