package com.bocloud.cmp.provider.convertor;

import java.util.List;

import org.openstack4j.model.network.ext.FirewallPolicy;
import org.openstack4j.model.network.ext.FirewallRule;

import com.bocloud.cmp.model.FirewallPolicyModel;
import com.bocloud.cmp.model.FirewallRuleModel;
import com.google.common.collect.Lists;

/**
 * Openstack防火墙策略数据转换器
 * 
 * @author weiwei
 *
 */
public class FirewallPolicyConvertor implements BeanConvertor<FirewallPolicyModel, FirewallPolicy> {

	@Override
	public List<FirewallPolicyModel> convertList(List<? extends FirewallPolicy> firewallPolicys) {
		List<FirewallPolicyModel> models = Lists.newArrayList();
		for (FirewallPolicy firewallPolicy : firewallPolicys) {
			models.add(convertModel(firewallPolicy));
		}
		return models;
	}

	@Override
	public FirewallPolicyModel convertModel(FirewallPolicy firewallPolicy) {
		FirewallPolicyModel model = new FirewallPolicyModel();
		model.setPolicyUuid(firewallPolicy.getId());
		model.setName(firewallPolicy.getName());
		model.setTenantUuid(firewallPolicy.getTenantId());
		model.setRemark(firewallPolicy.getDescription());
		model.setShared(firewallPolicy.isShared());
		model.setAudited(firewallPolicy.isAudited());
		model.setRuleUuids(firewallPolicy.getFirewallRuleIds());
		model.setFirewallUuids(firewallPolicy.getFirewallList());
		List<? extends FirewallRule> firewallRules = firewallPolicy.getNeutronFirewallRules();
		List<FirewallRuleModel> rules = Lists.newArrayList();
		BeanConvertor<FirewallRuleModel, FirewallRule> convertRule = new FirewallRuleConvertor();
		if (!firewallRules.isEmpty()) {
			for (FirewallRule firewallRule : firewallRules) {
				rules.add(convertRule.convertModel(firewallRule));
			}
		}
		model.setRules(rules);
		return model;
	}
}
