package com.bocloud.cmp.provider.convertor;

import java.math.BigDecimal;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.model.compute.Flavor;

import com.bocloud.cmp.model.FlavorModel;
import com.google.common.collect.Lists;

/**
 * Openstack配置规格数据转换器
 *
 * @author weiwei
 */
public class FlavorConvertor implements BeanConvertor<FlavorModel, Flavor> {

	@Override
	public List<FlavorModel> convertList(List<? extends Flavor> flavors) {
		List<FlavorModel> modes = Lists.newArrayList();
		for (Flavor flavor : flavors) {
			if (!StringUtils.isEmpty(flavor.getId())) { // 发现存在ID为空的规格 过滤掉
				modes.add(this.convertModel(flavor));
			}
		}
		return modes;
	}

	@Override
	public FlavorModel convertModel(Flavor flavor) {
		FlavorModel model = new FlavorModel();
		model.setCpu(flavor.getVcpus());
		model.setMemory(
				BigDecimal.valueOf(flavor.getRam()).divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP));
		model.setName(flavor.getName());
		model.setFamily(null);
		model.setDisk(flavor.getDisk());
		model.setFlavorUuid(flavor.getId());
		return model;
	}

}
