package com.bocloud.cmp.provider.convertor;

import java.util.List;

import org.openstack4j.model.network.ext.HealthMonitorV2;
import org.openstack4j.openstack.networking.domain.ext.ListItem;

import com.bocloud.cmp.model.HealthMonitorModel;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * Openstack监控检查数据转换器
 *
 * @author weiwei
 */
public class HealthMonitorConvertor implements BeanConvertor<HealthMonitorModel, HealthMonitorV2> {

    @Override
    public List<HealthMonitorModel> convertList(List<? extends HealthMonitorV2> healthMonitorV2s) {
        List<HealthMonitorModel> healthMonitorModels = Lists.newArrayList();
        for (HealthMonitorV2 healthMonitorV2 : healthMonitorV2s) {
            healthMonitorModels.add(convertModel(healthMonitorV2));
        }
        return healthMonitorModels;
    }

    @Override
    public HealthMonitorModel convertModel(HealthMonitorV2 healthMonitorV2) {
        HealthMonitorModel model = new HealthMonitorModel();
        model.setHealthUuid(healthMonitorV2.getId());
        model.setTenantUuid(healthMonitorV2.getTenantId());
        model.setType(healthMonitorV2.getType().name());
        model.setDelay(healthMonitorV2.getDelay());
        model.setTimeout(healthMonitorV2.getTimeout());
        model.setMaxRetries(healthMonitorV2.getMaxRetries());
        model.setHttpMethod(healthMonitorV2.getHttpMethod());
        model.setUrlPath(healthMonitorV2.getUrlPath());
        model.setExpectedCodes(healthMonitorV2.getExpectedCodes());
        model.setAdminStateUp(healthMonitorV2.isAdminStateUp());
        List<ListItem> items = healthMonitorV2.getPools();
        List<String> pools = Lists.newArrayList();
        if (!ListTool.isEmpty(items)) {
            for (ListItem item : items) {
                pools.add(item.getId());
            }
        }
        model.setPools(pools);
        return model;
    }

}
