package com.bocloud.cmp.provider.convertor;

import java.math.BigDecimal;
import java.util.List;

import org.openstack4j.model.compute.ext.Hypervisor;
import org.openstack4j.model.compute.ext.Hypervisor.CPUInfo;
import org.openstack4j.model.compute.ext.Hypervisor.CPUTopology;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * Openstack宿主机数据转换器
 *
 * @author weiwei
 */
public class HypervisorConvertor implements BeanConvertor<HostModel, Hypervisor> {

    @Override
    public List<HostModel> convertList(List<? extends Hypervisor> hyperVs) {
        List<HostModel> hostModels = Lists.newArrayList();
        for (Hypervisor hyperv : hyperVs) {
            hostModels.add(convertModel(hyperv));
        }
        return hostModels;
    }

    @Override
    public HostModel convertModel(Hypervisor hyperv) {
        HostModel hostModel = new HostModel();
        hostModel.setUuid(hyperv.getId());// uuid
        hostModel.setCatalog("OPENSTACK");
        hostModel.setName(hyperv.getHypervisorHostname());
        hostModel.setHostName(hyperv.getService().getHost());
        hostModel.setHostIp(hyperv.getHostIP());
        hostModel.setCpuTotal((long) hyperv.getVirtualCPU());
        double memory = new BigDecimal(hyperv.getLocalMemory())
                .divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
        hostModel.setMemTotal(memory);
        hostModel.setDiskTotal((double) hyperv.getLocalDisk());
        hostModel.setCpuUsed((long) hyperv.getVirtualUsedCPU());
        double memUsed = new BigDecimal(hyperv.getLocalMemoryUsed())
                .divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
        hostModel.setMemUsed(memUsed);
        hostModel.setDiskUsed((double) hyperv.getLocalDiskUsed());
        CPUInfo cpuInfo = hyperv.getCPUInfo();
        if (null != cpuInfo) {
            List<Object> model = cpuInfo.getModel();
            if (ListTool.isEmpty(model)) {
                hostModel.setCpuModel(null);
            } else {
                hostModel.setCpuModel(JSONArray.toJSONString(hyperv.getCPUInfo().getModel()));
            }
        } else {
            hostModel.setCpuModel(null);
        }
        if (null != cpuInfo) {
            List<Object> vendor = cpuInfo.getVendor();
            if (ListTool.isEmpty(vendor)) {
                hostModel.setVendor(null);
            } else {
                hostModel.setVendor(JSONArray.toJSONString(hyperv.getCPUInfo().getVendor()));
            }
        } else {
            hostModel.setVendor(null);
        }
        hostModel.setVmNum(hyperv.getRunningVM());
        hostModel.setPowerState(hyperv.getState());
        hostModel.setStatus(hyperv.getStatus());
        if (null != cpuInfo) {
            CPUTopology topology = cpuInfo.getTopology();
            if (null != topology) {
                hostModel.setCpuNum(topology.getSockets());
                hostModel.setCpuCore(topology.getCores());
            } else {
                hostModel.setCpuNum(0);
                hostModel.setCpuCore(0);
            }
        }
        return hostModel;
    }

}