package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;

import org.openstack4j.model.network.ext.LbPoolV2;
import org.openstack4j.model.network.ext.SessionPersistence;
import org.openstack4j.openstack.networking.domain.ext.ListItem;

import com.bocloud.cmp.model.LbPoolModel;
import com.google.common.collect.Lists;

/**
 * 负载均衡器成员池数据转换器
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月4日
 */
public class LbPoolConvertor implements BeanConvertor<LbPoolModel, LbPoolV2> {

    @Override
    public List<LbPoolModel> convertList(List<? extends LbPoolV2> lbPoolV2s) {
        List<LbPoolModel> models = Lists.newArrayList();
        for (LbPoolV2 lbPoolV2 : lbPoolV2s) {
            models.add(convertModel(lbPoolV2));
        }
        return models;
    }

    @Override
    public LbPoolModel convertModel(LbPoolV2 lbPoolV2) {
        LbPoolModel model = new LbPoolModel();
        model.setPoolUuid(lbPoolV2.getId());
        model.setTenantUuid(lbPoolV2.getTenantId());
        model.setName(lbPoolV2.getName());
        model.setRemark(lbPoolV2.getDescription());
        model.setProtocol(lbPoolV2.getProtocol().name());
        model.setLbMethod(lbPoolV2.getLbMethod().name());
        SessionPersistence persistence = lbPoolV2.getSessionPersistence();
        if (null != persistence) {
            model.setCookieName(persistence.getCookieName());
            model.setType(null == persistence.getType() ? "" : persistence.getType().name());
        }
        model.setAdminStateUp(lbPoolV2.isAdminStateUp());
        List<ListItem> listeners = lbPoolV2.getListeners();
        if (listeners.isEmpty()) {
            model.setListenerUuids(new ArrayList<>());
        } else {
            List<String> listenerUuids = Lists.newArrayList();
            for (ListItem item : listeners) {
                listenerUuids.add(item.getId());
            }
            model.setListenerUuids(listenerUuids);
        }
        List<ListItem> members = lbPoolV2.getMembers();
        if (members.isEmpty()) {
            model.setMemberUuids(new ArrayList<>());
        } else {
            List<String> memberUuids = Lists.newArrayList();
            for (ListItem item : members) {
                memberUuids.add(item.getId());
            }
            model.setMemberUuids(memberUuids);
        }
        model.setHmUuid(lbPoolV2.getHealthMonitorId());
        return model;
    }

}
