package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.List;

import org.openstack4j.model.network.HostRoute;
import org.openstack4j.model.network.Network;
import org.openstack4j.model.network.Pool;
import org.openstack4j.model.network.Subnet;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.utils.StatusConverter;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * Openstack网络数据转换器
 *
 * @author weiwei
 */
public class NetworkConvertor implements BeanConvertor<NetworkModel, Network> {

    @Override
    public List<NetworkModel> convertList(List<? extends Network> networks) {
        List<NetworkModel> models = Lists.newArrayList();
        for (Network network : networks) {
            models.add(convertModel(network));
        }
        return models;
    }

    @Override
    public NetworkModel convertModel(Network network) {
        if (null == network) {
            return null;
        }
        NetworkModel model = new NetworkModel();
        model.setAdminStateup(network.isAdminStateUp());
        model.setName(network.getName());
        model.setNetworkUuid(network.getId());
        model.setShared(network.isShared());
        model.setStatus(StatusConverter.Network.converter(network.getStatus().name()));
        model.setType(null == network.getNetworkType() ? null : network.getNetworkType().name());
        model.setIsRouterExternal(network.isRouterExternal());
        model.setPhyNetwork(network.getProviderPhyNet());
        model.setSegmentationId(network.getProviderSegID());
        model.setTenantUuid(network.getTenantId());
        List<? extends Subnet> subnets = network.getNeutronSubnets();
        List<SubnetModel> subnetModels = Lists.newArrayList();
        if (null != subnets) {
            for (Subnet subnet : subnets) {
                subnetModels.add(convertSubnet(subnet));
            }
        }
        model.setSubnets(subnetModels);
        return model;
    }

    private SubnetModel convertSubnet(Subnet subnet) {
        SubnetModel model = new SubnetModel();
        if (null != subnet) {
            model.setName(subnet.getName());
            if (!ListTool.isEmpty(subnet.getAllocationPools())) {
                List<String> ipPools = new ArrayList<>();
                for (Pool pool : subnet.getAllocationPools()) {
                    ipPools.add(pool.getStart() + "-" + pool.getEnd());
                }
                model.setIpPools(JSONObject.toJSONString(ipPools));
            }
            model.setCidr(subnet.getCidr());
            model.setDhcpEnabled(subnet.isDHCPEnabled());
            model.setDnsNames(JSONObject.toJSONString(subnet.getDnsNames()));
            model.setSubnetUuid(subnet.getId());
            model.setIpVersion(subnet.getIpVersion().name());
            model.setNetworkUuid(subnet.getNetworkId());
            model.setGateway(subnet.getGateway());
            model.setTenantUuid(subnet.getTenantId());
            if (!ListTool.isEmpty(subnet.getHostRoutes())) {
                List<String> hostRouteList = new ArrayList<>();
                for (HostRoute hostRoute : subnet.getHostRoutes()) {
                    hostRouteList.add(hostRoute.getDestination() + "," + hostRoute.getNexthop());
                }
                model.setHostRoutes(JSONObject.toJSONString(hostRouteList));
            }
        }
        return model;
    }

}
