package com.bocloud.cmp.provider.convertor;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.storage.ObjectStorageObjectService;
import org.openstack4j.model.storage.object.SwiftContainer;
import org.openstack4j.model.storage.object.SwiftObject;

import com.bocloud.cmp.model.ObjectStorageContainerModel;
import com.bocloud.cmp.model.ObjectStorageItemModel;
import com.bocloud.cmp.provider.common.OpenStackLocationProvider;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * 对象存储转换
 *
 * @author hufy
 * @version 1.0
 * @since 2017年9月4日
 */
public class ObjectStorageContainerConvertor implements BeanConvertor<ObjectStorageContainerModel, SwiftContainer> {

    private OpenStackLocationProvider openStackLocationProvider;

    public ObjectStorageContainerConvertor(OpenStackLocationProvider openStackLocationProvider) {
        this.openStackLocationProvider = openStackLocationProvider;
    }

    @Override
    public List<ObjectStorageContainerModel> convertList(List<? extends SwiftContainer> containers) {
        List<ObjectStorageContainerModel> models = Lists.newArrayList();
        for (SwiftContainer container : containers) {
            if (container != null) {
                models.add(convertModel(container));
            }
        }
        return models;
    }

    @Override
    public ObjectStorageContainerModel convertModel(SwiftContainer model) {
        ObjectStorageObjectService objectService = openStackLocationProvider.getClient().objectStorage().objects();
        ObjectStorageContainerModel container = new ObjectStorageContainerModel();
        container.setName(model.getName());
        container.setItemCount(model.getTotalSize());
        Map<String, String> metadata = model.getMetadata();
        container.setCreateTime(new Date(Double.valueOf(metadata.get("X-Timestamp")).longValue() * 1000));
        container.setItemCount(Long.parseLong(metadata.get("X-Container-Object-Count")));
        container.setSize(Long.parseLong(metadata.get("X-Container-Bytes-Used")));

        List<ObjectStorageItemModel> items = new ArrayList<>();

        List<? extends SwiftObject> list = objectService.list(model.getName());
        if (!ListTool.isEmpty(list)) {
            for (SwiftObject swiftObject : list) {
                ObjectStorageItemModel storageItem = convertToItem(swiftObject);
                items.add(storageItem);
            }
        }
        container.setItems(items);
        return container;

    }

    private ObjectStorageItemModel convertToItem(SwiftObject swiftObject) {
        ObjectStorageItemModel storageItem = new ObjectStorageItemModel();
        storageItem.setContainerName(swiftObject.getContainerName());
        storageItem.setContentType(swiftObject.getMimeType());
        storageItem.setDirectoryName(swiftObject.getDirectoryName());
        storageItem.setSize(swiftObject.getSizeInBytes());
        // storageItem.setIsDirectory(swiftObject.isDirectory()?1:0); 99cloud不能用这个判断
        // FIXME
        storageItem
                .setIsDirectory(((StringUtils.isBlank(swiftObject.getMimeType()) && swiftObject.getSizeInBytes() == 0)
                        || "application/pseudo-folder".equals(swiftObject.getMimeType())) ? 1 : 0);
        String name = swiftObject.getName();

        if (name.indexOf("/") == 0) { // 处理首位 "/" 99cloud那边,对首位/的目录处理有问题
            name = name.substring(1);
        }
        storageItem.setName(name); // 目录的名称最后一位是/
        storageItem.setUpdateTime(swiftObject.getLastModified());
        storageItem.setHash(swiftObject.getETag());
        Map<String, String> storageMeta = swiftObject.getMetadata();
        if (storageMeta != null) {
            storageItem.setOrigFilename(storageMeta.get("Orig-Filename"));
        }
        return storageItem;
    }
}
