package com.bocloud.cmp.provider.convertor;

import java.util.Date;
import java.util.List;

import org.openstack4j.model.network.SecurityGroup;
import org.openstack4j.model.network.SecurityGroupRule;

import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.google.common.collect.Lists;

/**
 * Openstack安全组数据转换器
 *
 * @author weiwei
 */
public class SecurityGroupConvertor implements BeanConvertor<SecurityGroupModel, SecurityGroup> {

    @Override
    public List<SecurityGroupModel> convertList(List<? extends SecurityGroup> groups) {
        List<SecurityGroupModel> models = Lists.newArrayList();
        for (SecurityGroup group : groups) {
            models.add(convertModel(group));
        }
        return models;
    }

    @Override
    public SecurityGroupModel convertModel(SecurityGroup group) {
        SecurityGroupModel model = new SecurityGroupModel();
        model.setCreateTime(new Date());
        model.setGroupUuid(group.getId());
        model.setTenantUuid(group.getTenantId());
        model.setName(group.getName());
        model.setRemark(group.getDescription());
        List<? extends SecurityGroupRule> rules = group.getRules();
        List<SecurityGroupRuleModel> models = Lists.newArrayList();
        for (SecurityGroupRule rule : rules) {
            models.add(convertRule(rule));
        }
        model.setModels(models);
        return model;
    }

    private SecurityGroupRuleModel convertRule(SecurityGroupRule rule) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        model.setPortMin(rule.getPortRangeMin());
        model.setPortMax(rule.getPortRangeMax());
        if (null != rule.getProtocol() && !"".equals(rule.getProtocol())) {
            model.setProtocol(rule.getProtocol().toUpperCase());
        } else {
            model.setProtocol(rule.getProtocol());
        }
        model.setRuleUuid(rule.getId());
        model.setGroupUuid(rule.getSecurityGroupId());
        model.setDirection(rule.getDirection());
        model.setEtherType(rule.getEtherType());
        model.setRemoteIpPrefix(rule.getRemoteIpPrefix());
        model.setRemoteGroupUuid(rule.getRemoteGroupId());
        model.setTenantUuid(rule.getTenantId());
        return model;
    }

}
