package com.bocloud.cmp.provider.convertor;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.openstack4j.model.compute.Address;
import org.openstack4j.model.compute.SecurityGroup;
import org.openstack4j.model.compute.Server;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * Openstack云主机数据转换器
 *
 * @author weiwei
 */
public class ServerConvertor implements BeanConvertor<ServerModel, Server> {

    @Override
    public List<ServerModel> convertList(List<? extends Server> servers) {
        List<ServerModel> models = Lists.newArrayList();
        for (Server server : servers) {
            models.add(convertModel(server));
        }
        return models;
    }

    @Override
    public ServerModel convertModel(Server server) {
        if (null == server) {
            return null;
        }
        ServerModel model = new ServerModel();
        if (null != server.getFlavor()) {
            model.setCpu(server.getFlavor().getVcpus());
            model.setDisk(server.getFlavor().getDisk());
            model.setMemory(BigDecimal.valueOf(server.getFlavor().getRam()).divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP));
        }
        model.setGmtVmCreate(server.getCreated());
        model.setImageUuid(server.getImageId());
        model.setInstanceId(server.getId());
        model.setName(server.getName());
        model.setPassword(server.getAdminPass());
        List<? extends SecurityGroup> groups = server.getSecurityGroups();
        List<String> groupList = Lists.newArrayList();
        if (!ListTool.isEmpty(groups)) {
            for (SecurityGroup group : groups) {
                if (groupList.contains(group.getName())) {
                    continue;
                }
                groupList.add(group.getName());
            }
        }
        model.setGroups(JSONObject.toJSONString(groupList));
        Map<String, List<? extends Address>> addressMap = server.getAddresses().getAddresses();
        String managerIp = null;
        if (null != addressMap) {
            for (Entry<String, List<? extends Address>> entry : addressMap.entrySet()) {
                List<? extends Address> addresses = entry.getValue();
                for (Address address : addresses) {
                    managerIp = address.getAddr();
                    if (null != managerIp) {
                        break;
                    }
                }
                if (null != managerIp) {
                    break;
                }
            }
        }
        model.setManagerIp(managerIp);
        model.setFlavorUuid(server.getFlavorId());
        model.setStatus(convert(server.getStatus().name()));
        if (null != server.getHypervisorHostname()) {
            model.setHostName(server.getHypervisorHostname());
        } else {
            model.setHostName(server.getHost());
        }
        model.setGmtVmCreate(server.getCreated());
        model.setKeypairName(server.getKeyName());
        model.setZone(server.getAvailabilityZone());
        model.setTenantUuid(server.getTenantId());
        return model;
    }

    private String convert(String status) {
        if ("active".equalsIgnoreCase(status)) {
            return BocloudStatus.Server.RUNNING.name();
        } else if (status.equalsIgnoreCase(BocloudStatus.Server.PAUSED.name())) {
            return BocloudStatus.Server.PAUSED.name();
        } else if ("stopped".equalsIgnoreCase(status) || "shutoff".equalsIgnoreCase(status)) {
            return BocloudStatus.Server.STOPPED.name();
        } else if ("BUILD".equalsIgnoreCase(status)) {
            return BocloudStatus.Server.BUILDING.name();
        } else if (status.equalsIgnoreCase(BocloudStatus.Server.SUSPENDED.name())) {
            return BocloudStatus.Server.SUSPENDED.name();
        } else if ("VERIFY_RESIZE".equalsIgnoreCase(status)) { // 配置修改中
            return BocloudStatus.Server.RESIZING.name();
        } else if ("SHELVED".equalsIgnoreCase(status) || "SHELVED_OFFLOADED".equalsIgnoreCase(status)) {
            return BocloudStatus.Server.SHELVED.name();
        } else if ("error".equalsIgnoreCase(status)) {
            return BocloudStatus.Server.ERROR.name();
        } else {
            return BocloudStatus.Server.EXCEPTION.name();
        }
    }

}