package com.bocloud.cmp.provider.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SnapshotModel;
import com.google.common.collect.Lists;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

/**
 * Openstack快照数据转换器
 * 
 * @author weiwei
 *
 */
public class SnapshotApiConvertor implements BeanConvertor<SnapshotModel, JSONObject> {

	private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.SSS");
	
	@Override
	public List<SnapshotModel> convertList(List<? extends JSONObject> snapshots) {
		List<SnapshotModel> models = Lists.newArrayList();
		for (JSONObject snapshot : snapshots) {
			models.add(convertModel(snapshot));
		}
		return models;
	}

	@SuppressWarnings("deprecation")
	@Override
	public SnapshotModel convertModel(JSONObject snapshot) {
		if (null == snapshot) {
			return null;
		}
		SnapshotModel model = new SnapshotModel();
		String name = snapshot.getString("name");
		if (StringUtils.isEmpty(name)) {
			name = snapshot.getString("displayName");
		}
		model.setName(name);
		model.setSize(snapshot.getInteger("size"));
		model.setSnapshotUuid(snapshot.getString("id"));
		model.setVolumeUuid(snapshot.getString("volumeId"));
		model.setRemark(snapshot.getString("description"));
		if ("CREATING".equalsIgnoreCase(snapshot.getString("status"))) {
			model.setStatus("BUILDING");
		} else {
			model.setStatus(snapshot.getString("status") == null ? null : snapshot.getString("status").toUpperCase());
		}
		String createdAt = snapshot.getString("created_at");
		try {
			model.setCreateTime(dateFormat.parse(createdAt));
		} catch (ParseException e) {
		}
		return model;
	}
}
