package com.bocloud.cmp.provider.convertor;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * Openstack存储卷数据转换器
 *
 */
public class VolumeApiConvertor implements BeanConvertor<VolumeModel, JSONObject> {

	@Override
	public List<VolumeModel> convertList(List<? extends JSONObject> volumes) {
		List<VolumeModel> models = Lists.newArrayList();
		for (JSONObject volume : volumes) {
			models.add(convertModel(volume));
		}
		return models;
	}
	
	@Override
	public VolumeModel convertModel(JSONObject volume) {
		VolumeModel model = new VolumeModel();
		if (null != volume) {
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'hh:mm:ss.SSS");
			model.setCategory(volume.getString("volume_type"));
			String createdAt = volume.getString("created_at");
			if (!StringUtils.isEmpty(createdAt)) {
				try {
					model.setCreateTime(dateFormat.parse(createdAt));
				} catch (ParseException e) {
				}
			}
			model.setImageUuid(volume.getString("imageRef"));
			String name = volume.getString("name");
			if (StringUtils.isEmpty(name)) {
				name = volume.getString("id");
			}
			String remark = volume.getString("description");
			model.setRemark(remark);
			model.setName(name);

			model.setSize(volume.getInteger("size"));
			String status = volume.getString("status");
			if (!StringUtils.isEmpty(status)) {
				model.setStatus(status.toUpperCase());
			}
			model.setType(volume.getString("volume_type"));
			model.setVolumeUuid(volume.getString("id"));
			model.setVolumeType(volume.getString("volume_type"));
			model.setZone(volume.getString("availability_zone"));
			model.setBootVolume(volume.getBoolean("bootable"));
			model.setTenantUuid(volume.getString("os-vol-tenant-attr:tenant_id"));

			List<ServerVolumeModel> models = Lists.newArrayList();
			List<JSONObject> attachments = JSONObject.parseArray(volume.getString("attachments"), JSONObject.class);
			if (!ListTool.isEmpty(attachments)) {
				for (JSONObject attachment : attachments) {
					models.add(new ServerVolumeModel(attachment.getString("server_id"),
							attachment.getString("volume_id"), attachment.getString("device")));
				}
			}
			model.setModels(models);
		}

		return model;
	}
}
