package com.bocloud.cmp.provider.network;

import org.openstack4j.api.Builders;
import org.openstack4j.model.network.SecurityGroup;
import org.openstack4j.model.network.SecurityGroupRule;
import org.openstack4j.model.network.builder.NetSecurityGroupRuleBuilder;
import org.openstack4j.model.network.ext.Ethertype;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.utils.IPAddress;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

/**
 * openstack安全组规则操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackSecuGroupRuleProvider extends OpenStackProvider implements SecurityGroupRuleProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackSecuGroupRuleProvider.class);
    private static final String DEFAULT_IP = "0.0.0.0/0";

    public OpenStackSecuGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        SecurityGroup group = this.getClient().networking().securitygroup().get(model.getGroupUuid());
        if (null == group) {
            logger.error("security group not exist");
            return new BocloudResult(false, "安全组不存在");
        }
        String ipPrefix = null == model.getRemoteIpPrefix() ? DEFAULT_IP : model.getRemoteIpPrefix();
        String address = ipPrefix;
        if (ipPrefix.contains("/")) {
            address = ipPrefix.substring(0, ipPrefix.indexOf("/"));
        }
        String ethertype = IPAddress.ipv6Validate(address) ? Ethertype.IPv6.name() : Ethertype.IPv4.name();
        NetSecurityGroupRuleBuilder builder = Builders.securityGroupRule().securityGroupId(model.getGroupUuid())
                .remoteIpPrefix(ipPrefix).protocol(model.getProtocol()).direction(model.getDirection())
                .ethertype(ethertype);
        // ICMP协议无端口
        if (!"icmp".equalsIgnoreCase(model.getProtocol())) {
            if (null != model.getPortMin()) {
                builder.portRangeMin(model.getPortMin());
            }
            if (null != model.getPortMax()) {
                builder.portRangeMax(model.getPortMax());
            }
        }
        SecurityGroupRule rule = builder.build();
        rule = this.getClient().networking().securityrule().create(rule);
        model.setRuleUuid(rule.getId());
        model.setEtherType(rule.getEtherType());
        model.setDirection(rule.getDirection());
        return new BocloudResult(true, model, "succsss");
    }

    public static void main(String[] args) {
        String aa = "dd";
        System.out.println(aa.substring(0, aa.indexOf("/")));
    }

    @Override
    public BocloudResult remove(String ruleId) {
        SecurityGroupRule rule = this.getClient().networking().securityrule().get(ruleId);
        if (null == rule) {
            logger.error("security group rule not exist");
            return new BocloudResult(false, "规则不存在！");
        }
        this.getClient().networking().securityrule().delete(rule.getId());
        return new BocloudResult(true, "删除成功！");
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel ruleModel) {
        // TODO Auto-generated method stub
        return null;
    }

}
