package com.bocloud.cmp.provider.storage;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.openstack4j.api.Builders;
import org.openstack4j.api.exceptions.ClientResponseException;
import org.openstack4j.model.common.ActionResponse;
import org.openstack4j.model.compute.VolumeAttachment;
import org.openstack4j.model.image.DiskFormat;
import org.openstack4j.model.image.Image;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.model.storage.block.VolumeSnapshot;
import org.openstack4j.model.storage.block.VolumeType;
import org.openstack4j.model.storage.block.VolumeUploadImage;
import org.openstack4j.model.storage.block.builder.VolumeTypeBuilder;
import org.openstack4j.model.storage.block.options.UploadImageData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumeTypeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.OpenStackProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.ImageConvertor;
import com.bocloud.cmp.provider.convertor.SnapshotApiConvertor;
import com.bocloud.cmp.provider.convertor.VolumeApiConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;

/**
 * openstack卷操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.openstack")
public class OpenStackVolumeProvider extends OpenStackProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(OpenStackVolumeProvider.class);

    public OpenStackVolumeProvider(Butler butler) {
        super(butler);
    }

    BeanConvertor<VolumeModel, JSONObject> convertVolumes = new VolumeApiConvertor();
    
    @Override
    public BocloudResult list() {
		try {
			JSONObject params = new JSONObject();
			params.put("all_tenants", "true");
			Result result = blockStorageFromGet("/volumes/detail", params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "数据不存在");
			}
			List<JSONObject> list = JSONArray.parseArray(object.getString("volumes"), JSONObject.class);
			List<VolumeModel> convertList = convertVolumes.convertList(list);
			return new BocloudResult(true, convertList, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询硬盘数据失败：" + e);
		}
    }

    @Override
    public BocloudResult create(VolumeModel volume) {
        try {
//        	logger.info("volume create params : " + JSON.toJSONString(volume));
//            boolean bootable = false;
//            if (null != volume.getBootVolume()) {
//                bootable = volume.getBootVolume();
//            }
//            VolumeBuilder builder = Builders.volume();
//            builder.name(volume.getName()).description(volume.getRemark()).size(volume.getSize());
//            String sourceImgId = volume.getImageUuid();
//            if (!StringUtils.isEmpty(sourceImgId)) {
//                bootable = true;
//                builder.imageRef(sourceImgId);
//            }
//            String sourceVolidId = volume.getSourceVolidUuid();
//            if (!StringUtils.isEmpty(sourceVolidId)) {
//                builder.source_volid(sourceVolidId);
//            }
//            String sourceSnapId = volume.getSourceSnapUuid();
//            if (!StringUtils.isEmpty(sourceSnapId)) {
//                builder.snapshot(sourceSnapId);
//            }
//            String volumeType = volume.getVolumeType();
//            if (!StringUtils.isEmpty(volumeType)) {
//                builder.volumeType(volumeType);
//            }
//            logger.info("start create volume ==== ");
//            Volume volumeCreated = this.getClient().blockStorage().volumes().create(builder.bootable(bootable).build());
//            logger.info("volumeCreated : " + JSON.toJSONString(volumeCreated));
//            if (null == volumeCreated) {
//                return new BocloudResult(false, "create fail");
//            }
//            volume.setBootVolume(volumeCreated.bootable());
//            volume.setVolumeUuid(volumeCreated.getId());
//            volume.setCreateTime(volumeCreated.getCreated());
//            volume.setStatus(volumeCreated.getStatus().name());
//            volume.setZone(volumeCreated.getZone());
//            // 等待启动盘创建完毕
//            if (bootable && !"ERROR".equals(volumeCreated.getStatus().name())) {
//                Long startTime = System.currentTimeMillis();
//                Boolean flag = true;
//                while (flag) {
//                    volumeCreated = this.getClient().blockStorage().volumes().get(volumeCreated.getId());
//                    if (null == volumeCreated) {
//                        return new BocloudResult(false, "create fail");
//                    }
//                    if ("ERROR".equals(volumeCreated.getStatus().name())) {
//                        this.getClient().blockStorage().volumes().forceDelete(volumeCreated.getId());
//                        return new BocloudResult(false, volume, "根磁盘创建错误");
//                    }
//                    Long endTime = System.currentTimeMillis();
//                    if (endTime - startTime > 60 * 60 * 1000) {// 等待1小时
//                        flag = false;
//                        volume.setStatus(volumeCreated.getStatus().name());
//                        return new BocloudResult(false, volume, "create bootable volume timeout");
//                    }
//                    if ("available".equalsIgnoreCase(volumeCreated.getStatus().name()) && volumeCreated.bootable()
//                            && !StringUtils.isEmpty(volumeCreated.getImageRef())) {
//                        logger.info("Block device is ready...");
//                        volume.setStatus(volumeCreated.getStatus().name());
//                        flag = false;
//                    }
//                }
//            } else if (bootable && "ERROR".equals(volumeCreated.getStatus().name())) {
//                this.getClient().blockStorage().volumes().forceDelete(volumeCreated.getId());
//                return new BocloudResult(false, volume, "启动盘创建错误");
//            }
//            return new BocloudResult(true, this.detail(volumeCreated.getId()).getData(), "create success");
        	
        	
			JSONObject params = new JSONObject();
			JSONObject volumeParams = new JSONObject();

			volumeParams.put("name", volume.getName());
			volumeParams.put("description", volume.getRemark());
			volumeParams.put("size", volume.getSize());

			String sourceImgId = volume.getImageUuid();
			if (!StringUtils.isEmpty(sourceImgId)) {
				volumeParams.put("imageRef", sourceImgId);
			}
			String sourceVolidId = volume.getSourceVolidUuid();
			if (!StringUtils.isEmpty(sourceVolidId)) {
				volumeParams.put("source_volid", sourceVolidId);
			}
			String sourceSnapId = volume.getSourceSnapUuid();
			if (!StringUtils.isEmpty(sourceSnapId)) {
				volumeParams.put("snapshot_id", sourceSnapId);
			}
			String volumeType = volume.getVolumeType();
			if (!StringUtils.isEmpty(volumeType)) {
				volumeParams.put("volume_type", volumeType);
			}
			params.put("volume", volumeParams);
			Result result = blockStoragePost("/volumes", params);
			logger.info("create disk result: " + JSON.toJSONString(result));
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "数据不存在");
			}
			JSONObject createVolumeObject = JSONArray.parseObject(object.getString("volume"), JSONObject.class);
			logger.info("createVolumeObject : " + createVolumeObject.toJSONString());
			if (null == createVolumeObject) {
				return new BocloudResult(false, "云硬盘创建失败");
			}
			return new BocloudResult(true, this.detail(createVolumeObject.getString("id")).getData(), "create success");
        } catch (ClientResponseException e) {
            logger.error("云硬盘创建失败:", e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
	public BocloudResult remove(String volumeId) {
		BocloudResult detail = this.detail(volumeId);
		if (detail.isFailed()) {
			if(detail.getMessage().contains("could not be found")) {
				return new BocloudResult(true, "云硬盘不存在,底层已删除成功");
			}
			return detail;
		}
		VolumeModel volume = (VolumeModel) detail.getData();
		if (null == volume) {
			logger.info("volume not exist...");
			return new BocloudResult(true, "云硬盘不存在,底层已删除成功");
		}

		Result blockStorageDelete = blockStorageDelete("/volumes/" + volumeId, null);
		if (blockStorageDelete.isSuccess()) {
			return new BocloudResult(true, "云硬盘删除成功！");
		} else {
			return new BocloudResult(false, "云硬盘删除失败！【" + blockStorageDelete.getErrorMsg() + "】");
		}
	}

    @Override
    public BocloudResult detail(String volumeId) {
//        Volume volume = this.getClient().blockStorage().volumes().get(volumeId);
//        if (null == volume) {
//            return new BocloudResult(false, "云硬盘不存在！");
//        }
//        BeanConvertor<VolumeModel, Volume> volumeConvert = new VolumeConvertor();
//        return new BocloudResult(true, volumeConvert.convertModel(volume), "success");
    	try {
			JSONObject params = new JSONObject();
			Result result = blockStorageFromGet("/volumes/" + volumeId, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			logger.info("disk : " + JSON.toJSONString(result));
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "数据不存在");
			}
			JSONObject volumeDetailObject = JSONArray.parseObject(object.getString("volume"), JSONObject.class);
			VolumeModel volumeModel = convertVolumes.convertModel(volumeDetailObject);
			return new BocloudResult(true, volumeModel, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：", e);
			return new BocloudResult(false, "查询失败；" + e);
		}
    }

    @Override
    public BocloudResult modify(VolumeModel volume) {
        ActionResponse response = this.getClient().blockStorage().volumes().update(volume.getVolumeUuid(), volume.getName(),
                volume.getRemark());
        if (null != response) {
            return new BocloudResult(true, "操作成功！");
        } else {
            logger.error("");
            return new BocloudResult(false, "操作失败！");
        }
    }

    BeanConvertor<SnapshotModel, JSONObject> convertSnapshot = new SnapshotApiConvertor();
    
    @Override
    public BocloudResult snapshot(SnapshotModel model) {
        /*try {
            VolumeSnapshot volumnBuilder = Builders.volumeSnapshot().description(model.getRemark()).name(model.getName())
                    .volume(model.getVolumeUuid()).force(true).build(); // force =
            // true可以进行强制创建快照
            VolumeSnapshot snapshot = this.getClient().blockStorage().snapshots().create(volumnBuilder);
            if (null == snapshot) {
                return new BocloudResult(false, "failed!");
            } else {
                return new BocloudResult(true, convert(snapshot), "success");
            }
        } catch (ClientResponseException e) {
            logger.error("创建快照失败！", e);
            if (400 == e.getStatus()) {
                return new BocloudResult(false, e.getMessage());
            }
            return new BocloudResult(false, e.getMessage());
        }*/
        try {
            JSONObject params = new JSONObject();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            params.put("volume_id", model.getVolumeUuid());
            JSONObject snapshotParam = new JSONObject();
            snapshotParam.put("snapshot", params);
            Result result = blockStoragePost("/snapshots", snapshotParam);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage()).getJSONObject("snapshot");
            logger.info("create snapshot : " + jsonObject.toJSONString());
            String snapshotId = jsonObject.getString("id");
            
            JSONObject snapshotParams = new JSONObject();
			Result snapshotDetail = blockStorageGet("/snapshots/" + snapshotId, snapshotParams);
			logger.info("get snapshotDetail : " + JSON.toJSONString(snapshotDetail));
			if (snapshotDetail.isFailed()) {
				return new BocloudResult(false, snapshotDetail.getMessage());
			}
			JSONObject snapshotObject = JSONObject.parseObject(snapshotDetail.getMessage());
			JSONObject snpashotModel = JSONArray.parseObject(snapshotObject.getString("snapshot"), JSONObject.class);
            return new BocloudResult(true, convertSnapshot.convertModel(snpashotModel), "快照创建成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
        }
        return new BocloudResult(false, "快照创建失败");
    }

    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        List<String> list = this.getClient().compute().servers().get(serverId).getOsExtendedVolumesAttached();
        String device = "/dev/vda";
        List<String> strs = Lists.newArrayList();
        for (int i = 0; i < 26; i++) {
            strs.add((char) (97 + i) + "");
        }
        if (!ListTool.isEmpty(list)) {
            device = "/dev/vd" + strs.get(list.size());
        }
        VolumeAttachment attachment = this.getClient().compute().servers().attachVolume(serverId, volumeId, device);
        if (null == attachment) {
            return new BocloudResult(false, "failed!");
        } else {
            return new BocloudResult(true, attachment.getDevice(), "success");
        }
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        try {
            List<String> list = this.getClient().compute().servers().get(serverId).getOsExtendedVolumesAttached();
            if (ListTool.isEmpty(list)) {
                return new BocloudResult(true, "success");
            }
            if (!list.contains(volumeId)) {
                return new BocloudResult(true, "success");
            }
            this.getClient().compute().servers().detachVolume(serverId, volumeId);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("创建快照失败！", e);
            return new BocloudResult(false, "卸载硬盘失败！");
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        if (StringUtils.isEmpty(volumeId) || StringUtils.isEmpty(status)) {
            return new BocloudResult(false, "No required param");
        }
        Volume.Status targetStatus = Volume.Status.valueOf(status);
        if (targetStatus == null) {
            return new BocloudResult(false, "param not right");
        }
        ActionResponse actionResponse = this.getClient().blockStorage().volumes().resetState(volumeId, targetStatus);
        if (actionResponse.isSuccess()) {
            return new BocloudResult(true, "");
        } else {
            return new BocloudResult(false, actionResponse.getFault());
        }

    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        UploadImageData data = UploadImageData.create(imageModel.getName());
        if (!StringUtils.isEmpty(imageModel.getOsType())) {
            data.diskFormat(DiskFormat.value(imageModel.getOsType()));
        }
        Boolean force = imageModel.getForce();
        if (null == force) {
            force = false;
        }
        data.force(force);
        ActionResponse response = this.getClient().blockStorage().volumes().resetState(volumeId, Volume.Status.AVAILABLE);
        if (response.isSuccess()) {
            VolumeUploadImage image = this.getClient().blockStorage().volumes().uploadToImage(volumeId, data);
            if (null == image) {
                return new BocloudResult(false, "upload to image fail");
            }
            ImageModel model = this.getActiveImage(image.getImageId());
            return new BocloudResult(true, model, "upload to image success");
        } else {
            return new BocloudResult(false, response.getFault());
        }
    }

	@Override
	public BocloudResult extend(String volumeId, Integer newSize) {
		JSONObject params = new JSONObject();
		JSONObject extendParams = new JSONObject();
		extendParams.put("new_size", newSize);
		params.put("os-extend", extendParams);
		logger.info("extend params : " + params.toJSONString());
		Result extend = blockStorageFromPost("/volumes/" + volumeId + "/action", params);
		logger.info("blockStorageDelete : " + JSON.toJSONString(extend));
		if (extend.isSuccess()) {
			return new BocloudResult(true, "扩容成功");
		} else {
			return new BocloudResult(false, "云硬盘扩容失败！【" + extend.getErrorMsg() + "】");
		}
	}

    @Override
    public BocloudResult list(String s, String s1) {
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel h3cVolumeModel) {
        return null;
    }

    @Override
    public BocloudResult createVolumeType(VolumeTypeModel model) {
        try {
            VolumeTypeBuilder builder = Builders.volumeType().name(model.getName());
            if (null != model.getExtraSpecMap() && !model.getExtraSpecMap().isEmpty()) {
                builder.extraSpecs(model.getExtraSpecMap());
            }
            VolumeType type = this.getClient().blockStorage().volumes().createVolumeType(builder.build());
            if (null == type) {
                return new BocloudResult(false, "创建云硬盘类型失败！");
            }
            model.setVolumeTypeUuid(type.getId());
            model.setName(type.getName());
            if (null != type.getExtraSpecs() && !type.getExtraSpecs().isEmpty()) {
                model.setExtraSpecs(JSONObject.toJSONString(type.getExtraSpecs()));
            }
            return new BocloudResult(true, model, "success");
        } catch (Exception e) {
            logger.error("创建云硬盘类型失败：", e);
            return new BocloudResult(false, "创建云硬盘类型失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult deleteVolumeType(String volumeTypeUuid) {
        try {
            this.getClient().blockStorage().volumes().deleteVolumeType(volumeTypeUuid);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("删除云硬盘类型失败：", e);
            return new BocloudResult(false, "删除云硬盘类型失败：" + e.getMessage());
        }
    }

    private ImageModel getActiveImage(String imageId) {
        ImageModel imageModel = null;
        Image image = null;
        boolean flag = true;
        BeanConvertor<ImageModel, Image> convertImage = new ImageConvertor();
        Long startTime = System.currentTimeMillis();
        while (flag) {
            image = this.getClient().images().get(imageId);
            // 镜像已经为运行中
            if ("active".equalsIgnoreCase(image.getStatus().name())) {
                imageModel = convertImage.convertModel(image);
                flag = false;
            }
            Long endTime = System.currentTimeMillis();
            // 因为备份镜像的过程比较漫长，因此此处超时时间设置为1小时
            if (endTime - startTime > 60 * 60 * 1000) {
                imageModel = convertImage.convertModel(image);
                flag = false;
            }
        }
        return imageModel;
    }

    private SnapshotModel convert(VolumeSnapshot snapshot) {
        if (null == snapshot) {
            return null;
        }
        SnapshotModel model = new SnapshotModel();
        model.setCreateTime(snapshot.getCreated());
        model.setName(snapshot.getName());
        model.setSize(snapshot.getSize());
        model.setSnapshotUuid(snapshot.getId());
        model.setVolumeUuid(snapshot.getVolumeId());
        model.setStatus(snapshot.getStatus().name());
        model.setRemark(snapshot.getDescription());
        return model;
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }
}
