package com.bocloud.cmp.provider.utils;

import org.openstack4j.model.compute.Server;
import org.openstack4j.model.image.v2.Image;
import org.openstack4j.model.network.State;
import org.openstack4j.model.storage.block.Volume;
import org.openstack4j.openstack.networking.domain.ext.NeutronFirewall;

/**
 * 资源过滤器(openstack)
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月19日
 */
public class ResourcePercolator {

    /**
     * 云主机过滤器
     *
     * @param status
     * @return
     */
    public static boolean serverFilter(Server.Status status) {
        if (null == status) {
            return false;
        }
        if (Server.Status.ACTIVE == status || Server.Status.SUSPENDED == status || Server.Status.PAUSED == status
                || Server.Status.RESIZE == status || Server.Status.VERIFY_RESIZE == status
                || Server.Status.REVERT_RESIZE == status || Server.Status.STOPPED == status
                || Server.Status.SHUTOFF == status || Server.Status.SHELVED == status
                || Server.Status.SHELVED_OFFLOADED == status || Server.Status.ERROR == status
                || Server.Status.DELETED == status || Server.Status.UNKNOWN == status
                || Server.Status.UNRECOGNIZED == status) {
            return true;
        }
        return false;
    }

    /**
     * 网络过滤器
     *
     * @param state
     * @return
     */
    public static boolean networkFilter(State state) {
        if (null == state) {
            return false;
        }
        if (State.ACTIVE == state || State.DOWN == state || State.ERROR == state) {
            return true;
        }
        return false;
    }

    /**
     * 卷过滤器
     *
     * @param status
     * @return
     */
    public static boolean volumeFilter(Volume.Status status) {
        if (null == status) {
            return false;
        }
        if (Volume.Status.AVAILABLE == status || Volume.Status.IN_USE == status || Volume.Status.BACKING_UP == status) {
            return true;
        }
        return false;
    }

    /**
     * 快照过滤器
     *
     * @param status
     * @return
     */
    public static boolean snapshotFilter(Volume.Status status) {
        if (null == status) {
            return false;
        }
        if (Volume.Status.AVAILABLE == status || Volume.Status.IN_USE == status || Volume.Status.BACKING_UP == status) {
            return true;
        }
        return false;
    }

    /**
     * 镜像过滤器
     *
     * @param status
     * @return
     */
    public static boolean imageFilter(Image.ImageStatus status) {
        if (null == status) {
            return false;
        }
        if (Image.ImageStatus.ACTIVE == status) {
            return true;
        }
        return false;
    }

    /**
     * 端口过滤器
     *
     * @param state
     * @return
     */
    public static boolean portFilter(State state) {
        if (null == state) {
            return false;
        }
        if (State.ACTIVE == state || State.DOWN == state || State.ERROR == state || State.UNRECOGNIZED == state) {
            return true;
        }
        return false;
    }

    /**
     * 路由过滤器
     *
     * @param state
     * @return
     */
    public static boolean routerFilter(State state) {
        if (null == state) {
            return false;
        }
        if (State.ACTIVE == state || State.DOWN == state || State.ERROR == state) {
            return true;
        }
        return false;
    }

    /**
     * 防火墙过滤器
     *
     * @param status
     * @return
     */
    public static boolean firewallFilter(NeutronFirewall.FirewallStatus status) {
        if (null == status) {
            return false;
        }
        if (NeutronFirewall.FirewallStatus.ACTIVE == status || NeutronFirewall.FirewallStatus.DOWN == status
                || NeutronFirewall.FirewallStatus.ERROR == status) {
            return true;
        }
        return false;
    }
}
