package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorExtraSpecsModel;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC配置规则驱动
 *
 * @author sxp
 * @since Mar 27, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcFlavorProvider extends PowervcProvider implements FlavorProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcFlavorProvider.class);

    public PowervcFlavorProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult sync() {
        List<FlavorModel> flavors = listFlavor();
        return new BocloudResult(flavors != null, flavors, "");
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        BocloudResult result = this.getClient().compute(getVersion()).flavors(getVersion()).create(flavor);
        if (result.isFailed()) {
            return result;
        }
        FlavorModel flavorNew = (FlavorModel) result.getData();
        if (StringUtils.isEmpty(flavorNew.getFlavorUuid())) {
            return new BocloudResult(false, "创建Flavor后返回的uuid为空");
        }
        result = this.getClient().compute(getVersion()).flavorExtras(getVersion()).create(flavorNew.getFlavorUuid(), flavor.getExtraSpecs());
        if (result.isFailed()) {
            return result;
        }
        flavorNew.setExtraSpecs((FlavorExtraSpecsModel) result.getData());
        return new BocloudResult(true, flavorNew, "创建成功");
    }

    @Override
    public BocloudResult remove(String flavorId) {
        return this.getClient().compute(getVersion()).flavors(getVersion()).delete(flavorId);
    }

    @Override
    public BocloudResult update(FlavorModel flavor) {
        BocloudResult result = remove(flavor.getFlavorUuid());
        if (result.isFailed()) {
            result.setMessage("修改配置规则流程中，先删除原来的配置规则出错。" + result.getMessage());
            return result;
        }

        result = create(flavor);
        if (result.isFailed()) {
            result.setMessage("修改配置规则流程中，删除原来的配置规则后，创建新的配置规格出错。" + result.getMessage());
            return result;
        }

        return result;
    }

    @Override
    public BocloudResult get(String flavorId) {
        FlavorModel model = this.getClient().compute(getVersion()).flavors(getVersion()).get(flavorId);
        if (model != null) {
            FlavorExtraSpecsModel extra = this.getClient().compute(getVersion()).flavorExtras(getVersion()).get(flavorId);
            model.setExtraSpecs(extra);
            return new BocloudResult(true, model, "");
        } else {
            return new BocloudResult(false, null, "未获取到指定唯一标识为[" + flavorId + "]的配置规格");
        }
    }

}
