package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.HmcHostModel;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.PowervcSyncModel;
import com.bocloud.cmp.model.SanFabricModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.StorageModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.VolumeTemplateModel;
import com.bocloud.cmp.model.powervc.HostSeaAdapterModel;
import com.bocloud.cmp.model.powervc.NetworkSeaAdaterModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC同步驱动
 * 
 * @author sxp
 * @since Mar 20, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcLocationProvider extends PowervcProvider implements LocationProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcLocationProvider.class);

    public PowervcLocationProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult sync() {
        try {
            List<HostModel> hypervisors = listHypervisor();
            List<ServerModel> servers = listServer();
            List<ClusterModel> clusters = listCluster();
            List<NetworkModel> networks = listNetwork();
            List<StorageModel> stroages = listStorageDevice();
            List<VolumeModel> volumes = listVolume();
            List<VolumeTemplateModel> templates = listVolumeTemplate();
            List<ImageModel> images = listImage();
            List<FlavorModel> flavors = listFlavor();
            List<PortModel> ports = listPort();
            List<HmcHostModel> hmcs = listHmc();
            List<HostSeaAdapterModel> hostSeas = listHostSea();
            List<NetworkSeaAdaterModel> networkSeas = listNetworkSea();
            List<SanFabricModel> fabrics = listFabrics();

            PowervcSyncModel syncModel = new PowervcSyncModel();
            syncModel.setHypervisors(hypervisors);
            syncModel.setServers(servers);
            syncModel.setClusters(clusters);
            syncModel.setNetworks(networks);
            syncModel.setStorages(stroages);
            syncModel.setVolumes(volumes);
            syncModel.setTemplates(templates);
            syncModel.setImages(images);
            syncModel.setFlavors(flavors);
            syncModel.setPorts(ports);
            syncModel.setHmcs(hmcs);
            syncModel.setHostSeas(hostSeas);
            syncModel.setNetworkSeas(networkSeas);
            syncModel.setFabrics(fabrics);

            return new BocloudResult(true, syncModel, "success");
        } catch (Exception e) {
            String errorMsg = "同步失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        return new BocloudResult(false, "PowerVC不支持该接口");
    }

    @Override
    public BocloudResult getManagedSystemSyncInfo(String uuid) {
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel host) {
        return null;
    }

    @Override
    public BocloudResult listRegions() {
        return null;
    }

    @Override
    public BocloudResult listZones(String region) {
        return null;
    }

}
