package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.model.ClusterOperateModel;
import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

;

/**
 * 主机组
 * @author luyanchao
 * @since 2019/2/11 14:38
 */
@Provider("bocloud.cmp.powervc")
public class PowervcClusterProvider extends PowervcProvider implements ClusterProvider {

    private static Logger logger = LoggerFactory.getLogger(PowervcOsHostProvider.class);

    public PowervcClusterProvider(Butler butler) {
        super(butler);
    }

    private BocloudResult create(String clusterName, String initialPolicy, List<String> hostNames) {
        // 创建一个空的主机组
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).create(clusterName);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        Long osAggregateId = JSON.parseObject(bocloudResult.getData().toString()).getJSONObject("aggregate").getLong("id");
        if (hostNames != null && hostNames.size() > 0) {
            for (String hostName : hostNames) {
                // 将主机从默认的主机组中移除
                bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                // 将主机加入到新的主机组中
                bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(osAggregateId.toString(), hostName);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
            }
        }
        bocloudResult =  this.getClient().compute(getVersion()).osAggregate(getVersion()).setMetadata(osAggregateId.toString(), initialPolicy);
        return bocloudResult;
    }

    @Override
    public BocloudResult create(ClusterOperateModel model) {
        return create(model.getClusterName(), model.getInitialPolicy(), model.getHostNames());
    }

    @Override
    public BocloudResult remove(ClusterOperateModel model) {
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).remove(model.getUuid());
    }

    @Override
    public BocloudResult detail(String clusterVal) {
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).get(clusterVal);
    }

    @Override
    public BocloudResult create(ClusterModel clusterModel) {
        return create(clusterModel.getName(), clusterModel.getInitialPolicy(), clusterModel.getHostNames());
    }

    @Override
    public BocloudResult modify(ClusterModel clusterModel) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).get(clusterModel.getUuid());
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        ClusterModel clusterModelNew = clusterModel;
        ClusterModel clusterModelOld = (ClusterModel) bocloudResult.getData();
        if (clusterModelNew.getHostNames() == null) {
            clusterModelNew.setHostNames(new ArrayList<>());
        }
        if (clusterModelOld.getHostNames() == null) {
            clusterModelOld.setHostNames(new ArrayList<>());
        }
        List<String> hostToAdd = new ArrayList<>(clusterModelNew.getHostNames());
        List<String> hostToRemove = new ArrayList<>(clusterModelOld.getHostNames());
        hostToAdd.removeAll(clusterModelOld.getHostNames());
        hostToRemove.removeAll(clusterModelNew.getHostNames());
        for (String hostName : hostToRemove) {
            bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(clusterModel.getUuid(), hostName);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
        }
        for (String hostName : hostToAdd) {
            bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(clusterModel.getUuid(), hostName);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
        }
        bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).setMetadata(clusterModel.getUuid(), clusterModel.getInitialPolicy());
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).modify(clusterModel.getUuid(), clusterModel.getName());
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).get(clusterModel.getUuid());
    }

    @Override
    public BocloudResult remove(String aggregateId) {
        BocloudResult bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).get(aggregateId);
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        ClusterModel clusterModel = (ClusterModel) bocloudResult.getData();
        if (clusterModel.getHostNames() != null && clusterModel.getHostNames().size() > 0) {
            for (String hostName : clusterModel.getHostNames()) {
                // 将该主机组中的主机移除
                bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(aggregateId, hostName);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
                // 将该主机组中的主机添加到默认组
                bocloudResult = this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
                if (bocloudResult.isFailed()) {
                    return bocloudResult;
                }
            }
        }
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).remove(aggregateId);
    }

    @Override
    public BocloudResult addHost(String aggregateId, String hostName) {
        if (PowervcConstant.DEFAULT_OS_AGGREGATE_ID.equals(aggregateId)) {
            return new BocloudResult(false, "不支持从默认主机组中移入/移出主机");
        }
        BocloudResult result = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
        if (result.isFailed()) {
            return result;
        }
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(aggregateId, hostName);
    }

    @Override
    public BocloudResult removeHost(String aggregateId, String hostName) {
        if (PowervcConstant.DEFAULT_OS_AGGREGATE_ID.equals(aggregateId)) {
            return new BocloudResult(false, "不支持从默认主机组中移入/移出主机");
        }
        BocloudResult result = this.getClient().compute(getVersion()).osAggregate(getVersion()).removeServer(aggregateId, hostName);
        if (result.isFailed()) {
            return result;
        }
        return this.getClient().compute(getVersion()).osAggregate(getVersion()).addServer(PowervcConstant.DEFAULT_OS_AGGREGATE_ID, hostName);
    }

    @Override
    public BocloudResult summary(String clusterVal) {
        return null;
    }

    @Override
    public BocloudResult queryByName(String name) {
        List<ClusterModel> list = this.getClient().compute(getVersion()).osAggregate(getVersion()).list();
        if (list != null && list.size() > 0) {
            for (ClusterModel clusterModel : list) {
                if (clusterModel.getName().equals(name)){
                    return new BocloudResult(true, clusterModel, "查询成功");
                }
            }
        }
        return new BocloudResult(true, "查询成功");
    }

    @Override
    public BocloudResult list() {
        List<ClusterModel> list = this.getClient().compute(getVersion()).osAggregate(getVersion()).list();
        return new BocloudResult(true, list, "查询成功");
    }
}
