package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.HmcHostModel;
import com.bocloud.cmp.model.OsHosts;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

@Provider("bocloud.cmp.powervc")
public class PowervcHmcProvider extends PowervcProvider implements HmcHostProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcHmcProvider.class);

    public PowervcHmcProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        try {
            List<HmcHostModel> hosts = this.getClient().compute(getVersion()).hmcs(getVersion()).list();
            if (hosts == null) {
                return new BocloudResult(false, "获取hmc机器失败");
            } else {
                return new BocloudResult(true, hosts, "获取hmc机器成功");
            }
        } catch (Exception e) {
            logger.error("获取hmc机器失败", e);
            return new BocloudResult(false, "获取hmc机器失败!");
        }
    }

    @Override
    public BocloudResult get(String hmcUuid) {
        try {
            HmcHostModel hmcHost = this.getClient().compute(getVersion()).hmcs(getVersion()).get(hmcUuid);
            if (hmcHost == null) {
                return new BocloudResult(false, "获取hmc机器" + hmcUuid + "失败");
            } else {
                return new BocloudResult(true, hmcHost, "success");
            }
        } catch (Exception e) {
            String errorMsg = "获取数据失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult create(HmcHostModel hmc) {
        try {
            BocloudResult hmcHostResult = this.getClient().compute(getVersion()).hmcs(getVersion()).create(hmc);
            if (hmcHostResult.isSuccess()){
                String hmcUuid = JSON.parseObject(hmcHostResult.getData().toString()).
                        getJSONObject("hmc").getJSONObject("registration").getString("hmc_uuid");
                HmcHostModel hmcHost = this.getClient().compute(getVersion()).hmcs(getVersion()).get(hmcUuid);
                long start = System.currentTimeMillis();
                while (System.currentTimeMillis() - start < 5 * 60 * 1000 && !hmcHost.getStatus().getStatus().equals("OK")){
                    logger.info("正在检测HMC[" + hmcUuid + "]状态，5秒后将重新检测");
                    Thread.sleep(5000);
                    hmcHost = this.getClient().compute(getVersion()).hmcs(getVersion()).get(hmcUuid);
                }
                if (System.currentTimeMillis() - start >= 5 * 60 * 1000) {
                    throw new RuntimeException("新增hmc机器失败，经过5分钟后hmc状态仍未正常");
                }
                return hmcHostResult;
            }else {
                return hmcHostResult;
            }
        } catch (Exception e) {
            logger.error("新增hmc机器失败", e);
            return new BocloudResult(false, "新增hmc机器失败!");
        }
    }

    @Override
    public BocloudResult modify(HmcHostModel hmc) {
        try {
            return this.getClient().compute(getVersion()).hmcs(getVersion()).modify(hmc);
        } catch (Exception e) {
            logger.error("修改hmc机器失败", e);
            return new BocloudResult(false, "修改hmc机器失败!");
        }
    }

    @Override
    public BocloudResult delete(String hmcUuid) {
        try {
            return this.getClient().compute(getVersion()).hmcs(getVersion()).delete(hmcUuid);
        } catch (Exception e) {
            logger.error("移除hmc机器失败", e);
            return new BocloudResult(false, "移除hmc机器失败!");
        }
    }

    @Override
    public BocloudResult getHosts(String hmcUuid) {
        try {
            OsHosts hosts = this.getClient().compute(getVersion()).hmcs(getVersion()).getHosts(hmcUuid);
            if (hosts == null) {
                return new BocloudResult(false, "获取hmc上的小型机失败");
            } else {
                return new BocloudResult(true, hosts, "获取hmc上的小型机成功");
            }
        } catch (Exception e) {
            logger.error("获取hmc上的小型机失败:", e);
            return new BocloudResult(false, "获取hmc上的小型机失败!");
        }
    }
}
