package com.bocloud.cmp.provider.compute;


import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.powervc.HostSeaAdapterModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

@Provider("bocloud.cmp.powervc")
public class PowervcHostSeaProvider extends PowervcProvider implements HostSeaProvider {

    private static Logger logger = LoggerFactory.getLogger(PowervcHostSeaProvider.class);

    public PowervcHostSeaProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<HostSeaAdapterModel> seas = this.getClient().compute(getVersion()).hostSea(getVersion()).list();
            if (seas == null) {
                return new BocloudResult(false, "获取主机sea适配器失败");
            } else {
                return new BocloudResult(true, seas, "获取主机sea适配器成功");
            }
        } catch (Exception e) {
            logger.error("获取主机sea适配器失败", e);
            return new BocloudResult(false, "获取主机sea适配器失败!");
        }
    }

    @Override
    public BocloudResult list(long vlanId) {
        try {
            List<HostSeaAdapterModel> seas = this.getClient().compute(getVersion()).hostSea(getVersion()).list(vlanId);
            if (seas == null) {
                return new BocloudResult(false, "获取主机sea适配器失败");
            } else {
                return new BocloudResult(true, seas, "获取主机sea适配器成功");
            }
        } catch (Exception e) {
            logger.error("获取主机sea适配器失败", e);
            return new BocloudResult(false, "获取主机sea适配器失败!");
        }
    }

    @Override
    public BocloudResult list(String networkUuid, long vlanId) {
        try {
            List<HostSeaAdapterModel> seas = this.getClient().compute(getVersion()).hostSea(getVersion()).list(networkUuid, vlanId);
            if (seas == null) {
                return new BocloudResult(false, "获取主机sea适配器失败");
            } else {
                return new BocloudResult(true, seas, "获取主机sea适配器成功");
            }
        } catch (Exception e) {
            logger.error("获取主机sea适配器失败", e);
            return new BocloudResult(false, "获取主机sea适配器失败!");
        }
    }

    @Override
    public BocloudResult createMapping(List<HostSeaAdapterModel> seas) {
        try {
            return this.getClient().compute(getVersion()).hostSea(getVersion()).createMapping(seas);
        } catch (Exception e) {
            logger.error("创建host-sea-mapping失败", e);
            return new BocloudResult(false, "创建host-sea-mapping失败!");
        }
    }
}
