package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.model.NovaHost;
import com.bocloud.cmp.provider.powervc4j.model.NovaHostResource;
import com.bocloud.cmp.provider.powervc4j.model.NovaHostResourceBody;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * PowerVC宿主机驱动
 *
 * @author sxp
 * @since Mar 28, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcHyperVisorProvider extends PowervcProvider implements HostProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcHyperVisorProvider.class);

    public PowervcHyperVisorProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult create(HostOperateModel model) {
        OsHost osHost = new OsHost();
        osHost.setForceSwitch(true);
        osHost.setHmcUuids(Collections.singletonList(model.getHmcUuid()));
        osHost.setNickName(model.getName());
        osHost.setName(model.getHostName());
        osHost.setHostType("powervm");
        return this.getClient().compute(getVersion()).oshost(getVersion()).create(osHost);
    }

    @Override
    public BocloudResult modify(HostOperateModel model) {
        OsHost osHost = new OsHost();
        osHost.setHmcUuids(Collections.singletonList(model.getHmcUuid()));
        osHost.setName(model.getHostName());
        return this.getClient().compute(getVersion()).oshost(getVersion()).modify(osHost);
    }

    @Override
    public BocloudResult remove(HostOperateModel model) {
        return this.getClient().compute(getVersion()).oshost(getVersion()).delete(model.getHostName());
    }

    @Override
    public BocloudResult enterMode(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult exitMode(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult disconn(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult reconn(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult moveTo(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult shutdown(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult reboot(HostOperateModel model) {
        return null;
    }

    @Override
    public BocloudResult detail(HostOperateModel model) {
        HostModel hostModel = this.getClient().compute(getVersion()).hypervisors(getVersion()).get(model.getUuid());
        return new BocloudResult(true, hostModel, "查询成功");
    }

    @Override
    public BocloudResult list() {
        List<HostModel> hypervisors = listHypervisor();
        return new BocloudResult(hypervisors != null, hypervisors, "");
    }

    @Override
    public BocloudResult monitor() {
        return null;
    }

    @Override
    public BocloudResult listPerf() {
        try {
            List<HostModel> hypervisors = listHypervisor();
            List<HostPerfModel> models = new ArrayList<>(hypervisors.size());

            HostPerfModel model;
            NovaHost novaHost;
            String name;

            for (HostModel host : hypervisors) {
                name = host.getName();
                novaHost = this.getClient().compute(getVersion()).host(getVersion()).hostDescribe(name);
                if (novaHost != null) {
                    model = convert(novaHost);
                    if (model != null) {
                        models.add(model);
                    }
                }
            }

            return new BocloudResult(true, models, "success");
        } catch (Exception e) {
            String errorMsg = "获取数据失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult summary(Long id) {
        return null;
    }

    @Override
    public BocloudResult detail(Long hostId) {
        HostModel hostModel = this.getClient().compute(getVersion()).hypervisors(getVersion()).get(hostId.toString());
        return new BocloudResult(true, hostModel, "查询成功");
    }

    @Override
    public BocloudResult list(String clusterId) {
        return null;
    }

    @Override
    public BocloudResult listUnmanagedPowervm(String hostname) {
        return null;
    }

    private HostPerfModel convert(NovaHost novaHost) {
        Objects.requireNonNull(novaHost, "novaHost");

        List<NovaHostResourceBody> hostItems = novaHost.getHostItems();
        if (hostItems == null || hostItems.size() == 0) {
            return null;
        }

        HostPerfModel hostPerfModel = new HostPerfModel();
        NovaHostResource resource;

        for (NovaHostResourceBody body : hostItems) {
            resource = body.getResource();
            if (resource == null) {
                continue;
            }
            if ("(total)".equalsIgnoreCase(resource.getProject())) {
                hostPerfModel.setCpuTotal(resource.getCpu());
                hostPerfModel.setDiskTotal(resource.getDiskGb());
                hostPerfModel.setMemTotal(resource.getMemoryMb());
                hostPerfModel.setHostName(resource.getHost());
            } else if ("(used_now)".equalsIgnoreCase(resource.getProject())) {
                hostPerfModel.setCpuUsed(resource.getCpu());
                hostPerfModel.setDiskUsed(resource.getDiskGb());
                hostPerfModel.setMemUsed(resource.getMemoryMb());
            }
        }

        hostPerfModel.setGmtCreate(Calendar.getInstance().getTime());
        return hostPerfModel;
    }

}
