package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ConsoleMessage;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * powervc 控制台消息驱动
 *
 * @author lkx007
 */
@Provider("bocloud.cmp.powervc")
public class PowervcMessageProvider extends PowervcProvider implements MessageProvider {
    private static Logger logger = LoggerFactory.getLogger(com.bocloud.cmp.provider.compute.PowervcMessageProvider.class);

    public PowervcMessageProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        try {
            List<ConsoleMessage> messages = this.getClient().compute(getVersion()).message(getVersion()).list();
            if (messages == null) {
                return new BocloudResult(false, "获取powervc消息失败");
            } else {
                return new BocloudResult(true, messages, "获取powervc消息成功");
            }
        } catch (Exception e) {
            logger.error("获取powervc消息失败", e);
            return new BocloudResult(false, "获取powervc消息失败!");
        }
    }

    @Override
    public BocloudResult delete(Integer messageId) {
        try {
            return this.getClient().compute(getVersion()).message(getVersion()).delete(messageId);
        } catch (Exception e) {
            logger.error("删除powervc消息失败", e);
            return new BocloudResult(false, "删除powervc消息失败!");
        }
    }

    @Override
    public BocloudResult deleteAll() {
        try {
            return this.getClient().compute(getVersion()).message(getVersion()).deleteAll();
        } catch (Exception e) {
            logger.error("删除powervc消息失败", e);
            return new BocloudResult(false, "删除powervc所有消息失败!");
        }
    }
}
