package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * PowerVC网络驱动
 *
 * @author sxp
 * @since Mar 29, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcNetworkProvider extends PowervcProvider implements NetworkProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcNetworkProvider.class);

    public PowervcNetworkProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        List<NetworkModel> networks = this.listNetwork();
        return new BocloudResult(true, networks, "查询成功");
    }

    @Override
    public BocloudResult create(NetworkModel model) {
        BocloudResult result = this.getClient().networking(getVersion()).network(getVersion()).create(model);
        if (result.isFailed()) {
            return result;
        }
        NetworkModel network = (NetworkModel) result.getData();
        List<SubnetModel> subnets = new ArrayList<>(network.getSubnetIds().size());
        for (String subnetId : network.getSubnetIds()) {
            SubnetModel subnetModel = this.getClient().networking(getVersion()).subnet(getVersion()).get(subnetId);
            subnets.add(subnetModel);
        }
        network.setSubnets(subnets);
        return new BocloudResult(true, network, "创建成功");
    }

    @Override
    public BocloudResult remove(String networkId) {
        return this.getClient().networking(getVersion()).network(getVersion()).delete(networkId);
    }

    @Override
    public BocloudResult detail(String networkId) {
        NetworkModel model = this.getClient().networking(getVersion()).network(getVersion()).get(networkId);
        return new BocloudResult(true, model, "查询成功");
    }

    @Override
    public BocloudResult modify(NetworkModel model) {
        BocloudResult result = this.getClient().networking(getVersion()).network(getVersion()).update(model);
        if (result.isFailed()) {
            return result;
        }
        NetworkModel network = (NetworkModel) result.getData();
        List<SubnetModel> subnets = new ArrayList<>(network.getSubnetIds().size());
        for (String subnetId : network.getSubnetIds()) {
            SubnetModel subnetModel = this.getClient().networking(getVersion()).subnet(getVersion()).get(subnetId);
            subnets.add(subnetModel);
        }
        network.setSubnets(subnets);
        return new BocloudResult(true, network, "修改");
    }

    // ----

    @Override
    public BocloudResult createDSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult createVSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult removeDSwitch(String name) {
        return null;
    }

    @Override
    public BocloudResult renameDSwitch(String oldName, String newName) {
        return null;
    }

    @Override
    public BocloudResult updateDVSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
        return null;
    }

    @Override
    public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
        return null;
    }

    @Override
    public BocloudResult removePortGroup(String hostName, String pgName, String type) {
        return null;
    }

    @Override
    public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
        return null;
    }

    @Override
    public BocloudResult addHost(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult listVSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult listProfile() {
        return null;
    }

}
