package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PortModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC子网驱动
 *
 * @author sxp
 * @since Apr 3, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcPortProvider extends PowervcProvider implements PortProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcPortProvider.class);

    public PowervcPortProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult sync() {
        List<PortModel> ports = this.listPort();
        return new BocloudResult(ports != null, ports, "");
    }

    @Override
    public BocloudResult create(PortModel portModel) {
        return this.getClient().networking(getVersion()).port(getVersion()).create(portModel);
    }

    @Override
    public BocloudResult modify(PortModel portModel) {
        return this.getClient().networking(getVersion()).port(getVersion()).modify(portModel);
    }

    @Override
    public BocloudResult remove(String portId) {
        return this.getClient().networking(getVersion()).port(getVersion()).remove(portId);
    }

    @Override
    public BocloudResult detail(String portId) {
        try {
            PortModel model = this.getClient().networking(getVersion()).port(getVersion()).get(portId);
            return new BocloudResult(model != null, model, "");
        } catch (Exception e) {
            String msg = "查询失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }
}
