package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * PowerVC子网驱动
 *
 * @author sxp
 * @since Apr 3, 2018
 */
@Provider("bocloud.cmp.powervc")
public class PowervcSubnetProvider extends PowervcProvider implements SubnetProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcSubnetProvider.class);

    public PowervcSubnetProvider(Butler bulter) {
        super(bulter);
    }

    @Override
    public BocloudResult list() {
        List<SubnetModel> subnets = this.listSubnet();
        return new BocloudResult(subnets != null, subnets, "");
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        try {
            BocloudResult result = this.getClient().networking(getVersion()).subnet(getVersion()).create(subnet);
            if (result.isFailed()) {
                return result;
            }

            SubnetModel createdSubnet = (SubnetModel) result.getData();
            String subnetId = createdSubnet.getSubnetUuid();
            result = detail(subnetId);

            if (result.isFailed()) {
                result.setMessage("创建子网成功，但是根据返回的唯一标识查询子网信息出错");
            }

            return result;
        } catch (Exception e) {
            String msg = "创建子网失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult modify(SubnetModel subnet) {
        try {
            BocloudResult result = this.getClient().networking(getVersion()).subnet(getVersion()).update(subnet);
            return result;
        } catch (Exception e) {
            String msg = "修改子网信息失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        try {
            BocloudResult result = this.getClient().networking(getVersion()).subnet(getVersion()).delete(subnetId);
            return result;
        } catch (Exception e) {
            String msg = "删除子网信息失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, "删除失败!");
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        try {
            SubnetModel model = this.getClient().networking(getVersion()).subnet(getVersion()).get(subnetId);

            return new BocloudResult(model != null, model, "");
        } catch (Exception e) {
            String msg = "查询子网信息失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        return null;
    }

}
