package com.bocloud.cmp.provider.powervc4j.api;

import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.image.ImageService;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkingService;
import com.bocloud.cmp.provider.powervc4j.api.notification.NotificationService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageService;
import com.bocloud.cmp.provider.powervc4j.api.validation.ValidateService;
import com.bocloud.cmp.provider.powervc4j.provider.DefaultAPIProvider;

;

/**
 * API工具类
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class Apis {
    public static final String API_PREFIX = "/powervc/openstack";

    private static final APIProvider provider = initializeProvider();

    /**
     * 获取指定接口类型的实现类
     *
     * @param <T>
     *            PowerVC API的类型
     * @param api
     *            API接口
     * @return T类型API的实现
     */
    public static <T> T get(Class<T> api, PowervcConstant.Version version) {
        return provider.get(api, version);
    }

    /**
     * 获取指定接口类型的从属的顶层接口类
     * 
     * @param api
     *            API接口
     * @return 从属的顶层接口类
     */
    public static Class<?> getTop(Class<?> api) {
        return provider.getTop(api);
    }

    /**
     * 获取Compute(Nova)服务
     * 
     * @author sxp
     * @since Mar 20, 2018
     */
    public static ComputeService getComputeServices(PowervcConstant.Version version) {
        return get(ComputeService.class, version);
    }

    /**
     * 获取Networks(Quantum)服务
     * 
     * @author sxp
     * @since Mar 21, 2018
     */
    public static NetworkingService getNetworkingServices(PowervcConstant.Version version) {
        return get(NetworkingService.class, version);
    }

    /**
     * 获取Volumes(Cinder)服务
     * 
     * @author sxp
     * @since Mar 21, 2018
     */
    public static BlockStorageService getBlockStorageServices(PowervcConstant.Version version) {
        return get(BlockStorageService.class, version);
    }

    /**
     * 获取Images(Glance)服务
     * 
     * @author sxp
     * @since Mar 27, 2018
     */
    public static ImageService getImageServices(PowervcConstant.Version version) {
        return get(ImageService.class, version);
    }

    /**
     * 获取validation(ttv)服务
     */
    public static ValidateService getValidateServices(PowervcConstant.Version version) {
        return get(ValidateService.class, version);
    }

    /**
     * 获取Notify服务
     */
    public static NotificationService getNotificationService(PowervcConstant.Version version) {
        return get(NotificationService.class, version);
    }

    // ----private methods

    /**
     * 初始化Provider提供者
     * <p>
     * 配置文件：/src/main/resources/META-INF/services/APIProvider
     * </p>
     * 
     * @author sxp
     * @since Mar 20, 2018
     */
    private static APIProvider initializeProvider() {
        APIProvider p = new DefaultAPIProvider();
        p.initialize();
        return p;
    }

}
