package com.bocloud.cmp.provider.powervc4j.api;

import com.bocloud.cmp.model.powervc.PowervcConstant;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.image.ImageService;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkingService;
import com.bocloud.cmp.provider.powervc4j.api.notification.NotificationService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageService;
import com.bocloud.cmp.provider.powervc4j.api.validation.ValidateService;

;

/**
 * PowerVC客户端
 * <p>
 * 这个虚拟的客户端，目的是将获取各种服务的操作和PowervcDriver解耦。
 * </p>
 * 
 * @author sxp
 * @since Mar 20, 2018
 */
public class PVClient {
    private static final ThreadLocal<PowervcProvider> drivers = new ThreadLocal<PowervcProvider>();

    public PVClient(PowervcProvider driver) {
        drivers.set(driver);
    }

    /**
     * 获取当前线程中使用的PowervcDriver
     * 
     * @author sxp
     * @since Mar 20, 2018
     */
    public static PowervcProvider getCurrent() {
        return drivers.get();
    }

    /**
     * 获取Compute(Nova)服务
     * 
     * @author sxp
     * @since Mar 20, 2018
     */
    public ComputeService compute(PowervcConstant.Version version) {
        return Apis.getComputeServices(version);
    }

    /**
     * 获取Networks(Quantum)服务
     * 
     * @author sxp
     * @since Mar 21, 2018
     */
    public NetworkingService networking(PowervcConstant.Version version) {
        return Apis.getNetworkingServices(version);
    }

    /**
     * 获取Volumes(Cinder)服务
     *
     * @author sxp
     * @since Mar 21, 2018
     */
    public BlockStorageService blockStorage(PowervcConstant.Version version) {
        return Apis.getBlockStorageServices(version);
    }

    /**
     * 获取Images(Glance)服务
     *
     * @author sxp
     * @since Mar 27, 2018
     */
    public ImageService image(PowervcConstant.Version version) {
        return Apis.getImageServices(version);
    }

    public ValidateService ttv(PowervcConstant.Version version) {
        return Apis.getValidateServices(version);
    }

    public NotificationService notify(PowervcConstant.Version version) {
        return Apis.getNotificationService(version);
    }

}
