package com.bocloud.cmp.provider.powervc4j.api.compute;

import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.powervc4j.api.base.BaseService;
import com.bocloud.cmp.provider.powervc4j.model.compute.RebootType;
import com.bocloud.cmp.provider.powervc4j.model.compute.ServerAction;
import com.bocloud.common.model.BocloudResult;

import java.util.List;
import java.util.Map;

/**
 * 云主机服务接口
 *
 * @author sxp
 * @since Mar 20, 2018
 */
public interface ServerService extends BaseService<ServerModel> {

    /**
     * 查询云主机列表
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    List<ServerModel> list(boolean allTenants);

    /**
     * 创建云主机
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    BocloudResult create(Map<String, Object> params);

    /**
     * 根据唯一标识获取云主机信息
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    ServerModel get(String instanceId);

    /**
     * 云主机启动、停止等操作
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    BocloudResult action(String instanceId, ServerAction action);

    /**
     * 云主机重启
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    BocloudResult reboot(String instanceId, RebootType type);

    /**
     * 云主机捕获前操作
     *
     * @author
     * @since
     */
    BocloudResult beforeCaptureImage(String instanceId, Map<String, Object> params);

    /**
     * 云主机捕获
     *
     * @author Tanshaolong
     * @since Jan 29, 2019
     */
    BocloudResult createImage(String instanceId, Map<String, Object> params);

    /**
     * 删除云主机
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    BocloudResult delete(String instanceId);

    /**
     * 修改云主机信息
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    BocloudResult update(ServerModel server);

    /**
     * 动态迁移云主机
     *
     * @author sxp
     * @since Apr 2, 2018
     */
    BocloudResult liveMigrate(String instanceId, Map<String, Object> params);

    /**
     * 修改云主机配置
     *
     * @author sxp
     * @since Apr 3, 2018
     */
    BocloudResult resize(String instanceId, Map<String, Object> params);

    /**
     * 创建云主机镜像
     *
     * @author sxp
     * @since Apr 3, 2018
     */
    BocloudResult createImage(String instanceId, String imageName);

    /**
     * 挂载卷
     * @param instanceId
     * @param volumeId
     * @return
     */
    BocloudResult attachVolume(String instanceId, VolumeModel volumeModel);

    /**
     * 设定卷是否在删除云主机的时候也随之删除
     * @param instanceId
     * @param volumeId
     * @param deleteVolumeWithVM
     * @return
     */
    BocloudResult retachVolume(String instanceId, VolumeModel volumeModel);

    /**
     * 移除卷
     * @param instanceId
     * @param volumeId
     * @return
     */
    BocloudResult detachVolume(String instanceId, VolumeModel volumeModel);

    /**
     * 挂载卷
     * @param instanceId
     * @param volumeId
     * @return
     */
    BocloudResult attachNetwork(String instanceId, String portId);

    /**
     * 移除卷
     * @param instanceId
     * @param volumeId
     * @return
     */
    BocloudResult detachNetworik(String instanceId, String portId);

    /**
     * 重置状态
     * @param instanceId
     * @return
     */
    public BocloudResult resetStatus(String instanceId);

}
