package com.bocloud.cmp.provider.powervc4j.api.storage;

import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.powervc4j.api.base.BaseService;
import com.bocloud.common.model.BocloudResult;

import java.util.List;

/**
 * 存储卷管理服务接口
 *
 * @author sxp
 * @since Mar 21, 2018
 */
public interface BlockVolumeService extends BaseService<VolumeModel> {

    /**
     * 查询存储卷信息
     *
     * @author sxp
     * @since Mar 30, 2018
     */
    VolumeModel get(String volumeId);

    /**
     * 新增存储卷
     *
     * @author sxp
     * @since Apr 4, 2018
     */
    BocloudResult create(VolumeModel model);

    /**
     * 修改存储卷
     *
     * @author sxp
     * @since Apr 4, 2018
     */
    BocloudResult modify(VolumeModel model);

    /**
     * 删除存储卷
     *
     * @author sxp
     * @since Apr 4, 2018
     */
    BocloudResult delete(String volumeId);

    /**
     * 扩容存储卷
     *
     * @author sxp
     * @since Apr 4, 2018
     */
    BocloudResult extend(String volumeId, int size);

    /**
     * 查询volume的metadata
     *
     * @param volumeId
     * @return
     */
    BocloudResult getMetadata(String volumeId);

    /**
     * 更新volume的metadata
     *
     * @param volumeId
     * @param metadata
     * @return
     */
    BocloudResult setMetadata(String volumeId, VolumeMetadataModel metadata);

    /**
     * 管理存储器现有卷
     *
     * @param volumeIds
     * @return
     */
    BocloudResult onboard(String storageName, List<String> volumeIds);

    /**
     * 取消管理存储上的卷
     *
     * @param volumeIds
     * @return
     */
    BocloudResult unmanage(String storageName, List<String> volumeIds);

    /**
     * 查询存储器上的所有可管理的卷
     *
     * @param storageName
     * @return
     */
    public BocloudResult allVolumes(String storageName);

}
