package com.bocloud.cmp.provider.powervc4j.impl.compute.v13;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VmNetworkBean;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ServerService;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.cmp.provider.powervc4j.model.compute.RebootType;
import com.bocloud.cmp.provider.powervc4j.model.compute.ServerAction;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.*;

/**
 * 云主机服务实现类
 *
 * @author sxp
 * @since Mar 20, 2018
 */
public class ServerServiceImpl extends BaseServiceImpl<ServerModel> implements ServerService {
    private static Logger logger = LoggerFactory.getLogger(ServerServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(ServerService.class);
    }

    /**
     * 将powervc中的状态按照CloudServer表的状态进行转换<br/><br/>
     * ============lyc 更新于2019-03-01============ <br/><br/>
     * <b>======PowervcStatus 来源于openstack======</b> <br/>
     * ACTIVE. The server is active. <br/>
     * BUILD. The server has not finished the original build process. <br/>
     * DELETED. The server is permanently deleted. <br/>
     * ERROR. The server is in error. <br/>
     * HARD_REBOOT. The server is hard rebooting. This is equivalent to pulling the power plug on a physical server, plugging it back in, and rebooting it. <br/>
     * MIGRATING. The server is being migrated to a new host. <br/>
     * PASSWORD. The password is being reset on the server. <br/>
     * PAUSED. In a paused state, the state of the server is stored in RAM. A paused server continues to run in frozen state. <br/>
     * REBOOT. The server is in a soft reboot state. A reboot command was passed to the operating system. <br/>
     * REBUILD. The server is currently being rebuilt from an image. <br/>
     * RESCUE. The server is in rescue mode. A rescue image is running with the original server image attached. <br/>
     * RESIZE. Server is performing the differential copy of data that changed during its initial copy. Server is down for this stage. <br/>
     * REVERT_RESIZE. The resize or migration of a server failed for some reason. The destination server is being cleaned up and the original source server is restarting. <br/>
     * SHELVED: The server is in shelved state. Depending on the shelve offload time, the server will be automatically shelved offloaded. <br/>
     * SHELVED_OFFLOADED: The shelved server is offloaded (removed from the compute host) and it needs unshelved action to be used again. <br/>
     * SHUTOFF. The server is powered off and the disk image still persists. <br/>
     * SOFT_DELETED. The server is marked as deleted but the disk images are still available to restore. <br/>
     * SUSPENDED. The server is suspended, either by request or necessity. This status appears for only the XenServer/XCP, KVM, and ESXi hypervisors. Administrative users can suspend an instance if it is infrequently used or to perform system maintenance. When you suspend an instance, its VM state is stored on disk, all memory is written to disk, and the virtual machine is stopped. Suspending an instance is similar to placing a device in hibernation; memory and vCPUs become available to create other instances. <br/>
     * UNKNOWN. The state of the server is unknown. Contact your cloud provider. <br/>
     * VERIFY_RESIZE. System is awaiting confirmation that the server is operational after a move or resize. <br/>
     * <b>======CloudServerStatus 来源于前端======</b> <br/>
     * RUNNING: '运行中', <br/>
     * STOPPED: '关机', <br/>
     * STOPPING: '关机中', <br/>
     * STARTING: '开机中', <br/>
     * SUSPENDED: '挂起', <br/>
     * SUSPENDING: '挂起中', <br/>
     * ACTIVING: '激活中', <br/>
     * PAUSED: '停止', <br/>
     * PAUSING: '停止中', <br/>
     * RECOVERING: '恢复中', <br/>
     * BUILDING: '创建中', <br/>
     * RESTARTING: '重启中', <br/>
     * EXCEPTION: '异常', <br/>
     * SYNSEXCEPTION: '同步异常', <br/>
     * UNKNOWN: '已断开', <br/>
     * UNKNOWNON: '已断开', <br/>
     * UNKNOWNOFF: '已断开', <br/>
     * UNKNOWNSUSPEND: '已断开' <br/>
     *
     * @param status
     * @return
     */
    private String convertStatus(String status) {
        switch (status) {
            case "ACTIVE": return BocloudStatus.Server.RUNNING.name();
            case "BUILD": return BocloudStatus.Server.BUILDING.name();
            case "DELETED": return BocloudStatus.Server.UNKNOWN.name();
            case "ERROR": return BocloudStatus.Server.ERROR.name();
            case "HARD_REBOOT": return BocloudStatus.Server.RESTARTING.name();
            case "MIGRATING": return BocloudStatus.Server.BUILDING.name();
            case "PASSWORD": return BocloudStatus.Server.RESIZING.name();
            case "PAUSED": return BocloudStatus.Server.PAUSED.name();
            case "REBOOT": return BocloudStatus.Server.SOFTRESTARTING.name();
            case "REBUILD": return BocloudStatus.Server.BUILDING.name();
            case "RESCUE": return BocloudStatus.Server.UNKNOWN.name();
            case "RESIZE": return BocloudStatus.Server.RESIZING.name();
            case "REVERT_RESIZE": return BocloudStatus.Server.RESIZING.name();
            case "SHELVED": return BocloudStatus.Server.SHELVED.name();
            case "SHELVED_OFFLOADED": return BocloudStatus.Server.SHELVED.name();
            case "SHUTOFF": return BocloudStatus.Server.STOPPED.name();
            case "SOFT_DELETED": return BocloudStatus.Server.UNKNOWN.name();
            case "SUSPENDED": return BocloudStatus.Server.SUSPENDED.name();
            case "UNKNOWN": return BocloudStatus.Server.UNKNOWN.name();
            case "VERIFY_RESIZE": return BocloudStatus.Server.RESIZING.name();
            default: return BocloudStatus.Server.UNKNOWN.name();
        }
    }

    @Override
    public List<ServerModel> list() {
        return list(false);
    }

    @Override
    public List<ServerModel> list(boolean allTenants) {
        String apiSuffix = "/servers/detail";

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));

        // Specify the all_tenants query parameter to list all instances for all
        // projects.
        // If the value of this parameter is not specified, it is treated as
        // True.
        // If the value is specified, 1, t, true, on, y and yes are treated as
        // True.
        // 0, f, false, off, n and no are treated as False. (They are
        // case-insensitive.)
        params.put("all_tenants", allTenants);

        return doList(apiSuffix, params);
    }

    @Override
    protected List<ServerModel> processListResult(String message) {
        JSONArray servers = JSON.parseObject(message).getJSONArray("servers");
        List<ServerModel> models = new ArrayList<>(servers.size());
        for (int i = 0; i < servers.size(); i++) {
            models.add(convertToServerModel(servers.getJSONObject(i)));
        }
        return models;
    }

    private ServerModel convertToServerModel(JSONObject server) {
        ServerModel serverModel = new ServerModel();
        serverModel.setInstanceId(server.getString("id"));
        serverModel.setName(server.getString("name"));
        serverModel.setPassword(server.getString("adminPass"));
        serverModel.setKeypairName(server.getString("key_name"));
        serverModel.setGmtVmCreate(server.getDate("created"));
        serverModel.setGmtModify(server.getDate("updated"));
        serverModel.setCpu(server.getInteger("cpus"));
        serverModel.setHmcCpu(server.getDouble("vcpus"));
        serverModel.setMemory(server.getInteger("memory_mb") == null ? null : new BigDecimal(server.getInteger("memory_mb")));
        serverModel.setDisk(server.getInteger("root_gb"));
        serverModel.setOsName(server.getString("operating_system"));
        serverModel.setFlavorUuid(server.getJSONObject("flavor") == null ? null : server.getJSONObject("flavor").getString("id"));
        serverModel.setHealthStatus(server.getJSONObject("health_status"));
        serverModel.setTaskState(server.getString("OS-EXT-STS:task_state"));
        serverModel.setImageUuid(server.getJSONObject("image") == null ? null : server.getJSONObject("image").getString("id"));

        JSONArray groups = server.getJSONArray("security_groups");
        if (groups != null && (groups.size()) > 0) {
            Set<String> groupNames = new HashSet<>(MapUtils.capacity(groups.size()));
            for (Object group : groups) {
                if (group instanceof JSONObject) {
                    groupNames.add(((JSONObject) group).getString("name"));
                }
            }
            serverModel.setGroups(JSON.toJSONString(groupNames));
        }

        if (server.containsKey("addresses")) {
            List<VmNetworkBean> networkBeans = new ArrayList<>();
            JSONObject addresses = server.getJSONObject("addresses");
            if (addresses != null && addresses.size() > 0) {
                for (Map.Entry<String, Object> network : addresses.entrySet()) {
                    JSONArray ips = (JSONArray) network.getValue();
                    for (Object ip : ips) {
                        JSONObject ipObject = (JSONObject) ip;
                        VmNetworkBean networkBean = new VmNetworkBean();
                        networkBean.setAddress(ipObject.getString("addr"));
                        networkBean.setMacAddress(ipObject.getString("OS-EXT-IPS-MAC:mac_addr"));
                        networkBean.setConnected(true);
                        networkBean.setNetworkName(network.getKey());
                        networkBeans.add(networkBean);
                    }
                }
            }
            serverModel.setPrivateIps(JSON.toJSONString(networkBeans));
        }

        String status = server.getString("status");
        if (StringUtils.isNotEmpty(status)) {
            serverModel.setStatus(convertStatus(status));
        }

        String hypervisorHostname = server.getString("OS-EXT-SRV-ATTR:hypervisor_hostname");
        if (StringUtils.isNotEmpty(hypervisorHostname)) {
            serverModel.setHostName(hypervisorHostname);
        } else {
            serverModel.setHostName(server.getString("OS-EXT-SRV-ATTR:host"));
        }

        JSONArray volumes = server.getJSONArray("os-extended-volumes:volumes_attached");
        if (volumes != null){
            List<VolumeModel> volumeModels = new ArrayList<>(volumes.size());
            for (int i = 0; i < volumes.size(); i++) {
                JSONObject volume = volumes.getJSONObject(i);
                VolumeModel volumeModel = new VolumeModel();
                volumeModel.setVolumeUuid(volume.getString("id"));
                volumeModels.add(volumeModel);
            }
            serverModel.setDiskDevices(volumeModels);
        }

        String rmcStatus = "active";
        if (server.containsKey("health_status") && server.getJSONObject("health_status").containsKey("health_value")) {
            if (server.getJSONObject("health_status").getString("health_value").equals("WARNING")) {
                for (int i = 0; i < server.getJSONObject("health_status").getJSONArray("value_reason").size(); i++) {
                    JSONObject valueReason = server.getJSONObject("health_status").getJSONArray("value_reason").getJSONObject(i);
                    if (valueReason.getString("resource_property_key").equals("rmc_state")) {
                        rmcStatus = valueReason.getString("resource_property_value");
                        break;
                    }
                }
            } else if (server.getJSONObject("health_status").getString("health_value").equals("PENDING")) {
                rmcStatus = "unknown";
            }
        }
        JSONObject props = new JSONObject();
        props.put("rmcStatus", rmcStatus);
        serverModel.setProps(props.toJSONString());

        return serverModel;
    }

    @Override
    public ServerModel get(String instanceId) {
        if (StringUtils.isEmpty(instanceId)) {
            throw new DriverException("查询云主机信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/servers/%s", instanceId);
        return doGet(apiSuffix);
    }

    @Override
    protected ServerModel processGetResult(String message) {
        return convertToServerModel(JSONTools.isJSONObj(message).getJSONObject("server"));
    }

    @Override
    public BocloudResult create(Map<String, Object> params) {
        try {
            String apiSuffix = "/servers";

            Result result = doCreate(apiSuffix, params);

            ServerModel createdServer = null;
            if (result.isSuccess()) {
                createdServer = processGetResult(result.getMessage());
                if (createdServer != null) {
                    String instanceId = createdServer.getInstanceId();
                    createdServer = get(instanceId);
                }
            }

            return new BocloudResult(result.isSuccess(), createdServer, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "创建云主机失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult action(String instanceId, ServerAction action) {
        if (StringUtils.isEmpty(instanceId)) {
            throw new DriverException("操作云主机信息时，唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s/action", instanceId);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        // 注意:value值不能使用null
        params.put(action.getUrl(), "");

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "操作云主机失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult reboot(String instanceId, RebootType type) {
        if (StringUtils.isEmpty(instanceId)) {
            throw new DriverException("操作云主机信息时，唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s/action", instanceId);

        Map<String, Object> typeMap = new HashMap<>(MapUtils.capacity(1));
        typeMap.put("type", type.name());

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("reboot", typeMap);

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "重启云主机失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult beforeCaptureImage(String instanceId, Map<String, Object> params) {
        String apiSuffix = String.format("/servers/%s/action", instanceId);
        Result result = work(RequestMethod.POST, apiSuffix, params);
        return new BocloudResult(result.isSuccess(), null, result.getMessage());
    }

    @Override
    public BocloudResult createImage(String instanceId, Map<String, Object> params) {
        if (StringUtils.isEmpty(instanceId)) {
            throw new DriverException("操作云主机信息时，唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s/action", instanceId);

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "捕获云主机快照失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult delete(String instanceId) {
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "云主机唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s", instanceId);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult update(ServerModel server) {
        Objects.requireNonNull(server, "model");

        String instanceId = server.getInstanceId();
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "云主机唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s", instanceId);
        Map<String, Object> params = getUpdateServerParams(server);

        Result result = doUpdate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        String message = result.getMessage();

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject serverJson = json.getJSONObject("server");

            ServerModel model = convertToServerModel(serverJson);

            return new BocloudResult(true, model, result.getMessage());
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            return new BocloudResult(false, msg);
        }
    }

    private Map<String, Object> getUpdateServerParams(ServerModel server) {
        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("name", server.getName());

        // 还支持修改以下属性:
        // accessIPv4
        // accessIPv6
        // OS-DCF:diskConfig
        // description

        return params;
    }

    @Override
    public BocloudResult liveMigrate(String instanceId, Map<String, Object> params) {
        if (StringUtils.isEmpty(instanceId)) {
            throw new DriverException("操作云主机信息时，唯一标识不能为空");
        }

        String apiSuffix = String.format("/servers/%s/action", instanceId);

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "动态迁移云主机失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult resize(String instanceId, Map<String, Object> params) {
        String apiSuffix = String.format("/servers/%s/action", instanceId);

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "修改云主机配置失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName) {
        String apiSuffix = String.format("/servers/%s/action", instanceId);

        JSONObject json = new JSONObject(MapUtils.capacity(1));
        json.put("name", imageName);

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put(ServerAction.CREATE_IMAGE.getUrl(), json);

        try {
            Result result = work(RequestMethod.POST, apiSuffix, params);

            return new BocloudResult(result.isSuccess(), null, result.getMessage());
        } catch (Exception e) {
            String errorMsg = "云主机创建镜像失败:" + e.getMessage();
            logger.error(errorMsg, e);

            return new BocloudResult(false, errorMsg);
        }
    }

    @Override
    public BocloudResult attachVolume(String instanceId, VolumeModel volumeModel) {
        String apiSuffix = String.format("/servers/%s/os-volume_attachments", instanceId);
        JSONObject attachment = new JSONObject();
        attachment.put("volumeId", volumeModel.getVolumeUuid());
        JSONObject params = new JSONObject();
        params.put("volumeAttachment", attachment);
        Result result = doCreate(apiSuffix, params);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult retachVolume(String instanceId, VolumeModel volumeModel) {
        String apiSuffix = String.format("/servers/%s/os-volume_attachments/%s", instanceId, volumeModel.getVolumeUuid());
        Map<String, Object> volumeParams = new HashMap<>(MapUtils.capacity(1));
        volumeParams.put("volumeId", volumeModel.getVolumeUuid());
        volumeParams.put("delete_on_termination", volumeModel.getDeleteWithInstance());
        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("volumeAttachment", volumeParams);
        Map<String, Object> header = new HashMap<>();
        header.put("X-OpenStack-Nova-API-Version", 2.22);
        Result result = work(RequestMethod.PUT, apiSuffix, params, header, 60 * 1000);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult detachVolume(String instanceId, VolumeModel volumeModel) {
        String apiSuffix = String.format("/servers/%s/os-volume_attachments/%s", instanceId, volumeModel.getVolumeUuid());
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult attachNetwork(String instanceId, String portId) {
        String apiSuffix = String.format("/servers/%s/os-interface", instanceId);
        JSONObject attachment = new JSONObject();
        attachment.put("port_id", portId);
        JSONObject params = new JSONObject();
        params.put("interfaceAttachment", attachment);
        Result result = doCreate(apiSuffix, params);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult detachNetworik(String instanceId, String portId) {
        String apiSuffix = String.format("/servers/%s/os-interface/%s", instanceId, portId);
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }


    @Override
    public BocloudResult resetStatus(String instanceId){
        String apiSuffix = String.format("/servers/%s/action", instanceId);
        JSONObject resetState = new JSONObject();
        resetState.put("state", "active");
        JSONObject params = new JSONObject();
        params.put("os-resetState",resetState);
        Result result = doCreate(apiSuffix, params);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }
}
