package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FlavorExtraSpecsModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.FlavorExtraService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * FlavorExtraService(配置规格扩展信息)服务接口
 *
 * @author lyc
 * @since 2019-01-29
 */
public class FlavorExtraServiceImpl extends BaseServiceImpl<FlavorExtraSpecsModel> implements FlavorExtraService {
    private static Logger logger = LoggerFactory.getLogger(FlavorExtraServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(FlavorExtraService.class);
    }

    @Override
    protected List<FlavorExtraSpecsModel> processListResult(String message) {
        return null;
    }

    @Override
    protected FlavorExtraSpecsModel processGetResult(String message) {
        return convertToFlavorExtraSpecsModel(JSON.parseObject(message).getJSONObject("extra_specs"));
    }

    private FlavorExtraSpecsModel convertToFlavorExtraSpecsModel(JSONObject flavorExtra) {
        FlavorExtraSpecsModel model = new FlavorExtraSpecsModel();
        model.setPriority(flavorExtra.getInteger("powervm:availability_priority"));
        model.setDedicatedProc(flavorExtra.getBoolean("powervm:dedicated_proc"));
        // 2019-07-17 : 需要统一将内存代为转换成GB
        model.setMaxMem(flavorExtra.get("powervm:max_mem") == null ? null : flavorExtra.getBigDecimal("powervm:max_mem").divide(new BigDecimal(1024), 10, RoundingMode.HALF_UP));
        model.setMinMem(flavorExtra.get("powervm:min_mem") == null ? null : flavorExtra.getBigDecimal("powervm:min_mem").divide(new BigDecimal(1024), 10, RoundingMode.HALF_UP));
        model.setMaxProcUnits(flavorExtra.getDouble("powervm:max_proc_units"));
        model.setMinProcUnits(flavorExtra.getDouble("powervm:min_proc_units"));
        model.setProcUnits(flavorExtra.getDouble("powervm:proc_units"));
        model.setMaxVcpu(flavorExtra.getInteger("powervm:max_vcpu"));
        model.setMinVcpu(flavorExtra.getInteger("powervm:min_vcpu"));
        model.setSharedWeight(flavorExtra.getInteger("powervm:shared_weight"));
        model.setUncapped(flavorExtra.getBoolean("powervm:uncapped"));
        model.setProcCompatibility(flavorExtra.getString("powervm:processor_compatibility"));
        return model;
    }


    @Override
    public FlavorExtraSpecsModel get(String flavorUuid) {
        String apiSuffix = String.format("/flavors/%s/os-extra_specs", flavorUuid);
        return doGet(apiSuffix);
    }

    @Override
    public BocloudResult create(String flavorUuid, FlavorExtraSpecsModel model) {
        String apiSuffix = String.format("/flavors/%s/os-extra_specs", flavorUuid);

        JSONObject flavorExtraJson = new JSONObject();
        flavorExtraJson.put("powervm:availability_priority", model.getPriority());
        flavorExtraJson.put("powervm:dedicated_proc", model.getDedicatedProc());
        // 2019-07-17 : 需要统一将内存代为转换成GB
        flavorExtraJson.put("powervm:max_mem", model.getMaxMem() == null ? null : model.getMaxMem().multiply(new BigDecimal(1024)).intValue());
        flavorExtraJson.put("powervm:min_mem", model.getMinMem() == null ? null : model.getMinMem().multiply(new BigDecimal(1024)).intValue());
        flavorExtraJson.put("powervm:max_proc_units", model.getMaxProcUnits());
        flavorExtraJson.put("powervm:min_proc_units", model.getMinProcUnits());
        flavorExtraJson.put("powervm:proc_units", model.getProcUnits());
        flavorExtraJson.put("powervm:max_vcpu", model.getMaxVcpu());
        flavorExtraJson.put("powervm:min_vcpu", model.getMinVcpu());
        flavorExtraJson.put("powervm:shared_weight", model.getSharedWeight());
        flavorExtraJson.put("powervm:uncapped", model.getUncapped());
        flavorExtraJson.put("powervm:processor_compatibility", model.getProcCompatibility());

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("extra_specs", flavorExtraJson);

        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        model = convertToFlavorExtraSpecsModel(JSON.parseObject(result.getMessage()).getJSONObject("extra_specs"));

        return new BocloudResult(result.isSuccess(), model, result.getMessage());
    }

    @Override
    public List<FlavorExtraSpecsModel> list() {
        return null;
    }
}
