package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.FlavorService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * FlavorService(配置规格)服务实现
 * 
 * @author sxp
 * @since Mar 27, 2018
 */
public class FlavorServiceImpl extends BaseServiceImpl<FlavorModel> implements FlavorService {
    private static Logger logger = LoggerFactory.getLogger(FlavorServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(FlavorService.class);
    }

    @Override
    public List<FlavorModel> list() {
        return list(false);
    }

    @Override
    public List<FlavorModel> list(boolean showDetail) {
        String apiSuffix = "/flavors";
        if (showDetail) {
            apiSuffix += "/detail";
        }
        return doList(apiSuffix);
    }

    @Override
    protected List<FlavorModel> processListResult(String message) {
        JSONArray flavors = JSON.parseObject(message).getJSONArray("flavors");
        List<FlavorModel> models = new ArrayList<>(flavors.size());
        for (int i = 0; i < flavors.size(); i++) {
            models.add(convertToFlavorModel(flavors.getJSONObject(i)));
        }
        return models;
    }

    @Override
    public FlavorModel get(String id) {
        String apiSuffix = String.format("/flavors/%s", id);
        return doGet(apiSuffix);
    }

    @Override
    protected FlavorModel processGetResult(String message) {
        return convertToFlavorModel(JSONTools.isJSONObj(message).getJSONObject("flavor"));
    }

    private FlavorModel convertToFlavorModel(JSONObject flavor) {
        FlavorModel model = new FlavorModel();
        model.setFlavorUuid(flavor.getString("id"));
        model.setName(flavor.getString("name"));
        model.setCpu(flavor.getIntValue("vcpus"));
        // 2019-07-17 : 需要统一将内存代为转换成GB
        model.setMemory(flavor.get("ram") == null ? null : flavor.getBigDecimal("ram").divide(new BigDecimal(1024), 10, RoundingMode.HALF_UP)); // 单位:MiB
        model.setDisk(flavor.getIntValue("disk")); // 单位:GiB
        model.setStatus(flavor.getBoolean("os-flavor-access:is_public") ? "public" : "private");
        return model;
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        String apiSuffix = "/flavors";

        if (StringUtils.isEmpty(flavor.getFlavorUuid())) {
            flavor.setFlavorUuid(UUID.randomUUID().toString());
        }

        JSONObject flavorJson = new JSONObject();
        flavorJson.put("id", flavor.getFlavorUuid());
        flavorJson.put("name", flavor.getName());
        // 2019-07-17 : 需要统一将内存代为转换成GB
        flavorJson.put("ram", flavor.getMemory().multiply(new BigDecimal(1024)).intValue());
        flavorJson.put("vcpus", flavor.getCpu());
        flavorJson.put("disk", flavor.getDisk());

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("flavor", flavorJson);

        Result result = doCreate(apiSuffix, params);
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }

        flavor = convertToFlavorModel(JSON.parseObject(result.getMessage()).getJSONObject("flavor"));

        return new BocloudResult(result.isSuccess(), flavor, result.getMessage());
    }

    @Override
    public BocloudResult delete(String flavorId) {
        if (StringUtils.isEmpty(flavorId)) {
            return new BocloudResult(false, "配置规则唯一标识不能为空");
        }

        String apiSuffix = String.format("/flavors/%s", flavorId);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

}
