package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.HmcHostModel;
import com.bocloud.cmp.model.OsHosts;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.compute.HmcService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.MapTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;

public class HmcServiceImpl extends BaseServiceImpl<HmcHostModel> implements HmcService {
    private static Logger logger = LoggerFactory.getLogger(HmcServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(ComputeService.class);
    }

    @Override
    public List<HmcHostModel> list() {
        return doList("/ibm-hmcs/detail");
    }

    @Override
    protected List<HmcHostModel> processListResult(String message) {
        JSONArray hmcs = JSON.parseObject(message).getJSONArray("hmcs");
        return JSON.parseArray(hmcs.toJSONString(), HmcHostModel.class);
    }

    @Override
    protected HmcHostModel processGetResult(String message) {
        JSONObject hmcHost = JSON.parseObject(message).getJSONObject("hmc").getJSONObject("registration");
        return JSON.parseObject(hmcHost.toString(), HmcHostModel.class);
    }

    /**
     * 向powervc请求新增hmc
     *
     * <pre>
     * 新增hmc机器的请求powervc的参数
     * {
     * "hmc": {
     * "registration": {
     * "access_ip": "1.2.3.4",
     * "user_id": "hscroot",
     * "password": "passw0rd",
     * "hmc_display_name": "My HMC 1"
     * }
     * }
     * }
     * </pre>
     */
    @SuppressWarnings("unchecked")
    @Override
    public BocloudResult create(HmcHostModel host) {
        String apiSuffix = "/ibm-hmcs";
        if (host == null || !StringUtils.hasText(host.getIp())) {
            return new BocloudResult(false, "新增hmc主机，请求参数不正确");
        }

        JSONObject params = new JSONObject();
        params.put("hmc", MapTools.simpleMap("registration", host));
        Result result = doCreate(apiSuffix, params);
        logger.info("create hmc result:{}", result);
        if (result.isFailed()) {
            return new BocloudResult(false, "新增hmc主机失败:" + result.getMessage());
        } else {
            return new BocloudResult(result.isSuccess(), result.getMessage(), "新增hmc主机成功");
        }
    }

    @Override
    public BocloudResult modify(HmcHostModel host) {
        String apiSuffix = String.format("/ibm-hmcs/%s", host.getUuid()) ;
        if (host == null || !StringUtils.hasText(host.getIp())) {
            return new BocloudResult(false, "修改hmc主机，请求参数不正确");
        }

        JSONObject params = new JSONObject();
        params.put("registration", host);
        Result result = doUpdate(apiSuffix, params);
        logger.info("modify hmc result:{}", result);
        if (result.isFailed()) {
            return new BocloudResult(false, "修改hmc主机失败:" + result.getMessage());
        } else {
            return new BocloudResult(result.isSuccess(), result.getMessage(), "修改hmc主机成功");
        }
    }

    @Override
    public HmcHostModel get(String uuid) {
        if (StringUtils.isEmpty(uuid)) {
            throw new DriverException("查询hmc信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/ibm-hmcs/%s", uuid);
        return doGet(apiSuffix);
    }

    @Override
    public BocloudResult delete(String uuid) {
        if (StringUtils.isEmpty(uuid)) {
            return new BocloudResult(false, "移除hmc时，hmc唯一标识不能为空");
        }

        String apiSuffix = String.format("/ibm-hmcs/%s", uuid);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public OsHosts getHosts(String uuid) {
        if (StringUtils.isEmpty(uuid)) {
            throw new DriverException("查询hmc上的小机信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/ibm-hmcs/%s/hosts", uuid);
        return customGet(OsHosts.class, apiSuffix);
    }


}
