package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.powervc.HostSeaAdapterModel;
import com.bocloud.cmp.model.powervc.NetworkSeaAdaterModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.HostSeaService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class HostSeaServiceImpl extends BaseServiceImpl<HostSeaAdapterModel> implements HostSeaService {
    private static Logger logger = LoggerFactory.getLogger(HostSeaServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(HostSeaService.class);
    }

    @Override
    protected List<HostSeaAdapterModel> processListResult(String message) {
        logger.debug("HostSeaService processListResult info:{}", message);
        try {
            List<HostSeaAdapterModel> hostSeaAdapters = new ArrayList<>();

            JSONObject json = JSONTools.isJSONObj(message);
            for (int i = 0; i < json.getJSONArray("host-seas").size(); i++) {
                JSONObject sea = json.getJSONArray("host-seas").getJSONObject(i);
                for (int j = 0; j < sea.getJSONArray("adapters").size(); j++) {
                    JSONObject adapter = sea.getJSONArray("adapters").getJSONObject(j);

                    HostSeaAdapterModel hostSeaAdapter = new HostSeaAdapterModel();
                    hostSeaAdapter.setSeaName(adapter.getString("sea_name"));
                    hostSeaAdapter.setVswitch(adapter.getString("vswitch"));
                    hostSeaAdapter.setPvid(adapter.getInteger("pvid"));
                    hostSeaAdapter.setLparId(adapter.getInteger("lpar_id"));
                    hostSeaAdapter.setInital(adapter.getBoolean("default"));
                    hostSeaAdapter.setState(adapter.getString("state"));
                    hostSeaAdapter.setLparName(adapter.getString("lpar_name"));
                    hostSeaAdapter.setServerHost(sea.getString("host_name"));
                    hostSeaAdapters.add(hostSeaAdapter);
                }
            }
            return hostSeaAdapters;
        } catch (Exception e) {
            String msg = "解析数据[" + message + "]出错:" + e.getMessage();
            logger.error(msg);
            throw new DriverException(msg, e);
        }
    }

    @Override
    protected HostSeaAdapterModel processGetResult(String message) {
        return null;
    }

    @Override
    public List<HostSeaAdapterModel> list() {
        return doList("/host-seas");
    }

    @Override
    public List<HostSeaAdapterModel> list(long vlanId) {
        return doList("/host-seas?vlanid=" + vlanId);
    }

    @Override
    public List<HostSeaAdapterModel> list(String networId, long vlanId) {
        return doList("/host-seas?vlanid=" + vlanId + "&network_id=" + networId);
    }

    @Override
    public BocloudResult createMapping(List<HostSeaAdapterModel> seas) {
        JSONArray array = new JSONArray();
        for (HostSeaAdapterModel sea : seas) {
            JSONObject object = new JSONObject();
            object.put("host_name", sea.getServerHost());
            JSONArray networks = new JSONArray();
            JSONObject network = new JSONObject();
            network.put("network_id", sea.getNetworkId());
            if (sea.getSeaName().equals("not-applicable")) {
                network.put("primary_sea", "not-applicable");
            } else {
                JSONObject primarySea = new JSONObject();
                primarySea.put("lpar_id", sea.getLparId());
                primarySea.put("name", sea.getSeaName());
                network.put("primary_sea", primarySea);
            }
            networks.add(network);
            object.put("networks", networks);
            array.add(object);
        }
        JSONObject params = new JSONObject();
        params.put("host-network-mapping", array);
        Result result = doCreate("/host-network-mapping", params);
        if (result.isFailed()) {
            logger.error("创建host-network-mapping失败:{}", StringEscapeUtils.unescapeJava(result.getMessage()));
            return new BocloudResult(false, StringEscapeUtils.unescapeJava(result.getMessage()));
        }
        return new BocloudResult(true, "创建host-network-mapping成功");
    }

    @Override
    public List<NetworkSeaAdaterModel> listMapping() {
        List<NetworkSeaAdaterModel> models = new ArrayList<>();

        JSONObject jsonObject = customGet(JSONObject.class, "/host-network-mapping");
        JSONArray servers = jsonObject.getJSONArray("host-network-mapping");
        for (int i = 0; i < servers.size(); i++) {
            JSONObject server = servers.getJSONObject(i);
            JSONArray networks = server.getJSONArray("networks");
            for (int j = 0; j < networks.size(); j++) {
                JSONObject network = networks.getJSONObject(j);
                if (!network.getString("primary_sea").equals("not-applicable")) {
                    // Powervc的网络可以设定“请勿在此主机上使用此网络”，此时返回数据中的primary_sea是字符串值为not-applicable，不是一个JSONObject
                    NetworkSeaAdaterModel model = new NetworkSeaAdaterModel();
                    model.setServerHost(server.getString("host_name"));
                    model.setNetworkUuid(network.getString("network_id"));
                    model.setLparId(network.getJSONObject("primary_sea").getInteger("lpar_id"));
                    model.setSeaName(network.getJSONObject("primary_sea").getString("name"));
                    models.add(model);
                }
            }
        }

        return models;
    }
}