package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ext.HypervisorService;
import com.bocloud.cmp.provider.powervc4j.core.Constant;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.utils.JSONTools;

/**
 * 宿主机服务实现类
 * 
 * @author sxp
 * @since Mar 20, 2018
 */
public class HypervisorServiceImpl extends BaseServiceImpl<HostModel> implements HypervisorService {

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(HypervisorService.class);
    }

    // ----list

    @Override
    public List<HostModel> list() {
        String apiSuffix = "/os-hypervisors/detail";
        return doList(apiSuffix);
    }

    private HostModel convertToHostModel(JSONObject hyperv) {
        HostModel model = new HostModel();

        model.setUuid(hyperv.getString("id"));
        model.setCatalog(Constant.VENDOR_TYPE);
        model.setName(hyperv.getJSONObject("service").getString("host_display_name"));
        model.setHostName(hyperv.getString("hypervisor_hostname"));
        // PBSM-7022 POWERVC中就没有主机ip地址，所以下面这句话不要了
        //model.setHostIp(hyperv.getString("host_ip"));
        model.setCpuNum(hyperv.getInteger("vcpus"));
        model.setCpuCore(hyperv.getInteger("vcpus"));
        model.setCpuTotal(hyperv.getLong("vcpus"));
        model.setCpuUsed(hyperv.getLong("vcpus_used"));
        model.setCpuHz(hyperv.getLong("cpu_frequency"));
        model.setHmcCpu(hyperv.getDouble("proc_units"));
        model.setHmcCpuReserved(hyperv.getDouble("proc_units_reserved"));
        model.setHmcCpuUsed(hyperv.getDouble("proc_units_used"));
        // 2019-07-17 : 需要统一将内存代为转换成GB
        model.setMemTotal(hyperv.get("memory_mb") == null ? null : hyperv.getDouble("memory_mb") / 1024);
        model.setMemUsed(hyperv.get("memory_mb_used") == null ? null : hyperv.getDouble("memory_mb_used") / 1024);
        model.setMemReserved(hyperv.get("memory_mb_reserved") == null ? null : hyperv.getDouble("memory_mb_reserved") / 1024);
        model.setDiskTotal(hyperv.getDouble("local_gb"));
        model.setDiskUsed(hyperv.getDouble("local_gb_used"));

        String cpuModel = hyperv.getJSONObject("cpu_info").getString("model");
        if (StringUtils.isNotEmpty(cpuModel)) {
            model.setCpuModel(cpuModel);
        }

        String cpuVendor = hyperv.getJSONObject("cpu_info").getString("vendor");
        if (StringUtils.isNotEmpty(cpuVendor)) {
            model.setCpuVendor(cpuVendor);
        }

        model.setVmNum(hyperv.getInteger("running_vms"));
        switch (hyperv.getString("state")) {
        case "up":
            model.setPowerState(BocloudStatus.Server.RUNNING.name());
            break;
        case "down":
            model.setPowerState(BocloudStatus.Server.STOPPED.name());
            break;
        default:
            model.setPowerState(BocloudStatus.Server.UNKNOWN.name());
            break;
        }
        switch (hyperv.getString("status")) {
        case "enabled":
            model.setStatus(BocloudStatus.Server.RUNNING.name());
            break;
        case "disabled":
            model.setStatus(BocloudStatus.Server.STOPPED.name());
            break;
        default:
            model.setStatus(BocloudStatus.Server.UNKNOWN.name());
            break;
        }

        JSONObject topologyJson = hyperv.getJSONObject("cpu_info").getJSONObject("topology");
        if (topologyJson != null) {
            model.setCpuNum(topologyJson.getInteger("sockets"));
            model.setCpuCore(topologyJson.getInteger("cores"));
        }

        if (hyperv.containsKey("service")) {
            List<String> hmcUuids = hyperv.getJSONObject("service").getJSONArray("hmc_uuids").toJavaList(String.class);
            model.setHmcUuids(hmcUuids);
        }
        ;
        return model;
    }

    @Override
    protected List<HostModel> processListResult(String message) {
        JSONArray hypervisors = JSONTools.isJSONObj(message).getJSONArray("hypervisors");
        List<HostModel> models = new ArrayList<>(hypervisors.size());
        for (Object obj : hypervisors) {
            models.add(convertToHostModel((JSONObject) obj));
        }
        return models;
    }

    @Override
    protected HostModel processGetResult(String message) {
        return convertToHostModel(JSON.parseObject(message).getJSONObject("hypervisor"));
    }

    @Override
    public HostModel get(String id) {
        String apiSuffix = String.format("/os-hypervisors/%s", id);
        return doGet(apiSuffix);
    }
}
