package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ConsoleMessage;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.compute.MessageService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;

public class MessageServiceImpl extends BaseServiceImpl<ConsoleMessage> implements MessageService {
    private static Logger logger = LoggerFactory.getLogger(MessageServiceImpl.class);

    @Override
    public List<ConsoleMessage> list() {
        return doList("/ibm-console-messages");
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(ComputeService.class);
    }

    @Override
    protected List<ConsoleMessage> processListResult(String message) {
        logger.info("MessageService processListResult info:{}", message);
        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray messages = json.getJSONArray("messages");
            return JSON.parseArray(messages.toJSONString(), ConsoleMessage.class);
        } catch (Exception e) {
            String msg = "解析数据[" + message + "]出错:" + e.getMessage();
            logger.error(msg);
            throw new DriverException(msg, e);
        }
    }

    @Override
    protected ConsoleMessage processGetResult(String message) {
        return null;
    }

    @Override
    public BocloudResult delete(Integer messageId) {
        if (StringUtils.isEmpty(messageId)) {
            return new BocloudResult(false, "删除消息时，消息id不能为空");
        }

        String apiSuffix = String.format("/ibm-console-messages/%s", messageId);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult deleteAll() {
        String apiSuffix = String.format("/ibm-console-messages//all");
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }
}
