package com.bocloud.cmp.provider.powervc4j.impl.compute.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.OsHost;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.compute.OsHostService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;

public class OsHostServiceImpl extends BaseServiceImpl<OsHost> implements OsHostService {
    private static Logger logger = LoggerFactory.getLogger(OsHostServiceImpl.class);

    @Override
    public BocloudResult create(OsHost host) {
        String apiSuffix = "/os-hosts";
        if (host == null || !StringUtils.hasText(host.getName())) {
            return new BocloudResult(false, "创建主机信息，请求参数不正确");
        }

        JSONObject params = new JSONObject();
        params.put("host", MapTools.simpleMap("registration", host));
        Result result = doCreate(apiSuffix, params, 5 * 60 * 1000);
        logger.info("create hmc os host result:{}", result);
        if (result.isFailed()) {
            return new BocloudResult(false, "创建主机失败:" + result.getMessage());
        } else {
            return new BocloudResult(result.isSuccess(), result.getMessage(), "创建主机成功");
        }
    }


    @Override
    public BocloudResult modify(OsHost host) {
        if (StringUtils.isEmpty(host.getName())) {
            throw new DriverException("修改hmc上的小机信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/os-hosts/%s/update-registration", host.getName());
        JSONObject registration = new JSONObject();
        registration.put("hmc_uuids", host.getHmcUuids());
        JSONObject params = new JSONObject();
        params.put("registration", registration);
        Result result = doUpdate(apiSuffix, params);
        return new BocloudResult(result.isSuccess(), result.getMessage(), result.getErrorMsg());
    }

    @Override
    public OsHost get(String hostName) {
        if (StringUtils.isEmpty(hostName)) {
            throw new DriverException("查询hmc上的小机信息时，唯一标识不能为空");
        }
        String apiSuffix = String.format("/os-hosts/%s", hostName);
        return doGet(apiSuffix);
    }

    @Override
    public BocloudResult delete(String hostName) {
        if (StringUtils.isEmpty(hostName)) {
            return new BocloudResult(false, "移除hmc上的小机时，hmc唯一标识不能为空");
        }

        String apiSuffix = String.format("/os-hosts/%s", hostName);
        Result result = doDelete(apiSuffix, null);

        return new BocloudResult(result.isSuccess(), result.isSuccess() ? "删除主机成功" : "删除主机失败：" + result.getMessage());
    }

    @Override
    public BocloudResult onboard(String hostName, List<String> serverUuids) {
        if (StringUtils.isEmpty(hostName)) {
            return new BocloudResult(false, "管理小机上现有云主机时，小机唯一标识不能为空");
        }
        JSONObject params = new JSONObject();
        params.put("synchronous", false);
        if (serverUuids == null || serverUuids.isEmpty()) {
            serverUuids = Lists.newArrayList();
            serverUuids.add("*ALL");
        }
        params.put("servers", serverUuids);
        String preString = "{\"servers\":[],\"network_vswitch\":\"\",\"move_network_components\":false}";
        params.put("preparation", JSONObject.parseObject(preString, JSONObject.class));
        String apiSuffix = String.format("/os-hosts/%s/onboard", hostName);
        Result result = doCreate(apiSuffix, params);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult unmanage(String hostName, List<String> serverUuids) {
        if (StringUtils.isEmpty(hostName)) {
            return new BocloudResult(false, "管理小机上现有云主机时，小机唯一标识不能为空");
        }
        if (serverUuids == null || serverUuids.isEmpty()) {
            return new BocloudResult(false, "要取消管理的云主机UUID必须提供");
        }
        JSONObject params = new JSONObject();
        params.put("servers", serverUuids);
        String apiSuffix = String.format("/os-hosts/%s/unmanage", hostName);
        Result result = doCreate(apiSuffix, params);

        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult allServers(String hostName) {
        if (StringUtils.isEmpty(hostName)) {
            return new BocloudResult(false, "查询小机上所有云主机时，小机唯一标识不能为空");
        }

        String apiSuffix = String.format("/os-hosts/%s/all-servers", hostName);
        String message = customGet(String.class, apiSuffix);

        if (null == message) {
            return new BocloudResult(false, "获取服务器上云主机失败");
        } else {
            return new BocloudResult(true, message, "获取服务器上云主机成功");
        }
    }

    @Override
    public List<OsHost> list() {
        return doList("/os-hosts");
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(ComputeService.class);
    }


    @Override
    protected List<OsHost> processListResult(String message) {
        JSONObject json = JSONTools.isJSONObj(message);
        JSONArray osHosts = json.getJSONArray("hosts");
        return JSON.parseArray(osHosts.toJSONString(), OsHost.class);
    }

    @Override
    protected OsHost processGetResult(String message) {
        JSONArray osHosts = JSON.parseObject(message).getJSONArray("host");
        OsHost osHost = new OsHost();
        OsHost registerHost = new OsHost();
        for (Object obj : osHosts) {
            JSONObject jsonObject = (JSONObject) obj;
            if (jsonObject.containsKey("resource")) {
                JSONObject resourceObj = jsonObject.getJSONObject("resource");
                if ("(total)".equalsIgnoreCase(resourceObj.getString("project"))) {
                    osHost.setCpuNumTotal(resourceObj.getInteger("cpu"));
                    osHost.setDiskTotal((double) resourceObj.getInteger("disk_gb"));
                    osHost.setMemTotal((long) resourceObj.getInteger("memory_mb"));
                } else if ("(used_now)".equalsIgnoreCase(resourceObj.getString("project"))) {
                    osHost.setCpuNumUsed(resourceObj.getInteger("cpu"));
                    osHost.setDiskUsed((double) resourceObj.getInteger("disk_gb"));
                    osHost.setMemUsed((long) resourceObj.getInteger("memory_mb"));
                } else if ("(used_max)".equalsIgnoreCase(resourceObj.getString("project"))) {
                    osHost.setCpuNumMax(resourceObj.getInteger("cpu"));
                    osHost.setDiskMax((double) resourceObj.getInteger("disk_gb"));
                    osHost.setMemMax((long) resourceObj.getInteger("memory_mb"));
                }
            } else if (jsonObject.containsKey("registration")) {
                registerHost = JSON.parseObject(jsonObject.getJSONObject("registration").toJSONString(), OsHost.class);
            }
        }
        osHost.setNickName(registerHost.getNickName());
        osHost.setName(registerHost.getName());
        osHost.setStatus(registerHost.getStatus());
        osHost.setViosClusters(registerHost.getViosClusters());
        osHost.setType(registerHost.getType());
        osHost.setHmcUuids(registerHost.getHmcUuids());
        osHost.setGmtCreate(registerHost.getGmtCreate());
        osHost.setManagerType(registerHost.getManagerType());
        return osHost;
    }
}
