package com.bocloud.cmp.provider.powervc4j.impl.networking.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.cmp.provider.powervc4j.utils.StatusConverter;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * PowerVC网络服务实现类
 *
 * @author sxp
 * @since Mar 21, 2018
 */
public class NetworkServiceImpl extends BaseServiceImpl<NetworkModel> implements NetworkService {
    private static Logger logger = LoggerFactory.getLogger(NetworkServiceImpl.class);

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(NetworkService.class);
    }

    // ----list

    @Override
    public List<NetworkModel> list() {
        String apiSuffix = "/networks";
        return doList(apiSuffix);
    }

    @Override
    protected List<NetworkModel> processListResult(String message) {
        JSONArray networks = JSON.parseObject(message).getJSONArray("networks");
        List<NetworkModel> networkModels = new ArrayList<>(networks.size());
        for (int i = 0; i < networks.size(); i++) {
            networkModels.add(convert(networks.getJSONObject(i)));
        }
        return networkModels;
    }

    private NetworkModel convert(JSONObject network) {
        NetworkModel model = new NetworkModel();
        model.setAdminStateup(network.getBooleanValue("admin_state_up"));
        model.setName(network.getString("name"));
        model.setNetworkUuid(network.getString("id"));
        model.setShared(network.getBooleanValue("shared"));
        model.setType(network.getString("provider:network_type"));
        model.setIsRouterExternal(network.getBooleanValue("router:external"));
        model.setPhyNetwork(network.getString("provider:physical_network"));
        model.setSegmentationId(network.getString("provider:segmentation_id"));
        model.setSubnetIds(network.getJSONArray("subnets").toJavaList(String.class));
        model.setMtu(network.getInteger("mtu"));
        String status = network.getString("status");
        if (StringUtils.isNotEmpty(status)) {
            model.setStatus(StatusConverter.Network.converter(status));
        }
        return model;
    }


    @Override
    public NetworkModel get(String networkId) {
        String apiSuffix = String.format("/networks/%s", networkId);
        return doGet(apiSuffix);
    }

    @Override
    protected NetworkModel processGetResult(String message) {
        JSONObject networkJson = JSON.parseObject(message).getJSONObject("network");
        return convert(networkJson);
    }

    // ----create

    @Override
    public BocloudResult create(NetworkModel model) {
        String apiSuffix = "/networks";
        Result result = doCreate(apiSuffix, getCreateNetworkParams(model));
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        JSONObject networkJson = JSON.parseObject(result.getMessage()).getJSONObject("network");
        NetworkModel networkModel = convert(networkJson);
        return new BocloudResult(true, networkModel, result.getMessage());
    }

    private Map<String, Object> getCreateNetworkParams(NetworkModel model) {
        JSONObject network = new JSONObject();
        network.put("name", model.getName());
        network.put("mtu", model.getMtu());
        network.put("provider:segmentation_id", Integer.parseInt(model.getSegmentationId()));
        network.put("provider:network_type", model.getType());
        network.put("provider:physical_network", model.getPhyNetwork());
        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("network", network);
        return params;
    }

    private Map<String, Object> getUpdateNetworkParams(NetworkModel model) {
        JSONObject network = new JSONObject();
        network.put("name", model.getName());
        network.put("mtu", model.getMtu());

        Map<String, Object> params = new HashMap<>(MapUtils.capacity(1));
        params.put("network", network);
        return params;
    }

    @Override
    public BocloudResult delete(String networkId) {
        String apiSuffix = String.format("/networks/%s", networkId);
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public BocloudResult update(NetworkModel model) {
        String apiSuffix = String.format("/networks/%s", model.getNetworkUuid());
        Result result = doUpdate(apiSuffix, getUpdateNetworkParams(model));
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        JSONObject networkJson = JSON.parseObject(result.getMessage()).getJSONObject("network");
        NetworkModel networkModel = convert(networkJson);
        return new BocloudResult(true, networkModel, result.getMessage());
    }
}
