package com.bocloud.cmp.provider.powervc4j.impl.notification.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.powervc.PowervcWebEventModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.notification.NotificationService;
import com.bocloud.cmp.provider.powervc4j.api.notification.WebEventService;
import com.bocloud.cmp.provider.powervc4j.common.enums.RequestMethod;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Powervc事件Service
 */
public class WebEventServiceImpl extends BaseServiceImpl<PowervcWebEventModel> implements WebEventService {
    private static Logger logger = LoggerFactory.getLogger(WebEventServiceImpl.class);

    private Date lastListDate = new Date();

    @Override
    public synchronized List<PowervcWebEventModel> list() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");
        simpleDateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
        Map<String, Object> header = new HashMap<>();
        Result result = work(RequestMethod.GET, "/web/events?" + simpleDateFormat.format(lastListDate), null, header, 60 * 60 * 1000);
        if (result.isFailed()) {
            throw new DriverException(result.getMessage());
        }
        lastListDate = new Date();
        return processListResult(result.getMessage());
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(NotificationService.class);
    }

    private PowervcWebEventModel convert(JSONObject eventObject) {
        PowervcWebEventModel result = new PowervcWebEventModel();
        result.setEventType(eventObject.getString("event_type"));
        result.setRole(eventObject.getString("role"));
        result.setPayload(eventObject.getString("payload"));
        result.setMessage(eventObject.getString("saved_message"));
        return result;
    }

    @Override
    protected List<PowervcWebEventModel> processListResult(String message) {
        List<PowervcWebEventModel> result = new ArrayList<>();

        JSONObject messageObject = JSON.parseObject(message);
        if (messageObject.containsKey("events")) {
            JSONArray eventObjects = messageObject.getJSONArray("events");
            for (int i = 0; i < eventObjects.size(); i++) {
                JSONObject eventObject = eventObjects.getJSONObject(i);
                result.add(convert(eventObject));
            }
        } else {
            result.add(convert(messageObject));
        }
        return result;
    }

    @Override
    protected PowervcWebEventModel processGetResult(String message) {
        return null;
    }
}
