package com.bocloud.cmp.provider.powervc4j.impl.storage.v14;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.StorageModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageDeviceService;
import com.bocloud.cmp.provider.powervc4j.exception.DriverException;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.MapTools;
import org.apache.commons.lang3.BooleanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 存储设备
 *
 * @author luyanchao
 * @since 2019/1/25 13:58
 */
public class BlockStorageDeviceServiceImpl extends BaseServiceImpl<StorageModel> implements BlockStorageDeviceService {

    private static Logger logger = LoggerFactory.getLogger(BlockVolumeTemplateServiceImpl.class);

    @Override
    public BocloudResult get(String uuid) {
        String apiSuffix = String.format("/storage-providers/%s", uuid);

        StorageModel model = doGet(apiSuffix);
        return new BocloudResult(model != null, model, "");
    }

    @Override
    public BocloudResult connect(StorageModel storage) {
        String apiSuffix = "/os-hosts/connect";
        JSONObject param = new JSONObject();
        param.put("access_ip", storage.getIp());
        param.put("host_type", storage.getType());
        param.put("user_id", storage.getUsername());
        param.put("password", storage.getPassword());

        JSONObject params = new JSONObject();
        params.put("host", MapTools.simpleMap("registration", param));

        Result result = doCreate(apiSuffix, params);
        if (result.isSuccess()) {
            return new BocloudResult(true, result.getMessage(), "连接存储器成功");
        } else {
            return new BocloudResult(false, "连接存储器失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult create(StorageModel storage) {
        String apiSuffix = "/os-hosts";
        JSONObject param = new JSONObject();
        param.put("access_ip", storage.getIp());
        param.put("host_type", storage.getType());
        param.put("user_id", storage.getUsername());
        param.put("password", storage.getPassword());
        param.put("host_display_name", storage.getName());
        param.put("volume_pool_name", storage.getVolumePool());

        JSONObject params = new JSONObject();
        params.put("host", MapTools.simpleMap("registration", param));

        Result result = doCreate(apiSuffix, params);
        if (result.isSuccess()) {
            return new BocloudResult(true, result.getMessage(), "连接存储器成功");
        } else {
            return new BocloudResult(false, "连接存储器失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult modify(StorageModel storage) {
        String apiSuffix = String.format("/os-hosts/%s", storage.getHost());
        JSONObject param = new JSONObject();
        param.put("access_ip", storage.getIp());
        param.put("host_display_name", storage.getName());
        param.put("user_id", storage.getUsername());
        param.put("password", storage.getPassword());
        JSONObject params = new JSONObject();
        params.put("registration", param);
        Result result = doUpdate(apiSuffix, params);
        if (result.isSuccess()) {
            return new BocloudResult(true, result.getMessage(), "修改存储器成功");
        } else {
            return new BocloudResult(false, "修改存储器失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String hostName) {
        String apiSuffix = String.format("/os-hosts/%s", hostName);
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public List<StorageModel> list() {
        String apiSuffix = "/storage-providers/detail";
        return doList(apiSuffix);
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(BlockStorageDeviceService.class);
    }

    @Override
    protected List<StorageModel> processListResult(String message) {
        logger.debug(message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONArray templates = json.getJSONArray("storage_providers");

            List<StorageModel> models = new ArrayList<>(templates.size());

            JSONObject storage;
            StorageModel model;

            for (Object obj : templates) {
                storage = (JSONObject) obj;
                model = convert(storage);

                models.add(model);
            }

            return models;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }

    private StorageModel convert(JSONObject storage) {
        StorageModel model = new StorageModel();
        model.setUuid(storage.getString("id"));
        model.setName(storage.getJSONObject("service").getString("host_display_name"));
        model.setIp(storage.getJSONObject("service").getString("access_ip"));
        model.setHost(storage.getJSONObject("service").getString("host"));
        model.setStatus(storage.getString("backend_state"));
        model.setType(storage.getString("backend_type"));
        model.setTotalCapacity(storage.getDouble("total_capacity_gb"));
        model.setFreeCapaticy(storage.getDouble("free_capacity_gb"));
        model.setCompressionSuppoerd(BooleanUtils.toBoolean(storage.getString("compression_supported")));
        model.setDefaultTemplateUuid(storage.getJSONObject("service").getString("default_volume_type"));
        return model;
    }

    @Override
    protected StorageModel processGetResult(String message) {
        logger.debug("powervc volume template info {}", message);

        try {
            JSONObject json = JSONTools.isJSONObj(message);
            JSONObject storage = json.getJSONObject("storage_provider");

            StorageModel model = convert(storage);

            return model;
        } catch (Exception e) {
            String msg = "解析数据出错。" + e.getMessage();
            logger.error(msg);

            throw new DriverException(msg, e);
        }
    }
}
