package com.bocloud.cmp.provider.powervc4j.impl.storage.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumePoolModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumePoolService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockVolumeTemplateService;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.cmp.provider.powervc4j.utils.CapacityUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 存储池
 * @author luyanchao
 * @since 2019/1/24 15:47
 */
public class BlockVolumePoolServiceImpl extends BaseServiceImpl<VolumePoolModel> implements BlockVolumePoolService {

    private static Logger logger = LoggerFactory.getLogger(BlockVolumePoolServiceImpl.class);

    @Override
    public List<VolumePoolModel> list(String storageHostName) {
        String apiSuffix = String.format("/os-hosts/%s/volume-pools", storageHostName);
        return doList(apiSuffix);
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(BlockVolumeTemplateService.class);
    }

    private VolumePoolModel convert(JSONObject pool) {
        VolumePoolModel model = new VolumePoolModel();
        model.setName(pool.getString("name"));
        model.setStatus(pool.getString("status"));
        // 当使用EMC存储时，存储池返回数据没有id字段，所以增加如下判断
        if (pool.containsKey("id")) {
            model.setUuid(pool.getString("id"));
        } else {
            model.setUuid(DigestUtils.md5Hex(pool.getString("name").getBytes()));
        }
        model.setWarning(pool.getDouble("warning"));
        model.setOverallocation(pool.getDouble("overallocation"));
        model.setExtentSize(pool.getDouble("extent_size"));
        model.setCapacity(CapacityUtils.transportCapacity(pool.getString("capacity")));
        model.setRealCapacity(CapacityUtils.transportCapacity(pool.getString("real_capacity")));
        model.setUsedCapacity(CapacityUtils.transportCapacity(pool.getString("used_capacity")));
        model.setVirtualCapacity(CapacityUtils.transportCapacity(pool.getString("virtual_capacity")));
        model.setFreeCapacity(CapacityUtils.transportCapacity(pool.getString("free_capacity")));
        model.setCompressionActive(BooleanUtils.toBoolean(pool.getString("compression_active")));
        model.setCompressionVirtualCapacity(CapacityUtils.transportCapacity(pool.getString("compression_compressed_capacity")));
        model.setCompressionCompressedCapacity(CapacityUtils.transportCapacity(pool.getString("compression_uncompressed_capacity")));
        model.setCompressionUncompressedCapacity(CapacityUtils.transportCapacity(pool.getString("compression_virtual_capacity")));
        model.setEasytier(pool.getString("easy_tier"));
        model.setEasytierStatus(pool.getString("easy_tier_status"));
        model.setMdiskCount(pool.getInteger("mdisk_count"));
        model.setVdiskCount(pool.getInteger("vdisk_count"));
        return model;
    }

    @Override
    protected List<VolumePoolModel> processListResult(String message) {
        JSONObject jsonObject = JSON.parseObject(message);
        if (jsonObject.containsKey("volume_pools")) {
            JSONArray pools = jsonObject.getJSONArray("volume_pools");
            List<VolumePoolModel> models = new ArrayList<>(pools.size());
            for (int i = 0; i < pools.size(); i++) {
                JSONObject template = pools.getJSONObject(i);
                models.add(convert(template));
            }
            return models;
        } else {
            return null;
        }
    }

    @Override
    protected VolumePoolModel processGetResult(String message) {
        JSONObject jsonObject = JSON.parseObject(message);
        if (jsonObject.containsKey("volume_pool")) {
            return convert(jsonObject.getJSONObject("volume_pool"));
        } else {
            return null;
        }
    }

    @Override
    public List<VolumePoolModel> list() {
        return null;
    }
}
