package com.bocloud.cmp.provider.powervc4j.impl.storage.v14;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SanFabricModel;
import com.bocloud.cmp.provider.powervc4j.api.Apis;
import com.bocloud.cmp.provider.powervc4j.api.storage.FabircService;
import com.bocloud.cmp.provider.powervc4j.impl.base.BaseServiceImpl;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.MapTools;

import java.util.List;

/**
 * @author luyanchao
 * @since 2019/2/22 14:06
 */
public class FabircServiceImpl extends BaseServiceImpl<SanFabricModel> implements FabircService {

    @Override
    public BocloudResult create(SanFabricModel sanFc) {
        String apiSuffix = "/san-fabrics";
        JSONObject params = new JSONObject();
        params.put("fabric", MapTools.simpleMap("registration", sanFc));
        Result result = doCreate(apiSuffix, params);
        if (result.isSuccess()) {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            SanFabricModel sanFabrics = jsonObject.getJSONObject("fabric").getObject("registration", SanFabricModel.class);
            return new BocloudResult(true, sanFabrics, "连接光纤存储器成功");
        } else {
            return new BocloudResult(false, "连接光纤存储器失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult modify(SanFabricModel sanFc) {
        String apiSuffix = String.format("/san-fabrics/%s", sanFc.getName());
        JSONObject params = new JSONObject();
        params.put("registration", sanFc);
        Result result = doUpdate(apiSuffix, params);
        if (result.isSuccess()) {
            JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
            SanFabricModel sanFabrics = jsonObject.getObject("registration", SanFabricModel.class);
            sanFabrics.setName(jsonObject.getString("fabric"));
            return new BocloudResult(true, sanFabrics, "修改光纤存储器成功");
        } else {
            return new BocloudResult(false, "修改光纤存储器失败:" + result.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String fcName) {
        String apiSuffix = String.format("/san-fabrics/%s", fcName);
        SanFabricModel sanFabrics = doGet(apiSuffix);
        return new BocloudResult(true, sanFabrics, "获取光纤交换机详情成功");
    }

    @Override
    public BocloudResult remove(String fcName) {
        String apiSuffix = String.format("/san-fabrics/%s", fcName);
        Result result = doDelete(apiSuffix, null);
        return new BocloudResult(result.isSuccess(), result.getMessage());
    }

    @Override
    public List<SanFabricModel> list() {
        String apiSuffix = "/san-fabrics/detail";
        return doList(apiSuffix, null);
    }

    @Override
    protected Class<?> initServiceType() {
        return Apis.getTop(FabircService.class);
    }

    @Override
    protected List<SanFabricModel> processListResult(String message) {
        return JSONObject.parseArray(JSON.parseObject(message).getString("fabrics"), SanFabricModel.class);
    }

    @Override
    protected SanFabricModel processGetResult(String message) {
        return JSONObject.parseObject(JSON.parseObject(message).getJSONObject("fabric").getString("registration"), SanFabricModel.class);
    }
}
