package com.bocloud.cmp.provider.powervc4j.model.identity;

import com.alibaba.fastjson.JSON;

import java.util.Date;
import java.util.Map;

/**
 * PowerVC授权令牌
 * 
 * @author sxp
 * @since Mar 19, 2018
 */
public class Token {
    /**
     * 令牌的唯一标识
     * <p>
     * X-Subject-Token: b4a68ae988774c089d59db802c007777
     * </p>
     */
    private String tokenId;

    /**
     * 令牌过期时间
     * <p>
     * "expires_at": "2018-03-19T07:37:41.037174Z",
     * </p>
     */
    private Date expires;

    /**
     * 存储访问PowerVC各种服务的url前缀
     * 
     * <pre>
     * 数据格式示例：
     * api_prefix = { 
     *     "metering": "https://192.168.2.61/powervc/openstack/metering", 
     *     "identity": "https://192.168.2.61/powervc/openstack/identity/v3", 
     *     "network": "https://192.168.2.61/powervc/openstack/network", 
     *     "volume": "https://192.168.2.61/powervc/openstack/volume/v2/e81cd703865d4dbc9de354dd36faf5a9", 
     *     "image": "https://192.168.2.61/powervc/openstack/image", 
     *     "compute": "https://192.168.2.61/powervc/openstack/compute/v2/e81cd703865d4dbc9de354dd36faf5a9", 
     *     "ttv": "https://192.168.2.61/powervc/openstack/validation" 
     * }
     * </pre>
     */
    private Map<String, String> apiPrefix;

    /**
     * 租户唯一标识
     * 
     * <pre>
     * project下的id就是tenantId
     * "project": {
     * 	"domain": {
     * 		"id": "default",
     * 		"name": "Default"
     * 	},
     * 	"id": "e81cd703865d4dbc9de354dd36faf5a9",
     * 	"name": "ibm-default"
     * }
     * </pre>
     */
    private String tenantId;

    public Token(String id, Date expires, Map<String, String> apiPrefix, String tenantId) {
        this.tokenId = id;
        this.expires = expires;
        this.apiPrefix = apiPrefix;
        this.tenantId = tenantId;
    }

    public String getTokenId() {
        return tokenId;
    }

    public void setTokenId(String tokenId) {
        this.tokenId = tokenId;
    }

    public Date getExpires() {
        return expires;
    }

    public void setExpires(Date expires) {
        this.expires = expires;
    }

    public Map<String, String> getApiPrefix() {
        return apiPrefix;
    }

    public void setApiPrefix(Map<String, String> apiPrefix) {
        this.apiPrefix = apiPrefix;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    @Override
    public String toString() {
        return JSON.toJSONString(this);
    }

}
