package com.bocloud.cmp.provider.powervc4j.utils;

import com.bocloud.cmp.provider.powervc4j.api.compute.ComputeService;
import com.bocloud.cmp.provider.powervc4j.api.image.ImageService;
import com.bocloud.cmp.provider.powervc4j.api.networking.NetworkingService;
import com.bocloud.cmp.provider.powervc4j.api.storage.BlockStorageService;
import com.bocloud.cmp.provider.powervc4j.api.validation.ValidateService;
import com.bocloud.cmp.provider.powervc4j.common.util.MapUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * Token中Map<String, String> apiPrefix的工具类
 * 
 * @author sxp
 * @since Mar 20, 2018
 */
public class ApiPrefixUtils {
    private static Map<Class<?>, String> apiKeys = new HashMap<>(MapUtils.capacity(10));

    static {
        apiKeys.put(ComputeService.class, "compute");
        apiKeys.put(NetworkingService.class, "network");
        apiKeys.put(BlockStorageService.class, "volume");
        apiKeys.put(ImageService.class, "image");
        apiKeys.put(ValidateService.class, "ttv");
    }

    /**
     * apiPrefix数据格式示例
     * 
     * <pre>
     * api_prefix = { 
     *     "metering": "https://192.168.2.61/powervc/openstack/metering", 
     *     "identity": "https://192.168.2.61/powervc/openstack/identity/v3", 
     *     "network": "https://192.168.2.61/powervc/openstack/network", 
     *     "volume": "https://192.168.2.61/powervc/openstack/volume/v2/e81cd703865d4dbc9de354dd36faf5a9", 
     *     "image": "https://192.168.2.61/powervc/openstack/image", 
     *     "compute": "https://192.168.2.61/powervc/openstack/compute/v2/e81cd703865d4dbc9de354dd36faf5a9", 
     *     "ttv": "https://192.168.2.61/powervc/openstack/validation" 
     * }
     * </pre>
     */

    /**
     * 根据服务接口的类型获得相应的API前缀
     * 
     * @author sxp
     * @since Mar 20, 2018
     * @since 2018-03-21 modify by sxp 增加版本号的特殊处理
     */
    public static String getPrefix(Map<String, String> apiPrefix, Class<?> serviceType) {
        if (apiPrefix == null || apiPrefix.isEmpty()) {
            return null;
        }

        String key = apiKeys.get(serviceType);
        if (StringUtils.isEmpty(key)) {
            return null;
        }

        String prefix = apiPrefix.get(key);

        // TODO: 2019/2/22 升级powervc版本后，这里的版本号需要适配 
        if ("network".equals(key)) {
            prefix += "/v2.0";
        } else if ("image".equals(key)) {
            prefix += "/v2";
        } else if ("ttv".equals(key)) {
            prefix += "/v1";
        }

        return prefix;
    }

}
