package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.StorageModel;
import com.bocloud.cmp.model.VolumePoolModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @author luyanchao
 * @since 2019/1/25 15:40
 */
@Provider("bocloud.cmp.powervc")
public class PowervcStorageProvider extends PowervcProvider implements StorageProvider {
    private static Logger logger = LoggerFactory.getLogger(PowervcVolumeTemplateProvider.class);

    public PowervcStorageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<StorageModel> storages = listStorageDevice();
            return new BocloudResult(storages != null, storages, "");
        } catch (Exception e) {
            String msg = "查询存储卷列表信息失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult connect(StorageModel model) {
        try {
            BocloudResult result = this.getClient().blockStorage(getVersion()).storageDevices(getVersion()).connect(model);
            if (result.isFailed()) {
                logger.error("连接存储器失败:{}", result);
                return new BocloudResult(false, "连接存储器失败：" + result.getMessage());
            } else {
                // 连接成功，返回volumePools信息
                JSONArray jsonArray = JSONObject.parseObject((String) result.getData(), JSONObject.class).
                        getJSONObject("host").getJSONArray("volumePools");
                List<VolumePoolModel> volumePoolModels = new ArrayList<>(jsonArray.size());
                for (int i = 0; i < jsonArray.size(); i++) {
                    JSONObject jsonObject = jsonArray.getJSONObject(i);
                    VolumePoolModel volumePoolModel = new VolumePoolModel();
                    volumePoolModel.setName(jsonObject.getString("pool_name"));
                    volumePoolModel.setCapacity(jsonObject.getDouble("pool_total_capacity_gb"));
                    volumePoolModel.setFreeCapacity(jsonObject.getDouble("pool_free_capacity_gb"));
                    volumePoolModels.add(volumePoolModel);
                }
                return new BocloudResult(true, volumePoolModels, "连接存储器成功");
            }
        } catch (Exception e) {
            String msg = "连接存储器失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult create(StorageModel model) {
        try {
            BocloudResult result = this.getClient().blockStorage(getVersion()).storageDevices(getVersion()).create(model);
            if (result.isFailed()) {
                logger.error("添加存储器失败:{}", result);
                return new BocloudResult(false, "添加存储器失败：" + result.getMessage());
            } else {
                // 连接成功，返回storage信息
                JSONObject jsonObject = JSONObject.parseObject((String) result.getData());
                JSONObject storage = JSONObject.parseObject(jsonObject.getJSONObject("host").getString("registration"), JSONObject.class);
                model.setHost(storage.getString("host_name"));
                return new BocloudResult(true, model, "连接存储器成功");
            }
        } catch (Exception e) {
            String msg = "添加存储器失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult modify(StorageModel model) {
        try {
            BocloudResult result = this.getClient().blockStorage(getVersion()).storageDevices(getVersion()).modify(model);
            if (result.isFailed()) {
                logger.error("修改存储器失败:{}", result);
                return new BocloudResult(false, "修改存储器失败：" + result.getMessage());
            } else {
                // 连接成功，返回storage信息
                JSONObject jsonObject = JSONObject.parseObject((String) result.getData());
                model.setHost(jsonObject.getString("host"));
                JSONObject registration = jsonObject.getJSONObject("registration");
                model.setIp(registration.getString("access_ip"));
                model.setName(registration.getString("host_display_name"));
                model.setUsername(registration.getString("user_id"));
                return new BocloudResult(true, model, "修改存储器成功");
            }
        } catch (Exception e) {
            String msg = "修改存储器失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult remove(String hostName) {
        try {
            return this.getClient().blockStorage(getVersion()).storageDevices(getVersion()).remove(hostName);
        } catch (Exception e) {
            String msg = "移除存储器失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            BocloudResult bocloudResult = this.getClient().blockStorage(getVersion()).storageDevices(getVersion()).get(uuid);
            if (bocloudResult.isFailed()) {
                logger.error("查询存储器失败：" + bocloudResult.getMessage());
                return new BocloudResult(false, "查询存储器失败：" + bocloudResult.getMessage());
            }
            StorageModel storageModel = (StorageModel) bocloudResult.getData();
            if (storageModel != null) {
                List<VolumePoolModel> poolModels = this.getClient().blockStorage(getVersion()).pools(getVersion()).list(storageModel.getHost());
                storageModel.setPools(poolModels);
            }
            return new BocloudResult(true, storageModel, "查询存储器成功");
        } catch (Exception e) {
            String msg = "查询存储器失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }
}
