package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeTemplateModel;
import com.bocloud.cmp.provider.PowervcProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author luyanchao
 * @since 2019/1/24 18:10
 */
@Provider("bocloud.cmp.powervc")
public class PowervcVolumeTemplateProvider extends PowervcProvider implements VolumeTemplateProvider{
    private static Logger logger = LoggerFactory.getLogger(PowervcVolumeTemplateProvider.class);

    public PowervcVolumeTemplateProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<VolumeTemplateModel> templates = listVolumeTemplate();
            return new BocloudResult(templates != null, templates, "");
        } catch (Exception e) {
            String msg = "查询存储卷列表信息失败，" + e.getMessage();
            logger.error(msg, e);
            return new BocloudResult(false, msg);
        }
    }

    @Override
    public BocloudResult create(VolumeTemplateModel template) {
        return this.getClient().blockStorage(getVersion()).templates(getVersion()).create(template);
    }

    @Override
    public BocloudResult modify(VolumeTemplateModel template) {
        BocloudResult modifyResult = this.getClient().blockStorage(getVersion()).templates(getVersion()).modify(template);
        if (modifyResult.isFailed()) {
            return modifyResult;
        }
        return this.getClient().blockStorage(getVersion()).templates(getVersion()).get(template.getUuid());
    }

    @Override
    public BocloudResult remove(String uuid) {
        return this.getClient().blockStorage(getVersion()).templates(getVersion()).remove(uuid);
    }

    @Override
    public BocloudResult detail(String uuid) {
        return this.getClient().blockStorage(getVersion()).templates(getVersion()).get(uuid);
    }
}
