package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.ClusterModel;
import com.bocloud.cmp.provider.compute.PowervcClusterProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.Collections;

/**
 * @author luyanchao
 * @since 2019/2/11 15:01
 */
public class PowervcOsAggregateDriverTest extends BaseDriverTest {
    private static PowervcClusterProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcClusterProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }


    @Test
    public void testCreate() {
        ClusterModel model = new ClusterModel();
        model.setName("test");
        model.setInitialPolicy("填满");
        model.setHostNames(Collections.singletonList("8203E4A_10F8563"));

        BocloudResult result = instance.create(model);
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testModify() {
        ClusterModel model = new ClusterModel();
        model.setUuid("28");
        model.setName("test");
        model.setInitialPolicy("内存分配平衡");
        model.setHostNames(Collections.singletonList("8203E4A_10F8563"));

        BocloudResult result = instance.modify(model);
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testRemove() {
        BocloudResult result = instance.remove("28");
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testAddServer() {
        BocloudResult result = instance.addHost("28", "8203E4A_10F8563");
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testRemoveServer() {
        BocloudResult result = instance.removeHost("28", "8203E4A_10F8563");
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testGet() {
        BocloudResult result = instance.detail("28");
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testGetByName() {
        BocloudResult result = instance.queryByName("test");
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

}
