package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.HostModel;
import com.bocloud.cmp.model.OsHost;
import com.bocloud.cmp.model.OsHosts;
import com.bocloud.cmp.provider.compute.PowervcHmcProvider;
import com.bocloud.cmp.provider.compute.PowervcOsHostProvider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.List;

public class PowervcOsHostDriverTest extends BaseDriverTest {
    private static PowervcOsHostProvider instance;
    private static PowervcHmcProvider hmcInstance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcOsHostProvider(bulter);
        hmcInstance = new PowervcHmcProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.get();
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            List<HostModel> models = (List<HostModel>) result.getData();
            System.out.println(models.get(0).getName());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        // 链接时间比较长，5分钟以内
        OsHost osHost = new OsHost();
        String hmcUuid = "88713e01-5962-35dc-a69a-589f503d04d1";
        BocloudResult result1 = hmcInstance.getHosts(hmcUuid);
        Assert.assertTrue(result1.isSuccess());
        OsHosts hosts = (OsHosts) result1.getData();
        osHost.setName(hosts.getHosts().get(0).getName());
        List<String> hmcUuids = Lists.newArrayList();
        hmcUuids.add(hmcUuid);
        osHost.setHmcUuids(hmcUuids);
        osHost.setHostType("powervm");
        osHost.setNickName(hosts.getHosts().get(0).getNickName());
        BocloudResult result = instance.create(osHost);
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDelete() {
        BocloudResult result = instance.delete("8203E4A_10F8563");
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetail() {
//        BocloudResult result = instance.get("powervc");
        BocloudResult result = instance.get("8203E4A_10F8563");
        if (result.isSuccess()) {
            OsHost model = (OsHost) result.getData();
            System.out.println("model name= " + model.getName());
            System.out.println("model disk total= " + model.getDiskTotal());
            System.out.println(JSON.toJSONString(model));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testOnboard() {
//        BocloudResult result = instance.onboard("8203E4A_10F8563", null);
        List<String> serverUuids = Lists.newArrayList();
        serverUuids.add("4d2deabd-5ec2-4077-8edb-f0668f86f975");
        BocloudResult result = instance.onboard("8203E4A_10F8563", serverUuids);
        System.out.println("result.getMessage() = " + result.getMessage());
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testUnmanage() {
        List<String> serverUuids = Lists.newArrayList();
        serverUuids.add("4d2deabd-5ec2-4077-8edb-f0668f86f975");
        BocloudResult result = instance.unmanage("8203E4A_10F8563", serverUuids);
        System.out.println("result.getMessage() = " + result.getMessage());
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testAllServers() {
        BocloudResult result = instance.allServers("8203E4A_10F8563");
        System.out.println("result message = " + result.getMessage());
        Assert.assertTrue(result.isSuccess());
    }
}
