package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.common.PowervcFlavorProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.math.BigDecimal;

public class PvFlavorDriverTest extends BaseDriverTest {
    private static PowervcFlavorProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcFlavorProvider(bulter);
    }
    @Test
    public void testSync() {
        BocloudResult result = instance.sync();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        FlavorModel flavor = new FlavorModel();
        flavor.setName("sxp.test.tiny");
        flavor.setMemory(new BigDecimal(256));
        flavor.setCpu(1);
        flavor.setDisk(1);
        flavor.setFlavorUuid("03d8d484-91eb-4428-94de-d1295fb70b8a");

        BocloudResult result = instance.create(flavor);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());

        try {
            instance.remove(flavor.getFlavorUuid());
        } catch (Exception e) {
            // ignore
        }
    }

    @Test
    public void testRemove() {
        FlavorModel flavor = new FlavorModel();
        flavor.setName("sxp.test.tiny");
        flavor.setMemory(new BigDecimal(256));
        flavor.setCpu(1);
        flavor.setDisk(1);
        flavor.setFlavorUuid("03d8d484-91eb-4428-94de-d1295fb70b8a");

        try {
            instance.create(flavor);
        } catch (Exception e) {
            // ignore
        }

        String flavorId = flavor.getFlavorUuid();
        BocloudResult result = instance.remove(flavorId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testUpdate() {
        FlavorModel flavor = new FlavorModel();
        flavor.setName("sxp.test");
        flavor.setMemory(new BigDecimal(512));
        flavor.setCpu(1);
        flavor.setDisk(1);
        flavor.setFlavorUuid("03d8d484-91eb-4428-94de-d1295fb70b8a");

        try {
            instance.create(flavor);
        } catch (Exception e) {
            // ignore
        }

        FlavorModel newFlavor = new FlavorModel();
        newFlavor.setName("sxp.test.update");
        newFlavor.setMemory(new BigDecimal(512 + 1));
        newFlavor.setCpu(1);
        newFlavor.setDisk(32);
        newFlavor.setFlavorUuid(flavor.getFlavorUuid());

        BocloudResult result = instance.update(newFlavor);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testGet() {
        String flavorId = "03d8d484-91eb-4428-94de-d1295fb70b8a";
        BocloudResult result = instance.get(flavorId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

}
