package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.compute.PowervcImageProvider;
import com.bocloud.cmp.provider.compute.PowervcMessageProvider;
import com.bocloud.cmp.provider.storage.PowervcVolumeProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class PvImageDriverTest extends BaseDriverTest {
    private static PowervcImageProvider instance;
    private static PowervcVolumeProvider volumeProvider;
    private static PowervcMessageProvider messageProvider;


    @BeforeClass
    public static void setUp() {
        instance = new PowervcImageProvider(bulter);
        volumeProvider = new PowervcVolumeProvider(bulter);
        messageProvider = new PowervcMessageProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetail() {
        String imageId = "test-id";
        BocloudResult result = instance.detail(imageId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testRemove() {
        String imageId = "test-id";
        BocloudResult result = instance.remove(imageId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        String volumeId = "3a9b6889-6783-4f8b-80af-4259b36b9e74";
        ImageModel imageModel = new ImageModel();
        imageModel.setName("lkxtest");
        imageModel.setArchitecture("ppc64");
        imageModel.setOsName("aix");
        imageModel.setRemark("this is lkx test image");
        System.out.println("JSON.toJSONString(imageModel = " + JSON.toJSONString(imageModel));
        BocloudResult result = instance.create(imageModel);
        Assert.assertTrue(result.isSuccess());
        JSONObject object = (JSONObject) result.getData();
        System.out.println("object = " + object);

        BocloudResult file = instance.file(object.getString("id"));
        System.out.println("file = " + file);
    }
}
