package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.provider.network.PowervcNetworkProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class PvNetworkDriverTest extends BaseDriverTest {
    private static PowervcNetworkProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcNetworkProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() throws Exception {
        NetworkModel network = new NetworkModel();
        // network.setNetworkUuid("03d8d123-91eb-4658-94de-d1295fb70574");
        network.setName("sxp.test.network");
        network.setAdminStateup(true);
        network.setShared(true);
        network.setSegmentationId("1001");
        network.setPhyNetwork("default");
        network.setType("vlan");
        network.setIsRouterExternal(true);
        network.setShared(true);
        // network.setTenant(instance.getToken().getTenantId());

        BocloudResult result = instance.create(network);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(result.getMessage());
            network = (NetworkModel) result.getData();
        }

        Assert.assertTrue(result.isSuccess());
        instance.remove(network.getNetworkUuid());
    }

    @Test
    public void testRemove() throws Exception {
        NetworkModel network = new NetworkModel();
        // network.setNetworkUuid("03d8d123-91eb-4658-94de-d1295fb70574");
        network.setName("sxp.test.network");
        network.setAdminStateup(true);
        network.setShared(true);
        network.setSegmentationId("1001");
        network.setPhyNetwork("default");
        network.setType("vlan");
        network.setIsRouterExternal(true);
        network.setShared(true);
        // network.setTenant(instance.getToken().getTenantId());


        BocloudResult craeteResult = instance.create(network);
        if (craeteResult.isFailed()) {
            System.out.println(craeteResult);
        } else {
            network = (NetworkModel) craeteResult.getData();
        }

        String networkId = network.getNetworkUuid();
        // String networkId = "403b2ddb-2e8c-4c5b-97bf-c630c4b22749";

        BocloudResult removeResult = instance.remove(networkId);

        if (removeResult.isFailed()) {
            System.out.println(removeResult.getMessage());
        }
        Assert.assertTrue(removeResult.isSuccess());
    }

    @Test
    public void testModify() throws Exception {
        NetworkModel network = new NetworkModel();
        // network.setNetworkUuid("03d8d123-91eb-4658-94de-d1295fb70574");
        network.setName("sxp.test.network");
        network.setAdminStateup(true);
        network.setShared(true);
        network.setSegmentationId("1001");
        network.setPhyNetwork("default");
        network.setType("vlan");
        network.setIsRouterExternal(true);
        network.setShared(true);
        // network.setTenant(instance.getToken().getTenantId());


        BocloudResult createResult = instance.create(network);
        if (createResult.isFailed()) {
            System.out.println(createResult);
        } else {
            network = (NetworkModel) createResult.getData();
        }


        network.setName("sxp.test.network.update");
        network.setAdminStateup(true);
        network.setShared(true);

        BocloudResult result = instance.modify(network);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(result.getMessage());
            network = (NetworkModel) result.getData();
        }
        Assert.assertTrue(result.isSuccess());


        instance.remove(network.getNetworkUuid());


    }

    @Test
    public void testGet() throws Exception {
        NetworkModel network = new NetworkModel();
        // network.setNetworkUuid("03d8d123-91eb-4658-94de-d1295fb70574");
        network.setName("sxp.test.network");
        network.setAdminStateup(true);
        network.setShared(true);
        network.setSegmentationId("1001");
        network.setPhyNetwork("default");
        network.setType("vlan");
        network.setIsRouterExternal(true);
        network.setShared(true);
        // network.setTenant(instance.getToken().getTenantId());


        BocloudResult createResult = instance.create(network);
        if (createResult.isFailed()) {
            System.out.println(createResult);
        } else {
            network = (NetworkModel) createResult.getData();
        }


        String networkId = network.getNetworkUuid();
        BocloudResult result = instance.detail(networkId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
            network = (NetworkModel) result.getData();
        }
        Assert.assertTrue(result.isSuccess());


        instance.remove(network.getNetworkUuid());

    }

}
