package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.compute.PowervcServerProvider;
import com.bocloud.common.model.BocloudResult;
import com.google.common.collect.Lists;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.Collections;
import java.util.List;

public class PvServerDriverTest extends BaseDriverTest {
    private static PowervcServerProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcServerProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetail() {
        BocloudResult result = instance.detail("sxp-test-vm-id");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        ServerModel server = new ServerModel();
        server.setName("lkx-vm-lyc");
        server.setHostName("8203E4A_10F8563");
        server.setFlavorUuid("99e5075c-c401-4535-95c2-355dc9f6dc3e");
        server.setImageUuid("a69116ec-0a28-4954-8b37-5243994dfb7c");
        server.setPassword("123456");
        //server.setSubnetUuid("1e04605d-5734-46cf-b544-305a0b296b4b");
        //server.setNetworkUuid("1c49c7dc-645a-42e6-8911-dda226c94c92");
        List<NetworkModel> networks = Lists.newArrayList();
        NetworkModel network = new NetworkModel();
        network.setNetworkUuid("1c49c7dc-645a-42e6-8911-dda226c94c92");
        network.setFixedIp("192.168.2.203");
        networks.add(network);
        server.setNetworks(networks);

        List<VolumeModel> disks = Lists.newArrayList();
        VolumeModel v1 = new VolumeModel();
        v1.setVolumeUuid("a0e75f3a-6850-4a1f-9c80-e113e5311a63");
        v1.setDeleteWithInstance(false);
        VolumeModel v2 = new VolumeModel();
        v2.setVolumeUuid("19abff1b-d366-4744-8832-fd0667371dce");
        v2.setDeleteWithInstance(false);
        VolumeModel v3 = new VolumeModel();
        v3.setVolumeUuid("20315772-b614-469e-bb9f-941d49a68a0b");
        v3.setDeleteWithInstance(true);
        //disks.add(v1);
        //disks.add(v2);
        //disks.add(v3);
        server.setDiskDevices(disks);

        BocloudResult result = instance.create(server);

        System.out.println(JSON.toJSONString(result));
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testModify() {
        ServerModel serverModel = new ServerModel();
        serverModel.setInstanceId("aa2c1dd0-fbf2-4f17-9f34-a248d6bb20af");
        serverModel.setName("lycDisabled");
        serverModel.setRemark("this is a remark");
        BocloudResult result = instance.modify(serverModel);
        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testStart() {
        BocloudResult result = instance.start("-vm-instanceId-");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testStop() {
        BocloudResult result = instance.stop("-vm-instanceId-");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testReboot() {
        BocloudResult result = instance.reboot("-vm-instanceId-");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testSoftReboot() {
        BocloudResult result = instance.softReboot("-vm-instanceId-");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testMigrate() {
        ServerModel server = new ServerModel();
        server.setInstanceId("-vm-instanceId-");
        BocloudResult result = instance.migrate(server);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testMigrate2() {
        OSMigrateModel model = new OSMigrateModel();
        model.setInstanceId("-vm-instanceId-");
        model.setLive(true);
        BocloudResult result = instance.migrate(model);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testAttachVolume() {
        VolumeModel volumeModel = new VolumeModel();
        volumeModel.setVolumeUuid("8f1f241d-da78-4a52-a90a-589d1200ace7");

        BocloudResult result = instance.attachVolume("cabb27f7-ea12-4fc7-9a43-9676825fe9a3", volumeModel);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDettachVolume() {
        VolumeModel volumeModel = new VolumeModel();
        volumeModel.setVolumeUuid("8f1f241d-da78-4a52-a90a-589d1200ace7");

        BocloudResult result = instance.detachVolume("cabb27f7-ea12-4fc7-9a43-9676825fe9a3", volumeModel);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testAttachNetwork() {
        AddressBean addressBean = new AddressBean();
        addressBean.setAddress("192.168.2.79");
        addressBean.setSubnetUuid("57ff7b79-a5a6-437b-965a-0d44fe5e467f");

        NetworkBean networkBean = new NetworkBean();
        networkBean.setNetworkUuid("4f2270d7-f84b-4a13-90f2-4c2fc2597c04");
        networkBean.setAddresses(Collections.singletonList(addressBean));

        ServerModel serverModel = new ServerModel();
        serverModel.setNetworkBeans(Collections.singletonList(networkBean));
        serverModel.setInstanceId("2b30b8c0-fd5f-4b4c-bfcf-296c46fbb391");

        BocloudResult result = instance.attachNetwork(serverModel);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetachNetwork() {
        BocloudResult result = instance.detachNetwork("3308c22f-837e-4ab5-bd00-47fd02bf8bfd", "f9d4a1c8-954f-4a46-90a6-7d34203d54db");

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }
}
