package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.network.PowervcSubnetProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

public class PvSubnetDriverTest extends BaseDriverTest {
    private static PowervcSubnetProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcSubnetProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        SubnetModel subnet = new SubnetModel();
        subnet.setNetworkUuid("5334381a-26af-4edb-b33c-e8bf5834708c");
        subnet.setName("sxp.test.subnet");
        subnet.setIpVersion("V4");
        subnet.setDhcpEnabled(true);
        subnet.setCidr("10.10.100.0/24");
        subnet.setGateway("10.10.100.1");

        BocloudResult result = instance.create(subnet);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testModify() {
        SubnetModel subnet = new SubnetModel();
        subnet.setSubnetUuid("bf8949d6-ecef-483e-9738-07547f68dae3");
        subnet.setName("sxp.test.subnet.modify");
        subnet.setDhcpEnabled(true);

        BocloudResult result = instance.modify(subnet);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testRemove() {
        String subnetId = "bf8949d6-ecef-483e-9738-07547f68dae3";

        BocloudResult result = instance.remove(subnetId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetail() {
        String subnetId = "a932e4c0-afe8-4dd4-bb9a-5ed1cb3cb7ef";
        BocloudResult result = instance.detail(subnetId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

}
