package com.bocloud.cmp.driver.provider.powervc;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumeMetadataModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.storage.PowervcVolumeProvider;
import com.bocloud.common.model.BocloudResult;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.ArrayList;

public class PvVolumeDriverTest extends BaseDriverTest {
    private static PowervcVolumeProvider instance;

    @BeforeClass
    public static void setUp() {
        instance = new PowervcVolumeProvider(bulter);
    }

    @Test
    public void testList() {
        BocloudResult result = instance.list();

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testCreate() {
        VolumeModel volume = new VolumeModel();
        volume.setName("sxp.test.volume");
        volume.setRemark("施小平测试");
        volume.setSize(2);
        volume.setBootVolume(false);
        // volume.setImageUuid("test-image-id");

        BocloudResult result = instance.create(volume);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testGet() {
        BocloudResult result = instance.detail("ef78f3fa-ca6b-4287-9fcb-a814ebadfce9");
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testModify() {
        VolumeModel volume = new VolumeModel();
        volume.setVolumeUuid("f654129d-6e83-4ab0-9576-7a3114f956df");
        volume.setName("sxp.test.volume.modify");
        volume.setRemark("施小平测试.modify");

        BocloudResult result = instance.modify(volume);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }

        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testRemove() {
        String volumeId = "a481c6cd-dd98-4994-b125-c5ebbf083ecc";

        BocloudResult result = instance.remove(volumeId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testDetail() {
        String volumeId = "f654129d-6e83-4ab0-9576-7a3114f956df";
        BocloudResult result = instance.detail(volumeId);

        if (result.isFailed()) {
            System.out.println(result.getMessage());
        } else {
            System.out.println(JSON.toJSONString(result.getData(), true));
        }
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testOnboard() {
        BocloudResult result = instance.onboard("192_168_0_8", new ArrayList<>());
        System.out.println(result.getData());
        Assert.assertTrue(result.isSuccess());
    }

    @Test
    public void testUpdateMetadata() {
        VolumeMetadataModel metadata = new VolumeMetadataModel("60050768029E0EB8C8000000000000B8", true, true, "7110bdee-9bb5-4bc5-b2f1-84c4b484bdd8");
        BocloudResult result = instance.setMetadata("c052f907-de59-4420-b036-8b9534ae6678", metadata);
        System.out.println(result.getData());
        Assert.assertTrue(result.isSuccess());
    }


    @Test
    public void testGetMetaData() {
        String volumeId = "da9a734e-8cdf-4df1-9560-1d04004fc82d";
        BocloudResult mtResult = instance.getMetadata(volumeId);
        Assert.assertTrue(mtResult.isSuccess());
        JSONObject mt = (JSONObject) mtResult.getData();
        System.out.println("JSON.toJSONString(mt) = " + JSON.toJSONString(mt));
        System.out.println("volume wwn = " + mt.getString("volume_wwn"));
    }

    @Test
    public void testVolumes() {
        BocloudResult mtResult = instance.allVolumes("192_168_0_8");
        Assert.assertTrue(mtResult.isSuccess());
        System.out.println("mtResult = " + mtResult);
    }
}

