package com.bocloud.cmp.pqcloudconvertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: SecurityGroupRuleConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class SecurityGroupRuleConvertor implements BeanJsonConvertor<SecurityGroupRuleModel> {

    private static Logger logger = LoggerFactory.getLogger(SecurityGroupRuleConvertor.class);

    @Override
    public List<SecurityGroupRuleModel> convertList(JSONArray array) {
        List<SecurityGroupRuleModel> list = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            list.add(convertModel(array.getJSONObject(i)));
        }
        return list;
    }

    private String[] protocols = new String[]{"ssh", "http", "https", "ftp", "remote", "openvpn", "pptp", "l2tp",
            "tcp", "udp"};


    @Override
    public SecurityGroupRuleModel convertModel(JSONObject object) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        model.setGroupUuid(object.getString("security_group_id"));
        model.setRuleUuid(object.getString("security_group_rule_id"));
        model.setName(object.getString("security_group_rule_name"));
        model.setAction("action");
        String protocol = object.getString("protocol");
        model.setProtocol(protocol);

        if (Arrays.asList(protocols).contains(protocol.toLowerCase())) {
            model.setRemoteIpPrefix(object.getString("val3"));
            model.setPortMin(object.getString("val2") == null ? null : Integer.valueOf(object.getString("val2")));
            model.setPortMax(object.getString("val1") == null ? null : Integer.valueOf(object.getString("val1")));
        }
        model.setVar1(object.getString("val1"));
        model.setVar2(object.getString("val2"));
        model.setVar3(object.getString("val3"));

        // ingress 0 egress 1
        Integer direction = object.getInteger("direction");
        if (0 == direction) {
            model.setDirection("ingress");
        }
        if (1 == direction) {
            model.setDirection("egress");
        }

        return model;
    }
}
