package com.bocloud.cmp.pqcloudconvertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.utils.DateTools;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: ServerConvertor
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
public class ServerConvertor implements BeanJsonConvertor<ServerModel> {


    @Override
    public List<ServerModel> convertList(JSONArray array) {
        List<ServerModel> list = new ArrayList<>();
        for (int i = 0; i < array.size(); i++) {
            list.add(convertModel(array.getJSONObject(i)));
        }
        return list;
    }

    @Override
    public ServerModel convertModel(JSONObject object) {
        ServerModel model = new ServerModel();
        model.setRemark(object.getString("description"));
        model.setCpu(object.getInteger("vcpus_current"));
        model.setMemory(BigDecimal.valueOf(object.getInteger("memory_current") / 1024));
        model.setOsCategory(object.getString("platform"));
        model.setZone(object.getString("zone_id"));
        JSONObject image = object.getJSONObject("image");
        model.setImageUuid(image.getString("image_id"));
        String createdTime = object.getString("create_time");
        Date date = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z");
        model.setGmtVmCreate(date);
        model.setName(object.getString("instance_name"));
        model.setInstanceId(object.getString("instance_id"));
        model.setFlavorUuid(object.getString("instance_type"));
        model.setStatus(convert(object.getString("status")));
        model.setRegionId(object.getString("zone_id"));
        return model;
    }

    private String convert(String status) {
        if (status.equalsIgnoreCase("running")) {
            return BocloudStatus.Server.RUNNING.name();
        } else if (status.equalsIgnoreCase("stopped")) {
            return BocloudStatus.Server.STOPPED.name();
        } else if (status.equalsIgnoreCase("pending")) {
            return BocloudStatus.Server.BUILDING.name();
        } else if (status.equalsIgnoreCase("ceased")) {
            return "ceased";
        } else {
            return BocloudStatus.Server.EXCEPTION.name();
        }
    }


}
