package com.bocloud.cmp.provider;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.utils.JSONTools;
import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TreeMap;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QingRestAPIProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/11/11 15:42
 */
public class PqcloudRestApiProvider extends GenericProvider implements SuperProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudRestApiProvider.class);

    private final static String CHARSET = "UTF-8";
    private final static String https = "https://";
    private final static String GET = "GET";
    private final static String uri = "/iaas/";

    private String domin;
    private String secretId;
    private String secretKey;
    private Butler butler;
    private String regionId = "nanjing";

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public PqcloudRestApiProvider(Butler butler) {
        super(butler);
        this.secretId = butler.getAccessKey();
        this.secretKey = butler.getSecretKey();
        this.regionId = butler.getRegionId();
        this.domin = butler.getUrl();
    }

    //私有云可用区为公共参数，必传值
    public JSONObject connection(String action, String zone, TreeMap<String, Object> param) {
        return this.toPrivate(action, this.domin, this.secretId, this.secretKey, zone, param);
    }

    //私有云·区译域·
    public JSONObject connection(String action, TreeMap<String, Object> param) {
        return this.toPrivate(action, this.domin, this.secretId, this.secretKey, this.regionId, param);
    }


    @Override
    public boolean accessable() {
        return true;
    }


//    public static void main(String[] args) {
//        Butler butler = new Butler();
//        butler.setAccessKey("QJJYBMXCEQXDCRISTNNO");
//        butler.setSecretKey("ev81uz0wFGlajKKErvgQj6JAcd7qLEqvlCalqRxz");
//        butler.setUrl("api.qingcloud.com/iaas/");
//        butler.setRegionId("pek3c");
//        PqcloudRestApiProvider apiProvider = new PqcloudRestApiProvider(butler);
//        TreeMap<String, Object> param = new TreeMap<>();
//        JSONObject describeInstances = apiProvider.connection("DescribeZones", "", param);
//        System.out.println("args = [" + args + "]");
//    }


    public JSONObject toPrivate(String action, String domain, String accessKey, String secretKey, String zone, TreeMap<String, Object> param) {
        try {
            TreeMap<String, Object> params = new TreeMap<String, Object>();
            params.put("action", action); // 公共参数
            params.put("zone", zone); // 公共参数
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
            String format = simpleDateFormat.format(new Date());
            params.put("time_stamp", format); // 公共参数
            params.put("access_key_id", accessKey);
            params.put("version", "1");
            params.put("signature_method", "HmacSHA256");
            params.put("signature_version", "1");
            params.putAll(param);
            //请求参数
            String queryParams = queryParams(params);
            //构建被签名串
            String strToSign = getStringToSign(queryParams);
            //生成签名串
            String signature = genHMAC(strToSign, secretKey);
            //拼接到最后
            String queryParams2 = queryParams + "&signature=" + signature;
            //构建url
            String url = getUrl(https, domain, queryParams2);
            HttpClient httpClient = new HttpClient();
            JSONObject object = JSONTools.isJSONObj(httpClient.get(url).getMessage());
            return object;
        } catch (Exception e) {
            logger.error("云平台连接错误：" + e.getMessage());
            return null;
        }
    }

    //请求参数排序
    public String queryParams(TreeMap<String, Object> params) throws UnsupportedEncodingException {
        StringBuilder s2s = new StringBuilder();
        // 签名时要求对参数进行字典排序，此处用TreeMap保证顺序
        for (String k : params.keySet()) {
            s2s.append(URLEncoder.encode(k, CHARSET)).append("=").append(URLEncoder.encode(params.get(k).toString(), CHARSET)).append("&");
        }
        return s2s.toString().substring(0, s2s.length() - 1);
    }

    //公共方法-字典排序   通用
    public String getStringToSign(String queryParams) throws UnsupportedEncodingException {
        StringBuilder s2s = new StringBuilder(GET + "\n" + uri + "\n");
        s2s.append(queryParams);
        return s2s.toString();
    }

    // 公共方法-获取地址  通用
    public String getUrl(String http, String domain, String queryParams2) throws UnsupportedEncodingException {
        StringBuilder url = new StringBuilder(http + domain + "?");
        url.append(queryParams2);
        return url.toString();
    }


    public String genHMAC(String srcStr, String secretKey) throws Exception {
        byte[] result = null;
        SecretKeySpec signinKey = new SecretKeySpec(secretKey.getBytes(), "HmacSHA256");
        // 生成一个指定 Mac 算法 的 Mac 对象
        Mac mac = Mac.getInstance("HmacSHA256");
        // 用给定密钥初始化 Mac 对象
        mac.init(signinKey);
        // 完成 Mac 操作
        byte[] rawHmac = mac.doFinal(srcStr.getBytes());
        result = Base64.encodeBase64(rawHmac);
        if (null != result) {
            String signatureResult = new String(result);
            String signature = URLEncoder.encode(signatureResult, "UTF-8");
            return signature;
        } else {
            return null;
        }
    }


    public String getDomin() {
        return domin;
    }

    public void setDomin(String domin) {
        this.domin = domin;
    }

    public String getSecretId() {
        return secretId;
    }

    public void setSecretId(String secretId) {
        this.secretId = secretId;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public void setSecretKey(String secretKey) {
        this.secretKey = secretKey;
    }

    public Butler getButler() {
        return butler;
    }

    public void setButler(Butler butler) {
        this.butler = butler;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }
}











