package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.pqcloudconvertor.JobConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.PqcloudJobModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudJobProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/11/13 14:13
 */
public class PqcloudJobProvider extends PqcloudRestApiProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudJobProvider.class);

    private BeanJsonConvertor<PqcloudJobModel> convertor = new JobConvertor();

    public PqcloudJobProvider(Butler butler) {
        super(butler);
    }


    public PqcloudJobModel getJob(String jobId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("jobs.0", jobId);
        JSONObject object = this.connection("DescribeJobs", params);
        if (object.getInteger("ret_code") == 0) {
            List<PqcloudJobModel> pqcloudJobModels = convertor.convertList(object.getJSONArray("job_set"));
            return pqcloudJobModels.get(0);
        } else {
            logger.error("list image error ： " + object.getString("message"));
            return null;
        }
    }

    /**
     * 等待任务执行结束
     *
     * @param jobId
     * @return
     */
    public boolean waitJob(String jobId) {
        boolean flag = true;
        PqcloudJobModel model = this.getJob(jobId);
        Long startTime = System.currentTimeMillis();
        while (flag) {
            if (null == model) {
                flag = false;
                break;
            }
            if ("successful".equalsIgnoreCase(model.getStatus())) {
                flag = true;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待10分钟
            if (endTime - startTime > 10 * 60 * 1000) {
                flag = false;
                break;
            }
            model = this.getJob(jobId);
        }
        return flag;
    }


}
