package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.pqcloudconvertor.RegionConvertor;
import com.bocloud.cmp.pqcloudconvertor.ZoneConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.compute.PqcloudImageProvider;
import com.bocloud.cmp.provider.compute.PqcloudServerProvider;
import com.bocloud.cmp.provider.compute.PqcloudSnapshotProvider;
import com.bocloud.cmp.provider.security.PqcloudKeypairProvider;
import com.bocloud.cmp.provider.storage.PqcloudVolumeProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.compute.*;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.cmp.provider.network.*;
import com.bocloud.cmp.provider.security.KeypairProvider;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudLocationProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudLocationProvider extends PqcloudRestApiProvider implements LocationProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudLocationProvider.class);

    private Butler butler;
    private FlavorProvider flavorProvider = null;
    private ImageProvider imageProvider = null;
    private KeypairProvider keypairProvider = null;
    private SecurityGroupProvider secgProvider = null;
    private ServerProvider serverProvider = null;
    private SnapshotProvider snapshotProvider = null;
    private SubnetProvider subnetProvider = null;
    private VolumeProvider volumeProvider = null;
    private VpcProvider vpcProvider = null;
    private SecurityGroupRuleProvider securityGroupRuleProvider = null;
    private PqcloudFloatingIpProvider floatingIpProvider;

    public PqcloudLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
        flavorProvider = new PqcloudFlavorProvider(butler);
        imageProvider = new PqcloudImageProvider(butler);
        keypairProvider = new PqcloudKeypairProvider(butler);
        serverProvider = new PqcloudServerProvider(butler);
        snapshotProvider = new PqcloudSnapshotProvider(butler);
        volumeProvider = new PqcloudVolumeProvider(butler);
        vpcProvider = new PqcloudVpcProvider(butler);
        subnetProvider = new PqcloudSubnetProvider(butler);
        secgProvider = new PqcloudSecuGroupProvider(butler);
        securityGroupRuleProvider = new PqcloudSecuGroupRuleProvider(butler);
        floatingIpProvider = new PqcloudFloatingIpProvider(butler);

    }

    private BeanJsonConvertor<ZoneModel> zoneConvertor = new ZoneConvertor();
    private BeanJsonConvertor<RegionModel> regionConvertor = new RegionConvertor();

    @Override
    public BocloudResult sync() {
        try {
            PqcloudSyncModel syncModel = new PqcloudSyncModel();
            List<FlavorModel> flavorModels = new ArrayList<>();
            List<ImageModel> imageModels = new ArrayList<>();
            List<VpcModel> vpcModels = new ArrayList<>();
            List<SecurityGroupModel> securityGroupModels = new ArrayList<>();
            List<VolumeModel> volumeModels = new ArrayList<>();
            List<ServerModel> serverModels = new ArrayList<>();
            List<SnapshotModel> snapshotModels = new ArrayList<>();
            List<SubnetModel> subnetModels = new ArrayList<>();
            List<KeypairModel> keypairModels = new ArrayList<>();
            List<ZoneModel> zoneModels = new ArrayList<>();
            List<FloatingIpModel> floatingIpModels = new ArrayList<>();
            List<RegionModel> regionModels = new ArrayList<>();

            logger.info("{} 区译域 同步-->Start", this.butler.getName());
            BocloudResult listRegions = this.listRegions();
            if (listRegions.isSuccess()) {
                regionModels = JSONObject.parseArray(JSONObject.toJSONString(listRegions.getData()), RegionModel.class);
            }
            logger.info("{} 区译域 同步-->end", this.butler.getName());


            logger.info("{} 可用区 同步-->Start", this.butler.getName());
            BocloudResult listZones = this.listZones(null);
            if (listZones.isSuccess()) {
                zoneModels = JSONObject.parseArray(JSONObject.toJSONString(listZones.getData()), ZoneModel.class);
            }
            logger.info("{} 可用区 同步-->end", this.butler.getName());


            logger.info("{} 规格 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult flavorResult = flavorProvider.list(zoneModel.getName());
                if (flavorResult.isSuccess()) {
                    flavorModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(flavorResult.getData()), FlavorModel.class));
                }
            }
            logger.info("{} 规格 同步-->End", this.butler.getName());


            logger.info("{} 镜像 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult imageResult = imageProvider.list(zoneModel.getName());
                if (imageResult.isSuccess()) {
                    imageModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(imageResult.getData()), ImageModel.class));
                }
            }
            logger.info("{} 镜像 同步-->End", this.butler.getName());


            logger.info("{} 硬盘 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult volumeResult = volumeProvider.list(zoneModel.getName());
                if (volumeResult.isSuccess()) {
                    volumeModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(volumeResult.getData()), VolumeModel.class));
                }
            }
            logger.info("{} 硬盘 同步-->End", this.butler.getName());


            logger.info("{} - 快照 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult snapshotResult = snapshotProvider.list(zoneModel.getName());
                if (snapshotResult.isSuccess()) {
                    snapshotModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(snapshotResult.getData()), SnapshotModel.class));
                }
            }
            logger.info("{} - 快照 同步-->End", this.butler.getName());

            logger.info("{} 密钥 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult keypairResult = keypairProvider.list(zoneModel.getName());
                if (keypairResult.isSuccess()) {
                    keypairModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(keypairResult.getData()), KeypairModel.class));
                }
            }
            logger.info("{} 密钥 同步-->End", this.butler.getName());

            logger.info("{} -  vpc 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult vpcResult = vpcProvider.list(zoneModel.getName());
                if (vpcResult.isSuccess()) {
                    vpcModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(vpcResult.getData()), VpcModel.class));
                }
            }
            logger.info("{} -  vpc 同步-->End", this.butler.getName());


            logger.info("{} -  子网 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult subnetResult = subnetProvider.list(zoneModel.getName());
                if (subnetResult.isSuccess()) {
                    subnetModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(subnetResult.getData()), SubnetModel.class));
                }
            }
            logger.info("{} -  子网 同步-->End", this.butler.getName());


            logger.info("{} 安全组 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult securityResult = secgProvider.list(zoneModel.getName());
                if (securityResult.isSuccess()) {
                    securityGroupModels = JSONObject.parseArray(JSONObject.toJSONString(securityResult.getData()), SecurityGroupModel.class);
                    for (SecurityGroupModel securityGroupModel : securityGroupModels) {
                        BocloudResult secRuleResult = securityGroupRuleProvider.list(securityGroupModel.getGroupUuid(), zoneModel.getName());
                        if (secRuleResult.isSuccess()) {
                            List<SecurityGroupRuleModel> parseArray = JSONObject.parseArray(
                                    JSONObject.toJSONString(secRuleResult.getData()), SecurityGroupRuleModel.class);
                            securityGroupModel.setModels(parseArray);
                        }
                    }
                }
            }
            logger.info("{} 安全组 同步-->End", this.butler.getName());

            logger.info("{} 云主机 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult serverResult = serverProvider.listByZone(zoneModel.getName());
                if (serverResult.isSuccess()) {
                    serverModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(serverResult.getData()), ServerModel.class));
                }
            }
            logger.info("{} 云主机 同步-->End", this.butler.getName());

            logger.info("{} 浮动ip 同步-->Start", this.butler.getName());
            for (ZoneModel zoneModel : zoneModels) {
                BocloudResult serverResult = floatingIpProvider.list(zoneModel.getName());
                if (serverResult.isSuccess()) {
                    floatingIpModels.addAll(JSONObject.parseArray(JSONObject.toJSONString(serverResult.getData()), FloatingIpModel.class));
                }
            }
            logger.info("{} 浮动ip 同步-->End", this.butler.getName());


            syncModel.setFloatingIpModels(floatingIpModels);
            syncModel.setFlavorModels(flavorModels);
            syncModel.setImageModels(imageModels);
            syncModel.setServerModels(serverModels);
            syncModel.setVolumeModels(volumeModels);
            syncModel.setSnapshotModels(snapshotModels);
            syncModel.setVpcModels(vpcModels);
            syncModel.setSubnetModels(subnetModels);
            syncModel.setSecurityGroupModels(securityGroupModels);
            syncModel.setKeypairModels(keypairModels);
            syncModel.setRegionModels(regionModels);
            syncModel.setZoneModels(zoneModels);

            return new BocloudResult(true, syncModel, "青云私有云 同步成功！");
        } catch (Exception e) {
            logger.error("青云私有云同步失败： " + e);
            return new BocloudResult(false, e.getMessage(), "青云私有云同步失败");
        }
    }

    @Override
    public BocloudResult listRegions() {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeZones", params);
        if (object.getInteger("ret_code") == 0) {
            List<RegionModel> regionModels = regionConvertor.convertList(object.getJSONArray("zone_set"));
            return new BocloudResult(true, regionModels, "查询成功");
        } else {
            logger.error("list zone error ： " + object.getString("message"));
            return new BocloudResult(false, "查询失败", object.getString("message"));
        }
    }

    @Override
    public BocloudResult listZones(String region) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeZones", params);
        if (object.getInteger("ret_code") == 0) {
            List<ZoneModel> zoneModels = zoneConvertor.convertList(object.getJSONArray("zone_set"));
            return new BocloudResult(true, zoneModels, "查询成功");
        } else {
            logger.error("list zone error ： " + object.getString("message"));
            return new BocloudResult(false, "查询失败", object.getString("message"));
        }
    }

    @Override
    public BocloudResult syncPortByNetworkUuid(String networkUuid) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getHostModel(HostModel arg0) {
        // TODO Auto-generated method stub
        return null;
    }


}
