package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.pqcloudconvertor.ImageConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudImageProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudImageProvider extends PqcloudRestApiProvider implements ImageProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudImageProvider.class);

    private BeanJsonConvertor<ImageModel> convertor = new ImageConvertor();

    public PqcloudImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeImages", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<ImageModel> imageModels = convertor.convertList(object.getJSONArray("image_set"));
            for (ImageModel imageModel : imageModels) {
                imageModel.setRegionId(zone);
            }
            return new BocloudResult(true, imageModels, "image list success!");
        } else {
            logger.error("image list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String imageId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("images.0", imageId);
        JSONObject object = this.connection("DescribeImages", params);
        if (object.getInteger("ret_code") == 0) {
            List<ImageModel> imageModels = convertor.convertList(object.getJSONArray("image_set"));
            ImageModel imageModel = imageModels.get(0);
            imageModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, imageModel, "image detail success!");
        } else {
            logger.error("image detail error ：" + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(String imageId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("images.0", imageId);
        JSONObject object = this.connection("DeleteImages", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "image remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "image remove success!");
        } else {
            logger.error("image remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("image", image.getImageUuid());
        params.put("image_name", image.getName());
        params.put("description", image.getRemark());
        JSONObject object = this.connection("ModifyImageAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "image modify success!");
        } else {
            logger.error("image modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult create(ImageModel imageModel) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("instance", imageModel.getInstanceId());
        params.put("image_name", imageModel.getName());
        JSONObject object = this.connection("CaptureInstance", params);
        if (object.getInteger("ret_code") == 0) {
            String imageId = object.getString("image_id");
            return getImage(imageId);
        } else {
            logger.error("create image error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult shareImage(ImageModel imageModel) {
        return null;
    }

    @Override
    public BocloudResult syncImages(ImageModel imageModel) {
        return null;
    }

    public BocloudResult getImage(String imageId) {
        BocloudResult detailResult = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(6000);
                BocloudResult detail = this.detail(imageId);
                ImageModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ImageModel.class);
                if (model == null) {
                    flag = false;
                    break;
                }
                if (("ACTIVE".equalsIgnoreCase(model.getStatus()))) {
                    flag = false;
                    detailResult = this.detail(imageId);
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    detailResult = this.detail(imageId);
                    break;
                }
            }
            return detailResult;
        } catch (Exception e) {
            logger.error("查询状态失败： " + e);
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }
}
