package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.pqcloudconvertor.SnapshotConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;


/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudSnapshotProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudSnapshotProvider extends PqcloudRestApiProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudSnapshotProvider.class);

    private BeanJsonConvertor<SnapshotModel> convertor = new SnapshotConvertor();

    public PqcloudSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeSnapshots", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<SnapshotModel> snapshotModels = convertor.convertList(object.getJSONArray("snapshot_set"));
            return new BocloudResult(true, snapshotModels, "查询成功");
        } else {
            logger.error("list snapshot error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("snapshots.0", model.getSnapshotUuid());
        JSONObject object = this.connection("DeleteSnapshots", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "snapshot remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "snapshot remove success!");
        } else {
            logger.error("snapshot remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("snapshots.0", model.getSnapshotUuid());
        JSONObject object = this.connection("DescribeSnapshots", params);
        if (object.getInteger("ret_code") == 0) {
            List<SnapshotModel> snapshotModels = convertor.convertList(object.getJSONArray("snapshot_set"));
            return new BocloudResult(true, snapshotModels.get(0), "snapshot detail success!");
        } else {
            logger.error("snapshot detail error ：" + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("snapshot", model.getSnapshotUuid());
        params.put("snapshot_name", model.getName());
        params.put("description", model.getRemark());
        JSONObject object = this.connection("ModifySnapshotAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, this.detail(model), "snapshot modify success!");
        } else {
            logger.error("snapshot modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        Boolean flag = true;
        BocloudResult result = new BocloudResult(false, "查询失败");
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            SnapshotModel model = new SnapshotModel();
            model.setSnapshotUuid(snapshotId);
            BocloudResult detail = this.detail(model);
            if (detail.isSuccess()) {
                SnapshotModel snapshot = (SnapshotModel) detail.getData();
                if (null == snapshot) {
                    logger.error("snapshot [{}] doesnot exsit!", snapshotId);
                    return new BocloudResult(false, "快照不存在！");
                }
                String snapshotState = snapshot.getStatus();
                if ("NORMAL".equalsIgnoreCase(snapshotState) ||
                        "AVAILABLE".equalsIgnoreCase(snapshotState)) {
                    flag = false;
                    result = detail;
                    break;
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待5分钟
            if (endTime - startTime > 5 * 60 * 1000) {
                flag = false;
                result = detail;
                break;
            }
        }
        return result;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult list(Long arg0) {
        // TODO Auto-generated method stub
        return null;
    }

}
