package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.pqcloudconvertor.FloatingIpConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;


/**
 * 青云公网IP操作类
 *
 * @author wangyu
 * @时间 2019年10月24日
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudFloatingIpProvider extends PqcloudRestApiProvider implements FloatingIPProvider {

    public PqcloudFloatingIpProvider(Butler butler) {
        super(butler);
    }

    private static Logger logger = LoggerFactory.getLogger(PqcloudFloatingIpProvider.class);

    private BeanJsonConvertor<FloatingIpModel> convertor = new FloatingIpConvertor();

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeEips", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<FloatingIpModel> floatingIpModels = convertor.convertList(object.getJSONArray("eip_set"));
            for (FloatingIpModel floatingIpModel : floatingIpModels) {
                floatingIpModel.setRegionId(zone);
            }
            return new BocloudResult(true, floatingIpModels, "eip list success!");
        } else {
            logger.error("eip list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(FloatingIpModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("bandwidth", Integer.valueOf(model.getBandwidth()));
        params.put("billing_mode", model.getInternetChargeType());
        params.put("eip_name", model.getName());
        params.put("count", 1);
        params.put("need_icp", model.getNeedIcp());
        JSONObject object = this.connection("AllocateEips", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray eips = object.getJSONArray("eips");
            return getEip(eips.get(0).toString());
        } else {
            logger.error("create vpc error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    public BocloudResult getEip(String eipId) {
        BocloudResult detailResult = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(6000);
                BocloudResult detail = this.detail(eipId);
                FloatingIpModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), FloatingIpModel.class);
                if (model == null) {
                    flag = false;
                    break;
                }
                if ("active".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    detailResult = this.detail(eipId);
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    detailResult = this.detail(eipId);
                    break;
                }
            }
            return detailResult;
        } catch (Exception e) {
            logger.error("查询状态失败： " + e);
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }


    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(String fipId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("eips.0", fipId);
        JSONObject object = this.connection("ReleaseEips", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "eip remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "eip remove success!");
        } else {
            logger.error("eip remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String fipId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("eips.0", fipId);
        JSONObject object = this.connection("DescribeEips", params);
        if (object.getInteger("ret_code") == 0) {
            List<FloatingIpModel> floatingIpModels = convertor.convertList(object.getJSONArray("eip_set"));
            FloatingIpModel floatingIpModel = floatingIpModels.get(0);
            floatingIpModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, floatingIpModel, "floatingIp detail success!");
        } else {
            logger.error("floatingIp detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult associate(String fipId, String portId) {
        return null;
    }

    @Override
    public BocloudResult associate(FloatingIpModel modeld) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("eip", modeld.getFloatingIpUuid());
        params.put("instance", modeld.getInstanceId());
        JSONObject object = this.connection("AssociateEip", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "eip associate fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "eip associate success!");
        } else {
            logger.error("eip associate error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult disassociate(String fipId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("eips.0", fipId);
        JSONObject object = this.connection("DissociateEips", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "eip dissociate fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "eip dissociate success!");
        } else {
            logger.error("eip dissociate error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }
}
