package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.pqcloudconvertor.SecurityGroupConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudSecuGroupProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudSecuGroupProvider extends PqcloudRestApiProvider implements SecurityGroupProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudSecuGroupProvider.class);

    private BeanJsonConvertor<SecurityGroupModel> convertor = new SecurityGroupConvertor();

    public PqcloudSecuGroupProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeSecurityGroups", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<SecurityGroupModel> securityGroupModels = convertor.convertList(object.getJSONArray("security_group_set"));
            for (SecurityGroupModel securityGroupModel : securityGroupModels) {
                securityGroupModel.setRegionId(zone);
            }
            return new BocloudResult(true, securityGroupModels, "secuGroup list success!");
        } else {
            logger.error("secuGroup list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(SecurityGroupModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group_name", model.getName());
        JSONObject object = this.connection("CreateSecurityGroup", params);
        if (object.getInteger("ret_code") == 0) {
            String sgId = object.getString("security_group_id");
            if (StringUtils.isEmpty(sgId)) {
                return new BocloudResult(false, "secuGroup create error ： " + object.getString("message"));
            }
            return new BocloudResult(true, this.detail(sgId).getData(), "secuGroup create success!");
        } else {
            logger.error("secuGroup create error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult remove(String sgId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_groups.0", sgId);
        JSONObject object = this.connection("DeleteSecurityGroups", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "secuGroup remove success!");
        } else {
            logger.error("secuGroup remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String sgId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_groups.0", sgId);
        JSONObject object = this.connection("DescribeSecurityGroups", params);
        if (object.getInteger("ret_code") == 0) {
            List<SecurityGroupModel> securityGroupModels = convertor.convertList(object.getJSONArray("security_group_set"));
            SecurityGroupModel securityGroupModel = securityGroupModels.get(0);
            securityGroupModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, securityGroupModel, "secuGroup detail success!");
        } else {
            logger.error("secuGroup detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult modify(SecurityGroupModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group", model.getGroupUuid());
        params.put("security_group_name", model.getName());
        params.put("description", model.getRemark());
        JSONObject object = this.connection("ModifySecurityGroupAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            String sgId = object.getString("security_group_id");
            if (StringUtils.isEmpty(sgId)) {
                return new BocloudResult(false, "secuGroup modify error ： " + object.getString("message"));
            }
            return new BocloudResult(true, this.detail(sgId).getData(), "secuGroup modify success!");
        } else {
            logger.error("secuGroup modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }
}
