package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.pqcloudconvertor.SecurityGroupRuleConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudSecuGroupRuleProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudSecuGroupRuleProvider extends PqcloudRestApiProvider implements SecurityGroupRuleProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudSecuGroupRuleProvider.class);

    private BeanJsonConvertor<SecurityGroupRuleModel> convertor = new SecurityGroupRuleConvertor();

    public PqcloudSecuGroupRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String zone) {
        return null;
    }

    @Override
    public BocloudResult list(String sgId, String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group", sgId);
        JSONObject object = this.connection("DescribeSecurityGroupRules", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<SecurityGroupRuleModel> securityGroupRuleModels = convertor.convertList(object.getJSONArray("security_group_rule_set"));
            return new BocloudResult(true, securityGroupRuleModels, "secuGroupRule list success!");
        } else {
            logger.error("secuGroupRule list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }

    }


    @Override
    public BocloudResult detail(String sgrlId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group_rules.0", sgrlId);
        JSONObject object = this.connection("DescribeSecurityGroupRules", params);
        if (object.getInteger("ret_code") == 0) {
            List<SecurityGroupRuleModel> securityGroupRuleModels = convertor.convertList(object.getJSONArray("security_group_rule_set"));
            return new BocloudResult(true, securityGroupRuleModels.get(0), "secuGroupRule detail success!");
        } else {
            logger.error("secuGroupRule detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult create(SecurityGroupRuleModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group", model.getGroupUuid());
        params.put("rules.0.priority", 1);
        params.put("rules.0.security_group_rule_name", model.getName());
        params.put("rules.0.action", model.getAction().toLowerCase());
        String direction = model.getDirection();
        if ("ingress".equalsIgnoreCase(direction)) {
            params.put("rules.0.direction", 0);
        }
        if ("egress".equalsIgnoreCase(direction)) {
            params.put("rules.0.direction", 1);
        }
        String lowerCase = model.getProtocol().toLowerCase();
        params.put("rules.0.protocol", model.getProtocol().toLowerCase());
        Integer portMin = model.getPortMin();
        Integer portMax = model.getPortMax();
        // icmp特殊处理
        if ("icmp".equalsIgnoreCase(lowerCase)) {
            portMin = 8;
            portMax = 0;
        }
        if (null == portMax || null == portMin) {
            return new BocloudResult(false, "起始端口、结束端口不允许为空！");
        }
        params.put("rules.n.val1", portMin + "");
        params.put("rules.n.val2", portMax + "");
        params.put("rules.n.val3", model.getRemoteIpPrefix());
        JSONObject object = this.connection("AddSecurityGroupRules", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray securityGroupRules = object.getJSONArray("security_group_rules");
            if (ListTool.isEmpty(securityGroupRules)) {
                return new BocloudResult(true, "安全组添加成功！");
            }
            return new BocloudResult(true, this.detail(securityGroupRules.get(0).toString()).getData(), "secuGroupRule create success!");
        } else {
            logger.error("secuGroupRule create error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult remove(String ruleId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("security_group_rules.0", ruleId);
        JSONObject object = this.connection("DeleteSecurityGroupRules", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "secuGroupRule remove success!");
        } else {
            logger.error("secuGroupRule remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult remove(SecurityGroupRuleModel model) {
        return null;
    }


}
