package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.pqcloudconvertor.SubnetConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudSubnetProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudSubnetProvider extends PqcloudRestApiProvider implements SubnetProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudSubnetProvider.class);

    private BeanJsonConvertor<SubnetModel> convertor = new SubnetConvertor();

    public PqcloudSubnetProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeVxnets", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<SubnetModel> subnetModels = convertor.convertList(object.getJSONArray("vxnet_set"));
            return new BocloudResult(true, subnetModels, "查询成功");
        } else {
            logger.error("list subnet error ： " + object.getString("message"));
            return new BocloudResult(false, "查询失败", object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(SubnetModel subnet) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnet_name", subnet.getName());
        params.put("vxnet_type", 1);
        params.put("count", 1);
        JSONObject object = this.connection("CreateVxnets", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray vxnets = object.getJSONArray("vxnets");
            return joinRouter(subnet.getVpcUuid(), subnet.getCidr(), vxnets.get(0).toString());
        } else {
            logger.error("create subnet error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    private BocloudResult joinRouter(String routerId, String IpNetwork, String vxNetId) {
        try {
            Thread.sleep(3000);
            TreeMap<String, Object> params = new TreeMap<String, Object>();
            params.put("vxnet", vxNetId);
            params.put("router", routerId);
            params.put("ip_network", IpNetwork);
            JSONObject object = this.connection("JoinRouter", params);
            if (object.getInteger("ret_code") == 0) {
                String jobId = object.getString("job_id");
                if (!this.waitJob(jobId)) {
                    return new BocloudResult(false, "subnet joinRouter fail!: " + object.getString("message"));
                }
                return new BocloudResult(true, "subnet joinRouter success!");
            } else {
                logger.error("subnet joinRouter error ： " + object.getString("message"));
                return new BocloudResult(false, object.getString("message"));
            }
        } catch (Exception e) {
            logger.error("subnet joinRouter error ： " + e.getMessage());
            return new BocloudResult(false, e.getMessage());
        }
    }

    private BocloudResult leaveRouter(String routerId, String vxNetId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnets.0", vxNetId);
        params.put("router", routerId);
        JSONObject object = this.connection("LeaveRouter", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "subnet LeaveRouter fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "subnet LeaveRouter success!");
        } else {
            logger.error("subnet LeaveRouter error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult modify(SubnetModel subnet) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnet", subnet.getSubnetUuid());
        params.put("vxnet_name", subnet.getName());
        params.put("description", subnet.getRemark());
        JSONObject object = this.connection("ModifyVxnetAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "subnet modify success!");
        } else {
            logger.error("subnet modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult remove(String subnetId) {
        BocloudResult detail = this.detail(subnetId);
        if (detail.isFailed()) {
            return new BocloudResult(false, detail.getMessage());
        }
        SubnetModel subnetModel = (SubnetModel) detail.getData();
        BocloudResult leaveRouter = leaveRouter(subnetModel.getVpcUuid(), subnetId);
        if (leaveRouter.isFailed()) {
            return leaveRouter;
        }
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnets.0", subnetId);
        JSONObject object = this.connection("DeleteVxnets", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "subnet remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "subnet remove success!");
        } else {
            logger.error("subnet remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String subnetId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnets", subnetId);
        JSONObject object = this.connection("DescribeVxnets", params);
        if (object.getInteger("ret_code") == 0) {
            List<SubnetModel> subnetModels = convertor.convertList(object.getJSONArray("vxnet_set"));
            return new BocloudResult(true, subnetModels.get(0), "subnet detail success!");
        } else {
            logger.error("subnet detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(SubnetModel subnet) {
        BocloudResult detail = this.detail(subnet.getSubnetUuid());
        if (detail.isFailed()) {
            return new BocloudResult(false, detail.getMessage());
        }
        SubnetModel subnetModel = (SubnetModel) detail.getData();
        BocloudResult leaveRouter = leaveRouter(subnetModel.getVpcUuid(), subnet.getSubnetUuid());
        if (leaveRouter.isFailed()) {
            return leaveRouter;
        }
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("vxnets.0", subnet.getSubnetUuid());
        JSONObject object = this.connection("DeleteVxnets", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "subnet remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "subnet remove success!");
        } else {
            logger.error("subnet remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }
}
