package com.bocloud.cmp.provider.network;

import com.bocloud.cmp.pqcloudconvertor.VpcConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudVpcProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudVpcProvider extends PqcloudRestApiProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudVpcProvider.class);

    private BeanJsonConvertor<VpcModel> convertor = new VpcConvertor();

    public PqcloudVpcProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeRouters", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<VpcModel> vpcModels = convertor.convertList(object.getJSONArray("router_set"));
            for (VpcModel vpcModel : vpcModels) {
                vpcModel.setRegionId(zone);
            }
            return new BocloudResult(true, vpcModels, "vpc list success!");
        } else {
            logger.error("vpc list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(VpcModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("router_name", model.getName());
        params.put("router_type", 1);
        params.put("vpc_network", model.getCidr());
        params.put("count", 1);
        JSONObject object = this.connection("CreateRouters", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray routers = object.getJSONArray("routers");
            return getVpc(routers.get(0).toString());
        } else {
            logger.error("create vpc error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    public BocloudResult getVpc(String vpcId) {
        BocloudResult detailResult = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(6000);
                BocloudResult detail = this.detail(vpcId, null);
                VpcModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), VpcModel.class);
                if (model == null) {
                    flag = false;
                    break;
                }
                if ("active".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    detailResult = this.detail(vpcId, null);
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    detailResult = this.detail(vpcId, null);
                    break;
                }
            }
            return detailResult;
        } catch (Exception e) {
            logger.error("查询状态失败： " + e);
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(String vpcId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("routers.0", vpcId);
        JSONObject object = this.connection("DeleteRouters", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "vpc remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "vpc remove success!");
        } else {
            logger.error("vpc remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("routers.0", vpcId);
        JSONObject object = this.connection("DescribeRouters", params);
        if (object.getInteger("ret_code") == 0) {
            List<VpcModel> vpcModels = convertor.convertList(object.getJSONArray("router_set"));
            VpcModel vpcModel = vpcModels.get(0);
            vpcModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, vpcModel, "vpc detail success!");
        } else {
            logger.error("vpc detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("router", model.getVpcId());
        params.put("router_name", model.getName());
        params.put("description", model.getRemark());
        JSONObject object = this.connection("ModifyRouterAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return this.detail(model.getVpcId(), this.getRegionId());
        } else {
            logger.error("vpc modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

}
