package com.bocloud.cmp.provider.security;

import com.bocloud.cmp.pqcloudconvertor.KeypairConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: QcloudKeypairProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudKeypairProvider extends PqcloudRestApiProvider implements KeypairProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudKeypairProvider.class);

    private BeanJsonConvertor<KeypairModel> convertor = new KeypairConvertor();

    public PqcloudKeypairProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeKeyPairs", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<KeypairModel> keypairModels = convertor.convertList(object.getJSONArray("keypair_set"));
            for (KeypairModel keypairModel : keypairModels) {
                keypairModel.setRegionId(zone);
            }
            return new BocloudResult(true, keypairModels, "keypair list success!");
        } else {
            logger.error("keypair list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String keypairName) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeKeyPairs", params);
        if (object.getInteger("ret_code") == 0) {
            List<KeypairModel> keypairModels = convertor.convertList(object.getJSONArray("keypair_set"));
            KeypairModel keypairModel = keypairModels.get(0);
            keypairModel.setRegionId(this.getRegionId());
            return new BocloudResult(true, keypairModel, "keypair detail success!");
        } else {
            logger.error("keypair list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypair_name", keypairName);
        if (null != publicKey && !"".equals(publicKey)) {
            params.put("mode", "user");
            params.put("public_key", publicKey);
        } else {
            params.put("mode", "system");
        }
        JSONObject object = this.connection("CreateKeyPair", params);
        if (object.getInteger("ret_code") == 0) {
            String keypairId = object.getString("keypair_id");
            String privateKey = object.getString("private_key");
            BocloudResult bocloudResult = this.detail(keypairId);
            KeypairModel model = JSONObject.parseObject(JSONObject.toJSONString(bocloudResult.getData()), KeypairModel.class);
            model.setPrivateKey(privateKey);
            return new BocloudResult(true, model, "keypair create success!");
        } else {
            logger.error("keypair create error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult delete(String keypairName) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypairs.0", keypairName);
        JSONObject object = this.connection("DeleteKeyPairs", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "keypair remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "keypair remove success!");
        } else {
            logger.error("keypair remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult modify(KeypairModel model) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypair", model.getKeyId());
        params.put("keypair_name", model.getName());
        params.put("description", model.getRemark());
        JSONObject object = this.connection("ModifyKeyPairAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "keypair modify success!");
        } else {
            logger.error("keypair modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult create(String keypairName, String publicKey, String encryptMethod) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("keypair_name", keypairName);
        if (null != publicKey && !"".equals(publicKey)) {
            params.put("mode", "user");
            params.put("public_key", publicKey);
        } else {
            params.put("mode", "system");
            params.put("encrypt_method", encryptMethod);
        }
        JSONObject object = this.connection("CreateKeyPair", params);
        if (object.getInteger("ret_code") == 0) {
            String keypairId = object.getString("keypair_id");
            String privateKey = object.getString("private_key");
            BocloudResult bocloudResult = this.detail(keypairId);
            KeypairModel model = JSONObject.parseObject(JSONObject.toJSONString(bocloudResult.getData()), KeypairModel.class);
            model.setPrivateKey(privateKey);
            return new BocloudResult(true, model, "keypair create success!");
        } else {
            logger.error("keypair create error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

}
