package com.bocloud.cmp.provider.storage;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.pqcloudconvertor.VolumeConvertor;
import com.bocloud.cmp.provider.PqcloudRestApiProvider;
import com.bocloud.cmp.provider.common.PqcloudJobProvider;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.h3c.H3cVolumeModel;
import com.bocloud.cmp.provider.convertor.BeanJsonConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.TreeMap;

/**
 * @Company: 博云 www.bocloud.com
 * @ClassName: PqcloudVolumeProvider
 * @Description:
 * @author: zhenpingLi/李振平
 * @date: 2019/10/28 16:47
 */
@Provider("bocloud.cmp.pqcloud")
public class PqcloudVolumeProvider extends PqcloudRestApiProvider implements VolumeProvider {

    private static Logger logger = LoggerFactory.getLogger(PqcloudVolumeProvider.class);

    private BeanJsonConvertor<VolumeModel> convertor = new VolumeConvertor();

    private Butler butler;

    public PqcloudVolumeProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeVolumes", zone, params);
        if (object.getInteger("ret_code") == 0) {
            List<VolumeModel> volumeModels = convertor.convertList(object.getJSONArray("volume_set"));
            return new BocloudResult(true, volumeModels, "volume list success!");
        } else {
            logger.error("volume list error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    @Override
    public BocloudResult create(VolumeModel volume) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("size", volume.getSize());
        params.put("volume_name", volume.getName());
        params.put("volume_type", Integer.valueOf(volume.getVolumeType()));
        params.put("count", 1);
        JSONObject object = this.connection("CreateVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            JSONArray routers = object.getJSONArray("volumes");
            return getVolume(routers.get(0).toString());
        } else {
            logger.error("create image error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }


    public BocloudResult getVolume(String volumeId) {
        BocloudResult detailResult = null;
        try {
            Boolean flag = true;
            Long startTime = System.currentTimeMillis();
            while (flag) {
                Thread.sleep(6000);
                BocloudResult detail = this.detail(volumeId);
                VolumeModel model = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), VolumeModel.class);
                if (model == null) {
                    flag = false;
                    break;
                }
                if ("AVAILABLE".equalsIgnoreCase(model.getStatus())) {
                    flag = false;
                    detailResult = this.detail(volumeId);
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待60s
                if (endTime - startTime > 5 * 60 * 1000) {
                    flag = false;
                    detailResult = this.detail(volumeId);
                    break;
                }
            }
            return detailResult;
        } catch (Exception e) {
            logger.error("查询状态失败： " + e);
            return new BocloudResult(false, "查询状态失败", e.getMessage());
        }
    }


    @Override
    public BocloudResult modify(VolumeModel volume) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("volume", volume.getVolumeUuid());
        params.put("volume_name", volume.getName());
        params.put("description", volume.getRemark());
        JSONObject object = this.connection("ModifyVolumeAttributes", params);
        if (object.getInteger("ret_code") == 0) {
            return new BocloudResult(true, "volume modify success!");
        } else {
            logger.error("volume modify error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    public boolean waitJob(String jobId) {
        PqcloudJobProvider provider = new PqcloudJobProvider(this.getButler());
        return provider.waitJob(jobId);
    }

    @Override
    public BocloudResult remove(String volumeId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("volumes.0", volumeId);
        JSONObject object = this.connection("DeleteVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "volume remove fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "volume remove success!");
        } else {
            logger.error("volume remove error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult detail(String volumeId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        JSONObject object = this.connection("DescribeVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            List<VolumeModel> volumeModels = convertor.convertList(object.getJSONArray("volume_set"));
            return new BocloudResult(true, volumeModels.get(0), "volume detail success!");
        } else {
            logger.error("volume detail error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }


    @Override
    public BocloudResult attach(String serverId, String volumeId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("volumes.0", volumeId);
        params.put("instance", serverId);
        JSONObject object = this.connection("AttachVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "volume attach fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "volume attach success!");
        } else {
            logger.error("volume attach error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult attach(String serverId, String mountpoint, String volumeId) {
        return null;
    }

    @Override
    public BocloudResult detach(String serverId, String volumeId) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("volumes.0", volumeId);
        params.put("instance", serverId);
        JSONObject object = this.connection("DetachVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "volume detach fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "volume detach success!");
        } else {
            logger.error("volume detach error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult resetStatus(String volumeId, String status) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult uploadToImage(String volumeId, ImageModel imageModel) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult extend(String volumeId, Integer newSize) {
        TreeMap<String, Object> params = new TreeMap<String, Object>();
        params.put("volumes.0", volumeId);
        params.put("size", newSize);
        JSONObject object = this.connection("ResizeVolumes", params);
        if (object.getInteger("ret_code") == 0) {
            String jobId = object.getString("job_id");
            if (!this.waitJob(jobId)) {
                return new BocloudResult(false, "volume resize fail!: " + object.getString("message"));
            }
            return new BocloudResult(true, "volume resize success!");
        } else {
            logger.error("volume resize error ： " + object.getString("message"));
            return new BocloudResult(false, object.getString("message"));
        }
    }

    @Override
    public BocloudResult list(String hostId, String storagePoolName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(H3cVolumeModel arg0) {
        // TODO Auto-generated method stub
        return null;
    }

}
