package com.bocloud.cmp.provider;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.qingcloud.sdk.config.EnvContext;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.JobService;
import com.qingcloud.sdk.service.QingCloudService;
import com.qingcloud.sdk.service.Types;

/**
 * @ Description： @ Author ：王宇 @ Date ：Created in 17:08 2019/10/23 @ Modified
 * By：
 */
public class QcloudProvider extends GenericProvider implements SuperProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudProvider.class);
    private String accessKey;// access key
    private String secretKey;// secret key
    private String regionId;// 可用域
    private EnvContext context;// 上下文

    public QcloudProvider(Butler butler) {
        super(butler);
        this.accessKey = butler.getAccessKey(); // LCAICZKUEIECMGTGFRKR
        this.secretKey = butler.getSecretKey(); // sQEfttXEFIjnbnUOWgfYAW31sQh0VqQdLTPnb1l1
        this.regionId = butler.getRegionId(); // pek3
        this.context = new EnvContext(this.accessKey, this.secretKey);
        this.context.setProtocol("https");
        this.context.setHost("api.qingcloud.com");
        this.context.setPort("443");
        if (StringUtils.isEmpty(this.regionId)) {
            this.context.setZone("pek3b");
        } else {
            this.context.setZone(this.regionId.toLowerCase());
        }
        this.context.setApiLang("zh-cn"); // optional, set return message i18n,
                                          // default to us-en
    }

    public String getAccessKey() {
        return accessKey;
    }

    public String getSecretKey() {
        return secretKey;
    }

    public String getRegionId() {
        return regionId;
    }

    public EnvContext getContext() {
        return context;
    }

    /**
     * 云平台是否可以访问
     *
     * @return
     */
    @Override
    public boolean accessable() {
        try {
            QingCloudService service = new QingCloudService(context);
            QingCloudService.DescribeZonesInput input = new QingCloudService.DescribeZonesInput();
            service.describeZones(input);
            return true;
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return false;
        }
    }

    /**
     * 获取任务
     *
     * @param jobId
     * @return
     */
    public Types.JobModel getJob(String jobId) {
        try {
            QingCloudService service = new QingCloudService(context);
            JobService.DescribeJobsInput input = new JobService.DescribeJobsInput();
            List<String> jobs = new ArrayList<>();
            jobs.add(jobId);
            input.setJobs(jobs);
            JobService.DescribeJobsOutput output = service.getJobService(this.getRegionId()).describeJobs(input);
            Types.JobModel model = output.getJobSet().get(0);
            if (null == model) {
                return null;
            }
            return model;
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return null;
        }
    }

    /**
     * 等待任务执行结束
     *
     * @param jobId
     * @return
     */
    public boolean waitJob(String jobId) {
        boolean flag = true;
        Types.JobModel model = this.getJob(jobId);
        Long startTime = System.currentTimeMillis();
        while (flag) {
            if (null == model) {
                flag = false;
                break;
            }
            if ("successful".equalsIgnoreCase(model.getStatus())) {
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待10分钟
            if (endTime - startTime > 10 * 60 * 1000) {
                flag = false;
                break;
            }
            model = this.getJob(jobId);
        }
        return flag;
    }
}