package com.bocloud.cmp.provider.common;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.QingCloudService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 青云认证
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/10/23
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudAuthProvider extends QcloudProvider implements AuthProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudAuthProvider.class);

    public QcloudAuthProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult auth() {
        QingCloudService service = new QingCloudService(this.getContext());
        QingCloudService.DescribeZonesInput input = new QingCloudService.DescribeZonesInput();
        try {
            QingCloudService.DescribeZonesOutput output = service.describeZones(input);
            Integer zoneCount = output.getTotalCount();
            if (null == zoneCount) {
                return new BocloudResult(false, "认证失败");
            }
            return new BocloudResult(true, "认证通过");
        } catch (QCException e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "认证失败：" + e.getMessage());
        }
    }
}
