package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.api.QcloudDescribeInstanceTypesOutput;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.FlavorConvertor;
import com.bocloud.cmp.provider.sdk.QcloudDescribeInstanceTypesInput;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.InstanceService;
import com.qingcloud.sdk.service.Types;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * qcloud云主机规格操作类
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2018/11/15
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudFlavorProvider extends QcloudProvider implements FlavorProvider {

	private static Logger logger = LoggerFactory.getLogger(QcloudFlavorProvider.class);
	private BeanConvertor<FlavorModel, Types.InstanceTypeModel> convertor = new FlavorConvertor();
	private InstanceService service = new InstanceService(this.getContext());

	private static final int PAGE_SIZE = 100;

	public QcloudFlavorProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult sync() {
		List<FlavorModel> models = new ArrayList<>();
		QcloudDescribeInstanceTypesInput input = new QcloudDescribeInstanceTypesInput();
		input.setLimit(PAGE_SIZE);
		input.setOffset(0);
		try {
			InstanceService.DescribeInstanceTypesOutput output = service.describeInstanceTypes(input);
			if (null == output || null == output.getTotalCount()) {
				return new BocloudResult(false, "规格获取失败！");
			}
			Integer totalCount = output.getTotalCount();
			if (null != output.getInstanceTypeSet() && !ListTool.isEmpty(output.getInstanceTypeSet())) {
				models.addAll(convertor.convertList(output.getInstanceTypeSet()));
			}
			int totalPage = totalCount / PAGE_SIZE + 1;
			if (totalPage == 1) {
				return new BocloudResult(true, models, "success");
			}
			for (int i = 2; i <= totalPage; i++) {
				input.setLimit(PAGE_SIZE);
				input.setOffset((i - 1) * PAGE_SIZE);
				output = service.describeInstanceTypes(input);
				if (null != output.getInstanceTypeSet() && !ListTool.isEmpty(output.getInstanceTypeSet())) {
					models.addAll(convertor.convertList(output.getInstanceTypeSet()));
				}
			}
			for (FlavorModel flavorModel : models) {
				flavorModel.setRegionId(this.getRegionId());
			}
			return new BocloudResult(true, models, "success");
		} catch (Exception e) {
			logger.error("list image error:", e);
			return new BocloudResult(false, "list image error!");
		}
	}

	@Override
	public BocloudResult create(FlavorModel flavor) {
		return null;
	}

	@Override
	public BocloudResult remove(String flavorId) {
		return null;
	}

	@Override
	public BocloudResult update(FlavorModel flavorModel) {
		return null;
	}

	@Override
	public BocloudResult list(String zone) {
		try {
			if (StringUtils.isEmpty(zone)) {
                return new BocloudResult(false, "可用区为空！");
            }
            InstanceService service = new InstanceService(this.getContext(), zone);
            InstanceService.DescribeInstanceTypesInput input = new InstanceService.DescribeInstanceTypesInput();
            InstanceService.DescribeInstanceTypesOutput output = service.describeInstanceTypes(input);
            List<FlavorModel> models = new ArrayList<>();
            if (!ListTool.isEmpty(output.getInstanceTypeSet())) {
                models = convertor.convertList(output.getInstanceTypeSet());
            }
			return new BocloudResult(true, models, "查询成功");
		} catch (QCException e) {
			logger.error("Get error message：{}", e);
			return new BocloudResult(false, "查询失败");
		}
	}

}
