package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.ServerConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.qingcloud.sdk.constants.QCConstant;
import com.qingcloud.sdk.request.ResourceRequestFactory;
import com.qingcloud.sdk.service.InstanceService;
import com.qingcloud.sdk.service.KeyPairService;
import com.qingcloud.sdk.service.KeyPairService.AttachKeyPairsInput;
import com.qingcloud.sdk.service.KeyPairService.AttachKeyPairsOutput;
import com.qingcloud.sdk.service.KeyPairService.DetachKeyPairsInput;
import com.qingcloud.sdk.service.KeyPairService.DetachKeyPairsOutput;
import com.qingcloud.sdk.service.SecurityGroupService;
import com.qingcloud.sdk.service.SecurityGroupService.ApplySecurityGroupInput;
import com.qingcloud.sdk.service.SecurityGroupService.ApplySecurityGroupOutput;
import com.qingcloud.sdk.service.InstanceService.*;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.InstanceModel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 云主机操作
 *
 * @author ymh
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudServerProvider extends QcloudProvider implements ServerProvider {

    private BeanConvertor<ServerModel, Types.InstanceModel> convertor = new ServerConvertor();

    private static Logger logger = LoggerFactory.getLogger(QcloudServerProvider.class);
    private static final String APPLY_SECURITY_GROUP = "ApplySecurityGroup";

    public QcloudServerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult list() {
        try {
            InstanceService service = new InstanceService(this.getContext());
            DescribeInstancesInput input = new DescribeInstancesInput();
            input.setVerbose(1);
            DescribeInstancesOutput describeInstances = service.describeInstances(input);
            return new BocloudResult(true, convertor.convertList(describeInstances.getInstanceSet()), "查询列表成功！");
        } catch (Exception e) {
            logger.error("查询列表异常：" + e.getMessage());
            return new BocloudResult(false, "查询列表异常：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list(String hostId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            // imageUuid : centos76x64
            // flavor c1m1
            InstanceService service = new InstanceService(this.getContext());
            RunInstancesInput input = new RunInstancesInput();
            input.setZone(server.getZone());
            input.setImageID(server.getImageUuid());
            input.setInstanceType(server.getFlavorUuid());

            input.setOSDiskSize(server.getRootSize());
            input.setInstanceName(server.getName());
            /**
             * 指定登录方式。当为 linux 主机时，有效值为 keypair 和 passwd; 当为 windows 主机时，只能选用
             * passwd 登录方式。 当登录方式为 keypair 时，需要指定 login_keypair 参数； 当登录方式为
             * passwd 时，需要指定 login_passwd 参数。
             */
            if (StringUtils.isEmpty(server.getKeypairName())) {
                input.setLoginMode("passwd");
                input.setLoginPasswd(server.getPassword());
            } else {
                input.setLoginMode("keypair");
                input.setLoginKeyPair(server.getKeypairName());
            }
            // 网络
            List<String> vxNets = new ArrayList<String>();
            List<NetworkBean> networkBeans = server.getNetworkBeans();
            for (NetworkBean networkBean : networkBeans) {
                String vxnet = networkBean.getNetworkUuid();
                if (StringUtils.isNotEmpty(networkBean.getFixIp())) {
                    vxnet += ("|" + networkBean.getFixIp());
                }
                vxNets.add(vxnet);
            }
            // "vxnet-f8b2mgr|192.168.45.12"
            input.setVxNets(vxNets);
            List<String> securityGroups = server.getSecurityGroups();
            if (null != securityGroups && securityGroups.size() > 0) {
                input.setSecurityGroup(securityGroups.get(0));
            }
            RunInstancesOutput runInstances = service.runInstances(input);
            if (0 == runInstances.getRetCode()) {
                List<String> instances = runInstances.getInstances();
                ServerModel instacesDetail = getInstacesDetail(instances.get(0));
                return new BocloudResult(instacesDetail != null, instacesDetail, "创建成功！");
            } else {
                return new BocloudResult(false, runInstances.getMessage());
            }
        } catch (Exception e) {
            logger.error("云主机创建失败：" + e.getMessage());
            return new BocloudResult(false, "云主机创建失败：" + e.getMessage());
        }
    }

    private ServerModel getInstacesDetail(String instanceId) throws Exception {
        ServerModel convertModel = null;
        InstanceService service = new InstanceService(this.getContext());
        DescribeInstancesInput input = new DescribeInstancesInput();
        input.setVerbose(1);
        input.setInstances(Arrays.asList(new String[] { instanceId }));
        boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            DescribeInstancesOutput describeInstances = service.describeInstances(input);
            List<InstanceModel> instanceSet = describeInstances.getInstanceSet();
            if (null == instanceSet || instanceSet.size() == 0) {
                flag = false;
                break;
            }
            convertModel = convertor.convertModel(instanceSet.get(0));
            if (null == convertModel) {
                flag = false;
                break;
            }
            String privateIps = convertModel.getPrivateIps();
            List<JSONObject> parseArray = JSON.parseArray(privateIps, JSONObject.class);
            String address = null;
            if (null != parseArray && parseArray.size() > 0) {
                JSONObject jsonObject = parseArray.get(0);
                if (null != jsonObject) {
                    address = jsonObject.getString("address");
                }
            }
            if ("running".equalsIgnoreCase(convertModel.getStatus()) && null != address && !"".equals(address)) {
                flag = true;
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待10分钟
            if (endTime - startTime > 10 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        return convertModel;
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            DescribeInstancesInput input = new DescribeInstancesInput();
            input.setVerbose(1);
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            DescribeInstancesOutput describeInstances = service.describeInstances(input);
            List<InstanceModel> instanceSet = describeInstances.getInstanceSet();
            if (null == instanceSet || instanceSet.size() == 0) {
                return new BocloudResult(false, "数据不存在！");
            }
            return new BocloudResult(true, convertor.convertModel(instanceSet.get(0)), "查询成功！");
        } catch (Exception e) {
            logger.error("查询失败：" + e.getMessage());
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            ModifyInstanceAttributesInput input = new ModifyInstanceAttributesInput();
            input.setInstance(server.getInstanceId());
            if (StringUtils.isNotEmpty(server.getName())) {
                input.setInstanceName(server.getName());
            }
            if (StringUtils.isNotEmpty(server.getRemark())) {
                input.setDescription(server.getRemark());
            }
            ModifyInstanceAttributesOutput modifyInstanceAttributes = service.modifyInstanceAttributes(input);
            if (0 == modifyInstanceAttributes.getRetCode()) {
                return new BocloudResult(true, "修改成功！");
            }
            return new BocloudResult(false, "修改失败");
        } catch (Exception e) {
            logger.error("修改失败：" + e.getMessage());
            return new BocloudResult(false, "修改失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            TerminateInstancesInput input = new TerminateInstancesInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            TerminateInstancesOutput terminateInstances = service.terminateInstances(input);
            String jobID = terminateInstances.getJobID();
            return new BocloudResult(true, jobID, "删除成功！");
        } catch (Exception e) {
            logger.error("删除失败：" + e.getMessage());
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            StartInstancesInput input = new StartInstancesInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            StartInstancesOutput startInstances = service.startInstances(input);
            if (!this.waitJob(startInstances.getJobID())) {
                return new BocloudResult(false, "启动失败");
            }
            return new BocloudResult(true, "启动成功");
        } catch (Exception e) {
            logger.error("启动失败：" + e.getMessage());
            return new BocloudResult(false, "启动失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            StopInstancesInput input = new StopInstancesInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            StopInstancesOutput stopInstances = service.stopInstances(input);
            if (!this.waitJob(stopInstances.getJobID())) {
                return new BocloudResult(false, "关闭失败");
            }
            return new BocloudResult(true, "关闭成功");
        } catch (Exception e) {
            logger.error("关闭失败：" + e.getMessage());
            return new BocloudResult(false, "关闭失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            RestartInstancesInput input = new RestartInstancesInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            RestartInstancesOutput restartInstances = service.restartInstances(input);
            if (!this.waitJob(restartInstances.getJobID())) {
                return new BocloudResult(false, "重启失败");
            }
            return new BocloudResult(true, "重启成功");
        } catch (Exception e) {
            logger.error("重启失败：" + e.getMessage());
            return new BocloudResult(false, "重启失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        try {
            SecurityGroupService service = new SecurityGroupService(this.getContext());
            ApplySecurityGroupInput input = new ApplySecurityGroupInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            input.setSecurityGroup(group);
            ApplySecurityGroupOutput applySecurityGroup = service.applySecurityGroup(input);
            if (!this.waitJob(applySecurityGroup.getJobID())) {
                return new BocloudResult(false, "安全组绑定失败");
            }
            return new BocloudResult(true, "安全组绑定成功！");
        } catch (Exception e) {
            logger.error("安全组绑定失败：" + e.getMessage());
            return new BocloudResult(false, "安全组绑定失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        try {
            ApplySecurityGroupInput input = new ApplySecurityGroupInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            input.setSecurityGroup(group);
            Map<String, Object> contextMap = new HashMap<String, Object>();
            contextMap.put(QCConstant.ENV_CONTEXT_KEY, this.getContext());
            contextMap.put("OperationName", APPLY_SECURITY_GROUP);
            contextMap.put("APIName", APPLY_SECURITY_GROUP);
            contextMap.put("ServiceName", APPLY_SECURITY_GROUP);
            contextMap.put("RequestMethod", "GET");
            contextMap.put("RequestURI", "/iaas/" + APPLY_SECURITY_GROUP);
            input.setZone(this.getRegionId());
            input.setAction("RemoveSecurityGroup");
            Object backModel = ResourceRequestFactory.getResourceRequest().sendApiRequest(contextMap, input,
                    ApplySecurityGroupOutput.class);
            if (backModel != null) {
                ApplySecurityGroupOutput applySecurityGroup = (ApplySecurityGroupOutput) backModel;
                if (!this.waitJob(applySecurityGroup.getJobID())) {
                    return new BocloudResult(false, "安全组解绑失败");
                } else {
                    return new BocloudResult(true, "安全组解绑失败");
                }
            }
            return new BocloudResult(false, "安全组解绑失败！");
        } catch (Exception e) {
            logger.error("安全组解绑失败：" + e.getMessage());
            return new BocloudResult(false, "安全组解绑失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        return null;
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            ResizeInstancesInput input = new ResizeInstancesInput();
            input.setInstances(Arrays.asList(new String[] { instanceId }));
            input.setInstanceType(flavorId);
            ResizeInstancesOutput resizeInstances = service.resizeInstances(input);
            if (!this.waitJob(resizeInstances.getJobID())) {
                return new BocloudResult(false, "重置失败");
            }
            return new BocloudResult(true, this.detail(instanceId).getData(), "重置成功！");
        } catch (Exception e) {
            logger.error("重置失败：" + e.getMessage());
            return new BocloudResult(false, "重置失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getVmIps(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult migrate(H3cMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult create(H3cServerModel server) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        try {
            InstanceService service = new InstanceService(this.getContext());
            ResetInstancesInput input = new ResetInstancesInput();
            input.setInstances(Arrays.asList(new String[] { serverModel.getInstanceId() }));
            if (StringUtils.isEmpty(serverModel.getKeypairName())) {
                input.setLoginMode("passwd");
                input.setLoginPasswd(serverModel.getPassword());
            } else {
                input.setLoginMode("keypair");
                input.setLoginKeyPair(serverModel.getKeypairName());
            }
            ResetInstancesOutput resetInstances = service.resetInstances(input);
            if (0 == resetInstances.getRetCode()) {
                if (!this.waitJob(resetInstances.getJobID())) {
                    return new BocloudResult(false, "重置系统失败");
                }
                return new BocloudResult(true, this.detail(serverModel.getInstanceId()).getData(), "重置系统成功");
            } else {
                return new BocloudResult(false, resetInstances.getMessage());
            }
        } catch (Exception e) {
            logger.error("重置系统异常：" + e.getMessage());
            return new BocloudResult(false, "重置系统异常：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        try {
            KeyPairService service = new KeyPairService(this.getContext());
            AttachKeyPairsInput input = new AttachKeyPairsInput();
            input.setInstances(Arrays.asList(new String[] { serverModel.getInstanceIds()[0] }));
            input.setKeyPairs(Arrays.asList(new String[] { serverModel.getKeypairIds()[0] }));
            AttachKeyPairsOutput attachKeyPairs = service.attachKeyPairs(input);
            if (0 == attachKeyPairs.getRetCode()) {
                return new BocloudResult(true, "绑定成功！");
            } else {
                return new BocloudResult(false, attachKeyPairs.getMessage());
            }
        } catch (Exception e) {
            logger.error("绑定秘钥失败：" + e.getMessage());
            return new BocloudResult(false, "绑定秘钥失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        try {
            KeyPairService service = new KeyPairService(this.getContext());
            DetachKeyPairsInput input = new DetachKeyPairsInput();
            input.setInstances(Arrays.asList(new String[] { serverModel.getInstanceIds()[0] }));
            input.setKeyPairs(Arrays.asList(new String[] { serverModel.getKeypairIds()[0] }));
            DetachKeyPairsOutput detachKeyPairs = service.detachKeyPairs(input);
            if (0 == detachKeyPairs.getRetCode()) {
                return new BocloudResult(true, "解绑成功！");
            } else {
                return new BocloudResult(false, detachKeyPairs.getMessage());
            }
        } catch (Exception e) {
            logger.error("解绑秘钥失败：" + e.getMessage());
            return new BocloudResult(false, "解绑秘钥失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        return null;
    }

    @Override
    public BocloudResult listByZone(String zone) {
        try {
            InstanceService service = new InstanceService(this.getContext(), zone);
            DescribeInstancesInput input = new DescribeInstancesInput();
            input.setVerbose(1);
            DescribeInstancesOutput describeInstances = service.describeInstances(input);
            List<ServerModel> convertList = convertor.convertList(describeInstances.getInstanceSet());
            for (ServerModel serverModel : convertList) {
                serverModel.setRegionId(this.getRegionId());
                serverModel.setZone(zone);
            }
            return new BocloudResult(true, convertList, "查询成功！");
        } catch (Exception e) {
            logger.error("查询异常：" + e.getMessage());
            return new BocloudResult(false, "查询异常：" + e.getMessage());
        }
    }

}