package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.SnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.exception.QCException;
import com.qingcloud.sdk.service.SnapshotService;
import com.qingcloud.sdk.service.Types;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 快照操作
 *
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudSnapshotProvider extends QcloudProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudSnapshotProvider.class);
    private BeanConvertor<SnapshotModel, Types.SnapshotModel> convertor = new SnapshotConvertor();
    private SnapshotService service = new SnapshotService(this.getContext());

    public QcloudSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult list() {
        try {
            SnapshotService.DescribeSnapshotsInput input = new SnapshotService.DescribeSnapshotsInput();
            SnapshotService.DescribeSnapshotsOutput output = service.describeSnapshots(input);
            List<SnapshotModel> snapshots = new ArrayList<>();
            if (!ListTool.isEmpty(output.getSnapshotSet())) {
                snapshots = convertor.convertList(output.getSnapshotSet());
            }
            return new BocloudResult(true, snapshots, "查询成功");
        } catch (QCException e) {
            logger.error("查询列表失败：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            SnapshotService.DeleteSnapshotsInput input = new SnapshotService.DeleteSnapshotsInput();
            List<String> snapshots = new ArrayList<>();
            snapshots.add(model.getSnapshotUuid());
            input.setSnapshots(snapshots);
            SnapshotService.DeleteSnapshotsOutput output = service.deleteSnapshots(input);
            if (!this.waitJob(output.getJobID())) {
                return new BocloudResult(false, "删除失败：" + output.getMessage());
            }
            return new BocloudResult(true, "删除成功");
        } catch (QCException e) {
            logger.error("删除失败：{}", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        try {
            SnapshotService.DescribeSnapshotsInput input = new SnapshotService.DescribeSnapshotsInput();
            List<String> snapshots = new ArrayList<>();
            snapshots.add(model.getSnapshotUuid());
            input.setSnapshots(snapshots);
            SnapshotService.DescribeSnapshotsOutput output = service.describeSnapshots(input);
            if (ListTool.isEmpty(output.getSnapshotSet())) {
                return new BocloudResult(false, "查询失败：" + output.getMessage());
            }
            return new BocloudResult(true, convertor.convertModel(output.getSnapshotSet().get(0)), "查询成功");
        } catch (QCException e) {
            logger.error("查询详情异常：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult modify(SnapshotModel model) {
        try {
            SnapshotService.ModifySnapshotAttributesInput input = new SnapshotService.ModifySnapshotAttributesInput();
            input.setSnapshot(model.getSnapshotUuid());
            input.setSnapshotName(model.getName());
            input.setDescription(model.getRemark());
            SnapshotService.ModifySnapshotAttributesOutput output = service.modifySnapshotAttributes(input);
            if (0 != output.getRetCode()) {
                return new BocloudResult(false, "修改失败：" + output.getMessage());
            }
            return new BocloudResult(true, this.detail(model), "修改成功");
        } catch (QCException e) {
            logger.error("快照编辑异常：{}", e);
            return new BocloudResult(false, "修改失败");
        }
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult removeByVolumeId(String volumeId) {
        return null;
    }

    @Override
    public BocloudResult getSnapshot(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult getAvaliableSnap(String snapshotId) {
        return null;
    }

    @Override
    public BocloudResult list(Long cloudServerId) {
        return null;
    }

    @Override
    public BocloudResult list(String zone) {
        try {
            SnapshotService.DescribeSnapshotsInput input = new SnapshotService.DescribeSnapshotsInput();
            input.setZone(zone);
            SnapshotService.DescribeSnapshotsOutput output = service.describeSnapshots(input);
            List<SnapshotModel> snapshots = new ArrayList<>();
            if (!ListTool.isEmpty(output.getSnapshotSet())) {
                snapshots = convertor.convertList(output.getSnapshotSet());
            }
            return new BocloudResult(true, snapshots, "查询成功");
        } catch (QCException e) {
            logger.error("查询列表异常：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }
}
