package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.qingcloud.sdk.service.RouterService;
import com.qingcloud.sdk.service.VxNetService;
import com.qingcloud.sdk.service.RouterService.JoinRouterInput;
import com.qingcloud.sdk.service.RouterService.JoinRouterOutput;
import com.qingcloud.sdk.service.RouterService.LeaveRouterInput;
import com.qingcloud.sdk.service.RouterService.LeaveRouterOutput;
import com.qingcloud.sdk.service.Types.VxNetModel;
import com.qingcloud.sdk.service.VxNetService.CreateVxNetsInput;
import com.qingcloud.sdk.service.VxNetService.CreateVxNetsOutput;
import com.qingcloud.sdk.service.VxNetService.DeleteVxNetsInput;
import com.qingcloud.sdk.service.VxNetService.DeleteVxNetsOutput;
import com.qingcloud.sdk.service.VxNetService.DescribeVxNetsInput;
import com.qingcloud.sdk.service.VxNetService.DescribeVxNetsOutput;

/**
 * 子网操作
 * 
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudSubnetProvider extends QcloudProvider implements SubnetProvider {

	private static Logger logger = LoggerFactory.getLogger(QcloudSubnetProvider.class);

	private BeanConvertor<SubnetModel, VxNetModel> convertor = new SubnetConvertor();

	public QcloudSubnetProvider(Butler butler) {
		super(butler);
	}

	@Override
	public boolean accessable() {
		return super.accessable();
	}

	@Override
	public BocloudResult list() {
		try {
			VxNetService service = new VxNetService(this.getContext());
			DescribeVxNetsInput input = new DescribeVxNetsInput();
			input.setVxNetType(1);
			DescribeVxNetsOutput describeVxNets = service.describeVxNets(input);
			List<VxNetModel> vxNetSet = describeVxNets.getVxNetSet();
			return new BocloudResult(true, convertor.convertList(vxNetSet), "查询成功！");
		} catch (Exception e) {
			logger.error("查询列表异常：" + e.getMessage());
			return new BocloudResult(false, "查询列表异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult list(String zone) {
		try {
			VxNetService service = new VxNetService(this.getContext(), zone);
			DescribeVxNetsInput input = new DescribeVxNetsInput();
			input.setVxNetType(1);
			DescribeVxNetsOutput describeVxNets = service.describeVxNets(input);
			List<VxNetModel> vxNetSet = describeVxNets.getVxNetSet();
			List<SubnetModel> convertList = convertor.convertList(vxNetSet);
			if (null != convertList && convertList.size() > 0) {
				for (SubnetModel subnetModel : convertList) {
					subnetModel.setZone(zone);
					subnetModel.setRegionId(this.getRegionId());
				}
			}
			return new BocloudResult(true, convertList, "查询成功！");
		} catch (Exception e) {
			logger.error("根据zone查询列表异常：" + e.getMessage());
			return new BocloudResult(false, "根据zone查询列表异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SubnetModel subnet) {
		try {
			VxNetService vxNetService = new VxNetService(this.getContext(), subnet.getZone());
			CreateVxNetsInput input = new CreateVxNetsInput();
			input.setVxNetName(subnet.getName());
			input.setVxNetType(1);
			CreateVxNetsOutput createVxNets = vxNetService.createVxNets(input);
			if (0 == createVxNets.getRetCode()) {
				List<String> vxNets = createVxNets.getVxNets();
				if (null != vxNets && vxNets.size() == 1) {
					String vxNetsId = vxNets.get(0);
					BocloudResult joinRouter = joinRouter(subnet.getVpcUuid(), subnet.getCidr(), vxNetsId);
					if (joinRouter.isSuccess()) {
						return new BocloudResult(true, this.detail(vxNetsId).getData(), "子网创建成功！");
					} else {
						this.remove(vxNetsId);
						return joinRouter;
					}
				}
			}
			return new BocloudResult(false, "创建失败：" + createVxNets.getMessage());
		} catch (Exception e) {
			logger.error("子网创建异常：" + e.getMessage());
			return new BocloudResult(false, "子网创建异常：" + e.getMessage());
		}
	}

	private BocloudResult joinRouter(String routerId, String IpNetwork, String vxNetId) {
		try {
			Thread.sleep(3000);
			RouterService service = new RouterService(this.getContext());
			JoinRouterInput input = new JoinRouterInput();
			input.setRouter(routerId);
			input.setIPNetwork(IpNetwork);
			input.setVxNet(vxNetId);
			JoinRouterOutput joinRouter = service.joinRouter(input);
			if (0 == joinRouter.getRetCode()) {
				return new BocloudResult(true, waitJob(joinRouter.getJobID()), "success");
			} else {
				return new BocloudResult(false, joinRouter.getMessage());
			}
		} catch (Exception e) {
			logger.error("子网绑定VPC异常：" + e.getMessage());
			return new BocloudResult(false, "子网绑定VPC异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SubnetModel subnet) {
		return null;
	}

	private BocloudResult leaveRouter(String routerId, String vxNetId) {
		try {
			RouterService service = new RouterService(this.getContext());
			LeaveRouterInput input = new LeaveRouterInput();
			input.setRouter(routerId);
			input.setVxNets(Arrays.asList(new String[] { vxNetId }));
			LeaveRouterOutput leaveRouter = service.leaveRouter(input);
			if (0 == leaveRouter.getRetCode()) {
				return new BocloudResult(true, waitJob(leaveRouter.getJobID()), "success");
			} else {
				return new BocloudResult(false, leaveRouter.getMessage());
			}
		} catch (Exception e) {
			logger.error("子网解绑异常：" + e.getMessage());
			return new BocloudResult(false, "子网解绑异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String subnetId) {
		try {
			BocloudResult detail = this.detail(subnetId);
			if (detail.isFailed()) {
				return new BocloudResult(false, "子网数据错误！");
			}
			SubnetModel convertModel = (SubnetModel) detail.getData();
			BocloudResult leaveRouter = leaveRouter(convertModel.getVpcUuid(), subnetId);
			if (leaveRouter.isFailed()) {
				return leaveRouter;
			}
			VxNetService vxNetService = new VxNetService(this.getContext());
			DeleteVxNetsInput input = new DeleteVxNetsInput();
			input.setVxNets(Arrays.asList(new String[] { subnetId }));
			DeleteVxNetsOutput deleteVxNets = vxNetService.deleteVxNets(input);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("删除失败:" + e.getMessage());
			return new BocloudResult(false, "删除失败:" + e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String subnetId) {
		try {
			VxNetService service = new VxNetService(this.getContext());
			DescribeVxNetsInput input = new DescribeVxNetsInput();
			input.setVxNetType(1);
			input.setVxNets(Arrays.asList(new String[] { subnetId }));
			DescribeVxNetsOutput describeVxNets = service.describeVxNets(input);
			List<VxNetModel> vxNetSet = describeVxNets.getVxNetSet();
			SubnetModel convertModel = convertor.convertModel(vxNetSet.get(0));
			return new BocloudResult(true, convertModel, "查询详情成功！");
		} catch (Exception e) {
			logger.error("查询详情异常：" + e.getMessage());
			return new BocloudResult(false, "查询详情异常：" + e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(SubnetModel subnet) {
		try {
			BocloudResult detail = this.detail(subnet.getSubnetUuid());
			if (detail.isFailed()) {
				return new BocloudResult(false, "子网数据错误！");
			}
			SubnetModel convertModel = (SubnetModel) detail.getData();
			if (null != convertModel.getVpcUuid()) {
				BocloudResult leaveRouter = leaveRouter(convertModel.getVpcUuid(), subnet.getSubnetUuid());
				if (leaveRouter.isFailed()) {
					return leaveRouter;
				}
			}
			VxNetService vxNetService = new VxNetService(this.getContext());
			DeleteVxNetsInput input = new DeleteVxNetsInput();
			List<String> vxNets = new ArrayList<>();
			vxNets.add(subnet.getSubnetUuid());
			input.setVxNets(vxNets);
			DeleteVxNetsOutput deleteVxNets = vxNetService.deleteVxNets(input);
			return new BocloudResult(true, "删除成功！");
		} catch (Exception e) {
			logger.error("子网删除失败:" + e.getMessage());
			return new BocloudResult(false, "子网删除失败:" + e.getMessage());
		}
	}

}
