package com.bocloud.cmp.provider.network;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.QcloudProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.qcloudconvertor.VpcConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.qingcloud.sdk.service.RouterService;
import com.qingcloud.sdk.service.RouterService.CreateRoutersInput;
import com.qingcloud.sdk.service.RouterService.CreateRoutersOutput;
import com.qingcloud.sdk.service.RouterService.DeleteRoutersInput;
import com.qingcloud.sdk.service.RouterService.DeleteRoutersOutput;
import com.qingcloud.sdk.service.RouterService.DescribeRoutersInput;
import com.qingcloud.sdk.service.RouterService.DescribeRoutersOutput;
import com.qingcloud.sdk.service.RouterService.ModifyRouterAttributesInput;
import com.qingcloud.sdk.service.RouterService.ModifyRouterAttributesOutput;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.RouterModel;

/**
 * vpc
 * 
 * @时间 2019年9月24日
 */
@Provider("bocloud.cmp.qcloud")
public class QcloudVpcProvider extends QcloudProvider implements VpcProvider {

    private static Logger logger = LoggerFactory.getLogger(QcloudVpcProvider.class);

    private BeanConvertor<VpcModel, Types.RouterModel> convertor = new VpcConvertor();

    public QcloudVpcProvider(Butler butler) {
        super(butler);
    }

    @Override
    public boolean accessable() {
        return super.accessable();
    }

    @Override
    public BocloudResult create(VpcModel model) {
        try {
            RouterService service = new RouterService(this.getContext());
            CreateRoutersInput input = new CreateRoutersInput();
            input.setRouterName(model.getName());
            input.setRouterType(1);
            input.setVpcNetwork(model.getCidr());
            CreateRoutersOutput createRouters = service.createRouters(input);
            createRouters.getJobID();
            List<String> routers = createRouters.getRouters();
            if (null == routers || routers.size() == 0) {
                return new BocloudResult(false, createRouters.getMessage());
            }
            VpcModel vpcDetail = getVpcDetail(routers.get(0));
            return new BocloudResult(vpcDetail != null, vpcDetail, "创建成功！");
        } catch (Exception e) {
            logger.error("删除失败：" + e.getMessage());
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }

    }

    private VpcModel getVpcDetail(String vpcId) {
        VpcModel model = null;
        try {
            RouterService service = new RouterService(this.getContext());
            DescribeRoutersInput descirbInput = new DescribeRoutersInput();
            descirbInput.setRouters(Arrays.asList(new String[] { vpcId }));
            Long startTime = System.currentTimeMillis();
            boolean flag = true;
            while (flag) {
                DescribeRoutersOutput describeRouters = service.describeRouters(descirbInput);
                List<RouterModel> routerSet = describeRouters.getRouterSet();
                if (null == routerSet || routerSet.size() == 0) {
                    flag = false;
                    break;
                }
                model = convertor.convertModel(routerSet.get(0));
                if ("active".equalsIgnoreCase(model.getStatus())) {
                    flag = true;
                    break;
                }
                Long endTime = System.currentTimeMillis();
                // 等待10分钟
                if (endTime - startTime > 10 * 60 * 1000) {
                    flag = false;
                    break;
                }
            }
        } catch (Exception e) {
            logger.error("查询vpc异常：" + e.getMessage());
        }
        return model;
    }

    @Override
    public BocloudResult remove(String vpcId) {
        try {
            RouterService service = new RouterService(this.getContext());
            DescribeRoutersInput describeRoutersInput = new DescribeRoutersInput();
            describeRoutersInput.setRouters(Arrays.asList(new String[] { vpcId }));
            DescribeRoutersOutput describeRouters = service.describeRouters(describeRoutersInput);
            List<RouterModel> routerSet = describeRouters.getRouterSet();
            if (null == routerSet || routerSet.size() <= 0) {
                return new BocloudResult(true, "删除成功！");
            }
            DeleteRoutersInput input = new DeleteRoutersInput();
            input.setRouters(Arrays.asList(new String[] { vpcId }));
            DeleteRoutersOutput deleteRouters = service.deleteRouters(input);
            if (0 == deleteRouters.getRetCode()) {
                if (!this.waitJob(deleteRouters.getJobID())) {
                    return new BocloudResult(false, "删除失败");
                } else {
                    return new BocloudResult(true, "删除成功！");
                }
            } else {
                return new BocloudResult(false, deleteRouters.getMessage());
            }
        } catch (Exception e) {
            logger.error("删除失败：" + e.getMessage());
            return new BocloudResult(false, "删除失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        try {
            RouterService service = new RouterService(this.getContext());
            DescribeRoutersInput input = new DescribeRoutersInput();
            DescribeRoutersOutput describeRouters = service.describeRouters(input);
            List<RouterModel> routerSet = describeRouters.getRouterSet();
            if (null == routerSet) {
                routerSet = new ArrayList<Types.RouterModel>();
            }
            return new BocloudResult(true, convertor.convertList(routerSet), "查询成功！");
        } catch (Exception e) {
            logger.error("查询失败：" + e.getMessage());
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String vpcId, String regionId) {
        try {
            RouterService service = new RouterService(this.getContext());
            DescribeRoutersInput input = new DescribeRoutersInput();
            input.setRouters(Arrays.asList(new String[] { vpcId }));
            DescribeRoutersOutput describeRouters = service.describeRouters(input);
            List<RouterModel> routerSet = describeRouters.getRouterSet();
            return new BocloudResult(true, convertor.convertList(routerSet), "查询成功！");
        } catch (Exception e) {
            logger.error("查询失败：" + e.getMessage());
            return new BocloudResult(false, "查询失败：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult update(VpcModel model) {
        try {
            RouterService service = new RouterService(this.getContext());
            ModifyRouterAttributesInput input = new ModifyRouterAttributesInput();
            input.setRouter(model.getVpcId());
            if (StringUtils.isNotEmpty(model.getName())) {
                input.setRouterName(model.getName());
            }
            if (StringUtils.isNotEmpty(model.getRemark())) {
                input.setRouterName(model.getRemark());
            }
            ModifyRouterAttributesOutput modifyRouterAttributes = service.modifyRouterAttributes(input);
            return new BocloudResult(true, modifyRouterAttributes.getMessage());
        } catch (Exception e) {
            logger.error("修改失败：" + e.getMessage());
            return new BocloudResult(false, "修改失败：" + e.getMessage());
        }
    }

}
