package com.bocloud.cmp.provider.qcloudconvertor;

import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.EIPResourceModel;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 弹性IP转换
 *
 * @author wangyu
 * @时间 2019年10月24日
 */
public class FloatingIpConvertor implements BeanConvertor<FloatingIpModel, Types.EIPModel> {

	private static Logger logger = LoggerFactory.getLogger(FloatingIpConvertor.class);

	@Override
	public List<FloatingIpModel> convertList(List<? extends Types.EIPModel> objects) {
		List<FloatingIpModel> models = new ArrayList<FloatingIpModel>();
		for (Types.EIPModel object : objects) {
			String status = object.getStatus();
			if (!StringUtils.isEmpty(status) && !"CEASED".equalsIgnoreCase(status)) {
				models.add(convertModel(object));
			}
		}
		return models;
	}

	@Override
	public FloatingIpModel convertModel(Types.EIPModel eipModel) {
		FloatingIpModel model = new FloatingIpModel();
		if (null != eipModel) {
			model.setBandwidth(eipModel.getBandwidth() + "");
			model.setFloatingIpAddress(eipModel.getEIPAddr());
			model.setFloatingIpUuid(eipModel.getEIPID());
			model.setName(eipModel.getEIPName());
			
			EIPResourceModel resource = eipModel.getResource();
			if (null != resource) {
				model.setInstanceId(resource.getResourceID());
			}
			
			String status = eipModel.getStatus();
			if ("ASSOCIATED".equalsIgnoreCase(status)) {
				model.setStatus("BIND");
			} else if("AVAILABLE".equalsIgnoreCase(status)) {
				model.setStatus("UNBIND");
			} else {
				model.setStatus(eipModel.getStatus().toUpperCase());
			}
			model.setNeedIcp(eipModel.getNeedICP());
			String createdTime = eipModel.getCreateTime();
			try {
				Date date = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z");
				model.setGmtCreate(date);
			} catch (Exception e) {
				logger.error("时间转换Error : " + e);
			}
		}
		return model;
	}
}
