package com.bocloud.cmp.provider.qcloudconvertor;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.qingcloud.sdk.service.Types;

/**
 * 镜像转换器
 *
 * @author wangyu
 * @时间 2019年10月24日
 */
public class ImageConvertor implements BeanConvertor<ImageModel, Types.ImageModel> {

    @Override
    public List<ImageModel> convertList(List<? extends Types.ImageModel> objects) {
        List<ImageModel> imageModels = new ArrayList<>();
        for (Types.ImageModel object : objects) {
            if ("available".equalsIgnoreCase(object.getStatus())) {
                imageModels.add(convertModel(object));
            }
        }
        return imageModels;
    }

    @Override
    public ImageModel convertModel(Types.ImageModel image) {
        ImageModel model = new ImageModel();
        model.setImageUuid(image.getImageID());
        model.setOsVersion(image.getImageID());
        model.setOsName(image.getOSFamily() == null ? "" : image.getOSFamily().toUpperCase());
        model.setOsType(image.getOSFamily() == null ? "" : image.getOSFamily().toUpperCase());
        String platform = image.getPlatform();
        if (null != platform && "linux".equals(platform)) {
            model.setOsCategory("Linux");
        } else {
            model.setOsCategory("Windows");
        }
        String visibility = image.getVisibility();
        if (!StringUtils.isEmpty(visibility)) {
            model.setImageType(visibility.toUpperCase());
        }
        String description = image.getDescription();
        model.setRemark(description);
        String imageName = image.getImageName();
        if (null != description && description.contains("IPv6")) {
            imageName = imageName + "_IPv6";
        }
        model.setName(imageName);
        model.setSize(Double.valueOf(image.getSize()));
        model.setMinDisk(Long.valueOf(image.getSize()));
        model.setArchitecture(image.getArchitecture());
        String status = image.getStatus();
        if("available".equalsIgnoreCase(status)) {
            model.setStatus("ACTIVE");
        } else {
            model.setStatus(status);
        }
        return model;
    }
}
