package com.bocloud.cmp.provider.qcloudconvertor;

import com.bocloud.cmp.model.SecurityGroupRuleModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.qingcloud.sdk.service.Types;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SecurityGroupRuleConvertor implements BeanConvertor<SecurityGroupRuleModel, Types.SecurityGroupRuleModel> {

    @Override
    public List<SecurityGroupRuleModel> convertList(List<? extends Types.SecurityGroupRuleModel> objects) {
        List<SecurityGroupRuleModel> ruleModels = new ArrayList<>();
        for (Types.SecurityGroupRuleModel object : objects) {
            ruleModels.add(convertModel(object));
        }
        return ruleModels;
    }

    private String[] protocols = new String[] { "ssh", "http", "https", "ftp", "remote", "openvpn", "pptp", "l2tp",
            "tcp", "udp" };

    @Override
    public SecurityGroupRuleModel convertModel(Types.SecurityGroupRuleModel info) {
        SecurityGroupRuleModel model = new SecurityGroupRuleModel();
        if (null != info) {
            String protocol = info.getProtocol() == null ? "" : info.getProtocol().toUpperCase();
            model.setProtocol(protocol);
            model.setRuleUuid(info.getSecurityGroupRuleID());
            model.setName(info.getSecurityGroupRuleName());
            model.setGroupUuid(info.getSecurityGroupID());
            model.setAction(info.getAction() == null ? null : info.getAction().toUpperCase());
            // ingress 0 egress 1
            Integer direction = info.getDirection();
            if (0 == direction) {
                model.setDirection("ingress");
            }
            if (1 == direction) {
                model.setDirection("egress");
            }
            if (Arrays.asList(protocols).contains(protocol.toLowerCase())) {
                model.setRemoteIpPrefix(info.getVal3());
                model.setPortMax(info.getVal2() == null ? null : Integer.valueOf(info.getVal2()));
                model.setPortMin(info.getVal1() == null ? null : Integer.valueOf(info.getVal1()));
            }
            model.setVar1(info.getVal1());
            model.setVar2(info.getVal2());
            model.setVar3(info.getVal3());
        }
        return model;
    }
}
