
package com.bocloud.cmp.provider.qcloudconvertor;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.EIPModel;
import com.qingcloud.sdk.service.Types.ExtraModel;

/**
 * 云主机转换器
 *
 * @author wangyu
 * @时间 2019年10月24日
 */
public class ServerConvertor implements BeanConvertor<ServerModel, Types.InstanceModel> {

    private static Logger logger = LoggerFactory.getLogger(ServerConvertor.class);

    @Override
    public List<ServerModel> convertList(List<? extends Types.InstanceModel> objects) {
        List<ServerModel> serverModels = new ArrayList<>();
        for (Types.InstanceModel object : objects) {
            // 页面点击删除之后的状态
            if (!"ceased".equalsIgnoreCase(object.getStatus()) && !"terminated".equalsIgnoreCase(object.getStatus())) {
                serverModels.add(convertModel(object));
            }
        }
        return serverModels;
    }

    @Override
    public ServerModel convertModel(Types.InstanceModel server) {
        if (null == server) {
            return null;
        }
        ServerModel model = new ServerModel();
        model.setFlavorUuid(server.getInstanceType());
        String createdTime = server.getCreateTime();
        try {
            Date date = DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z");
            model.setGmtVmCreate(date);
        } catch (Exception e) {
            logger.error("时间转换Error : " + e);
        }
        Types.ImageModel imageModel = server.getImage();
        if (null != imageModel) {
            model.setImageUuid(imageModel.getImageID());
        }
        model.setInstanceId(server.getInstanceID());
        model.setName(server.getInstanceName());
        Types.SecurityGroupModel securityGroupModel = server.getSecurityGroup();
        // 服务器只有一个安全组
        if (null != securityGroupModel) {
            model.setGroups(securityGroupModel.getSecurityGroupID());
        }
        // [{"subnetId":"vxnet-6ffbekm","address":"","networkName":"subnetbbb","mac":"52:54:9e:3c:cd:d5","subnetName":"subnetbbb"}]
        List<Types.NICVxNetModel> vxNetModels = server.getVxNets();
        if (!ListTool.isEmpty(vxNetModels)) {
            String managerIp = null;
            String subnetUuid = null;
            List<JSONObject> privateIps = new ArrayList<>();
            for (Types.NICVxNetModel netModel : vxNetModels) {
                if (!StringUtils.isEmpty(netModel.getPrivateIP()) && StringUtils.isEmpty(managerIp)) {
                    managerIp = netModel.getPrivateIP();
                }
                JSONObject object = new JSONObject();
                object.put("address", netModel.getPrivateIP());
                object.put("mac", netModel.getNICID());
                object.put("networkName", netModel.getVxNetName());
                object.put("subnetId", netModel.getVxNetID());
                object.put("subnetName", netModel.getVxNetName());
                subnetUuid = netModel.getVxNetID();
                privateIps.add(object);
            }
            model.setManagerIp(managerIp);
            model.setSubnetUuid(subnetUuid);
            model.setPrivateIps(JSONArray.toJSONString(privateIps));
        }
        model.setCpu(server.getVCPUsCurrent());
        model.setMemory(BigDecimal.valueOf(server.getMemoryCurrent() / 1024));
        // disk显示系统盘的大小
        ExtraModel extra = server.getExtra();
        if (null != extra) {
            model.setDisk(extra.getOSDiskSize());
        }
        model.setStatus(convert(server.getStatus()));
        List<String> keypairIds = server.getKeyPairIDs();
        if (!ListTool.isEmpty(keypairIds)) {
            model.setKeypairName(JSONObject.toJSONString(keypairIds));
        }
        EIPModel eipModel = server.getEIP();
        if (null != eipModel) {
            List<String> publicIp = new ArrayList<>();
            publicIp.add(eipModel.getEIPAddr());
            model.setPublicIps(JSON.toJSONString(publicIp));
        }
        return model;
    }

    private String convert(String status) {
        if (status.equalsIgnoreCase("running")) {
            return BocloudStatus.Server.RUNNING.name();
        } else if (status.equalsIgnoreCase("stopped")) {
            return BocloudStatus.Server.STOPPED.name();
        } else if (status.equalsIgnoreCase("pending")) {
            return BocloudStatus.Server.BUILDING.name();
        } else {
            return BocloudStatus.Server.EXCEPTION.name();
        }
    }
}
