package com.bocloud.cmp.provider.qcloudconvertor;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.common.QcloudLocationProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.qingcloud.sdk.service.Types.RouterModel;
import com.qingcloud.sdk.service.Types.VxNetModel;
import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * subnet
 * 
 * @时间 2019年10月24日
 */
public class SubnetConvertor implements BeanConvertor<SubnetModel, VxNetModel> {

    private static Logger logger = LoggerFactory.getLogger(QcloudLocationProvider.class);

    @Override
    public List<SubnetModel> convertList(List<? extends VxNetModel> objects) {
        List<SubnetModel> models = new ArrayList<SubnetModel>();
        for (VxNetModel object : objects) {
            RouterModel router = object.getRouter();
            if (null != router) {
                models.add(convertModel(object));
            }
        }
        return models;
    }

    @Override
    public SubnetModel convertModel(VxNetModel info) {
        SubnetModel model = new SubnetModel();
        if (null != info) {
            model.setSubnetUuid(info.getVxNetID());
            model.setName(info.getVxNetName());
            model.setVpcUuid(info.getVpcRouterID());
            RouterModel router = info.getRouter();
            if (null != router) {
                model.setCidr(router.getIPNetwork());
                model.setIpPools(
                        JSON.toJSONString(new String[] { router.getDYNIPStart() + "-" + router.getDYNIPEnd() }));
                model.setVpcName(router.getRouterName());
                model.setGateway(router.getManagerIP());
            }
            String createdTime = info.getCreateTime();
            if (null != createdTime) {
                try {
                    model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z"));
                } catch (Exception e) {
                    logger.error("时间转换Error : " + e);
                }
            }
        }
        return model;
    }
}
