
package com.bocloud.cmp.provider.qcloudconvertor;

import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.qingcloud.sdk.service.Types;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class VolumeConvertor implements BeanConvertor<VolumeModel, Types.VolumeModel> {

    private static Logger logger = LoggerFactory.getLogger(VolumeConvertor.class);

    @Override
    public List<VolumeModel> convertList(List<? extends Types.VolumeModel> objects) {
        List<VolumeModel> volumeModels = new ArrayList<>();
        for (Types.VolumeModel object : objects) {
            if (!"CEASED".equalsIgnoreCase(object.getStatus()) && !"deleted".equalsIgnoreCase(object.getStatus())) {
                volumeModels.add(convertModel(object));
            }
        }
        return volumeModels;
    }

    @Override
    public VolumeModel convertModel(Types.VolumeModel disk) {
        VolumeModel model = new VolumeModel();
        if (disk != null) {
            model.setVolumeUuid(disk.getVolumeID());
            model.setName(disk.getVolumeName());
            model.setSize(disk.getSize());
            if (null != disk.getVolumeType()) {
                model.setType(String.valueOf(disk.getVolumeType()));
            }
            String diskState = disk.getStatus();
            if ("in-use".equalsIgnoreCase(diskState)) {
                model.setStatus("IN_USE");
            } else if ("available".equalsIgnoreCase(diskState)) {
                model.setStatus("AVAILABLE");
            } else {
                model.setStatus(diskState);
            }
            Types.InstanceModel instanceModel = disk.getInstance();
            if (null != instanceModel) {
                model.setInstanceId(instanceModel.getInstanceID());
                model.setDevice(instanceModel.getDevice());
                model.setDeviceName(instanceModel.getDevice());
            }
            if (null != disk.getVolumeType()) {
                model.setVolumeType(String.valueOf(disk.getVolumeType()));
            }
            model.setRemark(disk.getDescription());
            String createdTime = disk.getCreateTime();
            if (null != createdTime) {
                try {
                    model.setCreateTime(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z"));
                } catch (Exception e) {
                    logger.error("时间转换Error : " + e);
                }
            }
            model.setVolumeType(disk.getVolumeType() + "");
            model.setCategory(getVolumeType(disk.getVolumeType()));
            model.setBootVolume(false);
            model.setDeleteWithInstance(false);
        }
        return model;
    }

    private String getVolumeType(Integer typeNum) {
        String volumeType = null;
        switch (typeNum) {
        case 100:
            volumeType = "基础型";
            break;
        case 6:
        case 2:
            volumeType = "容量型";
            break;
        case 5:
            volumeType = "企业级分布式 SAN (NeonSAN)";
            break;
        case 200:
            volumeType = "SSD企业级";
            break;
        default:
            break;
        }
        return volumeType;
    }

}
