
package com.bocloud.cmp.provider.qcloudconvertor;

import java.util.ArrayList;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.VpcModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.qingcloud.sdk.service.Types;
import com.qingcloud.sdk.service.Types.EIPModel;

/**
 * VPC转换器
 *
 * @author wangyu
 * @时间 2019年10月24日
 */

public class VpcConvertor implements BeanConvertor<VpcModel, Types.RouterModel> {

    private static Logger logger = LoggerFactory.getLogger(VpcConvertor.class);

    @Override
    public List<VpcModel> convertList(List<? extends Types.RouterModel> objects) {
        List<VpcModel> vpcModels = new ArrayList<>();
        for (Types.RouterModel object : objects) {
            if ("ACTIVE".equalsIgnoreCase(object.getStatus())) {
                vpcModels.add(convertModel(object));
            }
        }
        return vpcModels;
    }

    @Override
    public VpcModel convertModel(Types.RouterModel vpc) {
        VpcModel model = new VpcModel();
        model.setName(vpc.getRouterName());
        model.setStatus(vpc.getStatus().toUpperCase());
        model.setRemark(vpc.getDescription());
        model.setVpcId(vpc.getRouterID());
        model.setSecurityGroupId(vpc.getSecurityGroupID());
        model.setPrivateIp(vpc.getPrivateIP());
        model.setCidr(vpc.getVpcNetwork());
        EIPModel eip = vpc.getEIP();
        if (null != eip) {
            model.setEip(JSON.toJSONString(eip));
        }
        String createdTime = vpc.getCreateTime();
        if (null != createdTime) {
            try {
                model.setGmtCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", createdTime, "Z"));
            } catch (Exception e) {
                logger.error("时间转换Error : " + e);
            }
        }
        return model;
    }
}
