package com.bocloud.cmp.provider.common;

/**
 * 字节工具类
 *
 * @author Shawpin Shi
 * @since 2017/9/29
 */
public class Bytes {
    private final static char[] HEX_ARRAY = "0123456789abcdef".toCharArray();
    private final static int BYTE_ARRAY_MAX_LEN = Integer.MAX_VALUE >> 1;

    /**
     * 将字节数组转化成16进制的字符串
     * <p>
     * 参考资料：https://stackoverflow.com/questions/9655181/how-to-convert-a-byte-array-to-a-hex-string-in-java
     * <p/>
     *
     * @author Shawpin Shi
     * @since 2017/9/29
     */
    public static String toHexString(byte[] bytes) {
        int len = bytes.length;
        if (len > BYTE_ARRAY_MAX_LEN) {
            throw new RuntimeException("字节数组的长度不能超过" + BYTE_ARRAY_MAX_LEN);
        }

        int newLen = len << 1;// len * 2
        char[] hexChars = new char[newLen];

        int v, index;

        for (int i = 0; i < len; i++) {
            v = bytes[i] & 0xFF; // 保留低8位，高24位置0
            index = i << 1;// i * 2
            hexChars[index] = HEX_ARRAY[v >>> 4];
            hexChars[index + 1] = HEX_ARRAY[v & 0x0F];
        }

        return new String(hexChars);
    }

    /**
     * 16进制字符串还原为字节数组
     *
     * @author Shawpin Shi
     * @since 2021/12/20
     */
    public static byte[] hexStringToByteArray(String hexString) {
        int len = hexString.length();

        int newLen = len >> 1;// len / 2
        byte[] data = new byte[newLen];

        int index;
        for (int i = 0; i < len; i += 2) {
            index = i >> 1;// i / 2
            data[index] = (byte) ((Character.digit(hexString.charAt(i), 16) << 4)
                    + Character.digit(hexString.charAt(i + 1), 16));
        }
        return data;
    }
}
