package com.bocloud.cmp.provider.common;

import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.Result;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.util.Asserts;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.Map;

/**
 * 深信服专用的HttpClient
 * <p>
 * 和HttpClien的差异，HttpDelete的定义不一样
 * </p>
 *
 * @author sxp
 * @since 2023/5/30
 */
public class SangforHttpClient extends HttpClient {
    private static final Logger logger = LoggerFactory.getLogger(SangforHttpClient.class);
    protected SangforHttpDelete httpDelete;

    public SangforHttpClient() {
        super();
    }

    public SangforHttpClient(Integer timeout, PostDataFormat dataFormat) {
        super(timeout, dataFormat);
    }

    @Override
    public Result delete(Map<String, Object> header, Object params, String uri) {
        Asserts.notBlank(uri, "URI IS EMPTY！");
        this.httpDelete = new SangforHttpDelete(uri);
        logger.debug("uri is = {} | params is {}", uri, params);
        this.httpDelete.setConfig(this.config);
        this.buildHeader(this.httpDelete, header);
        this.buildEntity(this.httpDelete, params);
        this.remoteInvoke(this.httpDelete);
        return this.result;
    }

    private void remoteInvoke(HttpUriRequest request) {
        try {
            this.response = this.httpClient.execute(request);
            this.handleResponse();
        } catch (IOException e) {
            logger.error("HttpClient IOException:", e);
            this.result = new Result(false, e.getMessage());
        } finally {
            this.close();
        }

    }
}
