package com.bocloud.cmp.provider.v63.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SangforModel;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.v63.compute.SangforHostProvider;
import com.bocloud.cmp.provider.v63.compute.SangforServerProvider;
import com.bocloud.cmp.provider.v63.network.SangforNetworkProvider;
import com.bocloud.cmp.provider.v63.security.SangforFirewallRuleProvider;
import com.bocloud.cmp.provider.v63.storage.SangforVolumeProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforLocationProvider extends SangforProvider implements LocationProvider {
    private Butler butler;
    public SangforLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult sync() {

        try {
            log.info("开始数据抽取");
            SangforModel sangforModel = new SangforModel();

            log.info("主机数据抽取开始...");
            syncHost(sangforModel);
            log.info("主机数据抽取结束...");

            log.info("主机数据抽取开始...");
            syncIface(sangforModel);
            log.info("主机数据抽取结束...");

            log.info("虚拟机数据抽取开始...");
            syncServer(sangforModel);
            log.info("虚拟机数据抽取结束...");

            log.info("存储数据抽取开始...");
            syncVolume(sangforModel);
            log.info("存储数据抽取结束...");

            log.info("网络数据抽取开始...");
            syncNetwork(sangforModel);
            log.info("网络数据抽取结束...");

            log.info("防火墙数据抽取开始...");
            syncFirewall(sangforModel);
            log.info("防火墙数据抽取结束...");

            log.info("虚拟机分组数据抽取开始...");
            syncServerGroup(sangforModel);
            log.info("虚拟机分组数据抽取结束...");

            return new BocloudResult(true,sangforModel,"数据抽取成功");
        } catch (Exception e) {
            log.error("数据抽取失败",e);
            return  new BocloudResult(false,"数据抽取失败");
        }
    }

    private void syncServerGroup(SangforModel sangforModel) {
        SangforServerProvider provider = new SangforServerProvider(this.butler);
        BocloudResult result = provider.listGroup();
        if (result.isSuccess() && result.getData() != null){
            List<SangForServerGroupModel> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangForServerGroupModel.class);
            sangforModel.setServerGroupModels(serverGroupModels);
        }
    }

    private void syncFirewall(SangforModel sangforModel) {
        SangforFirewallRuleProvider provider = new SangforFirewallRuleProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforFirewallRuleModel> firewallModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforFirewallRuleModel.class);
            sangforModel.setFirewallModels(firewallModelList);
        }
    }

    private void syncIface(SangforModel sangforModel) {
        SangforHostProvider provider = new SangforHostProvider(this.butler);
        BocloudResult result = provider.ifaces();
        if (result.isSuccess() && result.getData() != null){
            List<SangforIfacesModel> ifaceModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforIfacesModel.class);
            sangforModel.setIfacesModels(ifaceModelList);
        }
    }

    private void syncNetwork(SangforModel sangforModel) {
        SangforNetworkProvider provider = new SangforNetworkProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforVirtualNetworkDeviceModel> networkDeviceModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforVirtualNetworkDeviceModel.class);
            sangforModel.setVirtualNetworkDeviceModels(networkDeviceModelList);
        }
    }

    private void syncVolume(SangforModel sangforModel) {

        SangforVolumeProvider provider = new SangforVolumeProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            SangforVolumeModel volumeModel = JSON.parseObject(JSON.toJSON(result.getData()).toString(), SangforVolumeModel.class);
            sangforModel.setVolumeModels(volumeModel);
        }
    }

    private void syncServer(SangforModel sangforModel) {
        SangforServerProvider provider = new SangforServerProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforServerModel> serverModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforServerModel.class);
            sangforModel.setServerModels(serverModelList);
        }
    }

    private void syncHost(SangforModel sangforModel) {
        SangforHostProvider provider = new SangforHostProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforHostModel> hostModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforHostModel.class);
            sangforModel.setHostModels(hostModelList);
        }
    }
}
