package com.bocloud.cmp.provider.v63.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.provider.compute.HostProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforHostProvider extends SangforProvider implements HostProvider {
    public SangforHostProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Result result = this.doGet("/vapi/extjs/index/host_list", null);
            List<SangforHostModel> hosts = new ArrayList<>();
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    JSONArray data = returnData.getJSONArray("data");
                    List<SangforHostModel> hostModels = JSONObject.parseArray(data.toJSONString(), SangforHostModel.class);
                    hosts.addAll(hostModels);

                    // 宿主机详情
                    for (SangforHostModel host : hosts) {
                        String hostId = host.getId();
                        Result infoResult = this.doGet("/vapi/extjs/nodes/"+ hostId +"/info", null);
                        if (infoResult.isSuccess() && !StringUtils.isEmpty(infoResult.getMessage())) {
                            JSONObject returnInfoData = JSONObject.parseObject(infoResult.getMessage());
                            if (null != returnInfoData && returnInfoData.getBoolean("success")) {
                                JSONObject info = returnInfoData.getJSONObject("data");
                                host.setDetail(info.toString());
                            }
                        }
                    }
                    return new BocloudResult(true, hosts, "获取主机列表成功");
                }
            }
            log.error(result.getMessage());
            return new BocloudResult(false, "获取主机列表失败");
        } catch (Exception e) {
            log.error("获取主机列表失败", e);
            return new BocloudResult(false, "获取主机列表失败");
        }
    }
    public BocloudResult<List<SangforIfacesModel>>  ifaces () {
        List<SangforIfacesModel> sangforIfacesModels = new ArrayList<>();
        Result result = this.doGet("/vapi/json/cluster/network/ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null != returnData && returnData.getBoolean("success")) {
                JSONArray data = returnData.getJSONArray("data");
                for (int i = 0; i <data.size(); i++) {
                    JSONObject nodeObj = JSONObject.parseObject(data.get(i).toString());
                    String node_id = nodeObj.getString("node_id");
                    String node_name = nodeObj.getString("node_name");
                    String node_status = nodeObj.getString("node_status");
                    Integer channel_protection = nodeObj.getInteger("channel_protection");

                    JSONArray ethObj = nodeObj.getJSONArray("data");
                    List<SangforIfacesModel> models = JSONObject.parseArray(ethObj.toJSONString(), SangforIfacesModel.class);
                    models.forEach(e ->{
                        e.setNode_id(node_id);
                        e.setNode_name(node_name);
                        e.setNode_status(node_status);
                        e.setChannel_protection(channel_protection);
                    });
                    sangforIfacesModels.addAll(models);
                }
                return new BocloudResult<>(true,sangforIfacesModels,"获取物理网络成功");
            }
        }
        return new BocloudResult<>(true,sangforIfacesModels,"获取物理网络失败");
    }
    /**
     * 网口-管理通信口
     * @return
     */
    public BocloudResult<List<SangforIfacesModel>>  ifacesMgmt (){

        Result result = this.doGet("/vapi/json/cluster/network/mgmt-ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData || !returnData.getBoolean("success")) {
                return new BocloudResult<>(false,"失败");
            }
            JSONArray data = returnData.getJSONArray("data");
            String text = data.toJSONString();
            List<SangforIfacesModel> sangforIfacesModels = JSONObject.parseArray(text, SangforIfacesModel.class);
            return new BocloudResult<>(true,sangforIfacesModels,"成功");
        }
        return new BocloudResult<>(false,"失败");
    }

    /**
     * 网口-数据通信口
     * @return
     */
    public BocloudResult<List<SangforIfacesModel>> ifacesVxlan (){

        Result result = this.doGet("/vapi/json/cluster/network/vxlan-ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData || !returnData.getBoolean("success")) {
                return new BocloudResult<>(false,"失败");
            }
            JSONArray data = returnData.getJSONArray("data");
            String text = data.toJSONString();
            List<SangforIfacesModel> sangforIfacesModels = JSONObject.parseArray(text, SangforIfacesModel.class);
            return new BocloudResult<>(true,sangforIfacesModels,"成功");
        }
        return new BocloudResult<>(false,"失败");
    }

    /**
     * 网口-业务通信口
     * @return
     */
    public BocloudResult<List<SangforIfacesModel>> ifacesBusiness (){

        Result result = this.doGet("/vapi/json/cluster/network/business-ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData || !returnData.getBoolean("success")) {
                return new BocloudResult<>(false,"失败");
            }
            JSONArray data = returnData.getJSONArray("data");
            String text = data.toJSONString();
            List<SangforIfacesModel> sangforIfacesModels = JSONObject.parseArray(text, SangforIfacesModel.class);
            return new BocloudResult<>(true,sangforIfacesModels,"成功");
        }
        return new BocloudResult<>(false,"失败");
    }

    /**
     * 网口-存储通信口
     * @return
     */
    public BocloudResult<List<SangforVsModel>> ifacesVs (){

        Result result = this.doGet("/vapi/json/cluster/network/vs-ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData || !returnData.getBoolean("success")) {
                return new BocloudResult<>(false,"失败");
            }
            JSONArray data = returnData.getJSONArray("data");
            String text = data.toJSONString();
            List<SangforVsModel> sangforVsModels = JSONObject.parseArray(text, SangforVsModel.class);
            return new BocloudResult<>(true,sangforVsModels,"成功");
        }
        return new BocloudResult<>(false,"失败");
    }

    /**
     * 网口-网络流量控制
     * @return
     */
    public BocloudResult<List<SangforQosModel>> ifacesQos (){

        Result result = this.doGet("/vapi/json/cluster/network/qos-ifaces", null);
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData || !returnData.getBoolean("success")) {
                return new BocloudResult<>(false,"失败");
            }
            JSONArray data = returnData.getJSONArray("data");
            String text = data.toJSONString();
            List<SangforQosModel> sangforQosModel = JSONObject.parseArray(text, SangforQosModel.class);
            return new BocloudResult<>(true,sangforQosModel,"成功");
        }
        return new BocloudResult<>(false,"失败");
    }
}
