package com.bocloud.cmp.provider.v63.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforServerProvider extends SangforProvider implements ServerProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforServerProvider.class);

    public SangforServerProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list(String instanceName) {
        try {
            List<SangforServerModel> servers = new ArrayList<>();
            // 获取所有虚拟机的ip地址
            JSONObject ipData = null;
            Result ipResult = this.doGet("/vapi/json/cluster/get_vms_ip_list", null);
            if (ipResult.isSuccess() && !StringUtils.isEmpty(ipResult.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(ipResult.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    ipData = returnData.getJSONObject("data");
                }
            } else {
                logger.warn(ipResult.getMessage());
                return new BocloudResult(false, servers, "获取虚拟机IP列表失败:" + ipResult.getMessage());
            }
            if (ipData == null) {
                ipData = new JSONObject();
            }

            Set<String> set = ipData.keySet();
            HashMap<String, String> ipMap = new HashMap<>();
            for (String hostId : set) {
                JSONObject serverIps = ipData.getJSONObject(hostId);
                for (String serverIp : serverIps.keySet()) {
                    JSONObject serverIpJO = serverIps.getJSONObject(serverIp);
                    ipMap.put(serverIp, serverIpJO.toJSONString());
                }
            }

            // 获取所有虚拟机
            HashMap<String, Object> params = new HashMap<>();
            params.put("desc", 1);
            params.put("group_type", "group");
            if (instanceName != null && !"".equals(instanceName)) {
                params.put("sname", instanceName);
            }
            Result result = this.doGet("/vapi/extjs/cluster/vms", params);

            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    JSONArray data = returnData.getJSONArray("data");
                    for (int i = 0; i < data.size(); i++) {
                        JSONObject jsonObject = JSONObject.parseObject(data.get(i).toString());
                        List<SangforServerModel> serverList = JSONObject.parseArray(jsonObject.getString("data"), SangforServerModel.class);
                        servers.addAll(serverList);
                    }
                    for (SangforServerModel server : servers) {
                        String vmid = server.getVmid();
                        // 设置ip
                        String privateIps = ipMap.get(vmid);
                        String managerIp = null;
                        server.setPrivateIps(privateIps);
                        if (StringUtils.isNotEmpty(privateIps)) {
                            JSONObject jsonObject = JSONObject.parseObject(privateIps);
                            Boolean flag = false;
                            for (Object ipObj : jsonObject.values()) {
                                JSONObject ips = JSONObject.parseObject(ipObj.toString());
                                for (Map.Entry<String, Object> entry : ips.entrySet()) {
                                    String key = entry.getKey();
                                    Object value = entry.getValue();
                                    if (Integer.parseInt(value.toString()) == 1) {
                                        managerIp = key;
                                        flag = true;
                                        break;
                                    }
                                }
                                if (flag) break;
                            }
                        }
                        server.setManagerIp(managerIp);
                        // 虚拟机详情
                        Result infoResult = this.doGet("/vapi/extjs/cluster/vm/" + vmid + "/info", null);
                        if (infoResult.isSuccess() && !StringUtils.isEmpty(infoResult.getMessage())) {
                            JSONObject returnInfoData = JSONObject.parseObject(infoResult.getMessage());
                            if (null != returnInfoData && returnInfoData.getBoolean("success")) {
                                JSONObject info = returnInfoData.getJSONObject("data");
                                JSONObject memStatus = info.getJSONObject("mem_status");
                                Long memTotal = memStatus.getLong("total");
                                BigDecimal mem = BigDecimal.valueOf(memTotal).divide(BigDecimal.valueOf(1073741824), 2, BigDecimal.ROUND_HALF_UP);
                                // 设置内存大小， byte -> GB
                                server.setMemoryTotal(mem);
                                info.remove("mem_sheet");
                                info.remove("net_sheet");
                                info.remove("cpu_sheet");
                                info.remove("disk_sheet");
                                server.setDetail(info.toString());
                            }
                        }

                        // 虚拟机操作日志
//                        JSONObject logParams = new JSONObject();
//                        logParams.put("start",0);
//                        logParams.put("limit",1000);
//                        logParams.put("vmid",vmid);
//                        Result logResult = this.doGet("/vapi/extjs/log/all?start=0&limit=1000&vmid=2526945914924", logParams);
//                        if (logResult.isSuccess() && !StringUtils.isEmpty(logResult.getMessage())) {
//                            JSONObject logReturnData = JSONObject.parseObject(logResult.getMessage());
//                            if (null != logReturnData && logReturnData.getBoolean("success")) {
//                                List<SangforServerLogModel> serverLogModels = JSONObject.parseArray(logReturnData.getString("data"), SangforServerLogModel.class);
//                                if (!serverLogModels.isEmpty()){
//                                    log.info(String.format("%s虚拟机存在%s条日志信息",server.getName(),serverLogModels.size()));
//                                }
//                                server.setLogModels(serverLogModels);
//                            }
//                        }

                        // 虚拟机快照
                        Result snapshotResult = this.doGet("/vapi/json/vapimain/vm/" + vmid + "/snapshots", null);
                        if (snapshotResult.isSuccess() && !StringUtils.isEmpty(snapshotResult.getMessage())) {
                            JSONObject snapshotReturnData = JSONObject.parseObject(snapshotResult.getMessage());
                            if (null != snapshotReturnData && snapshotReturnData.getBoolean("success")) {
                                JSONObject snapshotData = snapshotReturnData.getJSONObject("data");
                                String snaps = snapshotData.getJSONObject("snapshot").getString("snaps");
                                List<SangforSnapshotModel> snapshotModels = JSONObject.parseArray(snaps, SangforSnapshotModel.class);
                                if (!snapshotModels.isEmpty()) {
                                    log.info(String.format("%s虚拟机存在%s个快照", server.getName(), snapshotModels.size()));
                                }
                                server.setSnapshotModels(snapshotModels);
                            }
                        }
                    }
                    return new BocloudResult(true, servers, "获取虚拟机列表成功");
                }
            }
            log.info(result.getMessage());
            return new BocloudResult(false, "获取虚拟机列表失败");
        } catch (Exception e) {
            log.error("获取虚拟机列表失败", e);
            return new BocloudResult(false, "获取虚拟机列表失败");
        }
    }

    @Override
    public BocloudResult list() {
        return this.list(null);
    }

    /**
     * 克隆虚拟机
     *
     * @param model
     * @return
     */
    @Override
    public BocloudResult cloneVm(SangforServerModel model) {
        model.setTargetNode("cluster");
        Assert.notNull(model, "创建(克隆)虚拟机参数为空");
        JSONObject params = new JSONObject();
        Assert.notNull(model.getVmid(), "克隆虚拟机id为空");
        Assert.notNull(model.getName(), "新建虚拟机名字为空");
        Assert.notNull(model.getStorage(), "存储为空");
//        Assert.notNull(model.getTargetNode(),"目标节点为空");
        // 必选参数
        String vmid = model.getVmid();
        params.put("clonename", model.getName());
        params.put("storage", model.getStorage());
        params.put("targetnode", "cluster");
        // 可选参数
        params.put("clone_amount", 1);
        params.put("clone_share_disk", 0);
        params.put("start", model.getStart() ? 1 : 0);
        params.put("ha_enable", model.getHa() ? 1 : 0);
        if (!"default".equals(model.getGroup())){
            params.put("group",model.getGroup());
        }
        params.put("clone_type", model.getCloneType());

        Map<String, Object> netConfig = new HashMap<>();
        netConfig.put("interface_type", model.getInterfaceType());
        netConfig.put("interface_id", "net0");
        netConfig.put("mac", model.getMac().toLowerCase());
        JSONObject link_info = new JSONObject();
        link_info.put("bvs_id", model.getBvsId());  // 网卡设备的uuid
        String portUuid = model.getPortUuid();
        if (StringUtils.isEmpty(portUuid)) {
            portUuid = "12345678";
        }
        link_info.put("port", portUuid);
        netConfig.put("link_info", link_info);
        if (StringUtils.isNotEmpty(model.getIp())) {
            Map<String, Object> ipConfig = new HashMap<>();
            ipConfig.put("ip", model.getIp());
            ipConfig.put("mask", model.getMask());
            ipConfig.put("gateway", model.getGateway());
            String primaryDns = model.getPrimaryDns();
            if(StringUtils.isNotEmpty(primaryDns)){
                ipConfig.put("primary_dns", primaryDns);
            }
            String secondaryDns = model.getSecondaryDns();
            if(StringUtils.isNotEmpty(secondaryDns)){
                ipConfig.put("secondary_dns", secondaryDns);
            }
            netConfig.put("ip_info", ipConfig);
        }
        JSONArray netConfigs = new JSONArray();
        netConfigs.add(netConfig);
        params.put("net_config", netConfigs);

        params.put("hostname", model.getHostname());
        params.put("disable_net", model.getDisableNet() ? 1 : 0);
        if (StringUtils.isNotEmpty(model.getStoragePolicy())) {
            // 如果为虚拟机存储，存储策略不为空；如果是其他存储，存储策略为空
            params.put("storage_policy", model.getStoragePolicy());
            JSONArray disk_policies = new JSONArray();
            JSONObject disk_policy = new JSONObject();
            disk_policy.put("ds", "ide0");
            disk_policy.put("policy_id", model.getStoragePolicy());
            disk_policies.add(disk_policy);
            params.put("disk_policies", disk_policies);
        }
        params.put("always_try_host", model.getAlwaysTryHost());

        params.put("remark", model.getRemark());
        log.info("源主机id为:{}", vmid);
        log.info("clone params: {}", params);
        Result result = this.doPost(String.format("/vapi/json/vapimain/vm/%s/vm_clone", vmid), params, "application/json", PostDataFormat.RAW);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "创建(克隆)虚拟机失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "创建(克隆)虚拟机成功");
        } else {
            return new BocloudResult(false, "创建(克隆)虚拟机失败，失败原因：" + result.getMessage());
        }

    }

    /**
     * 创建全新的虚拟机
     *
     * @param model
     * @return 创建成功返回虚拟机的id
     */
    @Override
    public BocloudResult createVm(SangforServerModel model) {
        Assert.notNull(model, "创建(全新)虚拟机参数为空");

        Assert.notNull(model.getCfgstorage(), "虚拟机配置文件存放存储位置为空");
        Assert.notNull(model.getMemory(), "虚拟机分配内存为空");
        Assert.notNull(model.getName(), "虚拟机名字为空");
//        Assert.notNull(model.getNode(),"虚拟机所在主机");

        JSONObject params = new JSONObject();
        /**
         * 基础信息
         */
        //名称
        params.put("name", model.getName());
        // 分组
        if (!"default".equals(model.getDir())){
            params.put("dir", model.getDir());
        }
        // 标签
        params.put("label", model.getLabel());
        // 故障迁移
        params.put("ha_enable", model.getHa_enable() != null && model.getHa_enable());
        // 存储位置
        params.put("cfgstorage", model.getCfgstorage());
        // 存储策略
        if (StringUtils.isNotEmpty(model.getStoragePolicy())) {
            params.put("storage_policy", model.getStoragePolicy());
            params.put("disk_policies", String.format("{\"ide0\":\"%s\"}", model.getStoragePolicy()));
        }

        // 运行位置
        params.put("always_try_host", model.getAlwaysTryHost());
        // 操作系统
        params.put("ostype", model.getOstype());
        if (!StringUtils.isEmpty(model.getOsname())) {
            params.put("osname", model.getOsname());
        }
        // 重要虚拟机
        params.put("schedopt", model.getSchedopt() != null && model.getSchedopt());

        /**
         * 硬件配置
         */
        //---------------cpu---------------
        // cpu类型。kvm32|kvm64(,\+vmx)|core2duo|host 注: 默认使用 core2duo
        params.put("cpu", model.getCpu() == null ? "core2duo" : model.getCpu());
        // 虚拟机插槽数
        params.put("sockets", model.getSockets());
        // 每个插槽的核数
        params.put("cores", model.getCores());
        params.put("numa", model.getNuma() != null && model.getNuma());
        params.put("if_numa_on", model.getIf_numa_on() != null && model.getIf_numa_on());

        //---------------内存---------------
        params.put("memory", model.getMemory() * 1024);
        params.put("hugepage", model.getHugepage() != null && model.getHugepage());

        //---------------磁盘---------------
        String storageId = model.getCfgstorage();
        Long disk_total = model.getDisk_total().longValue();
        String preallocate = model.getPreallocate();
        // b2114b88_vs_vol_rep3:60,format=qcow2,cache=directsync,preallocate=metadata,forecast=disable,cache_size=256
        params.put("ide0", String.format("%s:%s,format=%s,cache=directsync,preallocate=%s,forecast=disable,cache_size=256", storageId, disk_total, "qcow2", preallocate));
        // 块设备准虚拟化。[yes,no]，默认 no
        params.put("use_vblk", model.getUse_vblk() == null ? "no" : model.getUse_vblk());

        //---------------光驱---------------
        String iso = model.getIso() == null ? "none" : model.getIso();
        params.put("ide2", String.format("%s,media=cdrom", iso));

        //---------------网卡---------------
        String networkcardType = model.getNetworkcardType();
        String mac = model.getMac();
        String bridge = model.getBridge();
        String bridgename = model.getBridgename();
        String portUuid = model.getPortUuid();
        if (StringUtils.isEmpty(portUuid)) {
            portUuid = "12345678";
        }
        // e1000=FE:FC:FE:C6:E2:C6,bridge=bvs2da525b,bridgename=物理出口2,port=12345678,connect=on
        params.put("net0", String.format("%s=%s,bridge=%s,bridgename=%s,port=%s,connect=%s", networkcardType, mac, bridge, bridgename, portUuid, "on"));
        // 是否为新虚拟机指定了ip信息。 例如：1,0 表明第一个网卡有指定ip 信息，对应信息在ip_net0 字段；第二个网卡没有指定 ip 信息
        params.put("set_net", model.getSet_net() == null ? "0" : model.getSet_net());
        String ip = model.getIp() == null ? "" : model.getIp();
        String mask = model.getMask() == null ? "" : model.getMask();
        String gateway = model.getGateway() == null ? "" : model.getGateway();
        String dns1 = model.getDns1() == null ? "" : model.getDns1();
        String dns2 = model.getDns2() == null ? "" : model.getDns2();
        // net0=FE:FC:FE:C6:E2:C6,ip=10.101.253.252,mask=255.255.255.0,gateway=10.101.253.252,dns1=114.114.114.114,dns2=8.8.8.8
        params.put("ip_net0", String.format("net0=%s,ip=%s,mask=%s,gateway=%s,dns1=%s,dns2=%s", mac, ip, mask, gateway, dns1, dns2));
        String ipv6 = model.getIpv6() == null ? "" : model.getIpv6();
        String ipv6Gateway = model.getIpv6Gateway() == null ? "" : model.getIpv6Gateway();
        String ipv6Dns1 = model.getIpv6Dns1() == null ? "" : model.getIpv6Dns1();
        String ipv6Dns2 = model.getIpv6Dns2() == null ? "" : model.getDns2();
        String prefixlen = model.getPrefixlen() == null ? "" : model.getPrefixlen();
        // net0=FE:FC:FE:C6:E2:C6,ipv6=,prefixlen=,gateway=,dns1=,dns2=
        params.put("ip6_net0", String.format("net0=%s,ipv6=%s,prefixlen=%s,gateway=%s,dns1=%s,dns2=%s", mac, ipv6, prefixlen, ipv6Gateway, ipv6Dns1, ipv6Dns2));

        //---------------其他硬件---------------
        // 虚拟显卡类型 默认为cirrus
        params.put("graphic_type", model.getGraphic_type() == null ? "cirrus" : model.getGraphic_type());
        // 鼠标类型。 usb 和 ps/2 两种
        params.put("mouse_type", model.getMousetype() == null ? "usb" : model.getMousetype());
        // BIOS停留时间
        params.put("splash_time", model.getSplash_time() == null ? 0 : model.getSplash_time());
        // 使用 uefi bios
        params.put("uefi_bios", model.getUefi_bios() == null ? "" : model.getUefi_bios());

        /**
         * 高级配置
         */
        // hostname
        params.put("hostname", model.getHostname() == null ? "" : model.getHostname());
        if ("".equals(params.getString("hostname"))) {
            params.remove("hostname");
        }
        // 虚拟机异常时重启
        params.put("abnormal_recovery", model.getAbnormal_recovery() != null && model.getAbnormal_recovery());
        // 主机启动时，是否自动运行此虚拟机
        params.put("onboot", model.getOnboot() != null && model.getOnboot());
//        // 虚拟机log编号
//        params.put("logo",model.getLogo());
        // 虚拟机启动方式（引导顺序）：硬盘(d)、光驱(c)、网络(n) cdn | cnd | dcn | dnc | ncd | ndc | dn | nd | nd | c | d | n 启动方式可以按参数组合使用
        params.put("boot", model.getBoot() == null ? "d" : model.getBoot());
        // 启动盘 ide[n]
        params.put("bootdisk", model.getBootdisk() == null ? "ide0" : model.getBootdisk());
        // 热插拔
        params.put("hotplug", model.getHotplug() == null ? 0 : model.getHotplug());
        // 虚拟机生命周期。-1 为不启用，设置为日期为使用该虚拟机直至指定日期。 例如：2022-11-01
        params.put("lifespan", model.getLifespan() == null ? -1 : model.getLifespan());
        // 是否启动uuid.
        params.put("use_uuid", model.getUse_uuid() != null && model.getUse_uuid());
        // 是否启用磁盘加密
        params.put("encrypted", model.getEncrypted() != null && model.getEncrypted());
        /**
         * 其他或未知
         */
        // 禁止使用 kvm_clock时钟
        params.put("no_kvm_clock", model.getNo_kvm_clock() != null && model.getNo_kvm_clock());
        // 是否禁用 Pause-Loop Exiting
        params.put("disable_ple", model.getDisable_ple() != null && model.getDisable_ple());
        // 是否备份过滤页面文件
        params.put("file_filter", model.getFile_filter() != null && model.getFile_filter());
        // 是否自动启动
        params.put("autostart", model.getAutostart() != null && model.getAutostart());


        // 未知。默认传值
        params.put("high_performance_disks", "ide0");
        params.put("balloon_memory", model.getBalloon_memory() != null && model.getBalloon_memory());
        params.put("host_gpu", false);
        params.put("invtsc", false);

        for (Map.Entry<String, Object> entry : params.entrySet()) {
            String key = entry.getKey();
            Object obj = entry.getValue();
            if (obj instanceof Boolean && params.getBoolean(key)) {
                params.put(key, 1);
            }
            if (obj instanceof Boolean && !params.getBoolean(key)) {
                params.put(key, 0);
            }
        }
        log.info("全新虚拟机创建params参数:{}", params.toJSONString());
        Result result = this.doPost("/vapi/json/cluster/qemu/create", params);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "创建(全新)虚拟机失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }

            String str = returnData.getString("data");
            str = str.substring(str.indexOf("新建虚拟机:"), str.length());
            str = str.split(":")[1];
            return new BocloudResult(true, str, "创建(全新)虚拟机成功");
        }
        return new BocloudResult(false, "创建(全新)虚拟机失败：" + result.getMessage());
    }

    @Override
    public BocloudResult start(String instanceId) {
        Assert.notNull(instanceId, "虚拟机id为空");

        Result result = this.doPost(String.format("/vapi/extjs/cluster/vm/%s/status/start", instanceId), null);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机开机失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "虚拟机开机成功");
        }
        return new BocloudResult(false, "虚拟机开机失败");
    }

    @Override
    public BocloudResult stop(String instanceId) {
        Assert.notNull(instanceId, "虚拟机id为空");

        Result result = this.doPost(String.format("/vapi/extjs/cluster/vm/%s/status/shutdown", instanceId), null);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机关机失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "虚拟机关机成功");
        }
        return new BocloudResult(false, "虚拟机关机失败");
    }


    @Override
    public BocloudResult reboot(String instanceId) {
        Assert.notNull(instanceId, "虚拟机id为空");

        Result result = this.doPost(String.format("/vapi/extjs/cluster/vm/%s/status/reset", instanceId), null);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机重启失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "虚拟机重启成功");
        }
        return new BocloudResult(false, "虚拟机重启失败");
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        Assert.notNull(instanceId, "虚拟机id为空");

        Result result = this.doPost(String.format("/vapi/extjs/cluster/vm/%s/status/suspend", instanceId), null);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机挂起失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "虚拟机挂起成功");
        }
        return new BocloudResult(false, "虚拟机挂起失败");
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        Assert.notNull(instanceId, "虚拟机id为空");

        JSONObject params = new JSONObject();
        params.put("vms", instanceId);
        params.put("diskremain", 1);
        Result result = this.doPost("/vapi/json/cluster/vm/delete_vms", params);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机删除失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            return new BocloudResult(true, returnData.getString("data"), "虚拟机删除成功");
        }
        return new BocloudResult(false, "虚拟机删除失败");
    }


    @Override
    public BocloudResult detail(String instanceId) {
        Result infoResult = this.doGet("/vapi/extjs/cluster/vm/" + instanceId + "/info", null);
        if (infoResult.isSuccess() && !StringUtils.isEmpty(infoResult.getMessage())) {
            JSONObject returnInfoData = JSONObject.parseObject(infoResult.getMessage());
            if (null != returnInfoData && returnInfoData.getBoolean("success")) {
                JSONObject info = returnInfoData.getJSONObject("data");
                info.put("instanceId", info.getString("vmid"));
                return new BocloudResult(true, info, "虚拟机详情查询成功");
            }
            log.error(infoResult.getMessage());
        }
        log.error(infoResult.getMessage());
        return new BocloudResult(false, "虚拟机详情查询失败");
    }

//    @Override
//    public BocloudResult modify(ServerModel server) {
//        Assert.notNull(server, "虚拟机参数为空");
//        // 不能为空
//        Assert.notNull(server.getInstanceId(), "虚拟机id为空");
//        Assert.notNull(server.getName(), "虚拟机名称为空");
//        Assert.notNull(server.getMemory(), "虚拟机内存为空");
//        Assert.notNull(server.getCfgstorage(), "虚拟机存储位置为空");
//        // 可以为空
//
//
//        JSONObject params = new JSONObject();
//        String instanceId = server.getInstanceId();
//        params.put("name", server.getName());
//        params.put("memory", server.getMemory().intValue());
//        params.put("cfgstorage", server.getCfgstorage());
//        Result result = this.doPut(String.format("/vapi/extjs/cluster/vm/%s/edit", instanceId), params);
//        log.info(result.toString());
//        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
//            JSONObject returnData = JSONObject.parseObject(result.getMessage());
//            if (null == returnData) {
//                return new BocloudResult(false, "虚拟机编辑失败");
//            }
//            if (!returnData.getBoolean("success")) {
//                return new BocloudResult(false, returnData.getString("message"));
//            }
//            return new BocloudResult(true, returnData.getString("data"), "虚拟机编辑成功");
//        }
//        return new BocloudResult(false, "虚拟机编辑失败");
//
//    }

    @Override
    public BocloudResult listGroup() {
        try {
            List<SangForServerGroupModel> groupModelList;
            Result result = this.doGet("/vapi/extjs/cluster/group_list", null);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage());
                JSONArray groupArr = data.getJSONArray("data");
                groupModelList = groupArr.toJavaList(SangForServerGroupModel.class);
                return new BocloudResult(true, groupModelList, "查询虚拟机分组列表成功");
            }
        } catch (Exception e) {
            log.error("查询虚拟机分组列表失败", e);
        }
        return BocloudResult.FAILED("查询虚拟机分组列表失败");
    }


    @Override
    public BocloudResult listLabel() {
        try {
            List<SangForServerLabelModel> labelModelList;
            Result result = this.doGet("/vapi/json/cluster/label_list", null);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage());
                JSONArray labelArray = data.getJSONArray("data");
                labelModelList = labelArray.toJavaList(SangForServerLabelModel.class);
                return new BocloudResult(true, labelModelList, "查询虚拟机标签列表成功");
            }
        } catch (Exception e) {
            log.error("查询虚拟机标签列表失败", e);
        }
        return BocloudResult.FAILED("查询虚拟机标签列表失败");
    }

    @Override
    public BocloudResult listCon(String storage) {
        try {
            List<SangforConModel> conModelList;
            JSONObject params = new JSONObject();
            params.put("storage", storage);
            params.put("nodeid", "cluster");

            Result result = this.doGet("/vapi/extjs/network/vm_connection", params);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage());
                JSONArray labelArray = data.getJSONArray("data");
                conModelList = labelArray.toJavaList(SangforConModel.class);
                return new BocloudResult(true, conModelList, "查询网络设备连接列表成功");
            }
        } catch (Exception e) {
            log.error("查询网络设备连接列表失败", e);
        }
        return BocloudResult.FAILED("查询网络设备连接列表失败");
    }


    @Override
    public BocloudResult listIso(String storageId) {
        try {
            List<SangforIsosModel> isoModelList;
            JSONObject params = new JSONObject();
            params.put("vm_storeid", storageId);
            params.put("filetype", "iso");
            params.put("storeids", null);
            Result result = this.doPost("/vapi/extjs/vtpstorage/load_all_iso", params);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage()).getJSONObject("data");
                JSONArray labelArray = data.getJSONArray("isos");
                isoModelList = labelArray.toJavaList(SangforIsosModel.class);
                return new BocloudResult(true, isoModelList, "获取iso列表成功");
            }
        } catch (Exception e) {
            log.error("获取iso列表失败", e);
        }
        return BocloudResult.FAILED("获取iso列表失败");
    }

    @Override
    public BocloudResult process(String upid) {
        try {
            JSONObject params = new JSONObject();
            params.put("upid", upid);
            params.put("noerr", 1);

            Result result = this.doGet("/vapi/extjs/log/process", params);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    JSONObject data = returnData.getJSONObject("data");
                    SangforTaskInfo taskInfo = JSONObject.parseObject(data.toJSONString(), SangforTaskInfo.class);
                    return new BocloudResult(true, taskInfo, "查询任务操作进度成功");
                }
            }
        } catch (Exception e) {
            log.error("查询任务操作进度失败", e);
        }
        return BocloudResult.FAILED("查询任务操作进度失败");
    }

    public BocloudResult<JSONObject> vncLogin(String instanceId) {
        try {
            Result result = this.doGet("/vapi/extjs/cluster/vm/" + instanceId + "/vnclogin", null);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                return new BocloudResult(true,"VNC登陆成功");
            }
        } catch (Exception e) {
            log.error("VNC登陆失败", e);
        }
        return BocloudResult.FAILED("VNC登陆失败");
    }
    public BocloudResult<JSONObject> vnc(String instanceId) {
        try {
            JSONObject params = new JSONObject();
            params.put("vncws", 1);
            Map<String, Object> header = this.buildHeader();
            Result result = this.doPost("/vapi/extjs/cluster/vm/" + instanceId + "/vncproxy", params);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage()).getJSONObject("data");
                data.put(CSRFPreventionToken,header.get(CSRFPreventionToken));
                data.put(Cookie,header.get(Cookie));
                data.put("vncloginUrl",buildUrl("/vapi/extjs/cluster/vm/" + instanceId + "/vnclogin"));
                return new BocloudResult(true,  data,"获取VNC端口");
            }
        } catch (Exception e) {
            log.error("获取VNC端口", e);
        }
        return BocloudResult.FAILED("获取VNC端口");
    }

    @Override
    public BocloudResult<JSONObject> config(String instanceId) {
        try {
            Assert.notNull(instanceId, "实例id不能为空");
            Result result = this.doGet("/vapi/extjs/cluster/vm/" + instanceId + "/config", null);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    JSONObject data = returnData.getJSONObject("data");
                    return new BocloudResult(true, data, "查询任务操作进度成功");
                }
            }
        } catch (Exception e) {
            log.error("查询任务操作进度失败", e);
        }
        return BocloudResult.FAILED("查询任务操作进度失败");
    }

    @Override
    public BocloudResult modify(ServerModel model) {
        String instanceId = model.getInstanceId();
        Assert.notNull(instanceId, "虚拟机id为空");

        String name = model.getName();
        // 先获取到虚拟机的详细配置信息
        BocloudResult<JSONObject> configResult = this.config(instanceId);
        if (configResult.isFailed()){
            return new BocloudResult(false,"获取config失败");
        }
        JSONObject config = configResult.getData();
        // 移除多余参数
        config.remove("cfgstorageats");
        config.remove("cfgstoragelvtype");
        config.remove("cfgstoragename");
        config.remove("cfgstoragenfstype");
        config.remove("cfgstorageshared");
        config.remove("cfgstoragestretch");
        config.remove("ctrl");
        config.remove("ip_unequal");
        config.remove("location");
        config.remove("location_config");
        // 部分参数格式需要重新处理
        JSONObject disk_policies = config.getJSONObject("disk_policies");
        if (Objects.nonNull(disk_policies)){
            Set<String> keySet_disk_policies = disk_policies.keySet();
            for (String key : keySet_disk_policies) {
                JSONObject policies = disk_policies.getJSONObject(key);
                String policiesId = policies.getString("id");
                JSONObject replace_disk_policies = new JSONObject();
                replace_disk_policies.put(key, policiesId);
                config.put("disk_policies",replace_disk_policies);
            }
        }
        String hostname = config.getString("hostname");
        if (StringUtils.isNotEmpty(hostname)) {
            config.put("hostname",Arrays.asList(StringUtils.split(hostname, ",")).get(0));
        }

        JSONObject storage_policy = config.getJSONObject("storage_policy");
        if (Objects.nonNull(storage_policy)){
            config.put("storage_policy",storage_policy.getString("id"));
        }


        //---------------名称变更---------------
        if (Objects.nonNull(name) &&  !StringUtils.equals(config.getString("name"), name)){
            config.put("name", name);
        }
        //---------------cpu变更---------------
        Integer cpu = model.getCpu();
        Integer configSockets = config.getInteger("sockets");
        // 深信服cpu核心数 = cores(核心数/槽) * sockets(槽)
        if (Objects.nonNull(cpu)){
            if (Objects.isNull(configSockets)){
                return new BocloudResult(false,"虚拟机插槽数未知");
            } else if (configSockets == 1 ){
                config.put("cores",cpu);
            } else {
                int i = cpu % configSockets;
                int cores = cpu / configSockets;
                if (i != 0){
                    return new BocloudResult(false,String.format("cpu插槽数为%s，cpu不支持变更为%s",configSockets, cpu));
                } else {
                    config.put("cores",cores);
                }
            }
        }

        //---------------内存变更---------------
        BigDecimal memory = model.getMemory();
        if(Objects.nonNull(memory)){
            int memoryIntValue = 1024 * memory.intValue();
            if (!config.getInteger("memory").equals(memoryIntValue)){
                config.put("memory", memoryIntValue);
            }
        }


        //---------------存储位置变更---------------
        String cfgstorage = model.getCfgstorage();  //存储位置变更。一般不会用到
        if (Objects.nonNull(cfgstorage) && !config.getString("cfgstorage").equals(cfgstorage)){
            return new BocloudResult(false,"暂不支持存储位置变更");
        }

        //---------------新增磁盘变更---------------
        Integer addDiskTotal = model.getAddDiskTotal();
        String preallocate = model.getPreallocate();
        if (Objects.nonNull(addDiskTotal)){
            Assert.notNull(cfgstorage,"新增磁盘cfgstorage不能为空");
            Assert.notNull(preallocate,"新增磁盘preallocate不能为空");
            int index = 0;
            while (true){
                if (!config.containsKey("ide"+ index)){
                    break;
                }
                index++;
            }
            config.put("ide"+ index, String.format("%s:%s,format=%s,cache=directsync,preallocate=%s,forecast=disable,cache_size=256", cfgstorage, addDiskTotal, "qcow2", preallocate));
        }

        JSONObject params = config;
        Result result = this.doPut(String.format("/vapi/extjs/cluster/vm/%s/edit", instanceId), params);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false, "虚拟机编辑失败");
            }
            if (!returnData.getBoolean("success")) {
                return new BocloudResult(false, returnData.getString("message"));
            }
            // 取修改后的虚拟机详情返回 sxp:2023.6.7
            BocloudResult bocloudResult = detail(instanceId);
            if (bocloudResult.isSuccess()) {
                return new BocloudResult(true, bocloudResult.getData(), "虚拟机编辑成功");
            } else {
                return new BocloudResult(true, returnData.getString("data"), "虚拟机编辑成功");
            }
        }
        return new BocloudResult(false, "虚拟机编辑失败");

    }
    public static void main(String[] args) {
        String str = "{\n" +
                " \"alwaysTryHost\": \"host-f46b8c24c9cd\",\n" +
                " \"boot\": \"cd\",\n" +
                " \"bridge\": \"bvs2da525b\",\n" +
                " \"bridgename\": \"物理出口2\",\n" +
                " \"cfgstorage\": \"b2114b88_vs_vol_rep3\",\n" +
                " \"cores\": 4,\n" +
                " \"cpu\": \"core2duo\",\n" +
                " \"dir\": \"4483cf8494e6\",\n" +
                " \"disk_total\": \"60\",\n" +
                " \"dns1\": \"8.8.8.8\",\n" +
                " \"dns2\": \"114.114.114.114\",\n" +
                " \"gateway\": \"10.10.253.1\",\n" +
                " \"graphic_type\": \"cirrus\",\n" +
                " \"hostname\": \"\",\n" +
                " \"if_numa_on\": true,\n" +
                " \"ip\": \"10.10.253.252\",\n" +
                " \"label\": \"67f9faa7872b\",\n" +
                " \"lifespan\": \"-1\",\n" +
                " \"mac\": \"FE:FC:FE:CC:27:DA\",\n" +
                " \"mask\": \"255.255.255.0\",\n" +
                " \"memory\": 8,\n" +
                " \"mousetype\": \"usb\",\n" +
                " \"name\": \"bocloud-test-webxxx\",\n" +
                " \"networkcardType\": \"rtl8139\",\n" +
                " \"numa\": true,\n" +
                " \"osname\": \"linux-centos\",\n" +
                " \"ostype\": \"l2664\",\n" +
                " \"preallocate\": \"metadata\",\n" +
                " \"set_net\": \"true\",\n" +
                " \"sockets\": 1,\n" +
                " \"splash_time\": 0,\n" +
                " \"storagePolicy\": \"6268f0bc\",\n" +
                " \"uefi_bios\": false,\n" +
                " \"use_vblk\": \"yes\"\n" +
                "}";
        str = "{\n" +
                "    \"alwaysTryHost\":\"host-f46b8c24c9cd\",\n" +
                "    \"bvsId\":\"bvs0af632d\",\n" +
                "    \"storage\":\"b2114b88_vs_vol_rep3\",\n" +
                "    \"cloneType\":\"fast\",\n" +
                "    \"disableNet\":true,\n" +
                "    \"gateway\":\"1.1.1.1\",\n" +
                "    \"group\":\"4483cf8494e6\",\n" +
                "    \"ha\":false,\n" +
                "    \"hostname\":\"hostname\",\n" +
                "    \"interfaceType\":\"rtl8139\",\n" +
                "    \"ip\":\"10.101.253.253\",\n" +
                "    \"mac\":\"FE:FC:FE:77:37:52\",\n" +
                "    \"mask\":\"255.255.255.0\",\n" +
                "    \"name\":\"bocloud-test-clone\",\n" +
                "    \"primaryDns\":\"8.8.8.8\",\n" +
                "    \"secondaryDns\":\"114.114.114.114\",\n" +
                "    \"sourceVmId\":\"2526945914924\",\n" +
                "    \"start\":false,\n" +
                "    \"storagePolicy\":\"6268f0bc\",\n" +
                "    \"vmid\":\"1769121205780\"\n" +
                "}";
        BocloudResult<JSONObject> bocloudResult = null;
        Butler butler = new Butler();
        butler.setUrl("https://10.101.253.37");
        butler.setUsername("admin");
        butler.setPassword("1qaz@WSX12");
        SangforServerProvider provider = new SangforServerProvider(butler);
        SangforServerModel serverModel = JSONObject.parseObject(str, SangforServerModel.class);
//        bocloudResult = provider.start("2307602279048");
//        bocloudResult = provider.stop("2307602279048");
//        Object obj = bocloudResult.getData();
//        String upid = obj.toString();
//        bocloudResult = provider.process("host-f46b8c24c9cd:46929:1669877446:5471931205736");
//        bocloudResult = provider.vncLogin("1778815745610");
//        bocloudResult = provider.vnc("1778815745610");
//        bocloudResult = provider.cloneVm(serverModel);
//        bocloudResult = provider.createVm(serverModel);
//        bocloudResult = provider.listLabel();
//        bocloudResult = provider.listCon("b2114b88_vs_vol_rep3");
//        bocloudResult = provider.listIso("b2114b88_vs_vol_rep3");
        ServerModel modifyModel = new ServerModel();
        modifyModel.setInstanceId("7595323042690");
        modifyModel.setName("bocloud-clone-x");
//        modifyModel.setMemory(BigDecimal.valueOf(8192L));
//        modifyModel.setCfgstorage("b2114b88_vs_vol_rep3");
        bocloudResult = provider.modify(modifyModel);

//        JSONObject jsonObject = bocloudResult.getData();
//        str = "{\"success\":1,\"data\":\"UPID:host-f46b8c24c9cd:0000D971:6C47B0C1:6363A1A7:新建虚拟机:3197933851414:admin@vtp:\"}";
//        str = str.substring(str.indexOf("新建虚拟机:"), str.length());
//        str = str.split(":")[1];
        System.out.println(str);
    }
}
