package com.bocloud.cmp.provider.v63.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.sangfor.SangforSnapshotModel;
import com.bocloud.cmp.provider.compute.SnapshotProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.List;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforSnapshotProvider extends SangforProvider implements SnapshotProvider {
    public SangforSnapshotProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list(String vmId) {
        Result snapshotResult = this.doGet("/vapi/json/vapimain/vm/" + vmId +"/snapshots", null);
        if (snapshotResult.isSuccess() && !StringUtils.isEmpty(snapshotResult.getMessage())) {
            JSONObject snapshotReturnData = JSONObject.parseObject(snapshotResult.getMessage());
            if (null != snapshotReturnData && snapshotReturnData.getBoolean("success")) {
                JSONObject snapshotData = snapshotReturnData.getJSONObject("data");
                String snaps = snapshotData.getJSONObject("snapshot").getString("snaps");
                List<SangforSnapshotModel> snapshotModels = JSONObject.parseArray(snaps, SangforSnapshotModel.class);
                return new BocloudResult(true,snapshotModels,"获取虚拟机快照列表成功");
            }
        }
        return new BocloudResult(true,"获取虚拟机快照列表失败");
    }
    @Override
    public BocloudResult create(SnapshotModel model) {
        Assert.notNull(model,"创建虚拟机快照参数为空");
        Assert.notNull(model.getInstanceId(),"虚拟机id为空");
        Assert.notNull(model.getName(),"快照名称为空");

        JSONObject params = new JSONObject();
        // 必选参数
        String vmid = model.getInstanceId();
        JSONObject snapshot = new JSONObject();
        params.put("snapshot",snapshot);
        // 可选参数
        snapshot.put("name",model.getName());
        snapshot.put("desc",model.getRemark());
        snapshot.put("snap_type","vs");
        snapshot.put("forced_create",1);
        Result result = this.doPost(String.format("/vapi/json/vapimain/vm/%s/snapshots",vmid), params, "application/json", PostDataFormat.RAW);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false,  "创建虚拟机快照失败");
            }
            if (!returnData.getBoolean("success")){
                return new BocloudResult(false,  returnData.getString("message"));
            }
            return new BocloudResult(true,  returnData.getString("data"),"创建虚拟机快照成功");
        }
        return new BocloudResult(false,  "创建虚拟机快照失败");

    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        Assert.notNull(model,"创建虚拟机快照参数为空");
        Assert.notNull(model.getSnapshotUuid(),"快照id为空");
        Assert.notNull(model.getInstanceId(),"虚拟机id为空");

        String vmid = model.getInstanceId();
        String snap_id = model.getSnapshotUuid();
        JSONObject params = new JSONObject();
        JSONObject snapshot = new JSONObject();
        params.put("snapshot",snapshot);
        snapshot.put("need_create_snap",model.getNeedCreateSnap() ? 1:0);
        snapshot.put("auto_startvm",model.getAutoStartvm() ? 1:0);

        Result result = this.doPost(String.format("/vapi/json/vapimain/vm/%s/snapshots/%s/recover",vmid,snap_id), params,"application/json", PostDataFormat.RAW);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false,  "虚拟机快照恢复失败");
            }
            if (!returnData.getBoolean("success")){
                return new BocloudResult(false,  returnData.getString("message"));
            }
            return new BocloudResult(true,  returnData.getString("data"),"虚拟机快照恢复成功");
        }
        return new BocloudResult(false,  "虚拟机快照恢复失败");
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        Assert.notNull(model,"创建虚拟机快照参数为空");
        Assert.notNull(model.getSnapshotUuid(),"快照id为空");
        Assert.notNull(model.getInstanceId(),"虚拟机id为空");

        String vmid = model.getInstanceId();
        String snap_id = model.getSnapshotUuid();
        JSONObject params = new JSONObject();
        JSONArray snap_ids = new JSONArray();
        snap_ids.add(snap_id);
        params.put("snap_ids",snap_ids);

        Result result = this.doPost(String.format("/vapi/json/vapimain/vm/%s/snapshots/delete",vmid), params,"application/json", PostDataFormat.RAW);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false,  "删除虚拟机快照失败");
            }
            if (!returnData.getBoolean("success")){
                return new BocloudResult(false,  returnData.getString("message"));
            }
            return new BocloudResult(true,  returnData.getString("data"),"删除虚拟机快照成功");
        }
        return new BocloudResult(false,  "删除虚拟机快照失败");
    }

    public static void main(String[] args) {
        BocloudResult bocloudResult = null;
        Butler butler = new Butler();
        butler.setUrl("https://10.101.253.37");
        butler.setUsername("admin");
        butler.setPassword("1qaz@WSX13");
        SangforSnapshotProvider provider = new SangforSnapshotProvider(butler);

        SnapshotModel model = new SnapshotModel();
        model.setName("20221021-test-02");
        model.setRemark("快照接口测试");
        model.setInstanceId("1769121205780");

        bocloudResult = provider.create(model);

        System.out.println("67f9faa7872b");
    }
}
