package com.bocloud.cmp.provider.v63.network;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.provider.network.NetworkProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.regex.Pattern;


@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforNetworkProvider extends SangforProvider implements NetworkProvider {
    public SangforNetworkProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list() {
        try {
            HashMap<String, Object> params = new HashMap<>();
            params.put("group_type","group");
            Result result = this.doGet("/vapi/json/network/allnet", params);
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null != returnData && returnData.getBoolean("success")) {
                    JSONArray data = returnData.getJSONArray("data");
                    List<SangforVirtualNetworkDeviceModel> virtualNetworkDeviceModels = JSONObject.parseArray(data.toJSONString(), SangforVirtualNetworkDeviceModel.class);
                    for (SangforVirtualNetworkDeviceModel mode : virtualNetworkDeviceModels) {
                        List<SangforBVSModel> bvsList = new ArrayList<>();
                        List<SangforDVSModel> dvsList = new ArrayList<>();
                        List<SangforVRouterModel> vRouterList = new ArrayList<>();
                        List<SangforNFVModel> nfvList = new ArrayList<>();
                        List<SangforBVSInfoModel> bvsInfoList = new ArrayList<>();
                        mode.setBvsModels(bvsList);
                        mode.setDvsModels(dvsList);
                        mode.setVRouterModels(vRouterList);
                        mode.setNfvModels(nfvList);
                        mode.setBvsInfoModels(bvsInfoList);
                        JSONArray vdns = mode.getData();
                        for (int i = 0; i < vdns.size(); i++) {
                            String object = vdns.get(i).toString();
                            String type = JSONObject.parseObject(object).getString("type");
                            if ("bvs".equals(type)){    // 物理出口
                                SangforBVSModel bvsModel = JSONObject.parseObject(object, SangforBVSModel.class);
                                bvsList.add(bvsModel);
                            } else if ("dvs".equals(type)){ // 分布式交换机
                                SangforDVSModel dvsModel = JSONObject.parseObject(object, SangforDVSModel.class);
                                dvsList.add(dvsModel);
                            } else if ("vrouter".equals(type)){ // 虚拟路由器
                                SangforVRouterModel vRouterModel = JSONObject.parseObject(object, SangforVRouterModel.class);
                                vRouterList.add(vRouterModel);
                            } else {    // nfv
                                SangforNFVModel nfvModel = JSONObject.parseObject(object, SangforNFVModel.class);
                                nfvList.add(nfvModel);
                            }
                        }
                        // 获取物理出口的详细配置信息（端口组）
                        for (SangforBVSModel model : bvsList) {
                            String id = model.getId();
                            params.clear();
                            params.put("id", id);
                            result = this.doGet("/vapi/extjs/network/borderswitch/info", params);
                            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                                returnData = JSONObject.parseObject(result.getMessage());
                                if (null != returnData && returnData.getBoolean("success")) {
                                    JSONObject infodata = returnData.getJSONObject("data");
                                    SangforBVSInfoModel bvsInfoModel = JSONObject.parseObject(infodata.toJSONString(), SangforBVSInfoModel.class);
                                    // status参数在不同深信服环境类型不一致，此处要兼容处理
                                    JSONArray bridgeList = infodata.getJSONArray("bridge_list");
                                    if (!ListTool.isEmpty(bridgeList)) {
                                        for (int i = 0; i < bridgeList.size(); i++) {
                                            JSONObject bridge = bridgeList.getJSONObject(i);
                                            String status = bridge.getString("status");
                                            Pattern pattern = Pattern.compile("[0-9]*");
                                            if (StringUtils.isBlank(status)) {
                                                bvsInfoModel.getBridge_list().get(i).setBridgeStatus(null);
                                                continue;
                                            }
                                            if (pattern.matcher(status).matches()) {
                                                bvsInfoModel.getBridge_list().get(i).setBridgeStatus(Integer.parseInt(status));
                                            } else if ("up".equalsIgnoreCase(status)) {
                                                bvsInfoModel.getBridge_list().get(i).setBridgeStatus(1);
                                            } else {
                                                bvsInfoModel.getBridge_list().get(i).setBridgeStatus(0);
                                            }
                                        }

                                    }
                                    bvsInfoModel.setBvsUuid(id);
                                    bvsInfoList.add(bvsInfoModel);
                                }
                            }
                        }
                    }
                    return new BocloudResult(true, virtualNetworkDeviceModels, "获取网络设备列表成功");
                }
            }
            return new BocloudResult(false, "获取网络设备列表失败");
        } catch (Exception e) {
            log.error("获取网络设备列表失败", e);
            return new BocloudResult(false, "获取网络设备列表失败");
        }
    }

    @Override
    public BocloudResult listMac() {
        try {
            Result result = this.doGet("/vapi/extjs/cluster/gen_mac_addr", null);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage());
                String mac = data.getString("data");
                return new BocloudResult(true, mac, "获取mac地址成功");
            }
        } catch (Exception e) {
            log.error("获取mac地址失败:", e.getMessage());
            return BocloudResult.FAILED("获取mac地址失败");
        }
        return BocloudResult.FAILED("获取mac地址失败");
    }


    @Override
    public BocloudResult addVlanGroup(SangforVlanGroupModel model) {
        JSONObject params = new JSONObject();
        String id = model.getId();
        String name = model.getName();
        String type = model.getType();
        Assert.notNull(id,"物理出口uuid不能为空");
        Assert.notNull(name,"名称不能为空");
        Assert.notNull(type,"类型不能为空");
        params.put("id", id);
        params.put("name", name);
        params.put("porttype", type);
        if ("trunk".equals(model.getType())){
            params.put("pvid",model.getPvid());
            params.put("vlanrange",model.getVlanrange());
        } else if ("access".equals(model.getType())){
            params.put("vlanid",model.getVlanid());
        } else {
            Assert.isTrue(false,"类型未知");
        }
        Result ruleResult = this.doPost("/vapi/extjs/network/borderswitch/vlangroup/add", params);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != returnData && returnData.getBoolean("success")) {
                String data = returnData.getString("data");// 端口组uuid
                return new BocloudResult(true,data,"防火墙策略编辑成功");
            }
        }
        return new BocloudResult(true,"防火墙策略编辑失败");
    }

    @Override
    public BocloudResult editVlanGroup(SangforVlanGroupModel model) {
        JSONObject params = new JSONObject();
        String id = model.getId();
        String groupid = model.getGroupid();
        String name = model.getName();
        String type = model.getType();
        Assert.notNull(id,"物理出口uuid不能为空");
        Assert.notNull(groupid,"端口组uuid不能为空");
        Assert.notNull(name,"名称不能为空");
        Assert.notNull(type,"类型不能为空");
        params.put("id", id);
        params.put("groupid", groupid);
        params.put("name", name);
        params.put("porttype", type);
        if ("trunk".equals(model.getType())){
            params.put("pvid",model.getPvid());
            params.put("vlanrange",model.getVlanrange());
        } else if ("access".equals(model.getType())){
            params.put("vlanid",model.getVlanid());
        } else {
            Assert.isTrue(false,"类型未知");
        }
        Result ruleResult = this.doPut("/vapi/extjs/network/borderswitch/vlangroup/edit", params);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != returnData && returnData.getBoolean("success")) {
                return new BocloudResult(true,"防火墙策略编辑成功");
            }
        }
        return new BocloudResult(true,"防火墙策略编辑失败");
    }

    public static void main(String[] args) {
        Butler butler = new Butler();
        butler.setUrl("https://10.101.253.37");
        butler.setUsername("admin");
        butler.setPassword("1qaz@WSX12");
        SangforNetworkProvider provider = new SangforNetworkProvider(butler);
        BocloudResult bocloudResult = provider.list();
        System.out.println("--");
    }
}
