package com.bocloud.cmp.provider.v63.security;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.SangforFirewallRuleModel;
import com.bocloud.cmp.model.sangfor.SangforIpgpModel;
import com.bocloud.cmp.model.sangfor.SangforServiceModel;
import com.bocloud.cmp.provider.security.FirewallRuleProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforFirewallRuleProvider extends SangforProvider implements FirewallRuleProvider {

    public SangforFirewallRuleProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        Result ruleResult = this.doGet("/vapi/json/network/dfwall/get_acl", null);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject ruleReturnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != ruleReturnData && ruleReturnData.getBoolean("success")) {
                List<SangforFirewallRuleModel> firewallRuleModels = new ArrayList<>();
                JSONArray data = ruleReturnData.getJSONArray("data");
                for (int i = 0; i < data.size(); i++) {
                    JSONObject firewallRule = data.getJSONObject(i);
                    SangforFirewallRuleModel model = JSONObject.parseObject(firewallRule.toJSONString(), SangforFirewallRuleModel.class);
                    model.setIsDefault(firewallRule.getBoolean("default"));
                    firewallRuleModels.add(model);
                }
                return new BocloudResult(true,firewallRuleModels,"获取防火墙列表成功");
            }
        }
        return new BocloudResult(true,"获取防火墙列表失败");
    }


    @Override
    public BocloudResult create(SangforFirewallRuleModel model) {
        Assert.notNull(model,"编辑策略参数为空");
        Assert.notNull(model.getName(),"策略名称为空");
        Assert.notNull(model.getEnable(),"策略状态为空");
        Assert.notNull(model.getAction(),"策略动作为空");
        Assert.notNull(model.getSrc_type(),"策略源类型为空");
        Assert.notNull(model.getDst_type(),"策略目的类型为空");
        Assert.notNull(model.getSrv(),"策略服务列表为空");

        JSONObject params = new JSONObject();
        // 必选参数
        params.put("name",model.getName());
        params.put("enable",model.getEnable());
        params.put("action",model.getAction());
        params.put("src_type",model.getSrc_type());
        params.put("dst_type",model.getDst_type());
        params.put("srv",model.getSrv() == null ? null: model.getSrv().toJSONString());
        // 可选参数
        params.put("domain_id",model.getDomain_id());
        params.put("srv_flag",model.getSrvFlag());
        params.put("srcip",model.getSrcip() == null ? null:model.getSrcip().toJSONString());
        params.put("dstip",model.getDstip() == null ? null:model.getDstip().toJSONString());
        params.put("src_vm",model.getSrc_vm());
        params.put("dst_vm",model.getDst_vm());
        params.put("srcipg",model.getSrcipg());
        params.put("dstipg",model.getDstipg());

        Result ruleResult = this.doPost("/vapi/json/network/dfwall/add_acl", params);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject ruleReturnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != ruleReturnData && ruleReturnData.getBoolean("success")) {
                return new BocloudResult(true,ruleReturnData.getString("data"),"防火墙策略编辑成功");
            }
        }
        return new BocloudResult(true,"防火墙策略编辑失败");
    }

    @Override
    public BocloudResult updateRule(SangforFirewallRuleModel model) {
        Assert.notNull(model,"编辑策略参数为空");
        Assert.notNull(model.getId(),"策略id为空");
        Assert.notNull(model.getName(),"策略名称为空");
        Assert.notNull(model.getEnable(),"策略状态为空");
        Assert.notNull(model.getAction(),"策略动作为空");
        Assert.notNull(model.getSrc_type(),"策略源类型为空");
        Assert.notNull(model.getDst_type(),"策略目的类型为空");
        Assert.notNull(model.getSrv(),"策略服务列表为空");
        Assert.notNull(model.getSrvFlag(),"策略服务标识为空");

        JSONObject params = new JSONObject();
        // 必选参数
        params.put("id",model.getId());
        params.put("name",model.getName());
        params.put("enable",model.getEnable());
        params.put("action",model.getAction());
        params.put("src_type",model.getSrc_type());
        params.put("dst_type",model.getDst_type());
        params.put("srv",model.getSrv() == null ? null: model.getSrv().toJSONString());
        params.put("srv_flag",model.getSrvFlag());
        // 可选参数
        params.put("domain_id",model.getDomain_id());
        params.put("srcip",model.getSrcip() == null ? null:model.getSrcip().toJSONString());
        params.put("dstip",model.getDstip() == null ? null:model.getDstip().toJSONString());
        params.put("src_vm",model.getSrc_vm());
        params.put("dst_vm",model.getDst_vm());
        params.put("srcipg",model.getSrcipg());
        params.put("dstipg",model.getDstipg());

        Result ruleResult = this.doPut("/vapi/json/network/dfwall/edit_acl", params);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject ruleReturnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != ruleReturnData && ruleReturnData.getBoolean("success")) {
                return new BocloudResult(true,"防火墙策略编辑成功");
            }
        }
        return new BocloudResult(true,"防火墙策略编辑失败");
    }

    @Override
    public BocloudResult remove(String firewallRuleId) {
        Assert.notNull(firewallRuleId,"策略id为空");
        JSONObject params = new JSONObject();
        params.put("acl_ids",firewallRuleId);
        Result ruleResult = this.doPost("/vapi/json/network/dfwall/delete_acl", params);
        if (ruleResult.isSuccess() && !StringUtils.isEmpty(ruleResult.getMessage())) {
            JSONObject ruleReturnData = JSONObject.parseObject(ruleResult.getMessage());
            if (null != ruleReturnData && ruleReturnData.getBoolean("success")) {
                return new BocloudResult(true,"防火墙策略删除成功");
            }
        }
        return new BocloudResult(true,"防火墙策略删除失败");
    }

    @Override
    public BocloudResult getIpgps(String query) {
        JSONObject params = new JSONObject();
        if (query !=null){
            params.put("query",query);
        }
        Result ipgpsResult = this.doGet("/vapi/json/network/dfwall/get_ipgps", params);
        if (ipgpsResult.isSuccess() && !StringUtils.isEmpty(ipgpsResult.getMessage())) {
            JSONObject ipgpsReturnData = JSONObject.parseObject(ipgpsResult.getMessage());
            if (null != ipgpsReturnData && ipgpsReturnData.getBoolean("success")) {
                JSONArray data = ipgpsReturnData.getJSONArray("data");
                List<SangforIpgpModel> ipgpModels = JSONObject.parseArray(data.toJSONString(), SangforIpgpModel.class);
                return new BocloudResult(true,ipgpModels,"获取IP组成功");
            }
        }
        return new BocloudResult(true,"获取IP组失败");
    }

    @Override
    public BocloudResult serviceDefault(String query_srv) {
        JSONObject params = new JSONObject();
        if (query_srv !=null){
            params.put("query_srv",query_srv);
        }
        Result ipgpsResult = this.doGet("/vapi/extjs/network/service/default", params);
        if (ipgpsResult.isSuccess() && !StringUtils.isEmpty(ipgpsResult.getMessage())) {
            JSONObject ipgpsReturnData = JSONObject.parseObject(ipgpsResult.getMessage());
            if (null != ipgpsReturnData && ipgpsReturnData.getBoolean("success")) {
                JSONArray data = ipgpsReturnData.getJSONArray("data");
                List<SangforServiceModel> serviceModels = JSONObject.parseArray(data.toJSONString(), SangforServiceModel.class);
                return new BocloudResult(true,serviceModels,"获取内置服务成功");
            }
        }
        return new BocloudResult(true,"获取内置服务失败");
    }

    @Override
    public BocloudResult serviceCustomdefault() {
        Result ipgpsResult = this.doGet("/vapi/json/network/dfwall/service/custom", null);
        if (ipgpsResult.isSuccess() && !StringUtils.isEmpty(ipgpsResult.getMessage())) {
            JSONObject ipgpsReturnData = JSONObject.parseObject(ipgpsResult.getMessage());
            if (null != ipgpsReturnData && ipgpsReturnData.getBoolean("success")) {
                JSONArray data = ipgpsReturnData.getJSONArray("data");
                List<SangforServiceModel> serviceModels = JSONObject.parseArray(data.toJSONString(), SangforServiceModel.class);
                return new BocloudResult(true,serviceModels,"获取内置服务成功");
            }
        }
        return new BocloudResult(true,"获取内置服务失败");
    }
}
