package com.bocloud.cmp.provider.v63.storage;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.provider.storage.VolumeProvider;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Provider("bocloud.cmp.sangfor.v63")
public class SangforVolumeProvider extends SangforProvider implements VolumeProvider {
    public SangforVolumeProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            SangforVolumeModel volumeModel = null;
            HashMap<String, Object> params = new HashMap<>();
            params.put("lt256g_not_display",1);
            //其他存储列表
            Result result = this.doGet("/vapi/extjs/vtpstorage/storelist", params);
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null == returnData || !returnData.getBoolean("success")) {
                    return new BocloudResult(false, "获取存储列表失败");
                }
                volumeModel = JSONObject.parseObject(returnData.getJSONObject("data").toJSONString(), SangforVolumeModel.class);
                log.info("其他存储列表数据：{}",JSONObject.toJSONString(volumeModel));
            }


            // 虚拟存储卷列表
            result = this.doGet("/vapi/json/vs/vs_status/vs_get_volume_list", null);
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null == returnData || !returnData.getBoolean("success")) {
                    return new BocloudResult(false, "获取存储列表失败");
                }
                JSONArray data = returnData.getJSONObject("data").getJSONArray("volumes");
                List<SangforVirtualStorageModel> virtualStorage = JSONObject.parseArray(data.toJSONString(), SangforVirtualStorageModel.class);
                volumeModel.setVirtual_storage(virtualStorage);
            }

            // 获取存储存储列表（用于创建虚拟机选择存储）
            result = this.doGet("/vapi/extjs/vtpstorage/storagetree", params);
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null == returnData || !returnData.getBoolean("success")) {
                    return new BocloudResult(false, "获取存储列表失败");
                }
                JSONArray data = returnData.getJSONArray("data");
                String text = data.toJSONString();
                List<SangforStorageModel> storage = JSONObject.parseArray(text, SangforStorageModel.class);
                volumeModel.setStorage(storage);
            }

            // 获取实体机系统盘
            result = this.doGet("/vapi/json/vapimain/host/sysdisk", null);
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(result.getMessage());
                if (null == returnData || !returnData.getBoolean("success")) {
                    return new BocloudResult(false, "获取存储列表失败");
                }
                JSONArray data = returnData.getJSONArray("data");
                String text = data.toJSONString();
                List<SangforSysDiskModel> sysDisk = JSONObject.parseArray(text, SangforSysDiskModel.class);
                volumeModel.setSysDisk(sysDisk);
            }

            return new BocloudResult(true, volumeModel, "获取存储列表成功");
        } catch (Exception e) {
            log.error("获取存储列表失败", e);
            return new BocloudResult(false, "获取存储列表失败");
        }
    }


    @Override
    public BocloudResult create(VolumeModel volume) {

        JSONObject params = new JSONObject();
        Result result = this.doPost("/vapi/json/vtpstorage/addlv" , params);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false,  "扩容失败失败");
            }
            if (!returnData.getBoolean("success")){
                return new BocloudResult(false,  returnData.getString("message"));
            }
            return new BocloudResult(true,  returnData.getString("data"),"扩容失败成功");
        }
        return new BocloudResult(false,  "扩容失败失败");
    }

    /**
     * 扩容
     * @param instanceId
     * @param newSize
     * @return
     */
    @Override
    public BocloudResult extend(String instanceId, Integer newSize) {
        Assert.notNull(instanceId,"存储id为空");
        Result result = this.doPost("/vapi/json/vtpstorage/lv/" + instanceId, null);
        log.info(result.toString());
        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            JSONObject returnData = JSONObject.parseObject(result.getMessage());
            if (null == returnData) {
                return new BocloudResult(false,  instanceId + "扩容失败失败");
            }
            if (!returnData.getBoolean("success")){
                return new BocloudResult(false,  returnData.getString("message"));
            }
            return new BocloudResult(true,  returnData.getString("data"),instanceId + "扩容失败成功");
        }
        return new BocloudResult(false,  instanceId + "扩容失败失败");
    }

    @Override
    public BocloudResult listPolicy(String volumeUuid) {
        try {
            List<SangForStoragePolicyModel> policyModels;
            Map<String, Object> param = new HashMap<>();
            param.put("storage_id", volumeUuid);
            Result result = this.doGet("/vapi/json/vapimain/storage_policy/policy_list", param);
            log.info(result.toString());
            if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
                JSONObject data = JSONObject.parseObject(result.getMessage());
                JSONArray groupArr = data.getJSONArray("data");
                policyModels = groupArr.toJavaList(SangForStoragePolicyModel.class);
                return new BocloudResult(true, policyModels, "查询存储策略列表成功");
            }
        } catch (Exception e) {
            log.error("查询存储策略列表失败", e);
            return BocloudResult.FAILED("查询存储策略列表失败: " + e.getMessage());
        }
        return BocloudResult.FAILED("查询存储策略列表失败");
    }

    public static void main(String[] args) {
        Butler butter  = new Butler();
        butter.setUrl("https://10.101.253.37");
        butter.setUsername("admin");
        butter.setPassword("1qaz@WSX12");
        SangforVolumeProvider provider = new SangforVolumeProvider(butter);
        BocloudResult br = provider.list();
        System.out.println(br);
    }
}
