package com.bocloud.cmp.provider.v68.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SangforModel;
import com.bocloud.cmp.model.sangfor.SangForServerGroupModel;
import com.bocloud.cmp.model.sangfor.SangforImage;
import com.bocloud.cmp.model.sangfor.SangforServerModel;
import com.bocloud.cmp.params.Sangfor.*;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.LocationProvider;
import com.bocloud.cmp.provider.v68.compute.SangforImageProvider;
import com.bocloud.cmp.provider.v68.compute.SangforServerProvider;
import com.bocloud.cmp.provider.v68.compute.SangforGroupServerProvider;
import com.bocloud.cmp.provider.v68.compute.SangforHostProvider;
import com.bocloud.cmp.provider.v68.security.SangforFileWallProvider;
import com.bocloud.cmp.provider.v68.network.SangforNetworkProvider;
import com.bocloud.cmp.provider.v68.storage.SangforStorageProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class SangforLocationProvider extends SangforProvider implements LocationProvider {
    private Butler butler;
    public SangforLocationProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult sync() {

        try {
            log.info("开始数据抽取");
            SangforModel sangforModel = new SangforModel();

            log.info("租户列表数据抽取开始....");
            syncProjects(sangforModel);
            log.info("租户列表数据抽取结束....");

            log.info("镜像数据抽取开始....");
            syncImage(sangforModel);
            log.info("镜像数据抽取结束....");

            log.info("资源池数据抽取开始....");
            syncAz(sangforModel);
            log.info("资源池数据抽取结束....");

            log.info("物理主机列表数据抽取开始....");
            syncPhysicalServer(sangforModel);
            log.info("物理主机列表数据抽取结束....");

            log.info("物理主机物理网络数据抽取开始...");
            syncInterfaces(sangforModel);
            log.info("物理主机物理网络数据抽取结束...");

            log.info("物理主机通信口数据抽取开始...");
            syncCommunication(sangforModel);
            log.info("物理主机通信口数据抽取结束...");

            log.info("物理主机系统盘数据抽取开始...");
            syncSystemDisk(sangforModel);
            log.info("物理主机系统盘数据抽取结束...");

            log.info("数据存储列表抽取开始...");
            syncStorage(sangforModel);
            log.info("数据存储列表抽取结束...");

            log.info("虚拟机分组数据抽取开始...");
            syncServerGroup(sangforModel);
            log.info("虚拟机分组数据抽取结束...");

            log.info("网络列表数据抽取开始...");
            syncNetWork(sangforModel);
            log.info("网络列表数据抽取结束...");

            log.info("网络列表端口组数据抽取开始...");
            syncDvVlanGroup(sangforModel);
            log.info("网络列表端口组数据抽取结束...");

            log.info("网络列表网口组数据抽取开始...");
            syncDvPhyIf(sangforModel);
            log.info("网络列表网口组数据抽取结束...");

            log.info("防火墙列表数据抽取开始...");
            syncFireWall(sangforModel);
            log.info("防火墙列表数据抽取结束...");

            log.info("虚拟机数据抽取开始...");
            syncServer(sangforModel);
            log.info("虚拟机数据抽取结束...");

            return new BocloudResult(true,sangforModel,"数据抽取成功");
        } catch (Exception e) {
            log.error("数据抽取失败",e);
            return  new BocloudResult(false,"数据抽取失败");
        }
    }

    private void syncImage(SangforModel sangforModel){
        SangforImageProvider provider = new SangforImageProvider(butler);
        Map<String, Object> params = new HashMap<>();
        BocloudResult result = provider.list(params);
        if (result.isSuccess()){
            List<SangforImage> images = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforImage.class);
            sangforModel.setScpImageModels(images);
        }
    }

    private void syncProjects(SangforModel sangforModel){
        SangforHostProvider provider = new SangforHostProvider(butler);
        BocloudResult result = provider.listProject();
        if (result.isSuccess() && result.getData() != null){
            List<SangforProjectsParams> azModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforProjectsParams.class);
            sangforModel.setProjectsParamsList(azModels);
        }
    }

    private void syncAz(SangforModel sangforModel){
        SangforHostProvider provider =new SangforHostProvider(butler);
        BocloudResult result = provider.listLocation();
        if (result.isSuccess() && result.getData() != null){
            List<SangforAzParams> azModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforAzParams.class);
            sangforModel.setAzModels(azModels);
        }
    }

    private void syncServerGroup(SangforModel sangforModel) {
        SangforGroupServerProvider provider = new SangforGroupServerProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangForServerGroupModel> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangForServerGroupModel.class);
            sangforModel.setServerGroupModels(serverGroupModels);
        }
    }
    private void syncFireWall(SangforModel sangforModel){
        SangforFileWallProvider provider = new SangforFileWallProvider(butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforFireWallParams> fireWallParams = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforFireWallParams.class);
            sangforModel.setFirewallRuleModels(fireWallParams);
        }
    }
    private void syncDvPhyIf(SangforModel sangforModel){
        SangforNetworkProvider provider = new SangforNetworkProvider(butler);
        BocloudResult result = provider.dvPhyIfList();
        if (result.isSuccess()){
            List<SangforPhyIfParams> phyIfParamsList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforPhyIfParams.class);
            sangforModel.setPhyIfParamsModel(phyIfParamsList);
        }
    }
    private void syncDvVlanGroup(SangforModel sangforModel){
        SangforNetworkProvider provider = new SangforNetworkProvider(butler);
        BocloudResult result = provider.dvVlanGroupList();
        if (result.isSuccess()){
            List<SangforVlanGroupParams> vlanGroupParams = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforVlanGroupParams.class);
            sangforModel.setVlanGroupParams(vlanGroupParams);
        }
    }
    private void syncCommunication(SangforModel sangforModel){
        SangforHostProvider provider = new SangforHostProvider(butler);
        BocloudResult result = provider.sangforCommunication();
        if (result.isSuccess() && result.getData() != null){
            List<SangforInterfacesParams> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforInterfacesParams.class);
            sangforModel.setCommunicationModels(serverGroupModels);
        }
    }
    private void syncNetWork (SangforModel sangforModel){
        SangforNetworkProvider provider = new SangforNetworkProvider(butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforScpNetWorkParams> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforScpNetWorkParams.class);
            sangforModel.setScpNetWorkParams(serverGroupModels);
        }
    }

    private void syncInterfaces(SangforModel sangforModel){
        SangforHostProvider provider = new SangforHostProvider(this.butler);
        BocloudResult result = provider.sangforInterfacesList();
        if (result.isSuccess() && result.getData() !=null){
            List<SangforInterfacesParams> interfacesList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforInterfacesParams.class);
            sangforModel.setInterfacesModels(interfacesList);
        }
    }
    private void syncStorage(SangforModel sangforModel){
        SangforStorageProvider provider = new SangforStorageProvider(butler);
        BocloudResult result = provider.listServerStorage();
        if (result.isSuccess() && result.getData() !=null){
            List<SangforStorageParams> storageParamsList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforStorageParams.class);
            sangforModel.setStorageParamsModels(storageParamsList);
        }
    }

    private void syncSystemDisk(SangforModel sangforModel){
        SangforStorageProvider provider = new SangforStorageProvider(this.butler);
        BocloudResult result = provider.sangforSystemDisk();
        if (result.isSuccess() && result.getData() !=null){
            List<SangforStorageParams> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforStorageParams.class);
            sangforModel.setStorageModels(serverGroupModels);
        }
    }

    private void syncPhysicalServer(SangforModel sangforModel){
        SangforHostProvider provider = new SangforHostProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() !=null){
            List<SangforHostParams> serverGroupModels = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforHostParams.class);
            sangforModel.setScpHostModels(serverGroupModels);
        }
    }

    private void syncServer(SangforModel sangforModel) {
        SangforServerProvider provider = new SangforServerProvider(this.butler);
        BocloudResult result = provider.list();
        if (result.isSuccess() && result.getData() != null){
            List<SangforServerModel> serverModelList = JSON.parseArray(JSON.toJSON(result.getData()).toString(), SangforServerModel.class);
            sangforModel.setServerModels(serverModelList);
        }
    }
}
