package com.bocloud.cmp.provider.v68.common;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.cmp.provider.v68.model.TaskResponseModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.concurrent.TimeUnit;

/**
 * 任务信息
 *
 * @author sxp
 * @since 2023/5/29
 */
@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class TaskProvider extends SangforProvider {

    public TaskProvider(Butler butler) {
        super(butler);
    }

    /**
     * 查询任务信息
     *
     * @author sxp
     * @since 2023/5/29
     */
    public BocloudResult detail(String taskId) {
        if (StringUtils.isEmpty(taskId)) {
            return new BocloudResult(false, "任务ID为空，查询失败");
        }

        HashMap<String, Object> params = new HashMap<>(0);
        String url = String.format("/janus/20180725/tasks/%s", taskId);

        Result result;
        try {
            result = this.doGet(url, params);
            if (result.isFailed()) {
                return new BocloudResult(false, "查询任务" + taskId + "信息失败，失败原因为：" + result.getMessage());
            }
            String message = result.getMessage();
            SangforResult sfResult = JSON.parseObject(message, SangforResult.class);
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                TaskResponseModel taskResponse = JSON.parseObject(JSON.toJSONString(data), TaskResponseModel.class);
                return new BocloudResult(true, taskResponse, "success");
            } else {
                return new BocloudResult(false, "查询任务" + taskId + "信息失败，失败原因为：" + sfResult.getMessage());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult(false, "查询任务" + taskId + "信息失败，失败原因为：" + e.getMessage());
        }
    }


    /**
     * 等待任务执行完毕后返回结果
     *
     * @author sxp
     * @since 2023/5/29
     */
    public BocloudResult waitTask(String taskId) {
        Long startTime = System.currentTimeMillis();
        Long endTime = System.currentTimeMillis();
        long costMills = TimeUnit.HOURS.toMillis(1);// 时间最长等待1小时

        TaskResponseModel taskModel;
        String status;
        Integer process = null;

        try {
            while (endTime - startTime <= costMills) {
                log.info("执行任务查询：{}", taskId);
                BocloudResult result = detail(taskId);
                if (result.isFailed()) {
                    return new BocloudResult(false, result.getMessage());
                }

                taskModel = (TaskResponseModel) result.getData();
                status = taskModel.getStatus();
                process = taskModel.getProgress();
                log.info("获取任务状态：{}，任务进度百分比：{}", status, process);

                if (TaskStatus.finish.name().equals(status)) {
                    return new BocloudResult(true, "任务执行成功");
                } else if (TaskStatus.failure.name().equals(status)) {
                    return new BocloudResult(false, "任务执行失败，" + taskModel.getDescription());
                } else if (TaskStatus.canceled.name().equals(status)) {
                    return new BocloudResult(true, "任务已取消");
                }

                if (process != null && process.intValue() == 100) {
                    break;
                }

                endTime = System.currentTimeMillis();
                TimeUnit.SECONDS.sleep(3);
            }
        } catch (Exception e) {
            log.error("查询任务信息失败", e);
            return new BocloudResult(false, "任务执行失败:" + e.getMessage());
        }

        if (process != null && process.intValue() == 100) {
            return new BocloudResult(true, "任务执行完毕，进度100");
        } else {
            return new BocloudResult(true, "任务执行完毕，进度" + process);
        }
    }


}
