package com.bocloud.cmp.provider.v68.common;

/**
 * 任务状态
 * <pre>
 * waiting 排队，对应进度为：-2
 * failure 失败，对应进度为：-1
 * doing 正在执行，对应进度：0~99
 * finish 完成，对应进度为：100
 * canceling 正在取消
 * canceled 已取消
 * </pre>
 *
 * @author sxp
 * @since 2023/5/29
 */
public enum TaskStatus {
    waiting("排队"),// 对应进度为：-2
    failure("失败"),// 对应进度为：-1
    doing("正在执行"),// 对应进度：0~99
    finish("完成"), // 对应进度为：100
    canceling("正在取消"),
    canceled("已取消");

    private String description;

    TaskStatus(String description) {
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    /**
     * 根据字符串类型的状态获取对应的枚举值
     *
     * @author sxp
     * @since 2023/5/29
     */
    public TaskStatus getByStatus(String status) {
        TaskStatus[] allValue = TaskStatus.values();
        for (TaskStatus v : allValue) {
            if (v.name().equals(status)) {
                return v;
            }
        }
        return null;
    }
}
