package com.bocloud.cmp.provider.v68.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.SangForServerGroupModel;
import com.bocloud.cmp.model.sangfor.SangforGroupModel;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.FastjsonUtils;
import com.bocloud.cmp.provider.compute.GroupServerProvider;
import com.bocloud.cmp.provider.v68.common.TaskProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class SangforGroupServerProvider extends SangforProvider implements GroupServerProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforGroupServerProvider.class);

    public SangforGroupServerProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            List<SangForServerGroupModel> allGroupList = new ArrayList<>();
            String url = "/janus/20180725/server-groups";

            Map<String, Object> params = new HashMap<>();
            params.put("cloud_type", SANGFOR_TYPE);
            params.put("page_num", 0);
            params.put("page_size", PAGE_SIZE);
            Result result = this.doGet(url, params);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject dataJson = JSON.parseObject(JSON.toJSONString(data));
                    List<SangForServerGroupModel> groups = JSON.parseArray(dataJson.getString("data"), SangForServerGroupModel.class);
                    if (groups != null) {
                        allGroupList.addAll(groups);
                    }

                    Integer totalSize = dataJson.getInteger("total_size");
                    String nextPageString = dataJson.getString("next_page_num");

                    Integer totalPage, nextPage;
                    // 下一页数，整型的字符串，为""表示当前是最后一页
                    if (StringUtils.isEmpty(nextPageString)) {
                        nextPage = 0;
                        totalPage = 0;
                    } else {
                        nextPage = dataJson.getInteger("next_page_num");
                        totalPage = totalSize / PAGE_SIZE;
                        if (totalSize % PAGE_SIZE > 0) {
                            totalPage++;
                        }
                    }

                    if (totalPage > 0) {
                        while (nextPage < totalPage) {
                            params.put("cloud_type", SANGFOR_TYPE);
                            params.put("page_num", nextPage);
                            params.put("page_size", PAGE_SIZE);
                            result = this.doGet(url, params);
                            message = result.getMessage();
                            sfResult = null;
                            if (FastjsonUtils.isJSONObject(message)) {
                                sfResult = JSON.parseObject(message, SangforResult.class);
                            }
                            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                                    data = sfResult.getData();
                                    dataJson = JSON.parseObject(JSON.toJSONString(data));
                                    groups = JSON.parseArray(dataJson.getString("data"), SangForServerGroupModel.class);
                                    if (groups != null) {
                                        allGroupList.addAll(groups);
                                    }
                                }
                            }
                            nextPage++;
                        }
                    }
                }
                return new BocloudResult(true, allGroupList, "查询分组成功");
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "查询分组出错，失败原因为：" + errorMessage);
        } catch (Exception e) {
            logger.error("查询分组出错", e);
            return new BocloudResult(false, "查询分组出错，失败原因为：" + e.getMessage());
        }
    }


    @Override
    public BocloudResult update(SangforGroupModel model) {
        try {
            Map<String, Object> params = new HashMap<>();
            if (StringUtils.isNotEmpty(model.getName())) {
                params.put("name", model.getName());
            }
            params.put("server_ids", model.getServerIds());
            Result result = this.doPut("/janus/20180725/servers/groups/" + model.getGroupId(), params, "application/json", PostDataFormat.RAW);
            List<SangForServerGroupModel> groupModels = new ArrayList<>();
            if (result.isSuccess()) {
                SangForServerGroupModel groupModel = JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data"),
                        SangForServerGroupModel.class);
                groupModels.add(groupModel);
            }
            return new BocloudResult<>(true, groupModels, "更新成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(SangforGroupModel model) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("name", model.getName());
            params.put("az_id", model.getAzId());
            params.put("project_id", model.getProjectId());
            params.put("parent_id", model.getParentId());
            Result result = this.doPost("/janus/20180725/servers/groups", params, "application/json", PostDataFormat.RAW);
            List<SangForServerGroupModel> groupModels = new ArrayList<>();
            if (result.isSuccess()) {
                SangForServerGroupModel groupModel = JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data"),
                        SangForServerGroupModel.class);
                groupModels.add(groupModel);
                return new BocloudResult<>(true, groupModels, "创建虚拟机分组成功");
            } else {
                return new BocloudResult<>(false,
                        JSONObject.parseObject(result.getMessage()).getString("message"));
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, "创建虚拟机分组失败");
        }
    }

    @Override
    public BocloudResult delete(String groupUuid) {
        try {
            Map<String, Object> params = new HashMap<>();
            String url = "/janus/20180725/servers/groups/" + groupUuid;
            Result result = this.doDelete(url, params, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            return new BocloudResult(true, "删除虚拟机分组成功", "success");
                        } else {
                            return new BocloudResult(true, "删除虚拟机分组失败，" + taskResult.getMessage(), "failure");
                        }
                    } else {
                        return new BocloudResult(false, "删除虚拟机分组失败，返回数据为:" + text);
                    }
                } else {
                    return new BocloudResult(false, "删除虚拟机分组失败，原因为：" + sfResult.getMessage());
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "删除虚拟机分组失败，失败原因：" + errorMessage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, "删除失败");
        }
    }
}
