package com.bocloud.cmp.provider.v68.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.params.Sangfor.SangforHostParams;
import com.bocloud.cmp.params.Sangfor.SangforInterfacesParams;
import com.bocloud.cmp.params.Sangfor.SangforProjectsParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.compute.HostProvider;
import com.bocloud.cmp.provider.v68.common.BaseProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


@Provider("bocloud.cmp.sangfor.v68")
public class SangforHostProvider extends SangforProvider implements HostProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforHostProvider.class);
    public SangforHostProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(){
        try {

        Map<String, Object> params =new HashMap<>();
        params.put("type","hci");
        Result result = this.doGet("/janus/20180725/azs", params);
        List<SangforScpHostModel> serverList = new ArrayList<>();
        if (result.isSuccess()) {
            //数据可能存在这里
            List<SangforAzModel> sangforAzModels =
                    JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                            SangforAzModel.class);
            //假设查询的hci资源池的
            //物理机集合
            for (int i = 0; i < sangforAzModels.size(); i++) {
                //获取到其中一条数据
                String id = sangforAzModels.get(i).getId();
                Map<String, Object> physicalMachineParams =new HashMap<>();
                physicalMachineParams.put("az_id",sangforAzModels.get(i).getId());
                Result result1 = this.doGet("/janus/20180725/hosts", physicalMachineParams);
                if (result1.isSuccess()){
                    String data1 = JSONObject.parseObject(result1.getMessage()).getString("data");
                    JSONObject jsonObject = JSONObject.parseObject(data1);
                    String data = jsonObject.getString("data");
                    List<SangforScpHostModel> sangforHostModels = JSONObject.parseArray(data, SangforScpHostModel.class);
                    serverList.addAll(sangforHostModels);
                }
            }
        }
            return new BocloudResult<>(true,HostConver(serverList),"获取物理机列表成功");
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return new BocloudResult<>(false,"获取物理机列表失败");
        }
    }

    @Override
    public BocloudResult listLocation(){
        BaseProvider provider = new BaseProvider(this.butler);
        return provider.listAzs();
    }

    @Override
    public BocloudResult listProject(){
        try {
            Map<String, Object> params =new HashMap<>();
            Result result = this.doGet("/janus/20180725/projects", params);
            List<SangforProjectsModel> projectsList = new ArrayList<>();
            if (result.isSuccess()) {
                //数据可能存在这里
                List<SangforProjectsModel> modelsList =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforProjectsModel.class);
                projectsList.addAll(modelsList);
                return new BocloudResult<>(true,prConver(projectsList),"获取租户列表成功");
            }else {
                return new BocloudResult<>(false,
                        JSONObject.parseObject(result.getMessage()).getString("message"));
            }
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return new BocloudResult<>(false,e.getMessage());
        }
    }

    //物理网络
    public BocloudResult sangforInterfacesList(){
        try {
            Map<String, Object> params =new HashMap<>();
            params.put("type","hci");
            Result result = this.doGet("/janus/20180725/azs", params);
            List<SangforScpHostModel> serverList = new ArrayList<>();
            if (result.isSuccess()) {
                //数据可能存在这里
                List<SangforAzModel> sangforAzModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforAzModel.class);
                //假设查询的hci资源池的
                //物理机集合
                for (int i = 0; i < sangforAzModels.size(); i++) {
                    //获取到其中一条数据
                    String id = sangforAzModels.get(i).getId();
                    Map<String, Object> physicalMachineParams =new HashMap<>();
                    physicalMachineParams.put("az_id",sangforAzModels.get(i).getId());
                    Result result1 = this.doGet("/janus/20180725/hosts", physicalMachineParams);
                    if (result1.isSuccess()){
                        List<SangforScpHostModel> sangforHostModels =
                                JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result1.getMessage()).getString("data")).getString("data"), SangforScpHostModel.class);
                        serverList.addAll(sangforHostModels);
                    }
                }
            }
            List<SangforScpInfacesModel> infacesList = new ArrayList<>();
            for (SangforScpHostModel model:serverList){
                Result inResult = this.doGet(String.format("/janus/20180725/hosts/%s/interfaces", model.getId()), null);
                if (inResult.isSuccess()){
                    List<SangforScpInfacesModel> infacesModels =
                            JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(inResult.getMessage())
                                    .getString("data")).getString("data"), SangforScpInfacesModel.class);
                    List<SangforScpInfacesModel> infacesModelList = new ArrayList<>();
                    for (SangforScpInfacesModel models : infacesModels){
                        SangforScpInfacesModel infacesModel = new SangforScpInfacesModel();
                        BeanUtils.copyProperties(models,infacesModel);
                        infacesModel.setNode_id(model.getId());
                        infacesModelList.add(infacesModel);
                    }
                    infacesList.addAll(infacesModelList);
                }
            }
            return new BocloudResult<>(true,infacesConver(infacesList),"获取物理网络列表成功");
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return new BocloudResult<>(false,"获取物理网络列表失败");
        }
    }
    //通信口
    public BocloudResult sangforCommunication(){
        try {
            Map<String, Object> params =new HashMap<>();
            params.put("type","hci");
            Result result = this.doGet("/janus/20180725/azs", params);
            List<SangforScpHostModel> serverList = new ArrayList<>();
            if (result.isSuccess()) {
                //数据可能存在这里
                List<SangforAzModel> sangforAzModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforAzModel.class);
                //假设查询的hci资源池的
                //物理机集合
                for (int i = 0; i < sangforAzModels.size(); i++) {
                    //获取到其中一条数据
                    String id = sangforAzModels.get(i).getId();
                    Map<String, Object> physicalMachineParams =new HashMap<>();
                    physicalMachineParams.put("az_id",sangforAzModels.get(i).getId());
                    Result result1 = this.doGet("/janus/20180725/hosts", physicalMachineParams);
                    if (result1.isSuccess()){
                        List<SangforScpHostModel> sangforHostModels =
                                JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result1.getMessage()).getString("data")).getString("data"), SangforScpHostModel.class);
                        serverList.addAll(sangforHostModels);
                    }
                }
            }
            List<SangforScpInfacesModel> infacesList = new ArrayList<>();
            int count =0;
            List<SangforInterfacesParams> modelList = new ArrayList<>();
            for (SangforScpHostModel model:serverList){
                count++;
                Result inResult = this.doGet(String.format("/janus/20180725/hosts/%s/interfaces", model.getId()), null);
                if (inResult.isSuccess()){
                    List<SangforScpInfacesModel> infacesModels =
                            JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(inResult.getMessage())
                                    .getString("data")).getString("data"), SangforScpInfacesModel.class);
                    for (SangforScpInfacesModel model1 :infacesModels){
                        List<SangforCommunicationModel> communication_interface = model1.getCommunication_interface();
                        for (SangforCommunicationModel model2 : communication_interface){
                            String function = model2.getFunction();

                            String type = model1.getType();
                            List<String> phy_if = new ArrayList<>();
                            JSONArray array = new JSONArray();
                            SangforInterfacesParams infacesModel = new SangforInterfacesParams();
                            JSONObject object = new JSONObject();
                            List<SangforScpInfacesModel> collect = new ArrayList<>();
                            List<SangforCommunicationModel> mgmt = new ArrayList<>();
                            JSONArray array1 = new JSONArray();

                            if (type.equalsIgnoreCase("phy-if")){
                                switch (function){
                                    case "mgmt":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel = new SangforInterfacesParams();
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName("phyIf"+count);
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                    case "vxlan":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName("phyIf"+count);
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if1 = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if1.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if1.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        infacesModel.setMtu(model1.getMtu());
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                    case "business":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName(model1.getName());
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if2 = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if2.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if2.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        infacesModel.setMtu(model1.getMtu());
                                        infacesModel.setCustomId((long) (count+1));
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals("business")));
                                        break;
                                    case "vs":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        String join = String.join(",", phy_if);
                                        infacesModel.setMember(join);
                                        infacesModel.setIfaces(function);
                                        infacesModel.setMtu(model1.getMtu());
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setHost_ip(model.getIp());
                                        infacesModel.setHost_alias(model.getIp());
                                        infacesModel.setNode_status(model1.getStatus());
                                        infacesModel.setSpeed(model1.getSpeed()+"Mb/s");
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                }
                            }else {
                                switch (function){
                                    case "mgmt":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel = new SangforInterfacesParams();
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName("phyIf"+count);
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                    case "vxlan":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName("phyIf"+count);
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if1 = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if1.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if1.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                    case "business":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setNode_name(model.getName());
                                        infacesModel.setName(model1.getName());
                                        infacesModel.setNodeMembers(array.toJSONString());
                                        infacesModel.setNetmask(model1.getNetmask());
                                        infacesModel.setGateway(model1.getGateway());
                                        infacesModel.setIp(model1.getIp());
                                        List<String> finalPhy_if2 = phy_if;
                                        collect = infacesModels.stream().filter(inModel -> inModel.getName().equals(finalPhy_if2.get(0))).collect(Collectors.toList());
                                        object.put("unknow",0);
                                        object.put("negotiation",collect.get(0).getNegotiation());
                                        object.put("duplex",collect.get(0).getDuplex());
                                        object.put("value",collect.get(0).getSpeed());
                                        infacesModel.setSpeed(object.toJSONString());
                                        infacesModel.setDriverType(model1.getDriver_type());
                                        infacesModel.setEnable(model1.getEnable());
                                        mgmt = infacesModels.stream().filter(inModel -> finalPhy_if2.contains(inModel.getName()))
                                                .flatMap(inModel -> inModel.getCommunication_interface().stream())
                                                .filter(com -> com.getFunction().equals(function))
                                                .collect(Collectors.toList());
                                        array1.addAll(mgmt);
                                        infacesModel.setCustomId((long) (count+1));
                                        infacesModel.setPrimarys(array1.toJSONString());
                                        infacesModel.setIfaces(function);
                                        infacesModel.setMtu(model1.getMtu());
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                    case "vs":
                                        phy_if = infacesModels.stream().filter(inModel -> inModel.getType().equals(type))
                                                .filter(inModel -> inModel.getCommunication_interface().stream().anyMatch(com ->com.getFunction().equals(function)))
                                                .map(SangforScpInfacesModel::getName).collect(Collectors.toList());
                                        array.addAll(phy_if);
                                        String join = String.join(",", phy_if);
                                        infacesModel.setMember(join);
                                        infacesModel.setIfaces(function);
                                        infacesModel.setMtu(model1.getMtu());
                                        infacesModel.setNode_id(model.getId());
                                        infacesModel.setHost_ip(model.getIp());
                                        infacesModel.setHost_alias(model.getIp());
                                        infacesModel.setNode_status(model1.getStatus());
                                        infacesModel.setSpeed(model1.getSpeed()+"Mb/s");
                                        modelList.add(infacesModel);
                                        infacesModels.stream()
                                                .filter(models ->!models.getName().equals(model1.getName()))
                                                .forEach(user -> user.getCommunication_interface().removeIf(commt -> commt.getFunction().equals(function)));
                                        break;
                                }
                            }
                        }
                    }

                }
            }

            List<SangforInterfacesParams> businessModel = modelList.stream().filter(models -> models.getIfaces().equalsIgnoreCase("business")).collect(Collectors.toList());
            List<SangforInterfacesParams> businessRemove = modelList.stream().filter(models -> !models.getIfaces().equals("business")).collect(Collectors.toList());
            SangforInterfacesParams busParams = new SangforInterfacesParams();
            JSONArray busArray = new JSONArray();
            busArray.addAll(businessModel);
            busParams.setBdr_switch("网络出口");
            busParams.setIfaces("business");
            busParams.setCustomId((long) count-(businessModel.size()-1));
            busParams.setSubdatas(busArray.toJSONString());
            businessRemove.add(busParams);
            List<SangforInterfacesParams> newModelList = new ArrayList<>(businessRemove);
            return new BocloudResult(true,newModelList,"获取物理机通信口列表成功");
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return new BocloudResult<>(false,"获取物理机通信口列表失败");
        }
    }

    private List<SangforInterfacesParams> infacesConver(List<SangforScpInfacesModel> infacesModels){
        List<SangforInterfacesParams> paramsList = new ArrayList<>();
        for (SangforScpInfacesModel model : infacesModels){
            SangforInterfacesParams interfaces = new SangforInterfacesParams();
            BeanUtils.copyProperties(model,interfaces);
            interfaces.setUuid(model.getId());
            interfaces.setDriverType(model.getDriver_type());
            interfaces.setSupportSpeed(model.getSupport_speed());
            interfaces.setSriovSupport(model.getSriov_support());
            Long speed = Long.valueOf(model.getSpeed());
            Long  duplexs = null;
            String duplex = model.getDuplex();
            if (duplex.equalsIgnoreCase("Full")){
                duplexs = 2L;
            }else if (duplex.equalsIgnoreCase("Half")){
                duplexs = 1L;
            }else {
                duplexs = 0L;
            }

            JSONObject object = new JSONObject();
            object.put("unknow",0);
            object.put("negotiation", model.getNegotiation());
            object.put("duplex",duplexs);
            object.put("value",speed);
            interfaces.setSpeed(object.toJSONString());


            interfaces.setVsUsed(model.getVs_used());
            interfaces.setNodeId(model.getNode_id());
            paramsList.add(interfaces);
        }
        return paramsList;
    }

    private List<SangforHostParams> HostConver(List<SangforScpHostModel> serverList){
        List<SangforHostParams> hostResultList = new ArrayList<>();
        for (SangforScpHostModel model:serverList){
            BigDecimal diskDm = new BigDecimal(model.getStorage().getString("total_mb"));
            String disk = diskDm.multiply(new BigDecimal(1024*1024))
                    .divide(new BigDecimal(1000*1000*1000), 0, RoundingMode.DOWN).toString();

            String memoryTotal = new BigDecimal(model.getMemory().getString("total_mb"))
                    .divide(new BigDecimal(1024), 0, RoundingMode.DOWN).toString();

            String memoryTotalBy = new BigDecimal(model.getMemory().getString("total_mb"))
                    .multiply(new BigDecimal(1024*1024)).setScale(0,RoundingMode.DOWN).toString();


            String memoryUsedMb =  new BigDecimal(model.getMemory().getString("used_mb"))
                    .divide(new BigDecimal(1024),2,RoundingMode.HALF_UP).toString();

            String memoryUsedBy =  new BigDecimal(model.getMemory().getString("used_mb"))
                    .multiply(new BigDecimal(1024*1024)).setScale(0,RoundingMode.DOWN).toString();

            String memoryRatio = null;
            if (model.getMemory().getString("ratio").compareTo(String.valueOf(new BigDecimal("0.1"))) <0){
                memoryRatio = new BigDecimal(model.getMemory().getString("ratio"))
                        .multiply(new BigDecimal(100)).setScale(1,RoundingMode.DOWN).toString();
            }else {
                memoryRatio = new BigDecimal(model.getMemory().getString("ratio"))
                        .multiply(new BigDecimal(100)).setScale(0,RoundingMode.DOWN).toString();
            }


            String cpuTotal = new BigDecimal(model.getCpu().getString("total_mhz"))
                    .divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).toString();

            String cpuUsedMhz = new BigDecimal(model.getCpu().getString("used_mhz"))
                    .divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).toString();
            String cpuRatio = null;
            if (model.getCpu().getString("ratio").compareTo(String.valueOf(new BigDecimal("0.1"))) <0){
                cpuRatio = new BigDecimal(model.getCpu().getString("ratio"))
                        .multiply(new BigDecimal(100)).setScale(1,RoundingMode.DOWN).toString();
            }else {
                cpuRatio = new BigDecimal(model.getCpu().getString("ratio"))
                        .multiply(new BigDecimal(10)).setScale(0,RoundingMode.DOWN).toString();
            }


            String coreCount = model.getCpu().getString("core_count");
            String cpuModel = model.getCpu().getString("type");
            String socketCount = model.getCpu().getString("socket_count");

            SangforHostParams result = new SangforHostParams();
            result.setUuid(model.getId());
            result.setHostname(model.getName());
            result.setName(model.getName());
            result.setManageIp(model.getIp());
            result.setRemark(model.getDescription());
            result.setType(model.getType());
            result.setStatus(model.getStatus().toUpperCase());
            result.setServiceIp(model.getIp());
            result.setSangforAzId(model.getAz_id());

            result.setCpuModel(cpuModel);
            result.setCpuCore(Long.valueOf(coreCount));
            result.setSocketCount(Long.valueOf(socketCount));

            result.setDisk(disk);
            result.setMemory(memoryTotal);
            result.setMemRatio(model.getMemory().getString("ratio"));
            result.setUsedMemory(memoryUsedMb);

            result.setCpuTotalMhz(cpuTotal);
            result.setUsedCpu(cpuUsedMhz);
            result.setCpuRatio(model.getCpu().getString("ratio"));

            SangforHostDetail detail = new SangforHostDetail();
            detail.setIp(model.getIp());
            detail.setName(model.getIp());
            detail.setOs_version(model.getOs_version());
            detail.setRemark(model.getDescription());

            HostConfMemStatus conf = new HostConfMemStatus();
            conf.setBase_total_byte(Long.valueOf(memoryTotalBy));
            conf.setBase_used_byte(Long.valueOf(memoryUsedBy));
            conf.setBase_used_rate(model.getMemory().getString("ratio"));
            conf.setConf_total_byte(null);
            conf.setConf_used_byte(null);
            conf.setConf_used_rate(null);


            HostRaidCard card = new HostRaidCard();
            HostRaidConModel conModel = new HostRaidConModel();
            conModel.setController_model("");
            conModel.setController_status("");
            conModel.setRaid_number("");
            card.setController0(conModel);

            HostCpuModel cpus = new HostCpuModel();
            cpus.setType(model.getCpu().getString("type"));
            String mhz = cpuMhzUtils(model.getCpu().getString("type"));
            cpus.setMhz(mhz);
            cpus.setCache_size("");

            HostNetCard netCard = new HostNetCard();
            List<HostNetCardArray> net_card_array = new ArrayList<>();
            HostNetCardArray cardArray = new HostNetCardArray();
            net_card_array.add(cardArray);
            netCard.setNet_card_array(net_card_array);

            detail.setNet_cards_status(netCard);
            detail.setConf_mem_status(conf);
            detail.setRaid_card_status(card);
            detail.setCpu_status(cpus);
            detail.setRunning_vms(0L);
            result.setDetail(JSONObject.toJSONString(detail, SerializerFeature.WriteMapNullValue));
            hostResultList.add(result);
        }
        return hostResultList;
    }



    private List<SangforProjectsParams> prConver(List<SangforProjectsModel> models){
        if (ListTool.isEmpty(models)){
            return null;
        }
        List<SangforProjectsParams> paramsList = new ArrayList<>();
        for (SangforProjectsModel model:models){
            SangforProjectsParams params = new SangforProjectsParams();
            BeanUtils.copyProperties(model,params);
            params.setRoleType(model.getRole_type());
            params.setUuid(model.getId());
            paramsList.add(params);
        }
        return paramsList;
    }

    private String cpuMhzUtils(String cpuType){
        String regex = "@\\s*([0-9.]+)GHz";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(cpuType);

        String frequency = null;
        if (matcher.find()) {
            frequency = matcher.group(1);
        } else {
            System.out.println("No match found.");
        }
        if (frequency!=null){
            frequency = new BigDecimal(frequency).multiply(new BigDecimal(1000)).setScale(3, RoundingMode.UP).toString();
        }
        return frequency;
    }
}
