package com.bocloud.cmp.provider.v68.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.sangfor.*;
import com.bocloud.cmp.params.Sangfor.SangforStorageParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.FastjsonUtils;
import com.bocloud.cmp.provider.common.IpUtils;
import com.bocloud.cmp.provider.compute.ServerProvider;
import com.bocloud.cmp.provider.v68.common.BaseProvider;
import com.bocloud.cmp.provider.v68.common.TaskProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.cmp.provider.v68.storage.SangforStorageProvider;
import com.bocloud.cmp.provider.v68.utils.SangForServerHelper;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Assert;

import javax.crypto.Cipher;
import javax.xml.bind.DatatypeConverter;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.RSAPublicKeySpec;
import java.util.*;
import java.util.concurrent.TimeUnit;


@Provider("bocloud.cmp.sangfor.v68")
public class SangforServerProvider extends SangforProvider implements ServerProvider {
    private static Logger logger = LoggerFactory.getLogger(SangforServerProvider.class);

    // 正在被克隆的虚拟机唯一标识
    private volatile static Set<String> cloningVmIdSet = new HashSet<>();
    private static long CLONING_SET_MAX_WAIT_MILLIS = TimeUnit.MINUTES.toMillis(60);

    // 最大等待时长:5分钟
    private static long MAX_WAIT_MILLIS = TimeUnit.MINUTES.toMillis(5);

    public SangforServerProvider(Butler butler) {
        super(butler);
    }

    public BocloudResult list() {
        List<SangforServerModel> servers = new ArrayList<>();
        HashMap<String, Object> params = new HashMap<>();
        try {
            params.put("type", SANGFOR_TYPE);// 只获取深信服的虚拟机
            params.put("page_num", 0);
            params.put("page_size", PAGE_SIZE);
            Result result = this.doGet("/janus/20200725/servers", params);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }
            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject dataJson = JSON.parseObject(JSON.toJSONString(data));
                    List<SangforServerModel> list = JSON.parseArray(dataJson.getString("data"), SangforServerModel.class);
                    servers.addAll(list);
                    Integer totalSize = dataJson.getInteger("total_size");
                    String nextPageString = dataJson.getString("next_page_num");

                    Integer totalPage, nextPage;
                    // 下一页数，整型的字符串，为""表示当前是最后一页
                    if (StringUtils.isEmpty(nextPageString)) {
                        nextPage = 0;
                        totalPage = 0;
                    } else {
                        nextPage = dataJson.getInteger("next_page_num");
                        totalPage = totalSize / PAGE_SIZE;
                        if (totalSize % PAGE_SIZE > 0) {
                            totalPage++;
                        }
                    }

                    if (totalPage > 0) {
                        while (nextPage < totalPage) {
                            params.clear();
                            params.put("type", SANGFOR_TYPE);
                            params.put("page_num", nextPage);
                            params.put("page_size", PAGE_SIZE);
                            result = this.doGet("/janus/20200725/servers", params);
                            message = result.getMessage();
                            sfResult = null;
                            if (FastjsonUtils.isJSONObject(message)) {
                                sfResult = JSON.parseObject(message, SangforResult.class);
                            }
                            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                                    data = sfResult.getData();
                                    dataJson = JSON.parseObject(JSON.toJSONString(data));
                                    list = JSONObject.parseArray(dataJson.getString("data"), SangforServerModel.class);
                                    servers.addAll(list);
                                } else {
                                    logger.warn("本轮获取虚拟机列表失败:" + sfResult.getMessage());
                                }
                            } else {
                                logger.warn("本轮获取虚拟机列表失败:" + result.getMessage());
                            }
                            nextPage++;
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error("获取虚拟机列表失败: {}", e);
            return new BocloudResult(false, "获取虚拟机列表失败");
        }


        int total = servers.size();
        int process = 0;
        if (servers != null && !servers.isEmpty()) {
            for (SangforServerModel server : servers) {
                process++;
                fixServerModel(server);
                logger.info("虚拟机列表获取进度{}/{}", process, total);
            }
        }
        return new BocloudResult(true, servers, "查询成功");
    }

    /**
     * 对虚拟机信息进行补充
     *
     * @author sxp
     * @since 2023/5/29
     */
    private void fixServerModel(SangforServerModel server) {
        String vmIdV68 = server.getVmIdV68();
        if (StringUtils.isNotEmpty(vmIdV68)) {
            server.setInstanceId(vmIdV68);
        }

        String status = server.getStatus();
        String newStatus = SangForServerHelper.getCloudServerStatus(status);
        server.setStatus(newStatus);

        List<SangforNetwork> networks = server.getNetworks();
        Map<String, Integer> network2ConnectMap = new HashMap<>();
        for (SangforNetwork network : networks) {
            network2ConnectMap.put(network.getIpAddress(), network.getConnect());
        }

        String managerIp = null;
        List<String> ips = server.getIps();
        if (ips != null && !ips.isEmpty()) {
            for (String ip : ips) {
                if (!network2ConnectMap.containsKey(ip)) {
                    continue;
                }
                Integer connect = network2ConnectMap.get(ip);
                if (connect == null || connect.intValue() == 0) {
                    continue;
                }
                managerIp = ip;
                if (StringUtils.isNotEmpty(managerIp)) {
                    server.setManagerIp(managerIp);
                    break;
                }
            }
            if (managerIp == null) {
                managerIp = ips.get(0);
                server.setManagerIp(managerIp);
            }
        }

        String privateIps = SangForServerHelper.getPrivateIps(ips, networks);
        if (StringUtils.isNotEmpty(managerIp)) {
            server.setPrivateIps(privateIps);
        }

        String osName = server.getOs_name();
        osName = SangForServerHelper.getDetailOsName(osName);
        if (StringUtils.isEmpty(osName) && StringUtils.isNotEmpty(server.getOs_type())) {
            String osType = server.getOs_type();
            osName = SangForServerHelper.getDetailOsName(osType);
        }
        server.setOsname(osName);

        try {
            String url = "/janus/20190725/servers/" + server.getInstanceId() + "/snapshots";
            Result snapShotResult = this.doGet(url, new HashMap<>(0));
            if (snapShotResult.isSuccess() && !StringUtils.isEmpty(snapShotResult.getMessage())) {
                JSONObject returnData = JSONObject.parseObject(snapShotResult.getMessage());
                if (null != returnData && returnData.getInteger("code").equals(0)) {
                    JSONObject data = returnData.getJSONObject("data");
                    List<SangforSnapshotModel> snapshotModels = JSON.parseArray(data.getString("data"), SangforSnapshotModel.class);
                    server.setSnapshotModels(snapshotModels);
                } else {
                    logger.warn("获取虚拟机" + server.getName() + "快照列表失败，返回数据为空");
                }
            } else {
                logger.warn("获取虚拟机" + server.getName() + "快照列表失败，调用接口失败");
            }
        } catch (Exception e) {
            logger.warn("获取虚拟机" + server.getName() + "快照列表失败", e);
        }
    }

    public BocloudResult createVm(SangforServerModel model) {
        Assert.notNull(model, "创建(全新)虚拟机参数为空");

        Assert.notNull(model.getMemory(), "虚拟机分配内存为空");
        Assert.notNull(model.getCores(), "虚拟机CPU总核数为空");
        Assert.notNull(model.getSockets(), "socket数目为空");
        Assert.notNull(model.getDisks(), "磁盘信息为空");
        Assert.notNull(model.getNetworks(), "网络信息为空");
        Assert.notNull(model.getImage_id(), "镜像ID为空");
        Assert.notNull(model.getAz_id(), "资源池ID为空");

        JSONObject params = new JSONObject();
        /**
         * 基础信息
         */
        //名称
        params.put("name", model.getName());
        // 分组
        params.put("group_id", model.getGroup_id());
        //内存大小(MB)
        params.put("memory_mb", model.getMemory() * 1024);
        //磁盘信息
        params.put("disks", model.getDisks());
        //网络信息
        params.put("networks", model.getNetworks());
        //镜像id
        params.put("image_id", model.getImage_id());
        //资源池id
        params.put("az_id", model.getAz_id());
        //专属服务器组id
        if (StringUtils.isNotEmpty(model.getDh_id())) {
            params.put("dh_id", model.getDh_id());
        }
        //云主机描述
        if (StringUtils.isNotEmpty(model.getDescription())) {
            params.put("description", model.getDescription());
        }
        //云主机数量
        if (null != model.getCount()) {
            params.put("count", model.getCount());
        }
        //云主机类型
        if (StringUtils.isNotEmpty(model.getType())) {
            params.put("type", model.getType());
        }
        //云主机UUID列表
        if (CollectionUtils.isNotEmpty(model.getUuids())) {
            params.put("uuids", model.getUuids());
        }
        //云主机存储标签id
        if (StringUtils.isNotEmpty(model.getStorage_tag_id())) {
            params.put("storage_tag_id", model.getStorage_tag_id());
        }
        //云主机存储id
        if (StringUtils.isNotEmpty(model.getStorage_id())) {
            params.put("storage_id", model.getStorage_id());
        }
        //运行位置
        Object locationObj = model.getLocation();
        if (null != locationObj) {
            if ((locationObj instanceof JSONObject) || (locationObj instanceof Map)) {
                params.put("location", model.getLocation());
            } else {
                params.put("location", locationObj);
            }
        }
        //是否创建完之后开机
        if (null != model.getPower_on()) {
            params.put("power_on", model.getPower_on());
        }
        //光驱信息
        if (CollectionUtils.isNotEmpty(model.getCdroms())) {
            params.put("cdroms", model.getCdroms());
        }
        //USB信息
        if (CollectionUtils.isNotEmpty(model.getUsbs())) {
            params.put("usbs", model.getUsbs());
        }
        //超融合高级参数
        if (null != model.getAdvance_param()) {
            params.put("advance_param", model.getAdvance_param());
        }
        //云主机密码
        if (StringUtils.isNotEmpty(model.getPassword())) {
            params.put("password", model.getPassword());
        }
        //弹性公网IP创建信息
        if (null != model.getFloatingip()) {
            params.put("floatingip", model.getFloatingip());
        }
        //云主机hostname
        if (StringUtils.isNotEmpty(model.getHostname())) {
            params.put("hostname", model.getHostname());
        }
        //云主机失效时间 参数格式须为 %Y-%m-%d %H:%M:%S，例: 2021-10-25 23:59:59
        if (StringUtils.isNotEmpty(model.getLifespan())) {
            params.put("lifespan", model.getLifespan());
        }
        //云主机hostname后缀起始值
        if (null != model.getHostname_index()) {
            params.put("hostname_index", model.getHostname_index());
        }
        //加密信息
        if (null != model.getEncryption()) {
            params.put("encryption", model.getEncryption());
        }
        //GPU配置信息
        if (null != model.getGpu()) {
            params.put("gpu", model.getGpu());
        }
        //密钥信息
        if (null != model.getKeypair()) {
            params.put("keypair", model.getKeypair());
        }
        //租户id
        if (StringUtils.isNotEmpty(model.getProject_id())) {
            params.put("project_id", model.getProject_id());
        }
        //用户id
        if (StringUtils.isNotEmpty(model.getUser_id())) {
            params.put("user_id", model.getUser_id());
        }

        // 虚拟机插槽数
        params.put("sockets", model.getSockets());
        // 每个插槽的核数
        params.put("cores", model.getCores());

        logger.info("全新虚拟机创建params参数:{}", params.toJSONString());
        Result result = this.doPost("/janus/20180725/servers", params, "application/json", PostDataFormat.RAW);
        logger.info(result.toString());
        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }
        if (result.isSuccess() && !StringUtils.isEmpty(message)) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    String uuids = json.getString("uuids");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        List<String> newInstanceIds = JSON.parseArray(uuids, String.class);
                        List<SangforServerModel> newServerList = new ArrayList<>(newInstanceIds.size());
                        SangforServerModel vm;
                        BocloudResult bocloudResult;
                        for (String instanceId : newInstanceIds) {
                            bocloudResult = this.detail(instanceId);
                            if (bocloudResult.isSuccess()) {
                                vm = (SangforServerModel) bocloudResult.getData();
                                if (vm != null) {
                                    newServerList.add(vm);
                                }
                            }
                        }
                        return new BocloudResult(true, newServerList, "创建(全新)云主机成功");
                    } else {
                        return new BocloudResult(false, "创建(全新)云主机失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "创建(全新)云主机失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "创建(全新)云主机失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "创建(全新)虚拟机失败：" + errorMessage);
    }

    public BocloudResult cloneVm(SangforServerModel server) {
        Assert.notNull(server, "虚拟机克隆参数为空");
        String vmId = server.getVmid();
        if (StringUtils.isEmpty(vmId)) {
            return new BocloudResult(false, "虚拟机被克隆实例id为空");
        }

        Map<String, Object> param = new HashMap<>();
        if (StringUtils.isEmpty(server.getName())) {
            return new BocloudResult(false, "虚拟机名称不能为空");
        }

        try {
            // 如果虚拟机正在被克隆，说明有一台虚拟机正在进行克隆操作，需要等待状态变化 sxp:2023.6.12
            SangforServerModel oldVm = null;
            BocloudResult bocloudResult = this.detail(vmId);
            if (bocloudResult.isSuccess()) {
                oldVm = (SangforServerModel) bocloudResult.getData();
            }
            if (oldVm == null) {
                logger.error("被克隆的虚拟机未查询到详情信息,vmId=" + vmId);
                return new BocloudResult(false, "被克隆的虚拟机未查询到详情信息,vmId=" + vmId);
            }
            String azId = oldVm.getAz_id();// 资源池ID
            String status = oldVm.getStatus();
            if (!BocloudStatus.Server.RUNNING.name().equals(status)
                    && !BocloudStatus.Server.STOPPED.name().equals(status)) {
                long begin = System.currentTimeMillis();
                while (System.currentTimeMillis() - begin < MAX_WAIT_MILLIS) {
                    logger.info("虚拟机{}状态为{}，只有运行中和已关闭的云主机可以执行克隆,等待状态变化...", oldVm.getName(), status);
                    TimeUnit.SECONDS.sleep(5);
                    bocloudResult = this.detail(vmId);
                    if (bocloudResult.isSuccess()) {
                        oldVm = (SangforServerModel) bocloudResult.getData();
                    }
                    if (oldVm == null) {
                        logger.error("被克隆的虚拟机未查询到详情信息,vmId=" + vmId);
                        return new BocloudResult(false, "被克隆的虚拟机未查询到详情信息,vmId=" + vmId);
                    }
                    status = oldVm.getStatus();
                    if (BocloudStatus.Server.RUNNING.name().equals(status)
                            || BocloudStatus.Server.STOPPED.name().equals(status)) {
                        logger.info("等待完毕，虚拟机{}的状态已经变成{}", oldVm.getName(), status);
                        break;
                    }
                }
            }

            long begin = System.currentTimeMillis();
            while (cloningVmIdSet.contains(vmId)) {
                logger.info("虚拟机{}正在进行克隆操作，等待克隆完成再进行本次克隆操作...", oldVm.getName());
                TimeUnit.SECONDS.sleep(5);
                while (System.currentTimeMillis() - begin > CLONING_SET_MAX_WAIT_MILLIS) {
                    logger.info("等待克隆超时，不再等待。");
                    break;
                }
            }

            Integer powerOn = server.getPower_on();
            if (powerOn == null) {
                powerOn = 0;
            }
            Boolean start = server.getStart();
            if (start != null && start.booleanValue()) {
                powerOn = 1;
            }

            boolean needStartVm = false;
            if (powerOn.intValue() == 1) {
                needStartVm = true;
            }

            Integer count = server.getCount();
            if (count == null) {
                count = 1;
            }
            logger.info("count={}", count);

            List<SangforDiskModel> newDisks = server.getDisks();

            String serverName = server.getName();

            param.put("name", serverName);
            param.put("count", count);
            param.put("power_on", 0);// 先不要启动机器，因为还要进行编辑操作

            if (!StringUtils.isEmpty(server.getDescription())) {
                param.put("description", server.getDescription());
            }
            if (!StringUtils.isEmpty(server.getRemark())) {
                param.put("description", server.getDescription());
            }

            String alwaysTryHost = server.getAlwaysTryHost();
            if (StringUtils.isNotEmpty(alwaysTryHost)) {
                param.put("compute_location_id", alwaysTryHost);
            }
            if (!StringUtils.isEmpty(server.getStorage_tag_id())) {
                param.put("storage_tag_id", server.getStorage_tag_id());
            }
            if (!StringUtils.isEmpty(server.getStorage())) {
                param.put("storage_location", server.getStorage());
            }
            if (!StringUtils.isEmpty(server.getGroup())) {
                param.put("group_id", server.getGroup());
            }
            if (!StringUtils.isEmpty(server.getGroup_id())) {
                param.put("group_id", server.getGroup_id());
            }

            String cloneType = server.getCloneType();
            if (!StringUtils.isEmpty(cloneType)) {
                param.put("clone_type", cloneType);
            }

            JSONObject advanceJson = new JSONObject();
            advanceJson.put("return_uuids", 1);
            param.put("advance_param", advanceJson);

            cloningVmIdSet.add(vmId);

            logger.info("source instanceId is {}, clone params: {}", vmId, JSON.toJSONString(param));
            String url = String.format("/janus/20180725/servers/%s/clone", vmId);
            Result result = this.doPost(url, param, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            List<String> newInstanceIds = null;
            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        if (json.containsKey("uuids")) {
                            String uuids = json.getString("uuids");
                            newInstanceIds = JSON.parseArray(uuids, String.class);
                        }
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            logger.info("克隆虚拟机成功，下面进行编辑操作");
                        } else {
                            return new BocloudResult(true, "虚拟机克隆失败，" + taskResult.getMessage(), "failure");
                        }
                    } else {
                        return new BocloudResult(false, "虚拟机克隆失败，返回数据为:" + text);
                    }
                } else {
                    return new BocloudResult(false, "虚拟机克隆失败，原因为：" + sfResult.getMessage());
                }
            } else {
                String errorMessage;
                if (sfResult != null) {
                    errorMessage = sfResult.getMessage();
                } else {
                    errorMessage = message;
                }
                return new BocloudResult(false, "虚拟机克隆失败，失败原因：" + errorMessage);
            }

            // 获取资源池对应的HCI版本 sxp:2023.6.20
            BaseProvider provider = new BaseProvider(this.butler);
            String hciVersion = provider.getVersionByAzId(azId);
            if (StringUtils.isEmpty(hciVersion)) {
                logger.warn("获取资源池{}对应的HCI版本为空", azId);
                hciVersion = "6.7";
            }
            logger.info("HCI版本:" + hciVersion);

            // 如果是快速克隆，以便快速克隆能够完成 sxp:2023.6.20
            if (!hciVersion.startsWith("6.8") && "fast".equals(cloneType)) {
                logger.info("等待3分钟，后台正在进行快速克隆操作");
                TimeUnit.MINUTES.sleep(3);
            }

            cloningVmIdSet.remove(vmId);

            String oldVmOsName = oldVm.getOsname();
            List<SangforDiskModel> oldDisks = oldVm.getDisks();
            Map<String, String> diskId2StorageFileMap = new HashMap<>();
            Map<String, String> diskId2StorageIdMap = new HashMap<>();
            if (oldDisks != null) {
                for (SangforDiskModel diskModel : oldDisks) {
                    diskId2StorageFileMap.put(diskModel.getId(), diskModel.getStorage_file());
                    diskId2StorageIdMap.put(diskModel.getId(), diskModel.getStorage_id());
                }
            }

            boolean addDisk = false;
            List<SangforServerModel> newServerList = null;
            if (newInstanceIds != null) {
                int size = newInstanceIds.size();
                if (size != count.intValue()) {
                    logger.warn("克隆{}台虚拟机，但是只有{}台克隆成功", count, size);
                }

                newServerList = new ArrayList<>(newInstanceIds.size());
                SangforServerModel vm;
                for (String instanceId : newInstanceIds) {
                    bocloudResult = this.detail(instanceId);
                    if (bocloudResult.isSuccess()) {
                        vm = (SangforServerModel) bocloudResult.getData();
                        if (vm != null) {
                            newServerList.add(vm);
                        }
                    }
                }

                // is_old_disk为1时，storage_file参数不能为空或者不传 sxp:2023.6.7
                if (newDisks != null) {
                    String diskId;
                    Integer isOldDisk;
                    for (SangforDiskModel diskModel : newDisks) {
                        diskId = diskModel.getId();
                        isOldDisk = diskModel.getIs_old_disk();
                        if (isOldDisk == null) {
                            isOldDisk = 0;
                        }
                        if (isOldDisk.intValue() == 0) {
                            addDisk = true;
                        }
                        if (isOldDisk.intValue() == 1 && diskId2StorageFileMap.containsKey(diskId)) {
                            diskModel.setStorage_file(diskId2StorageFileMap.get(diskId));
                        }
                        if (isOldDisk.intValue() == 1 && diskId2StorageIdMap.containsKey(diskId)) {
                            diskModel.setStorage_id(diskId2StorageIdMap.get(diskId));
                        }
                    }
                    logger.info("disks:" + JSON.toJSONString(newDisks));
                }


                if (newServerList.size() != size) {
                    return new BocloudResult(false, "成功克隆" + size + "台虚拟机，但是查询得到的虚拟机结果只有" + newServerList.size() + "台");
                }

                String mac = server.getMac();
                if (size > 1) {
                    mac = null;
                }
                if (StringUtils.isNotEmpty(mac)) {
                    // 6.8要修mac地址中的字母是小写字母
                    mac = mac.trim().toLowerCase();
                }

                String mask = server.getMask();
                String gateway = server.getGateway();
                String dns1 = server.getPrimaryDns();
                String dns2 = server.getSecondaryDns();
                String bridgeName = server.getBridgename();
                String portUuid = server.getPortUuid();
                Integer cores = server.getCores();
                String networkDeviceId = server.getBvsId();// 网口的uuid，即networkUuid
                Integer memory = server.getMemory();
                List<String> dnsList = new ArrayList<>(2);
                if (StringUtils.isNotEmpty(dns1)) {
                    dnsList.add(dns1);
                }
                if (StringUtils.isNotEmpty(dns2)) {
                    dnsList.add(dns2);
                }
                String ip = server.getIp();
                if (ip == null) {
                    ip = "";
                }
                if (ip != null) {
                    ip = ip.trim();
                    ip = ip.replace("\r", "");
                    ip = ip.replace("\n", "");
                }
                List<String> ipList = new ArrayList<>();
                if (size > 1) {
                    if (StringUtils.isNotEmpty(ip)) {
                        /*
                        "10.101.20.200"
                        " 10.101.20.200, 10.101.20.201"
                        "10.101.20.203-10.101.20.204"
                        */
                        if (ip.indexOf(",") >= 0) {
                            String[] ipArray = ip.split(",");
                            for (String item : ipArray) {
                                if (item != null) {
                                    item = item.trim();
                                }
                                if (StringUtils.isNotEmpty(item)) {
                                    ipList.add(item);
                                }
                            }
                        } else if (ip.indexOf("-") >= 0) {
                            int pos = ip.indexOf("-");
                            String startIp = ip.substring(0, pos);
                            String endIp = ip.substring(pos + 1);
                            if (startIp != null) {
                                startIp = startIp.trim();
                            }
                            if (endIp != null) {
                                endIp = endIp.trim();
                            }
                            ipList = IpUtils.findIPsForIpv4(startIp, endIp);
                            logger.info("findIPsForIpv4({},{})的结果为{}", startIp, endIp, JSON.toJSONString(ipList));
                        } else {
                            logger.warn("克隆{}台虚拟机，但是从IP地址{}无法解析出多个IP", size, ip);
                        }
                    }
                } else {
                    ipList.add(ip);
                }
                logger.info("ipList:" + JSON.toJSONString(ipList));

                String vmIdV68, name;
                ServerModel newVm;
                List<SangforNetwork> networks;

                SangforNetworkIpInfo ipInfo;
                BocloudResult subResult;
                SangforNetwork network;
                String loopIp;
                for (int i = 0; i < size; i++) {
                    vm = newServerList.get(i);
                    vmIdV68 = vm.getVmIdV68();

                    newVm = new ServerModel();
                    newVm.setInstanceId(vmIdV68);

                    if (size > 1) {
                        name = serverName + (i + 1);
                        newVm.setName(name);
                    } else {
                        name = serverName;
                    }

                    if (cores != null && !cores.equals(vm.getCores())) {
                        Integer sockets = vm.getSockets();
                        if (sockets == null) {
                            sockets = 1;
                        }
                        // cores值须大于等于sockets sxp:2023.6.20
                        if (cores < sockets) {
                            if (cores % 2 == 0) {
                                newVm.setSockets(cores / 2);
                            } else {
                                newVm.setSockets(cores);
                            }
                        }
                        newVm.setCores(cores);
                    }

                    if (memory != null && memory.intValue() > 0) {
                        newVm.setMemory(new BigDecimal(String.valueOf(memory)));
                    }

                    if (newDisks != null && !newDisks.isEmpty()) {
                        newVm.setSangforDisks(newDisks);
                    }
                    if (server.getBootdisk() !=null){
                        newVm.setSystemDiskSize(Integer.valueOf(server.getBootdisk()));
                    }

                    if (i < ipList.size()) {
                        loopIp = ipList.get(i);
                    } else {
                        loopIp = null;
                    }

//                    networkDeviceId = "d4e3c77f-7673-4a00-8449-80813563ee4c";
//                    mac ="1a:61:4b:c3:2d:8c";
//                    bridgeName ="物理出口1";
//                    portUuid ="ef8cfe56-8b54-47b0-988b-2c1b048af355";
//                    mask = "255.255.255.0";
//                    gateway = "192.168.24.1";
//                    loopIp = "192.168.24.252";
                    logger.info("create network net0");
                    List<SangforNetWorkConfig> netWorkConfigs = server.getNetWorkConfigs();
                    if (ListTool.isEmpty(netWorkConfigs)){
                        network = new SangforNetwork();
                        network.setVif_id("net0");
                        network.setConnect(1);
                        network.setDevice_id(networkDeviceId);
                        network.setModel("virtio");
                        network.setMac_address(mac);
                        if (StringUtils.isNotEmpty(bridgeName)) {
                            network.setName(bridgeName);
                        }
                        if (StringUtils.isNotEmpty(portUuid)) {
                            network.setPort_id(portUuid);
                        }
                        if (StringUtils.isNotEmpty(loopIp)) {
                            ipInfo = new SangforNetworkIpInfo();
                            ipInfo.setIp_address(loopIp);
                            ipInfo.setNetmask(mask);
                            ipInfo.setGateway(gateway);
                            ipInfo.setDns(dnsList);
                            network.setIp_info(ipInfo);
                        } else {
                            network.setIp_info(null);
                        }
                        networks = new ArrayList<>(1);
                        networks.add(network);
                        newVm.setSangforNetworks(networks);
                    }else {
                        networks = new ArrayList<>();
                        for (SangforNetWorkConfig netWorkConfig : netWorkConfigs) {
                            network = new SangforNetwork();
                            network.setVif_id(netWorkConfig.getVifId());
                            network.setConnect(1);
                            network.setDevice_id(netWorkConfig.getBvsId());
                            network.setModel("virtio");
                            network.setMac_address(netWorkConfig.getMac());
                            network.setName(netWorkConfig.getBridgename());
                            network.setPort_id(netWorkConfig.getPortUuid());
                            ipInfo = new SangforNetworkIpInfo();
                            ipInfo.setIp_address(netWorkConfig.getIp());
                            ipInfo.setNetmask(netWorkConfig.getMask());
                            ipInfo.setGateway(netWorkConfig.getGateway());
                            network.setIp_info(ipInfo);
                            networks.add(network);
                        }
                        newVm.setSangforNetworks(networks);
                    }
                    if (server.getDiskStorageUuid() !=null){
                        newVm.setDiskStorageUuid(server.getDiskStorageUuid());
                    }
                    newVm.setStorage(server.getStorage());
                    logger.info("虚拟机{}修改配置...", name);
                    subResult = this.modify(newVm);
                    if (subResult.isFailed()) {
                        logger.error("虚拟机{}修改失败，原因是:{}", name, subResult.getMessage());
                    } else {
                        logger.info("虚拟机{}修改配置成功", name);
                    }
                    logger.info("虚拟机密码:{}",JSON.toJSONString(server.getPassword()));
                    newVm.setPassword(server.getPassword());
                    //重置密码
                    subResult = this.resetVmPassword(newVm);
                    if (subResult.isFailed()){
                        logger.error("虚拟机{}密码修改失败，原因是:{}",name,subResult.getMessage());
                    }else {
                        logger.info("虚拟机{}修改密码成功",name);
                    }
                    needStartVm =true;
                    if (needStartVm) {
                        logger.info("虚拟机{}启动...", name);
                        subResult = this.start(newVm.getInstanceId());
                        if (subResult.isFailed()) {
                            logger.error("虚拟机{}启动失败，原因是:{}", name, subResult.getMessage());
                        } else {
                            logger.info("虚拟机{}启动成功", name);
                        }
                    }
                }
            }

            // 如果编辑过程中有增加磁盘的操作，等待一段时间 sxp:2023.6.20
            if (!hciVersion.startsWith("6.8") && addDisk) {
                logger.info("编辑过程中有增加磁盘的操作,等待2分钟");
                TimeUnit.MINUTES.sleep(2);
            }

            // 重新获取newServerList
            newServerList.clear();
            SangforServerModel vm;
            String currentOsName;
            for (String instanceId : newInstanceIds) {
                bocloudResult = this.detail(instanceId);
                if (bocloudResult.isSuccess()) {
                    vm = (SangforServerModel) bocloudResult.getData();
                    if (vm != null) {
                        currentOsName = vm.getOsname();
                        if (StringUtils.isEmpty(currentOsName) && StringUtils.isNotEmpty(oldVmOsName)) {
                            vm.setOsname(oldVmOsName);
                        }
                        newServerList.add(vm);
                    }
                }
            }
            return new BocloudResult(true, newServerList, "虚拟机克隆并修改配置成功");

        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "虚拟机克隆失败，失败原因：" + e.getMessage());
        } finally {
            if (cloningVmIdSet.contains(vmId)) {
                cloningVmIdSet.remove(vmId);
            }
        }
    }

    public String getPublicKey(){
        try {
            Result result = this.doGet("/janus/public-key", null);
            if (result.isSuccess()){
                String string = JSONObject.parseObject(result.getMessage()).getJSONObject("data").getString("public_key");
                return string.replace("\n", "");
            }else {
                return null;
            }
        }catch (Exception e){
            return null;
        }

    }
    public  BocloudResult resetVmPassword(ServerModel server){
        String instanceId = server.getInstanceId();
        Map<String, Object> params = new HashMap<>();
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "虚拟机配置变更实例id不能为空");
        }
        //需要在此处进行password操作
        if (!StringUtils.isEmpty(server.getPassword())){
            String password = encryptPassword(server.getPassword());
            params.put("password",password);
        }
        try {
            logger.info("变更主机id为:{}, resetVm params: {}", instanceId, JSON.toJSONString(params));
            Result result = this.doPost(String.format("/janus/20180725/servers/%s/server-password", instanceId), params, "application/json", PostDataFormat.RAW);
            logger.info("变更主机结果:" + JSON.toJSONString(result));
            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }
            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            // 返回修改后的虚拟机信息
                            BocloudResult bocloudResult = this.detail(instanceId);
                            SangforServerModel newServer = null;
                            if (bocloudResult.isSuccess()) {
                                newServer = (SangforServerModel) bocloudResult.getData();
                            }
                            return new BocloudResult(true, newServer, "虚拟机修改成功");
                        } else {
                            logger.error("虚拟机修改失败，" + taskResult.getMessage());
                            return new BocloudResult(false, "虚拟机修改失败，" + taskResult.getMessage());
                        }
                    } else {
                        logger.error("虚拟机修改失败，返回数据为:" + text);
                        return new BocloudResult(false, "虚拟机修改失败，返回数据为:" + text);
                    }
                } else {
                    logger.error("虚拟机修改失败，原因为：" + sfResult.getMessage());
                    return new BocloudResult(false, "虚拟机修改失败，原因为：" + sfResult.getMessage());
                }
            }
            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            logger.error("虚拟机修改失败，原因为：" + errorMessage);
            return new BocloudResult(false, "虚拟机修改失败，失败原因：" + errorMessage);
        }catch (Exception e){
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "虚拟机修改失败，失败原因：" + e.getMessage());
        }
    }



    public String encryptPassword(String password){
        String modulus = this.getPublicKey();
        try {
            byte[] contentBytes = password.getBytes(StandardCharsets.UTF_8);
            BigInteger e = new BigInteger("10001", 16);
            BigInteger n = new BigInteger(modulus, 16);
            RSAPublicKeySpec keySpec = new RSAPublicKeySpec(n, e);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            RSAPublicKey publicKey = (RSAPublicKey) keyFactory.generatePublic(keySpec);
            Cipher cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding");
            cipher.init(Cipher.ENCRYPT_MODE, publicKey);
            byte[] encryptedBytes = cipher.doFinal(contentBytes);
            String encryptedContent = DatatypeConverter.printHexBinary(encryptedBytes);
            return encryptedContent;
        }catch (Exception e){
            logger.error("编码密码报错", e);
            return null;
        }
    }
    @Override
    public BocloudResult modify(ServerModel server) {
        Assert.notNull(server, "虚拟机配置变更参数为空");
        Map<String, Object> params = new HashMap<>();

        boolean modifyDisk = false;

        String instanceId = server.getInstanceId();
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "虚拟机配置变更实例id不能为空");
        }
        if (!StringUtils.isEmpty(server.getName())) {
            params.put("name", server.getName());
        }
        if (!StringUtils.isEmpty(server.getRemark())) {
            params.put("description", server.getRemark());
        }
        if (!StringUtils.isEmpty(server.getGroup())) {
            params.put("group_id", server.getGroup());
        }

        if (null != server.getCores()) {
            params.put("cores", server.getCores());
        }
        Integer cpu = server.getCpu();
        if (cpu != null && cpu.intValue() > 0) {
            params.put("cores", cpu);
        }

        if (null != server.getSockets()) {
            params.put("sockets", server.getSockets());
        }

        Object coresObj = params.get("cores");
        Integer cores = null;
        if (coresObj != null) {
            cores = (Integer) coresObj;
        }

        Object socketsObj = params.get("sockets");
        Integer sockets = null;
        if (socketsObj != null) {
            sockets = (Integer) socketsObj;
        }

        // cores值须大于等于sockets sxp:2023.9.20
        if (cores != null) {
            if (sockets == null) {
                sockets = 1;
                params.put("sockets", sockets);
            }
            if (cores < sockets) {
                if (cores % 2 == 0) {
                    params.put("sockets", cores / 2);
                } else {
                    params.put("sockets", cores);
                }
            }
        }

        if (null != server.getMemory()) {
            params.put("memory_mb", server.getMemory().intValue() * 1024);
        }

        if (!StringUtils.isEmpty(server.getOstype())) {
            params.put("os_type", server.getOstype());
        }
        if (!StringUtils.isEmpty(server.getStorage())) {
            params.put("storage_location", server.getStorage());
        }

        if (!CollectionUtils.isEmpty(server.getSangforDisks())) {
            modifyDisk = true;

            List<SangforDiskModel> disks = server.getSangforDisks();
            BocloudResult diskResult = fixDiskList(instanceId, disks);
            if (diskResult.isFailed()) {
                return diskResult;
            }
            params.put("disks", disks);
        }

        if (!CollectionUtils.isEmpty(server.getSangforNetworks())) {
            params.put("networks", server.getSangforNetworks());
        }
        if (server.getSystemDiskSize() !=null){
            JSONArray array = new JSONArray();
            JSONObject object =new JSONObject();
            object.put("id","ide0");
            object.put("size_mb",server.getSystemDiskSize());
            object.put("is_old_disk",1);
            object.put("storage_file",server.getStorage());//存储的uuid
            object.put("storage_id",server.getStorage());
            object.put("preallocate",1);
            object.put("type","derive_disk");
            array.add(object);
            params.put("disks",array);
        }
        // begin--增加磁盘  sxp:2023.6.13
        Integer addDiskTotal = server.getAddDiskTotal();// 单位：GB
        if (addDiskTotal != null && addDiskTotal.intValue() > 0) {
            modifyDisk = true;

            String preallocate = server.getPreallocate();
            Integer newDiskSizeMb = addDiskTotal * 1024;

            SangforServerModel theVm = null;
            BocloudResult bocloudResult = this.detail(instanceId);
            if (bocloudResult.isSuccess()) {
                theVm = (SangforServerModel) bocloudResult.getData();
            }
            if (theVm == null) {
                logger.error("被克隆的虚拟机未查询到详情信息,name=" + theVm.getName());
                return new BocloudResult(false, "被克隆的虚拟机未查询到详情信息,name=" + theVm.getName());
            }
            List<SangforDiskModel> theVmDisks = theVm.getDisks();
            for (SangforDiskModel oldDisk : theVmDisks) {
                oldDisk.setIs_old_disk(1);
                oldDisk.setType("derive_disk");
            }

            int oldDiskCount = theVmDisks.size();
            String newDiskId;
            if (oldDiskCount == 1) {
                newDiskId = "ide1";
            } else if (oldDiskCount == 2) {
                newDiskId = "ide3";
            } else {
                newDiskId = "ide" + (oldDiskCount + 1);
            }
            SangforDiskModel newDisk = new SangforDiskModel();
            newDisk.setId(newDiskId);
            newDisk.setIs_old_disk(0);
            newDisk.setType("new_disk");
            newDisk.setPreallocate(preallocate);
            newDisk.setSize_mb(newDiskSizeMb);

            theVmDisks.add(newDisk);
            params.put("disks", theVmDisks);
        }
        // end----增加磁盘

        try {
            logger.info("变更主机id为:{}, modify params: {}", instanceId, JSON.toJSONString(params));
            Result result = this.doPut(String.format("/janus/20180725/servers/%s", instanceId), params, "application/json", PostDataFormat.RAW);
            logger.info("变更主机结果:" + JSON.toJSONString(result));

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            if (modifyDisk) {
                                logger.info("检测到修改磁盘的操作,等待1分钟再获取最新的虚拟机信息...");
                                TimeUnit.MINUTES.sleep(1);
                            }
                            // 返回修改后的虚拟机信息
                            BocloudResult bocloudResult = this.detail(instanceId);
                            SangforServerModel newServer = null;
                            if (bocloudResult.isSuccess()) {
                                newServer = (SangforServerModel) bocloudResult.getData();
                            }
                            return new BocloudResult(true, newServer, "虚拟机修改成功");
                        } else {
                            logger.error("虚拟机修改失败，" + taskResult.getMessage());
                            return new BocloudResult(false, "虚拟机修改失败，" + taskResult.getMessage());
                        }
                    } else {
                        logger.error("虚拟机修改失败，返回数据为:" + text);
                        return new BocloudResult(false, "虚拟机修改失败，返回数据为:" + text);
                    }
                } else {
                    logger.error("虚拟机修改失败，原因为：" + sfResult.getMessage());
                    return new BocloudResult(false, "虚拟机修改失败，原因为：" + sfResult.getMessage());
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            logger.error("虚拟机修改失败，原因为：" + errorMessage);
            return new BocloudResult(false, "虚拟机修改失败，失败原因：" + errorMessage);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "虚拟机修改失败，失败原因：" + e.getMessage());
        }
    }

    /**
     * 为磁盘数据填充字段storage_file,storage_id
     */
    private BocloudResult fixDiskList(String instanceId, List<SangforDiskModel> newDiskList) {
        if (newDiskList == null || newDiskList.isEmpty()) {
            return new BocloudResult(true, "success");
        }

        String diskId;
        Integer isOldDisk;
        boolean needUpdate = false;

        // is_old_disk为1时，storage_file,storage_id不能为空
        for (SangforDiskModel diskModel : newDiskList) {
            isOldDisk = diskModel.getIs_old_disk();
            if (isOldDisk == null) {
                isOldDisk = new Integer(0);
            }
            if (isOldDisk.intValue() == 1) {
                if (StringUtils.isEmpty(diskModel.getStorage_file())) {
                    needUpdate = true;
                    break;
                }
                if (StringUtils.isEmpty(diskModel.getStorage_id())) {
                    needUpdate = true;
                    break;
                }
            }
        }

        if (needUpdate) {
            SangforServerModel oldVm = null;
            BocloudResult bocloudResult = this.detail(instanceId);
            if (bocloudResult.isSuccess()) {
                oldVm = (SangforServerModel) bocloudResult.getData();
            }
            if (oldVm == null) {
                logger.error("未查询到当前虚拟机详情信息,vmId=" + instanceId);
                return new BocloudResult(false, "未查询到当前虚拟机详情信息,vmId=" + instanceId);
            }
            List<SangforDiskModel> oldDisks = oldVm.getDisks();
            Map<String, String> diskId2StorageFileMap = new HashMap<>();
            Map<String, String> diskId2StorageIdMap = new HashMap<>();
            if (oldDisks != null) {
                for (SangforDiskModel diskModel : oldDisks) {
                    diskId2StorageFileMap.put(diskModel.getId(), diskModel.getStorage_file());
                    diskId2StorageIdMap.put(diskModel.getId(), diskModel.getStorage_id());
                }
            }

            for (SangforDiskModel diskModel : newDiskList) {
                diskId = diskModel.getId();
                isOldDisk = diskModel.getIs_old_disk();
                if (isOldDisk == null) {
                    isOldDisk = new Integer(0);
                }
                if (isOldDisk.intValue() == 1 && diskId2StorageFileMap.containsKey(diskId)) {
                    diskModel.setStorage_file(diskId2StorageFileMap.get(diskId));
                }
                if (isOldDisk.intValue() == 1 && diskId2StorageIdMap.containsKey(diskId)) {
                    diskModel.setStorage_id(diskId2StorageIdMap.get(diskId));
                }
            }
            logger.info("new disks:" + JSON.toJSONString(newDiskList));
        }

        return new BocloudResult(true, "success");
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        Map<String, Object> params = new HashMap<>(0);
        Result result = this.doDelete(String.format("/janus/20180725/servers/%s", instanceId), params, "application/json", PostDataFormat.RAW);
        logger.info(result.toString());

        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }

        if (result.isSuccess() && !StringUtils.isEmpty(message)) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        return new BocloudResult(true, "虚拟机删除成功");
                    } else {
                        return new BocloudResult(false, "虚拟机删除失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "虚拟机删除失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "虚拟机删除失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "虚拟机删除失败，失败原因：" + errorMessage);
    }

    @Override
    public BocloudResult start(String vmId) {
        Map<String, Object> params = new HashMap<>();
        Result result = this.doPost(String.format("/janus/20180725/servers/%s/start", vmId), params, "application/json", PostDataFormat.RAW);

        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }

        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        return new BocloudResult(true, "虚拟机启动成功");
                    } else {
                        return new BocloudResult(false, "虚拟机启动失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "虚拟机启动失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "虚拟机启动失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "虚拟机启动失败，失败原因：" + errorMessage);
    }

    @Override
    public BocloudResult stop(String vmId) {
        Map<String, Object> params = new HashMap<>();
        Result result = this.doPost(String.format("/janus/20180725/servers/%s/stop", vmId), params, "application/json", PostDataFormat.RAW);

        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }

        if (result.isSuccess()) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        return new BocloudResult(true, "虚拟机关闭成功");
                    } else {
                        return new BocloudResult(false, "虚拟机关闭失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "虚拟机关闭失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "虚拟机关闭失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "虚拟机关闭失败，失败原因：" + errorMessage);
    }

    @Override
    public BocloudResult reboot(String vmId) {
        Map<String, Object> params = new HashMap<>();
        Result result = this.doPost(String.format("/janus/20180725/servers/%s/reboot", vmId), params, "application/json", PostDataFormat.RAW);

        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }

        if (result.isSuccess() && !StringUtils.isEmpty(result.getMessage())) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        return new BocloudResult(true, "虚拟机重启成功");
                    } else {
                        return new BocloudResult(false, "虚拟机重启失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "虚拟机重启失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "虚拟机重启失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "虚拟机重启失败，失败原因：" + errorMessage);
    }

    /**
     * 虚拟机挂起
     *
     * @author sxp
     * @since 2023/5/29
     */
    @Override
    public BocloudResult suspend(String instanceId) {
        Map<String, Object> params = new HashMap<>();
        Result result = this.doPost(String.format("/janus/20180725/servers/%s/suspend", instanceId), params, "application/json", PostDataFormat.RAW);

        String message = result.getMessage();
        SangforResult sfResult = null;
        if (FastjsonUtils.isJSONObject(message)) {
            sfResult = JSON.parseObject(message, SangforResult.class);
        }

        if (result.isSuccess()) {
            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                String text = JSON.toJSONString(data);
                JSONObject json = JSON.parseObject(text, JSONObject.class);
                if (json != null) {
                    String taskId = json.getString("task_id");
                    TaskProvider provider = new TaskProvider(butler);
                    BocloudResult taskResult = provider.waitTask(taskId);
                    if (taskResult.isSuccess()) {
                        return new BocloudResult(true, "虚拟机挂起成功");
                    } else {
                        return new BocloudResult(true, "虚拟机挂起失败，" + taskResult.getMessage());
                    }
                } else {
                    return new BocloudResult(false, "虚拟机挂起失败，返回数据为:" + text);
                }
            } else {
                return new BocloudResult(false, "虚拟机挂起失败，原因为：" + sfResult.getMessage());
            }
        }

        String errorMessage;
        if (sfResult != null) {
            errorMessage = sfResult.getMessage();
        } else {
            errorMessage = message;
        }
        return new BocloudResult(false, "虚拟机挂起失败，失败原因：" + errorMessage);
    }


    /**
     * 查询虚拟机明细信息
     *
     * @author sxp
     * @since 2023/5/29
     */
    @Override
    public BocloudResult detail(String instanceId) {
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "虚拟机ID为空，查询失败");
        }

        // 详情接口有部分字段获取不到，改成使用云主机列表查询接口并制定查询条件 sxp:2023.5.30
        // String url = String.format("/janus/20180725/servers/%s", instanceId);
        HashMap<String, Object> params = new HashMap<>();
        params.put("page_num", 0);
        params.put("page_size", PAGE_SIZE);
        params.put("ids", instanceId);
        String url = "/janus/20200725/servers";

        Result result;
        try {
            result = this.doGet(url, params);
            if (result.isFailed()) {
                return new BocloudResult(false, "查询虚拟机详情" + instanceId + "信息失败，失败原因为：" + result.getMessage());
            }

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                JSONObject json = JSON.parseObject(JSON.toJSONString(data));
                List<SangforServerModel> list = JSON.parseArray(json.getString("data"), SangforServerModel.class);

                SangforServerModel serverModel = null;
                if (list != null && !list.isEmpty()) {
                    serverModel = list.get(0);
                }
                if (serverModel != null) {
                    fixServerModel(serverModel);
                    return new BocloudResult(true, serverModel, "success");
                } else {
                    return new BocloudResult(false, "查询虚拟机详情" + instanceId + "信息失败，返回数据为:" + JSON.toJSONString(data));
                }
            } else {
                String errorMessage;
                if (sfResult != null) {
                    errorMessage = sfResult.getMessage();
                } else {
                    errorMessage = message;
                }
                return new BocloudResult(false, "查询虚拟机详情" + instanceId + "信息失败，失败原因为：" + errorMessage);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "查询虚拟机详情" + instanceId + "信息失败，失败原因为：" + e.getMessage());
        }
    }

    /**
     * 查询虚拟机明细信息(使用/janus/20180725/servers/{server_id}接口调用)
     *
     * @author sxp
     * @since 2023/5/29
     */
    public BocloudResult detailOfRealAPI(String instanceId) {
        if (StringUtils.isEmpty(instanceId)) {
            return new BocloudResult(false, "虚拟机ID为空，查询失败");
        }

        HashMap<String, Object> params = new HashMap<>();
        String url = String.format("/janus/20180725/servers/%s", instanceId);
        Result result;

        try {
            result = this.doGet(url, params);
            if (result.isFailed()) {
                return new BocloudResult(false, "查询虚拟机详情[*]" + instanceId + "信息失败，失败原因为：" + result.getMessage());
            }

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                Object data = sfResult.getData();
                JSONObject json = JSON.parseObject(JSON.toJSONString(data));
                SangforServerModel serverModel = JSON.parseObject(json.toJSONString(), SangforServerModel.class);
                if (serverModel != null) {
                    fixServerModel(serverModel);
                    return new BocloudResult(true, serverModel, "success");
                } else {
                    return new BocloudResult(false, "查询虚拟机详情[*]" + instanceId + "信息失败，返回数据为:" + JSON.toJSONString(data));
                }
            } else {
                String errorMessage;
                if (sfResult != null) {
                    errorMessage = sfResult.getMessage();
                } else {
                    errorMessage = message;
                }
                return new BocloudResult(false, "查询虚拟机详情[*]" + instanceId + "信息失败，失败原因为：" + errorMessage);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return new BocloudResult(false, "查询虚拟机详情[*]" + instanceId + "信息失败，失败原因为：" + e.getMessage());
        }
    }

    /**
     * 获取VNC控制台信息
     *
     * @author sxp
     * @since 2023/5/30
     */
    @Override
    public BocloudResult vncLogin(String instanceId) {
        Map<String, Object> consoleMap = new HashMap<>();
        consoleMap.put("protocol", "vnc");
        consoleMap.put("type", "novnc");

        Map<String, Object> params = new HashMap<>();
        params.put("remote_console", consoleMap);

        String url = String.format("/janus/20180725/servers/%s/remote-consoles", instanceId);

        try {
            Result result = this.doPost(url, params, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject json = JSON.parseObject(JSON.toJSONString(data), JSONObject.class);
                    if (json != null) {
                        return new BocloudResult(true, json, "success");
                    } else {
                        return new BocloudResult(false, "获取VNC控制台地址出错，返回数据为：" + JSON.toJSONString(data));
                    }
                }
            }
            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "获取VNC控制台地址出错，失败原因为：" + errorMessage);
        } catch (Exception e) {
            logger.error("获取VNC控制台地址出错", e);
            return new BocloudResult(false, "获取VNC控制台地址出错，失败原因为：" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listLabel() {
        List<SangForServerLabelModel> labelModelList = new ArrayList<>(0);
        return new BocloudResult(true, labelModelList, "SCP中没有标签");
    }

    @Override
    public BocloudResult listIso(String storageId) {
        String azId = null;
        if (StringUtils.isNotEmpty(storageId)) {
            SangforStorageProvider storageProvider = new SangforStorageProvider(this.butler);
            BocloudResult<SangforStorageParams> storagResult = storageProvider.detailStorage(storageId);
            if (storagResult.isSuccess()) {
                SangforStorageParams storageParams = storagResult.getData();
                if (storageParams != null) {
                    azId = storageParams.getAzId();
                    logger.info("存储{}的azId为{}", storageId, azId);
                }
            }
        }

        Map<String, Object> params = new HashMap<>();
        params.put("disk_format", "ISO");
        if (StringUtils.isNotEmpty(azId)) {
            params.put("az_id", azId);
        }
        SangforImageProvider imageProvider = new SangforImageProvider(this.butler);
        BocloudResult<List<SangforImage>> imageResult = imageProvider.list(params);

        if (imageResult.isFailed()) {
            return new BocloudResult(true, Collections.emptyList(), "sucess");
        }
        List<SangforImage> isoImages = imageResult.getData();
        List<SangforIsosModel> isoModels = convertImageToIsoModel(isoImages);

        return new BocloudResult(true, isoModels, "success");
    }


    private List<SangforIsosModel> convertImageToIsoModel(List<SangforImage> isoImages) {
        if (isoImages == null || isoImages.isEmpty()) {
            return Collections.emptyList();
        }

        List<SangforIsosModel> isoModelList = new ArrayList<>(isoImages.size());

        SangforIsosModel isoModel;
        Double sizeMb;
        Long total;
        JSONArray disks;
        JSONObject disk;
        String storageId;
        for (SangforImage image : isoImages) {
            isoModel = new SangforIsosModel();
            isoModel.setImageUuid(image.getImageUuid());
            isoModel.setName(image.getName());
            isoModel.setAzs(image.getAzs());

            sizeMb = image.getSizeMb();
            if (sizeMb == null) {
                sizeMb = 0D;
            }
            total = (long) (sizeMb * 1024 * 1024);
            isoModel.setTotal(total);

            disks = image.getDisks();
            if (disks != null && disks.isEmpty()) {
                for (Object obj : disks) {
                    disk = JSON.parseObject(JSON.toJSONString(obj));
                    storageId = disk.getString("storage_id");
                    if (StringUtils.isNotEmpty(storageId)) {
                        isoModel.setStoreid(storageId);
                        break;
                    }
                }
            }

            isoModelList.add(isoModel);
        }

        return isoModelList;
    }
}
