package com.bocloud.cmp.provider.v68.compute;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.model.sangfor.SangforSnapshotModel;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.DateUtils;
import com.bocloud.cmp.provider.common.FastjsonUtils;
import com.bocloud.cmp.provider.compute.SnapshotProvider;
import com.bocloud.cmp.provider.v68.common.TaskProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class SangforSnapshotProvider extends SangforProvider implements SnapshotProvider {

    public SangforSnapshotProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(String instanceId) {
        try {
            Result result = this.doGet(String.format("/janus/20190725/servers/%s/snapshots", instanceId), null);
            List<SangforSnapshotModel> snapshotModels =
                    JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                            SangforSnapshotModel.class);
            if (result.isSuccess()) {
                return new BocloudResult<>(true, snapShotConvert(snapshotModels), "创建云主机快照成功");
            } else {
                return new BocloudResult<>(false,
                        JSONObject.parseObject(result.getMessage()).getString("message"));
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, "创建云主机失败");
        }
    }

    @Override
    public BocloudResult create(SnapshotModel model) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("name", model.getName());
            params.put("description", model.getRemark());
            params.put("snap_type", model.getSnapshotType());
            model.setServerUri(model.getInstanceId());
            Result result = this.doPost(String.format("/janus/20190725/servers/%s/snapshots", model.getServerUri()), params, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            return new BocloudResult(true, "创建云主机快照成功");
                        } else {
                            return new BocloudResult(false, "创建云主机快照失败，" + taskResult.getMessage());
                        }
                    } else {
                        return new BocloudResult(false, "创建云主机快照失败，返回数据为:" + text);
                    }
                } else {
                    return new BocloudResult(false, "创建云主机快照失败，原因为：" + sfResult.getMessage());
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "创建云主机快照失败，失败原因：" + errorMessage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, "创建云主机失败");
        }
    }

    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
            Result result = this.doDelete(String.format("/janus/20180725/servers/%s/snapshots/%s", model.getInstanceId(), model.getSnapshotUuid()),
                    null, "application/json", PostDataFormat.RAW);
            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            return new BocloudResult(true, "云主机快照删除成功");
                        } else {
                            return new BocloudResult(false, "云主机快照删除失败，" + taskResult.getMessage());
                        }
                    } else {
                        return new BocloudResult(false, "云主机快照删除失败，返回数据为:" + text);
                    }
                } else {
                    return new BocloudResult(false, "云主机快照删除失败，原因为：" + sfResult.getMessage());
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "云主机快照删除失败，失败原因：" + errorMessage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult(false, "云快照删除失败");

        }
    }


    @Override
    public BocloudResult revert(SnapshotModel model) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("backup", model.getNeedCreateSnap() ? 1 : 0);
            params.put("power_on", model.getAutoStartvm() ? 1 : 0);

            String url = String.format("/janus/20180725/servers/%s/snapshots/%s/recovery", model.getInstanceId(), model.getSnapshotUuid());

            Result result = this.doPost(url, params, "application/json", PostDataFormat.RAW);

            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    String text = JSON.toJSONString(data);
                    JSONObject json = JSON.parseObject(text, JSONObject.class);
                    if (json != null) {
                        String taskId = json.getString("task_id");
                        TaskProvider provider = new TaskProvider(butler);
                        BocloudResult taskResult = provider.waitTask(taskId);
                        if (taskResult.isSuccess()) {
                            return new BocloudResult(true, "云主机快照恢复成功");
                        } else {
                            return new BocloudResult(false, "云主机快照恢复失败，" + taskResult.getMessage());
                        }
                    } else {
                        return new BocloudResult(false, "云主机快照恢复失败，返回数据为:" + text);
                    }
                } else {
                    return new BocloudResult(false, "云主机快照恢复失败，原因为：" + sfResult.getMessage());
                }
            }

            String errorMessage;
            if (sfResult != null) {
                errorMessage = sfResult.getMessage();
            } else {
                errorMessage = message;
            }
            return new BocloudResult(false, "云主机快照恢复失败，失败原因：" + errorMessage);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult<>(false, "云快照恢复失败");
        }
    }


    private List<SangforSnapshotModel> snapShotConvert(List<SangforSnapshotModel> models) {
        try {
            if (ListTool.isEmpty(models)) {
                return new ArrayList<>(0);
            }

            Date createTime;
            List<SangforSnapshotModel> modelList = new ArrayList<>();
            for (SangforSnapshotModel model : models) {
                model.setSnap_id(model.getIdV68());
                model.setParent(model.getParentId());
                model.setSnap_type(model.getType());
                model.setDesc(model.getDescription());

                // 时间格式"2020-12-01T03:17:42Z"
                try {
                    createTime = DateUtils.utcStringToDate(model.getCreatedAt(), DateUtils.FORMAT_UTC);
                } catch (Exception e) {
                    createTime = null;
                    log.warn("将时间{}转化为日期是出错:{}", model.getCreatedAt(), e.getMessage());
                }

                model.setCreate_time(Integer.valueOf((int) (createTime.getTime() / 1000)));
                modelList.add(model);
            }
            return modelList;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new ArrayList<>(0);
        }

    }
}
