package com.bocloud.cmp.provider.v68.network;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.SangforPhyIfModel;
import com.bocloud.cmp.model.sangfor.SangforScpNetWorkModel;
import com.bocloud.cmp.model.sangfor.SangforVlanGroupS;
import com.bocloud.cmp.params.Sangfor.SangforPhyIfParams;
import com.bocloud.cmp.params.Sangfor.SangforScpNetWorkParams;
import com.bocloud.cmp.params.Sangfor.SangforVlanGroupParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.common.FastjsonUtils;
import com.bocloud.cmp.provider.network.NetworkProvider;
import com.bocloud.cmp.provider.v68.model.SangforResult;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import java.util.*;

@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class SangforNetworkProvider extends SangforProvider implements NetworkProvider {

    public SangforNetworkProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            Map<String, Object> params = new HashMap<>();
            Result result = this.doGet("/janus/20180725/classic-bvswitches", null);
            List<SangforScpNetWorkModel> groupModels = new ArrayList<>();
            if (result.isSuccess()) {
                List<SangforScpNetWorkModel> bvNWModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforScpNetWorkModel.class);
                List<SangforScpNetWorkModel> newBvModels = new ArrayList<>();
                for (SangforScpNetWorkModel model : bvNWModels) {
                    SangforScpNetWorkModel netWorkModel = new SangforScpNetWorkModel();
                    BeanUtils.copyProperties(model, netWorkModel);
                    netWorkModel.setNetWorkType("bv");
                    newBvModels.add(netWorkModel);
                }
                groupModels.addAll(newBvModels);
            }

            Result dvResult = this.doGet("/janus/20180725/classic-dvswitches", null);
            if (dvResult.isSuccess()) {
                List<SangforScpNetWorkModel> dvNWModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(dvResult.getMessage()).getString("data")).getString("data"),
                                SangforScpNetWorkModel.class);
                List<SangforScpNetWorkModel> newDvModels = new ArrayList<>();
                for (SangforScpNetWorkModel model : dvNWModels) {
                    SangforScpNetWorkModel netWorkModel = new SangforScpNetWorkModel();
                    BeanUtils.copyProperties(model, netWorkModel);
                    netWorkModel.setNetWorkType("dv");
                    newDvModels.add(netWorkModel);
                }
                groupModels.addAll(newDvModels);
            }
            return new BocloudResult<>(true, netWorkConvert(groupModels), "获取网络列表成功");
        } catch (Exception e) {
            e.printStackTrace();
            return new BocloudResult<>(false, "获取网络列表失败");
        }
    }

    public BocloudResult dvVlanGroupList() {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("page_num", 0);
            params.put("page_size", PAGE_SIZE);
            Result result = this.doGet("/janus/20180725/classic-bvswitches", params);
            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            List<SangforVlanGroupS> vlanGroup;
            List<SangforVlanGroupS> vlanGroupS = new ArrayList<>();

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject dataJson = JSON.parseObject(JSON.toJSONString(data));
                    List<SangforScpNetWorkModel> bvNwModels = JSON.parseArray(dataJson.getString("data"), SangforScpNetWorkModel.class);
                    for (SangforScpNetWorkModel model : bvNwModels) {
                        vlanGroup = model.getVlan_group();
                        for (SangforVlanGroupS vlan : vlanGroup) {
                            SangforVlanGroupS vlanModels = new SangforVlanGroupS();
                            BeanUtils.copyProperties(vlan, vlanModels);
                            vlanModels.setNetworkUuid(model.getId());
                            vlanGroupS.add(vlanModels);
                        }
                    }

                    Integer totalSize = dataJson.getInteger("total_size");
                    String nextPageString = dataJson.getString("next_page_num");

                    Integer totalPage, nextPage;
                    // 下一页数，整型的字符串，为""表示当前是最后一页
                    if (StringUtils.isEmpty(nextPageString)) {
                        nextPage = 0;
                        totalPage = 0;
                    } else {
                        nextPage = dataJson.getInteger("next_page_num");
                        totalPage = totalSize / PAGE_SIZE;
                        if (totalSize % PAGE_SIZE > 0) {
                            totalPage++;
                        }
                    }

                    if (totalPage > 0) {
                        while (nextPage < totalPage) {
                            params.clear();
                            params.put("page_num", nextPage);
                            params.put("page_size", PAGE_SIZE);
                            result = this.doGet("/janus/20180725/classic-bvswitches", params);
                            message = result.getMessage();
                            sfResult = null;
                            if (FastjsonUtils.isJSONObject(message)) {
                                sfResult = JSON.parseObject(message, SangforResult.class);
                            }
                            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                                    data = sfResult.getData();
                                    dataJson = JSON.parseObject(JSON.toJSONString(data));
                                    bvNwModels = JSON.parseArray(dataJson.getString("data"), SangforScpNetWorkModel.class);
                                    for (SangforScpNetWorkModel model : bvNwModels) {
                                        vlanGroup = model.getVlan_group();
                                        for (SangforVlanGroupS vlan : vlanGroup) {
                                            SangforVlanGroupS vlanModels = new SangforVlanGroupS();
                                            BeanUtils.copyProperties(vlan, vlanModels);
                                            vlanModels.setNetworkUuid(model.getId());
                                            vlanGroupS.add(vlanModels);
                                        }
                                    }
                                }
                            }
                            nextPage++;
                        }
                    }
                }
            }
            return new BocloudResult<>(true, vlanGroupConvert(vlanGroupS), "查询vlan成功");
        } catch (Exception e) {
            return new BocloudResult<>(false, "查询端口组失败");
        }
    }

    public BocloudResult dvPhyIfList() {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("page_num", 0);
            params.put("page_size", PAGE_SIZE);
            Result result = this.doGet("/janus/20180725/classic-bvswitches", params);
            String message = result.getMessage();
            SangforResult sfResult = null;
            if (FastjsonUtils.isJSONObject(message)) {
                sfResult = JSON.parseObject(message, SangforResult.class);
            }

            List<SangforPhyIfModel> phyIfList;
            List<SangforPhyIfModel> phyIfModels = new ArrayList<>();

            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                    Object data = sfResult.getData();
                    JSONObject dataJson = JSON.parseObject(JSON.toJSONString(data));
                    List<SangforScpNetWorkModel> bvNwModels = JSON.parseArray(dataJson.getString("data"), SangforScpNetWorkModel.class);
                    for (SangforScpNetWorkModel model : bvNwModels) {
                        phyIfList = model.getPhy_if();
                        for (SangforPhyIfModel phyIf : phyIfList) {
                            SangforPhyIfModel models = new SangforPhyIfModel();
                            BeanUtils.copyProperties(phyIf, models);
                            models.setNetworkUuid(model.getId());
                            phyIfModels.add(models);
                        }
                    }

                    Integer totalSize = dataJson.getInteger("total_size");
                    String nextPageString = dataJson.getString("next_page_num");

                    Integer totalPage, nextPage;
                    // 下一页数，整型的字符串，为""表示当前是最后一页
                    if (StringUtils.isEmpty(nextPageString)) {
                        nextPage = 0;
                        totalPage = 0;
                    } else {
                        nextPage = dataJson.getInteger("next_page_num");
                        totalPage = totalSize / PAGE_SIZE;
                        if (totalSize % PAGE_SIZE > 0) {
                            totalPage++;
                        }
                    }

                    if (totalPage > 0) {
                        while (nextPage < totalPage) {
                            params.clear();
                            params.put("page_num", nextPage);
                            params.put("page_size", PAGE_SIZE);
                            result = this.doGet("/janus/20180725/classic-bvswitches", params);
                            message = result.getMessage();
                            sfResult = null;
                            if (FastjsonUtils.isJSONObject(message)) {
                                sfResult = JSON.parseObject(message, SangforResult.class);
                            }
                            if (result.isSuccess() && !StringUtils.isEmpty(message)) {
                                if (sfResult != null && SangforResult.SUCCESS_CODE.equals(sfResult.getCode())) {
                                    data = sfResult.getData();
                                    dataJson = JSON.parseObject(JSON.toJSONString(data));
                                    bvNwModels = JSON.parseArray(dataJson.getString("data"), SangforScpNetWorkModel.class);
                                    for (SangforScpNetWorkModel model : bvNwModels) {
                                        phyIfList = model.getPhy_if();
                                        for (SangforPhyIfModel phyIf : phyIfList) {
                                            SangforPhyIfModel models = new SangforPhyIfModel();
                                            BeanUtils.copyProperties(phyIf, models);
                                            models.setNetworkUuid(model.getId());
                                            phyIfModels.add(models);
                                        }
                                    }
                                }
                            }
                            nextPage++;
                        }
                    }
                }
            }
            return new BocloudResult<>(true, phyIfConvert(phyIfModels), "查询端口组成功");
        } catch (Exception e) {
            return new BocloudResult<>(false, "查询端口组失败");
        }
    }

    private List<SangforScpNetWorkParams> netWorkConvert(List<SangforScpNetWorkModel> modelList) {
        if (CollectionUtils.isEmpty(modelList)) {
            return null;
        }
        List<SangforScpNetWorkParams> netWorkParams = new ArrayList<>();
        for (SangforScpNetWorkModel model : modelList) {
            SangforScpNetWorkParams params = new SangforScpNetWorkParams();
            params.setNetworkUuid(model.getId());
            params.setNetWorkType(model.getNetWorkType());
            params.setAzId(model.getAz_id());
            params.setRemark(model.getDescription());
            params.setEnable(model.getEnable());
            params.setName(model.getName());
            params.setProjectId(model.getProject_id());
            netWorkParams.add(params);
        }
        return netWorkParams;
    }

    private List<SangforVlanGroupParams> vlanGroupConvert(List<SangforVlanGroupS> vlanGroupList) {
        if (ListTool.isEmpty(vlanGroupList)) {
            return null;
        }
        List<SangforVlanGroupParams> paramsList = new ArrayList<>();
        for (SangforVlanGroupS groupS : vlanGroupList) {
            SangforVlanGroupParams params = new SangforVlanGroupParams();
            params.setUuid(groupS.getId());
            params.setName(groupS.getName());
            params.setCount(groupS.getCount());
            params.setVlanId(groupS.getVlan_id());
            params.setVlanRange(groupS.getVlan_range());
            params.setPvid(groupS.getPvid());
            params.setType(groupS.getType());
            params.setNetworkUuid(groupS.getNetworkUuid());
            paramsList.add(params);
        }
        return paramsList;
    }

    private List<SangforPhyIfParams> phyIfConvert(List<SangforPhyIfModel> phyIfModels) {
        if (ListTool.isEmpty(phyIfModels)) {
            return null;
        }
        List<SangforPhyIfParams> phyIfParams = new ArrayList<>();
        for (SangforPhyIfModel model : phyIfModels) {
            SangforPhyIfParams params = new SangforPhyIfParams();
            params.setType(model.getType());
            params.setVlanId(model.getVlan_id());
            params.setHostId(model.getHost_id());
            params.setPortUuid(model.getPort());
            params.setStatus(String.valueOf(model.getStatus()));
            params.setNetworkUuid(model.getNetworkUuid());
            phyIfParams.add(params);
        }
        return phyIfParams;
    }

    /**
     * 随机生成MAC地址
     *
     * @author sxp
     * @since 2023/5/30
     */
    @Override
    public BocloudResult listMac() {
        String mac = randomMacAddress();
        return new BocloudResult(true, mac, "获取mac地址成功");
    }

    /**
     * 随机生成MAC地址
     * <p>
     * API中没有【随机生成MAC地址】的接口，自己写一个。
     * MAC地址的长度为48位(6个字节)，通常表示为12个16进制数，如：00-16-ea-ae-3c-40就是一个MAC地址，其中前3个字节，
     * 16进制数00-16-ea代表网络硬件制造商的编号，它由IEEE(电气与电子工程师协会)分配，而后3个字节，16进制数ae-3c-40
     * 代表该制造商所制造的某个网络产品(如网卡)的系列号。
     *
     * 单播MAC：第一字节第8个比特为0。
     * </p>
     *
     * @author sxp
     * @since 2023/5/30
     */
    private static String randomMacAddress() {
        Random random = new Random();
        StringBuilder sb = new StringBuilder();

        int value;
        String element;
        for (int i = 0; i < 6; i++) {
            if (sb.length() > 0) {
                sb.append(":");
            }

            value = random.nextInt(256);
            // 单播MAC：第一字节第8个比特为0 sxp:2023.6.5
            if (i == 0) {
                if (value % 2 != 0) {
                    value = value + 1;// 设置为偶数
                    if (value >= 256) {
                        value = 254;
                    }
                }
            }

            element = Integer.toHexString(value);
            if (element.length() < 2) {
                sb.append(0);
            }

            sb.append(element);
        }

        //return sb.toString().toUpperCase();
        // 6.8接口会校验MAC地址（字母小写，以:号分隔）
        return sb.toString();
    }

}
