package com.bocloud.cmp.provider.v68.security;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FirewallModel;
import com.bocloud.cmp.model.sangfor.SangForServerGroupModel;
import com.bocloud.cmp.model.sangfor.SangforFirewallRuleModel;
import com.bocloud.cmp.params.Sangfor.SangforFireWallParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.security.FirewallProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.omg.CORBA.PUBLIC_MEMBER;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Provider("bocloud.cmp.sangfor.v68")
public class SangforFileWallProvider extends SangforProvider implements FirewallProvider {

    public SangforFileWallProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list(){
        try {
            Result result = this.doGet("/janus/20180725/dfw-policies", null);
            List<SangforFirewallRuleModel> firewallModelList = new ArrayList<>();
            if (result.isSuccess()){
                List<SangforFirewallRuleModel> fireWallModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforFirewallRuleModel.class);
                firewallModelList.addAll(fireWallModels);
            }
            return new BocloudResult<>(true,fireWallConvert(firewallModelList),"获取防火墙列表成功");
        }catch (Exception e){
            e.printStackTrace();
            return new BocloudResult<>(false,"获取防火墙列表失败");
        }
    }

    @Override
    public BocloudResult createV68(SangforFireWallParams model){
        try {
            Map<String, Object> params =new HashMap<>();
            params.put("name",model.getName());
            params.put("network_type",model.getNetworkType());
            params.put("az_id",model.getAzId());
            params.put("project_id",model.getSangforProjectId());
            Result result = this.doPost("/janus/20180725/dfw-policies", params,"application/json", PostDataFormat.RAW);
            List<SangforFirewallRuleModel> firewallModelList = new ArrayList<>();
            if (result.isSuccess()){
                SangforFirewallRuleModel ruleModel = JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data"),
                        SangforFirewallRuleModel.class);
                result = this.doGet("/janus/20180725/dfw-policies/"+ruleModel.getId(), null);
                if (result.isSuccess()){
                    SangforFirewallRuleModel fireWallModel = JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data"),
                            SangforFirewallRuleModel.class);
                    firewallModelList.add(fireWallModel);
                }
            }
            return new BocloudResult<>(true,fireWallConvert(firewallModelList),"创建防火墙策略成功");
        }catch (Exception e){
            e.printStackTrace();
            return new BocloudResult<>(false,"创建防火墙策略失败");
        }
    }

    @Override
    public BocloudResult remove(String firewallId){
        try {
            Result result = this.doDelete("/janus/20180725/dfw-policies/" + firewallId, null, "application/json", PostDataFormat.RAW);
            if (result.isSuccess()){
                return new BocloudResult<>(true,"删除防火墙成功");
            }else {
                return new BocloudResult<>(false,
                        JSONObject.parseObject(result.getMessage()).getString("message"));
            }
        }catch (Exception e){
            e.printStackTrace();
            return new BocloudResult<>(false,"删除防火墙失败");
        }
    }

    @Override
    public BocloudResult updateV68(SangforFireWallParams firewall) {
        try {
            Map<String, Object> params =new HashMap<>();
            params.put("name",firewall.getName());
//            params.put("az_id",firewall.getAzId());
//            params.put("project_id",firewall.getSangforProjectId());
            Result result = this.doPut("/janus/20180725/dfw-policies/" + firewall.getSangforPolicyId(), params, "application/json", PostDataFormat.RAW);
            List<SangforFirewallRuleModel> firewallModelList = new ArrayList<>();
            if (result.isSuccess()){
                result = this.doGet("/janus/20180725/dfw-policies/"+firewall.getSangforPolicyId(), null);
                if (result.isSuccess()){
                    SangforFirewallRuleModel ruleModel = JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data"),
                            SangforFirewallRuleModel.class);
                    firewallModelList.add(ruleModel);
                }
                return new BocloudResult<>(true,fireWallConvert(firewallModelList),"更新防火墙成功");
            }else {
                return new BocloudResult<>(false,
                        JSONObject.parseObject(result.getMessage()).getString("message"));
            }
        }catch (Exception e){
            e.printStackTrace();
            return new BocloudResult<>(false,"更新防火墙失败");
        }
    }

    private List<SangforFireWallParams> fireWallConvert(List<SangforFirewallRuleModel> models){
        if (ListTool.isEmpty(models)){
            return null;
        }
        List<SangforFireWallParams> fireWallParamsList = new ArrayList<>();
        for (SangforFirewallRuleModel model : models){
            SangforFireWallParams params = new SangforFireWallParams();
            params.setSangforPolicyId(model.getId());
            params.setPolicyType(model.getPolicy_type());
            params.setName(model.getName());
            params.setSangforProjectId(model.getProject_id());
            params.setAzId(model.getAz_id());
            params.setRuleIds("[\"" + String.join("\",\"", model.getRule_ids()) + "\"]");
            params.setNetworkType(model.getNetwork_type());
            params.setVpcId(model.getVpc_id());
            fireWallParamsList.add(params);
        }
        return fireWallParamsList;
    }
}
