package com.bocloud.cmp.provider.v68.storage;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.sangfor.SangforAzModel;
import com.bocloud.cmp.model.sangfor.SangforScpHostModel;
import com.bocloud.cmp.model.sangfor.SangforScpStorageModel;
import com.bocloud.cmp.params.Sangfor.SangforStorageParams;
import com.bocloud.cmp.provider.SangforProvider;
import com.bocloud.cmp.provider.storage.StorageServerProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Slf4j
@Provider("bocloud.cmp.sangfor.v68")
public class SangforStorageProvider extends SangforProvider implements StorageServerProvider {

    public SangforStorageProvider(Butler butler) {
        super(butler);
    }

    /**
     * * 获取虚拟机存储列表,Vs代表虚拟存储类型
     *
     * @return
     */
    @Override
    public BocloudResult<List<SangforStorageParams>> listServerStorage() {
        return this.listServerStorage(new HashMap<>());
    }

    public BocloudResult<List<SangforStorageParams>> listServerStorage(Map<String, Object> storagesParams) {
        try {
            if (storagesParams == null) {
                storagesParams = new HashMap<>();
            }

            List<SangforScpStorageModel> storageList = new ArrayList<>();

            Map<String, Object> params = new HashMap<>();
            params.put("type", "hci");
            Result result = this.doGet("/janus/20180725/azs", params);
            if (result.isSuccess()) {
                List<SangforAzModel> sangforAzModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforAzModel.class);

                for (SangforAzModel model : sangforAzModels) {
                    storagesParams.put("az_id", model.getId());
                    result = this.doGet("/janus/20200725/storages", storagesParams);
                    if (result.isSuccess()) {
                        List<SangforScpStorageModel> storageModels =
                                JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                        SangforScpStorageModel.class);
                        storageList.addAll(storageModels);
                    }
                }
            }
            return new BocloudResult<>(true, storageConver(storageList), "获取所有存储列表成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new BocloudResult(false, "获取所有存储列表失败");
        }

    }

    /**
     * 查询某个指定的存储
     *
     * @author sxp
     * @since 2023/5/31
     */
    public BocloudResult<SangforStorageParams> detailStorage(String storageUuid) {
        Map<String, Object> storagesParams = new HashMap<>();
        storagesParams.put("storage_ids", storageUuid);
        BocloudResult<List<SangforStorageParams>> result = this.listServerStorage(storagesParams);
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        List<SangforStorageParams> storageParamsList = result.getData();
        if (storageParamsList == null || storageParamsList.isEmpty()) {
            return new BocloudResult<>(false, null, "未查询到唯一标识为" + storageUuid + "的存储");
        }
        SangforStorageParams storageParams = storageParamsList.get(0);
        return new BocloudResult<>(true, storageParams, "success");
    }

    //物理主机系统盘
    @Override
    public BocloudResult sangforSystemDisk() {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("type", "hci");
            Result result = this.doGet("/janus/20180725/azs", params);
            List<SangforScpHostModel> serverList = new ArrayList<>();
            if (result.isSuccess()) {
                //数据可能存在这里
                List<SangforAzModel> sangforAzModels =
                        JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result.getMessage()).getString("data")).getString("data"),
                                SangforAzModel.class);
                //假设查询的hci资源池的
                //物理机集合
                Map<String, Object> physicalMachineParams = new HashMap<>();
                for (int i = 0; i < sangforAzModels.size(); i++) {
                    //获取到其中一条数据
                    physicalMachineParams.clear();
                    physicalMachineParams.put("az_id", sangforAzModels.get(i).getId());
                    Result result1 = this.doGet("/janus/20190725/hosts", physicalMachineParams);
                    if (result1.isSuccess()) {
                        List<SangforScpHostModel> sangforHostModels =
                                JSONObject.parseArray(JSONObject.parseObject(JSONObject.parseObject(result1.getMessage()).getString("data")).getString("data"), SangforScpHostModel.class);
                        serverList.addAll(sangforHostModels);
                    }
                }
            }
            List<SangforScpStorageModel> storageModels = new ArrayList<>();
            for (SangforScpHostModel model : serverList) {
                SangforScpStorageModel storageModel = new SangforScpStorageModel();
                SangforScpStorageModel system_disk = model.getSystem_disk();
                BeanUtils.copyProperties(system_disk, storageModel);
                storageModel.setAz_id(model.getAz_id());
                storageModel.setName(model.getName());
                storageModel.setSangforType(system_disk.getType());
                storageModel.setHost(model.getId());
                storageModel.setGuid(system_disk.getUuid());
                storageModels.add(storageModel);
            }
            return new BocloudResult<>(true, systemDiskConver(storageModels), "获取物理机系统盘成功");
        } catch (Exception e) {
            e.printStackTrace();
            return new BocloudResult<>(false, "获取物理机系统盘失败");
        }
    }

    private List<SangforStorageParams> systemDiskConver(List<SangforScpStorageModel> storageModels) {
        if (CollectionUtils.isEmpty(storageModels)) {
            return null;
        }
        List<SangforStorageParams> paramsList = new ArrayList<>();
        for (SangforScpStorageModel model : storageModels) {
            SangforStorageParams params = new SangforStorageParams();
            params.setStatus(model.getStatus());
            params.setType(model.getType());
            params.setUuid(model.getUuid());
            params.setName(model.getName());
            params.setAzId(model.getAz_id());
            params.setHost(model.getHost());
            params.setSangforType(model.getSangforType());
            params.setGuid(model.getGuid());
            params.setIsSangforSystemDisk("sangforSystem");
            params.setOnlineStatus(model.getOnline_status());
            params.setSsdRemainingLife(model.getSsd_remaining_life());
            paramsList.add(params);
        }
        return paramsList;
    }

    private List<SangforStorageParams> storageConver(List<SangforScpStorageModel> storageList) {
        if (CollectionUtils.isEmpty(storageList)) {
            return null;
        }
        List<SangforStorageParams> paramsList = new ArrayList<>();
        for (SangforScpStorageModel model : storageList) {
            SangforStorageParams params = new SangforStorageParams();
            BeanUtils.copyProperties(model, params);
            BigDecimal bgTotal = new BigDecimal(model.getTotal_mb());
            BigDecimal bgSize = new BigDecimal(1024);
            BigDecimal newTotal = bgTotal.divide(bgSize, 0, RoundingMode.UP);

            long freeMb = model.getTotal_mb() - model.getUsed_mb();
            BigDecimal freeTotal = new BigDecimal(freeMb);
            BigDecimal newFreeTotal = freeTotal.divide(bgSize, 0, RoundingMode.UP);

            BigDecimal maxReadBy = new BigDecimal(model.getMax_read_byteps());
            BigDecimal maxReadWrSize = new BigDecimal(1024 * 1024);
            BigDecimal maxWriteby = new BigDecimal(model.getMax_write_byteps());

            BigDecimal newIoRead = maxReadBy.divide(maxReadWrSize, 0, RoundingMode.UP);
            BigDecimal newIoWrRead = maxWriteby.divide(maxReadWrSize, 0, RoundingMode.UP);

            params.setHostNum(Integer.valueOf(model.getConnected_hosts()));//连接主机数量
            params.setIoRead(Integer.valueOf(newIoRead.toString()));//最大Io读速率
            params.setIoWrite(Integer.valueOf(newIoWrRead.toString()));//最大Io读写速率

            params.setUuid(model.getId());
            params.setStorageTagId(model.getStorage_tag_id());
            params.setStorageTagName(model.getStorage_tag_name());
            params.setStorageLabel(model.getStorage_label());
            params.setClusterId(model.getCluster_id());
            params.setClusterName(model.getCluster_name());
            params.setClusterType(model.getCluster_type());
            params.setAzId(model.getAz_id());
            params.setCapacity(Long.valueOf(newTotal.toString()));//总容量
            params.setUsedMb(model.getUsed_mb());
            params.setFreeSpace(Long.valueOf(newFreeTotal.toString()));//可用容量
            String type = null;
            if (!model.getType().equals("vs")){
                type="OTHER";
            }else {
                type="VS";

            }
            params.setSangforType(type);//磁盘类型
            String status = model.getStatus();
            if (model.getStatus().equals("normal")) {
                status = "active";
            }
            params.setStatus(status);//状态
            params.setName(model.getName());//名称
            params.setVersion("6.8");
            params.setRatio(String.valueOf(model.getRatio()));
            params.setOperationalStatus(model.getStatus());
            paramsList.add(params);
        }
        return paramsList;
    }
}
