package com.bocloud.cmp.provider.v68.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.enums.BocloudStatus;
import com.bocloud.cmp.model.sangfor.SangforNetwork;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * SCP 6.8 虚拟机辅助类
 *
 * @author sxp
 * @since 2023/5/26
 */
public class SangForServerHelper {
    /**
     * 虚拟机状态
     */
    private static Map<String, String> vmStatusMap = new HashMap<>();
    /**
     * 操作系统名称
     */
    private static Map<String, String> osNameMap = new HashMap<>();

    private static final String UNKNOWN_OS_NAME = "未知";

    static {
        vmStatusMap.put("running", BocloudStatus.Server.RUNNING.name());// 正在运行
        vmStatusMap.put("suspend", BocloudStatus.Server.SUSPENDED.name()); // 已挂起
        vmStatusMap.put("stopped", BocloudStatus.Server.STOPPED.name()); // 已关机
        vmStatusMap.put("alert", BocloudStatus.Server.RUNNING.name());  // 警告
        vmStatusMap.put("overdue", BocloudStatus.Server.EXPIRED.name()); // 过期的
        vmStatusMap.put("lost", BocloudStatus.Server.LOST.name());  // /存储丢失
        vmStatusMap.put("paused_io_error", BocloudStatus.Server.ERROR.name()); // 持续IO错误
        vmStatusMap.put("starting", BocloudStatus.Server.STARTING.name()); // 正在开机
        vmStatusMap.put("stoping", BocloudStatus.Server.STOPPING.name()); // 正在关机
        vmStatusMap.put("tpl_lost", BocloudStatus.Server.RUNNING.name()); // 模板丢失
        vmStatusMap.put("clone", BocloudStatus.Server.CLONE.name());// 正在克隆
        vmStatusMap.put("backup", BocloudStatus.Server.BACKUP.name());// 正在备份
        vmStatusMap.put("recovery", BocloudStatus.Server.RECOVERING.name()); // 正在恢复
        vmStatusMap.put("run_backup", BocloudStatus.Server.RUNNING.name()); // 正在备份
        vmStatusMap.put("resetting", BocloudStatus.Server.RESETTING.name()); // 正在重启
        vmStatusMap.put("suspending", BocloudStatus.Server.SUSPENDING.name()); // 正在挂起
        vmStatusMap.put("updating", BocloudStatus.Server.UPDATING.name()); // 正在更新
        vmStatusMap.put("deleting", BocloudStatus.Server.DELETING.name()); // 正在删除
        vmStatusMap.put("snapshotting", BocloudStatus.Server.SNAPSHOTTING.name()); // 正在快照
        vmStatusMap.put("creating", BocloudStatus.Server.BUILDING.name()); // 正在创建
        vmStatusMap.put("migrate", BocloudStatus.Server.MIGRATE.name()); // 正在迁移
        vmStatusMap.put("deriving", BocloudStatus.Server.BUILDING.name()); // 正在部署
        vmStatusMap.put("tpl_updating", BocloudStatus.Server.UPDATING.name()); // 正在更新
        vmStatusMap.put("tpl_moving", BocloudStatus.Server.UPDATING.name()); // 准备回拷
        vmStatusMap.put("tpl_syncing", BocloudStatus.Server.UPDATING.name()); // 模板回拷
        vmStatusMap.put("offline", BocloudStatus.Server.STOPPED.name()); // 已经离线
    }
    static {
        osNameMap.put("linux-centos", "CentOS");
        osNameMap.put("linux-suse", "SUSE Linux");
        osNameMap.put("linux-oracle", "Oracle Linux");
        osNameMap.put("l26", "Linux 3.X/2.6 Kernel（32位）");
        osNameMap.put("l2664", "Linux 3.X/2.6 Kernel（64位）");
        osNameMap.put("l24", "Linux 2.4 Kernel（32位）");
        osNameMap.put("l2464", "Linux 2.4 Kernel（64位）");
        osNameMap.put("linux-freebsd", "FreeBSD");
        osNameMap.put("linux-redflag", "红旗Linux");
        osNameMap.put("linux-neokylin", "中标麒麟");
        osNameMap.put("linux-neoshine", "中标普华Linux ");
        osNameMap.put("linux-deepin", "深度Linux ");
        osNameMap.put("linux-ubuntu", "Ubuntu");
        osNameMap.put("linux-debian", "Debian GNU/Linux");
        osNameMap.put("linux-redhat", "Red Hat Enterprise Linux");
        osNameMap.put("linux-solaris", "Solaris");
        osNameMap.put("linux-fedora", "Fedora");
        osNameMap.put("linux-kylinos", "银河麒麟 ");
        osNameMap.put("linux-arm", "Linux ARM ");
        osNameMap.put("ws1964", "Windows Server 2019（64位）");
        osNameMap.put("ws1664", "Windows Server 2016（64位）");
        osNameMap.put("ws1264", "Windows Server 2012（64位）");
        osNameMap.put("ws08", "Windows Server 2008（32位）");
        osNameMap.put("ws0864", "Windows Server 2008（64位）");
        osNameMap.put("ws03", "Windows Server 2003（32位）");
        osNameMap.put("ws0364", "Windows Server 2003（64位）");
        osNameMap.put("win10", "Microsoft Windows 10（32位）");
        osNameMap.put("win1064", "Microsoft Windows 10（64位）");
        osNameMap.put("win8", " Microsoft Windows 8（32位）");
        osNameMap.put("win864", "Microsoft Windows 8（64位）");
        osNameMap.put("win7", " Microsoft Windows 7（32位）");
        osNameMap.put("win764", "Microsoft Windows 7（64位）");
        osNameMap.put("wxp", "Microsoft Windows XP（32位）");
        osNameMap.put("wxp64", "Microsoft Windows XP（64位）");
        osNameMap.put("w2k", "Microsoft Windows 2000");
        osNameMap.put("other", "32位系统");
        osNameMap.put("other64", "64位系统");
    }

    /**
     * 根据深信服接口中的状态获取博云虚拟机的状态
     *
     * @param status 从接口中获取得到的虚拟机状态
     * @author sxp
     * @since 2023/5/26
     */
    public static String getCloudServerStatus(String status) {
        if (StringUtils.isEmpty(status)) {
            return BocloudStatus.Server.UNKNOWN.name();
        }
        String cloudServerStatus = vmStatusMap.get(status);
        if (StringUtils.isEmpty(cloudServerStatus)) {
            return BocloudStatus.Server.UNKNOWN.name();
        }

        return cloudServerStatus;
    }

    /**
     * 根据简称获取详细的操作系统名称
     *
     * @author sxp
     * @since 2023/5/26
     */
    public static String getDetailOsName(String osName) {
        if (StringUtils.isEmpty(osName)) {
            return osName;
        }
        String newOsName = osNameMap.get(osName);
        if (StringUtils.isEmpty(newOsName)) {
            return osName;
        }

        return newOsName;
    }


    /**
     * 将IP数组转换为具备以下格式的IP
     * <p>
     * {"net0":{"ip1":1},"net1":{"ip2":1}}
     * 或者
     * {"net0":{"ip1":1,"ip2":1}}
     * </p>
     *
     * @author sxp
     * @since 2023/5/26
     */
    public static String getPrivateIps(List<String> ips, List<SangforNetwork> networks) {
        if (ips == null || ips.isEmpty()) {
            return "{}";
        }
        int size = ips.size();

        if (networks == null) {
            networks = new ArrayList<>(0);
        }
        int networkCount = networks.size();

        JSONObject result = new JSONObject();
        if (size == 1) {
            JSONObject valueJson = new JSONObject();
            valueJson.put(ips.get(0), 1);
            result.put("net0", valueJson);
        } else {
            Map<String, Integer> networkMap = new HashMap<>();
            for (SangforNetwork network : networks) {
                networkMap.put(network.getIpAddress(), network.getConnect());
            }

            int connect;
            if (networkCount == 1) {
                JSONObject valueJson = new JSONObject();
                for (String ip : ips) {
                    connect = 1;
                    if (networkMap.containsKey(ip)) {
                        connect = networkMap.get(ip);
                    }
                    valueJson.put(ip, connect);
                }
                result.put("net0", valueJson);
            } else {
                String key;
                int index = 0;
                JSONObject valueJson;
                for (String ip : ips) {
                    key = "net" + index;
                    valueJson = new JSONObject();
                    connect = 1;
                    if (networkMap.containsKey(ip)) {
                        connect = networkMap.get(ip);
                    }
                    valueJson.put(ip, connect);
                    result.put(key, valueJson);
                    index++;
                }
            }
        }

        return JSON.toJSONString(result);

    }
}
