package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.ClusterModel;
import com.bocloud.cmp.provider.smart.sdk.model.ComputeSummaryModel;
import com.bocloud.cmp.provider.smart.sdk.model.MetricModel;
import com.bocloud.cmp.provider.smart.sdk.model.StorageSummaryModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.ClusterCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.ClusterModifyModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;

import java.util.List;

public class ClusterApi extends SmartApi {
    
    public ClusterApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public List<ClusterModel> list(String view) {
        // BASIC： 基本视图，含集群的主要信息，FULL： 完整视图， 集群更详细的信息
        String url = this.getSessionParams().getHost() + Constant.URL_CLUSTERS;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response).getJSONArray("clusters").toJavaList(ClusterModel.class);
    }

    public ClusterModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_CLUSTER, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, ClusterModel.class);
    }

    public ClusterModel create(ClusterCreateModel param) {
        String url = this.getSessionParams().getHost() + Constant.URL_CLUSTERS;
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response, ClusterModel.class);
    }

    public ClusterModel modify(String uuid, ClusterModifyModel param) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_CLUSTER, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "PUT", entity);
        return JSON.parseObject(response, ClusterModel.class);
    }

    public void remove(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_CLUSTER, uuid);
        this.getDefaultHttpInvoke().request(url, "DELETE");
    }

    public StorageSummaryModel storageSummary() {
        String url = this.getSessionParams().getHost() + Constant.URL_CLUSTER_STORAGE;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, StorageSummaryModel.class);
    }

    public ComputeSummaryModel computeSummary() {
        String url = this.getSessionParams().getHost() + Constant.URL_COMPUTE_SUMMARY;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, ComputeSummaryModel.class);
    }

    public MetricModel metric(String metric) {
        String url = this.getSessionParams().getHost() + Constant.URL_METRICS_QUERY + "?query=" + metric;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, MetricModel.class);
    }

}
