package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.http.*;
import com.bocloud.cmp.provider.smart.sdk.model.ImageModel;
import com.bocloud.cmp.provider.smart.sdk.model.ImageStatsModel;
import com.bocloud.cmp.provider.smart.sdk.model.ImageUploadModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.ImageCreateModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class ImageApi extends SmartApi {

    private static final Logger logger = LoggerFactory.getLogger(ImageApi.class);

    public ImageApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public List<ImageModel> list() {
        String url = this.getSessionParams().getHost() + Constant.URL_IMAGES;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response).getJSONArray("images").toJavaList(ImageModel.class);
    }

    public ImageModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_IMAGE, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response).getJSONObject("image").toJavaObject(ImageModel.class);
    }

    public ImageStatsModel stats(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_IMAGE_STATS, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, ImageStatsModel.class);
    }

    public ImageUploadModel create(ImageCreateModel param) {
        String url = this.getSessionParams().getHost() + Constant.URL_IMAGE_UPLOAD_VOLUME;
        url += "?" + URLEncodedUtils.format(param.toNameValuePairs(), StandardCharsets.UTF_8);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response, ImageUploadModel.class);
    }

    public ImageUploadModel upload(byte[] bytes, String zbsVolumeUuid, String imageUuid, Long chunkNum) {
        HttpInvoker httpInvoker;
        String debug = System.getProperty("smart.debug", Boolean.FALSE.toString());
        if (Boolean.TRUE.toString().equals(debug)) {
            httpInvoker = new SmartInvoke(new ThrowsInvoke(new DebugInvoke()));
        } else {
            httpInvoker = new SmartInvoke(new SessionInvoker(new ThrowsInvoke(new ClientInvoker()), this.getSessionParams()));
        }
        String url = this.getSessionParams().getHost() + Constant.URL_IMAGE_UPLOAD;
        List<NameValuePair> nameValuePairs = new ArrayList<>();
        nameValuePairs.add(new BasicNameValuePair("zbs_volume_id", zbsVolumeUuid));
        nameValuePairs.add(new BasicNameValuePair("image_uuid", imageUuid));
        nameValuePairs.add(new BasicNameValuePair("chunk_num", chunkNum.toString()));
        url += "?" + URLEncodedUtils.format(nameValuePairs, StandardCharsets.UTF_8);
        HttpEntity entity = MultipartEntityBuilder.create().addBinaryBody("file", bytes, ContentType.DEFAULT_BINARY, "file").build();
        logger.debug("========调用Rest API========");
        logger.debug("==>> [{}] {} ", "POST", url);
        String response = httpInvoker.request(url, "POST", entity);
        logger.debug("<<== [RESPONSE] {} ", response);
        return JSON.parseObject(response, ImageUploadModel.class);
    }

    public void modify(String uuid, String name, String description) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_IMAGE, uuid);
        Map<String, Object> param = MapBuilder.init().put("name", name).put("description", description).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        this.getDefaultHttpInvoke().request(url, "PUT", entity);
    }

    public void remove(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_IMAGE, uuid);
        this.getDefaultHttpInvoke().request(url, "DELETE");
    }


}
