package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.http.*;
import com.bocloud.cmp.provider.smart.sdk.model.JobModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class JobApi extends SmartApi {

    public JobApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    @Override
    public HttpInvoker getDefaultHttpInvoke() {
        String debug = System.getProperty("smart.debug", Boolean.FALSE.toString());
        if (Boolean.TRUE.toString().equals(debug)) {
            return  new SmartInvoke(
                    new ThrowsInvoke(
                    new ReplaceInvoke(
                    new ReplaceInvoke(
                    new LoggingInvoker(
                    new DebugInvoke())))));
        } else {
            return  new SmartInvoke(
                    new SessionInvoker(
                    new ThrowsInvoke(
                    new ReplaceInvoke(
                    new LoggingInvoker(
                    new ClientInvoker()))), this.getSessionParams()));
        }
    }

    public List<JobModel> page(long pageSize, long pageNum) {
        String url = this.getSessionParams().getHost() + Constant.URL_JOBS +
                "?page=" + pageNum + "&length=" + pageSize;
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response).getJSONArray("jobs").toJavaList(JobModel.class);
    }

    public List<JobModel> list() {
        final int pageSize = 100;
        int pageNum = 1;
        List<JobModel> list = new ArrayList<>();
        while (true) {
            List<JobModel> pager = this.page(pageSize, pageNum);
            if (pager.isEmpty()) {
                break;
            }
            list.addAll(pager);
            pageNum++;
        }
        return list;
    }

    public JobModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_JOB, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response).getJSONObject("job").toJavaObject(JobModel.class);
    }

    private class ReplaceInvoke implements HttpInvoker {

        private final HttpInvoker target;

        public ReplaceInvoke(HttpInvoker target) {
            this.target = target;
        }

        @Override
        public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity reqEntity) {
            HttpResponse response = target.invoke(url, method, headers, reqEntity);
            String body;
            try {
                body = EntityUtils.toString(response.getEntity());
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            body = body.replace("@", "");
            ContentType contentType = ContentType.get(response.getEntity());
            response.setEntity(new StringEntity(body, contentType));
            return response;
        }
    }
}
