package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VlanModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VlanCreateModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class VlanApi extends SmartApi {

    public VlanApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VlanModel> page(String vdsUuid, long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLANS, vdsUuid) + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VlanModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VlanModel>>() {});
        List<VlanModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VlanModel.class);
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VlanModel> list(String vdsUuid) {
        return list(vdsUuid, null);
    }

    public List<VlanModel> list(String vdsUuid, String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VlanModel> list = new ArrayList<>();
        while (true) {
            PageModel<VlanModel> pager = this.page(vdsUuid, pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public List<String> listAvailableIp(String vdsUuid, String vlanUuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLAN_AVAILABLE_IPS, vdsUuid, vlanUuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseArray(response, String.class);
    }

    public VlanModel get(String vdsUuid, String vlanUuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLAN, vdsUuid, vlanUuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        return JSON.parseObject(response, VlanModel.class);
    }

    public VlanModel create(String vdsUuid, VlanCreateModel param) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLANS, vdsUuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response, VlanModel.class);
    }

    public VlanModel modify(String vdsUuid, String vlanUuid, String name, Integer vlanId) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLAN, vdsUuid, vlanUuid);
        Map<String, Object> param = MapBuilder.init().put("name", name).put("vlan_id", vlanId).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "PUT", entity);
        return JSON.parseObject(response, VlanModel.class);
    }

    public void remove(String vdsUuid, String vlanUuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VDS_VLAN, vdsUuid, vlanUuid);
        this.getDefaultHttpInvoke().request(url, "DELETE");
    }

}
