package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VmCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VmModifyModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmCloneModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class VmApi extends SmartApi {

    public VmApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VmModel> page(long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + Constant.URL_VMS + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VmModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VmModel>>() {});
        List<VmModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VmModel.class);
        tList.forEach(t -> t.setVmType(VmModel.VmType.VM));
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VmModel> list() {
        return list(null);
    }

    public List<VmModel> list(String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VmModel> list = new ArrayList<>();
        while (true) {
            PageModel<VmModel> pager = this.page(pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public VmModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        VmModel vmModel = JSON.parseObject(response, VmModel.class);
        vmModel.setVmType(VmModel.VmType.VM);
        return vmModel;
    }

    public String create(VmCreateModel params) {
        String url = this.getSessionParams().getHost() + Constant.URL_VMS;
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String modify(String uuid, VmModifyModel params) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "PUT", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String remove(String uuid, Boolean includeVmSnapshots, Boolean includeVolumes, Boolean includeVolumeSnapshots) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM, uuid);
        Map<String, Object> params = MapBuilder.init()
                .put("include_vm_snapshots", includeVmSnapshots)
                .put("include_volumes", includeVolumes)
                .put("include_volume_snapshots", includeVolumeSnapshots)
                .get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "DELETE", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String start(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_START, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response).getString("job_id");
    }

    public String pause(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_PAUSE, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response).getString("job_id");
    }

    public String resume(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_RESUME, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response).getString("job_id");
    }

    public String reboot(String uuid, boolean force) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_REBOOT, uuid);
        Map<String, Boolean> params = Collections.singletonMap("force", force);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String stop(String uuid, boolean force) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_STOP, uuid);
        Map<String, Boolean> params = Collections.singletonMap("force", force);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String migrate(String uuid, String nodeIp, boolean autoSchedule) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_MIGRATE, uuid);
        Map<String, Object> params = MapBuilder.init().put("auto_schedule", autoSchedule).put("node_ip", nodeIp).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String clone(String uuid, VmCloneModel params) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_CLONE, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public void asTemplate(String uuid, String name, boolean cloudInitSupport) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_AS_VM_TEMPLATE, uuid);
        Map<String, Object> params = MapBuilder.init().put("name", name).put("cloud_init_supported", cloudInitSupport).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        this.getDefaultHttpInvoke().request(url, "POST", entity);
    }

}
