package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmSnapshotRebuildModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class VmSnapshotApi extends SmartApi {

    public VmSnapshotApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VmModel> page(long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + Constant.URL_VM_SNAPSHOTS + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VmModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VmModel>>() {});
        List<VmModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VmModel.class);
        tList.forEach(t -> t.setVmType(VmModel.VmType.VM_SNAPSHOT));
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VmModel> list() {
        return list(null);
    }

    public List<VmModel> list(String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VmModel> list = new ArrayList<>();
        while (true) {
            PageModel<VmModel> pager = this.page(pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public VmModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_SNAPSHOT, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        VmModel vmModel = JSON.parseObject(response, VmModel.class);
        vmModel.setVmType(VmModel.VmType.VM_SNAPSHOT);
        return vmModel;
    }

    /**
     * 创建云主机快照
     * @param vmUuid 云主机uuid
     * @param snapshotName 快照名称
     * @param snapshotDescription 快照描述
     * @param consistentType 一致性类型，可选crash_consistent（默认）, file_system_consistent
     * @return
     */
    public String create(String vmUuid, String snapshotName, String snapshotDescription, String consistentType) {
        String url = this.getSessionParams().getHost() + Constant.URL_VM_SNAPSHOTS;
        Map<String, Object> param = MapBuilder.init().put("vm_uuid", vmUuid).put("name", snapshotName)
                .put("description", snapshotDescription).put("consistent_type", consistentType).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String remove(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_SNAPSHOT, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "DELETE");
        return JSON.parseObject(response).getString("job_id");
    }

    public String rollback(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_SNAPSHOT_ROLLBACK, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response).getString("job_id");
    }

    public String rebuild(String uuid, VmSnapshotRebuildModel param) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_SNAPSHOT_REBUILD, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

}
