package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VmModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VmTemplateCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmCreateVmTemplateModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmTemplateCreateVmModel;
import com.bocloud.cmp.provider.smart.sdk.model.other.VmTemplateCreateVmsModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class VmTemplateApi extends SmartApi {

    public VmTemplateApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VmModel> page(long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + Constant.URL_VM_TEMPLATES + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VmModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VmModel>>() {});
        List<VmModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VmModel.class);
        tList.forEach(t -> t.setVmType(VmModel.VmType.VM_TEMPLATE));
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VmModel> list() {
        return list(null);
    }

    public List<VmModel> list(String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VmModel> list = new ArrayList<>();
        while (true) {
            PageModel<VmModel> pager = this.page(pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public VmModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        VmModel vmModel = JSON.parseObject(response, VmModel.class);
        vmModel.setVmType(VmModel.VmType.VM_TEMPLATE);
        return vmModel;
    }

    public String create(VmTemplateCreateModel params) {
        String url = this.getSessionParams().getHost() + Constant.URL_VM_TEMPLATES;
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String create(VmCreateVmTemplateModel params) {
        String url = this.getSessionParams().getHost() + Constant.URL_VM_TEMPLATES;
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public void modify(String uuid, String name, String description) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE, uuid);
        Map<String, Object> params = MapBuilder.init().put("name", name).put("description", description).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        this.getDefaultHttpInvoke().request(url, "PUT", entity);
    }

    public String remove(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "DELETE");
        return JSON.parseObject(response).getString("job_id");
    }

    public String createVm(String uuid, VmTemplateCreateVmModel params) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE_CREATE_VM, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String createVms(String uuid, VmTemplateCreateVmsModel params) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE_CREATE_VMS, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public void asVm(String uuid, String vmName) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VM_TEMPLATE_AS_VM, uuid);
        Map<String, String> params = Collections.singletonMap("name", vmName);
        HttpEntity entity = new StringEntity(JSON.toJSONString(params), ContentType.APPLICATION_JSON);
        this.getDefaultHttpInvoke().request(url, "POST", entity);
    }

}
