package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VolumeModel;
import com.bocloud.cmp.provider.smart.sdk.model.create.VolumeCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.VolumeModifyModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class VolumeApi extends SmartApi {

    public VolumeApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VolumeModel> page(long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + Constant.URL_VOLUMES + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VolumeModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VolumeModel>>() {});
        List<VolumeModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VolumeModel.class);
        tList.forEach(v -> v.setVolumeType(VolumeModel.VolumeType.VOLUME));
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VolumeModel> list() {
        return list(null);
    }

    public List<VolumeModel> list(String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VolumeModel> list = new ArrayList<>();
        while (true) {
            PageModel<VolumeModel> pager = this.page(pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public VolumeModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        VolumeModel volumeModel = JSON.parseObject(response, VolumeModel.class);
        volumeModel.setVolumeType(VolumeModel.VolumeType.VOLUME);
        return volumeModel;
    }

    public String create(VolumeCreateModel param) {
        String url = this.getSessionParams().getHost() + Constant.URL_VOLUMES;
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String modify(String uuid, VolumeModifyModel param) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME, uuid);
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "PUT", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String remove(String uuid, boolean includeSnapshot) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME, uuid);
        Map<String, Object> param = Collections.singletonMap("include_snapshots", includeSnapshot);
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "DELETE", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String clone(String uuid, String name, String remark) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME_CLONE, uuid);
        Map<String, Object> param = MapBuilder.init().put("name", name).put("description", remark).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }
}
