package com.bocloud.cmp.provider.smart.sdk.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.model.PageModel;
import com.bocloud.cmp.provider.smart.sdk.model.VolumeModel;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicNameValuePair;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class VolumeSnapshotApi extends SmartApi {

    public VolumeSnapshotApi(SessionParams sessionParams) {
        super(sessionParams);
    }

    public PageModel<VolumeModel> page(long pageSize, long pageNum, String order) {
        List<NameValuePair> pairs = new ArrayList<>();
        pairs.add(new BasicNameValuePair("count", String.valueOf(pageSize)));
        pairs.add(new BasicNameValuePair("skip_page", String.valueOf(pageNum)));
        if (StringUtils.isNotEmpty(order)) {
            pairs.add(new BasicNameValuePair("sort_criteria", order));
        }
        String finalUrl = this.getSessionParams().getHost() + Constant.URL_VOLUME_SNAPSHOTS + "?" +
                URLEncodedUtils.format(pairs, StandardCharsets.UTF_8);
        String body = this.getDefaultHttpInvoke().request(finalUrl, "GET");
        PageModel<VolumeModel> pageModel = JSON.parseObject(body, new TypeReference<PageModel<VolumeModel>>() {});
        List<VolumeModel> tList = JSON.parseArray(JSON.toJSONString(pageModel.getEntities()), VolumeModel.class);
        tList.forEach(v -> v.setVolumeType(VolumeModel.VolumeType.VOLUME_SNAPSHOT));
        pageModel.setEntities(tList);
        return pageModel;
    }

    public List<VolumeModel> list() {
        return list(null);
    }

    public List<VolumeModel> list(String order) {
        final int pageSize = 100;
        int pageNum = 1;
        List<VolumeModel> list = new ArrayList<>();
        while (true) {
            PageModel<VolumeModel> pager = this.page(pageSize, pageNum, order);
            list.addAll(pager.getEntities());
            if (pager.getPageEntities() < pageSize) {
                break;
            }
            pageNum++;
        }
        return list;
    }

    public VolumeModel get(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME_SNAPSHOT, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "GET");
        VolumeModel volumeModel = JSON.parseObject(response, VolumeModel.class);
        volumeModel.setVolumeType(VolumeModel.VolumeType.VOLUME_SNAPSHOT);
        return volumeModel;
    }

    public String create(String volumeUuid, String snapshotName, String snapshotDescription) {
        String url = this.getSessionParams().getHost() + Constant.URL_VOLUME_SNAPSHOTS;
        Map<String, Object> param = MapBuilder.init().put("volume_uuid", volumeUuid)
                .put("name", snapshotName).put("description", snapshotDescription).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }

    public String remove(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME_SNAPSHOT, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "DELETE");
        return JSON.parseObject(response).getString("job_id");
    }

    public String rollback(String uuid) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME_SNAPSHOT_ROLLBACK, uuid);
        String response = this.getDefaultHttpInvoke().request(url, "POST");
        return JSON.parseObject(response).getString("job_id");
    }

    public String rebuild(String uuid, String name, String description) {
        String url = this.getSessionParams().getHost() + String.format(Constant.URL_VOLUME_SNAPSHOT_REBUILD, uuid);
        Map<String, Object> param = MapBuilder.init().put("name", name).put("description", description).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(param), ContentType.APPLICATION_JSON);
        String response = this.getDefaultHttpInvoke().request(url, "POST", entity);
        return JSON.parseObject(response).getString("job_id");
    }
}
