package com.bocloud.cmp.provider.smart.sdk.convert;

import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.smart.sdk.model.AvailableNicModel;
import com.bocloud.cmp.provider.smart.sdk.model.HostModel;
import com.bocloud.cmp.provider.smart.sdk.model.StorageSummaryModel;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

public class HostConvert {

    public List<com.bocloud.cmp.model.HostModel> convert(List<StorageSummaryModel.Host> storages, List<HostModel> hosts, List<AvailableNicModel> availableNics) {
        List<com.bocloud.cmp.model.HostModel> results = new ArrayList<>();
        for (HostModel host : hosts) {
            AvailableNicModel availableNic = availableNics.stream()
                    .filter(a -> host.getHostUuid().equals(a.getHostUuid())).findFirst().orElse(null);
            StorageSummaryModel.Host storage = storages.stream()
                    .filter(a -> host.getHostUuid().equals(a.getHost())).findFirst().orElse(null);
            com.bocloud.cmp.model.HostModel result = this.convert(storage, host, availableNic);
            results.add(result);
        }
        return results;
    }

    public com.bocloud.cmp.model.HostModel convert(StorageSummaryModel.Host storage, HostModel host, AvailableNicModel availableNic) {
        com.bocloud.cmp.model.HostModel result = new com.bocloud.cmp.model.HostModel();
        result.setUuid(host.getHostUuid());
        result.setName(host.getName());
        result.setStatus(host.getStatus().toString());
        result.setHostIp(host.getManagementIp());
        result.setServiceIp(host.getAccessIp());
        result.setVmotionIp(host.getDataIp());
        result.setIpmiAddress(host.getIpmiIp());
        result.setHostName(host.getCvmResource().getHostname());
        result.setModelName(host.getModel());

        result.setCpuModel(host.getCpu().getBrand());
        result.setCpuCore(host.getCpu().getNumCores().intValue());
        result.setCpuHz(host.getCpu().getHzAdvertisedRaw().stream().filter(h -> h > 0).findFirst().orElse(0L));
        result.setMemTotal(Math.round(host.getMemory().getTotal() / 1024D / 1024D / 1024D * 1000D) / 1000D);
        result.setDiskTotal(Math.round(host.getSummary().getDisks().getTotalSize() / 1024D / 1024D / 1024D * 1000D) / 1000D);

        result.setDisks(new HostDiskConvert().convert(host.getDisks()));
        result.setNetcards(new HostNicConvert().convert(host.getNics()));

        Optional<HostModel.Summary.Disks> disksOpt = Optional.of(host).map(HostModel::getSummary).map(HostModel.Summary::getDisks);
        Optional<HostModel.Summary.Disks.HSD> hddOpt = disksOpt.map(HostModel.Summary.Disks::getHdd);
        Optional<HostModel.Summary.Disks.HSD> ssdOpt = disksOpt.map(HostModel.Summary.Disks::getSsd);
        hddOpt.map(HostModel.Summary.Disks.HSD::getMounted).map(HostModel.TotalAndSize::getTotal).map(Long::intValue).ifPresent(result::setHddCount);
        hddOpt.map(HostModel.Summary.Disks.HSD::getMounted).map(HostModel.TotalAndSize::getSize).ifPresent(result::setHddTotalSizeByte);
        ssdOpt.map(HostModel.Summary.Disks.HSD::getMounted).map(HostModel.TotalAndSize::getTotal).map(Long::intValue).ifPresent(result::setSsdCount);
        ssdOpt.map(HostModel.Summary.Disks.HSD::getMounted).map(HostModel.TotalAndSize::getSize).ifPresent(result::setSsdTotalSizeByte);

        result.setDiskTotalSizeByte(storage.getData().getDataSpace());
        result.setDiskUsedSizeByte(storage.getData().getUsedDataSpace());
        result.setDiskCacheSizeByte(storage.getData().getCacheSpace());

        for (NetworkCardModel netcard : result.getNetcards()) {
            Optional<AvailableNicModel.Nic> nicOpt = Optional.ofNullable(availableNic)
                    .map(AvailableNicModel::getNics).orElseGet(ArrayList::new).stream()
                    .filter(nic -> netcard.getMac().equals(nic.getHwaddr())).findFirst();
            netcard.setSwitcherAvailable(nicOpt.isPresent());
        }

        return result;
    }
}
