package com.bocloud.cmp.provider.smart.sdk.convert;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.provider.smart.sdk.model.JobModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

public class JobConvert {

    private static final Logger logger = LoggerFactory.getLogger(JobConvert.class);
    private static final String[] VM = new String[]{
            "VM_CREATE", "VM_EDIT", "VM_DELETE", "VM_EDIT_NIC", "VM_EDIT_VOL", "VM_EDIT_DEVICE", "VM_CLONE",
            "VM_MIGRATE", "VM_MOUNT_SVT", "VM_START", "VM_STOP", "VM_STOP_F", "VM_REBOOT", "VM_REBOOT_F", "VM_PAUSE",
            "VM_RESUME", "VM_RECOVER", "UMOUNT_SVT", "VM_SNAPSHOT_REBUILD"
    };
    private static final String[] VM_SNAPSHOT = new String[]{
            "VM_SNAPSHOT_CREATE"/*, "VM_SNAPSHOT_DELETE"*/, "VM_SNAPSHOT_ROLLBACK"
    };
    private static final String[] VM_TEMPLATE = new String[]{
            "VM_TEMPLATE_CREATE"/*, "VM_TEMPLATE_DELETE"*/, "VM_TEMPLATE_VM_CREATE"
    };
    private static final String[] VOLUME = new String[]{
            "VOL_CREATE", "VOL_EDIT", "VOL_DELETE", "VOL_CLONE", "VOLUME_SNAPSHOT_REBUILD"
    };
    private static final String[] VOLUME_SNAPSHOT = new String[]{
            "VOLUME_SNAPSHOT_CREATE", "VOLUME_SNAPSHOT_DELETE", "VOLUME_SNAPSHOT_ROLLBACK"
    };
    private static final String[] HOST = new String[]{
            "DATA_MOUNT", "SMTX_SYSTEM_MOUNT"
    };
    private static final String[] SWITCHER = new String[]{
            "NETWORK_CREATE_VDS", "NETWORK_EDIT_VDS"
    };

    public List<com.bocloud.cmp.model.TaskMsgModel> convert(List<JobModel> params) {
        return params.stream().map(this::convert).collect(Collectors.toList());
    }

    public Map<String, JSONObject> getResourceByType(String resourceJson, String type) {
        if (resourceJson == null) {
            return null;
        }
        JSONObject resourceObj = JSON.parseObject(resourceJson);
        Map<String, JSONObject> map = new HashMap<>();
        for (String resourceKey : resourceObj.keySet()) {
            JSONObject resourceValue = resourceObj.getJSONObject(resourceKey);
            if (type.equals(resourceValue.getString("type"))) {
                map.put(resourceKey, resourceValue);
            }
        }
        return map;
    }

    public com.bocloud.cmp.model.TaskMsgModel convert(JobModel param) {
        com.bocloud.cmp.model.TaskMsgModel result = new com.bocloud.cmp.model.TaskMsgModel();
        result.setUuid(param.getJobId());
        result.setName(param.getDescription());
        result.setState(param.getState());
        Optional.of(param).map(JobModel::getCtime).map(t -> t * 1000).map(Date::new).ifPresent(result::setStartTime);
        Optional.of(param).map(JobModel::getFtime).map(t -> t * 1000).map(Date::new).ifPresent(result::setCompleteTime);
        Optional<JobModel.Event.Data> dataOpt = Optional.of(param).map(JobModel::getEvent).map(JobModel.Event::getData);
        if (Arrays.asList(VM).contains(param.getDescription())) {
            result.setTargetType("VM");
            dataOpt.map(JobModel.Event.Data::getVmId).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getVmName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(VM_SNAPSHOT).contains(param.getDescription())) {
            result.setTargetType("VM_SNAPSHOT");
            dataOpt.map(JobModel.Event.Data::getVmSnapshotId).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getVmSnapshotName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(VM_TEMPLATE).contains(param.getDescription())) {
            result.setTargetType("VM_TEMPLATE");
            dataOpt.map(JobModel.Event.Data::getVmTemplateId).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getVmTemplateName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(VOLUME).contains(param.getDescription())) {
            result.setTargetType("VOLUME");
            dataOpt.map(JobModel.Event.Data::getVolumeId).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getVolumeName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(VOLUME_SNAPSHOT).contains(param.getDescription())) {
            result.setTargetType("VOLUME_SNAPSHOT");
            dataOpt.map(JobModel.Event.Data::getVolumeSnapshotId).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getVolumeSnapshotName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(SWITCHER).contains(param.getDescription())) {
            result.setTargetType("SWITCHER");
            dataOpt.map(JobModel.Event.Data::getUuid).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getName).ifPresent(result::setTargetName);
        } else if (Arrays.asList(HOST).contains(param.getDescription())) {
            result.setTargetType("HOST");
            dataOpt.map(JobModel.Event.Data::getDataIp).ifPresent(result::setTargetUuid);
            dataOpt.map(JobModel.Event.Data::getHostname).ifPresent(result::setTargetName);
        } else if ("VM_SNAPSHOT_DELETE".equals(param.getDescription())) {
            result.setTargetType("VM_SNAPSHOT");
            Optional<Map<String, JSONObject>> snapshotObjs = Optional.of(param)
                    .map(JobModel::getResources).map(r -> this.getResourceByType(r, "KVM_VM_SNAPSHOT"));
            Optional<Map.Entry<String, JSONObject>> snapshotObj = snapshotObjs
                    .map(Map::entrySet).orElseGet(HashSet::new).stream().findFirst();
            snapshotObj.map(Map.Entry::getKey).ifPresent(result::setTargetUuid);
            snapshotObj.map(Map.Entry::getValue).map(o -> o.getString("name")).ifPresent(result::setTargetName);
        } else if ("VM_TEMPLATE_DELETE".equals(param.getDescription())) {
            result.setTargetType("VM_TEMPLATE");
            Optional<Map<String, JSONObject>> snapshotObjs = Optional.of(param)
                    .map(JobModel::getResources).map(r -> this.getResourceByType(r, "KVM_VM_TEMPLATE"));
            Optional<Map.Entry<String, JSONObject>> snapshotObj = snapshotObjs
                    .map(Map::entrySet).orElseGet(HashSet::new).stream().findFirst();
            snapshotObj.map(Map.Entry::getKey).ifPresent(result::setTargetUuid);
            snapshotObj.map(Map.Entry::getValue).map(o -> o.getString("name")).ifPresent(result::setTargetName);
        } else {
            logger.warn("未知的任务目标类型。任务id: {}, {}, {}", param.getJobId(), param.getDescription(), dataOpt.map(JSON::toJSONString).orElse(""));
        }
        Optional<JobModel.Event.Language> detailOpt = Optional.of(param).map(JobModel::getEvent).map(JobModel.Event::getMessage);
        detailOpt.map(JobModel.Event.Language::getEnUS).ifPresent(result::setMessageEnUS);
        detailOpt.map(JobModel.Event.Language::getZhCN).ifPresent(result::setMessageZhCN);
        Optional<JobModel.Event.Language> messageOpt = Optional.of(param).map(JobModel::getEvent).map(JobModel.Event::getDetail);
        messageOpt.map(JobModel.Event.Language::getEnUS).ifPresent(result::setDetailEnUS);
        messageOpt.map(JobModel.Event.Language::getZhCN).ifPresent(result::setDetailZhCN);

        result.setResources(param.getResources());
        result.setResourceGroup(param.getResourceGroup());

        List<JobModel.Task> taskList = Optional.of(param).map(JobModel::getTaskList).orElseGet(ArrayList::new);
        result.setSubTasks(new TaskConvert().convert(taskList));

        return result;
    }
}
