package com.bocloud.cmp.provider.smart.sdk.convert;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.model.NetworkModel;
import com.bocloud.cmp.provider.smart.sdk.model.VlanModel;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

public class VlanConvert {

    public List<NetworkModel> convert(List<VlanModel> params) {
        return params.stream().map(this::convert).collect(Collectors.toList());
    }

    public com.bocloud.cmp.model.NetworkModel convert(VlanModel params) {
        com.bocloud.cmp.model.NetworkModel network = new com.bocloud.cmp.model.NetworkModel();
        com.bocloud.cmp.model.SubnetModel subnet = new com.bocloud.cmp.model.SubnetModel();

        network.setNetworkUuid(params.getUuid());
        network.setName(params.getName());
        network.setSwitchUUid(params.getVdsUuid());

        subnet.setSubnetUuid(params.getUuid());
        subnet.setNetworkUuid(params.getUuid());
        subnet.setName(params.getName());
        subnet.setVlan(params.getVlanId());
        subnet.setGateway(params.getGateway());
        subnet.setNetmask(params.getSubnetmask());

        if (params.getDhcpConfig() == null) {
            subnet.setDhcpEnabled(false);
        } else {
            subnet.setDhcpEnabled(true);
            List<String> ipPools = new ArrayList<>();
            for (VlanModel.DhcpConfig.IpRange ipRange : params.getDhcpConfig().getIpRanges()) {
                ipPools.add(ipRange.getStartIp() + "-" + ipRange.getEndIp());
            }
            subnet.setIpPools(JSON.toJSONString(ipPools));
            subnet.setCidr(params.getDhcpConfig().getCidr());
            subnet.setDhcpServerIp(params.getDhcpConfig().getDhcpInterfaceIp());
            subnet.setDnsNames(String.join(",", params.getDhcpConfig().getDnsServers()));
            subnet.setGateway(params.getDhcpConfig().getGatewayIp());
        }

        network.setSubnets(Collections.singletonList(subnet));
        return network;
    }
}
