package com.bocloud.cmp.provider.smart.sdk.convert;

import com.bocloud.cmp.provider.smart.sdk.model.VmModel;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class VmConvert {

    public List<com.bocloud.cmp.model.ServerModel> convert(List<VmModel> params) {
        return params.stream().map(this::convert).collect(Collectors.toList());
    }

    public com.bocloud.cmp.model.ServerModel convert(VmModel params) {
        com.bocloud.cmp.model.ServerModel result = new com.bocloud.cmp.model.ServerModel();
        result.setIsTemplate(false);
        result.setInstanceId(params.getUuid());
        result.setName(params.getVmName());
        result.setStatus(params.getStatus());
        result.setRemark(params.getDescription());

        result.setCpu(params.getVcpu());
        result.setMemory(new BigDecimal(params.getMemory()).divide(new BigDecimal(1024 * 1024 * 1024), 3, RoundingMode.HALF_UP));
        result.setDiskDevices(new VmDiskConvert().convert(params.getVmType(), params.getDisks()));
        result.setNetworkCards(new VmNicConvert().convertToNetworkCard(params.getNics()));
        Optional.of(params).map(VmModel::getUpdateTime).map(t -> t * 1000).map(Date::new).ifPresent(result::setGmtVmModify);
        Optional.of(params).map(VmModel::getCreateTime).map(t -> t * 1000).map(Date::new).ifPresent(result::setGmtVmCreate);

        result.setIps(new VmNicConvert().convertToIp(params.getNics()));
        result.setManagerIp(params.getNics().stream().findFirst().map(VmModel.Nic::getIpAddress).orElse(null));

        Optional.of(params).map(VmModel::getGuestInfo).map(VmModel.GuestInfo::getHostname).ifPresent(result::setHostName);
        Optional.of(params).map(VmModel::getGuestInfo).map(VmModel.GuestInfo::getOsVersion).ifPresent(result::setOsVersion);

        result.setServerVal(params.getNodeIp());
        return result;
    }

}
