package com.bocloud.cmp.provider.smart.sdk.exception;

import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;

import java.io.IOException;

public class HttpException extends RuntimeException {

    private int responseCode;
    private Header[] responseHeaders;
    private String responseBody;

    public HttpException(HttpResponse response) {
        this.responseCode = response.getStatusLine().getStatusCode();
        this.responseHeaders = response.getAllHeaders();
        if (response.getEntity() == null) {
            this.responseBody = "";
        } else {
            try {
                this.responseBody = EntityUtils.toString(response.getEntity());
            } catch (IOException e) {
                this.responseBody = String.format("Read response's entity error: %s", e.getMessage());
            }
        }
    }

    public int getResponseCode() {
        return responseCode;
    }

    public Header[] getResponseHeaders() {
        return responseHeaders;
    }

    public String getResponseBody() {
        return responseBody;
    }

    @Override
    public String getMessage() {
        return String.format("[%d] %s", this.getResponseCode(), this.getResponseBody());
    }
}
