package com.bocloud.cmp.provider.smart.sdk.http;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.BufferedHttpEntity;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

public class ClientInvoker implements HttpInvoker {

    private static final Logger logger = LoggerFactory.getLogger(ClientInvoker.class);
    private static HttpClient httpClient = null;
    private static Throwable throwable = null;

    static {
        try {
            SSLContext sslContext = SSLContext.getInstance("TLS");
            sslContext.init(null, new TrustManager[]{ new EmptyX509ExtendedTrustManager() }, null);
            httpClient = HttpClients.custom().setSSLContext(sslContext).setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE).build();
        } catch (NoSuchAlgorithmException | KeyManagementException e) {
            logger.warn("Init HttpClient Error: {}", e.getMessage());
            throwable = e;
        }
    }

    @Override
    public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity entity) {
        if (httpClient == null) {
            throw new RuntimeException(throwable);
        }
        try {
            HttpEntityEnclosingRequestBase request = new HttpEntityEnclosingRequestBase() {
                @Override
                public String getMethod() {
                    return method;
                }
            };
            request.setURI(new URI(url));
            request.setEntity(entity);
            request.setHeaders(headers);
            HttpResponse response = httpClient.execute(request);
            response.setEntity(new BufferedHttpEntity(response.getEntity()));
            return response;
        } catch (IOException | URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }
}
