package com.bocloud.cmp.provider.smart.sdk.http;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.session.SessionToken;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.*;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.message.BasicHttpResponse;
import org.apache.http.message.BasicStatusLine;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.UUID;

public class DebugInvoke implements HttpInvoker {

    private HttpResponse load(String path)  {
        try {
            InputStream inputStream = this.getClass().getResourceAsStream(path);
            String json = IOUtils.toString(inputStream, StandardCharsets.UTF_8);
            return this.response200(json);
        } catch (IOException e) {
            return this.response500(e);
        }
    }

    private HttpResponse response200(String body) {
        StatusLine statusLine = new BasicStatusLine(HttpVersion.HTTP_1_1, 200, "Success");
        HttpEntity entity = new StringEntity(body, ContentType.APPLICATION_JSON);
        HttpResponse response = new BasicHttpResponse(statusLine);
        response.setEntity(entity);
        return response;
    }

    private HttpResponse response500(Throwable e) {
        StatusLine statusLine = new BasicStatusLine(HttpVersion.HTTP_1_1, 500, "Internal Error");
        HttpEntity entity = new StringEntity(e.getMessage(), ContentType.APPLICATION_JSON);
        HttpResponse response = new BasicHttpResponse(statusLine);
        response.setEntity(entity);
        return response;
    }

    private HttpResponse response404() {
        StatusLine statusLine = new BasicStatusLine(HttpVersion.HTTP_1_1, 404, "Not Found");
        return new BasicHttpResponse(statusLine);
    }

    @Override
    public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity entity) {
        URL urlObj;
        try {
            urlObj = new URL(url);
        } catch (MalformedURLException e) {
            return this.response500(e);
        }
        String path = urlObj.getPath();
        String query = urlObj.getQuery();

        if ("GET".equals(method)) {
            if (path.equals(Constant.URL_STORAGE_POLICIES)) return load("/listStoragePolicy.json");
            if (path.equals(Constant.URL_IMAGES)) return load("/listISO.json");
            if (path.equals(Constant.URL_CLUSTERS)) return load("/listCluster.json");
            if (path.equals(Constant.URL_HOSTS)) return load("/listHost.json");
            if (path.equals(Constant.URL_VDSES)) return load("/listVDS.json");
            if (path.equals(Constant.URL_VMS)) return load("/listVM.json");
            if (path.equals(Constant.URL_VM_SNAPSHOTS)) return load("/listVMSnapshot.json");
            if (path.equals(Constant.URL_VM_TEMPLATES)) return load("/listVMTemplate.json");
            if (path.equals(Constant.URL_VOLUMES)) return load("/listVolume.json");
            if (path.equals(Constant.URL_VOLUME_SNAPSHOTS)) return load("/listVolumeSnapshot.json");
            if (path.equals(Constant.URL_AVAILABLE_NICS)) return load("/listAvailableNic.json");
            if (path.equals(Constant.URL_JOBS)) {
                if (StringUtils.isEmpty(query) || query.contains("page=1")) {
                    return load("/listJob.json");
                } else {
                    String body = "{\"data\": {\"jobs\": []}, \"ec\": \"EOK\", \"error\": {}}";
                    return response200(body);
                }
            }
            if (path.contains(Constant.URL_VDSES) && path.endsWith("vlans")) {
                String[] urls = url.split("/");
                String vdsUuid = urls[7];
                return load("/listVLAN" + vdsUuid + ".json");
            }
            if (path.contains(Constant.URL_JOBS + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getJob_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VMS + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVM_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VOLUMES + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVolume_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VM_SNAPSHOTS + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVMSnapshot_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VM_TEMPLATES + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVMTemplate_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VOLUME_SNAPSHOTS + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVolumeSnapshot_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_IMAGES + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getISO_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_VDSES) && path.contains("vlans/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getVLAN_" + uuid + ".json");
            }
            if (path.contains(Constant.URL_STORAGE_POLICIES + "/")) {
                String uuid = path.substring(path.lastIndexOf("/") + 1);
                return load("/getStoragePolicy_" + uuid + ".json");
            }
        } else if ("POST".equals(method)) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            if (path.equals(Constant.URL_SESSION)) {
                SessionToken sessionToken = new SessionToken();
                sessionToken.setToken(UUID.randomUUID().toString());
                sessionToken.setUserId("root");
                sessionToken.setExpireTime("2099-12-31 23:59:59");
                sessionToken.setCreateTime("2000-01-01 00:00:00");
                String json = JSON.toJSONString(sessionToken);
                return response200(json);
            }
            if (path.equals(Constant.URL_VMS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2CreateVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VMS) && path.contains("migrate")) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2MigrateVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VMS) && path.contains("clone")) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2CloneVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VMS) && path.contains("as_vm_template")) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.equals(Constant.URL_VM_SNAPSHOTS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2CreateSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.equals(Constant.URL_VM_TEMPLATES)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest5CreateVMTemplateJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_TEMPLATES) && path.contains("vm_create")) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest6CreateVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_TEMPLATES) && path.contains("as_vm")) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.equals(Constant.URL_VOLUMES)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9CreateVolumeJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.equals(Constant.URL_VOLUME_SNAPSHOTS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9CreateSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_SNAPSHOTS) && path.contains("rollback")) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2RollbackSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VOLUME_SNAPSHOTS) && path.contains("rollback")) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9RollbackSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_IMAGE_UPLOAD_VOLUME)) {
                String body = "{\"data\": {\"image_uuid\": \"BoyunTest\", \"zbs_volume_id\": \"BoyunTest\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_IMAGE_UPLOAD)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VDSES) && path.endsWith("vlans")) {
                return load("/getVLAN_BoyunTest7.json");
            }
            if (path.equals(Constant.URL_STORAGE_POLICIES)) {
                return load("/getStoragePolicy_BoyunTest8.json");
            }
        } else if ("PUT".equals(method)) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            if (path.contains(Constant.URL_VMS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2ModifyVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VOLUMES)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9ModifyVolumeJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_IMAGES)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_TEMPLATES)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VDSES) && path.contains("vlans/")) {
                return load("/getVLAN_BoyunTest7.json");
            }
            if (path.contains(Constant.URL_STORAGE_POLICIES)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
        } else if ("DELETE".equals(method)) {
            try {
                Thread.sleep(5000);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
            if (path.contains(Constant.URL_VMS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2RemoveVMJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VOLUMES)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9RemoveVolumeJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_SNAPSHOTS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest2RemoveSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VM_TEMPLATES)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest5RemoveVMTemplateJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VOLUME_SNAPSHOTS)) {
                String body = "{\"data\": {\"job_id\": \"BoyunTest9RemoveSnapshotJob\"}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_IMAGES)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_VDSES) && path.contains("vlans/")) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
            if (path.contains(Constant.URL_STORAGE_POLICIES)) {
                String body = "{\"data\": {}, \"ec\": \"EOK\", \"error\": {}}";
                return response200(body);
            }
        }
        StatusLine statusLine404 = new BasicStatusLine(HttpVersion.HTTP_1_1, 404, "Not Found");
        return new BasicHttpResponse(statusLine404);
    }
}
