package com.bocloud.cmp.provider.smart.sdk.http;

import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.exception.HttpException;
import com.bocloud.cmp.provider.smart.sdk.session.SessionManager;
import com.bocloud.cmp.provider.smart.sdk.session.SessionParams;
import com.bocloud.cmp.provider.smart.sdk.session.SessionToken;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.message.BasicHeader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SessionInvoker implements HttpInvoker {

    private static final Logger logger = LoggerFactory.getLogger(SessionInvoker.class);
    private static final int RETRY = 3;
    private final SessionParams sessionParams;
    private final HttpInvoker target;

    public SessionInvoker(HttpInvoker target, SessionParams sessionParams) {
        this.sessionParams = sessionParams;
        this.target = target;
    }

    @Override
    public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity entity) {
        if (headers == null) {
            headers = new Header[2];
        } else {
            headers = Arrays.copyOf(headers, headers.length + 2);
        }
        SessionManager sessionManager = new SessionManager();
        SessionToken token = sessionManager.getSession(sessionParams);
        headers[headers.length - 1] = new BasicHeader(Constant.HEADER_TOKEN_V2, token.getToken());
        headers[headers.length - 2] = new BasicHeader(Constant.HEADER_TOKEN_V3, token.getToken());
        int currTryCnt = 0;
        while (true) {
            try {
                return target.invoke(url, method, headers, entity);
            } catch (HttpException e) {
                if (currTryCnt == RETRY) throw e;
                if (e.getResponseCode() != 401) throw e;
                logger.warn("Invoke error: {]", e.getMessage());
                sessionManager.clearSession(sessionParams);
                currTryCnt++;
            }
        }
    }
}
