package com.bocloud.cmp.provider.smart.sdk.http;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.exception.SmartException;
import com.bocloud.cmp.provider.smart.sdk.model.ResultModel;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class SmartInvoke implements HttpInvoker {

    private static final Logger logger = LoggerFactory.getLogger(SmartInvoke.class);
    private final HttpInvoker target;

    public SmartInvoke(HttpInvoker target) {
        this.target = target;
    }

    private String unicodeToString(String string) {
        if (string == null) {
            return null;
        }
        StringBuffer stringBuffer = new StringBuffer();
        String pattern = "\\\\u[0-9a-fA-F]{4}";
        Matcher matcher = Pattern.compile(pattern).matcher(string);
        while (matcher.find()) {
            char c = (char) Integer.parseInt(matcher.group().replace("\\u", ""), 16);
            matcher.appendReplacement(stringBuffer, new String(new char[]{c}));
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }

    private String stringToUnicode(String string) {
        if (string == null) {
            return null;
        }
        StringBuffer stringBuffer = new StringBuffer();
        String pattern = "[\\u4e00-\\u9fa5]";
        Matcher matcher = Pattern.compile(pattern).matcher(string);
        while (matcher.find()) {
            String hex = Integer.toHexString(matcher.group().charAt(0));
            matcher.appendReplacement(stringBuffer, "\\u" + hex);
        }
        matcher.appendTail(stringBuffer);
        return stringBuffer.toString();
    }


    @Override
    public HttpResponse invoke(String url, String method, Header[] headers, HttpEntity entity) {
        HttpResponse response = target.invoke(url, method, headers, entity);
        String body;
        try {
            body = EntityUtils.toString(response.getEntity());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        body = this.unicodeToString(body);
        JSONObject bodyObj = JSON.parseObject(body);
        if (!bodyObj.containsKey("ec")) {
            logger.warn("The response not container key[ec], return directly");
            return response;
        }
        ResultModel resultModel = JSON.parseObject(body, ResultModel.class);
        if (!Constant.RESPONSE_EOK.equals(resultModel.getEc())) {
            logger.warn("==>> [{}] {}", method, url);
            if (headers != null && headers.length > 0) {
                for (Header header : headers) {
                    logger.warn("==>> [HEADER] {}={}", header.getName(), header.getValue());
                }
            }
            logger.warn("==>> [ENTITY] {}", body);
            logger.warn("==>> [{}] {}", resultModel.getEc(), resultModel.getError());
            throw new SmartException(url, method, headers, entity, resultModel);
        }
        String data = JSON.toJSONString(resultModel.getData());
        ContentType contentType = ContentType.get(response.getEntity());
        Charset charset = contentType == null ? StandardCharsets.UTF_8 : contentType.getCharset();
        charset = charset == null ? StandardCharsets.UTF_8 : charset;
        response.setEntity(new StringEntity(data, charset));
        return response;
    }
}
