package com.bocloud.cmp.provider.smart.sdk.model;

import com.alibaba.fastjson.annotation.JSONField;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@NoArgsConstructor
public class HostModel{

    private String name;
    private String role;
    private String sn;
    private Integer status; // 文档未说明汉化
    @JSONField(name = "use_state")
    private Integer useState; // 文档未说明汉化

    @JSONField(name = "access_ip")
    private String accessIp;
    @JSONField(name = "data_ip")
    private String dataIp;
    @JSONField(name = "hypervisor_ip")
    private String hypervisorIp;
    @JSONField(name = "management_ip")
    private String managementIp;
    @JSONField(name = "rpc_ip")
    private String rpcIp;
    @JSONField(name = "ipmi_ip")
    private String ipmiIp;


    @JSONField(name = "zone_id")
    private String zoneId;
    @JSONField(name = "zone_name")
    private String zoneName;

    private String model;
    private Cpu cpu;
    private Memory memory;
    private List<Disk> disks;
    private List<Nic> nics;
    private List<String> tags;

    @JSONField(name = "storage_pool_id")
    private String storagePoolId;
    @JSONField(name = "brick_topo_id")
    private String brickTopoId;
    @JSONField(name = "topo_id")
    private String topoId;
    @JSONField(name = "chunk_id")
    private String chunkId;
    @JSONField(name = "elf_nested_kvm")
    private Boolean elfNestedKvm;
    @JSONField(name = "host_uuid")
    private String hostUuid;
    @JSONField(name = "last_disk_modified")
    private String lastDiskModified;
    private String modified;
    private Summary summary;
    @JSONField(name = "sys_path")
    private SysPath sysPath;
    @JSONField(name = "cvm_resource")
    private CvmResource cvmResource;

    @Data
    @NoArgsConstructor
    public static class Cpu{
        private String brand;
        private String flags;
        @JSONField(name = "hz_advertised")
        private String hzAdvertised;
        @JSONField(name = "hz_advertised_raw")
        private List<Long> hzAdvertisedRaw;
        @JSONField(name = "num_cores")
        private Long numCores;
    }

    @Data
    @NoArgsConstructor
    public static class Memory{
        private Long total;
    }

    @Data
    @NoArgsConstructor
    public static class Disk{
        private String id;
        private String name; // "sdb"
        private String path; // "/dev/sdb"
        private String serial; // "PHYG113105FR1P9DGN"
        private Long size; // byte
        private String type; // "SSD" "HDD"

        @JSONField(name = "is_healthy")
        private Boolean isHealthy;
        @JSONField(name = "is_mounted")
        private Boolean isMounted;
        @JSONField(name = "is_offline")
        private Boolean isOffline;
        @JSONField(name = "is_os_disk")
        private Boolean isOsDisk;
        @JSONField(name = "is_physical_healthy")
        private Boolean isPhysicalHealthy;
        @JSONField(name = "is_rotational")
        private Boolean isRotational;

        private String firmware;
        private String controller;
        @JSONField(name = "celsius_temperature")
        private String celsiusTemperature;
        @JSONField(name = "num_io_errors")
        private Long numIoErrors;
        private String model;
        @JSONField(name = "mount_status")
        private String mountStatus;
        private Boolean staging;

        private List<Partition> partitions;

        @Data
        @NoArgsConstructor
        public static class Partition{
            private String name; // "sdb2"
            private String path; // "/dev/sdb2"
            private Long size; // byte
            @JSONField(name = "used_size")
            private Long usedSize; // byte
            private String usage; // metad 元数据分区 journal Journal分区 cache 缓存分区 system 系统分区 unparted 未分区
        }
    }

    @Data
    @NoArgsConstructor
    public static class Nic{
        private String name;
        private String usage;
        private Boolean running;
        private Boolean up;
        @JSONField(name = "is_physical")
        private Boolean isPhysical;

        private String gateway;
        private String hwaddr;
        private String ipaddr;
        private String netmask;
        private Integer vlan;
        private Integer mtu;
        private Long speed;

        @JSONField(name = "bonding_detail")
        private String bondingDetail;
        @JSONField(name = "bonding_master")
        private String bondingMaster;
        @JSONField(name = "bonding_mode")
        private String bondingMode;
    }

    @Data
    @NoArgsConstructor
    public static class CvmResource{
        private String hostname;
        private Long cpus;
        private Long memory;
    }

    @Data
    @NoArgsConstructor
    public static class Summary{

        private Disks disks;

        @Data
        @NoArgsConstructor
        public static class Disks{

            private HSD hdd;
            private HSD ssd;
            @JSONField(name = "total_size")
            private Long totalSize;

            @Data
            @NoArgsConstructor
            public static class HSD{

                private TotalAndSize mounted;
                @JSONField(name = "not_mounted")
                private TotalAndSize notMounted;
                private Total unhealthy;
            }
        }
    }

    @Data
    @NoArgsConstructor
    public static class SysPath{
        private TotalAndSize meta;
        private TotalAndSize os;
        private TotalAndSize zk;
    }

    @Data
    @NoArgsConstructor
    public static class TotalAndSize{
        private Long size;
        private Long total;
    }

    @Data
    @NoArgsConstructor
    public static class Total{
        private Long total;
    }
}
