package com.bocloud.cmp.provider.smart.sdk.model;

import com.alibaba.fastjson.annotation.JSONField;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@NoArgsConstructor
public class VmModel {

    private VmType vmType;

    /**
     * VM: 云主机uuid
     * VM_SNAPSHOT: 云主机快照uuid
     * VM_TEMPLATE: 云主机模板uuid
     */
    private String uuid;
    /**
     * VM: 尚未使用
     * VM_SNAPSHOT: 云主机快照名称
     * VM_TEMPLATE: 云主机模板名称
     */
    private String name;
    /**
     * VM: 尚未使用
     * VM_SNAPSHOT: 云主机uuid
     * VM_TEMPLATE: 云主机uuid
     */
    @JSONField(name = "vm_uuid")
    private String vmUuid;
    /**
     * VM: 云主机名称
     * VM_SNAPSHOT: 云主机名称
     * VM_TEMPLATE: 云主机名称
     */
    @JSONField(name = "vm_name")
    private String vmName;
    @JSONField(name = "vm_snapshot")
    private String vmSnapshot;
    private String type;
    private String status;
    @JSONField(name = "resource_state")
    private String resourceState;
    private String description;
    @JSONField(name = "node_ip")
    private String nodeIp;

    private Integer vcpu;
    private Cpu cpu;
    @JSONField(name = "cpu_model")
    private String cpuModel;
    private Long memory;
    private List<Disk> disks;
    private List<Nic> nics;

    @JSONField(name = "create_time")
    private Long createTime;
    @JSONField(name = "update_time")
    private Long updateTime;
    @JSONField(name = "last_start_time")
    private Long lastStartTime;
    @JSONField(name = "last_down_time")
    private Long lastDownTime;

    private String firmware;
    @JSONField(name = "nested_virtualization")
    private Double nestedVirtualization;
    @JSONField(name = "win_opt")
    private Boolean winOpt;
    @JSONField(name = "clock_offset")
    private String clockOffset;
    private List<HostDev> hostdevs;
    @JSONField(name = "quota_policy")
    private QuotaPolicy quotaPolicy;
    @JSONField(name = "video_type")
    private String videoType;
    private Folder folder;

    private Boolean ha;
    private Boolean internal;
    @JSONField(name = "auto_schedule")
    private Boolean autoSchedule;
    @JSONField(name = "cluster_cpu_model")
    private String clusterCpuModel;
    @JSONField(name = "guest_cpu_model")
    private String guestCpuModel;
    @JSONField(name = "guest_info")
    private GuestInfo guestInfo;
    @JSONField(name = "unique_size")
    private Long uniqueSize; // 云主机快照：独立占用空间

    @JSONField(name = "cloud_init_supported")
    private Boolean cloudInitSupported;

    @Data
    @NoArgsConstructor
    public static class Cpu {
        private Topology topology;

        @Data
        @NoArgsConstructor
        public static class Topology {
            private Long cores;
            private Long sockets;
        }
    }

    @Data
    @NoArgsConstructor
    public static class Disk {
        private Integer boot;
        private Long size; // 云主机使用、云主机模板使用
        private String name; // 云主机使用
        @JSONField(name = "diff_size")
        private Long diffSize; // 云主机模板使用
        @JSONField(name = "export_name")
        private String exportName; // 云主机模板使用
        private String type;
        private String bus;
        private String path;
        private String serial;
        private Boolean disabled;
        @JSONField(name = "volume_uuid")
        private String volumeUuid;
        @JSONField(name = "snapshot_uuid")
        private String snapshotUuid; // 云主机快照使用
        @JSONField(name = "volume_template_uuid")
        private String volumeTemplateUuid; // 云主机模板使用
        @JSONField(name = "volume_is_sharing")
        private Boolean volumeIsSharing;
        @JSONField(name = "quota_policy")
        private QuotaPolicy quotaPolicy;
        @JSONField(name = "storage_policy_uuid")
        private String storagePolicyUuid;
        private Long key;
    }

    @Data
    @NoArgsConstructor
    public static class Nic {
        private String uuid;
        private String ovs;
        @JSONField(name = "vlan_uuid")
        private String vlanUuid;
        private List<Vlan> vlans;
        @JSONField(name = "ip_address")
        private String ipAddress;
        @JSONField(name = "mac_address")
        private String macAddress;
        @JSONField(name = "subnet_mask")
        private String subnetMask;
        private String gateway;
        private String link;
        private String model;
        private Long queues;
        private Boolean mirror;

        @Data
        @NoArgsConstructor
        public static class Vlan {
            @JSONField(name = "vlan_id")
            private Integer vlanId;
        }
    }

    @Data
    @NoArgsConstructor
    public static class GuestInfo {
        private String hostname;
        @JSONField(name = "kernel_info")
        private String kernelInfo;
        private Boolean expired;
        private Boolean mounted;
        @JSONField(name = "ga_state")
        private String gaState;
        @JSONField(name = "ga_version")
        private String gaVersion;
        @JSONField(name = "os_version")
        private String osVersion;
        @JSONField(name = "gateway_ips")
        private List<String> gatewayIps;
        @JSONField(name = "dns_servers")
        private List<String> dnsServers;
        private List<Nic> nics;
        private String vip;

        @Data
        @NoArgsConstructor
        public static class Nic {
            private String name;
            @JSONField(name = "ip_addresses")
            private List<IpAddress> ipAddresses;

            @Data
            @NoArgsConstructor
            public static class IpAddress {
                @JSONField(name = "ip_address")
                private String ipAddress;
                @JSONField(name = "ip_address_type")
                private String ipAddressType;
                private Integer prefix;
                private String netmask;
            }
        }
    }

    @Data
    @NoArgsConstructor
    public static class HostDev {
        private String uuid;
        private String name;
        private String type;
    }

    @Data
    @NoArgsConstructor
    public static class QuotaPolicy {
        @JSONField(name = "max_iops_policy")
        private String maxIopsPolicy;
        @JSONField(name = "max_bandwidth_policy")
        private String maxBandwidthPolicy;
        @JSONField(name = "max_iops")
        private String maxIops;
        @JSONField(name = "max_bandwidth")
        private String maxBandwidth;
    }

    @Data
    @NoArgsConstructor
    public static class Folder {
        private String uuid;
        private String name;
    }

    public enum VmType {
        VM, VM_SNAPSHOT, VM_TEMPLATE
    }
}
