package com.bocloud.cmp.provider.smart.sdk.model.other;

import com.alibaba.fastjson.annotation.JSONField;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@Data
@NoArgsConstructor
public class VmTemplateCreateVmsModel {

    private List<Vm> vms;

    @Data
    @NoArgsConstructor
    public static class Vm {
        @JSONField(name = "vm_name")
        private String vmName;
        private String status;
        @JSONField(name = "node_ip")
        private String nodeIp;

        private Integer vcpu;
        private Cpu cpu;
        @JSONField(name = "cpu_model")
        private String cpuModel;
        private Long memory;
        private List<Nic> nics;
        @JSONField(name = "new_disks")
        private List<NewDisk> newDisks;
        @JSONField(name = "modify_disks")
        private List<ModifyDisk> modifyDisks;
        @JSONField(name = "delete_disks")
        private List<DeleteDisk> deleteDisks;

        private Boolean ha;
        private String firmware;
        private String description;
        @JSONField(name = "nested_virtualization")
        private Double nestedVirtualization;
        @JSONField(name = "win_opt")
        private Boolean winOpt;
        @JSONField(name = "clock_offset")
        private String clockOffset;
        private List<HostDev> hostdevs;
        @JSONField(name = "quota_policy")
        private QuotaPolicy quotaPolicy;
        @JSONField(name = "video_type")
        private VideoType videoType;
        @JSONField(name = "auto_schedule")
        private Boolean autoSchedule;
        @JSONField(name = "is_full_copy")
        private Boolean isFullCopy;
        @JSONField(name = "folder_uuid")
        private String folderUuid;
        @JSONField(name = "cloud_init")
        private CloudInit cloudInit;

        @JSONField(serialize = false)
        public synchronized CloudInit getCloudInitSafely() {
            if (this.cloudInit == null) {
                this.cloudInit = new CloudInit();
            }
            return this.cloudInit;
        }

        @Data
        @NoArgsConstructor
        @AllArgsConstructor
        public static class Cpu {
            private Topology topology;

            @Data
            @NoArgsConstructor
            @AllArgsConstructor
            public static class Topology {
                private Long cores;
                private Long sockets;
            }
        }

        @Data
        @NoArgsConstructor
        public static class Nic {
            private String ovs;
            @JSONField(name = "vlan_uuid")
            private String vlanUuid;
            private List<Vlan> vlans;
            @JSONField(name = "ip_address")
            private String ipAddress;
            @JSONField(name = "mac_address")
            private String macAddress;
            @JSONField(name = "subnet_mask")
            private String subnetMask;
            private String gateway;
            private String link;
            private String model;
            private Boolean mirror;

            @Data
            @NoArgsConstructor
            public static class Vlan {
                @JSONField(name = "vlan_id")
                private Integer vlanId;
            }
        }

        @Data
        @NoArgsConstructor
        public static class HostDev {
            private String uuid;
            private String name;
            private String type;
        }

        @Data
        @NoArgsConstructor
        public static class QuotaPolicy {
            @JSONField(name = "max_iops_policy")
            private String maxIopsPolicy;
            @JSONField(name = "max_bandwidth_policy")
            private String maxBandwidthPolicy;
            @JSONField(name = "max_iops")
            private String maxIops;
            @JSONField(name = "max_bandwidth")
            private String maxBandwidth;
        }

        @Data
        @NoArgsConstructor
        public static class VideoType {
            @JSONField(name = "Video_type")
            private String VideoType;
        }

        @Data
        @NoArgsConstructor
        public static class NewDisk {

            @JSONField(name = "size_in_byte")
            private Long sizeInByte;
            @JSONField(name = "new_size_in_byte")
            private Long newSizeInByte;
            private String name;
            private String type;
            private String bus;
            private String path;
            private Boolean disabled;
            @JSONField(name = "quota_policy")
            private QuotaPolicy quotaPolicy;
            @JSONField(name = "storage_policy_uuid")
            private String storagePolicyUuid;
            @JSONField(name = "src_target_name")
            private String srcTargetName;
            @JSONField(name = "src_lun_id")
            private String srcLunId;
            @JSONField(name = "src_export_id")
            private String srcExportId;
            @JSONField(name = "src_inode_id")
            private String srcInodeId;
            @JSONField(name = "clone_before_create")
            private String cloneBeforeCreate;
        }

        @Data
        @NoArgsConstructor
        public static class ModifyDisk {

            private Long key;
            private String type;
            private String name;
            private String path;
            private String bus;
            @JSONField(name = "volume_template_uuid")
            private String volumeTemplateUuid;
            private Boolean disabled;
        }

        @Data
        @NoArgsConstructor
        public static class DeleteDisk {

            private Long key;
            private String type;
            private String path;
        }

        @Data
        @NoArgsConstructor
        public static class CloudInit {

            private String hostname;
            @JSONField(name = "default_user_password")
            private String defaultUserPassword;
            @JSONField(name = "public_keys")
            private List<String> publicKeys;
            @JSONField(name = "network_info")
            private NetworkInfo networkInfo;
            @JSONField(name = "user_data")
            private String userData;

            @JSONField(serialize = false)
            public synchronized NetworkInfo getNetworkInfoSafely() {
                if (this.networkInfo == null) {
                    this.networkInfo = new NetworkInfo();
                }
                return this.networkInfo;
            }

            @Data
            @NoArgsConstructor
            public static class NetworkInfo {

                private String gateway;
                private List<String> nameservers;
                private List<Network> networks;

                @Data
                @NoArgsConstructor
                public static class Network {

                    @JSONField(name = "nic_index")
                    private Integer nicIndex;
                    @JSONField(name = "ip_address")
                    private String ipAddress;
                    private String netmask;
                    private String type; // ipv4，ipv4_dhcp
                    private List<Route> routes;

                    @Data
                    @NoArgsConstructor
                    public static class Route {

                        private String network;
                        private String netmask;
                        private String gateway;
                    }
                }
            }
        }
    }
}
