package com.bocloud.cmp.provider.smart.sdk.session;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.provider.smart.sdk.MapBuilder;
import com.bocloud.cmp.provider.smart.sdk.constant.Constant;
import com.bocloud.cmp.provider.smart.sdk.http.*;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Map;

public class SessionManager{

    private static final Logger logger = LoggerFactory.getLogger(SessionManager.class);
    private final HttpInvoker httpInvoker;

    public SessionManager() {
        String debug = System.getProperty("smart.debug", Boolean.FALSE.toString());
        if (Boolean.TRUE.toString().equals(debug)) {
            httpInvoker = new ThrowsInvoke(new LoggingInvoker(new DebugInvoke()));
        } else {
            httpInvoker = new ThrowsInvoke(new LoggingInvoker(new ClientInvoker()));
        }
    }

    public void clearSession(SessionParams params) {
        SessionHolder.clearSession(params);
    }

    public SessionToken getSession(SessionParams params) {
        synchronized (SessionHolder.class) {
            SessionToken token = SessionHolder.getSession(params);
            if (token != null) {
                return token;
            }
            token = this.getSessionFromSmartx(params);
            SessionHolder.setSession(params, token);
            return token;
        }
    }

    private SessionToken getSessionFromSmartx(SessionParams params) {
        String url = params.getHost() + Constant.URL_SESSION;
        Map<String, Object> request = MapBuilder.init()
                .put("username", params.getUser())
                .put("password", params.getPass()).get();
        HttpEntity entity = new StringEntity(JSON.toJSONString(request), ContentType.APPLICATION_JSON);
        String response = httpInvoker.request(url, "POST", entity);
        SessionToken token = JSON.parseObject(response, SessionToken.class);
        logger.info("Get Smart-x Token: userId={}", token.getUserId());
        logger.info("Get Smart-x Token: token={}", token.getToken());
        logger.info("Get Smart-x Token: createTime={}", token.getCreateTime());
        logger.info("Get Smart-x Token: expireTime={}", token.getExpireTime());
        return token;
    }


}
