package com.bocloud.cmp.provider.storage;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.VolumeTemplateModel;
import com.bocloud.cmp.provider.SmartProvider;
import com.bocloud.cmp.provider.smart.sdk.api.StoragePolicyApi;
import com.bocloud.cmp.provider.smart.sdk.convert.StoragePolicyConvert;
import com.bocloud.cmp.provider.smart.sdk.model.create.StoragePolicyCreateModel;
import com.bocloud.cmp.provider.smart.sdk.model.modify.StoragePolicyModifyModel;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

@Provider("bocloud.cmp.smartx")
public class SmartVolumeTemplateProvider extends SmartProvider implements VolumeTemplateProvider {

    private static final Logger logger = LoggerFactory.getLogger(SmartVolumeTemplateProvider.class);

    private final StoragePolicyApi storagePolicyApi;
    private final StoragePolicyConvert storagePolicyConvert = new StoragePolicyConvert();

    public SmartVolumeTemplateProvider(Butler butler) {
        super(butler);
        this.storagePolicyApi = new StoragePolicyApi(this.getSessionParams());
    }

    @Override
    public BocloudResult list() {
        try {
            List<VolumeTemplateModel> templateModels = storagePolicyConvert.convert(storagePolicyApi.list());
            return new BocloudResult(true, templateModels, "success");
        } catch (Exception e) {
            logger.error("查询存储策略失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult create(VolumeTemplateModel template) {
        try {
            StoragePolicyCreateModel params = new StoragePolicyCreateModel();
            params.setName(template.getName());
            params.setDescription(template.getDescription());

            params.setReplicaNum(template.getReplicaNum());
            params.setThinProvision(template.getThin());
            params.setWhitelist(template.getWhitelist());

            params.setStripeNum(template.getStripeNum());
            params.setStripeSize(template.getStripeSize());

            params.setStoragePoolId("system");
            VolumeTemplateModel templateModel = storagePolicyConvert.convert(storagePolicyApi.create(params));

            return new BocloudResult(true, templateModel, "success");
        } catch (Exception e) {
            logger.error("创建存储策略失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(VolumeTemplateModel template) {
        try {
            StoragePolicyModifyModel params = new StoragePolicyModifyModel();
            params.setName(template.getName());
            params.setDescription(template.getDescription());
            params.setWhitelist(template.getWhitelist());

            storagePolicyApi.modify(template.getUuid(), params);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("修改存储策略失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult remove(String uuid) {
        try {
            storagePolicyApi.remove(uuid);
            return new BocloudResult(true, "success");
        } catch (Exception e) {
            logger.error("删除存储策略失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String uuid) {
        try {
            VolumeTemplateModel templateModel = storagePolicyConvert.convert(storagePolicyApi.get(uuid));
            return new BocloudResult(true, templateModel, "success");
        } catch (Exception e) {
            logger.error("查询存储策略失败: {}", e.getMessage(), e);
            return new BocloudResult(false, e.getMessage());
        }
    }
}
