package com.bocloud.cmp.provider.handler;

import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.w3c.dom.DOMException;

public final class VcSessionHandler implements SOAPHandler<SOAPMessageContext> {

	private final String vcSessionCookie;

	public VcSessionHandler(String vcSessionCookie) {
		this.vcSessionCookie = vcSessionCookie;
	}

	@Override
	public boolean handleMessage(SOAPMessageContext smc) {
		if (isOutgoingMessage(smc)) {
			try {
				SOAPHeader header = getSOAPHeader(smc);

				SOAPElement vcsessionHeader = header
						.addChildElement(new javax.xml.namespace.QName("#", "vcSessionCookie"));
				vcsessionHeader.setValue(vcSessionCookie);

			} catch (DOMException e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			} catch (SOAPException e) {
				e.printStackTrace();
				throw new RuntimeException(e);
			}
		}
		return true;
	}

	@Override
	public void close(MessageContext arg0) {
	}

	@Override
	public boolean handleFault(SOAPMessageContext arg0) {
		return false;
	}

	@Override
	public Set<QName> getHeaders() {
		return null;
	}

	/**
	 * Returns the header. If not present then adds one and return the same
	 * 
	 * @param smc
	 * @return
	 * @throws SOAPException
	 */
	SOAPHeader getSOAPHeader(SOAPMessageContext smc) throws SOAPException {
		return smc.getMessage().getSOAPPart().getEnvelope().getHeader() == null
				? smc.getMessage().getSOAPPart().getEnvelope().addHeader()
				: smc.getMessage().getSOAPPart().getEnvelope().getHeader();
	}

	/**
	 * Returns true if the {@link SOAPMessageContext} is part of the request
	 * 
	 * @param smc
	 * @return
	 */
	boolean isOutgoingMessage(SOAPMessageContext smc) {
		Boolean outboundProperty = (Boolean) smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);
		return outboundProperty.booleanValue();
	}
}
