package com.bocloud.cmp.provider.pbm;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.pbm.PbmProfileModel;
import com.bocloud.cmp.provider.PbmProvider;
import com.bocloud.cmp.provider.utils.PbmDataUtil;
import com.bocloud.cmp.provider.utils.PbmUtil;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.vmware.pbm.PbmProfile;
import com.vmware.pbm.PbmProfileCategoryEnum;
import com.vmware.pbm.PbmProfileId;

/**
 * 存储策略操作类
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年6月15日
 *
 */
@Provider("bocloud.cmp.vmware")
public class SpbmProfileProvider extends PbmProvider implements PbmProfileProvider {

	public SpbmProfileProvider(Butler butler) {
		super(butler);
	}

	private static Logger logger = LoggerFactory.getLogger(SpbmProfileProvider.class);

	@Override
	public BocloudResult list() {
		try {
			List<PbmProfileModel> profileModels = new ArrayList<>();
			// 查询REQUIREMENT 类型profile
			List<PbmProfileId> profileIds = this.getPbmPort().pbmQueryProfile(
					this.getPbmServiceContent().getProfileManager(), PbmUtil.getStorageResourceType(),
					PbmProfileCategoryEnum.REQUIREMENT.name());
			if (!ListTool.isEmpty(profileIds)) {
				List<PbmProfile> profiles = this.getPbmPort()
						.pbmRetrieveContent(this.getPbmServiceContent().getProfileManager(), profileIds);
				for (PbmProfile profile : profiles) {
					profileModels.add(PbmDataUtil.assembleProfile(profile, PbmProfileCategoryEnum.REQUIREMENT.name()));
				}
			}
			// 查询DATA_SERVICE_POLICY 类型profile
			profileIds = this.getPbmPort().pbmQueryProfile(this.getPbmServiceContent().getProfileManager(),
					PbmUtil.getStorageResourceType(), PbmProfileCategoryEnum.DATA_SERVICE_POLICY.name());
			if (!ListTool.isEmpty(profileIds)) {
				List<PbmProfile> profiles = this.getPbmPort()
						.pbmRetrieveContent(this.getPbmServiceContent().getProfileManager(), profileIds);
				for (PbmProfile profile : profiles) {
					profileModels.add(
							PbmDataUtil.assembleProfile(profile, PbmProfileCategoryEnum.DATA_SERVICE_POLICY.name()));
				}
			}
			// 查询RESOURCE 类型profile
			profileIds = this.getPbmPort().pbmQueryProfile(this.getPbmServiceContent().getProfileManager(),
					PbmUtil.getStorageResourceType(), PbmProfileCategoryEnum.RESOURCE.name());
			if (!ListTool.isEmpty(profileIds)) {
				List<PbmProfile> profiles = this.getPbmPort()
						.pbmRetrieveContent(this.getPbmServiceContent().getProfileManager(), profileIds);
				for (PbmProfile profile : profiles) {
					profileModels.add(PbmDataUtil.assembleProfile(profile, PbmProfileCategoryEnum.RESOURCE.name()));
				}
			}
			return new BocloudResult(true, profileModels, "查询成功");
		} catch (Exception e) {
			logger.error("Get error message：{}", e.getMessage());
			return new BocloudResult(false, "查询失败");
		}
	}

}
