package com.bocloud.cmp.provider.network;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.bocloud.cmp.common.enums.TencentEnum;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SecurityGroupModel;
import com.bocloud.cmp.provider.TencentProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.tencentconvertor.SecurityGroupConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.tencentcloudapi.vpc.v20170312.VpcClient;
import com.tencentcloudapi.vpc.v20170312.models.CreateSecurityGroupRequest;
import com.tencentcloudapi.vpc.v20170312.models.CreateSecurityGroupResponse;
import com.tencentcloudapi.vpc.v20170312.models.DeleteSecurityGroupRequest;
import com.tencentcloudapi.vpc.v20170312.models.DeleteSecurityGroupResponse;
import com.tencentcloudapi.vpc.v20170312.models.DescribeSecurityGroupsRequest;
import com.tencentcloudapi.vpc.v20170312.models.DescribeSecurityGroupsResponse;
import com.tencentcloudapi.vpc.v20170312.models.Filter;
import com.tencentcloudapi.vpc.v20170312.models.ModifySecurityGroupAttributeRequest;
import com.tencentcloudapi.vpc.v20170312.models.ModifySecurityGroupAttributeResponse;
import com.tencentcloudapi.vpc.v20170312.models.SecurityGroup;

/**
 * 
 * @author ymh
 * @时间 2018年11月15日
 */
@Provider("bocloud.cmp.tencent")
public class TencentSecuGroupProvider extends TencentProvider implements SecurityGroupProvider {

	private static Logger logger = LoggerFactory.getLogger(TencentSecuGroupProvider.class);

	private BeanConvertor<SecurityGroupModel, SecurityGroup> convertor = new SecurityGroupConvertor();

	public TencentSecuGroupProvider(Butler butler) {
		super(butler);
	}

	@Override
	public BocloudResult list() {
		try {
			DescribeSecurityGroupsRequest req = new DescribeSecurityGroupsRequest();
			Filter filter = new Filter();
			filter.setName("project-id");
			filter.setValues(new String[] { this.getProjectId() });
			Filter[] filters = new Filter[] { filter };
			req.setFilters(filters);
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DescribeSecurityGroupsResponse resp = client.DescribeSecurityGroups(req);
			SecurityGroup[] groupSet = resp.getSecurityGroupSet();
			return new BocloudResult(true, convertor.convertList(Arrays.asList(groupSet)), "查询成功");
		} catch (Exception e) {
			logger.error("查询失败 ： " + e);
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult create(SecurityGroupModel group) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			CreateSecurityGroupRequest req = new CreateSecurityGroupRequest();
			req.setGroupName(group.getName());
			String remark = group.getRemark();
			if (null == remark || "".equals(remark)) {
				remark = "";
			}
			req.setGroupDescription(remark);
			String proId = this.getProjectId();
			if (null != proId && !"".equals(proId)) {
				req.setProjectId(proId);
			}
			CreateSecurityGroupResponse resp = client.CreateSecurityGroup(req);
			SecurityGroup securityGroup = resp.getSecurityGroup();
			SecurityGroupModel groupModel = convertor.convertModel(securityGroup);
			groupModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, groupModel, "创建成功");
		} catch (Exception e) {
			logger.error("创建失败 ： " + e);
			return new BocloudResult(false, "创建失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult remove(String sgId) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			DeleteSecurityGroupRequest req = new DeleteSecurityGroupRequest();
			req.setSecurityGroupId(sgId);
			DeleteSecurityGroupResponse resp = client.DeleteSecurityGroup(req);
			return new BocloudResult(true, resp.getRequestId(), "删除成功");
		} catch (Exception e) {
			logger.error("删除失败 ： " + e);
			return new BocloudResult(false, "删除失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult detail(String sgId) {
		try {
			DescribeSecurityGroupsRequest req = new DescribeSecurityGroupsRequest();
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			req.setSecurityGroupIds(new String[] { sgId });
			DescribeSecurityGroupsResponse resp = client.DescribeSecurityGroups(req);
			SecurityGroup[] securityGroupSet = resp.getSecurityGroupSet();
			SecurityGroup group = null;
			if (null != securityGroupSet && securityGroupSet.length > 0) {
				group = securityGroupSet[0];
			}
			return new BocloudResult(true, convertor.convertModel(group), "查询成功");
		} catch (Exception e) {
			logger.error("查询失败 ： " + e);
			return new BocloudResult(false, "查询失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult modify(SecurityGroupModel model) {
		try {
			VpcClient client = (VpcClient) this.getClient(TencentEnum.VPC);
			ModifySecurityGroupAttributeRequest req = new ModifySecurityGroupAttributeRequest();
			req.setSecurityGroupId(model.getGroupUuid());
			if (null != model.getName() && !"".equals(model.getName())) {
				req.setGroupName(model.getName());
			}
			if (null != model.getRemark() && !"".equals(model.getRemark())) {
				req.setGroupDescription(model.getRemark());
			}
			ModifySecurityGroupAttributeResponse resp = client.ModifySecurityGroupAttribute(req);
			return new BocloudResult(true, resp.getRequestId(), "修改成功");
		} catch (Exception e) {
			logger.error("修改失败 ： " + e);
			return new BocloudResult(false, "修改失败", e.getMessage());
		}
	}

}
