package com.bocloud.cmp.tencentconvertor;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.bocloud.cmp.common.enums.TencentConstants;
import com.bocloud.cmp.model.FloatingIpModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.tencentcloudapi.vpc.v20170312.models.Address;

/**
 * 地域 转换
 * 
 * @author ymh
 * @时间 2018年11月22日
 */
public class FloatingIpConvertor implements BeanConvertor<FloatingIpModel, Address> {

    private static Logger logger = LoggerFactory.getLogger(FloatingIpConvertor.class);

    @Override
    public List<FloatingIpModel> convertList(List<? extends Address> objects) {
        List<FloatingIpModel> regionModels = new ArrayList<>();
        for (Address object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public FloatingIpModel convertModel(Address info) {
        FloatingIpModel model = new FloatingIpModel();
        if (null != info) {

            String addressName = info.getAddressName();
            if (StringUtils.isEmpty(addressName)) {
                addressName = info.getAddressId();
            }
            model.setName(addressName);
            model.setFloatingIpAddress(info.getAddressIp());
            model.setFloatingIpUuid(info.getAddressId());
            model.setFloatingNetworkName(info.getNetworkInterfaceId());
            model.setInstanceId(info.getInstanceId());
            model.setStatus(info.getAddressStatus());
            model.setNetworkCardUuid(info.getNetworkInterfaceId());
            String createdTime = info.getCreatedTime();
            try {
                Date date = DateTools.transfer(TencentConstants.DATE_FORMAT, createdTime, "Z");
                model.setGmtCreate(date);
            } catch (Exception e) {
                logger.error("时间转换Error : " + e);
            }
        }
        return model;
    }
}
