package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.compute.TianyiServerProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.FlavorConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Hu Wentao
 * @create 2020-10-10 15:41
 * @Description: 规格
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiFlavorProvider extends TianyiServerProvider implements FlavorProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiFlavorProvider.class);

    private BeanConvertor<FlavorModel, JSONObject> convertor = new FlavorConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiFlavorProvider(Butler butler) {
        super(butler);
    }

    //查询服务器规格列表
    //GET http://ctyun-api-url/apiproxy/v3/order/getFlavors
    @Override
    public BocloudResult sync() {
        try {
        	String apiUrl = "/apiproxy/v3/order/getFlavors";
        	JSONObject headers = new JSONObject();
        	headers.put("regionId", this.getRegionId());
        	headers.put("zoneId", "cn-jssz1a");

        	Result result = doGet(apiUrl, headers);
        	if (result.isFailed()) {
        	    logger.error("查询规则列表失败:" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "查询规则列表失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        	    logger.error("查询规则列表失败：" + object.getString("message"));
        		return new BocloudResult(false, "查询规则列表失败：" + object.getString("message"));
        	}
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
        		return new BocloudResult(true, "规则列表为空");
        	}
            // 接口问题导致
            List<FlavorModel> flavorModels = convertor.convertList(returnObj);
			// 由于接口问题导致 缺少一条数据 该数据底层确实存在 但接口没有返回 现特殊处理
			if (!ListTool.isEmpty(flavorModels)) {
				List<String> flavorUuids = flavorModels.stream().map(FlavorModel::getFlavorUuid)
						.collect(Collectors.toList());
				if (!flavorUuids.contains("s3.4xlarge.4")) {
					FlavorModel flavorModel = flavorModels.get(0);
					FlavorModel newFlavorModel = new FlavorModel();
					newFlavorModel.setFlavorUuid("s3.4xlarge.4");
					newFlavorModel.setName("通用计算型s3");
					newFlavorModel.setCpu(16);
					newFlavorModel.setMemory(new BigDecimal(64));
					newFlavorModel.setFamily(flavorModel.getFamily());
					newFlavorModel.setStatus(flavorModel.getStatus());
					newFlavorModel.setZone(flavorModel.getZone());
					newFlavorModel.setPerformanceType(flavorModel.getPerformanceType());
					flavorModels.add(newFlavorModel);
				}
			}

            return new BocloudResult(true, flavorModels, "查询规则列表成功");
        } catch (Exception e) {
        	logger.error("查询规则列表失败 ： " + e);
        	return new BocloudResult(false, "查询规则列表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult create(FlavorModel flavorModel) {
        return null;
    }

    @Override
    public BocloudResult remove(String s) {
        return null;
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        return null;
    }
}
