package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.storage.TianyiVolumeProvider;
import com.bocloud.cmp.provider.tianyiconvertor.ClusterConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.apache.commons.collections.ListUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author Hu Wentao
 * @Description: 集群资源
 * @date 2020/12/30 11:51
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiClusterProvider extends TianyiProvider implements ClusterProvider {

    Butler butler;

    private static Logger logger = LoggerFactory.getLogger(TianyiClusterProvider.class);

    private BeanConvertor<ClusterModel, JSONObject> convertor = new ClusterConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiClusterProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult create(ClusterOperateModel clusterOperateModel) {
        return null;
    }

    @Override
    public BocloudResult remove(ClusterOperateModel clusterOperateModel) {
        return null;
    }

    @Override
    public BocloudResult detail(String s) {
        return null;
    }

    @Override
    public BocloudResult detailCluster(String clusterVal) {
        return null;
    }

    @Override
    public BocloudResult create(ClusterModel clusterModel) {
        return null;
    }

    @Override
    public BocloudResult modify(ClusterModel clusterModel) {
        return null;
    }

    @Override
    public BocloudResult remove(String s) {
        return null;
    }

    @Override
    public BocloudResult addHost(String s, String s1) {
        return null;
    }

    @Override
    public BocloudResult removeHost(String s, String s1) {
        return null;
    }

    @Override
    public BocloudResult summary(String s) {
        return null;
    }

    @Override
    public BocloudResult queryByName(String s) {
        return null;
    }

    @Override
    public BocloudResult list() {
        try {
            BocloudResult computeColony = this.getComputeColony();

            List<JSONObject> cluseterList = ((List<JSONObject>) computeColony.getData()).stream().peek(item -> item.put("projectId", this.getProjectId())).collect(Collectors.toList());

            List<ClusterModel> clusterModels = convertor.convertList(cluseterList);

            BocloudResult queryEvsQuotaResult = this.queryEvsQuota();
            if (queryEvsQuotaResult.isFailed()) {
                return queryEvsQuotaResult;
            }
            JSONObject data = (JSONObject) queryEvsQuotaResult.getData();
            for (ClusterModel clusterModel : clusterModels) {
                clusterModel.setDiskTotal(data.getDouble("gigabytes"));
                clusterModel.setDiskUsed(this.getUsedVolume());
            }

            return new BocloudResult(true, clusterModels, "查询集群资源成功");
        } catch (Exception e) {
            logger.error("查询集群资源失败 ： " + e);
            return new BocloudResult(false, "查询集群资源失败", e.getMessage());
        }
    }

    // 查询projectId
    //GET http://ctyun-api-url/apiproxy/v3/ondemand/queryProjectIds
    public BocloudResult getProject() {
        try {
            String apiUrl = "/apiproxy/v3/ondemand/queryProjectIds";
            JSONObject headers = new JSONObject();

            Result result = doGet(apiUrl, headers);

            if (result.isFailed()) {
                logger.error("查询项目列表失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "查询规则列表失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询项目列表失败：" + object.getString("message"));
                return new BocloudResult(false, "查询项目列表失败：" + object.getString("message"));
            }
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                logger.error("项目列表为空");
                return new BocloudResult(true, "项目列表为空");
            }
            for (JSONObject project : returnObj) {
                if (project.getString("id").equals(this.getProjectId())) {
                    return new BocloudResult(true, project, "查询项目列表成功");
                }
            }
            logger.error("此项目不存在");
            return new BocloudResult(false, "此项目不存在");
        } catch (Exception e) {
            logger.error("查询项目列表失败 ： " + e);
            return new BocloudResult(false, "查询项目列表失败", e.getMessage());
        }
    }

    //获取专属计算集群资源
    //GET http://ctyun-api-url /apiproxy/v3/hws/getComputeColony
    public BocloudResult getComputeColony() {
        try {
            String apiUrl = "/apiproxy/v3/hws/getComputeColony";
            JSONObject headers = new JSONObject();

            BocloudResult project = this.getProject();
            if (project.isFailed()) {
                logger.error("查询项目列表失败");
                return new BocloudResult(true, "查询项目列表失败");
            }

            JSONObject projectData = (JSONObject) project.getData();
            headers.put("regionId", projectData.getString("name"));

            Result result = doGet(apiUrl, headers);

            if (result.isFailed()) {
                logger.error("获取专属计算集群资源失败:" + result.getMessage());
                return new BocloudResult(false, "获取专属计算集群资源失败:" + result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("获取专属计算集群资源失败");
                return new BocloudResult(false, "获取专属计算集群资源失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("获取专属计算集群资源失败：" + object.getString("message"));
                return new BocloudResult(false, "获取专属计算集群资源失败：" + object.getString("message"));
            }
            JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
            List<JSONObject> projects = JSONArray.parseArray(returnObj.getString("dedicated_cluster"), JSONObject.class);
            List<JSONObject> projectList = new ArrayList<>();
            for (JSONObject data : projects) {
                if (data.getString("project_id").equals(this.getProjectId())) {
                    projectList.add(data);
                }
            }
            return new BocloudResult(true, projectList, "获取专属计算集群资源成功");
        } catch (Exception e) {
            logger.error("获取专属计算集群资源失败 ： " + e);
            return new BocloudResult(false, "获取专属计算集群资源失败", e.getMessage());
        }
    }

    //查询磁盘配额
    //GET http://ctyun-api-url/apiproxy/v3/queryEVsQuota
    public BocloudResult queryEvsQuota() {
        try {
            String apiUrl = "/apiproxy/v3/queryEVsQuota";
            JSONObject headers = new JSONObject();

            headers.put("regionId", this.getRegionId());
            headers.put("projectId", this.getProjectId());

            System.out.println(headers);

            Result result = doGet(apiUrl, headers);

            if (result.isFailed()) {
                logger.error("查询磁盘配额失败:" + result.getMessage());
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                logger.error("查询磁盘配额失败");
                return new BocloudResult(false, "查询磁盘配额失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                logger.error("查询磁盘配额失败：" + object.getString("message"));
                return new BocloudResult(false, "查询磁盘配额失败：" + object.getString("message"));
            }
            JSONObject returnObj = object.getJSONObject("returnObj");

            return new BocloudResult(true, returnObj, "查询磁盘配额成功");
        } catch (Exception e) {
            logger.error("查询磁盘配额失败 ： " + e);
            return new BocloudResult(false, "查询磁盘配额失败", e.getMessage());
        }
    }

    //计算磁盘已用量
    public Double getUsedVolume() {
        Double usedVolume = 0d;

        TianyiVolumeProvider tianyiVolumeProvider = new TianyiVolumeProvider(butler);
        BocloudResult bocloudResult = tianyiVolumeProvider.list();
        List<VolumeModel> volumeModel = (List<VolumeModel>) bocloudResult.getData();

        if (!ListTool.isEmpty(volumeModel)) {
            for (VolumeModel volume : volumeModel) {
                usedVolume += volume.getSize();
            }
        }
        return usedVolume;
    }
}
