package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.model.h3c.H3cMigrateModel;
import com.bocloud.cmp.model.h3c.H3cServerModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.TianyiSubnetProvider;
import com.bocloud.cmp.provider.storage.TianyiVolumeProvider;
import com.bocloud.cmp.provider.tianyiconvertor.ServerConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @Package: com.bocloud.cmp.provider.compute
 * @ClassName: TianyiServerProvider
 * @Author: 胡文涛
 * @CreateTime: 2020/9/22 11:50
 * @Description: 云主机
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiServerProvider extends TianyiProvider implements ServerProvider {

	private Logger logger = LoggerFactory.getLogger(TianyiServerProvider.class);

	BeanConvertor<ServerModel, JSONObject> serverConvertor = new ServerConvertor();

	Butler butler = null;

	/**
	 * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
	 *
	 * @param butler 云平台认证信息
	 */
	public TianyiServerProvider(Butler butler) {
		super(butler);
		this.butler = butler;
	}

	// 查询云主机列表
	// GET http://ctyun-api-url/apiproxy/v3/ondemand/queryVMs
	@Override
	public BocloudResult list() {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/queryVMs";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				logger.error(result.getMessage());
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				logger.error("查询云主机列表失败:" + returnObj);
				return new BocloudResult(false, "查询云主机列表失败:" + returnObj);
			}
			List<ServerModel> serverList = new ArrayList<>();

			List<JSONObject> servers = JSONObject.parseArray(returnObj.getString("servers"), JSONObject.class);
			if (ListTool.isEmpty(servers)) {
				return new BocloudResult(true, serverList, "云主机列表为空");
			}

			return new BocloudResult(true, serverConvertor.convertList(servers), "查询云主机列表成功");
		} catch (Exception e) {
			logger.error("查询云主机列表失败 ： " + e);
			return new BocloudResult(false, "查询云主机列表失败", e.getMessage());
		}

	}

	@Override
	public BocloudResult list(String s) {
		return null;
	}

	// 创建主机时使用秘钥验证方式
	// POST http://ctyun-api-url/apiproxy/v3/order/placeNewPurchaseOrder
	public BocloudResult createByKeypair(ServerModel serverModel) {
		try {
			String apiUrl = "/apiproxy/v3/order/placeNewPurchaseOrder";

			JSONObject params = new JSONObject();
			JSONObject orderDetailJson = new JSONObject();

			JSONArray orders = new JSONArray();
			JSONObject order = new JSONObject();

			order.put("instanceCnt", serverModel.getCount());
			order.put("cycleCnt", "1");
			order.put("cycleType", "3");

			JSONArray items = new JSONArray();
			JSONObject item1 = new JSONObject();
			item1.put("master", "true");
			item1.put("resourceType", "VMC");
			item1.put("serviceTag", "HWS");

			JSONObject itemConfig1 = new JSONObject();

			itemConfig1.put("memSize", serverModel.getMemory());
			itemConfig1.put("cpuNum", serverModel.getCpu());
			itemConfig1.put("flavorType", serverModel.getInstanceType());
			itemConfig1.put("osType", serverModel.getImageType());
			itemConfig1.put("availability_zone", serverModel.getZone());
			itemConfig1.put("name", serverModel.getName());
			itemConfig1.put("imageRef", serverModel.getImageUuid());
			itemConfig1.put("vpcid", serverModel.getVpcUuid());

			JSONArray securityGroups = new JSONArray();
			for (String id : serverModel.getSecurityGroups()) {
				JSONObject securityGroup = new JSONObject();
				securityGroup.put("id", id);
				securityGroups.add(securityGroup);
			}

			itemConfig1.put("security_groups", securityGroups);

			JSONArray nics = new JSONArray();
			JSONObject nic = new JSONObject();
			nic.put("subnet_id", serverModel.getSubnetUuid());
			TianyiSubnetProvider tianyiSubnetProvider = new TianyiSubnetProvider(butler);
			BocloudResult detail = tianyiSubnetProvider.detail(serverModel.getSubnetUuid());
			if (detail.isSuccess()) {
				SubnetModel subnetModel = (SubnetModel) detail.getData();
				nic.put("ip_address", subnetModel.getAddress());
			}
			nics.add(nic);
			itemConfig1.put("nics", nics);

			itemConfig1.put("key_name", serverModel.getKeypairName());
			itemConfig1.put("regionId", this.getRegionId());

			item1.put("itemConfig", itemConfig1);
			item1.put("itemValue", 1);
			items.add(item1);

			JSONObject item2 = new JSONObject();
			item2.put("master", "false");
			item2.put("resourceType", "EBSC");
			item2.put("serviceTag", "HWS");

			JSONObject itemConfig2 = new JSONObject();
			itemConfig2.put("volumeType", serverModel.getDiskType());

			item2.put("itemConfig", itemConfig2);
			item2.put("isSystemVolume", true);
			items.add(item2);

			if (serverModel.getDataDisks() != null && !"".equals(serverModel.getDataDisks())) {
				List<JSONObject> jsonObjects = JSONArray.parseArray(serverModel.getDataDisks(), JSONObject.class);
				for (JSONObject object : jsonObjects) {
					JSONObject item3 = new JSONObject();
					item3.put("master", "false");
					item3.put("resourceType", "EBSC");
					item3.put("serviceTag", "HWS");

					JSONObject itemConfig3 = new JSONObject();
					itemConfig3.put("volumeType", object.getString("type"));

					item3.put("itemConfig", itemConfig3);
					item3.put("itemValue", object.getString("size"));
					items.add(item3);
				}
			}

			order.put("items", items);
			orders.add(order);
			orderDetailJson.put("orders", orders);
			params.put("orderDetailJson", orderDetailJson);

			Result result = doPost(apiUrl, params);

			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
					String message = jsonObject.getString("message");
					message = message.replace(';', ' ');
					JSONObject object = JSONObject.parseObject(message);
					if (object != null && object.getString("error") != null) {
						String errorMessage = JSONObject.parseObject(object.getString("error")).getString("message");
						return new BocloudResult(false, errorMessage);
					}
				}
				return new BocloudResult(false, "创建云主机失败");
			}
			JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
			if (!returnObj.getBoolean("status")) {
				return new BocloudResult(false, "创建云主机失败");
			}
			String serverId = (String) this.queryServerJob(this.getRegionId(), returnObj.getString("data")).getData();
			if (serverId == null) {
				return new BocloudResult(false, "创建云主机失败：所选规格下的云主机已售尽！");
			}
			Long startTime = System.currentTimeMillis();
			while (true) {
				ServerModel model = (ServerModel) detail(serverId).getData();
				if (model != null && !"".equals(model)) {
					model.setRegionId(serverModel.getRegionId());
					return new BocloudResult(true, model, "创建云主机成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
		} catch (Exception e) {
			logger.error("创建云主机失败 ： " + e);
			return new BocloudResult(false, "创建云主机失败", e.getMessage());
		}
	}

	// 创建主机（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/createVM
	@Override
	public BocloudResult create(ServerModel serverModel) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/createVM";

			JSONObject params = new JSONObject();

			JSONObject server = new JSONObject();
			server.put("availability_zone", serverModel.getZone());
			server.put("name", serverModel.getName());
			server.put("imageRef", serverModel.getImageUuid());
			server.put("osType", serverModel.getImageType());
			// 系统盘
			JSONObject rootVolume = new JSONObject();
			rootVolume.put("volumetype", serverModel.getSystemDiskCategory());
			rootVolume.put("size", serverModel.getSystemDiskSize());
			server.put("root_volume", rootVolume);
			// 数据盘
			/*
			 * [{ "volumetype":"SATA", "size":10 }]
			 */
			if (serverModel.getDataDisks() != null && !"".equals(serverModel.getDataDisks())) {
				List<JSONObject> volumes = new ArrayList<>();
				List<JSONObject> jsonObjects = JSONArray.parseArray(serverModel.getDataDisks(), JSONObject.class);
				for (JSONObject object : jsonObjects) {
					JSONObject volume = new JSONObject();
					volume.put("volumetype", object.getString("type"));
					volume.put("size", object.getString("size"));
					volumes.add(volume);
				}
				server.put("data_volumes", volumes);
			}

			server.put("flavorRef", serverModel.getInstanceType());
			server.put("vpcid", serverModel.getVpcUuid());
			// 安全组
			List<JSONObject> securityGroups = new ArrayList<>();

			for (String id : serverModel.getSecurityGroups()) {
				JSONObject securityGroup = new JSONObject();
				securityGroup.put("id", id);
				securityGroups.add(securityGroup);
			}
			server.put("security_groups", securityGroups);
			// 子网
			List<JSONObject> nics = new ArrayList<>();
			JSONObject subnetId = new JSONObject();
			subnetId.put("subnet_id", serverModel.getSubnetUuid());
			nics.add(subnetId);
			server.put("nics", nics);

			server.put("adminPass", serverModel.getPassword());
			server.put("key_name", serverModel.getKeypairName());

			server.put("count", serverModel.getCount());

			JSONObject extendparam = new JSONObject();
			extendparam.put("regionID", serverModel.getRegionId());
			server.put("extendparam", extendparam);

			JSONObject createVMInfo = new JSONObject();
			createVMInfo.put("server", server);
			params.put("createVMInfo", createVMInfo);

			Result result = doPost(apiUrl, params);

			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				if (jsonObject.getInteger("statusCode").equals(900) && jsonObject.getString("message") != null) {
					String message = jsonObject.getString("message");
					message = message.replace(';', ' ');
					JSONObject object = JSONObject.parseObject(message);
					if (object != null && object.getString("error") != null) {
						String errorMessage = JSONObject.parseObject(object.getString("error")).getString("message");
						return new BocloudResult(false, errorMessage);
					}
				}
				return new BocloudResult(false, "创建云主机失败");
			}
			JSONObject returnObj = JSONObject.parseObject(jsonObject.getString("returnObj"));
			if (!returnObj.getBoolean("status")) {
				return new BocloudResult(false, "创建云主机失败");
			}
			String serverId = (String) this.queryServerJob(this.getRegionId(), returnObj.getString("data")).getData();
			if (serverId == null) {
				return new BocloudResult(false, "创建云主机失败：所选规格下的云主机已售尽！");
			}
			Long startTime = System.currentTimeMillis();
			while (true) {
				ServerModel model = (ServerModel) detail(serverId).getData();
				if (model != null && !"".equals(model)) {
					model.setRegionId(serverModel.getRegionId());
					return new BocloudResult(true, model, "创建云主机成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
		} catch (Exception e) {
			logger.error("创建云主机失败 ： " + e);
			return new BocloudResult(false, "创建云主机失败", e.getMessage());
		}
	}

	// 查询JOB状态信息
	// GET http://ctyun-api-url/apiproxy/v3/queryJobStatus
	public BocloudResult queryServerJob(String regionId, String jobId) {
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(100);
				String apiUrl = "/apiproxy/v3/queryJobStatus";
				JSONObject params = new JSONObject();
				params.put("regionId", regionId);
				params.put("jobId", jobId);

				Result result = doGet(apiUrl, params);
				if (result.isFailed()) {
					return new BocloudResult(false, result.getMessage());
				}
				JSONObject object = JSONObject.parseObject(result.getMessage());
				if (null == object) {
					return new BocloudResult(false, "根据jobId查询JOB状态信息失败");
				}
				JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
				if (object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)
						&& "SUCCESS".equals(returnObj.getString("status"))) {
					JSONObject entities = JSONObject
							.parseObject(JSONObject.parseObject(object.getString("returnObj")).getString("entities"));
					List<JSONObject> subJobs = JSONArray.parseArray(entities.getString("sub_jobs"), JSONObject.class);
					if (ListTool.isEmpty(subJobs)) {
						return new BocloudResult(true, entities.getString("server_id"), "根据jobId查询任务成功");
					} else {
						for (JSONObject jsonObject : subJobs) {
							if ("SUCCESS".equals(jsonObject.getString("status"))
									&& !"".equals(jsonObject.getString("entities"))
									&& null != jsonObject.getString("entities")) {
								return new BocloudResult(true,
										JSONObject.parseObject(jsonObject.getString("entities")).getString("server_id"),
										"根据jobId查询JOB状态信息成功");
							}
						}
					}
				} else if (object.getInteger("statusCode").equals(900) && object.getString("message") != null
						&& !"".equals(object.getString("message"))) {
					return new BocloudResult(false, "创建云主机失败");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}

		} catch (Exception e) {
			logger.error("根据jobId查询JOB状态信息失败 ： " + e);
			return new BocloudResult(false, "根据jobId查询磁盘JOB状态信息失败", e.getMessage());
		}
	}

	// 修改主机名称
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/reviseVM
	@Override
	public BocloudResult modify(ServerModel serverModel) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/reviseVM";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			headers.put("serverId", serverModel.getInstanceId());
			if (!StringUtils.isEmpty(serverModel.getName())) {
				headers.put("name", serverModel.getName());
			}
			if (!StringUtils.isEmpty(serverModel.getRemark())) {
				headers.put("description", serverModel.getRemark());
			}
			Result result = doPost(apiUrl, headers);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "修改云主机失败");
			}
			return new BocloudResult(true, "修改主机名称成功");
		} catch (Exception e) {
			logger.error("查询云主机列表失败 ： " + e);
			return new BocloudResult(false, "查询云主机列表失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult migrate(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult migrate(OSMigrateModel osMigrateModel) {
		return null;
	}

	// 删除云主机（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/deleteVM
	@Override
	public BocloudResult remove(String vmId, String name) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/deleteVM";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject jsonObject = JSONObject.parseObject(result.getMessage());
			if (!jsonObject.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "删除云主机失败");
			}
			Boolean status = JSONObject.parseObject(jsonObject.getString("returnObj")).getBoolean("status");
			if (!status) {
				return new BocloudResult(false, "删除云主机失败");
			}
			return new BocloudResult(true, "删除云主机成功");
		} catch (Exception e) {
			logger.error("删除云主机失败 ： " + e);
			return new BocloudResult(false, "删除云主机失败", e.getMessage());
		}
	}

	// 查询云主机详情
	// GET http://ctyun-api-url/apiproxy/v3/ondemand/queryVMDetail
	public BocloudResult getServerDetail(String vmId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/queryVMDetail";
			JSONObject headers = new JSONObject();
			headers.put("regionId", this.getRegionId());
			headers.put("vmId", vmId);
			Result result = doGet(apiUrl, headers);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "查询云主机详情失败");
			}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			if (null == returnObj) {
				return new BocloudResult(true, "该云主机不存在");
			}
			ServerModel serverModel = convertModel(returnObj);

			return new BocloudResult(true, serverModel, "查询云主机详情成功");
		} catch (Exception e) {
			logger.error("查询云主机详情失败 ： " + e);
			return new BocloudResult(false, "查询云主机详情失败", e.getMessage());
		}
	}

	public ServerModel convertModel(JSONObject jsonObject) {
		ServerModel serverModel = new ServerModel();
		if (null != jsonObject) {
			serverModel.setInstanceId(jsonObject.getString("hostId"));
			serverModel.setName(jsonObject.getString("name"));

			serverModel.setTenantUuid(jsonObject.getString("tenant_id"));
			// 网卡
			List<JSONObject> privateIpList = JSONArray.parseArray(jsonObject.getString("privateIps"), JSONObject.class);
			if (!ListTool.isEmpty(privateIpList)) {
				JSONArray privateIps = new JSONArray();
				for (JSONObject networkInterface : privateIpList) {
					privateIps.add(networkInterface.getString("address"));

				}
				serverModel.setPrivateIps(String.valueOf(privateIps));
			}

			// 公网
			List<JSONObject> publicIpList = JSONArray.parseArray(jsonObject.getString("publicIps"), JSONObject.class);
			if (!ListTool.isEmpty(publicIpList)) {
				JSONArray publicIps = new JSONArray();
				for (JSONObject publicIp : publicIpList) {
					publicIps.add(publicIp.getString("address"));

				}
				serverModel.setPublicIps(String.valueOf(publicIps));
			}

			// 云盘
			TianyiVolumeProvider tianyiVolumeProvider = new TianyiVolumeProvider(butler);
			List<JSONObject> volumes = JSONArray.parseArray(jsonObject.getString("volumes"), JSONObject.class);
			if (!ListTool.isEmpty(volumes)) {
				List<VolumeModel> volumeModels = new ArrayList<>();
				for (JSONObject volume : volumes) {
					BocloudResult volumeProvider = tianyiVolumeProvider.detail(volume.getString("id"));
					if (volumeProvider.isSuccess()) {
						VolumeModel volumeModel = (VolumeModel) volumeProvider.getData();
						volumeModels.add(volumeModel);
					}

				}
				serverModel.setDiskDevices(volumeModels);
			}
			String created = jsonObject.getString("created");
			if (created != null && !"".equals(created)) {
				serverModel.setGmtVmCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", created, "Z"));
			}
			String status = jsonObject.getString("status");
			if ("ACTIVE".equalsIgnoreCase(status)) {
				serverModel.setStatus("RUNNING");
			} else if ("SHUTOFF".equalsIgnoreCase(status)) {
				serverModel.setStatus("STOPPED");
			} else if ("ERROR".equalsIgnoreCase(status)) {
				serverModel.setStatus("EXCEPTION");
			} else {
				serverModel.setStatus(status);
			}

			serverModel.setFlavorUuid(JSONObject.parseObject(jsonObject.getString("flavorObj")).getString("id"));
			serverModel.setCpu(JSONObject.parseObject(jsonObject.getString("flavorObj")).getInteger("cpuNum"));
			serverModel.setMemory(JSONObject.parseObject(jsonObject.getString("flavorObj")).getBigDecimal("memSize"));
			serverModel.setImageUuid(jsonObject.getString("imageId"));

		}
		return serverModel;
	}

	// 查询云主机详情
	// GET http://ctyun-api-url/apiproxy/v3/ondemand/queryVMDetail
	@Override
	public BocloudResult detail(String vmId) {
		try {
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(100);
				BocloudResult serverDetail = getServerDetail(vmId);
				if (serverDetail.isSuccess()) {
					ServerModel serverModel = (ServerModel) serverDetail.getData();
					return new BocloudResult(true, serverModel, "查询云主机详情成功");
				}

				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
		} catch (Exception e) {
			logger.error("查询云主机详情失败 ： " + e);
			return new BocloudResult(false, "查询云主机详情失败", e.getMessage());
		}
	}

	// 开启主机
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/startVM
	@Override
	public BocloudResult start(String vmId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/startVM";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "开启云主机失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "开启云主机失败：" + object.getString("message"));
			}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			if (!returnObj.getBoolean("status")) {
				return new BocloudResult(false, "开启云主机失败");
			}
			String serverId = (String) this.queryServerJob(this.getRegionId(), returnObj.getString("data")).getData();
			ServerModel model = (ServerModel) detail(serverId).getData();
			return new BocloudResult(true, model, "开启云主机成功");
		} catch (Exception e) {
			logger.error("开启云主机失败 ： " + e);
			return new BocloudResult(false, "开启云主机失败", e.getMessage());
		}
	}

	// 关闭主机
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/stopVM
	@Override
	public BocloudResult stop(String vmId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/stopVM";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "关闭云主机失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "关闭云主机失败：" + object.getString("message"));
			}
			Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
			if (!status) {
				return new BocloudResult(false, "关闭云主机失败");
			}
			return new BocloudResult(true, "关闭云主机成功");
		} catch (Exception e) {
			logger.error("关闭云主机失败 ： " + e);
			return new BocloudResult(false, "关闭云主机失败", e.getMessage());
		}
	}

	// 重启主机
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/restartVM
	@Override
	public BocloudResult reboot(String vmId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/restartVM";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			// 重启类型：SOFT：普通重启。HARD：强制重启。
			params.put("type", "SOFT");

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "重启云主机失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "重启云主机失败：" + object.getString("message"));
			}
			Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
			if (!status) {
				return new BocloudResult(false, "重启云主机失败");
			}
			return new BocloudResult(true, "重启云主机成功");
		} catch (Exception e) {
			logger.error("重启云主机失败 ： " + e);
			return new BocloudResult(false, "重启云主机失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult softReboot(String s) {
		return null;
	}

	@Override
	public BocloudResult pause(String s) {
		return null;
	}

	@Override
	public BocloudResult unPause(String s) {
		return null;
	}

	@Override
	public BocloudResult suspend(String s) {
		return null;
	}

	@Override
	public BocloudResult resume(String s) {
		return null;
	}

	// 主机加入安全组
	// POST http://ctyun-api-url/apiproxy/v3/addSecurityGroup
	@Override
	public BocloudResult joinSGroup(String vmId, String securityGroupId) {
		try {
			String apiUrl = "/apiproxy/v3/addSecurityGroup";
			JSONObject params = new JSONObject();
			JSONObject securityGroup = new JSONObject();
			securityGroup.put("regionId", this.getRegionId());
			securityGroup.put("projectId", this.getProjectId());
			securityGroup.put("vmId", vmId);
			securityGroup.put("securityGroupRuleId", securityGroupId);
			params.put("securityGroup", securityGroup);
			System.out.println(params);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "主机加入安全组失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "主机加入安全组失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "主机加入安全组成功");
		} catch (Exception e) {
			logger.error("主机加入安全组失败 ： " + e);
			return new BocloudResult(false, "主机加入安全组失败", e.getMessage());
		}
	}

	// 主机移出安全组
	// POST http://ctyun-api-url/apiproxy/v3/removeSecurityGroup
	@Override
	public BocloudResult leaveSGroup(String vmId, String securityGroupId) {
		try {
			String apiUrl = "/apiproxy/v3/removeSecurityGroup";
			JSONObject params = new JSONObject();
			JSONObject securityGroup = new JSONObject();
			securityGroup.put("regionId", this.getRegionId());
			securityGroup.put("vmId", vmId);
			securityGroup.put("securityGroupRuleId", securityGroupId);
			params.put("securityGroup", securityGroup);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "主机移出安全组失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "主机移出安全组失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "主机移出安全组成功");
		} catch (Exception e) {
			logger.error("主机移出安全组失败 ： " + e);
			return new BocloudResult(false, "主机移出安全组失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult snapshot(SnapshotModel snapshotModel) {
		return null;
	}

	@Override
	public BocloudResult vncConsole(String vncType, String serverId) {
		try {
			String apiUrl = "/apiproxy/v3/queryVncUrl";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", serverId);

			Result result = doGet(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "查询云主机列表失败");
			}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			if (null == returnObj) {
				return new BocloudResult(false, "获取控制台失败！");
			}
			JSONObject console = returnObj.getJSONObject("console");
			if (null == console) {
				return new BocloudResult(false, "获取控制台失败！");
			}
			return new BocloudResult(true, console.getString("url"), "获取控制台成功！");
		} catch (Exception e) {
			logger.error("主机密码重置失败 ： " + e);
			return new BocloudResult(false, "主机密码重置失败", e.getMessage());
		}
	}

	// 主机密码重置
	// POST http://ctyun-api-url/apiproxy/v3/resetVmPassword
	@Override
	public BocloudResult changePassword(String vmId, String password) {
		try {
			String apiUrl = "/apiproxy/v3/resetVmPassword";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			params.put("password", password);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "主机密码重置失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "主机密码重置失败：" + object.getString("message"));
			}
			ServerModel serverModel = (ServerModel) detail(vmId).getData();
			return new BocloudResult(true, serverModel, "主机密码重置成功");
		} catch (Exception e) {
			logger.error("主机密码重置失败 ： " + e);
			return new BocloudResult(false, "主机密码重置失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult attachNetwork(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult detachNetwork(String s, String s1) {
		return null;
	}

	// 主机规格变更（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/upgradeVM
	@Override
	public BocloudResult resize(String vmId, String flavorId) {
		try {
			String apiUrl = "/apiproxy/v3/ondemand/upgradeVM";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			params.put("flavorId", flavorId);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "主机规格变更失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "主机规格变更失败：" + object.getString("message"));
			}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
			Boolean status = returnObj.getBoolean("status");
			if (!status) {
				return new BocloudResult(false, "主机规格变更失败");
			}
			String serverId = (String) this.queryServerJob(this.getRegionId(), returnObj.getString("data")).getData();
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(2000);
				ServerModel model = (ServerModel) detail(serverId).getData();
				if (model != null && !"".equals(model)) {
					return new BocloudResult(true, model, "主机规格变更成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}

		} catch (Exception e) {
			logger.error("主机规格变更失败 ： " + e);
			return new BocloudResult(false, "主机规格变更失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult confirmResize(String s) {
		return null;
	}

	@Override
	public BocloudResult revertResize(String s) {
		return null;
	}

	@Override
	public BocloudResult hotConfigEnable(String s, Boolean aBoolean, Boolean aBoolean1) {
		return null;
	}

	@Override
	public BocloudResult getTicket(String s) {
		return null;
	}

	@Override
	public BocloudResult createInResPool(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult mountToolsInstaller(String s) {
		return null;
	}

	@Override
	public BocloudResult unmountToolsInstaller(String s) {
		return null;
	}

	@Override
	public BocloudResult upgradeToolsTask(String s, String s1) {
		return null;
	}

	@Override
	public BocloudResult getNetCard(String s) {
		return null;
	}

	@Override
	public BocloudResult markAsTemplate(String s) {
		return null;
	}

	// 主机密码重置
	// POST http://ctyun-api-url/apiproxy/v3/resetVmPassword
	@Override
	public BocloudResult changeVspVmPasswd(String vmId, String s1, String password) {
		try {
			String apiUrl = "/apiproxy/v3/resetVmPassword";
			JSONObject params = new JSONObject();
			params.put("regionId", this.getRegionId());
			params.put("vmId", vmId);
			params.put("password", password);

			Result result = doPost(apiUrl, params);
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "主机密码重置失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "主机密码重置失败：" + object.getString("message"));
			}
			return new BocloudResult(true, "主机密码重置成功");
		} catch (Exception e) {
			logger.error("主机密码重置失败 ： " + e);
			return new BocloudResult(false, "主机密码重置失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult diskExtend(String s, String s1, Integer integer) {
		return null;
	}

	@Override
	public BocloudResult getMetric(String s) {
		return null;
	}

	@Override
	public BocloudResult vmrcConsole(String s) {
		return null;
	}

	@Override
	public BocloudResult migrate(H3cMigrateModel h3cMigrateModel) {
		return null;
	}

	@Override
	public BocloudResult create(H3cServerModel h3cServerModel) {
		return null;
	}

	@Override
	public BocloudResult getJnlpXml(Long aLong, String s) {
		return null;
	}

	@Override
	public BocloudResult resetInternetMaxBandwidthOut(String s, Integer integer) {
		return null;
	}

	// 重装系统（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/reInstallSys
	// 切换系统（按需开通）
	// POST http://ctyun-api-url/apiproxy/v3/ondemand/switchSys
	@Override
	public BocloudResult resetInstance(ServerModel serverModel) {
		try {
			String reInstallApiUrl = "/apiproxy/v3/ondemand/reInstallSys";
			String switchApiUrl = "/apiproxy/v3/ondemand/switchSys";
			JSONObject params = new JSONObject();
			params.put("regionId", serverModel.getRegionId());
			params.put("vmId", serverModel.getInstanceId());
			params.put("adminPass", serverModel.getPassword());
			Result result = null;
			// 如果不传镜像，重装系统
			if (!serverModel.getImageUuid().isEmpty()) {
				params.put("imageRef", serverModel.getImageUuid());
				result = doPost(switchApiUrl, params);
			} else {
				// 如果传系统id，切换系统
				result = doPost(reInstallApiUrl, params);
			}
			if (result.isFailed()) {
				return new BocloudResult(false, result.getMessage());
			}
			JSONObject object = JSONObject.parseObject(result.getMessage());
			if (null == object) {
				return new BocloudResult(false, "重装/切换系统（按需开通）失败");
			}
			if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
				return new BocloudResult(false, "重装/切换系统（按需开通）失败：" + object.getString("message"));
			}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"), JSONObject.class);
			String serverId = (String) this.queryServerJob(serverModel.getRegionId(), returnObj.getString("data"))
					.getData();
			Long startTime = System.currentTimeMillis();
			while (true) {
				sleepSomeTime(2000);
				ServerModel model = (ServerModel) detail(serverId).getData();
				if (model != null && !"".equals(model)) {
					return new BocloudResult(true, model, "重装/切换系统（按需开通）成功");
				}
				Long endTime = System.currentTimeMillis();
				// 等待三分钟
				if (endTime - startTime > 3 * 60 * 1000) {
					return new BocloudResult(false, "查询资源详情超时！");
				}
			}
		} catch (Exception e) {
			logger.error("重装/切换系统（按需开通）失败 ： " + e);
			return new BocloudResult(false, "重装/切换系统（按需开通）失败", e.getMessage());
		}
	}

	@Override
	public BocloudResult bindKeypair(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult unbindKeypair(ServerModel serverModel) {
		return null;
	}

	@Override
	public BocloudResult resize(String s, String s1, Boolean aBoolean) {
		return null;
	}

	@Override
	public BocloudResult createImage(String s, String s1, List<String> list) {
		return null;
	}
}
