package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.SnapshotBackupModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.common.TianyiFlavorProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SnapshotBackupConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author Hu Wentao
 * @create 2020-10-13 16:10
 * @Description: 备份
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiSnapshotBackupProvider extends TianyiProvider implements SnapshotBackupProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiFlavorProvider.class);

    private BeanConvertor<SnapshotBackupModel, JSONObject> convertor = new SnapshotBackupConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiSnapshotBackupProvider(Butler butler) {
        super(butler);
    }

    //根据资源池及磁盘ID查询云磁盘备份详细列表
    //GET http://ctyun-api-url/apiproxy/v3/ondemand/queryVBSDetails
    @Override
    public BocloudResult list() {
        try {
        	String apiUrl = "/apiproxy/v3/ondemand/queryVBSDetails";
        	JSONObject headers = new JSONObject();
        	headers.put("regionId", this.getRegionId());

        	Result result = doGet(apiUrl, headers);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "查询备份失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		return new BocloudResult(false, "查询备份失败：" + object.getString("message"));
        	}
			List<JSONObject> jsonObjects = JSONArray.parseArray(JSONObject.parseObject(object.getString("returnObj")).getString("backups"), JSONObject.class);
			List<SnapshotBackupModel> snapshotBackupModels = convertor.convertList(jsonObjects);

			return new BocloudResult(true, snapshotBackupModels, "查询备份成功");
        } catch (Exception e) {
        	logger.error("查询备份失败 ： " + e);
        	return new BocloudResult(false, "查询备份失败", e.getMessage());
        }
    }

    //创建云硬盘备份 （按需开通）
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/createVBS
    @Override
    public BocloudResult create(SnapshotBackupModel snapshotBackupModel) {
        try {
        	String apiUrl = "/apiproxy/v3/ondemand/createVBS";
        	JSONObject params = new JSONObject();
        	params.put("regionId", this.getRegionId());
        	params.put("volumeId", snapshotBackupModel.getVolumeUuid());
        	params.put("name", snapshotBackupModel.getName());

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "创建云硬盘备份 （按需开通）失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		return new BocloudResult(false, "创建云硬盘备份 （按需开通）失败：" + object.getString("message"));
        	}
        	Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
        	if (!status) {
        		return new BocloudResult(false, "创建云硬盘备份 （按需开通）失败");
        	}
        	return new BocloudResult(true, "创建云硬盘备份 （按需开通）成功");
        } catch (Exception e) {
        	logger.error("创建云硬盘备份 （按需开通）失败 ： " + e);
        	return new BocloudResult(false, "创建云硬盘备份 （按需开通）失败", e.getMessage());
        }
    }

    //删除云硬盘备份
    //POST http://ctyun-api-url/apiproxy/v3/ondemand/deleteVBS
    @Override
    public BocloudResult remove(String vbsId) {
        try {
        	String apiUrl = "/apiproxy/v3/ondemand/deleteVBS";
        	JSONObject params = new JSONObject();
        	params.put("regionId", this.getRegionId());
        	params.put("vbsId", vbsId);

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "删除云硬盘备份失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		return new BocloudResult(false, "删除云硬盘备份失败：" + object.getString("message"));
        	}
        	Boolean status = JSONObject.parseObject(object.getString("returnObj")).getBoolean("status");
        	if (!status) {
        		return new BocloudResult(false, "删除云硬盘备份失败");
        	}
        	return new BocloudResult(true, "删除云硬盘备份成功");
        } catch (Exception e) {
        	logger.error("删除云硬盘备份失败 ： " + e);
        	return new BocloudResult(false, "删除云硬盘备份失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotBackupModel snapshotBackupModel) {
        return null;
    }

    @Override
    public BocloudResult remove(SnapshotBackupModel snapshotBackupModel) {
        return null;
    }

    @Override
    public BocloudResult restore(SnapshotBackupModel snapshotBackupModel) {
        return null;
    }
}
