package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.SnapshotModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.common.TianyiFlavorProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.ImageConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SnapshotConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.ListTool;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author Hu Wentao
 * @create 2020-10-09 11:36
 * @Description: //快照
 * //TODO 快照和备份不知道要哪个，暂时放着，后面补写
 * //TODO 测试：快照接口只有文档3、自营资源云里面的接口，测试发现用3里面的接口调不通
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiSnapshotProvider extends TianyiProvider implements SnapshotProvider {

    private static Logger logger = LoggerFactory.getLogger(TianyiFlavorProvider.class);

    private BeanConvertor<SnapshotModel, JSONObject> convertor = new SnapshotConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiSnapshotProvider(Butler butler) {
        super(butler);
    }

    //根据主机Id查询快照列表
    //url GET http://ctyun-api-url/apiproxy/v3/queryVMSnapshotsByVmId
    @Override
    public BocloudResult list(String vmId) {
        try {
        	String apiUrl = "/apiproxy/v3/queryVMSnapshotsByVmId";
        	JSONObject headers = new JSONObject();
        	headers.put("regionId", this.getRegionId());
        	headers.put("vmId", vmId);

        	Result result = doGet(apiUrl, headers);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "根据主机Id查询快照列表失败");
        	}
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return new BocloudResult(false, "查询镜像列表失败：" + object.getString("message"));
            }
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                return new BocloudResult(true, "快照列表为空");
            }
            List<SnapshotModel> snapshotModels = convertor.convertList(returnObj);
            return new BocloudResult(true, snapshotModels, "根据主机Id查询快照列表成功");
        } catch (Exception e) {
        	logger.error("根据主机Id查询快照列表失败 ： " + e);
        	return new BocloudResult(false, "根据主机Id查询快照列表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult list() {
        return null;
    }

    //删除快照
    //url POST http://ctyun-api-url/apiproxy/v3/removeVMSnapshot
    @Override
    public BocloudResult remove(SnapshotModel model) {
        try {
        	String apiUrl = "/apiproxy/v3/removeVMSnapshot";
        	JSONObject params = new JSONObject();
        	params.put("regionId", model.getRegionId());
        	params.put("vmSnapshotId", model.getSnapshotUuid());

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "删除快照失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		return new BocloudResult(false, "删除快照失败：" + object.getString("message"));
        	}
        	return new BocloudResult(true, "删除快照成功");
        } catch (Exception e) {
        	logger.error("删除快照失败 ： " + e);
        	return new BocloudResult(false, "删除快照失败", e.getMessage());
        }
    }

    //创建快照
    //url POST http://ctyun-api-url/apiproxy/v3/createVMSnapshot   /apiproxy/v3/ondemand/createVM
    @Override
    public BocloudResult create(SnapshotModel model) {
        try {
        	String apiUrl = "/apiproxy/v3/createVMSnapshot";
        	JSONObject params = new JSONObject();
        	params.put("regionId", model.getRegionId());
        	params.put("vmId", model.getVolumeUuid());
        	params.put("vmSnapshotName", model.getSnapName());

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		return new BocloudResult(false, "失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		return new BocloudResult(false, "失败：" + object.getString("message"));
        	}
        	return new BocloudResult(true, "成功");
        } catch (Exception e) {
        	logger.error("失败 ： " + e);
        	return new BocloudResult(false, "失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(SnapshotModel model) {
        return null;
    }

    @Override
    public BocloudResult revert(SnapshotModel model) {
        return null;
    }

    //根据快照Id查询快照详情
    //url GET http://ctyun-api-url/apiproxy/v3/queryVMSnapshotsByVmId
    @Override
    public BocloudResult detail(String regionId, String id) {
        try {
            String apiUrl = "/apiproxy/v3/queryVMSnapshotsByVmId";
            JSONObject headers = new JSONObject();
            headers.put("regionId", this.getRegionId());
            headers.put("id", id);

            Result result = doGet(apiUrl, headers);
            if (result.isFailed()) {
                return new BocloudResult(false, result.getMessage());
            }
            JSONObject object = JSONObject.parseObject(result.getMessage());
            if (null == object) {
                return new BocloudResult(false, "根据快照Id查询快照详情失败");
            }
            if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
                return new BocloudResult(false, "根据快照Id查询快照详情失败：" + object.getString("message"));
            }
            List<JSONObject> returnObj = JSONArray.parseArray(object.getString("returnObj"), JSONObject.class);
            if (ListTool.isEmpty(returnObj)) {
                return new BocloudResult(true, "该快照不存在");
            }
            List<SnapshotModel> snapshotModels = convertor.convertList(returnObj);
            return new BocloudResult(true, snapshotModels, "根据快照Id查询快照详情成功");
        } catch (Exception e) {
            logger.error("根据快照Id查询快照详情失败 ： " + e);
            return new BocloudResult(false, "根据快照Id查询快照详情失败", e.getMessage());
        }
    }
}
