package com.bocloud.cmp.provider.security;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.TianyiProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.network.TianyiSubnetProvider;
import com.bocloud.cmp.provider.tianyiconvertor.KeypairConvertor;
import com.bocloud.cmp.provider.tianyiconvertor.SubnetConvertor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Result;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author Hu Wentao
 * @create 2020-10-13 10:22
 * @Description: 密钥
 */
@Provider("bocloud.cmp.tianyi")
public class TianyiKeypairProvider extends TianyiProvider implements KeypairProvider {

    private Logger logger = LoggerFactory.getLogger(TianyiSubnetProvider.class);

    BeanConvertor<KeypairModel, JSONObject> convertor = new KeypairConvertor();

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public TianyiKeypairProvider(Butler butler) {
        super(butler);
    }

    //查询SSH密钥列表
    //GET http://ctyun-api-url/apiproxy/v3/querySSH
    @Override
    public BocloudResult list() {
        try {
        	String apiUrl = "/apiproxy/v3/querySSH";
        	JSONObject headers = new JSONObject();
        	headers.put("regionId", this.getRegionId());


        	Result result = doGet(apiUrl, headers);
        	if (result.isFailed()) {
        		logger.error("查询密钥列表失败：" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		logger.error("查询密钥列表失败");
        		return new BocloudResult(false, "查询SSH密钥列表失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("查询SSH密钥列表失败：" + object.getString("message"));
        		return new BocloudResult(false, "查询SSH密钥列表失败：" + object.getString("message"));
        	}
            List<JSONObject> jsonObjects = JSONArray.parseArray(JSONObject.parseObject(object.getString("returnObj")).getString("keypairs"), JSONObject.class);
            List<KeypairModel> keypairModels = convertor.convertList(jsonObjects);
            return new BocloudResult(true, keypairModels, "查询SSH密钥列表成功");
        } catch (Exception e) {
        	logger.error("查询SSH密钥列表失败 ： " + e);
        	return new BocloudResult(false, "查询SSH密钥列表失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult detail(String s) {
        return null;
    }

    //创建SSH密钥
    //POST http://ctyun-api-url/apiproxy/v3/createSSH
    @Override
    public BocloudResult create(String name, String publicKey) {
        try {
        	String apiUrl = "/apiproxy/v3/createSSH";
        	JSONObject params = new JSONObject();
        	params.put("regionId", this.getRegionId());
        	params.put("name", name);
        	if (publicKey != null && !"".equals(publicKey)) {
				params.put("publicKey", publicKey);
			}
        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		logger.error("创建密钥失败：" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		logger.error("创建密钥失败");
        		return new BocloudResult(false, "创建SSH密钥失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("创建SSH密钥失败：" + object.getString("message"));
        		return new BocloudResult(false, "创建SSH密钥失败：" + object.getString("message"));
        	}
			JSONObject returnObj = JSONObject.parseObject(object.getString("returnObj"));
        	if (null == returnObj || "".equals(returnObj)) {
				logger.error("创建密钥失败");
				return new BocloudResult(false, "创建SSH密钥失败");
			}
			JSONObject keyPair = JSONObject.parseObject(returnObj.getString("keypair"));
			if (null == keyPair || "".equals(keyPair)) {
				logger.error("创建密钥失败");
				return new BocloudResult(false, "创建SSH密钥失败");
			}
			KeypairModel keypairModel = convertor.convertModel(returnObj);
			keypairModel.setRegionId(this.getRegionId());
			return new BocloudResult(true, keypairModel, "创建SSH密钥成功");
        } catch (Exception e) {
        	logger.error("创建SSH密钥失败 ： " + e);
        	return new BocloudResult(false, "创建SSH密钥失败", e.getMessage());
        }
    }

    //删除SSH密钥
    //POST http://ctyun-api-url/apiproxy/v3/deleteSSH
    @Override
    public BocloudResult delete(String name) {
        try {
        	String apiUrl = "/apiproxy/v3/deleteSSH";
        	JSONObject params = new JSONObject();
        	params.put("regionId", this.getRegionId());
        	params.put("name", name);

        	Result result = doPost(apiUrl, params);
        	if (result.isFailed()) {
        		logger.error("删除密钥失败：" + result.getMessage());
        		return new BocloudResult(false, result.getMessage());
        	}
        	JSONObject object = JSONObject.parseObject(result.getMessage());
        	if (null == object) {
        		logger.error("删除密钥失败");
        		return new BocloudResult(false, "删除SSH密钥失败");
        	}
        	if (!object.getInteger("statusCode").equals(SUCCESS_STATUS_CODE)) {
        		logger.error("删除SSH密钥失败：" + object.getString("message"));
        		return new BocloudResult(false, "删除SSH密钥失败：" + object.getString("message"));
        	}
        	return new BocloudResult(true, "删除SSH密钥成功");
        } catch (Exception e) {
        	logger.error("删除SSH密钥失败 ： " + e);
        	return new BocloudResult(false, "删除SSH密钥失败", e.getMessage());
        }
    }

    @Override
    public BocloudResult modify(KeypairModel keypairModel) {
        return null;
    }

}
