package com.bocloud.cmp.provider.tianyiconvertor;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.DateTools;
import com.bocloud.common.utils.ListTool;
import java.util.ArrayList;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

/**
 * @author Hu Wentao
 * @create 2020-10-09 16:21
 * @Description: 云主机转换
 */
public class ServerConvertor implements BeanConvertor<ServerModel, JSONObject> {

	BeanConvertor<VolumeModel, JSONObject> volumeConvertor = new VolumeConvertor();

	@Override
	public List<ServerModel> convertList(List<? extends JSONObject> list) {
		List<ServerModel> serverModels = new ArrayList<>();
		for (JSONObject object : list) {
			serverModels.add(convertModel(object));
		}
		return serverModels;
	}

	@Override
	public ServerModel convertModel(JSONObject jsonObject) {
		try {
			ServerModel serverModel = new ServerModel();
			if (null != jsonObject) {
				serverModel.setInstanceId(jsonObject.getString("id"));
				serverModel.setName(jsonObject.getString("name"));

				JSONObject addressesObject = jsonObject.getJSONObject("addresses");
				List<String> privateIps = new ArrayList<String>();
				List<String> publicIps = new ArrayList<String>();
				Set<Entry<String, Object>> entrySet = addressesObject.entrySet();
				for (Entry<String, Object> entry : entrySet) {
					List<JSONObject> addressList = JSON.parseArray(JSON.toJSONString(entry.getValue()),
							JSONObject.class);
					if (!ListTool.isEmpty(addressList)) {
						for (JSONObject addr : addressList) {
							if ("fixed".equalsIgnoreCase(addr.getString("OS-EXT-IPS:type"))) {
								privateIps.add(addr.getString("addr"));
							}
							if ("floating".equalsIgnoreCase(addr.getString("OS-EXT-IPS:type"))) {
								publicIps.add(addr.getString("addr"));
							}
						}
					}
				}
				serverModel.setPrivateIps(JSON.toJSONString(privateIps));
				serverModel.setPublicIps(JSON.toJSONString(publicIps));

				// 云盘
				List<JSONObject> volumes = JSONArray.parseArray(
						jsonObject.getString("os-extended-volumes:volumes_attached"), JSONObject.class);
				if (!ListTool.isEmpty(volumes)) {
					List<VolumeModel> volumeModels = new ArrayList<>();
					VolumeModel volumeModel = null;
					for (JSONObject volume : volumes) {
						volumeModel = new VolumeModel();
						volumeModel.setVolumeUuid(volume.getString("id"));
						volumeModels.add(volumeModel);
					}
					serverModel.setDiskDevices(volumeModels);
				}
				String created = jsonObject.getString("created");
				if (created != null && !"".equals(created)) {
					serverModel.setGmtVmCreate(DateTools.transfer("yyyy-MM-dd'T'HH:mm:ssZ", created, "Z"));
				}
				String status = jsonObject.getString("status");
				if ("ACTIVE".equalsIgnoreCase(status)) {
					serverModel.setStatus("RUNNING");
				} else if ("SHUTOFF".equalsIgnoreCase(status)) {
					serverModel.setStatus("STOPPED");
				} else if ("ERROR".equalsIgnoreCase(status)) {
					serverModel.setStatus("EXCEPTION");
				} else {
					serverModel.setStatus(status);
				}

				serverModel.setFlavorUuid(JSONObject.parseObject(jsonObject.getString("flavor")).getString("id"));
				serverModel.setImageUuid(JSONObject.parseObject(jsonObject.getString("image")).getString("id"));

				List<String> securityGroups = new ArrayList<String>();
				List<JSONObject> securityObjects = JSONObject.parseArray(jsonObject.getString("security_groups"),
						JSONObject.class);
				if (!ListTool.isEmpty(securityObjects)) {
					for (JSONObject object : securityObjects) {
						securityGroups.add(object.getString("name"));
					}
				}
				serverModel.setSecurityGroups(securityGroups);
				serverModel.setZone(jsonObject.getString("OS-EXT-AZ:availability_zone"));

			}
			return serverModel;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
}
