package com.bocloud.cmp.provider.tianyiconvertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.common.utils.ListTool;
import lombok.SneakyThrows;

import java.util.ArrayList;
import java.util.List;


/**
 * @Package: com.bocloud.cmp.provider.tianyiconvertor
 * @ClassName: VolumeConvertor
 * @Author: 胡文涛
 * @CreateTime: 2020/9/23 10:57
 * @Description: 云硬盘转换
 */
public class VolumeConvertor implements BeanConvertor<VolumeModel, JSONObject> {

    @Override
    public List<VolumeModel> convertList(List<? extends JSONObject> list) {
        List<VolumeModel> volumeModels = new ArrayList<>();
        for (JSONObject object : list) {
            volumeModels.add(convertModel(object));
        }
        return volumeModels;
    }

    @SneakyThrows
    @Override
    public VolumeModel convertModel(JSONObject jsonObject) {
        VolumeModel model = new VolumeModel();
        if (null != jsonObject && !jsonObject.isEmpty()) {
            model.setVolumeUuid(jsonObject.getString("id"));
            model.setName(jsonObject.getString("name"));
            model.setCreateTime(jsonObject.getDate("created_at"));
            model.setUpdateTime(jsonObject.getDate("updated_at"));
            model.setMultiattach(jsonObject.getBoolean("multiattach"));
            model.setActionStatus(jsonObject.getString("replication_status"));
            model.setSize(jsonObject.getInteger("size"));
            model.setVolumeType(jsonObject.getString("volume_type"));
            model.setUsage(jsonObject.getString("user_id"));
            model.setBootVolume(jsonObject.getBoolean("bootable"));
            model.setZone(jsonObject.getString("availability_zone"));

            String status = jsonObject.getString("status");
            if (status == null || "".equals(status)) {
                model.setStatus("");
            } else if (status == "in-use" || "in-use".equals(status)) {
                model.setStatus("IN_USE");
            } else {
                model.setStatus(status.toUpperCase());
            }

            List<JSONObject> attachments = JSONObject.parseArray(jsonObject.getString("attachments"), JSONObject.class);
            if (!ListTool.isEmpty(attachments)) {
                model.setInstanceId(attachments.get(0).getString("server_id"));
                model.setDevice(attachments.get(0).getString("device"));
            }
            model.setRegionId(jsonObject.getString("regionId"));

        }
        return model;
    }
}
