package com.bocloud.cmp.provider;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.contentlibrary.ClsApiClient;
import com.bocloud.cmp.provider.contentlibrary.VapiConnector;
import com.bocloud.vmware.vim25.OvfOptionInfo;
import com.bocloud.vmware.vim25.mo.Folder;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import com.bocloud.vmware.vim25.mo.ServiceInstance;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

/**
 * vmware连接验证通用类
 *
 * @author weiwei
 */
public class VmwareProvider extends GenericProvider implements SuperProvider {
    public static final String VM = "VirtualMachine";
    public static final String DT = "Datastore";
    public static final String CR = "ComputeResource";
    public static final String DC = "Datacenter";
    public static final String HS = "HostSystem";
    public static final String RP = "ResourcePool";
    public static final String VAPP = "VirtualApp";

    private static Logger logger = LoggerFactory.getLogger(VmwareProvider.class);
    private String endpoint;
    private String username;
    private String password;
    private URL url;
    private ServiceInstance serviceInstance;
    private Folder rootFolder;
    private boolean accessable;

    /**
     * @return the rootFolder
     */
    public Folder getRootFolder() {
        return rootFolder;
    }

    /**
     * @return the accessable
     */
    public boolean isAccessable() {
        return accessable;
    }

    public String getEndpoint() {
        return endpoint;
    }

    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    /**
     * @return the serviceInstance
     */
    public ServiceInstance getServiceInstance() {
        return serviceInstance;
    }

    /**
     * @return the url
     */
    public URL getUrl() {
        return url;
    }

    public void init() {
        try {
            Assert.notNull(endpoint, "VCenter EndPoint is Empty!");
            if (endpoint.contains(":")) {
                String[] endpoints = endpoint.split(":");
                Assert.isTrue(Integer.valueOf(endpoints[1]) > 0, "VCenter EndPoint has Wrong Port");
                url = new URL("https", endpoints[0], Integer.valueOf(endpoints[1]), "/sdk");
            } else {
                url = new URL("https", endpoint, "/sdk");
            }
            serviceInstance = new ServiceInstance(url, username, password, true);
            serviceInstance.getSessionManager().setLocale("zh-CN");
            rootFolder = serviceInstance.getRootFolder();
            this.accessable = true;
        } catch (MalformedURLException | RemoteException e) {
            this.accessable = false;
            logger.error("Init Driver Error:", e);
        }

    }

    public VmwareProvider(Butler butler) {
        super(butler);
        this.endpoint = butler.getUrl();
        this.username = butler.getUsername();
        this.password = butler.getPassword();
        this.init();
    }

    /**
     * 云平台是否可以访问
     *
     * @return
     */
    @Override
    public boolean accessable() {
        return this.accessable;
    }

    /**
     * 获取内容库访问client
     *
     * @return
     */
    public ClsApiClient getClsApiClient() {
        return VapiConnector.getClsApiClient(this.endpoint, this.username, this.password);
    }
    
    public static void sleepSomeTime(int sleepTime) {
		try {
			Thread.sleep(sleepTime);
		} catch (InterruptedException e) {
			logger.error("睡眠失败: " + e.getMessage());
		}
	}

    public static void main(String[] args) throws Exception {
        Butler butler = new Butler();
        butler.setUrl("10.20.2.66");
        butler.setUsername("administrator@cmp.vsphere");
        butler.setPassword("BeyondCMP@v587");
        VmwareProvider provider = new VmwareProvider(butler);
        ManagedEntity dcEntity = new InventoryNavigator(provider.getRootFolder()).searchManagedEntity(DC, "北京研发中心");
        OvfOptionInfo[] ovfImportOption = provider.getServiceInstance().getOvfManager().getOvfImportOption();
        for (OvfOptionInfo ovfOptionInfo : ovfImportOption) {
            System.out.println(ovfOptionInfo.getOption());
        }

    }
}
