package com.bocloud.cmp.provider.compute;

import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FolderModel;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.vmware.vim25.mo.Datacenter;
import com.bocloud.vmware.vim25.mo.Folder;
import com.bocloud.vmware.vim25.mo.InventoryNavigator;
import com.bocloud.vmware.vim25.mo.ManagedEntity;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: VmwareFolderProvider
 * @Description: Vmware文件夹操作类
 * @Author lxy
 * @Date 2021/5/12 14:40
 */
@Provider("bocloud.cmp.vmware")
public class VmwareFolderProvider extends VmwareProvider implements FolderProvider {
    private static Logger logger = LoggerFactory.getLogger(VmwareFolderProvider.class);

    public VmwareFolderProvider(Butler butler) {
        super(butler);
    }


    @Override
    public BocloudResult list(String vdcName) {
        try {
            ManagedEntity center = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC, vdcName);
            Assert.notNull(center, "数据中心不存在！");
            Datacenter datacenter = (Datacenter) center;
            Folder vmFolder = datacenter.getVmFolder();

            List<FolderModel> modelList = new ArrayList<>();
            getVmFolder(modelList, vmFolder);
            return new BocloudResult(true, modelList, "获取云主机文件夹成功");
        } catch (Exception e) {
            logger.error("Get vm datacenter {} exception:{}", vdcName, e.getMessage());
            return new BocloudResult(false, "获取云主机文件夹失败：" + e.getMessage());
        }
    }

    private static void getVmFolder(List<FolderModel> models, Folder rootFolder)
            throws Exception {

        ManagedEntity[] entities = rootFolder.getChildEntity();
        if (null != entities && entities.length > 0) {
            for (ManagedEntity entity : entities) {
                FolderModel model = null;
                if (entity instanceof Folder) {
                    model = new FolderModel();
                    Folder childFolder = (Folder) entity;
                    //父文件夹
                    model.setRootFolder(childFolder.getName());
                    List<FolderModel> child = new ArrayList<>();

                    getVmFolder(child, childFolder);
                    model.setModelList(child);
                }
                if (model != null) {
                    models.add(model);
                }
            }
        }
    }

}
