package com.bocloud.cmp.provider.network;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.entity.enums.NetworkType;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.VmwareProvider;
import com.bocloud.cmp.provider.utils.AssembleTools;
import com.bocloud.cmp.provider.utils.SingleAssembleTools;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.vmware.vim25.*;
import com.bocloud.vmware.vim25.mo.HostSystem;
import com.bocloud.vmware.vim25.mo.*;
import io.jsonwebtoken.lang.Assert;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * vmware网络操作类
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/11/14
 */
@Provider("bocloud.cmp.vmware")
public class VmwareNetworkProvider extends VmwareProvider implements NetworkProvider {

    private static Logger logger = LoggerFactory.getLogger(VmwareNetworkProvider.class);

    public VmwareNetworkProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult listByVdc(String vdcName) {
        logger.info("查询数据中心{}下网络", vdcName);
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    vdcName);
            Assert.notNull(managedEntity, "数据中心" + vdcName + "不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            List<NetworkModel> networks = new ArrayList<>();
            List<SwitchModel> switchers = new ArrayList<>();
            Map<String, Object> networkMap = SingleAssembleTools.getInstance().assembleNetwork(datacenter);
            if (null != networkMap) {
                networks = (List<NetworkModel>) networkMap.get("network");
                switchers = (List<SwitchModel>) networkMap.get("switcher");
            }
            VDCModel vdc = new VDCModel();
            vdc.setName(vdcName);
            vdc.setVdcVal(datacenter.getMOR().getVal());
            vdc.setNetworks(networks);
            vdc.setSwitchers(switchers);
            return new BocloudResult(true, vdc, "数据中心" + vdcName + "下网络查询成功");
        } catch (Exception e) {
            logger.error("数据中心" + vdcName + "下网络查询失败:" + e);
            return new BocloudResult(false, "数据中心" + vdcName + "下网络查询失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult list() {
        try {
            ManagedEntity[] hostSystemArray = new InventoryNavigator(this.getRootFolder()).searchManagedEntities(HS);
            List<NetworkModel> networkModels = new ArrayList<>();
            List<SubnetModel> subnetModels = new ArrayList<>();
            SubnetModel switchSubnetModel = new SubnetModel();
            for (ManagedEntity hostSystemManagedEntity : hostSystemArray) {
                HostSystem hostSystem = (HostSystem) hostSystemManagedEntity;
                Assert.notNull(hostSystem, "host system is null");
                SubnetModel nicSubnetModel = new SubnetModel();
                HostNetworkInfo nwi = hostSystem.getConfig().getNetwork();
                HostIpRouteConfig hostIpRouteConfig = nwi.getIpRouteConfig();
                if (hostIpRouteConfig != null) {
                    switchSubnetModel.setGateway(hostIpRouteConfig.getDefaultGateway());
                    nicSubnetModel.setGateway(hostIpRouteConfig.getDefaultGateway());
                }

                HostPortGroup[] portgroups = nwi.getPortgroup();
                Map<String, String> pgMap = new HashMap<String, String>();
                for (HostPortGroup portgroup : portgroups) {
                    pgMap.put(portgroup.getKey(), portgroup.getSpec().getName());
                }
                PhysicalNic[] pnics = nwi.getPnic();
                Map<String, String> pnicMap = new HashMap<>();
                for (PhysicalNic pnic : pnics) {
                    pnicMap.put(pnic.getKey(), pnic.getMac());
                }

                HostVirtualSwitch[] vswtichs = nwi.getVswitch();
                for (HostVirtualSwitch vswitch : vswtichs) {
                    NetworkModel switchModel = new NetworkModel();
                    switchModel.setType("VirtualSwitch");
                    switchModel.setName(vswitch.getName());
                    String[] macKeys = vswitch.getPnic();
                    String macValue = "";
                    if (macKeys != null) {
                        for (String key : macKeys) {
                            macValue = pnicMap.get(key) + ";" + macValue;
                        }
                        switchSubnetModel.setMac(macValue);
                    }
                    String[] pgs = vswitch.getPortgroup();
                    String pgValue = "";
                    if (pgs != null) {
                        for (String pg : pgs) {
                            pgValue = (pgMap.get(pg)) + ";" + pgValue;
                        }
                        switchSubnetModel.setPortgroup(pgValue);
                    }
                    subnetModels.add(switchSubnetModel);
                    switchModel.setSubnets(subnetModels);
                    subnetModels.add(switchSubnetModel);
                }

                HostVirtualNic[] virtualnics = nwi.getVnic();
                for (HostVirtualNic virtualnic : virtualnics) {
                    NetworkModel nicModel = new NetworkModel();
                    nicModel.setType("VirtualNic");
                    nicModel.setName(virtualnic.getDevice());
                    nicSubnetModel.setPortgroup(virtualnic.getPortgroup());
                    nicSubnetModel.setMac(virtualnic.getSpec().getMac());
                    nicSubnetModel.setAddress(virtualnic.getSpec().getIp().getIpAddress());
                    nicSubnetModel.setHostRoutes(virtualnic.getSpec().getIp().getSubnetMask());
                    if (virtualnic.getSpec().getIp().getIpV6Config().getDhcpV6Enabled()) {
                        nicSubnetModel.setIpVersion("IPv6");
                    } else {
                        nicSubnetModel.setIpVersion("IPv4");
                    }
                    networkModels.add(nicModel);
                }
            }
            return new BocloudResult(true, networkModels, "获取网络信息成功!");
        } catch (Exception e) {
            logger.error("get network failure:", e);
            return new BocloudResult(false, "获取网络信息失败!");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }

    }

    @Override
    public BocloudResult createDSwitch(SwitchModel switchModel) {
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    switchModel.getVdcName());
            Assert.notNull(managedEntity, "数据中心不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            // 创建分布式交换机使用folder且必须是networkFolder
            Folder folder = datacenter.getNetworkFolder();
            // 获取已经存在的网络名称
            ManagedEntity[] managedEntities = folder.getChildEntity();
            managedEntities = Optional.ofNullable(managedEntities).orElse(new ManagedEntity[0]);
            boolean nameAbsent = Arrays.stream(managedEntities)
                    .noneMatch(entity -> switchModel.getName().equalsIgnoreCase(entity.getName()));
            Assert.isTrue(nameAbsent, "创建失败：名称不能重复");
            // 获取所有的物理机器以及物理网卡信息
            Map<String, List<String>> hostMap = switchModel.getValues();
            // 存放物理机的名称及key值
            Map<String, String> hostKeyMap = new HashMap<>();
            // 获取hostName对应的物理机的key值
            for (String hostName : hostMap.keySet()) {
                ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
                if (null != entity) {
                    HostSystem hostSystem = (HostSystem) entity;
                    hostKeyMap.put(hostName, hostSystem.getMOR().getVal());
                } else {
                    return new BocloudResult(false, "创建分布式交换机失败：获取物理主机value值失败");
                }
            }
            // 物理机数组
            DistributedVirtualSwitchHostMemberConfigSpec[] dvshmcsArray = new DistributedVirtualSwitchHostMemberConfigSpec[hostMap
                    .size()];
            int dvshmcsIndex = 0;
            for (String hostName : hostMap.keySet()) {
                List<String> pNics = hostMap.get(hostName);
                // hostName主机对应的物理网卡
                if (!ListTool.isEmpty(pNics)) {
                    DistributedVirtualSwitchHostMemberPnicSpec[] pnicSpecs = new DistributedVirtualSwitchHostMemberPnicSpec[pNics
                            .size()];
                    int specIndex = 0;
                    for (String pNic : pNics) {
                        DistributedVirtualSwitchHostMemberPnicSpec pnicSpc = new DistributedVirtualSwitchHostMemberPnicSpec();
                        pnicSpc.setPnicDevice(pNic);
                        pnicSpecs[specIndex++] = pnicSpc;
                    }

                    // 某一个物理机所包含的物理网卡的信息
                    DistributedVirtualSwitchHostMemberPnicBacking dvshmpb = new DistributedVirtualSwitchHostMemberPnicBacking();
                    dvshmpb.setPnicSpec(pnicSpecs);
                    // 要创建的分布式交换机所需的物理机相关配置
                    DistributedVirtualSwitchHostMemberConfigSpec dvshmcs = new DistributedVirtualSwitchHostMemberConfigSpec();
                    // 设置物理网卡
                    dvshmcs.setBacking(dvshmpb);
                    // 某一台特定物理机
                    ManagedObjectReference reference = new ManagedObjectReference();
                    reference.setType(HS);// 类型
                    reference.setVal(hostKeyMap.get(hostName));// 物理机的唯一value值
                    HostSystem hostSystem = new HostSystem(this.getRootFolder().getServerConnection(), reference);
                    // 设置物理机
                    dvshmcs.setHost(hostSystem.getMOR());
                    dvshmcs.setOperation("add");

                    dvshmcsArray[dvshmcsIndex++] = dvshmcs;
                }
            }
            // 设置物理机
            DVSConfigSpec dvsConfigSpec = new DVSConfigSpec();
            dvsConfigSpec.setHost(dvshmcsArray);
            dvsConfigSpec.setName(switchModel.getName());
            // 分布式云主机配置
            DVSCreateSpec dvs = new DVSCreateSpec();
            dvs.setConfigSpec(dvsConfigSpec);
            // 创建分布式云主机任务
            Task dvsTask = folder.createDVS_Task(dvs);
            if (dvsTask.waitForTask().equals(Task.SUCCESS)) {
                // 初始化所创建的分布式云主机
                DistributedVirtualSwitch dvSwitch = new DistributedVirtualSwitch(folder.getServerConnection(),
                        (ManagedObjectReference) dvsTask.getTaskInfo().getResult());
                // 分布式端口组配置
                DVPortgroupConfigSpec dvP = new DVPortgroupConfigSpec();
                dvP.setType("earlyBinding");// 分布式端口组的类型
                dvP.setName(switchModel.getName() + "PortGroup");// 分布式端口组的名称
                dvP.setNumPorts(switchModel.getPortGroupNum());// 分布式端口组端口数量
                // 分布式端口组创建任务
                Task groupTask = dvSwitch.createDVPortgroup_Task(dvP);
                if (groupTask.waitForTask().equals(Task.SUCCESS)) {
                    // return new BocloudResult(true, networks, "创建分布式交换机成功");
                    ManagedEntity entity = new InventoryNavigator(this.getRootFolder())
                            .searchManagedEntity("VmwareDistributedVirtualSwitch", switchModel.getName());
                    if (null == entity) {
                        return new BocloudResult(true, "创建分布式交换机失败");
                    }
                    VmwareDistributedVirtualSwitch vSwitch = (VmwareDistributedVirtualSwitch) entity;
                    List<PortGroupModel> groups = new ArrayList<>();
                    Map<String, String> portMap = new HashMap<>();
                    ManagedObjectReference[] references = vSwitch.getConfig().getUplinkPortgroup();
                    if (null != references) {
                        for (ManagedObjectReference reference : references) {
                            DistributedVirtualPortgroup uplinkPort = new DistributedVirtualPortgroup(
                                    entity.getServerConnection(), reference);
                            PortGroupModel group = new PortGroupModel(uplinkPort.getName(),
                                    NetworkType.PortGroupType.UPLINK.name(), uplinkPort.getConfig().getNumPorts(),
                                    uplinkPort.getMOR().getVal(), uplinkPort.getConfig().getDescription());
                            portMap.put(uplinkPort.getMOR().getVal(), uplinkPort.getName());
                            groups.add(group);
                        }
                    }
                    DistributedVirtualPortgroup[] pgs = vSwitch.getPortgroup();
                    Arrays.stream(pgs).filter(pg -> !portMap.containsKey(pg.getMOR().getVal())).forEach(pg -> {
                        PortGroupModel group = new PortGroupModel(pg.getName(), NetworkType.PortGroupType.DVPORT.name(),
                                pg.getConfig().getNumPorts(), pg.getMOR().getVal(), pg.getConfig().getDescription());
                        groups.add(group);
                    });
                    int portNum = vSwitch.getConfig().getNumPorts();
                    String val = vSwitch.getMOR().getVal();
                    String status = vSwitch.getOverallStatus().toString();
                    SwitcherModel switcher = new SwitcherModel(vSwitch.getName(), status,
                            NetworkType.SwitchType.DVSWITCH.name(), portNum, val, groups);
                    NetworkModel network = new NetworkModel(vSwitch.getName(), NetworkType.SwitchType.DVSWITCH.name(),
                            status, portNum, val);
                    switcher.setNetwork(network);
                    return new BocloudResult(true, switcher, "创建分布式交换机成功");
                }
            } else {
                return new BocloudResult(false, "创建分布式交换机失败：" + dvsTask.getTaskInfo().getError().getLocalizedMessage());
            }
        } catch (Exception e) {
            logger.error("", e);
            return new BocloudResult(false, "创建分布式交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
        return new BocloudResult(false, "创建分布式交换机失败");
    }

    @Override
    public BocloudResult createVSwitch(SwitchModel switchModel) {
        try {
            // 获取所有的物理机器以及物理网卡信息(此时该map中只有一条记录，即一台物理机)
            Map<String, List<String>> hostMap = switchModel.getValues();
            String hostName = hostMap.keySet().iterator().next();
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
            HostSystem host = (HostSystem) entity;
            // 获取主机当前连接状态
            HostSystemConnectionState connState = host.getSummary().getRuntime().getConnectionState();
            if (connState.equals(HostSystemConnectionState.disconnected)) {
                return new BocloudResult(false, "主机处于断开状态，无法创建网络");
            }
            // 如果该机器上已存在相应的标准交换机名称和云主机端口组名称，标准交换机创建不成功
            // 获取该物理机上的所有云主机端口组
            Network[] networks = host.getNetworks();
            // 判断该机器上是否已存在该端口组名称
            boolean pgNameAbsent = Arrays.asList(networks).stream()
                    .noneMatch(network -> network.getName().equals(switchModel.getPortGroupName()));
            Assert.isTrue(pgNameAbsent, "创建失败：该机器上已存在该端口组名称!");
            // 获取机器上的标准交换机
            HostVirtualSwitch[] vSwitchs = host.getHostNetworkSystem().getNetworkInfo().getVswitch();
            // 判断该机器上是否已存在该交换机名称
            boolean switchAbsent = Arrays.stream(vSwitchs).noneMatch(vs -> vs.getName().equals(switchModel.getName()));
            Assert.isTrue(switchAbsent, "创建失败:该机器上已存在该交换机名称！");
            List<String> pNicNames = hostMap.get(hostName);// 该物理机对应的物理网卡
            HostVirtualSwitchSpec vSwitchSpec = new HostVirtualSwitchSpec();
            if (pNicNames.size() != 0) {
                // 使用BondBridge方式创建
                HostVirtualSwitchBondBridge bondBridge = new HostVirtualSwitchBondBridge();
                // 设置该物理机下物理网卡的名称
                bondBridge.setNicDevice(pNicNames.toArray(new String[pNicNames.size()]));
                vSwitchSpec.setBridge(bondBridge);
            }
            // 设置端口数量
            vSwitchSpec.setNumPorts(switchModel.getPortGroupNum() + 8);
            // 物理机上创建标准交换机
            host.getHostNetworkSystem().addVirtualSwitch(switchModel.getName(), vSwitchSpec);
            // 添加云主机端口组
            HostPortGroupSpec portGroupSpec = new HostPortGroupSpec();
            portGroupSpec.setVswitchName(switchModel.getName());
            portGroupSpec.setName(switchModel.getPortGroupName());
            portGroupSpec.setVlanId(switchModel.getVlanId());
            portGroupSpec.setPolicy(new HostNetworkPolicy());
            // 添加端口组
            host.getHostNetworkSystem().addPortGroup(portGroupSpec);
            entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
            host = (HostSystem) entity;
            vSwitchs = host.getHostNetworkSystem().getNetworkInfo().getVswitch();
            SwitcherModel switcher = null;
            for (HostVirtualSwitch vSwitch : vSwitchs) {
                if (!vSwitch.getName().equals(switchModel.getName())) {
                    continue;
                }
                Map<String, String> netNames = new HashMap<>();
                NetworkModel networkModel = null;
                networks = host.getNetworks();
                for (Network network : networks) {
                    if (network.getName().equals(switchModel.getPortGroupName())) {
                        networkModel = new NetworkModel(network.getName(), NetworkType.SwitchType.VSWITCH.name(),
                                network.getOverallStatus().name(), null, network.getMOR().getVal());
                    }
                    netNames.put(network.getName(), network.getName());
                }
                // 有可能会出现第一次拿不到网络的情况，此处容错做轮训处理
                if (null == networkModel) {
                    logger.info("未获取到网络数据，开始轮询");
                    networkModel = this.getNetwork(host, switchModel.getPortGroupName());
                    if (null != networkModel) {
                        netNames.put(networkModel.getName(), networkModel.getName());
                    }
                    logger.info("获取网络数据结束");
                }
                // 获取主机上的所有端口
                HostPortGroup[] portGroups = host.getHostNetworkSystem().getNetworkInfo().getPortgroup();
                portGroups = Optional.ofNullable(portGroups).orElse(new HostPortGroup[0]);
                List<PortGroupModel> groups = new ArrayList<>();
                Arrays.stream(portGroups).filter(pg -> pg.getSpec().getVswitchName().equals(vSwitch.getName()))
                        .forEach(pg -> {
                            String type = netNames.containsKey(pg.getSpec().getName())
                                    ? NetworkType.PortGroupType.VIRTUAL.name()
                                    : NetworkType.PortGroupType.VMKERNEL.name();
                            PortGroupModel group = new PortGroupModel(pg.getSpec().getName(), type,
                                    pg.getSpec().getVlanId());
                            groups.add(group);
                        });
                switcher = new SwitcherModel(vSwitch.getName(), null, NetworkType.SwitchType.VSWITCH.name(),
                        vSwitch.getSpec().getNumPorts() - 8, vSwitch.getKey(), groups);
                networkModel.setPortNum(vSwitch.getSpec().getNumPorts() - 8);
                switcher.setNetwork(networkModel);
                break;
            }
            return new BocloudResult(true, switcher, "创建标准交换机成功");
        } catch (Exception e) {
            logger.error("创建标准交换机异常", e);
            return new BocloudResult(false, "创建标准交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    /**
     * 查询网络
     *
     * @param host
     * @param networkName
     * @return
     * @throws Exception
     */
    private NetworkModel getNetwork(HostSystem host, String networkName) throws Exception {
        NetworkModel networkModel = null;
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();
        while (flag) {
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                logger.error("Get error message:", e);
                Thread.currentThread().interrupt();
            }
            Network[] networks = host.getNetworks();
            Network findNetwork = Arrays.asList(networks).stream().filter(n -> n.getName().equals(networkName))
                    .findFirst().orElse(null);
            if (null != findNetwork) {
                flag = false;
                networkModel = new NetworkModel(findNetwork.getName(), NetworkType.SwitchType.VSWITCH.name(),
                        findNetwork.getOverallStatus().name(), null, findNetwork.getMOR().getVal());
                break;
            }
            Long endTime = System.currentTimeMillis();
            // 等待3分钟
            if (endTime - startTime > 3 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        return networkModel;
    }

    @Override
    public BocloudResult removeDSwitch(String name) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder())
                    .searchManagedEntity("VmwareDistributedVirtualSwitch", name);
            Assert.notNull(entity, "分布式交换机不存在！！");
            VmwareDistributedVirtualSwitch virtualSwitch = (VmwareDistributedVirtualSwitch) entity;
            Task task = virtualSwitch.destroy_Task();
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "删除分布式交换机成功");
            }
            return new BocloudResult(false, "删除分布式交换机失败");
        } catch (Exception e) {
            logger.error("Remove distribute virtual switch error:", e);
            return new BocloudResult(false, "删除分布式交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult renameDSwitch(String oldName, String newName) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder())
                    .searchManagedEntity("VmwareDistributedVirtualSwitch", oldName);
            Assert.notNull(entity, "分布式交换机不存在！！");
            VmwareDistributedVirtualSwitch virtualSwitch = (VmwareDistributedVirtualSwitch) entity;
            Task task = virtualSwitch.rename_Task(newName);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, "修改分布式交换机成功");
            }
            return new BocloudResult(false, "修改分布式交换机失败");
        } catch (Exception e) {
            logger.error("Rename distribute virtual switch error:", e);
            return new BocloudResult(false, "修改分布式交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult updateDVSwitch(SwitchModel model) {
        try {
            Datacenter center = (Datacenter) new InventoryNavigator(this.getRootFolder())
                    .searchManagedEntity("Datacenter", model.getVdcName());
            Assert.notNull(center, "数据中心不存在！！");
            Datacenter dc = center;
            ManagedEntity entity = new InventoryNavigator(dc).searchManagedEntity("VmwareDistributedVirtualSwitch",
                    model.getOldName());
            Assert.notNull(entity, "分布式交换机不存在！！");
            VmwareDistributedVirtualSwitch virtualSwitch = (VmwareDistributedVirtualSwitch) entity;
            VMwareDVSConfigSpec spec = new VMwareDVSConfigSpec();
            spec.setConfigVersion(virtualSwitch.getConfig().getConfigVersion());
            // 常规
            spec.setName(model.getName());
            spec.setDescription(model.getRemark());
            List<String> dvportNames = model.getDvportNames();
            if (!ListTool.isEmpty(dvportNames)) {
                String[] dvports = dvportNames.toArray(new String[dvportNames.size()]);
                // 上行链路端口
                DVSNameArrayUplinkPortPolicy uplinkPortPolicy = new DVSNameArrayUplinkPortPolicy();
                uplinkPortPolicy.setUplinkPortName(dvports);
                // 上行链路端口数
                model.setDvportNum(dvportNames.size());
                spec.setUplinkPortPolicy(uplinkPortPolicy);
            }
            // 高级
            DVSContactInfo contact = new DVSContactInfo();
            // 管理员联系信息
            contact.setName(model.getManagerName());
            contact.setContact(model.getManagerContact());
            spec.setContact(contact);
            spec.setMaxMtu(model.getMtu());
            if (!StringUtils.isEmpty(model.getProtocol())
                    && !LinkDiscoveryProtocolConfigOperationType.none.name().equals(model.getProtocol())) {
                // 发现协议
                LinkDiscoveryProtocolConfig linkDiscoveryProtocolConfig = new LinkDiscoveryProtocolConfig();
                // 发现协议类型
                linkDiscoveryProtocolConfig.setProtocol(model.getProtocol());
                // 发现协议操作
                linkDiscoveryProtocolConfig.setOperation(model.getOperation());
                spec.setLinkDiscoveryProtocolConfig(linkDiscoveryProtocolConfig);
            }
            // 专用VLAN
            List<VMwareDVSPvlanConfigSpec> pvlanSpecs = new ArrayList<>();
            String pVlans = model.getPVlans();
            if (StringUtils.isNotBlank(pVlans)) {
                List<PVlanModel> pVlanModels = JSONArray.parseArray(pVlans, PVlanModel.class);
                for (PVlanModel pVlanModel : pVlanModels) {
                    VMwareDVSPvlanConfigSpec pvlanSpec = new VMwareDVSPvlanConfigSpec();
                    pvlanSpec.setOperation(ConfigSpecOperation.edit.name());
                    VMwareDVSPvlanMapEntry pvlanEntry = new VMwareDVSPvlanMapEntry();
                    pvlanEntry.setPrimaryVlanId(pVlanModel.getPrimaryVlanId());
                    pvlanEntry.setSecondaryVlanId(pVlanModel.getSecondaryVlanId());
                    pvlanEntry.setPvlanType(pVlanModel.getPvlanType());
                    pvlanSpec.setPvlanEntry(pvlanEntry);
                    pvlanSpecs.add(pvlanSpec);
                }
            }
            if (!ListTool.isEmpty(pvlanSpecs)) {
                VMwareDVSPvlanConfigSpec[] pvlanConfigSpec = pvlanSpecs.toArray(new VMwareDVSPvlanConfigSpec[0]);
                spec.setPvlanConfigSpec(pvlanConfigSpec);
            }
            // NetFlow
            VMwareIpfixConfig ipfixConfig = new VMwareIpfixConfig();
            // 收集器IP
            ipfixConfig.setCollectorIpAddress(model.getCollectorIpAddress());
            // 收集器端口
            ipfixConfig.setCollectorPort(model.getCollectorPort());
            // 活动流导出超时
            ipfixConfig.setActiveFlowTimeout(model.getActiveFlowTimeout());
            // 闲置流导出超时
            ipfixConfig.setIdleFlowTimeout(model.getIdleFlowTimeout());
            // 采样率
            ipfixConfig.setSamplingRate(model.getSamplingRate());
            // 仅处理内部流
            ipfixConfig.setInternalFlowsOnly(model.getInternalFlowsOnly());
            // VDS IP地址
            spec.setSwitchIpAddress(model.getVdsIpAddress());
            spec.setIpfixConfig(ipfixConfig);
            Task task = virtualSwitch.reconfigureDvs_Task(spec);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                VmwareDistributedVirtualSwitch dvSwitch = (VmwareDistributedVirtualSwitch) new InventoryNavigator(dc)
                        .searchManagedEntity("VmwareDistributedVirtualSwitch", model.getName());
                return new BocloudResult(true, AssembleTools.getInstance().assembleDvSwitch(model, dvSwitch),
                        "分布式交换机更新成功");
            }
            return new BocloudResult(false, "分布式交换机更新失败：" + task.getTaskInfo().getError().getLocalizedMessage());
        } catch (Exception e) {
            logger.error("Update distribute virtual switch error:", e);
            return new BocloudResult(false, "分布式交换机更新失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult removeVSwitch(String hostName, String vswitchName, List<String> pgNames) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
            Assert.notNull(entity, "宿主机不存在！！");
            HostSystem host = (HostSystem) entity;
            HostVirtualSwitch[] vSwitches = host.getHostNetworkSystem().getNetworkInfo().getVswitch();
            Assert.notEmpty(vSwitches, "删除失败，该宿主机下的标准交换机不存在!");
            HostVirtualSwitch virtualSwitch = Arrays.stream(vSwitches)
                    .filter(vSwitch -> vSwitch.getName().equals(vswitchName)).findFirst().orElse(null);
            Assert.notNull(virtualSwitch, "删除失败，标准交换机不存在");
            logger.info("Start to remove port group...");
            // 删除端口组
            for (String pgName : pgNames) {
                host.getHostNetworkSystem().removePortGroup(pgName);
            }
            logger.info("End to remove port group...");
            // 删除标准交换机
            logger.info("Start to remove virtual switch...");
            host.getHostNetworkSystem().removeVirtualSwitch(vswitchName);
            logger.info("End to remove virtual switch...");
            return new BocloudResult(true, "删除标准交换机成功");
        } catch (Exception e) {
            return new BocloudResult(false, "删除标准交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult updateVSwitch(String hostName, String vswitchName, SwitchModel model) {
        try {
            ManagedEntity entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
            Assert.notNull(entity, "宿主机不存在！");
            HostSystem host = (HostSystem) entity;
            // 网络故障切换检测
            HostNicFailureCriteria failureCriteria = new HostNicFailureCriteria();
            failureCriteria.setCheckBeacon(model.getCheckBeacon());
            failureCriteria.setCheckDuplex(Boolean.FALSE);
            failureCriteria.setCheckErrorPercent(Boolean.FALSE);
            failureCriteria.setCheckSpeed("minimum");
            failureCriteria.setFullDuplex(Boolean.FALSE);
            failureCriteria.setSpeed(1);
            failureCriteria.setPercentage(100);
            HostNicOrderPolicy nicOrder = new HostNicOrderPolicy();
            // 活动适配器
            List<String> activeNics = model.getActiveNics();
            if (!ListTool.isEmpty(activeNics)) {
                nicOrder.setActiveNic(activeNics.toArray(new String[activeNics.size()]));
            }
            // 备用适配器
            List<String> standbyNics = model.getStandbyNics();
            if (!ListTool.isEmpty(standbyNics)) {
                nicOrder.setActiveNic(standbyNics.toArray(new String[standbyNics.size()]));
            }
            // 安全设置
            HostNetworkSecurityPolicy security = new HostNetworkSecurityPolicy();
            // 混杂模式
            security.setAllowPromiscuous(model.getAllowPromiscuous());
            // 伪传输
            security.setForgedTransmits(model.getForgedTransmits());
            // MAC地址更改
            security.setMacChanges(model.getMacChanges());
            // 流量调整
            HostNetworkTrafficShapingPolicy shapingPolicy = new HostNetworkTrafficShapingPolicy();
            // 状态
            shapingPolicy.setEnabled(model.getTrafficEnabled());
            if (model.getTrafficEnabled()) {
                // 平均带宽
                shapingPolicy.setAverageBandwidth(model.getAverageBandWidth());
                // 带宽峰值
                shapingPolicy.setPeakBandwidth(model.getPeakBandWidth());
                // 突发大小
                shapingPolicy.setBurstSize(model.getBurstSize());
            }
            // 网卡绑定
            HostNicTeamingPolicy nicTeaming = new HostNicTeamingPolicy();
            nicTeaming.setFailureCriteria(failureCriteria);
            nicTeaming.setNicOrder(nicOrder);
            // 通知交换机
            nicTeaming.setNotifySwitches(model.getNotifySwitches());
            // 负载平衡
            nicTeaming.setPolicy(model.getPolicy());
            nicTeaming.setReversePolicy(Boolean.TRUE);
            // 故障恢复
            nicTeaming.setRollingOrder(model.getRollingOrder());
            HostNetworkPolicy policy = new HostNetworkPolicy();
            policy.setNicTeaming(nicTeaming);
            policy.setSecurity(security);
            policy.setShapingPolicy(shapingPolicy);
            HostVirtualSwitchSpec spec = new HostVirtualSwitchSpec();
            spec.setPolicy(policy);
            // HostVirtualSwitchBeaconConfig beacon = new HostVirtualSwitchBeaconConfig();
            // beacon.setInterval(10);
            if (null != activeNics && activeNics.size() > 0) {
                HostVirtualSwitchBondBridge bridge = new HostVirtualSwitchBondBridge();
                bridge.setNicDevice(activeNics.toArray(new String[activeNics.size()]));
                spec.setBridge(bridge);
            }
            // bridge.setBeacon(beacon);
            spec.setPolicy(policy);
            // 常规设置
            spec.setNumPorts(model.getPortGroupNum() + 8);
            // mtu
            if (null != model.getMtu() && 0 != model.getMtu()) {
                spec.setMtu(model.getMtu());
            } else {
                spec.setMtu(1280);
            }
            host.getHostNetworkSystem().updateVirtualSwitch(vswitchName, spec);
            return new BocloudResult(true, model, "更新标准交换机成功");
        } catch (Exception e) {
            return new BocloudResult(false, "更新标准交换机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult createPortGroup(String hostName, String vswitchName, PortGroupModel pgModel) {
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    pgModel.getVdcName());
            Assert.notNull(managedEntity, "数据中心不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            HostPortGroupSpec portgrp = new HostPortGroupSpec();
            portgrp.setName(pgModel.getName());
            portgrp.setVswitchName(vswitchName);
            HostNetworkPolicy policy = null;
            BocloudResult BocloudResult = null;
            HostSystem host = null;
            if (!StringUtils.isEmpty(hostName)) {
                ManagedEntity entity = new InventoryNavigator(datacenter).searchManagedEntity(HS, hostName);
                Assert.notNull(entity, "宿主机不存在！");
                host = (HostSystem) entity;
            }
            switch (NetworkType.PortGroupType.valueOf(pgModel.getType())) {
                case VIRTUAL:
                    portgrp.setVlanId(pgModel.getVlanId());
                    policy = new HostNetworkPolicy();
                    portgrp.setPolicy(policy);
                    logger.info("Start to create virtual machine port group for host 【" + hostName + "】...");
                    if (null != host && null != host.getHostNetworkSystem()) {
                        host.getHostNetworkSystem().addPortGroup(portgrp);
                        // 组装新添加的端口组
                        HostPortGroup[] groups = host.getHostNetworkSystem().getNetworkInfo().getPortgroup();
                        for (HostPortGroup group : groups) {
                            HostPortGroupSpec groupSpec = group.getSpec();
                            if (null != groupSpec && groupSpec.getName().equals(pgModel.getName())) {
                                pgModel = AssembleTools.getInstance().assemblePortGroup(pgModel, group);
                                break;
                            }
                        }
                    }
                    logger.info("End to create virtual machine port group for host 【" + hostName + "】...");
                    NetworkModel model = new NetworkModel();
                    // 获取新创建端口组所在网络
                    Folder folder = datacenter.getNetworkFolder();
                    ManagedEntity[] entities = null;
                    boolean flag = true;
                    Long start = System.currentTimeMillis();
                    Network network = null;
                    while (flag) {
                        Thread.sleep(5 * 1000L);
                        entities = folder.getChildEntity();
                        // 超时和延时
                        if (System.currentTimeMillis() - start > 5 * 1000 * 60) {
                            flag = false;
                            break;
                        }
                        for (ManagedEntity networkEntity : entities) {
                            // 查询标准交换机网络
                            if (networkEntity instanceof Network
                                    && !(networkEntity instanceof DistributedVirtualPortgroup)) {
                                network = (Network) networkEntity;
                                if (network.getName().equals(pgModel.getName())) {
                                    flag = false;
                                    break;
                                }
                            }
                        }
                        if (!flag) {
                            break;
                        }
                    }
                    if (null == network && null != host) {
                        logger.warn("Start to remove error port group...");
                        host.getHostNetworkSystem().removePortGroup(pgModel.getName());
                        logger.warn("End to remove error port group...");
                        return new BocloudResult(false, "创建端口组失败");
                    } else if (null != network) {
                        logger.info("The network name of the port group is 【" + network.getName() + "】");
                        model.setName(network.getName());
                        model.setType(NetworkType.SwitchType.VSWITCH.name());
                        model.setStatus(network.getConfigStatus().name());
                        model.setNetworkVal(network.getMOR().getVal());
                    }
                    Integer portNum = 0;
                    if (null != host) {
                        // 获取机器上的标准交换机
                        HostVirtualSwitch[] vSwitchs = host.getHostNetworkSystem().getNetworkInfo().getVswitch();
                        for (HostVirtualSwitch virtualSwitch : vSwitchs) {
                            if (virtualSwitch.getName().equals(vswitchName)) {
                                portNum = virtualSwitch.getSpec().getNumPorts() - 8;
                                logger.info("Number of network ports is 【" + portNum + "】");
                                break;
                            }
                        }
                    }
                    model.setPortNum(portNum);
                    pgModel.setNetworkModel(model);
                    BocloudResult = new BocloudResult(true, pgModel, "创建端口组成功");
                    break;
                case DVPORT:
                    ManagedEntity dvswitchEntity = new InventoryNavigator(datacenter)
                            .searchManagedEntity("VmwareDistributedVirtualSwitch", vswitchName);
                    Assert.notNull(dvswitchEntity, "分布式交换机不存在！");
                    DistributedVirtualSwitch dvSwitch = (DistributedVirtualSwitch) dvswitchEntity;
                    DVPortgroupConfigSpec spec = new DVPortgroupConfigSpec();
                    spec.setName(pgModel.getName());
                    spec.setNumPorts(pgModel.getPortNum());
                    VMwareDVSPortSetting defaultPortConfig = new VMwareDVSPortSetting();
                    String vlanType = pgModel.getVlanType();
                    if (StringUtils.isEmpty(vlanType)) {
                        vlanType = NetworkType.VlanType.UNKNOWN.name();
                    }
                    switch (NetworkType.VlanType.valueOf(vlanType)) {
                        case Vlan:
                            // vlan
                            VmwareDistributedVirtualSwitchVlanIdSpec vlanSpec = new VmwareDistributedVirtualSwitchVlanIdSpec();
                            vlanSpec.setVlanId(pgModel.getVlanId());
                            defaultPortConfig.setVlan(vlanSpec);
                            break;
                        case TrunkVlan:
                            // 中继vlan
                            VmwareDistributedVirtualSwitchTrunkVlanSpec trunkVlanSpec = new VmwareDistributedVirtualSwitchTrunkVlanSpec();
                            String trunkVlan = pgModel.getTrunkVlan();
                            if (!StringUtils.isEmpty(trunkVlan)) {
                                List<NumericRange> vlanIds = JSONArray.parseArray(trunkVlan, NumericRange.class);
                                NumericRange[] vlanId = vlanIds.toArray(new NumericRange[vlanIds.size()]);
                                trunkVlanSpec.setVlanId(vlanId);
                                defaultPortConfig.setVlan(trunkVlanSpec);
                            }
                            break;
                        case Pvlan:
                            // 专用vlan
                            VmwareDistributedVirtualSwitchPvlanSpec pVlanSpec = new VmwareDistributedVirtualSwitchPvlanSpec();
                            pVlanSpec.setPvlanId(pgModel.getVlanId());
                            defaultPortConfig.setVlan(pVlanSpec);
                            break;
                        case UNKNOWN:
                        default:
                            break;
                    }
                    spec.setDefaultPortConfig(defaultPortConfig);
                    // earlyBinding静态绑定/ephemeral极短-无绑定/lateBinding动态绑定（已弃用）
                    spec.setType(DistributedVirtualPortgroupPortgroupType.earlyBinding.name());
                    logger.info("Start to create distributed virtual port group");
                    Task task = dvSwitch.createDVPortgroup_Task(spec);
                    if (task.waitForTask().equals(Task.SUCCESS)) {
                        logger.info("Success to create distributed virtual port group");
                        ManagedEntity portEntity = new InventoryNavigator(datacenter)
                                .searchManagedEntity("DistributedVirtualPortgroup", pgModel.getName());
                        if (null == portEntity) {
                            logger.warn("Distributed virtual port group create fail");
                            BocloudResult = new BocloudResult(false, "端口组创建失败");
                        } else {
                            DistributedVirtualPortgroup group = (DistributedVirtualPortgroup) portEntity;
                            pgModel.setGroupVal(group.getMOR().getVal());
                            BocloudResult = new BocloudResult(true, pgModel, "端口组创建成功");
                        }
                    } else {
                        BocloudResult = new BocloudResult(false,
                                "分布式端口组创建失败：" + task.getTaskInfo().getError().getLocalizedMessage());
                    }
                    break;
                case VMKERNEL:
                    // HostPortGroupSpec hpgs = new HostPortGroupSpec();
                    // hpgs.setName(pgModel.getName());
                    // hpgs.setVlanId(0); // not associated with a VLAN
                    // hpgs.setVswitchName(vswitchName);
                    // hpgs.setPolicy(new HostNetworkPolicy());
                    // host.getHostNetworkSystem().addPortGroup(hpgs);
                    // // add a virtual NIC to VMKernel
                    // HostVirtualNicSpec hvns = new HostVirtualNicSpec();
                    // hvns.setMac("00:50:56:7d:5e:0b");
                    // HostIpConfig hic = new HostIpConfig();
                    // hic.setDhcp(false);
                    // hic.setIpAddress("10.20.143.204");
                    // hic.setSubnetMask("255.255.252.0");
                    // hvns.setIp(hic);
                    // host.getHostNetworkSystem().addVirtualNic("VMKernel", hvns);
                    BocloudResult = new BocloudResult(false, "暂不支持此端口组的创建");
                    break;
                case UPLINK:
                    BocloudResult = new BocloudResult(false, "暂不支持此端口组的创建");
                    break;
                default:
                    BocloudResult = new BocloudResult(false, "暂不支持此端口组的创建");
                    break;
            }
            return BocloudResult;
        } catch (DuplicateName e) {
            logger.error("Port group name already exists");
            return new BocloudResult(false, "端口组创建失败：该名称已经存在");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "创建端口组失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult removePortGroup(String hostName, String pgName, String type) {
        try {
            ManagedEntity entity = null;
            BocloudResult BocloudResult = null;
            switch (NetworkType.PortGroupType.valueOf(type)) {
                case VIRTUAL:
                    entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
                    Assert.notNull(entity, "宿主机不存在！");
                    HostSystem host = (HostSystem) entity;
                    host.getHostNetworkSystem().removePortGroup(pgName);
                    BocloudResult = new BocloudResult(true, "删除端口组成功");
                    break;
                case DVPORT:
                    entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity("DistributedVirtualPortgroup",
                            pgName);
                    Assert.notNull(entity, "端口组不存在！");
                    DistributedVirtualPortgroup portGroup = (DistributedVirtualPortgroup) entity;
                    Task task = portGroup.destroy_Task();
                    if (task.waitForTask().equals(Task.SUCCESS)) {
                        BocloudResult = new BocloudResult(true, "端口组删除成功");
                    } else {
                        BocloudResult = new BocloudResult(false,
                                "端口组删除失败：" + task.getTaskInfo().getError().getLocalizedMessage());
                    }
                    break;
                case VMKERNEL:
                    BocloudResult = new BocloudResult(false, "端口组删除失败");
                    break;
                case UPLINK:
                    BocloudResult = new BocloudResult(false, "端口组删除失败");
                    break;
                default:
                    BocloudResult = new BocloudResult(false, "端口组删除失败");
                    break;
            }
            return BocloudResult;
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "删除端口组失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult updatePortGroup(String hostName, PortGroupModel pgModel) {
        try {
            ManagedEntity managedEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    pgModel.getVdcName());
            Assert.notNull(managedEntity, "数据中心不存在！");
            Datacenter datacenter = (Datacenter) managedEntity;
            ManagedEntity entity = null;
            BocloudResult BocloudResult = null;
            switch (NetworkType.PortGroupType.valueOf(pgModel.getType())) {
                case VMKERNEL:
                    break;
                case VIRTUAL:
                    entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS, hostName);
                    Assert.notNull(entity, "宿主机不存在！");
                    HostSystem host = (HostSystem) entity;
                    HostPortGroupSpec portgrp = new HostPortGroupSpec();
                    // 常规设置
                    portgrp.setName(pgModel.getName());
                    portgrp.setVlanId(pgModel.getVlanId());
                    portgrp.setVswitchName(pgModel.getSwitcherName());
                    HostNetworkPolicy policy = new HostNetworkPolicy();
                    // 安全设置
                    HostNetworkSecurityPolicy security = new HostNetworkSecurityPolicy();
                    // 混杂模式
                    security.setAllowPromiscuous(pgModel.getAllowPromiscuous());
                    // MAC地址更改
                    security.setMacChanges(pgModel.getMacChanges());
                    // 伪传输
                    security.setForgedTransmits(pgModel.getForgedTransmits());
                    policy.setSecurity(security);
                    // 流量调整
                    HostNetworkTrafficShapingPolicy shapingPolicy = new HostNetworkTrafficShapingPolicy();
                    // 状态
                    shapingPolicy.setEnabled(pgModel.getInTrafficEnabled());
                    if (pgModel.getInTrafficEnabled()) {
                        // 平均带宽
                        shapingPolicy.setAverageBandwidth(pgModel.getInAverageBandWidth());
                        // 突发大小
                        shapingPolicy.setBurstSize(pgModel.getInBurstSize());
                        // 带宽峰值
                        shapingPolicy.setPeakBandwidth(pgModel.getInPeakBandWidth());
                    }
                    policy.setShapingPolicy(shapingPolicy);
                    // 网卡绑定
                    HostNicTeamingPolicy nicTeaming = new HostNicTeamingPolicy();
                    // 负载平衡
                    nicTeaming.setPolicy(pgModel.getPolicy());
                    // 网络故障切换检测
                    HostNicFailureCriteria failureCriteria = new HostNicFailureCriteria();
                    failureCriteria.setCheckBeacon(pgModel.getCheckBeacon());
                    nicTeaming.setFailureCriteria(failureCriteria);
                    // 通知交换机
                    nicTeaming.setNotifySwitches(pgModel.getNotifySwitches());
                    // 故障恢复
                    nicTeaming.setRollingOrder(pgModel.getRollingOrder());
                    HostNicOrderPolicy nicOrder = new HostNicOrderPolicy();
                    // 活动适配器
                    List<String> activeNics = pgModel.getActiveNics();
                    if (!ListTool.isEmpty(activeNics)) {
                        nicOrder.setActiveNic(activeNics.toArray(new String[activeNics.size()]));
                    }
                    // 备用适配器
                    List<String> standbyNics = pgModel.getStandbyNics();
                    if (!ListTool.isEmpty(standbyNics)) {
                        nicOrder.setActiveNic(standbyNics.toArray(new String[standbyNics.size()]));
                    }
                    if (null != nicOrder) {
                        nicTeaming.setNicOrder(nicOrder);
                    }
                    policy.setNicTeaming(nicTeaming);
                    // 设置属性
                    portgrp.setPolicy(policy);
                    host.getHostNetworkSystem().updatePortGroup(pgModel.getOldName(), portgrp);
                    NetworkModel model = new NetworkModel();
                    // 获取新创建网络组所在网络
                    Folder folder = datacenter.getNetworkFolder();
                    ManagedEntity[] entities = null;
                    boolean flag = true;
                    Long start = System.currentTimeMillis();
                    Network network = null;
                    while (flag) {
                        Thread.sleep(5 * 1000L);
                        entities = folder.getChildEntity();
                        // 超时和延时
                        if (System.currentTimeMillis() - start > 5 * 1000 * 60L) {
                            flag = false;
                            break;
                        }
                        for (ManagedEntity networkEntity : entities) {
                            // 查询标准交换机网络
                            if (networkEntity instanceof Network
                                    && !(networkEntity instanceof DistributedVirtualPortgroup)) {
                                network = (Network) networkEntity;
                                if (network.getName().equals(pgModel.getName())) {
                                    flag = false;
                                    break;
                                }
                            }
                        }
                        if (!flag) {
                            break;
                        }
                    }
                    if (null != network) {
                        model.setName(network.getName());
                        model.setType(NetworkType.SwitchType.VSWITCH.name());
                        model.setStatus(network.getConfigStatus().name());
                        model.setNetworkVal(network.getMOR().getVal());
                    }
                    // 获取机器上的标准交换机
                    HostVirtualSwitch[] vSwitchs = host.getHostNetworkSystem().getNetworkInfo().getVswitch();
                    Integer portNum = 0;
                    for (HostVirtualSwitch virtualSwitch : vSwitchs) {
                        if (virtualSwitch.getName().equals(pgModel.getSwitcherName())) {
                            portNum = virtualSwitch.getSpec().getNumPorts() - 8;
                            logger.info("Number of network ports is 【" + portNum + "】");
                            break;
                        }
                    }
                    model.setPortNum(portNum);
                    pgModel.setNetworkModel(model);
                    BocloudResult = new BocloudResult(true, pgModel, "更新端口组成功");
                    break;
                case UPLINK:
                case DVPORT:
                    entity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity("DistributedVirtualPortgroup",
                            pgModel.getOldName());
                    Assert.notNull(entity, "端口组不存在！");
                    DistributedVirtualPortgroup portGroup = (DistributedVirtualPortgroup) entity;
                    DVPortgroupConfigSpec spec = new DVPortgroupConfigSpec();
                    spec.setConfigVersion(portGroup.getConfig().getConfigVersion());
                    // 常规
                    spec.setName(pgModel.getName());
                    if (pgModel.getType().equals(NetworkType.PortGroupType.DVPORT.name())) {
                        spec.setNumPorts(pgModel.getPortNum());
                    }
                    // spec.setType(DistributedVirtualPortgroupPortgroupType.earlyBinding.name());
                    spec.setDescription(pgModel.getRemark());
                    // 策略
                    VMwareDVSPortSetting defaultPortConfig = new VMwareDVSPortSetting();
                    if (pgModel.getType().equals(NetworkType.PortGroupType.DVPORT.name())) {
                        // 安全
                        DVSSecurityPolicy securityPolicy = new DVSSecurityPolicy();
                        // 混杂模式
                        BoolPolicy allowPromiscuous = new BoolPolicy();
                        allowPromiscuous.setValue(pgModel.getAllowPromiscuous());
                        securityPolicy.setAllowPromiscuous(allowPromiscuous);
                        // MAC地址更改
                        BoolPolicy macChanges = new BoolPolicy();
                        macChanges.setValue(pgModel.getMacChanges());
                        securityPolicy.setMacChanges(macChanges);
                        // 伪传输
                        BoolPolicy forgedTransmits = new BoolPolicy();
                        forgedTransmits.setValue(pgModel.getForgedTransmits());
                        securityPolicy.setForgedTransmits(forgedTransmits);
                        defaultPortConfig.setSecurityPolicy(securityPolicy);
                        // 输入流量调整
                        DVSTrafficShapingPolicy inShapingPolicy = new DVSTrafficShapingPolicy();
                        BoolPolicy inTrafficEnabled = new BoolPolicy();
                        inTrafficEnabled.setValue(pgModel.getInTrafficEnabled());
                        inShapingPolicy.setEnabled(inTrafficEnabled);
                        if (pgModel.getInTrafficEnabled()) {
                            // 输入流量平均带宽
                            LongPolicy inAverage = new LongPolicy();
                            inAverage.setValue(pgModel.getInAverageBandWidth());
                            inShapingPolicy.setAverageBandwidth(inAverage);
                            // 输入流量带宽峰值
                            LongPolicy inPeakBand = new LongPolicy();
                            inPeakBand.setValue(pgModel.getInPeakBandWidth());
                            inShapingPolicy.setPeakBandwidth(inPeakBand);
                            // 输入流量突发大小
                            LongPolicy inBurstSize = new LongPolicy();
                            inBurstSize.setValue(pgModel.getInBurstSize());
                            inShapingPolicy.setBurstSize(inBurstSize);
                        }
                        defaultPortConfig.setInShapingPolicy(inShapingPolicy);
                        // 输出流量调整
                        DVSTrafficShapingPolicy outShapingPolicy = new DVSTrafficShapingPolicy();
                        BoolPolicy outTrafficEnabled = new BoolPolicy();
                        outTrafficEnabled.setValue(pgModel.getOutTrafficEnabled());
                        outShapingPolicy.setEnabled(outTrafficEnabled);
                        if (pgModel.getOutTrafficEnabled()) {
                            // 输出流量平均带宽
                            LongPolicy outAverage = new LongPolicy();
                            outAverage.setValue(pgModel.getOutAverageBandWidth());
                            outShapingPolicy.setAverageBandwidth(outAverage);
                            // 输出流量带宽峰值
                            LongPolicy outPeakBand = new LongPolicy();
                            outPeakBand.setValue(pgModel.getOutPeakBandWidth());
                            outShapingPolicy.setPeakBandwidth(outPeakBand);
                            // 输出流量突发大小
                            LongPolicy outBurstSize = new LongPolicy();
                            outBurstSize.setValue(pgModel.getOutBurstSize());
                            outShapingPolicy.setBurstSize(outBurstSize);
                        }
                        defaultPortConfig.setOutShapingPolicy(outShapingPolicy);
                        // vlan
                        String vlanType = pgModel.getVlanType();
                        if (StringUtils.isEmpty(vlanType)) {
                            vlanType = NetworkType.VlanType.UNKNOWN.name();
                        }
                        switch (NetworkType.VlanType.valueOf(vlanType)) {
                            case Vlan:
                                // vlan
                                VmwareDistributedVirtualSwitchVlanIdSpec vlanSpec = new VmwareDistributedVirtualSwitchVlanIdSpec();
                                vlanSpec.setVlanId(pgModel.getVlanId());
                                defaultPortConfig.setVlan(vlanSpec);
                                break;
                            case TrunkVlan:
                                // 中继vlan
                                VmwareDistributedVirtualSwitchTrunkVlanSpec trunkVlanSpec = new VmwareDistributedVirtualSwitchTrunkVlanSpec();
                                String trunkVlan = pgModel.getTrunkVlan();
                                if (!StringUtils.isEmpty(trunkVlan)) {
                                    List<NumericRange> vlanIds = JSONArray.parseArray(trunkVlan, NumericRange.class);
                                    NumericRange[] vlanId = vlanIds.toArray(new NumericRange[vlanIds.size()]);
                                    trunkVlanSpec.setVlanId(vlanId);
                                    defaultPortConfig.setVlan(trunkVlanSpec);
                                }
                                break;
                            case Pvlan:
                                // 专用vlan
                                VmwareDistributedVirtualSwitchPvlanSpec pVlanSpec = new VmwareDistributedVirtualSwitchPvlanSpec();
                                pVlanSpec.setPvlanId(pgModel.getVlanId());
                                defaultPortConfig.setVlan(pVlanSpec);
                                break;
                            default:
                                break;
                        }
                        // 成组和故障切换
                        VmwareUplinkPortTeamingPolicy uplinkTeamingPolicy = new VmwareUplinkPortTeamingPolicy();
                        // 负载平衡
                        StringPolicy policyVal = new StringPolicy();
                        policyVal.setValue(pgModel.getPolicy());
                        uplinkTeamingPolicy.setPolicy(policyVal);
                        // 网络故障切换检测
                        DVSFailureCriteria failureCriteriaVal = new DVSFailureCriteria();
                        BoolPolicy checkBeaconVal = new BoolPolicy();
                        checkBeaconVal.setValue(pgModel.getCheckBeacon());
                        failureCriteriaVal.setCheckBeacon(checkBeaconVal);
                        uplinkTeamingPolicy.setFailureCriteria(failureCriteriaVal);
                        // 通知交换机
                        BoolPolicy notifySwitchesVal = new BoolPolicy();
                        notifySwitchesVal.setValue(pgModel.getNotifySwitches());
                        uplinkTeamingPolicy.setNotifySwitches(notifySwitchesVal);
                        // 故障恢复
                        BoolPolicy rollingOrderVal = new BoolPolicy();
                        rollingOrderVal.setValue(pgModel.getRollingOrder());
                        uplinkTeamingPolicy.setRollingOrder(rollingOrderVal);
                        VMwareUplinkPortOrderPolicy uplinkPortOrder = new VMwareUplinkPortOrderPolicy();
                        // 活动上行链路
                        List<String> activeUplinkPorts = pgModel.getActiveUplinks();
                        if (!ListTool.isEmpty(activeUplinkPorts)) {
                            uplinkPortOrder
                                    .setActiveUplinkPort(activeUplinkPorts.toArray(new String[activeUplinkPorts.size()]));
                        }
                        // 备用上行链路
                        List<String> standbyUplinkPorts = pgModel.getStandbyUplinks();
                        if (!ListTool.isEmpty(standbyUplinkPorts)) {
                            uplinkPortOrder.setStandbyUplinkPort(
                                    standbyUplinkPorts.toArray(new String[standbyUplinkPorts.size()]));
                        }
                        if (null != uplinkPortOrder) {
                            uplinkTeamingPolicy.setUplinkPortOrder(uplinkPortOrder);
                        }
                        defaultPortConfig.setUplinkTeamingPolicy(uplinkTeamingPolicy);
                        // 资源分配(已过时)
                    } else {
                        // 上行链路端口组只能使用中继vlan
                        VmwareDistributedVirtualSwitchTrunkVlanSpec trunkVlanSpec = new VmwareDistributedVirtualSwitchTrunkVlanSpec();
                        String trunkVlan = pgModel.getTrunkVlan();
                        if (!StringUtils.isEmpty(trunkVlan)) {
                            List<NumericRange> vlanIds = JSONArray.parseArray(trunkVlan, NumericRange.class);
                            NumericRange[] vlanId = vlanIds.toArray(new NumericRange[vlanIds.size()]);
                            trunkVlanSpec.setVlanId(vlanId);
                            defaultPortConfig.setVlan(trunkVlanSpec);
                        }
                    }
                    // 正在监控(NetFlow状态)
                    BoolPolicy ipfixEnabled = new BoolPolicy();
                    ipfixEnabled.setValue(pgModel.getIpfixEnabled());
                    defaultPortConfig.setIpfixEnabled(ipfixEnabled);
                    // 其他(阻止所有端口)
                    BoolPolicy blocked = new BoolPolicy();
                    blocked.setValue(pgModel.getBlocked());
                    defaultPortConfig.setBlocked(blocked);
                    // 高级
                    VMwareDVSPortgroupPolicy groupPolicy = new VMwareDVSPortgroupPolicy();
                    // 断开连接时配置重置
                    groupPolicy.setPortConfigResetAtDisconnect(pgModel.getDisconnectConfigReset());
                    // 允许替代端口策略
                    groupPolicy.setUplinkTeamingOverrideAllowed(pgModel.getAllowOverridePolicy());
                    if (null != groupPolicy) {
                        spec.setPolicy(groupPolicy);
                    }
                    spec.setDefaultPortConfig(defaultPortConfig);
                    Task task = portGroup.reconfigureDVPortgroup_Task(spec);
                    if (task.waitForTask().equals(Task.SUCCESS)) {
                        logger.info("Success to update distributed virtual port group");
                        BocloudResult = new BocloudResult(true, pgModel, "端口组更新成功");
                    } else {
                        BocloudResult = new BocloudResult(false,
                                "端口组更新失败：" + task.getTaskInfo().getError().getLocalizedMessage());
                    }
                    break;
                default:
                    break;
            }
            return BocloudResult;
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "端口组更新失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult addHost(SwitchModel model) {
        try {
            ManagedEntity center = (Datacenter) new InventoryNavigator(this.getRootFolder()).searchManagedEntity(DC,
                    model.getVdcName());
            Assert.notNull(center, "数据中心不存在！");
            Datacenter dc = (Datacenter) center;
            ManagedEntity entity = new InventoryNavigator(dc).searchManagedEntity("VmwareDistributedVirtualSwitch",
                    model.getName());
            Assert.notNull(entity, "分布式交换机不存在！");
            VmwareDistributedVirtualSwitch virtualSwitch = (VmwareDistributedVirtualSwitch) entity;
            // 获取所有的物理机器以及物理网卡信息
            Map<String, List<String>> hostMap = model.getValues();
            // 存放物理机的名称及key值
            Map<String, String> hostKeyMap = new HashMap<>();
            // 获取hostName对应的物理机的key值
            for (String hostName : hostMap.keySet()) {
                ManagedEntity hostEntity = new InventoryNavigator(this.getRootFolder()).searchManagedEntity(HS,
                        hostName);
                if (null != hostEntity) {
                    HostSystem hostSystem = (HostSystem) hostEntity;
                    hostKeyMap.put(hostName, hostSystem.getMOR().getVal());
                } else {
                    return new BocloudResult(false, "创建分布式交换机失败：获取主机信息失败");
                }
            }
            // 物理机数组
            DistributedVirtualSwitchHostMemberConfigSpec[] dvshmcsArray = new DistributedVirtualSwitchHostMemberConfigSpec[hostMap
                    .size()];
            int dvshmcsIndex = 0;
            for (String hostName : hostMap.keySet()) {
                List<String> pNics = hostMap.get(hostName);
                // hostName主机对应的物理网卡
                if (!ListTool.isEmpty(pNics)) {
                    DistributedVirtualSwitchHostMemberPnicSpec[] pnicSpecs = new DistributedVirtualSwitchHostMemberPnicSpec[pNics
                            .size()];
                    int specIndex = 0;
                    for (String pNic : pNics) {
                        DistributedVirtualSwitchHostMemberPnicSpec pnicSpc = new DistributedVirtualSwitchHostMemberPnicSpec();
                        pnicSpc.setPnicDevice(pNic);
                        pnicSpecs[specIndex++] = pnicSpc;
                    }
                    DistributedVirtualSwitchHostMemberPnicBacking dvshmpb = new DistributedVirtualSwitchHostMemberPnicBacking();
                    dvshmpb.setPnicSpec(pnicSpecs);
                    DistributedVirtualSwitchHostMemberConfigSpec dvshmcs = new DistributedVirtualSwitchHostMemberConfigSpec();
                    // 设置物理网卡
                    dvshmcs.setBacking(dvshmpb);
                    ManagedObjectReference reference = new ManagedObjectReference();
                    // 类型
                    reference.setType(HS);
                    // 物理机的唯一value值
                    reference.setVal(hostKeyMap.get(hostName));
                    HostSystem hostSystem = new HostSystem(this.getRootFolder().getServerConnection(), reference);
                    // 设置物理机
                    dvshmcs.setHost(hostSystem.getMOR());
                    dvshmcs.setOperation("add");
                    dvshmcsArray[dvshmcsIndex++] = dvshmcs;
                } else {
                    return new BocloudResult(false, "主机参数不正确,网卡不能为空");
                }
            }
            VMwareDVSConfigSpec spec = new VMwareDVSConfigSpec();
            spec.setConfigVersion(virtualSwitch.getConfig().getConfigVersion());
            spec.setHost(dvshmcsArray);
            Task task = virtualSwitch.reconfigureDvs_Task(spec);
            if (task.waitForTask().equals(Task.SUCCESS)) {
                return new BocloudResult(true, model, "添加主机成功");
            }
            return new BocloudResult(false, "添加主机失败：" + task.getTaskInfo().getError().getLocalizedMessage());
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "添加主机失败");
        } finally {
            this.getServiceInstance().getServerConnection().logout();
        }
    }

    @Override
    public BocloudResult listVSwitch(SwitchModel model) {
        return null;
    }

    @Override
    public BocloudResult listProfile() {
        return null;
    }

    @Override
    public BocloudResult create(NetworkModel model) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult remove(String networkId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult detail(String networkId) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public BocloudResult modify(NetworkModel model) {
        // TODO Auto-generated method stub
        return null;
    }

}
