package com.bocloud.cmp.provider.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolFlavorConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Flavor;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineFlavorProvider
 * @Description:
 * @Author lxy
 * @Date 2021/9/14 16:02
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineFlavorProvider extends VolcengineProvider implements FlavorProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineFlavorProvider.class);
    private BeanConvertor<FlavorModel, Flavor> flavorConvertor = new VolFlavorConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineFlavorProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult sync() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.LISTFLAVORS, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Flavor> flavorList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Flavors")), Flavor.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询规格列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.LISTFLAVORS, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Flavor> flavors = JSONArray.parseArray(JSONObject.toJSONString(result.get("Flavors")), Flavor.class);
                    flavorList.addAll(flavors);
                }
            }
            List<FlavorModel> convertList = flavorConvertor.convertList(flavorList);
            for (FlavorModel flavorModel : convertList) {
                flavorModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("sync Flavor error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(FlavorModel flavor) {
        return null;
    }

    @Override
    public BocloudResult remove(String flavorId) {
        return null;
    }

    @Override
    public BocloudResult update(FlavorModel flavorModel) {
        return null;
    }
}
