package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.model.KeypairModel;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolImageConvertor;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Image;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.volcengine.helper.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: VolcengineImageProvider
 * @Description: 镜像
 * @Author lxy
 * @Date 2021/9/10 10:53
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineImageProvider extends VolcengineProvider implements ImageProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineImageProvider.class);
    private BeanConvertor<ImageModel, Image> imageConvertor = new VolImageConvertor();
    private static final int PAGE_SIZE = 100;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineImageProvider(Butler butler) {
        super(butler);
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEIMAGES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Image> images = JSONArray.parseArray(JSONObject.toJSONString(result.get("Images")), Image.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询镜像列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEIMAGES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Image> imageList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Images")), Image.class);
                    images.addAll(imageList);
                }
            }

            List<ImageModel> convertList = imageConvertor.convertList(images);
            for (ImageModel imageModel : convertList) {
                imageModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list image error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }


    @Override
    public BocloudResult detail(String imageId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ImageIds.1", imageId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEIMAGES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Image> images = JSONArray.parseArray(JSONObject.toJSONString(result.get("Images")), Image.class);
            List<ImageModel> convertList = imageConvertor.convertList(images);
            for (ImageModel imageModel : convertList) {
                imageModel.setRegionId(this.getRegionId());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("detail image error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult remove(String imageId) {
        return null;
    }

    @Override
    public BocloudResult modify(ImageModel image) {
        return null;
    }

    @Override
    public BocloudResult create(ImageModel imageModel) {
        return null;
    }

}
