package com.bocloud.cmp.provider.compute;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.*;
import com.bocloud.cmp.provider.VolcengineProvider;
import com.bocloud.cmp.provider.convertor.BeanConvertor;
import com.bocloud.cmp.provider.convertor.VolServerConvertor;
import com.bocloud.cmp.provider.network.VolcengineNetworkCardProvider;
import com.bocloud.cmp.provider.utils.VolcengineConst;
import com.bocloud.cmp.provider.volmodel.Instances;
import com.bocloud.cmp.provider.volmodel.NetworkInterfaceRes;
import com.bocloud.common.annotations.Provider;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.volcengine.helper.Utils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName: VolcengineServerProvider
 * @Description: 云主机
 * @Author lxy
 * @Date 2021/9/10 18:26
 */
@Provider("bocloud.cmp.volcengine")
public class VolcengineServerProvider extends VolcengineProvider implements ServerProvider {
    private static Logger logger = LoggerFactory.getLogger(VolcengineServerProvider.class);
    private BeanConvertor<ServerModel, Instances> convertor = new VolServerConvertor();
    private static final int PAGE_SIZE = 100;
    private Butler butler;

    /**
     * 构造方法，所有的云平台驱动都必须要实现或重写该构造方法
     *
     * @param butler 云平台认证信息
     */
    public VolcengineServerProvider(Butler butler) {
        super(butler);
        this.butler = butler;
    }

    @Override
    public BocloudResult list() {
        try {
            int pageNumber = 1;
            Map<String, String> params = new HashMap<>();
            params.put("PageSize", String.valueOf(PAGE_SIZE));
            params.put("PageNumber", String.valueOf(pageNumber));
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEINSTANCES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Instances> instancesList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Instances")), Instances.class);
            Integer totalCount = result.getInteger("TotalCount");
            if (totalCount == 0) {
                logger.info("查询云主机列表:totalCount为0");
                return new BocloudResult(true, new ArrayList<KeypairModel>(), "empty");
            }
            int totalPage = (totalCount.intValue() % PAGE_SIZE) == 0
                    ? totalCount.intValue() / PAGE_SIZE : (totalCount.intValue() / PAGE_SIZE + 1);

            if (totalPage > pageNumber) {
                for (int i = pageNumber + 1; i <= totalPage; i++) {
                    params.put("PageSize", String.valueOf(PAGE_SIZE));
                    params.put("PageNumber", String.valueOf(i));
                    response = operation(VolcengineConst.DESCRIBEINSTANCES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
                    if (response.isFailed()) {
                        return response;
                    }
                    data = response.getData();
                    result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
                    List<Instances> instances = JSONArray.parseArray(JSONObject.toJSONString(result.get("Instances")), Instances.class);
                    instancesList.addAll(instances);
                }
            }

            List<ServerModel> convertList = convertor.convertList(instancesList);
            VolcengineImageProvider imageProvider = new VolcengineImageProvider(butler);
            BocloudResult imageList = imageProvider.list();
            if (imageList.isFailed() || null == imageList.getData()) {
                return new BocloudResult(false, "查询失败,调用镜像查询异常");
            }
            List<ImageModel> imageModels = JSONArray.parseArray(JSONObject.toJSONString(imageList.getData()), ImageModel.class);
            Map<String, ImageModel> imageModelMap = imageModels.stream().collect(Collectors.toMap(ImageModel::getImageUuid, a -> a, (k1, k2) -> k1));
            for (ServerModel serverModel : convertList) {
                serverModel.setRegionId(this.getRegionId());
                ImageModel imageModel = imageModelMap.get(serverModel.getImageUuid());
                serverModel.setOsName(imageModel.getName());
                serverModel.setOsCategory(imageModel.getOsCategory());
            }
            return new BocloudResult(true, convertList, "查询成功");
        } catch (Exception e) {
            logger.error("list Server error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult list(String hostId) {
        return null;
    }

    @Override
    public BocloudResult create(ServerModel server) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("ZoneId", server.getZone());
            params.put("ImageId", server.getImageUuid());
            params.put("InstanceType", server.getFlavorUuid());
            params.put("InstanceName", server.getName());
            if (StringUtils.isNotBlank(server.getRemark())) {
                params.put("Description", server.getRemark());
            }
            if (StringUtils.isNotBlank(server.getHostName())) {
                params.put("HostName", server.getHostName());
            }
            if (StringUtils.isNotBlank(server.getPassword())) {
                params.put("Password", server.getPassword());
            }
            if (StringUtils.isNotBlank(server.getKeypairName())) {
                params.put("KeyPairName", server.getKeypairName());
            }
            params.put("Volumes.1.VolumeType", server.getRootVolumeType());
            params.put("Volumes.1.Size", String.valueOf(server.getRootSize()));
            List<VolumeModel> diskDevices = server.getDiskDevices();
            if (!ListTool.isEmpty(diskDevices)) {
                for (int i = 1; i <= diskDevices.size(); i++) {
                    params.put("Volumes." + (i + 1) + ".VolumeType", diskDevices.get(i - 1).getType());
                    params.put("Volumes." + (i + 1) + ".Size", String.valueOf(diskDevices.get(i - 1).getSize()));
                }
            }
            params.put("NetworkInterfaces.1.SubnetId", server.getSubnetUuid());
            List<String> securityGroups = server.getSecurityGroups();
            String securityGroupId = "";
            for (int i = 1; i <= securityGroups.size(); i++) {
                params.put("NetworkInterfaces.1.SecurityGroupIds." + i, securityGroups.get(i - 1));
            }

            BocloudResult<JSONObject> response = operation(VolcengineConst.CREATEINSTANCES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<String> instanceIds = JSONArray.parseArray(result.getString("InstanceIds"), String.class);
            if (ListTool.isEmpty(instanceIds)) {
                return new BocloudResult(false, "云主机" + server.getName() + "创建失败");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(instanceIds.get(0)).getData(), "创建云主机成功");
        } catch (Exception e) {
            logger.error("create server message:", e);
            return new BocloudResult(false, "创建云主机失败");
        }
    }

    @Override
    public BocloudResult modify(ServerModel server) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", server.getInstanceId());
            if (StringUtils.isNotBlank(server.getRemark())) {
                params.put("Description", server.getRemark());
            }
            if (StringUtils.isNotBlank(server.getName())) {
                params.put("InstanceName", server.getName());
            }
            BocloudResult<JSONObject> response = operation(VolcengineConst.MODIFYINSTANCEATTRIBUTE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "云主机: " + server.getInstanceId() + " 编辑失败");
            }
            Thread.sleep(3000);
            return new BocloudResult(true, this.detail(server.getInstanceId()).getData(), "编辑云主机成功");
        } catch (Exception e) {
            logger.error("modify server message:", e);
            return new BocloudResult(false, "编辑云主机失败");
        }
    }

    @Override
    public BocloudResult migrate(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult migrate(OSMigrateModel model) {
        return null;
    }

    @Override
    public BocloudResult remove(String instanceId, String name) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", instanceId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DELETEINSTANCE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "删除云主机: " + instanceId + " 失败");
            }
            return new BocloudResult(true, "删除云主机成功");
        } catch (Exception e) {
            logger.error("remove server error message:", e);
            return new BocloudResult(false, "删除云主机失败");
        }
    }

    @Override
    public BocloudResult detail(String instanceId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceIds.1", instanceId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.DESCRIBEINSTANCES, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("Result")));
            List<Instances> instancesList = JSONArray.parseArray(JSONObject.toJSONString(result.get("Instances")), Instances.class);
            List<ServerModel> convertList = convertor.convertList(instancesList);
            VolcengineImageProvider imageProvider = new VolcengineImageProvider(butler);

            for (ServerModel serverModel : convertList) {
                BocloudResult image = imageProvider.detail(serverModel.getImageUuid());
                serverModel.setRegionId(this.getRegionId());
                ImageModel imageModel = JSONArray.parseArray(JSONObject.toJSONString(image.getData()), ImageModel.class).get(0);
                serverModel.setOsName(imageModel.getName());
                serverModel.setOsCategory(imageModel.getOsCategory());
            }
            return new BocloudResult(true, convertList.get(0), "查询成功");
        } catch (Exception e) {
            logger.error("detail server error:", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult start(String instanceId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", instanceId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.STARTINSTANCE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "开启云主机: " + instanceId + " 失败");
            }
            return waitForJobStatus(requestId, instanceId, "RUNNING", "开启");
        } catch (Exception e) {
            logger.error("start server error message:", e);
            return new BocloudResult(false, "开启云主机失败");
        }
    }

    @Override
    public BocloudResult stop(String instanceId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", instanceId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.STOPINSTANCE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "关闭云主机: " + instanceId + " 失败");
            }
            return waitForJobStatus(requestId, instanceId, "STOPPED", "关闭");
        } catch (Exception e) {
            logger.error("stop server error message:", e);
            return new BocloudResult(false, "关闭云主机失败");
        }
    }

    @Override
    public BocloudResult reboot(String instanceId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", instanceId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.REBOOTINSTANCE, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "重启云主机: " + instanceId + " 失败");
            }
            return waitForJobStatus(requestId, instanceId, "RUNNING", "重启");
        } catch (Exception e) {
            logger.error("reboot server error message:", e);
            return new BocloudResult(false, "重启云主机失败");
        }
    }

    public BocloudResult waitForJobStatus(String jobId, String instanceId, String status, String message) {
        logger.info(message + " wait job ==  jobId :" + jobId);
        BocloudResult result = new BocloudResult(false, message + "失败");
        Boolean flag = true;
        Long startTime = System.currentTimeMillis();

        while (flag) {
            BocloudResult detail = this.detail(instanceId);
            ServerModel serverModel = JSONObject.parseObject(JSONObject.toJSONString(detail.getData()), ServerModel.class);
            String serverStatus = serverModel.getStatus();

            if (status.equalsIgnoreCase(serverStatus)) {
                result = new BocloudResult(true, serverModel, message + "成功");
                flag = false;
                break;
            } else {
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            Long endTime = System.currentTimeMillis();
            // 等待三分钟
            if (endTime - startTime > 20 * 60 * 1000) {
                flag = false;
                break;
            }
        }
        logger.info("wait job end ==  jobId :" + jobId);
        return result;
    }

    @Override
    public BocloudResult softReboot(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult pause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unPause(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult suspend(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult resume(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult joinSGroup(String instanceId, String group) {
        BocloudResult serverDetail = this.detail(instanceId);
        ServerModel serverModel = JSONObject.parseObject(JSONObject.toJSONString(serverDetail.getData()), ServerModel.class);
        List<NetworkInterfaceRes> networkInterfaceRes = JSONArray.parseArray(serverModel.getPrivateIps(), NetworkInterfaceRes.class);
        for (NetworkInterfaceRes network : networkInterfaceRes) {
            if ("primary".equalsIgnoreCase(network.getType())) {
                List<String> securityGroupIds = JSONArray.parseArray(network.getSecurityGroupIds(), String.class);
                securityGroupIds.add(group);

                VolcengineNetworkCardProvider networkCardProvider = new VolcengineNetworkCardProvider(butler);
                BocloudResult networkCardDetail = networkCardProvider.detail(network.getNetworkInterfaceId());
                NetworkCardModel networkCardModel = JSONObject.parseObject(JSONObject.toJSONString(networkCardDetail.getData()), NetworkCardModel.class);
                networkCardModel.setSecurityGroupIds(securityGroupIds);
                return networkCardProvider.modify(networkCardModel);
            }
        }
        return new BocloudResult(false, "添加网卡失败，请查看底层数据");
    }

    @Override
    public BocloudResult leaveSGroup(String instanceId, String group) {
        BocloudResult serverDetail = this.detail(instanceId);
        ServerModel serverModel = JSONObject.parseObject(JSONObject.toJSONString(serverDetail.getData()), ServerModel.class);
        List<NetworkInterfaceRes> networkInterfaceRes = JSONArray.parseArray(serverModel.getPrivateIps(), NetworkInterfaceRes.class);
        for (NetworkInterfaceRes network : networkInterfaceRes) {
            if ("primary".equalsIgnoreCase(network.getType())) {
                List<String> securityGroupIds = JSONArray.parseArray(network.getSecurityGroupIds(), String.class);
                securityGroupIds.remove(group);

                VolcengineNetworkCardProvider networkCardProvider = new VolcengineNetworkCardProvider(butler);
                BocloudResult networkCardDetail = networkCardProvider.detail(network.getNetworkInterfaceId());
                NetworkCardModel networkCardModel = JSONObject.parseObject(JSONObject.toJSONString(networkCardDetail.getData()), NetworkCardModel.class);
                networkCardModel.setSecurityGroupIds(securityGroupIds);
                return networkCardProvider.modify(networkCardModel);
            }
        }
        return new BocloudResult(false, "移除网卡失败，请查看底层数据");
    }

    @Override
    public BocloudResult snapshot(SnapshotModel snapshotModel) {
        return null;
    }

    @Override
    public BocloudResult vncConsole(String vncType, String serverId) {
        return null;
    }

    @Override
    public BocloudResult changePassword(String instanceId, String password) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("Id", instanceId);
            params.put("Password", password);
            BocloudResult<JSONObject> response = operation(VolcengineConst.RESETAUTHENTICATION, VolcengineConst.VERSION_20180101, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "云主机重置密码: " + instanceId + " 失败");
            }
            return new BocloudResult(true, "云主机重置密码成功");
        } catch (Exception e) {
            logger.error(" server changePassword error message:", e);
            return new BocloudResult(false, "云主机重置密码失败");
        }
    }

    @Override
    public BocloudResult attachNetwork(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult detachNetwork(String instanceId, String portUuid) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId) {
        try {
            Map<String, String> params = new HashMap<>();
            params.put("InstanceId", instanceId);
            params.put("InstanceType", flavorId);
            BocloudResult<JSONObject> response = operation(VolcengineConst.INNERCREATERESIZEORDER, VolcengineConst.VERSION_20200401, Utils.mapToPairList(params), VolcengineConst.service.ECS);
            if (response.isFailed()) {
                return response;
            }
            JSONObject data = response.getData();
            JSONObject result = JSONObject.parseObject(JSONObject.toJSONString(data.get("ResponseMetadata")));
            String requestId = result.getString("RequestId");
            if (StringUtils.isBlank(requestId)) {
                return new BocloudResult(false, "变更云主机: " + instanceId + " 失败");
            }
            return waitForJobStatus(requestId, instanceId, "RUNNING", "配置变更");
        } catch (Exception e) {
            logger.error("resize server error message:", e);
            return new BocloudResult(false, "变更云主机失败");
        }
    }

    @Override
    public BocloudResult confirmResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult revertResize(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult hotConfigEnable(String instanceId, Boolean cpuHotEnable, Boolean MemHotEnable) {
        return null;
    }

    @Override
    public BocloudResult getTicket(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult createInResPool(ServerModel server) {
        return null;
    }

    @Override
    public BocloudResult mountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult unmountToolsInstaller(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult upgradeToolsTask(String instanceId, String options) {
        return null;
    }

    @Override
    public BocloudResult getNetCard(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult markAsTemplate(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult changeVspVmPasswd(String instanceId, String oldPasswd, String newPasswd) {
        return null;
    }

    @Override
    public BocloudResult diskExtend(String instanceId, String diskLabel, Integer extendSize) {
        return null;
    }

    @Override
    public BocloudResult getMetric(String type) {
        return null;
    }

    @Override
    public BocloudResult vmrcConsole(String instanceId) {
        return null;
    }

    @Override
    public BocloudResult getJnlpXml(Long vmId, String address) {
        return null;
    }

    @Override
    public BocloudResult resetInternetMaxBandwidthOut(String instanceId, Integer internetMaxBandwidthOut) {
        return null;
    }

    @Override
    public BocloudResult resetInstance(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult bindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult unbindKeypair(ServerModel serverModel) {
        return null;
    }

    @Override
    public BocloudResult resize(String instanceId, String flavorId, Boolean forceStop) {
        return null;
    }

    @Override
    public BocloudResult createImage(String instanceId, String imageName, List<String> volumes) {
        return null;
    }
}
