package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ElasticIpModel;
import com.bocloud.cmp.provider.volmodel.EipAddresses;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: VolEipConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/17 17:26
 */
public class VolEipConvertor implements BeanConvertor<ElasticIpModel, EipAddresses> {
    @Override
    public List<ElasticIpModel> convertList(List<? extends EipAddresses> objects) {
        List<ElasticIpModel> regionModels = new ArrayList<ElasticIpModel>();
        for (EipAddresses object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public ElasticIpModel convertModel(EipAddresses eipAddress) {
        ElasticIpModel model = null;
        if (null != eipAddress) {
            model = new ElasticIpModel();
            model.setElasticIpUuid(eipAddress.getAllocationId());
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            String creationTime = eipAddress.getAllocationTime();
            if (null != creationTime && !"".equals(creationTime)) {
                try {
                    model.setCreateTime(sdf.parse(creationTime));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            model.setBandwidth(String.valueOf(eipAddress.getBandwidth()));
            model.setRemark(eipAddress.getDescription());
            if (!StringUtils.isEmpty(eipAddress.getInstanceId())) {
                model.setInstanceId(eipAddress.getInstanceId());
                model.setInstanceType(eipAddress.getInstanceType());
            }
            model.setPublicIp(eipAddress.getEipAddress());
            if (StringUtils.isEmpty(eipAddress.getName())) {
                model.setName(eipAddress.getAllocationId());
            } else {
                model.setName(eipAddress.getName());
            }
            model.setStatus(eipAddress.getStatus().toUpperCase());
            model.setType(eipAddress.getISP());
        }
        return model;
    }
}
