package com.bocloud.cmp.provider.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.FlavorModel;
import com.bocloud.cmp.provider.volmodel.Flavor;
import com.bocloud.common.utils.ListTool;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: VolFlavorConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/14 17:25
 */
public class VolFlavorConvertor implements BeanConvertor<FlavorModel, Flavor> {
    @Override
    public List<FlavorModel> convertList(List<? extends Flavor> objects) {
        List<FlavorModel> regionModels = new ArrayList<FlavorModel>();
        for (Flavor object : objects) {
            regionModels.add(convertModel(object));
        }
        return regionModels;
    }

    @Override
    public FlavorModel convertModel(Flavor info) {
        FlavorModel model = new FlavorModel();
        if (null != info) {
            FlavorModel flavorModel = new FlavorModel();
            flavorModel.setCpuArchitecture(info.getArchitecture());
            flavorModel.setName(info.getId());
            flavorModel.setCpu(info.getCpu());
            flavorModel.setFlavorUuid(info.getId());
            flavorModel.setMemory(
                    BigDecimal.valueOf(info.getMemMB()).divide(BigDecimal.valueOf(1024), 2, BigDecimal.ROUND_HALF_UP));
            flavorModel.setUuid(info.getId());
            String typeFamily = info.getTypeFamily();
            flavorModel.setFamily(typeFamily);

            List<String> zones = info.getZones();
            if (!ListTool.isEmpty(zones)) {
                flavorModel.setZone(JSONObject.toJSONString(zones));
                flavorModel.setAvailabilityZoneIds(JSONObject.toJSONString(zones));
            }

            if (typeFamily.equalsIgnoreCase("ecs-g1")) {
                flavorModel.setPerformanceType("通用型g1");
            } else if (typeFamily.equalsIgnoreCase("ecs-g1e")) {
                flavorModel.setPerformanceType("通用增强型g1e");
            } else if (typeFamily.equalsIgnoreCase("ecs-c1")) {
                flavorModel.setPerformanceType("计算型c1");
            } else if (typeFamily.equalsIgnoreCase("ecs-c1e")) {
                flavorModel.setPerformanceType("计算增强型c1e");
            } else if (typeFamily.equalsIgnoreCase("ecs-r1")) {
                flavorModel.setPerformanceType("内存型r1");
            } else if (typeFamily.equalsIgnoreCase("ecs-r1e")) {
                flavorModel.setPerformanceType("内存增强型r1e");
            } else if (typeFamily.equalsIgnoreCase("ecs-d1s")) {
                flavorModel.setPerformanceType("大数据存储型d1s");
            } else if (typeFamily.equalsIgnoreCase("ecs-g1t")) {
                flavorModel.setPerformanceType("GPU型g1t");
            } else if (typeFamily.equalsIgnoreCase("ecs-g1tl")) {
                flavorModel.setPerformanceType("GPU型g1tl");
            } else if (typeFamily.equalsIgnoreCase("ecs-g1v")) {
                flavorModel.setPerformanceType("GPU型g1v");
            }
            return flavorModel;
        }
        return model;
    }
}
