package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.ImageModel;
import com.bocloud.cmp.provider.volmodel.Image;
import com.google.common.collect.Lists;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * @ClassName: VolImageConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/10 15:36
 */
public class VolImageConvertor implements BeanConvertor<ImageModel, Image> {
    @Override
    public List<ImageModel> convertList(List<? extends Image> images) {
        List<ImageModel> models = Lists.newArrayList();
        for (Image image : images) {
            models.add(convertModel(image));
        }
        return models;
    }

    @Override
    public ImageModel convertModel(Image image) {
        ImageModel imageModel = new ImageModel();
        imageModel.setName(image.getImageName());
        imageModel.setStatus(convertStatus(image.getStatus()).toUpperCase());
        imageModel.setRemark(image.getDescription());
        imageModel.setOsCategory(image.getOsType());
        imageModel.setOsName(image.getPlatform());
        //首字母大写转换
        imageModel.setOsType(image.getOsType().substring(0, 1).toUpperCase() + image.getOsType().substring(1));
        imageModel.setImageUuid(image.getImageId());
        imageModel.setArchitecture(image.getArchitecture());
        imageModel.setImageType(image.getVisibility().toUpperCase());
        imageModel.setSize(Double.valueOf(image.getVirtualSize() / 1024 / 1024 / 1024));
        imageModel.setMinDisk(image.getVirtualSize() / 1024 / 1024 / 1024);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        String creationTime = image.getCreatedAt();
        if (null != creationTime && !"".equals(creationTime)) {
            try {
                imageModel.setGmtCreate(sdf.parse(creationTime));
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        return imageModel;
    }

    /**
     * 状态转换
     *
     * @param status
     * @return
     */
    public String convertStatus(String status) {
        if ("available".equalsIgnoreCase(status)) {// 可用
            return "available";
        } else if ("creating".equalsIgnoreCase(status)) {// 创建中国
            return "creating";
        } else if ("error".equalsIgnoreCase(status)) {// 错误
            return "ERROR";
        } else { //下载中、上传中、格式转换中
            return "ERROR";
        }
    }
}
