package com.bocloud.cmp.provider.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.LbModel;
import com.bocloud.cmp.provider.volmodel.LoadBalancers;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ClassName: VolLoadBalanceConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/10/11 11:54
 */
public class VolLoadBalanceConvertor implements BeanConvertor<LbModel, LoadBalancers> {
    @Override
    public List<LbModel> convertList(List<? extends LoadBalancers> balancers) {
        List<LbModel> models = Lists.newArrayList();
        for (LoadBalancers loadBalancer : balancers) {
            models.add(convertModel(loadBalancer));
        }
        return models;
    }

    @Override
    public LbModel convertModel(LoadBalancers info) {
        LbModel model = new LbModel();
        if (null != info) {
            model.setLbUuid(info.getLoadBalancerId());
            if (StringUtils.isEmpty(info.getLoadBalancerName())) {
                model.setName(info.getLoadBalancerId());
            } else {
                model.setName(info.getLoadBalancerName());
            }
            model.setStatus(info.getStatus());
            model.setRemark(info.getDescription());
            model.setCreateTime(dateConver(info.getCreateTime()));
            model.setLoadBalancerType(info.getType());
            model.setVpcUuid(info.getVpcId());
            model.setVipSubnetUuid(info.getSubnetId());
            model.setModificationProtection(info.getModificationProtectionStatus());
            if ("NonProtection".equalsIgnoreCase(info.getModificationProtectionStatus())) {
                model.setDeleteProtection("off");
            } else if ("ConsoleProtection".equalsIgnoreCase(info.getModificationProtectionStatus())) {
                model.setDeleteProtection("on");
            } else {
                model.setDeleteProtection("off");
            }
            model.setModificationProtectionReason(info.getModificationProtectionReason());
            model.setEipUuid(info.getEipID());
            model.setElasticIpAddress(info.getEipAddress());
            model.setNetworkCardUuid(info.getEniID());
            model.setVipAddress(info.getEniAddress());
            model.setExpiredTime(dateConver(info.getDeletedTime()));
            model.setSpec(info.getLoadBalancerSpec());
            model.setInternetChargeType(String.valueOf(info.getLoadBalancerBillingType()));
            JSONObject eip = info.getEip();
            if (null != eip) {
                model.setBroadbandValue(Long.valueOf(eip.getInteger("Bandwidth")));
            }
            // 监听
            List<JSONObject> listeners = info.getListeners();
            if (CollectionUtils.isNotEmpty(listeners)) {
                List<String> listenerList = new ArrayList<>();
                for (JSONObject resource : listeners) {
                    listenerList.add(resource.getString("ListenerId"));
                }
                model.setListeners(listenerList);
            }
            // 服务器组 resourceGroupUuid
            List<JSONObject> pools = info.getServerGroups();
            if (CollectionUtils.isNotEmpty(pools)) {
                List<String> lbPoolUuids = new ArrayList<>();
                for (JSONObject resource : pools) {
                    lbPoolUuids.add(resource.getString("ServerGroupId"));
                }
                model.setLbPoolUuids(lbPoolUuids);
            }
        }
        return model;
    }

    private Date dateConver(String dateStr) {
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
            if (null != dateStr && !"".equals(dateStr)) {
                Date date = format.parse(dateStr);
                return date;
            }
            return null;
        } catch (Exception e) {
            return null;
        }
    }
}
