package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.provider.volmodel.AssociatedElasticIp;
import com.bocloud.cmp.provider.volmodel.NetworkInterfaceSets;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * @ClassName: VolNetworkConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/16 17:59
 */
public class VolNetworkConvertor implements BeanConvertor<NetworkCardModel, NetworkInterfaceSets> {
    @Override
    public List<NetworkCardModel> convertList(List<? extends NetworkInterfaceSets> networkInterfaceSets) {
        List<NetworkCardModel> models = Lists.newArrayList();
        for (NetworkInterfaceSets networkInterfaceSet : networkInterfaceSets) {
            models.add(convertModel(networkInterfaceSet));
        }
        return models;
    }

    @Override
    public NetworkCardModel convertModel(NetworkInterfaceSets network) {
        NetworkCardModel model = new NetworkCardModel();
        String creationTime = network.getCreatedAt();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        if (null != creationTime && !"".equals(creationTime)) {
            try {
                model.setGmtCreate(sdf.parse(creationTime));
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        model.setRemark(network.getDescription());
        model.setNetworkcardUuid(network.getNetworkInterfaceId());
        if (StringUtils.isNotBlank(network.getNetworkInterfaceName())) {
            model.setName(network.getNetworkInterfaceName());
        } else {
            model.setName(network.getNetworkInterfaceId());
        }
        model.setVpcUuid(network.getVpcId());
        model.setZone(network.getZoneId());
        model.setSubnetUuid(network.getSubnetId());
        model.setMac(network.getMacAddress());
        model.setInstanceId(network.getDeviceId());
        if (StringUtils.isNotBlank(network.getType()) && "Primary".equalsIgnoreCase(network.getType())) {
            model.setIsPrimary(true);
        } else {
            model.setIsPrimary(false);
        }
        model.setIpAddress(network.getPrimaryIpAddress());
        AssociatedElasticIp associatedElasticIp = network.getAssociatedElasticIp();
        if (null != associatedElasticIp) {
            model.setFloatingIpUuid(associatedElasticIp.getAllocationId());
        }
        if (StringUtils.isNotBlank(network.getStatus()) && "InUse".equalsIgnoreCase(network.getStatus())) {
            model.setUsed(true);
        } else {
            model.setUsed(false);
        }
        model.setSecurityGroupIds(network.getSecurityGroupIds());
        return model;
    }
}
