package com.bocloud.cmp.provider.convertor;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.NetworkCardModel;
import com.bocloud.cmp.model.NetworkInterfaceModel;
import com.bocloud.cmp.model.ServerModel;
import com.bocloud.cmp.model.VolumeModel;
import com.bocloud.cmp.provider.volmodel.InstanceType;
import com.bocloud.cmp.provider.volmodel.Instances;
import com.bocloud.cmp.provider.volmodel.NetworkInterfaceRes;
import com.bocloud.cmp.provider.volmodel.VolumeRes;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName: VolServerConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/14 15:22
 */
public class VolServerConvertor implements BeanConvertor<ServerModel, Instances> {
    @Override
    public List<ServerModel> convertList(List<? extends Instances> instances) {
        List<ServerModel> serverModels = new ArrayList<>();
        for (Instances instance : instances) {
            serverModels.add(convertModel(instance));
        }
        return serverModels;
    }

    @Override
    public ServerModel convertModel(Instances instance) {
        ServerModel model = null;
        if (null != instance) {
            model = new ServerModel();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            String creationTime = instance.getCreatedAt();
            if (null != creationTime && !"".equals(creationTime)) {
                try {
                    model.setGmtVmCreate(sdf.parse(creationTime));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            model.setInstanceId(instance.getId());
            model.setZone(instance.getZoneId());
            model.setImageUuid(instance.getImageId());

            model.setStatus(getStatus(instance.getStatus()));
            model.setName(instance.getInstanceName());
            model.setRemark(instance.getDescription());
            model.setHostName(instance.getHostName());
            model.setVpcUuid(instance.getVpcId());
            model.setKeypairName(instance.getKeyPairName());
            model.setKeypairIds(new String[]{instance.getKeyPairId()});

            InstanceType instanceType = instance.getInstanceType();
            model.setFlavorUuid(instanceType.getId());
            model.setCpu(instanceType.getCpu());
            model.setMemory(BigDecimal.valueOf(instanceType.getMem()).divide(BigDecimal.valueOf(1024), 2,
                    BigDecimal.ROUND_HALF_UP));
            this.volumeConvertor(instance, model);
            model.setKeypairName(instance.getKeyPairName());
            this.securityConvertor(instance, model);
//            String osType = instance.getOSType();
//            model.setOsCategory(osType);
//            if (null != osType && "linux".equalsIgnoreCase(osType)) {
//                model.setAccount("root");
//            } else if ("windows".equalsIgnoreCase(osType)) {
//                model.setAccount("administrator");
//            }

        }
        return model;
    }

    private void securityConvertor(Instances instance, ServerModel model) {
        List<NetworkInterfaceRes> networkInterfaces = instance.getNetworkInterfaces();
        model.setPrivateIps(JSONObject.toJSONString(networkInterfaces));
        if (null != networkInterfaces && !networkInterfaces.isEmpty()) {
            List<String> secgList = new ArrayList<String>();
            List<NetworkCardModel> networkCards = Lists.newArrayList();
            List<NetworkInterfaceModel> nicsModels = new ArrayList<NetworkInterfaceModel>();
            NetworkCardModel networkCard = null;
            for (NetworkInterfaceRes networkInterface : networkInterfaces) {
                networkCard = new NetworkCardModel();
                networkCard.setMac(networkInterface.getMacAddress());
                networkCard.setIpAddress(networkInterface.getPrimaryIpAddress());
                networkCard.setNetworkcardUuid(networkInterface.getNetworkInterfaceId());
                networkCard.setVpcUuid(networkInterface.getVpcId());
                if (StringUtils.isNotBlank(networkInterface.getNetworkInterfaceName())) {
                    networkCard.setName(networkInterface.getNetworkInterfaceName());
                } else {
                    networkCard.setName(networkInterface.getNetworkInterfaceId());
                }
                networkCard.setZone(instance.getZoneId());
                networkCard.setSubnetUuid(networkInterface.getSubnetId());
                networkCard.setInstanceId(instance.getId());
                if (StringUtils.isNotBlank(networkInterface.getType()) && "Primary".equalsIgnoreCase(networkInterface.getType())) {
                    networkCard.setIsPrimary(true);
                } else {
                    networkCard.setIsPrimary(false);
                }
                networkCard.setIpAddress(networkInterface.getPrimaryIpAddress());
                networkCard.setFloatingIpUuid(networkInterface.getEipAddress());
                if (StringUtils.isNotBlank(networkInterface.getStatus()) && "InUse".equalsIgnoreCase(networkInterface.getStatus())) {
                    networkCard.setUsed(true);
                } else {
                    networkCard.setUsed(false);
                }
                networkCard.setSecurityGroupIds(Lists.newArrayList(networkInterface.getSecurityGroupIds()));
                networkCards.add(networkCard);
                secgList.addAll(JSONArray.parseArray(networkInterface.getSecurityGroupIds(), String.class));
            }
            model.setNetworkCards(networkCards);
            model.setGroups(JSONArray.toJSONString(secgList));
        }
        networkInterfaces = networkInterfaces.stream().filter(network -> "primary".equals(network.getType())).collect(Collectors.toList());
        model.setManagerIp(networkInterfaces.get(0).getPrimaryIpAddress());
    }

    private void volumeConvertor(Instances instance, ServerModel model) {
        List<VolumeRes> volumes = instance.getVolumes();
        List<VolumeModel> volumeModels = new ArrayList<VolumeModel>();
        if (null != volumes && volumes.size() > 0) {
            VolumeModel volumeModel = null;
            for (VolumeRes vol : volumes) {
                volumeModel = new VolumeModel();
                volumeModel.setVolumeUuid(vol.getVolumeId());
                String kind = vol.getKind();
                if (null != kind && "system".equals(kind)) {
                    volumeModel.setBootVolume(true);
                } else {
                    volumeModel.setBootVolume(false);
                }
                volumeModel.setType(kind);
                volumeModels.add(volumeModel);
            }
        }
        model.setDiskDevices(volumeModels);
        volumes = volumes.stream().filter(volume -> "system".equals(volume.getKind())).collect(Collectors.toList());
        Integer disk = 0;
        for (VolumeRes volume : volumes) {
            disk += Integer.valueOf(volume.getSize());
        }
        model.setDisk(disk);
    }

    private String getStatus(String status) {
        if ("CREATING".equalsIgnoreCase(status)) {
            return "BUILDING";
        } else if ("RUNNING".equalsIgnoreCase(status)) {
            return "RUNNING";
        } else if ("STOPPING".equalsIgnoreCase(status)) {
            return "PAUSING";
        } else if ("STOPPED".equalsIgnoreCase(status)) {
            return "STOPPED";
        } else if ("REBOOTING".equalsIgnoreCase(status)) {
            return "RESTARTING";
        } else {
            return "ERROR";
        }
    }
}
