package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.LbMemberModel;
import com.bocloud.cmp.model.LbPoolModel;
import com.bocloud.cmp.provider.volmodel.ServerGroups;
import com.bocloud.cmp.provider.volmodel.Servers;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: VolServerGroupConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/10/12 17:16
 */
public class VolServerGroupConvertor implements BeanConvertor<LbPoolModel, ServerGroups> {
    @Override
    public List<LbPoolModel> convertList(List<? extends ServerGroups> groups) {
        List<LbPoolModel> models = Lists.newArrayList();
        for (ServerGroups serverGroups : groups) {
            models.add(convertModel(serverGroups));
        }
        return models;
    }

    @Override
    public LbPoolModel convertModel(ServerGroups serverGroup) {
        LbPoolModel poolModel = new LbPoolModel();
        if (serverGroup != null) {
            poolModel.setPoolUuid(serverGroup.getServerGroupId());
            poolModel.setName(serverGroup.getServerGroupName() == null ? serverGroup.getServerGroupId() : serverGroup.getServerGroupName());
            poolModel.setLbUuid(serverGroup.getLoadBalancerId());
            poolModel.setRemark(serverGroup.getDescription());
            List<String> listeners = serverGroup.getListeners();
            if (CollectionUtils.isNotEmpty(listeners)) {
                poolModel.setListenerUuid(listeners.get(0));
                poolModel.setListenerUuids(listeners);
            }
            //后端服务器
            List<Servers> servers = serverGroup.getServers();
            if (CollectionUtils.isNotEmpty(servers)) {
                List<String> memberUuidList = new ArrayList<>();
                List<LbMemberModel> memberList = new ArrayList<>();
                for (Servers server : servers) {
                    memberUuidList.add(server.getServerId());
                    LbMemberModel memberModel = new LbMemberModel();
                    memberModel.setMemberUuid(server.getServerId());
                    memberModel.setInstanceId(server.getInstanceId());
                    memberModel.setPort(Integer.valueOf(server.getPort()));
                    memberModel.setWeight(Integer.valueOf(server.getWeight()));
                    memberModel.setType(server.getType());
                    memberModel.setRemark(server.getDescription());
                    memberModel.setPoolUuid(serverGroup.getServerGroupId());
                    memberModel.setAddress(server.getIp());
                    memberList.add(memberModel);
                }
                poolModel.setMemberUuids(memberUuidList);
                poolModel.setMemberModels(memberList);
            }
        }
        return poolModel;
    }
}
