package com.bocloud.cmp.provider.convertor;

import com.bocloud.cmp.model.SubnetModel;
import com.bocloud.cmp.provider.volmodel.Subnets;
import com.bocloud.common.utils.ListTool;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

/**
 * @ClassName: VolSubnetConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/13 16:19
 */
public class VolSubnetConvertor implements BeanConvertor<SubnetModel, Subnets> {
    @Override
    public List<SubnetModel> convertList(List<? extends Subnets> subnets) {
        List<SubnetModel> models = Lists.newArrayList();
        if (ListTool.isEmpty(subnets)) {
            return models;
        }
        for (Subnets subnet : subnets) {
            models.add(convertModel(subnet));
        }
        return models;
    }

    @Override
    public SubnetModel convertModel(Subnets vSwitch) {
        SubnetModel model = new SubnetModel();
        if (null != vSwitch) {
            model.setSubnetUuid(vSwitch.getSubnetId());
            model.setVpcUuid(vSwitch.getVpcId());
            String status = vSwitch.getStatus();
            if (null != status) {
                model.setStatus(status.toUpperCase());
            }
            model.setCidr(vSwitch.getCidrBlock());
            model.setZone(vSwitch.getZoneId());
            model.setRemark(vSwitch.getDescription());
            if (StringUtils.isEmpty(vSwitch.getSubnetName())) {
                model.setName(vSwitch.getSubnetId());
            } else {
                model.setName(vSwitch.getSubnetName());
            }
            String creationTime = vSwitch.getCreationTime();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            if (null != creationTime && !"".equals(creationTime)) {
                try {
                    model.setGmtCreate(sdf.parse(creationTime));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            model.setIpVersion("V4");
            model.setResourceGroupUuid(vSwitch.getRouteTable().getRouteTableId());

        }
        return model;
    }

    public static void main(String[] args) throws ParseException {
        String s = "2021-09-13T17:49:57+08:00";
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
        System.out.println(sdf.parse(s));
    }
}
