package com.bocloud.cmp.provider.convertor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.ServerVolumeModel;
import com.bocloud.cmp.model.VolumeModel;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: VolVolumeConvertor
 * @Description:
 * @Author lxy
 * @Date 2021/9/15 14:03
 */
public class VolVolumeConvertor implements BeanConvertor<VolumeModel, JSONObject> {
    @Override
    public List<VolumeModel> convertList(List<? extends JSONObject> volums) {
        List<VolumeModel> models = Lists.newArrayList();
        for (JSONObject volum : volums) {
            models.add(convertModel(volum));
        }
        return models;
    }

    @Override
    public VolumeModel convertModel(JSONObject disk) {
        VolumeModel model = new VolumeModel();
        if (null != disk) {
            String creationTime = disk.getString("CreatedAt");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            if (null != creationTime && !"".equals(creationTime)) {
                try {
                    model.setCreateTime(sdf.parse(creationTime));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            model.setRemark(disk.getString("Description"));
            model.setVolumeUuid(disk.getString("Id"));
            JSONObject instance = JSONObject.parseObject(disk.getString("Instance"));
            if (null != instance) {
                model.setInstanceId(instance.getString("Id"));
                List<ServerVolumeModel> serverVolumeModels = new ArrayList<>();
                ServerVolumeModel volumeModel = new ServerVolumeModel();
                volumeModel.setInstanceId(instance.getString("Id"));
                volumeModel.setVolumeUuid(disk.getString("Id"));
                serverVolumeModels.add(volumeModel);
                model.setModels(serverVolumeModels);
            }

            String kind = disk.getString("Kind");
            if (null != kind && "system".equals(kind)) {
                model.setBootVolume(true);
            } else {
                model.setBootVolume(false);
            }
            model.setType(disk.getString("Type"));
            if (!StringUtils.isEmpty(disk.getString("Name"))) {
                model.setName(disk.getString("Name"));
            } else {
                model.setName(disk.getString("Id"));
            }
            model.setSize(disk.getInteger("SizeGB"));
            model.setStatus(disk.getString("Status").toUpperCase());
            JSONObject pool = JSONObject.parseObject(disk.getString("Pool"));
            if (null != pool) {
                model.setZone(pool.getString("Zone"));
            }
        }
        return model;
    }
}
