package com.bocloud.cmp.provider.volmodel;

import lombok.Data;

/**
 * @ClassName: HealthCheck
 * @Description:
 * @Author lxy
 * @Date 2021/10/12 9:59
 */
@Data
public class HealthCheck {
    /**
     * 监听器是否开启健康检查功能。取值：
     * ● on ：开启（默认值）。
     * ● off：不开启。
     * 说明： 以下参数仅当本参数的值为 on 时有效。
     */
    private String Enabled;
    //执行健康检查的时间间隔，取值：1 ～ 300 ，默认值为2，单位：秒。
    private Integer Interval;
    /**
     * 健康检查的响应超时时间，表示如果后端服务器在指定的时间内没有正确响应，则判定为健康检查“异常”。
     * 取值：1 ～ 60，默认值为2，单位：秒。
     * 说明： 响应超时时间需小于检查时间间隔。
     */
    private Integer Timeout;
    /**
     * 健康检查的健康阈值，表示连续执行指定次数的健康检查，结果均为“正常”的后端服务器将被判定为健康检查“正常”。
     * 取值： 2 ～ 10， 默认值为3，单位：次。
     */
    private Integer HealthyThreshold;
    /**
     * 健康检查的不健康阈值，表示连续执行指定次数的健康检查，结果均为“异常”的后端服务器将被判定为健康检查异常。
     * 取值： 2 ～ 10， 默认值为3，单位：次。
     */
    private Integer UnHealthyThreshold;
    /**
     * 监听器健康检查的方法。
     * ● GET：服务器需支持GET方法。
     * ● HEAD：服务器仅返回HEAD头部信息，可以降低后端开销，但要求服务器支持HEAD方法。
     * 说明： TCP/UDP监听器返回为空。
     */
    private String Method;
    //健康检查的路径。说明： TCP/UDP监听器返回为空。
    private String Uri;
    //健康检查的域名。说明： TCP/UDP监听器返回为空。
    private String Domain;
    //健康检查正常的HTTP状态码。● http_2xx● http_3xx● http_4xx● http_5xx 说明： TCP/UDP监听器返回为空。
    private String HttpCode;
}
